##!/usr/software/bin/python3.4.3
from NATE.Log import log_global
from NACL.APISet.Response import Response
import re 
import requests
import json
import time
import sys
from requests.utils import quote

log = log_global()
class APISet(object):
    "This class will create new object for APISet"
    def __init__(self,**data):
        self.connection = data['connection']
        self.category   = data['category']
        self.interface  = data['interface']
        self.set        = data['set']
        if 'port' in data:
            self.port       = data['port']
        if 'api_server' in data:
            self.apiserv = data['api_server']
        self.log        = log
        self.log.info("Connection : %s" %self.connection)
        self.log.info("Category   : %s " %self.category)
        self.log.info("Interface  : %s" %self.interface)
        self.log.info("Set        : %s" %self.set)
        self.prep_service()
        
    def translate(self,**args):
        "Taking hash values and framing cmode command"
        ret_cmd=""
        for key in args:
            smf_field = '-'+key + ' ' + args[key]
            ret_cmd += ' ' + smf_field 
        log.info(ret_cmd)
        return ret_cmd
        
    def create_uri(self):
        log.info("Creating URI for running commands")
        return "https://"+self.apiserv+":"+self.port+"/api"+"/1.0/slo/"

    def get_key(self,**data):
        "Takes two params - api to run and search field"
        apiserv_uri=self.create_uri()
        api=data['api']
        payload=data
        del payload['api']
        log.info(payload)
        log.info(apiserv_uri+api)
        res = requests.get(apiserv_uri+api,
                    auth=("admin","Netapp1!"), 
                    headers={'Content-Type': 'application/json', 'Accept': 'application/json'},
                    params=payload,
                    verify=False)
        if(res.status_code == 200):
            i=0
            rec = json.loads(res.text)["result"]["total_records"]
            #Need code to iterate if records are more then  500
            rec=rec-1
            while(rec>=0):
                if(api == 'controllers'):
                    if(data['name']==json.loads(res.text)["result"]["records"][rec]["name"]):
                        return json.loads(res.text)["result"]["records"][rec]["storage_system_key"]
                elif(api == "data-protection-nodes"):
                        log.info(" Data protection nodes status")
                        return json.loads(res.text)["result"]["records"][rec]["protection_relationship_state"]
                else:
                    if(data['name']==json.loads(res.text)["result"]["records"][rec]["name"]):
                        return json.loads(res.text)["result"]["records"][rec]["key"]
                rec=rec-1
        return 1
        
    def invoke_apiservice(self,operation,final_comm):
        "This method will invoke apiservice calls for get,put,delete and post"
        if(operation == "get"):
            self.request_get(final_comm)
        elif(operation =="put"):
            log.info("operation is put")
        elif(operation == "delete"):
            self.request_delete(final_comm)
            log.info("operations is delete")
        elif(operation == "post"):
            self.request_post(final_comm)
            log.info("operation is post")
            
    def request_get(self,final_comm):
        apiserv_uri=self.create_uri()
        log.info(apiserv_uri+final_comm)
        res = requests.get(apiserv_uri+final_comm,
                    auth=("admin","Netapp1!"), 
                    headers={'Content-Type': 'application/json', 'Accept': 'application/json'},
                    verify=False)
        self.key_val={}
        # Run request and check for next tag 
        # If next tag exists run get request again 
        # Else close the loop 
        if(res.status_code == 200):
            rec = json.loads(res.text)["result"]["total_records"]
            rec=rec-1
            while(rec>=0):
               name = json.loads(res.text)["result"]["records"][rec]["name"]
               key_name = json.loads(res.text)["result"]["records"][rec]["key"]
               log.info(name+":"+key_name)
               self.key_val[key_name]=name
               rec=rec-1
        
    def getkey_from_parsed_output(self,search):
        for k, v in self.key_val.items():
            if(search == v):
                return k
        
    def request_post(self,final_comm):
        apiserv_uri=self.create_uri()
        log.info(self.post_req)
        uri=apiserv_uri+final_comm
        log.info("POST URL -> " + uri)
        log.info(self.post_req)
        res = requests.post(uri,
                    auth=("admin","Netapp1!"), 
                    headers={'Content-Type': 'application/json', 'Accept': 'application/json'},
                    json=self.post_req,
                    verify=False)
        log.info(res.status_code)
        log.info(res.headers)
        if(res.status_code == 202):
            log.info(res.json())
            job_url=res.headers["Location"]
            print("Job URL -> " + job_url)
            time_to_wait=30
            while(time_to_wait):
                log.info(job_url)
                res = requests.get(job_url,
                    auth=("admin","Netapp1!"), 
                    headers={'Content-Type': 'application/json', 'Accept': 'application/json'},
                    verify=False)
                status=json.loads(res.text)["result"]["records"][0]["status"]
                print("Job Status ->" + status)
                if(status == "COMPLETED"):
                    log.info("Job completed")
                    self.resource_key=json.loads(res.text)["result"]["records"][0]["resource_key"]
                    print(self.resource_key)
                    break
                elif(status == "FAILED"):
                    log.info("Job failed")
                    log.info(res.text)
                    break
                else:
                    print("Waiting for job to complete - 10sec sleep")
                    time.sleep(10)
                    time_to_wait=time_to_wait-10
        else:
            log.info(res.text)
            log.info(res.headers)
            print("Status code -> "+ res.status_code)
            
    def request_delete(self,final_comm):
        apiserv_uri=self.create_uri()
        print("URL for delete: " + apiserv_uri+final_comm)
        print(self.post_req)
        uri=apiserv_uri+final_comm
        res = requests.delete(uri,
                    auth=("admin","Netapp1!"), 
                    headers={'Content-Type': 'application/json', 'Accept': 'application/json'},
                    verify=False)
        if(res.status_code == 202):
            log.info(res.json())
            job_url=res.headers["Location"]
            time_to_wait=30
            while(time_to_wait):
                log.info(job_url)
                res = requests.get(job_url,
                    auth=("admin","Netapp1!"), 
                    headers={'Content-Type': 'application/json', 'Accept': 'application/json'},
                    verify=False)
                status=json.loads(res.text)["result"]["records"][0]["status"]
                print("Job Status :" + status)
                if(status == "COMPLETED"):
                    print("Job completed")
                    break
                elif(status == "FAILED"):
                    print("Job failed")
                    print(res.text)
                    break
                else:
                    print("Waiting for job to complete - 10sec sleep")
                    time.sleep(10)
                    time_to_wait=time_to_wait-10
        else:
            print(res.status_code)
        pass
            
    def __getattr__(self,command):
        "Used to create runtime methods"
        def translate(**args):
            "Taking hash values and framing cmode command"
            ret_cmd=""
            for key in args:
                smf_field = '-'+key + ' ' + args[key]
                ret_cmd += ' ' + smf_field 
            print(ret_cmd)
            return ret_cmd
        def handlerFunction(*args,**kwargs):
            comm=command.replace('_',' ')
            searchObj=re.search(r'(\w+)',comm)
            self.command=searchObj.group(0)
            if(self.command == "get"):
                log.info("Call routine for apiservice")
                final_comm=comm.split(" ")
                operation=final_comm[0]
                del final_comm[0]
                final_comm = "-".join(final_comm)
                url_key=""
                url_field=""
                temp_json={}
                self.post_req={}
                for k, v in kwargs.items():
                    if(k=="url_key"):
                        url_key=quote(v,safe='')
                    elif(k=="url_field"):
                        url_field=v
                    else:
                        temp_json[k]=v
                out="/"+url_key +"/"+url_field
                self.post_req=temp_json
                final_comm=final_comm+out
                temp=final_comm.replace('//','/')
                final_comm=temp
                self.invoke_apiservice(operation,final_comm)
            else:
                log.info("Call routine for NACL APISet")
                out_cmd=translate(**kwargs)
                comm=comm + out_cmd
                response=self.connection.execute(command = comm )
                res_obj=Response(output=response,CLI_separator="X;X")
                return res_obj
        return handlerFunction
     
    def send_command(self,comm=None,**param):
        out_cmd=self.translate(**param)
        comm=comm + out_cmd
        response=self.connection.execute(command = comm )
        res_obj=Response(output=response,CLI_separator="X;X")
        return res_obj
    
    def prep_service(self):
        self.connection.execute(command = "set -confirmations off")
        self.connection.execute(command = 'set -showallfields true -showseparator "X;X" ')
        self.connection.execute(command = "system timeout modify -timeout 0")
        self.connection.execute(command = "rows 0" )