#
# Copyright (c) 2009-2010 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NACL::APISet::Exceptions::ResponseException that will be thrown if 
##          command execution does not work as expected. It is intended to be
##          a generic exception to be thrown when command failure occurs 
##          for a reason other than those covered by the other exceptions.
##          This would include cases such as trying to stop an aggr scrub
##          after it has already been stopped, or when it is unclear from
##          the command output why the command failed.
##
## @author dl-nacl-dev@netapp.com
## @status private
## @pod here

=pod

=head1 Package Name

NACL::APISet::Exceptions::CommandFailedException.pm

=head1 Author

dl-nacl-dev@netapp.com

=head1 Description

An exception to be thrown if command execution does not work as expected. It
is intended to be a generic exception to be thrown when command failure occurs
for a reason other than those covered by the other exceptions. This would 
include cases such as trying to stop an aggr scrub which has already been 
stopped, or when it is unclear from the output why the command failed.

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.

B<NACL/APISet/Exceptions/ResponseException.pm>

=head1 NACL::APISet::Exceptions::CommandFailedException

=head2 Expected use

An exception to be thrown if command execution does not work as expected. It
is intended to be a generic exception to be thrown when command failure occurs
for a reason other than those covered by the other exceptions. This would
include cases such as trying to stop an aggr scrub which has already been
stopped, or when it is unclear from the output why the command failed.

=head2 Usage

Same as for exceptions defined by ResponseException

=head2 Base Class

NACL::APISet::Exceptions::ResponseException

=cut

package NACL::APISet::Exceptions::CommandFailedException;
use base qw(NACL::APISet::Exceptions::ResponseException);
1;
