#
# Copyright (c) 2009-2010 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NACL::APISet::Exceptions::TimeoutException that will be thrown if
##          interrupt+resync is failed after the timeout.
##
## @author dl-nacl-dev@netapp.com
## @status private
## @pod here

=pod

=head1 Package Name

NACL::APISet::Exceptions::CouldNotResyncConnectionException.pm

=head1 Author

dl-nacl-dev@netapp.com

=head1 Description

An exception to be thrown if interrupt+resync times out or is idle

=head2 Base Class

NACL::APISet::Exceptions::TimeoutException

=head2 Examples

=over

=item B<Catching the exception>

    try {
        $connection->send_intr();
        $connection->resync(%args_to_resync);
    } catch NATE::BaseException with {
        NACL::APISet::Exceptions::CouldNotResyncConnectionException->detect_convert_and_throw();
    };

=back

=cut

package NACL::APISet::Exceptions::CouldNotResyncConnectionException;
use base qw(NACL::APISet::Exceptions::TimeoutException);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter();
    my ( $pkg, @args ) = @_;

    my $detected =
      $pkg->_detect_helper( @args, regexes => [qr/Timeout\s+waiting\s+for\s+command\s+to\s+complete/i] );

    $Log->exit();
    return $detected;
}

1;
