# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is thrown when
##          an attempt was made to run a 7Mode command that does not exist.
##          This also gets NACL::APISet::Exceptions::ResponseException in its
##          inheritance hierarchy.
##
## @author dl-nacl-dev@netapp.com
## @status private
## @pod here

=pod

=head1 Package Name

NACL::APISet::Exceptions::FailedAndNotFoundException.pm

=head1 Author

dl-nacl-dev@netapp.com

=head1 Description

This exception was introduced purely for backwards compatibility reasons.
Earlier, a C<CommandFailedException> was thrown when attempting to invoke
a 7Mode command that does not exist. This should more correctly have been
C<CommandNotFoundException>. Changing this is potentially backwards
incompatible. This new exception subclasses both of these exceptions,
so that earlier calls which were explicitly catching C<CommandFailedException>
continue to work, while newer code that expects a C<CommandNotFoundException>
would also work.

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.

B<NACL/APISet/Exceptions/CommandNotFoundException.pm>
B<NACL/APISet/Exceptions/CommandFailedException.pm>

=head1 NACL::APISet::Exceptions::FailedAndNotFoundException

=head2 Expected use

Use when an attempt was made to run a 7Mode command that does not exist.

=head2 Usage

Same as for exceptions defined by ResponseException

=head2 Base Classes

NACL::APISet::Exceptions::CommandNotFoundException
NACL::APISet::Exceptions::CommandFailedException

=cut

package NACL::APISet::Exceptions::FailedAndNotFoundException;
use base qw(
    NACL::APISet::Exceptions::CommandNotFoundException
    NACL::APISet::Exceptions::CommandFailedException
);
1;
