#
# Copyright (c) 2015-2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base for Flashray API Sets CLI commands
## @author nsesha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Flashray;

#
# Compiler Directives
#
use strict;
use warnings;

# NATE imports
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use Params::Validate qw(validate_with SCALAR SCALARREF);

use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument;

# Base class
use base qw(NACL::APISet);

sub _new {
    $Log->enter() if $may_enter;

    my ( $class, %args ) = @_;
    my $self = {};

    # Bless the hash returned which will make it a Perl object
    bless $self, $class;

    # And give the caller the reference to the bless object.
    $Log->exit() if $may_exit;
    return $self;
}

1;

=head1 NAME

NACL::APISet::Flashray - Base class for API Sets for Flashray

=head1 SYNOPSIS

 use NACL::APISet::Flashray;

=head1 DESCRIPTION

A C<NACL::APISet::Flashray> is used to create API Set objects for Flashray. This acts
as a factory class for creation of API Set objects for various Flashray interfaces
such as CLI, REST

=cut

=head1 EXCEPTIONS

=over

=item NATE::BaseException

=item NATE::Exceptions::Argument

=back

=cut

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

 $Id: //depot/prod/DOT/dev/test/lib/NACL/APISet/Flashray.pm#4 $
 Last revised $Date:$ by $Author:$

=cut
