#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/AIX/default/cfgmgr.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse AIX cfgmgr command
##
## @description
##      This library supports error checking for cfgmgr command, but output parsing is not supported as of now.
##
## @synopsis
##    use cfgmgr;
##
##    cfgmgr(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::AIX::default::cfgmgr;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

## @name ::cfgmgr
##
## @summary Parses the raw output from the "cfgmgr" command
##
## @description
##     This library supports error checking for cfgmgr command, but output parsing is not supported as of now.
## @synopsis
##      cfgmgr(%args);
## INPUTS TO METHOD CALL 'cfgmgr' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'phase-1' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'specifies-device-to-configure' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'phase-2' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'device' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'specified-phase' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
##  Returns: Output parsing is not supported.
sub cfgmgr {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub cfgmgr

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /([Uu]sage:|cfgmgr:)\s+Not a recognized.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ( $raw_output =~ /(cfgmgr:)\s+\d+.*/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;

