#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/AIX/default/chtz.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse AIX chtz command
##
## @description
##    This is a library to parse the raw output of AIX chtz command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "chtz" will chtz()
##
## @synopsis
##    use chtz;
##
##    chtz(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::AIX::default::chtz;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::chtz
##
## @summary Parses the raw output from the "chtz" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from chtz with any of its options as described in the
##    corresponding cdef file 'chtz.cdefs'
## @synopsis
##      chtz(%args);
## INPUTS TO METHOD CALL 'chtz' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'time-zone' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub chtz {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];

} ## end sub chtz

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error       = undef;
    my $error_level = $resp->get_execution_status();
    if ( $raw_output =~ /(Usage:)\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
