#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/AIX/default/dumpfs.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse AIX dumpfs command
##
## @description
##   This library supports error checking for dumpfs command, but output parsing is not supported as of now
## @synopsis
##    use dumpfs;
##
##    dumpfs(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::AIX::default::dumpfs;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::dumpfs
##
## @summary Parses the raw output from the "dumpfs" command
##
## @description 
##     This library supports error checking for dumpfs command, but output parsing is not supported as of now
## @synopsis
##      dumpfs(%args);
## INPUTS TO METHOD CALL 'dumpfs' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'dir-path' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns: Output parsing is not supported.

sub dumpfs {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];

} ## end sub dumpfs

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error       = undef;
    my $error_level = $resp->get_execution_status();
    if (
        $raw_output =~ /Usage:|dumpfs:|A file or directory in the path name does not exist/i )
    {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;
