# //depot/prod/DOT/dev/test/nacldev/lib/NACL/APISet/Host/CLI/AIX/default/lscfg.pm#1 $
#
#copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse AIX lscfg command
##
## @description
##    This is a library to parse the raw output of AIX lspath command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "lspath" lspath()
##
## @synopsis
##    use lscfg;
##
##    lscfg(%args);
##
## @author saparna, Aparna.S2@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
#
package NACL::APISet::Host::CLI::AIX::default::lscfg;

use strict;

#use  NACL::APISet::Exceptions::InvalidParamException qw(:try) ;
use NACL::APISet::Response::CLI::ParserUtils;

## @name ::lscfg
##
## @summary Parses the raw output from the "lscfg" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lscfg with any of its options as described in the
##    corresponding cdef file 'lscfg.cdefs'
## @synopsis
##      lscfg(%args);
## INPUTS TO METHOD CALL 'lscfg' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-l' </td> <td> 'device-description' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'platform-specific-info' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'vpd' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'device-info' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = lscfg -s
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;INSTALLED RESOURCE LIST
##&nbsp;
##&nbsp;The following resources are installed on the machine.
##&nbsp;+/- = Added or deleted from Resource List.
##&nbsp;*   = Diagnostic support not available.
##&nbsp;
##&nbsp;  Model Architecture: chrp
##&nbsp;  Model Implementation: Multiple Processor, PCI bus
##&nbsp;
##&nbsp;+ sys0
##&nbsp;        System Object
##&nbsp;+ sysplanar0
##&nbsp;        System Planar
##&nbsp;* vio0
##&nbsp;        Virtual I/O Bus
##&nbsp;* vsa1             U787F.001.DPM1FFD-P1-T2
##&nbsp;        LPAR Virtual Serial Adapter
##&nbsp;* vty1             U787F.001.DPM1FFD-P1-T2-L0
##&nbsp;        Asynchronous Terminal
##&nbsp;* vsa0             U787F.001.DPM1FFD-P1-T1
##&nbsp;        LPAR Virtual Serial Adapter
##&nbsp;* vty0             U787F.001.DPM1FFD-P1-T1-L0
##&nbsp;        Asynchronous Terminal
##&nbsp;* pci2             U787F.001.DPM1FFD-P1
##&nbsp;        PCI Bus
##&nbsp;* pci10            U787F.001.DPM1FFD-P1
##&nbsp;        PCI Bus
##&nbsp;+ ent2             U787F.001.DPM1FFD-P1-C1-T1
##&nbsp;        Gigabit Ethernet-SX PCI-X Adapter (14106802)
##&nbsp;* pci11            U787F.001.DPM1FFD-P1
##&nbsp;        PCI Bus
##&nbsp;* pci8             U787F.001.DPM1FFD-P1
##&nbsp;        PCI Bus
##&nbsp;+ usbhc0           U787F.001.DPM1FFD-P1
##&nbsp;        USB Host Controller (33103500)
##&nbsp;+ usbhc1           U787F.001.DPM1FFD-P1
##&nbsp;        USB Host Controller (33103500)
##&nbsp;+ usbhc2           U787F.001.DPM1FFD-P1
##&nbsp;        USB Enhanced Host Controller (3310e000)
##&nbsp;* pci9             U787F.001.DPM1FFD-P1
##&nbsp;        PCI Bus
##&nbsp;+ ent0             U787F.001.DPM1FFD-P1-T5
##&nbsp;        2-Port 10/100/1000 Base-TX PCI-X Adapter (14108902)
##&nbsp;+ ent1             U787F.001.DPM1FFD-P1-T6
##&nbsp;        2-Port 10/100/1000 Base-TX PCI-X Adapter (14108902)
##&nbsp;* pci1             U787F.001.DPM1FFD-P1
##&nbsp;        PCI Bus
##&nbsp;* pci7             U787F.001.DPM1FFD-P1
##&nbsp;        PCI Bus
##&nbsp;* pci5             U787F.001.DPM1FFD-P1
##&nbsp;        PCI Bus
##&nbsp;* pci3             U787F.001.DPM1FFD-P1
##&nbsp;        PCI Bus
##&nbsp;* ide0             U787F.001.DPM1FFD-P1-T12
##&nbsp;        ATA/IDE Controller Device
##&nbsp;+ cd0              U787F.001.DPM1FFD-P4-D2
##&nbsp;        IDE DVD-ROM Drive
##&nbsp;* pci4             U787F.001.DPM1FFD-P1
##&nbsp;        PCI Bus
##&nbsp;+ sisscsia0        U787F.001.DPM1FFD-P1
##&nbsp;        PCI-X Dual Channel Ultra320 SCSI Adapter
##&nbsp;+ scsi0            U787F.001.DPM1FFD-P1-T10
##&nbsp;        PCI-X Dual Channel Ultra320 SCSI Adapter bus
##&nbsp;+ hdisk0           U787F.001.DPM1FFD-P1-T10-L5-L0
##&nbsp;        16 Bit LVD SCSI Disk Drive (73400 MB)
##&nbsp;+ hdisk1           U787F.001.DPM1FFD-P1-T10-L8-L0
##&nbsp;        16 Bit LVD SCSI Disk Drive (73400 MB)
##&nbsp;+ ses0             U787F.001.DPM1FFD-P1-T10-L15-L0
##&nbsp;        SCSI Enclosure Services Device
##&nbsp;+ scsi1            U787F.001.DPM1FFD-P1-T11
##&nbsp;        PCI-X Dual Channel Ultra320 SCSI Adapter bus
##&nbsp;* pci6             U787F.001.DPM1FFD-P1
##&nbsp;        PCI Bus
##&nbsp;* pci0             U787F.001.DPM1FFD-P1
##&nbsp;        PCI Bus
##&nbsp;+ fcs0             U787F.001.DPM1FFD-P1-C4-T1
##&nbsp;        FC Adapter
##&nbsp;* fcnet0           U787F.001.DPM1FFD-P1-C4-T1
##&nbsp;        Fibre Channel Network Protocol Device
##&nbsp;+ fscsi0           U787F.001.DPM1FFD-P1-C4-T1
##&nbsp;        FC SCSI I/O Controller Protocol Device
##&nbsp;* sfwcomm0         U787F.001.DPM1FFD-P1-C4-T1-W0-L0
##&nbsp;        Fibre Channel Storage Framework Comm
##&nbsp;+ fcs1             U787F.001.DPM1FFD-P1-C4-T2
##&nbsp;        FC Adapter
##&nbsp;* fcnet1           U787F.001.DPM1FFD-P1-C4-T2
##&nbsp;        Fibre Channel Network Protocol Device
##&nbsp;+ fscsi1           U787F.001.DPM1FFD-P1-C4-T2
##&nbsp;        FC SCSI I/O Controller Protocol Device
##&nbsp;* sfwcomm1         U787F.001.DPM1FFD-P1-C4-T2-W0-L0
##&nbsp;        Fibre Channel Storage Framework Comm
##&nbsp;+ L2cache0
##&nbsp;        L2 Cache
##&nbsp;+ mem0
##&nbsp;        Memory
##&nbsp;+ proc0
##&nbsp;        Processor
##&nbsp;+ proc2
##&nbsp;        Processor
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'model_architecture' => ' chrp',
##&nbsp;            'model_implementation' => ' Multiple Processor, PCI bus',
##&nbsp;            'installed_resource_list' => [
##&nbsp;                                           {
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'sys0',
##&nbsp;                                             'type' => 'System Object'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'sysplanar0',
##&nbsp;                                             'type' => 'System Planar'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'vio0',
##&nbsp;                                             'type' => 'Virtual I/O Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T2',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'vsa1',
##&nbsp;                                             'type' => 'LPAR Virtual Serial Adapter'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T2-L0',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'vty1',
##&nbsp;                                             'type' => 'Asynchronous Terminal'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T1',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'vsa0',
##&nbsp;                                             'type' => 'LPAR Virtual Serial Adapter'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T1-L0',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'vty0',
##&nbsp;                                             'type' => 'Asynchronous Terminal'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'pci2',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'pci10',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C1-T1',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'ent2',
##&nbsp;                                             'type' => 'Gigabit Ethernet-SX PCI-X Adapter (14106802)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'pci11',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'pci8',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'usbhc0',
##&nbsp;                                             'type' => 'USB Host Controller (33103500)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'usbhc1',
##&nbsp;                                             'type' => 'USB Host Controller (33103500)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'usbhc2',
##&nbsp;                                             'type' => 'USB Enhanced Host Controller (3310e000)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'pci9',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T5',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'ent0',
##&nbsp;                                             'type' => '2-Port 10/100/1000 Base-TX PCI-X Adapter (14108902)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T6',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'ent1',
##&nbsp;                                             'type' => '2-Port 10/100/1000 Base-TX PCI-X Adapter (14108902)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'pci1',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'pci7',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'pci5',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'pci3',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T12',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'ide0',
##&nbsp;                                             'type' => 'ATA/IDE Controller Device'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P4-D2',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'cd0',
##&nbsp;                                             'type' => 'IDE DVD-ROM Drive'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'pci4',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'sisscsia0',
##&nbsp;                                             'type' => 'PCI-X Dual Channel Ultra320 SCSI Adapter'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T10',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'scsi0',
##&nbsp;                                             'type' => 'PCI-X Dual Channel Ultra320 SCSI Adapter bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T10-L5-L0',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'hdisk0',
##&nbsp;                                             'type' => '16 Bit LVD SCSI Disk Drive (73400 MB)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T10-L8-L0',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'hdisk1',
##&nbsp;                                             'type' => '16 Bit LVD SCSI Disk Drive (73400 MB)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T10-L15-L0',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'ses0',
##&nbsp;                                             'type' => 'SCSI Enclosure Services Device'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T11',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'scsi1',
##&nbsp;                                             'type' => 'PCI-X Dual Channel Ultra320 SCSI Adapter bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'pci6',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'pci0',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T1',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'fcs0',
##&nbsp;                                             'type' => 'FC Adapter'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T1',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'fcnet0',
##&nbsp;                                             'type' => 'Fibre Channel Network Protocol Device'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T1',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'fscsi0',
##&nbsp;                                             'type' => 'FC SCSI I/O Controller Protocol Device'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T1-W0-L0',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'sfwcomm0',
##&nbsp;                                             'type' => 'Fibre Channel Storage Framework Comm'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T2',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'fcs1',
##&nbsp;                                             'type' => 'FC Adapter'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T2',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'fcnet1',
##&nbsp;                                             'type' => 'Fibre Channel Network Protocol Device'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T2',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'fscsi1',
##&nbsp;                                             'type' => 'FC SCSI I/O Controller Protocol Device'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T2-W0-L0',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'name' => 'sfwcomm1',
##&nbsp;                                             'type' => 'Fibre Channel Storage Framework Comm'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'L2cache0',
##&nbsp;                                             'type' => 'L2 Cache'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'mem0',
##&nbsp;                                             'type' => 'Memory'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'proc0',
##&nbsp;                                             'type' => 'Processor'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'name' => 'proc2',
##&nbsp;                                             'type' => 'Processor'
##&nbsp;                                           }
##&nbsp;                                         ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lscfg -p
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;INSTALLED RESOURCE LIST
##&nbsp;
##&nbsp;The following resources are installed on the machine.
##&nbsp;
##&nbsp;  Model Architecture: chrp
##&nbsp;  Model Implementation: Multiple Processor, PCI bus
##&nbsp;
##&nbsp;  sys0                                               System Object
##&nbsp;  sysplanar0                                         System Planar
##&nbsp;  vio0                                               Virtual I/O Bus
##&nbsp;  vsa1             U787F.001.DPM1FFD-P1-T2           LPAR Virtual Serial Adapter
##&nbsp;  vty1             U787F.001.DPM1FFD-P1-T2-L0        Asynchronous Terminal
##&nbsp;  vsa0             U787F.001.DPM1FFD-P1-T1           LPAR Virtual Serial Adapter
##&nbsp;  vty0             U787F.001.DPM1FFD-P1-T1-L0        Asynchronous Terminal
##&nbsp;  pci2             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;  pci10            U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;  ent2             U787F.001.DPM1FFD-P1-C1-T1        Gigabit Ethernet-SX PCI-X Adapter (14106802)
##&nbsp;  pci11            U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;  pci8             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;  usbhc0           U787F.001.DPM1FFD-P1              USB Host Controller (33103500)
##&nbsp;  usbhc1           U787F.001.DPM1FFD-P1              USB Host Controller (33103500)
##&nbsp;  usbhc2           U787F.001.DPM1FFD-P1              USB Enhanced Host Controller (3310e000)
##&nbsp;  pci9             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;  ent0             U787F.001.DPM1FFD-P1-T5           2-Port 10/100/1000 Base-TX PCI-X Adapter (14108902)
##&nbsp;  ent1             U787F.001.DPM1FFD-P1-T6           2-Port 10/100/1000 Base-TX PCI-X Adapter (14108902)
##&nbsp;  pci1             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;  pci7             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;  pci5             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;  pci3             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;  ide0             U787F.001.DPM1FFD-P1-T12          ATA/IDE Controller Device
##&nbsp;  cd0              U787F.001.DPM1FFD-P4-D2           IDE DVD-ROM Drive
##&nbsp;  pci4             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;  sisscsia0        U787F.001.DPM1FFD-P1              PCI-X Dual Channel Ultra320 SCSI Adapter
##&nbsp;  scsi0            U787F.001.DPM1FFD-P1-T10          PCI-X Dual Channel Ultra320 SCSI Adapter bus
##&nbsp;  hdisk0           U787F.001.DPM1FFD-P1-T10-L5-L0    16 Bit LVD SCSI Disk Drive (73400 MB)
##&nbsp;  hdisk1           U787F.001.DPM1FFD-P1-T10-L8-L0    16 Bit LVD SCSI Disk Drive (73400 MB)
##&nbsp;  ses0             U787F.001.DPM1FFD-P1-T10-L15-L0   SCSI Enclosure Services Device
##&nbsp;  scsi1            U787F.001.DPM1FFD-P1-T11          PCI-X Dual Channel Ultra320 SCSI Adapter bus
##&nbsp;  pci6             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;  pci0             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;  fcs0             U787F.001.DPM1FFD-P1-C4-T1        FC Adapter
##&nbsp;  fcnet0           U787F.001.DPM1FFD-P1-C4-T1        Fibre Channel Network Protocol Device
##&nbsp;  fscsi0           U787F.001.DPM1FFD-P1-C4-T1        FC SCSI I/O Controller Protocol Device
##&nbsp;  sfwcomm0         U787F.001.DPM1FFD-P1-C4-T1-W0-L0  Fibre Channel Storage Framework Comm
##&nbsp;  fcs1             U787F.001.DPM1FFD-P1-C4-T2        FC Adapter
##&nbsp;  fcnet1           U787F.001.DPM1FFD-P1-C4-T2        Fibre Channel Network Protocol Device
##&nbsp;  fscsi1           U787F.001.DPM1FFD-P1-C4-T2        FC SCSI I/O Controller Protocol Device
##&nbsp;  sfwcomm1         U787F.001.DPM1FFD-P1-C4-T2-W0-L0  Fibre Channel Storage Framework Comm
##&nbsp;  L2cache0                                           L2 Cache
##&nbsp;  mem0                                               Memory
##&nbsp;  proc0                                              Processor
##&nbsp;  proc2                                              Processor
##&nbsp;
##&nbsp;  PLATFORM SPECIFIC
##&nbsp;
##&nbsp;  Name:  IBM,9131-52A
##&nbsp;    Model:  IBM,9131-52A
##&nbsp;    Node:  /
##&nbsp;    Device Type:  chrp
##&nbsp;
##&nbsp;  Name:  openprom
##&nbsp;    Model:  IBM,SF240_259
##&nbsp;    Node:  openprom
##&nbsp;
##&nbsp;  Name:  interrupt-controller
##&nbsp;    Model:  IBM, Logical PowerPC-PIC, 00
##&nbsp;    Node:  interrupt-controller@3fe0f000000
##&nbsp;    Device Type:  PowerPC-External-Interrupt-Presentation
##&nbsp;
##&nbsp;  Name:  interrupt-controller
##&nbsp;    Model:  IBM,Logical PHB
##&nbsp;    Node:  interrupt-controller@3fe0000a400
##&nbsp;    Device Type:  PowerPC-LSI-Source
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  Name:  interrupt-controller
##&nbsp;    Model:  IBM,Logical PHB
##&nbsp;    Node:  interrupt-controller@3fe0000b400
##&nbsp;    Device Type:  PowerPC-LSI-Source
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  Name:  interrupt-controller
##&nbsp;    Model:  IBM,Logical PHB
##&nbsp;    Node:  interrupt-controller@3fe00008400
##&nbsp;    Device Type:  PowerPC-LSI-Source
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  Name:  pci
##&nbsp;    Model:  IBM,Logical_PHB
##&nbsp;    Node:  pci@800000020000002
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  Name:  pci
##&nbsp;    Model:  IBM,Logical_PHB
##&nbsp;    Node:  pci@800000020000003
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  Name:  pci
##&nbsp;    Model:  IBM,Logical_PHB
##&nbsp;    Node:  pci@800000020000004
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  Name:  vty
##&nbsp;    Node:  vty@30000000
##&nbsp;    Device Type:  serial
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1-T1
##&nbsp;
##&nbsp;  Name:  vty
##&nbsp;    Node:  vty@30000001
##&nbsp;    Device Type:  serial
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1-T2
##&nbsp;
##&nbsp;  Name:  fibre-channel
##&nbsp;    Model:  LP11000
##&nbsp;    Node:  fibre-channel@1
##&nbsp;    Device Type:  fcp
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1-C4-T1
##&nbsp;
##&nbsp;  Name:  fibre-channel
##&nbsp;    Model:  LP11000
##&nbsp;    Node:  fibre-channel@1,1
##&nbsp;    Device Type:  fcp
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1-C4-T2
##&nbsp;
##&nbsp;  Name:  pci
##&nbsp;    Node:  pci@2,3
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  Name:  pci
##&nbsp;    Node:  pci@2,4
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  Name:  pci
##&nbsp;    Node:  pci@2,2
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  Name:  pci
##&nbsp;    Node:  pci@2,6
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  Name:  pci
##&nbsp;    Node:  pci@2
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  Name:  pci
##&nbsp;    Node:  pci@2,3
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  Name:  pci
##&nbsp;    Node:  pci@2,4
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  Name:  pci
##&nbsp;    Node:  pci@2
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  Name:  pci
##&nbsp;    Node:  pci@2,2
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  Name:  ide
##&nbsp;    Node:  ide@1
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1-T12
##&nbsp;
##&nbsp;  Name:  pci1069,b166
##&nbsp;    Node:  pci1069,b166@1
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  Name:  usb
##&nbsp;    Node:  usb@1
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  Name:  usb
##&nbsp;    Node:  usb@1,1
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  Name:  usb
##&nbsp;    Node:  usb@1,2
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  Name:  ethernet
##&nbsp;    Node:  ethernet@1
##&nbsp;    Device Type:  network
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1-T5
##&nbsp;
##&nbsp;  Name:  ethernet
##&nbsp;    Node:  ethernet@1,1
##&nbsp;    Device Type:  network
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1-T6
##&nbsp;
##&nbsp;  Name:  ethernet
##&nbsp;    Node:  ethernet@1
##&nbsp;    Device Type:  network
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1-C1-T1
##&nbsp;
##&nbsp;  Name:  disk
##&nbsp;    Node:  disk@0
##&nbsp;    Device Type:  block
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P4-D2
##&nbsp;
##&nbsp;  Name:  scsi
##&nbsp;    Node:  scsi@0
##&nbsp;    Device Type:  scsi-2
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1-T10
##&nbsp;
##&nbsp;  Name:  scsi
##&nbsp;    Node:  scsi@1
##&nbsp;    Device Type:  scsi-2
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1-T11
##&nbsp;
##&nbsp;  Name:  hub
##&nbsp;    Node:  hub@1
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1-T7
##&nbsp;
##&nbsp;  Name:  hub
##&nbsp;    Node:  hub@1
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1-T8
##&nbsp;
##&nbsp;  Name:  hub
##&nbsp;    Node:  hub@1
##&nbsp;    Physical Location: U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'model_architecture' => ' chrp',
##&nbsp;            'model_implementation' => ' Multiple Processor, PCI bus',
##&nbsp;            'installed_resource_list' => [
##&nbsp;                                           {
##&nbsp;                                             'name' => 'sysplanar0',
##&nbsp;                                             'type' => 'System Planar'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'name' => 'vio0',
##&nbsp;                                             'type' => 'Virtual I/O Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T2',
##&nbsp;                                             'name' => 'vsa1',
##&nbsp;                                             'type' => 'LPAR Virtual Serial Adapter'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T2-L0',
##&nbsp;                                             'name' => 'vty1',
##&nbsp;                                             'type' => 'Asynchronous Terminal'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T1',
##&nbsp;                                             'name' => 'vsa0',
##&nbsp;                                             'type' => 'LPAR Virtual Serial Adapter'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T1-L0',
##&nbsp;                                             'name' => 'vty0',
##&nbsp;                                             'type' => 'Asynchronous Terminal'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci2',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci10',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C1-T1',
##&nbsp;                                             'name' => 'ent2',
##&nbsp;                                             'type' => 'Gigabit Ethernet-SX PCI-X Adapter (14106802)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci11',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci8',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'usbhc0',
##&nbsp;                                             'type' => 'USB Host Controller (33103500)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'usbhc1',
##&nbsp;                                             'type' => 'USB Host Controller (33103500)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'usbhc2',
##&nbsp;                                             'type' => 'USB Enhanced Host Controller (3310e000)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci9',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T5',
##&nbsp;                                             'name' => 'ent0',
##&nbsp;                                             'type' => '2-Port 10/100/1000 Base-TX PCI-X Adapter (14108902)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T6',
##&nbsp;                                             'name' => 'ent1',
##&nbsp;                                             'type' => '2-Port 10/100/1000 Base-TX PCI-X Adapter (14108902)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci1',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci7',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci5',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci3',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T12',
##&nbsp;                                             'name' => 'ide0',
##&nbsp;                                             'type' => 'ATA/IDE Controller Device'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P4-D2',
##&nbsp;                                             'name' => 'cd0',
##&nbsp;                                             'type' => 'IDE DVD-ROM Drive'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci4',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'sisscsia0',
##&nbsp;                                             'type' => 'PCI-X Dual Channel Ultra320 SCSI Adapter'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T10',
##&nbsp;                                             'name' => 'scsi0',
##&nbsp;                                             'type' => 'PCI-X Dual Channel Ultra320 SCSI Adapter bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T10-L5-L0',
##&nbsp;                                             'name' => 'hdisk0',
##&nbsp;                                             'type' => '16 Bit LVD SCSI Disk Drive (73400 MB)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T10-L8-L0',
##&nbsp;                                             'name' => 'hdisk1',
##&nbsp;                                             'type' => '16 Bit LVD SCSI Disk Drive (73400 MB)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T10-L15-L0',
##&nbsp;                                             'name' => 'ses0',
##&nbsp;                                             'type' => 'SCSI Enclosure Services Device'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T11',
##&nbsp;                                             'name' => 'scsi1',
##&nbsp;                                             'type' => 'PCI-X Dual Channel Ultra320 SCSI Adapter bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci6',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci0',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T1',
##&nbsp;                                             'name' => 'fcs0',
##&nbsp;                                             'type' => 'FC Adapter'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T1',
##&nbsp;                                             'name' => 'fcnet0',
##&nbsp;                                             'type' => 'Fibre Channel Network Protocol Device'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T1',
##&nbsp;                                             'name' => 'fscsi0',
##&nbsp;                                             'type' => 'FC SCSI I/O Controller Protocol Device'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T1-W0-L0',
##&nbsp;                                             'name' => 'sfwcomm0',
##&nbsp;                                             'type' => 'Fibre Channel Storage Framework Comm'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T2',
##&nbsp;                                             'name' => 'fcs1',
##&nbsp;                                             'type' => 'FC Adapter'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T2',
##&nbsp;                                             'name' => 'fcnet1',
##&nbsp;                                             'type' => 'Fibre Channel Network Protocol Device'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T2',
##&nbsp;                                             'name' => 'fscsi1',
##&nbsp;                                             'type' => 'FC SCSI I/O Controller Protocol Device'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T2-W0-L0',
##&nbsp;                                             'name' => 'sfwcomm1',
##&nbsp;                                             'type' => 'Fibre Channel Storage Framework Comm'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'name' => 'L2cache0',
##&nbsp;                                             'type' => 'L2 Cache'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'name' => 'mem0',
##&nbsp;                                             'type' => 'Memory'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'name' => 'proc0',
##&nbsp;                                             'type' => 'Processor'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'name' => 'proc2',
##&nbsp;                                             'type' => 'Processor'
##&nbsp;                                           }
##&nbsp;                                         ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'platform_specific' => [
##&nbsp;                                     {
##&nbsp;                                       'name' => 'hub',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'hub@1'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'hub',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1-T8',
##&nbsp;                                       'node' => 'hub@1'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'hub',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1-T7',
##&nbsp;                                       'node' => 'hub@1'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'device_type' => 'scsi-2',
##&nbsp;                                       'name' => 'scsi',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1-T11',
##&nbsp;                                       'node' => 'scsi@1'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'device_type' => 'scsi-2',
##&nbsp;                                       'name' => 'scsi',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1-T10',
##&nbsp;                                       'node' => 'scsi@0'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'device_type' => 'block',
##&nbsp;                                       'name' => 'disk',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P4-D2',
##&nbsp;                                       'node' => 'disk@0'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'device_type' => 'network',
##&nbsp;                                       'name' => 'ethernet',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1-C1-T1',
##&nbsp;                                       'node' => 'ethernet@1'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'device_type' => 'network',
##&nbsp;                                       'name' => 'ethernet',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1-T6',
##&nbsp;                                       'node' => 'ethernet@1,1'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'device_type' => 'network',
##&nbsp;                                       'name' => 'ethernet',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1-T5',
##&nbsp;                                       'node' => 'ethernet@1'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'usb',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'usb@1,2'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'usb',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'usb@1,1'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'usb',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'usb@1'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'pci1069,b166',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'pci1069,b166@1'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'ide',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1-T12',
##&nbsp;                                       'node' => 'ide@1'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'pci',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'pci@2,2'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'pci',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'pci@2'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'pci',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'pci@2,4'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'pci',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'pci@2,3'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'pci',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'pci@2'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'pci',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'pci@2,6'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'pci',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'pci@2,2'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'pci',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'pci@2,4'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'pci',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'pci@2,3'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'device_type' => 'fcp',
##&nbsp;                                       'name' => 'fibre-channel',
##&nbsp;                                       'model' => 'LP11000',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1-C4-T2',
##&nbsp;                                       'node' => 'fibre-channel@1,1'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'device_type' => 'fcp',
##&nbsp;                                       'name' => 'fibre-channel',
##&nbsp;                                       'model' => 'LP11000',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1-C4-T1',
##&nbsp;                                       'node' => 'fibre-channel@1'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'device_type' => 'serial',
##&nbsp;                                       'name' => 'vty',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1-T2',
##&nbsp;                                       'node' => 'vty@30000001'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'device_type' => 'serial',
##&nbsp;                                       'name' => 'vty',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1-T1',
##&nbsp;                                       'node' => 'vty@30000000'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'pci',
##&nbsp;                                       'model' => 'IBM,Logical_PHB',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'pci@800000020000004'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'pci',
##&nbsp;                                       'model' => 'IBM,Logical_PHB',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'pci@800000020000003'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'pci',
##&nbsp;                                       'model' => 'IBM,Logical_PHB',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'pci@800000020000002'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'device_type' => 'PowerPC-LSI-Source',
##&nbsp;                                       'name' => 'interrupt-controller',
##&nbsp;                                       'model' => 'IBM,Logical PHB',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'interrupt-controller@3fe00008400'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'device_type' => 'PowerPC-LSI-Source',
##&nbsp;                                       'name' => 'interrupt-controller',
##&nbsp;                                       'model' => 'IBM,Logical PHB',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'interrupt-controller@3fe0000b400'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'device_type' => 'PowerPC-LSI-Source',
##&nbsp;                                       'name' => 'interrupt-controller',
##&nbsp;                                       'model' => 'IBM,Logical PHB',
##&nbsp;                                       'physical_location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                       'node' => 'interrupt-controller@3fe0000a400'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'device_type' => 'PowerPC-External-Interrupt-Presentation',
##&nbsp;                                       'name' => 'interrupt-controller',
##&nbsp;                                       'model' => 'IBM, Logical PowerPC-PIC, 00',
##&nbsp;                                       'node' => 'interrupt-controller@3fe0f000000'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'name' => 'openprom',
##&nbsp;                                       'model' => 'IBM,SF240_259',
##&nbsp;                                       'node' => 'openprom'
##&nbsp;                                     },
##&nbsp;                                     {
##&nbsp;                                       'device_type' => 'chrp',
##&nbsp;                                       'name' => 'IBM,9131-52A',
##&nbsp;                                       'model' => 'IBM,9131-52A',
##&nbsp;                                       'node' => '/'
##&nbsp;                                     }
##&nbsp;                                   ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lscfg -l cd0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  cd0              U787F.001.DPM1FFD-P4-D2  IDE DVD-ROM Drive
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'name' => 'cd0',
##&nbsp;            'physical_location' => 'U787F.001.DPM1FFD-P4-D2',
##&nbsp;            'type' => 'IDE DVD-ROM Drive'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = lscfg -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;INSTALLED RESOURCE LIST WITH VPD
##&nbsp;
##&nbsp;The following resources are installed on your machine.
##&nbsp;
##&nbsp;  Model Architecture: chrp
##&nbsp;  Model Implementation: Multiple Processor, PCI bus
##&nbsp;
##&nbsp;  sys0                                               System Object
##&nbsp;  sysplanar0                                         System Planar
##&nbsp;  vio0                                               Virtual I/O Bus
##&nbsp;  vsa1             U787F.001.DPM1FFD-P1-T2           LPAR Virtual Serial Adapter
##&nbsp;
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1-T2
##&nbsp;
##&nbsp;  vty1             U787F.001.DPM1FFD-P1-T2-L0        Asynchronous Terminal
##&nbsp;  vsa0             U787F.001.DPM1FFD-P1-T1           LPAR Virtual Serial Adapter
##&nbsp;
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1-T1
##&nbsp;
##&nbsp;  vty0             U787F.001.DPM1FFD-P1-T1-L0        Asynchronous Terminal
##&nbsp;  pci2             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  pci10            U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  ent2             U787F.001.DPM1FFD-P1-C1-T1        Gigabit Ethernet-SX PCI-X Adapter (14106802)
##&nbsp;
##&nbsp;      Gigabit Ethernet-SX PCI-X Adapter:
##&nbsp;        Part Number.................03N6981
##&nbsp;        FRU Number..................03N6981
##&nbsp;        EC Level....................H14079
##&nbsp;        Manufacture ID..............YL1021
##&nbsp;        Network Address.............001125BD87BD
##&nbsp;        ROM Level.(alterable).......GOL021
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1-C1-T1
##&nbsp;
##&nbsp;  pci11            U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  pci8             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  usbhc0           U787F.001.DPM1FFD-P1              USB Host Controller (33103500)
##&nbsp;
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  usbhc1           U787F.001.DPM1FFD-P1              USB Host Controller (33103500)
##&nbsp;
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  usbhc2           U787F.001.DPM1FFD-P1              USB Enhanced Host Controller (3310e000)
##&nbsp;
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  pci9             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  ent0             U787F.001.DPM1FFD-P1-T5           2-Port 10/100/1000 Base-TX PCI-X Adapter (14108902)
##&nbsp;
##&nbsp;      2-Port 10/100/1000 Base-TX PCI-X Adapter:
##&nbsp;        Network Address.............00145E965898
##&nbsp;        ROM Level.(alterable).......DV0210
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1-T5
##&nbsp;
##&nbsp;  ent1             U787F.001.DPM1FFD-P1-T6           2-Port 10/100/1000 Base-TX PCI-X Adapter (14108902)
##&nbsp;
##&nbsp;      2-Port 10/100/1000 Base-TX PCI-X Adapter:
##&nbsp;        Network Address.............00145E965899
##&nbsp;        ROM Level.(alterable).......DV0210
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1-T6
##&nbsp;
##&nbsp;  pci1             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  pci7             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  pci5             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  pci3             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  ide0             U787F.001.DPM1FFD-P1-T12          ATA/IDE Controller Device
##&nbsp;
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1-T12
##&nbsp;
##&nbsp;  cd0              U787F.001.DPM1FFD-P4-D2           IDE DVD-ROM Drive
##&nbsp;
##&nbsp;        Manufacturer................IBM
##&nbsp;        Machine Type and Model......DROM00205L1   H0
##&nbsp;        ROS Level and ID............P533
##&nbsp;        Device Specific.(Z0)........058002028F000030
##&nbsp;        Brand.......................H0
##&nbsp;
##&nbsp;  pci4             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  sisscsia0        U787F.001.DPM1FFD-P1              PCI-X Dual Channel Ultra320 SCSI Adapter
##&nbsp;
##&nbsp;      PCI-X Dual Channel Ultra320 SCSI Adapter:
##&nbsp;        Part Number.................00P0000
##&nbsp;        FRU Number..................00P0000
##&nbsp;        Serial Number...............YL3000000001
##&nbsp;        Manufacture ID..............0000
##&nbsp;        EC Level....................0
##&nbsp;        ROM Level.(alterable).......0510006d
##&nbsp;        Product Specific.(Z0).......573E
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  scsi0            U787F.001.DPM1FFD-P1-T10          PCI-X Dual Channel Ultra320 SCSI Adapter bus
##&nbsp;
##&nbsp;        ROM Level.(alterable).......0510006d
##&nbsp;        Customer Card ID Number.....573E
##&nbsp;
##&nbsp;  hdisk0           U787F.001.DPM1FFD-P1-T10-L5-L0    16 Bit LVD SCSI Disk Drive (73400 MB)
##&nbsp;
##&nbsp;        Manufacturer................IBM   H0
##&nbsp;        Machine Type and Model......HUS103073FL3800
##&nbsp;        FRU Number..................03N5262
##&nbsp;        ROS Level and ID............52505152
##&nbsp;        Serial Number...............0057694B
##&nbsp;        EC Level....................H17923D
##&nbsp;        Part Number.................26K5573
##&nbsp;        Device Specific.(Z0)........000004129F00013E
##&nbsp;        Device Specific.(Z1)........RPQR
##&nbsp;        Device Specific.(Z2)........0068
##&nbsp;        Device Specific.(Z3)........06199
##&nbsp;        Device Specific.(Z4)........0001
##&nbsp;        Device Specific.(Z5)........22
##&nbsp;        Device Specific.(Z6)........H17923D
##&nbsp;        Brand.......................H0
##&nbsp;
##&nbsp;  hdisk1           U787F.001.DPM1FFD-P1-T10-L8-L0    16 Bit LVD SCSI Disk Drive (73400 MB)
##&nbsp;
##&nbsp;        Manufacturer................IBM   H0
##&nbsp;        Machine Type and Model......HUS103073FL3800
##&nbsp;        FRU Number..................03N5262
##&nbsp;        ROS Level and ID............52505152
##&nbsp;        Serial Number...............00577E10
##&nbsp;        EC Level....................H17923D
##&nbsp;        Part Number.................26K5573
##&nbsp;        Device Specific.(Z0)........000004129F00013E
##&nbsp;        Device Specific.(Z1)........RPQR
##&nbsp;        Device Specific.(Z2)........0068
##&nbsp;        Device Specific.(Z3)........06201
##&nbsp;        Device Specific.(Z4)........0001
##&nbsp;        Device Specific.(Z5)........22
##&nbsp;        Device Specific.(Z6)........H17923D
##&nbsp;        Brand.......................H0
##&nbsp;
##&nbsp;  ses0             U787F.001.DPM1FFD-P1-T10-L15-L0   SCSI Enclosure Services Device
##&nbsp;
##&nbsp;        Manufacturer................IBM
##&nbsp;        Machine Type and Model......VSBPD4E1  U4SCSI
##&nbsp;        ROS Level and ID............6000
##&nbsp;        Serial Number...............2621718F
##&nbsp;        Device Specific.(Z0)........0D0002022F004000
##&nbsp;        FRU Number..................03N6000
##&nbsp;        FRU Label...................P3
##&nbsp;        Frame Serial Number.........787F-001 DPM1FFD
##&nbsp;
##&nbsp;  scsi1            U787F.001.DPM1FFD-P1-T11          PCI-X Dual Channel Ultra320 SCSI Adapter bus
##&nbsp;
##&nbsp;        ROM Level.(alterable).......0510006d
##&nbsp;        Customer Card ID Number.....573E
##&nbsp;
##&nbsp;  pci6             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  pci0             U787F.001.DPM1FFD-P1              PCI Bus
##&nbsp;
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1
##&nbsp;
##&nbsp;  fcs0             U787F.001.DPM1FFD-P1-C4-T1        FC Adapter
##&nbsp;
##&nbsp;        Part Number.................03N5029
##&nbsp;        EC Level....................A
##&nbsp;        Serial Number...............1B62804868
##&nbsp;        Manufacturer................001B
##&nbsp;        Customer Card ID Number.....5759
##&nbsp;        FRU Number.................. 03N5029
##&nbsp;        Device Specific.(ZM)........3
##&nbsp;        Network Address.............10000000C9538A20
##&nbsp;        ROS Level and ID............02C82138
##&nbsp;        Device Specific.(Z0)........1036406D
##&nbsp;        Device Specific.(Z1)........00000000
##&nbsp;        Device Specific.(Z2)........00000000
##&nbsp;        Device Specific.(Z3)........03000909
##&nbsp;        Device Specific.(Z4)........FFC01159
##&nbsp;        Device Specific.(Z5)........02C82138
##&nbsp;        Device Specific.(Z6)........06C12138
##&nbsp;        Device Specific.(Z7)........07C12138
##&nbsp;        Device Specific.(Z8)........20000000C9538A20
##&nbsp;        Device Specific.(Z9)........BS2.10X8
##&nbsp;        Device Specific.(ZA)........B1F2.10X8
##&nbsp;        Device Specific.(ZB)........B2F2.10X8
##&nbsp;        Device Specific.(ZC)........00000000
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1-C4-T1
##&nbsp;
##&nbsp;  fcnet0           U787F.001.DPM1FFD-P1-C4-T1        Fibre Channel Network Protocol Device
##&nbsp;  fscsi0           U787F.001.DPM1FFD-P1-C4-T1        FC SCSI I/O Controller Protocol Device
##&nbsp;  sfwcomm0         U787F.001.DPM1FFD-P1-C4-T1-W0-L0  Fibre Channel Storage Framework Comm
##&nbsp;  fcs1             U787F.001.DPM1FFD-P1-C4-T2        FC Adapter
##&nbsp;
##&nbsp;        Part Number.................03N5029
##&nbsp;        EC Level....................A
##&nbsp;        Serial Number...............1B62804868
##&nbsp;        Manufacturer................001B
##&nbsp;        Customer Card ID Number.....5759
##&nbsp;        FRU Number.................. 03N5029
##&nbsp;        Device Specific.(ZM)........3
##&nbsp;        Network Address.............10000000C9538A21
##&nbsp;        ROS Level and ID............02C82138
##&nbsp;        Device Specific.(Z0)........1036406D
##&nbsp;        Device Specific.(Z1)........00000000
##&nbsp;        Device Specific.(Z2)........00000000
##&nbsp;        Device Specific.(Z3)........03000909
##&nbsp;        Device Specific.(Z4)........FFC01159
##&nbsp;        Device Specific.(Z5)........02C82138
##&nbsp;        Device Specific.(Z6)........06C12138
##&nbsp;        Device Specific.(Z7)........07C12138
##&nbsp;        Device Specific.(Z8)........20000000C9538A21
##&nbsp;        Device Specific.(Z9)........BS2.10X8
##&nbsp;        Device Specific.(ZA)........B1F2.10X8
##&nbsp;        Device Specific.(ZB)........B2F2.10X8
##&nbsp;        Device Specific.(ZC)........00000000
##&nbsp;        Hardware Location Code......U787F.001.DPM1FFD-P1-C4-T2
##&nbsp;
##&nbsp;  fcnet1           U787F.001.DPM1FFD-P1-C4-T2        Fibre Channel Network Protocol Device
##&nbsp;  fscsi1           U787F.001.DPM1FFD-P1-C4-T2        FC SCSI I/O Controller Protocol Device
##&nbsp;  sfwcomm1         U787F.001.DPM1FFD-P1-C4-T2-W0-L0  Fibre Channel Storage Framework Comm
##&nbsp;  L2cache0                                           L2 Cache
##&nbsp;  mem0                                               Memory
##&nbsp;  proc0                                              Processor
##&nbsp;  proc2                                              Processor
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'model_architecture' => ' chrp',
##&nbsp;            'model_implementation' => ' Multiple Processor, PCI bus',
##&nbsp;            'installed_resource_list' => [
##&nbsp;                                           {
##&nbsp;                                             'name' => 'sys0',
##&nbsp;                                             'type' => 'System Object'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'name' => 'sysplanar0',
##&nbsp;                                             'type' => 'System Planar'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'name' => 'vio0',
##&nbsp;                                             'type' => 'Virtual I/O Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T2',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1-T2',
##&nbsp;                                             'name' => 'vsa1',
##&nbsp;                                             'type' => 'LPAR Virtual Serial Adapter'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T2-L0',
##&nbsp;                                             'name' => 'vty1',
##&nbsp;                                             'type' => 'Asynchronous Terminal'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T1',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1-T1',
##&nbsp;                                             'name' => 'vsa0',
##&nbsp;                                             'type' => 'LPAR Virtual Serial Adapter'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T1-L0',
##&nbsp;                                             'name' => 'vty0',
##&nbsp;                                             'type' => 'Asynchronous Terminal'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci2',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci10',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C1-T1',
##&nbsp;                                             'name' => 'ent2',
##&nbsp;                                             'type' => 'Gigabit Ethernet-SX PCI-X Adapter (14106802)',
##&nbsp;                                             'gigabit_ethernet-sx_pci-x_adapter' => [
##&nbsp;                                                                                      {
##&nbsp;                                                                                        'part_number' => '03N6981',
##&nbsp;                                                                                        'hardware_location_code' => 'U787F.001.DPM1FFD-P1-C1-T1',
##&nbsp;                                                                                        'rom_level.alterable' => 'GOL021',
##&nbsp;                                                                                        'manufacture_id' => 'YL1021',
##&nbsp;                                                                                        'fru_number' => '03N6981',
##&nbsp;                                                                                        'network_address' => '001125BD87BD',
##&nbsp;                                                                                        'ec_level' => 'H14079'
##&nbsp;                                                                                      }
##&nbsp;                                                                                    ]
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci11',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci8',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'usbhc0',
##&nbsp;                                             'type' => 'USB Host Controller (33103500)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'usbhc1',
##&nbsp;                                             'type' => 'USB Host Controller (33103500)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'usbhc2',
##&nbsp;                                             'type' => 'USB Enhanced Host Controller (3310e000)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci9',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             '2-port_10/100/1000_base-tx_pci-x_adapter' => [
##&nbsp;                                                                                             {
##&nbsp;                                                                                               'hardware_location_code' => 'U787F.001.DPM1FFD-P1-T5',
##&nbsp;                                                                                               'rom_level.alterable' => 'DV0210',
##&nbsp;                                                                                               'network_address' => '00145E965898'
##&nbsp;                                                                                             }
##&nbsp;                                                                                           ],
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T5',
##&nbsp;                                             'name' => 'ent0',
##&nbsp;                                             'type' => '2-Port 10/100/1000 Base-TX PCI-X Adapter (14108902)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             '2-port_10/100/1000_base-tx_pci-x_adapter' => [
##&nbsp;                                                                                             {
##&nbsp;                                                                                               'hardware_location_code' => 'U787F.001.DPM1FFD-P1-T6',
##&nbsp;                                                                                               'rom_level.alterable' => 'DV0210',
##&nbsp;                                                                                               'network_address' => '00145E965899'
##&nbsp;                                                                                             }
##&nbsp;                                                                                           ],
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T6',
##&nbsp;                                             'name' => 'ent1',
##&nbsp;                                             'type' => '2-Port 10/100/1000 Base-TX PCI-X Adapter (14108902)'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci1',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci7',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci5',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci3',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T12',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1-T12',
##&nbsp;                                             'name' => 'ide0',
##&nbsp;                                             'type' => 'ATA/IDE Controller Device'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_specific.z0' => '058002028F000030',
##&nbsp;                                             'ros_level_and_id' => 'P533',
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P4-D2',
##&nbsp;                                             'machine_type_and_model' => 'DROM00205L1   H0',
##&nbsp;                                             'name' => 'cd0',
##&nbsp;                                             'brand' => 'H0',
##&nbsp;                                             'manufacturer' => 'IBM',
##&nbsp;                                             'type' => 'IDE DVD-ROM Drive'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci4',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'pci-x_dual_channel_ultra320_scsi_adapter' => [
##&nbsp;                                                                                             {
##&nbsp;                                                                                               'serial_number' => 'YL3000000001',
##&nbsp;                                                                                               'part_number' => '00P0000',
##&nbsp;                                                                                               'hardware_location_code' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                                                                               'product_specific.z0' => '573E',
##&nbsp;                                                                                               'rom_level.alterable' => '0510006d',
##&nbsp;                                                                                               'manufacture_id' => '0000',
##&nbsp;                                                                                               'fru_number' => '00P0000',
##&nbsp;                                                                                               'ec_level' => '0'
##&nbsp;                                                                                             }
##&nbsp;                                                                                           ],
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'sisscsia0',
##&nbsp;                                             'type' => 'PCI-X Dual Channel Ultra320 SCSI Adapter'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'customer_card_id_number' => '573E',
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T10',
##&nbsp;                                             'rom_level.alterable' => '0510006d',
##&nbsp;                                             'name' => 'scsi0',
##&nbsp;                                             'type' => 'PCI-X Dual Channel Ultra320 SCSI Adapter bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_specific.z0' => '000004129F00013E',
##&nbsp;                                             'serial_number' => '0057694B',
##&nbsp;                                             'ros_level_and_id' => '52505152',
##&nbsp;                                             'brand' => 'H0',
##&nbsp;                                             'device_specific.z5' => '22',
##&nbsp;                                             'device_specific.z6' => 'H17923D',
##&nbsp;                                             'device_specific.z1' => 'RPQR',
##&nbsp;                                             'fru_number' => '03N5262',
##&nbsp;                                             'ec_level' => 'H17923D',
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T10-L5-L0',
##&nbsp;                                             'device_specific.z4' => '0001',
##&nbsp;                                             'machine_type_and_model' => 'HUS103073FL3800',
##&nbsp;                                             'name' => 'hdisk0',
##&nbsp;                                             'part_number' => '26K5573',
##&nbsp;                                             'manufacturer' => 'IBM   H0',
##&nbsp;                                             'type' => '16 Bit LVD SCSI Disk Drive (73400 MB)',
##&nbsp;                                             'device_specific.z2' => '0068',
##&nbsp;                                             'device_specific.z3' => '06199'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_specific.z0' => '000004129F00013E',
##&nbsp;                                             'serial_number' => '00577E10',
##&nbsp;                                             'ros_level_and_id' => '52505152',
##&nbsp;                                             'brand' => 'H0',
##&nbsp;                                             'device_specific.z5' => '22',
##&nbsp;                                             'device_specific.z6' => 'H17923D',
##&nbsp;                                             'device_specific.z1' => 'RPQR',
##&nbsp;                                             'fru_number' => '03N5262',
##&nbsp;                                             'ec_level' => 'H17923D',
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T10-L8-L0',
##&nbsp;                                             'device_specific.z4' => '0001',
##&nbsp;                                             'machine_type_and_model' => 'HUS103073FL3800',
##&nbsp;                                             'name' => 'hdisk1',
##&nbsp;                                             'part_number' => '26K5573',
##&nbsp;                                             'manufacturer' => 'IBM   H0',
##&nbsp;                                             'type' => '16 Bit LVD SCSI Disk Drive (73400 MB)',
##&nbsp;                                             'device_specific.z2' => '0068',
##&nbsp;                                             'device_specific.z3' => '06201'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_specific.z0' => '0D0002022F004000',
##&nbsp;                                             'serial_number' => '2621718F',
##&nbsp;                                             'ros_level_and_id' => '6000',
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T10-L15-L0',
##&nbsp;                                             'machine_type_and_model' => 'VSBPD4E1  U4SCSI',
##&nbsp;                                             'name' => 'ses0',
##&nbsp;                                             'fru_label' => 'P3',
##&nbsp;                                             'manufacturer' => 'IBM',
##&nbsp;                                             'fru_number' => '03N6000',
##&nbsp;                                             'type' => 'SCSI Enclosure Services Device',
##&nbsp;                                             'frame_serial_number' => '787F-001 DPM1FFD'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'customer_card_id_number' => '573E',
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-T11',
##&nbsp;                                             'rom_level.alterable' => '0510006d',
##&nbsp;                                             'name' => 'scsi1',
##&nbsp;                                             'type' => 'PCI-X Dual Channel Ultra320 SCSI Adapter bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci6',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1',
##&nbsp;                                             'name' => 'pci0',
##&nbsp;                                             'type' => 'PCI Bus'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_specific.z0' => '1036406D',
##&nbsp;                                             'ros_level_and_id' => '02C82138',
##&nbsp;                                             'customer_card_id_number' => '5759',
##&nbsp;                                             'serial_number' => '1B62804868',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1-C4-T1',
##&nbsp;                                             'device_specific.z5' => '02C82138',
##&nbsp;                                             'device_specific.zb' => 'B2F2.10X8',
##&nbsp;                                             'device_specific.z6' => '06C12138',
##&nbsp;                                             'device_specific.z1' => '00000000',
##&nbsp;                                             'device_specific.z9' => 'BS2.10X8',
##&nbsp;                                             'device_specific.z7' => '07C12138',
##&nbsp;                                             'device_specific.zm' => '3',
##&nbsp;                                             'fru_number' => ' 03N5029',
##&nbsp;                                             'ec_level' => 'A',
##&nbsp;                                             'device_specific.zc' => '00000000',
##&nbsp;                                             'device_specific.z4' => 'FFC01159',
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T1',
##&nbsp;                                             'name' => 'fcs0',
##&nbsp;                                             'network_address' => '10000000C9538A20',
##&nbsp;                                             'device_specific.z8' => '20000000C9538A20',
##&nbsp;                                             'part_number' => '03N5029',
##&nbsp;                                             'manufacturer' => '001B',
##&nbsp;                                             'type' => 'FC Adapter',
##&nbsp;                                             'device_specific.z2' => '00000000',
##&nbsp;                                             'device_specific.za' => 'B1F2.10X8',
##&nbsp;                                             'device_specific.z3' => '03000909'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T1',
##&nbsp;                                             'name' => 'fcnet0',
##&nbsp;                                             'type' => 'Fibre Channel Network Protocol Device'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T1',
##&nbsp;                                             'name' => 'fscsi0',
##&nbsp;                                             'type' => 'FC SCSI I/O Controller Protocol Device'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T1-W0-L0',
##&nbsp;                                             'name' => 'sfwcomm0',
##&nbsp;                                             'type' => 'Fibre Channel Storage Framework Comm'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_specific.z0' => '1036406D',
##&nbsp;                                             'ros_level_and_id' => '02C82138',
##&nbsp;                                             'customer_card_id_number' => '5759',
##&nbsp;                                             'serial_number' => '1B62804868',
##&nbsp;                                             'hardware_location_code' => 'U787F.001.DPM1FFD-P1-C4-T2',
##&nbsp;                                             'device_specific.z5' => '02C82138',
##&nbsp;                                             'device_specific.zb' => 'B2F2.10X8',
##&nbsp;                                             'device_specific.z6' => '06C12138',
##&nbsp;                                             'device_specific.z1' => '00000000',
##&nbsp;                                             'device_specific.z9' => 'BS2.10X8',
##&nbsp;                                             'device_specific.z7' => '07C12138',
##&nbsp;                                             'device_specific.zm' => '3',
##&nbsp;                                             'fru_number' => ' 03N5029',
##&nbsp;                                             'ec_level' => 'A',
##&nbsp;                                             'device_specific.zc' => '00000000',
##&nbsp;                                             'device_specific.z4' => 'FFC01159',
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T2',
##&nbsp;                                             'name' => 'fcs1',
##&nbsp;                                             'network_address' => '10000000C9538A21',
##&nbsp;                                             'device_specific.z8' => '20000000C9538A21',
##&nbsp;                                             'part_number' => '03N5029',
##&nbsp;                                             'manufacturer' => '001B',
##&nbsp;                                             'type' => 'FC Adapter',
##&nbsp;                                             'device_specific.z2' => '00000000',
##&nbsp;                                             'device_specific.za' => 'B1F2.10X8',
##&nbsp;                                             'device_specific.z3' => '03000909'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T2',
##&nbsp;                                             'name' => 'fcnet1',
##&nbsp;                                             'type' => 'Fibre Channel Network Protocol Device'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T2',
##&nbsp;                                             'name' => 'fscsi1',
##&nbsp;                                             'type' => 'FC SCSI I/O Controller Protocol Device'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'location' => 'U787F.001.DPM1FFD-P1-C4-T2-W0-L0',
##&nbsp;                                             'name' => 'sfwcomm1',
##&nbsp;                                             'type' => 'Fibre Channel Storage Framework Comm'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'name' => 'L2cache0',
##&nbsp;                                             'type' => 'L2 Cache'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'name' => 'mem0',
##&nbsp;                                             'type' => 'Memory'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'name' => 'proc0',
##&nbsp;                                             'type' => 'Processor'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'name' => 'proc2',
##&nbsp;                                             'type' => 'Processor'
##&nbsp;                                           }
##&nbsp;                                         ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub lscfg {
    my %args          = @_;
    my $result_arrayp = $args{response_object};
    my $output        = $result_arrayp->get_processed_output();
    my $cmd           = $result_arrayp->get_command();
    _check_error( raw_output => $output, response_object => $result_arrayp );
    my ( @result_array, @values );
    my @lines = get_lines(
        raw_output      => $output,
        first_line      => 'Model Architecture:\s+',
        response_object => $result_arrayp
    );
    my $count = 0;
    my @heads;
    my $linex = shift(@lines);
    my @line1 = split( ':', $linex );
    $result_array[0]->{ to_key_name( item => $line1[0] ) } = $line1[1];

    my $liney = shift(@lines);
    my @line2 = split( ':', $liney );
    $result_array[0]->{ to_key_name( item => $line2[0] ) } = $line2[1];
    my %hash;
    $hash{'+'} = "added resource";
    $hash{'-'} = "deleted resource";
    $hash{'*'} = "diagnostic support not available";

    my $j = 0;
    if ( $cmd !~ /-p|-l\s.*|-v/ ) {
        while ( $j <= $#lines ) {
            @values = split( / {2,}/, $lines[$j] );
            if ( $cmd =~ /-s/ ) {

                my $k = $j + 1;
                push( @values, $lines[$k] );
                shift(@lines);
            }
            my $x = 1;
            @heads = ("name");
            my @values1 = split( / {1,}/, $values[0] );
            shift(@values);
            unshift( @values, $values1[1] );
            unshift( @heads,  'status' );
            unshift( @values, $hash{ $values1[0] } );
            $x++;

            if ( $values[$x] =~ /U\d+\.*/ ) {
                push( @heads, 'location' );
            }
            push( @heads, 'type' );

            for ( my $i = 0 ; $i <= $#heads ; $i++ ) {

                $result_array[0]->{installed_resource_list}->[$count]
                  ->{ to_key_name( item => $heads[$i] ) } = $values[$i];
            }
            $count++;
            $j++;
        }
        return \@result_array;
    }

    if ( $cmd =~ /-l\s.*/ ) {
        my @line = get_lines(
            raw_output      => $output,
            response_object => $result_arrayp
        );
        my @values1 = split( / {2,}/, $line[0] );
        my @result_new;
        $result_new[0]->{name}              = $values1[0];
        $result_new[0]->{type}              = $values1[2];
        $result_new[0]->{physical_location} = $values1[1];
        return \@result_new;

    }
    if ( $cmd =~ /-p/ ) {
        my @result1;
        my $j = 0;
        $result1[0]->{ to_key_name( item => $line1[0] ) } = $line1[1];
        $result1[0]->{ to_key_name( item => $line2[0] ) } = $line2[1];
        $j++;
        while ( $lines[$j] !~ /PLATFORM SPECIFIC/ ) {
            @values = split( / {2,}/, $lines[$j] );
            my $x = 1;
            @heads = ("name");
            if ( $values[$x] =~ /U\d+\.*/ ) {
                push( @heads, 'location' );
            }
            push( @heads, 'type' );

            for ( my $i = 0 ; $i <= $#heads ; $i++ ) {

                $result1[0]->{installed_resource_list}->[$count]
                  ->{ to_key_name( item => $heads[$i] ) } = $values[$i];
            }
            $count++;
            $j++;
        }
        if ( $lines[$j] =~ /PLATFORM SPECIFIC/ ) {
            my $i = 0;
            $j = $#lines;
            until ( $lines[$j] =~ /PLATFORM SPECIFIC/ ) {
                until ( $lines[$j] =~ /Name:\s+(.*)/ ) {
                    my @value1 = split( ':\s+', $lines[$j] );

                    $result1[1]->{platform_specific}->[$i]
                      ->{ to_key_name( item => $value1[0] ) } = $value1[1];
                    $j--;
                }
                if ( $lines[$j] =~ /Name:\s+(.*)/ ) {
                    my @value1 = split( ':\s+', $lines[$j] );

                    $result1[1]->{platform_specific}->[$i]
                      ->{ to_key_name( item => $value1[0] ) } = $value1[1];
                    $i++;

                }
                $j--;
            }
        }

        return \@result1;
    }
    if ( $cmd =~ /-v/ ) {
        my @resultv;
        my @heads1;
        my @values1;
        my $count1 = 0;
        my $j      = 0;
        $resultv[0]->{ to_key_name( item => $line1[0] ) } = $line1[1];
        $resultv[0]->{ to_key_name( item => $line2[0] ) } = $line2[1];
        while ( $j <= $#lines ) {
            my $y = 0;
            if ( $lines[$j] !~ /((\w.*)\.\.+(.*))|(\w+:)/ ) {
                @values1 = split( / {2,}/, $lines[$j] );
                my $x = 1;
                @heads1 = ("name");
                if ( $values1[$x] =~ /U\d+\.*/ ) {
                    push( @heads1, 'location' );
                }
                push( @heads1, 'type' );

                for ( my $i = 0 ; $i <= $#heads1 ; $i++ ) {

                    $resultv[0]->{installed_resource_list}->[$count1]
                      ->{ to_key_name( item => $heads1[$i] ) } = $values1[$i];
                }
            }
            my $k = $j + 1;
            if ( $lines[$k] !~ /((\w.*)\.\.+(.*))/ and $lines[$k] !~ /(\w+:)/ )
            {
                $count1++;
            }
            elsif ( $lines[$k] =~ /(\.\.\.+)|(\w+:)/ ) {
                if ( $lines[$k] =~ /\w+:/ ) {
                    my $str = $lines[$k];
                    $k++;
                    while ( $lines[$k] =~ /\.\.\.+/ ) {
                        my @value1 = split( /\.\.+/, $lines[$k] );
                        $k++;
                        $resultv[0]->{installed_resource_list}->[$count1]
                          ->{ to_key_name( item => $str ) }->[$y]
                          ->{ to_key_name( item => $value1[0] ) } = $value1[1];
                    }
                    $y++;
                    $j = $k - 1;
                }
                elsif ( $lines[$k] =~ /\.\.\.+/ ) {
                    while ( $lines[$k] =~ /\.\.\.+/ ) {
                        my @value1 = split( /\.\.+/, $lines[$k] );
                        $k++;
                        $resultv[0]->{installed_resource_list}->[$count1]
                          ->{ to_key_name( item => $value1[0] ) } = $value1[1];

                    }
                    $j = $k - 1;
                }

                $count1++;
            }
            $j++;
        }
        return \@resultv;
    }
}

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $result_arrayp = $args{response_object};
    my $error_level   = $result_arrayp->get_execution_status();
    my ( $error, $error_message );
    if ( $raw_output =~ /[Uu]sage:\s+lscfg.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /lscfg:\s+device\.*/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(
            text            => $error_message,
            response_object => $result_arrayp
        );

    }

}

1;

