# //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/AIX/default/prtconf.pm#1 $
#
#copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse AIX prtconf command
##
## @description
##    This is a library to parse the raw output of AIX lspath command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "lspath" lspath()
##
## @synopsis
##    use prtconf;
##
##    prtconf(%args);
##
## @author sheethal@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
#
package NACL::APISet::Host::CLI::AIX::default::prtconf;

use strict;

#use  NACL::APISet::Exceptions::InvalidParamException qw(:try) ;
use NACL::APISet::Response::CLI::ParserUtils;

## @name ::prtconf
##
## @summary Parses the raw output from the "prtconf" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from prtconf with any of its options as described in the
##    corresponding cdef file 'prtconf.cdefs'
## @synopsis
##      prtconf(%args);
## INPUTS TO METHOD CALL 'prtconf' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-k' </td> <td> 'kernel' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'cpu_type' </td> <td> '1' </td> </tr>
## <tr> <td> '-L' </td> <td> 'lpar_partition' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'processor_clock_speed' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'system_memory' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = prtconf
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;System Model: IBM,9111-520
##&nbsp;Machine Serial Number: 6598EBA
##&nbsp;Processor Type: PowerPC_POWER5
##&nbsp;Processor Implementation Mode: POWER 5
##&nbsp;Processor Version: PV_5
##&nbsp;Number Of Processors: 2
##&nbsp;Processor Clock Speed: 1499 MHz
##&nbsp;CPU Type: 64-bit
##&nbsp;Kernel Type: 64-bit
##&nbsp;LPAR Info: 2 aix221-96
##&nbsp;Memory Size: 1824 MB
##&nbsp;Good Memory Size: 1824 MB
##&nbsp;Platform Firmware level: SF235_160
##&nbsp;Firmware Version: IBM,SF235_160
##&nbsp;Console Login: enable
##&nbsp;Auto Restart: true
##&nbsp;Full Core: false
##&nbsp;
##&nbsp;Network Information
##&nbsp;        Host Name: aix221-96
##&nbsp;        IP Address: 10.72.221.96
##&nbsp;        Sub Netmask: 255.255.252.0
##&nbsp;        Gateway: 10.72.220.1
##&nbsp;        Name Server: 10.72.8.29
##&nbsp;        Domain Name: eng.btc.netapp.in
##&nbsp;
##&nbsp;Paging Space Information
##&nbsp;        Total Paging Space: 4352MB
##&nbsp;        Percent Used: 1%
##&nbsp;
##&nbsp;Volume Groups Information
##&nbsp;==============================================================================
##&nbsp;Inactive VGs
##&nbsp;==============================================================================
##&nbsp;
##&nbsp;==============================================================================
##&nbsp;Active VGs
##&nbsp;==============================================================================
##&nbsp;rootvg:
##&nbsp;PV_NAME           PV STATE          TOTAL PPs   FREE PPs    FREE DISTRIBUTION
##&nbsp;hdisk0            active            546         181         72..00..00..00..109
##&nbsp;==============================================================================
##&nbsp;
##&nbsp;INSTALLED RESOURCE LIST
##&nbsp;
##&nbsp;The following resources are installed on the machine.
##&nbsp;+/- = Added or deleted from Resource List.
##&nbsp;*   = Diagnostic support not available.
##&nbsp;
##&nbsp;  Model Architecture: chrp
##&nbsp;  Model Implementation: Multiple Processor, PCI bus
##&nbsp;
##&nbsp;+ sys0                                               System Object
##&nbsp;+ sysplanar0                                         System Planar
##&nbsp;* vio0                                               Virtual I/O Bus
##&nbsp;* vsa0             U9111.520.6598EBA-V2-C0           LPAR Virtual Serial Adapter
##&nbsp;* vty0             U9111.520.6598EBA-V2-C0-L0        Asynchronous Terminal
##&nbsp;* pci1             U787A.001.DPM175T-P1              PCI Bus
##&nbsp;* pci8             U787A.001.DPM175T-P1              PCI Bus
##&nbsp;* ide0             U787A.001.DPM175T-P1-T12          ATA/IDE Controller Device
##&nbsp;+ cd0              U787A.001.DPM175T-P4-D2           IDE DVD-ROM Drive
##&nbsp;* pci9             U787A.001.DPM175T-P1              PCI Bus
##&nbsp;+ sisscsia0        U787A.001.DPM175T-P1              PCI-X Dual Channel Ultra320 SCSI Adapter
##&nbsp;+ scsi0            U787A.001.DPM175T-P1-T10          PCI-X Dual Channel Ultra320 SCSI Adapter bus
##&nbsp;+ hdisk0           U787A.001.DPM175T-P1-T10-L5-L0    16 Bit LVD SCSI Disk Drive (73400 MB)
##&nbsp;+ hdisk1           U787A.001.DPM175T-P1-T10-L8-L0    16 Bit LVD SCSI Disk Drive (73400 MB)
##&nbsp;+ ses0             U787A.001.DPM175T-P1-T10-L15-L0   SCSI Enclosure Services Device
##&nbsp;+ scsi1            U787A.001.DPM175T-P1-T11          PCI-X Dual Channel Ultra320 SCSI Adapter bus
##&nbsp;* pci0             U787A.001.DPM175T-P1              PCI Bus
##&nbsp;* pci4             U787A.001.DPM175T-P1              PCI Bus
##&nbsp;+ fcs0             U787A.001.DPM175T-P1-C1-T1        FC Adapter
##&nbsp;+ fscsi0           U787A.001.DPM175T-P1-C1-T1        FC SCSI I/O Controller Protocol Device
##&nbsp;* sfwcomm0         U787A.001.DPM175T-P1-C1-T1-W0-L0  Fibre Channel Storage Framework Comm
##&nbsp;* fcnet0           U787A.001.DPM175T-P1-C1-T1        Fibre Channel Network Protocol Device
##&nbsp;* pci5             U787A.001.DPM175T-P1              PCI Bus
##&nbsp;* pci6             U787A.001.DPM175T-P1-C2           PCI Bus
##&nbsp;+ lai0             U787A.001.DPM175T-P1-C2-T1        GXT135P Graphics Adapter
##&nbsp;* pci2             U787A.001.DPM175T-P1              PCI Bus
##&nbsp;+ usbhc0           U787A.001.DPM175T-P1              USB Host Controller (33103500)
##&nbsp;+ usbhc1           U787A.001.DPM175T-P1              USB Host Controller (33103500)
##&nbsp;+ usbhc2           U787A.001.DPM175T-P1              USB Enhanced Host Controller (3310e000)
##&nbsp;* pci3             U787A.001.DPM175T-P1              PCI Bus
##&nbsp;+ ent0             U787A.001.DPM175T-P1-T5           2-Port 10/100/1000 Base-TX PCI-X Adapter (14108902)
##&nbsp;+ ent1             U787A.001.DPM175T-P1-T6           2-Port 10/100/1000 Base-TX PCI-X Adapter (14108902)
##&nbsp;* pci7             U787A.001.DPM175T-P1              PCI Bus
##&nbsp;+ fcs1             U787A.001.DPM175T-P1-C4-T1        FC Adapter
##&nbsp;+ fscsi1           U787A.001.DPM175T-P1-C4-T1        FC SCSI I/O Controller Protocol Device
##&nbsp;* sfwcomm1         U787A.001.DPM175T-P1-C4-T1-W0-L0  Fibre Channel Storage Framework Comm
##&nbsp;* fcnet1           U787A.001.DPM175T-P1-C4-T1        Fibre Channel Network Protocol Device
##&nbsp;+ L2cache0                                           L2 Cache
##&nbsp;+ mem0                                               Memory
##&nbsp;+ proc0                                              Processor
##&nbsp;+ proc2                                              Processor
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'good_memory_size' => '1824 MB',
##&nbsp;            'processor_version' => 'PV_5',
##&nbsp;            'console_login' => 'enable',
##&nbsp;            'cpu_type' => '64-bit',
##&nbsp;            'model_implementation' => 'Multiple Processor, PCI bus',
##&nbsp;            'memory_size' => '1824 MB',
##&nbsp;            'network_information' => {
##&nbsp;                                       'domain_name' => 'eng.btc.netapp.in',
##&nbsp;                                       'name_server' => '10.72.8.29',
##&nbsp;                                       'gateway' => '10.72.220.1',
##&nbsp;                                       'host_name' => 'aix221-96',
##&nbsp;                                       'ip_address' => '10.72.221.96',
##&nbsp;                                       'sub_netmask' => '255.255.252.0'
##&nbsp;                                     },
##&nbsp;            'platform_firmware_level' => 'SF235_160',
##&nbsp;            'installed_resource_list' => [
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'sys0',
##&nbsp;                                             'device_type' => 'System Object',
##&nbsp;                                             'status' => 'added resource'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'sysplanar0',
##&nbsp;                                             'device_type' => 'System Planar',
##&nbsp;                                             'status' => 'added resource'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'vio0',
##&nbsp;                                             'device_type' => 'Virtual I/O Bus',
##&nbsp;                                             'status' => 'diagnostic support not available'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'vsa0',
##&nbsp;                                             'device_type' => 'LPAR Virtual Serial Adapter',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'physical_location' => 'U9111.520.6598EBA-V2-C0'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'vty0',
##&nbsp;                                             'device_type' => 'Asynchronous Terminal',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'physical_location' => 'U9111.520.6598EBA-V2-C0-L0'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'pci1',
##&nbsp;                                             'device_type' => 'PCI Bus',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'pci8',
##&nbsp;                                             'device_type' => 'PCI Bus',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'ide0',
##&nbsp;                                             'device_type' => 'ATA/IDE Controller Device',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1-T12'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'cd0',
##&nbsp;                                             'device_type' => 'IDE DVD-ROM Drive',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P4-D2'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'pci9',
##&nbsp;                                             'device_type' => 'PCI Bus',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'sisscsia0',
##&nbsp;                                             'device_type' => 'PCI-X Dual Channel Ultra320 SCSI Adapter',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'scsi0',
##&nbsp;                                             'device_type' => 'PCI-X Dual Channel Ultra320 SCSI Adapter bus',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1-T10'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'hdisk0',
##&nbsp;                                             'device_type' => '16 Bit LVD SCSI Disk Drive (73400 MB)',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1-T10-L5-L0'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'hdisk1',
##&nbsp;                                             'device_type' => '16 Bit LVD SCSI Disk Drive (73400 MB)',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1-T10-L8-L0'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'ses0',
##&nbsp;                                             'device_type' => 'SCSI Enclosure Services Device',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1-T10-L15-L0'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'scsi1',
##&nbsp;                                             'device_type' => 'PCI-X Dual Channel Ultra320 SCSI Adapter bus',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1-T11'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'pci0',
##&nbsp;                                             'device_type' => 'PCI Bus',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'pci4',
##&nbsp;                                             'device_type' => 'PCI Bus',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'fcs0',
##&nbsp;                                             'device_type' => 'FC Adapter',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1-C1-T1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'fscsi0',
##&nbsp;                                             'device_type' => 'FC SCSI I/O Controller Protocol Device',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1-C1-T1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'sfwcomm0',
##&nbsp;                                             'device_type' => 'Fibre Channel Storage Framework Comm',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1-C1-T1-W0-L0'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'fcnet0',
##&nbsp;                                             'device_type' => 'Fibre Channel Network Protocol Device',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1-C1-T1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'pci5',
##&nbsp;                                             'device_type' => 'PCI Bus',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'pci6',
##&nbsp;                                             'device_type' => 'PCI Bus',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1-C2'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'lai0',
##&nbsp;                                             'device_type' => 'GXT135P Graphics Adapter',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1-C2-T1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'pci2',
##&nbsp;                                             'device_type' => 'PCI Bus',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'usbhc0',
##&nbsp;                                             'device_type' => 'USB Host Controller (33103500)',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'usbhc1',
##&nbsp;                                             'device_type' => 'USB Host Controller (33103500)',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'usbhc2',
##&nbsp;                                             'device_type' => 'USB Enhanced Host Controller (3310e000)',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'pci3',
##&nbsp;                                             'device_type' => 'PCI Bus',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'ent0',
##&nbsp;                                             'device_type' => '2-Port 10/100/1000 Base-TX PCI-X Adapter (14108902)',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1-T5'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'ent1',
##&nbsp;                                             'device_type' => '2-Port 10/100/1000 Base-TX PCI-X Adapter (14108902)',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1-T6'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'pci7',
##&nbsp;                                             'device_type' => 'PCI Bus',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'fcs1',
##&nbsp;                                             'device_type' => 'FC Adapter',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1-C4-T1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'fscsi1',
##&nbsp;                                             'device_type' => 'FC SCSI I/O Controller Protocol Device',
##&nbsp;                                             'status' => 'added resource',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1-C4-T1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'sfwcomm1',
##&nbsp;                                             'device_type' => 'Fibre Channel Storage Framework Comm',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1-C4-T1-W0-L0'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'fcnet1',
##&nbsp;                                             'device_type' => 'Fibre Channel Network Protocol Device',
##&nbsp;                                             'status' => 'diagnostic support not available',
##&nbsp;                                             'physical_location' => 'U787A.001.DPM175T-P1-C4-T1'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'L2cache0',
##&nbsp;                                             'device_type' => 'L2 Cache',
##&nbsp;                                             'status' => 'added resource'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'mem0',
##&nbsp;                                             'device_type' => 'Memory',
##&nbsp;                                             'status' => 'added resource'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'proc0',
##&nbsp;                                             'device_type' => 'Processor',
##&nbsp;                                             'status' => 'added resource'
##&nbsp;                                           },
##&nbsp;                                           {
##&nbsp;                                             'device_name' => 'proc2',
##&nbsp;                                             'device_type' => 'Processor',
##&nbsp;                                             'status' => 'added resource'
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;            'full_core' => 'false',
##&nbsp;            'processor_clock_speed' => '1499 MHz',
##&nbsp;            'lpar_info' => '2 aix221-96',
##&nbsp;            'kernel_type' => '64-bit',
##&nbsp;            'inactive_vgs' => undef,
##&nbsp;            'paging_space_information' => {
##&nbsp;                                            'percent_used' => '1%',
##&nbsp;                                            'total_paging_space' => '4352MB'
##&nbsp;                                          },
##&nbsp;            'firmware_version' => 'IBM,SF235_160',
##&nbsp;            'processor_type' => 'PowerPC_POWER5',
##&nbsp;            'system_model' => 'IBM,9111-520',
##&nbsp;            'auto_restart' => 'true',
##&nbsp;            'number_of_processors' => '2',
##&nbsp;            'processor_implementation_mode' => 'POWER 5',
##&nbsp;            'active_vgs' => [
##&nbsp;                              {
##&nbsp;                                'rootvg' => {
##&nbsp;                                              'pps' => '72..00..00..00..109',
##&nbsp;                                              'pv' => 'RESOURCE',
##&nbsp;                                              'total' => '181',
##&nbsp;                                              'state' => 'LIST',
##&nbsp;                                              'pv_name' => 'INSTALLED'
##&nbsp;                                            }
##&nbsp;                              }
##&nbsp;                            ],
##&nbsp;            'machine_serial_number' => '6598EBA'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = prtconf -c
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;CPU Type: 64-bit
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cpu_type' => '64-bit'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = prtconf -k
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Kernel Type: 64-bit
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'kernel_type' => '64-bit'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = prtconf -L
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;LPAR Info: 2 aix221-96
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lpar_info' => '2 aix221-96'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = prtconf -m
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Memory Size: 1824 MB
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'memory_size' => '1824 MB'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = prtconf -s
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Processor Clock Speed: 1499 MHz
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'processor_clock_speed' => '1499 MHz'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = prtconf -c -k -L -m -s
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;CPU Type: 64-bit
##&nbsp;Kernel Type: 64-bit
##&nbsp;LPAR Info: 2 aix221-96
##&nbsp;Memory Size: 1824 MB
##&nbsp;Processor Clock Speed: 1499 MHz
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'kernel_type' => '64-bit',
##&nbsp;            'cpu_type' => '64-bit',
##&nbsp;            'memory_size' => '1824 MB',
##&nbsp;            'processor_clock_speed' => '1499 MHz',
##&nbsp;            'lpar_info' => '2 aix221-96'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub prtconf {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    my $str;
    my @head;
    my @proc_info;
    my $i = 0;
    my $key;
    _check_error(raw_output => $output, response_object => $resp);
    my (@result_array);
    my @lines = get_lines(raw_output => $output, response_object => $resp);
    my $count = 0;
    @lines = grep (!(/^(===.*)/), @lines);

    while (@lines) {
        if (   $cmd =~ /-c/
            || $cmd =~ /-k/
            || $cmd =~ /-L/
            || $cmd =~ /-m/
            || $cmd =~ /-s/) {
            while (@lines) {
                $str = shift(@lines);
                my @head = split(/:\s+/, $str, 2);
                $result_array[$count]->{to_key_name(item => $head[0])} = $head[1];
            }
        }

        if ($str =~ /^(System.*:\s+((\w|\s)*))/) {
            until ($str =~ /^Network/ || !@lines) {
                my @head = split(/:\s+/, $str, 2);
                $result_array[$count]->{to_key_name(item => $head[0])} = $head[1];
                $str = shift(@lines);
            }
        }
        if ($str =~ /^Network/) {
            $str = shift(@lines);
            until ($str =~ /^Paging/ || !@lines) {
               my @head = split(/:\s+/, $str, 2);
                $result_array[$count]->{network_information}
                    ->{to_key_name(item => $head[0])} = $head[1];
                $str = shift(@lines);
            }
        }
        if ($str =~ /^Paging/) {
            $str = shift(@lines);
            until ($str =~ /^Volume/ || !@lines) {
               my @head = split(/:\s+/, $str, 2);
                $result_array[$count]->{paging_space_information}
                    ->{to_key_name(item => $head[0])} = $head[1];
                $str = shift(@lines);
            }
        }
        if ($str =~ /^Inactive/) {

            until ($str =~ /^Active/ || !@lines) {
                $result_array[$count]->{inactive_vgs} = undef;

                $str = shift(@lines);
            }
        }
        if ($str =~ /^Active/) {
            until ($str =~ /^INSTALLED/ || !@lines) {
                my $j = -1;
                $str = shift(@lines);
                if ($str =~ /:/) {
                    $j++;
                    my $vg = $str;
                    $vg =~ s/://;
                    $str = shift(@lines);
                    @head = split(/\s+/, $str);
                    until ($str =~ /:/ || $str =~ /^INSTALLED/ || !@lines) {
                        $str = shift(@lines);
                        @proc_info = split(/\s+/, $str);
                        for (my $i = 0; $i < scalar(@proc_info); $i++) {
                            $result_array[$count]->{active_vgs}->[$j]->{$vg}
                                ->{to_key_name(item => $head[$i])}
                                = $proc_info[$i];
                        }
                    }
                }
            }
        }
        if ($str =~ /^Model/) {
            until ($str =~ /^\+/ || $str =~ /^\*/ || $str =~ /^\-/ || !@lines) {
                my @head = split(/:\s+/, $str, 2);
                $result_array[$count]->{to_key_name(item => $head[0])} = $head[1];
                $str = shift(@lines);
            }
        }
        if ($str =~ /^\+/ || $str =~ /^\*/ || $str =~ /^\-/) {
            my $j = 0;
            my %hash;
            $hash{'+'} = "added resource";
            $hash{'-'} = "deleted resource";
            $hash{'*'} = "diagnostic support not available";

            while (($str =~ /^\+/ || $str =~ /^\*/ || $str =~ /^\-/ ) && scalar(@lines)) {
                my @values  = split(/ {2,}/, $str);
                my @values1 = split(/ {1,}/, $values[0]);
                shift(@values);
                unshift(@values, $values1[1]);

                my @heads = ("device_name");

                if ($values[1] =~ /U\d+\.*/) {
                    push(@heads, 'physical_location');

                }
                push(@heads, 'device_type');
                unshift(@heads,  'status');
                unshift(@values, $hash{$values1[0]});

                for (my $i = 0; $i <= $#heads; $i++) {
                    $result_array[$count]->{installed_resource_list}->[$j]
                        ->{to_key_name(item => $heads[$i])} = $values[$i];
                }
                $str = shift(@lines);
                $j++;
            }
        }
        $str = shift(@lines);
    }
    return \@result_array;
} ## end sub prtconf

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $result_arrayp = $args{response_object};
    my $error_level   = $result_arrayp->get_execution_status();
    my ($error, $error_message);
    if ($raw_output =~ /([Uu]sage:|prtconf:)\s+\w+.*/) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($raw_output
        =~ /prtconf:\s+\d+.*|prtconf: Invalid page.*|prtconf: A flag requires.*/
        ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } ## end sub _check_error
} ## end sub _check_error

1;

