#Copyright (c) 2013 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse rmdev command output
##
## @description
##    This is a library to parse the raw output of  rmdev command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::AIX::default::rmdev;
##
##    rmdev(%args);
##
## @author sheethal@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::AIX::default::rmdev;

use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Response::CLI::ParserUtils;
use strict;

## @name ::rmdev
##
## @summary Parses the raw output from the "rmdev" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from rmdev with any of its options as described in the 
##    corresponding cdef file 'rmdev.cdefs'
## @synopsis
##      rmdev(%args);
## INPUTS TO METHOD CALL 'rmdev' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-q' </td> <td> 'quiet' </td> <td> '1' </td> </tr> 
## <tr> <td> '-p' </td> <td> 'parent' </td> <td> '0' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'file' </td> <td> '0' </td> </tr> 
## <tr> <td> '-l' </td> <td> 'name' </td> <td> '0' </td> </tr> 
## <tr> <td> '-R' </td> <td> 'unconfigure' </td> <td> '1' </td> </tr> 
## <tr> <td> '-S' </td> <td> 'stop_unavailable' </td> <td> '1' </td> </tr> 
## <tr> <td> '-g' </td> <td> 'force_remove' </td> <td> '1' </td> </tr> 
## <tr> <td> '-h' </td> <td> 'help' </td> <td> '1' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'remove_definition' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub rmdev {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error(raw_output => $output, response_object => $resp);
    return [];
} ## end sub rmdev

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ($error, $error_message) = undef;
    if ($raw_output =~ /(Usage:)/) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $raw_output,
            response_object => $resp
        );
    }
} ## end sub _check_error

1;
