#
# Copyright (c) 2013 NetApp, Inc.
# All rights reserved.
#
## @summary Routines to parse SystemShell envcmdtool  commands
##
## @description
##     This is a library to parse the raw output of SystemShell envcmdtool command.
##     The function name will be the same as command name with underscores being
##     used instead of spaces.
##
## @synopsis
##     use envcmdtool ;
##
##     envcmdtool (%args);
##
## @author sheethal@netapp.com , dl-nacl-dev
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::envcmdtool;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args            = @_;
    my $raw_output      = $args{raw_output};
    my $command         = $args{command};
    my $response_object = $args{response_object};
    my $error_message;
    my $error;
    if ($raw_output =~ /([Uu]sage: envcmdtool.*)/) {
        $error_message = $1;
        $error         = "InvalidParam";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(
            text            => $error_message,
            response_object => $response_object
        );
    }

} ## end sub _check_error

## @name ::envcmdtool
##
## @summary Parses the raw output from the "envcmdtool " command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from envcmdtool with any of its options as described in the
##    corresponding cdef file 'envcmdtool.cdefs'
## @synopsis
##      envcmdtool(%args);
## INPUTS TO METHOD CALL 'envcmdtool' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-u' </td> <td> 'microseconds' </td> <td> '0' </td> </tr>
## <tr> <td> '-show_inactive' </td> <td> 'show_inactive' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'crithigh' </td> <td> '0' </td> </tr>
## <tr> <td> '-show_all' </td> <td> 'show_all' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'hystlow' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'seconds' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'set_state' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'newstate' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'warnlow' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'newstatename' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'id' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'critlow' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'get_state' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'num_retries' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'extrlow' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'hours' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'value' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'warnhigh' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'version' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'minutes' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'reason' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'hysthigh' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'extrhigh' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'full' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'socket' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'newval' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'action' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'type' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'newmask' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'retry_interval' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub envcmdtool {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );
    return [];
} ## end sub envcmdtool

## @name ::envcmdtool_sensors
##
## @summary Parses the raw output from the "envcmdtool sensors" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from envcmdtool_sensors with any of its options as described in the
##    corresponding cdef file 'envcmdtool.cdefs'
## @synopsis
##      envcmdtool_sensors(%args);
## INPUTS TO METHOD CALL 'envcmdtool_sensors' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-n' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'id' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'full' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub envcmdtool_sensors {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );
    return [];
} ## end sub envcmdtool_sensors

## @name ::envcmdtool_fru_info
##
## @summary Parses the raw output from the "envcmdtool fru_info" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from envcmdtool_fru_info with any of its options as described in the
##    corresponding cdef file 'envcmdtool.cdefs'
## @synopsis
##      envcmdtool_fru_info(%args);
## INPUTS TO METHOD CALL 'envcmdtool_fru_info' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-n' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'id' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'full' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub envcmdtool_fru_info {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );
    return [];
} ## end sub envcmdtool_fru_info

## @name ::envcmdtool_get_state
##
## @summary Parses the raw output from the "envcmdtool get_state" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from envcmdtool_get_state with any of its options as described in the
##    corresponding cdef file 'envcmdtool.cdefs'
## @synopsis
##      envcmdtool_get_state(%args);
## INPUTS TO METHOD CALL 'envcmdtool_get_state' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-n' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'id' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'full' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = envcmdtool get_state -n "PSU1 Fan2 Fault"
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;sid: PSU1_Fan2_Fault
##&nbsp;value: 1
##&nbsp;units: OK
##&nbsp;state: 196615
##&nbsp;state name: normal
##&nbsp;enabled: yes
##&nbsp;initialized: yes
##&nbsp;valid: yes
##&nbsp;failed: no
##&nbsp;stale: no
##&nbsp;ignored: no
##&nbsp;forced: no
##&nbsp;hold: no
##&nbsp;extrlow: 0
##&nbsp;critlow: 0
##&nbsp;warnlow: 0
##&nbsp;warnhigh: 0
##&nbsp;crithigh: 0
##&nbsp;extrhigh: 0
##&nbsp;hystlow: 0
##&nbsp;hysthigh: 0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'sid' => 'PSU1_Fan2_Fault',
##&nbsp;            'forced' => 'no',
##&nbsp;            'state_name' => 'normal',
##&nbsp;            'crithigh' => '0',
##&nbsp;            'failed' => 'no',
##&nbsp;            'hystlow' => '0',
##&nbsp;            'valid' => 'yes',
##&nbsp;            'state' => '196615',
##&nbsp;            'initialized' => 'yes',
##&nbsp;            'warnlow' => '0',
##&nbsp;            'critlow' => '0',
##&nbsp;            'enabled' => 'yes',
##&nbsp;            'extrlow' => '0',
##&nbsp;            'value' => '1',
##&nbsp;            'warnhigh' => '0',
##&nbsp;            'hysthigh' => '0',
##&nbsp;            'extrhigh' => '0',
##&nbsp;            'stale' => 'no',
##&nbsp;            'ignored' => 'no',
##&nbsp;            'hold' => 'no',
##&nbsp;            'units' => 'OK'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub envcmdtool_get_state {

    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my @result_array;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );
    my @lines
        = get_lines(raw_output => $raw_output, response_object => $resp);
    foreach my $line (@lines) {
        my @values = split(/:\s+/, $line);
        $result_array[0]->{to_key_name(item => $values[0])} = $values[1];
    }
    return \@result_array;
} ## end sub envcmdtool_get_state

## @name ::envcmdtool_set_state
##
## @summary Parses the raw output from the "envcmdtool set_state" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from envcmdtool_set_state with any of its options as described in the
##    corresponding cdef file 'envcmdtool.cdefs'
## @synopsis
##      envcmdtool_set_state(%args);
## INPUTS TO METHOD CALL 'envcmdtool_set_state' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'newstate' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'newstatename' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'newmask' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub envcmdtool_set_state {

    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );
    return [];
} ## end sub envcmdtool_set_state

## @name ::envcmdtool_set_val
##
## @summary Parses the raw output from the "envcmdtool set_val" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from envcmdtool_set_val with any of its options as described in the
##    corresponding cdef file 'envcmdtool.cdefs'
## @synopsis
##      envcmdtool_set_val(%args);
## INPUTS TO METHOD CALL 'envcmdtool_set_val' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'newval' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub envcmdtool_set_val {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );
    return [];
} ## end sub envcmdtool_set_val

## @name ::envcmdtool_set_thresh
##
## @summary Parses the raw output from the "envcmdtool set_thresh" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from envcmdtool_set_thresh with any of its options as described in the
##    corresponding cdef file 'envcmdtool.cdefs'
## @synopsis
##      envcmdtool_set_thresh(%args);
## INPUTS TO METHOD CALL 'envcmdtool_set_thresh' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'extrlow' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'warnhigh' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'crithigh' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'hysthigh' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'extrhigh' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'hystlow' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'warnlow' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'id' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'critlow' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub envcmdtool_set_thresh {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );
    return [];
} ## end sub envcmdtool_set_thresh

## @name ::envcmdtool_set_retry
##
## @summary Parses the raw output from the "envcmdtool set_retry" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from envcmdtool_set_retry with any of its options as described in the
##    corresponding cdef file 'envcmdtool.cdefs'
## @synopsis
##      envcmdtool_set_retry(%args);
## INPUTS TO METHOD CALL 'envcmdtool_set_retry' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-n' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'id' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'retry_interval' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'num_retries' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub envcmdtool_set_retry {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );
    return [];
} ## end sub envcmdtool_set_retry

## @name ::envcmdtool_groups
##
## @summary Parses the raw output from the "envcmdtool groups" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from envcmdtool_groups with any of its options as described in the
##    corresponding cdef file 'envcmdtool.cdefs'
## @synopsis
##      envcmdtool_groups(%args);
## INPUTS TO METHOD CALL 'envcmdtool_groups' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-n' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub envcmdtool_groups {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );
    return [];
} ## end sub envcmdtool_groups

## @name ::envcmdtool_get_group_policy
##
## @summary Parses the raw output from the "envcmdtool get_group_policy" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from envcmdtool_get_group_policy with any of its options as described in the
##    corresponding cdef file 'envcmdtool.cdefs'
## @synopsis
##      envcmdtool_get_group_policy(%args);
## INPUTS TO METHOD CALL 'envcmdtool_get_group_policy' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-n' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub envcmdtool_get_group_policy {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );
    return [];
} ## end sub envcmdtool_get_group_policy

## @name ::envcmdtool_get_sensor_policy
##
## @summary Parses the raw output from the "envcmdtool get_sensor_policy" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from envcmdtool_get_sensor_policy with any of its options as described in the
##    corresponding cdef file 'envcmdtool.cdefs'
## @synopsis
##      envcmdtool_get_sensor_policy(%args);
## INPUTS TO METHOD CALL 'envcmdtool_get_sensor_policy' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-n' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub envcmdtool_get_sensor_policy {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );
    return [];
} ## end sub envcmdtool_get_sensor_policy

## @name ::envcmdtool_get_action_enabled
##
## @summary Parses the raw output from the "envcmdtool get_action_enabled" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from envcmdtool_get_action_enabled with any of its options as described in the
##    corresponding cdef file 'envcmdtool.cdefs'
## @synopsis
##      envcmdtool_get_action_enabled(%args);
## INPUTS TO METHOD CALL 'envcmdtool_get_action_enabled' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'action' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub envcmdtool_get_action_enabled {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );
    return [];
} ## end sub envcmdtool_get_action_enabled

## @name ::envcmdtool_enable_action
##
## @summary Parses the raw output from the "envcmdtool enable_action" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from envcmdtool_enable_action with any of its options as described in the
##    corresponding cdef file 'envcmdtool.cdefs'
## @synopsis
##      envcmdtool_enable_action(%args);
## INPUTS TO METHOD CALL 'envcmdtool_enable_action' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'action' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub envcmdtool_enable_action {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );
    return [];
} ## end sub envcmdtool_enable_action

## @name ::envcmdtool_disable_action
##
## @summary Parses the raw output from the "envcmdtool disable_action" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from envcmdtool_disable_action with any of its options as described in the
##    corresponding cdef file 'envcmdtool.cdefs'
## @synopsis
##      envcmdtool_disable_action(%args);
## INPUTS TO METHOD CALL 'envcmdtool_disable_action' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'action' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub envcmdtool_disable_action {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );
    return [];
} ## end sub envcmdtool_disable_action

## @name ::envcmdtool_action
##
## @summary Parses the raw output from the "envcmdtool action" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from envcmdtool_action with any of its options as described in the
##    corresponding cdef file 'envcmdtool.cdefs'
## @synopsis
##      envcmdtool_action(%args);
## INPUTS TO METHOD CALL 'envcmdtool_action' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'value' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'reason' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub envcmdtool_action {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );
    return [];
} ## end sub envcmdtool_action

## @name ::envcmdtool_fast_forward
##
## @summary Parses the raw output from the "envcmdtool fast_forward" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from envcmdtool_fast_forward with any of its options as described in the
##    corresponding cdef file 'envcmdtool.cdefs'
## @synopsis
##      envcmdtool_fast_forward(%args);
## INPUTS TO METHOD CALL 'envcmdtool_fast_forward' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'seconds' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'microseconds' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'hours' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'minutes' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub envcmdtool_fast_forward {
    my %args         = @_;
    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();
    my $command_line = $resp->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $raw_output,
        "command"         => $command_line,
        "response_object" => $resp
    );
    return [];
} ## end sub envcmdtool_fast_forward
1;
