#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Freebsd/default/kenv.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse kenv command output
##
## @description
##    This library supports error checking for kenv command, but output parsing is not supported as of now.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::kenv;
##
##  kenv(%args);
##
## @author skotagir@netapp.com , dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::kenv;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

## @name ::kenv
##
## @summary Parses the raw output from the "kenv" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from kenv with any of its options as described in the
##    corresponding cdef file 'kenv.cdefs'
## @synopsis
##      kenv(%args);
## INPUTS TO METHOD CALL 'kenv' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
##  Returns: Output parsing is not supported

## @name ::kenv
##
## @summary Parses the raw output from the "kenv" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from kenv with any of its options as described in the
##    corresponding cdef file 'kenv.cdefs'
## @synopsis
##      kenv(%args);
## INPUTS TO METHOD CALL 'kenv' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = kenv
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;AUTOBOOT="true"
##&nbsp;BIOS_VERSION="245"
##&nbsp;BOARDNAME="VMWARE"
##&nbsp;BOOTED_FROM="PRIMARY"
##&nbsp;BOOT_FILE="x86_64/freebsd/image1/kernel"
##&nbsp;LINES="24"
##&nbsp;LOADER_VERSION="1.0"
##&nbsp;MOBO_REV="ZZ"
##&nbsp;MOBO_SERIAL_NUM="999999"
##&nbsp;NETAPP_BACKUP_KERNEL_URL="x86_64/freebsd/image2/kernel"
##&nbsp;NETAPP_PRIMARY_KERNEL_URL="x86_64/freebsd/image1/kernel"
##&nbsp;SYS_MODEL="SIMBOX"
##&nbsp;SYS_REV="ZZ"
##&nbsp;SYS_SERIAL_NUM="4073324-73-5"
##&nbsp;acpi_load="YES"
##&nbsp;bootarg.bsdportname="e0c"
##&nbsp;bootarg.dblade.wafl_use_delete_log="false"
##&nbsp;bootarg.init.boot_clustered="true"
##&nbsp;bootarg.init.cfdevice="/dev/ad0s2"
##&nbsp;bootarg.init.clearvarfsnvram="false"
##&nbsp;bootarg.init.dhcp.disable="true"
##&nbsp;bootarg.mgwd.autoconf.disable="true"
##&nbsp;bootarg.mgwd.scsi_blade_uuid="ab7cce7d-fd77-11e2-a315-ed1b290567fe"
##&nbsp;bootarg.nvram.sysid="4073324735"
##&nbsp;bootarg.setup.auto.internal="true"
##&nbsp;bootarg.sim="true"
##&nbsp;bootarg.srm.disk.config.pci="true"
##&nbsp;bootarg.srm.disk.simnames="false"
##&nbsp;bootarg.srm.disk.simulated="false"
##&nbsp;bootarg.srm.nvram.setup_sim="true"
##&nbsp;bootarg.srm.nvram.vnvram="true"
##&nbsp;bootarg.srm.virtual.adapter="true"
##&nbsp;bootarg.vm="true"
##&nbsp;bootarg.vm.data_diskmodel="vscsi"
##&nbsp;bootarg.vm.no_poweroff_on_halt="true"
##&nbsp;bootarg.vm.rapidsavecore="true"
##&nbsp;bootarg.vm.run_vmtools="true"
##&nbsp;bootarg.vm.scsi_wwns="50 05 05 6c 62 65 0c 80,50 05 05 6c 62 65 0d 80,50 05 05 6c 62 65 0e 80,50 05 05 6c 62 65 0f 80"
##&nbsp;bootarg.vm.sim="true"
##&nbsp;bootarg.vm.sys_diskmodel="standard"
##&nbsp;bootarg.vm.vnvram="panic"
##&nbsp;bootarg.vnvram.size="256"
##&nbsp;bootfile="kernel"
##&nbsp;comconsole_speed="9600"
##&nbsp;console="comconsole"
##&nbsp;currdev="disk1s2"
##&nbsp;interpret="OK"
##&nbsp;kernel="kernel"
##&nbsp;kernel_options=""
##&nbsp;kernelname="x86_64/freebsd/image1/kernel"
##&nbsp;loader_conf_files="/boot/device.hints /boot/loader.conf /boot/loader.conf.local"
##&nbsp;mac_ifoff="NO"
##&nbsp;module_path="/boot/kernel;/boot/modules"
##&nbsp;ntap.init.cfdevice="/dev/ad0s2"
##&nbsp;nvram_discard="false"
##&nbsp;nvram_emulation="true"
##&nbsp;prompt="VLOADER>"
##&nbsp;setvsimenv="true"
##&nbsp;smbios.bios.reldate="10/13/2009"
##&nbsp;smbios.bios.vendor="Phoenix Technologies LTD"
##&nbsp;smbios.bios.version="6.00"
##&nbsp;smbios.chassis.maker="No Enclosure"
##&nbsp;smbios.chassis.serial="None"
##&nbsp;smbios.chassis.tag="No Asset Tag"
##&nbsp;smbios.chassis.version="N/A"
##&nbsp;smbios.memory.enabled="5242880"
##&nbsp;smbios.planar.maker="Intel Corporation"
##&nbsp;smbios.planar.product="440BX Desktop Reference Platform"
##&nbsp;smbios.planar.serial="None"
##&nbsp;smbios.planar.version="None"
##&nbsp;smbios.socket.enabled="2"
##&nbsp;smbios.socket.populated="2"
##&nbsp;smbios.system.maker="VMware, Inc."
##&nbsp;smbios.system.product="VMware Virtual Platform"
##&nbsp;smbios.system.serial="VMware-56 4d 5d 6c 62 65 0c 83-e2 c4 e0 aa 91 99 aa 83"
##&nbsp;smbios.system.uuid="564d5d6c-6265-0c83-e2c4-e0aa9199aa83"
##&nbsp;smbios.system.version="None"
##&nbsp;smbios.version="2.4"
##&nbsp;sysvar.system_id="0"
##&nbsp;wafl-disable-mbuf-backed-buffers?="TRUE"
##&nbsp;hint.vga.0.at="isa"
##&nbsp;hint.sc.0.at="isa"
##&nbsp;hint.sc.0.flags="0x100"
##&nbsp;hint.atkbdc.0.at="isa"
##&nbsp;hint.atkbdc.0.port="0x060"
##&nbsp;hint.atkbd.0.at="atkbdc"
##&nbsp;hint.atkbd.0.irq="1"
##&nbsp;hint.uart.0.at="isa"
##&nbsp;hint.uart.0.port="0x3F8"
##&nbsp;hint.uart.0.flags="0x40"
##&nbsp;hint.uart.0.irq="4"
##&nbsp;hint.uart.1.at="isa"
##&nbsp;hint.uart.1.port="0x2F8"
##&nbsp;hint.uart.1.flags="0x20"
##&nbsp;hint.uart.1.irq="3"
##&nbsp;hint.atrtc.0.at="isa"
##&nbsp;hint.atrtc.0.port="0x70"
##&nbsp;hint.atrtc.0.irq="8"
##&nbsp;bootarg.init.booterr="0"
##&nbsp;bootarg.init.cf_mounted="true"
##&nbsp;bootarg.init.rootimage="/cfcard/x86_64/freebsd/image1/rootfs.img"
##&nbsp;bootarg.init.cfimagebase="/cfcard/x86_64/freebsd"
##&nbsp;ntap.init.kernelname="/cfcard/x86_64/freebsd/image1/kernel"
##&nbsp;bootarg.init.defaultimage="image1"
##&nbsp;bootarg.init.kldload_mem_size="386433024"
##&nbsp;bootarg.init.wipeclean="false"
##&nbsp;bootarg.init.cf_new="false"
##&nbsp;bootarg.init.last_boot="true"
##&nbsp;bootarg.notifyd.optout="false"
##&nbsp;bootarg.bootmenu.selection="4a"
##&nbsp;sysvar.ngsh_remote_ip=""
##&nbsp;bootarg.mgwd.nblade_uuid="ab7cb023-fd77-11e2-b9e8-717c7aff181e"
##&nbsp;bootarg.mgwd.spinvfs_uuid="ab7d8ea9-fd77-11e2-8bc9-67e3ee58dda1"
##&nbsp;bootarg.bootmenu.root_dsid="1"
##&nbsp;bootarg.bootmenu.root_msid="2147483649"
##&nbsp;bootarg.bootmenu.root_uuid="0500b0f9-f4b8-4078-96b7-b44685e78ec2"
##&nbsp;bootarg.bootmenu.node_uuid="ab7d8f8b-fd77-11e2-ad00-81308873a5f8"
##&nbsp;bootarg.prevent_sendhome="false"
##&nbsp;fc-ports-valid="true"
##&nbsp;bootarg.dblade.root_volume.local_dsid="0x8e57f785"
##&nbsp;bootarg.dblade.root_volume.local_uuid="0500b0f9-f4b8-4078-96b7-b44685e78ec2"
##&nbsp;bootarg.dblade.root_volume.local_name="vol0"
##&nbsp;bootarg.dblade.root_volume.local_aggr_uuid="2f075439-4c80-4937-acb8-88fa9c0cd582"
##&nbsp;bootarg.dblade.root_volume.local_event="INIT"
##&nbsp;bootarg.dblade.root_volume.local_size="2781_MB"
##&nbsp;bootarg.dblade.root_volume.local_space="2781_MB"
##&nbsp;REBOOT_REASON="REBOOT_UNKNOWN"
##&nbsp;last-OS-booted-raid-ver="12"
##&nbsp;last-OS-booted-wafl-ver="24851"
##&nbsp;last-OS-booted-ver="devN_130803_0500"
##&nbsp;bootarg.init.boot_mode="normal"
##&nbsp;bootarg.mgwd.mroot_found="true"
##&nbsp;bootarg.mgwd.booted="true"
##&nbsp;bootarg.mgwd.movestatus.sampling_int="20"
##&nbsp;bootarg.mgwd.movestatus.cleanup_int="288"
##&nbsp;bootarg.mgwd.movestatus.cleanup_now="0"
##&nbsp;sysvar.init.populate_mroot_late_done="true"
##&nbsp;bootarg.bootmenu.root_aggr_name="kjp_vsim6_01_aggr0"
##&nbsp;bootarg.mgwd.cluster_uuid="2fc679fa-fd9b-11e2-ac86-123478563412"
##&nbsp;bootarg.mgwd.cluster_name="kjp-vsim6"
##&nbsp;bootarg.libc.nameservices="true"
##&nbsp;bootarg.GM_DNSLB_Nameservices="true"
##&nbsp;bootarg.login_running="true"
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bootarg_prevent_sendhome' => 'false',
##&nbsp;            'smbios_socket_populated' => '2',
##&nbsp;            'setvsimenv' => 'true',
##&nbsp;            'bios_version' => '245',
##&nbsp;            'bootarg_mgwd_cluster_uuid' => '2fc679fa-fd9b-11e2-ac86-123478563412',
##&nbsp;            'bootarg_init_boot_mode' => 'normal',
##&nbsp;            'bootarg_vm_rapidsavecore' => 'true',
##&nbsp;            'bootarg_dblade_wafl_use_delete_log' => 'false',
##&nbsp;            'bootarg_init_clearvarfsnvram' => 'false',
##&nbsp;            'hint_uart_0_irq' => '4',
##&nbsp;            'bootarg_srm_nvram_setup_sim' => 'true',
##&nbsp;            'mac_ifoff' => 'NO',
##&nbsp;            'prompt' => 'VLOADER>',
##&nbsp;            'netapp_backup_kernel_url' => 'x86_64/freebsd/image2/kernel',
##&nbsp;            'bootarg_mgwd_mroot_found' => 'true',
##&nbsp;            'ntap_init_kernelname' => '/cfcard/x86_64/freebsd/image1/kernel',
##&nbsp;            'smbios_chassis_serial' => 'None',
##&nbsp;            'bootarg_srm_disk_config_pci' => 'true',
##&nbsp;            'bootarg_vm' => 'true',
##&nbsp;            'bootarg_vm_sys_diskmodel' => 'standard',
##&nbsp;            'bootarg_sim' => 'true',
##&nbsp;            'hint_uart_0_at' => 'isa',
##&nbsp;            'nvram_discard' => 'false',
##&nbsp;            'bootarg_vm_no_poweroff_on_halt' => 'true',
##&nbsp;            'bootarg_vm_scsi_wwns' => '50 05 05 6c 62 65 0c 80,50 05 05 6c 62 65 0d 80,50 05 05 6c 62 65 0e 80,50 05 05 6c 62 65 0f 80',
##&nbsp;            'sysvar_system_id' => '0',
##&nbsp;            'bootarg_vm_run_vmtools' => 'true',
##&nbsp;            'kernel' => 'kernel',
##&nbsp;            'bootarg_dblade_root_volume_local_size' => '2781_MB',
##&nbsp;            'acpi_load' => 'YES',
##&nbsp;            'smbios_bios_version' => '6.00',
##&nbsp;            'bootarg_init_cf_new' => 'false',
##&nbsp;            'bootarg_mgwd_movestatus_cleanup_now' => '0',
##&nbsp;            'hint_uart_1_port' => '0x2F8',
##&nbsp;            'smbios_system_product' => 'VMware Virtual Platform',
##&nbsp;            'bootarg_bootmenu_root_msid' => '2147483649',
##&nbsp;            'currdev' => 'disk1s2',
##&nbsp;            'hint_vga_0_at' => 'isa',
##&nbsp;            'last_os_booted_wafl_ver' => '24851',
##&nbsp;            'bootarg_dblade_root_volume_local_uuid' => '0500b0f9-f4b8-4078-96b7-b44685e78ec2',
##&nbsp;            'fc_ports_valid' => 'true',
##&nbsp;            'hint_sc_0_flags' => '0x100',
##&nbsp;            'bootarg_bootmenu_root_dsid' => '1',
##&nbsp;            'bootarg_init_defaultimage' => 'image1',
##&nbsp;            'smbios_system_version' => 'None',
##&nbsp;            'autoboot' => 'true',
##&nbsp;            'hint_sc_0_at' => 'isa',
##&nbsp;            'bootarg_dblade_root_volume_local_dsid' => '0x8e57f785',
##&nbsp;            'hint_uart_1_flags' => '0x20',
##&nbsp;            'bootarg_bootmenu_root_aggr_name' => 'kjp_vsim6_01_aggr0',
##&nbsp;            'bootarg_bootmenu_node_uuid' => 'ab7d8f8b-fd77-11e2-ad00-81308873a5f8',
##&nbsp;            'smbios_planar_maker' => 'Intel Corporation',
##&nbsp;            'bootarg_mgwd_movestatus_cleanup_int' => '288',
##&nbsp;            'bootarg_mgwd_booted' => 'true',
##&nbsp;            'bootarg_init_kldload_mem_size' => '386433024',
##&nbsp;            'bootarg_mgwd_spinvfs_uuid' => 'ab7d8ea9-fd77-11e2-8bc9-67e3ee58dda1',
##&nbsp;            'smbios_version' => '2.4',
##&nbsp;            'bootarg_srm_disk_simnames' => 'false',
##&nbsp;            'bootarg_bootmenu_root_uuid' => '0500b0f9-f4b8-4078-96b7-b44685e78ec2',
##&nbsp;            'bootarg_gm_dnslb_nameservices' => 'true',
##&nbsp;            'kernelname' => 'x86_64/freebsd/image1/kernel',
##&nbsp;            'bootarg_mgwd_cluster_name' => 'kjp-vsim6',
##&nbsp;            'mobo_rev' => 'ZZ',
##&nbsp;            'sys_serial_num' => '4073324-73-5',
##&nbsp;            'bootarg_mgwd_movestatus_sampling_int' => '20',
##&nbsp;            'bootarg_dblade_root_volume_local_event' => 'INIT',
##&nbsp;            'bootarg_libc_nameservices' => 'true',
##&nbsp;            'last_os_booted_raid_ver' => '12',
##&nbsp;            'hint_atkbdc_0_at' => 'isa',
##&nbsp;            'ntap_init_cfdevice' => '/dev/ad0s2',
##&nbsp;            'bootarg_vnvram_size' => '256',
##&nbsp;            'interpret' => 'OK',
##&nbsp;            'bootarg_nvram_sysid' => '4073324735',
##&nbsp;            'smbios_memory_enabled' => '5242880',
##&nbsp;            'bootarg_init_dhcp_disable' => 'true',
##&nbsp;            'smbios_bios_vendor' => 'Phoenix Technologies LTD',
##&nbsp;            'smbios_planar_product' => '440BX Desktop Reference Platform',
##&nbsp;            'bootarg_init_wipeclean' => 'false',
##&nbsp;            'bootarg_dblade_root_volume_local_space' => '2781_MB',
##&nbsp;            'bootarg_init_cfdevice' => '/dev/ad0s2',
##&nbsp;            'smbios_planar_serial' => 'None',
##&nbsp;            'mobo_serial_num' => '999999',
##&nbsp;            'console' => 'comconsole',
##&nbsp;            'hint_atkbdc_0_port' => '0x060',
##&nbsp;            'bootarg_init_cf_mounted' => 'true',
##&nbsp;            'smbios_chassis_maker' => 'No Enclosure',
##&nbsp;            'hint_atrtc_0_irq' => '8',
##&nbsp;            'bootarg_mgwd_nblade_uuid' => 'ab7cb023-fd77-11e2-b9e8-717c7aff181e',
##&nbsp;            'bootarg_init_boot_clustered' => 'true',
##&nbsp;            'bootarg_setup_auto_internal' => 'true',
##&nbsp;            'bootarg_login_running' => 'true',
##&nbsp;            'bootarg_srm_nvram_vnvram' => 'true',
##&nbsp;            'boardname' => 'VMWARE',
##&nbsp;            'hint_atrtc_0_at' => 'isa',
##&nbsp;            'bootarg_srm_virtual_adapter' => 'true',
##&nbsp;            'sys_rev' => 'ZZ',
##&nbsp;            'smbios_system_uuid' => '564d5d6c-6265-0c83-e2c4-e0aa9199aa83',
##&nbsp;            'wafl_disable_mbuf_backed_buffers' => 'TRUE',
##&nbsp;            'boot_file' => 'x86_64/freebsd/image1/kernel',
##&nbsp;            'sysvar_init_populate_mroot_late_done' => 'true',
##&nbsp;            'bootarg_init_cfimagebase' => '/cfcard/x86_64/freebsd',
##&nbsp;            'hint_uart_0_port' => '0x3F8',
##&nbsp;            'bootarg_init_rootimage' => '/cfcard/x86_64/freebsd/image1/rootfs.img',
##&nbsp;            'smbios_chassis_tag' => 'No Asset Tag',
##&nbsp;            'hint_uart_1_at' => 'isa',
##&nbsp;            'hint_atrtc_0_port' => '0x70',
##&nbsp;            'sys_model' => 'SIMBOX',
##&nbsp;            'comconsole_speed' => '9600',
##&nbsp;            'loader_conf_files' => '/boot/device.hints /boot/loader.conf /boot/loader.conf.local',
##&nbsp;            'bootarg_vm_vnvram' => 'panic',
##&nbsp;            'netapp_primary_kernel_url' => 'x86_64/freebsd/image1/kernel',
##&nbsp;            'module_path' => '/boot/kernel;/boot/modules',
##&nbsp;            'smbios_system_serial' => 'VMware-56 4d 5d 6c 62 65 0c 83-e2 c4 e0 aa 91 99 aa 83',
##&nbsp;            'last_os_booted_ver' => 'devN_130803_0500',
##&nbsp;            'bootarg_dblade_root_volume_local_aggr_uuid' => '2f075439-4c80-4937-acb8-88fa9c0cd582',
##&nbsp;            'nvram_emulation' => 'true',
##&nbsp;            'smbios_bios_reldate' => '10/13/2009',
##&nbsp;            'bootarg_vm_sim' => 'true',
##&nbsp;            'bootarg_vm_data_diskmodel' => 'vscsi',
##&nbsp;            'smbios_planar_version' => 'None',
##&nbsp;            'bootarg_srm_disk_simulated' => 'false',
##&nbsp;            'smbios_socket_enabled' => '2',
##&nbsp;            'bootarg_bsdportname' => 'e0c',
##&nbsp;            'kernel_options' => '',
##&nbsp;            'lines' => '24',
##&nbsp;            'hint_atkbd_0_irq' => '1',
##&nbsp;            'smbios_system_maker' => 'VMware, Inc.',
##&nbsp;            'reboot_reason' => 'REBOOT_UNKNOWN',
##&nbsp;            'sysvar_ngsh_remote_ip' => '',
##&nbsp;            'booted_from' => 'PRIMARY',
##&nbsp;            'hint_uart_1_irq' => '3',
##&nbsp;            'bootarg_notifyd_optout' => 'false',
##&nbsp;            'smbios_chassis_version' => 'N/A',
##&nbsp;            'bootarg_mgwd_autoconf_disable' => 'true',
##&nbsp;            'bootfile' => 'kernel',
##&nbsp;            'hint_atkbd_0_at' => 'atkbdc',
##&nbsp;            'bootarg_bootmenu_selection' => '4a',
##&nbsp;            'bootarg_init_last_boot' => 'true',
##&nbsp;            'bootarg_mgwd_scsi_blade_uuid' => 'ab7cce7d-fd77-11e2-a315-ed1b290567fe',
##&nbsp;            'loader_version' => '1.0',
##&nbsp;            'hint_uart_0_flags' => '0x40',
##&nbsp;            'bootarg_init_booterr' => '0',
##&nbsp;            'bootarg_dblade_root_volume_local_name' => 'vol0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub kenv {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    my $count=0;
    foreach my $line (@lines) {
        my @values = split (/=/,$line);
        $values[0] =~ s/\.|-/_/g;
        $values[0] =~ s/\?//g;
        $values[1] =~ s/"//g;
        $result_array[$count]->{to_key_name( item => $values[0] )} = $values[1];
    }
   return \@result_array;
} ## end sub kenv

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error_message, $error ) = undef;
    if ( $raw_output =~ /kenv: unable to.*/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /([Uu]sage:|kenv:)\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

1;
