#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Freebsd/default/reboot.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse reboot command output
##
## @description
##    This is a library to parse the raw output of reboot command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::reboot;
##
##  reboot(%args);
##
## @author skotagir@netapp.com , dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::reboot;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::reboot
##
## @summary Parses the raw output from the "sudo reboot" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from reboot with any of its options as described in the
##    corresponding cdef file 'reboot.cdefs'
## @synopsis
##      reboot(%args);
## INPUTS TO METHOD CALL 'reboot' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub reboot {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub reboot

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error_message, $error ) = undef;
    if ( $raw_output =~ /Operation not permitted.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /([Uu]sage:|reboot:)\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

1;
