# Copyright (c) 2011 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse Freebsd pwd command
##
## @description
##    This is a library to parse the raw output of Freebsd pwd command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "pwd" will be pwd()
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::rmdir;
##
##    rmdir(%args);
##
## @author anerudh@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::rmdir;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);


## @name ::rmdir
##
## @summary Parses the raw output from the "rmdir" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from rmdir with any of its options as described in the
##    corresponding cdef file 'rmdir.cdefs'
## @synopsis
##      rmdir(%args);
## INPUTS TO METHOD CALL 'rmdir' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '--ignore-fail-on-non-empty' </td> <td> 'ignore_fail_on_non_empty' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'paths' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'parents' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = rmdir directoryname
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub rmdir {
    # currently does not support vserbose mode output

    my %args = @_;

    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $error_level = $response->get_execution_status();
    my $result;

    # To check if there is any error
    if($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text=>"Command failed: $raw_output",
            response_object => $response);
    }
    return [];
}

1;
