#
#
#Copyright (c) 2013 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse SystemShell sptool command
##
## @description
##    This is a library to parse the raw output of SystemShell sptool command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "sptool" will be sptool()
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::sptool;
##
##    sptool(%args);
##
## @author Veerabhadraswamy.P@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
package NACL::APISet::Host::CLI::Freebsd::default::sptool;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::InvalidParamException;

## @name ::sptool
##
## @summary Parses the raw output from the "sptool" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sptool with any of its options as described in the
##    corresponding cdef file 'sptool.cdefs'
## @synopsis
##      sptool(%args);
## INPUTS TO METHOD CALL 'sptool' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'boot-value' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'add-subcommand' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub sptool {

    my %args    = @_;
    my $resp    = $args{response_object};
    my $command = $resp->get_actual_command();
    my $output  = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );

    return [];
} ## end of sub sptool

sub _check_error {

    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error ) = undef;
    if ( $raw_output =~ /SP error: Invalid.*|Error opening dev.*/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /([Uu]sage:\s+sptool).*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

} ## end sub _check_error
1;

