#$Id$
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse ssram command output
##
## @description
##   This library supports error checking for ssram command, but output parsing is not supported as of now.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::ssram;
##
##  ssram(%args);
##
## @author veerap@netapp.com , dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::ssram;

use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::MissingValueException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;
use strict;
use warnings;

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /ssram: offset not specified|ssram: data size not specified/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /No such file or directory |ssram: cannot open/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /ssram: insufficent write data/ ) {
        $error         = "MissingValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

## @name ::ssram
##
## @summary Parses the raw output from the "sudo ssram" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ssram with any of its options as described in the
##    corresponding cdef file 'ssram.cdefs'
## @synopsis
##      ssram(%args);
## INPUTS TO METHOD CALL 'ssram' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-t' </td> <td> 'record-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-R' </td> <td> 'raw-read' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'set-mode' </td> <td> '1' </td> </tr>
## <tr> <td> '-g' </td> <td> 'region-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'validate-region' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'initialize-region' </td> <td> '1' </td> </tr>
## <tr> <td> '-o' </td> <td> 'offset' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'format-text' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'read-region' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'region' </td> <td> '1' </td> </tr>
## <tr> <td> '-x' </td> <td> 'data-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'region-header' </td> <td> '1' </td> </tr>
## <tr> <td> '-W' </td> <td> 'raw-write' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'log-string' </td> <td> '0' </td> </tr>
## <tr> <td> '-z' </td> <td> 'access-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-w' </td> <td> 'write-region' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns: Output parsing is not supported
## @examples

##&nbsp;Command = sudo ssram -v -g 2 -o 20 -x 2 -z 1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Region 2 is valid
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub ssram {
    my %args = @_;
    my (@result_array);
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    if ($error_level) {
        _check_error(
            raw_output      => $output,
            response_object => $resp,
        );
    }
    return [];
} ## end sub ssram
1;
