#$Id$
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse watchdog command output
##
## @description
##    This library supports error checking for watchdog command, but output parsing is not supported as of now.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::watchdog;
##
##    watchdog(%args);
##
## @author tanil@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Freebsd::default::watchdog;

use NACL::APISet::Exceptions::CommandFailedException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;
use strict;
use warnings;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error         = undef;
    if ($raw_output =~ /Usage:.*/) {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    }
    return $error;
} ## end sub _check_error

## @name :: watchdog
##
## @summary Parses the raw output from the "watchdog" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from scp with any of its options as described in the
##    corresponding cdef file 'watchdog.cdefs'
## @synopsis
##      watchdog(%args);
## INPUTS TO METHOD CALL 'watchdog' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td>'wd_reset' </td> <td> 'reset' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)

## @return
## Returns: Output parsing is not supported
## @examples

##&nbsp;Command = sudo watchdog wd_reset
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub watchdog{
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'watchdog',
                             "ptr_error_message" => \$error_message,);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub watchdog

1;
