#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/HPUX/default/insf.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse HPUX insf command
##
## @description
##    This is a library to parse the raw output of HPUX insf command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "insf" will insf()
##
## @synopsis
##    use insf;
##
##    insf(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::HPUX::default::insf;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::insf
##
## @summary Parses the raw output from the "insf" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from insf with any of its options as described in the
##    corresponding cdef file 'insf.cdefs'
## @synopsis
##      insf(%args);
## INPUTS TO METHOD CALL 'insf' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-q' </td> <td> 'quiet' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'npty' </td> <td> '0' </td> </tr>
## <tr> <td> '-H' </td> <td> 'hw-path' </td> <td> '0' </td> </tr>
## <tr> <td> '-I' </td> <td> 'instance' </td> <td> '0' </td> </tr>
## <tr> <td> '-D' </td> <td> 'directory' </td> <td> '0' </td> </tr>
## <tr> <td> '-L' </td> <td> 'legacy' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'nstrpty' </td> <td> '0' </td> </tr>
## <tr> <td> '-C' </td> <td> 'class' </td> <td> '0' </td> </tr>
## <tr> <td> '-e' </td> <td> 'exists' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'driver' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = insf -C disk
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub insf {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    return [];

} ## end sub insf

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error       = undef;
    my $error_level = $resp->get_execution_status();
    if ( $raw_output =~ /(insf:|Usage)\s+.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;

    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;

