#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/HPUX/default/ioinit.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse HPUX ioinit command
##
## @description
##    This is a library to parse the raw output of HPUX ioinit command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "ioinit" will ioinit()
##
## @synopsis
##    use ioinit;
##
##    ioinit(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::HPUX::default::ioinit;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

## @name ::ioinit
##
## @summary Parses the raw output from the "ioinit" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ioinit with any of its options as described in the
##    corresponding cdef file 'ioinit.cdefs'
## @synopsis
##      ioinit(%args);
## INPUTS TO METHOD CALL 'ioinit' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-A' </td> <td> 'critical-resource-analysis' </td> <td> '1' </td> </tr>
## <tr> <td> '-C' </td> <td> 'class-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'infile' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'recreate' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'invoke-insf-command' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'reboot-system' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'ioconfig-file' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
sub ioinit {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub ioinit

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ($raw_output =~ /ioinit: verification.*failed/) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }elsif ($raw_output =~ /([Uu]sage:)\s+.*|ioinit: illegal.*/ ){
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /ioinit: Unable.*/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }
} ## end sub _check_error

1;

