#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux flg command output
##
## @description
##    This is a library to parse the raw output of Linux flg command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Linux::flg;
##
##    flg(%args);
##
## @author Abhsihek Kumar(kabhishe@netapp.com), dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::flg;

use strict;
use warnings;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);


## @name ::flg
##
## @summary Parses the raw output from the "flg" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from flg with any of its options as described in the
##    corresponding cdef file 'flg.cdefs'
## @synopsis
##      flg(%args);
## INPUTS TO METHOD CALL 'flg' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-maxlat' </td> <td> 'maxlat' </td> </tr>
## <tr> <td> '-contfile' </td> <td> 'contfile' </td> </tr>
## <tr> <td> '-rexmitage' </td> <td> 'rexmitage' </td> </tr>
## <tr> <td> '-rate' </td> <td> 'rate' </td> </tr>
## <tr> <td> '-invalfh' </td> <td> 'invalfh' </td> </tr>
## <tr> <td> '-random_uids' </td> <td> 'random-uids' </td> </tr>
## <tr> <td> '-random_gids' </td> <td> 'random-gids' </td> </tr>
## <tr> <td> '-savefile' </td> <td> 'savefile' </td> </tr>
## <tr> <td> '-nclients' </td> <td> 'nclients' </td> </tr>
## <tr> <td> '-logfile' </td> <td> 'logfile' </td> </tr>
## <tr> <td> '-maxinuse' </td> <td> 'maxinuse' </td> </tr>
## <tr> <td> '-rexmitmax' </td> <td> 'rexmitmax' </td> </tr>
## <tr> <td> '-transp' </td> <td> 'trans-protocol' </td> </tr>
## <tr> <td> '-maxios' </td> <td> 'maxios' </td> </tr>
## <tr> <td> '-enospc_nofail' </td> <td> 'enospc-nofail' </td> </tr>
## <tr> <td> '-nsprefix' </td> <td> 'nsprefix' </td> </tr>
## <tr> <td> '-host' </td> <td> 'path' </td> </tr>
## <tr> <td> '-duration' </td> <td> 'duration' </td> </tr>
## <tr> <td> '-quitfile' </td> <td> 'quitfile' </td> </tr>
## <tr> <td> '-rusage' </td> <td> 'rusage' </td> </tr>
## <tr> <td> '-rsizedist' </td> <td> 'rsizedist' </td> </tr>
## <tr> <td> '-wsizedist' </td> <td> 'wsizedist' </td> </tr>
## <tr> <td> '-opdist' </td> <td> 'opdist' </td> </tr>
## <tr> <td> '-maxops' </td> <td> 'maxops' </td> </tr>
## <tr> <td> '-workload' </td> <td> 'workload' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = flg -host 10.238.3.164:/vol/vol0 -duration 5
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub flg {
    my %args = @_;
    
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    
    if($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(text=>$output,response_object => $resp);
    }
    return [];
}

1;
