# $Id$
#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux ln command output
##
## @description
##    This library supports error checking for kenv command, but output parsing is not supported as of now.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Linux::ln;
##
##    ln(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::ln;

use strict;
use warnings;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);


## @name ::ln
##
## @summary Parses the raw output from the "ln" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ln with any of its options as described in the 
##    corresponding cdef file 'ln.cdefs'
## @synopsis
##      ln(%args);
## INPUTS TO METHOD CALL 'ln' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-d' </td> <td> 'allow-directory' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'no-deference' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'files' </td> </tr> 
## <tr> <td> '-T' </td> <td> 'no-target-directory' </td> </tr> 
## <tr> <td> '--backup' </td> <td> 'backup-control' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'force' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'symlink' </td> </tr> 
## <tr> <td> '-S' </td> <td> 'suffix' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'target-directory' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'link-name' </td> </tr> 
## <tr> <td> '-b' </td> <td> 'backup' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
##  Returns: Output parsing is not supported
## @examples

sub ln {
    my %args = @_;
    
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    
    if($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(text=>$output,response_object => $resp);
    }
    return [];
}

1;
