#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Linux/Redhat/ls.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary ls command output
##
## @description
##This library supports error checking for kenv command, but output parsing is not supported as of now
## @synopsis
##    use NACL::APISet::Host::CLI::Linux::Redhat::ls;
##
##    ls(%args);
##
## @author skotagir@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Host::CLI::Linux::Redhat::ls;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidOptionException qw(:try);
use strict;

sub _check_error {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $raw_output  = $args{raw_output};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /invalid option|unrecognized option|No such file or directory/ ) {
        $error         = "InvalidOption";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /invalid (block size|tab size|argument)/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /option.*requires an argument/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => $raw_output,
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

## @name ::ls
##
## @summary Parses the raw output from the "ls" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ls with any of its options as described in the
##    corresponding cdef file 'ls.cdefs'
## @synopsis
##      ls(%args);
## INPUTS TO METHOD CALL 'ls' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-A' </td> <td> 'almost-all' </td> <td> '1' </td> </tr>
## <tr> <td> '-C' </td> <td> 'list-by-column' </td> <td> '1' </td> </tr>
## <tr> <td> '-S' </td> <td> 'sort-by-size' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'directory' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'paths' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'no-group-information' </td> <td> '1' </td> </tr>
## <tr> <td> '-N' </td> <td> 'literal' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'long-list' </td> <td> '1' </td> </tr>
## <tr> <td> '-g' </td> <td> 'do-not-list-owner' </td> <td> '1' </td> </tr>
## <tr> <td> '-L' </td> <td> 'dereference' </td> <td> '1' </td> </tr>
## <tr> <td> '-G' </td> <td> 'do-not-list-group' </td> <td> '1' </td> </tr>
## <tr> <td> '-Q' </td> <td> 'enclose-in-quotes' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'comma-separated' </td> <td> '1' </td> </tr>
## <tr> <td> '-F' </td> <td> 'classify' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'size-in-blocks' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'escape' </td> <td> '1' </td> </tr>
## <tr> <td> '-1' </td> <td> 'one-per-line' </td> <td> '1' </td> </tr>
## <tr> <td> '-q' </td> <td> 'hide-control-chars' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'human-readable' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'inode' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'reverse' </td> <td> '1' </td> </tr>
## <tr> <td> '-H' </td> <td> 'follow-symbolic-links' </td> <td> '1' </td> </tr>
## <tr> <td> '-R' </td> <td> 'recursive' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'numeric-uid-gid' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'sort-by-modification-time' </td> <td> '1' </td> </tr>
## <tr> <td> '-X' </td> <td> 'sort-alphabetically-by-extension' </td> <td> '1' </td> </tr>
## <tr> <td> '-B' </td> <td> 'ignore-backups' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
##  Returns: Output parsing is not supported
## @examples

sub ls {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output,
                 "command" => $command,
                 response_object => $resp );
    return [];
} ## end sub ls
1;

