# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Linux/Redhat/lsmod.pm#1 $
#
# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse Linux lsmod command output
##
## @description
##    This is a library to parse the raw output of Linux lsmod command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Linux::lsmod;
##
##    lsmod(%args);
##
## @author sangavai@netapp.con, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::lsmod;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

## @name ::lsmod
##
## @summary Parses the raw output from the "lsmod" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from lsmod with any of its options as described in the 
##    corresponding cdef file 'lsmod.cdefs'
## @synopsis
##      lsmod(%args);
## INPUTS TO METHOD CALL 'lsmod' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = lsmod
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Module                  Size  Used by
##&nbsp;ib_iser                69569  0 
##&nbsp;rdma_cm                68437  1 ib_iser
##&nbsp;ib_cm                  72809  1 rdma_cm
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'used_by_modules_count' => '0',
##&nbsp;            'used_by_modules' => [
##&nbsp;                                   undef
##&nbsp;                                 ],
##&nbsp;            'size' => '69569',
##&nbsp;            'module' => 'ib_iser'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'used_by_modules_count' => '1',
##&nbsp;            'used_by_modules' => [
##&nbsp;                                   'ib_iser'
##&nbsp;                                 ],
##&nbsp;            'size' => '68437',
##&nbsp;            'module' => 'rdma_cm'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'used_by_modules_count' => '1',
##&nbsp;            'used_by_modules' => [
##&nbsp;                                   'rdma_cm'
##&nbsp;                                 ],
##&nbsp;            'size' => '72809',
##&nbsp;            'module' => 'ib_cm'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub lsmod {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my @result_array;
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $output,
            response_object => $resp
        );
    }
    my @lines = get_lines( response_object => $resp, raw_output => $output );
    my @head = split( /\s+/, $lines[0] );
    for ( my $i = 1; $i <= $#lines; $i++ ) {
        my @values = split( /\s+/, $lines[$i] );
        for ( my $j = 0; $j < $#head; $j++ ) {
            $head[$j] = "used_by_modules_count" if($head[$j] =~ /Used/);
            $result_array[ $i - 1 ]->{ to_key_name( item => $head[$j] ) }
                = $values[$j];
        }
        my @used;
        if($#head == $#values) {
            @used = split( /\,/, $values[-1] );
        } else {
            $used[0] = undef;
        }
        $head[-1] = "used_by_modules" if($head[-1] =~ /by/);
        for ( my $k = 0; $k <= $#used; $k++ ) {
            $result_array[ $i - 1 ]->{ to_key_name( item => $head[-1] ) }
                ->[$k] = $used[$k];
        }
    }
    return \@result_array;
} ## end sub lsmod

1;
