# $Id: //depot/prod/test/main/lib/NACL/APISet/Host/CLI/Linux/Redhat/lvm_version.pm#1
#
#Copyright (c) 2012 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse lvm_version command
##
## @description
##    This is a library to parse the raw output of the lvm_version command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "lvm version" will be lvm_version
##
## @synopsis
##    use  NACL::APISet::Host::CLI::Linux::Redhat::lvm_version;
##
##    lvm_version(%args);
##
## @author saparna Aparna.S2@netapp.com
##
## @status public
#
################################################################################

package NACL::APISet::Host::CLI::Linux::Redhat::lvm_version;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Import exceptions
use NACL::APISet::Exceptions::CommandNotFoundException qw(:try);

## @name ::lvm_version
##
## @summary Parses the raw output from the "lvm version" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from lvm_version with any of its options as described in the
##    corresponding cdef file 'lvm_version.cdefs'
## @synopsis
##      lvm_version(%args);
## INPUTS TO METHOD CALL 'lvm_version' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = lvm version
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  LVM version:     2.02.87(2)-RHEL6 (2011-10-12)
##&nbsp;  Library version: 1.02.66-RHEL6 (2011-10-12)
##&nbsp;  Driver version:  4.22.6
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'library_version' => '1.02.66-RHEL6 (2011-10-12)',
##&nbsp;            'driver_version' => '4.22.6',
##&nbsp;            'lvm_version' => '2.02.87(2)-RHEL6 (2011-10-12)'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub lvm_version {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();

    _check_error( raw_output => $output, response_object => $resp );

    my ( @result_array, @heads, @value );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@lines) {
        @heads = split( ':\s+', $line );
        $result_array[0]->{ to_key_name( item => $heads[0] ) } = $heads[1];
    }
    return \@result_array;
} ## end sub lvm_version

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    if ( $raw_output =~ /command not found/ ) {
        NACL::APISet::Exceptions::CommandNotFoundException->throw(
            text            => $raw_output,
            response_object => $resp
        );
    }
} ## end sub _check_error

1;

