#Copyright (c) 2013 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse mkfile command output
##
## @description
##    This is a library to parse the raw output of Linux mkfile command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Linux::Redhat::mkfile;
##
##    mkfile(%args);
##
## @author sheethal@netapp.com,dl-nacl-dev;
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::mkfile;

use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Response::CLI::ParserUtils;
use strict;

## @name ::mkfile
##
## @summary Parses the raw output from the "mkfile" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mkfile with any of its options as described in the
##    corresponding cdef file 'mkfile.cdefs'
## @synopsis
##      mkfile(%args);
## INPUTS TO METHOD CALL 'mkfile' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'filename' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'size' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub mkfile {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error(raw_output => $output, response_object => $resp);
    return [];
} ## end sub mkfile

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ($error, $error_message) = undef;
    if ($raw_output =~ /(Usage:)/) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $raw_output,
            response_object => $resp
        );
    }
} ## end sub _check_error

1;
