# Hey emacs, this is basically -*- perl -*-
#$Id:
#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse  ndmpcopy oftap utility
##
## @description
##    This is a library to parse the raw outputs of ndmpcopy oftap utility
##    The function name will be the same as command name.
##    For Example, the parser function for "ndmpcopy" will be ndmpcopy(%args)
##
## @synopsis
##    use ndmpcopy;
##
##    ndmpcopy(%args);
##
## @author c-dl-nb-tape-qa-functional, dl-nacl-dev
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::ndmpcopy;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

## @name ::ndmpcopy
##
## @summary Parses the raw output from the "ndmpcopy" oftap utility
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ndmpcopy with any of its options as described in the
##    corresponding cdef file 'ndmpcopy.cdefs'
## @synopsis
##      ndmpcopy(%args);
## INPUTS TO METHOD CALL 'ndmpcopy' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> PROGRAMMATIC NAME </th> <th> OPTION </th> </tr>
## <tr> <th> source-filer-authentication' </td> <td> '-sa' </td> </tr>
## <tr> <th> destination-filer-authentication' </td> <td> '-da' </td> </tr>
## <tr> <th> all-authentication' </td> <td> '-aa' </td> </tr>
## <tr> <th> source-filer-authentication-type' </td> <td> '-st' </td> </tr>
## <tr> <th> destination-filer-authentication-type' </td> <td> '-dt' </td> </tr>
## <tr> <th> all-authentication-type' </td> <td> '-at' </td> </tr>
## <tr> <th> source-control-connection-mode' </td> <td> '-mcs' </td> </tr>
## <tr> <th> destination-control-connection-mode' </td> <td> '-mcd' </td> </tr>
## <tr> <th> all-control-connection-mode' </td> <td> '-mcd' </td> </tr>
## <tr> <th> incremental-level' </td> <td> '-l' </td> </tr>
## <tr> <th> debug-mode' </td> <td> '-d' </td> </tr>
## <tr> <th> help-message' </td> <td> '-h' </td> </tr>
## <tr> <th> advance-help-message' </td> <td> ' -H' </td> </tr>
## <tr> <th> append-debug-traces-file' </td> <td> 'no swith for this key' </td> </tr>
## <tr> <th> engine' </td> <td> 'no switch for this key </td> </tr>
## <tr> <th> backup-null' </td> <td> '-null' </td> </tr>
## <tr> <th> file-history' </td> <td> '-fh' </td> </tr>
## <tr> <th> env-variables' </td> <td> ' -e' </td> </tr>
## <tr> <th> print-file-history' </td> <td> '-p' </td> </tr>
## <tr> <th> data-connection-mode' </td> <td> '-mca' </td> </tr>
## <tr> <th> unicode-file-history' </td> <td> '-fhu' </td> </tr>
## <tr> <th> file-history-interval' </td> <td> '-internval' </td> </tr>
## <tr> <th> file-history-sleep' </td> <td> '-sleep' </td> </tr>
## <tr> <th> no-cab' </td> <td> '-nocab' </td> </tr>
## <tr> <th> secure-ndmp' </td> <td> '-sec' </td> </tr>
## <tr> <th> backup-type' </td> <td> '-t' </td> </tr>
## <tr> <th> source' </td> <td> 'no switch for this key' </td> </tr>
## <tr> <th> destination' </td> <td> 'no switch for this key' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub ndmpcopy {
    my %args = @_;

    my $error_message;
    my $response     = $args{response_object};
    my $raw_output   = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $result;
    my $error = _check_error(
        "raw_output"        => $raw_output,
        "command"           => $command_line,
        "response_object"   => $response
    );
    # Now we can parse the output
    $result = _ndmpcopy(
        raw_output      => $raw_output,
        command         => $command_line,
        response_object => $response
    );
    return $result;
} ## end sub ndmpcopy

sub _ndmpcopy {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my ( %res_hash, @result_array, $help_message );

    if ( $command =~ /ndmpcopy\s+?-h/ ) {
        ($help_message)
            = ( $raw_output =~ /(usage:[^#]+?display this message)/i );
        $res_hash{"help_output"} = $help_message;
        return ( [\%res_hash] );
    }

    if ( $raw_output =~ /Starting copy/i ) {

        #Parsing the output for getting overall Ndmpcopy command status
        ( $res_hash{"ndpmcopy_status"} )
            = ( $raw_output =~ /(Transfer\s+?.+?)\s+\[/i );

        #Parsing the raw output for getting time taken for Ndmpcopy
        ( $res_hash{"ndmpcopy_time_taken"} )
            = ( $raw_output =~ /Transfer\s+?.+?\s+\[\s(.+?)\s\]\n/i );

        #Parsing the raw output for getting dump status on the source side
        ( $res_hash{"dump_status"} )
            = ( $raw_output =~ /Notify:\s(dump.+?)\nNdmpcopy:/i );

        #Parsing the raw output for getting dump type on the source side
        ( $res_hash{"dump_type"} )
            = ( $raw_output =~ /DUMP:\s+Using\s(.+?)\nNdmpcopy:/i );

#Parsing the raw output for getting the restore status on the destination side
        ( $res_hash{"restore_status"} )
            = ( $raw_output =~ /Notify:\s(restore.+?)\nNdmpcopy:/i );

        #Parsing the raw output for getting the snapshot name
        ( $res_hash{"snapshot_name"} )
            = ( $raw_output =~ /DUMP: creating ".*?\.\.\/(.+?)" snapshot/i );

        #Parsing the raw output for getting estiated dump data size
        ( $res_hash{"estimated_dump_data"} )
            = ( $raw_output =~ /DUMP: estimated (\d+? KB)/i );

        #Parsing the raw output for getting actual dumped data size
        ( $res_hash{"actual_dump_data"} )
            = ( $raw_output =~ /DUMP: (\d+? KB)/i );

    } else {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw(
            text            => "Output is not in the expected format",
            response_object => $response
        );
    }

    return ( [\%res_hash] );
} ## end sub _ndmpcopy

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
	my $response      = $args{"response_object"};

    if ( $raw_output =~ /(Invalid syntax.*)/i ) {
		NACL::APISet::Exceptions::MissingRequiredParamException->throw(response_obj => $response, error_text => $raw_output);
    } elsif (
        ( $raw_output    =~ /(Ndmpcopy:.*?Error.*)/i )
        or ( $raw_output =~ /(Ndmpcopy:.*?failed.*)/i )
        or ( $raw_output =~ /(Received interrupt.*)/i )
        or ( $raw_output =~ /(Could not create NDMP connection)/i )
        or ( $raw_output =~ /(Client authentication request failed)/i )
        or ( $raw_output =~ /(Version negotiation failed)/i )
        or ( $raw_output =~ /(Authentication failed.*)/i )
        or ( $raw_output =~ /(Failed to start.*)/i )
        or ( $raw_output =~ /(User requesting abort)/i )
        or ( $raw_output =~ /(kmem_zalloc failed)/i )
        or ( $raw_output =~ /(Notify:\sTransfer failed)/i )
        or ( $raw_output =~ /(Socket connection to source\s.*?failed)/i )
        or ( $raw_output
            =~ /(Issue 'ndmpd on'.+?NDMP request then retry the connection)/i
        )
        ) {
		NACL::APISet::Exceptions::CommandFailedException->throw(response_obj => $response, error_text => $raw_output);
    } elsif ( $raw_output =~ /usage:/ && !( $command =~ /ndmpcopy\s+?-h/i ) )
    {
        NACL::APISet::Exceptions::CommandFailedException->throw(response_obj => $response, error_text => $raw_output);
	}
} ## end sub _check_error

1;

