#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Linux/Redhat/pvscan.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary pvscan command output
##
## @description
##    This is a library to parse the raw output of pvscan command.
##
## @synopsis
##    use pvscan;
##
##    pvscan(%args);
##
## @author sangavai@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Host::CLI::Linux::Redhat::pvscan;

use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

use strict;

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my $error_message;
    my $error_level = $resp->get_execution_status();
    my $error       = undef;
    if ( $raw_output =~ /invalid option/ ) {
        $error         = "InvalidParam";
        $error_message = "Invalid Options are used in the Command";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $raw_output,
            response_object => $resp
        );
    }
} ## end sub _check_error

## @name ::pvscan
##
## @summary Parses the raw output from the "pvscan" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from pvscan with any of its options as described in the
##    corresponding cdef file 'pvscan.cdefs'
## @synopsis
##      pvscan(%args);
## INPUTS TO METHOD CALL 'pvscan' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'short' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'novolgrp' </td> <td> '1' </td> </tr>
## <tr> <td> '--ignorelockingfailure' </td> <td> 'ignorelockfail' </td> <td> '1' </td> </tr>
## <tr> <td> '-e' </td> <td> 'exported' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr>
## <tr> <td> '-u' </td> <td> 'uuid' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = pvscan
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  PV /dev/sda4   VG vol_grp1        lvm2 [40.26 GB / 40.26 GB free]
##&nbsp;  PV /dev/sdb2                      lvm2 [147.05 GB]
##&nbsp;  Total: 2 [187.31 GB] / in use: 1 [40.26 GB] / in no VG: 1 [147.05 GB]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lvm' => 'lvm2',
##&nbsp;            'physical_volume' => '/dev/sda4 ',
##&nbsp;            'free_space' => '40.26 GB free',
##&nbsp;            'available_space' => '40.26 GB',
##&nbsp;            'volume_group' => 'vol_grp1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lvm' => 'lvm2',
##&nbsp;            'physical_volume' => '/dev/sdb2',
##&nbsp;            'available_space' => '147.05 GB'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'in_no_vg' => {
##&nbsp;                            'physical_volumes' => '1',
##&nbsp;                            'space' => '147.05'
##&nbsp;                          },
##&nbsp;            'in_use' => {
##&nbsp;                          'physical_volumes' => '1',
##&nbsp;                          'space' => '40.26'
##&nbsp;                        },
##&nbsp;            'total' => {
##&nbsp;                         'physical_volumes' => '2',
##&nbsp;                         'space' => '187.31'
##&nbsp;                       }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pvscan -d
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  PV /dev/sda4   VG vol_grp1        lvm2 [40.26 GB / 40.26 GB free]
##&nbsp;  PV /dev/sdb2                      lvm2 [147.05 GB]
##&nbsp;  Total: 2 [187.31 GB] / in use: 1 [40.26 GB] / in no VG: 1 [147.05 GB]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lvm' => 'lvm2',
##&nbsp;            'physical_volume' => '/dev/sda4 ',
##&nbsp;            'free_space' => '40.26 GB free',
##&nbsp;            'available_space' => '40.26 GB',
##&nbsp;            'volume_group' => 'vol_grp1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lvm' => 'lvm2',
##&nbsp;            'physical_volume' => '/dev/sdb2',
##&nbsp;            'available_space' => '147.05 GB'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'in_no_vg' => {
##&nbsp;                            'physical_volumes' => '1',
##&nbsp;                            'space' => '147.05'
##&nbsp;                          },
##&nbsp;            'in_use' => {
##&nbsp;                          'physical_volumes' => '1',
##&nbsp;                          'space' => '40.26'
##&nbsp;                        },
##&nbsp;            'total' => {
##&nbsp;                         'physical_volumes' => '2',
##&nbsp;                         'space' => '187.31'
##&nbsp;                       }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pvscan --ignorelockingfailure
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  PV /dev/sda4   VG vol_grp1        lvm2 [40.26 GB / 40.26 GB free]
##&nbsp;  PV /dev/sdb2                      lvm2 [147.05 GB]
##&nbsp;  Total: 2 [187.31 GB] / in use: 1 [40.26 GB] / in no VG: 1 [147.05 GB]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lvm' => 'lvm2',
##&nbsp;            'physical_volume' => '/dev/sda4 ',
##&nbsp;            'free_space' => '40.26 GB free',
##&nbsp;            'available_space' => '40.26 GB',
##&nbsp;            'volume_group' => 'vol_grp1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lvm' => 'lvm2',
##&nbsp;            'physical_volume' => '/dev/sdb2',
##&nbsp;            'available_space' => '147.05 GB'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'in_no_vg' => {
##&nbsp;                            'physical_volumes' => '1',
##&nbsp;                            'space' => '147.05'
##&nbsp;                          },
##&nbsp;            'in_use' => {
##&nbsp;                          'physical_volumes' => '1',
##&nbsp;                          'space' => '40.26'
##&nbsp;                        },
##&nbsp;            'total' => {
##&nbsp;                         'physical_volumes' => '2',
##&nbsp;                         'space' => '187.31'
##&nbsp;                       }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pvscan -n
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  WARNING: only considering physical volumes in no volume group
##&nbsp;  PV /dev/sdb2                      lvm2 [147.05 GB]
##&nbsp;  Total: 1 [147.05 GB] / in use: 0 [0   ] / in no VG: 1 [147.05 GB]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lvm' => 'lvm2',
##&nbsp;            'physical_volume' => '/dev/sdb2',
##&nbsp;            'available_space' => '147.05 GB'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'in_no_vg' => {
##&nbsp;                            'physical_volumes' => '1',
##&nbsp;                            'space' => '147.05'
##&nbsp;                          },
##&nbsp;            'in_use' => {
##&nbsp;                          'physical_volumes' => '0',
##&nbsp;                          'space' => '0'
##&nbsp;                        },
##&nbsp;            'total' => {
##&nbsp;                         'physical_volumes' => '1',
##&nbsp;                         'space' => '147.05'
##&nbsp;                       }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pvscan -u
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  PV /dev/sda4 with UUID ioJOeh-2z6s-b3HZ-OHoD-1zHR-qRb8-gwUaL2 VG vol_grp1        lvm2 [40.26 GB / 40.26 GB free]
##&nbsp;  PV /dev/sdb2 with UUID 33kise-jXRT-DLvp-nusl-4FSy-Zq5H-goOfE7                    lvm2 [147.05 GB]
##&nbsp;  Total: 2 [187.31 GB] / in use: 1 [40.26 GB] / in no VG: 1 [147.05 GB]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lvm' => 'lvm2',
##&nbsp;            'physical_volume' => '/dev/sda4 with',
##&nbsp;            'free_space' => '40.26 GB free',
##&nbsp;            'available_space' => '40.26 GB',
##&nbsp;            'volume_group' => 'vol_grp1',
##&nbsp;            'uuid' => 'ioJOeh-2z6s-b3HZ-OHoD-1zHR-qRb8-gwUaL2'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'in_no_vg' => {
##&nbsp;                            'physical_volumes' => '1',
##&nbsp;                            'space' => '147.05'
##&nbsp;                          },
##&nbsp;            'in_use' => {
##&nbsp;                          'physical_volumes' => '1',
##&nbsp;                          'space' => '40.26'
##&nbsp;                        },
##&nbsp;            'total' => {
##&nbsp;                         'physical_volumes' => '2',
##&nbsp;                         'space' => '187.31'
##&nbsp;                       }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pvscan -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;    Wiping cache of LVM-capable devices
##&nbsp;    Wiping internal VG cache
##&nbsp;    Walking through all physical volumes
##&nbsp;  PV /dev/sda4   VG vol_grp1        lvm2 [40.26 GB / 40.26 GB free]
##&nbsp;  PV /dev/sdb2                      lvm2 [147.05 GB]
##&nbsp;  Total: 2 [187.31 GB] / in use: 1 [40.26 GB] / in no VG: 1 [147.05 GB]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'lvm' => 'lvm2',
##&nbsp;            'physical_volume' => '/dev/sda4 ',
##&nbsp;            'free_space' => '40.26 GB free',
##&nbsp;            'available_space' => '40.26 GB',
##&nbsp;            'volume_group' => 'vol_grp1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lvm' => 'lvm2',
##&nbsp;            'physical_volume' => '/dev/sdb2',
##&nbsp;            'available_space' => '147.05 GB'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'in_no_vg' => {
##&nbsp;                            'physical_volumes' => '1',
##&nbsp;                            'space' => '147.05'
##&nbsp;                          },
##&nbsp;            'in_use' => {
##&nbsp;                          'physical_volumes' => '1',
##&nbsp;                          'space' => '40.26'
##&nbsp;                        },
##&nbsp;            'total' => {
##&nbsp;                         'physical_volumes' => '2',
##&nbsp;                         'space' => '187.31'
##&nbsp;                       }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pvscan -s
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  /dev/sda4
##&nbsp;  /dev/sdb2
##&nbsp;  Total: 2 [187.31 GB] / in use: 1 [40.26 GB] / in no VG: 1 [147.05 GB]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'physical_volume' => '/dev/sda4'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'physical_volume' => '/dev/sdb2'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'in_no_vg' => {
##&nbsp;                            'physical_volumes' => '1',
##&nbsp;                            'space' => '147.05'
##&nbsp;                          },
##&nbsp;            'in_use' => {
##&nbsp;                          'physical_volumes' => '1',
##&nbsp;                          'space' => '40.26'
##&nbsp;                        },
##&nbsp;            'total' => {
##&nbsp;                         'physical_volumes' => '2',
##&nbsp;                         'space' => '187.31'
##&nbsp;                       }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pvscan -u
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  PV /dev/sda4 with UUID ioJOeh-2z6s-b3HZ-OHoD-1zHR-qRb8-gwUaL2  is in exported VG vol_grp1 [40.26 GB / 40.26 GB free]
##&nbsp;  PV /dev/sdb2 with UUID 33kise-jXRT-DLvp-nusl-4FSy-Zq5H-goOfE7                    lvm2 [147.05 GB]
##&nbsp;  Total: 2 [187.31 GB] / in use: 1 [40.26 GB] / in no VG: 1 [147.05 GB]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'physical_volume' => '/dev/sda4',
##&nbsp;            'free_space' => '40.26 GB free',
##&nbsp;            'available_space' => '40.26 GB',
##&nbsp;            'volume_group' => 'vol_grp1',
##&nbsp;            'uuid' => 'ioJOeh-2z6s-b3HZ-OHoD-1zHR-qRb8-gwUaL2',
##&nbsp;            'volgrp_status' => 'exported'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lvm' => 'lvm2',
##&nbsp;            'physical_volume' => '/dev/sdb2',
##&nbsp;            'available_space' => '147.05 GB',
##&nbsp;            'uuid' => '33kise-jXRT-DLvp-nusl-4FSy-Zq5H-goOfE7'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'in_no_vg' => {
##&nbsp;                            'physical_volumes' => '1',
##&nbsp;                            'space' => '147.05'
##&nbsp;                          },
##&nbsp;            'in_use' => {
##&nbsp;                          'physical_volumes' => '1',
##&nbsp;                          'space' => '40.26'
##&nbsp;                        },
##&nbsp;            'total' => {
##&nbsp;                         'physical_volumes' => '2',
##&nbsp;                         'space' => '187.31'
##&nbsp;                       }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pvscan -e
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  WARNING: only considering physical volumes of exported volume group(s)
##&nbsp;  PV /dev/sda4    is in exported VG vol_grp1 [40.26 GB / 40.26 GB free]
##&nbsp;  Total: 1 [40.26 GB] / in use: 1 [40.26 GB] / in no VG: 0 [0   ]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'physical_volume' => '/dev/sda4',
##&nbsp;            'free_space' => '40.26 GB free',
##&nbsp;            'available_space' => '40.26 GB',
##&nbsp;            'volume_group' => 'vol_grp1',
##&nbsp;            'volgrp_status' => 'exported'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'in_no_vg' => {
##&nbsp;                            'physical_volumes' => '0',
##&nbsp;                            'space' => '0'
##&nbsp;                          },
##&nbsp;            'in_use' => {
##&nbsp;                          'physical_volumes' => '1',
##&nbsp;                          'space' => '40.26'
##&nbsp;                        },
##&nbsp;            'total' => {
##&nbsp;                         'physical_volumes' => '1',
##&nbsp;                         'space' => '40.26'
##&nbsp;                       }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pvscan -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;    Wiping cache of LVM-capable devices
##&nbsp;    Wiping internal VG cache
##&nbsp;    Walking through all physical volumes
##&nbsp;  PV /dev/sda4    is in exported VG vol_grp1 [40.26 GB / 40.26 GB free]
##&nbsp;  PV /dev/sdb2                      lvm2 [147.05 GB]
##&nbsp;  Total: 2 [187.31 GB] / in use: 1 [40.26 GB] / in no VG: 1 [147.05 GB]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'physical_volume' => '/dev/sda4',
##&nbsp;            'free_space' => '40.26 GB free',
##&nbsp;            'available_space' => '40.26 GB',
##&nbsp;            'volume_group' => 'vol_grp1',
##&nbsp;            'volgrp_status' => 'exported'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'lvm' => 'lvm2',
##&nbsp;            'physical_volume' => '/dev/sdb2',
##&nbsp;            'available_space' => '147.05 GB'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'in_no_vg' => {
##&nbsp;                            'physical_volumes' => '1',
##&nbsp;                            'space' => '147.05'
##&nbsp;                          },
##&nbsp;            'in_use' => {
##&nbsp;                          'physical_volumes' => '1',
##&nbsp;                          'space' => '40.26'
##&nbsp;                        },
##&nbsp;            'total' => {
##&nbsp;                         'physical_volumes' => '2',
##&nbsp;                         'space' => '187.31'
##&nbsp;                       }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub pvscan {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );

    my $c = 0;
    foreach my $line (@lines) {
        if ( $line
            =~ /PV\s+(\/.*\/\w+)\s+(.*\s+)*VG\s+(.*)\s+(\w+.*\s+)*\[(.*)\s+\/\s+(.*)\]/
            ) {
            my @arr = ( $1, $3, $4, $5, $6 );
            $result_array[$c]->{physical_volume} = $arr[0];
            if ( $line =~ /export/ ) {
                $result_array[$c]->{volgrp_status} = "exported";
            }
            if ( $line =~ /UUID\s+((\w+-){6}\w+)\s+/ ) {
                $result_array[$c]->{uuid} = $1;
            }
            if($arr[1] =~ /\w+\s+(\w+)/) {
                $result_array[$c]->{lvm} = $1;
                $arr[1] =~ s/\s+$1//;
            }
            $arr[1] =~ s/\s+//;
            $result_array[$c]->{volume_group} = $arr[1];
            if ( $arr[2] ) {
                chomp( $arr[2] );
                $result_array[$c]->{lvm} = $arr[2];
            }
            $result_array[$c]->{available_space} = $arr[3];
            $result_array[$c]->{free_space}      = $arr[4];
            $c++;
        } elsif ( $line
            =~ /PV\s+(\/.*\/\w+)\s+(with UUID\s+.*\-\w+\s+)*(\w+.*)\s+\[(.*)\]/
            ) {
            $result_array[$c]->{physical_volume} = $1;
            my $temp1 = $3;
            my $temp2 = $4;
            if ( $line =~ /UUID\s+((\w+-){6}\w+)\s+/ ) {
                $result_array[$c]->{uuid} = $1;
            }
            $result_array[$c]->{lvm}             = $temp1;
            $result_array[$c]->{available_space} = $temp2;
            $c++;
        } elsif ( $line =~ /^(\/.*\/\w+)$/ ) {
            $result_array[$c]->{physical_volume} = $line;
            $c++;
        } elsif ( $line =~ /Total/ ) {
            my @temp = split( /\s+\/\s+/, $line );
            foreach my $var (@temp) {
                my @t1 = split( /:\s+/, $var );
                my @t2 = split( /\s+/,  $t1[1] );
                $t2[1] =~ s/\[//;
                $result_array[$c]->{ to_key_name( item => $t1[0] ) }
                    ->{physical_volumes} = $t2[0];
                $result_array[$c]->{ to_key_name( item => $t1[0] ) }->{space}
                    = $t2[1];
            }
        }
    }
    return \@result_array;
} ## end sub pvscan
1;

