#Copyright (c) 2014 Network Appliance, Inc.
#All rights reserved.
#
## @summary reboot command output
##
## @description
##    This is a library to parse the raw output of reboot command.
##
## @synopsis
##    use reboot;
##
##    reboot(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Host::CLI::Linux::Redhat::reboot;
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandNotFoundException;
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Response::CLI::ParserUtils;
use strict;
use warnings;

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my $error_level = $resp->get_execution_status();
    my $error = undef;

    if ( $raw_output =~ /[Uu]sage/ ) {
        $error         = "MissingRequiredParam";
    } elsif ( $raw_output =~ /command not found.*/i ) {
        $error         = "CommandNotFound";
    } elsif ( $raw_output =~ /Need to be root/i ) {
        $error         = "CommandFailed";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $raw_output, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => $raw_output,
            response_object => $resp
        );
    }

} ## end sub _check_error

## @name ::reboot
##
## @summary Parses the raw output from the "reboot" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from reboot with any of its options as described in the
##    corresponding cdef file 'reboot.cdefs'
## @synopsis
##      reboot(%args);
## INPUTS TO METHOD CALL 'reboot' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-d' </td> <td> 'no-wtmp-record' </td> <td> '1' </td> </tr>
## <tr> <td> '-w' </td> <td> 'dry-run' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'poweroff' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'no-sync' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'harddrive-standby' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'shutdown-interfaces' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub reboot {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();

    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );

    return [];
}
1;

