#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Linux/Redhat/rm.pm#5 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Linux rm command
##
## @description
##    This is a library to parse the raw output of Linux rm command.
##
## @synopsis
##    use rm;
##
##    rm(%args);
##
## @author sheethal@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::rm;
use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

## @name ::rm
##
## @summary Parses the raw output from the "rm" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from rm with any of its options as described in the
##    corresponding cdef file 'rm.cdefs'
## @synopsis
##      rm(%args);
## INPUTS TO METHOD CALL 'rm' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-R' </td> <td> 'recursive' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'interactive' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'directory' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'paths' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub rm {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub rm

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~ /missing.*/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /No such file or directory.*/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /rm:.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => $raw_output,
            response_object => $resp
        );
    }
} ## end sub _check_error

1;
