#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Linux/Redhat/san_version.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary san_version command output
##
## @description
##    This is a library to parse the raw output of san_version command.
##
## @synopsis
##    use san_version;
##
##    san_version(%args);
##
## @author sangavai@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Host::CLI::Linux::Redhat::san_version;

use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

use strict;

sub _check_error {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $raw_output  = $args{raw_output};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /illegal option/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /usage:\s+.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

## @name ::san_version
##
## @summary Parses the raw output from the "san_version" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from san_version with any of its options as described in the
##    corresponding cdef file 'san_version.cdefs'
## @synopsis
##      san_version(%args);
## INPUTS TO METHOD CALL 'san_version' ON APISET OBJECT
##
## SPECIAL OPTIONS
##
## tool_full_path
## (Optional)
## The complete path to the tool.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /opt/netapp/santools/san_version
##
## tool_dir_path
## (Optional)
## The absolute path to the dir which contains the executable of the tool.
## This shouldn't include version or name of the tool.
## Defaults to the test param TESTBINDIR_UNIX (/usr/software/test/bin is the usual
## default) in unix like OSes and TESTBINDIR_WINDOWS (x:/bin is the usual default)
## in Windows.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /opt/netapp/santools
##
## tool_version
## (Optional)
## The specific version of the tool to be selected.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: '6.0'
##
## TOOL SPECIFIC OPTIONS:
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'version' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = /opt/netapp/santools/san_version -v
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;NetApp Linux Host Utilities version 6.0
##&nbsp;sanlun version 6.0.144.0839
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;              'sanlun_version' => '6.0.144.0839',
##&nbsp;              'netapp_linux_host_utilities_version' => '6.0'
##&nbsp;          }
##&nbsp;         ];
##&nbsp;
sub san_version {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );

    foreach my $line (@lines) {
        if ( $line =~ /(NetApp Linux Host Utilities version)\s+(\S+)/ ) {
            $result_array[0]->{ to_key_name( item => $1 ) } = $2;
        } elsif ( $line =~ /(sanlun version)\s+(\S+)/ ) {
            $result_array[0]->{ to_key_name( item => $1 ) } = $2;
        }
    }

    return \@result_array;
} ## end sub san_version
1;

