#Copyright (c) 2013 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse systemctl command output
##
## @description
##    This is a library to parse the raw output of  systemctl command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Linux::Redhat::systemctl;
##
##    systemctl(%args);
##
## @author sheethal@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::systemctl;

use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;
use strict;


sub systemctl {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub systemctl

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~ /(Usage:)/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => $raw_output,
            response_object => $resp);
    }
} ## end sub _check_error

1;
