#$Id: //depot/prod/test/nacld/lib/NACL/APISet/Host/CLI/Linux/Redhat/ttcp.pm $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux ttcp command
##
## @description
##    This is a library to parse the raw output of Linux ttcp command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "ttcp" will be ttcp()
##
## @synopsis
##    use ttcp;
##
##    ttcp(%args);
##
## @author udaykk@netapp.com , dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Linux::Redhat::ttcp;

#use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::CommandNotFoundException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if ( $raw_output =~ /([Uu]sage:\s)/ ) {
        $$error_message = $1;
        $error          = "CommandFailed";
    }
    elsif ( $raw_output =~ /(bad hostname.*)/ ) {
        $$error_message = $1;
        $error          = "Invalidparam";
    }
    elsif ( $raw_output =~ /(bind.*)/ ) {
        $$error_message = $1;
        $error          = "CommandFailed";
    }
    elsif ( $raw_output =~ /(.*command not found)/ ) {
        $$error_message = $1;
        $error          = "CommandFailed";
    }
    elsif ( $raw_output =~ /(.*Connection.*)/ ) {
        $$error_message = $1;
        $error          = "CommandFailed";
    }
    elsif ( $raw_output =~ /(.*IO:.*)/ ) {
        $$error_message = $1;
        $error          = "Invalidparam";
    }
    elsif ( $raw_output =~ /(can't resolve host.*)/ ) {
        $$error_message = $1;
        $error          = "Invalidparam";
   }
    elsif ( $raw_output =~ /(Timeout has occured.*)/ ) {
        $$error_message = $1;
        $error          = "CommandFailed";
   }
 return $error;
}

## @name ::ttcp
##
## @summary Parses the raw output from the "ttcp" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ttcp with any of its options as described in the
##    corresponding cdef file 'ttcp.cdefs'
## @synopsis
##      ttcp(%args);
## INPUTS TO METHOD CALL 'ttcp' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'sink' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'source' </td> <td> '1' </td> </tr>
## <tr> <td> '-4' </td> <td> 'ipv4' </td> <td> '1' </td> </tr>
## <tr> <td> '-6' </td> <td> 'ipv6' </td> <td> '1' </td> </tr>
## <tr> <td> '-w' </td> <td> 'wait' </td> <td> '1' </td> </tr>
## <tr> <td> '-I' </td> <td> 'interface' </td> <td> '1' </td> </tr>
## <tr> <td>  No switch for this key </td> <td> 'address' </td> <td> '1' </td> </tr>
## <tr> <td>  No switch for this key </td> <td> 'multicast-group' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-B' </td> <td> 'full-blocks' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'transmit-mode' </td> <td> '1' </td> </tr>
## <tr> <td> '-u' </td> <td> 'udp' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'socket-buffer-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-V' </td> <td> 'version-number' </td> <td> '1' </td> </tr>
## <tr> <td> '-O' </td> <td> 'offset' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'receive-mode' </td> <td> '1' </td> </tr>
## <tr> <td> '-A' </td> <td> 'align' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'socket-debug' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'port' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'host' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'length-of-bufs-read' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'format' </td> <td> '0' </td> </tr>
## <tr> <td> '-D' </td> <td> 'dont-buffer' </td> <td> '1' </td> </tr>
## <tr> <td> '-T' </td> <td> 'touch' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'number-of-source-bufs' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = ttcp -r -s -p 5000
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'KB_per_sec' => '0.000',
##&nbsp;            'bytes' => '0',
##&nbsp;            'real_seconds' => '0.000',
##&nbsp;            'tcp_info' => 'tcp',
##&nbsp;            'io_calls' => '1',
##&nbsp;            'sockbufsize' => '51882',
##&nbsp;            'sockbufrcvsize' => '87380',
##&nbsp;            'align' => '16384/0',
##&nbsp;            'calls_per_sec' => '0.006',
##&nbsp;            'transmitted_data' => '0.000user 0.000sys 0:00real 0% 0i+0d 0maxrss 0+1pf 0+0csw',
##&nbsp;            'buflen' => '8192',
##&nbsp;            'port' => '5000',
##&nbsp;            'sockbufsndsize' => '16384',
##&nbsp;            'nbuf' => '2048',
##&nbsp;            'msec_per_call' => '0.006',
##&nbsp;            'connect' => '10.72.219.45'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
##&nbsp;Command = ttcp -r host 10.72.219.45
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'KB_per_sec' => '0.000',
##&nbsp;            'bytes' => '0',
##&nbsp;            'real_seconds' => '0.000',
##&nbsp;            'tcp_info' => 'tcp',
##&nbsp;            'io_calls' => '1',
##&nbsp;            'sockbufsize' => '51882',
##&nbsp;            'sockbufrcvsize' => '87380',
##&nbsp;            'align' => '16384/0',
##&nbsp;            'calls_per_sec' => '0.127',
##&nbsp;            'transmitted_data' => '0.000user 0.000sys 0:00real 0% 0i+0d 0maxrss 0+1pf 1+0csw',
##&nbsp;            'buflen' => '8192',
##&nbsp;            'port' => '5001',
##&nbsp;            'sockbufsndsize' => '16384',
##&nbsp;            'nbuf' => '2048',
##&nbsp;            'msec_per_call' => '0.127',
##&nbsp;            'connect' => '10.72.219.45'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;Command = ttcp -t -V
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'revision' => '1.1 $ 2-Dec-1997'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
sub ttcp {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @result_array;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @arraylines =
      get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@arraylines) {
        if ( $line =~ /^ttcp.c\s\$(Revision):\s([\w+\.\$\s-]+)/ ) {
            $result_array[0]->{"revision"} = $2;
        }
        elsif ( $line =~
 /^(ttcp-r|ttcp-t):\s+buflen+=(\d+)[\s,]+nbuf=(\d+)[\s,]+align=(\d+\/\d+)[\s,]+port=(\d+)\s*(tcp\s+->\s+(\S+)|tcp|)/
           )
         {
             $result_array[0]->{"buflen"} = $2;
             $result_array[0]->{"nbuf"}   = $3;
             $result_array[0]->{"align"}  = $4;
             $result_array[0]->{"port"}   = $5;
             if($7){
             $result_array[0]->{"tcp_info"} = $7;
             }
         }
        elsif ( $line =~
/^(ttcp-r|ttcp-t):\s+sockbufsndsize=(\d+)[\s,]+sockbufrcvsize=(\d+)[\s,]+sockbufsize=(\d+),\s+\#\s(\w+).*\s\#/
          )
        {
            $result_array[0]->{"sockbufsndsize"} = $2;
            $result_array[0]->{"sockbufrcvsize"} = $3;
            $result_array[0]->{"sockbufsize"}    = $4;
            $result_array[0]->{"tcp_info"}       = $5;
        }
        elsif ( $line =~ /(ttcp-r|ttcp-t):\s+(accept from|connect)\s(.*)/ ) {
            $result_array[0]->{"connect"} = $3;
        }
        elsif ( $line =~
/^(ttcp-r|ttcp-t):\s+(\d+)\s(\w+)\s\w+\s(\d+\.\d+)\s(\w+\s\w+)\s=\s+(\d+\.\d+)\s(\S+)/
          )
        {
            $result_array[0]->{ to_key_name( item => $5 ) } = $4;
            $result_array[0]->{"$3"}         = $2;
            $result_array[0]->{"KB_per_sec"} = $6;
        }
        elsif ( $line =~
/^(ttcp-r|ttcp-t):\s+(\d+)\s+([\w\/\s]+),\s+([\w\/]+)\s+=\s+([\d\w\.]+),\s+([\w\/]+)\s+=\s+([\d\.]+)/
          )
        {
            $result_array[0]->{"io_calls"}  = $2;
            $result_array[0]->{"msec_per_call"} = $5;
            $result_array[0]->{"calls_per_sec"} = $7;
        }
        elsif ( $line =~ /^(ttcp-r|ttcp-t):\s+(\S+user.*)/ ) {
            $result_array[0]->{"transmitted_data"} = $2;
        }
    }
    return \@result_array;
}    # end sub ttcp;
1;
