#$Id$
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux ypcat command output
##
## @description
##      This library supports error checking for kenv command, but output parsing is not supported as of now.
##
## @synopsis
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Host::CLI::Linux::Redhat::ypcat;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::CommandFailedException;


sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error = "CommandFailed"; # we'll try to classify it if possible
    $$error_message = "Command failed - Check command line and dependencies";
    if ($raw_output =~ /:\s(.*)/) {
        $$error_message = $1;
        if($$error_message =~ /(Missing)/) {
            $error = "MissingRequiredParam";
        } elsif($$error_message =~ /([Ii]nvalid)|([Uu]nrecognized option)/) {
            $error = "InvalidParam";
        } elsif($$error_message =~ /(argument to)|(unknown)/) {
            $error = "InvalidParamValue";
        }
    }
    return $error;
}

## @name ::ypcat
##
## @summary Parses the raw output from the "ypcat" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ypcat with any of its options as described in the 
##    corresponding cdef file 'ypcat.cdefs'
## @synopsis
##      ypcat(%args);
## INPUTS TO METHOD CALL 'ypcat' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-d' </td> <td> 'domain' </td> </tr> 
## <tr> <td> '-x' </td> <td> 'disp-trans-table' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'map-nickname-trans' </td> </tr> 
## <tr> <td> '-k' </td> <td> 'display-map-keys' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'mapname' </td> </tr> 
## <tr> <td> '-h' </td> <td> 'hostname' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
##  Returns: Output parsing is not supported

## @examples

##&nbsp;Command = ypcat -d csiqa.rtp.netapp.com -h 10.227.64.31 passwd
##&nbsp;$VAR1 = [];
##&nbsp;
sub ypcat {
    my %args = @_;
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;
    if($error_level) {
        $error = _check_error(raw_output=>$output,
                              error_message=>\$error_message);
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
	}
    return [];
}
1;
