#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary API Set for 7-Mode transition tool CLI commands
## @author rohit4@netapp.com, dl-nacl-dev@netapp.com
## @status Review
## @pod here

package NACL::APISet::Host::CLI::SMTT;

#
# Compiler Directives
#
use strict;
use warnings;

use base qw(NACL::APISet::Host::CLI);

#
# Module imports - APISet
#
use NACL::APISet::Session::CLI::Windows;
use NACL::APISet::Session::CLI::Linux;
#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);

=head1 NAME

NACL::APISet::Host::CLI::SMTT - API Set for Windows CLI commands

=head1 SYNOPSIS

use NACL::APISet::Host::CLI::SMTT;

=head1 DESCRIPTION

A C<NACL::APISet::Host::CLI::SMTT> is used to connect to and send CLI commands
to the Windows Host running 7-Mode transition tool.

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Host::CLI::SMTT>.

=over

=item Synopsis

 $api_set_obj = NACL::APISet::Host::CLI::SMTT->new( %args );

=item Arguments

=over

=item %args

(Optional)
Parameters passed to parent classes.  See NACL::APISet and
NACL::APISet::Host::CLI for details.

=back

=item Returns

A reference to a new SMTT API Set (NACL::APISet::Host::CLI::SMTT)

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

=head2 add_paths_for_transition

	$Api_Set_Obj-> add_paths_for_transition();
		OR
	$Api_Set_Obj-> add_paths_for_transition(
	        path => 'C:\ProgramData\7-Mode Transition Tool\bin',
	);

This subroutine is used to set the PATH environment variable for 7-Mode transition
tool .

There are two ways to invoke the method.

1) Without providing the parameters i.e "path"

2) With providing the parameters i.e "path"

In the first case default path for transition tool will be set i.e.
'C:\Program Files\NetApp\7-Mode Transition Tool\bin;' .

In the second case user defined path will be set. The path till the transition
binary has to be supplied . Multiple paths can be set by separating the paths with ';'.

Supported for Windows clients

=over

=back

=back

=cut

sub add_paths_for_transition {
    $Log->enter();
    my ($self,%path_to_bin)=@_;
    my $command='set PATH=%path%;C:\Windows\system32;C:\Windows;'
	.'C:\Program Files\Java\jre6\bin;'
	.'C:\Program Files\Java\jre7\bin;'
	.'C:\Program Files\NetApp\7-Mode Transition Tool\bin;';
    if (defined $path_to_bin{path}){
	$command .= $path_to_bin{path};
    }
    $self->execute_raw_command(command => $command);
    $Log->exit();
}

1;
