# $Id: //depot/prod/test/main/lib/NACL/APISet/Host/CLI/SQL.pm#3 $

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

## @summary API Set for SQL CLI commands
## @author dl-nacl-dev@netapp.com,Shilpi.Nagpal@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Host::CLI::SQL;

#
# Compiler Directives
#
use strict;
use warnings;

use base qw(NACL::APISet::Host::CLI);

#
# APISet imports
#
use NACL::APISet::Session::CLI::SQL;

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

##############################################################################
# Method:    execute_raw_command
# Objective: Execute a raw CLI command on the SQL shell and return the raw
#            output
# Details:   See POD documentation at the end of this file.
##############################################################################
sub execute_raw_command {
    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;

    $self->timeout_scaling_factor(
        Tharn::param("APISET_TIMEOUT_SCALING_FACTOR"))
        if defined Tharn::param("APISET_TIMEOUT_SCALING_FACTOR");

    $args{"on_timeout"}
        = $self->_throw_timeout_exception( before => $args{"on_timeout"}
            || delete $args{"connectrec-on_timeout"} );
    $args{"on_idle"}
        = $self->_throw_timeout_exception( before => $args{"on_idle"}
            || delete $args{"connectrec-on_idle"} );

    my $output;

    $output = $self->{session}->send_command(%args);

    $Log->exit() if $may_exit;
    return $output;
} ## end sub execute_raw_command

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Host::CLI::SQL - API Set for SQL CLI commands

=head1 SYNOPSIS

use NACL::APISet::Host::CLI::SQL;

=head1 DESCRIPTION

A C<NACL::APISet::Host::CLI::SQL> is used to connect to and send CLI commands
to the SQL Host.

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Host::CLI::SQL>.

=over

=item Synopsis

 $api_set_obj = NACL::APISet::Host::CLI::SQL->new( %args );

=item Arguments

=over

=item %args

(Optional)
Parameters passed to parent classes.  See NACL::APISet and
NACL::APISet::Host::CLI for details.

=back

=item Returns

A reference to a new SQL API Set.

=back

=head1 METHODS

=head2 execute_raw_command

This method is used to execute raw CLI commands on the SQL shell. The entire command
executed and the raw output is returned as a scalar. This is useful for cases
where we want to invoke a command without parsing the output, or for cases
raw output.

=over

=item Synopsis

$raw_output = $Api_Set_Obj->execute_raw_command(command => "select * from v\$instance;");

=item Arguments

execute_raw_command accepts all arguments accepted by Connectrec->execute().
Of these, only command is mandatory.

=over

=item command

(Mandatory)
The command to be invoked. This is the full command line as it will be entered on the filer.

=back

=item Returns

(Scalar)
Raw output

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

$Id: //depot/prod/test/main/lib/NACL/APISet/Host/CLI/SQL.pm#3 $

=cut

