#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Solaris/default/devfsadm.pm# $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse solaris devfsadm command
##
## @description
##    This is a library to parse the raw outputs of any of the solaris devfsadm command.
##
## @synopsis
##    use devfsadm;
##    devfsadm(%args);
##
## @author veerr@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Host::CLI::Solaris::default::devfsadm;

use Data::Dumper;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error = undef;
    if($raw_output =~ /Usage/) {
        $error = "InvalidParam";
        $$error_message = $raw_output;
    } elsif($raw_output =~ /driver failed to attach|No such file or directory|no modules loaded from MODPATH/) {
        $error = "InvalidParamValue";
        $$error_message = $raw_output;
    }
    return $error;
}

## @name ::devfsadm
##
## @summary Parses the raw output from the "devfsadm" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from devfsadm with any of its options as described in the
##    corresponding cdef file 'devfsadm.cdefs'
## @synopsis
##      devfsadm(%args);
## INPUTS TO METHOD CALL 'devfsadm' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub devfsadm {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
	my $command = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'devfsadm',
                             "ptr_error_message" => \$error_message,
                             );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
	return [];
}
1;
