#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Solaris/default/format.pm# $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse solaris format command
##
## @description
##    This is a library to parse the raw outputs of any of the solaris format command.
##
## @synopsis
##    use format;
##    format(%args);
##
## @author veerr@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Host::CLI::Solaris::default::format;

use Data::Dumper;
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error = undef;
    if($raw_output =~ /(Unable to find specified disk .*|Must specify disk as well as type|
        Unable to open command file|No such file or directory|No disks found)/) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    } elsif($raw_output =~ /Usage/) {
       $error = "CommandFailed";
        $$error_message = $raw_output;
    }
    return $error;
}

## @name ::format
##
## @summary Parses the raw output from the "format" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from format with any of its options as described in the
##    corresponding cdef file 'format.cdefs'
## @synopsis
##      format(%args);
## INPUTS TO METHOD CALL 'format' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub format {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
	my $command = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'format',
                             "ptr_error_message" => \$error_message,
                             );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
	return [];
}
1;
