#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Solaris/default/pagesize.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Solaris pagesize command output
##
## @description
##    This is a library to parse the raw output of Solaris pagesize command.
##
## @synopsis
##    use pagesize;
##
##    pagesize(%args);
##
## @author sangavai@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Solaris::default::pagesize;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error      = undef;
    my $error_message;
    if ( $raw_output =~ /(pagesize|[Uu]sage):\s(.*)/ ) {
        $error_message = $1;
        $error         = "CommandFailed";
        if ( $raw_output =~ /(illegal option)/ ) {
            $error         = "InvalidParam";
            $error_message = $1;
        }
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

## @name ::pagesize
##
## @summary Parses the raw output from the "pagesize" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from pagesize with any of its options as described in the 
##    corresponding cdef file 'pagesize.cdefs'
## @synopsis
##      pagesize(%args);
## INPUTS TO METHOD CALL 'pagesize' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = pagesize
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;4096
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'supported_sizes' => [
##&nbsp;                                   '4096'
##&nbsp;                                 ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = pagesize -a
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;4096
##&nbsp;2097152
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'supported_sizes' => [
##&nbsp;                                   '4096',
##&nbsp;                                   '2097152'
##&nbsp;                                 ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub pagesize {
    my %args = @_;
    my (@result_array);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();

    _check_error( raw_output => $output, response_object => $resp );
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    for ( my $i = 0; $i < (@lines); $i++ ) {
        $result_array[0]->{supported_sizes}->[$i] = $lines[$i];
    }
    return \@result_array;
} ## end sub pagesize

1;
