#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Solaris/default/rsh.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Solaris rsh command output
##
## @description
##    This is a library to parse the raw output of Solaris rsh command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Solaris::default::rsh;
##
##    rsh(%args);
##
## @author skotagir@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Solaris::default::rsh;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidOptionException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error       = undef;
    my $error_message;
    if ( $raw_output =~ /rsh:\s+option requires an argument.*/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output
        =~ /rsh:\s+(illegal option.*|Only one of \S+ and \S+ allowed)/ ) {
        $error         = "InvalidOption";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /.*(Network is unreachable|:unknown host)/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /usage:/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }
} ## end sub _check_error

## @name ::rsh
##
## @summary Parses the raw output from the "rsh" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from rsh with any of its options as described in the
##    corresponding cdef file 'rsh.cdefs'
## @synopsis
##      rsh(%args);
## INPUTS TO METHOD CALL 'rsh' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'hostname' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'redirect-input' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'command' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'login' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub rsh {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );

    return [];
} ## end sub rsh

1;
