# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Solaris/default/update_drv.pm#1 $
#
# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse Solaris update_drv command
##
## @description
##     This library supports error checking for kenv command, but output parsing is not supported as of now.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Solaris::default::update_drv;
##
##    update_drv(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Solaris::default::update_drv;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::update_drv
##
## @summary Parses the raw output from the "update_drv" command
##
## @description
##    This library supports error checking for kenv command, but output parsing is not supported as of now.
## @synopsis
##      update_drv(%args);
## INPUTS TO METHOD CALL 'update_drv' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'driver-module' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'permission' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'add-attributes' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'base-directory' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'policy' </td> <td> '0' </td> </tr>
## <tr> <td> '-P' </td> <td> 'privilege' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'delete-attributes' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'identify-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns: Output parsing is not supported

sub update_drv {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );

    return [];
} ## end sub update_drv

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /(update_drv:|Usage:|Failed to update)\s+.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;
