#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Unix/init.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Solaris init command
##
## @description
##    This is a library to parse the raw output of Solaris init command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "init" will be init()
##
## @synopsis
##    use NACL::APISet::Host::CLI::Unix::init;
##
##    init(%args);
##
## @author skotagir@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Unix::init;

use strict;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::init
##
## @summary Parses the raw output from the "init" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from init with any of its options as described in the
##    corresponding cdef file 'init.cdefs'
## @synopsis
##      init(%args);
## INPUTS TO METHOD CALL 'init' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'level' </td> <td> '0' </td> </tr>
## <tr> <td> 'Q' </td> <td> 're-examine' </td> <td> '1' </td> </tr>
## <tr> <td> 'S' </td> <td> 'single-user-mode' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'pseudo-states' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub init {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub init

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error ) = undef;
    if ( $raw_output =~ /([uU]sage|init):\s+.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;
