#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Freebsd kill command output
##
## @description
##    This is a library to parse the raw output of Freebsd kill command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Freebsd::default::kill;
##
##    kill(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Unix::kill;

use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Response::CLI::ParserUtils;
use strict;


sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error = "CommandFailed"; # we'll try to classify it if possible
    $$error_message = "Command failed - Check command line and dependencies";
    return $error;
}

## @name ::kill
##
## @summary Parses the raw output from the "kill" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from kill with any of its options as described in the
##    corresponding cdef file 'kill.cdefs'
## @synopsis
##      kill(%args);
## INPUTS TO METHOD CALL 'kill' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-s' </td> <td> 'signal' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'pid' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'commandname' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'do-not-restrict' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'print-pid-only' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'signal-list' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = kill -s SIGKILL 3960;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub kill {
    my %args = @_;
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;
    if($error_level) {
        $error = _check_error(raw_output=>$output,
                              error_message=>\$error_message);
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
    return [];
}

1;
