#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Unix/lvcreate.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux lvcreate command output
##
## @description
##   This library supports error checking for lvcreate command, but output parsing is not supported as of now.
##
## @synopsis
##    use lvcreate;
##
##    lvcreate(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Unix::lvcreate;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error       = undef;
    my $error_level = $resp->get_execution_status();
    my $error_message;
    if( $raw_output =~ /requires an argument/) {
        $error_message = $raw_output;
        $error         = "MissingRequiredParam";
    } elsif ( $raw_output =~ /not found in Volume Group|Volume Group.*not found/) {
        $error_message = $raw_output;
        $error         = "CommandFailed";
    } elsif ( $raw_output =~ /invalid option|Invalid argument|Error during parsing of command line/i) {
        $error_message = $raw_output;
        $error         = "InvalidParam";
    } elsif ( $raw_output =~ /Run `lvcreate --help' for more information/) {
        $error_message = $raw_output;
        $error         =  "InvalidParamValue";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

    return $error;
} ## end sub _check_error

## @name ::lvcreate
##
## @summary Parses the raw output from the "lvcreate" command
##
## @description 
##    This library supports error checking for lvcreate command, but output parsing is not supported as of now
## @synopsis
##      lvcreate(%args);
## INPUTS TO METHOD CALL 'lvcreate' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## The below options apllicable for Linux only
## <tr> <td> '--virtualsize' </td> <td> 'virtual-size' </td> <td> '0' </td> </tr> 
## <tr> <td> '--addtag' </td> <td> 'addtag' </td> <td> '0' </td> </tr> 
## <tr> <td> '--alloc' </td> <td> 'alloc-policy' </td> <td> '0' </td> </tr> 
## <tr> <td> '--minor' </td> <td> 'minor' </td> <td> '0' </td> </tr> 
## <tr> <td> '--nosync' </td> <td> 'nosync' </td> <td> '1' </td> </tr> 
## <tr> <td> '--mirrorlog' </td> <td> 'mirrorlog' </td> <td> '0' </td> </tr> 
## <tr> <td> '--corelog' </td> <td> 'corelog' </td> <td> '1' </td> </tr> 
## <tr> <td> '-R' </td> <td> 'regionsize' </td> <td> '0' </td> </tr> 
## <tr> <td> '-r' </td> <td> 'readahead' </td> <td> '0' </td> </tr> 
## <tr> <td> '-s' </td> <td> 'snapshot' </td> <td> '0' </td> </tr> 
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'test' </td> <td> '1' </td> </tr> 
## <tr> <td> '-Z' </td> <td> 'zero' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'phy-volumes' </td> <td> '0' </td> </tr> 
##
## The below options supports both Linux & HPUX
## <tr> <td> '-l' </td> <td> 'logicalextents' </td> <td> '0' </td> </tr> 
## <tr> <td> '-m' </td> <td> 'mirrors' </td> <td> '0' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'chunksize' </td> <td> '0' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr> 
## <tr> <td> '-p' </td> <td> 'permission' </td> <td> '0' </td> </tr> 
## <tr> <td> '-L' </td> <td> 'lvsize' </td> <td> '0' </td> </tr> 
## <tr> <td> '-A' </td> <td> 'autobackup' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'vgname' </td> <td> '0' </td> </tr> 
## <tr> <td> '-C' </td> <td> 'contiguous' </td> <td> '0' </td> </tr> 
## <tr> <td> '-M' </td> <td> 'persistent' </td> <td> '0' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'stripes' </td> <td> '0' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'mirror-consistency' </td> <td> '0' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'lvname' </td> <td> '0' </td> </tr> 
## <tr> <td> '-I' </td> <td> 'stripesize' </td> <td> '0' </td> </tr> 
##
## The below options support HPUX only
## <tr> <td> '-s' </td> <td> 'strict' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'timestamp' </td> <td> '0' </td> </tr> 
## <tr> <td> '-S' </td> <td> 'snap-type' </td> <td> '0' </td> </tr> 
## <tr> <td> '-D' </td> <td> 'distributed' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns: Output parsing is not supported
## @examples
##&nbsp;Command = lvcreate -L 20m vol_grp1 /dev/sda1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  Logical volume "lvol1" created
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = lvcreate -L 20m /dev/vg00
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Warning: rounding up logical volume size to extent boundary at size "24" MB.
##&nbsp;Logical volume "/dev/vg00/lvol9" has been successfully created with
##&nbsp;character device "/dev/vg00/rlvol9".
##&nbsp;Logical volume "/dev/vg00/lvol9" has been successfully extended.
##&nbsp;Volume Group configuration for /dev/vg00 has been saved in /etc/lvmconf/vg00.conf
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub lvcreate {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub lvcreate

1;
