#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Unix/lvremove.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux lvremove command output
##
## @description
##    This library supports error checking for lvremove command, but output parsing is not supported as of now..
##
## @synopsis
##    use lvremove;
##
##    lvremove(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Unix::lvremove;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error       = undef;
    my $error_level = $resp->get_execution_status();
    my $error_message;
    if( $raw_output =~ /requires an argument/) {
        $error_message = $raw_output;
        $error         = "MissingRequiredParam";
    } elsif ( $raw_output =~ /not found in Volume Group|Volume Group.*not found/) {
        $error_message = $raw_output;
        $error         = "CommandFailed";
    } elsif ( $raw_output =~ /invalid option|Invalid argument|Error during parsing of command line/i) {
        $error_message = $raw_output;
        $error         = "InvalidParam";
    } elsif ( $raw_output =~ /Run `lvremove --help' for more information/) {
        $error_message = $raw_output;
        $error         =  "InvalidParamValue";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

    return $error;
} ## end sub _check_error

## @name ::lvremove
##
## @summary Parses the raw output from the "lvremove" command
##
## @description 
##    This library supports error checking for kenv command, but output parsing is not supported as of now.
## @synopsis
##      lvremove(%args);
## INPUTS TO METHOD CALL 'lvremove' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## The below options support both Linux & HPUX
## <tr> <td> '-A' </td> <td> 'autobackup' </td> <td> '1' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'logical-volume-path' </td> <td> '0' </td> </tr> 
##
## The below options support Linux only
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr> 
## <tr> <td> '-t' </td> <td> 'test' </td> <td> '1' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr> 
##
## The below options supports HPUX only
## <tr> <td> '-F' </td> <td> 'force' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns: Output parsing is not supported.
## @examples
##&nbsp;Command = lvremove -f /dev/vg00/lvol9
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Logical volume "/dev/vg00/lvol9" has been successfully removed.
##&nbsp;Volume Group configuration for /dev/vg00 has been saved in /etc/lvmconf/vg00.conf
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = lvremove -f /dev/vol_grp1/lvol1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  Logical volume "lvol1" successfully removed
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub lvremove {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub lvremove

1;
