#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Linux random command output
##
## @description
##    This is a library to parse the raw output of Linux random command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Linux::random;
##
##    random(%args);
##
## @author Abhsihek Kumar(kabhishe@netapp.com), dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Unix::random;

use strict;
use warnings;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);


## @name ::random
##
## @summary Parses the raw output from the "random" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from random with any of its options as described in the
##    corresponding cdef file 'random.cdefs'
## @synopsis
##      random(%args);
## INPUTS TO METHOD CALL 'random' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> </tr>
## <tr> <td> '-l' </td> <td> 'maxfilelen' </td> </tr>
## <tr> <td> '-u' </td> <td> 'uid' </td> </tr>
## <tr> <td> '-e' </td> <td> 'max-diff-length' </td> </tr>
## <tr> <td> '-i' </td> <td> 'iterations' </td> </tr>
## <tr> <td> '-s' </td> <td> 'seed' </td> </tr>
## <tr> <td> '-b' </td> <td> 'byte-value' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'scratch-dir' </td> </tr>
## <tr> <td> '-R' </td> <td> 'random-secondary-groups' </td> </tr>
## <tr> <td> '-m' </td> <td> 'maxfiles' </td> </tr>
## <tr> <td> '-r' </td> <td> 'random-uid-gid' </td> </tr>
## <tr> <td> '-g' </td> <td> 'gid' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = random -t 5 /root
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub random {
    my %args = @_;
 
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
 
    if($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(text=>$output,response_object => $resp);
    }
    return [];
}

1;
