#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Unix/rm.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse rm command
##
## @description
##    This is a library to parse rm command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "rm" will rm()
##
## @synopsis
##    use rm;
##    rm(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Host::CLI::Unix::rm;

use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidOptionException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /(rm|Usage):/i && $raw_output !~ /remove/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    if ( $raw_output =~ /(.*: No such file or directory|cannot remove directory.*)/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /missing operand/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /invalid option/ ) {
        $error         = "InvalidOption";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

## @name ::rm
##
## @summary Parses the raw output from the "rm" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from rm with any of its options as described in the 
##    corresponding cdef file 'rm.cdefs'
## @synopsis
##      rm(%args);
## INPUTS TO METHOD CALL 'rm' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
##
## The below option supports Linux only
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr> 
##
## The below options support Linux & Solaris
## <tr> <td> '-r' </td> <td> 'recursive' </td> <td> '1' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'interactive' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'paths' </td> <td> '0' </td> </tr> 
##
## The below option support Linux only
## <tr> <td> '-I' </td> <td> 'prompt-once' </td> <td> '1' </td> </tr> 
##
## The below option supports both Linux & Solaris
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = rm -f -v /root/dir/file1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;removed `/root/dir/file1'
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = rm /dir/file1.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command=rm -i /dir/file1.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;rm: remove file1.txt (yes/no)?
##&nbsp;y
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = rm -r /dir/file1.txt
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub rm {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    _check_error( "raw_output" => $raw_output, response_object => $resp );
    return [];
} ## end sub rm
1;
