#$Id$
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse scp command output
##
## @description
##    This is a library to parse the raw output of scp command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Linux:Redhat::scp;
##
##    scp(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Host::CLI::Unix::scp;

use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;
use strict;
use warnings;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error         = undef;
    if ($raw_output =~
        /No such file or directory|hostname nor servname provided, or not known/
      ) {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /Permission denied/) {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    }
    return $error;
} ## end sub _check_error

## @name ::scp
##
## @summary Parses the raw output from the "scp" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from scp with any of its options as described in the
##    corresponding cdef file 'scp.cdefs'
## @synopsis
##      scp(%args);
## INPUTS TO METHOD CALL 'scp' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'dest-path' </td> </tr>
## <tr> <td> '-r' </td> <td> 'recursive-copy' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'source-path' </td> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> </tr>
## <tr> <td>  No switch for this key </td> <td> 'limit_kbits' </td> </tr>
## <tr> <td> '-4' </td> <td> 'ipv4_addr' </td> </tr>
## <tr> <td> '-6' </td> <td> 'ipv6_addr' </td> </tr>
## <tr> <td> '-i' </td> <td> 'identity_file' </td> </tr>
## <tr> <td> '-S' </td> <td> 'program' </td> </tr>
## <tr> <td> '-2' </td> <td> 'protocol_2' </td> </tr>
## <tr> <td> '-F' </td> <td> 'ssh_config' </td> </tr>
## <tr> <td> '-C' </td> <td> 'compression' </td> </tr>
## <tr> <td> '-c' </td> <td> 'cipher' </td> </tr>
## <tr> <td> '-p' </td> <td> 'preserve' </td> </tr>
## <tr> <td> '-l' </td> <td> 'limit' </td> </tr>
## <tr> <td> '-q' </td> <td> 'disable' </td> </tr>
## <tr> <td> '-P' </td> <td> 'port' </td> </tr>
## <tr> <td> '-B' </td> <td> 'batch_mode' </td> </tr>
## <tr> <td> '-o' </td> <td> 'ssh_option' </td> </tr>
## <tr> <td> '-1' </td> <td> 'protocol_1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = scp  /var/home/diag/SRC1/* /var/home/diag/DESTN
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub scp {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'scp',
                             "ptr_error_message" => \$error_message,);

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub scp
1;

