# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Unix/vgchange.pm#1 $
#
# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse vgchange command
##
## @description
##    This is a library to parse vgchange command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "vgchange" will be vgchange()
##
## @synopsis
##    use NACL::APISet::Host::CLI::Unix::vgchange;
##
##    vgchange(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Unix::vgchange;
use strict;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

## @name ::vgchange
##
## @summary Parses the raw output from the "vgchange" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vgchange with any of its options as described in the
##    corresponding cdef file 'vgchange.cdefs'
## @synopsis
##      vgchange(%args);
## INPUTS TO METHOD CALL 'vgchange' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
##
## The below option support Linux only
##
## <tr> <td> '--ignoremonitoring' </td> <td> 'ignore-monitoring' </td> <td> '1' </td> </tr>
## <tr> <td> '--monitor' </td> <td> 'monitor' </td> <td> '0' </td> </tr>
## <tr> <td> '-u' </td> <td> 'uuid' </td> <td> '1' </td> </tr>
## <tr> <td> ' --refresh' </td> <td> 'refresh' </td> <td> '1' </td> </tr>
## <tr> <td> '-x' </td> <td> 'resizeable' </td> <td> '0' </td> </tr>
##
## The below option supports both Linux & HPUX
##
## <tr> <td> '-l' </td> <td> 'disable-opening-of-lv' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'disable-sync-of-stale-physical-extents' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'activate-vg-if-pv-are-available' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'clustered' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'autobackup' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'available' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'vg-name' </td> <td> '0' </td> </tr>
##
## The below option support HPUX only
##
## <tr> <td> '-x' </td> <td> 'cross-activate' </td> <td> '1' </td> </tr>
## <tr> <td> '-Q' </td> <td> 'quiesce-mode' </td> <td> '0' </td> </tr>
## <tr> <td> '-R' </td> <td> 'resume' </td> <td> '1' </td> </tr>
## <tr> <td> '-S' </td> <td> 'sharable ' </td> <td> '0' </td> </tr>
## <tr> <td> '-T' </td> <td> 'threads' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '-q' </td> <td> 'quorum ' </td> <td> '0' </td> </tr>
## <tr> <td> '-P' </td> <td> 'resync-daemon-count' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'quiesce-time' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.

sub vgchange {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];

} ## end sub vgchange

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /Usage:|Error during parsing/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;
