#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Unix/vgcreate.pm#1 $#
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse vgcreate command
##
## @description
##    This is a library to parse the raw output of vgcreate command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "vgcreate" will vgcreate()
##
## @synopsis
##    use vgcreate;
##
##    vgcreate(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Unix::vgcreate;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::MissingRequiredParamException;

## @name ::vgcreate
##
## @summary Parses the raw output from the "vgcreate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vgcreate with any of its options as described in the
##    corresponding cdef file 'vgcreate.cdefs'
## @synopsis
##      vgcreate(%args);
## INPUTS TO METHOD CALL 'vgcreate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-y' </td> <td> 'yes' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'max-logical-volumes' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'test' </td> <td> '1' </td> </tr>
## <tr> <td> '--dataalignmentoffset' </td> <td> 'data-alignment-offset' </td> <td> '0' </td> </tr>
## <tr> <td> '--vgmetadatacopies' </td> <td> 'vgmeta-datacopies' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'physical-extent-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr>
## <tr> <td> '-z' </td> <td> 'zero' </td> <td> '0' </td> </tr>
## <tr> <td> '--dataalignment' </td> <td> 'data-alignment' </td> <td> '0' </td> </tr>
## <tr> <td> '--labelsector' </td> <td> 'label-sector' </td> <td> '0' </td> </tr>
## <tr> <td> '--version' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'volume-group-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'physical-device-path' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'clustered' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '--alloc' </td> <td> 'allocation-policy' </td> <td> '0' </td> </tr>
## <tr> <td> '--metadatasize' </td> <td> 'metadata-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-M' </td> <td> 'meta-datatype' </td> <td> '0' </td> </tr>
## <tr> <td> '--addtag' </td> <td> 'addtag' </td> <td> '0' </td> </tr>
## <tr> <td> '--pvmetadatacopies' </td> <td> 'pvmetadata-copies' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'autobackup' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'max-physical-volumes' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = vgcreate  vg0 /dev/sdb
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Volume group "vg0" successfully created
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'volume_group_created' => 'vg0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vgcreate  -v vg0 /dev/sdb
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Wiping cache of LVM-capable devices
##&nbsp;Wiping cache of LVM-capable devices
##&nbsp;Adding physical volume '/dev/sdb' to volume group 'vg0'
##&nbsp;Archiving volume group "vg0" metadata (seqno 0).
##&nbsp;Creating volume group backup "/etc/lvm/backup/vg0" (seqno 1).
##&nbsp;Volume group "vg0" successfully created
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'archiving_sequence_no' => 0,
##&nbsp;            'backup_sequence_no' => '1',
##&nbsp;            'physical_volume_added' => '/dev/sdb',
##&nbsp;            'volume_group_created' => 'vg0',
##&nbsp;            'backup_file' => '/etc/lvm/backup/vg0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vgcreate  --version
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;LVM version:     2.02.72(2) (2010-07-28)
##&nbsp;Library version: 1.02.53 (2010-07-28)
##&nbsp;Driver version:  4.17.0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'library_version' => '1.02.53 (2010-07-28)',
##&nbsp;            'driver_version' => '4.17.0',
##&nbsp;            'lvm_version' => '2.02.72(2) (2010-07-28)'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = vgcreate  -c y vg0 /dev/sdb
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Clustered Volume group "vg0" successfully created
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'clustered' => 1,
##&nbsp;            'volume_group_created' => 'vg0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub vgcreate {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my ( @result_array, @values );
    my @lines = get_lines( raw_output => $output, response_object => $resp );

    if ( $output =~ /(Volume\s+group)\s+\"(.*)\"\s+successfully\s+created/ ) {
        $result_array[0]->{volume_group_created} = $2;
    }
    elsif ( $output =~
        /(Clustered\s+volume\s+group)\s+"(.*)"\s+successfully\s+(created)/ )
    {
        $result_array[0]->{clustered} = 1;
        $result_array[0]->{volume_group_created} = $2;
    }
    elsif ( $cmd =~ /--version/ ) {
        foreach my $line (@lines) {
            @values = split( /:\s+/, $line );
            for ( my $j = 0 ; $j <= $#values ; $j = $j + 2 ) {
                $result_array[0]->{ to_key_name( item => $values[$j] ) } =
                  $values[ $j + 1 ];
            }
        }
    }
    if ( $cmd =~ /-v/ ) {
        foreach my $line (@lines) {
            if ( $line =~
/Adding\s+physical\s+volume\s+\'(.*)\'\s+to\s+volume\s+group\s+\'(.*)\'/
              )
            {
                $result_array[0]->{physical_volume_added} = $1;
                $result_array[0]->{volume_group_created}  = $2;
            }
            elsif ( $line =~
/Archiving\s+volume\s+group\s+\".*"\s+metadata\s+\(seqno\s+(\w+)\)/
              )
            {
                $result_array[0]->{archiving_sequence_no} = $1;
            }
            elsif ( $line =~
/Creating\s+volume\s+group\s+backup\s+\"(.*)\"\s+\(seqno\s+(\w+)\)/
              )
            {
                $result_array[0]->{backup_file}        = $1;
                $result_array[0]->{backup_sequence_no} = $2;
            }
        }
    }
    return \@result_array;
} ## end sub vgcreate

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error ) = undef;
    if ( $raw_output =~
        /Please\s+provide\s+volume\s+group\s+name\s+and\s+physical\s+volumes/ )
    {
        $error         = "MissingRequired";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /Unable\s+to\s+add\s+physical\s+volume.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~
/A\s+volume\s+group\s+called\s+.*\s+already\s+exists|Device\s+.*\s+not\s+found/
      )
    {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~
/vgcreate:\s+.*|[Uu]sage:\s+.*|Invalid\s+argument.*|Run\s+\'vgcreate\s+--help\'.*/
      )
    {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $resp
        );
    }
} ## end sub _check_error

1;
