# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Unix/vgscan.pm#1 $
#
# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse vgscan command
##
## @description
##    This library supports error checking for kenv command, but output parsing is not supported as of now
##
## @synopsis
##    use NACL::APISet::Host::CLI::Unix::vgscan;
##
##    vgscan(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Unix::vgscan;
use strict;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

## @name ::vgscan
##
## @summary Parses the raw output from the "vgscan" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vgscan with any of its options as described in the
##    corresponding cdef file 'vgscan.cdefs'
## @synopsis
##      vgscan(%args);
## INPUTS TO METHOD CALL 'vgscan' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
##
## The below option supports both Linux & HPUX
##
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
##
## The below option support Linux only
##
## <tr> <td> '--ignorelockingfailure' </td> <td> 'ignore-locking-failure' </td> <td> '1' </td> </tr>
## <tr> <td> '-P' </td> <td> 'partial' </td> <td> '1' </td> </tr>
## <tr> <td> '--mknodes' </td> <td> 'mknodes' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'debug' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'help' </td> <td> '1' </td> </tr>
##
## The below option support HPUX only
##
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '-N' </td> <td> 'populate-persistent-dsfs' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '0' </td> </tr>
## <tr> <td> '-B' </td> <td> 'populate-persistent-and-legacy-dsfs' </td> <td> '1' </td> </tr>
## <tr> <td> '-k' </td> <td> 'skip-disk-probe-portion' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'preview' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns: Output parsing is not supported.
sub vgscan {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];

} ## end sub vgscan

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /Usage:|Error during parsing/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }

} ## end sub _check_error

1;
