#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse cacls command on Windows 7
##
## @description
##    This is a library to parse the raw output of command cacls on Windows 7.
##
## @synopsis
##  use NACL::APISet::Host::CLI::Windows::default::cacls
##  
##  icacls(%args)
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Windows::default::cacls;

use NACL::APISet::Host::CLI::Windows::default::icacls;

## @name ::cacls
##
## @summary Parses the raw output from the "cacls" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from cacls with any of its options as described in the 
##    corresponding cdef file 'cacls.cdefs'
## @synopsis
##      cacls(%args);
## INPUTS TO METHOD CALL 'cacls' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '/D' </td> <td> 'deny' </td> </tr> 
## <tr> <td> '/E' </td> <td> 'edit' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'user-perm' </td> </tr> 
## <tr> <td> '/P' </td> <td> 'replace-perm' </td> </tr> 
## <tr> <td> '/C' </td> <td> 'continue-on-err' </td> </tr> 
## <tr> <td> '/T' </td> <td> 'recursive' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'pattern' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'user' </td> </tr> 
## <tr> <td> '/R' </td> <td> 'revoke' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = cacls C:\Users\Administrator\Music\ab /T
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;C:\Users\Administrator\Music\ab NT AUTHORITY\SYSTEM:(OI)(CI)(ID)F 
##&nbsp;                                BUILTIN\Administrators:(OI)(CI)(ID)F 
##&nbsp;                                VMWIN64-75\Administrator:(OI)(CI)(ID)F 
##&nbsp;
##&nbsp;C:\Users\Administrator\Music\ab\a NT AUTHORITY\SYSTEM:(ID)F 
##&nbsp;                                  BUILTIN\Administrators:(ID)F 
##&nbsp;                                  VMWIN64-75\Administrator:(ID)F 
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'file_path' => 'C:\\Users\\Administrator\\Music\\ab',
##&nbsp;            'permissions' => [
##&nbsp;                               {
##&nbsp;                                 'permission' => '(OI)(CI)(ID)F',
##&nbsp;                                 'user_group' => 'NT AUTHORITY\\SYSTEM'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'permission' => '(OI)(CI)(ID)F',
##&nbsp;                                 'user_group' => 'BUILTIN\\Administrators'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'permission' => '(OI)(CI)(ID)F',
##&nbsp;                                 'user_group' => 'VMWIN64-75\\Administrator'
##&nbsp;                               }
##&nbsp;                             ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'file_path' => 'C:\\Users\\Administrator\\Music\\ab\\a',
##&nbsp;            'permissions' => [
##&nbsp;                               {
##&nbsp;                                 'permission' => '(ID)F',
##&nbsp;                                 'user_group' => 'NT AUTHORITY\\SYSTEM'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'permission' => '(ID)F',
##&nbsp;                                 'user_group' => 'BUILTIN\\Administrators'
##&nbsp;                               },
##&nbsp;                               {
##&nbsp;                                 'permission' => '(ID)F',
##&nbsp;                                 'user_group' => 'VMWIN64-75\\Administrator'
##&nbsp;                               }
##&nbsp;                             ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub cacls {
    my %args = @_;

    my @result_array = NACL::APISet::Host::CLI::Windows::default::icacls::icacls(%args);

    return \@result_array;
}

1;
