#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Windows/default/dir.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Windows dir command output
##
## @description
##    This is a library to parse the raw output of dir command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "dir" will dir()
##
## @synopsis
##    use NACL::APISet::Host::CLI::Windows::default::dir;
##
##    dir(%args);
##
## @author skotagir@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Host::CLI::Windows::default::dir;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /Invalid switch/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /Parameter format not correct|File Not Found/i )
    {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(
            text            => $error_message,
            response_object => $resp
        );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $args{response_object}
        );
    }
} ## end sub _check_error

## @name ::dir
##
## @summary Parses the raw output from the "dir" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from dir with any of its options as described in the
##    corresponding cdef file 'dir.cdefs'
## @synopsis
##      dir(%args);
## INPUTS TO METHOD CALL 'dir' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '/L' </td> <td> 'lowercase' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## <tr> <td> '/S' </td> <td> 'recursive' </td> <td> '1' </td> </tr>
## <tr> <td> '/C' </td> <td> 'file-size-separator' </td> <td> '1' </td> </tr>
## <tr> <td> '/N' </td> <td> 'long-list' </td> <td> '1' </td> </tr>
## <tr> <td> '/O' </td> <td> 'sort' </td> <td> '0' </td> </tr>
## <tr> <td> '/-C' </td> <td> 'no-file-size-separator' </td> <td> '1' </td> </tr>
## <tr> <td> '/X' </td> <td> 'short-names' </td> <td> '1' </td> </tr>
## <tr> <td> '/T' </td> <td> 'timefield' </td> <td> '0' </td> </tr>
## <tr> <td> '/B' </td> <td> 'bare-format' </td> <td> '1' </td> </tr>
## <tr> <td> '/W' </td> <td> 'wide-list' </td> <td> '1' </td> </tr>
## <tr> <td> '/Q' </td> <td> 'display-owner' </td> <td> '1' </td> </tr>
## <tr> <td> '/A' </td> <td> 'attributes' </td> <td> '0' </td> </tr>
## <tr> <td> '/4' </td> <td> 'four-digit-years' </td> <td> '1' </td> </tr>
## <tr> <td> '/D' </td> <td> 'sort-by-column' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub dir {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    _check_error( "raw_output" => $raw_output, response_object => $resp );
    return [];
} ## end sub dir
1;

