#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Windows/default/route.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse route command on Windows 7
##
## @description
##    This is a library to parse the raw output of command route on Windows client.
##
## @synopsis
##  use NACL::APISet::Host::CLI::Windows::default::route
##
##  route(%args)
##
## @author rahula@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Windows::default::route;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;
    $$error_message = $raw_output;
    if ( $$error_message =~ /(.*Manipulates network routing tables.*)/i ) {
        $$error_message = $1;
        $error          = "MissingRequiredParam";
    }
    elsif ( $$error_message =~ /(.*The parameter is incorrect.*)/i ) {
        $$error_message = $1;
        $error          = "InvalidParamValue";
    }
    elsif ( $$error_message =~ /(.*interfaces can not be.*)/i ) {
        $$error_message = $1;
        $error          = "InvalidParam";
    }

    return $error;

}

## @name ::route_add
##
## @summary Parses the raw output from the "route add" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from route_add with any of its options as described in the
##    corresponding cdef file 'route.cdefs'
## @synopsis
##      route_add(%args);
## INPUTS TO METHOD CALL 'route_add' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'gateway' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination' </td> </tr>
## <tr> <td> 'METRIC' </td> <td> 'metric' </td> </tr>
## <tr> <td> 'IF' </td> <td> 'if' </td> </tr>
## <tr> <td> 'MASK' </td> <td> 'mask' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub route_add {
    my %args = @_;

    my ($error_message);

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        command       => "route add",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # No need to parse because no output is generated
    return [];
}

## @name ::route_delete
##
## @summary Parses the raw output from the "route delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from route_delete with any of its options as described in the
##    corresponding cdef file 'route.cdefs'
## @synopsis
##      route_delete(%args);
## INPUTS TO METHOD CALL 'route_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = route delete 10.72.46.73
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub route_delete {
    my %args = @_;

    my ($error_message);

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        command       => "route delete",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # No need to parse because no output is generated
    return [];
}

1;
