#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary shutdown command 
##
## @description
##    This is a library to parse the raw output of shutdown command.
##
## @synopsis
##    use shutdown;
##
##    shutdown(%args);
##
## @author tanil@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Host::CLI::Linux::Redhat::shutdown;
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Response::CLI::ParserUtils;
use strict;
use warnings;

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my $error_level = $resp->get_execution_status();
    my $error = undef;

    if ( $raw_output =~ /[Uu]sage/ ) {
        $error         = "MissingRequiredParam";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $raw_output, response_object => $resp );
    }

    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => $raw_output,
            response_object => $resp
        );
    }

} ## end sub _check_error

## @name ::shutdown
##
## @summary Parses the raw output from the "shutdown" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from shutdown with any of its options as described in the
##    corresponding cdef file 'shutdown.cdefs'
## @synopsis
##      shutdown(%args);
## INPUTS TO METHOD CALL 'shutdown' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-e' </td> <td> 'unexpected-shutdown' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'reboot' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'abort' </td> <td> '1' </td> </tr>
## <tr> <td> '-h' </td> <td> 'hibernate' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'comment-shutdown' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'turn-off' </td> <td> '1' </td> </tr>
## <tr> <td> '-m' </td> <td> 'target' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force-shutdown' </td> <td> '1' </td> </tr>
## <tr> <td> '-t' </td> <td> 'timeout-shutdown' </td> <td> '0' </td> </tr>
## <tr> <td> '-i' </td> <td> 'gui' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'shutdown' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'reason-shutdow' </td> <td> '0' </td> </tr>
## <tr> <td> '-l' </td> <td> 'log-off' </td> <td> '1' </td> </tr>
## <tr> <td> '-g' </td> <td> 'restart-application' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference which contains the parsed output.
## @examples
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = shutdown -r & echo __status__=!errorlevel!
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub shutdown {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();

    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    
    return [];
}
1;

