#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Windows/default/time.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse Windows time command output
##
## @description
##    This is a library to parse the raw output of time command.
##
## @synopsis
##    use NACL::APISet::Host::CLI::Windows::default::time;
##
##    time(%args);
##
## @author sangavai@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Host::CLI::Windows::default::time;

use strict;

use NACL::APISet::Exceptions::CommandFailedException;

sub _check_error {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

## @name ::time
##
## @summary Parses the raw output from the "time" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from time with any of its options as described in the
##    corresponding cdef file 'time.cdefs'
## @synopsis
##      time(%args);
## INPUTS TO METHOD CALL 'time' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '/T' </td> <td> 'show' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'time' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = time /T
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;01:11 PM
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'system_time' => '01:11 PM'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = time 5:34:49.75
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub time {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    _check_error( "raw_output" => $raw_output, response_object => $resp );
    my @result_array;
    my @array = split( /\s+/, $raw_output );
    if ( $raw_output =~ /(\d+:\d+\s+\w+)/ ) {
        $result_array[0]->{system_time} = $1;
    }
    return \@result_array;

} ## end sub time
1;

