#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/Windows/default/ver.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse the output of the Windows ver command output
##
## @description
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ver with any of its options as described in the 
##    corresponding cdef file 'ver.cdefs'
##
## @synopsis
##    use NACL::APISet::Host::CLI::Windows::default::ver;
##
##    ver();
##
## @author leahk@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::Windows::default::ver;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

## @name ::ver
##
## @summary Parses the raw output from the "ver" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ver with any of its options as described in the 
##    corresponding cdef file 'ver.cdefs'
## @synopsis
##      ver();
## INPUTS TO METHOD CALL 'ver' ON APISET OBJECT 
## 'ver' does not take any inputs.
## 
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = ver
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;& echo __status__=!errorlevel!
##&nbsp;
##&nbsp;Microsoft Windows [Version 6.1.7600]
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'number' => '6.1.7600',
##&nbsp;            'name' => 'Windows'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ver {
    my %args = @_;

    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(text=>$output,
                                               response_object => $resp);
    }

    $output =~ s/^.*Microsoft/Microsoft/s; # get rid of echo status
    ##################################################################
    # Microsoft Windows XP [Version 5.1.2600]
    # Microsoft Windows [Version 6.0.6001] # Vista
    # Microsoft Windows [Version 6.1.7600] # Win7
    # 
    $output =~ /Microsoft (.*) \[Version (\d+\.\d+\.\d+)\]/;
    # these were tested while getting the regex right
    unless ($1) {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw( 
            text=> "parser unable to find version name in\n$output",
            response_object => $resp
        );
    } 
    unless ($2) {
        NACL::APISet::Exceptions::UnexpectedOutputException->throw( 
            text=> "parser unable to find version number in\n$output",
            response_object => $resp
        );
    }
    return [{ name => $1, number => $2 }];
}

1;
