# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/default/dt.pm#3 $
#
#Copyright (c) 2011-2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse dt command output
##
## @description
##     This library supports error checking for dt command, but output parsing is not supported as of now.
##
## @synopsis
##    use NACL::APISet::Host::CLI::default::dt;
##
##    dt(%args);
##
## @author Abhsihek Kumar(kabhishe@netapp.com), dl-nacl-dev@netapp.com
##
## @status public
## @pod here
#
package NACL::APISet::Host::CLI::default::dt;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Response::CLI::ParserUtils;
use strict;
use warnings;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error         = undef;
    if ( $raw_output =~ /dt: .* No such file or directory/ 
        || $raw_output =~ /The system cannot find the drive specified/) {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /(dt: ERROR:.*)/
	|| $raw_output =~ /(: ERROR:.*The handle is invalid)/
        || $raw_output =~ /.*not allowed with.*/
        || $raw_output =~ /.*is not a valid operation.*/ ) {
        $error          = "InvalidParam";
        $$error_message = $raw_output;
    }
    return $error;
} ## end sub _check_errorrgs{raw_output}

## @name ::dt
##
## @summary Parses the raw output from the "dt" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from dt with any of its options as described in the
##    corresponding cdef file 'dt.cdefs'
## @synopsis
##      dt(%args);
## INPUTS TO METHOD CALL 'dt' ON APISET OBJECT
##
## SPECIAL OPTIONS
##
## tool_full_path
## (Optional)
## The complete path to the tool.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /usr/software/test/bin/dt.17.52/dt, /u/myuser/dir/dt.staging
##
## tool_dir_path
## (Optional)
## The absolute path to the dir which contains the executable of the tool.
## This shouldn't include version or name of the tool.
## Defaults to the test param TESTBINDIR_UNIX (/usr/software/test/bin is the usual
## default) in unix like OSes and TESTBINDIR_WINDOWS (x:/bin is the usual default)
## in Windows.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /usr/software/test/bin, /u/myuser/dir
##
## tool_version
## (Optional)
## The specific version of the tool to be selected.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: 'current', '17.52'
## This would select dt.current, dt.17.52 respectively.
##
## TOOL SPECIFIC OPTIONS:
## <table border = "1" >
## <tr> <th> ALL OPTIONS SUPPORTED </th> </tr>
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'rdelay' </td> <td> 'rdelay' </td> <td> '0' </td> </tr>
## <tr> <td> 'files' </td> <td> 'files' </td> <td> '0' </td> </tr>
## <tr> <td> '%iodir' </td> <td> 'log_iodir' </td> <td> '0' </td> </tr>
## <tr> <td> 'iomode' </td> <td> 'iomode' </td> <td> '0' </td> </tr>
## <tr> <td> 'if' </td> <td> 'if' </td> <td> '0' </td> </tr>
## <tr> <td> 'dtype' </td> <td> 'dtype' </td> <td> '0' </td> </tr>
## <tr> <td> '%dsf' </td> <td> 'device_name' </td> <td> '0' </td> </tr>
## <tr> <td> 'dfmt' </td> <td> 'dfmt' </td> <td> '0' </td> </tr>
## <tr> <td> 'flow' </td> <td> 'flow' </td> <td> '0' </td> </tr>
## <tr> <td> 'dsize' </td> <td> 'dsize' </td> <td> '0' </td> </tr>
## <tr> <td> 'passes' </td> <td> 'passes' </td> <td> '0' </td> </tr>
## <tr> <td> 'vrecords' </td> <td> 'vrecords' </td> <td> '0' </td> </tr>
## <tr> <td> 'history' </td> <td> 'history' </td> <td> '0' </td> </tr>
## <tr> <td> 'maxdata' </td> <td> 'maxdata' </td> <td> '0' </td> </tr>
## <tr> <td> 'align' </td> <td> 'align' </td> <td> '0' </td> </tr>
## <tr> <td> '%iotype' </td> <td> 'log_iotype' </td> <td> '0' </td> </tr>
## <tr> <td> 'slices' </td> <td> 'slices' </td> <td> '0' </td> </tr>
## <tr> <td> 'iotpass' </td> <td> 'iotpass' </td> <td> '0' </td> </tr>
## <tr> <td> 'ontap_cserver' </td> <td> 'ontap_cserver' </td> <td> '0' </td> </tr>
## <tr> <td> 'logt' </td> <td> 'logt' </td> <td> '0' </td> </tr>
## <tr> <td> 'ontap_password' </td> <td> 'ontap_password' </td> <td> '0' </td> </tr>
## <tr> <td> 'maxbad' </td> <td> 'maxbad' </td> <td> '0' </td> </tr>
## <tr> <td> 'iodir' </td> <td> 'iodir' </td> <td> '0' </td> </tr>
## <tr> <td> 'boff' </td> <td> 'boff' </td> <td> '0' </td> </tr>
## <tr> <td> 'zapipanic' </td> <td> 'zapipanic' </td> <td> '0' </td> </tr>
## <tr> <td> 'errors' </td> <td> 'errors' </td> <td> '0' </td> </tr>
## <tr> <td> 'of' </td> <td> 'of' </td> <td> '0' </td> </tr>
## <tr> <td> 'bs' </td> <td> 'bs' </td> <td> '0' </td> </tr>
## <tr> <td> 'dlimit' </td> <td> 'dlimit' </td> <td> '0' </td> </tr>
## <tr> <td> 'dir' </td> <td> 'dir' </td> <td> '0' </td> </tr>
## <tr> <td> 'ontap_nodes' </td> <td> 'ontap_nodes' </td> <td> '0' </td> </tr>
## <tr> <td> 'iotune' </td> <td> 'iotune' </td> <td> '0' </td> </tr>
## <tr> <td> 'idtype' </td> <td> 'idtype' </td> <td> '0' </td> </tr>
## <tr> <td> 'alarm' </td> <td> 'alarm' </td> <td> '0' </td> </tr>
## <tr> <td> 'disable' </td> <td> 'disable' </td> <td> '0' </td> </tr>
## <tr> <td> 'ralign' </td> <td> 'ralign' </td> <td> '0' </td> </tr>
## <tr> <td> 'enable' </td> <td> 'enable' </td> <td> '0' </td> </tr>
## <tr> <td> 'bufmodes' </td> <td> 'bufmodes' </td> <td> '0' </td> </tr>
## <tr> <td> 'stats' </td> <td> 'stats' </td> <td> '0' </td> </tr>
## <tr> <td> 'odtype' </td> <td> 'odtype' </td> <td> '0' </td> </tr>
## <tr> <td> 'skip' </td> <td> 'skip' </td> <td> '0' </td> </tr>
## <tr> <td> 'retry_delay' </td> <td> 'retry_delay' </td> <td> '0' </td> </tr>
## <tr> <td> 'iotype' </td> <td> 'iotype' </td> <td> '0' </td> </tr>
## <tr> <td> 'oncerr' </td> <td> 'oncerr' </td> <td> '0' </td> </tr>
## <tr> <td> 'pf' </td> <td> 'pf' </td> <td> '0' </td> </tr>
## <tr> <td> 'max' </td> <td> 'max' </td> <td> '0' </td> </tr>
## <tr> <td> 'slice' </td> <td> 'slice' </td> <td> '0' </td> </tr>
## <tr> <td> 'pkeepalive' </td> <td> 'pkeepalive' </td> <td> '0' </td> </tr>
## <tr> <td> 'dispose' </td> <td> 'dispose' </td> <td> '0' </td> </tr>
## <tr> <td> 'cdelay' </td> <td> 'cdelay' </td> <td> '0' </td> </tr>
## <tr> <td> 'step' </td> <td> 'step' </td> <td> '0' </td> </tr>
## <tr> <td> '%pid' </td> <td> 'pid' </td> <td> '0' </td> </tr>
## <tr> <td> 'sdelay' </td> <td> 'sdelay' </td> <td> '0' </td> </tr>
## <tr> <td> 'ffreq' </td> <td> 'ffreq' </td> <td> '0' </td> </tr>
## <tr> <td> '%user' </td> <td> 'user' </td> <td> '0' </td> </tr>
## <tr> <td> 'maxfiles' </td> <td> 'maxfiles' </td> <td> '0' </td> </tr>
## <tr> <td> 'd' </td> <td> 'device_size' </td> <td> '0' </td> </tr>
## <tr> <td> 'rlimit' </td> <td> 'rlimit' </td> <td> '0' </td> </tr>
## <tr> <td> 'records' </td> <td> 'records' </td> <td> '0' </td> </tr>
## <tr> <td> 'oflags' </td> <td> 'oflags' </td> <td> '0' </td> </tr>
## <tr> <td> 'ontapi_path' </td> <td> 'ontapi_path' </td> <td> '0' </td> </tr>
## <tr> <td> 'depth' </td> <td> 'depth' </td> <td> '0' </td> </tr>
## <tr> <td> 'runtime' </td> <td> 'runtime' </td> <td> '0' </td> </tr>
## <tr> <td> 'aios' </td> <td> 'aios' </td> <td> '0' </td> </tr>
## <tr> <td> 'procs' </td> <td> 'procs' </td> <td> '0' </td> </tr>
## <tr> <td> 'incr' </td> <td> 'incr' </td> <td> '0' </td> </tr>
## <tr> <td> 'trigger' </td> <td> 'trigger' </td> <td> '0' </td> </tr>
## <tr> <td> 'rseed' </td> <td> 'rseed' </td> <td> '0' </td> </tr>
## <tr> <td> 'lbs' </td> <td> 'lbs' </td> <td> '0' </td> </tr>
## <tr> <td> 'noprogtt' </td> <td> 'noprogtt' </td> <td> '0' </td> </tr>
## <tr> <td> 'position' </td> <td> 'position' </td> <td> '0' </td> </tr>
## <tr> <td> 'notime' </td> <td> 'notime' </td> <td> '0' </td> </tr>
## <tr> <td> 'stopon' </td> <td> 'stopon' </td> <td> '0' </td> </tr>
## <tr> <td> 'nice' </td> <td> 'nice' </td> <td> '0' </td> </tr>
## <tr> <td> 'wdelay' </td> <td> 'wdelay' </td> <td> '0' </td> </tr>
## <tr> <td> 'tkeepalive' </td> <td> 'tkeepalive' </td> <td> '0' </td> </tr>
## <tr> <td> 'hdsize' </td> <td> 'hdsize' </td> <td> '0' </td> </tr>
## <tr> <td> 'volumes' </td> <td> 'volumes' </td> <td> '0' </td> </tr>
## <tr> <td> '%device' </td> <td> 'device_path' </td> <td> '0' </td> </tr>
## <tr> <td> 'tdelay' </td> <td> 'tdelay' </td> <td> '0' </td> </tr>
## <tr> <td> 'keepalive' </td> <td> 'keepalive' </td> <td> '0' </td> </tr>
## <tr> <td> 'limit' </td> <td> 'limit' </td> <td> '0' </td> </tr>
## <tr> <td> 'noprogt' </td> <td> 'noprogt' </td> <td> '0' </td> </tr>
## <tr> <td> 'edelay' </td> <td> 'edelay' </td> <td> '0' </td> </tr>
## <tr> <td> 'flags' </td> <td> 'flags' </td> <td> '0' </td> </tr>
## <tr> <td> 'min' </td> <td> 'min' </td> <td> '0' </td> </tr>
## <tr> <td> 'sdirs' </td> <td> 'sdirs' </td> <td> '0' </td> </tr>
## <tr> <td> 'keepalivet' </td> <td> 'keepalivet' </td> <td> '0' </td> </tr>
## <tr> <td> 'capacity' </td> <td> 'capacity' </td> <td> '0' </td> </tr>
## <tr> <td> '%host' </td> <td> 'host' </td> <td> '0' </td> </tr>
## <tr> <td> 'ontap_username' </td> <td> 'ontap_username' </td> <td> '0' </td> </tr>
## <tr> <td> 'lba' </td> <td> 'lba' </td> <td> '0' </td> </tr>
## <tr> <td> 'iotseed' </td> <td> 'iotseed' </td> <td> '0' </td> </tr>
## <tr> <td> 'pattern' </td> <td> 'pattern' </td> <td> '0' </td> </tr>
## <tr> <td> 'parity' </td> <td> 'parity' </td> <td> '0' </td> </tr>
## <tr> <td> 'dirp' </td> <td> 'dirp' </td> <td> '0' </td> </tr>
## <tr> <td> 'seek' </td> <td> 'seek' </td> <td> '0' </td> </tr>
## <tr> <td> 'logu' </td> <td> 'logu' </td> <td> '0' </td> </tr>
## <tr> <td> 'prefix' </td> <td> 'prefix' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns undef. Parsed output is not yet supported

## @name ::dt
##
## @summary Parses the raw output from the "dt" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from dt with any of its options as described in the 
##    corresponding cdef file 'dt.cdefs'
## @synopsis
##      dt(%args);
## INPUTS TO METHOD CALL 'dt' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '--verbose' </td> <td> 'verbose' </td> <td> '1' </td> </tr> 
## <tr> <td> '-ignoredatacorruption' </td> <td> 'ignoredatacorruption' </td> <td> '1' </td> </tr> 
## <tr> <td> 'rdelay' </td> <td> 'rdelay' </td> <td> '0' </td> </tr> 
## <tr> <td> '-magic_trigger' </td> <td> 'magic_trigger' </td> <td> '0' </td> </tr> 
## <tr> <td> '--closedelete' </td> <td> 'closedelete' </td> <td> '1' </td> </tr> 
## <tr> <td> '-shmprot' </td> <td> 'shared_mem_prot' </td> <td> '0' </td> </tr> 
## <tr> <td> '-unlockchance' </td> <td> 'unlockchance' </td> <td> '0' </td> </tr> 
## <tr> <td> 'vrecords' </td> <td> 'vrecords' </td> <td> '0' </td> </tr> 
## <tr> <td> 'history' </td> <td> 'history' </td> <td> '0' </td> </tr> 
## <tr> <td> '-heap' </td> <td> 'use_heap' </td> <td> '1' </td> </tr> 
## <tr> <td> 'slices' </td> <td> 'slices' </td> <td> '0' </td> </tr> 
## <tr> <td> 'zapipanic' </td> <td> 'zapipanic' </td> <td> '0' </td> </tr> 
## <tr> <td> 'of' </td> <td> 'of' </td> <td> '0' </td> </tr> 
## <tr> <td> '-seed' </td> <td> 'seed' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'dir' </td> <td> '0' </td> </tr> 
## <tr> <td> 'idtype' </td> <td> 'idtype' </td> <td> '0' </td> </tr> 
## <tr> <td> 'ralign' </td> <td> 'ralign' </td> <td> '0' </td> </tr> 
## <tr> <td> 'enable' </td> <td> 'enable' </td> <td> '0' </td> </tr> 
## <tr> <td> '-version' </td> <td> 'version' </td> <td> '1' </td> </tr> 
## <tr> <td> '-bg' </td> <td> 'background' </td> <td> '1' </td> </tr> 
## <tr> <td> '-aio' </td> <td> 'async_io' </td> <td> '0' </td> </tr> 
## <tr> <td> '-iterations' </td> <td> 'iterations' </td> <td> '0' </td> </tr> 
## <tr> <td> '-rshuserfirst' </td> <td> 'rshuserfirst' </td> <td> '1' </td> </tr> 
## <tr> <td> '-nofilercore' </td> <td> 'nofilercore' </td> <td> '1' </td> </tr> 
## <tr> <td> '--write_offset_max' </td> <td> 'write_offset_max' </td> <td> '0' </td> </tr> 
## <tr> <td> '-ignorelockerrors' </td> <td> 'ignorelockerrors' </td> <td> '1' </td> </tr> 
## <tr> <td> '--filename' </td> <td> 'filename' </td> <td> '0' </td> </tr> 
## <tr> <td> '--read_offset_max' </td> <td> 'read_offset_max' </td> <td> '0' </td> </tr> 
## <tr> <td> '-noflock' </td> <td> 'noflock' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'devices_list' </td> <td> '0' </td> </tr> 
## <tr> <td> '-detailed_logging' </td> <td> 'detailed_logging' </td> <td> '1' </td> </tr> 
## <tr> <td> 'iotype' </td> <td> 'iotype' </td> <td> '0' </td> </tr> 
## <tr> <td> 'oncerr' </td> <td> 'oncerr' </td> <td> '0' </td> </tr> 
## <tr> <td> '--write_size' </td> <td> 'write_size' </td> <td> '0' </td> </tr> 
## <tr> <td> 'pkeepalive' </td> <td> 'pkeepalive' </td> <td> '0' </td> </tr> 
## <tr> <td> '--read_offset_min' </td> <td> 'read_offset_min' </td> <td> '0' </td> </tr> 
## <tr> <td> '%pid' </td> <td> 'pid' </td> <td> '0' </td> </tr> 
## <tr> <td> 'ffreq' </td> <td> 'ffreq' </td> <td> '0' </td> </tr> 
## <tr> <td> '%user' </td> <td> 'user' </td> <td> '0' </td> </tr> 
## <tr> <td> 'maxfiles' </td> <td> 'maxfiles' </td> <td> '0' </td> </tr> 
## <tr> <td> '-shmkey' </td> <td> 'shared_mem_key' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'duration_in_secs' </td> <td> '0' </td> </tr> 
## <tr> <td> '-mfiler' </td> <td> 'mfiler' </td> <td> '0' </td> </tr> 
## <tr> <td> 'ontapi_path' </td> <td> 'ontapi_path' </td> <td> '0' </td> </tr> 
## <tr> <td> 'runtime' </td> <td> 'runtime' </td> <td> '0' </td> </tr> 
## <tr> <td> 'aios' </td> <td> 'aios' </td> <td> '0' </td> </tr> 
## <tr> <td> '-instrumentation' </td> <td> 'instrumentation' </td> <td> '1' </td> </tr> 
## <tr> <td> '-rshuser' </td> <td> 'rsh_user_name' </td> <td> '0' </td> </tr> 
## <tr> <td> '-nostreams' </td> <td> 'nostreams' </td> <td> '1' </td> </tr> 
## <tr> <td> 'trigger' </td> <td> 'trigger' </td> <td> '0' </td> </tr> 
## <tr> <td> 'notime' </td> <td> 'notime' </td> <td> '0' </td> </tr> 
## <tr> <td> 'nice' </td> <td> 'nice' </td> <td> '0' </td> </tr> 
## <tr> <td> 'tkeepalive' </td> <td> 'tkeepalive' </td> <td> '0' </td> </tr> 
## <tr> <td> '-noflush' </td> <td> 'noflush' </td> <td> '1' </td> </tr> 
## <tr> <td> '-0' </td> <td> '0' </td> <td> '0' </td> </tr> 
## <tr> <td> '-verify' </td> <td> 'verify' </td> <td> '0' </td> </tr> 
## <tr> <td> '-break_on_dc' </td> <td> 'break_on_dc' </td> <td> '1' </td> </tr> 
## <tr> <td> 'noprogt' </td> <td> 'noprogt' </td> <td> '0' </td> </tr> 
## <tr> <td> '-minbsize' </td> <td> 'min_block_size' </td> <td> '0' </td> </tr> 
## <tr> <td> 'flags' </td> <td> 'flags' </td> <td> '0' </td> </tr> 
## <tr> <td> '--write_size_min' </td> <td> 'write_size_min' </td> <td> '0' </td> </tr> 
## <tr> <td> 'keepalivet' </td> <td> 'keepalivet' </td> <td> '0' </td> </tr> 
## <tr> <td> '%host' </td> <td> 'host' </td> <td> '0' </td> </tr> 
## <tr> <td> '-8' </td> <td> '8' </td> <td> '0' </td> </tr> 
## <tr> <td> 'pattern' </td> <td> 'pattern' </td> <td> '0' </td> </tr> 
## <tr> <td> '-R' </td> <td> 'R' </td> <td> '0' </td> </tr> 
## <tr> <td> 'dirp' </td> <td> 'dirp' </td> <td> '0' </td> </tr> 
## <tr> <td> '--min_sl' </td> <td> 'min_sl' </td> <td> '0' </td> </tr> 
## <tr> <td> '-no_statit' </td> <td> 'no_statit' </td> <td> '1' </td> </tr> 
## <tr> <td> '-5' </td> <td> '5' </td> <td> '0' </td> </tr> 
## <tr> <td> 'prefix' </td> <td> 'prefix' </td> <td> '0' </td> </tr> 
## <tr> <td> '-ramp' </td> <td> 'ramp' </td> <td> '0' </td> </tr> 
## <tr> <td> '-blocksize' </td> <td> 'blocksize' </td> <td> '0' </td> </tr> 
## <tr> <td> '-mode' </td> <td> 'mode' </td> <td> '0' </td> </tr> 
## <tr> <td> '--max_hl' </td> <td> 'max_hl' </td> <td> '0' </td> </tr> 
## <tr> <td> '%iodir' </td> <td> 'log_iodir' </td> <td> '0' </td> </tr> 
## <tr> <td> '%dsf' </td> <td> 'device_name' </td> <td> '0' </td> </tr> 
## <tr> <td> 'dfmt' </td> <td> 'dfmt' </td> <td> '0' </td> </tr> 
## <tr> <td> '--timeout' </td> <td> 'timeout' </td> <td> '0' </td> </tr> 
## <tr> <td> '--write_offset_min' </td> <td> 'write_offset_min' </td> <td> '0' </td> </tr> 
## <tr> <td> '-logfile' </td> <td> 'logfile' </td> <td> '0' </td> </tr> 
## <tr> <td> '--stripe_width_min' </td> <td> 'stripe_width_min' </td> <td> '0' </td> </tr> 
## <tr> <td> '--read_offset' </td> <td> 'read_offset' </td> <td> '0' </td> </tr> 
## <tr> <td> 'align' </td> <td> 'align' </td> <td> '0' </td> </tr> 
## <tr> <td> '-nopread' </td> <td> 'nopread' </td> <td> '1' </td> </tr> 
## <tr> <td> '%iotype' </td> <td> 'log_iotype' </td> <td> '0' </td> </tr> 
## <tr> <td> '-globalfds' </td> <td> 'globalfds' </td> <td> '1' </td> </tr> 
## <tr> <td> '-stats_reset' </td> <td> 'stats_reset' </td> <td> '0' </td> </tr> 
## <tr> <td> 'ontap_password' </td> <td> 'ontap_password' </td> <td> '0' </td> </tr> 
## <tr> <td> '-z' </td> <td> 'z' </td> <td> '0' </td> </tr> 
## <tr> <td> '-hard' </td> <td> 'hard' </td> <td> '1' </td> </tr> 
## <tr> <td> 'boff' </td> <td> 'boff' </td> <td> '0' </td> </tr> 
## <tr> <td> '--stripe_width_max' </td> <td> 'stripe_width_max' </td> <td> '0' </td> </tr> 
## <tr> <td> 'errors' </td> <td> 'errors' </td> <td> '0' </td> </tr> 
## <tr> <td> '-noheader' </td> <td> 'noheader' </td> <td> '1' </td> </tr> 
## <tr> <td> '-lockall' </td> <td> 'lockall' </td> <td> '1' </td> </tr> 
## <tr> <td> '--seed' </td> <td> 'crud_seed' </td> <td> '1' </td> </tr> 
## <tr> <td> 'disable' </td> <td> 'disable' </td> <td> '0' </td> </tr> 
## <tr> <td> '-create' </td> <td> 'create' </td> <td> '1' </td> </tr> 
## <tr> <td> '-nolseek' </td> <td> 'nolseek' </td> <td> '1' </td> </tr> 
## <tr> <td> '-no_sync' </td> <td> 'no_sync' </td> <td> '0' </td> </tr> 
## <tr> <td> '--style' </td> <td> 'style' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr> 
## <tr> <td> 'odtype' </td> <td> 'odtype' </td> <td> '0' </td> </tr> 
## <tr> <td> '--min_hl' </td> <td> 'min_hl' </td> <td> '0' </td> </tr> 
## <tr> <td> '--read_size_max' </td> <td> 'read_size_max' </td> <td> '0' </td> </tr> 
## <tr> <td> '-openfailok' </td> <td> 'ignore_open_fails' </td> <td> '1' </td> </tr> 
## <tr> <td> '-T' </td> <td> 'T' </td> <td> '0' </td> </tr> 
## <tr> <td> 'max' </td> <td> 'max' </td> <td> '0' </td> </tr> 
## <tr> <td> '-maxfsize' </td> <td> 'max_file_size' </td> <td> '0' </td> </tr> 
## <tr> <td> '-1' </td> <td> '1' </td> <td> '0' </td> </tr> 
## <tr> <td> 'step' </td> <td> 'step' </td> <td> '0' </td> </tr> 
## <tr> <td> 'cdelay' </td> <td> 'cdelay' </td> <td> '0' </td> </tr> 
## <tr> <td> '-nosysinfo' </td> <td> 'disable_host_monitoring' </td> <td> '1' </td> </tr> 
## <tr> <td> '-partial_write_test' </td> <td> 'partial_write_test' </td> <td> '1' </td> </tr> 
## <tr> <td> '-fileperthread' </td> <td> 'one_file_per_thread' </td> <td> '1' </td> </tr> 
## <tr> <td> '-verify' </td> <td> 'sio_verify' </td> <td> '1' </td> </tr> 
## <tr> <td> '-t' </td> <td> 't' </td> <td> '0' </td> </tr> 
## <tr> <td> '-numops' </td> <td> 'numops' </td> <td> '0' </td> </tr> 
## <tr> <td> 'depth' </td> <td> 'depth' </td> <td> '0' </td> </tr> 
## <tr> <td> '-nocleanup' </td> <td> 'nocleanup' </td> <td> '1' </td> </tr> 
## <tr> <td> 'procs' </td> <td> 'procs' </td> <td> '0' </td> </tr> 
## <tr> <td> '-direct' </td> <td> 'direct' </td> <td> '1' </td> </tr> 
## <tr> <td> 'rseed' </td> <td> 'rseed' </td> <td> '0' </td> </tr> 
## <tr> <td> 'noprogtt' </td> <td> 'noprogtt' </td> <td> '0' </td> </tr> 
## <tr> <td> 'hdsize' </td> <td> 'hdsize' </td> <td> '0' </td> </tr> 
## <tr> <td> '-filercore' </td> <td> 'filercore' </td> <td> '0' </td> </tr> 
## <tr> <td> '%device' </td> <td> 'device_path' </td> <td> '0' </td> </tr> 
## <tr> <td> '-max_latency' </td> <td> 'max_latency' </td> <td> '0' </td> </tr> 
## <tr> <td> '-6' </td> <td> '6' </td> <td> '0' </td> </tr> 
## <tr> <td> 'iobehavior' </td> <td> 'iobehavior' </td> <td> '0' </td> </tr> 
## <tr> <td> '-n' </td> <td> 'n' </td> <td> '0' </td> </tr> 
## <tr> <td> 'min' </td> <td> 'min' </td> <td> '0' </td> </tr> 
## <tr> <td> 'capacity' </td> <td> 'capacity' </td> <td> '0' </td> </tr> 
## <tr> <td> 'ontap_username' </td> <td> 'ontap_username' </td> <td> '0' </td> </tr> 
## <tr> <td> 'iotseed' </td> <td> 'iotseed' </td> <td> '0' </td> </tr> 
## <tr> <td> '--max_sl' </td> <td> 'max_sl' </td> <td> '0' </td> </tr> 
## <tr> <td> 'parity' </td> <td> 'parity' </td> <td> '0' </td> </tr> 
## <tr> <td> 'logu' </td> <td> 'logu' </td> <td> '0' </td> </tr> 
## <tr> <td> 'seek' </td> <td> 'seek' </td> <td> '0' </td> </tr> 
## <tr> <td> '-S' </td> <td> 'S' </td> <td> '1' </td> </tr> 
## <tr> <td> 'files' </td> <td> 'files' </td> <td> '0' </td> </tr> 
## <tr> <td> 'iomode' </td> <td> 'iomode' </td> <td> '0' </td> </tr> 
## <tr> <td> 'if' </td> <td> 'if' </td> <td> '0' </td> </tr> 
## <tr> <td> 'dtype' </td> <td> 'dtype' </td> <td> '0' </td> </tr> 
## <tr> <td> '-y' </td> <td> 'y' </td> <td> '0' </td> </tr> 
## <tr> <td> '-fixedfill' </td> <td> 'fixedfill' </td> <td> '0' </td> </tr> 
## <tr> <td> 'flow' </td> <td> 'flow' </td> <td> '0' </td> </tr> 
## <tr> <td> 'dsize' </td> <td> 'dsize' </td> <td> '0' </td> </tr> 
## <tr> <td> 'maxdata' </td> <td> 'maxdata' </td> <td> '0' </td> </tr> 
## <tr> <td> '-checkinodes' </td> <td> 'checkinodes' </td> <td> '1' </td> </tr> 
## <tr> <td> 'iotpass' </td> <td> 'iotpass' </td> <td> '0' </td> </tr> 
## <tr> <td> '-shm' </td> <td> 'use_shared_mem' </td> <td> '1' </td> </tr> 
## <tr> <td> '-shmI' </td> <td> 'use_intimate_shared_mem' </td> <td> '1' </td> </tr> 
## <tr> <td> '-iofailok' </td> <td> 'iofailok' </td> <td> '1' </td> </tr> 
## <tr> <td> 'ontap_cserver' </td> <td> 'ontap_cserver' </td> <td> '0' </td> </tr> 
## <tr> <td> 'maxbad' </td> <td> 'maxbad' </td> <td> '0' </td> </tr> 
## <tr> <td> '-w' </td> <td> 'w' </td> <td> '0' </td> </tr> 
## <tr> <td> '-fillonce' </td> <td> 'fillonce' </td> <td> '1' </td> </tr> 
## <tr> <td> '-partition_among_threads' </td> <td> 'partition_among_threads' </td> <td> '1' </td> </tr> 
## <tr> <td> 'ontap_nodes' </td> <td> 'ontap_nodes' </td> <td> '0' </td> </tr> 
## <tr> <td> '-think' </td> <td> 'think' </td> <td> '0' </td> </tr> 
## <tr> <td> '-rep_latency' </td> <td> 'rep_latency' </td> <td> '0' </td> </tr> 
## <tr> <td> 'bufmodes' </td> <td> 'bufmodes' </td> <td> '0' </td> </tr> 
## <tr> <td> 'stats' </td> <td> 'stats' </td> <td> '0' </td> </tr> 
## <tr> <td> '-truncate' </td> <td> 'truncate' </td> <td> '1' </td> </tr> 
## <tr> <td> 'skip' </td> <td> 'skip' </td> <td> '0' </td> </tr> 
## <tr> <td> '--read_size_min' </td> <td> 'read_size_min' </td> <td> '0' </td> </tr> 
## <tr> <td> 'retry_delay' </td> <td> 'retry_delay' </td> <td> '0' </td> </tr> 
## <tr> <td> '-kstat' </td> <td> 'enable_solaris_kstat' </td> <td> '1' </td> </tr> 
## <tr> <td> 'pf' </td> <td> 'pf' </td> <td> '0' </td> </tr> 
## <tr> <td> 'slice' </td> <td> 'slice' </td> <td> '0' </td> </tr> 
## <tr> <td> '-2' </td> <td> '2' </td> <td> '0' </td> </tr> 
## <tr> <td> '--write_size_max' </td> <td> 'write_size_max' </td> <td> '0' </td> </tr> 
## <tr> <td> '-filer' </td> <td> 'filer' </td> <td> '0' </td> </tr> 
## <tr> <td> 'sdelay' </td> <td> 'sdelay' </td> <td> '0' </td> </tr> 
## <tr> <td> '-niceoutput' </td> <td> 'nice_output' </td> <td> '1' </td> </tr> 
## <tr> <td> '-debug' </td> <td> 'debug' </td> <td> '1' </td> </tr> 
## <tr> <td> '-runtime' </td> <td> 'hammer_runtime' </td> <td> '0' </td> </tr> 
## <tr> <td> '-align' </td> <td> 'sio_align' </td> <td> '0' </td> </tr> 
## <tr> <td> 'rlimit' </td> <td> 'rlimit' </td> <td> '0' </td> </tr> 
## <tr> <td> '-s' </td> <td> 's' </td> <td> '0' </td> </tr> 
## <tr> <td> '-c' </td> <td> 'c' </td> <td> '1' </td> </tr> 
## <tr> <td> '-nomutex' </td> <td> 'nomutex' </td> <td> '1' </td> </tr> 
## <tr> <td> 'position' </td> <td> 'position' </td> <td> '0' </td> </tr> 
## <tr> <td> 'lbs' </td> <td> 'lbs' </td> <td> '0' </td> </tr> 
## <tr> <td> 'wdelay' </td> <td> 'wdelay' </td> <td> '0' </td> </tr> 
## <tr> <td> 'volumes' </td> <td> 'volumes' </td> <td> '0' </td> </tr> 
## <tr> <td> '-fill' </td> <td> 'fill' </td> <td> '1' </td> </tr> 
## <tr> <td> 'tdelay' </td> <td> 'tdelay' </td> <td> '0' </td> </tr> 
## <tr> <td> 'limit' </td> <td> 'limit' </td> <td> '0' </td> </tr> 
## <tr> <td> 'keepalive' </td> <td> 'keepalive' </td> <td> '0' </td> </tr> 
## <tr> <td> '--stripe_width' </td> <td> 'stripe_widht' </td> <td> '0' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'i' </td> <td> '0' </td> </tr> 
## <tr> <td> '-ignorefileerrors' </td> <td> 'ignorefileerrors' </td> <td> '1' </td> </tr> 
## <tr> <td> 'retry_limit' </td> <td> 'retry_limit' </td> <td> '0' </td> </tr> 
## <tr> <td> 'sdirs' </td> <td> 'sdirs' </td> <td> '0' </td> </tr> 
## <tr> <td> '-m' </td> <td> 'm' </td> <td> '0' </td> </tr> 
## <tr> <td> '-lockfiles' </td> <td> 'lockfiles' </td> <td> '1' </td> </tr> 
## <tr> <td> 'lba' </td> <td> 'lba' </td> <td> '0' </td> </tr> 
## <tr> <td> '-ignoreallerrors' </td> <td> 'ignoreallerrors' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'file_size' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'start_byte' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'block_size' </td> <td> '0' </td> </tr> 
## <tr> <td> '-7' </td> <td> '7' </td> <td> '0' </td> </tr> 
## <tr> <td> '-verify_retry' </td> <td> 'verify_retry' </td> <td> '0' </td> </tr> 
## <tr> <td> '-affine' </td> <td> 'affine' </td> <td> '1' </td> </tr> 
## <tr> <td> '-stream_per_thread' </td> <td> 'stream_per_thread' </td> <td> '1' </td> </tr> 
## <tr> <td> 'passes' </td> <td> 'passes' </td> <td> '0' </td> </tr> 
## <tr> <td> '-no_dsync' </td> <td> 'no_dsync' </td> <td> '1' </td> </tr> 
## <tr> <td> '-blocksize random' </td> <td> 'blocksize_random' </td> <td> '1' </td> </tr> 
## <tr> <td> 'logt' </td> <td> 'logt' </td> <td> '0' </td> </tr> 
## <tr> <td> 'iodir' </td> <td> 'iodir' </td> <td> '0' </td> </tr> 
## <tr> <td> '-onlyflush' </td> <td> 'onlyflush' </td> <td> '1' </td> </tr> 
## <tr> <td> '-maxbsize' </td> <td> 'max_block_size' </td> <td> '0' </td> </tr> 
## <tr> <td> '-mmap' </td> <td> 'use_mem_mapped_io' </td> <td> '0' </td> </tr> 
## <tr> <td> 'bs' </td> <td> 'bs' </td> <td> '0' </td> </tr> 
## <tr> <td> 'dlimit' </td> <td> 'dlimit' </td> <td> '0' </td> </tr> 
## <tr> <td> '-soak' </td> <td> 'soak' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'num_of_threads' </td> <td> '0' </td> </tr> 
## <tr> <td> 'iotune' </td> <td> 'iotune' </td> <td> '0' </td> </tr> 
## <tr> <td> 'alarm' </td> <td> 'alarm' </td> <td> '0' </td> </tr> 
## <tr> <td> '--write_offset' </td> <td> 'write_offset' </td> <td> '0' </td> </tr> 
## <tr> <td> '-rshhostfirst' </td> <td> 'rshhostfirst' </td> <td> '1' </td> </tr> 
## <tr> <td> '--read_size' </td> <td> 'read_size' </td> <td> '0' </td> </tr> 
## <tr> <td> '-minfsize' </td> <td> 'min_file_size' </td> <td> '0' </td> </tr> 
## <tr> <td> 'dispose' </td> <td> 'dispose' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'read' </td> <td> '0' </td> </tr> 
## <tr> <td> '-metadata' </td> <td> 'metadata' </td> <td> '1' </td> </tr> 
## <tr> <td> 'd' </td> <td> 'device_size' </td> <td> '0' </td> </tr> 
## <tr> <td> '-iscsi' </td> <td> 'iscsi' </td> <td> '1' </td> </tr> 
## <tr> <td> '-max_blksize' </td> <td> 'max_blksize' </td> <td> '0' </td> </tr> 
## <tr> <td> '-W' </td> <td> 'W' </td> <td> '0' </td> </tr> 
## <tr> <td> '-blockno' </td> <td> 'blockno' </td> <td> '1' </td> </tr> 
## <tr> <td> '-interactive' </td> <td> 'interactive' </td> <td> '1' </td> </tr> 
## <tr> <td> '--locking' </td> <td> 'locking' </td> <td> '1' </td> </tr> 
## <tr> <td> 'records' </td> <td> 'records' </td> <td> '0' </td> </tr> 
## <tr> <td> 'oflags' </td> <td> 'oflags' </td> <td> '0' </td> </tr> 
## <tr> <td> '-v' </td> <td> 'v' </td> <td> '1' </td> </tr> 
## <tr> <td> '--interval' </td> <td> 'interval' </td> <td> '0' </td> </tr> 
## <tr> <td> 'incr' </td> <td> 'incr' </td> <td> '0' </td> </tr> 
## <tr> <td> '-r' </td> <td> 'r' </td> <td> '0' </td> </tr> 
## <tr> <td> 'retry_error' </td> <td> 'retry_error' </td> <td> '0' </td> </tr> 
## <tr> <td> '-profile' </td> <td> 'collect_filer_profile' </td> <td> '1' </td> </tr> 
## <tr> <td> '-x' </td> <td> 'x' </td> <td> '0' </td> </tr> 
## <tr> <td> 'stopon' </td> <td> 'stopon' </td> <td> '0' </td> </tr> 
## <tr> <td> '-api' </td> <td> 'api' </td> <td> '0' </td> </tr> 
## <tr> <td> '-h' </td> <td> 'h' </td> <td> '1' </td> </tr> 
## <tr> <td> '-help' </td> <td> 'help' </td> <td> '1' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'f' </td> <td> '0' </td> </tr> 
## <tr> <td> 'edelay' </td> <td> 'edelay' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'rand' </td> <td> '0' </td> </tr> 
## <tr> <td> '-stop' </td> <td> 'stop' </td> <td> '0' </td> </tr> 
## <tr> <td> '-3' </td> <td> '3' </td> <td> '0' </td> </tr> 
## <tr> <td> '-9' </td> <td> '9' </td> <td> '0' </td> </tr> 
## <tr> <td> '-iops' </td> <td> 'iops' </td> <td> '0' </td> </tr> 
## <tr> <td> '--threads' </td> <td> 'threads' </td> <td> '0' </td> </tr> 
## <tr> <td> '-dafs' </td> <td> 'dafs' </td> <td> '0' </td> </tr> 
## <tr> <td> '-l' </td> <td> 'l' </td> <td> '1' </td> </tr> 
## <tr> <td> '-4' </td> <td> '4' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub dt {
    my %args = @_;
    my $error_message;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my $error       = _check_error(
        "raw_output"        => $output,
        "command"           => $command,
        "ptr_error_message" => \$error_message,
    );
    if($error) {
        if($error_level == 0) {
            $Log->warn ( "Due to a bug in 'dt', a zero execution status"
                         ."\n was returned even when the command"
                         ."\nexecution has failed.\n" );
        }
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text =>  "Execution status was $error_level "
                    ."indicating that the command has failed",
            response_object => $args{response_object}
        );
    }
    #Output is not required to be parsed
    return [];
} ## end sub dt

1;
