# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/default/iozone.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse iozone command output
##
## @description
##     This library supports error checking for iozone command, but output parsing is not supported as of now.
##
## @synopsis
##    use NACL::APISet::Host::CLI::default::iozone;
##
##    iozone(%args);
##
## @author Abhsihek Kumar(kabhishe@netapp.com), dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::default::iozone;

use strict;
use warnings;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

## @name ::iozone
##
## @summary Parses the raw output from the "iozone" command
##
## @description
##   This library supports error checking for iozone command, but output parsing is not supported as of now.
## @synopsis
##      iozone(%args);
## INPUTS TO METHOD CALL 'iozone' ON APISET OBJECT
##
## SPECIAL OPTIONS
##
## tool_full_path
## (Optional)
## The complete path to the tool.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /usr/software/test/bin/iozone_6_22, /u/myuser/dir/iozone.experimental
##
## tool_dir_path
## (Optional)
## The absolute path to the dir which contains the executable of the tool.
## This shouldn't include version or name of the tool.
## Defaults to the test param TESTBINDIR_UNIX (/usr/software/test/bin is the usual
## default) in unix like OSes and TESTBINDIR_WINDOWS (x:/bin is the usual default)
## in Windows.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: /usr/software/test/bin, /u/myuser/dir
##
## tool_version
## (Optional)
## The specific version of the tool to be selected.
## Note that tool_full_path is mutually exclusive with tool_dir_path and tool_version.
## Eg: 'current', '3.397'
## This would select iozone.current, iozone.3.397 respectively.
##
## TOOL SPECIFIC OPTIONS:
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-M' </td> <td> 'uname' </td> <td> '1' </td> </tr>
## <tr> <td> '-O' </td> <td> 'ops-per-sec' </td> <td> '1' </td> </tr>
## <tr> <td> '-g' </td> <td> 'maxfilesize' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'children' </td> <td> '0' </td> </tr>
## <tr> <td> '-B' </td> <td> 'map' </td> <td> '1' </td> </tr>
## <tr> <td> '-K' </td> <td> 'jitter' </td> <td> '1' </td> </tr>
## <tr> <td> '-l' </td> <td> 'min-number-procs' </td> <td> '0' </td> </tr>
## <tr> <td> '-+p' </td> <td> 'percentage' </td> <td> '0' </td> </tr>
## <tr> <td> '-z' </td> <td> 'test-all' </td> <td> '1' </td> </tr>
## <tr> <td> '-+X' </td> <td> 'enable-short-circuit' </td> <td> '1' </td> </tr>
## <tr> <td> '-q' </td> <td> 'maxrecsize' </td> <td> '0' </td> </tr>
## <tr> <td> '-k' </td> <td> 'kdepth' </td> <td> '0' </td> </tr>
## <tr> <td> '-x' </td> <td> 'off-stone-walling' </td> <td> '1' </td> </tr>
## <tr> <td> '-L' </td> <td> 'cacheline-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'include-close' </td> <td> '1' </td> </tr>
## <tr> <td> '-F' </td> <td> 'paths' </td> <td> '0' </td> </tr>
## <tr> <td> '-G' </td> <td> 'msync' </td> <td> '1' </td> </tr>
## <tr> <td> '-S' </td> <td> 'cache-size' </td> <td> '0' </td> </tr>
## <tr> <td> '-D' </td> <td> 'masync' </td> <td> '1' </td> </tr>
## <tr> <td> '-P' </td> <td> 'start-cpu' </td> <td> '0' </td> </tr>
## <tr> <td> '-U' </td> <td> 'mount-point' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'auto2-mode' </td> <td> '1' </td> </tr>
## <tr> <td> '-v' </td> <td> 'version' </td> <td> '1' </td> </tr>
## <tr> <td> '-N' </td> <td> 'microsec-per-ops' </td> <td> '1' </td> </tr>
## <tr> <td> '-Y' </td> <td> 'read-telemetry-filename' </td> <td> '0' </td> </tr>
## <tr> <td> '-+t' </td> <td> 'network-performance' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'filename' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'purge-on' </td> <td> '1' </td> </tr>
## <tr> <td> '-+w' </td> <td> 'in-buffer' </td> <td> '0' </td> </tr>
## <tr> <td> '-y' </td> <td> 'minrecsize' </td> <td> '0' </td> </tr>
## <tr> <td> '-+Z' </td> <td> 'data-compatibility' </td> <td> '1' </td> </tr>
## <tr> <td> '-J' </td> <td> 'milliseconds' </td> <td> '0' </td> </tr>
## <tr> <td> '-+d' </td> <td> 'file-io-diagnostic' </td> <td> '1' </td> </tr>
## <tr> <td> '-+n' </td> <td> 'no-retest' </td> <td> '1' </td> </tr>
## <tr> <td> '-W' </td> <td> 'lock' </td> <td> '1' </td> </tr>
## <tr> <td> '-u' </td> <td> 'max-number-procs' </td> <td> '0' </td> </tr>
## <tr> <td> '-w' </td> <td> 'not-unlink' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'auto-mode' </td> <td> '1' </td> </tr>
## <tr> <td> '-X' </td> <td> 'write-telemetry-filename' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'excel-worksheet' </td> <td> '0' </td> </tr>
## <tr> <td> '-+r' </td> <td> 'enable-sync' </td> <td> '1' </td> </tr>
## <tr> <td> '-Z' </td> <td> 'enable-nmap-file-io' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'test-to-run' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'writes-sync' </td> <td> '1' </td> </tr>
## <tr> <td> '-+y' </td> <td> 'buffer-across' </td> <td> '0' </td> </tr>
## <tr> <td> '-E' </td> <td> 'extension-test' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'minfilesize' </td> <td> '0' </td> </tr>
## <tr> <td> '-H' </td> <td> 'depth' </td> <td> '0' </td> </tr>
## <tr> <td> '-C' </td> <td> 'show-byte-trans' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'filesize' </td> <td> '0' </td> </tr>
## <tr> <td> '-R' </td> <td> 'excel-report' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'microsec-delay' </td> <td> '0' </td> </tr>
## <tr> <td> '-+C' </td> <td> 'buffer-not-across' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'multiple-buffers' </td> <td> '1' </td> </tr>
## <tr> <td> '-j' </td> <td> 'stride' </td> <td> '0' </td> </tr>
## <tr> <td> '-Q' </td> <td> 'offset-file' </td> <td> '1' </td> </tr>
## <tr> <td> '-e' </td> <td> 'include-flush' </td> <td> '1' </td> </tr>
## <tr> <td> '-+u' </td> <td> 'enable-cpu-utilization' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'record_size' </td> <td> '0' </td> </tr>
## <tr> <td> '-h' </td> <td> 'help' </td> <td> '1' </td> </tr>
## <tr> <td> '-I' </td> <td> 'use-vxfs' </td> <td> '1' </td> </tr>
## <tr> <td> '-T' </td> <td> 'posix-threads' </td> <td> '1' </td> </tr>
## <tr> <td> '-V' </td> <td> 'pattern' </td> <td> '0' </td> </tr>
## <tr> <td> '-+m' </td> <td> 'cluster-filename' </td> <td> '0' </td> </tr>
## <tr> <td> '-+x' </td> <td> 'multiplier' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
##   Returns: Output parsing is not supported.
## @examples

sub iozone {
    my %args = @_;

    my $error_message;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();

    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text            => $output,
            response_object => $resp
        );
    }

    #Output is not required to be parsed
    return [];
}

1;
