# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/default/vxdmpadm.pm#1 $
#
# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse vxdmpadm command output
##
## @description
##    This is a library to parse the raw output of vxdmpadm command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "vxdmpadm" will be vxdmpadm()
##
## @synopsis
##    use NACL::APISet::Host::CLI::default::vxdmpadm;
##
##    vxdmpadm(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Host::CLI::default::vxdmpadm;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

## @name ::vxdmpadm_stat_restored
##
## @summary Parses the raw output from the "vxdmpadm stat restored" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from vxdmpadm_stat_restored with any of its options as described in the
##    corresponding cdef file 'vxdmpadm.cdefs'
## @synopsis
##      vxdmpadm_stat_restored(%args);
## INPUTS TO METHOD CALL 'vxdmpadm_stat_restored' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = vxdmpadm stat restored
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;The number of daemons running : 1
##&nbsp;The interval of daemon: 300
##&nbsp;The policy of daemon: check_disabled
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'the_policy_of_daemon' => 'check_disabled',
##&nbsp;             'the_interval_of_daemon' => '300',
##&nbsp;             'the_number_of_daemons_running' => '1'
##&nbsp;           }
##&nbsp;        ];
##&nbsp;
sub vxdmpadm_stat_restored {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );

    for ( my $i = 1; $i <= $#lines; $i++ ) {
        my @values = split( /\s*:\s*/, $lines[$i] );
        $result_array[0]->{ to_key_name( item => $values[0] ) } = $values[1];
    }
    return \@result_array;

} ## end sub vxdmpadm_stat_restored

sub _check_error {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

1;
