#
# Copyright (c) 2001-2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary API Set for SMTT GUI commands
## @author santosh6@netapp.com, rohit4@netapp.com, dl-nacl-dev@netapp.com
## @status Review
## @pod here

package NACL::APISet::Host::GUI::SMTT;

#
# Compiler Directives
#
use strict;
use warnings;

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();

#use Module Imports
use Data::Dumper;
use NACL::APISet::Response::GUI;
use NACL::APISet::Exceptions::ConnectionFailedException qw(:try);
use Params::Validate qw(validate_pos SCALAR );
use NACL::APISet::Session::GUI::SMTT::V1_3;
use NACL::APISet::Session::GUI::SMTT::V2_2;
use NACL::APISet::Session::GUI::SMTT::V2_3;
use NACL::APISet::Session::GUI::SMTT::V2_0;
use NACL::APISet::Session::GUI::SMTT::V1_4;
use NACL::APISet::Session::GUI::SMTT::V3_0;
use NACL::APISet::Session::GUI::SMTT::V3_1;

my $nate_params = NATE::ParamSet->new(global => 1);

# Import Exception class
use NATE::BaseException qw(:try);

our $AUTOLOAD;

# Base class
use base qw(NACL::APISet::Host);

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_Session
# Objective: Initialize the SMTT Session object after checking version
# Details:   This method establishes the Session and initializes the APISet
#            object with the session object.
##############################################################################

sub _init_Session {

    my ($self, %args) = @_;
    $Log->enter();

    # Read all the parameters from Tharnhost
    my $hostobj = $self->{hostobj};

    $args{remote_server_addr} = $nate_params->get("SMTTSERVERIP") || $hostobj->{SMTTSERVERIP} || $hostobj->{default_ip}
        if($nate_params->get("SMTTSERVERIP") || $hostobj->{SMTTSERVERIP} || $hostobj->{default_ip}
    );

    $args{port}               = $nate_params->get("SELPORT")  || $hostobj->{SELPORT}
        if($nate_params->get("SELPORT")   || $hostobj->{SELPORT}
    );

    $args{browser_name}       = $nate_params->get("BROWSER")  || $hostobj->{BROWSER}
        if($nate_params->get("BROWSER")   || $hostobj->{BROWSER}
    );

    $args{browser_url}        = $nate_params->get("SMTTURL")  || $hostobj->{SMTTURL}
        if($nate_params->get("SMTTURL")   || $hostobj->{SMTTURL}
    );

    $args{username}           = $nate_params->get("username") || $hostobj->{conn}->{telnet}->{username} || $hostobj->{username}
        if($nate_params->get("username") || $hostobj->{conn}->{telnet}->{username} || $hostobj->{username}
    );

    $args{password}           = $nate_params->get("password") || $hostobj->{conn}->{telnet}->{password} || $hostobj->{password}
        if($nate_params->get("password") || $hostobj->{conn}->{telnet}->{password} || $hostobj->{password}
    );

    $args{hostobj_for_windows}= $hostobj;

    # Finding out the version of SMTT
    my $session_pkg;
    my $conn = Connectrec->new(
        host=>$hostobj,
        template=>"login",
    );
    my $command='set PATH=%path%;C:\Windows\system32;C:\Windows;'
        .'C:\Program Files\Java\jre1.8.0_45\bin;'
        .'C:\Program Files\Java\jre1.8.0_112\bin;'
        .'C:\Program Files\Java\jre7\bin;'
        .'C:\Program Files\NetApp\7-Mode Transition Tool\bin;';
    $command .= $hostobj->{Path_To_SMTT_Bin};
    $conn->execute(command => "$command");
    my $version = $conn->execute(command => "transition version");
    chomp($version);
    if( $version =~ /2.2\.\d/ ){
        $Log->comment("Pointing package to V2.2 with respect to the 7MTT version");
        $session_pkg = "NACL::APISet::Session::GUI::SMTT::V2_2";
    }elsif(($version =~ /2\.3\.\d/) ||($version =~ /2\.3/) ){
        $Log->comment("Pointing package to V2.3 with respect to the 7MTT version");
        $session_pkg = "NACL::APISet::Session::GUI::SMTT::V2_3";
    }elsif( $version =~ /1.3\.\d/ ){
        $Log->comment("Pointing package to V1.3 with respect to the 7MTT version");
        $session_pkg = "NACL::APISet::Session::GUI::SMTT::V1_3";
    }elsif( $version =~ /2.0\.\d/ ){
        $Log->comment("Pointing package to V2.0 with respect to the 7MTT version");
        $session_pkg = "NACL::APISet::Session::GUI::SMTT::V2_0";
    }elsif( $version =~ /1.4\.\d/ ){
        $Log->comment("Pointing package to V1.4 with respect to the 7MTT version");
        $session_pkg = "NACL::APISet::Session::GUI::SMTT::V1_4";
    }elsif( $version =~ /3.0\.\d/ ){
        $Log->comment("Pointing package to V3.0 with respect to the 7MTT version");
        $session_pkg = "NACL::APISet::Session::GUI::SMTT::V3_0";
    }elsif( $version =~ /3.1\.\d/ ){
        $Log->comment("Pointing package to V3.1 with respect to the 7MTT version");
        $session_pkg = "NACL::APISet::Session::GUI::SMTT::V3_1";
    }else{
        $Log->warn("Unable to do versioning.. Defaulting to V3.1");
        $session_pkg = "NACL::APISet::Session::GUI::SMTT::V3_1";
    }

    try {
        $self->{session} = $session_pkg->new(%args);
    }
    otherwise {
        my $exception = shift;
        $Log->exit();
        NACL::APISet::Exceptions::ConnectionFailedException->throw(
            "Failed to Create Selenium Session object : " . $exception->text());
    }

    $Log->exit();
}

###############################################################################
# Method Name: AUTOLOAD
# Objective:  This function will be called when method is missing
###############################################################################

sub AUTOLOAD {
    my ($self, %args) = @_;
    $Log->enter();
    my ($response, $auto);
    my $method = $AUTOLOAD;
    $method =~ s/.*://;    # strip fully-qualified portion
    if ($method eq "DESTROY") {
        $Log->exit();
        return;
    }

    # Call invoke with the arguments given to execute the command
    $Log->exit();
    my $output;
    eval {
        my $exception;
        try {
            $auto = $self->{session}->$method(%args);
        } otherwise {
            $exception = shift;
        };
        $output=($exception->text()) if ($exception);
        die $output if ($exception);
        unless (ref $auto) {
            $auto = {};
        }
        $auto->{'method_name'} = $method;
        $auto->{execution_status} = "PASS";
    };
    if ($@) {
        $auto                     = {};
        $auto->{'method_name'}    = $method;
        $auto->{execution_status} = "FAIL";
        $auto->{output}           = $output;
    }

    $response = NACL::APISet::Response::GUI->new($auto);
    $Log->exit();
    return $response;
} ## end sub AUTOLOAD

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Host::GUI::SMTT - Base class for API Sets for GUI operations

=head1 SYNOPSIS

use NACL::APISet::Host::GUI::SMTT;

=head1 DESCRIPTION

A C<NACL::APISet::Host::GUI::SMTT> is used to Initialize the API Set and make the API Set ready for work.

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (rohit4@netapp.com, dl-nacl-dev@netapp.com)

=back

 $Id$
 Last revised  $Date: 04/18/2014 $ by $Author: rohit4 $
               $Date: 06/24/2015 $ by $Author: santosh6 $
               $Date: 07/18/2016 $ by $Author: dinoc $
=cut
