# $Id$
#---------------------------------------------------
# Copyright (c) 2014 Network Appliance, Inc.
# All rights reserved.
#---------------------------------------------------
## @summary
##      Wrapper Utility for Selnium webDriver functions.
##
## @description
##      This modules contains all helper functions for SM libraries
##      to interact with selenium webdriver
##
## @synopsis
##      use NACL::APISet::Host::GUI::SysManager::NWFUtils;
##
##      _double_click(%args);
##      _find_element(%args);
##      check_ascending(%args);
##      check_descending(%args);
##      check_for_error(%args);
##      check_warning(%args);
##      click(%args);
##      click_child_row_with_texts(%args);
##      click_item_in_list(%args);
##      click_row_with_texts(%args);
##      click_row_with_texts_without_ascending(%args);
##      control_key_down(%args);
##      control_key_up(%args);
##      does_row_contain_exact_text(%args);
##      does_row_contain_texts(%args);
##      does_row_contain_texts_without_ascending(%args);
##      drag_and_drop(%args);
##      fire_event(%args);
##      get_all_window_ids(%args);
##      get_all_window_names(%args);
##      get_all_window_titles(%args);
##      get_attribute(%args);
##      get_grid_data(%args);
##      get_row_data_with_text(%args);
##      get_text(%args);
##      get_value(%args);
##      is_button_enabled(%args);
##      is_checked(%args);
##      is_element_present(%args);
##      key_down(%args);
##      key_up(%args);
##      keys(%args);
##      mouse_down(%args);
##      mouse_down_only(%args);
##      mouse_over(%args);
##      mouse_up(%args);
##      select_window(%args);
##      set_checkbox(%args);
##      set_checkbox_in_row_with_text(%args);
##      set_combobox_value(%args);
##      set_option(%args);
##      set_radiobutton_in_row_with_text(%args);
##      set_timeout(%args);
##      sort_filter(%args);
##      select_from_dropdown(%args)
##      stop(%args);
##      type(%args);
##      type_in_row_with_text(%args);
##      type_keys(%args);
##      wait_for_element_not_present(%args);
##      wait_for_element_present(%args);
##      wait_for_loading(%args);
##      wait_for_page_to_load(%args);
##
## @status public
##
## @author arunw@netapp.com, dl-nacl-dev@netapp.com
## @change 2014 neelkant@netapp.com Added webdriver capability

package NACL::APISet::Host::GUI::SysManager::NWFUtils;

#
## Compiler directives.
#
use strict;
use warnings;

## Module import.
use Selenium::Remote::Driver;
use Selenium::Remote::WebElement;
use Selenium::Remote::WDKeys;

use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::APISet::Host::GUI::SysManager::GConst;
use NATE::BaseException qw(:try);

use vars qw(@ISA);
@ISA = qw(Selenium::Remote::Driver);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

###############################################################################
## @name ::select_from_dropdown
##
## @summary
##     Set a value in latest NWF dropdown list widget
## @description
##     This method can be used to Setting a value in the Combobox.
##     This method is used to select a value from the list in the Combobox.
## @synopsis
##     select_from_dropdown(%args);
## INPUTS TO METHOD CALL 'select_from_dropdown' ON APISET OBJECT

sub select_from_dropdown {
    my ( $self, $debugID, $value, $partial ) = @_;
    $Log->enter() if $may_enter;
    $Log->comment("Selecting \"$value\" from dropdown");

    my ( $retValue, $comboInputID );

    $self->wait_for_element_present("$debugID");
    if ( $debugID =~ /input$/ ) {
        $comboInputID = $debugID;
        $retValue = $self->get_value($comboInputID);
    } elsif ( $debugID =~ /img$/ ) {
        $comboInputID = $debugID;
        $debugID =~ s/img$/input/;
        $retValue = $self->get_value($debugID);
    }else {
        if ( $self->is_element_present("$debugID-input") ) {
            $comboInputID = "$debugID-input";
        } elsif ( $self->is_element_present("$debugID/input") ) {
            $comboInputID = "$debugID/input";
        } elsif ( $self->is_element_present("//div[\@id='$debugID']/input") ) {
            $comboInputID = "//div[\@id='$debugID']/input";
        } else {
            $Log->comment("Debug ID not found $debugID");
            NATE::BaseException->throw("Debug ID not found $debugID");
        }
        $retValue = $self->get_value($comboInputID);
    }
    $Log->comment("default-selected value : $retValue");

    if ( $partial and $retValue !~ /$value/ ) {
        $self->mouse_down($comboInputID);
        if ( !$self->is_element_present("//div[contains(\@class,'x-combo-list-inner')]")
         and !$self->is_element_present("//div[contains(\@class,'x-combo-list-item')]")
            ) {
            $self->mouse_down($comboInputID);
        }
        if ( $self->is_element_present("//div[contains(\@class,'x-combo-list-inner')]") ) {
            my $div_item = "//div[contains(\@class,'x-combo-list-inner')]"
                     . "//div[contains(text(),'$value')]";
            my $span_item = "//div[contains(\@class,'x-combo-list-inner')]"
                     . "//span[contains(text(),'$value')]";
            my $div_2_item = "//div[contains(\@class,'x-combo-list-inner') and contains(text(),'$value')]";

            if ( $self->is_element_present($div_item) ){
                $self->mouse_down($div_item);
            } elsif( $self->is_element_present($span_item) ){
                $self->mouse_down($span_item);
            } elsif ( $self->is_element_present($div_2_item) ){
                $self->mouse_down($div_2_item);
            } else {
                 $Log->comment("Item $value not found in $comboInputID");
                 NATE::BaseException->throw(
                     "Value: \"$value\" not present in combobox list");
            }
        } elsif (
            $self->is_element_present("//div[contains(\@class,'x-combo-list-item')]") ){
            my $div_item = "//div[contains(\@class,'x-combo-list-item')]"
                     . "//div[contains(text(),'$value')]";
            my $span_item = "//div[contains(\@class,'x-combo-list-item')]"
                     . "//span[contains(text(),'$value')]";
            my $div_2_item = "//div[contains(\@class,'x-combo-list-item') and contains(text(),'$value')]";

            if ( $self->is_element_present($div_item) ){
                $self->mouse_down($div_item);
            } elsif( $self->is_element_present($span_item) ){
                $self->mouse_down($span_item);
            } elsif ( $self->is_element_present($div_2_item) ){
                $self->mouse_down($div_2_item);
            } else {
                 $Log->comment("Item $value not found in $comboInputID");
                 NATE::BaseException->throw(
                     "Value: \"$value\" not present in combobox list");
            }
        }
    } elsif ( $retValue ne $value ) {
        $self->mouse_down($comboInputID);
        my $div_item = "//div[contains(\@class,'x-combo-list-inner')]"
                 . "/div[text()='$value')]";
        my $span_item = "//div[contains(\@class,'x-combo-list-inner')]"
                 . "/div/span[text()='$value']";

        if ( $self->is_element_present($div_item) ){
            $self->mouse_down($div_item);
        } elsif ($self->is_element_present($span_item) ){
            $self->mouse_down($span_item);
        } else {
            NATE::BaseException->throw(
                "Value: \"$value\" not present in combobox list");
         }
    } else {
        $Log->comment("Item $value already selected ");
    }

    $Log->exit() if $may_exit;
    return 1;
} ## end sub select_from_dropdown
###############################################################################
## @name ::set_combobox_value
##
## @summary
##     Set a value in NWF combobox using GUI.
## @description
##     This method can be used to Setting a value in the Combobox.
##     This method is used to select a value from the list in the Combobox.
## @synopsis
##     set_combobox_value(%args);
## INPUTS TO METHOD CALL 'set_combobox_value' ON APISET OBJECT

sub set_combobox_value {
    my ( $self, $debugID, $value, $comp ) = @_;
    $Log->enter() if $may_enter;

    $Log->comment("Selecting \"$value\" from combobox");

    my ( $retValue, $comboInputID );

    $self->wait_for_element_present("$debugID");

    if ( $self->is_element_present("$debugID-input") ) {
        $comboInputID = "$debugID-input";
    }
    elsif ( $self->is_element_present("$debugID/input") ) {
        $comboInputID = "$debugID/input";
    }
    elsif ( $self->is_element_present("//div[\@id='$debugID']/input") ) {
        $comboInputID = "//div[\@id='$debugID']/input";
    }
    else {
        $Log->comment("Debug ID not found $debugID");
        return 0;
    }

    $retValue = $self->get_value($comboInputID);
    if ( defined $comp ) {
        if ( $comp eq 'partial' ) {
            $self->mouse_down("//div[\@id='$debugID']/img");
            eval {
                $self->wait_for_element_present(
"//div[contains(\@class,'x-combo-list-item') and contains(text(),'$value')]",
                    $GENERIC_WAIT
                );
            };
            if ($@) {
                NATE::BaseException->throw(
                    "Value: \"$value\" not present in combobox list");
            }
            $self->mouse_down(
"//div[contains(\@class,'x-combo-list-item') and contains(text(),'$value')]"
            );

        } ## end if ($comp eq 'partial')
    }
    else {
        if ( $retValue ne $value ) {
            $self->mouse_down("//div[\@id='$debugID']/img");
            $self->wait_for_loading();
	    sleep(2);
            eval {
                $self->mouse_down(
"//div[contains(\@class,'x-combo-list-item') and text()='$value']"
                );

            };
            if ($@) {
                $Log->comment(
                    "couldn't find vaule in combox with default path");
                eval {
                    $self->mouse_down(
"//div[contains(\@class,'x-combo-list-item')]/span[text()='$value']"
                    );
                };
                if ($@) {
                    NATE::BaseException->throw(
                        "Value: \"$value\" not present in combobox list");
                }
            }
        } ## end if ($retValue ne $value)
    } ## end else [ if (defined $comp)
    $Log->exit() if $may_exit;
    return 1;
} ## end sub set_combobox_value

###############################################################################
## @name ::set_radiobutton_in_row_with_text
## @summary
##     Enable/Disable Radio Button using GUI.
## @description
##     Based on the input provided, the radio button is enabled or disabled.
## @synopsis
##     set_radiobutton_in_row_with_text(%args);
## INPUTS TO METHOD CALL 'set_radiobutton_in_row_with_text' ON APISET OBJECT
##
sub set_radiobutton_in_row_with_text {
    my ( $self, $debugID, @texts ) = @_;
    $Log->enter() if $may_enter;
    my $baseDebugID =
      "//div[\@id='$debugID']/div/div/div/div/div/table/tbody/tr";

    my ( $rowDebugID, $radioID );

    my ( $tempID1, $tempID2 );

    foreach my $text (@texts) {
        $tempID1 = $baseDebugID
          . "/td/div/div/div/table/tbody/tr[td/span/div/label[text()='$text']]";
        $tempID2 = $baseDebugID . "[td/div/div/label[text()='$text']]";

        if ( $self->is_element_present($tempID1) ) {
            $rowDebugID = $tempID1;
            $radioID    = $rowDebugID . "/td/span/div/input";
        }
        elsif ( $self->is_element_present($tempID2) ) {
            $rowDebugID = $tempID2;
            $radioID    = $rowDebugID . "/td/div/div/input";
        }
        else {
            NATE::BaseException->throw(
                "Row with \"$text\" not present in grid \"$debugID\" ");
        }
    } ## end foreach my $text (@texts)

    if ($radioID) {
        $self->click($radioID);
        $Log->comment("Radiobox with @texts in table row is selected");
    }
    else {
        $Log->comment("No such text box found in row with @texts");
    }
    $Log->exit() if $may_exit;
    return 1;
} ## end sub set_radiobutton_in_row_with_text

###############################################################################
## @name ::sort_filter
##
## @summary
##     Set multiple filters.
## @description
##     This function can be used to set filter values in columns inside the table.
## @synopsis
##     sort_filter(%args);
## INPUTS TO METHOD CALL 'sort_filter' ON APISET OBJECT
##
sub sort_filter {

    my ( $self, $gridID, $text ) = @_;
    foreach my $text1 ( @{$text} ) {
        my @column_value = split( /:/, $text1 );

        foreach my $count ( 1 .. 10 ) {
            my $baseID =
"//div[contains(\@id,'$gridID')]/div/div/div/div/div/div/table/tbody/tr/td[$count]/div/";
            my $columnID =
              "//table/tbody/tr/td[$count]/div/span[text()='$column_value[0]']";

            if ( $self->is_element_present($columnID) ) {
                $Log->comment("Filter the column for $column_value[0]");
                my $dowID = $baseID . "a";
                $self->click($dowID);
                my $filterID =
                  "//div[contains(\@class,'filter-menu-item')]/input";
                if ( $self->is_element_present($filterID) ) {
                    $self->type( $filterID, "$column_value[1]" );
                    $self->fire_event( $filterID, "focus" );

# using ascii values for entering keys are not best to use but for time being we are using
# this and will be replaced once we find a better way to do this. Also we can use below 2 lines
# but this has another problem so time being i am using ascii values
                    $self->key_down( $filterID, "\13" );
                    $self->key_up( $filterID, "\13" );
                    $Log->comment(
"Filter for $column_value[0] column is set to $column_value[1]"
                    );

                }
                else {
                    $Log->comment(
                        "No filer option available for $column_value[0] column"
                    );
                }
                $self->mouse_down($dowID);
                $self->wait_for_element_not_present($filterID);
                last;
            } ## end if ($self->is_element_present...
        } ## end foreach my $count (1 .. 10)
    } ## end foreach my $text1 (@{$text})

    return 1;
} ## end sub sort_filter

###############################################################################
## @name ::set_checkbox
##
## @summary
##     Set value for a checkbox.
## @description
##     This function can be used to enable or disable the checkbox.
## @synopsis
##     set_checkbox(%args);
## INPUTS TO METHOD CALL 'set_checkbox' ON APISET OBJECT
##
sub set_checkbox {
    my ( $self, $debugID, $value ) = @_;
    $Log->enter() if $may_enter;
    my $toogleButtonID = "//div[\@id='" . $debugID . "']/input";
    $value = lc $value;

    my $retValue = $self->is_checked($toogleButtonID);
    if (   ( $value eq "enable" && not $retValue )
        || ( $retValue && $value eq "disable" ) )
    {
        $Log->comment("Clicking $toogleButtonID to $value it");
        $self->click($toogleButtonID);
    }
    elsif ( $value ne "enable" && $value ne "disable" ) {
        NATE::BaseException->throw("Invalid value. Use enable/disable");
    }
    else {
        $Log->comment("$toogleButtonID is already $value");
    }
    $Log->exit() if $may_exit;
    return 1;
} ## end sub set_checkbox

###############################################################################
## @name ::get_row_data_with_text
##
## @summary
##     Get row data from a table.
## @description
##    This method is used to fetch the row data from a grid, which contains the input text.
## @synopsis
##     get_row_data_with_text(%args);
## INPUTS TO METHOD CALL 'get_row_data_with_text' ON APISET OBJECT
##
sub get_row_data_with_text {
    my ( $self, $debugID, @texts ) = @_;
    $Log->enter() if $may_enter;
    my $baseDebugID = "//div[\@id='$debugID']//tbody/tr";

    my $rowDebugID;

    foreach my $text (@texts) {
        my @tempIDS;
        push( @tempIDS, $baseDebugID . "[td//div[text(),'$text']]" );
        push( @tempIDS, $baseDebugID . "[td//div//span[text(),'$text']]" );
        push( @tempIDS, $baseDebugID . "[td//div[contains(text(),'$text')]]" );
        push( @tempIDS,
            $baseDebugID . "[td//div//span[contains(text(),'$text')]]" );
        push( @tempIDS,
            "//div[\@id='$debugID']//div[contains(text(),'$text')]" );
        push( @tempIDS,
            "//div[\@id='$debugID']//tbody/tr/td/div[contains(text(),'$text')]"
        );

        foreach my $tempid (@tempIDS) {
            if ( $self->is_element_present($tempid) ) {
                $rowDebugID = $tempid;
                $Log->comment("Row with \"$text\" found");
                last;
            }
        }
        unless ($rowDebugID) {
            NATE::BaseException->throw(
                "Row with \"$text\" not present in grid \"$debugID\" ");
        }
    } ## end foreach $text (@texts)
    $Log->comment( "Row data : " . $self->get_text($rowDebugID) );
    $Log->exit() if $may_exit;
    return $self->get_text($rowDebugID);
} ## end sub get_row_data_with_text

###############################################################################
## @name ::set_checkbox_in_row_with_text
##
## @summary
##     Set value for checkbox in a grid row.
## @description
##     This method is used to enable or disable checkbox in a row of
##      the grid which contains the input text.
## @synopsis
##     set_checkbox_in_row_with_text(%args);
## INPUTS TO METHOD CALL 'set_checkbox_in_row_with_text' ON APISET OBJECT
##
sub set_checkbox_in_row_with_text {
    my ( $self, $debugID, $value, @texts ) = @_;
    $Log->enter() if $may_enter;
    my $baseDebugID  = "//div[\@id='$debugID']//div/table/tbody/tr";
    my $baseDebugID1 = "//div[\@id='$debugID']//table/tbody/tr";
    my $rowDebugID;
    my $checkboxDebugID;
    $Log->comment(" The Values are $debugID, $value, @texts");

    my (
        $text,        $checkboxID,  $checkboxIDOn, $checkboxID1,
        $checkboxID2, $checkboxID3, $checkboxID4
    );

    foreach my $text (@texts) {
        my @tempIDS;
        push( @tempIDS, $baseDebugID . "[td//div[text(),'$text']]" );
        push( @tempIDS, $baseDebugID . "[td//div//span[text(),'$text']]" );
        push( @tempIDS, $baseDebugID . "[td//div[contains(text(),'$text')]]" );
        push( @tempIDS, $baseDebugID . "/td/div/div/div[div[text()='$text']]" );
        push( @tempIDS, $baseDebugID );
        push( @tempIDS, $baseDebugID1 . "[td/div[text()='$text']]" );

        foreach my $tempid (@tempIDS) {
            if ( $self->is_element_present($tempid) ) {
                $rowDebugID = $tempid;
                $Log->comment("Row with \"$text\" found");
                last;
            }
        }
        unless ($rowDebugID) {
            NATE::BaseException->throw(
                "Row with \"$text\" not present in grid \"$debugID\" ");
        }
    } ## end foreach $text (@texts)

    $checkboxID = $rowDebugID
      . "/td/div/div[contains(\@class,'x-grid3-check-col x-grid3-check-col')]";
    $checkboxIDOn = $rowDebugID
      . "/td/div/div[contains(\@class,'x-grid3-check-col x-grid3-check-col-on')]";
    $checkboxID1 =
        $rowDebugID
      . "/td/div[contains(\@class,'x-grid3-cell-inner "
      . "x-grid3-col-disk-added')]/div/input[\@type='checkbox']";
    $checkboxID2 = $rowDebugID . "/td/input";
    $checkboxID3 = $rowDebugID . "/div/input";
    $checkboxID4 = $rowDebugID . "/td/div/input";

    if ( $value eq "enable" && $self->is_element_present($checkboxID) ) {
        $self->mouse_down($checkboxID);
    }
    elsif ( $value eq "disable" && $self->is_element_present($checkboxIDOn) ) {
        $self->mouse_down($checkboxIDOn);
    }
    elsif ( $value eq "enable" && $self->is_element_present($checkboxID1) ) {
        $self->mouse_down($checkboxID1);
    }
    elsif ( $value eq "enable" && $self->is_element_present($checkboxID2) ) {
        $self->mouse_down($checkboxID2);
    }
    elsif ( $value eq "enable" && $self->is_element_present($checkboxID3) ) {
        $Log->comment(" Printing the ID for temp $checkboxID3");
        $self->mouse_down($checkboxID3);
    }
    elsif ( $value eq "enable" && $self->is_element_present($checkboxID4) ) {
        $self->mouse_down($checkboxID4);
    }
    elsif ( $value ne "enable" && $value ne "disable" ) {
        NATE::BaseException->throw(
            "Invalid option ($value). Put enable/disable");
    }
    elsif ($self->is_element_present($checkboxID)
        || $self->is_element_present($checkboxIDOn) )
    {
        $Log->comment( "Already $value" . "d" );
    }
    $Log->exit() if $may_exit;
    return 1;
} ## end sub set_checkbox_in_row_with_text

###############################################################################
## @name ::type_in_row_with_text
##
## @summary
##    Enter input value in the textbox in a row in a grid.
## @description
##     This method is used to enter the input value in the textbox,
##     which is present in the row of a grid which contains the
##     input text.
## @synopsis
##     type_in_row_with_text(%args);
## INPUTS TO METHOD CALL 'type_in_row_with_text' ON APISET OBJECT
##
sub type_in_row_with_text {
    my ( $self, $debugID, $typeText, @texts ) = @_;
    $Log->enter() if $may_enter;
    my $baseDebugID =
      "//div[\@id='$debugID']/div/div/div/div/div/table/tbody/tr";
    my $rowDebugID = $baseDebugID;
    my $checkboxDebugID;

    my ( $text,    $checkboxID, $checkboxIDOn, $checkboxID1 );
    my ( $tempID1, $tempID2,    $tempID3,      $tempID4 );
    foreach $text (@texts) {
        $tempID1 = $baseDebugID . "[td/div[text()='$text']]";
        $tempID2 = $baseDebugID . "[td/div/span[text()=' $text']]";
        $tempID3 = $baseDebugID . "[td/div/div/div/span[text()='$text']]";
        $tempID4 = $baseDebugID . "[td/div/div[contains(text(),'$text')]]";

        if ( $self->is_element_present($tempID1) ) {
            $rowDebugID = $tempID1;
        }
        elsif ( $self->is_element_present($tempID2) ) {
            $rowDebugID = $tempID2;
        }
        elsif ( $self->is_element_present($tempID3) ) {
            $rowDebugID = $tempID3;
        }
        elsif ( $self->is_element_present($tempID4) ) {
            $rowDebugID = $tempID4;
        }
        else {
            NATE::BaseException->throw(
                "Row with \"$text\" not present in grid \"$debugID\" ");
        }
    } ## end foreach $text (@texts)

    my $textboxID = $rowDebugID . "/td/div/div/input[\@type='text']";
    if ($textboxID) {
        $self->type( $textboxID, $typeText );
    }
    else {
        $Log->nlog("No such text box found in row with $text");
    }
    $Log->enter() if $may_enter;
    return 1;
} ## end sub type_in_row_with_text

###############################################################################
## @name ::set_option
##
## @summary
##     Set an option button.
## @description
##     This method is used to set the option.
## @synopsis
##     set_option(%args);
## INPUTS TO METHOD CALL 'set_option' ON APISET OBJECT
##
sub set_option {
    my ( $self, $debugID ) = @_;
    $Log->enter() if $may_enter;
    my $toogleButtonID = "//div[\@id='" . $debugID . "']/input";

    if (
        $self->is_element_present(
            "//div[\@id='$debugID' and contains(\@class,'x-item-disabled')]")
      )
    {
        NATE::BaseException->throw("Option $debugID is disabled");
    }

    $Log->comment("Enabling $toogleButtonID");
    $self->click($toogleButtonID);
    $Log->exit() if $may_exit;
    return 1;
} ## end sub set_option

###############################################################################
## @name ::click_row_with_texts
##
## @summary
##     Click a row with matching texts in columns.
## @description
##     This method is used to click on the row of a grid which contains the input text.
## @synopsis
##     click_row_with_texts(%args);
## INPUTS TO METHOD CALL 'click_row_with_texts' ON APISET OBJECT
##
sub click_row_with_texts {
    my ( $self, $debugID, $partial, $doubleclick, @texts );
    $Log->enter() if $may_enter;
    my $rowDebugID;
    $self    = shift;
    $debugID = shift;

    if (   ( $_[0] eq "1" || $_[0] eq "0" || $_[0] eq "2" )
        && ( $_[1] eq "1" || $_[1] eq "0" ) )
    {
        $partial     = shift;
        $doubleclick = shift;
    }
    else {
        $partial     = 0;
        $doubleclick = 0;
    }

    @texts = @_;

    if ( $partial eq 2 ) {
        my $baseDebugID = "//div[\@id='$debugID']//tbody/tr";
        foreach my $text (@texts) {
            my $tempID1 = $baseDebugID . "[td//div[text()='$text']]";
            my $tempID2 = $baseDebugID . "[td//div/span[text()=' $text']]";

            if ( $self->is_element_present($tempID1) ) {
                $rowDebugID = $tempID1;
            }
            elsif ( $self->is_element_present($tempID2) ) {
                $rowDebugID = $tempID2;
            }
            else {
                $Log->comment(
                    "Row with \"$text\" not present in grid \"$debugID\" ");
                NATE::BaseException->throw(
                    "Row with \"$text\" not present in grid \"$debugID\" ");
            }
        } ## end foreach my $text (@texts)
        $Log->comment("Row with \"@texts\" is present in grid \"$debugID\" ");
    }
    else {
        my $baseDebugID = "//div[\@id='$debugID']//tbody/tr";
        my $notfound    = 0;
        $Log->comment("Clicking row with text: @texts from the grid");
        my $baseSortID = "//div[\@id='$debugID']//tbody/tr/td[1]/div";

        # looking for string with Default Order
        $Log->comment(
            "Looking for text: @texts with Default order from the grid");

        foreach my $text (@texts) {
            my @tempIDS;
            push( @tempIDS,
                $baseDebugID . "[td//div[contains(text(),'$text')]]" );
            push( @tempIDS,
                $baseDebugID . "[td//div/span[contains(text(),'$text')]]" );
            push( @tempIDS,
                "//div[\@id='$debugID']/div[contains(text(),'$text')]" );
            push( @tempIDS,
"//div[\@id='$debugID']//tbody/tr/td/div[contains(text(),'$text')]"
            );

            foreach my $tempid (@tempIDS) {
                if ( $self->is_element_present($tempid) ) {
                    $rowDebugID = $tempid;
                    $Log->comment("Row with \"$text\" found");
                    last;
                }
            }
            unless ($rowDebugID) {
                $notfound = 1;
            }
        }

        if ( $notfound eq 1 ) {

            $notfound = 0;

            # looking for string with ascending order
            $Log->comment(
                "Looking for text: @texts with ascending order from the grid");
            $self->check_ascending($baseSortID);
            sleep(5);

            foreach my $text (@texts) {
                my @tempIDS;
                push( @tempIDS,
                    $baseDebugID . "[td/div[contains(text(),'$text')]]" );
                push( @tempIDS,
                    $baseDebugID . "[td/div/span[contains(text(),'$text')]]" );
                push( @tempIDS,
                    $baseDebugID
                      . "[td/div/div/div/span[contains(text(),'$text')]]" );
                push( @tempIDS,
                    $baseDebugID . "[td/div/div[contains(text(),'$text')]]" );
                push( @tempIDS,
                    "//div[\@id='$debugID']/div[contains(text(),'$text')]" );
                push( @tempIDS,
                    "//div[\@id='$debugID']/div/div/div/div/div/div/div/div/"
                      . "table/tbody/tr/td/div[contains(text(),'$text')]" );
                push( @tempIDS,
                    $baseDebugID
                      . "[td/div/div/span[contains(text(),'$text')]]" );

                foreach my $tempid (@tempIDS) {
                    if ( $self->is_element_present($tempid) ) {
                        $rowDebugID = $tempid;
                        $Log->comment("Row with \"$text\" found");
                        last;
                    }
                }
                unless ($rowDebugID) {
                    $notfound = 1;
                }
            }
        }

        # Try one more to look for string with desecending order
        if ( $notfound eq 1 ) {

            # looking for string with descending order
            $self->check_descending($baseSortID);
            sleep(5);
            $Log->comment(
                "Looking for text: @texts with descending order from the grid");

            foreach my $text (@texts) {
                my @tempIDS;
                push( @tempIDS,
                    $baseDebugID . "[td/div[contains(text(),'$text')]]" );
                push( @tempIDS,
                    $baseDebugID . "[td/div/span[contains(text(),'$text')]]" );
                push( @tempIDS,
                    $baseDebugID
                      . "[td/div/div/div/span[contains(text(),'$text')]]" );
                push( @tempIDS,
                    $baseDebugID . "[td/div/div[contains(text(),'$text')]]" );
                push( @tempIDS,
                    "//div[\@id='$debugID']/div[contains(text(),'$text')]" );
                push( @tempIDS,
                    "//div[\@id='$debugID']/div/div/div/div/div/div/div/div/"
                      . "table/tbody/tr/td/div[contains(text(),'$text')]" );
                push( @tempIDS,
                    $baseDebugID
                      . "[td/div/div/span[contains(text(),'$text')]]" );

                foreach my $tempid (@tempIDS) {
                    if ( $self->is_element_present($tempid) ) {
                        $rowDebugID = $tempid;
                        $Log->comment("Row with \"$text\" found");
                        last;
                    }
                }
                unless ($rowDebugID) {
                    $Log->comment(
                        "Row with \"$text\" not present in grid \"$debugID\" ");
                    NATE::BaseException->throw(
                        "Row with \"$text\" not present in grid \"$debugID\" ");
                }
            }
        }

        # Found element in the grid
        $Log->comment("Row with \"@texts\" is present in grid \"$debugID\" ");
    }

    if ( $doubleclick eq "1" ) {
        $Log->comment("Double click row $rowDebugID");
        $self->_double_click($rowDebugID);
    }
    else {
        $Log->comment("Selecting row $rowDebugID");
        $self->mouse_down($rowDebugID);
    }
    $Log->exit() if $may_exit;
    return 1;
} ## end sub click_row_with_texts

###############################################################################
# Method:    does_row_contain_texts
# Objective: matched texts with row cells and return true/false.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub does_row_contain_texts {
    my ( $self, $debugID, $partial, @texts );
    $Log->enter() if $may_enter;
    $self    = shift;
    $debugID = shift;

    if ( ( $_[0] eq "1" || $_[0] eq "0" ) ) {
        $partial = shift;
    }
    else {
        $partial = 0;
    }

    @texts = @_;

    my $baseDebugID =
      "//div[\@id='$debugID']/div/div/div/div/div/table/tbody/tr";

    my $rowDebugID = $baseDebugID;

    # Clicking the ascending order button
    my $baseSortID =
      "//div[\@id='$debugID']/div/div/div/div/div/div/table/tbody/tr/td[1]/div";

    $self->check_ascending($baseSortID);

    my $text;
    my ( $tempID1, $tempID2, $tempID3, $tempID4 );

    foreach $text (@texts) {
        $tempID1 = $baseDebugID . "[td/div[contains(text(),'$text')]]";
        $tempID2 = $baseDebugID . "[td/div/span[contains(text(),'$text')]]";
        $tempID3 =
          $baseDebugID . "[td/div/div/div/span[contains(text(),'$text')]]";
        $tempID4 = $baseDebugID . "[td/div/div[contains(text(),'$text')]]";

        if (   $self->is_element_present($tempID1)
            || $self->is_element_present($tempID2)
            || $self->is_element_present($tempID3)
            || $self->is_element_present($tempID4) )
        {

            # Found element and return true
            $Log->comment(
                "Row with \"@texts\" is present in grid \"$debugID\" ");
            return 1;
        }
        else {
            $Log->comment(
                "Row with \"$text\" not present in grid \"$debugID\" ");
            return 0;
        } ## end else [ if ( $self->is_element_present...)]
    } ## end foreach $text (@texts)
    $Log->exit() if $may_exit;
    return 0;
} ## end sub does_row_contain_texts

###############################################################################
## @name ::click_child_row_with_texts
##
## @summary
##     Click a row with matching texts in columns.
## @description
##     This method will select the child row under the parent row which contains the input text.
## @synopsis
##     click_child_row_with_texts(%args);
## INPUTS TO METHOD CALL 'click_child_row_with_texts' ON APISET OBJECT
##
sub click_child_row_with_texts {
    $Log->enter() if $may_enter;
    my ( $self, $debugID, $parent_values, $text1 ) = @_;
    my ( $parentID, $childID1, $childID2, $baseDebugID, $expand_parent );
    my $rowDebugID;
    if ( $parent_values =~ /(Kerberos|UNIX|Windows)/ ) {
        $expand_parent = 0;
        $baseDebugID =
          "//div[\@id='$debugID']//div[//div[text()='$parent_values']]";
    }
    else {
        $expand_parent = 1;
        $baseDebugID   = "//div[\@id='$debugID']//div/tbody/tr";
    }

    foreach my $text ( @{$text1} ) {

        $parentID = $baseDebugID . "[td//span[text()='$parent_values']]";

        $childID1 =
            $baseDebugID
          . "[td/div/div[contains(\@id,'$parent_values"
          . "')]/div/span[text()='$text']]";

        $childID2 = $baseDebugID . "//div[text()='$text']";

        if ( $expand_parent == 1 ) {
            if ( not $self->is_element_present($childID1) ) {
                $Log->comment(" ******  $childID1 and $parentID ");
                if ( $self->is_element_present($parentID) ) {
                    $Log->comment(
"Expanding parent $parent_values row to display child rows "
                    );
                    $self->_double_click($parentID);
                    sleep(5);
                }
                else {
                    $Log->comment("$childID1 and $parentID ");
                    NATE::BaseException->throw(
                            "Parent \"$parent_values\" or child row"
                          . " \"$text\" not present in grid \"$debugID\" " );
                }
            }
        }

        if ( $self->is_element_present($childID1) ) {
            $rowDebugID = $childID1;
        }
        elsif ( $self->is_element_present($childID2) ) {
            $rowDebugID = $childID2;
        }
        else {
            NATE::BaseException->throw(
                "Row with texts \"@$text1\" not present in grid \"$debugID\" ");
        }

        # Found element and return true
        $Log->comment("Row with \"$text\" is present in grid \"$debugID\" ");

        $Log->comment("Selecting row $rowDebugID");
        $self->mouse_down($rowDebugID);
    }
    $Log->exit() if $may_exit;
    return 1;
} ## end sub click_child_row_with_texts

###############################################################################
## @name ::click_row_with_texts_without_ascending
##
## @summary
##     Click a row with matching texts in columns.
## @description
##     This method is used to select a particular row without ascending the column,
##       and also containing the input text.
## @synopsis
##     click_row_with_texts_without_ascending(%args);
## INPUTS TO METHOD CALL 'click_row_with_texts_without_ascending' ON APISET OBJECT
##
sub click_row_with_texts_without_ascending {
    my ( $self, $debugID, $partial, $doubleclick, @texts );
    $Log->enter() if $may_enter;

    $self    = shift;
    $debugID = shift;

    if (   ( $_[0] eq "1" || $_[0] eq "0" )
        && ( $_[1] eq "1" || $_[1] eq "0" ) )
    {
        $partial     = shift;
        $doubleclick = shift;
    }
    else {
        $partial     = 0;
        $doubleclick = 0;
    }

    @texts = @_;

    my $baseDebugID =
      "//div[\@id='$debugID']/div/div/div/div/div/table/tbody/tr";

    my $rowDebugID;

    $Log->comment(
        "partial is $partial and doubleclick is $doubleclick and text is @texts"
    );

    # Clicking the ascending order button
    my $baseSortID =
      "//div[\@id='$debugID']/div/div/div/div/div/div/table/tbody/tr/td[1]/div";

    foreach my $text (@texts) {
        my @tempIDS;
        push( @tempIDS, $baseDebugID . "[td/div[contains(text(),'$text')]]" );
        push( @tempIDS,
            $baseDebugID . "[td/div/span[contains(text(),'$text')]]" );
        push( @tempIDS,
            $baseDebugID . "[td/div/div/div/span[contains(text(),'$text')]]" );
        push( @tempIDS,
            $baseDebugID . "[td/div/div[contains(text(),'$text')]]" );
        push( @tempIDS,
            "//div[\@id='$debugID']/div[contains(text(),'$text')]" );
        push( @tempIDS,
                "//div[\@id='$debugID']/div/div/div/div/div/div/div/div/"
              . "table/tbody/tr/td/div[contains(text(),'$text')]" );
        push( @tempIDS,
            $baseDebugID . "[td/div/div/span[contains(text(),'$text')]]" );

        foreach my $tempid (@tempIDS) {
            if ( $self->is_element_present($tempid) ) {
                $rowDebugID = $tempid;
                $Log->comment("Row with \"$text\" found");
                last;
            }
        }
        unless ($rowDebugID) {
            NATE::BaseException->throw(
                "Row with \"$text\" not present in grid \"$debugID\" ");
        }
    } ## end foreach $text (@texts)

    # Found element and return true
    $Log->comment("Row with \"@texts\" is present in grid \"$debugID\" ");

    if ( $doubleclick eq "1" ) {
        $Log->comment("Double click row $rowDebugID");
        $self->_double_click($rowDebugID);
    }
    else {
        $Log->comment("Selecting row $rowDebugID");
        $self->mouse_down($rowDebugID);
    }
    $Log->exit() if $may_exit;
    return 1;
} ## end sub click_row_with_texts_without_ascending

###############################################################################
# Method:    does_row_contain_texts_without_ascending
# Objective: matched texts with row cells and return true/false (without ascending)
# Details:   See POD documentation at the end of this file.
###############################################################################
sub does_row_contain_texts_without_ascending {
    my ( $self, $debugID, $partial, @texts );
    $Log->enter() if $may_enter;

    $self    = shift;
    $debugID = shift;

    if ( ( $_[0] eq "1" || $_[0] eq "0" ) ) {
        $partial = shift;
    }
    else {
        $partial = 0;
    }

    @texts = @_;

    my $baseDebugID =
      "//div[\@id='$debugID']/div/div/div/div/div/table/tbody/tr";

    my $rowDebugID;

    # Clicking the ascending order button
    my $baseSortID =
      "//div[\@id='$debugID']/div/div/div/div/div/div/table/tbody/tr/td[1]/div";

    my $text;
    my ( $tempID1, $tempID2, $tempID3, $tempID4 );

    foreach $text (@texts) {
        $tempID1 = $baseDebugID . "[td/div[contains(text(),'$text')]]";
        $tempID2 = $baseDebugID . "[td/div/span[contains(text(),'$text')]]";
        $tempID3 =
          $baseDebugID . "[td/div/div/div/span[contains(text(),'$text')]]";
        $tempID4 = $baseDebugID . "[td/div/div[contains(text(),'$text')]]";

        if ( $self->is_element_present($tempID1) ) {
            $rowDebugID = $tempID1;
        }
        elsif ( $self->is_element_present($tempID2) ) {
            $rowDebugID = $tempID2;
        }
        elsif ( $self->is_element_present($tempID3) ) {
            $rowDebugID = $tempID3;
        }
        elsif ( $self->is_element_present($tempID4) ) {
            $rowDebugID = $tempID4;
        }
        else {
            $Log->comment(
                "Row with \"$text\" not present in grid \"$debugID\" ");
            return 0;
        } ## end else [ if ( $self->is_element_present...)]
    } ## end foreach $text (@texts)

    # Found element and return true
    $Log->comment("Row with \"@texts\" is present in grid \"$debugID\" ");
    $Log->exit() if $may_exit;
    return $self->is_element_present($rowDebugID);

} ## end sub does_row_contain_texts_without_ascending

###############################################################################
# Method:    does_row_contain_exact_text
# Objective: matched texts with row cells and return true/false.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub does_row_contain_exact_text {
    my ( $self, $debugID, @texts ) = @_;
    $Log->enter() if $may_enter;

    my $baseDebugID =
      "//div[\@id='$debugID']/div/div/div/div/div/table/tbody/tr";
    my $rowDebugID;

    foreach my $text (@texts) {
        my @tempIDS;
        push( @tempIDS, $baseDebugID . "[td/div[text()='$text']]" );
        push( @tempIDS, $baseDebugID . "[td/div/span[text()=' $text']]" );
        push( @tempIDS,
            $baseDebugID . "[td/div/div/div/span[text()='$text']]" );
        push( @tempIDS, $baseDebugID . "[td/div/div[text()='$text']]" );

        foreach my $tempid (@tempIDS) {
            if ( $self->is_element_present($tempid) ) {
                $rowDebugID = $tempid;
                $Log->comment("Row with \"$text\" found");
                last;
            }
        }
        unless ($rowDebugID) {
            $Log->comment(
                "Row with \"$text\" not present in grid \"$debugID\" ");
            return 0;
        }
    } ## end foreach my $text (@texts)

    $Log->comment("Row with \"@texts\" is present in grid \"$debugID\" ");
    $Log->exit() if $may_exit;
    return $self->is_element_present($rowDebugID);
} ## end sub does_row_contain_exact_text

###############################################################################
## @name ::check_ascending
##
## @summary
##   Check if the column is in ascending order.
## @description
##     This method is used to verify if a column is set to ascending order or not.
## @synopsis
##     check_ascending(%args);
## INPUTS TO METHOD CALL 'check_ascending' ON APISET OBJECT
##
sub check_ascending {
    my ( $self, $debugID ) = @_;
    $Log->enter() if $may_enter;
    sleep(1);

    #Check if the element present
    if ( $self->is_element_present($debugID) ) {

        #click on the tab if its not ascending
        my $btn_id = $debugID . "[contains(\@aria-sort,'descending')]";
        if ( not $self->is_element_present($btn_id) ) {
            $Log->comment("Sorting in ascending order");
            $self->click($debugID);
        }
    }
    else {
        $Log->comment("Invalid id $debugID to sort ascending order");
    }
    $Log->exit() if $may_exit;
    return 1;
} ## end sub check_ascending

###############################################################################
## @name ::check_descending
##
## @summary
##   Check if the column is in descending order.
## @description
##     This method is used to verify if a column is set to descending order or not.
## @synopsis
##     check_descending(%args);
## INPUTS TO METHOD CALL 'check_descending' ON APISET OBJECT
##
sub check_descending {
    my ( $self, $debugID ) = @_;
    $Log->enter() if $may_enter;
    sleep("1");

    #Check if the element present
    if ( $self->is_element_present($debugID) ) {

        #click on the tab if its not descending
        my $btn_id = $debugID . "[contains(\@aria-sort,'ascending')]";
        if ( not $self->is_element_present($btn_id) ) {
            $Log->comment("Sorting in descending order");
            $self->click($debugID);
        }
    }
    else {
        $Log->comment("Invalid id $debugID to sort descending order");
    }
    $Log->exit() if $may_exit;
    return 1;
} ## end sub check_descending

###############################################################################
# Method:    click_text_in_list
# Objective: Click a list item.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub click_item_in_list {
    my ( $self, $debugID, $item ) = @_;
    $Log->enter() if $may_enter;
    $self->mouse_down("//div[\@id='$debugID']/div[text()='$item']");
    $Log->exit() if $may_exit;
    return 1;
} ## end sub click_item_in_list

###############################################################################
## @name ::is_button_enabled
##
## @summary
##    Checks if button is enabled/disabled.
## @description
##     This method is used to verify whether the button is enable or not.
## @synopsis
##     is_button_enabled(%args);
## INPUTS TO METHOD CALL 'is_button_enabled' ON APISET OBJECT
##
sub is_button_enabled {
    my ( $self, $debugID ) = @_;
    $Log->enter() if $may_enter;
    my $retStatus = 1;

    if (
        $self->is_element_present(
            "//table[\@id='$debugID' and contains(\@class,'x-item-disabled')]")
      )
    {
        $retStatus = 0;
    }
    elsif (
        $self->is_element_present(
            "//a[\@id='$debugID' and contains(\@class,'x-item-disabled')]")
      )
    {
        $retStatus = 0;
    }

    ( $retStatus == 1 )
      ? $Log->comment("Button: $debugID is Enabled")
      : $Log->comment("Button: $debugID is Disabled");

    $Log->exit() if $may_exit;
    return $retStatus;
} ## end sub is_button_enabled

###############################################################################
## @name ::check_warning
##
## @summary
##   This lists any warning message present in the page.
## @description
##     This method is used to check if any warning is present in a page.
## @synopsis
##     check_warning(%args);
## INPUTS TO METHOD CALL 'check_warning' ON APISET OBJECT
##
sub check_warning {

    my ( $self, $debugID ) = @_;
    my $warning;
    $Log->enter() if $may_enter;

    NATE::BaseException->throw("Invalid ID : $debugID")
      if ( not $self->is_element_present($debugID) );

    if (
        not $self->is_element_present(
            "//div[\@id = '$debugID' and contains(\@class,'x-hide-display')]")
      )
    {
        my $textfield =
            "//div[\@id ='" 
          . $debugID
          . "']/table/tbody/tr/td/div[contains(\@class,'x-form-label')]";
        $warning = $self->get_text($textfield);
        my $imageid =
            "//div[\@id ='" 
          . $debugID
          . "']/table/tbody/tr/td/img[contains(\@src,'icon-error.gif')]";
        if ( $self->is_element_present($imageid) ) {
            NATE::BaseException->throw("/n Error message : $warning /n");
        }
        $Log->comment("******* Waring message found : $warning *****");
    } ## end if (not $self->is_element_present...
    $Log->exit() if $may_exit;

    return $warning;
} ## end sub check_warning

###############################################################################
## @name ::get_grid_data
##
## @summary
##   Fetch the grid data.
## @description
##     This method returns the whole grid values.
## @synopsis
##     get_grid_data(%args);
## INPUTS TO METHOD CALL 'get_grid_data' ON APISET OBJECT
##
sub get_grid_data {
    my ( $self, $debugID ) = @_;
    $Log->enter() if $may_enter;
    my $gridID = "//div[\@id='$debugID']";
    my $gridData;
    my $tablepresent;

    $Log->comment("Tally the table iD's to be read inside the grid");
    my $tableID1 = $gridID . "/div/div/div/div[\@class='x-grid3-body']";
    my @tableIDs = ($tableID1);

    $Log->comment(
        "Read the data in each tallied and existing table & add to gridData");
    foreach my $tableID (@tableIDs) {
        if ( $self->is_element_present("$tableID") ) {
            my $tableData = $self->get_text($tableID);
            $gridData     = $gridData . $tableData;
            $tablepresent = 'true';
        }
    }
    $Log->comment("If no table exists, get the data for the base gridID");
    if ( not $tablepresent eq 'true' ) {
        $gridData = $self->get_text($gridID);
    }
    $Log->exit() if $may_exit;
    return $gridData;
} ## end sub get_grid_data

#-------------------------------------------------------------------------------
# WebDriver Overloaded functions, for Selenium RC backward compatibility support
#-------------------------------------------------------------------------------
sub click {
    my ( $self, $debugID ) = @_;
    $Log->enter() if $may_enter;
    if ( ref($self) =~ /WebElement/ ) {
        $self->click();
        return 1;
    }
    $Log->comment("Click Element : $debugID");
    $self->set_implicit_wait_timeout(30000);
    my $elem = $self->_find_element($debugID);
    $elem->click();
    $Log->comment("Click on Element $debugID successfull");
    $Log->exit() if $may_exit;
    return 1;
}

sub _double_click {
    my ( $self, $ID ) = @_;
    $Log->enter() if $may_enter;
    unless ($ID) {
        $self->double_click();
        return 1;
    }
    $self->set_implicit_wait_timeout(30000);
    my $elem = $self->_find_element("$ID");
    $self->mouse_move_to_location( 'element' => $elem );
    $self->double_click();
    $Log->exit() if $may_exit;
    return 1;
}

sub get_text {
    my ( $self, $debugID ) = @_;
    $Log->enter() if $may_enter;

    my $output_text;
    if ( ref($self) =~ /WebElement/ ) {
        $output_text = $self->get_text();
        return $output_text;
    }
    $self->set_implicit_wait_timeout(30000);
    my $elem = $self->_find_element($debugID);
    $output_text = $elem->get_text();
    $Log->exit() if $may_exit;
    return $output_text;
}

sub get_value {
    my ( $self, $debugID ) = @_;
    $Log->enter() if $may_enter;

    my $output_value;
    if ( ref($self) =~ /WebElement/ ) {
        $output_value = $self->get_value();
        return $output_value;
    }

    $self->set_implicit_wait_timeout(30000);
    my $elem = $self->_find_element($debugID);
    $output_value = $elem->get_value();

    $Log->exit() if $may_exit;
    return $output_value;
}

sub is_checked {
    my ( $self, $debugID ) = @_;
    $Log->enter() if $may_enter;
    my $value;

    if ( ref($self) =~ /WebElement/ ) {
        $value = $self->is_selected();
        return $value;
    }
    $self->set_implicit_wait_timeout(30000);
    $value = $self->_find_element($debugID)->is_selected();
    $Log->exit() if $may_exit;
    return $value;
}

sub key_down {
    my ( $self, $debugID, $kbKey ) = @_;
    $self->set_implicit_wait_timeout(30000);
    $self->_find_element($debugID)->send_keys( KEYS->{$kbKey} );
    return 1;
}

sub key_up {
    my ( $self, $debugID, $kbKey ) = @_;
    $self->set_implicit_wait_timeout(30000);
    $self->_find_element($debugID)->send_keys( KEYS->{$kbKey} );
    return 1;
}

sub mouse_down {
    my ( $self, $debugID ) = @_;
    $self->set_implicit_wait_timeout(30000);
    my $elem = $self->_find_element($debugID);
    $self->mouse_move_to_location( 'element' => $elem );
    $self->button_down;
    sleep 1;
    $self->button_up;
    return 1;
}

sub mouse_up {
    my ( $self, $debugID ) = @_;
    $self->set_implicit_wait_timeout(30000);
    my $elem = $self->_find_element($debugID);
    $self->mouse_move_to_location( 'element' => $elem );
    $self->button_up;
    return 1;
}

sub select_window {
    my ( $self, $selWin ) = @_;
    $self->switch_to_window($selWin);

    return 1;
}

sub set_timeout {
    my ( $self, $timeoutVal ) = @_;
    my $res = { 'command' => 'setTimeout' };
    foreach my $type ( 'script', 'page load' ) {
        my $params = { 'type' => $type, 'ms' => $timeoutVal };
        $self->_execute_command( $res, $params );
    }
    return 1;
}

sub type {
    my ( $self, $debugID, $typeText ) = @_;
    $self->set_implicit_wait_timeout(30000);
    my $elem = $self->_find_element($debugID);
    $elem->clear();
    $elem->send_keys($typeText);
    return 1;
}

sub wait_for_element_not_present {
    my ( $self, $debugID, $timeout ) = @_;
    my $retVal = 1;
    $Log->enter() if $may_enter;

    for ( my $i = 0 ; $i < 10 ; $i++ ) {
        $self->set_implicit_wait_timeout(1000);
        eval {
            $self->_find_element($debugID);
            $Log->comment("Element $debugID still exists");
            sleep 1;
            $retVal = 0;
        };
        if ($@) {
            $retVal = 1;
            last;
        }
    }
    unless ($retVal) {
        $Log->comment( "Sub : wait_for_element_not_present()"
              . " , on element '$debugID' Failed" );
    }
    $Log->exit() if $may_exit;
    return $retVal;
}

sub wait_for_element_present {
    my ( $self, $debugID ) = @_;
    my $retVal = 0;
    $Log->enter() if $may_enter;

    sleep 1;
    for ( my $i = 0 ; $i < 10 ; $i++ ) {
        $self->set_implicit_wait_timeout(10000);
        eval {
            $self->_find_element($debugID);
            $retVal = 1;
        };
        if ($@) {
            $Log->comment( "Waiting for $debugID : " . $@ );
            sleep 5;
        }
        else {
            last;
        }
    }
    unless ($retVal) {
        $Log->comment( "Sub : wait_for_element_present()"
              . " , on element '$debugID' Failed" );
    }
    $Log->exit() if $may_exit;
    return $retVal;
}

sub wait_for_page_to_load {
    $Log->enter() if $may_enter;
    my ( $self, $timeoutVal ) = @_;
    $self->wait_for_loading($timeoutVal);
    $Log->exit() if $may_exit;
    return 1;
}

sub get_all_window_titles {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    $self->set_implicit_wait_timeout(30000);
    my $handles = $self->get_window_handles;
    $Log->exit() if $may_exit;
    return @$handles;
}

sub is_element_present {
    $Log->enter() if $may_enter;
    my ( $self, $debugID ) = @_;
    $self->set_implicit_wait_timeout(1000);
    eval { $self->_find_element($debugID); };
    if ($@) {
        return 0;
    }
    $Log->comment(
        "Sub : is_element_present()," . " on element '$debugID' Passed" );
    $Log->exit() if $may_exit;
    return 1;
}

sub get_all_window_ids {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    $self->set_implicit_wait_timeout(30000);
    my $handles = $self->get_window_handles;
    $Log->exit() if $may_exit;
    return @$handles;
}

sub get_all_window_names {
    my ($self) = @_;
    $self->set_implicit_wait_timeout(30000);
    my $handles = $self->get_window_handles;
    return @$handles;
}

sub stop {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    $self->quit();
    $Log->exit() if $may_exit;
    return 1;
}

sub fire_event {
    return 1;
}

sub type_keys {
    my ( $self, $debugID, $typeText ) = @_;
    return 1 if ( $typeText =~ /^\s*$/ );
    $self->set_implicit_wait_timeout(30000);
    my $elem = $self->_find_element($debugID);
    $elem->clear();
    $elem->send_keys($typeText);
    return 1;
}

sub _find_element {
    my ( $self, $str ) = @_;
    my $elem_obj;
    if ( $str =~ /^\/\// ) {
        $elem_obj = $self->find_element($str);
    }
    else {
        $elem_obj = $self->find_element( $str, 'id' );
    }
    return $elem_obj;
}

sub mouse_over {
    my ( $self, $ID ) = @_;
    $self->set_implicit_wait_timeout(30000);
    my $elem = $self->_find_element($ID);
    $self->mouse_move_to_location( 'element' => $elem );
    return 1;
}

sub get_attribute {
    my ( $self, $id, $attr_name ) = @_;
    my $val;
    if ( ref($self) =~ /WebElement/ ) {
        $val = $self->get_attribute($id);
    }
    else {
        $self->set_implicit_wait_timeout(30000);
        my $elem_obj;
        $elem_obj = $self->_find_element($id);
        $val      = $elem_obj->get_attribute($attr_name);
    }
    $Log->comment("Get Attrinute on $id successfull : $val");
    return $val;
}

sub control_key_down {
    my $self = shift;
    $Log->comment("Pressing control key down");
    $self->send_keys_to_active_element( KEYS->{control} );
    return 1;
}

sub control_key_up {
    my $self = shift;
    $Log->comment("Pressing control key up");
    $self->send_keys_to_active_element( KEYS->{control} );
    return 1;
}

sub mouse_down_only {
    my ( $self, $debugID ) = @_;
    $self->set_implicit_wait_timeout(30000);
    my $elem = $self->_find_element($debugID);
    $self->mouse_move_to_location( 'element' => $elem );
    $self->button_down;
    return 1;
}

1;
