# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/VMAPI.pm $
#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Initialize VIM APISet Session object creation and call VIM object method on it
## @author bkprasha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Host::VMAPI;


# NATE imports
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

# Module import - APISets
use base qw(NACL::APISet::Host);
use NACL::APISet::Session::VMAPI::ESX;
use NACL::APISet::Response::VMAPI::ESX;
use NACL::APISet::Exceptions::ResponseException qw(:try);

our $AUTOLOAD;
#####################################################################################
# Method Name: AUTOLOAD
# Objective:  Catch all commands not specifically implemented and call the
#             method on vim object which is the value of $self-{session}-{connection}.
#             This method also constructs and returns a response object
#             
######################################################################################

sub AUTOLOAD {  
    $Log->enter() if $may_enter;
    
    my ($self, %args) = @_;

    my $method = $AUTOLOAD;

    $method =~ s/.*://;    # strip fully-qualified portion
    if ($method eq "DESTROY") {
        $Log->exit() if $may_exit;
        return;
    }
    
    my $time_start = time();
    my $time_end;
    my $execution_time;
    my ( $esx_output, $result_obj);
    try {
        $esx_output = $self->{session}->{connection}->$method(%args);
    } otherwise {
        $time_end = time();
        $execution_time = $time_end - $time_start;
        $result_obj = NACL::APISet::Response::VMAPI::ESX->new(
            output          => $@,
            method_name     => $method,
            
        );
        
        NACL::APISet::Exceptions::ResponseException->throw(
            text                => "Error: $@",
            response_object     => $result_obj,
            execution_time      => $execution_time,
        );
    };
    
    $time_end = time();
    $execution_time = $time_end - $time_start;
    
    $result_obj = NACL::APISet::Response::VMAPI::ESX->new(
        parsed_output   => $esx_output,
        output          => $esx_output,
        method_name     => $method,
        execution_time  => $execution_time,
    );
    
    $Log->exit() if $may_exit;
    return $result_obj;
}

sub _init_Session {
    $Log->enter() if $may_enter;

    my ($self, %args) = @_;
    my $interface = $self->{interface};
    my $session_type = $self->{set};

    my $session_pkg = "NACL::APISet::Session::" . $interface . "::" . $session_type;
    $self->{session} = $session_pkg->new(%args, hostobj => $self->{hostobj});

    $Log->exit();   
}

=head1 NAME

NACL::APISet::Host::VMAPI - Initialize VIM APISet Session object creation and call
VIM object method on it

=head1 SYNOPSIS

 use NACL::APISet::Host::VMAPI;

=head1 DESCRIPTION

A C<NACL::APISet::Host::VMAPI> is used to construct the VIM APISet session object. 
This is subclassed as an additional Host interface

=head1 EXCEPTIONS
NACL::APISet::Exceptions::ResponseException

The error output from the VIM object method call is passed on through
NACL::APISet::Exceptions::ResponseException

=head1 Methods

=head2 AUTOLOAD

Catch all commands not specifically implemented and call the
method on vim object which is the value of $self-{session}-{connection}, 
where self is the APISet object. This method also constructs and returns 
a response object which includes the output from the VIM object method 

=over

=item Synopsis

This method is automatically called when there is no definition for the method 
called on APISet object 


=item Arguments

The arguments passed to the method unknown to APISet is got automatically to 
this method

=item Returns

Returns a NACL::APISet::Response::VMAPI::ESX object

=item output

Output returned by the VIM object method. 

=item execution_time

The amount of time taken by the method call to return

=item method_name

The name of the method called by the user

=back

=head2 _init_API

Completes the APISet VIM object creation by creating a {session}->{connection},
which contains a VIM object that is connected to the ESX server specified by the
user during APISet object creation

=over

=item Synopsis

This is called from _init_API function in NACL::APISet::Host::VMAPI::ESX 
during APISet object creation

=back

# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/VMAPI.pm $

=cut

1;
