# Copyright (c) 2012-* NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary Initialize Xen APISet object construction
## @author rawat@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here
package NACL::APISet::Host::XAPI::Xen;

# APISet imports
use base qw(NACL::APISet::Host::XAPI);
use NACL::APISet::Session::XAPI::Xen;

# Nate imports
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use Class::MethodMaker [
    scalar => [
        {   -default_ctor => sub {
                    return $_[0]->{session}->{rpc_clientobj};
                }
        },
        'rpc_clientobj'
    ],
    scalar => [
        {   -default_ctor => sub {
                    return $_[0]->{session}->{xen_host};
                }
        },
        'xen_host'
    ],
    scalar => [
        {   -default_ctor => sub {
                    return $_[0]->{session}->{xen_session};
                }
        },
        'xen_session'
    ],
];

sub new {
    $Log->enter();

    my ($class, %args) = @_;
    my $self = {};

    bless $self, $class;

	$Log->exit();

    return $self;
}

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Host::XAPI::Xen - Initialize Xen APISet object construction

=head1 SYNOPSIS

 use NACL::APISet::Host::XAPI::Xen;

=head1 DESCRIPTION

A C<NACL::APISet::Host::XAPI::Xen> is used to initialize Xen object construction.
This is further subclassed to XAPI to call the methods as specified by the user
on the Xen pointed to by the constructed Xen object.

=head1 EXCEPTIONS

None

=head1 Constructors

=over

=back

=head2 new

Returns a reference to a newly created C<NACL::APISet::Host::XAPI::Xen>.

=over

=item Synopsis

$api_esx_pkg = NACL::APISet::Host::XAPI::Xen->new( %args );

=item Arguments

=back

=cut

1;
