# $Id: //depot/prod/test/main/lib/NACL/APISet/Host/ZAPI/Windows.pm#1 $

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary API Set for Windows ZAPI commands
## @author yatin@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Host::ZAPI::Windows;

#
# Compiler Directives
#
use strict;
use warnings;

use base qw(NACL::APISet::Host::ZAPI);

#
# Module imports - APISet
#
use NACL::APISet::Session::ZAPI::Windows;

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);

sub new
{
    $Log->enter() if $may_enter;

    my ($class, %args) = @_;
    my $self = {};
    $self = NACL::APISet::validate_params
      ($class,
       params => \%args,
       spec   =>
             {
              version   => { type => SCALAR, optional => 1, default => 99.99   },
             }
      );
    bless $self, $class;
    $Log->exit() if $may_exit;
    return $self;
}

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the Windows API Set
# Details:   Initializes the API Set.  This method drives the workflow needed
#            to make the API Set ready for work.  This is accomplished by
#            calling the parents' init_API, assigning a default CDEF path
##############################################################################
sub _init_API
{
    $Log->enter() if $may_enter;

    my ($self, %args) = @_;

    $self->{cdef_path} = NATE::Inc::find_file($self->{cdef_path});
	$args{set} = $self->{actual_set};
    $self->SUPER::_init_API(%args);

    $Log->exit() if $may_exit;
}

=head1 NAME

NACL::APISet::Host::ZAPI::Windows - API Set for Windows ZAPI commands

=head1 SYNOPSIS

use NACL::APISet::Host::ZAPI::Windows;

=head1 DESCRIPTION

A C<NACL::APISet::Host::ZAPI::Windows> is used to connect to and send ZAPI commands
to the Windows Host.

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Host::ZAPI::Windows>.

=over

=item Synopsis

 $api_set_obj = NACL::APISet::Host::ZAPI::Windows->new( %args );

=item Arguments

=over

=item %args

(Optional)
Parameters passed to parent classes.  See NACL::APISet and
NACL::APISet::Host::ZAPI for details.

=back

=item Returns

A reference to a new Windows API Set (NACL::APISet::Host::ZAPI::Windows)

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

1;
