#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/arp.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse 7Mode arp command output
##
## @description
##    This is a library to parse the raw output of 7Mode arp command.
##
## @synopsis
##
##    arp(%args);
##
## @author skotagir@netapp.com, dl-nacl-dev@netapp.com
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::arp;
use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

## @name ::arp
##
## @summary Parses the raw output from the "arp" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from arp with any of its options as described in the
##    corresponding cdef file 'arp.cdefs'
## @synopsis
##      arp(%args);
## INPUTS TO METHOD CALL 'arp' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'temp' </td> <td> 'temp' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'ether_addr' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'display-ipaddress' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'delete-entry' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'hostname' </td> <td> '0' </td> </tr>
## <tr> <td> 'pub' </td> <td> 'pub' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'create-entry' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'display-arp-entries' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub arp {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub arp

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error, $error_message ) = undef;
    if ( $raw_output
        =~ /.*no entry|cannot locate.*|Failed.*|invalid Ethernet address.*/ )
    {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /([Uu]sage:|arp:).*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

1;

