#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse backup command output
##
## @description
##    This is a library to parse the raw output of backup command.
##
## @synopsis
##    use backup;
##
##    backup(%args);
##
## @author anupamap@netapp.com
##
## @status public
#

package NACL::APISet::Node::CLI::7Mode::default::backup;
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;
use strict;
use warnings;

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my $error_message;
    my $error = undef;

    if ( $raw_output =~ /[Uu]sage/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /Specific backup does not exist./ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

} ## end sub _check_error

## @name ::backup_status
##
## @summary Parses the raw output from the "backup status" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from backup_status with any of its options as described in the 
##    corresponding cdef file 'backup.cdefs'
## @synopsis
##      backup_status(%args);
## INPUTS TO METHOD CALL 'backup_status' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'id' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; backup status; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;No backup activity.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub backup_status {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;
    if ($error_level) {
        $error = _check_error(
            raw_output      => $output,
            response_object => $resp,
        );
    }
    return [];
}

sub backup_terminate {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;
    if ($error_level) {
        $error = _check_error(
            raw_output      => $output,
            response_object => $resp,
        );
    }
    return [];
}
1;
