#
# Copyright (c) 2015 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse 7Mode boootfs command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode boootfs commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "bootfs" will be boootfs stats error()
##
## @synopsis
##     use NACL::APISet::Node::CLI::7Mode::default::bootfs;
## @author saparna(Aparna.S2@netapp.com),dl-nacl-dev@netapp.com
##
## @status publboootfs
#
package NACL::APISet::Node::CLI::7Mode::default::bootfs;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

## @name ::bootfs
##
## @summary Parses the raw output from the "bootfs" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from bootfs with any of its options as described in the 
##    corresponding cdef file 'bootfs.cdefs'
## @synopsis
##      bootfs(%args);
## INPUTS TO METHOD CALL 'bootfs' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'rename' </td> <td> 'rename' </td> <td> '1' </td> </tr> 
## <tr> <td> 'copy' </td> <td> 'copy' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'dfile' </td> <td> '0' </td> </tr> 
## <tr> <td> 'move' </td> <td> 'move' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'sfile' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.

sub bootfs {
    my %args    = @_;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @resultv;

    if ( $command =~ /copy|move|rename/ ) {
        my @val =split(/\s*->\s*/, $output);
        $resultv[0]->{source_file} = $val[0];
        $val[1] =~ s/^\s+|\s+$//g;
        $resultv[0]->{destination_file} = $val[1];
    }
    elsif ($command =~ /dir/ ) {
            $resultv[0]->{output} = $output;
    }
    elsif ($command =~ /type/ ) {
        if ($output =~ '.'){
            $resultv[0]->{output} =  "Type is directory";}
	else {
	    $resultv[0]->{output} = "Directory path does not exists";}
    }
    return \@resultv;
}


sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $result_arrayp = $args{response_object};
    my $error_level   = $result_arrayp->get_execution_status();
    my ( $error, $error_message );

    if ( $raw_output =~ /mv:|Usage: /i ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(
            text            => $error_message,
            response_object => $result_arrayp
        );

    }

}

1;
