#
#Copyright (c) 2013 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse ct_phy_read_t3 command output
##
## @description
##    This is a library to parse the raw output of ct_phy_read_t3 command.
##
## @synopsis
##    use ct_phy_read_t3;
##
##    ct_phy_read_t3(%args);
##
## @author sheethal@netapp.com
##
## @status public
#

package NACL::APISet::Node::CLI::7Mode::default::ct_phy_read_t3;
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;
use strict;
use warnings;

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my $error_message;
    my $error = undef;
    if ($raw_output =~ /<cmd>.*/) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

} ## end sub _check_error

## @name ::ct_phy_read_t3
##
## @summary Parses the raw output from the "ct_phy_read_t3" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ct_phy_read_t3 with any of its options as described in the
##    corresponding cdef file 'ct_phy_read_t3.cdefs'
## @synopsis
##      ct_phy_read_t3(%args);
## INPUTS TO METHOD CALL 'ct_phy_read_t3' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'value' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'address' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; ct_phy_read_t3 0 0 3d7f3; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;(18)rc=0 val=0x20
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            '18_rc' => '0',
##&nbsp;            'val' => '0x20'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ct_phy_read_t3 {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;
    if ($error_level) {
        $error = _check_error(
            raw_output      => $output,
            response_object => $resp,
        );
    }
    my (@result_array);
    my @values;
    my @values1;
    my @values2;
    my $i = 0;
    my @lines = get_lines(raw_output => $output, response_object => $resp);
    foreach my $line (@lines) {
        $line =~ s/\(//;
        $line =~ s/\)/_/;
        @values  = split(/\s+/, $line);
        @values1 = split(/=/,   $values[0]);
        @values2 = split(/=/,   $values[1]);
        $result_array[$i]->{to_key_name(item => $values1[0])} = $values1[1];
        $result_array[$i]->{to_key_name(item => $values2[0])} = $values2[1];
        $i++;
    }

    return \@result_array;
} ## end sub ct_phy_read_t3
1;

