#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse ct_reset_t3 command output
##
## @description
##    This is a library to parse the raw output of ct_reset_t3 command.
##
## @synopsis
##    use ct_reset_t3;
##
##    ct_reset_t3(%args);
##
## @author anupamap@netapp.com
##
## @status public
#

package NACL::APISet::Node::CLI::7Mode::default::ct_reset_t3;
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

use strict;
use warnings;

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my $error_message;
    my $error = undef;

    if ( $raw_output =~ /ct_reset_t3 not found.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } if ( $raw_output =~ /Adapter\s+\S+.not\s+found/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } if ( $raw_output =~ /<cmd>.*/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

} ## end sub _check_error


## @name ::ct_reset_t3
##
## @summary Parses the raw output from the "ct_reset_t3" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ct_reset_t3 with any of its options as described in the
##    corresponding cdef file 'ct_reset_t3.cdefs'
## @synopsis
##      ct_reset_t3(%args);
## INPUTS TO METHOD CALL 'ct_reset_t3' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'adapter-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; ct_reset_t3 e6; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub ct_reset_t3 {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;
    if ($error_level) {
        $error = _check_error(
            raw_output      => $output,
            response_object => $resp,
        );
    }
    return [];
}
1;
