#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/dbg.pm $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode dbg commands output
##
## @description
##    This is a library to parse the raw output of 7Mode dbg commands.
##
## @synopsis
##    use dbg;
##
##    dbg_level(%args);
##    dbg_to(%args);
##
## @author tanil@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::dbg;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $raw_output =~ /dbg:\s+(unknown\s+module.*)/ ) {
        $$error_message = $1;
        $error          = "InvalidParamValue";
    } elsif($raw_output =~ /[Uu]sage:/) {
        $error          = "InvalidParam";
        $$error_message = $raw_output;
    }

    return $error;
} ## end sub _check_error

## @name ::dbg_level
##
## @summary Parses the raw output from the "dbg level" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from dbg_level with any of its options as described in the
##    corresponding cdef file 'dbg.cdefs'
## @synopsis
##      dbg_level(%args);
## INPUTS TO METHOD CALL 'dbg_level' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'module-levels' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = dbg level
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'mc' => '0',
##&nbsp;            'smb' => '0',
##&nbsp;            'hamsg' => '0',
##&nbsp;            'vdisk' => '0',
##&nbsp;            'adminapi' => '0',
##&nbsp;            'ems_flush' => '0',
##&nbsp;            'gridiron' => '0',
##&nbsp;            'rr' => '0',
##&nbsp;            'fio' => '0',
##&nbsp;            'hat' => '0',
##&nbsp;            'fat' => '0',
##&nbsp;            'tcm' => '0',
##&nbsp;            'diskio' => '0',
##&nbsp;            'tn' => '0',
##&nbsp;            'trunc' => '0',
##&nbsp;            'nvsave' => '0',
##&nbsp;            'pipeline' => '0',
##&nbsp;            'upgrade_64bit' => '0',
##&nbsp;            'rra' => '0',
##&nbsp;            'incpr' => '0',
##&nbsp;            'sinai' => '0',
##&nbsp;            'vi_if' => '0',
##&nbsp;            'sem' => '0',
##&nbsp;            'nfsv4' => '0',
##&nbsp;            'tavornv5' => '0',
##&nbsp;            'nsm' => '0',
##&nbsp;            'raid2obj' => '0',
##&nbsp;            'acp' => '0',
##&nbsp;            'vvol_wafliron' => '0',
##&nbsp;            'sanown' => '0',
##&nbsp;            'chipset' => '0',
##&nbsp;            'diskmaint' => '0',
##&nbsp;            'nht' => '0',
##&nbsp;            'nsdb' => '0',
##&nbsp;            'array_persist' => '0',
##&nbsp;            'sp_notification_sender' => '0',
##&nbsp;            'tm' => '0',
##&nbsp;            'bdb' => '0',
##&nbsp;            'nvram5' => '0',
##&nbsp;            'scsi' => '0',
##&nbsp;            'sis_clone' => '0',
##&nbsp;            'fmm' => '0',
##&nbsp;            'rlm_cnsl' => '0',
##&nbsp;            'delayed_free' => '0',
##&nbsp;            'sonic' => '0',
##&nbsp;            'ndr' => '0',
##&nbsp;            'nc_registry' => '0',
##&nbsp;            'nfsv4_XX' => '0',
##&nbsp;            'version' => '0',
##&nbsp;            'iwcm' => '0',
##&nbsp;            'sparecore' => '0',
##&nbsp;            'ib_core' => '0',
##&nbsp;            'vdom' => '0',
##&nbsp;            'cmt' => '0',
##&nbsp;            'ndmpd' => '0',
##&nbsp;            'netinet' => '0',
##&nbsp;            'lmgr' => '0',
##&nbsp;            'snapvault' => '0',
##&nbsp;            'cma' => '0',
##&nbsp;            'od' => '0',
##&nbsp;            'toe' => '0',
##&nbsp;            'worm' => '0',
##&nbsp;            'vldb' => '0',
##&nbsp;            'pc nmi' => '0',
##&nbsp;            'tdd' => '0',
##&nbsp;            'spinnp' => '0',
##&nbsp;            'fcal' => '0',
##&nbsp;            'qos' => '0',
##&nbsp;            'environ' => '0',
##&nbsp;            'array_mon' => '0',
##&nbsp;            'ra' => '0',
##&nbsp;            'uipc' => '0',
##&nbsp;            'i2p' => '0',
##&nbsp;            'ic' => '0',
##&nbsp;            'fold' => '0',
##&nbsp;            'timed' => '0',
##&nbsp;            'eeprom_sensor' => '0',
##&nbsp;            'httpd' => '0',
##&nbsp;            'esh' => '0',
##&nbsp;            'sli' => '0',
##&nbsp;            'vm' => '0',
##&nbsp;            'mnode' => '0',
##&nbsp;            'mbuf' => '0',
##&nbsp;            'lmem' => '0',
##&nbsp;            'replckpt' => '0',
##&nbsp;            'diskfw' => '0',
##&nbsp;            'sp_mgmt' => '0',
##&nbsp;            'vfiler' => '0',
##&nbsp;            'aggr_gb_arl' => '0',
##&nbsp;            'tar' => '0',
##&nbsp;            'quota' => '0',
##&nbsp;            'registry' => '0',
##&nbsp;            'replpedigree' => '0',
##&nbsp;            'ndvm' => '0',
##&nbsp;            'snapdiff' => '0',
##&nbsp;            'sparse' => '0',
##&nbsp;            'sysconfig' => '0',
##&nbsp;            'lockops_XX' => '0',
##&nbsp;            'auth' => '0',
##&nbsp;            'fct' => '32773',
##&nbsp;            'pcnfsd' => '0',
##&nbsp;            'sp_state_manager' => '0',
##&nbsp;            'pccfe' => '0',
##&nbsp;            'aha' => '0',
##&nbsp;            'v4lock_XX' => '0',
##&nbsp;            'pmap' => '0',
##&nbsp;            'lsiscsi' => '0',
##&nbsp;            'icbulk' => '1',
##&nbsp;            'nfsd' => '0',
##&nbsp;            'scanflag' => '0',
##&nbsp;            'cd' => '0',
##&nbsp;            'rpc' => '0',
##&nbsp;            'ups' => '0',
##&nbsp;            'lrep' => '0',
##&nbsp;            'shm' => '0',
##&nbsp;            'scsit' => '16777200',
##&nbsp;            'cp' => '0',
##&nbsp;            'wafl' => '0',
##&nbsp;            'htbt' => '0',
##&nbsp;            'idump' => '0',
##&nbsp;            'perfmon' => '0',
##&nbsp;            'software' => '0',
##&nbsp;            'via' => '0',
##&nbsp;            'ltm' => '0',
##&nbsp;            'rsm' => '0',
##&nbsp;            'waflng' => '0',
##&nbsp;            'ofed' => '0',
##&nbsp;            'incpd' => '0',
##&nbsp;            'ngwatch' => '0',
##&nbsp;            'nfsd_XX' => '0',
##&nbsp;            'icrdma' => '0',
##&nbsp;            'wafl_XX' => '0',
##&nbsp;            'sp_cluster_user_mgmt' => '0',
##&nbsp;            'waflra' => '0',
##&nbsp;            'avoa' => '0',
##&nbsp;            'lockops' => '0',
##&nbsp;            'qual_file' => '0',
##&nbsp;            'xlate' => '0',
##&nbsp;            'ibmlx_core' => '0',
##&nbsp;            'ext_cache' => '0',
##&nbsp;            'ctrlrdma' => '0',
##&nbsp;            'sasmon' => '0',
##&nbsp;            'dense' => '0',
##&nbsp;            'fcmon' => '0',
##&nbsp;            'config_repl' => '0',
##&nbsp;            'dim' => '0',
##&nbsp;            'ibmlx_mlx4' => '0',
##&nbsp;            'replsrc_wafl' => '0',
##&nbsp;            'pac' => '0',
##&nbsp;            'ics' => '0',
##&nbsp;            'rdr' => '0',
##&nbsp;            'raid2io' => '0',
##&nbsp;            'mgmt' => '0',
##&nbsp;            'flint_mlx4' => '0',
##&nbsp;            'shout' => '0',
##&nbsp;            'sas' => '0',
##&nbsp;            'rv' => '0',
##&nbsp;            'fdisk' => '0',
##&nbsp;            'v4lock' => '0',
##&nbsp;            'fal' => '0',
##&nbsp;            'fault' => '0',
##&nbsp;            'sfsr' => '0',
##&nbsp;            'naloader' => '0',
##&nbsp;            'core' => '0',
##&nbsp;            'pciconfig' => '0',
##&nbsp;            'lprdma' => '0',
##&nbsp;            'asup' => '0',
##&nbsp;            'ibnet_mlx4' => '0',
##&nbsp;            'cat' => '0',
##&nbsp;            'vseries' => '0',
##&nbsp;            'wafliron' => '0',
##&nbsp;            'pfs' => '0',
##&nbsp;            'replchecker' => '0',
##&nbsp;            'orftp' => '0',
##&nbsp;            'vfmigrate' => '0',
##&nbsp;            'mlnx' => '0',
##&nbsp;            'mlm' => '0',
##&nbsp;            'vtic' => '0',
##&nbsp;            'rdmalib' => '0',
##&nbsp;            'memio' => '0',
##&nbsp;            'sk' => '0',
##&nbsp;            'kmod' => '0',
##&nbsp;            'dmp' => '0',
##&nbsp;            'repldst_wafl' => '0',
##&nbsp;            'gdb' => '0',
##&nbsp;            'raid2' => '0',
##&nbsp;            'mntd' => '0',
##&nbsp;            'page' => '0',
##&nbsp;            'rlm_mgmt' => '0',
##&nbsp;            'cxgb3' => '0',
##&nbsp;            'bplus' => '0',
##&nbsp;            'net' => '0',
##&nbsp;            'nlm' => '0',
##&nbsp;            'blkr' => '0',
##&nbsp;            'fm_hwassist' => '0',
##&nbsp;            'iscsi' => '0',
##&nbsp;            'replchecker_wafl' => '0',
##&nbsp;            'xfiler_CF' => '0',
##&nbsp;            'coral' => '0',
##&nbsp;            'scan' => '0',
##&nbsp;            'ad7994' => '0',
##&nbsp;            'guardrail' => '0',
##&nbsp;            'snap' => '0',
##&nbsp;            'vi_if_rnic' => '0',
##&nbsp;            'fm' => '0',
##&nbsp;            'ngsecurity' => '0',
##&nbsp;            'nic_mgr' => '0',
##&nbsp;            'rsh' => '0',
##&nbsp;            'utilclass' => '0',
##&nbsp;            'rebl' => '0',
##&nbsp;            'com_e' => '0',
##&nbsp;            'nodestatus' => '0',
##&nbsp;            'pccard' => '0',
##&nbsp;            'vvol' => '0',
##&nbsp;            'qlic' => '0',
##&nbsp;            'dsb' => '0',
##&nbsp;            'tape' => '0',
##&nbsp;            'sp_notification_receiver' => '0',
##&nbsp;            'lhm' => '0',
##&nbsp;            'sbs' => '0',
##&nbsp;            'ncstat' => '0',
##&nbsp;            'ib' => '0',
##&nbsp;            'ses' => '0',
##&nbsp;            'passthru' => '0',
##&nbsp;            'sagittarius' => '0',
##&nbsp;            'sdm' => '0',
##&nbsp;            'playlist' => '0',
##&nbsp;            'snmp' => '0',
##&nbsp;            'mvia' => '0',
##&nbsp;            'ata' => '0',
##&nbsp;            'sec' => '0',
##&nbsp;            'nrv' => '0',
##&nbsp;            'SMBUS' => '0',
##&nbsp;            'sesfw' => '0',
##&nbsp;            'adm1026' => '0',
##&nbsp;            'smb_mask' => '0',
##&nbsp;            'rlm_takeover' => '0',
##&nbsp;            'ocm' => '0',
##&nbsp;            'fcs' => '0',
##&nbsp;            'fiji' => '0',
##&nbsp;            'snaptags' => '0',
##&nbsp;            'snapmirror' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = dbg level fct auth=0 'pc nmi'
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'auth' => '0',
##&nbsp;            'fct' => '32773',
##&nbsp;            'pc nmi' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub dbg_level {
    my %args = @_;

    my ($error_message, @result_array);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $can_throw = 1;
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $can_throw = 0;
    }

    # For dbg level, some output is always expected. So we are making
    # $can_throw as 1 so that if the lines array got finally in _get_lines is
    # empty, UnexpectedOutputException can be thrown there.
    my @arr = get_lines(
        "raw_output"      => $output,
        "can_throw"       => $can_throw,
        "response_object" => $resp
    );

    my %dbg_level;

    for ( my $line = 0; $line < @arr; $line++ ) {
        chomp $line;
        if ( $arr[$line] =~ m/dbg:\s+(\w+\s?\w+)\s+\:?\=?\s?(\d+)/ ) {
            $dbg_level{$1} = $2;
        }
    }
    @result_array = (\%dbg_level);

    if ($error) {
        $resp->set_parsed_output(parsed_output=>\@result_array);
        $error->throw(text=>$error_message,response_object => $resp);
    }

    return \@result_array;
} # end sub _dbg_level

## @name ::dbg_to
##
## @summary Parses the raw output from the "dbg to" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from dbg_to with any of its options as described in the
##    corresponding cdef file 'dbg.cdefs'
## @synopsis
##      dbg_to(%args);
## INPUTS TO METHOD CALL 'dbg_to' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'console' </td> <td> 'console' </td> </tr>
## <tr> <td> 'off' </td> <td> 'off' </td> </tr>
## <tr> <td> 'log' </td> <td> 'log' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = dbg to off
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = dbg to log
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = dbg to console
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = dbg to
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'dbg_printf_targeted_to' => 'CONSOLE'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub dbg_to {
    my %args = @_;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my (%process_output, @final, $error_message);
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    } elsif($output =~ /dbg: dbg_printf targeted to:(\s+(\w+(\s+\w+)?))?/){
        $process_output{'dbg_printf_targeted_to'} = $2;
        push(@final,\%process_output);
    }
    return(\@final);
}

1;

