#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/disk.pm#30 $
#
#Copyright (c) 2014 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode disk commands
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode disk commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "disk show" will be disk_show()
##
## @synopsis
##    use disk;
##
##    disk_show(%args);
##    disk_zero_spares(%args);
##    disk_maint_start(%args);
##    disk_maint_abort(%args);
##    disk_maint_status(%args);
##    disk_maint_list(%args);
##    disk_remove_ownership(%args);
##    disk_unfail(%args);
##    disk_assign(%args);
##    disk_remove(%args);
##    disk_replace(%args);
##    disk_fail(%args);
##    disk_ddr_label_print(%args);
##    disk_ddr_label_start_scan(%args);
##    disk_ddr_label_delete(%args);
##    disk_ddr_label_repair(%args);
##    disk_ddr_label_dumpraw(%args);
##    disk_ddr_label_modify(%args);
##    disk_ddr_label_error_scan(%args);
##    disk_offline(%args);
##    disk_online(%args);
##    disk_sanitize_start(%args);
##    disk_sanitize_status(%args);
##    disk_sanitize_abort(%args);
##    disk_encrypt_show(%args);
##    disk_encrypt_lock(%args);
##    disk_encrypt_unlock(%args);
##    disk_encrypt_rekey(%args);
##    disk_encrypt_sanitize(%args);
##    disk_encrypt_destroy(%args);
##    disk_reassign(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::disk;

use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::CapabilityException;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $c             = $command . ':\s*';
    my $error;
    if ((   $output
            =~ /$c((Missing.*)|(at\sleast\sone\s.*\sargument\srequired)|(missing\sdisk\scount\sparameter\swith\s-n\soption.*)
        |(Missing\sstorage\stype\sparameter.*)|(Disk\sor\swildcarding\snot\sspecified.*)|([Pp]refail\s+of\s+disk.*))/
        )
        || ($output =~ /(at\s+least\s+one\s+<disk_name>\s+argument\s+required)|(option\s+requires\s+an\s+argument)|(No\s+argument\s+supplied\s+for\s+unpartition\s+option)|([mM]issing.*)/)
        ) {
        $error = "MissingRequiredParam";
    } elsif ($output
        =~ /$c((Unrecognized\soption.*)|(Has extra arguments)|(.* only allowed in Maintenance mode)|(Unrecognized\soption\s.*)|([Ii]nvalid\s+error\s+code.*))/
        ) {
        $error = "InvalidParam";
    } elsif (
        (   $output
            =~ /$c((Could\snot\sfind.*)|(Failed\s+to\s+add\s+disk.*)|(Disk\s+\w+\s+does\s+not\s+exist.)|(Disk.*not\sfound)|(duplicate disk name.*)|(Disk.*does not exist)|(Invalid\schecksum\sparameter.)|(Proper\sinteger\svalue\sis\srequired\sfor\soption\s-n.*)|(invalid\sstorage\stype\s.*)|(Incorrect\sshelf\sname\sis\sgiven.*)|(Unrecognized\spool\soption\s.*)|([Mm]issing\s+(on|off)line\s+type.*)|(.*Disk \w+ not found.*)|(Disk \w+(.\d+)? is not a file system disk.*)|(disk\s+start\s*:\s+\w+(.\d+)?\s+isn't\s+a\s+spare\s+disk.*))/
        )
        || ($output
            =~ /(.*disk\s+start\s*:\s+\w+.\d+.?\d+?\s+isn't\s+a\s+spare\s+disk.*)|(.*disk\s+sanitize:\s*Disk\s+\w+.\d+.?\d+?\s+not\s+found.*)|(Maximum\s+of\s+7\s+cycles\s+can\s+be\s+specified.*) |(.*disk\s+sanitize:\s+SSD\s+\w+.\d+.?\d+?\s+does\s+not\s+support\s+sanitize.*)|(.*invalid\s+parameter:\s+auto-sized\s+partitions\s+too\s+small)|(Too\s+many.*disk_name)|(.*num_parts.*invalid)|(disk\s+unpartition:\s+Disk.*not\s+found)/
        )
        ) {
		$error = "InvalidParamValue";
    } elsif ($output
        =~ /$c((Disk.*not\savailable)|([sS]tatus.*has\sfailed)|(Disk is not currently\s+failed)|(.*[Dd]isk\s+.*\s+is\s+not\s+a\s+file\s+.*)|(Could\snot\sfind\sany\sunowned\sdisks\sto\sassign.*)|([Cc]annot\s+(on|off)line\s+disk.*)|(Unowned\sdisks\sexist,\sbut\snot\svalid\sfor\sassignment\sto\spool\s.*)|(Could\snot\sfind\sany\s+unowned\sdisks\sto\sassign.)|(Disk\s.*\sis\snot\sowned\sby\sthis\snode.*)|(Disks\smay\sbe\sautomatically\sassigned\sto\sthis\snode,\ssince\soption\sdisk.auto_assign\sis\son.*)|(Not\senough\sunowned\sdisks\savailable)|(checksum\sread\sfor\sLUN\s.*\shas\sfailed\swith\sstatus\s'bad\svalue\sfor\sdisk'.\sAssign\srequest\shas\sfailed.*)|(Assign\srequest\sfailed\sfor\sdisk\s.*\sReason:Disk\sis\spart\sof\san\sonline\saggregate\sor\svolume.\sChanging\sits\sowner\smay\scause\saggregate,\svolume,\sor\sfiler\soutage.\sOwnership\smay\sbe\schanged\sonly\sby\susing\sthe\sappropriate\sforce\soption.*)|(Disk\s.*\not\saccessible.\sError\s16\s(disk does not exist))|(Disk\s+.*\s+is\s+not\s+a\s+spare\s+disk.*))|(disk\s+remove\s+.*:\s+Disk\s+.*\s+is\s+not\s+a\s+spare\s+disk.*)|(Prefail\s+of\s+disk.*)|(may\sbe\sfailed,\sfaulted,\s+unowned)|(Assign\srequest\sfailed\sfor\sdisk\s.*\sReason:Disk\sis\spart\sof\sa\sfailed\sor\soffline\saggregate\sor\svolume.\sChanging\sits\sowner\smay\sprevent\saggregate\sor\svolume\sfrom\scoming\sback\sonline.\sOwnership\smay\sbe\schanged\sonly\sby\susing\sthe\sappropriate\sforce\soption.*)|(Unable\sto\slocate\sa\ssystem\snamed)|(Assign\sfailed\sfor\sone\sor\smore\sdisks)|(Disk\sis\scurrently\sowned\sby\sthe\spartner\sand\scannot\sbe\sassigned)|(cannot\sassign\sthis\sdisk\sto\sanother\snode\swhile\schanging\sthe\schecksum\stype)|(Another\sdisk\sin\sthe\sdisk\scarrier\sis\sassigned\sto\sanother\snode\sor\sfailed)|(this\schecksum\stype\sis\snot\ssupported)/
        ) {
        $error = "CommandFailed";
    } elsif ($output =~ /(usage)|(.*WARNING!!.*)/i) {
        $error          = "CommandFailed";
        $$error_message = "Command failed because of the dependencies";
        return $error;
    } elsif ($output =~ /No such disk/) {
        $error          = "InvalidParamValue";
        $$error_message = "Command failed because no such disk exists";
        return $error;
    } elsif (
        $output =~ /((.*[Ii]nvalid\s+option.*)|(.*DDR\s+:.*check usage.*))/) {
        $error          = "InvalidParam";
        $$error_message = "Command failed because of invalid param";
        return $error;
    } elsif ($output
        =~ /((.*error\s+DDR\s+.*)|(.*DDR\s+\w+:\s+(Failed).*))|(.*disk\s+ddr_label\s+{.*error_scan.*)|(.*Could not access label.*)/
        ) {
        $error          = "CommandFailed";
        $$error_message = "Command failed because of dependencies";
        return $error;
    } elsif ($output
        =~ /(disk\s+assign:\s+Assign\s+failed\s+because\s+disk\s+(.*)\s+has\s+redundancy\s+errors)/
        ) {
        $error = "CommandFailed";
        $$error_message
            = "disk assign failed because disk has redundancy errors";
    } elsif ($output
        =~ /(disk\s+sanitize\s+(status|abort):\s+Couldn\'t\s+find\s+(.*)\.)/)
    { #disk sanitize status: Couldn't find 0b.35.#disk sanitize abort: Couldn't find 0b.35
        $error = "CommandFailed";
    } elsif ($output =~ /(disk\s+sanitize\s+abort\s+<disk_list>)/)
    {    # disk sanitize abort <disk_list>
        $error = "CommandFailed";
    } elsif (
        $output =~ /(at\s+least\s+one\s+<disk_name>\s+argument\s+required)/)
    {    # at least one <disk_name> argument required
        $error = "CommandFailed";
    } elsif ($output =~ /Use of sanitization is not enabled/) {
        $error = "Capability";
    } elsif ($output =~ /Maximum of 7 cycles can be specified/) {
        $error = "InvalidParamValue";
    } elsif ($output
        =~ /.*LUN\s+or\s+HDD.*file\s+system\s+data\s+and\s+unable\s+to\s+retrieve\s+current\s+checksum/
        ) {
        $error = "CommandFailed";
    } elsif (
        $output =~ /ERROR: The system is not configured to run this command/)
    {
        $error = "CommandFailed";
    }
     elsif ( $output =~ /disk reassign:.*/i ) {
        $error = "InvalidParam";
    }
     elsif ( $output =~ /Disk reassign request failed. Reason:.*/i ) {
        $error = "CommandFailed";
    } elsif ( $output =~ /disk\s+partition:.*too\s+big|.*partitions\s+too\s+small/i ) {
       $error = "InvalidParamValue";
    }
    $$error_message = $output;
    return $error;
} ## end sub _check_error

## @name ::disk_zero_spares
##
## @summary Parses the raw output from the "disk zero spares" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_zero_spares with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_zero_spares(%args);
## INPUTS TO METHOD CALL 'disk_zero_spares' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub disk_zero_spares {    #done
    my %args = @_;

    my ($error_message);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output    => $output,
        command       => "disk zero spares",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub disk_zero_spares

## @name ::disk_show
##
## @summary Parses the raw output from the "disk show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_show with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_show(%args);
## INPUTS TO METHOD CALL 'disk_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'not-owned' </td> <td> '1' </td> </tr>
## <tr> <td> '-o' </td> <td> 'owner-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'sysid' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '-z' </td> <td> 'prkey' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; disk show -v; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  DISK       OWNER                  POOL   SERIAL NUMBER           HOME                     CHKSUM
##&nbsp;------------ -------------          -----  -------------           -------------            ------
##&nbsp;v4.16        sti6-vsim-ucs043b(4068748814)    Pool0  14082500              sti6-vsim-ucs043b(4068748814)  Block
##&nbsp;v4.17        sti6-vsim-ucs043b(4068748814)    Pool0  14082501              sti6-vsim-ucs043b(4068748814)  Block
##&nbsp;v4.18        sti6-vsim-ucs043b(4068748814)    Pool0  14082502              sti6-vsim-ucs043b(4068748814)  Block
##&nbsp;v4.19        sti6-vsim-ucs043b(4068748814)    FAILED 14082503              sti6-vsim-ucs043b(4068748814)  Block
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'serial_number' => '14082500',
##&nbsp;            'owner' => 'sti6-vsim-ucs043b(4068748814)',
##&nbsp;            'disk' => 'v4.16',
##&nbsp;            'pool' => 'Pool0',
##&nbsp;            'chksum' => 'Block',
##&nbsp;            'home' => 'sti6-vsim-ucs043b(4068748814)'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'serial_number' => '14082501',
##&nbsp;            'owner' => 'sti6-vsim-ucs043b(4068748814)',
##&nbsp;            'disk' => 'v4.17',
##&nbsp;            'pool' => 'Pool0',
##&nbsp;            'chksum' => 'Block',
##&nbsp;            'home' => 'sti6-vsim-ucs043b(4068748814)'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'serial_number' => '14082502',
##&nbsp;            'owner' => 'sti6-vsim-ucs043b(4068748814)',
##&nbsp;            'disk' => 'v4.18',
##&nbsp;            'pool' => 'Pool0',
##&nbsp;            'chksum' => 'Block',
##&nbsp;            'home' => 'sti6-vsim-ucs043b(4068748814)'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'serial_number' => '14082503',
##&nbsp;            'owner' => 'sti6-vsim-ucs043b(4068748814)',
##&nbsp;            'disk' => 'v4.19',
##&nbsp;            'pool' => 'FAILED',
##&nbsp;            'chksum' => 'Block',
##&nbsp;            'home' => 'sti6-vsim-ucs043b(4068748814)'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; disk show -v; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;  DISK       OWNER                    POOL   SERIAL NUMBER         HOME                     DR HOME                CHKSUM
##&nbsp;------------ -------------            -----  -------------         -------------            -------------          --------
##&nbsp;0b.03.13     sti2240-2-072(1786433986)    Pool0  PNV0WGEB              sti2240-2-072(1786433986)                          Block
##&nbsp;0b.03.19     sti2240-2-072(1786433986)    Pool0  PNV0SP4B              sti2240-2-072(1786433986)                          Block
##&nbsp;0b.02.14     sti2240-2-072(1786433986)    Pool0  PNV0N2HB              sti2240-2-072(1786433986)                          Block
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'dr_home' => '',
##&nbsp;            'serial_number' => 'PNV0WGEB',
##&nbsp;            'owner' => 'sti2240-2-072(1786433986)',
##&nbsp;            'disk' => '0b.03.13',
##&nbsp;            'pool' => 'Pool0',
##&nbsp;            'chksum' => 'Block',
##&nbsp;            'home' => 'sti2240-2-072(1786433986)'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'dr_home' => '',
##&nbsp;            'serial_number' => 'PNV0SP4B',
##&nbsp;            'owner' => 'sti2240-2-072(1786433986)',
##&nbsp;            'disk' => '0b.03.19',
##&nbsp;            'pool' => 'Pool0',
##&nbsp;            'chksum' => 'Block',
##&nbsp;            'home' => 'sti2240-2-072(1786433986)'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'dr_home' => '',
##&nbsp;            'serial_number' => 'PNV0N2HB',
##&nbsp;            'owner' => 'sti2240-2-072(1786433986)',
##&nbsp;            'disk' => '0b.02.14',
##&nbsp;            'pool' => 'Pool0',
##&nbsp;            'chksum' => 'Block',
##&nbsp;            'home' => 'sti2240-2-072(1786433986)'
##&nbsp;           }
##&nbsp;        ];
##&nbsp;


sub disk_show {
    my %args = @_;

    my ($error_message, @result_array);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => "disk show",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    my $first_line = 'DISK\s+OWNER\s+POOL';

    my @lines = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );
    my @keys_array = split(/\s\s+/, $lines[0]);    # get the keys
    my $item;

    for my $i (0 .. @keys_array - 1) {
        $keys_array[$i] = to_key_name(item => $keys_array[$i]);
    }

    shift(@lines);    # remove keys from output lines
    shift(@lines);    # remove heading lines ------

    my ($i);

    pop(@lines) if ($lines[$#lines] =~ /NOTE: C.*/);

    NACL::APISet::Exceptions::InvalidParamValueException->throw(
        text            => $command . " contains invalid parameter value(s)",
        response_object => $resp
    ) unless @lines;

    foreach my $line (@lines) {
        my %hash = ();

        # get the disk name
        my @values;
        if ($line =~ s/([^\s]+)\s+//) {
            ($values[0]) = $1;    # disk name
        }

        if (   $command =~ /-z/ && $line =~ /(.*)PRkey=\s*(\d+)(\s*|\*+\s*)UpCnt=\s*(\d+)\s*duid=\s*(\S+)/) {
            ($line, $hash{prkey}, $hash{upcnt}, $hash{duid}) = ($1, $2, $4, $5);    # $line, prkey, upcnt, duid
        }

        if ($line =~ /(.*)\s*(Pool\d+|FAILED|NONE)\s+(.*)/) {
            my $pool_type = $2;
            ($values[1], $values[2], my $rest)
                = ($1, $2, $3);        # owner, pool, $rest
            $values[1] =~ s/\s*$//;
            if ($command =~ /^disk show -v$/) {
                if (   $rest =~ /(.+)(Block|Zoned)/
                    && $pool_type eq "NONE") {
                    ($values[3], $values[6])
                        = ($1, $2);    # serial_number, checksum
                } elsif ($rest =~ /(\S+)\s*(.+)\s*(.+)\s*(Block|Zoned)/) {
                    ($values[3], $values[4], $values[5], $values[6])
                        = ($1, $2, $3, $4)
                        ;              # serial_number, home,dr_home,checksum
                }
            } elsif ($rest =~ /(\S+)\s*(\S*(\s*\(\S*\))?)\s*(.*)\s*(\S*)/) {
                ($values[3], $values[4], $values[5], $values[6])
                    = ($1, $2, $4, $5);
            }
        }

        $i = 0;
        foreach $item (@keys_array) {
            $values[$i] =~ s/\s+$//;
            $hash{$item} = $values[$i] || '';
            $i = $i + 1;
        }
        push(@result_array, \%hash);
    }
    return \@result_array;
} ## end sub disk_show

## @name ::disk_maint_start
##
## @summary Parses the raw output from the "disk maint start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_maint_start with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_maint_start(%args);
## INPUTS TO METHOD CALL 'disk_maint_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-i' </td> <td> 'is-immediate-disks' </td> </tr>
## <tr> <td> '-t' </td> <td> 'maint-tests' </td> </tr>
## <tr> <td> '-d' </td> <td> 'disks' </td> </tr>
## <tr> <td> '-c' </td> <td> 'maint-cycle-count' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub disk_maint_start {
    my %args = @_;

    my ($error_message, @result_array);
    my $resp      = $args{response_object};
    my $output    = $resp->get_processed_output();
    my $can_throw = 1;

    my $error = _check_error(
        raw_output    => $output,
        command       => "disk maint",
        error_message => \$error_message
    );

    if ($error) {
        $error     = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $can_throw = 0;
    }

    my $result;
    my $first_line = '[Ss]tarted';
    my @lines      = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => $can_throw,
        response_object => $resp
    );

    foreach my $line (@lines) {
        if ($line =~ /Started\stesting\s(.*)/) {
            my %hash = ('disk' => $1, 'result' => 'started');
            push(@result_array, \%hash);
        }
    }

    if ($error) {
        $resp->set_parsed_output(parsed_output => \@result_array);
        $error->throw(text => $error_message, response_object => $resp);
    }
    return \@result_array;
} ## end sub disk_maint_start

## @name ::disk_maint_abort
##
## @summary Parses the raw output from the "disk maint abort" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_maint_abort with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_maint_abort(%args);
## INPUTS TO METHOD CALL 'disk_maint_abort' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-list' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub disk_maint_abort {
    my %args = @_;

    my $can_throw = 1;
    my ($error_message, @result_array);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output    => $output,
        command       => "disk maint",
        error_message => \$error_message
    );

    if ($error) {
        $can_throw = 0;
        $error     = 'NACL::APISet::Exceptions::' . $error . 'Exception';
    }
    my $result;

    my $first_line = 'disk maint: .* testing';
    my @lines      = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => $can_throw,
        response_object => $resp
    );

    foreach my $line (@lines) {
        $result = $line =~ /:\s(.*\stesting)\s(.*)/;

        if ($result) {
            my %hash = ('disk' => $2, 'result' => $1);
            push(@result_array, \%hash);
        }

    }

    if ($error) {
        $resp->set_parsed_output(parsed_output => \@result_array);
        $error->throw(text => $error_message, response_object => $resp);
    }
    return \@result_array;
} ## end sub disk_maint_abort

## @name ::_initial_parse
##
## @summary : returns 1 if a new section of output has started. 0 otherwise
##
## @description
##    This method is used for determining if a new hash has to be started to
##    store the next section of output.
##
## @synopsis
##    _initial_parse (%args);
##
## @arg %args Required:
## line => the line of output to be scanned
## hash1 => reference to hash holding current output section (before this line)
## hash2 => reference to empty hash which is used to copy old hash from hash1
##
## @return Returns 1 or 0

sub _initial_parse {
    my %args  = @_;
    my $line  = $args{line};
    my $hash1 = $args{hash1};
    my $hash2 = $args{hash2};

    if ($line =~ /(.*)\s(.*)\stest\s(.*)\scycle\s(.*)/) {
        %$hash2 = %$hash1;
        %$hash1 = (
            'status' => $1,
            'disk'   => $2,
            'test'   => $3,
            'cycle'  => $4
        );
    } elsif ($line =~ /Pending\stesting\s(.*)/) {
        %$hash2 = %$hash1;
        %$hash1 = ('status' => 'pending', 'disk' => $1);
    } elsif ($line =~ /Copying\s(.*)/) {
        %$hash2 = %$hash1;
        %$hash1 = ('status' => 'Copying', 'disk' => $1);
    } elsif ($line =~ /Not\stesting\s(.*)/) {
        %$hash2 = %$hash1;
        %$hash1 = ('status' => 'Not testing', 'disk' => $1);
    } else {
        return 0;
    }
    return 1;
} ## end sub _initial_parse

## @name ::disk_maint_status
##
## @summary Parses the raw output from the "disk maint status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_maint_status with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_maint_status(%args);
## INPUTS TO METHOD CALL 'disk_maint_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-list' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub disk_maint_status {
    my %args = @_;

    my $can_throw = 1;
    my ($error_message, @result_array);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        command       => "disk maint",
        error_message => \$error_message
    );

    if ($error) {
        $can_throw = 0;
        $error     = 'NACL::APISet::Exceptions::' . $error . 'Exception';
    }

    my $first_line = '(Not\s)?[TtCP]...ing';
    my @lines      = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => $can_throw,
        response_object => $resp
    );

    my ($verbose, %hash, $i, $pushable, $item);
    my %verbose_valid_keys = (
        'test_name' => 1,
        'iteration' => 1,
        'cycle'     => 1,
        'status'    => 1
    );

    if ($command =~ /\s*-v\s*/) {
        $verbose = 1;
        $hash{'tests'} = [];
    }

    #logic for the first section of output
    _initial_parse(line => $lines[0], hash1 => \%hash);

    #now ready the lines array for the loop
    shift(@lines);

    foreach my $line (@lines) {
        my %hash2;
        $pushable = _initial_parse(
            line  => $line,
            hash1 => \%hash,
            hash2 => \%hash2
        );

        if ($pushable) {
            push(@result_array, \%hash2);
            $i = 0;    #reset
        } elsif ($verbose) {
            my @items = split(/\s\s+/, $line);
            my %testline;
            foreach $item (@items) {
                my ($name, $value) = split(/:\s+/, $item);
                $name = to_key_name(item => $name);

                #die "Unexpected output" if(!$verbose_valid_keys{$name});
                $testline{$name} = $value;
                last if $name eq 'time';
                NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                    text            => "Invalid key $name found in $line",
                    response_object => $resp
                ) if (!$verbose_valid_keys{$name} && !$error);
            }

            if (exists($testline{'time'})) {
                $testline{'time'} = $items[$#items];
                delete $testline{$items[$#items]};
            }
            ${$hash{'tests'}}[$i] = \%testline;
            $i++;
        }
    }
    push(@result_array, \%hash);
    if ($error) {
        $resp->set_parsed_output(parsed_output => \@result_array);
        $error->throw(text => $error_message, response_object => $resp);
    }
    return \@result_array;
} ## end sub disk_maint_status

## @name ::disk_maint_list
##
## @summary Parses the raw output from the "disk maint list" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_maint_list with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_maint_list(%args);
## INPUTS TO METHOD CALL 'disk_maint_list' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub disk_maint_list {
    my %args = @_;

    my ($error_message, @result_array);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my %valid_keys = (
        'test_index' => 1,
        'test_id'    => 1,
        'test_name'  => 1
    );

    my $first_line = 'Test index.*Test Id.*Test name.*';
    my @lines      = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );

    foreach my $line (@lines) {
        my @items = split(/\s\s+/, $line);
        my %hash;
        foreach my $item (@items) {
            my ($name, $value) = split(/:\s/, $item);
            $name = to_key_name(item => $name);
            NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                text            => "Invalid key $name found in $line",
                response_object => $resp
            ) if (!$valid_keys{$name});
            $hash{$name} = $value;
        }
        push(@result_array, \%hash);
    }
    return \@result_array;
} ## end sub disk_maint_list

## @name ::disk_unfail
##
## @summary Parses the raw output from the "disk unfail" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_unfail with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_unfail(%args);
## INPUTS TO METHOD CALL 'disk_unfail' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-list' </td> </tr>
## <tr> <td> '-s' </td> <td> 'make-spare' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub disk_unfail {
    my %args = @_;

    my ($error_message, @result_array);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output    => $output,
        command       => 'disk unfail',
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    } elsif ($output
        =~ /(unfailing disk|clearing prefail status of disk) ([^\s]*)/) {
        @result_array = ({'disk' => $2, 'status' => $1});
        $result_array[0]->{'disk'} =~ s/\.+$//;
    }
    return \@result_array;
} ## end sub disk_unfail

## @name ::disk_remove_ownership
##
## @summary Parses the raw output from the "disk remove_ownership" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_remove_ownership with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_remove_ownership(%args);
## INPUTS TO METHOD CALL 'disk_remove_ownership' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'sysid' </td> <td> '0' </td> </tr>
## <tr> <td> 'all' </td> <td> 'all-disks' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub disk_remove_ownership {
    my %args = @_;

    my ($error_message, @result_array);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output    => $output,
        command       => 'disk remove_ownership',
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    } elsif ($output =~ /Disk (.*) will have its ownership removed/) {
        @result_array
            = ({'disk' => $1, 'status' => 'ownership removal initiated'});
    }
    return \@result_array;
} ## end sub disk_remove_ownership

## @name ::disk_assign
##
## @summary Parses the raw output from the "disk assign" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_assign with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_assign(%args);
## INPUTS TO METHOD CALL 'disk_assign' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'checksum' </td> </tr>
## <tr> <td> '-n' </td> <td> 'count' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-list' </td> </tr>
## <tr> <td> '-s' </td> <td> 'sysid' </td> </tr>
## <tr> <td> 'all' </td> <td> 'all-disks' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## <tr> <td> '-o' </td> <td> 'owner-name' </td> </tr>
## <tr> <td> '-shelf' </td> <td> 'shelf' </td> </tr>
## <tr> <td> '-p' </td> <td> 'pool' </td> </tr>
## <tr> <td> 'auto' </td> <td> 'auto-assign' </td> </tr>
## <tr> <td> '-T' </td> <td> 'storage-type' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = disk assign -n
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = disk assign auto
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = disk assign -o
##&nbsp;$VAR1 = [];
##&nbsp;
sub disk_assign {
    my %args = @_;
    my $error_message;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output    => $output,
        command       => 'disk assign',
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub disk_assign

## @name ::disk_remove
##
## @summary Parses the raw output from the "disk remove" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_remove with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_remove(%args);
## INPUTS TO METHOD CALL 'disk_remove' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk' </td> <td> '0' </td> </tr>
## <tr> <td> '-w' </td> <td> 'gateway-filer' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = disk remove disk-name
##&nbsp;$VAR1 [
##&nbsp;        {
##&nbsp;          'disk_removed' => disk-name,
##&nbsp;          'status' => 'remove and unload of disk has been initiated',
##&nbsp;        }
##&nbsp;      ]
sub disk_remove {
    my %args = @_;
    my ($error_message, @result_array);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output    => $output,
        command       => 'disk remove',
        error_message => \$error_message
    );
    my %hash;
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    } elsif ($output
        =~ /disk remove:\s+The\s+following\s+disk\s+was\s+removed:\s+(.*)/) {
        $hash{disk_removed} = $1;
        $hash{status}       = 'remove and unload of disk has been initiated';
        push(@result_array, \%hash);
    }
    return \@result_array;
} ## end sub disk_remove
## @name ::disk_replace
##
## @summary Parses the raw output from the "disk replace" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_replace with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_replace(%args);
## INPUTS TO METHOD CALL 'disk_replace' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'spare-disk-name' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk' </td> </tr>
## <tr> <td> 'stop' </td> <td> 'stop' </td> </tr>
## <tr> <td> '-m' </td> <td> 'mixing-disks' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## <tr> <td> 'start' </td> <td> 'start' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; disk replace start -f -m v0.17 v4.16; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'disk' => 'v0.17',
##&nbsp;            'status' => 'marked for replacement.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub disk_replace {
    my %args = @_;
    my ($error_message, @result_array);
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error  = _check_error(
        raw_output    => $output,
        command       => 'disk replace',
        error_message => \$error_message
    );
    my %info;
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if ($output =~ /^\w+\s+\w+\:\s+\w+\s+(.*)\s+\w+\s(\w+\s+\w+\s+\w.*)/) {
        $info{disk}   = $1;
        $info{status} = $2;
    } elsif ($output =~ /.*\s+You\s+are\s+about\s+to\s+copy.*/) {
        my @arr = get_lines(
            "raw_output"      => $output,
            "response_object" => $resp
        );
        if ($arr[4]
            =~ /(\w+)\s+(\w+.\d+)\s+(\w+)\s+(.)\s+(.)\s+(\w+.\w+)\s+(.)\s+(\w+)\s+(\d+)\s+(\d+.\d+)\s+(\d+.\d+)/
            ) {
            $info{"raid_disk"}      = $1;
            $info{"device"}         = $2;
            $info{"ha"}             = $3;
            $info{"shelf"}          = $4;
            $info{"bay"}            = $5;
            $info{"chan"}           = $6;
            $info{"pool"}           = $7;
            $info{"type"}           = $8;
            $info{"rpm"}            = $9;
            $info{"used_space"}     = $10;
            $info{"physical_space"} = $11;
        }
        if ($arr[6] =~ /\.*:\s+\w+\s+(\w+.\d+)\s+\w+\s+(\w+\s+\w+\s+\w+)/) {
            $info{disk}   = $1;
            $info{status} = $2;
        }
    } elsif ($output =~ /\w+\s+\w+\:\s+(\w+\s+\w+)\s+\w+\s+\w+\s+(\w+.\d+).*/)
    {
        $info{disk}   = $2;
        $info{status} = $1;
    }
    push(@result_array, \%info);
    return \@result_array;
} ## end sub disk_replace

## @name ::disk_fail
##
## @summary Parses the raw output from the "disk fail" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_fail with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_fail(%args);
## INPUTS TO METHOD CALL 'disk_fail' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-i' </td> <td> 'is-immediate-disks' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = disk fail -i -f  v5.22
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'disk' => 'v5.22',
##&nbsp;            'status' => 'failed'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub disk_fail {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @result_array;

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "disk fail",
        "error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my $value;
    my %info;
    my @arr = get_lines(
        "raw_output"      => $output,
        "response_object" => $resp
    );
    foreach $value (@arr) {
        if ($value =~ /[Dd]isk\s+(.*)\s+is\s+the\s+parity\s+disk\s+.*/) {
            $info{disk} = $1;
        } elsif ($value
            =~ /(volume\s+will\s+remain\s+24\/7\s+.*) | ([Tt]he\s+following\s+disk\s+was\s+failed)/
            ) {
            $info{status} = "failed";
        }
    }
    push(@result_array, \%info);
    return \@result_array;
} ## end sub disk_fail

## @name ::disk_ddr_label_print
##
## @summary Parses the raw output from the "disk ddr_label print" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_ddr_label_print with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_ddr_label_print(%args);
## INPUTS TO METHOD CALL 'disk_ddr_label_print' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-d' </td> <td> 'disks' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; disk ddr_label print -d 0c.24 0a.16; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Disk: 0c.24
##&nbsp;Datecode: 20111014
##&nbsp;Vendor_ID: NETAPP
##&nbsp;Product_ID: X269_HJUPI01TSSX
##&nbsp;Alias Product_ID:
##&nbsp;Firmware: NA00
##&nbsp;checkrev: 0
##&nbsp;fastzero: 15300
##&nbsp;bps: 512
##&nbsp;verify_size: 128
##&nbsp;rpm: 7200
##&nbsp;last_block: GIG847_89S
##&nbsp;last_block_azcs: GIG954_AZCS
##&nbsp;qual_flags: QDC_FAST_SBCS_SW_MP_BCS89_ATA_VFY_ZCS89_SCT_SEGFW
##&nbsp;qual_flag_index: 0
##&nbsp;fake SSD: 0
##&nbsp;ZCS Flag: Disabled
##&nbsp;Mini disk header present: No
##&nbsp;
##&nbsp;Disk: 0a.16
##&nbsp;Datecode: 20111014
##&nbsp;Vendor_ID: NETAPP
##&nbsp;Product_ID: X269_HJUPI01TSSX
##&nbsp;Alias Product_ID:
##&nbsp;Firmware: NA00
##&nbsp;checkrev: 0
##&nbsp;fastzero: 15300
##&nbsp;bps: 512
##&nbsp;verify_size: 128
##&nbsp;rpm: 7200
##&nbsp;last_block: GIG847_89S
##&nbsp;last_block_azcs: GIG954_AZCS
##&nbsp;qual_flags: QDC_FAST_SBCS_SW_MP_BCS89_ATA_VFY_ZCS89_SCT_SEGFW
##&nbsp;qual_flag_index: 0
##&nbsp;fake SSD: 0
##&nbsp;ZCS Flag: Disabled
##&nbsp;Mini disk header present: No
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'fake_ssd' => '0',
##&nbsp;            'disk' => '0c.24',
##&nbsp;            'firmware' => 'NA00',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'product_id' => 'X269_HJUPI01TSSX',
##&nbsp;            'datecode' => '20111014',
##&nbsp;            'mini_disk_header_present' => 'No',
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => 'QDC_FAST_SBCS_SW_MP_BCS89_ATA_VFY_ZCS89_SCT_SEGFW',
##&nbsp;            'vendor_id' => 'NETAPP',
##&nbsp;            'fastzero' => '15300',
##&nbsp;            'qual_flag_index' => '0',
##&nbsp;            'bps' => '512',
##&nbsp;            'checkrev' => '0',
##&nbsp;            'verify_size' => '128',
##&nbsp;            'last_block_azcs' => 'GIG954_AZCS',
##&nbsp;            'zcs_flag' => 'Disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'fake_ssd' => '0',
##&nbsp;            'disk' => '0a.16',
##&nbsp;            'firmware' => 'NA00',
##&nbsp;            'rpm' => '7200',
##&nbsp;            'product_id' => 'X269_HJUPI01TSSX',
##&nbsp;            'datecode' => '20111014',
##&nbsp;            'mini_disk_header_present' => 'No',
##&nbsp;            'last_block' => 'GIG847_89S',
##&nbsp;            'qual_flags' => 'QDC_FAST_SBCS_SW_MP_BCS89_ATA_VFY_ZCS89_SCT_SEGFW',
##&nbsp;            'vendor_id' => 'NETAPP',
##&nbsp;            'fastzero' => '15300',
##&nbsp;            'qual_flag_index' => '0',
##&nbsp;            'bps' => '512',
##&nbsp;            'checkrev' => '0',
##&nbsp;            'verify_size' => '128',
##&nbsp;            'last_block_azcs' => 'GIG954_AZCS',
##&nbsp;            'zcs_flag' => 'Disabled'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub disk_ddr_label_print {
    my %args = @_;

    my ($error_message, @Res, $key,);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => "disk ddr_label print",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @lines = get_lines(
        raw_output      => $output,
        can_throw       => 1,
        response_object => $resp,
    );
    my $i = 0;
    foreach my $line (@lines) {
        if ($line =~ /^(Mini.*:\s+(\w+))/) {
            $key = $1;
            $Res[$i]->{to_key_name(item => $key)} = $2;
	    $i++;
        } elsif ($line =~ /^(Qual.*:\s+(\w+))/) {
            $key = $1;
            $Res[$i]->{to_key_name(item => $key)} = $2;
        } elsif ($line =~ /^(Alias.*:\s+(\w+))/) {
            $key = $1;
            $Res[$i]->{to_key_name(item => $key)} = $2;
        } elsif ($line =~ /^(Last.*:\s+(\w+))/) {
            $key = $1;
            $Res[$i]->{to_key_name(item => $key)} = $2;
        } elsif ($line =~ /^(\w+\s+\w+ID):(\s+(\S+))?$/) {
            $key = $1;
            $Res[$i]->{to_key_name(item => $key)} = $2;
        } elsif ($line =~ /^(\S+(\s+\w+)?):(\s+(\S+))?$/) {
            $key = $1;
            $Res[$i]->{to_key_name(item => $key)} = $4;
        }
    }
    return \@Res;
} ## end sub disk_ddr_label_print

## @name ::disk_ddr_label_start_scan
##
## @summary Parses the raw output from the "disk ddr_label start_scan" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_ddr_label_start_scan with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_ddr_label_start_scan(%args);
## INPUTS TO METHOD CALL 'disk_ddr_label_start_scan' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-d' </td> <td> 'disks' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; disk ddr_label start_scan ; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;The DDR Label Scanner has been started
##&nbsp;
##&nbsp;
sub disk_ddr_label_start_scan {
    my %args = @_;

    my ($error_message, @Res, %hash);
    my $flag    = 0;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        command       => "disk ddr_label start_scan",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if ($output !~ /(\w+\s+DDR.*\w+)/s) {
        $error_message = "Command failed because of dependencies";
        $error = 'NACL::APISet::Exceptions::' . 'CommandFailed' . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
} ## end sub disk_ddr_label_start_scan

## @name ::disk_ddr_label_delete
##
## @summary Parses the raw output from the "disk ddr_label delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_ddr_label_delete with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_ddr_label_delete(%args);
## INPUTS TO METHOD CALL 'disk_ddr_label_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-d' </td> <td> 'disks' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; disk ddr_label delete -d 0c.24 0a.16; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;DDR Delete: Succeeded for 0c.24
##&nbsp;DDR Delete: Succeeded for 0a.16
##&nbsp;
##&nbsp;
##&nbsp;Command = priv set -q diag; disk ddr_label delete -d all; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;disk ddr_label deleted for 47 owned disks
##&nbsp;
sub disk_ddr_label_delete {
    my %args = @_;

    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my (@Res, %hash);
    my $error = _check_error(
        raw_output    => $output,
        command       => "disk ddr_label delete",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @lines = get_lines(
        raw_output      => $output,
        can_throw       => 1,
        response_object => $resp,
    );
    my $flag = 0;
    foreach my $line (@lines) {
        my %hash;
        if ($line =~ /DDR\s+\w+:\s+([Ss]ucceeded)\s+\w+\s+(\S+\d+).*/ || $line =~ /disk ddr_label deleted for.* owned disks/) {
            $flag = 1;
        }
    }
    if ($flag != 1) {
        $error_message = "Command failed because of dependencies";
        $error = 'NACL::APISet::Exceptions::' . 'CommandFailed' . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if ($error) {
        $resp->set_parsed_output(parsed_output => \@Res);
        $error->throw(text => $error_message, response_object => $resp);
    }
    return \@Res;

} ## end sub disk_ddr_label_delete

## @name ::disk_ddr_label_repair
##
## @summary Parses the raw output from the "disk ddr_label repair" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_ddr_label_repair with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_ddr_label_repair(%args);
## INPUTS TO METHOD CALL 'disk_ddr_label_repair' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-d' </td> <td> 'disks' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; disk ddr_label repair -d 0c.24 0a.16; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;DDR Repair: succeeded for 0c.24. Issue a scan to readd the label !
##&nbsp;DDR Repair: succeeded for 0a.16. Issue a scan to readd the label !
##&nbsp;
##&nbsp;
sub disk_ddr_label_repair {
    my %args = @_;

    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my (@Res, %hash);
    my $error = _check_error(
        raw_output    => $output,
        command       => "disk ddr_label repair",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @lines = get_lines(
        raw_output      => $output,
        can_throw       => 1,
        response_object => $resp,
    );
    my $flag = 0;
    foreach my $line (@lines) {
        my %hash;
        if ($line =~ /DDR\s+\w+:\s+([Ss]ucceeded)\s+\w+\s+(\S+\d+)./) {
            $flag = 1;
        }
    }
    if ($flag != 1) {
        $error_message = "Command failed because of dependencies";
        $error = 'NACL::APISet::Exceptions::' . 'CommandFailed' . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if ($error) {
        $resp->set_parsed_output(parsed_output => \@Res);
        $error->throw(text => $error_message, response_object => $resp);
    }
    return \@Res;

} ## end sub disk_ddr_label_repair

## @name ::disk_ddr_label_dumpraw
##
## @summary Parses the raw output from the "disk ddr_label dumpraw" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_ddr_label_dumpraw with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_ddr_label_dumpraw(%args);
## INPUTS TO METHOD CALL 'disk_ddr_label_dumpraw' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-d' </td> <td> 'disks' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; disk ddr_label dumpraw -d 0c.24 0a.16; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Dumping raw contents: \n
##&nbsp;Dumped the raw DDR label to /etc/ddr_label_raw file!\n
##&nbsp;Dumping raw contents: \n
##&nbsp;Dumped the raw DDR label to /etc/ddr_label_raw file!\n
##&nbsp;
##&nbsp;
sub disk_ddr_label_dumpraw {
    my %args = @_;

    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my (@Res, %hash);
    my $error = _check_error(
        raw_output    => $output,
        command       => "disk ddr_label dumpraw",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @lines = get_lines(
        raw_output      => $output,
        can_throw       => 1,
        response_object => $resp,
    );
    my $flag = 0;
    foreach my $line (@lines) {
        my %hash = ();
        if ($line =~ /Dumped\s+\w+.*to\s+(\S+).*/) {
            $flag = 1;
        }
    }
    if ($flag != 1) {
        $error_message = "Command failed because of dependencies";
        $error = 'NACL::APISet::Exceptions::' . 'CommandFailed' . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    } else {
        return [];
    }

} ## end sub disk_ddr_label_dumpraw

## @name ::disk_ddr_label_modify
##
## @summary Parses the raw output from the "disk ddr_label modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_ddr_label_modify with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_ddr_label_modify(%args);
## INPUTS TO METHOD CALL 'disk_ddr_label_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-o' </td> <td> 'offset' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'value' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'disks' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; disk ddr_label modify -o 11 -v 0 -d 0c.24 0a.16; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;DDR Modify: succeeded for 0c.24
##&nbsp;DDR Modify: succeeded for 0a.16
##&nbsp;
##&nbsp;
sub disk_ddr_label_modify {
    my %args = @_;

    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my (@Res, %hash);
    my $error = _check_error(
        raw_output    => $output,
        command       => "disk ddr_label modify",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @lines = get_lines(
        raw_output      => $output,
        can_throw       => 1,
        response_object => $resp,
    );
    my $flag = 0;
    foreach my $line (@lines) {
        my %hash;
        if ($line =~ /DDR\s+\w+:\s+([Ss]ucceeded)\s+\w+\s+(\S+\d+)/) {
            $flag = 1;
        }
    }
    if ($flag != 1) {
        $error_message = "Command failed because of dependencies";
        $error = 'NACL::APISet::Exceptions::' . 'CommandFailed' . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    } else {
        return [];
    }
} ## end sub disk_ddr_label_modify

## @name ::disk_ddr_label_error_scan
##
## @summary Parses the raw output from the "disk ddr_label error_scan" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_ddr_label_error_scan with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_ddr_label_error_scan();
## INPUTS TO METHOD CALL 'disk_ddr_label_error_scan' ON APISET OBJECT
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; disk ddr_label error_scan ; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;
sub disk_ddr_label_error_scan {
    my %args = @_;

    my ($error_message, @Res, %hash);
    my $flag    = 0;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        command       => "disk ddr_label error_scan",
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    } else {
        return [];
    }
} ## end sub disk_ddr_label_error_scan

## @name ::disk_offline
##
## @summary Parses the raw output from the "disk offline" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_offline with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_offline(%args);
## INPUTS TO METHOD CALL 'disk_offline' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'type-option' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'type' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; disk offline timeout v5.32; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;disk offline: Disk v5.32 was successfully offlined.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'disk' => 'v5.32',
##&nbsp;            'status' => 'offline'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub disk_offline {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @arr;

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "disk offline",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    my $value;
    my %info;

    if ($output
        =~ /[Dd]isk\s+(\S+)\s+was\s+\w+\s+(\w+)\./)
    {
        $info{disk}   = $1;
        $info{status} = $2;
    }
    push(@arr, \%info);
    return \@arr;
} ## end sub disk_offline

## @name ::disk_online
##
## @summary Parses the raw output from the "disk online" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_online with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_online(%args);
## INPUTS TO METHOD CALL 'disk_online' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'error' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; disk online v5.17; result -1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;disk offline: Disk v5.17 was successfully onlined.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'disk' => 'v5.17',
##&nbsp;            'status' => 'online'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub disk_online {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @arr;

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "disk online",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    my $value;
    my %info;
    if ($output
        =~ /[Dd]isk\s+(\S+)\s+was\s+\w+\s+(\w+)\./) {
        $info{disk}   = $1;
        $info{status} = $2;
    }
    push(@arr, \%info);
    return \@arr;
} ## end sub disk_online
## @name ::disk_sanitize_start
##
## @summary Parses the raw output from the "disk sanitize start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_sanitize_start with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_sanitize_start(%args);
## INPUTS TO METHOD CALL 'disk_sanitize_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-j' </td> <td> 'fast-sanitize' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## <tr> <td> '-p' </td> <td> 'pattern' </td> <td> '0' </td> </tr>
## <tr> <td> '-r' </td> <td> 'random-pattern' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'sanitize-only' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'cycle-count' </td> <td> '0' </td> </tr>
## <tr> <td> '-g' </td> <td> 'remove-defect-list' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; disk sanitize start -f 0b.39; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;The disk sanitization process has been initiated.  You will be notified via the system log when it is complete.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub disk_sanitize_start {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        command       => "disk sanitize start",
        error_message => \$error_message
    );
    my @returnarray;
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    } else {
        return \@returnarray;
    }
} ## end sub disk_sanitize_start
## @name ::disk_sanitize_status
##
## @summary Parses the raw output from the "disk sanitize status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_sanitize_status with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_sanitize_status(%args);
## INPUTS TO METHOD CALL 'disk_sanitize_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> OPTIONS </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-list' </td> <td> "" </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub disk_sanitize_status {
    my %args = @_;

    my $error_message;
    my $resp      = $args{response_object};
    my $output    = $resp->get_processed_output();
    my $command   = $resp->get_actual_command();
    my @arr       = ();
    my %temp_hash = ();
    my $error     = _check_error(
        "raw_output"    => $output,
        "command"       => "disk sanitize status",
        "error_message" => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my @lines = get_lines(
        raw_output      => $output,
        can_throw       => 0,
        response_object => $resp,
    );

    foreach my $line (@lines) {
        my %temp_hash = ();
        if ($line =~ /sanitization\s+for\s+(.*)\s+is\s+(\d+)\s+%\s+complete/)
        {    #sanitization for 1c.00.5 is 2 % complete
            $temp_hash{disk}              = $1;
            $temp_hash{sanitized_percent} = $2;
            if ($2 =~ /100/) {
                $temp_hash{sanitized_status} = "sanitized";
            } else {
                $temp_hash{sanitized_status} = "sanitizing";
            }
        } elsif ($line
            =~ /disk\s+sanitize\s+status:\s+(.*)\s+(not\s+sanitizing\.)/)
        {    #disk sanitize status: 1c.00.1 not sanitizing.
            $temp_hash{disk}             = $1;
            $temp_hash{sanitized_status} = $2;
        }
        push(@arr, \%temp_hash);
    }
    return \@arr;
} ## end sub disk_sanitize_status

## @name ::disk_sanitize_abort
##
## @summary Parses the raw output from the "disk sanitize abort" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_sanitize_abort with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_sanitize_abort(%args);
## INPUTS TO METHOD CALL 'disk_sanitize_abort' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> OPTIONS </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-list' </td> <td> "" </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub disk_sanitize_abort {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @arr;

    my $error = _check_error(
        "raw_output"    => $output,
        "command"       => "disk sanitize abort",
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    my $first_line = "";
    my @lines      = get_lines(
        raw_output      => $output,
        first_line      => $first_line,
        can_throw       => 1,
        response_object => $resp
    );
    my $flag = 0;
    foreach my $line (@lines) {
        if ($line
            =~ /[Ss]anitization\s+abort\s+initiated\.\s+[Dd]o\s+not\s+interrupt\s+drive\s+until\s+abort\s+has\s+completed\./
            )
        { # Sanitization abort initiated. Do not interrupt drive until abort has completed.
            $flag = 1;
        }
    }
    if ($flag) {
        return [];
    }
} ## end sub disk_sanitize_abort

## @name ::disk_encrypt_show
##
## @summary Parses the raw output from the "disk encrypt show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_encrypt_show with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_encrypt_show(%args);
## INPUTS TO METHOD CALL 'disk_encrypt_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-list' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub disk_encrypt_show {
    my %args = @_;
    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        "command"     => $command,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub disk_encrypt_show

## @name ::disk_encrypt_lock
##
## @summary Parses the raw output from the "disk encrypt lock" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_encrypt_lock with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_encrypt_lock(%args);
## INPUTS TO METHOD CALL 'disk_encrypt_lock' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-list' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub disk_encrypt_lock {
    my %args = @_;
    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        "command"     => $command,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub disk_encrypt_lock

## @name ::disk_encrypt_unlock
##
## @summary Parses the raw output from the "disk encrypt unlock" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_encrypt_unlock with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_encrypt_unlock(%args);
## INPUTS TO METHOD CALL 'disk_encrypt_unlock' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-list' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub disk_encrypt_unlock {
    my %args = @_;
    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        "command"     => $command,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub disk_encrypt_unlock

## @name ::disk_encrypt_rekey
##
## @summary Parses the raw output from the "disk encrypt rekey" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_encrypt_rekey with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_encrypt_rekey(%args);
## INPUTS TO METHOD CALL 'disk_encrypt_rekey' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-list' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'new-key-id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub disk_encrypt_rekey {
    my %args = @_;
    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        "command"     => $command,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub disk_encrypt_rekey

## @name ::disk_encrypt_sanitize
##
## @summary Parses the raw output from the "disk encrypt sanitize" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_encrypt_sanitize with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_encrypt_sanitize(%args);
## INPUTS TO METHOD CALL 'disk_encrypt_sanitize' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-all' </td> <td> 'emergency-sanitize-all-disks' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub disk_encrypt_sanitize {
    my %args = @_;
    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        "command"     => $command,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub disk_encrypt_sanitize

## @name ::disk_encrypt_destroy
##
## @summary Parses the raw output from the "disk encrypt destroy" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_encrypt_destroy with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_encrypt_destroy(%args);
## INPUTS TO METHOD CALL 'disk_encrypt_destroy' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk-list' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub disk_encrypt_destroy {
    my %args = @_;
    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        "command"     => $command,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub disk_encrypt_destroy

## @name ::disk_reassign
##
## @summary Parses the raw output from the "disk reassign" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_reassign with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_reassign(%args);
## INPUTS TO METHOD CALL 'disk_reassign' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-o' </td> <td> 'owner-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'new-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'sysid' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'new-sysid' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = disk reassign -s 4074497043 -d 4074541060
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub disk_reassign {
    my %args = @_;
    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        "command"     => $command,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    return [];
} ## end sub disk_reassign

## @name ::disk_partition
##
## @summary Parses the raw output from the "disk partition" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_partition with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_partition(%args);
## INPUTS TO METHOD CALL 'disk_partition' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-n' </td> <td> 'num-parts' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'pool' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub disk_partition {
    my %args = @_;
    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        "command"     => $command,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
}

## @name ::disk_unpartition
##
## @summary Parses the raw output from the "disk unpartition" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from disk_unpartition with any of its options as described in the
##    corresponding cdef file 'disk.cdefs'
## @synopsis
##      disk_unpartition(%args);
## INPUTS TO METHOD CALL 'disk_unpartition' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'disk' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub disk_unpartition {
    my %args = @_;
    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        "command"     => $command,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
}

1;
