#
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.
# 
## @summary Routines to parse 7Mode hostname commands
## 
## @description
##     This is a library to parse the raw output of 7Mode hostname commands.
##     The function name will be the same as command name with underscores being
##     used instead of spaces.
## 
## @synopsis
##     use hostname;
## 
##    hostname(%args);
## 
## @author ksmahesh@netapp.com, dl-nacl-dev@netapp.com
## 
## @status public
# 
package NACL::APISet::Node::CLI::7Mode::default::hostname;

use Data::Dumper;
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

sub _parse_for_errors {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command=$args{command};
    my $errormsg_ptr=$args{'errormsg_ptr'};
    
    if($raw_output=~/hostname: hostname must be in \/etc\/hosts/) {
        $$errormsg_ptr=$command." contains invalid param value(s)";
        return "InvalidParamValue";
    }elsif ($raw_output=~/hostname: cannot change hostname via rsh/){
        $$errormsg_ptr=$command." command failed";
        return "CommandFailed";
    }
    return undef;        
}

sub _hostname(%){
    my %args = @_;
    my @lines =@{$args{'lines'}};
    my $line = shift(@lines);
    my %hash;
    $hash{"hostname"}=$line;
    my @returnarray;
    push(@returnarray,\%hash);
    return \@returnarray;
}
## @name ::hostname
##
## @summary Parses the raw output from the "hostname" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from hostname with any of its options as described in the 
##    corresponding cdef file 'hostname.cdefs'
## @synopsis
##      hostname(%args);
## INPUTS TO METHOD CALL 'hostname' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'hostname' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub hostname (%){
    my %args = @_;
    
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    
    my $error;
    my $errormsg;
    my $first_line="";
    my $can_throw=0;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }    
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    if (scalar(@lines)==0){
        return [];
    }
    return (_hostname('lines'=>\@lines));
}#end of funtion hostname()
1;
