#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/iftest.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode iftest command output
##
## @description
##    This is a library to parse the raw output of 7Mode iftest command.
##
## @synopsis
##
##    iftest_start(%args);
##    iftest_stop(%args);
##    iftest_link(%args);
##    iftest_status(%args);
##    iftest_startold(%args);
##    iftest_multicast(%args);
##    iftest_pkt_create(%args);
##    iftest_pkt_delete(%args);
##    iftest_pkt_show(%args);
##    iftest_pkt_genmbuf(%args);
##    iftest_pkt_send(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::iftest;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

## @name ::iftest_start
##
## @summary Parses the raw output from the "iftest start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iftest_start with any of its options as described in the
##    corresponding cdef file 'iftest.cdefs'
## @synopsis
##      iftest_start(%args);
## INPUTS TO METHOD CALL 'iftest_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interfaces' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## Returns an empty array reference.

sub iftest_start {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub iftest_start

## @name ::iftest_stop
##
## @summary Parses the raw output from the "iftest stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iftest_stop with any of its options as described in the
##    corresponding cdef file 'iftest.cdefs'
## @synopsis
##      iftest_stop(%args);
## INPUTS TO METHOD CALL 'iftest_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## Returns an empty array reference.

sub iftest_stop {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub iftest_stop

## @name ::iftest_link
##
## @summary Parses the raw output from the "iftest link" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iftest_link with any of its options as described in the
##    corresponding cdef file 'iftest.cdefs'
## @synopsis
##      iftest_link(%args);
## INPUTS TO METHOD CALL 'iftest_link' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interfaces' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'status' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## Returns an empty array reference.

sub iftest_link {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub iftest_link

## @name ::iftest_startold
##
## @summary Parses the raw output from the "iftest startold" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iftest_startold with any of its options as described in the
##    corresponding cdef file 'iftest.cdefs'
## @synopsis
##      iftest_startold(%args);
## INPUTS TO METHOD CALL 'iftest_startold' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interfaces' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## Returns an empty array reference.

sub iftest_startold {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub iftest_startold

## @name ::iftest_status
##
## @summary Parses the raw output from the "iftest status" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iftest_status with any of its options as described in the
##    corresponding cdef file 'iftest.cdefs'
## @synopsis
##      iftest_status(%args);
## INPUTS TO METHOD CALL 'iftest_status' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; iftest status; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Listener #1 has processed 140 MB, 144164 iterations
##&nbsp;Connector #1 has processed 70 MB, 72608 iterations
##&nbsp;Pair #1: 0 data mismatches
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'connector_statistics' => [
##&nbsp;                                        {
##&nbsp;                                          'iterations' => '72608',
##&nbsp;                                          'connector' => '1',
##&nbsp;                                          'has_processed' => '70 MB'
##&nbsp;                                        }
##&nbsp;                                      ],
##&nbsp;            'listener_statistics' => [
##&nbsp;                                       {
##&nbsp;                                         'listener' => '1',
##&nbsp;                                         'iterations' => '144164',
##&nbsp;                                         'has_processed' => '140 MB'
##&nbsp;                                       }
##&nbsp;                                     ],
##&nbsp;            'pair_statistics' => [
##&nbsp;                                   {
##&nbsp;                                     'data_mismatches' => '0',
##&nbsp;                                     'pair' => '1'
##&nbsp;                                   }
##&nbsp;                                 ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iftest_status {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my (  $count, $limit, $limit1, $limit2, $limit3 ) = 0;
    for ( my $i = 0; $i <= $#lines; $i++ ) {

        if ( $lines[$i]
            =~ /(\w\d\w):\s+(\d+)\s+(\w+)\,\s+(\d+)\s+(\w+)\,\s+(\d+)\s+(\w+)\,\s+(\d+)\s+(\w+)/
            ) {
            $result_array[$count]->{interface_statistics}->[$limit]
                ->{interface} = $1;
            $result_array[$count]->{interface_statistics}->[$limit]
                ->{ to_key_name( item => $3 ) } = $2;
            $result_array[$count]->{interface_statistics}->[$limit]
                ->{ to_key_name( item => $5 ) } = $4;
            $result_array[$count]->{interface_statistics}->[$limit]
                ->{ to_key_name( item => $7 ) } = $6;
            $result_array[$count]->{interface_statistics}->[$limit]
                ->{ to_key_name( item => $9 ) } = $8;
            $limit++;
        } elsif ( $lines[$i] =~ /^(\d+)\s+(.*)\,\s+(\d+)\s+(.*)/ ) {
            $result_array[$count]->{ to_key_name( item => $2 ) } = $1;
            $result_array[$count]->{ to_key_name( item => $4 ) } = $3;
        } elsif ( $lines[$i]
            =~ /(Listener)\s+#(\d)\s+(.*)\s+(\d+\s+\w+)\,\s+(\d+)\s+(\w+)/ ) {
            $result_array[$count]->{listener_statistics}->[$limit1]
                ->{ to_key_name( item => $1 ) } = $2;
            $result_array[$count]->{listener_statistics}->[$limit1]
                ->{ to_key_name( item => $3 ) } = $4;
            $result_array[$count]->{listener_statistics}->[$limit1]
                ->{ to_key_name( item => $6 ) } = $5;
            $limit1++;
        } elsif ( $lines[$i]
            =~ /(Connector)\s+#(\d)\s+(.*)\s+(\d+\s+\w+)\,\s+(\d+)\s+(\w+)/ )
        {
            $result_array[$count]->{connector_statistics}->[$limit2]
                ->{ to_key_name( item => $1 ) } = $2;
            $result_array[$count]->{connector_statistics}->[$limit2]
                ->{ to_key_name( item => $3 ) } = $4;
            $result_array[$count]->{connector_statistics}->[$limit2]
                ->{ to_key_name( item => $6 ) } = $5;
            $limit2++;
        } elsif ( $lines[$i] =~ /(\w+)\s+#(\d):\s+(\d+)\s+(\w+\s+\w+)/ ) {
            $result_array[$count]->{pair_statistics}->[$limit3]
                ->{ to_key_name( item => $1 ) } = $2;
            $result_array[$count]->{pair_statistics}->[$limit3]
                ->{ to_key_name( item => $4 ) } = $3;
            $limit3++;
        }
    }
    return \@result_array;
} ## end sub iftest_status

## @name ::iftest_multicast
##
## @summary Parses the raw output from the "iftest multicast" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iftest_multicast with any of its options as described in the
##    corresponding cdef file 'iftest.cdefs'
## @synopsis
##      iftest_multicast(%args);
## INPUTS TO METHOD CALL 'iftest_multicast' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'source-ip' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## Returns an empty array reference.

sub iftest_multicast {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub iftest_multicast

## @name ::iftest_pkt_create
##
## @summary Parses the raw output from the "iftest pkt create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iftest_pkt_create with any of its options as described in the
##    corresponding cdef file 'iftest.cdefs'
## @synopsis
##      iftest_pkt_create(%args);
## INPUTS TO METHOD CALL 'iftest_pkt_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'packet-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-layers' </td> <td> 'layers' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## Returns an empty array reference.

sub iftest_pkt_create {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub iftest_pkt_create

## @name ::iftest_pkt_delete
##
## @summary Parses the raw output from the "iftest pkt delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iftest_pkt_delete with any of its options as described in the
##    corresponding cdef file 'iftest.cdefs'
## @synopsis
##      iftest_pkt_delete(%args);
## INPUTS TO METHOD CALL 'iftest_pkt_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'packet-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## Returns an empty array reference.

sub iftest_pkt_delete {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub iftest_pkt_delete

## @name ::iftest_pkt_show
##
## @summary Parses the raw output from the "iftest pkt show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iftest_pkt_show with any of its options as described in the
##    corresponding cdef file 'iftest.cdefs'
## @synopsis
##      iftest_pkt_show(%args);
## INPUTS TO METHOD CALL 'iftest_pkt_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-verbose' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-packets' </td> <td> 'packets' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'subfield' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'packet-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; iftest pkt show   mypkt4 -verbose; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Packet "mypkt4":
##&nbsp;  -tso		0
##&nbsp;  -tcpcksum	0
##&nbsp;  -tsosegsz	0
##&nbsp;  -vlan		0
##&nbsp;  -mbuf		0x0000000000000000
##&nbsp;
##&nbsp; Layers (84 bytes):
##&nbsp;	tcp	(20 bytes)
##&nbsp;	data	(64 bytes)
##&nbsp;
##&nbsp; Layer Contents:
##&nbsp;  TCP:
##&nbsp;	-sport		0
##&nbsp;	-dport		0
##&nbsp;	-seq		0
##&nbsp;	-acknum		0
##&nbsp;	-off		5
##&nbsp;	-urg		0
##&nbsp;	-ack		0
##&nbsp;	-psh		0
##&nbsp;	-rst		0
##&nbsp;	-syn		0
##&nbsp;	-fin		0
##&nbsp;	-window		0
##&nbsp;	-checksum	0x0000
##&nbsp;	-urgp		0
##&nbsp;  Data:
##&nbsp;	-length		64
##&nbsp;	-pattern	ladder
##&nbsp;	-seed		0x10000003
##&nbsp;	-byte		0x00
##&nbsp;	-moffset	0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'tcpcksum' => '0'
##&nbsp;            'packet_name' => 'mypkt4'
##&nbsp;            'layer_contents' => [
##&nbsp;                                  {
##&nbsp;                                    'data' => {
##&nbsp;                                                'moffset' => '0',
##&nbsp;                                                'byte' => '0x00',
##&nbsp;                                                'pattern' => 'ladder',
##&nbsp;                                                'length' => '64',
##&nbsp;                                                'seed' => '0x10000003'
##&nbsp;                                              },
##&nbsp;                                    'tcp' => {
##&nbsp;                                               'urg' => '0',
##&nbsp;                                               'window' => '0',
##&nbsp;                                               'psh' => '0',
##&nbsp;                                               'off' => '5',
##&nbsp;                                               'rst' => '0',
##&nbsp;                                               'dport' => '0',
##&nbsp;                                               'urgp' => '0',
##&nbsp;                                               'checksum' => '0x0000',
##&nbsp;                                               'ack' => '0',
##&nbsp;                                               'fin' => '0',
##&nbsp;                                               'syn' => '0',
##&nbsp;                                               'acknum' => '0',
##&nbsp;                                               'sport' => '0',
##&nbsp;                                               'seq' => '0'
##&nbsp;                                             }
##&nbsp;                                  }
##&nbsp;                                ],
##&nbsp;            'layers_info' => [
##&nbsp;                               {
##&nbsp;                                 'layers' => '84 bytes',
##&nbsp;                                 'data' => '64 bytes',
##&nbsp;                                 'tcp' => '20 bytes'
##&nbsp;                               }
##&nbsp;                             ],
##&nbsp;            'tso' => '0'
##&nbsp;            'tsosegsz' => '0'
##&nbsp;            'vlan' => '0'
##&nbsp;            'mbuf' => '0x0000000000000000'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; iftest pkt show  -packets; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Packets:
##&nbsp; mypkt10
##&nbsp; mypkt11
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'packets' => [
##&nbsp;                           'mypkt10',
##&nbsp;                           'mypkt11'
##&nbsp;                         ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iftest_pkt_show {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $cmd    = $resp->get_command();
    _check_error( raw_output => $output, response_object => $resp );
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my $c = 0;
    if ( $cmd =~ /-packets/ ) {
        my $head;
        for ( my $i = 1; $i <= $#lines; $i++ ) {
            $head = $lines[0];
            if ( $head =~ s/:// ) { }
            if ( $lines[$i] =~ /(\w+\d*)/ ) {
                $result_array[0]->{ to_key_name( item => $head ) }->[$c] = $1;
            }
            $c++;
        }
    } elsif ( $cmd =~ /-verbose/ ) {
        my $head;
        my $count = -1;
        my $limit = 0;
        for ( my $i = 0; $i <= $#lines; $i++ ) {
            if ( $lines[$i] =~ /(\w+):/ ) {
                $head = $1;
            } elsif ( $lines[$i] =~ /(Layers)\s+\((.*)\):/ ) {
                $head = $1;
                $head =~ s/Layers/layers_info/;
            } elsif ( $lines[$i] =~ /(Packet)\s+\"(\w+)\"\:/ ) {
                $count++;
                $head = $1;
            }
            my @array = split( /\s\s*/, $lines[$i] );
            if ( $#array == 1 ) {
                if ( $head =~ /TCP|Data|Ethernet|IPv4/ ) {
                    if ( $array[0] =~ s/-// ) { }
                    $result_array[$count]->{layer_contents}->[$limit]
                        ->{ to_key_name( item => $head ) }
                        ->{ to_key_name( item => $array[0] ) } = $array[1];
                } elsif ( $head =~ /Packet/ ) {
                    if ( $array[0] =~ s/-// )    { }
                    if ( $array[1] =~ s/"|://g ) { }
                    $result_array[$count]
                        ->{ to_key_name( item => $array[0] ) } = $array[1];
                }
            } elsif ( $lines[$i] =~ /(\w+)\s+\((.*)\)/ ) {
                $result_array[$count]->{ to_key_name( item => $head ) }
                    ->[$limit]->{ to_key_name( item => $1 ) } = $2;
            }
        }
    }
    return \@result_array;
} ## end sub iftest_pkt_show

## @name ::iftest_pkt_genmbuf
##
## @summary Parses the raw output from the "iftest pkt genmbuf" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iftest_pkt_genmbuf with any of its options as described in the
##    corresponding cdef file 'iftest.cdefs'
## @synopsis
##      iftest_pkt_genmbuf(%args);
## INPUTS TO METHOD CALL 'iftest_pkt_genmbuf' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'packet-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## Returns an empty array reference.

sub iftest_pkt_genmbuf {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub iftest_pkt_genmbuf

## @name ::iftest_pkt_send
##
## @summary Parses the raw output from the "iftest pkt send" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iftest_pkt_send with any of its options as described in the
##    corresponding cdef file 'iftest.cdefs'
## @synopsis
##      iftest_pkt_send(%args);
## INPUTS TO METHOD CALL 'iftest_pkt_send' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-netif' </td> <td> 'network-interface' </td> <td> '0' </td> </tr>
## <tr> <td> '-delay' </td> <td> 'delay' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'packet-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-context' </td> <td> 'context' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## Returns an empty array reference.

sub iftest_pkt_send {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub iftest_pkt_send

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my $error_message;
    my $error = undef;
    if ( $raw_output
        =~ /iftest start requires an even number of interface names|iftest status takes no arguments|iftest link requires two arguments.*/
        ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /(Usage:|iftest:|ltmtst:).*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

1;

