#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode log command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode log commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "log" will be log()
##
## @synopsis
##     use NACL::APISet::Node::CLI::7Mode::default::log;
##
## log(%args);
##
##
##
## @author Rahula.M (rahula@netapp.com), nacl-dev (dl-nacl-dev@netapp.com)
##
## @status public
#

package NACL::APISet::Node::CLI::7Mode::default::log;

###################
# Everytime
###################
use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use Data::Dumper;
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub _parse_for_errors {
    my %args = @_;
    
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if($output =~ /usage.+/si) {
        $error = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    }     
    return $error;
}

## @name ::log
##
## @summary Parses the raw output from the "log" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from log with any of its options as described in the 
##    corresponding cdef file 'log.cdefs'
## @synopsis
##      log(%args);
## INPUTS TO METHOD CALL 'log' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'count' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'message' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; log message; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = priv set -q diag; log 10; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub log {
    my %args = @_;
    
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();   
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }  
    
    return [];
}
1;


