#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/mkdir.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse 7Mode mkdir command output
##
## @description
##    This is a library to parse the raw output of 7Mode mkdir command.
##
## @synopsis
##
##    mkdir(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::mkdir;
use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

## @name ::mkdir
##
## @summary Parses the raw output from the "mkdir" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mkdir with any of its options as described in the
##    corresponding cdef file 'mkdir.cdefs'
## @synopsis
##      mkdir(%args);
## INPUTS TO METHOD CALL 'mkdir' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub mkdir {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub mkdir

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error, $error_message ) = undef;
    if ( $raw_output =~ /No such file or directory.*|File exists.*/ ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /Invalid\s+argument.*/ ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

1;

