#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/mkfile.pm#1 $
#
#Copyright (c) 2011-2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode mkfile command output
##
## @description
##    This is a library to parse the raw output of 7Mode mkfile command.
##
## @synopsis
##    use mkfile;
##
##    mkfile(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::mkfile;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $command          = $args{command};
    my $error;
    if ( $raw_output =~ /write failed|file creation failed/ ) {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    } elsif ( $raw_output =~ /(.*): Volume in path not found/ ) {
        $error          = "InvalidParamValue";
        $$error_message = "Command failed due to an invalid path specified";
    } elsif ( $raw_output =~ /[Uu]sage:\s(.*)/ ) {
        $error          = "CommandFailed";
        $$error_message = "Command failed due to wrong usage or dependencies";
    } elsif ($raw_output =~ /Filename too long/ ) {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    }
    return $error;
}

## @name ::mkfile
##
## @summary Parses the raw output from the "mkfile" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from mkfile with any of its options as described in the 
##    corresponding cdef file 'mkfile.cdefs'
## @synopsis
##      mkfile(%args);
## INPUTS TO METHOD CALL 'mkfile' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'step-size-in-blocks' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'filename' </td> </tr> 
## <tr> <td> '-x' </td> <td> 'stream-name' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'filecount' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'size' </td> </tr> 
## <tr> <td> '-z' </td> <td> 'deduplicate-zero-data' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns empty array ref.
## @examples

sub mkfile {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        command => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    } 
    
	# If it has reached this point then we assume that mkfile has 
	# succeeded and we return empty array ref
    return [];
}
1;
