#
# Copyright (c) 2014 NetApp.
# All rights reserved.
#
## @summary Routines to parse node scope metawafl mw_emap commands
##
## @description
##    This is a library to parse the raw output of node scope metawafl mw_emap commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "mw_emap_insert" will be mw_emap_insert()
##
## @synopsis
##    use mw_emap;
##
##    mw_emap_insert(%args);
##    mw_emap_delete(%args);
##    mw_emap_lookup(%args);
##
## @author chenp@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::mw_emap;

use strict;
use warnings;

use List::MoreUtils qw(natatime);

use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

my %method_hash = (

    mw_emap_insert => {
        output_parser_method => '_emap_parser',
    },

    mw_emap_lookup => {
        output_parser_method => '_emap_parser',
        cmd_fail_string      => 'PVBNs:$',
    },

    mw_emap_delete => {
        output_parser_method => '_emap_parser',
    },

);

parser_method_builder(__PACKAGE__, %method_hash);

sub _emap_parser {

    my $args       = shift;
    my $raw_output = $args->{response_object}->get_processed_output;

    my @all_lines = NACL::APISet::Response::CLI::ParserUtils::get_lines(
        'raw_output' => $raw_output,
        'can_throw'  => 1,
    );

    my @array;
    my $iter = natatime 2, @all_lines;
    while (my @lines = $iter->()) {
        my $hash;
        for my $line (@lines) {
            my ($key, $value) = $line =~ /^\s*(.+?):\s*(.*)/;
            if (defined $key) {
                if ($key =~ /pvbns/i) {
                    $value = [split(/\s/, $value)];
                }
                $hash->{
                    NACL::APISet::Response::CLI::ParserUtils::to_key_name(
                        item => $key
                    )
                } = $value;
            }
        }
        push @array, $hash;
    }

    return \@array;
} ## end _emap_parser

1;
