#
# Copyright (c) 2016 NetApp.
# All rights reserved.
#
## @summary Routines to parse node scope metawafl mw_zlog commands
##
## @description
##    This is a library to parse the raw output of node scope metawafl mw_zlog commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "mw_zlog_info" will be zlog info()
##
## @synopsis
##    use mw_zlog;
##
##    mw_zlog_info(%args);
##    mw_zlog_fetch_ents(%args);
##    mw_zlog_drain_ents(%args);
##
## @author jabhijit@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::mw_zlog;

use strict;
use warnings;

use List::MoreUtils qw(natatime);

use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

my %method_hash = (

    mw_zlog_info      => {
        output_parser_method => '_info',
        cmd_pass_string      =>
            'Inst(\s+)EnqIdxOD(\s+)DeqIdxOD(\s+)EnqIdxIC(\s+)DeqIdxIC',
    },

    mw_zlog_fetch_ents => {
        output_parser_method => '_fetch_ents',
        cmd_pass_string      => 'Fetching(\s+)(\d+)(\s+)entries(\s+)from(\s+)instance',
    },

    mw_zlog_drain_ents => {
        cmd_pass_string     => 'Drained(\s+)(\d+)(\s+)entries(\s+)in(\s+)instance|No entries to drain in instance',
    },

);

parser_method_builder(__PACKAGE__, %method_hash);

sub _info {
    my $args       = shift;
    my $raw_output = $args->{response_object}->get_processed_output;
    my @lines = get_lines(
        'raw_output' => $raw_output,
        'can_throw'  => 1,
        'first_line' => 'Inst(\s+)EnqIdxOD(\s+)DeqIdxOD(\s+)EnqIdxIC(\s+)DeqIdxIC',
    );

    my $hash;
    my (@keys, @values);
    for my $line (@lines) {
        if ( $line =~ /^([a-z]+)(\s+)([a-z]+)(\s+)([a-z]+)(\s+)([a-z]+)(\s+)([a-z]+)/i ) {
            @keys = ($1,$3,$5,$7,$9);
            next;
        }

        if ( $line =~ /^(\d+)(\s+)(\d+)(\s+)(\d+)(\s+)(\d+)(\s+)(\d+)$/ ) {
            @values = ($1,$3,$5,$7,$9);
        }
    }

    foreach my $index (0..4) {
        $hash->{ to_key_name( item => $keys[$index] ) } = $values[$index];
    }

    return [$hash];
} ## end sub _info

sub _fetch_ents {

    my $args       = shift;
    my $raw_output = $args->{response_object}->get_processed_output;

    my @lines = get_lines(
        'raw_output' => $raw_output,
        'can_throw'  => 1,
    );

    my @array;
    my @keys = map {
        to_key_name( item => $_ )} split(/\s+/, $lines[1]);
    my @values = @lines[2 .. scalar(@lines)-1];

    foreach my $value (@values) {
        my @each_values = split(/\s+/, $value);

        my %hash = map {$keys[$_] => $each_values[$_] } (0 .. scalar(@each_values)-1);

        push @array, \%hash;
    }

    return \@array;
} ## end _fetch_ents

1;

