# Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/ndmpcopy.pm#1 $
#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode ndmpcopy command
##
## @description
##    This is a library to parse the raw outputs of ndmpcopy command in 7Mode
##    The function name will be the same as command name.
##    For Example, the parser function for "ndmpcopy" will be ndmpcopy(%args)
##
## @synopsis
##    use ndmpcopy;
##
##    ndmpcopy(%args);
##
## @author c-dl-dpg-nb-automation, dl-nacl-dev
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::ndmpcopy;
use strict;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);

## @name ::ndmpcopy
##
## @summary Parses the raw output from the "ndmpcopy" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ndmpcopy with any of its options as described in the 
##    corresponding cdef file 'ndmpcopy.cdefs'
## @synopsis
##      ndmpcopy(%args);
## INPUTS TO METHOD CALL 'ndmpcopy' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'source' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'destination' </td> </tr> 
## <tr> <td> '-exclude' </td> <td> 'exclude-files-dirs-from-backup-path' </td> </tr> 
## <tr> <td> '-dt' </td> <td> 'destination-filer-authentication-type' </td> </tr> 
## <tr> <td> '-p' </td> <td> 'accept-password-interactively' </td> </tr> 
## <tr> <td> '-mcd' </td> <td> 'destination-control-connection-mode' </td> </tr> 
## <tr> <td> '-md' </td> <td> 'data-connection-mode' </td> </tr> 
## <tr> <td> '-l' </td> <td> 'incremental-level' </td> </tr> 
## <tr> <td> '-sa' </td> <td> 'source-filer-authentication' </td> </tr> 
## <tr> <td> '-st' </td> <td> 'source-filer-authentication-type' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'increment-forever' </td> </tr> 
## <tr> <td> '-da' </td> <td> 'destination-filer-authentication' </td> </tr> 
## <tr> <td> '-h' </td> <td> 'help-message' </td> </tr> 
## <tr> <td> '-d' </td> <td> 'debug-mode' </td> </tr> 
## <tr> <td> '-f' </td> <td> 'force' </td> </tr> 
## <tr> <td> '-mcs' </td> <td> 'source-control-connection-mode' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub ndmpcopy {
    my %args = @_;

    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command_line = $response->get_actual_command();

    my $result;
    my $error = _check_error("raw_output"=>$raw_output,
                             "command"=>$command_line,
                             "ptr_error_message"=>\$error_message);

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

    # Now we can parse the output
    $result = _ndmpcopy(raw_output=>$raw_output,command=>$command_line,
                                 response_object=>$response);
    return $result;
}


sub _ndmpcopy {
    my %args = @_;
    my $raw_output = $args{"raw_output"};
    my $command    = $args{"command"};
    my $response   = $args{"response_object"};
    my (%res_hash, @result_array, $help_message);

    if ($command =~ /ndmpcopy\s+?-h/) {
        ($help_message)= ($raw_output=~/(usage:[^#]+?display this message)/i);
        $res_hash{"help_output"} = $help_message;
        return ([\%res_hash]);
    }

    if ($raw_output =~ /Starting copy/i) {

        #Parsing the output for getting overall Ndmpcopy command status
        ($res_hash{"ndpmcopy_status"}) = ($raw_output =~ /(Transfer\s+?.+?)\s+\[/i);

        #Parsing the raw output for getting time taken for Ndmpcopy
        ($res_hash{"ndmpcopy_time_taken"}) = ($raw_output =~ /Transfer\s+?.+?\s+\[\s(.+?)\s\]\n/i);

        #Parsing the raw output for getting dump status on the source side
        ($res_hash{"dump_status"}) = ($raw_output =~ /Notify:\s(dump.+?)\nNdmpcopy:/i);

        #Parsing the raw output for getting dump type on the source side
        ($res_hash{"dump_type"}) = ($raw_output =~ /DUMP:\s+Using\s(.+?)\nNdmpcopy:/i);

        #Parsing the raw output for getting the restore status on the destination side
        ($res_hash{"restore_status"}) = ($raw_output =~ /Notify:\s(restore.+?)\nNdmpcopy:/i);

        #Parsing the raw output for getting the snapshot name
        ($res_hash{"snapshot_name"}) = ($raw_output =~ /DUMP: creating ".*?\.\.\/(.+?)" snapshot/i);

        #Parsing the raw output for getting estiated dump data size
        ($res_hash{"estimated_dump_data"}) = ($raw_output =~ /DUMP: estimated (\d+? KB)/i);

        #Parsing the raw output for getting actual dumped data size
        ($res_hash{"actual_dump_data"}) = ($raw_output =~ /DUMP: (\d+? KB)/i);

    } else {
           NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                        text =>"Output is not in the expected format",
                                         response_object => $response);
    }

    return ([\%res_hash]);
}

sub _check_error {
    my %args = @_;
    my $raw_output = $args{"raw_output"};
    my $command = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error;

    if($raw_output =~ /(Invalid syntax.*)/i) {
        $error = "MissingRequiredParam";
        $$error_message = $1;
    } elsif( ($raw_output =~ /(Ndmpcopy:.*?Error.*)/i) or
           ($raw_output =~ /(Ndmpcopy:.*?failed.*)/i) or
           ($raw_output =~ /(Received interrupt.*)/i) or
           ($raw_output =~ /(Could not create NDMP connection)/i) or
           ($raw_output =~ /(Client authentication request failed)/i) or
           ($raw_output =~ /(Version negotiation failed)/i) or
           ($raw_output =~ /(Authentication failed.*)/i) or
           ($raw_output =~ /(Failed to start.*)/i) or
           ($raw_output =~ /(User requesting abort)/i) or
           ($raw_output =~ /(kmem_zalloc failed)/i) or
           ($raw_output =~ /(Notify:\sTransfer failed)/i) or
           ($raw_output =~ /(Socket connection to source\s.*?failed)/i) or
           ($raw_output =~ /(Issue 'ndmpd on'.+?NDMP request then retry the connection)/i)
        ){
        $error = "CommandFailed";
        $$error_message = $1;
    } elsif( $raw_output =~ /usage:/ && !($command =~ /ndmpcopy\s+?-h/i) ) {
        $error = "CommandFailed";
        $$error_message = "Failure may be because of missing required params or invalid param or invalid param value";
    } else {
        return undef;
    }

    # Remove ctrl-M from error message which is taken from raw output.
    $$error_message =~ s/\cM//g;
    $$error_message =~ s/^$command:\s//;

    return $error;
}

1;
