# $Id$
#
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.
#
## @summary Routines to parse 7Mode nis commands
##
## @description
##     This is a library to parse the raw output of 7Mode nis commands.
##     The function name will be the same as command name with underscores being
##     used instead of spaces.
##
## @synopsis
##    use nis;
##
##    nis_info(%args);
##
## @author ksmahesh@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::nis;

use strict;
use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

sub _remove_null ($)
{
    my $items=shift;
    my $element_count =$#$items;
    for (my $iterate = 0; $iterate <= $element_count; $iterate++) {
        my $non_null = shift (@$items);
        if (length ($non_null)){
            push (@$items, $non_null);
        }
    }
}

sub _nis_info(%){
    my %args=@_;
    my @lines=@{$args{'lines'}};
    my $resp=$args{'response_obj'};
    my @returnarray;
    my %hash;

    if ($lines[0]=~/NIS is not enabled./) {
        $hash{'result'}=$lines[0];
        push(@returnarray,\%hash);
        return \@returnarray;
    }
    if (scalar(@lines)==0) {
        return [];
    }
    my $next_line;
    while ($next_line=shift(@lines)) {
        if ($next_line=~/NIS domain is (.*)/i) {
            $hash{domain}=$1;
        } elsif($next_line=~/group cache (.*)/i) {
            if($1=~/disabled/i){
                $hash{'group_cache'}="disabled";
            } elsif ($1=~/is (.*)/){
                $hash{'group_cache'}=$1;
            }
        } elsif($next_line=~/ip address/i){
             my @headings;
             my %ipdetails;
             my $heading_line=$next_line;
             @headings=split(/\s\s+/,$next_line);
             foreach my $key(@headings){
                $key=to_key_name('item'=>$key);
             }
             my $next_line;
             $next_line=shift(@lines);
             my $dividing_line=$next_line;
             $next_line=shift(@lines);
             my @values;
             @values=split(/\s\s+/,$next_line);
             _remove_null(\@values);
             if ($values[$#values]!~/:.*:/ && $heading_line =~/Last Polled/){
                 $values[$#values+1]= " ";
             }
             if($#headings!=$#values){
                 my $element_count=scalar(@values);
                 for (my $iterate = 0; $iterate <= $element_count; $iterate++){
                     if($values[$iterate]=~/:.*:/) {
                        $values[$iterate-1]=$values[$iterate-1]." ".$values[$iterate];
                        delete $values[$iterate];
                     }
                 }
             }
             _remove_null(\@values);
             if($#headings==$#values){
                 my $element_count = $#values;
                 for (my $iterate = 0; $iterate <= $element_count; $iterate++){
                     $ipdetails{$headings[$iterate]}=$values[$iterate];
                 }
             } else {
                 my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
                 $error->throw(text=>"Unexpected output",response_object => $resp);
             }

            if (($lines[0] !~ /^NIS/) && ($lines[0] =~ /Last Polled/)) {
                $next_line = shift(@lines);
                my ($key,$value)= split(/:/,$next_line);
                $key=to_key_name('item'=>$key);
                $ipdetails{$key}=$value;
                $next_line = shift(@lines);
                ($key,$value)= split(/:/,$next_line);
                $key=to_key_name('item'=>$key);
                $ipdetails{$key}=$value;
             }
             if (($lines[0] !~ /^NIS/) && ($lines[0] !~ /Last Polled/)){
                 unshift (@lines,$dividing_line);
                 unshift (@lines,$heading_line);
            }
             push(@{$hash{ipdetails}},\%ipdetails);
        } elsif($next_line=~/NIS Performance Statistics/){
            my %perfstatistics;
            while($next_line!~/recent/i){
                if ($next_line=~/Number of YP Lookups: (.*)/) {
                    $perfstatistics{'number_of_yplookups'}=$1;
                } elsif ($next_line=~/Total time spent in YP Lookups: (.*)/) {
                    $perfstatistics{'time_spent_in_yplookups'}=$1;
                } elsif ($next_line=~/Number of network re-transmissions: (.*)/){
                    $perfstatistics{'number_of_network_retransmissions'}=$1;
                } elsif ($next_line=~/Minimum time spent in a YP Lookup: (.*)/){
                    $perfstatistics{'min_timespent'}=$1;
                } elsif ($next_line=~/Maximum time spent in a YP Lookup: (.*)/){
                    $perfstatistics{'maxtimespent'}=$1;
                } elsif ($next_line=~/Average time spent in YP Lookups: (.*)/){
                    $perfstatistics{'avgtimespent'}=$1;
                }
                $next_line=shift(@lines);
            }
            unshift(@lines,$next_line);
            $hash{'perfstatistics'}=[\%perfstatistics];
        } elsif($next_line=~/recent lookups/i) {
            my @returnarray1;
            $next_line=shift(@lines);
            while($next_line!~/cache status/i) {
                my %newhash;
                if($next_line=~/\[(.*)\] Lookup time: (.*)Number of network re-transmissions:(.*)/i) {
                    my $lookuptime;
                    $lookuptime=$2;
                    trim('string_ptr'=>\$lookuptime);
                    my $network_retransmission;
                    $network_retransmission=$3;
                    trim('string_ptr'=>\$network_retransmission);
                    $newhash{'lookup_time'}=$lookuptime;
                    $newhash{'network_retransmissions'}=$network_retransmission;
                    push(@returnarray1,\%newhash);
                }
                $next_line=shift(@lines);
            }
            $hash{'recent_lookups'}= \@returnarray1;
            unshift (@lines,$next_line);
            next;
        } elsif($next_line=~/cache status/i) {
            my @fields;
            my %cachestatus;
            $next_line=shift(@lines);
            while ($next_line!~/NIS Slave/i){
                @fields=split(/:/,$next_line);
                $fields[0]=to_key_name('item'=>$fields[0]);
                trim('string_ptr'=>\$fields[1]);
                $cachestatus{$fields[0]}=$fields[1];
                $next_line=shift(@lines);
            }
            unshift(@lines,$next_line);
            $hash{cachestatus}=[\%cachestatus];
            next;
        } elsif($next_line=~/NIS Slave (.*)$/) {
            $hash{nisslave}=$1;
        } else {
            my $error = 'NACL::APISet::Exceptions::UnexpectedOutputException';
            $error->throw(text=>"Unexpected output",response_object => $resp);

        }
    }#end of while loop
    push(@returnarray,\%hash);
    return \@returnarray;
}

sub _parse_for_errors (%){
    my %args=@_;
    my $raw_output=$args{'raw_output'};
    my $command=$args{'command'};
    my $errormsg_ptr=$args{'errormsg_ptr'};
    if($raw_output=~/^Usage:$/) {
        $$errormsg_ptr=$command." failed";
        return "CommandFailed";
    }
    return undef;
}

## @name ::nis_info
##
## @summary Parses the raw output from the "nis info" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from nis_info with any of its options as described in the
##    corresponding cdef file 'nis.cdefs'
## @synopsis
##      nis_info(%args);
## INPUTS TO METHOD CALL 'nis_info' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-z' </td> <td> 'zero' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub nis_info(%){
    my %args=@_;
    my $resp = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $first_line="NIS";
    my $can_throw=0;
    my $error;
    my $errormsg;
    if ($error=_parse_for_errors('raw_output'=>$raw_output,'command'=>$command,'errormsg_ptr'=>\$errormsg)){
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$errormsg,response_object => $resp);
    }
    my @lines=get_lines('raw_output'=>$raw_output,'first_line'=>$first_line,'can_throw'=>$can_throw);
    return (_nis_info('lines'=>\@lines,'response_obj'=>$resp));
}
1;
