# $Id: //depot/prod/test/nacl/lib/NACL/APISet/Node/CLI/7Mode/default/passwd.pm $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode passwd command
##
## @description
##    This is a library to parse the raw output of 7Mode passwd command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "passwd" will be passwd()
##
## @synopsis
##    use passwd;
##
##    passwd(%args);
##
## @author udaykk@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::passwd;

# use strict;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;



sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if ($raw_output =~ /([Uu]sage:\s)/ ) {
        $$error_message = $1;
        $error = "CommandFailed";
    }elsif ($raw_output =~ /(Password.*)/ ) {
        $$error_message = $1;
        $error = "CommandFailed";
    }
    return $error;
}
## @name ::passwd
##
## @summary Parses the raw output from the "passwd" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from passwd with any of its options as described in the 
##    corresponding cdef file 'passwd.cdefs'
## @synopsis
##      passwd(%args);
## INPUTS TO METHOD CALL 'passwd' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub passwd {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my @result_array;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} # end sub passwd1;
1;
