#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary Routines to parse node scope metawafl mw teh commands
##
## @description
##    This is a library to parse the raw output of 7Mode persistent command.
##
## @synopsis
##    persistent_fence_set(%args);
##    persistent_fence_get(%args);
##    persistent_fence_clear(%args);
##
##
## @author rishis@netapp.com
## @status public

package NACL::APISet::Node::CLI::7Mode::default::persistent_fence;

use strict;
use warnings;
use NATE::BaseException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Response::CLI::CannedParsers;
use NACL::APISet::Exceptions::CommandFailedException ':try';

my %method_hash = (
    persistent_fence_set   => {cmd_pass_string => 'Success',},
    persistent_fence_clear => {cmd_pass_string => 'Success',},
    persistent_fence_get   => {
        first_line                 => 'Owner',
        canned_output_parser       => 'key_equal_value',
        cmd_fail_string            => 'No active persistent',
        dont_split_output_by_space => '1'
    },
);

parser_method_builder(__PACKAGE__, %method_hash);
1;

## @name ::persistent_fence_set
##
## @summary Validates the raw output from the "persistent_fence set" command
##
## @description
##    This method can be used to validathe raw output obtained after calling
##    the method from persistent_fence _set with any of its options as described in the
##    corresponding cdef file 'persistent_fence.cdefs'
## @synopsis
##      persistent_fence_set(%args);
## INPUTS TO METHOD CALL 'persistent_fence_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'volume' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'owner' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'caller_id' </td> <td> '0' </td> </tr>
## <tr> <td> '-t' </td> <td> 'timeout' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'suspension_bit' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
##  Null in case of success. Exception in case of failure
## @examples
##&nbsp;Command = persistent_fence set -v flexgroup__0001 -o 1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = []
##&nbsp;

## @name ::persistent_fence_clear
##
## @summary Validates the raw output from the "persistent_fence clear" command
##
## @description
##    This method can be used to validate raw output obtained after calling
##    the method from persistent_fence_clear with any of its options as described in the
##    corresponding cdef file 'persistent_fence.cdefs'
## @synopsis
##      persistent_fence_clear(%args);
## INPUTS TO METHOD CALL 'persistent_fence_clear' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'volume' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'owner' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'caller_id' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
##  Null in case of success. Exception in case of failure
## @examples
##&nbsp;Command = persistent_fence clear -v flexgroup__0001 -o 1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = []
##&nbsp;

## @name ::persistent_fence_get
##
## @summary Validates the raw output from the "persistent_fence get " command
##
## @description
##    This method can be used to validate raw output obtained after calling
##    the method from persistent_fence_get with any of its options as described in the
##    corresponding cdef file 'persistent_fence.cdefs'
## @synopsis
##      persistent_fence_get(%args);
## INPUTS TO METHOD CALL 'persistent_fence_clear' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'volume' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = persistent_fence get -v flexgroup__0001
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp  {
##&nbsp   'caller_id' => '0',
##&nbsp   'owner' => 'Coordinated Snaprestore'
##&nbsp }
##&nbsp];
##&nbsp;
