## @name ::qos_debug.pm
#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved.
# Any other use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Parses the raw output from the "qos_debug" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug .The parser function for "qos_debug" will be qos_debug()
##
## @synopsis
##    use NACL::APISet::Node::CLI::7Mode::default::qos_debug;
##
##   qos_debug_filersio_besteffort(%args);
##   qos_debug_filersio_background(%args);
##   qos_debug_filersio_lookup(%args);
##   qos_debug_filersio_rename(%args);
##   qos_debug_filersio_get(%args);
##   qos_debug_filersio_delete(%args);
##   qos_debug_filersio_set(%args);
##   qos_debug_rbr_show(%args);
##   qos_debug_rbr_enable(%args);
##   qos_debug_rbr_disable(%args);
##   qos_debug_rbr_simulate(%args);
##   qos_debug_rbr_test_discard(%args);
##   qos_debug_qmon_show(%args);
##   qos_debug_qmon_stress(%args);
##   qos_debug_qmon_validate(%args);
##   qos_debug_qmon_ageout(%args);
##   qos_debug_qmon_malloc(%args);
##   qos_debug_qmon_memlimit(%args);
##   qos_debug_qmon_debug_flag(%args);
##   qos_debug_wid_dealer_on(%args);
##   qos_debug_wid_dealer_off(%args);
##   qos_debug_wid_dealer_file(%args);
##   qos_debug_wid_dealer_prov_wids(%args);
##   qos_debug_workload_delete_later_set(%args);
##   qos_debug_workload_delete_later_clear(%args);
##   qos_debug_zero_suppression(%args);
##   qos_debug_wss(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public


package NACL::APISet::Node::CLI::7Mode::default::qos_debug;
use strict;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
## import exceptions
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::InvalidParamValueException;

## @name ::qos_debug_filersio_besteffort
##
## @summary Parses the raw output from the "qos_debug filersio besteffort" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_filersio_besteffort with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_filersio_besteffort(%args);
## INPUTS TO METHOD CALL 'qos_debug_filersio_besteffort' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug filersio besteffort; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub qos_debug_filersio_besteffort {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output   = $response->get_processed_output();
    my $command  = $response->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    return [];
}

## @name ::qos_debug_filersio_background
##
## @summary Parses the raw output from the "qos_debug filersio background" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_filersio_background with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_filersio_background(%args);
## INPUTS TO METHOD CALL 'qos_debug_filersio_background' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug filersio background; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub qos_debug_filersio_background {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output   = $response->get_processed_output();
    my $command  = $response->get_actual_command();

    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    return [];
}

## @name ::qos_debug_filersio_lookup
##
## @summary Parses the raw output from the "qos_debug filersio lookup" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_filersio_lookup with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_filersio_lookup(%args);
## INPUTS TO METHOD CALL 'qos_debug_filersio_lookup' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug filersio lookup; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Workload id is 1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'workload_id' => '1'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub qos_debug_filersio_lookup {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output   = $response->get_processed_output();
    my $command  = $response->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    my @lines = get_lines( raw_output => $output );
    my ( %hash, @result_array );

    foreach my $line (@lines) {
        if ( $line =~ /Workload\s+id\s+is\s+(.*)/ ) {
            $hash{"workload_id"} = $1;
        }
    }
    push( @result_array, \%hash );
    return \@result_array;
}

## @name ::qos_debug_filersio_rename
##
## @summary Parses the raw output from the "qos_debug filersio rename" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from qos_debug_filersio_rename with any of its options as described in the 
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_filersio_rename(%args);
## INPUTS TO METHOD CALL 'qos_debug_filersio_rename' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'workload-name' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug filersio rename newname; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Workload rename not supported for ERBs
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub qos_debug_filersio_rename {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output   = $response->get_processed_output();
    my $command  = $response->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    return [];
}

## @name ::qos_debug_filersio_get
##
## @summary Parses the raw output from the "qos_debug filersio get" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from qos_debug_filersio_get with any of its options as described in the 
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_filersio_get(%args);
## INPUTS TO METHOD CALL 'qos_debug_filersio_get' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug filersio get; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Policy get not supported for ERBs
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub qos_debug_filersio_get {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output   = $response->get_processed_output();
    my $command  = $response->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    return [];
}

## @name ::qos_debug_filersio_delete
##
## @summary Parses the raw output from the "qos_debug filersio delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_filersio_delete with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_filersio_delete(%args);
## INPUTS TO METHOD CALL 'qos_debug_filersio_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug filersio delete; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub qos_debug_filersio_delete {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output   = $response->get_processed_output();
    my $command  = $response->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    return [];
}

## @name ::qos_debug_filersio_set
##
## @summary Parses the raw output from the "qos_debug filersio set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_filersio_set with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_filersio_set(%args);
## INPUTS TO METHOD CALL 'qos_debug_filersio_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'operations' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'bytes' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'kbytes' </td> <td> '0' </td> </tr>
## <tr> <td> '--reserve' </td> <td> 'reserve' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'us' </td> <td> '0' </td> </tr>
## <tr> <td> '-latency' </td> <td> 'latency' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'iops' </td> <td> '1' </td> </tr>
## <tr> <td> '--limit' </td> <td> 'limit' </td> <td> '0' </td> </tr>
## <tr> <td> '-units' </td> <td> 'units' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug filersio set -reserve 10 -limit 5 -latency 1 -units bytes; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub qos_debug_filersio_set {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output   = $response->get_processed_output();
    my $command  = $response->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    return [];
}

## @name ::qos_debug_rbr_show
##
## @summary Parses the raw output from the "qos_debug rbr show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_rbr_show with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_rbr_show(%args);
## INPUTS TO METHOD CALL 'qos_debug_rbr_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug rbr show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;		QoS RateBucket Rebalance Finite State Machine info
##&nbsp;	QoS RBR state: Waiting for Ctran Ready
##&nbsp;	QoS RBR action: 0
##&nbsp;	QoS last_event generated to the FSM: no-event
##&nbsp;	Simulate RBR: 0
##&nbsp;	Simulate discard: 0
##&nbsp;	ctran ready events: 0
##&nbsp;	ctran not ready events: 0
##&nbsp;	join errors: 0
##&nbsp;	Events posted: 0
##&nbsp;	Committed proposals: 0
##&nbsp;	Discarded_Busy proposals: 0
##&nbsp;	Discarded proposals: 0
##&nbsp;	Discarded Resync Oid proposals: 0
##&nbsp;	Discarded Local node OOQ: 0
##&nbsp;	Discarded others: 0
##&nbsp;	proposal1 made: 0
##&nbsp;	proposal1 errors: 0
##&nbsp;	proposal2 made: 0
##&nbsp;	proposal2 errors: 0
##&nbsp;	response 1 made: 0
##&nbsp;	response 1 error: 0
##&nbsp;	response 2 made: 0
##&nbsp;	response 1 error: 0
##&nbsp;	prop_done : 0
##&nbsp;	prop_done error: 0
##&nbsp;	leave error: 0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'qos_ratebucket_rebalance_finite_state_machine_info' => {
##&nbsp;                                                                      'discarded_proposals' => '0',
##&nbsp;                                                                      'discarded_resync_oid_proposals' => '0',
##&nbsp;                                                                      'qos_rbr_action' => '0',
##&nbsp;                                                                      'join_errors' => '0',
##&nbsp;                                                                      'ctran_not_ready_events' => '0',
##&nbsp;                                                                      'ctran_ready_events' => '0',
##&nbsp;                                                                      'qos_rbr_state' => 'Waiting for Ctran Ready',
##&nbsp;                                                                      'response_1_error' => '0',
##&nbsp;                                                                      'leave_error' => '0',
##&nbsp;                                                                      'committed_proposals' => '0',
##&nbsp;                                                                      'response_1_made' => '0',
##&nbsp;                                                                      'discarded_local_node_ooq' => '0',
##&nbsp;                                                                      'discarded_busy_proposals' => '0',
##&nbsp;                                                                      'response_2_made' => '0',
##&nbsp;                                                                      'discarded_others' => '0',
##&nbsp;                                                                      'proposal2_made' => '0',
##&nbsp;                                                                      'proposal2_errors' => '0',
##&nbsp;                                                                      'qos_last_event_generated_to_the_fsm' => 'no-event',
##&nbsp;                                                                      'proposal1_made' => '0',
##&nbsp;                                                                      'simulate_discard' => '0',
##&nbsp;                                                                      'events_posted' => '0',
##&nbsp;                                                                      'prop_done_' => '0',
##&nbsp;                                                                      'simulate_rbr' => '0',
##&nbsp;                                                                      'prop_done_error' => '0',
##&nbsp;                                                                      'proposal1_errors' => '0'
##&nbsp;                                                                    }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub qos_debug_rbr_show {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output   = $response->get_processed_output();
    my $command  = $response->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );

    my @lines = get_lines( raw_output => $output );
    my ( %hash, @result_array );
    my ( $line, $key, $value );
    $line = shift(@lines);
    my $main_key = to_key_name( item => $line );
    while (@lines) {
        $line = shift(@lines);
        if ( $line =~ /(.*)\s*[:]+\s*(.*)/ ) {
            ( $key, $value ) = ( $1, $2 );
            $hash{$main_key}{ to_key_name( item => $key ) } = $value;
        }
    }
    push( @result_array, \%hash );
    return \@result_array;
}

## @name ::qos_debug_qmon_show
##
## @summary Parses the raw output from the "qos_debug qmon show" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from qos_debug_qmon_show with any of its options as described in the 
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_qmon_show(%args);
## INPUTS TO METHOD CALL 'qos_debug_qmon_show' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-d' </td> <td> 'detail' </td> <td> '1' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'resource-number' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; qos_debug qmon show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Resource name: CPU
##&nbsp;    Allocated pages (and map): 1
##&nbsp;        00001: 8000000000000000 0000000000000000 0000000000000000 0000000000000000 
##&nbsp;         ...
##&nbsp;
##&nbsp;    Next free page:      2
##&nbsp;    Elms needed:         0
##&nbsp;    Base element size:   480
##&nbsp;    Resource table:      0xffffff8000887000
##&nbsp;    Resource table size: 30200
##&nbsp;    Resource columns:    15100
##&nbsp;    Resource row_type:   1 (rows 2)
##&nbsp;    Free element list:   0xffffff8000edf3e8
##&nbsp;    Free elements:       476
##&nbsp;	Allocated elements:	 36
##&nbsp;    Page mem allocated:  245864
##&nbsp;Resource name: AGGR
##&nbsp;    Allocated pages (and map): 1
##&nbsp;        00001: 8000000000000000 0000000000000000 0000000000000000 0000000000000000 
##&nbsp;         ...
##&nbsp;
##&nbsp;    Next free page:      2
##&nbsp;    Elms needed:         0
##&nbsp;    Base element size:   72
##&nbsp;    Resource table:      0xffffff80008a5000
##&nbsp;    Resource table size: 770100
##&nbsp;    Resource columns:    15100
##&nbsp;    Resource row_type:   2 (rows 51)
##&nbsp;    Free element list:   0xffffff8000f1b2c0
##&nbsp;    Free elements:       333
##&nbsp;	Allocated elements:	 179
##&nbsp;    Page mem allocated:  36968
##&nbsp;Resource name: RT
##&nbsp;    Allocated pages (and map): 1
##&nbsp;        00001: 8000000000000000 0000000000000000 0000000000000000 0000000000000000 
##&nbsp;         ...
##&nbsp;
##&nbsp;    Next free page:      2
##&nbsp;    Elms needed:         0
##&nbsp;    Base element size:   104
##&nbsp;    Resource table:      0xffffff8000b96000
##&nbsp;    Resource table size: 15100
##&nbsp;    Resource columns:    15100
##&nbsp;    Resource row_type:   0 (rows 1)
##&nbsp;    Free element list:   0xffffff8000f222d8
##&nbsp;    Free elements:       506
##&nbsp;	Allocated elements:	 6
##&nbsp;    Page mem allocated:  53352
##&nbsp;Resource name: WAFL
##&nbsp;    Allocated pages (and map): 1
##&nbsp;        00001: 8000000000000000 0000000000000000 0000000000000000 0000000000000000 
##&nbsp;         ...
##&nbsp;
##&nbsp;    Next free page:      2
##&nbsp;    Elms needed:         0
##&nbsp;    Base element size:   104
##&nbsp;    Resource table:      0xffffff8000ba5000
##&nbsp;    Resource table size: 15100
##&nbsp;    Resource columns:    15100
##&nbsp;    Resource row_type:   0 (rows 1)
##&nbsp;    Free element list:   0xffffff8000f30548
##&nbsp;    Free elements:       500
##&nbsp;	Allocated elements:	 12
##&nbsp;    Page mem allocated:  53352
##&nbsp;Resource name: DCS
##&nbsp;    Allocated pages (and map): 1
##&nbsp;        00001: 8000000000000000 0000000000000000 0000000000000000 0000000000000000 
##&nbsp;         ...
##&nbsp;
##&nbsp;    Next free page:      2
##&nbsp;    Elms needed:         0
##&nbsp;    Base element size:   120
##&nbsp;    Resource table:      0xffffff8000bb4000
##&nbsp;    Resource table size: 30200
##&nbsp;    Resource columns:    15100
##&nbsp;    Resource row_type:   1 (rows 2)
##&nbsp;    Free element list:   0xffffff8000f3eb30
##&nbsp;    Free elements:       489
##&nbsp;	Allocated elements:	 23
##&nbsp;    Page mem allocated:  61544
##&nbsp;Resource name: DATA_OBJECT
##&nbsp;    Allocated pages (and map): 1
##&nbsp;        00001: 8000000000000000 0000000000000000 0000000000000000 0000000000000000 
##&nbsp;         ...
##&nbsp;
##&nbsp;    Next free page:      2
##&nbsp;    Elms needed:         0
##&nbsp;    Base element size:   232
##&nbsp;    Resource table:      0xffffff8000bd2000
##&nbsp;    Resource table size: 15100
##&nbsp;    Resource columns:    15100
##&nbsp;    Resource row_type:   0 (rows 1)
##&nbsp;    Free element list:   0xffffff8000f4e320
##&nbsp;    Free elements:       509
##&nbsp;	Allocated elements:	 3
##&nbsp;    Page mem allocated:  118888
##&nbsp;Resource name: POLICY_GROUP
##&nbsp;    Allocated pages (and map): 1
##&nbsp;        00001: 8000000000000000 0000000000000000 0000000000000000 0000000000000000 
##&nbsp;         ...
##&nbsp;
##&nbsp;    Next free page:      2
##&nbsp;    Elms needed:         0
##&nbsp;    Base element size:   152
##&nbsp;    Resource table:      0xffffff8000be1000
##&nbsp;    Resource table size: 8501
##&nbsp;    Resource columns:    8501
##&nbsp;    Resource row_type:   0 (rows 1)
##&nbsp;    Free element list:   0xffffff8000f6c068
##&nbsp;    Free elements:       512
##&nbsp;	Allocated elements:	 0
##&nbsp;    Page mem allocated:  77928
##&nbsp;Resource name: DEF_AGGR
##&nbsp;    Allocated pages (and map): 1
##&nbsp;        00001: 8000000000000000 0000000000000000 0000000000000000 0000000000000000 
##&nbsp;         ...
##&nbsp;
##&nbsp;    Next free page:      2
##&nbsp;    Elms needed:         0
##&nbsp;    Base element size:   96
##&nbsp;    Resource table:      0xffffff8000bea000
##&nbsp;    Resource table size: 770100
##&nbsp;    Resource columns:    15100
##&nbsp;    Resource row_type:   2 (rows 51)
##&nbsp;    Free element list:   0xffffff8000f83e48
##&nbsp;    Free elements:       347
##&nbsp;	Allocated elements:	 165
##&nbsp;    Page mem allocated:  49256
##&nbsp;Total elements allocated: 424
##&nbsp;Total memory allocated: 697152 bytes
##&nbsp;MAX PAGES     = 32768 (15 bits)
##&nbsp;MAX ELMS/PAGE = 512 (9 bits)
##&nbsp;QMON_INDEX_MAX = 512
##&nbsp;QMON_PAGE_HEADER_SIZE = 104
##&nbsp;QOS_WID_CACHE_RACE = 0
##&nbsp;Total elements aged out: = 0
##&nbsp;Element aging admin state: = ENABLED
##&nbsp;Total elements moved: = 0
##&nbsp;Total Physical Memory: 0 MB (5119 MB)
##&nbsp;Total ONTAP Memory: 4292 MB
##&nbsp;Total freeBSD Memory: 0 MB (4725 MB)
##&nbsp;Calculated ONTAP Memory: 4608 MB
##&nbsp;Qmon Memory Threshold: 230 MB
##&nbsp;Qmon ONTAP Memory Usage: 0 MB
##&nbsp;Max CPUs: 2
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'total_elements_moved' => '0',
##&nbsp;            'total_ontap_memory' => '4292 MB',
##&nbsp;            'aggr' => {
##&nbsp;                        'resource_columns' => '15100',
##&nbsp;                        'base_element_size' => '72',
##&nbsp;                        'free_element_list' => '0xffffff8000f1b2c0',
##&nbsp;                        'elms_needed' => '0',
##&nbsp;                        'page_mem_allocated' => '36968',
##&nbsp;                        '00001' => '8000000000000000 0000000000000000 0000000000000000 0000000000000000',
##&nbsp;                        'allocated_pages_and_map' => '1',
##&nbsp;                        'next_free_page' => '2',
##&nbsp;                        'resource_table_size' => '770100',
##&nbsp;                        'free_elements' => '333',
##&nbsp;                        'allocated_elements' => '179',
##&nbsp;                        'resource_row_type' => '2 (rows 51)',
##&nbsp;                        'resource_table' => '0xffffff80008a5000'
##&nbsp;                      },
##&nbsp;            'total_elements_aged_out' => '0',
##&nbsp;            'dcs' => {
##&nbsp;                       'resource_columns' => '15100',
##&nbsp;                       'base_element_size' => '120',
##&nbsp;                       'free_element_list' => '0xffffff8000f3eb30',
##&nbsp;                       'elms_needed' => '0',
##&nbsp;                       'page_mem_allocated' => '61544',
##&nbsp;                       '00001' => '8000000000000000 0000000000000000 0000000000000000 0000000000000000',
##&nbsp;                       'allocated_pages_and_map' => '1',
##&nbsp;                       'next_free_page' => '2',
##&nbsp;                       'resource_table_size' => '30200',
##&nbsp;                       'free_elements' => '489',
##&nbsp;                       'allocated_elements' => '23',
##&nbsp;                       'resource_row_type' => '1 (rows 2)',
##&nbsp;                       'resource_table' => '0xffffff8000bb4000'
##&nbsp;                     },
##&nbsp;            'total_memory_allocated' => '697152 bytes',
##&nbsp;            'rt' => {
##&nbsp;                      'resource_columns' => '15100',
##&nbsp;                      'base_element_size' => '104',
##&nbsp;                      'free_element_list' => '0xffffff8000f222d8',
##&nbsp;                      'elms_needed' => '0',
##&nbsp;                      'page_mem_allocated' => '53352',
##&nbsp;                      '00001' => '8000000000000000 0000000000000000 0000000000000000 0000000000000000',
##&nbsp;                      'allocated_pages_and_map' => '1',
##&nbsp;                      'next_free_page' => '2',
##&nbsp;                      'resource_table_size' => '15100',
##&nbsp;                      'free_elements' => '506',
##&nbsp;                      'allocated_elements' => '6',
##&nbsp;                      'resource_row_type' => '0 (rows 1)',
##&nbsp;                      'resource_table' => '0xffffff8000b96000'
##&nbsp;                    },
##&nbsp;            'total_elements_allocated' => '424',
##&nbsp;            'total_physical_memory' => '0 MB (5119 MB)',
##&nbsp;            'def_aggr' => {
##&nbsp;                            'resource_columns' => '15100',
##&nbsp;                            'base_element_size' => '96',
##&nbsp;                            'free_element_list' => '0xffffff8000f83e48',
##&nbsp;                            'elms_needed' => '0',
##&nbsp;                            'page_mem_allocated' => '49256',
##&nbsp;                            '00001' => '8000000000000000 0000000000000000 0000000000000000 0000000000000000',
##&nbsp;                            'allocated_pages_and_map' => '1',
##&nbsp;                            'next_free_page' => '2',
##&nbsp;                            'resource_table_size' => '770100',
##&nbsp;                            'free_elements' => '347',
##&nbsp;                            'allocated_elements' => '165',
##&nbsp;                            'resource_row_type' => '2 (rows 51)',
##&nbsp;                            'resource_table' => '0xffffff8000bea000'
##&nbsp;                          },
##&nbsp;            'cpu' => {
##&nbsp;                       'resource_columns' => '15100',
##&nbsp;                       'base_element_size' => '480',
##&nbsp;                       'free_element_list' => '0xffffff8000edf3e8',
##&nbsp;                       'elms_needed' => '0',
##&nbsp;                       'page_mem_allocated' => '245864',
##&nbsp;                       '00001' => '8000000000000000 0000000000000000 0000000000000000 0000000000000000',
##&nbsp;                       'allocated_pages_and_map' => '1',
##&nbsp;                       'next_free_page' => '2',
##&nbsp;                       'resource_table_size' => '30200',
##&nbsp;                       'free_elements' => '476',
##&nbsp;                       'allocated_elements' => '36',
##&nbsp;                       'resource_row_type' => '1 (rows 2)',
##&nbsp;                       'resource_table' => '0xffffff8000887000'
##&nbsp;                     },
##&nbsp;            'data_object' => {
##&nbsp;                               'resource_columns' => '15100',
##&nbsp;                               'base_element_size' => '232',
##&nbsp;                               'free_element_list' => '0xffffff8000f4e320',
##&nbsp;                               'elms_needed' => '0',
##&nbsp;                               'page_mem_allocated' => '118888',
##&nbsp;                               '00001' => '8000000000000000 0000000000000000 0000000000000000 0000000000000000',
##&nbsp;                               'allocated_pages_and_map' => '1',
##&nbsp;                               'next_free_page' => '2',
##&nbsp;                               'resource_table_size' => '15100',
##&nbsp;                               'free_elements' => '509',
##&nbsp;                               'allocated_elements' => '3',
##&nbsp;                               'resource_row_type' => '0 (rows 1)',
##&nbsp;                               'resource_table' => '0xffffff8000bd2000'
##&nbsp;                             },
##&nbsp;            'policy_group' => {
##&nbsp;                                'resource_columns' => '8501',
##&nbsp;                                'base_element_size' => '152',
##&nbsp;                                'free_element_list' => '0xffffff8000f6c068',
##&nbsp;                                'elms_needed' => '0',
##&nbsp;                                'page_mem_allocated' => '77928',
##&nbsp;                                '00001' => '8000000000000000 0000000000000000 0000000000000000 0000000000000000',
##&nbsp;                                'allocated_pages_and_map' => '1',
##&nbsp;                                'next_free_page' => '2',
##&nbsp;                                'resource_table_size' => '8501',
##&nbsp;                                'free_elements' => '512',
##&nbsp;                                'allocated_elements' => '0',
##&nbsp;                                'resource_row_type' => '0 (rows 1)',
##&nbsp;                                'resource_table' => '0xffffff8000be1000'
##&nbsp;                              },
##&nbsp;            'wafl' => {
##&nbsp;                        'resource_columns' => '15100',
##&nbsp;                        'base_element_size' => '104',
##&nbsp;                        'free_element_list' => '0xffffff8000f30548',
##&nbsp;                        'elms_needed' => '0',
##&nbsp;                        'page_mem_allocated' => '53352',
##&nbsp;                        '00001' => '8000000000000000 0000000000000000 0000000000000000 0000000000000000',
##&nbsp;                        'allocated_pages_and_map' => '1',
##&nbsp;                        'next_free_page' => '2',
##&nbsp;                        'resource_table_size' => '15100',
##&nbsp;                        'free_elements' => '500',
##&nbsp;                        'allocated_elements' => '12',
##&nbsp;                        'resource_row_type' => '0 (rows 1)',
##&nbsp;                        'resource_table' => '0xffffff8000ba5000'
##&nbsp;                      },
##&nbsp;            'qmon_ontap_memory_usage' => '0 MB',
##&nbsp;            'element_aging_admin_state' => 'ENABLED',
##&nbsp;            'calculated_ontap_memory' => '4608 MB',
##&nbsp;            'qmon_memory_threshold' => '230 MB',
##&nbsp;            'total_freebsd_memory' => '0 MB (4725 MB)'
##&nbsp;          }
##&nbsp;        ];


sub qos_debug_qmon_show {
    my %args     = @_;
    my $response = $args{response_object};
    my $output   = $response->get_processed_output();
    my $command  = $response->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    my @lines = get_lines( raw_output => $output );
    my ( %hash, @result_array );
    my ( $line, $key, $main_key, $value );
    $line = shift(@lines);

    while (@lines) {
        if ( $line =~ /Resource\s+name\s*:(.*)/i ) {
            $main_key = $1;
            $main_key =~ s/^\s+//;
            $line = shift(@lines);
            while (( $line !~ /Resource\s+name\s*[:]+(.*)/i )
                && ( $line !~ /Total\s+elements\s+Allocated/i ) )
            {
                if ( $line =~ /(.*)\s*[:]+\s*(.*)$/ ) {
                    my ( $key, $value ) = ( $1, $2 );
                    $key =~ s/^\s+//;
                    $key =~ s/\s+\(/_/g;
                    $key =~ s/\s+/_/;
                    $key =~ s/\)//g;
                    $hash{ to_key_name( item => $main_key ) }
                      { to_key_name( item => $key ) } = $value;
                }
                $line = shift(@lines);
            }
            if ( $line =~ /Total\s+elements\s+Allocated/i ) {
                while (@lines) {
                    if ( $line =~ /(.*)\s*[:]+\s*(.*)$/ ) {
                        my ( $key, $value ) = ( $1, $2 );
                        $key   =~ s/^\s+//;
                        $key   =~ s/\s+/_/;
                        $key   =~ s/\(/_/g;
                        $key   =~ s/\)//g;
                        $value =~ s/=//g;
                        $value =~ s/^\s+//;
                        $hash{ to_key_name( item => $key ) } = $value;
                    }
                    $line = shift(@lines);
                }
            }
        }

    }
    push( @result_array, \%hash );
    return \@result_array;
}

## @name ::qos_debug_qmon_stress
##
## @summary Parses the raw output from the "qos_debug qmon stress" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_qmon_stress with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_qmon_stress(%args);
## INPUTS TO METHOD CALL 'qos_debug_qmon_stress' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'resource number' </td> <td> 'resource' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'age' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug qmon stress 1 10; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub qos_debug_qmon_stress {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output   = $response->get_processed_output();
    my $command  = $response->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    return [];
}

## @name ::qos_debug_qmon_validate
##
## @summary Parses the raw output from the "qos_debug qmon validate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_qmon_validate with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_qmon_validate(%args);
## INPUTS TO METHOD CALL 'qos_debug_qmon_validate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'resource number' </td> <td> 'resource' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug qmon validate freelist 1; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;passed!
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'result' => 'passed!'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub qos_debug_qmon_validate {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output   = $response->get_processed_output();
    my $command  = $response->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    my @lines = get_lines( raw_output => $output );
    my ( %hash, @result_array );
    my ( $line, $value );

    foreach $line (@lines) {
        if ( $line =~ /(.*)/ ) {
            $value = $1;
            $hash{'result'} = $value;
        }
    }
    push( @result_array, \%hash );
    return \@result_array;
}

## @name ::qos_debug_qmon_malloc
##
## @summary Parses the raw output from the "qos_debug qmon malloc" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_qmon_malloc with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_qmon_malloc(%args);
## INPUTS TO METHOD CALL 'qos_debug_qmon_malloc' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'option1' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug qmon malloc  on; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub qos_debug_qmon_malloc {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output   = $response->get_processed_output();
    my $command  = $response->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    return [];
}

## @name ::qos_debug_qmon_ageout
##
## @summary Parses the raw output from the "qos_debug qmon ageout" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_qmon_ageout with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_qmon_ageout(%args);
## INPUTS TO METHOD CALL 'qos_debug_qmon_ageout' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'resource number' </td> <td> 'resource' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'option1' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'sparse' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug qmon ageout; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub qos_debug_qmon_ageout {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output   = $response->get_processed_output();
    my $command  = $response->get_actual_command();

    my $result;
    my $error = _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    return [];
}

## @name ::qos_debug_qmon_memlimit
##
## @summary Parses the raw output from the "qos_debug qmon memlimit" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_qmon_memlimit with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_qmon_memlimit(%args);
## INPUTS TO METHOD CALL 'qos_debug_qmon_memlimit' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'bytes' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug qmon memlimit 1; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Setting memory limit to 1 bytes
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'result' => 'Setting memory limit to 1 bytes'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub qos_debug_qmon_memlimit {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $command  = $response->get_actual_command();
    my $output   = $response->get_processed_output();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    my @lines = get_lines( raw_output => $output );
    my ( %hash, @result_array );
    my ( $line, $value );

    foreach $line (@lines) {
        if ( $line =~ /(.*)/ ) {
            $value = $1;
            $hash{'result'} = $value;
        }
    }
    push( @result_array, \%hash );
    return \@result_array;
}    ##

## @name ::qos_debug_qmon_debug_flag
##
## @summary Parses the raw output from the "qos_debug qmon debug_flag" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_qmon_debug_flag with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_qmon_debug_flag(%args);
## INPUTS TO METHOD CALL 'qos_debug_qmon_debug_flag' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'flag' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug qmon debug_flag; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;qmon_debug_flag = 0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'qmon_debug_flag' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub qos_debug_qmon_debug_flag {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output   = $response->get_processed_output();
    my $command  = $response->get_actual_command();
    my $result;

    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    my @lines = get_lines( raw_output => $output );
    my ( %hash, @result_array );
    my ( $line, $value );

    foreach $line (@lines) {
        if ( $line =~ /(.*)\s+[=]+\s+(.*)/ ) {
            my ( $key, $value ) = ( $1, $2 );
            $hash{ to_key_name( item => $key ) } = $value;
        }
    }
    push( @result_array, \%hash );
    return \@result_array;
}

## @name ::qos_debug_wid_dealer_off
##
## @summary Parses the raw output from the "qos_debug wid_dealer off" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from qos_debug_wid_dealer_off with any of its options as described in the 
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_wid_dealer_off(%args);
## INPUTS TO METHOD CALL 'qos_debug_wid_dealer_off' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug wid_dealer off; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;WID distribution off
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'result' => 'WID distribution off'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub qos_debug_wid_dealer_off {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output   = $response->get_processed_output();
    my $command  = $response->get_actual_command();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    my @lines = get_lines( raw_output => $output );
    my ( %hash, @result_array );
    my ( $line, $value );

    foreach $line (@lines) {
        if ( $line =~ /(.*)/ ) {
            $value = $1;
            $hash{'result'} = $value;
        }
    }
    push( @result_array, \%hash );
    return \@result_array;
}

## @name ::qos_debug_wid_dealer_on
##
## @summary Parses the raw output from the "qos_debug wid_dealer on" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_wid_dealer_on with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_wid_dealer_on(%args);
## INPUTS TO METHOD CALL 'qos_debug_wid_dealer_on' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'pattern' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug wid_dealer on gauss; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Gaussian WID distribution on
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'result' => 'Gaussian WID distribution on'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub qos_debug_wid_dealer_on {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $command  = $response->get_actual_command();
    my $output   = $response->get_processed_output();
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    my @lines = get_lines( raw_output => $output );
    my ( %hash, @result_array );
    my ( $line, $value );

    foreach $line (@lines) {
        if ( $line =~ /(.*)/ ) {
            $value = $1;
            $hash{'result'} = $value;
        }
    }
    push( @result_array, \%hash );
    return \@result_array;
}    ##

## @name ::qos_debug_wid_dealer_file
##
## @summary Parses the raw output from the "qos_debug wid_dealer file" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_wid_dealer_file with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_wid_dealer_file(%args);
## INPUTS TO METHOD CALL 'qos_debug_wid_dealer_file' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'file' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub qos_debug_wid_dealer_file {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $command  = $response->get_actual_command();
    my $output   = $response->get_processed_output();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    return [];
}

## @name ::qos_debug_workload_delete_later_set
##
## @summary Parses the raw output from the "qos_debug workload_delete_later set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_workload_delete_later_set with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_workload_delete_later_set(%args);
## INPUTS TO METHOD CALL 'qos_debug_workload_delete_later_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug workload_delete_later set; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub qos_debug_workload_delete_later_set {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $command  = $response->get_actual_command();
    my $output   = $response->get_processed_output();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    return [];
}

## @name ::qos_debug_workload_delete_later_clear
##
## @summary Parses the raw output from the "qos_debug workload_delete_later clear" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_workload_delete_later_clear with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_workload_delete_later_clear(%args);
## INPUTS TO METHOD CALL 'qos_debug_workload_delete_later_clear' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug workload_delete_later clear; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub qos_debug_workload_delete_later_clear {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $command  = $response->get_actual_command();
    my $output   = $response->get_processed_output();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    return [];
}

## @name ::qos_debug_wid_dealer_prov_wids
##
## @summary Parses the raw output from the "qos_debug wid_dealer prov_wids" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_wid_dealer_prov_wids with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_wid_dealer_prov_wids(%args);
## INPUTS TO METHOD CALL 'qos_debug_wid_dealer_prov_wids' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'pgid' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'workloads' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub qos_debug_wid_dealer_prov_wids {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $command  = $response->get_actual_command();
    my $output   = $response->get_processed_output();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    return [];
}

## @name ::qos_debug_zero_suppression
##
## @summary Parses the raw output from the "qos_debug zero_suppression" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_zero_suppression with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_zero_suppression(%args);
## INPUTS TO METHOD CALL 'qos_debug_zero_suppression' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'show' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'option1' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug zero_suppression on; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;zero_suppression set to ON (31) for all CM objects
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'result' => 'zero_suppression set to ON (31) for all CM objects'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub qos_debug_zero_suppression {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $output   = $response->get_processed_output();
    my $command  = $response->get_actual_command();
    my $result;

    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    my @lines = get_lines( raw_output => $output );
    my ( %hash, @result_array );
    my ( $line, $value );
    foreach my $line (@lines) {
        $hash{'result'} = $line;
    }
    push( @result_array, \%hash );
    return \@result_array;
}

## @name ::qos_debug_rbr_enable
##
## @summary Parses the raw output from the "qos_debug rbr enable" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_rbr_enable with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_rbr_enable(%args);
## INPUTS TO METHOD CALL 'qos_debug_rbr_enable' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug rbr enable; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub qos_debug_rbr_enable {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $command  = $response->get_actual_command();
    my $output   = $response->get_processed_output();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    return [];
}

## @name ::qos_debug_rbr_disable
##
## @summary Parses the raw output from the "qos_debug rbr disable" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_rbr_disable with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_rbr_disable(%args);
## INPUTS TO METHOD CALL 'qos_debug_rbr_disable' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug rbr disable; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub qos_debug_rbr_disable {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $command  = $response->get_actual_command();
    my $output   = $response->get_processed_output();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    return [];
}

## @name ::qos_debug_rbr_simulate
##
## @summary Parses the raw output from the "qos_debug rbr simulate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_rbr_simulate with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_rbr_simulate(%args);
## INPUTS TO METHOD CALL 'qos_debug_rbr_simulate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'simulate' </td> <td> 'simulate-value' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug rbr simulate; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;setting simulate_rbr to 0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'result' => 'setting simulate_rbr to 0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub qos_debug_rbr_simulate {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $command  = $response->get_actual_command();
    my $output   = $response->get_processed_output();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    my @lines = get_lines( raw_output => $output );
    my ( %hash, @result_array );
    my ( $line, $value );

    foreach $line (@lines) {
        if ( $line =~ /(.*)/ ) {
            $value = $1;
            $hash{'result'} = $value;
        }
    }
    push( @result_array, \%hash );
    return \@result_array;
}

## @name ::qos_debug_rbr_test_discard
##
## @summary Parses the raw output from the "qos_debug rbr test_discard" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from qos_debug_rbr_test_discard with any of its options as described in the
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_rbr_test_discard(%args);
## INPUTS TO METHOD CALL 'qos_debug_rbr_test_discard' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'test_discard' </td> <td> 'test-discard-value' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug rbr test_discard; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;setting test_discard to 0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'result' => 'setting test_discard to 0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub qos_debug_rbr_test_discard {
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $command  = $response->get_actual_command();
    my $output   = $response->get_processed_output();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
    my @lines = get_lines( raw_output => $output );
    my ( %hash, @result_array );
    my ( $line, $value );

    foreach $line (@lines) {
        if ( $line =~ /(.*)/ ) {
            $value = $1;
            $hash{'result'} = $value;
        }
    }
    push( @result_array, \%hash );
    return \@result_array;
}

## @name ::qos_debug_wss
##
## @summary Parses the raw output from the "qos_debug wss" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from qos_debug_wss with any of its options as described in the 
##    corresponding cdef file 'qos_debug.cdefs'
## @synopsis
##      qos_debug_wss(%args);
## INPUTS TO METHOD CALL 'qos_debug_wss' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'wid' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; qos_debug wss; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Global
##&nbsp;Time		 Unique				 Total				Re-read%
##&nbsp;----		 ------				 -----				----
##&nbsp; 0 00:01:00.000:   0.000   (        0 blks)	   0.000   (        0 blks)	  0
##&nbsp; 0 00:02:00.000:   0.000   (        0 blks)	   0.000   (        0 blks)	  0
##&nbsp; 0 00:05:00.000:   0.000   (        0 blks)	   0.000   (        0 blks)	  0
##&nbsp; 0 00:10:00.000:   0.000   (        0 blks)	   0.000   (        0 blks)	  0
##&nbsp; 0 01:00:00.000:   0.000   (        0 blks)	   0.000   (        0 blks)	  0
##&nbsp; 0 06:00:00.000:   0.000   (        0 blks)	   0.000   (        0 blks)	  0
##&nbsp; 0 14:19:00.512:   0.000   (        0 blks)	   0.000   (        0 blks)	  0
##&nbsp;Memsize:        8.000 G (  2097152 blks)
##&nbsp;Memory used:    1.220 M (      312 blks)
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'unique' => {
##&nbsp;                          'value' => '0.000',
##&nbsp;                          'blks' => '0 blks'
##&nbsp;                        },
##&nbsp;            're_read_percentage' => '0',
##&nbsp;            'global_time' => '0 00:01:00.000',
##&nbsp;            'total' => {
##&nbsp;                         'value' => '0.000',
##&nbsp;                         'blks' => '0 blks'
##&nbsp;                       }
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'unique' => {
##&nbsp;                          'value' => '0.000',
##&nbsp;                          'blks' => '0 blks'
##&nbsp;                        },
##&nbsp;            're_read_percentage' => '0',
##&nbsp;            'global_time' => '0 00:02:00.000',
##&nbsp;            'total' => {
##&nbsp;                         'value' => '0.000',
##&nbsp;                         'blks' => '0 blks'
##&nbsp;                       }
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'unique' => {
##&nbsp;                          'value' => '0.000',
##&nbsp;                          'blks' => '0 blks'
##&nbsp;                        },
##&nbsp;            're_read_percentage' => '0',
##&nbsp;            'global_time' => '0 00:05:00.000',
##&nbsp;            'total' => {
##&nbsp;                         'value' => '0.000',
##&nbsp;                         'blks' => '0 blks'
##&nbsp;                       }
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'unique' => {
##&nbsp;                          'value' => '0.000',
##&nbsp;                          'blks' => '0 blks'
##&nbsp;                        },
##&nbsp;            're_read_percentage' => '0',
##&nbsp;            'global_time' => '0 00:10:00.000',
##&nbsp;            'total' => {
##&nbsp;                         'value' => '0.000',
##&nbsp;                         'blks' => '0 blks'
##&nbsp;                       }
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'unique' => {
##&nbsp;                          'value' => '0.000',
##&nbsp;                          'blks' => '0 blks'
##&nbsp;                        },
##&nbsp;            're_read_percentage' => '0',
##&nbsp;            'global_time' => '0 01:00:00.000',
##&nbsp;            'total' => {
##&nbsp;                         'value' => '0.000',
##&nbsp;                         'blks' => '0 blks'
##&nbsp;                       }
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'unique' => {
##&nbsp;                          'value' => '0.000',
##&nbsp;                          'blks' => '0 blks'
##&nbsp;                        },
##&nbsp;            're_read_percentage' => '0',
##&nbsp;            'global_time' => '0 06:00:00.000',
##&nbsp;            'total' => {
##&nbsp;                         'value' => '0.000',
##&nbsp;                         'blks' => '0 blks'
##&nbsp;                       }
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'unique' => {
##&nbsp;                          'value' => '0.000',
##&nbsp;                          'blks' => '0 blks'
##&nbsp;                        },
##&nbsp;            're_read_percentage' => '0',
##&nbsp;            'global_time' => '0 14:19:00.512',
##&nbsp;            'total' => {
##&nbsp;                         'value' => '0.000',
##&nbsp;                         'blks' => '0 blks'
##&nbsp;                       }
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'memsize' => {
##&nbsp;                           'blks' => '2097152 blks',
##&nbsp;                           'size' => '8.000 G'
##&nbsp;                         }
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'memory_used' => {
##&nbsp;                               'blks' => '312 blks',
##&nbsp;                               'size' => '1.220 M'
##&nbsp;                             }
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub qos_debug_wss{
    my %args = @_;
    my $error_message;
    my $response = $args{response_object};
    my $command  = $response->get_actual_command();
    my $output   = $response->get_processed_output();
    my $result;
    _check_error(
        "raw_output"      => $output,
        "command"         => $command,
        "response_object" => $response,
    );
	my @lines = get_lines( raw_output => $output );
    my  @result_array ;
    my ( $line, $key, $value );
	while(@lines){
	   my %hash = ();
	   $line = shift(@lines);
	   if($line =~ /\s*(.+)[:]+\s+(.+)\s*[\(]+\s*(.+)[\)]+\s+(.+)\s*[\(]+\s*(.+)[\)]+\s+(.+)/){
	       my ($value1, $value2, $value3, $value4, $value5, $value6 ) = ($1,$2,$3,$4,$5,$6);
		   $value2 =~ s/\s+//;
		   $value4 =~ s/\s+//;
                   $hash{'global_time'}              = $value1 ;
		   $hash{'unique'}{'value'}          = $value2 ;
		   $hash{'unique'}{'blks'}           = $value3 ;
		   $hash{'total'}{'value'}           = $value4 ;
		   $hash{'total'}{'blks'}            = $value5 ;
		   $hash{'re_read_percentage'}       = $value6 ;
	   }elsif($line =~ /\w+[:]+/){
		  my @split_arr  = split ( /:/,$line);
	      $key = $split_arr[0];
		  if($split_arr[1] =~ /(.+)\s+[\(]\s+(.+)[\)]+/){
		    my $value1  = $1;
			my $value2  = $2;
            $value1  =~ s/^\s+//;
		    $hash{to_key_name( item => $key)}{'size'} = $value1;
		    $hash{to_key_name( item => $key)}{'blks'} = $value2;
		  }
	   }
	   push( @result_array, \%hash ) if(%hash);
	}
	return \@result_array;
}
sub _check_error {
    my %args       = @_;
    my $response   = $args{response_object};
    my $raw_output = $args{raw_output};
    my $command    = $args{command};
    my $error      = '';
    if ( $raw_output =~ /^(Could\s*not\s*create )/ ) {
        $error = "InvalidParam";
    }
    elsif ( $raw_output =~ /^(Error\s*-\s*policy\S*group)/ ) {
        $error = "CommandFailed";
    }
    elsif ( $raw_output =~ /^usage/i ) {
        $error = "CommandFailed";
    }
    elsif ( $raw_output =~ /^(Missing|expecting)/i ) {
        $error = "MissingRequiredParam";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $raw_output, response_object => $response );
    }
}
1;
