#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode rdfile command output
##
## @description
##    This is a library to parse the raw output of 7Mode rfdile command.
##
## @synopsis
##    use rdfile;
##
##    rdfile(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::rdfile;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $file          = $args{file};
    my $error;
    if ( $raw_output =~ /[Uu]sage:\s(.*)/ ) {
        $error          = "CommandFailed";
        $$error_message = "Command failed due to wrong usage or dependencies";
    }elsif ( $raw_output =~ /$file\s*:\s*(.*)/ ) {
        $error          = "CommandFailed";
        $$error_message = $1;
    }
    return $error;
}

## @name ::rdfile
##
## @summary Parses the raw output from the "rdfile" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from rdfile with any of its options as described in the 
##    corresponding cdef file 'rdfile.cdefs'
## @synopsis
##      rdfile(%args);
## INPUTS TO METHOD CALL 'rdfile' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'file' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub rdfile {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    $command =~ /rdfile\s+(.+)/;
    my $file = $1;
    my $error;

    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
        file => $file,
    );
    my %hash;
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    } else {
        $hash{file} = $file;
        $hash{contents} = $output;
    }
    @result_array = ( \%hash );
    return \@result_array;
}

1;
