#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/rm.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
### @summary Routine to parse 7Mode rm command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode rm
##    command. The function name will be the same as command name with
##    underscores being used instead of spaces.
##    For Example,
##      the parser function for "rm" will be rm().
##
## @synopsis
##    use rm;
##    rm(%args);
##
## @author kathar.hidayath@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
##

package NACL::APISet::Node::CLI::7Mode::default::rm;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown

use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args = @_;
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if($output =~ /(No such file or directory|Invalid argument)/si) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    }
    return $error;
}

## @name ::rm
##
## @summary Parses the raw output from the "rm" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from rm with any of its options as described in the 
##    corresponding cdef file 'rm.cdefs'
## @synopsis
##      rm(%args);
## INPUTS TO METHOD CALL 'rm' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'paths' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub rm {
    my %args = @_;
    my ($error_message);
    my $resp = $args{response_object};
    my $output = $resp -> get_processed_output();
    my $error = _check_error( raw_output    => $output,
                              command       => "rm",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
}
1;
