#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode rmcores command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode rmcores commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "rmcores" will be rmcores()
##
## @synopsis
##     use NACL::APISet::Node::CLI::7Mode::default::rmcores;
##
##rmcores(%args);
##
##
## @author udaykk@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Node::CLI::7Mode::default::rmcores;

use strict;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if ($raw_output =~ /$command(.*)/) {
        $$error_message = $1;
        if ($raw_output =~ /(No such .*)|( .* could not be found.)/) {
            $error = "InvalidParamValue";
        }
        return $error;
    }
    return undef;
} ## end sub _check_error

sub rmcores {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error = _check_error("raw_output"    => $output,
                             "command"       => $command,
                             "error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
} ## end sub rmcores
1;
