#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/rmdir.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse 7Mode rmdir command output
##
## @description
##    This is a library to parse the raw output of 7Mode rmdir command.
##
## @synopsis
##
##    rmdir(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::rmdir;

use strict;
use warnings;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::CommandFailedException;

## @name ::rmdir
##
## @summary Parses the raw output from the "rmdir" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from rmdir with any of its options as described in the
##    corresponding cdef file 'rmdir.cdefs'
## @synopsis
##      rmdir(%args);
## INPUTS TO METHOD CALL 'rmdir' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'path' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub rmdir {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub rmdir

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    
    my %excp_args = ( text => $raw_output, response_object => $resp );

    if ( $raw_output =~ /No such file or directory.*|Can't open directory.*/ ) {
        NACL::APISet::Exceptions::InvalidParamValueException->throw(%excp_args);
    } elsif ( $raw_output =~ /Invalid\s+argument.*/ ) {
        NACL::APISet::Exceptions::InvalidParamException->throw(%excp_args);
    } elsif ($raw_output =~ /Directory not empty|Undefined error.*/i) {
        NACL::APISet::Exceptions::CommandFailedException->throw(%excp_args);
    }
} ## end sub _check_error

1;

