#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/rshstat.pm# $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode rshstat command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode rshstat command.
##
## @synopsis
##    use rshstat;
##
##
## @author tanil@netapp.com, dl-nacl-dev@netapp.com
##
## @status public

package NACL::APISet::Node::CLI::7Mode::default::rshstat;

use strict;
use Data::Dumper;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

sub _check_error {
    my %args = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{ptr_error_message};
    my $error = undef;
    if($raw_output =~ /rshstat \[-a\|-t\]$/) {
        $error = "InvalidParam";
        $$error_message = $raw_output;
    }
    return $error;
}
## @name ::rshstat
##
## @summary Parses the raw output from the "rshstat" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from rshstat with any of its options as described in the
##    corresponding cdef file 'rshstat.cdefs'
## @synopsis
##      rshstat(%args);
## INPUTS TO METHOD CALL 'rshstat' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-t' </td> <td> 'toggle' </td> </tr>
## <tr> <td> '-a' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; rshstat -a; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'current_active_sessions' => '1',
##&nbsp;            'maximum_available_sessions' => '192',
##&nbsp;            'active_high_sessions' => '2',
##&nbsp;            'session_invocations' => '1162'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; rshstat -t; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'gethost_time' => '381ms',
##&nbsp;            'current_active_sessions' => '1',
##&nbsp;            'connection_time' => '381ms',
##&nbsp;            'maximum_available_sessions' => '192',
##&nbsp;            'active_high_sessions' => '2',
##&nbsp;            'command_time' => '381ms',
##&nbsp;            'session_invocations' => '1163'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub rshstat {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
	my $command = $resp->get_actual_command();
    my $raw_output = $resp->get_processed_output();
    my $error = _check_error("raw_output"        => $raw_output,
                             "command"           => 'rshstat',
                             "ptr_error_message" => \$error_message,
                             );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    my @result_array = ();
    my ($i, $key);
    $i = 0;
    my @lines = get_lines( raw_output => $raw_output,
	                       can_throw => 0, response_object => $resp );
    foreach my $line (@lines) {
        if($line =~ /^([A-Za-z\s0-9]+)\:\s+([a-z0-9]+)$/) {
            $key = $1;
            $result_array[$i] -> {to_key_name(item=>$key)} = $2;
        }
    }
   return \@result_array;
}
1;
