#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/setflag.pm $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
### @summary Routine to parse 7Mode setflag command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode setflag
##    command. The function name will be the same as command name with
##    underscores being used instead of spaces.
##    For Example,
##      the parser function for "setflag" will be setflag().
##
## @synopsis
##    use setflag;
##    setflag(%args);
##
## @author kathar.hidayath@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
##

package NACL::APISet::Node::CLI::7Mode::default::setflag;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args = @_;
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if($output =~ /usage.+/si) {
        $error = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    } elsif($output =~ /(no such flag|invalid integer value|can not be set)/si) {
        $error = "InvalidParamValue";
        $$error_message = $1;
    }
    return $error;
}

## @name ::setflag
##
## @summary Parses the raw output from the "setflag" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from setflag with any of its options as described in the
##    corresponding cdef file 'setflag.cdefs'
## @synopsis
##      setflag(%args);
## INPUTS TO METHOD CALL 'setflag' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'value' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'flag' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = setflag wafl_ignore_pseudobad 100
##&nbsp;$VAR1 = [
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = setflag wafl_ignore_pseudobad 0
##&nbsp;$VAR1 = [
##&nbsp;        ];
##&nbsp;

sub setflag {
    my %args = @_;
    my ($error_message);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error = _check_error( raw_output    => $output,
                              command       => "setflag",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    return [];
}
1;
