#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/setup.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode setup command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode setup commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "setup" will be setup()
##
## @synopsis
##     use NACL::APISet::Node::CLI::7Mode::default::setup;
##
## setup(%args);
##
## @author rahula@netapp.com , dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Node::CLI::7Mode::default::setup;
use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown

use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use Data::Dumper;
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use TCD;
sub _parse_for_errors {
    my %args = @_;
    
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
        if($output=~/Now type 'reboot'.*/){
        $error = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
        }
    return $error;
}
## @name ::setup
##
## @summary Parses the raw output from the "setup" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from setup with any of its options as described in the 
##    corresponding cdef file 'setup.cdefs'
## @synopsis
##      setup(%args);
## INPUTS TO METHOD CALL 'setup' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command =setup
##&nbsp;$VAR1 = [];
##&nbsp;
sub setup {
    my %args = @_;

    my ($error_message);
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}
1;

