#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/showfh.pm $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
### @summary Routine to parse 7Mode showfh command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode showfh
##    command. The function name will be the same as command name with
##    underscores being used instead of spaces.
##    For Example,
##      the parser function for "showfh" will be showfh().
##
## @synopsis
##    use showfh;
##    showfh(%args);
##
## @author kathar.hidayath@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
##

package NACL::APISet::Node::CLI::7Mode::default::showfh;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException;

sub _check_error {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $output =~ /usage.+/si ) {
        $error          = "InvalidParam";
        $$error_message = "Invalid Parameter Passed";
    } elsif ( $output =~ /(.*No such file or directory)/si ) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    }
    return $error;
}

## @name ::showfh
##
## @summary Parses the raw output from the "showfh" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from showfh with any of its options as described in the
##    corresponding cdef file 'showfh.cdefs'
## @synopsis
##      showfh(%args);
## INPUTS TO METHOD CALL 'showfh' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'filename' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;Command = priv set -q diag; showfh /vol/vol0; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'flags' => '0x00',
##&nbsp;            'fileid' => '0x000040',
##&nbsp;            'gen' => '0xffd04fa4',
##&nbsp;            'fsid' => '0xb6638c9f',
##&nbsp;            'snapid' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub showfh {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my ( $error, @temp );
    my $errormsg;
    if ($error = _check_error(
            'raw_output'   => $raw_output,
            'command'      => $command,
            'error_message' => \$errormsg
        )
        )
    {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }
    my @arr = get_lines(
        "raw_output"      => $raw_output,
        "first_line"      => 'flags=',
        "can_throw"       => 1,
        "response_object" => $resp
    );
    @arr = split( /\s+/, $arr[0] );
    foreach my $entry (@arr) {
        if ( $entry =~ /(\w+)=(.+)/ ) {
            $temp[0]->{$1} = $2;
        } else {
            NACL::APISet::Exceptions::UnexpectedOutputException->throw(
                "'$entry' is not in the expected format");
        }
    }

    return \@temp;
}    # end of function showfh
1;
