#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/spinnp_replay_stats.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode spinnp_replay_stats command output
##
## @description
##    This is a library to parse the raw output of 7Mode spinnp_replay_stats command.
##
## @synopsis
##    use spinnp_replay_stats;
##
##    spinnp_replay_stats(%args);
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::spinnp_replay_stats;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

## @name ::spinnp_replay_stats
##
## @summary Parses the raw output from the "spinnp_replay_stats" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from spinnp_replay_stats with any of its options as described in the
##    corresponding cdef file 'spinnp_replay_stats.cdefs'
## @synopsis
##      spinnp_replay_stats(%args);
## INPUTS TO METHOD CALL 'spinnp_replay_stats' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; spinnp_replay_stats; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Handles in use: 0
##&nbsp;LRU length: 0
##&nbsp;Bins in use: 0
##&nbsp;Max Bin allowed: 2340
##&nbsp;Total buckets in use: 0
##&nbsp;Max LRU buckets allowed: 160
##&nbsp;Total create bin calls: 0
##&nbsp;Total release bin calls: 0
##&nbsp;Total remove cache calls: 0
##&nbsp;Total find bucket calls: 0
##&nbsp;Total fill bucket calls: 0
##&nbsp;Total release handle calls: 0
##&nbsp;Total make space for bin from empty lru: 0
##&nbsp;Total make space for bin from expired bin: 0
##&nbsp;Total make space for bin from oldest: 0
##&nbsp;Failures to alloc bin: 0
##&nbsp;Failures to make space for bin: 0
##&nbsp;Failures to lookup bin: 0
##&nbsp;Failures to lookup bucket: 0
##&nbsp;Failures to allocate a handle: 0
##&nbsp;Failures to allocate a bucket: 0
##&nbsp;Failures to steal from lru chain: 0
##&nbsp;In progress buckets found: 0
##&nbsp;Replayed ops: 0
##&nbsp;LRU allocs taking expired bucket: 0
##&nbsp;LRU allocs taking over min bucket: 0
##&nbsp;LRU allocs due to over max buckets: 0
##&nbsp;Total times need to steal a chunk: 0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'failures_to_allocate_a_handle' => '0',
##&nbsp;            'max_lru_buckets_allowed' => '160',
##&nbsp;            'lru_allocs_due_to_over_max_buckets' => '0',
##&nbsp;            'total_make_space_for_bin_from_oldest' => '0',
##&nbsp;            'bins_in_use' => '0',
##&nbsp;            'total_make_space_for_bin_from_empty_lru' => '0',
##&nbsp;            'lru_allocs_taking_over_min_bucket' => '0',
##&nbsp;            'total_find_bucket_calls' => '0',
##&nbsp;            'total_remove_cache_calls' => '0',
##&nbsp;            'total_make_space_for_bin_from_expired_bin' => '0',
##&nbsp;            'total_create_bin_calls' => '0',
##&nbsp;            'total_release_bin_calls' => '0',
##&nbsp;            'max_bin_allowed' => '2340',
##&nbsp;            'handles_in_use' => '0',
##&nbsp;            'total_times_need_to_steal_a_chunk' => '0',
##&nbsp;            'failures_to_steal_from_lru_chain' => '0',
##&nbsp;            'replayed_ops' => '0',
##&nbsp;            'total_fill_bucket_calls' => '0',
##&nbsp;            'failures_to_allocate_a_bucket' => '0',
##&nbsp;            'lru_allocs_taking_expired_bucket' => '0',
##&nbsp;            'in_progress_buckets_found' => '0',
##&nbsp;            'failures_to_lookup_bin' => '0',
##&nbsp;            'total_release_handle_calls' => '0',
##&nbsp;            'lru_length' => '0',
##&nbsp;            'total_buckets_in_use' => '0',
##&nbsp;            'failures_to_alloc_bin' => '0',
##&nbsp;            'failures_to_lookup_bucket' => '0',
##&nbsp;            'failures_to_make_space_for_bin' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub spinnp_replay_stats {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my @result_array;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    for ( my $i = 0; $i <= $#lines; $i++ ) {
        my @values = split( /:\s*/, $lines[$i] );
        $result_array[0]->{ to_key_name( item => $values[0] ) } = $values[1];
    }
    return \@result_array;
} ## end sub spinnp_replay_stats

1;
