#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/sync.pm $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
### @summary Routine to parse 7Mode sync command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode sync
##    command. The function name will be the same as command name with
##    underscores being used instead of spaces.
##    For Example,
##      the parser function for "sync" will be sync().
##
## @synopsis
##    use sync;
##    sync(%args);
##
## @author kathar.hidayath@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
##

package NACL::APISet::Node::CLI::7Mode::default::sync;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my %args = @_;
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if($output =~ /usage: sync/si) {
        $error = "InvalidParam";
        $$error_message = "Invalid Options or "
                         ."Options are available only in test privileage";
    }
    return $error;
}

## @name ::sync
##
## @summary Parses the raw output from the "sync" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from sync with any of its options as described in the
##    corresponding cdef file 'sync.cdefs'
## @synopsis
##      sync(%args);
## INPUTS TO METHOD CALL 'sync' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'enable' </td> <td> 'enable' </td> </tr>
## <tr> <td> 'disable' </td> <td> 'disable' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = sync
##&nbsp;$VAR1 = [];
##&nbsp;

sub sync {
    my %args = @_;
    my ($error_message, %hash, @result);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error = _check_error( raw_output    => $output,
                              command       => "sync",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
    if($output =~ /CPs\s+(\w+)/) {
        $hash{ 'consistency_point' } = $1;
        push( @result, \%hash);
        return \@result;
    } else {
        return [];
    }
}
1;
