#
#Copyright (c) 2014 NetApp.
#All rights reserved.
#
## @summary Routines to parse 7Mode timezone command output
##
## @description
##    This is a library to parse the raw output of 7Mode timezone command.
##
## @synopsis
##    use timezone;
##
##    timezone(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Node::CLI::7Mode::default::timezone;
use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ($raw_output =~ /[Uu]sage:/) {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    }
    return $error;
}


## @name ::timezone
##
## @summary Parses the raw output from the "timezone" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from timezone with any of its options as described in the
##    corresponding cdef file 'timezone.cdefs'
## @synopsis
##      timezone(%args);
## INPUTS TO METHOD CALL 'timezone' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'set-timezone' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = timezone
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'timezone' => 'UTC'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;
sub timezone {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my (%process_output, @final);
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    } elsif($output =~ /^Current\s+time\s+zone\s+is\s+(\S+)$/i){
        $process_output{timezone} = $1;
        push(@final,\%process_output);
    }
    return (\@final);
}

1;

