#$id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/waffinity.pm# $
#
#Copyright (c) 2015 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode waffinity command
##
## @description
##    This is a library to parse the raw outputs of 7Mode waffinity_stats command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
## @author dl-nacl-dev@netapp.com
#

package NACL::APISet::Node::CLI::7Mode::default::waffinity;

use Data::Dumper;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandNotFoundException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub waffinity_stats {
    my %args       = @_;
    my $result;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my $command    = $resp->get_actual_command();
    my $error;
    my $errormsg;
    my $first_line = '';
    my $can_throw  = 0;
    $error = _parse_for_errors(
        "raw_output"   => $raw_output,
        "command"      => $command,
        "errormsg_ptr" => \$errormsg
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $errormsg, response_object => $resp );
    }

    # Now we can parse the output
    $result = _waffinity_stats(
        raw_output      => $raw_output,
        command         => $command,
        response_object => $resp
    );
    return $result;

}    #end of funtion waffinity_stats()

sub _parse_for_errors {
    my %args = @_;
    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if($output =~ /usage.+/si) {
        $error = "InvalidParam";
        $$error_message = $output;
    } elsif ( $output =~ /sp:\s+(unrecognized\s+command)\s*(.*)/) {
        $error = "CommandNotFound",
        $$error_message = $1;
    } elsif ( $output =~ /sp\s*:\s*ignoring\s*(.*)/) {
        $error = "InvalidParam",
        $$error_message = $output;
    } elsif ( $output =~ /sp\s+test\s+autosupport/ && $output =~ /sp\s+test\s+autosupport/ ){
        $error = "MissingRequiredParam",
        $$error_message = "incomplete command";
    } elsif($output =~ /sp\s+:\s+ignoring\s+.*/) {
        $error = "InvalidParam";
        $$error_message = $output;
    } elsif ( $output =~ /sp\s+notify\s+<command>.*/ ){
        $error = "MissingRequiredParam",
        $$error_message = $output;
    } elsif($output =~ /Unknown EMS.*/) {
        $error = "CommandFailed";
        $$error_message = $output;
    } elsif ( $output =~ /Insufficient\s+number\s+of\s+arguments/ ){
        $error = "MissingRequiredParam",
        $$error_message = $output;
    } elsif ( $output =~ /sp\s+(sendfile|getfile)\s+<srcFile>\s+<dstFile>.*/ ){
        $error = "MissingRequiredParam",
        $$error_message = $output;
    } elsif ( $output =~ /The\s+following\s+commands\s+are\s+available.*/ ){
       $error = "MissingRequiredParam",
       $$error_message = $output;
    } elsif ( $output =~ /There\s+is\s+no\s+SP.*/ ){
       $error = "CommandNotFound",
       $$error_message = $output;
    }
    return $error;
}

sub _waffinity_stats {
    my %args       = @_;
    my $raw_output = $args{"raw_output"};
    my $option     = $args{"command"};
    my $response   = $args{"response_object"};
    my $first_line = '';
    my $can_throw  = 0;
    my @returnarray;
    my @lines ;

    # For waffinity_stats, some output is always expected.
    # So we are making $can_throw as 1 so that if the
    # lines array got finally in _get_lines is empty,
    # UnexpectedOutputException can be thrown there.
    $can_throw = 1;

        @lines = get_lines(
            'raw_output' => $raw_output,
            'first_line' => $first_line,
            'can_throw'  => $can_throw
        );
	my %hash;
        my @reason;
        while ( my $line = shift(@lines) ) {
            if ( $line =~ /^Waffinity configured with\:$/ ) {
		my %hash_new;
                for ( my $count = 0 ; $count < @lines ; $count++ ) {
		    last if ( $lines[$count] =~ /^Waffinity slow-path stats$/ );
                    my @values = split( /\s+:\s+/, $lines[$count]);
                    $values[0] =~ s/\#\s//g;
                    $values[0] =~ s/\/.*//g;
                    $values[0] =~ s/\s+$//g;
                    $hash_new{to_key_name( item => $values[0] )} = $values[1] ;
                }
                push @{$hash{'waffinity_configured'}}, \%hash_new ;
            }elsif ( $line =~ /^Waffinity slow-path stats$/ ) {
                my %hash_new;
		my @array;
	        for ( my $count = 0 ; $count < @lines ; $count++ ) {
                    next if ( $lines[$count] =~ /^Switches/ );
                    last if ( $lines[$count] =~ /Waffinity message dispatch statistics$/ );
                    my @values = split( /\s+/, $lines[$count]);
		    $array[$count]->{'switch'}   = $values[0] ;
		    $array[$count]->{'location'} = $values[1] ;
                }
                @array = grep defined, @array;
                push @{$hash{'waffinity_slow_path_stats'}}, \@array ;
	    }elsif ( $line =~ /Waffinity message dispatch statistics/ ) {
                my %hash_new;
                for ( my $count = 0 ; $count < @lines ; $count++ ) {
		    last if ( $lines[$count] =~ /^Waffinity thread dispatch statistics$/ ) ;
                    my @values = split( /:\s+/, $lines[$count]);
                    $hash_new{to_key_name( item => $values[0] )} = $values[1] ;
                }
                push @{$hash{'waffinity_message_dispatch_statistics'}}, \%hash_new ;
	    }elsif ( $line =~ /^Waffinity thread dispatch statistics$/ ) {
                my %hash_new;
                for ( my $count = 0 ; $count < @lines ; $count++ ) {
                    last if ( $lines[$count] =~ /^Waffinity unsend statistics$/ ) ;
                    my @values = split( /:\s+/, $lines[$count]);
                    $hash_new{to_key_name( item => $values[0] )} = $values[1] ;
                }
                push @{$hash{'waffinity_thread_dispatch_statistics'}}, \%hash_new ;
	    }elsif ( $line =~ /^Waffinity unsend statistics$/ ) {
                my %hash_new;
                for ( my $count = 0 ; $count < @lines ; $count++ ) {
                    last if ( $lines[$count] =~ /^Waffinity domain statistics$/ ) ;
                    my @values = split( /:\s+/, $lines[$count]);
		    $values[0] =~ s/;//g;
                    $hash_new{to_key_name( item => $values[0] )} = $values[1] ;
                }
                push @{$hash{'waffinity_unsend_statistics'}}, \%hash_new ;
	    }elsif ( $line =~ /^Waffinity domain statistics$/ ) {
                my %hash_new;
                for ( my $count = 0 ; $count < @lines ; $count++ ) {
                    last if ( $lines[$count] =~ /^Aggregated waffinity thread statistics/ );
                    my @values = split( /:\s+/, $lines[$count]);
		    $values[0] =~ s/\./ /g;
		    $values[0] =~ s/\s+$//g;
		    $values[0] =~ s/\(\)//g;
                    $hash_new{to_key_name( item => $values[0] )} = $values[1] ;
                }
                push @{$hash{'waffinity_domain_statistics'}}, \%hash_new ;
            }elsif ( $line =~ /^Aggregated waffinity thread statistics/ ) {
                my %hash_new;
		my @array_var;
		my $key;
                for ( my $count = 0 ; $count < @lines ; $count++ ) {
                    last if ( $lines[$count] =~ /^cpu per slice histogram/ );
		    #last if ( $lines[$count] =~ /^wasted slices/ );
                    my @values = split( /:\s+/, $lines[$count]);
                    $values[0] =~ s/\#//g;
		    if ( defined $values[1] ) {
                        $hash_new{to_key_name( item => $values[0] )} = $values[1] ;
		    }else {
			$key = $values[0] if ( !(defined $key) ) ;
			next if ( $lines[$count] =~ /^wasted slices/ );
			push @array_var , $lines[$count];
			$hash_new{to_key_name( item => $key )} = \@array_var ;
		    }
                }
                push @{$hash{'aggregated_waffinity_thread_statistics'}}, \%hash_new ;
            }elsif ( $line =~ /^cpu per slice histogram/ ) {
                my %hash_new;
		my @array;
		my @keys;
                for ( my $count = 0 ; $count < @lines ; $count++ ) {
		    if ( $lines[$count] =~ /0\s+1\s+2\s+3\s+4\s+5\s+6\s+7/ ) {
		        @keys = split(/\s+/, $lines[$count]);
		    }
		    next if ( $lines[$count] =~ /0\s+1\s+2\s+3\s+4\s+5\s+6\s+7/ );
                    last if ( $lines[$count] =~ /^Waffinity Affinity statistics$/ );

                    my @values = split( /\s+/, $lines[$count]);
		    for ( my $i=0 ; $i < @keys ; $i++ ){
			$array[$count]->{$keys[$i]} = $values[$i];
		    }
                }
                @array = grep defined, @array;
                push @{$hash{'cpu_per_slice_histogram'}}, \@array ;
            }elsif ( $line =~ /^Waffinity Affinity statistics$/ ) {
                my %hash_new;
                my @array;
                for ( my $count = 0 ; $count < @lines ; $count++ ) {
                    next if ( $lines[$count] =~ /^Affi\s+Class/ );
		    next if ( $lines[$count] =~ /=/ ) ;
                    last if ( $lines[$count] =~ /^Affinity Message statistics/ );
                    my @values = split( /\s+/, $lines[$count]);
		    my @keys = ('affi','class','parent','queue','sched','sched/s','preempt','skip','skip/enq','cpu','wait_avg','skip_avg','cpu_avg','wait_max','skip_max','cpu_max','msgs_avg','msgs_var','wp1m_pre','wp1m_msg','wp1m_post','wp1m_pre_avg','wp1m_msg_avg','wp1m_post_avg','q_len_max','q_len_avg','sched_0','sched_1','sched_2','short_ts','maybe_rq','not_rq','yes_rq','enqueue','preempted','promoted');
                    for ( my $i=0 ; $i < @keys ; $i++ ){
                        $array[$count]->{$keys[$i]} = $values[$i];
                    }
                }
                @array = grep defined, @array;
                push @{$hash{'waffinity_affinity_statistics'}}, \@array ;
            }elsif ( $line =~ /^Affinity Message statistics$/ ) {
                my %hash_new;
                my @array;
                for ( my $count = 0 ; $count < @lines ; $count++ ) {
                    next if ( $lines[$count] =~ /^Class/ );
                    next if ( $lines[$count] =~ /=/ ) ;
                    last if ( $lines[$count] =~ /^Per Affinity Message statistics$/ );
                    my @values = split( /\s+/, $lines[$count]);
                    my @keys = ('class','queue','msg_type','msgs_new','msgs_restarts','cpu','over','per_msg_cpu_avg','per_msg_cpu_var','per_msg_cpu_max','per_msg_wait_avg','per_msg_wait_var','per_msg_wait_max','qos_msgs_u','qos_msgs_d','qos_msgs_be','qos_msgs_bg','qos_per_msg_wait_avg_u','qos_per_msg_wait_avg_d','qos_per_msg_wait_avg_be','qos_per_msg_wait_avg_bg','qos_per_msg_wait_max_u','qos_per_msg_wait_max_d','qos_per_msg_wait_max_be','qos_per_msg_wait_max_bg','wp1m_pre','wp1m_msg','wp1m_post','wp1m_pre_avg','wp1m_msg_avg','wp1m_post_avg');
                    for ( my $i=0 ; $i < @keys ; $i++ ){
                        $array[$count]->{$keys[$i]} = $values[$i];
                    }
                }
                @array = grep defined, @array;
                push @{$hash{'affinity_message_statistics'}}, \@array ;
            }elsif ( $line =~ /^Per Affinity Message statistics$/ ) {
                my %hash_new;
                my @array;
                for ( my $count = 0 ; $count < @lines  ; $count++ ) {
                    next if ( $lines[$count] =~ /^Affi\s+Class/ );
                    next if ( $lines[$count] =~ /=/ );
                    last if ( $lines[$count] =~ /^Class STRIPE WAFL/ );
                    my @values = split( /\s+/, $lines[$count]);
		    my @keys =('affi','class','queue','msg_type','msgs_new','msgs_restarts','cpu','over','per_msg_cpu_avg','per_msg_cpu_var','per_msg_cpu_max','per_msg_wait_avg','per_msg_wait_var','per_msg_wait_max','qos_msgs_u','qos_msgs_d','qos_msgs_be','qos_msgs_bg','qos_per_msg_wait_avg_u','qos_per_msg_wait_avg_d','qos_per_msg_wait_avg_be','qos_per_msg_wait_avg_bg','qos_per_msg_wait_max_u','qos_per_msg_wait_max_d','qos_per_msg_wait_max_be','qos_per_msg_wait_max_bg','wp1m_pre','wp1m_msg','wp1m_post','wp1m_pre_avg','wp1m_msg_avg','wp1m_post_avg');
                    for ( my $i=0 ; $i < @keys ; $i++ ){
                        $array[$count]->{$keys[$i]} = $values[$i];
                    }
		}
		@array = grep defined, @array;
                push @{$hash{'per_affinity_message_statistics'}}, \@array ;
            }

        }
        push( @returnarray, \%hash );
        return \@returnarray;
}

1;
