#
# Copyright (c) 2015 NetApp Inc.
# All rights reserved.
#
## @summary Routines to parse node scope metawafl wafl get commands
##
## @description
##    This is a library to parse the raw output of dblade wafl_iface commands.
##
## @synopsis
##
##    wafl_iface wafl_get_child_and_next_fbn(%args);
##    wafl_iface wafl_get_subtree_level(%args);
##
## @author onkarb@netapp.com,asmitak@netapp.com,kvishal@netapp.com

## @status public

package NACL::APISet::Node::CLI::7Mode::default::wafl_ifaces;

use strict;
use warnings;
use NATE::BaseException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

my %method_hash = (
    wafl_get_child_and_next_fbn  => {
        output_parser_method => '_wafl_ifaces_parser',
    },
    wafl_get_subtree_level  => {
        output_parser_method => '_wafl_ifaces_parser',
    },
);

parser_method_builder(__PACKAGE__, %method_hash);

sub _wafl_ifaces_parser {
    my $args = shift;
    my $raw_output = $args->{response_object}->get_processed_output;
    my $hash;

    my @lines = get_lines(
        'raw_output'      => $raw_output,
        'can_throw'       => 1,
    );
    my @list = split(" ",$lines[0]);
    foreach my $line (@list) {
	my @list2 = split("=",$line);
	my $key = $list2[0];
	my $value = $list2[1];
	$hash->{ to_key_name(item => $key) } = $value;
    }
    return [$hash];
}
1;

## @name ::wafl_get_child_and_next_fbn
##
## @summary Parses the raw output from the "wafl get child_and_next_fbn" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_get_child_and_next_fbn()
## @synopsis
##      wafl_get_child_and_next_fbn();
## INPUTS TO METHOD CALL 'wafl_get_child_and_next_fbn' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '' </td> <td> 'path' </td><td> Not a boolean option </td></tr>
## <tr> <td> '' </td> <td> 'fbn' </td><td> Not a boolean option </td></tr>
## <tr> <td> '' </td> <td> 'level' </td><td> Not a boolean option </td></tr>
## </table>
## @arg %args required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Does not return any value
#
#
## @name ::wafl_get_subtree_level
##
## @summary Parses the raw output from the "wafl get subtree_level" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wafl_get_subtree_level with any of its options as described in the
##    corresponding cdef file 'wafl_buftree_iterators.cdefs'
## @synopsis
##      wafl_get_subtree_level(%args);
## INPUTS TO METHOD CALL 'wafl_get_subtree_level' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '' </td> <td> 'path' </td><td> Not a boolean option </td></tr>
## <tr> <td> '' </td> <td> 'fbn' </td><td> Not a boolean option </td></tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Does not return any value
#
