#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse wcc command output
##
## @description
##    This is a library to parse the raw output of wcc command.
##
## @synopsis
##    use wcc;
##
##    wcc(%args);
##
## @author anupamap@netapp.com,dl-nacl-dev
##
## @status public
#

package NACL::APISet::Node::CLI::7Mode::default::wcc;
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

use NACL::APISet::Response::CLI::ParserUtils;
use strict;
use warnings;

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /[Uu]sage/ ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /no passwd.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /[Ii]nvalid.*/ ) {
        $error         = "CommandFailed";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
}
## @name ::wcc
##
## @summary Parses the raw output from the "wcc" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from wcc with any of its options as described in the
##    corresponding cdef file 'wcc.cdefs'
## @synopsis
##      wcc(%args);
## INPUTS TO METHOD CALL 'wcc' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-v' </td> <td> 'verbose' </td> <td> '1' </td> </tr>
## <tr> <td> '-u' </td> <td> 'uname' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'ntname' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'entries' </td> <td> '1' </td> </tr>
## <tr> <td> '-i' </td> <td> 'ip-address' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'add-uname' </td> <td> '1' </td> </tr>
## <tr> <td> '-x' </td> <td> 'remove-entries' </td> <td> '1' </td> </tr>
## <tr> <td> '-f' </td> <td> 'force' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; wcc -d; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;WAFL cred cache is empty
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub wcc {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;
    if ($error_level) {
        $error = _check_error(
            raw_output      => $output,
            response_object => $resp,
        );
    }
    return [];
}
1;
