#
# Copyright (c) 2014 NetApp.
# All rights reserved.
#
## @summary Routines to parse node scope metawafl wreckcxt commands
##
## @description
##    This is a library to parse the raw output of node scope metawafl mw_tec commands.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "mw_tec add" will be mw_tec_add()
##
## @synopsis
##    use wreckcxt;
##
##    wreckcxt(%args);
##
## @author chenp@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::wreckcxt;

use strict;
use warnings;
use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Response::CLI::ParserUtils;

use Data::Dumper;

my %method_hash = (
    wreckcxt => {
        output_parser_method => '_output_parser',
    },
);

parser_method_builder(__PACKAGE__, %method_hash);

sub _output_parser {
    my $args = shift;
    my $raw_output = $args->{response_object}->get_processed_output;
    my $actual_cmd = $args->{response_object}->get_actual_command;

    my $hash;

    # Command output is '' if -v is not specified
    if ( $actual_cmd !~ /-v/ ) {
        return [];
    }

    my @lines = get_lines(
        'raw_output'      => $raw_output,
        'can_throw'       => 1,
    );

    for my $line (@lines) {
        my ($key, $value) = $line =~ /(.+?):\s?(.*)/;
        if ($key) {
            if ($value =~ /\s/ ) {
                $value = [split(/\s/, $value)];
            }
            $hash->{ to_key_name(item => $key) } = $value;
        }
    }

    return [$hash];
} ## end sub _output_parser

1;
