#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode wrfile command output
##
## @description
##    This is a library to parse the raw output of 7Mode wrfile command.
##
## @synopsis
##    use wrfile;
##
##    wrfile(%args);
##
## @author govinda1@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::wrfile;

use strict;
use warnings;

use Params::Validate qw(:all);

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args = validate_with(
        params => \@_,
        spec   => {
            raw_output      => { type => SCALAR },
            response_object => { type => OBJECT, isa => 'NACL::APISet::Response' }
        }
    );
    my $raw_output    = $args{raw_output};
    my $exception = 'NACL::APISet::Exceptions::';
    my %excp_args = ( text => $raw_output,
        response_object => $args{response_object} );

    if ( $raw_output =~ /(.*:\s+No\s+such\s+file\s+or\s+directory)/) {
        $exception .= 'InvalidParamValueException';
        $exception->throw(%excp_args);
    } elsif ($raw_output =~ /[Uu]sage:\s(.*)|(.*:\s+Invalid\s+argument)/) {
        $exception .= 'InvalidParamException';
        $exception->throw(%excp_args);
    } elsif ($raw_output =~ /Unknown error/i) {
        $exception .= 'CommandFailedException';
        $exception->throw(%excp_args);
    }
}

## @name ::wrfile
##
## @summary Parses the raw output from the "wrfile" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from wrfile with any of its options as described in the 
##    corresponding cdef file 'wrfile.cdefs'
## @synopsis
##      wrfile(%args);
## INPUTS TO METHOD CALL 'wrfile' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'content' </td> </tr> 
## <tr> <td> '-i' </td> <td> 'inum' </td> </tr> 
## <tr> <td> '-private' </td> <td> 'private' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'file' </td> </tr> 
## <tr> <td> '-a' </td> <td> 'append' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
## $VAR1 = []
sub wrfile {
    my %args = @_;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();

    _check_error(
        raw_output      => $output,
        response_object => $resp
    );
    
    return [];
}

1;
