#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
##
##  @summary Routine to parse 7Mode ypwhich command
##
## @description
##    This is a library to parse the raw outputs of any of the 7Mode ypwhich
##    command. The function name will be the same as command name with
##    underscores being used instead of spaces.
##    For Example,
##      the parser function for "ypwhich" will be ypwhich().
##
## @synopsis
##    use ypwhich;
##    ypwhich(%args);
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
##

package NACL::APISet::Node::CLI::7Mode::default::ypwhich;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

## @name ::ypwhich
##
## @summary Parses the raw output from the "ypwhich" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ypwhich with any of its options as described in the
##    corresponding cdef file 'ypwhich.cdefs'
## @synopsis
##      ypwhich(%args);
## INPUTS TO METHOD CALL 'ypwhich' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; ypwhich; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;ns1.sim.eng.btc.netapp.in
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'nis' => 'ns1.sim.eng.btc.netapp.in'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub ypwhich {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my @lines  = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    $result_array[0]->{nis} = $lines[0];
    return \@result_array;

}
1;
