#
#Copyright (c) 2014 NetApp.
#All rights reserved.
#
## @summary Routines to parse 7Mode zeroblock command output
##
## @description
##    This is a library to parse the raw output of 7Mode zeroblock command.
##
## @synopsis
##    use zeroblock;
##
##    zeroblock(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#

package NACL::APISet::Node::CLI::7Mode::default::zeroblock;
use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingValueException;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $error;
    chomp $raw_output;
    my $error_message = $args{error_message};
    if ($raw_output =~ /zeroblock dbn <disk_name> <dbn>/) {
        $error          = "InvalidParamValue";
        $$error_message = $raw_output;
    } elsif ($raw_output =~ /zeroblock dbn/) {
        $error = "MissingValue";
        $$error_message = $raw_output;
    }

    return $error;
}


## @name ::zeroblock
##
## @summary Parses the raw output from the "zeroblock" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from zeroblock with any of its options as described in the
##    corresponding cdef file 'zeroblock.cdefs'
## @synopsis
##      zeroblock(%args);
## INPUTS TO METHOD CALL 'zeroblock' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'set-zeroblock' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference which contains the parsed output.
## @examples
##&nbsp;Command = zeroblock dbn 0b.14 111144
##&nbsp;$VAR1 = [];
##
sub zeroblock {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my (%process_output, @final);
    my $error;
    $error = _check_error(
        raw_output    => $output,
        error_message => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
    return [];
}

1;

