#
# $Id: //depot/prod/DOT/dev/test/lib/NACL/APISet/Node/CLI/SP/default/nslookup.pm#1 $
#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
##
## @synopsis
##    use nslookup;
##
##    nslookup(%args);
##
## @summary Routines to parse nslookup command output
## @author dl-nacl-dev

package NACL::APISet::Node::CLI::SP::default::nslookup;

###################
# Everytime
###################
use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;
# Include exceptions which will be thrown

use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandNotFoundException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _check_error {
    my %args   = @_;
    my $output = $args{raw_output};
    my $resp   = $args{"response_object"};
    my $error;
    my $error_message;
    if ( $output =~ /nslookup[:]+can\'t resolve/i ) {
	    $$error_message = $output;
        $error = "CommandFailed";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
}


## @name ::nslookup
##
## @summary Parses the raw output from the "nslookup" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from nslookup with any of its options as described in the 
##    corresponding cdef file 'nslookup.cdefs'
## @synopsis
##      nslookup(%args);
## INPUTS TO METHOD CALL 'nslookup' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'server' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'host' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = nslookup 10.224.203.68
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Server:    10.224.223.130
##&nbsp;Address 1: 10.224.223.130 acast-cns1.rtp.eng.netapp.com
##&nbsp;
##&nbsp;Name:      10.224.203.68
##&nbsp;Address 1: 10.224.203.68 f3240-203-68.rtp.netapp.com
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'host_name' => 'f3240-203-68.rtp.netapp.com',
##&nbsp;            'server' => '10.224.223.130',
##&nbsp;            'host_ip' => '10.224.203.68',
##&nbsp;            'host' => '10.224.203.68',
##&nbsp;            'server_name' => 'acast-cns1.rtp.eng.netapp.com',
##&nbsp;            'server_ip' => '10.224.223.130'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub nslookup {
    my %args = @_;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @result_array;
    _check_error( 
         "raw_output"      => $output, 
         "response_object" => $resp 
    ); 
    my $error;
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    if (grep /nslookup[:]+\s+can\'t\s+resolve/, @lines) {
        $error = 'NACL::APISet::Exceptions::CommandFailedException';
        $error->throw( text => $output, response_object => $resp );
    }

    #Now we can parse the output
    my (@result_array, $line, @split_line);
    my %hash;
    while(@lines) {
	    $line = shift(@lines);
	    if($line =~ /Server[:]+\s+(.+)/i){
	         $result_array[0]->{'server'} = $1;
		 $line = shift(@lines);
		 if($line =~ /Address\s+\d+[:]+\s+(.+)/){
		    @split_line = split(/\s+/, $1, 2);
		    $result_array[0]->{'server_ip'} = $split_line[0];
                    $result_array[0]->{'server_name'} = $split_line[1] if($split_line[1]);
                 }
	    }elsif($line =~ /Name[:]+\s+(.+)/i){
	         $result_array[0]->{'host'} = $1;
		 $line = shift(@lines);
		 if($line =~ /Address\s+\d+[:]+\s+(.+)/){
		    @split_line = split(/\s+/, $1, 2);
		    $result_array[0]->{'host_ip'} = $split_line[0];
                    $result_array[0]->{'host_name'} = $split_line[1] if($split_line[1]);
                 }
            }
    }
    return \@result_array;
}
1;
