# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/SP/default/system_sensors.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
##
## @synopsis
##    system_sensors_show(%args);
##    system_sensors_get(%args);
##    system_sensors_monitor(%args);
##    system_sensors_thermal_mgmt(%args);
##    system_sensors_set_thresh(%args);
##    system_sensors_set_fan(%args);
##
## @summary Routines to parse system sensors command output
## @author Akshay Mohit(makshay@netapp.com), dl-nacl-dev@netapp.com

package NACL::APISet::Node::CLI::SP::default::system_sensors;
###################
# Everytime
###################
use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

# Include exceptions which will be thrown

use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::CommandNotFoundException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

# Description

#  This Parser is used to run sp status command on the SP filer.

sub _parse_for_errors {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $output =~ /[Uu]sage.*/ ) {
        $error          = "InvalidParam";
        $$error_message = $output;
    }
    elsif ( $output =~ /(sensor data record.*)/i ) {
        $error = "CommandNotFound";
  $$error_message = $1;
    }
    elsif ( $output =~ /(invalid argument.*)/i ) {
        $error = "InvalidParam";
  $$error_message = $1;
    }
    elsif ( $output =~ /(^Error.*)/i ) {
        $error = "InvalidParam";
  $$error_message = $1;
    }
    elsif ( $output =~ /(^Valid Threshhold.*)/i ) {
        $error = "InvalidParamValue";
  $$error_message = $1;
    }
    return $error;
}

## @name ::system_sensors_show
##
## @summary Parses the raw output from the "system sensors show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system sensors show with any of its options as described in the
##    corresponding cdef file 'system_sensors.cdefs'
## @synopsis
##      system_sensors_show(%args);
## INPUTS TO METHOD CALL 'system sensors show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'show' </td> <td> 'show' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = system sensors show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;system sensors show
##&nbsp;Sensor Name      | Current    | Unit       | Status| LCR       | LNC       | UNC       | UCR
##&nbsp;-----------------+------------+------------+-------+-----------+-----------+-----------+-----------
##&nbsp;CPU0_Temp_Margin | -47.000    | degrees C  | ok    | na        | na        | -5.000    | 0.000
##&nbsp;In_Flow_Temp     | 35.000     | degrees C  | ok    | 0.000     | 10.000    | 49.000    | 55.000
##&nbsp;Out_Flow_Temp    | 41.000     | degrees C  | ok    | 0.000     | 10.000    | 62.000    | 67.000
##&nbsp;Memory_Hot       | 0x0        | discrete   | 0x0180| na        | na        | na        | na
##&nbsp;CPU_Hot          | 0x0        | discrete   | 0x0180| na        | na        | na        | na
##&nbsp;CPU_Cat_Error    | 0x0        | discrete   | 0x0180| na        | na        | na        | na
##&nbsp;CPU_Therm_Trip   | 0x0        | discrete   | 0x0180| na        | na        | na        | na
##&nbsp;CPU_VCC          | 0.902      | Volts      | ok    | 0.708     | 0.747     | 1.348     | 1.426
##&nbsp;CPU_1.05V        | 1.048      | Volts      | ok    | 0.892     | 0.941     | 1.154     | 1.203
##&nbsp;CPU_VTT          | 1.106      | Volts      | ok    | 0.931     | 0.989     | 1.213     | 1.261
##&nbsp;CPU_1.5V         | 1.494      | Volts      | ok    | 1.271     | 1.348     | 1.649     | 1.727
##&nbsp;CPU_1.8V         | 1.806      | Volts      | ok    | 1.535     | 1.625     | 1.974     | 2.064
##&nbsp;10G_1.0V         | 0.999      | Volts      | ok    | 0.854     | 0.902     | 1.096     | 1.154
##&nbsp;USB_5.0V         | 5.006      | Volts      | ok    | 4.253     | 4.495     | 5.492     | 5.759
##&nbsp;PCH_3.3V         | 3.291      | Volts      | ok    | 2.798     | 2.973     | 3.625     | 3.800
##&nbsp;SASS_1.0V        | 0.999      | Volts      | ok    | 0.854     | 0.902     | 1.096     | 1.154
##&nbsp;SASS_1.2V        | 1.193      | Volts      | ok    | 1.018     | 1.077     | 1.319     | 1.377
##&nbsp;STBY_1.8V        | 1.804      | Volts      | ok    | 1.533     | 1.620     | 1.979     | 2.066
##&nbsp;STBY_2.5V        | 2.496      | Volts      | ok    | 2.122     | 2.246     | 2.746     | 2.870
##&nbsp;STBY_5.0V        | 4.982      | Volts      | ok    | 4.253     | 4.495     | 5.492     | 5.759
##&nbsp;PSU1_Fault       | na         | discrete   | na    | na        | na        | na        | na
##&nbsp;PSU1_Present     | na         | discrete   | na    | na        | na        | na        | na
##&nbsp;PSU2_Fault       | na         | discrete   | na    | na        | na        | na        | na
##&nbsp;PSU2_Present     | na         | discrete   | na    | na        | na        | na        | na
##&nbsp;PSU3_Fault       | na         | discrete   | na    | na        | na        | na        | na
##&nbsp;PSU3_Present     | na         | discrete   | na    | na        | na        | na        | na
##&nbsp;PSU4_Fault       | na         | discrete   | na    | na        | na        | na        | na
##&nbsp;PSU4_Present     | na         | discrete   | na    | na        | na        | na        | na
##&nbsp;AC_Power_Fail    | 0x0        | discrete   | 0x0180| na        | na        | na        | na
##&nbsp;Power_SeqBootErr | 0x0        | discrete   | 0x0180| na        | na        | na        | na
##&nbsp;Power_Good       | 0x0        | discrete   | 0x0280| na        | na        | na        | na
##&nbsp;Power_Proc_OK    | 0x0        | discrete   | 0x0280| na        | na        | na        | na
##&nbsp;Bat_1.5V         | 1.522      | Volts      | ok    | 1.277     | 1.342     | 1.651     | 1.729
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'unc' => 'UNC',
##&nbsp;            'unit' => 'Unit',
##&nbsp;            'status' => 'Status',
##&nbsp;            'lnc' => 'LNC',
##&nbsp;            'current' => 'Current',
##&nbsp;            'lcr' => 'LCR',
##&nbsp;            'sensor_name' => 'Sensor Name',
##&nbsp;            'ucr' => 'UCR'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'unc' => '-5.000',
##&nbsp;            'unit' => 'degrees C',
##&nbsp;            'status' => 'ok',
##&nbsp;            'lnc' => 'na',
##&nbsp;            'current' => '-47.000',
##&nbsp;            'lcr' => 'na',
##&nbsp;            'sensor_name' => 'CPU0_Temp_Margin',
##&nbsp;            'ucr' => '0.000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'unc' => '49.000',
##&nbsp;            'unit' => 'degrees C',
##&nbsp;            'status' => 'ok',
##&nbsp;            'lnc' => '10.000',
##&nbsp;            'current' => '35.000',
##&nbsp;            'lcr' => '0.000',
##&nbsp;            'sensor_name' => 'In_Flow_Temp',
##&nbsp;            'ucr' => '55.000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'unc' => '62.000',
##&nbsp;            'unit' => 'degrees C',
##&nbsp;            'status' => 'ok',
##&nbsp;            'lnc' => '10.000',
##&nbsp;            'current' => '41.000',
##&nbsp;            'lcr' => '0.000',
##&nbsp;            'sensor_name' => 'Out_Flow_Temp',
##&nbsp;            'ucr' => '67.000'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'unc' => 'na',
##&nbsp;            'unit' => 'discrete',
##&nbsp;            'status' => ' 0x0180',
##&nbsp;            'lnc' => 'na',
##&nbsp;            'current' => '0x0',
##&nbsp;            'lcr' => 'na',
##&nbsp;            'sensor_name' => 'Memory_Hot',
##&nbsp;            'ucr' => 'na'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'unc' => 'na',
##&nbsp;            'unit' => 'discrete',
##&nbsp;            'status' => '0x0180',
##&nbsp;            'lnc' => 'na',
##&nbsp;            'current' => '0x0',
##&nbsp;            'lcr' => 'na',
##&nbsp;            'sensor_name' => 'CPU_Hot',
##&nbsp;            'ucr' => 'na'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'unc' => 'na',
#&nbsp;             'unit' => 'discrete',
##&nbsp;            'status' => '0x0180',
##&nbsp;            'lnc' => 'na',
##&nbsp;            'current' => '0x0',
##&nbsp;            'lcr' => 'na',
##&nbsp;            'sensor_name' => 'CPU_Cat_Error',
##&nbsp;            'ucr' => 'na'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => 'na',
##&nbsp;           'unit' => 'discrete',
##&nbsp;           'status' => '0x0180',
##&nbsp;           'lnc' => 'na',
##&nbsp;           'current' => '0x0',
##&nbsp;           'lcr' => 'na',
##&nbsp;           'sensor_name' => 'CPU_Therm_Trip',
##&nbsp;           'ucr' => 'na'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => '1.348',
##&nbsp;           'unit' => 'Volts',
##&nbsp;           'status' => 'ok',
##&nbsp;           'lnc' => '0.747',
##&nbsp;           'current' => '0.902',
##&nbsp;           'lcr' => '0.708',
##&nbsp;           'sensor_name' => 'CPU_VCC',
##&nbsp;           'ucr' => '1.426'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => '1.154',
##&nbsp;           'unit' => 'Volts',
##&nbsp;           'status' => 'ok',
##&nbsp;           'lnc' => '0.941',
##&nbsp;           'current' => '1.048',
##&nbsp;           'lcr' => '0.892',
##&nbsp;           'sensor_name' => 'CPU_1.05V',
##&nbsp;           'ucr' => '1.203'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => '1.213',
##&nbsp;           'unit' => 'Volts',
##&nbsp;           'status' => 'ok',
##&nbsp;           'lnc' => '0.989',
##&nbsp;           'current' => '1.106',
##&nbsp;           'lcr' => '0.931',
##&nbsp;           'sensor_name' => 'CPU_VTT',
##&nbsp;           'ucr' => '1.261'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unit' => 'Volts',
##&nbsp;           'status' => 'ok',
##&nbsp;           'lnc' => '1.348',
##&nbsp;           'current' => '1.494',
##&nbsp;           'lcr' => '1.271',
##&nbsp;           'sensor_name' => 'CPU_1.5V',
##&nbsp;           'ucr' => '1.727'
##&nbsp;          },
##&nbsp;          {
#&nbsp;            'unc' => '1.974',
##&nbsp;           'unit' => 'Volts',
##&nbsp;           'status' => 'ok',
##&nbsp;           'lnc' => '1.625',
##&nbsp;           'current' => '1.806',
##&nbsp;           'lcr' => '1.535',
##&nbsp;           'sensor_name' => 'CPU_1.8V',
##&nbsp;           'ucr' => '2.064'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => '1.096',
##&nbsp;           'unit' => 'Volts',
##&nbsp;           'status' => 'ok',
##&nbsp;           'lnc' => '0.902',
##&nbsp;           'current' => '0.999',
##&nbsp;           'lcr' => '0.854',
##&nbsp;           'sensor_name' => '10G_1.0V',
##&nbsp;           'ucr' => '1.154'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => '5.492',
##&nbsp;           'unit' => 'Volts',
##&nbsp;           'status' => 'ok',
##&nbsp;           'lnc' => '4.495',
##&nbsp;           'current' => '5.006',
##&nbsp;           'lcr' => '4.253',
##&nbsp;           'sensor_name' => 'USB_5.0V',
##&nbsp;           'ucr' => '5.759'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => '3.625',
##&nbsp;           'unit' => 'Volts',
##&nbsp;           'status' => 'ok',
##&nbsp;           'lnc' => '2.973',
##&nbsp;           'current' => '3.291',
##&nbsp;           'lcr' => '2.798',
##&nbsp;           'sensor_name' => 'PCH_3.3V',
##&nbsp;           'ucr' => '3.800'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => '1.096',
##&nbsp;           'unit' => 'Volts',
##&nbsp;           'status' => 'ok',
##&nbsp;           'lnc' => '0.902',
##&nbsp;           'current' => '0.999',
##&nbsp;           'lcr' => '0.854',
##&nbsp;           'sensor_name' => 'SASS_1.0V',
##&nbsp;           'ucr' => '1.154'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => '1.319',
##&nbsp;           'unit' => 'Volts',
##&nbsp;           'status' => 'ok',
##&nbsp;           'lnc' => '1.077',
##&nbsp;           'current' => '1.193',
##&nbsp;           'lcr' => '1.018',
##&nbsp;           'sensor_name' => 'SASS_1.2V',
##&nbsp;           'ucr' => '1.377'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => '1.979',
##&nbsp;           'unit' => 'Volts',
##&nbsp;           'status' => 'ok',
##&nbsp;           'lnc' => '1.620',
##&nbsp;           'current' => '1.804',
##&nbsp;           'lcr' => '1.533',
##&nbsp;           'sensor_name' => 'STBY_1.8V',
##&nbsp;           'ucr' => '2.066'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => '2.746',
##&nbsp;           'unit' => 'Volts',
##&nbsp;           'status' => 'ok',
##&nbsp;           'lnc' => '2.246',
##&nbsp;           'current' => '2.496',
##&nbsp;           'lcr' => '2.122',
##&nbsp;           'sensor_name' => 'STBY_2.5V',
##&nbsp;           'ucr' => '2.870'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => '5.492',
##&nbsp;           'unit' => 'Volts',
##&nbsp;           'status' => 'ok',
##&nbsp;           'lnc' => '4.495',
##&nbsp;           'current' => '4.982',
##&nbsp;           'lcr' => '4.253',
##&nbsp;           'sensor_name' => 'STBY_5.0V',
##&nbsp;           'ucr' => '5.759'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => 'na',
##&nbsp;           'unit' => 'discrete',
##&nbsp;           'status' => 'na',
##&nbsp;           'lnc' => 'na',
##&nbsp;           'current' => 'na',
##&nbsp;           'lcr' => 'na',
##&nbsp;           'sensor_name' => 'PSU1_Fault',
##&nbsp;           'ucr' => 'na'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => 'na',
##&nbsp;           'unit' => 'discrete',
##&nbsp;           'status' => 'na',
##&nbsp;           'lnc' => 'na',
##&nbsp;           'current' => 'na',
##&nbsp;           'lcr' => 'na',
##&nbsp;           'sensor_name' => 'PSU1_Present',
##&nbsp;           'ucr' => 'na'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => 'na',
##&nbsp;           'unit' => 'discrete',
##&nbsp;           'status' => 'na',
##&nbsp;           'lnc' => 'na',
##&nbsp;           'current' => 'na',
##&nbsp;           'lcr' => 'na',
##&nbsp;           'sensor_name' => 'PSU2_Fault',
##&nbsp;           'ucr' => 'na'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => 'na',
##&nbsp;           'unit' => 'discrete',
##&nbsp;           'status' => 'na',
##&nbsp;           'lnc' => 'na',
##&nbsp;           'current' => 'na',
##&nbsp;           'lcr' => 'na',
##&nbsp;           'sensor_name' => 'PSU2_Present',
##&nbsp;           'ucr' => 'na'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => 'na',
##&nbsp;           'unit' => 'discrete',
##&nbsp;           'status' => 'na',
##&nbsp;           'lnc' => 'na',
##&nbsp;           'current' => 'na',
##&nbsp;           'lcr' => 'na',
##&nbsp;           'sensor_name' => 'PSU3_Fault',
##&nbsp;           'ucr' => 'na'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => 'na',
##&nbsp;           'unit' => 'discrete',
##&nbsp;           'status' => 'na',
##&nbsp;           'lnc' => 'na',
##&nbsp;           'current' => 'na',
##&nbsp;           'lcr' => 'na',
##&nbsp;           'sensor_name' => 'PSU3_Present',
##&nbsp;           'ucr' => 'na'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => 'na',
##&nbsp;           'unit' => 'discrete',
##&nbsp;           'status' => 'na',
##&nbsp;           'lnc' => 'na',
##&nbsp;           'current' => 'na',
##&nbsp;           'lcr' => 'na',
##&nbsp;           'sensor_name' => 'PSU4_Fault',
##&nbsp;           'ucr' => 'na'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => 'na',
##&nbsp;           'unit' => 'discrete',
##&nbsp;           'status' => 'na',
##&nbsp;           'lnc' => 'na',
##&nbsp;           'current' => 'na',
##&nbsp;           'lcr' => 'na',
##&nbsp;           'sensor_name' => 'PSU4_Present',
##&nbsp;           'ucr' => 'na'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => 'na',
##&nbsp;           'unit' => 'discrete',
##&nbsp;           'status' => '0x0180',
##&nbsp;           'lnc' => 'na',
##&nbsp;           'current' => '0x0',
##&nbsp;           'lcr' => 'na',
##&nbsp;           'sensor_name' => 'AC_Power_Fail',
##&nbsp;           'ucr' => 'na'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => 'na',
##&nbsp;           'unit' => 'discrete',
##&nbsp;           'status' => '0x0180',
##&nbsp;           'lnc' => 'na',
##&nbsp;           'current' => '0x0',
##&nbsp;           'lcr' => 'na',
##&nbsp;           'sensor_name' => 'Power_SeqBootErr',
##&nbsp;           'ucr' => 'na'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => 'na',
##&nbsp;           'unit' => 'discrete',
##&nbsp;           'status' => '0x0280',
##&nbsp;           'lnc' => 'na',
##&nbsp;           'current' => '0x0',
##&nbsp;           'lcr' => 'na',
##&nbsp;           'sensor_name' => 'Power_Good',
##&nbsp;           'ucr' => 'na'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => 'na',
##&nbsp;           'unit' => 'discrete',
##&nbsp;           'status' => '0x0280',
##&nbsp;           'lnc' => 'na',
##&nbsp;           'current' => '0x0',
##&nbsp;           'lcr' => 'na',
##&nbsp;           'sensor_name' => 'Power_Proc_OK',
##&nbsp;           'ucr' => 'na'
##&nbsp;          },
##&nbsp;          {
##&nbsp;           'unc' => '1.651',
##&nbsp;           'unit' => 'Volts',
##&nbsp;           'status' => 'ok',
##&nbsp;           'lnc' => '1.342',
##&nbsp;           'current' => '1.522',
##&nbsp;           'lcr' => '1.277',
##&nbsp;           'sensor_name' => 'Bat_1.5V',
##&nbsp;           'ucr' => '1.729'
##&nbsp;          }
##&nbsp;       ];
##&nbsp;

sub system_sensors_show {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @result_array;
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $i = 0;

#Now we can parse the output
    my @arraylines = get_lines( raw_output => $output, response_object => $resp );
#To get the column names
    my $head=shift(@arraylines);
    my @headings =  split('\s*\|\s*', $head);
#To remove line -------
    shift(@arraylines);
    foreach my $lines (@arraylines) {
	    my @arry=split('\s*\|\s*', $lines);
	    for(my $j=0;$j<=$#arry;$j++){
		    $result_array[$i]->{to_key_name(item =>$headings[$j])} = $arry[$j];
	    }
	    $i++;
    }
    return \@result_array;
}    ##end of system_sensors_show

## @name ::system_sensors_get
##
## @summary Parses the raw output from the "system sensors get" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_sensors_get with any of its options as described in the
##    corresponding cdef file 'system_sensors.cdefs'
## @synopsis
##      system_sensors_get(%args);
## INPUTS TO METHOD CALL 'system_sensors_get' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'sensor-name' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = system sensors get P5V_STBY
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Locating sensor record...
##&nbsp;Sensor ID              : P5V_STBY (0x12)
##&nbsp; Entity ID             : 7.97
##&nbsp; Sensor Type (Analog)  : Voltage
##&nbsp; Sensor Reading        : 5.051 (+/- 0) Volts
##&nbsp; Status                : ok
##&nbsp; Lower Non-Recoverable : na
##&nbsp; Lower Critical        : 4.246
##&nbsp; Lower Non-Critical    : 4.343
##&nbsp; Upper Non-Critical    : 5.661
##&nbsp; Upper Critical        : 5.807
##&nbsp; Upper Non-Recoverable : na
##&nbsp; Assertion Events      :
##&nbsp; Assertions Enabled    : lnc- lcr- ucr+
##&nbsp; Deassertions Enabled  : lnc- lcr- ucr+
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'upper_non-recoverable' => 'na',
##&nbsp;            'status' => 'ok',
##&nbsp;            'sensor_id' => '0x12',
##&nbsp;            'deassertions_enabled' => 'lnc- lcr- ucr+',
##&nbsp;            'assertion_events_' => undef,
##&nbsp;            'lower_non-recoverable' => 'na',
##&nbsp;            'lower_non-critical' => '4.343',
##&nbsp;            'sensor_type_analog' => 'Voltage',
##&nbsp;            'sensor_name' => 'P5V_STBY',
##&nbsp;            'sensor_reading' => '5.051',
##&nbsp;            'upper_critical' => '5.807',
##&nbsp;            'lower_critical' => '4.246',
##&nbsp;            'upper_non-critical' => '5.661',
##&nbsp;            'entity_id' => '7.97',
##&nbsp;            'assertions_enabled' => 'lnc- lcr- ucr+',
##&nbsp;            'units' => 'Volts'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub system_sensors_get {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @result_array;
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @arraylines =
        get_lines( raw_output => $output, response_object => $resp );
        shift @arraylines if ($arraylines[0] =~ /Locating sensor record/);
        foreach my $lines (@arraylines) {
            my @values = split('\s+\:\s+', $lines);
            if ($values[0] =~ /Sensor ID/){
                if($values[1]=~/(\S+)\s+\((\S+)\)/){
                    $result_array[0]->{'sensor_name'} = $1;
                    $result_array[0]->{'sensor_id'} = $2;
                }
            }
            elsif($values[0] =~ /Sensor Reading/){
                if($values[1]=~/(\S+)\s+\(.*\)\s+(.*)/){
                    $result_array[0]->{'sensor_reading'} = $1;
                    $result_array[0]->{'units'} = $2;
			    }
            }
            else {
                $result_array[0]->{to_key_name(item=>$values[0])}=$values[1];
            }
        }
    return \@result_array;
}    ##end of system_sensors_get

## @name ::system_sensors_monitor
##
## @summary Parses the raw output from the "system sensors monitor" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system sensors monitor with any of its options as described in the
##    corresponding cdef file 'system_sensors.cdefs'
## @synopsis
##      system_sensors_monitor(%args);
## INPUTS TO METHOD CALL 'system sensors monitor' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'monitor' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = system sensors monitor 1; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;system sensors monitor 1
##&nbsp;The system sensors monitoring has been enabled.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;             'monitor' => 'The system sensors monitoring has been enabled.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub system_sensors_monitor {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @result_array;
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @arraylines =
      get_lines( raw_output => $output, response_object => $resp );
    foreach my $lines (@arraylines) {
        if ( $lines =~ /(.*)/ ) {
            $result_array[0]->{"monitor"} = $1;
        }
    }
    return \@result_array;
}    ##end of system_sensors_monitor

## @name ::system_sensors_thermal_mgmt
##
## @summary Parses the raw output from the "system sensors thermal-mgmt" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system sensors thermal-mgmt with any of its options as described in the
##    corresponding cdef file 'system_sensors.cdefs'
## @synopsis
##      system_sensors_thermal_mgmt(%args);
## INPUTS TO METHOD CALL 'system sensors thermal-mgmt' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'thermal-mgmt' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = system sensors thermal-mgmt 1; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;system sensors thermal-mgmt 1
##&nbsp;The system thermal management has been enabled.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;             'thermal_mgmt' => 'The system thermal management has been enabled.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub system_sensors_thermal_mgmt {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my @result_array;
    my $error = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @arraylines =
      get_lines( raw_output => $output, response_object => $resp );
    foreach my $lines (@arraylines) {
        if ( $lines =~ /(.*)/ ) {
            $result_array[0]->{"thermal_mgmt"} = $1;
        }
    }
    return \@result_array;
}    ##end of system_sensors_thermal_mgmt

## @name ::system_sensors_set_thresh
##
## @summary Parses the raw output from the "system sensors set-thresh" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system sensors set-thresh with any of its options as described in the
##    corresponding cdef file 'system_sensors.cdefs'
## @synopsis
##      system_sensors_set_thresh(%args);
## INPUTS TO METHOD CALL 'system sensors set-thresh' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'sensor_name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'set_thresh' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = system sensors set-thresh CPU_VCC unc 1.32; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;system sensors set-thresh CPU_VCC unc 1.32
##&nbsp;Setting sensor "CPU_VCC" Upper Non-Critical threshold to 1.320
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;             'set_thresh' => 'Setting sensor "CPU_VCC" Upper Non-Critical threshold to 1.320'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub system_sensors_set_thresh {
    my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;
    my $error_message;
    my @result_array;
    $error = _parse_for_errors(
        raw_output      => $output,
        response_object => $resp,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @arraylines =
      get_lines( raw_output => $output, response_object => $resp );
    foreach my $lines (@arraylines) {
        if ( $lines =~ /(.*)/ ) {
            $result_array[0]->{"set_thresh"} = $1;
        }
    }
    return \@result_array;
}    #end of system_sensors_set_thresh

## @name ::system_sensors_set_fan
##
## @summary Parses the raw output from the "system sensors set-fan" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system sensors set-fan with any of its options as described in the
##    corresponding cdef file 'system_sensors.cdefs'
## @synopsis
##      system_sensors_set_fan(%args);
## INPUTS TO METHOD CALL 'system sensors set-fan' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'sensor-name' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'duty-cycle' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
sub system_sensors_set_fan {
 my %args        = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    my $error;
    my $error_message;
    my @result_array;
    $error = _parse_for_errors(
        raw_output      => $output,
        response_object => $resp,
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
} #end of system_sensors_set_fan

1;
