#
#Copyright (c) 2014 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse cp command output
##
## @description
##    This is a library to parse the raw output of cp command.
##
## @synopsis
##    use NACL::APISet::Node::CLI::SPShell::default::cp;
##
##  cp(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::SPShell::default::cp;

use NACL::APISet::Exceptions::InvalidParamValueException;

use strict;
use warnings;

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $resp          = $args{response_object};
    my $error_message;
    my $error         = undef;
    if ( $raw_output =~ /cp:(.*)/ ) {
		$error         = "InvalidParamValue";
        $error_message = $1;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error


## @name ::cp
##
## @summary Parses the raw output from the "cp" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from cp with any of its options as described in the 
##    corresponding cdef file 'cp.cdefs'
## @synopsis
##      cp(%args);
## INPUTS TO METHOD CALL 'cp' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'destination-path' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'source-path' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = cp temp1 temp2
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub cp {
    my %args = @_;
    my ( @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    _check_error("raw_output" => $output, "response_object" => $resp );
    return [];
} ## end sub cp
1;
