# $Id$
#---------------------------------------------------
# Copyright (c) 2014 Network Appliance, Inc.
# All rights reserved.
#---------------------------------------------------
## @summary
##      Wrapper Utility for Selnium webDriver functions.
##
## @description
##      This modules contains all helper functions for SM libraries
##      to interact with selenium webdriver
##
## @synopsis
##      use NACL::APISet::Node::GUI::CMode::GUI_Utils;
##
##      _double_click(%args);
##      _find_element(%args);
##      captureResponseTime(%args);
##      captureStartTime(%args);
##      check_ascending(%args);
##      check_descending(%args);
##      check_for_error(%args);
##      _check_for_errordialog(%args);
##      _check_error_field(%args);
##      check_for_warning(%args);
##      clear_window(%args);
##      _click(%args);
##      click_button(%args);
##      click_child_row_with_texts(%args);
##      click_EventTime(%args);
##      click_item_in_list(%args);
##      click_nowait(%args);
##      click_row_with_texts(%args);
##      click_row_with_texts_without_ascending(%args);
##      click_timecount(%args);
##      cluster_tree_navigate(%args);
##      collapse_tree(%args);
##      config_info(%args);
##      control_key_down(%args);
##      control_key_up(%args);
##      does_row_contain_exact_text(%args);
##      does_row_contain_texts(%args);
##      does_row_contain_texts_without_ascending(%args);
##      drag_and_drop(%args);
##      eventWaitingTime(%args);
##      fire_event(%args);
##      get_all_window_ids(%args);
##      get_all_window_names(%args);
##      get_all_window_titles(%args);
##      get_attribute(%args);
##      get_disks_details(%args);
##      get_grid_data(%args);
##      get_row_data_with_text(%args);
##      get_text(%args);
##      _get_value(%args);
##      getGridDataAsHashMap(%args);
##      is_button_enabled(%args);
##      is_checked(%args);
##      is_element_present(%args);
##      key_down(%args);
##      key_up(%args);
##      keys(%args);
##      manage_system(%args);
##      mouse_down(%args);
##      mouse_down_only(%args);
##      mouse_over(%args);
##      mouse_up(%args);
##      node_tree_navigate(%args);
##      refreshBrowser(%args);
##      reset_filters(%args)
##      select_window(%args);
##      set_checkbox(%args);
##      set_checkbox_in_row_with_text(%args);
##      set_combobox_value(%args);
##      select_from_dropdown
##      set_option(%args);
##      set_radiobutton_in_row_with_text(%args);
##      set_timeout(%args);
##      set_filters(%args);
##      set_checkbox_filters(%args);
##      set_cond_filters(%args);
##      stop(%args);
##      type(%args);
##      type_in_row_with_text(%args);
##      type_keys(%args);
##      wait_for_element_not_present(%args);
##      wait_for_element_present(%args);
##      wait_for_loading(%args);
##      wait_for_page_to_load(%args);
##
## @status shared
## @author ng-sysmgr-automation-core
## @change 20/8/2014 : neelkant : Added webDriver capability
## @pod here

=head1 NAME

NACL::APISet::Node::GUI::CMode::GUI_Utils

=head1 DESCRIPTION

NACL::APISet::Node::GUI::CMode::GUI_Utils

=cut

package NACL::APISet::Node::GUI::CMode::GUI_Utils;

#
## Compiler directives.
#
use strict;
use warnings;

## Module import.
use Selenium::Remote::WDKeys;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use NACL::APISet;
use NATE::BaseException qw(:try);
use NACL::APISet::Node::GUI::CMode::GlobalConst;
use Params::Validate qw(validate_with SCALAR ARRAYREF UNDEF BOOLEAN);

use base qw( Selenium::Remote::Driver );

use Cwd qw(abs_path);
use NATE::ParamSet;

my $nate_params = NATE::ParamSet->new( global => 1 );
my $logdir      = $nate_params->get('TOP_LOGDIR') ||
                    $nate_params->get('NATE_DB_DIR_TEMP');
my $runid       = $nate_params->get("RUNID");

our $screenshot = abs_path($logdir) . "/$runid" . "_captured_screenshot_";
our $s_count    = 0;

=head1 METHODS

=head2 rightClick

Returns BOOLEAN

=over

=item Options

=over

=item arguments_array

$element,$linkText

=back

=back

=over

=back

=cut

sub rightClick {
    $Log->enter() if $may_enter;
    my ( $self, $element, $linkText ) = @_;

    $Log->comment("Right Click Element $linkText");
    $self->mouse_move_to_location( 'element' => $element );
    $Log->comment("Move to Successful");
    $self->click('RIGHT');
    sleep(2);
    $Log->comment("Click $element Successful");
    if ( $linkText ) {
        $self->_find_element($linkText,"link_text")->click();
    }
    $Log->comment("Right Click Element Successful");
    $Log->exit() if $may_exit;

    return 1;
}

=head2 getMatchedRow

Returns BOOLEAN

=over

=item Options

=over

=item arguments_array

$element,$linkText

=back

=back

=over

=back

=cut

sub getMatchedRow{
    return 1;
}

###############################################################################
## @name:: click_button
##
## @summary
##      Click the button with checking pre and post conditions.
## @description
##     This method can be used to click the button with checking pre and post conditions.
## @synopsis
##     click_button(%args);
## INPUTS TO METHOD CALL 'click_button' ON APISET OBJECT.
sub click_button {
    my ( $self, $debugID ) = @_;
    $Log->enter() if $may_enter;

    if ( $self->is_button_enabled($debugID) ) {
        $self->check_for_error();
        $self->_click($debugID);
        $self->check_for_error();
    }
    else {
        $self->capture_screenshot($screenshot.$s_count++.".png");
        NATE::BaseException->throw("The Button \"$debugID\" is not enabled");
    }
    $Log->exit() if $may_exit;
    return 1;

} ## end sub click_button

sub select_from_dropdown {
    my ( $self, $debugID, $value, $partial ) = @_;
    $Log->enter() if $may_enter;
    $Log->comment("Selecting \"$value\" from dropdown");

    my ( $retValue, $comboInputID );

    $self->wait_for_element_present("$debugID");
    if ( $debugID =~ /input$/ ) {
        $comboInputID = $debugID;
        $retValue = $self->_get_value($comboInputID);
    } elsif ( $debugID =~ /img$/ ) {
        $comboInputID = $debugID;
        $debugID =~ s/img$/input/;
        $retValue = $self->_get_value($debugID);
    }else {
        if ( $self->is_element_present("$debugID-input") ) {
            $comboInputID = "$debugID-input";
        } elsif ( $self->is_element_present("$debugID/input") ) {
            $comboInputID = "$debugID/input";
        } elsif ( $self->is_element_present("//div[\@id='$debugID']/input") ) {
            $comboInputID = "//div[\@id='$debugID']/input";
        } else {
            $Log->comment("Debug ID not found $debugID");
            NATE::BaseException->throw("Debug ID not found $debugID");
        }
        $retValue = $self->_get_value($comboInputID);
    }
    $Log->comment("default-selected value : $retValue");

    if ( $partial and $retValue !~ /$value/ ) {
        $self->mouse_down($comboInputID);
        if ( !$self->is_element_present("//div[contains(\@class,'x-combo-list-inner')]")
         and !$self->is_element_present("//div[contains(\@class,'x-combo-list-item')]")
            ) {
            $self->mouse_down($comboInputID);
        }
        if ( $self->is_element_present("//div[contains(\@class,'x-combo-list-inner')]") ) {
            my $div_item = "//div[contains(\@class,'x-combo-list-inner')]"
                     . "//div[contains(text(),'$value')]";
            my $span_item = "//div[contains(\@class,'x-combo-list-inner')]"
                     . "//span[contains(text(),'$value')]";
            my $div_2_item = "//div[contains(\@class,'x-combo-list-inner') and contains(text(),'$value')]";
    
            if ( $self->is_element_present($div_item) ){
                $self->mouse_down($div_item);
            } elsif( $self->is_element_present($span_item) ){
                $self->mouse_down($span_item);
            } elsif ( $self->is_element_present($div_2_item) ){
                $self->mouse_down($div_2_item);
            } else {
                 $Log->comment("Item $value not found in $comboInputID");
                 NATE::BaseException->throw(
                     "Value: \"$value\" not present in combobox list");
            }
        } elsif (
            $self->is_element_present("//div[contains(\@class,'x-combo-list-item')]") ){
            my $div_item = "//div[contains(\@class,'x-combo-list-item')]"
                     . "//div[contains(text(),'$value')]";
            my $span_item = "//div[contains(\@class,'x-combo-list-item')]"
                     . "//span[contains(text(),'$value')]";
            my $div_2_item = "//div[contains(\@class,'x-combo-list-item') and contains(text(),'$value')]";

            if ( $self->is_element_present($div_item) ){
                $self->mouse_down($div_item);
            } elsif( $self->is_element_present($span_item) ){
                $self->mouse_down($span_item);
            } elsif ( $self->is_element_present($div_2_item) ){
                $self->mouse_down($div_2_item);
            } else {
                 $Log->comment("Item $value not found in $comboInputID");
                 NATE::BaseException->throw(
                     "Value: \"$value\" not present in combobox list");
            }
        }
    } elsif ( $retValue ne $value ) {
        $self->mouse_down($comboInputID);
        my $div_item = "//div[contains(\@class,'x-combo-list-inner')]"
                 . "/div[text()='$value')]";
        my $span_item = "//div[contains(\@class,'x-combo-list-inner')]"
                 . "/div/span[text()='$value']";

        if ( $self->is_element_present($div_item) ){
            $self->mouse_down($div_item);
        } elsif ($self->is_element_present($span_item) ){
            $self->mouse_down($span_item);
        } else {
            NATE::BaseException->throw(
                "Value: \"$value\" not present in combobox list");
         }
    } else {
        $Log->comment("Item $value already selected ");
    }

    $Log->exit() if $may_exit;
    return 1;
} ## end sub select_from_dropdown

###############################################################################
## @name:: click_nowait
##
## @summary
##      This will click the button.
## @description
##     This method can be used to click the button.
## @synopsis
##     click_nowait(%args);
## INPUTS TO METHOD CALL 'click_nowait' ON APISET OBJECT.

sub click_nowait {
    $Log->enter() if $may_enter;
    my ( $self, $debugID ) = @_;
    $self->_click($debugID);
    $Log->exit() if $may_exit;
    return 1;
}
###############################################################################
## @name:: manage_system
##
## @summary
##      This function opens a system in new tab and authenticates it.
## @description
##      This function opens a system in new tab and authenticates it.
## @synopsis
##     manage_system(%args);
## INPUTS TO METHOD CALL 'manage_system' ON APISET OBJECT.

sub manage_system {

    my ($self) = @_;
    $Log->enter() if $may_enter;

    my $systemName  = $self->{$SYSTEM_NAME_KEY};

    if ( $self->is_element_present($CLUSTER_TAB)) {
        $Log->comment("system already open for management");
        $Log->exit() if $may_exit;
        return 1;
    }

    eval {
        # handling Authentication window
        $self->wait_for_element_present($LOGIN_PAGE_USERNAME, $GENERIC_WAIT);

        #Code for log popup
        if ( $self->is_element_present(
                "//button[\@class='gwt-Button' and text()='Minimize']")) {
            $Log->comment("Minimise Log popup .....");
            $self->_click(
                "//button[\@class='gwt-Button' and text()='Minimize']");
        }

        if ($self->is_element_present($LOGIN_PAGE_USERNAME)){

            $Log->comment( "Entering Username : "
                         . $self->{$SYSTEM_USERNAME_KEY} );
            $self->type( $LOGIN_PAGE_USERNAME,
                           $self->{$SYSTEM_USERNAME_KEY} );
            $Log->comment( "Entering Password : "
                         . $self->{$SYSTEM_PASSWORD_KEY} );
            $self->type( $LOGIN_PAGE_PASSWORD,
                           $self->{$SYSTEM_PASSWORD_KEY} );

            # Code to handle Code coverage runs.
            if ( $self->is_element_present($LOGIN_PAGE_SYSTEM_IP) ) {
                $Log->comment( "Entering Storage System IP: "
                             . $self->{$SYSTEM_IP_KEY} );
                $self->type($LOGIN_PAGE_SYSTEM_IP,
                            $self->{$SYSTEM_IP_KEY} );
            }

            $Log->comment("Clicking Connect button");
            $self->_click($LOGIN_PAGE_SUBMIT_BTN);

            $self->wait_for_element_present(
               "//span[\@aria-level=1 and text()='Cluster']",
               $GENERIC_WAIT );
        } else {
            die "Timed out Waiting for Login dialog";
        }
    };
    if ($@ =~ /Timed out Waiting/ ) {
        # Continue since authentication dialog may not appear for the
        # second time access to the same system
        $Log->comment("Error encountered.Authentication "
                    . "not required. Accessing second time");
    } elsif ($@) {
        $Log->comment(
            "Unkown Error encountered in GUI_Utils::manage_system() : $@");
    }
    $self->check_for_error();

    #Making sure that system Page is loaded
    my $treeElement =
        "//div[contains(\@id,'navigation-tree')]//span[contains(text(),"
      . " '$systemName' ) and $TREE_ELEMENT_CLASS_ID]";
    $Log->comment("Waiting for system page to load");
    $self->wait_for_element_present( $treeElement, $GENERIC_WAIT );
    $self->wait_for_loading();

    $Log->comment("System opened for management.\n");
    $Log->exit() if $may_exit;
    return 1;
} ## end sub manage_system

###############################################################################
## @name:: cluster_tree_navigate
##
## @summary
##      This function navigates to specific tree element in Clusters left tree of a node.
## @description
##      This function navigates to specific tree element in Clusters left tree of a node.
## @synopsis
##     node_tree_navigate(%args);
## INPUTS TO METHOD CALL 'cluster_tree_navigate' ON APISET OBJECT.

sub node_tree_navigate {
    my ( $self, $nodename, $element ) = @_;
    my ( $menus, $timetaken, $aria_level );

    $Log->enter() if $may_enter;

    $self->_tab_navigation("nodes");

    if($self->is_element_present("//span[contains(\@qtip,'$nodename')]")) {
        # HA
        $self->_double_click("//span[contains(\@qtip,'$nodename')]");
        $aria_level = 4;
    } else {
        # Non HA
        $aria_level = 3;
    }

    if ( $nodename ) {
        $menus = $nodename;
        if( $element ) {
            if ($element eq "Storage" || $element eq "Configuration" ) {
                $menus .= ">$element";
            } elsif ( $element eq "Aggregates" || $element eq "Disks" ) {
                $menus .= ">Storage>$element";
            } elsif ($element eq "AutoSupport" || $element eq "Flash Cache" ) {
                $menus .= ">Configuration>$element";
            }
        } 
    }
    
    $timetaken = $self->_left_tree_navigate( $aria_level, $menus );
    $Log->debug("Time taken for left pane Navigation : $timetaken");

    $Log->exit() if $may_exit;
    return $timetaken;
} ## end sub node_tree_navigate

###############################################################################
## @name:: cluster_tree_navigate
##
## @summary
##      This function navigates to specific tree element in Clusters left tree of a node.
## @description
##      This function navigates to specific tree element in Clusters left tree of a node.
## @synopsis
##     cluster_tree_navigate(%args);
## INPUTS TO METHOD CALL 'cluster_tree_navigate' ON APISET OBJECT.

sub cluster_tree_navigate {
    my ( $self, $scope, $element ) = @_;
    $Log->enter() if $may_enter;
    my ( $menus, $timetaken );

    $Log->debug(
       "sub cluster_tree_navigate(): Navigating to tab : $scope" );

    if ( $scope and $scope eq "cluster" ) {
        my $cluster = $scope;
        $self->_tab_navigation("cluster");

        if ( $element eq $cluster ) {
            $menus = $cluster;
        } elsif ($element eq "Storage"
            || $element eq "Configuration"
            || $element eq "Diagnostics"
            || $element eq "High Availability"
            || $element eq "Advanced" ) {
            $menus = $cluster . ">" . $element;
        } elsif ( $element eq "Aggregates" || $element eq "Storage Pools" || $element eq "Disks" ) {
            $menus = $cluster . ">Storage>" . $element;
        } elsif ($element eq "Services"
            || $element eq "Security"
            || $element eq "Network"
            || $element eq "System Tools"
            || $element eq "Snapshot Policies"
            || $element eq "Schedules"
            || $element eq "Peers" ) {
            $menus = $cluster . ">Configuration>" . $element;
        } elsif ( $element eq "LDAP" || $element eq "Kerberos" ) {
            $menus = $cluster . ">Configuration>Services>" . $element;
        } elsif ( $element eq "Users" || $element eq "Roles" ) {
            $menus = $cluster . ">Configuration>Security>" . $element;
        } elsif ($element eq "Licenses"
            || $element eq "Date and Time"
            || $element eq "SNMP" ) {
            $menus = $cluster . ">Configuration>System Tools>" . $element;
        } elsif ($element eq "Job"
            || $element eq "System Health"
            || $element eq "Logs" ) {
            $menus = $cluster . ">Diagnostics>" . $element;
        } elsif ( $element eq "Events" || $element eq "SnapMirror Log" ) {
            $menus = $cluster . ">Diagnostics>Logs>" . $element;
        } else {
            NATE::BaseException->throw("$element is a wrong Element in Tree.");
        }
        $timetaken = $self->_left_tree_navigate( 2, $menus );
    } else {
        my $vserverName = $scope or undef ;
        my $clusterName = $self->{$SYSTEM_NAME_KEY};
        $self->_tab_navigation("vserver");

        if ( $element eq $clusterName ) {
            $menus = $clusterName;
        } elsif ( $element eq $vserverName ) {
            $menus = $clusterName . ">" . $vserverName;
        } elsif ($element eq "Storage"
            || $element eq "Configuration"
            || $element eq "Policies"
            || $element eq "Protection" ) {
            $menus = $clusterName . ">" . $vserverName . ">" . $element;
        } elsif ($element eq "Volumes"
            || $element eq "LUNs"
            || $element eq "Qtrees"
            || $element eq "Quotas"
            || $element eq "Shares"
            || $element eq "Namespace" ) {
            $menus = $clusterName . ">" . $vserverName . ">Storage>" . $element;
        } elsif ($element eq "Export Policies"
            || $element eq "Snapshot Policies"
            || $element eq "Protection Policies"
            || $element eq "QoS Policy Groups"
            || $element eq "Efficiency Policies" ) {
            $menus =
              $clusterName . ">" . $vserverName . ">Policies>" . $element;
        } elsif ($element eq "Network Interfaces"
            || $element eq "Protocols"
            || $element eq "Security"
            || $element eq "Services"
            || $element eq "Local Users and Groups" ) {
            $menus =
              $clusterName . ">" . $vserverName . ">Configuration>" . $element;
        } elsif ($element eq "CIFS"
            || $element eq "NFS"
            || $element eq "iSCSI"
            || $element eq "FC/FCoE" ) {
            $menus =
                $clusterName . ">"
              . $vserverName
              . ">Configuration>Protocols>"
              . $element;
        } elsif ($element eq "DNS/DDNS Services"
            || $element eq "LDAP"
            || $element eq "LDAP Client"
            || $element eq "LDAP Configuration"
            || $element eq "Kerberos"
            || $element eq "NIS" ) {
            $menus =
                $clusterName . ">"
              . $vserverName
              . ">Configuration>Services>"
              . $element;
        } elsif ( $element eq "Name Mapping" || $element eq "UNIX" ) {
            $menus =
                $clusterName . ">"
              . $vserverName
              . ">Configuration>Local Users and Groups>"
              . $element;
        } elsif ( $element eq "Users" || $element eq "Roles" ) {
            $menus =
                $clusterName . ">"
              . $vserverName
              . ">Configuration>Security>"
              . $element;
        } elsif ( $element eq "Infinite Volume") {
            $menus = $clusterName . ">" . $vserverName . ">Storage>" . $element;
        }
        else {
            $self->capture_screenshot($screenshot.$s_count++.".png");
            NATE::BaseException->throw("$element is a wrong Element.");
        }

        $timetaken = $self->_left_tree_navigate( 2, $menus );
    } ## end else [ if ($scope eq $self->{...

    $Log->debug(
       "Time taken for left pane Navigation : $timetaken");
    $Log->exit() if $may_exit;
    return $timetaken;
} ## end sub cluster_tree_navigate

###############################################################################
## @name:: clear_window
##
## @summary
##      This function will check and close any leftover dialog boxes
## @description
##      This function will check and close any leftover dialog boxes
## @synopsis
##     clear_window(%args);
## INPUTS TO METHOD CALL 'clear_window' ON APISET OBJECT.

sub clear_window {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    my $loop_count = 0;

    while (  $self->is_element_present($GENERIC_DIALOG_TITLE_BAR)
          or $self->is_element_present($GENERIC_DIALOG_TITLE_BAR1) ) {
        $Log->comment(
            "Closing dialog boxes left over by previously failed testcase.");
        if ($self->is_element_present($GENERIC_DIALOG_CLOSE)) {
             $self->click($GENERIC_DIALOG_CLOSE);
             $self->wait_for_loading();
        }
        if ($self->is_element_present($GENERIC_DIALOG2_OK_BUTTON)) {
             $self->click($GENERIC_DIALOG2_OK_BUTTON);
             $self->wait_for_loading();
        }
        if ($self->is_element_present($GENERIC_DIALOG_OK_BUTTON)) {
             $self->click($GENERIC_DIALOG_OK_BUTTON);
             $self->wait_for_loading();
        }
        if ($self->is_element_present($GENERIC_DIALOG_CANCEL_BUTTON)) {
             $self->mouse_down($GENERIC_DIALOG_CANCEL_BUTTON);
             $self->wait_for_loading();
        }
        if ($self->is_element_present($GENERIC_DIALOG_YES_BUTTON)) {
             $self->mouse_down($GENERIC_DIALOG_YES_BUTTON);
             $self->wait_for_loading();
        }
        if ($loop_count++ > 5) {
            NATE::BaseException->throw("clear_window() exceeded loop count");
        }
    } ## end while ($self->is_element_present...

    $Log->exit() if $may_exit;
    return 1;
} ## end sub clear_window

###############################################################################
## @name:: collapse_tree
##
## @summary
##      This function will collapse the left system tree.
## @description
##      This function is used to collape the opened view.
## @synopsis
##     collapse_tree(%args);
## INPUTS TO METHOD CALL 'collapse_tree' ON APISET OBJECT.

sub collapse_tree {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    $Log->comment("Collapsing left tree.....");
    if ( $self->is_element_present(
            "//span[text()='Volumes' and $TREE_ELEMENT_CLASS_ID]")
      ) {
        $self->_double_click(
            "//span[text()='Storage' and $TREE_ELEMENT_CLASS_ID]");
    }
    if ( $self->is_element_present(
            "//span[text()='Syslog' and $TREE_ELEMENT_CLASS_ID]")
      ) {
        $self->_double_click(
            "//span[text()='System Tools' and $TREE_ELEMENT_CLASS_ID]");
    }
    if ( $self->is_element_present(
            "//span[text()='SSH/SSL' and $TREE_ELEMENT_CLASS_ID]")
      ) {
        $self->_double_click(
            "//span[text()='Security' and $TREE_ELEMENT_CLASS_ID]");
    }
    if ( $self->is_element_present(
            "//span[text()='NFS' and $TREE_ELEMENT_CLASS_ID]")
      ) {
        $self->_double_click(
            "//span[text()='Protocols' and $TREE_ELEMENT_CLASS_ID]");
    }
    if ( $self->is_element_present(
            "//span[text()='Interfaces' and $TREE_ELEMENT_CLASS_ID]")
      ) {
        $self->_double_click(
            "//span[text()='Networking' and $TREE_ELEMENT_CLASS_ID]");
    }
    if ( $self->is_element_present(
            "//span[text()='Users' and $TREE_ELEMENT_CLASS_ID]")
      ) {
        $self->_double_click(
            "//span[text()='Local Users and Groups' and $TREE_ELEMENT_CLASS_ID]"
        );
    }
    if ( $self->is_element_present(
            "//span[text()='Networking' and $TREE_ELEMENT_CLASS_ID]")
      ) {
        $self->_double_click(
            "//span[text()='Configuration' and $TREE_ELEMENT_CLASS_ID]");
    }
    if ( $self->is_element_present(
            "//span[text()='Session' and $TREE_ELEMENT_CLASS_ID]")
      ) {
        $self->_double_click(
            "//span[text()='Diagnostics' and $TREE_ELEMENT_CLASS_ID]");
    }
    if ( $self->is_element_present(
            "//span[text()='Storage' and $TREE_ELEMENT_CLASS_ID]")
      ) {
        $self->_double_click(
            "//span[text()='$self->{$SYSTEM_NAME_KEY}' and $TREE_ELEMENT_CLASS_ID]"
        );
    }
    $Log->exit() if $may_exit;
    return 1;
} ## end sub collapse_tree

###############################################################################
## @name:: check_for_error
##
## @summary
##      This function will check for all kinds of possible errors after completing a wizard.
## @description
##      This function will check for all kinds of possible errors after completing a wizard.
## @synopsis
##     check_for_error(%args);
## INPUTS TO METHOD CALL 'check_for_error' ON APISET OBJECT.

sub check_for_error {

    my ($self) = @_;
    $Log->enter() if $may_enter;
    $self->_check_error_field();
    $self->_check_for_errordialog();
    $Log->exit() if $may_exit;
    return 1;
}

###############################################################################
## @name:: wait_for_loading
##
## @summary
##      This function waits till the loading message disapears.
## @description
##      This function This function waits till the loading message disapears.
## @synopsis
##     check_for_error(%args);
## INPUTS TO METHOD CALL 'wait_for_loading' ON APISET OBJECT.

sub wait_for_loading {
    my ( $self, $timeout ) = @_;
    $Log->enter() if $may_enter;
    $Log->comment("Waiting for Page to load......");
    sleep(1);
    my $sleep_count;

    if ( defined $timeout ) {
        $sleep_count = $timeout / 1000;
    } else {
        $sleep_count = $GENERIC_WAIT / 1000;
    }

    while ( $self->is_element_present($GENERIC_PLEASE_WAIT) ) {
        $Log->comment("Waiting for Page to load......");
        sleep(1);
        $sleep_count = $sleep_count - 1;
        if ( $sleep_count == 0 ) {
            NATE::BaseException->throw("Page load timeout.");
        }
    }
    $Log->exit() if $may_exit;
} ## end sub wait_for_loading

###############################################################################
## @name:: click_timecount
## @summary
##      This function perform click action and return time required to complete the action.
## @description
##      This function Perform click action and return time required to complete the action.
## @synopsis
##     click_timecount(%args);
## INPUTS TO METHOD CALL 'click_timecount' ON APISET OBJECT.

sub click_timecount {
    my ( $self, $curentID, $type, $nextID ) = @_;
    my $time_out = 300000;
    my ( $sec, $end, $start, $time_sec, $final_values );
    $Log->enter() if $may_enter;

    #Check for loading times
    if ( $curentID =~ /gwt-debug/ ) {
        if ( not $self->is_button_enabled($curentID) ) {
            $self->capture_screenshot($screenshot.$s_count++.".png");
            NATE::BaseException->throw(
                "The Button \"$curentID\" is not enabled");
        }
    }
    my $sleep_count = 300;

    $start = (time)[0];
    $self->_click($curentID);
    if ( $type and $type eq 'refresh' ) {
        $self->wait_for_loading($time_out);
        $self->check_for_error();
    } elsif ( $type and $type eq 'next' ) {
        $self->wait_for_loading($time_out);
        $self->check_for_error();
        $self->wait_for_element_present( $nextID, $time_out );
        while ( not $self->is_button_enabled($nextID) ) {
            $Log->comment("Waiting for button $nextID to enable ......");
        }
    } else {
        $self->wait_for_loading($time_out);
        $self->check_for_error();
        while ( $self->is_element_present($curentID) ) {
            $Log->comment("Waiting for commit ......");
            sleep(1);
            $self->check_for_error();
            $sleep_count = $sleep_count - 1;
            if ( $sleep_count == 0 ) {
                NATE::BaseException->throw("Page load timeout.");
            }
        }

        $self->wait_for_element_not_present( $curentID, $time_out );
    } ## end else [ if ($type eq 'refresh')
    $end = (time)[0];

    #Timings
    $sec = $end - $start;
    $final_values = sprintf( "%.2f", $sec );
    $self->clear_window();
    return $final_values;
    $Log->exit() if $may_exit;
} ## end sub click_timecount

###############################################################################
## @name ::set_combobox_value
##
## @summary
##     Set a value in NWF combobox using GUI.
## @description
##     This method can be used to Setting a value in the Combobox.
##     This method is used to select a value from the list in the Combobox.
## @synopsis
##     set_combobox_value(%args);
## INPUTS TO METHOD CALL 'set_combobox_value' ON APISET OBJECT

sub set_combobox_value {
    my ( $self, $debugID, $value, $comp ) = @_;
    $Log->enter() if $may_enter;
    $Log->comment("Selecting \"$value\" from combobox");

    my ( $retValue, $comboInputID );

    $self->wait_for_element_present("$debugID");

    if ( $self->is_element_present("$debugID-input") ) {
        $comboInputID = "$debugID-input";
    } elsif ( $self->is_element_present("$debugID/input") ) {
        $comboInputID = "$debugID/input";
    } elsif ( $self->is_element_present("//div[\@id='$debugID']/input") ) {
        $comboInputID = "//div[\@id='$debugID']/input";
    } else {
        $Log->comment("Debug ID not found $debugID");
        return 0;
    }

    $retValue = $self->_get_value($comboInputID);
    if ( defined $comp ) {
        if ( $comp eq 'partial' ) {
            $self->mouse_down("//div[\@id='$debugID']/img");
            eval {
                $self->wait_for_element_present(
                     "//div[contains(\@class,'x-combo-list-item')"
                   . " and contains(text(),'$value')]",
                     $GENERIC_WAIT
                );
            };
            if ($@) {
                NATE::BaseException->throw(
                    "Value: \"$value\" not present in combobox list");
            }
            $self->mouse_down(
                     "//div[contains(\@class,'x-combo-list-item')"
                   . " and contains(text(),'$value')]"
            );

        } ## end if ($comp eq 'partial')
    }
    else {
        if ( $retValue ne $value ) {
            $self->mouse_down("//div[\@id='$debugID']/img");
            eval {
                $self->wait_for_element_present(
                      "//div[contains(\@class,'x-combo-list-item')"
                    . " and text()='$value']",
                    $GENERIC_WAIT
                );
            };
            if ($@) {
                NATE::BaseException->throw(
                    "Value: \"$value\" not present in combobox list");
            }
            $self->mouse_down(
                "//div[contains(\@class,'x-combo-list-item') and text()='$value']"
            );
        } ## end if ($retValue ne $value)
    } ## end else [ if (defined $comp)
    $Log->exit();    
    return 1;
} ## end sub set_combobox_value

###############################################################################
## @name ::set_radiobutton_in_row_with_text
## @summary
##     Enable/Disable Radio Button using GUI.
## @description
##     Based on the input provided, the radio button is enabled or disabled.
## @synopsis
##     set_radiobutton_in_row_with_text(%args);
## INPUTS TO METHOD CALL 'set_radiobutton_in_row_with_text' ON APISET OBJECT
##
sub set_radiobutton_in_row_with_text {
    my ( $self, $debugID, @texts ) = @_;
    $Log->enter() if $may_enter;
    my $baseDebugID =
      "//div[\@id='$debugID']//div/table/tbody/tr";

    my $rowDebugID = $baseDebugID;

    my ( $tempID1, $tempID2, $tempID3, $radioID );

    foreach my $text (@texts) {
        $tempID1 = $baseDebugID
          . "/td//div/table/tbody/tr[td/span/div/label[text()='$text']]";
        $tempID2 = $baseDebugID . "[td//div/label[text()='$text']]";

        if ( $self->is_element_present($tempID1) ) {
            $rowDebugID = $rowDebugID . $tempID1;
            $radioID = $rowDebugID . "//input";
        } elsif ( $self->is_element_present($tempID2) ) {
            $rowDebugID = $rowDebugID . $tempID2;
            $radioID    = $rowDebugID . "//input";
        } else {
            NATE::BaseException->throw(
                "Row with \"$text\" not present in grid \"$debugID\" ");
        }
    } ## end foreach my $text (@texts)

    if ($radioID) {
        $self->_click($radioID);
        $Log->comment("Radiobox with @texts in table row is selected");
    } else {
        $Log->comment("No such text box found in row with @texts");
    }
    $Log->exit() if $may_exit;
    return 1;
} ## end sub set_radiobutton_in_row_with_text

###############################################################################
# Method:    reset_filters
# Objective: reset multiple filters .
# Details:   See POD documentation at the end of this file.
###############################################################################
## ex: $self->reset_filters( $GlobalConst::VOLUME_GRID,
##                         ['Name:abc-vsim1', 'Aggregate:Aggr1']
##                       );
#
sub reset_filters {
    $Log->enter() if $may_enter;
    my ( $self, $gridID, $filters ) = @_;

    foreach my $filter ( @{$filters} ) {
        my ($column_name) = split( /:/, $filter ); 
        my $baseID         = "//div[contains(\@id,'$gridID')]";
        my $filterSortIcon = $baseID
          . "//div[contains(\@role,'columnheader')]/span[contains(text(),'$column_name')]/../a";
        my $filterTextBox = "//div[contains(\@class,'filter-menu-item')]/input";

        if ( $self->is_element_present($filterSortIcon) ) {
            $self->_click($filterSortIcon);
            if ( $self->is_element_present($filterTextBox) ) {
                $Log->comment(
                    "Existing Filter : " . $self->_get_value($filterTextBox) );
                if ( $self->_get_value($filterTextBox) =~ /\w+/ ) {
                    $Log->comment("Reset filters of the column '$column_name'");
                    $self->_find_element($filterTextBox)->clear();
                    if ( $self->_get_value($filterTextBox) =~ /\w+/ ) {
                        $self->type( $filterTextBox," ");
                        $self->type( $filterTextBox, KEYS->{'enter'} );
                    }
                } else {
                    # press Enter key
                    $self->type( $filterTextBox, KEYS->{'enter'} );
                }
            }
        } else {
            $Log->comment(
                "No filer option available for column $column_name" );
        }
    }
    $Log->comment("reset_filters() completed.");
    $Log->exit() if $may_exit;
    return 1;
} ## end sub reset_filters

###############################################################################
## @name ::set_filters
##
## @summary
##     Set multiple filters.
## @description
##     This function can be used to set filter values in columns inside the table.
## @synopsis
##     set_filters(gridid,\@cols);
##
## ex: $self->set_filters( $GlobalConst::VOLUME_GRID,
##                         ['Name:abc-vsim1,vsim2', 'Aggregate:Aggr1']
##                       );
## NOTE: call reset_filters after calling this function.
#
sub set_filters {
    $Log->enter() if $may_enter;

    my ( $self, $gridID, $filters ) = @_;

    foreach my $filter ( @{$filters} ) {

        my ( $column_name, $column_value ) = split( /:/, $filter );

        if ( $column_value =~ /,/ ) {
            my @values = split(/,/,$column_value);
            $column_value = $self->longestCommonPrefix(@values);
        }

        my $baseID         = "//div[contains(\@id,'$gridID')]";
        my $filterSortIcon = $baseID
          . "//div[contains(\@role,'columnheader')]"
          . "/span[contains(text(),'$column_name')]/../a";
        my $filterTextBox = "//div[contains(\@class,'filter-menu-item')]/input";

        $Log->comment( "Filter the column '$column_name' in grid"
              . " $gridID with value '$column_value'" );
        if ( $self->is_element_present($filterSortIcon) ) {
            $self->_click($filterSortIcon);
            $Log->comment("Filter the column '$column_name'");
            if ( $self->is_element_present($filterTextBox) ) {
                $Log->comment("Type in to Filter textbox : ");
                $self->type( $filterTextBox, "$column_value" );
                # press Enter
                $self->key_down(KEYS->{'enter'} );
                $self->key_up(KEYS->{'enter'} );
                $Log->comment($self->_get_value($filterTextBox));
                sleep 1;
                $Log->comment(
                  "Filter for $column_name column is set to $column_value" );
            }
        } else {
            $Log->comment(
                "No filer option available for column $column_name" );
        }
    }
    $Log->comment("set_filters() completed.");
    $Log->exit() if $may_exit;
    return 1;
} ## end sub set_filters

###############################################################################
## @name ::set_checkbox_filters
##
## @summary
##     Set multiple filters.
## @description
##     This function can be used to set filter values in columns inside the table.
## @synopsis
##     set_filters(gridid,\@cols);
##
## ex: $self->set_filters( $GlobalConst::VOLUME_GRID,
##                         ['Name:abc-vsim1,vsim2', 'Aggregate:Aggr1']
##                       );
## NOTE: call reset_filters after calling this function.
#
sub set_checkbox_filters {
    $Log->enter() if $may_enter;

    my ( $self, $gridID, $filters ) = @_;

    foreach my $filter ( @{$filters} ) {

        my ( $column_name, $column_value ) = split( /:/, $filter );

        if ( $column_value =~ /,/ ) {
            my @values = split(/,/,$column_value);
            $column_value = $self->longestCommonPrefix(@values);
        }

        my $baseID         = "//div[contains(\@id,'$gridID')]";
        my $filterSortIcon = $baseID
          . "//div[contains(\@role,'columnheader')]"
          . "/span[contains(text(),'$column_name')]/../a";
        my $filterTextBox = "//div[contains(\@class,'filter-menu-item')]/input";

        $Log->comment( "Filter the column '$column_name' in grid"
              . " $gridID with value '$column_value'" );
        if ( $self->is_element_present($filterSortIcon) ) {
            $self->_click($filterSortIcon);
            $Log->comment("Filter the column '$column_name'");
            if ( $self->is_element_present($filterTextBox) ) {
                $Log->comment("Type in to Filter textbox : ");
                $self->type( $filterTextBox, "$column_value" );
                # press Enter
                $self->key_down(KEYS->{'enter'} );
                $self->key_up(KEYS->{'enter'} );
                $Log->comment($self->_get_value($filterTextBox));
                sleep 1;
                $Log->comment(
                  "Filter for $column_name column is set to $column_value" );
            }
        } else {
            $Log->comment(
                "No filer option available for column $column_name" );
        }
    }
    $Log->comment("set_checkbox_filters() completed.");
    $Log->exit() if $may_exit;
    return 1;
} ## end sub set_checkbox_filters


###############################################################################
## @name ::set_cond_filters
##
## @summary
##     Set multiple filters.
## @description
##     This function can be used to set filter values in columns inside the table.
## @synopsis
##     set_filters(gridid,\@cols);
##
## ex: $self->set_filters( $GlobalConst::VOLUME_GRID,
##                         ['Name:abc-vsim1,vsim2', 'Aggregate:Aggr1']
##                       );
## NOTE: call reset_filters after calling this function.
#
sub set_cond_filters {
    $Log->enter() if $may_enter;

    my ( $self, $gridID, $filters ) = @_;

    foreach my $filter ( @{$filters} ) {

        my ( $column_name, $column_value ) = split( /:/, $filter );

        if ( $column_value =~ /,/ ) {
            my @values = split(/,/,$column_value);
            $column_value = $self->longestCommonPrefix(@values);
        }

        my $baseID         = "//div[contains(\@id,'$gridID')]";
        my $filterSortIcon = $baseID
          . "//div[contains(\@role,'columnheader')]"
          . "/span[contains(text(),'$column_name')]/../a";
        my $filterTextBox = "//div[contains(\@class,'filter-menu-item')]/input";

        $Log->comment( "Filter the column '$column_name' in grid"
              . " $gridID with value '$column_value'" );
        if ( $self->is_element_present($filterSortIcon) ) {
            $self->_click($filterSortIcon);
            $Log->comment("Filter the column '$column_name'");
            if ( $self->is_element_present($filterTextBox) ) {
                $Log->comment("Type in to Filter textbox : ");
                $self->type( $filterTextBox, "$column_value" );
                # press Enter
                $self->key_down(KEYS->{'enter'} );
                $self->key_up(KEYS->{'enter'} );
                $Log->comment($self->_get_value($filterTextBox));
                sleep 1;
                $Log->comment(
                  "Filter for $column_name column is set to $column_value" );
            }
        } else {
            $Log->comment(
                "No filer option available for column $column_name" );
        }
    }
    $Log->comment("set_cond_filters() completed.");
    $Log->exit() if $may_exit;
    return 1;
} ## end sub set_cond_filters

###############################################################################
## @name ::set_checkbox
##
## @summary
##     Set value for a checkbox.
## @description
##     This function can be used to enable or disable the checkbox.
## @synopsis
##     set_checkbox(%args);
## INPUTS TO METHOD CALL 'set_checkbox' ON APISET OBJECT
##
sub set_checkbox {
    my ( $self, $debugID, $value ) = @_;
    $Log->enter() if $may_enter;
    my $toogleButtonID;

    if ( $debugID =~ /input$/ ) {
        $toogleButtonID = $debugID;
    } else {
        $toogleButtonID = "//div[\@id='" . $debugID . "']/input";
    }

    $value = lc $value;

    my $retValue = $self->is_checked($toogleButtonID);

    if (   ( $value eq "enable" && not $retValue )
        || ( $retValue && $value eq "disable" ) )
    {
        $Log->comment("Clicking $toogleButtonID to $value it");
        $self->_click($toogleButtonID);
    } elsif ( $value ne "enable" && $value ne "disable" ) {
        NATE::BaseException->throw("Invalid value. Use enable/disable");
    } else {
        $Log->comment("$toogleButtonID is already $value");
    }
    $Log->exit() if $may_exit;
    return 1;
} ## end sub set_checkbox


###############################################################################
## @name ::get_row_data_with_text
##
## @summary
##     Get row data from a table.
## @description
##    This method is used to fetch the row data from a grid, which contains the input text.
## @synopsis
##     geo_row_data_with_text(%args);
## INPUTS TO METHOD CALL 'get_row_data_with_text' ON APISET OBJECT
##

sub get_row_data_with_text {
    my ( $self, $debugID, @texts ) = @_;
    $Log->enter() if $may_enter;
    my $baseDebugID =
      "//div[\@id='$debugID']//tbody/tr";

    my $rowDebugID;

    foreach my $text (@texts) {
        my @tempIDS;
        push( @tempIDS, $baseDebugID . "[td//div[text(),'$text']]" );
        push( @tempIDS, $baseDebugID . "[td//div//span[text(),'$text']]" );
        push( @tempIDS, $baseDebugID . "[td//div[contains(text(),'$text')]]" );
        push( @tempIDS, $baseDebugID . "[td//div//span[contains(text(),'$text')]]" );
        push( @tempIDS, "//div[\@id='$debugID']//div[contains(text(),'$text')]" );
        push( @tempIDS, "//div[\@id='$debugID']//tbody/tr/td/div[contains(text(),'$text')]" );

        foreach my $tempid ( @tempIDS ) {
            if ( $self->is_element_present($tempid) ) {
                $rowDebugID = $tempid;
                $Log->comment("Row with \"$text\" found");
                last;
            }
        }
        unless ($rowDebugID) {
            NATE::BaseException->throw(
                "Row with \"$text\" not present in grid \"$debugID\" ");
        }
    } ## end foreach $text (@texts)
    $Log->comment("Row data : " . $self->get_text($rowDebugID) );
    $Log->exit() if $may_exit;
    return $self->get_text($rowDebugID);
} ## end sub get_row_data_with_text

###############################################################################
## @name ::set_checkbox_in_row_with_text
##
## @summary
##     Set value for checkbox in a grid row.
## @description
##     This method is used to enable or disable checkbox in a row of
##      the grid which contains the input text.
## @synopsis
##     set_checkbox_in_row_with_text(%args);
## INPUTS TO METHOD CALL 'set_checkbox_in_row_with_text' ON APISET OBJECT
##
sub set_checkbox_in_row_with_text {
  my ( $self, $debugID, $value, @texts ) = @_;
    $Log->enter() if $may_enter;
    my $baseDebugID = "//div[\@id='$debugID']//div/table/tbody/tr";
    my $baseDebugID1 = "//div[\@id='$debugID']//table/tbody/tr";
    my $rowDebugID;
    my $checkboxDebugID;
    $Log->comment(" The Values are $debugID, $value, @texts");

    my ( $text,        $checkboxID,  $checkboxIDOn, $checkboxID1,
         $checkboxID2, $checkboxID3, $checkboxID4 );

    foreach my $text (@texts) {
        my @tempIDS;
        push( @tempIDS, $baseDebugID . "[td//div[text(),'$text']]" );
        push( @tempIDS, $baseDebugID . "[td//div//span[text(),'$text']]" );
        push( @tempIDS, $baseDebugID . "[td//div[contains(text(),'$text')]]" );
        push( @tempIDS, $baseDebugID . "/td/div/div/div[div[text()='$text']]" );
        push( @tempIDS, $baseDebugID );
        push( @tempIDS, $baseDebugID1 . "[td/div[text()='$text']]" );

        foreach my $tempid ( @tempIDS ) {
            if ( $self->is_element_present($tempid) ) {
                $rowDebugID = $tempid;
                $Log->comment("Row with \"$text\" found");
                last;
            }
        }
        unless ($rowDebugID) {
            NATE::BaseException->throw(
                "Row with \"$text\" not present in grid \"$debugID\" ");
        }
    } ## end foreach $text (@texts)

    $checkboxID = $rowDebugID
      . "/td/div/div[contains(\@class,'x-grid3-check-col x-grid3-check-col')]";
    $checkboxIDOn = $rowDebugID
      . "/td/div/div[contains(\@class,'x-grid3-check-col x-grid3-check-col-on')]";
    $checkboxID1 = $rowDebugID
      . "/td/div[contains(\@class,'x-grid3-cell-inner x-grid3-col-disk-added')]/div/input[\@type='checkbox']";
    $checkboxID2 = $rowDebugID . "/td/input";
    $checkboxID3 = $rowDebugID . "/div/input";
    $checkboxID4 = $rowDebugID . "/td/div/input";

    if ( $value eq "enable" && $self->is_element_present($checkboxID) ) {
        $self->mouse_down($checkboxID);
    } elsif ($value eq "disable"
        && $self->is_element_present($checkboxIDOn) ) {
        $self->mouse_down($checkboxIDOn);
    } elsif ( $value eq "enable" && $self->is_element_present($checkboxID1) ) {
        $self->_click($checkboxID1);
    } elsif ( $value eq "enable" && $self->is_element_present($checkboxID2) ) {
        $self->_click($checkboxID2);
    } elsif ( $value eq "enable" && $self->is_element_present($checkboxID3) ) {
        $Log->comment(" Printing the ID for temp $checkboxID3");
        $self->_click($checkboxID3);
    } elsif ( $value eq "enable" && $self->is_element_present($checkboxID4) ) {
        $self->_click($checkboxID4);
    } elsif ( $value ne "enable" && $value ne "disable" ) {
        NATE::BaseException->throw("Invalid option ($value). Put enable/disable");
    } elsif ($self->is_element_present($checkboxID)
        || $self->is_element_present($checkboxIDOn) ) {
        $Log->comment( "Already $value" . "d" );
    }
    $Log->exit() if $may_exit;
    return 1;
} ## end sub set_checkbox_in_row_with_text

###############################################################################
## @name ::type_in_row_with_text
##
## @summary
##    Enter input value in the textbox in a row in a grid.
## @description
##     This method is used to enter the input value in the textbox,
##     which is present in the row of a grid which contains the
##     input text.
## @synopsis
##     type_in_row_with_text(%args);
## INPUTS TO METHOD CALL 'type_in_row_with_text' ON APISET OBJECT
##
sub type_in_row_with_text {
    $Log->enter() if $may_enter;

    my ( $self, $debugID, $typeText, @texts ) = @_;

    my $baseDebugID =
      "//div[\@id='$debugID']/div/div/div/div/div/table/tbody/tr";
    my $rowDebugID = $baseDebugID;
    my $checkboxDebugID;

    my ( $text,    $checkboxID, $checkboxIDOn, $checkboxID1 );
    my ( $tempID1, $tempID2,    $tempID3,      $tempID4 );
    foreach $text (@texts) {
        $tempID1 = $baseDebugID . "[td/div[text()='$text']]";
        $tempID2 = $baseDebugID . "[td/div/span[text()=' $text']]";
        $tempID3 = $baseDebugID . "[td/div/div/div/span[text()='$text']]";
        $tempID4 = $baseDebugID . "[td/div/div[contains(text(),'$text')]]";

        if ( $self->is_element_present($tempID1) ) {
            $rowDebugID = $rowDebugID . "[td/div[text()='$text']]";
        } elsif ( $self->is_element_present($tempID2) ) {
            $rowDebugID = $rowDebugID . "[td/div/span[text()=' $text']]";
        } elsif ( $self->is_element_present($tempID3) ) {
            $rowDebugID = $rowDebugID . "[td/div/div/div/span[text()='$text']]";
        } elsif ( $self->is_element_present($tempID4) ) {
            $rowDebugID =
              $rowDebugID . "[td/div/div[contains(text(),'$text')]]";
        } else {
            NATE::BaseException->throw(
                "Row with \"$text\" not present in grid \"$debugID\" ");
        }
    } ## end foreach $text (@texts)

    my $textboxID = $rowDebugID . "/td/div/div/input[\@type='text']";
    if ($textboxID) {
        $self->type( $textboxID, $typeText );
    } else {
        $Log->nlog("No such text box found in row with $text");
    }
    $Log->exit() if $may_exit;

    return 1;
} ## end sub type_in_row_with_text

###############################################################################
## @name ::set_option
##
## @summary
##     Set an option button.
## @description
##     This method is used to set the option.
## @synopsis
##     set_option(%args);
## INPUTS TO METHOD CALL 'set_option' ON APISET OBJECT
##
sub set_option {
    $Log->enter() if $may_enter;

    my ( $self, $debugID ) = @_;
    my $toogleButtonID = "//div[\@id='" . $debugID . "']/input";

    if (
        $self->is_element_present(
            "//div[\@id='$debugID' and contains(\@class,'x-item-disabled')]")
      ) {
        NATE::BaseException->throw("Option $debugID is disabled");
    }

    $Log->comment("Enabling $toogleButtonID");
    $self->click($toogleButtonID);
    $Log->exit() if $may_exit;

    return 1;
} ## end sub set_option

###############################################################################
## @name ::click_row_with_texts
##
## @summary
##     Click a row with matching texts in columns.
## @description
##     This method is used to click on the row of a grid which contains the input text.
## @synopsis
##     click_row_with_texts(%args);
## INPUTS TO METHOD CALL 'click_row_with_texts' ON APISET OBJECT
##
sub click_row_with_texts {
    my ( $self, $debugID, $partial, $doubleclick, @texts );
    $Log->enter() if $may_enter;
    my $rowDebugID;
    $self    = shift;
    $debugID = shift;

    if (   ( $_[0] eq "1" || $_[0] eq "0" || $_[0] eq "2" )
        && ( $_[1] eq "1" || $_[1] eq "0" ) ) {
        $partial     = shift;
        $doubleclick = shift;
    } else {
        $partial     = 0;
        $doubleclick = 0;
    }

    @texts = @_;

    if ( $partial eq 2 ) {
        my $baseDebugID =
          "//div[\@id='$debugID']//tbody/tr";
        foreach my $text (@texts) {
            my $tempID1 = $baseDebugID . "[td//div[text()='$text']]";
            my $tempID2 = $baseDebugID . "[td//div/span[text()=' $text']]";

            if ( $self->is_element_present($tempID1) ) {
                $rowDebugID = $tempID1;
            } elsif ( $self->is_element_present($tempID2) ) {
                $rowDebugID = $tempID2;
            } else {
                $Log->comment(
                    "Row with \"$text\" not present in grid \"$debugID\" ");
                NATE::BaseException->throw(
                    "Row with \"$text\" not present in grid \"$debugID\" ");
            }
        } ## end foreach my $text (@texts)

        $Log->comment("Row with \"@texts\" is present in grid \"$debugID\" ");

    } else {
        my $baseDebugID =
          "//div[\@id='$debugID']//tbody/tr";

        my $notfound = 0;

        $Log->comment("Clicking row with text: @texts from the grid");
        my $baseSortID = "//div[\@id='$debugID']//tbody/tr/td[1]/div";

        # looking for string with Default Order
        $Log->comment(
            "Looking for text: @texts with Default order from the grid");

        foreach my $text (@texts) {
            my @tempIDS;
            push( @tempIDS, $baseDebugID . "[td//div[contains(text(),'$text')]]" );
            push( @tempIDS, $baseDebugID . "[td//div/span[contains(text(),'$text')]]" );
            push( @tempIDS, "//div[\@id='$debugID']/div[contains(text(),'$text')]" );
            push( @tempIDS, "//div[\@id='$debugID']//tbody/tr/td/div[contains(text(),'$text')]" );

            foreach my $tempid ( @tempIDS ) {
                if ( $self->is_element_present($tempid) ) {
                    $rowDebugID = $tempid;
                    $Log->comment("Row with \"$text\" found");
                    last;
                }
            }
            unless ($rowDebugID) {
                $notfound = 1;
            }
        }

        if ( $notfound eq 1 ) {

            $notfound = 0;

            # looking for string with ascending order
            $Log->comment(
                "Looking for text: @texts with ascending order from the grid");
            $self->check_ascending($baseSortID);
            sleep(5);

            foreach my $text (@texts) {
                my @tempIDS;
                push( @tempIDS, $baseDebugID . "[td/div[contains(text(),'$text')]]" );
                push( @tempIDS, $baseDebugID . "[td/div/span[contains(text(),'$text')]]" );
                push( @tempIDS, $baseDebugID . "[td/div/div/div/span[contains(text(),'$text')]]" );
                push( @tempIDS, $baseDebugID . "[td/div/div[contains(text(),'$text')]]" );
                push( @tempIDS, "//div[\@id='$debugID']/div[contains(text(),'$text')]" );
                push( @tempIDS, "//div[\@id='$debugID']/div/div/div/div/div/div/div/div/"
                         . "table/tbody/tr/td/div[contains(text(),'$text')]" );
                push( @tempIDS, $baseDebugID . "[td/div/div/span[contains(text(),'$text')]]" );

                foreach my $tempid ( @tempIDS ) {
                    if ( $self->is_element_present($tempid) ) {
                        $rowDebugID = $tempid;
                        $Log->comment("Row with \"$text\" found");
                        last;
                    }
                }
                unless ($rowDebugID) {
                    $notfound = 1;
                }
            }
        }
        # Try one more to look for string with desecending order
        if ( $notfound eq 1 ) {

            # looking for string with descending order
            $self->check_descending($baseSortID);
            sleep(5);
            $Log->comment(
                "Looking for text: @texts with descending order from the grid");

            foreach my $text (@texts) {
                my @tempIDS;
                push( @tempIDS, $baseDebugID . "[td/div[contains(text(),'$text')]]" );
                push( @tempIDS, $baseDebugID . "[td/div/span[contains(text(),'$text')]]" );
                push( @tempIDS, $baseDebugID . "[td/div/div/div/span[contains(text(),'$text')]]" );
                push( @tempIDS, $baseDebugID . "[td/div/div[contains(text(),'$text')]]" );
                push( @tempIDS, "//div[\@id='$debugID']/div[contains(text(),'$text')]" );
                push( @tempIDS, "//div[\@id='$debugID']/div/div/div/div/div/div/div/div/"
                         . "table/tbody/tr/td/div[contains(text(),'$text')]" );
                push( @tempIDS, $baseDebugID . "[td/div/div/span[contains(text(),'$text')]]" );

                foreach my $tempid ( @tempIDS ) {
                    if ( $self->is_element_present($tempid) ) {
                        $rowDebugID = $tempid;
                        $Log->comment("Row with \"$text\" found");
                        last;
                    }
                }
                unless ($rowDebugID) {
                    $Log->comment(
                       "Row with \"$text\" not present in grid \"$debugID\" ");
                    NATE::BaseException->throw(
                       "Row with \"$text\" not present in grid \"$debugID\" ");
                }
            }
        }

        # Found element in the grid
        $Log->comment("Row with \"@texts\" is present in grid \"$debugID\" ");
    }

    if ( $doubleclick eq "1" ) {
        $Log->comment("Double click row $rowDebugID");
        $self->_double_click($rowDebugID);
    } else {
        $Log->comment("Selecting row $rowDebugID");
        $self->mouse_down($rowDebugID);
    }
    $Log->exit() if $may_exit;
    return 1;
} ## end sub click_row_with_texts

###############################################################################
# Method:    does_row_contain_texts
# Objective: matched texts with row cells and return true/false.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub does_row_contain_texts {
    my ( $self, $debugID, $partial, @texts );
    $Log->enter() if $may_enter;

    $self    = shift;
    $debugID = shift;

    if ( ( $_[0] eq "1" || $_[0] eq "0" ) ) {
        $partial = shift;
    } else {
        $partial = 0;
    }

    @texts = @_;

    my $baseDebugID =
      "//div[\@id='$debugID']/div/div/div/div/div/table/tbody/tr";

    my $rowDebugID = $baseDebugID;

    # Clicking the ascending order button
    my $baseSortID =
      "//div[\@id='$debugID']/div/div/div/div/div/div/table/tbody/tr/td[1]/div";

    $self->check_ascending($baseSortID);

    my $text;
    my ( $tempID1, $tempID2, $tempID3, $tempID4 );

    foreach $text (@texts) {
        $tempID1 = $baseDebugID . "[td/div[contains(text(),'$text')]]";
        $tempID2 = $baseDebugID . "[td/div/span[contains(text(),'$text')]]";
        $tempID3 = $baseDebugID . "[td/div/div/div/span[contains(text(),'$text')]]";
        $tempID4 = $baseDebugID . "[td/div/div[contains(text(),'$text')]]";

        if ( $self->is_element_present($tempID1) ) {
            $rowDebugID = $rowDebugID . "[td/div[contains(text(),'$text')]]";
        } elsif ( $self->is_element_present($tempID2) ) {
            $rowDebugID =
              $rowDebugID . "[td/div/span[contains(text(),'$text')]]";
        } elsif ( $self->is_element_present($tempID3) ) {
            $rowDebugID =
              $rowDebugID . "[td/div/div/div/span[contains(text(),'$text')]]";
        } elsif ( $self->is_element_present($tempID4) ) {
            $rowDebugID =
              $rowDebugID . "[td/div/div[contains(text(),'$text')]]";
        } else {
            $Log->comment("Row with \"$text\" not present in grid \"$debugID\" ");
            return 0;
        } ## end else [ if ( $self->is_element_present...)]
    } ## end foreach $text (@texts)

    # Found element and return true
    $Log->comment("Row with \"@texts\" is present in grid \"$debugID\" ");
    $Log->exit() if $may_exit;

    return $self->is_element_present($rowDebugID);
} ## end sub does_row_contain_texts

###############################################################################
## @name ::click_child_row_with_texts
##
## @summary
##     Click a row with matching texts in columns.
## @description
##     This method will select the child row under the parent row which contains the input text.
## @synopsis
##     click_child_row_with_texts(%args);
## INPUTS TO METHOD CALL 'click_child_row_with_texts' ON APISET OBJECT
##

sub click_child_row_with_texts {
    $Log->enter() if $may_enter;
    my ( $self, $debugID, $parent_values, $text1 ) = @_;
    my ( $parentID, $childID1, $childID2, $baseDebugID, $expand_parent);
    my $rowDebugID;
    if ( $parent_values =~ /(Kerberos|UNIX|Windows)/ ) {
        $expand_parent = 0;
        $baseDebugID =
            "//div[\@id='$debugID']//div[//div[text()='$parent_values']]";
    } else {
        $expand_parent = 1;
        $baseDebugID = "//div[\@id='$debugID']//div/tbody/tr";
    }
    foreach my $text ( @{$text1} ) {

        $parentID =
          $baseDebugID . "[td//span[text()='$parent_values']]";

        $childID1 = $baseDebugID
          . "[td/div/div[contains(\@id,'$parent_values"
          . "')]/div/span[text()='$text']]";

        $childID2 = $baseDebugID . "//div[text()='$text']";

        if ( $expand_parent == 1 ) {
            if ( not $self->is_element_present($childID1) ) {
                $Log->comment(" ******  $childID1 and $parentID ");
                if ( $self->is_element_present($parentID) ) {
                    $Log->comment(
                       "Expanding parent $parent_values row to display child rows ");
                    $self->_double_click($parentID);
                    sleep(5);
                } else {
                    $Log->comment("$childID1 and $parentID ");
                    NATE::BaseException->throw( "Parent \"$parent_values\" or child row"
                                   . " \"$text\" not present in grid \"$debugID\" ");
                }
            }
        }

        if ( $self->is_element_present($childID1) ) {
            $rowDebugID = $childID1;
        } elsif ( $self->is_element_present($childID2) ) {
            $rowDebugID = $childID2;
        } else {
            NATE::BaseException->throw(
               "Row with texts \"@$text1\" not present in grid \"$debugID\" ");
        }

        # Found element and return true
        $Log->comment("Row with \"$text\" is present in grid \"$debugID\" ");

        $Log->comment("Selecting row $rowDebugID");
        $self->mouse_down($rowDebugID);
    }
    $Log->exit() if $may_exit;
    return 1;
} ## end sub click_child_row_with_texts


##-----------------------------------------------------------------------------
## @name     click_EventTime
## @summary
## @synopsis click_EventTime(%args);
## INPUTS TO METHOD CALL 'click_EventTime' ON AN APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> element </td> <td> SCALAR</td> </tr>
## </table>
## @return   long
##-----------------------------------------------------------------------------
sub click_EventTime {

    my( $self, $element ) = @_;
    $Log->enter() if $may_enter;
    my $log_record = "---Start---Action Triggered:" . $self->getActionLongName();
    my $startTime = time;
    $Log->comment( "Start Time(in MilliSeconds) : " . $startTime );
    $element->click();
    return $startTime;

    $Log->exit() if $may_exit;

} ## end click_EventTime

##-----------------------------------------------------------------------------
## @name     getActionLongName
## @summary
## @synopsis getActionLongName(%args);
## INPUTS TO METHOD CALL 'getActionLongName' ON AN APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## </table>
## @return   String
##-----------------------------------------------------------------------------
sub getActionLongName {

    my $self = shift;

    $Log->enter() if $may_enter;

    my %args = validate_with(
        params      => \@_,
        spec        => {},
        allow_extra => 1
    );

    my ( $package, $file, $line, $subname ) = caller(0);
    my $invokingTestCase = $file;
    my $invokingMethod = $subname;
    my $className = $package;

    return $invokingTestCase . ":" . $className;
    $Log->exit() if $may_exit;

} ## end getActionLongName

##-----------------------------------------------------------------------------
## @name     captureResponseTime
## @summary
## @synopsis captureResponseTime(%args);
## INPUTS TO METHOD CALL 'captureResponseTime' ON AN APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> methodName </td> <td> SCALAR</td> </tr>
## <tr> <td> startTime </td> <td> SCALAR</td> </tr>
## </table>
## @return   boolean
##-----------------------------------------------------------------------------
sub captureResponseTime {

    my ( $self, $startTime, $methodName) = @_;

    $Log->enter() if $may_enter;

    my $endTime = time;
    $Log->comment( "End Time(in MilliSeconds) : " . $endTime );
    my $elapsedTime = $endTime - $startTime;
    my $newTime     = $elapsedTime / 1000.00;
    $Log->comment(
        "Loading Time For Action: " . $methodName . ":" . $newTime );
    my $log_record =
        "---End---Action:"
      . getActionLongName() . ":"
      . $methodName
      . "--Total Time:"
      . $newTime;
    return $newTime;

} ## end captureResponseTime


##-----------------------------------------------------------------------------
## @name     captureStartTime
## @summary
## @synopsis captureStartTime(%args);
## INPUTS TO METHOD CALL 'captureStartTime' ON AN APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## </table>
## @return   long
##-----------------------------------------------------------------------------
sub captureStartTime {

    my $self = shift;

    $Log->enter() if $may_enter;

    my $log_record = "---Start---Action Triggered:" . getActionLongName();
    my $startTime = time;
    $Log->comment( "Start Time(in MilliSeconds) : " . $startTime );
    return $startTime;

} ## end captureStartTime


###############################################################################
## @name ::click_row_with_texts_without_ascending
##
## @summary
##     Click a row with matching texts in columns.
## @description
##     This method is used to select a particular row without ascending the column,
##       and also containing the input text.
## @synopsis
##     click_row_with_texts_without_ascending(%args);
## INPUTS TO METHOD CALL 'click_row_with_texts_without_ascending' ON APISET OBJECT
##

sub click_row_with_texts_without_ascending {
    my ( $self, $debugID, $partial, $doubleclick, @texts );
    $Log->enter() if $may_enter;

    $self    = shift;
    $debugID = shift;

    if (   ( $_[0] eq "1" || $_[0] eq "0" )
        && ( $_[1] eq "1" || $_[1] eq "0" ) )
    {
        $partial     = shift;
        $doubleclick = shift;
    } else {
        $partial     = 0;
        $doubleclick = 0;
    }

    @texts = @_;

    my $baseDebugID =
      "//div[\@id='$debugID']/div/div/div/div/div/table/tbody/tr";

    my $rowDebugID;

    $Log->comment(
        "partial is $partial and doubleclick is $doubleclick and text is @texts"
    );

    # Clicking the ascending order button
    my $baseSortID =
      "//div[\@id='$debugID']/div/div/div/div/div/div/table/tbody/tr/td[1]/div";

    foreach my $text (@texts) {
        my @tempIDS;
        push( @tempIDS, $baseDebugID . "[td/div[contains(text(),'$text')]]" );
        push( @tempIDS, $baseDebugID . "[td/div/span[contains(text(),'$text')]]" );
        push( @tempIDS, $baseDebugID . "[td/div/div/div/span[contains(text(),'$text')]]" );
        push( @tempIDS, $baseDebugID . "[td/div/div[contains(text(),'$text')]]" );
        push( @tempIDS, "//div[\@id='$debugID']/div[contains(text(),'$text')]" );
        push( @tempIDS, "//div[\@id='$debugID']/div/div/div/div/div/div/div/div/"
                 . "table/tbody/tr/td/div[contains(text(),'$text')]" );
        push( @tempIDS, $baseDebugID . "[td/div/div/span[contains(text(),'$text')]]" );

        foreach my $tempid ( @tempIDS ) {
            if ( $self->is_element_present($tempid) ) {
                $rowDebugID = $tempid;
                $Log->comment("Row with \"$text\" found");
                last;
            }
        }
        unless ($rowDebugID) {
            NATE::BaseException->throw(
               "Row with \"$text\" not present in grid \"$debugID\" ");
        }
    } ## end foreach $text (@texts)

    # Found element and return true
    $Log->comment("Row with \"@texts\" is present in grid \"$debugID\" ");

    if ( $doubleclick eq "1" ) {
        $Log->comment("Double click row $rowDebugID");
        $self->_double_click($rowDebugID);
    } else {
        $Log->comment("Selecting row $rowDebugID");
        $self->mouse_down($rowDebugID);
    }
    $Log->exit() if $may_exit;
    return 1;
} ## end sub click_row_with_texts_without_ascending

###############################################################################
# Method:    does_row_contain_texts_without_ascending
# Objective: matched texts with row cells and return true/false (without ascending)
# Details:   See POD documentation at the end of this file.
###############################################################################
sub does_row_contain_texts_without_ascending {
    my ( $self, $debugID, $partial, @texts );
    $Log->enter() if $may_enter;
    $self    = shift;
    $debugID = shift;

    if ( ( $_[0] eq "1" || $_[0] eq "0" ) ) {
        $partial = shift;
    } else {
        $partial = 0;
    }

    @texts = @_;

    my $baseDebugID =
      "//div[\@id='$debugID']/div/div/div/div/div/table/tbody/tr";

    my $rowDebugID = $baseDebugID;

    # Clicking the ascending order button
    my $baseSortID =
      "//div[\@id='$debugID']/div/div/div/div/div/div/table/tbody/tr/td[1]/div";

    #$self->check_ascending($baseSortID);

    my $text;
    my ( $tempID1, $tempID2, $tempID3, $tempID4 );

    foreach $text (@texts) {
        $tempID1 = $baseDebugID . "[td/div[contains(text(),'$text')]]";
        $tempID2 = $baseDebugID . "[td/div/span[contains(text(),'$text')]]";
        $tempID3 = $baseDebugID . "[td/div/div/div/span[contains(text(),'$text')]]";
        $tempID4 = $baseDebugID . "[td/div/div[contains(text(),'$text')]]";

        if ( $self->is_element_present($tempID1) ) {
            $rowDebugID = $rowDebugID . "[td/div[contains(text(),'$text')]]";
        } elsif ( $self->is_element_present($tempID2) ) {
            $rowDebugID =
              $rowDebugID . "[td/div/span[contains(text(),'$text')]]";
        } elsif ( $self->is_element_present($tempID3) ) {
            $rowDebugID =
              $rowDebugID . "[td/div/div/div/span[contains(text(),'$text')]]";
        } elsif ( $self->is_element_present($tempID4) ) {
            $rowDebugID =
              $rowDebugID . "[td/div/div[contains(text(),'$text')]]";
        } else {
            $Log->comment("Row with \"$text\" not present in grid \"$debugID\" ");
            return 0;
        } ## end else [ if ( $self->is_element_present...)]
    } ## end foreach $text (@texts)

    # Found element and return true
    $Log->comment("Row with \"@texts\" is present in grid \"$debugID\" ");
    $Log->exit() if $may_exit;
    return $self->is_element_present($rowDebugID);

} ## end sub does_row_contain_texts_without_ascending

###############################################################################
# Method:    does_row_contain_exact_text
# Objective: matched texts with row cells and return true/false.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub does_row_contain_exact_text {
    my ( $self, $debugID, @texts ) = @_;
    $Log->enter() if $may_enter;

    my $baseDebugID =
      "//div[\@id='$debugID']/div/div/div/div/div/table/tbody/tr";
    my $rowDebugID;

    foreach my $text (@texts) {
        my @tempIDS;
        push( @tempIDS, $baseDebugID . "[td/div[text()='$text']]" );
        push( @tempIDS, $baseDebugID . "[td/div/span[text()=' $text']]" );
        push( @tempIDS, $baseDebugID . "[td/div/div/div/span[text()='$text']]" );
        push( @tempIDS, $baseDebugID . "[td/div/div[text()='$text']]" );

        foreach my $tempid ( @tempIDS ) {
            if ( $self->is_element_present($tempid) ) {
                $rowDebugID = $tempid;
                $Log->comment("Row with \"$text\" found");
                last;
            }
        }
        unless ($rowDebugID) {
            $Log->comment(
               "Row with \"$text\" not present in grid \"$debugID\" ");
            return 0;
        }
    } ## end foreach my $text (@texts)

    $Log->comment("Row with \"@texts\" is present in grid \"$debugID\" ");
    $Log->exit() if $may_exit;
    return $self->is_element_present($rowDebugID);
} ## end sub does_row_contain_exact_text

###############################################################################
## @name ::check_ascending
##
## @summary
##   Check if the column is in ascending order.
## @description
##     This method is used to verify whether the grid is column is set to ascending order or not.
## @synopsis
##     check_ascending(%args);
## INPUTS TO METHOD CALL 'check_ascending' ON APISET OBJECT
##
sub check_ascending {
    my ( $self, $debugID ) = @_;
    $Log->enter() if $may_enter;

    #Check if the element present
    if ( $self->is_element_present($debugID) ) {

        #_click on the tab if its not ascending
        my $btn_id = $debugID . "[contains(\@aria-sort,'descending')]";
        if ( not $self->is_element_present($btn_id) ) {
            $Log->comment("Sorting in ascending order");
            $self->_click($debugID);
        }
    } else {
        $Log->comment("Invalid id $debugID to sort ascending order");
    }
    $Log->exit() if $may_exit;
    return 1;
} ## end sub check_ascending

###############################################################################
## @name ::check_descending
##
## @summary
##   Check if the column is in descending order.
## @description
##     This method is used to verify whether the grid is column is set to descending order or not.
## @synopsis
##     check_descending(%args);
## INPUTS TO METHOD CALL 'check_descending' ON APISET OBJECT
##
sub check_descending {
    $Log->enter() if $may_enter;
    my ( $self, $debugID ) = @_;

    #Check if the element present
    if ( $self->is_element_present($debugID) ) {

        #click on the tab if its not descending
        my $btn_id = $debugID . "[contains(\@aria-sort,'ascending')]";
        if ( not $self->is_element_present($btn_id) ) {
            $Log->comment("Sorting in descending order");
            $self->_click($debugID);
        }
    } else {
        $Log->comment("Invalid id $debugID to sort descending order");
    }
    $Log->exit() if $may_exit;
    return 1;
} ## end sub check_descending

###############################################################################
# Method:    click_text_in_list
# Objective: Click a list item.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub click_item_in_list {
    my ( $self, $debugID, $item ) = @_;
    $Log->enter() if $may_enter;
    $self->mouse_down("//div[\@id='$debugID']/div[text()='$item']");
    $Log->exit() if $may_exit;
    return 1;
} ## end sub click_item_in_list

###############################################################################
## @name ::is_button_enabled
##
## @summary
##    Checks if button is enabled/disabled.
## @description
##     This method is used to verify whether the button is enable or not.
## @synopsis
##     is_button_enabled(%args);
## INPUTS TO METHOD CALL 'is_button_enabled' ON APISET OBJECT
##
sub is_button_enabled {
    $Log->enter() if $may_enter;
    my ( $self, $debugID ) = @_;
    my $retStatus = 1;

    if (
        $self->is_element_present(
            "//table[\@id='$debugID' and contains(\@class,'x-item-disabled')]")
      ) {
        $retStatus = 0;
    } elsif (
        $self->is_element_present(
            "//a[\@id='$debugID' and contains(\@class,'x-item-disabled')]")
      ) {
        $retStatus = 0;
    }

    ( $retStatus == 1 ) ? $Log->comment("Button: $debugID is Enabled")
         : $Log->comment("Button: $debugID is Disabled");
    $Log->exit() if $may_exit;
    return $retStatus;
} ## end sub is_button_enabled

###############################################################################
## @name ::check_for_warning
##
## @summary
##   This lists any warning message present in the page.
## @description
##     This method is used to check if any warning is present in a page.
## @synopsis
##     check_for_warning(%args);
## INPUTS TO METHOD CALL 'check_for_warning' ON APISET OBJECT
##
sub check_for_warning {
    my ( $self, $debugID ) = @_;
    my $warning;
    $Log->enter() if $may_enter;

    if ( $debugID and $self->is_element_present($debugID) ) {
        if ( not $self->is_element_present(
                "//div[\@id = '$debugID' and contains(\@class,'x-hide-display')]")
          ) {
            my $textfield =
                "//div[\@id ='"
              . $debugID
              . "']//div[contains(\@class,'x-form-label')]";
            $warning = $self->get_text($textfield);
            my $imageid =
                "//div[\@id ='"
              . $debugID
              . "']//img[contains(\@src,'icon-error.gif')]";
            if ( $self->is_element_present($imageid) ) {
                $Log->comment( "Warning : $warning");
                NATE::BaseException->throw("/n Error message : $warning /n");
            }
        }
    }

    $Log->exit() if $may_exit;
    return $warning;
} ## end sub check_for_warning

###############################################################################
## @name ::get_grid_data
##
## @summary
##   Fetch the grid data.
## @description
##     This method returns the whole grid values.
## @synopsis
##     get_grid_data(%args);
## INPUTS TO METHOD CALL 'get_grid_data' ON APISET OBJECT
##
sub get_grid_data {

    my ( $self, $debugID ) = @_;
    $Log->enter() if $may_enter;

    my $gridID = "//div[\@id='$debugID']";
    my $gridData;
    my $tablepresent;

    $Log->comment("Tally the table iD's to be read inside the grid");
    my $tableID1 = $gridID . "/div/div/div/div[\@class='x-grid3-body']";
    my @tableIDs = ($tableID1);

    $Log->comment(
        "Read the data in each tallied and existing table & add to gridData");
    foreach my $tableID (@tableIDs) {
        if ( $self->is_element_present("$tableID") ) {
            my $tableData = $self->get_text($tableID);
            $gridData     = $gridData . $tableData;
            $tablepresent = 'true';
        }
    }
    $Log->comment("If no table exists, get the data for the base gridID");
    if ( not $tablepresent eq 'true' ) {
        $gridData = $self->get_text($gridID);
    }
    $Log->exit() if $may_exit;
    return $gridData;
} ## end sub get_grid_data

#-------------------------------------------------------------------------------
# WebDriver Overloaded functions, for Selenium RC backward compatibility support
#-------------------------------------------------------------------------------
sub _click {
    $Log->enter() if $may_enter;
    my ( $self, $debugID ) = @_;

    if ( ref($self) =~ /WebElement/ ) {
        $self->click();
        return 1;
    }
    $Log->comment("Click Element : $debugID");

    $self->set_implicit_wait_timeout(30000);
    my $elem = $self->_find_element($debugID);
    $elem->click();
    $Log->comment("Click on Element $debugID successfull");
    $Log->exit() if $may_exit;
    return 1;
}

sub _double_click {
    $Log->enter() if $may_enter;
    my ( $self, $ID ) = @_;
    unless ( $ID ) {
        $self->double_click();
        return 1;
    }
    $self->set_implicit_wait_timeout(30000);
    my $elem = $self->_find_element($ID);
    $self->mouse_move_to_location( 'element' => $elem );
    $self->double_click();
    $Log->exit() if $may_exit;
    return 1;
}

sub get_text {
    $Log->enter() if $may_enter;

    my ( $self, $debugID ) = @_;

    my $output_text;
    if ( ref($self) =~ /WebElement/ ) {
        $output_text = $self->get_text();
        return $output_text;
    }
    $self->set_implicit_wait_timeout(30000);
    my $elem        = $self->_find_element($debugID);
    $output_text = $elem->get_text();
    $Log->exit() if $may_exit;
 
    return $output_text;
}

sub _get_value {
    $Log->enter() if $may_enter;
    
    my ( $self, $debugID ) = @_;

    my $output_value;
    if ( ref($self) =~ /WebElement/ ) {
        $output_value = $self->get_value();
        return $output_value;
    }

    $self->set_implicit_wait_timeout(30000);
    my $elem      = $self->_find_element($debugID);
    $output_value = $elem->get_value();
    $Log->exit() if $may_exit;

    return $output_value;
}

sub is_checked {
    my ( $self, $debugID ) = @_;
    my $cssPath;
    my $value = undef;
    $Log->enter() if $may_enter;

    # if is_checked() is called as an instance
    if ( $debugID ) {
        $self->set_implicit_wait_timeout(30000);
        if ( ref($debugID) =~ /WebElement/ ) {
            $Log->comment("Find ID of element : $debugID");
            $cssPath = $debugID->get_attribute("id");
        } elsif ( $debugID =~ /input$/ ) {
            $cssPath = $self->XpathToCssConvertor($debugID);
        } else {
            NATE::BaseException->throw(
                "Css conversion not possible for id : $debugID,"
                . "Pass a complete xpath of input element");
        }
        if (!$cssPath) {
            $Log->comment("$debugID does not have an Id."
                . "Raise a burt and get it");
        }
        eval {
            $Log->comment("Checking Checkbox status of $cssPath");
            $self->_find_element("\#$cssPath\[checked\]", "css");
            $value = 1;
        };
        if ($@ =~ /An element could not be located/) {
            $Log->comment("Check box is unselected");
            $value = 0;
        } elsif ($@) {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw($@);
        }
        $Log->exit() if $may_exit;
        return $value;
    }

    # if is_checked() is called as a class method
    if ( ref($self) =~ /WebElement/ ) {
        $Log->comment("Find id of element : $debugID");
        $cssPath = $self->get_attribute("id");
    }

    if (!$cssPath) {
        NATE::BaseException->throw(
          "$debugID does not have an Id. Raise a burt and get it");
    }

    $Log->comment("Checking Checkbox status of $cssPath");

    # is_checked() called directly from input element
    eval { 
        $self->_find_element("\#$cssPath\[checked\]", "css");
        $value = 1;
    };
    if ($@) {
        $Log->comment( "Find CSS element returned error : $@");
    } else {
        $Log->exit() if $may_exit;
        return $value;
    }

    # is_checked() called from parent of input element
    eval {
        $self->_find_element("\#$cssPath>input[checked]", "css");
        $value = 1;
    };
    if ($@) {
        $Log->comment( "Find CSS element returned error : $@");
    }

    $Log->exit() if $may_exit;
    return $value;
}

sub key_down {
    $Log->enter();    
    my ( $self, $kbKey ) = @_;
    $self->send_keys_to_active_element(KEYS->{$kbKey});
    $Log->exit() if $may_exit;
    return 1;
}

sub key_up {
    $Log->enter();    
    my ( $self, $kbKey ) = @_;
    $self->send_keys_to_active_element(KEYS->{$kbKey});
    $Log->exit();    
    return 1;
}

sub mouse_down {
    $Log->enter();    
    my ( $self, $debugID ) = @_;
    $self->set_implicit_wait_timeout(30000);
    sleep(2);
    my $elem = $self->_find_element($debugID);
    $self->mouse_move_to_location( 'element' => $elem );
    $self->button_down;
    $self->button_up;
    sleep(1);
    $Log->exit() if $may_exit;
    return 1;
}

sub mouse_up {
    $Log->enter() if $may_enter;
    my ( $self, $debugID ) = @_;
    $self->set_implicit_wait_timeout(30000);
    my $elem = $self->_find_element($debugID);
    $self->mouse_move_to_location( 'element' => $elem );
    $self->button_up;
    $Log->exit() if $may_exit;
    return 1;
}

sub refreshBrowser {
    my ($self) = @_;
    $Log->enter() if $may_enter;
    $self->refresh();
    $self->manage_system();
    $Log->exit() if $may_exit;
    return 1;
}

sub select_window {
    $Log->enter() if $may_enter;
    my ( $self, $selWin ) = @_;
    $self->switch_to_window($selWin);
    $Log->exit() if $may_exit;
    return 1;
}

sub set_timeout {
    $Log->enter() if $may_enter;
    my ( $self, $timeoutVal ) = @_;
    my $res = { 'command' => 'setTimeout' };
    foreach my $type ( 'script', 'page load' ) {
        my $params = { 'type' => $type, 'ms' => $timeoutVal };
        $self->_execute_command( $res, $params );
    }
    $Log->exit() if $may_exit;
    return 1;
}

sub type {
    my ( $self, $debugID, $typeText ) = @_;
    $Log->enter() if $may_enter;
    $self->set_implicit_wait_timeout(30000);
    my $elem = $self->_find_element($debugID);
    $elem->clear();
    $elem->send_keys($typeText);
    $Log->exit() if $may_exit;
    return 1;
}


sub clear {
    $Log->enter() if $may_enter;
    my ( $self, $debugID ) = @_;
    $self->set_implicit_wait_timeout(30000);
    my $elem = $self->_find_element($debugID);
    $elem->clear();
    $Log->exit() if $may_exit;
    return 1;
}



sub wait_for_element_not_present {
    my ( $self, $debugID, $timeout ) = @_;
    my $retVal = 1;
    $Log->enter() if $may_enter;

    for ( my $i = 0 ; $i < 10 ; $i++ ) {
        $self->set_implicit_wait_timeout(1000);
        eval {
            $self->_find_element($debugID);
            $Log->comment("Element $debugID still exists");
            sleep(1);
            $retVal = 0;
        };
        if ($@) {
            $retVal = 1;
            last;
        }
    }
    unless ( $retVal ) {
        $Log->comment( "Sub : wait_for_element_not_present(), on element '$debugID' Failed");
    }
    $Log->exit() if $may_exit;

    return $retVal;
}

sub wait_for_element_present {
    my ( $self, $debugID ) = @_;
    my $retVal = 0;
    $Log->enter() if $may_enter;

    $Log->comment( "Waiting for Element : $debugID ");
    for ( my $i = 0 ; $i < 10 ; $i++ ) {
        $self->set_implicit_wait_timeout(10000);
        eval {
            $self->_find_element($debugID);
            $retVal = 1;
        };
        if ($@) {
            sleep(5);
            $Log->comment("Waiting for $debugID : " . $@);
        } else {
            last;
        }
    }
    unless ( $retVal ) {
        $Log->comment( "Sub : wait_for_element_present(), on element '$debugID' Failed");
    }
    $Log->exit() if $may_exit;

    return $retVal;
}

sub wait_for_page_to_load {
    $Log->enter() if $may_enter;
    my ( $self ) = @_;
    $self->wait_for_loading();
    $Log->exit() if $may_exit;
    return 1;
}

sub get_all_window_titles {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    $self->set_implicit_wait_timeout(30000);
    my $handles = $self->get_window_handles;
    $Log->exit() if $may_exit;
    return @$handles;
}

sub is_element_present {
    $Log->enter() if $may_enter;
    my ( $self, $debugID ) = @_;
    $self->set_implicit_wait_timeout(1000);
    eval { $self->_find_element($debugID); };
    if ($@) { 
        $Log->comment( "Sub : is_element_present(), on element '$debugID' failed");
        return 0;
    }
    $Log->comment( "Sub : is_element_present(), on element '$debugID' Passed");
    $Log->exit() if $may_exit;
    return 1;
}

sub get_all_window_ids {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    $self->set_implicit_wait_timeout(30000);
    my $handles = $self->get_window_handles;
    $Log->exit() if $may_exit;
    return @$handles;
}

sub get_all_window_names {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    $self->set_implicit_wait_timeout(30000);
    my $handles = $self->get_window_handles;
    $Log->exit() if $may_exit;
    return @$handles;
}

sub stop {
    $Log->enter() if $may_enter;
    my ( $self ) = @_;
    $self->quit();
    $Log->exit() if $may_exit;
    return 1;
}

sub fire_event {
    return 1;
}

sub type_keys {
    $Log->enter() if $may_enter;
    my ( $self, $debugID, $typeText ) = @_;
    return 1 if ( $typeText =~ /^\s*$/ );
    $self->set_implicit_wait_timeout(30000);
    my $elem = $self->_find_element($debugID);
    $elem->clear();
    $elem->send_keys($typeText);
    return 1;
    $Log->exit() if $may_exit;
}

sub _find_element {
    my ( $self, $str, $type ) = @_;
    my $elem_obj;
    $Log->enter() if $may_enter;
    if ($type) {
        $elem_obj = $self->find_element($str, $type);
    } else{
        if ( $str =~ /^\/\// ) {
            $elem_obj = $self->find_element($str);
        }
        else {
            $elem_obj = $self->find_element( $str, 'id' );
        }
    }
    $Log->exit() if $may_exit;
    return $elem_obj;
}

sub mouse_over {
    my ( $self, $ID ) = @_;
    $Log->enter() if $may_enter;
    $self->set_implicit_wait_timeout(30000);
    my $elem = $self->_find_element($ID);
    $self->mouse_move_to_location( 'element' => $elem );
    $Log->exit() if $may_exit;
    return 1;
}

sub get_attribute {
    my ( $self, $id, $attr_name ) = @_;
    my $val;
    $Log->enter() if $may_enter;
    if ( ref($self) =~ /WebElement/ ) {
        $val = $self->get_attribute($id);
    } else {
        $self->set_implicit_wait_timeout(30000);
        my $elem_obj;
        $elem_obj = $self->_find_element($id);
        $val = $elem_obj->get_attribute($attr_name);
    }
    $Log->comment("Get Attrinute on $id successfull : $val");
    $Log->exit() if $may_exit;
    return $val;
}

sub keys {
    my ($self,@keys) = shift;
    $Log->enter() if $may_enter;
    $self->set_implicit_wait_timeout(1000);
    $self->send_keys_to_active_element( @keys );
    $Log->exit() if $may_exit;
    return 1;
}

sub control_key_down {
    $Log->enter() if $may_enter;
    my $self = shift;
    $Log->comment("Comment pressing control key down");
    $self->key_down(KEYS->{control});
    $Log->exit() if $may_exit;
    return 1;
}

sub control_key_up {
    $Log->enter() if $may_enter;
    my $self = shift;
    $Log->comment("Comment pressing control key up");
    $self->key_up(KEYS->{control});
    $Log->exit() if $may_exit;
    return 1;
}

sub mouse_down_only {
    $Log->enter() if $may_enter;
    my ( $self, $debugID ) = @_;
    $self->set_implicit_wait_timeout(30000);
    my $elem = $self->_find_element($debugID);
    $self->mouse_move_to_location( 'element' => $elem );
    $self->button_down;
    $Log->exit() if $may_exit;
    return 1;
}

sub drag_and_drop {
    $Log->enter() if $may_enter;
    my ( $self, $source, $dest ) = @_;
    my ($elem1 , $elem2);

    if ( ref($source) =~ /WebElement/ ) {
        $elem1 = $source;
    } else {
        $elem1 = $self->find_element($source);
    }

    if ( ref($dest) =~ /WebElement/ ) {
        $elem2 = $dest;
    } else {
        $elem2 = $self->find_element($dest);
    }

    $self->mouse_move_to_location( 'element' => $elem1 );
    $self->button_down;

    $self->mouse_move_to_location( 'element' => $elem2 );
    $self->button_up;
    $Log->exit() if $may_exit;
    return 1;
}

##-----------------------------------------------------------------------------
## @name     eventWaitingTime
## @summary
## @synopsis eventWaitingTime(%args);
## INPUTS TO METHOD CALL 'eventWaitingTime' ON AN APISET OBJECT
## <table border = "1">
## <tr> <th> PARAM </th> <th> DESCRIPTION </th> </tr>
## <tr> <td> element </td> <td> SCALAR</td> </tr>
## <tr> <td> methodName </td> <td> SCALAR</td> </tr>
## <tr> <td> startTime </td> <td> SCALAR</td> </tr>
## </table>
## @return   boolean
##-----------------------------------------------------------------------------
sub eventWaitingTime {

    my ( $self, $element, $startTime, $methodName) = @_;

    $Log->enter() if $may_enter;

    $self->wait_for_element_not_present( $element );

    my $endTime = time;
    $Log->comment( "End Time(in MilliSeconds) : " . $endTime );
    my $elapsedTime = $endTime - $startTime;
    my $newTime   = $elapsedTime / 1000.00;
    $Log->comment(
        "Loading Time For Action: " . $methodName . ":" . $newTime );
    my $log_record =
        "---End---Action:"
      . getActionLongName() . ":"
      . $methodName
      . "--Total Time:"
      . $newTime;
    return 1;

} ## end eventWaitingTime


###############################################################################
# Method:    config_info
# Objective: This function will return data related to ONTAP version and config related information
# Details:   See POD documentation at the end of this file.
###############################################################################
sub config_info {
    my ($self) = @_;
    $Log->enter() if $may_enter;

    if ( $self->is_element_present($LOGGING_POPUP) ) {
        $Log->comment("Move the logging pop-up to center of menu bar");

        my $elem1 = $self->find_element($LOGGING_POPUP);
        my $elem2 = $self->find_element($MENU_BAR);

        $self->mouse_move_to_location( 'element' => $elem1 );
        $self->button_down;

        $self->mouse_move_to_location( 'element' => $elem2 );
        $self->button_up;
    }

    $Log->comment("Clicking help button");
    $self->_click($HELP_BUTTON);

    $Log->comment("Clicking Netapp About");
    $self->_click($ABOUT_NETAPP_DROPDOWN);

    $Log->comment("Clicking Configuration button");
    $self->_click($CONFIG_BUTTON);

    $Log->comment("Reading Netapp About data");
    my %value;
    my $elem_obj = $self->find_element($CONFIG_TEXTAREA);
    $value{"config"} = $elem_obj->get_value();

    $Log->comment("Collection of Config Information completed");

    $self->close();

    my $temp_value = {};
    $temp_value->{parsed_output} = \%value;
    $Log->exit() if $may_exit;

    return $temp_value;
}

###############################################################################
#                               Private Methods
###############################################################################

###############################################################################
# Method:    _check_error_field
# Objective: This function checks for any error field shown due to invalid entry.
# Details:   This function checks for any error field shown due to invalid entry.
###############################################################################
sub _check_error_field {

    my ($self) = @_;
    $Log->enter() if $may_enter;

    $Log->comment("Check if any invalid fields are displayed");
    if ( $self->is_element_present($GENERIC_INVALID_FIELD) ) {

        #Getting debugID of the invalid field
        my $debugID = $self->get_attribute( $GENERIC_INVALID_FIELD, 'id' );

        #Removing the -input part from ID
        $debugID =~ s/-input//;

        #Appending it with x-form-el to form the upper element's ID
        my $topElementID = "x-form-el-" . $debugID;

        #Check if a red color Error Icon is displayed
        if ( $self->is_element_present("//div[\@id='$topElementID']/img") ) {
            #If red color Error Icon is displayed then do mouse over it
            $self->mouse_over("//div[\@id='$topElementID']/img");
        } elsif ( $self->is_element_present($GENERIC_INVALID_ICON) ) {
            #If any other Error Icon is displayed then do mouse over it
            $self->mouse_over($GENERIC_INVALID_ICON);
        } else {
            # If no Error Icon is displayed then do mouse over on the error textbox
            $self->mouse_over($debugID);
        }

        #Check if tooltip is displayed
        if ( $self->is_element_present($GENERIC_TOOLTIP) ) {

            # Fetch the tooltip message
            my $tooltipMsg = $self->get_text($GENERIC_TOOLTIP);

            #Die with the tooltip message
            $self->capture_screenshot($screenshot.$s_count++.".png");
            NATE::BaseException->throw("Invalid input in $debugID \n : \"$tooltipMsg\"");
        }

        #Check for error icon is displayed
        if ( $self->is_element_present($GENERIC_ERROR_ICON) ) {
            # Fetch the error message along with error icon
            my $errorMsg = $self->get_text($GENERIC_ERROR_ICON);
            $Log->comment("Invalid Operation Error Message:$errorMsg");
            $self->capture_screenshot($screenshot.$s_count++.".png");
            #Die with error message
            NATE::BaseException->throw("\"$errorMsg\"");
        } ## end if ($self->is_element_present...
        $self->capture_screenshot($screenshot.$s_count++.".png");
        # If not tooltip then throw Generic error message
        NATE::BaseException->throw("Invalid input in $debugID");
    } ## end if ($self->is_element_present...
    $Log->exit() if $may_exit;
    return 1;
} ## end sub _check_error_field
###############################################################################
# Method:    _check_for_errordialog
# Objective: check if error dialog has popup and throw error message from error dialog.
# Details:   This Method checks if error dialog has popup/error dialoge.
###############################################################################
sub _check_for_errordialog {

    my ($self) = @_;
    $Log->enter() if $may_enter;
    sleep(1);

    $Log->comment("Check if any error dialogs are present");
    if ( $self->is_element_present($GENERIC_ERROR_DIALOG) ) {

        #Fetching the error message from Error dialog using Xpath
        my $errorMessage = $self->get_text(
                "//div[\@id='$GENERIC_ERROR_DIALOG']/div/div/div/div/div/"
              . "div/div/div/div/table/tbody/tr/td/div" );
        NATE::BaseException->throw("\"$errorMessage\"");
    }

    # Checking for alternate error dialog
    if ( $self->is_element_present($GENERIC_ERROR_DIALOG1) ) {

        #Fetching the error message from Error dialog using Xpath
        my $errorMessage = $self->get_text(
                "//div[\@id='$GENERIC_ERROR_DIALOG1']/div/div/div/div/div/div/"
              . "div/div/div/table/tbody/tr/td/div" );
        NATE::BaseException->throw("\"$errorMessage\"");
    }

    # Checking for error icon
    if ( $self->is_element_present($POPUP_ERROR_ICON) ) {
        my $errorMessage1 = $self->get_text($POPUP_ERROR_MESSAGE);
        $Log->comment("Error occurred: Pop-up error message is $errorMessage1");
        NATE::BaseException->throw("\"$errorMessage1\"");
    }

    # Checking for error window
    if ( $self->is_element_present($GENERIC_WARNING_DIALOG) ) {
        my $errorMessage1 = $self->get_text($GENERIC_WARNING_TEXT);
        $Log->comment("Error occurred: Pop-up error message is $errorMessage1");
        NATE::BaseException->throw("\"$errorMessage1\"");
    }
    $Log->exit() if $may_exit;
    return 1;
} ## end sub _check_for_errordialog

###############################################################################
# Method:    _left_tree_navigate
# Objective: This function navigates to specific tree element in left tree
# Details:   This function navigates to specific tree element in left tree.
###############################################################################
sub _left_tree_navigate {
    my ( $self, $aria, $menus ) = @_;
    my ( $sec, $end, $start, $final_values );
    $Log->enter() if $may_enter;

    $Log->comment("Setting browser width and height to :1280, 1024");
    $self->set_window_size(1280,1280);

    $Log->comment("Navigating to $menus");

    my @menu = split( />/, $menus );
    my $menu_count = scalar(@menu);
    my $count;

    if ( $menu_count == 1 ) {
        $self->wait_for_element_present(
                "//div[contains(\@id,'navigation-tree__navigation-tree') and "
              . "\@aria-level=$aria-1]/span[text()='$menu[$menu_count-1]' and $TREE_ELEMENT_CLASS_ID]"
        );
        $Log->comment("Clicking on $menu[$menu_count-1] in tree");
        $self->mouse_down(
                "//div[contains(\@id,'navigation-tree__navigation-tree') and "
              . "\@aria-level=$aria-1]/span[text()='$menu[$menu_count-1]' and $TREE_ELEMENT_CLASS_ID]"
        );
    } else {
        for ( $count = 1 ; $count < $menu_count ; $count++ ) {
            $Log->comment("-----> Checking if $menu[$count] menu is present");

            if ( not $self->is_element_present(
                   "//div[contains(\@id,'navigation-tree') and "
                   . "\@aria-level=$aria]/span[text()='$menu[$count]' and $TREE_ELEMENT_CLASS_ID]" )
              )
            {
                $Log->comment(
                    "-----> No, So Clicking on $menu[$count-1] in tree");
                $self->_double_click(
                        "//div[contains(\@id,'navigation-tree') and "
                      . "\@aria-level=$aria-1]/span[text()='$menu[$count-1]' and $TREE_ELEMENT_CLASS_ID]"
                );
                $self->wait_for_element_present(
                    "//div[contains(\@id,'navigation-tree') and "
                      . "\@aria-level=$aria]/span[text()='$menu[$count]' and $TREE_ELEMENT_CLASS_ID]",
                    $GENERIC_WAIT
                );
                $aria = $aria + 1;
                next;
            } ## end if (not $self->is_element_present...

            $aria = $aria + 1;
            $Log->comment("-----> OK $menu[$count] menu is present");
        } ## end for ($count = 1 ; $count...
        $Log->comment("---> Final Click on $menu[$menu_count-1] in tree");
        $self->mouse_down(
            "//div[contains(\@id,'navigation-tree') and \@aria-level=$aria-1]/"
              . "span[text()='$menu[$menu_count-1]' and $TREE_ELEMENT_CLASS_ID]"
        );
    } ## end else [ if ($menu_count == 1)

    $start = (time)[0];
    $self->wait_for_loading($GENERIC_WAIT);
    $end = (time)[0];

    #Timings
    $sec = $end - $start;
    $final_values = sprintf( "%.2f", $sec );
    $Log->comment( "Time measurement :Loading time for page $menus is $final_values Seconds");
    $Log->enter() if $may_enter;

    return $final_values;
} ## end sub _left_tree_navigate

###############################################################################
# Method:    _cluster_tab_navigation
# Objective: This function will navigate accross the available tabs in Cluster view.
# Details:   This function will navigate accross the available tabs in Cluster view.
###############################################################################
sub _tab_navigation {
    my ( $self, $tablevel ) = @_;
    $Log->enter() if $may_enter;

    $Log->comment("Navigating across TAB : $tablevel");

    if ( $tablevel eq 'cluster' ) {
        $Log->comment("Check if Cluster Tab is collapsed");
        if ( $self->is_element_present( $CLUSTER_TAB_COLLAPSED ) ) {
            $Log->comment( "Clicking Cluster Tab to expand for navigation");
            $self->mouse_down($CLUSTER_TAB_COLLAPSED);
            $self->wait_for_loading();
            $Log->comment("Cluster Tab is available for navigation");
        } else {
            $Log->comment("Cluster Tab is already open");
        }
    } elsif ( $tablevel eq 'vserver' ) {
        $Log->comment("Check if Vserver Tab is collapsed");
        if ( $self->is_element_present( $VSERVER_TAB_COLLAPSED )) {
            $Log->comment("Clicking Vserver Tab to expand for navigation");
            $self->mouse_down( $VSERVER_TAB_COLLAPSED );
            $self->wait_for_loading();
            $self->wait_for_element_present($CLUSTER_VSERVER, $GENERIC_WAIT);
            $Log->comment("Vserver Tab is available for navigation");
        } else {
            $Log->comment("Vserver Tab is already open");
        }
    } elsif ( $tablevel eq 'nodes' ) {
        $Log->comment("Check if Nodes Tab is collapsed");
        if ( $self->is_element_present( $NODES_TAB_COLLAPSED )) {
            $Log->comment("Clicking Nodes Tab to expand for navigation");
            $self->mouse_down( $NODES_TAB_COLLAPSED );
            $self->wait_for_loading();
            $Log->comment("Nodes is available for navigation");
        } else {
            $Log->comment("Nodes Tab is already open");
        }
    }

    $Log->exit() if $may_exit;
    return 1;
} ## end sub _tab_navigation

=head2 getGridDataAsHashMap

Returns Array <HashMap<String, String>>

=over

=item Options

=over

=item argument_list

 (Optional) $gridID, $columns, $colDisplayName

=back

=back

=over

=back

=cut

sub getGridDataAsHashMap {

    my ( $self, $gridID, $columns, $colDisplayName)  = @_;
    $Log->enter() if $may_enter;

    my (@colNames, @values);

    # populate hash between column name displayed and their
    # corresponding grid-td-classnames used in inner html
    foreach my $element ( $self->find_elements(
          "//div[\@id='$gridID']"
        . "//div[\@role='columnheader']" ) ) {
        my $class = $element->get_attribute("id");
        my ($col_class_name) = $class =~ /DEBUG-ID-NOT-SET(\S+)/;
        if ( $colDisplayName ) {
            my $col_name =
               $self->find_child_element($element,"./span")->get_text();
            push(@colNames, $col_name);
        } else {
            push(@colNames, $col_class_name);
        }
    }

    my @rowElements = $self->find_elements(
          "//div[\@id='$gridID']"
        . "//div[contains(\@class,'x-grid3-scroller')]"
        . "//div[contains(\@class,'x-grid3-row')]" );

    my $i = 0 ;
    foreach my $row (@rowElements) {
        $values[$i] = {};
        foreach my $column (@colNames) {
            # if user has requested specific columns then,
            # skip if current coloumn is not one of those.
            if ( @$columns and
                 !grep( /$column/i,@$columns ) ) {
                next;
            }
            my $value = $self->find_child_element($row,
                    "./table//div[contains(\@class,'x-grid3-col-$column"
                  . "')]//div[contains(\@class,'gwt-HTML')]")->get_text();
            my $name = $column;
            $name =~ s/_/-/g;
            $name =~ s/\s+/-/g;
            $name =~ s/[:;'",.]//g;
            $name = lc($name);
            $value = '-' unless $value;
            $value = '-' if ( $value =~ /-NA-/ );
            if ( $value =~ /,/ ) {
                my @vals = split(/,/,$value);
                $values[$i++]->{$name} = \@vals;
            } else {
                $values[$i++]->{$name} = $value;
            }
        }
    }
    return @values;
} ## end getGridDataAsHashMap

=head2 getRowDataAsHashMap

Returns List<HashMap<String, String>>

=over

=item Options

=over

=item argument_list

 (Optional) $gridID, $columns, $colDisplayName

=back

=back

=over

=back

=cut

sub getRowDataAsHashMap {

    my ( $self, $gridID, $columns, $colDisplayName)  = @_;
    $Log->enter() if $may_enter;

    my (@colNames, %values);

    # populate hash between column name displayed and their
    # corresponding grid-td-classnames used in inner html
    foreach my $element ( $self->find_elements(
          "//div[\@id='$gridID']"
        . "//div[\@role='columnheader']" ) ) {
        my $class = $element->get_attribute("id");
        my ($col_class_name) = $class =~ /DEBUG-ID-NOT-SET(\S+)/;
        if ( $colDisplayName ) {
            my $col_name =
               $self->find_child_element($element,"./span")->get_text();
            push(@colNames, $col_name);
        } else {
            push(@colNames, $col_class_name);
        }
    }

    my $row = $self->find_element(
          "//div[\@id='$gridID']"
        . "//div[contains(\@class,'x-grid3-scroller')]"
        . "//div[contains(\@class,'x-grid3-row')]" );

    foreach my $column (@colNames) {
        # if user has requested specific columns then,
        # skip if current coloumn is not one of those.
        if ( @$columns and
             !grep( /$column/i,@$columns ) ) {
            next;
        }
        unless ( $self->is_element_present(
             "//table//div[contains(\@class,'x-grid3-col-$column"
           . "')]//div[contains(\@class,'gwt-HTML')]")) {
            $Log->comment("Warning: requested field id : $column"
                . " was not found in the display Grid: $gridID" );
            next;
        }
        my $value = $self->find_child_element($row,
                   "./table//div[contains(\@class,'x-grid3-col-$column"
                 . "')]//div[contains(\@class,'gwt-HTML')]")->get_text();
        my $name = $column;
        $name =~ s/_/-/g;
        $name =~ s/\s+/-/g;
        $name =~ s/[:;'",.]//g;
        $name = lc($name);
        $value = '-' unless $value;
        $value = '-' if ( $value =~ /-NA-/ );
        if ( $value =~ /,/ ) {
            my @vals = split(/,/,$value);
            $values{$name} = \@vals;
        } else {
            $values{$name} = $value;
        }
    }
    return %values;
} ## end getRowDataAsHashMap

=head2 getPropertiesDataAsHashMap

Returns List<HashMap<String, String>>

=over

=item Options

=over

=item argument_list

 (Optional) $gridID, $columns, $colDisplayName

=back

=back

=over

=back

=cut

sub getPropertiesDataAsHashMap {
    my ( $self, $gridID, @keys ) = @_;
    $Log->enter() if $may_enter;
    my %values;
    foreach my $key (@keys) {
        $key = "gwt-debug-$self->{$SYSTEM_NAME_KEY}".$key;
        unless ( $self->is_element_present("//div[\@id='$gridID']"
                 . "//label[\@for='$key']") ) {
            next;
        }
        my $name = $self->_find_element( "//div[\@id='$gridID']"
            . "//label[\@for='$key']")->get_text();
        $name =~ s/\s+/-/g;
        $name =~ s/[:;'",.]//g;
        $name = lc($name);
        my $value =
          $self->_find_element( "//div[\@id='$gridID']"
              . "//div[\@id='$key']")->get_text();
        $value = '-' unless $value;
        $value = '-' if ( $value =~ /-NA-/ ); 
        if ( $value =~ /,/ ) {
            my @vals = split(/,/,$value);
            $values{$name} = \@vals;
        } else {
            $values{$name} = $value;
        }
    }
    return %values;
} ## end getPropertiesDataAsHashMap

sub stringTrim {
    my ($self, $str) = @_;
    $str =~ s/^\s*|\s*$//g;
    return $str;
}

sub XpathToCssConvertor {
    my ($self, $xpath) = @_;

    $xpath =~ s/\/+/>/g;
    $xpath =~ s/^.*\@id\s*=\s*'([\w\-]+)'[^>]/\#$1/g;
    $xpath =~ s/\[\s*contains\(\s*[\\\@]+class\s*,\s*[\'\"]([\w\-]+)[\'\"]\s*\)\s*\]/\.$1/g;
    $xpath =~ s/\[\s*[\\\@]+class\s*=\s*[\'\"]([\w\-]+)[\'\"]\s*\]/\.$1/g;
    $xpath =~ s/\[\s*[\\\@]+(\w+)\s*=\s*[\'\"]([\w\-]+)[\'\"]\s*\]/\[/g;
    $xpath =~ s/\[\s*text\s*\(\s*\)=\s*([\'\"][\w\-]+[\'\"])\s*\]/\:contains($1)/g;
    $xpath =~ s/\[\s*\]//g;
    $xpath =~ s/\[/\{/g;
    $xpath =~ s/\]/\}/g;

    return $xpath;
}

###############################################################################
# Method:    longestCommonPrefix
# Objective: This function will return the longest common prefix from a array of strings
###############################################################################
sub longestCommonPrefix {

    my $self = shift;

    return $_[0] if @_ == 1;

    my $i          = 0;
    my $first      = shift;
    my $min_length = length($first);
    foreach (@_) {
        $min_length = length($_) if length($_) < $min_length;
    }
  INDEX: foreach my $ch ( split //, $first ) {
        last INDEX unless $i < $min_length;
        foreach my $string (@_) {
            last INDEX if substr( $string, $i, 1 ) ne $ch;
        }
    }
    continue { $i++ }
    my $substr = substr $first, 0, $i;
    unless ($substr) {
        return "noLCPfound";
    }
    return $substr;
}

1;
