# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary API Set for 7mode SNMP commands
## @author Isshwarya.M@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Node::SNMP::7Mode;

#
# Compiler Directives
#
use strict;
use warnings;

# Base class
use base qw(NACL::APISet::Node::SNMP);

# Module import - APISets
use NACL::APISet::Session::SNMP::7Mode;

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

sub new {
    $Log->enter();

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my ( $class, %args ) = @_;
    my $self = {};

    # Bless the hash returned which will make it a Perl object
    bless $self, $class;

    # And give the caller the reference to the bless object.
    $Log->exit();
    return $self;
}

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Node::SNMP::7Mode - API Set for 7Mode SNMP commands

=head1 SYNOPSIS

 use NACL::APISet::Node::SNMP::7Mode;

=head1 DESCRIPTION

A C<NACL::APISet::Node::SNMP::7Mode> is used to connect to and send SNMP 
commands to the 7Mode node.

=cut

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Node::SNMP::7Mode>.

=over 

=item Synopsis

 $api_set_obj = NACL::APISet::Node::SNMP::7Mode->new( %args );

=item Arguments

=over 

=item %args

(Optional)
Parameters passed to parent classes.  See NACL::APISet for details.

=back

=item Returns

A reference to a new SNMP API Set for 7Mode.

=back

=cut

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut
