# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Resonse sub class for Firmware API Sets
## @author anerudh@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Response::CLI::Firmware;

use strict;
use warnings;

use base qw(NACL::APISet::Response::CLI);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(SCALAR);
use Data::Dumper;

sub new {
    $Log->enter();

    my ($class, %args) = @_;

    # Fill the member vars for this instance
    my $self = NACL::APISet::validate_params(
        $class,
        params => \%args,
        spec   => {
            output           => { type => SCALAR, required => 1},
            command          => { type => SCALAR, required => 1},
            method_name      => { type => SCALAR, required => 1}, 
            cdef_path        => { type => SCALAR, required => 1}, 
            execution_time   => { type => SCALAR, required => 1},
            parser_file      => { type => SCALAR, required => 1},           
        }
    );

    bless $self, $class;
    
    # Populate parsed output at time of invoking API
    $self->_parse_output();
    
    $Log->debug( Dumper($self) );
    $Log->exit();
    return $self;
}

1;

=head1 NAME

NACL::APISet::Response::CLI::Firmware - Firmware CLI Response class

=head1 SYNOPSIS

 use NACL::APISet::Response::CLI::Firmware;

=head1 DESCRIPTION

A C<NACL::APISet::Response::CLI::Firmware> object is used to recieve, parse and 
hold the output from Firmware CLI API methods.

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Response::CLI::Firmware>.

=over 

=item Synopsis

 $response = NACL::APISet::Response::CLI::Firmware->new( %args );

=item Arguments

=over 

=item output

The raw output from the command.  This can be parsed into an array of key value pairs.

=item command

The command invoked on the service

=item method_name

Name of the method invoked.

=item cdef_path

Path to the directory containing the Freebsd CDEF files

=item execution_time

Time taken to execute the command

=item parser_file

The name of the parser file

=back

=back

=cut

