# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Response/VMAPI/ESX.pm $
#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Used to construct a new Response object for the VIM object method output
## @author bkprasha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Response::VMAPI::ESX;

#
# Compiler Directives
#
use strict;
use warnings;

#
# NATE imports
#
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);

use Data::Dumper;

#
#APISet imports
#
use base qw(NACL::APISet::Response::VMAPI);

sub new {
    $Log->enter() if $may_enter;

    my ( $class, %args ) = @_;
    my $self = NACL::APISet::validate_params(
        $class,
        params => \%args,
        spec   => {
            parsed_output	 => { optional => 1},
            output			 => { required => 1,},
            execution_time   => { type => SCALAR, optional => 1, },
            method_name      => { type => SCALAR, required => 1, },
        }
    );

    bless $self, $class;

    $Log->debug( Dumper($self) );
    $Log->exit() if $may_exit;
    return $self;
}

=head1 NAME

NACL::APISet::Response::VMAPI::ESX - Used to construct a new Response object for the VIM object method output

=head1 SYNOPSIS

 use NACL::APISet::Response::VMAPI::ESX;

=head1 DESCRIPTION

A C<NACL::APISet::Response::VMAPI::ESX> constructs a new VIM APISet Response object. 

=head1 Constructors

=head2 new

Returns a reference to a newly created C<NACL::APISet::Response::VMAPI::ESX>.  

# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Response/VMAPI/ESX.pm $

=cut


1;
