#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Used to construct a new Response object for the Xen object method output
## @author bkprasha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Response::XAPI::Xen;

#
# Compiler Directives
#
use strict;
use warnings;

#
# NATE imports
#
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);

#APISet imports
use base qw(NACL::APISet::Response);

sub new {
    $Log->enter() if $may_enter;

    my ( $class, %args ) = @_;
    my $self = NACL::APISet::validate_params(
        $class,
        params => \%args,
        spec   => {
            parsed_output    => { type => ARRAYREF|HASHREF, },
            output           => { type => ARRAYREF|HASHREF, },
            execution_time   => { type => SCALAR, },
            method_name      => { type => SCALAR, },
        }
    );

    bless $self, $class;

    $Log->exit() if $may_exit;
    return $self;
}

=head1 NAME

NACL::APISet::Response::XAPI::Xen - Used to construct a new Response object for the Xen object method output

=head1 SYNOPSIS

 use NACL::APISet::Response::XAPI::Xen;

=head1 DESCRIPTION

A C<NACL::APISet::Response::XAPI::Xen> constructs a new Xen APISet Response object. 

=head1 Constructors

=over

=back

=head2 new

Returns a reference to a newly created C<NACL::APISet::Response::XAPI::Xen>.  


=cut

1;
