# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Response class for ZAPI DBlade APISet
## @author rawat@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Response::ZAPI::DBlade;

use strict;
use warnings;

use base qw(NACL::APISet::Response::ZAPI);

use NACL::APISet::Exceptions::NoMatchingEntriesException qw(:try);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

# Constructor
sub new {
    $Log->enter();

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my ($class, %args) = @_;

    # Fill the member vars for this instance
    my $self = NACL::APISet::validate_params
	   ($class,
	    params => \%args,
		spec   => {
		    output           => { type => SCALAR, required => 1,},
			command          => { type => SCALAR, required => 1,},
			execution_time   => { type => SCALAR, required => 1,},
			execution_status => { type => SCALAR, required => 1,},
			error_number     => { type => SCALAR, required => 1,},
			error_reason     => { type => SCALAR, required => 1,},
		}
	);

    # This tags the hash pointed to by $object as a
    # type of DBlade
    bless $self, $class;

    # Populate parsed output at time of invoking API
    $self->_parse_output();

    # And give the caller the referece to the bless object.
    $Log->debug( sub { "Dumper of $class object:\n" . Dumper($self) } );
    $Log->exit();
    return $self;
}

# For other sets, "complete" raw output contains the netapp tag. DBlade ZAPIs
# do not return this tag, so the complete raw output should be made to be the
# same as the raw output.
sub get_complete_raw_output {
    my ($self, %opts) = @_;
    return $self->get_raw_output(%opts);
}

1;

=head1 NAME

NACL::APISet::Response::ZAPI::DBlade - DBlade ZAPI Response class

=head1 SYNOPSIS

 use NACL::APISet::Response::ZAPI::DBlade;

=head1 DESCRIPTION

A C<NACL::APISet::Response::ZAPI::DBlade> object is used to recieve, parse and hold
the output from DBlade ZAPI API methods.

=cut

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Response::ZAPI::DBlade>.

=over

=item Synopsis

 $response = NACL::APISet::Response::ZAPI::DBlade->new( %args );

=item Arguments

=over

=item output

The raw xml output from the command.  This can be parsed into an array of
key value pairs.

=item command

The ZAPI command which has been sent to the filer.

=item execution_time

The time taken to execute the ZAPI command.

=item execution_status

The status returned after the execution of ZAPI command.
Possible values are passed and failed.

=item error_number

The error number returned on the execution of ZAPI

=item error_reason

The reason for error in case if any

=back

=back

=cut

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

