#
# Copyright (c) 2015-2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary CDEF class specific to CMode-CLI
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Schema::Cdef::CModeCLI;

use strict;
use warnings;

use base qw(NACL::APISet::Schema::Cdef);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

# A simplified translate for CMode-CLI. Does not require all of the
# complex logic in Schema::translate, which handles option_order,
# option separators and other such features.
# This simply needs to handle:
# * If extended_query is provided, construct it first.
# * For each set $key => $value, the options should be translated as -$key $value
# * If pass_through is specified, append it.
sub translate {
	$Log->enter();
	my ($self, %args) = @_;
	my $extended_query = delete $args{extended_query};
	my $pass_through   = delete $args{pass_through};
	my $ret_cmd        = $self->get_command();
	
	# The extended query needs to be supplied after the command
    # and before the other options. For example, this works:
    # volume modify { -state online } -state offline
    # But this does not:
    # volume modify -state offline { -state online }
    if ( keys %$extended_query ) {

        # This is CMode-only, so simply obtaining the key mapped to
        # and putting a space between the pairs of options. We don't
        # need to worry about insert characters and boolean options.
        $ret_cmd .= ' {';
        while ( my ( $key, $value ) = each %$extended_query ) {
            $ret_cmd .= " -$key $value" if (defined $value);
        }
        $ret_cmd .= ' }';
    }
	
	while (my ($key, $value) = each %args) {
		# as per burt#1078377, special handling for 'application show-detail' command
		# fields 'cifs-permissions' and 'nfs-permissions' containing spaces
		$value = '{'.$value.'}' if ( ($ret_cmd =~ /application\s+show-detail/) && ($value =~ /\S+\s+\S+/) && ($key =~ /cifs-permissions|nfs-permissions/) );
		$ret_cmd .= " -$key $value" if (defined $value);
	}
	
	$ret_cmd .= " $pass_through" if ( defined $pass_through );

    $ret_cmd =~ s/\s+$//;
	
	$Log->exit();
	return $ret_cmd;
}

1;
