# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Session/VMAPI/ESX.pm $
#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Used to construct a new APISet VIM Session object
## @author bkprasha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Session::VMAPI::ESX;

#
#Compiler directives
#
use strict;
use warnings;
use Params::Validate qw(:all);
use Data::Dumper;

#
#APISet imports
#
use base qw(NACL::APISet::Session::VMAPI);

#
#NATE imports
#
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();



sub new {
    $Log->enter() if $may_enter;

    my ($class, %args) = @_;
    
    my $self = NACL::APISet->validate_params(
        params => \%args,
        spec   => {
        	hostobj	  => { required => 1},
            username  => { type => SCALAR, optional => 1, },
            password  => { type => SCALAR, optional => 1, },
            ipaddress => { type => SCALAR, optional => 1, },
        },
    );
    
    bless $self, $class;

	if ( !($self->{username}) ) {
		$self->{username} = $self->{hostobj}->default_username();
	}
	
	if ( !($self->{password}) ) {
		$self->{password} = $self->{hostobj}->default_password();
	}
	
	if ( !($self->{ipaddress}) ) {
		$self->{ipaddress} = $self->{hostobj}->default_ip();
	}

    $self->esx_connect();

    $Log->exit() if $may_exit;
    return $self;
}

=head1 NAME

NACL::APISet::Session::VMAPI::ESX - Used to construct a new APISet VIM Session object

=head1 SYNOPSIS

 use NACL::APISet::Session::VMAPI::ESX;

=head1 DESCRIPTION

A C<NACL::APISet::Session::VMAPI::ESX> constructs a new VIM APISet session object. 

=head1 Constructors

=head2 new

Returns a reference to a newly created C<NACL::APISet::Session::VMAPI::ESX>.  

# $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Session/VMAPI/ESX.pm $

=cut


1;
