#
# Copyright (c) 2012-* NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Used to construct a new APISet Xen Session object
## @author rawat@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Session::XAPI::Xen;

#Compiler directives
use strict;
use warnings;
use Params::Validate qw(:all);

# Import Exceptions
use NATE::Exceptions::Argument qw(:try);

#APISet imports
use base qw(NACL::APISet::Session::XAPI);

#NATE imports
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub new {
    $Log->enter() if $may_enter;

    my ($class, %args) = @_;
    
    my $self = NACL::APISet->validate_params(
        params => \%args,
        spec   => {
            connection => { isa  => "Connectrec", optional => 1, },
            hostobj	   => { isa  => "Hostrec", optional => 1, },
            username   => { type => SCALAR, optional => 1, },
            password   => { type => SCALAR, optional => 1, },
            ipaddress  => { type => SCALAR, optional => 1, },
        },
    );
    
    bless $self, $class;
    
    my $hostobj =  $args{hostobj} || $args{connection}->hostp();
    
    $self->{username}  ||= $hostobj->default_username();
    $self->{password}  ||= $hostobj->default_password();
    $self->{ipaddress} ||= $hostobj->default_ip();

    $self->xen_connect();

    $Log->exit() if $may_exit;
    return $self;
}

=head1 NAME

NACL::APISet::Session::XAPI::Xen - Used to construct a new APISet Xen Session object

=head1 SYNOPSIS

 use NACL::APISet::Session::XAPI::Xen;

=head1 DESCRIPTION

A C<NACL::APISet::Session::XAPI::Xen> constructs a new Xen APISet session object. 

=over

=back

=head1 Constructors

=head2 new

Returns a reference to a newly created C<NACL::APISet::Session::XAPI::Xen>.  

=over

=back

=cut


1;
