# $Id: //depot/prod/test/main/lib/NACL/APISet/Session/ZAPI/Windows.pm#1 $

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Class to hold all connection information for Windows Sessions
## @author tanzim@netapp.com, dl-sm-nacl-dev@netapp.com, nsesha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

# Not all 'connections' to services will be in the form of NATE::Connectionrec
# The Session class will define all data needed for connections, whether
# NATE Connection or not.

package NACL::APISet::Session::ZAPI::Windows;

use strict;
use warnings;

use base qw(NACL::APISet::Session::ZAPI);

use NACL::APISet::Response::ZAPI::Windows;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;


##############################################################################
# Method:    send_command
# Objective: Send the ZAPI command to the Windows host.
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub send_command {
    $Log->enter();
    my ($self, %args) = @_;
    my $response = $self->SUPER::send_command(%args, _response_type => 'Windows');
    $Log->exit();
    return $response;
}

=head1 NAME

NACL::APISet::Session::ZAPI::Windows - Session class for Windows ZAPI commands

=head1 SYNOPSIS

 use NACL::APISet::Session::ZAPI::Windows;

=head1 DESCRIPTION

A C<NACL::APISet::Session::ZAPI::Windows> provides the methods needs to control the 
connection to a Windows host.  

=cut

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Session::ZAPI::Windows>.

=over

=item Synopsis

 $session = NACL::APISet::Session::ZAPI::Windows->new( %opt );

=item Arguments

For POD refer base class L<initialize|lib-NACL-APISet-Session-ZAPI-pm/initialize>

=item Returns

A reference to a new Session.

=back

=cut

=head1 METHODS

=head2 send_command

Send the command to the host.  The output of the command is returned in a 
NACL::APISet::Response::ZAPI::Windows object.

=over 

=item Synopsis

$response = $session->send_command( command => "<Windows-about/>" );

=item Arguments

=over 

=item command

The input raw xml to be sent

=back

=item Returns

Response object of type NACL::APISet::Response::ZAPI::Windows

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-sm-nacl-dev@netapp.com)

=back

 $Id: //depot/prod/test/R8.1x/lib/NACL/APISet/Session/ZAPI/Windows.pm#1 $
 Last revised $Date: 2011/05/12 $ by $Author: build $

=cut

1;
