#$Id$
#
# Copyright (c) 2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base for API Sets CLI commands
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Switch;

#
# Compiler Directives
#
use strict;
use warnings;
use Data::Dumper;

# NATE imports
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::BaseException qw(:try);

# Base class
use base qw(NACL::APISet);

##############################################################################
#                               Private Methods
##############################################################################
sub _read_default_cdef_dir {
    $Log->enter();
    my $pkg_or_obj = shift();
    
    my %args = $pkg_or_obj->validate_params(
        params => \@_,
        spec   => $pkg_or_obj->_read_default_cdef_dir_spec,
    );
    
    my @cdef_paths = $pkg_or_obj->_read_shared_cdef_dir(%args);
    
    $Log->exit();
    
    return @cdef_paths;
}

###############################################################################
# Method:    find_cdef_directory
# Objective: To find the cdef directory based on various version related
#            attributes obtained from the service.
# Details:   See POD documentation at the end of this file.
###############################################################################

sub find_cdef_directory {
    $Log->enter();
    my $self = shift();

    # Load CDEFs from the CDEF directories which we maintain based on branch
    # and release information.

    my $interface = lc( $self->{interface} );
    my $set       = lc( $self->{set} );

    my @cdef_paths = $self->get_default_cdef_dir_list();

    $Log->debug("CDEF directories found are:\n" . Dumper(\@cdef_paths));
    $Log->exit();
    
    return @cdef_paths;
}

1;

=head1 NAME

NACL::APISet::Switch - Base class for API Sets for Switchs

=head1 SYNOPSIS

 use NACL::APISet::Switch;

=head1 DESCRIPTION

A C<NACL::APISet::Switch> is used to create API Set objects for Switchs. This acts
as a factory class for creation of API Set objects for the interface CLI.

=cut

=head1 EXCEPTIONS

=over

=item NATE::BaseException

=back

=cut

=head1 METHODS

=head2 find_cdef_directory

This method gets the information of various version related attributes and uses
them to autodiscover the CDEF path. The data about the CDEF directory to
be loaded is maintained in a separate XML file from which it finds the CDEF path
based on the discovered version attributes. The version attributes to be
discovered will vary depending on the category and set.

=over

=item Synopsis

my $cdef_dir = $Api_Set_Obj->find_cdef_directory();

=item Arguments

Nothing.

=item Returns

Returns the relative path to CDEF directory.

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

 $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch.pm#1 $
 Last revised $Date: 2011/05/25 $ by $Author: rawat $

=cut

