# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
## @summary Routines to parse Armadillo bridge commands
##
## @description
##    This is a library to parse the raw output of Armadillo bridge commands.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::Armadillo::default::firmwarerestart
##
##  firmwarerestart(%args)
##
## @author dl-nacl-dev@netapp.com
##
## @status public


package NACL::APISet::Switch::CLI::Armadillo::default::firmwarerestart;

use NACL::APISet::Response::CLI::ParserUtils;

use NACL::APISet::Exceptions::InvalidParamValueException;

use strict;
use warnings;

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my $error_message;
    my $error = undef;
    
    if ( $raw_output =~ /Wrong\/Missing Parameters/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
}

## @name ::firmwarerestart
##
## @summary Parses the raw output from the "firmwarerestart" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from firmwarerestart with any of its options as described in the
##    corresponding cdef file 'firmwarerestart.cdefs'
## @synopsis
##      firmwarerestart(%args);
## INPUTS TO METHOD CALL 'firmwarerestart' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = firmwarerestart

##&nbsp;-------- Parsed Output ------
##&nbsp;
##&nbsp;$VAR1 = [];
##&nbsp;
sub firmwarerestart {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $output     = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $raw_output = $resp->get_raw_output();
    my $error      = _check_error(
        raw_output      => $raw_output,
        response_object => $resp
    );
    return [];
}

1;
