#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/BrocadeFC/default/cfgsave.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC cfgsave command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch cfgsave command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::cfgsave
##
##  cfgsave(%args)
##
## @author benjaram@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::cfgsave;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args = @_;

    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;

    # Check for the error
    if ( $raw_output =~ /Usage:(.*)/ ) {
        $$error_message = $1;
        $error          = "InvalidParam";
    }
    return $error;
} ## end sub _check_error

## @name ::cfgsave
##
## @summary Parses the raw output from the "cfgsave" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cfgsave with any of its options as described in the
##    corresponding cdef file 'cfgsave.cdefs'
## @synopsis
##      cfgsave(%args);
## INPUTS TO METHOD CALL 'cfgsave' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'Nothing changed',
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub cfgsave {
    my %args = @_;

    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key );

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _check_error(
        raw_output    => $output,
        command       => "cfgsave",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %hash;

    #To match with the multiple patterns like 'Openration cancelled,
    #Nothing changed, Updating flash'.
    if ( $output
        =~ /((Operation cancelled.*)|(Nothing changed.*)|(Updating flash.*))/i
        ) {
        $hash{'status'} = $1;
    }

    push( @result_array, \%hash );
    return \@result_array;
} ## end sub cfgsave
1;
