#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/BrocadeFC/default/chassisname.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC chassisname command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch chassisname command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::chassisname
##
##  chassisname(%args)
##
## @author sheethal@netapp.ocm,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::chassisname;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

## @name ::chassisname
##
## @summary Parses the raw output from the "chassisname" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from chassisname with any of its options as described in the
##    corresponding cdef file 'chassisname.cdefs'
## @synopsis
##      chassisname(%args);
## INPUTS TO METHOD CALL 'chassisname' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = chassisname
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Brocade6510
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'name' => 'Brocade6510'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub chassisname {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;
    my (@result_array);

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message,
        command       => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    my @lines = get_lines(raw_output => $output, response_object => $resp);
    foreach my $line (@lines) {
        if ($line =~ /^\w+/) {
            push(@result_array, {name => $line});
            last;
        }
    }
    return \@result_array;
}    # end sub chassisname

sub _parse_for_errors {
    my %args = @_;

    my $output        = $args{raw_output};
    my $error_message = $args{error_message};
    my $command       = $args{command};
    my $error;

    return $error;
} ## end sub _parse_for_errors

1;
