#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/BrocadeFC/default/fosexec.pm# $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC fosexec command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch fosexec command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::fosexec
##
##  fosexec(%args)
##
## @author udaykk@netapp.com dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::fosexec;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args = @_;

    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;

    # Check for the error
    if ( $raw_output =~ /Usage:(.*)/ ) {
        $$error_message = $1;
        $error          = "InvalidParam";
    } elsif ( $raw_output =~ /(FID \d+ is currently not associated with any logical switch)|(Failed to execute the command on logical switch with FID \d+)/ ) {
        $$error_message = $1;
        $error          = "InvalidParam";
    }
    return $error;
} ## end sub _check_error

## @name ::fosexec
##
## @summary Parses the raw output from the "fosexec" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from fosexec with any of its options as described in the 
##    corresponding cdef file 'fosexec.cdefs'
## @synopsis
##      fosexec(%args);
## INPUTS TO METHOD CALL 'fosexec' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '--fid' </td> <td> 'fid-number' </td> <td> '0' </td> </tr> 
## <tr> <td> '-cmd' </td> <td> 'command' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = fosexec --fid 128 -cmd switchenable
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'fid_number' => '128',
##&nbsp;            'command_name' => 'switchenable'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = fosexec --fid 128 -cmd switchdisable
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'fid_number' => '128',
##&nbsp;            'command_name' => 'switchdisable'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub fosexec {
    my %args = @_;

    my $error_message;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    my @result_array;

    $error = _check_error(
        raw_output    => $output,
        command       => "fosexec",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my %info;
    if( $output =~ /"(\w+)"\s+\w+\s+FID\s+(\d+):/) {
        $info{"command_name"} = $1;
        $info{"fid_number"} = $2;
        push(@result_array, \%info);
    }
    
    return \@result_array;
} ## end sub fosexec
1;
