#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/BrocadeFC/default/setcontext.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC setcontext command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch setcontext command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::setcontext
##
##  portcfg(%args)
##
## @author udaykk@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::setcontext;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error;
    if($raw_output =~ /(Invalid FID)|(FID not associated with a switch)/i) {
        $error = "InvalidParam";
    } elsif($raw_output =~ /Usage/i) {
        $error = "InvalidParamValue";
    }
    return $error;
}

## @name ::setcontext
##
## @summary Parses the raw output from the "setcontext" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from setcontext with any of its options as described in the 
##    corresponding cdef file 'setcontext.cdefs'
## @synopsis
##      setcontext(%args);
## INPUTS TO METHOD CALL 'setcontext' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'fid-number' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = setcontext 128
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub setcontext {
    my %args         = @_;
    my $error_message;
    my $response = $args{response_object};
    my $raw_output = $response->get_processed_output();
    my $command = $response->get_command();
    my $error = _check_error("raw_output"    => $raw_output,
                             "command"       => "setcontext",
                             "error_message" => \$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $response );
    }

   return  [];
}
1;
