#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/BrocadeFC/default/tsclockserver.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC  tsclockserver command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch  tsclockserver command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::tsclockserver
##
##   tsclockserver(%args)
##
## @author sangavai@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::tsclockserver;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my ( $error_message, $error );
    my $error_level = $resp->get_execution_status();
    if ( $raw_output =~ /Invalid NTP servers|NTP Query Failed/i ) {
        $error          = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

## @name ::tsclockserver
##
## @summary Parses the raw output from the "tsclockserver" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from tsclockserver with any of its options as described in the
##    corresponding cdef file 'tsclockserver.cdefs'
## @synopsis
##      tsclockserver(%args);
## INPUTS TO METHOD CALL 'tsclockserver' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'ntp-server' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = tsclockserver
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;; echo __status__=$?
##&nbsp;Active NTP Server		LOCL
##&nbsp;Configured NTP Server List  LOCL
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'configured_ntp_server_list' => 'LOCL',
##&nbsp;            'active_ntp_server' => 'LOCL'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub tsclockserver {
    my %args = @_;
    my (@result_array);
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    @lines = grep (! /echo __status__=\$\?/, @lines);
    for ( my $i = 0; $i <= $#lines; $i++ ) {
        if($lines[$i] =~ /^(\w.*\w)\s+(\w+.*)$/) {
            $result_array[0]->{ to_key_name( item => $1)} = $2;
        }
    }
    return \@result_array;
} ## end sub tsclockserver

1;
