#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/CiscoFC/default/fcalias.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoFC fcalias command
##
## @description
##    This is a library to parse the raw output of CiscoFC switch fcalias command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::fcalias
##
##  fcalias(%args)
##
## @author udaykk@netapp.com dl-nacl-dev@netapp.com
##
## @status public
#
package  NACL::APISet::Switch::CLI::CiscoFC::default::fcalias;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args = @_;

    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;

    # Check for the error
    if ( $raw_output =~ /Usage:(.*)/ ) {
        $$error_message = $1;
        $error          = "InvalidParam";
    }
    return $error;
} ## end sub _check_error

## @name ::fcalias
##
## @summary Parses the raw output from the "fcalias" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from fcalias with any of its options as described in the 
##    corresponding cdef file 'fcalias.cdefs'
## @synopsis
##      fcalias(%args);
## INPUTS TO METHOD CALL 'fcalias' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'name' </td> <td> 'fcalias-name' </td> <td> '0' </td> </tr> 
## <tr> <td> 'vsan' </td> <td> 'vsan' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = fcalias name Alias vsan 1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub fcalias {
    my %args = @_;

    my $error_message;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error = _check_error(
        raw_output    => $output,
        command       => "fcalias",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
} ## end sub fcalias
1;
