#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoFC member command
##
## @description
##    This is a library to parse the raw output of CiscoFC member command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::member
##
##  member(%args)
##
## @author skotagir@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoFC::default::member;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my $error;
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    if($raw_output =~ /Invalid\s+command/) {
        $error = "InvalidParam";
        $$error_message = $1 ;
    }elsif($raw_output =~ /Incomplete\s+command/){
        $error = "MissingRequiredParam";
        $$error_message = $1 ;
    }elsif($raw_output =~ /(.*\s+not\s+present)/){
        $error = "InvalidParamValue";
        $$error_message = $1 ;
    }elsif($raw_output =~ /(Incomplete\s+IPv6\s+prefix)/){
        $error = "InvalidParamValue";
        $$error_message = $1 ;
    }elsif($raw_output =~ /(Invalid\s+ip\s+address)/){
        $error = "InvalidParamValue";
        $$error_message = $1 ;
    }
    return $error;
}

## @name ::member
##
## @summary Parses the raw output from the "member" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from member with any of its options as described in the
##    corresponding cdef file 'member.cdefs'
## @synopsis
##      member(%args);
## INPUTS TO METHOD CALL 'member' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> 'domain-id' </td> <td> 'domain-id' </td> </tr>
## <tr> <td> 'fcalias' </td> <td> 'fcalias-name' </td> </tr>
## <tr> <td> 'fwwn' </td> <td> 'fwwn-id' </td> </tr>
## <tr> <td> 'swwn' </td> <td> 'swwn-id' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'zone-name' </td> </tr>
## <tr> <td> 'symbolic-nodename' </td> <td> 'symbolic-nodename' </td> </tr>
## <tr> <td> 'interface' </td> <td> 'interface' </td> </tr>
## <tr> <td> 'pwwn' </td> <td> 'pwwn-id' </td> </tr>
## <tr> <td> 'port-number' </td> <td> 'port-number' </td> </tr>
## <tr> <td> 'device-alias' </td> <td> 'device-alias-name' </td> </tr>
## <tr> <td> 'ip-address' </td> <td> 'ip-address' </td> </tr>
## <tr> <td> 'lun' </td> <td> 'lun-id' </td> </tr>
## <tr> <td> 'fcid' </td> <td> 'fcid' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub member {
    my %args = @_;
    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my $error = _check_error(raw_output=>$output,error_message=>\$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message, response_object => $resp);
    }
    return [] ;
}
1;

