#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/CiscoNetwork/Nexus/reload.pm# $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoNetwork reload command
##
## @description
##    This is a library to parse the raw output of CiscoNetwork switch reload command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoNetwork::Nexus::reload
##
##  reload(%args)
##
## @author skotagir@netapp.com ,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoNetwork::Nexus::reload;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error = undef;

    # Check for the error
    if ($raw_output =~ /FEX\s+\d+\s+is\s+not\s+present/) {
        $$error_message = $raw_output;
        $error = "InvalidParamValue";
    } elsif ($raw_output =~ /(Incomplete\s+command)/i) {
        $$error_message = $raw_output;
        $error = "MissingRequiredParam";
    } elsif($raw_output =~ /Invalid number, range is (100:199)/) {
        $$error_message = $raw_output;
        $error = "InvalidParamValue";
    }

    return $error;
}

## @name ::reload
##
## @summary Parses the raw output from the "reload" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from reload with any of its options as described in the
##    corresponding cdef file 'reload.cdefs'
## @synopsis
##      reload(%args);
## INPUTS TO METHOD CALL 'reload' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'fex' </td> <td> 'fex' </td> <td> '0' </td> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub reload {
    my %args = @_;

    my ($error_message);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my $error ;

    $error = _check_error(raw_output    => $output,
                              command       => "reload",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }

    return [];
}

1;
