#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/CiscoNetwork/Nexus/snmp_server.pm# $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoNetwork snmp_server and its subcommands
##
## @description
##    This is a library to parse the raw output of CiscoNetwork switch snmp_server and its subcommands.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoNetwork::Nexus::snmp_server
##
##  snmp_server(%args)
##
## @author dl-nacl-dev@netapp.com, skotagiri@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoNetwork::Nexus::snmp_server;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error ;

    if($raw_output =~ /group\s+not\s+found/){
        $$error_message = $raw_output;
        $error = "InvalidParamValue";
    } elsif($raw_output =~ /SNMP\s+community\s+entry\s+not\s+found/){
        $$error_message = $raw_output;
        $error = "InvalidParamValue";
    }
    return $error;
}

## @name ::snmp_server_community
##
## @summary Parses the raw output from the "snmp-server community" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from snmp_server_community with any of its options as described in the
##    corresponding cdef file 'snmp_server.cdefs'
## @synopsis
##      snmp_server_community(%args);
## INPUTS TO METHOD CALL 'snmp_server_community' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'group' </td> <td> 'group' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'community' </td> <td> '0' </td> </tr>
## <tr> <td> 'ro' </td> <td> 'read_only' </td> <td> '1' </td> </tr>
## <tr> <td> 'use-acl' </td> <td> 'acl_name' </td> <td> '0' </td> </tr>
## <tr> <td> 'rw' </td> <td> 'read_write' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = snmp-server community comm1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;config)# snmp-server community comm1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub snmp_server_community {
    my %args = @_;
    my ($error_message ,$error);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error_level = $resp->get_execution_status();

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _check_error(raw_output    => $output,
                          command       => $command,
                          error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }

    return [];
}

1;
