#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/NetAppNetwork/CN1601/interface.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary : interface command
##
## @description
##    This is a library to parse the raw output of NetAppNetwork switch interface command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::NetAppNetwork::CN1601::interface
##
##  interface(%args)
##
## @author veerap@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::NetAppNetwork::CN1601::interface;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error_message, $error );

    # Check for the error
    if ( $raw_output =~ /.*Invalid input detected.*|.*Command not found.*/ ) {
        $error_message = $raw_output;
        $error         = "InvalidParam";
    }
    elsif ( $raw_output =~ /An invalid interface range.*/ ) {
        $error_message = $raw_output;
        $error         = "InvalidParamValue";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
}

## @name ::interface
##
## @summary Parses the raw output from the "interface" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from interface with any of its options as described in the
##    corresponding cdef file 'interface.cdefs'
## @synopsis
##      interface(%args);
## INPUTS TO METHOD CALL 'interface' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub interface {
    my %args = @_;

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();

    my $error = _check_error(
        raw_output      => $output,
        response_object => $resp
    );

    return [];
}

1;
