#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/NetAppNetwork/CN1601/storm_control.pm# $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary : storm_control command
##
## @description
##    This is a library to parse the raw output of NetAppNetwork switch storm_control command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::NetAppNetwork::CN1610::storm_control
##
##  storm_control(%args)
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::NetAppNetwork::CN1601::storm_control;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error_message, $error );

    # Check for the error
    if ( $raw_output =~ /Invalid input detected.*/ ) {
        $error_message = $raw_output;
        $error         = "InvalidParamValue";
    } elsif ( $raw_output =~ /Command not found/ ) {
        $error_message = $raw_output;
        $error         = "CommandFailed";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error
## @name ::storm_control
##
## @summary Parses the raw output from the "storm-control" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from storm_control with any of its options as described in the 
##    corresponding cdef file 'storm_control.cdefs'
## @synopsis
##      storm_control(%args);
## INPUTS TO METHOD CALL 'storm_control' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'rate' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'feature' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'level' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub storm_control {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    _check_error(
        raw_output      => $output,
        response_object => $resp
    );

    return [];
} ## end sub storm_control

1;
