#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/NetAppNetwork/default/isdp.pm# $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary :  isdp command
##
## @description
##    This is a library to parse the raw output of NetAppNetwork switch isdp command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::NetAppNetwork::default::isdp
##
##  isdp(%args)
##
## @author sangavai@netapp.com , dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::NetAppNetwork::default::isdp;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ($error_message);
    my $error = undef;

    # Check for the error
   if ( $raw_output =~ /(Invalid\s+input\s+detected)/i ) {
        $error_message = $1;
        $error         = "InvalidParam";
    }
	elsif ( $raw_output =~ /(Please specify an integer in the range \d+ to \d+|Value is out of range)/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

## @name ::isdp_run
##
## @summary Parses the raw output from the "isdp" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from isdp with any of its options as described in the
##    corresponding cdef file 'isdp.cdefs'
## @synopsis
##      isdp(%args);
## INPUTS TO METHOD CALL 'isdp' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub isdp_run {
    my %args = @_;

    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );

    return [];
} ## end sub isdp_run


## @name ::isdp_advertisev2
##
## @summary Parses the raw output from the "isdp advertise-v2" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from isdp_advertisev2 with any of its options as described in the 
##    corresponding cdef file 'isdp.cdefs'
## @synopsis
##      isdp_advertisev2(%args);
## INPUTS TO METHOD CALL 'isdp_advertisev2' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub isdp_advertisev2 {
     my %args = @_;

    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );
    

    return [];
} ## end sub isdp_advertisev2


## @name ::isdp_holdtime
##
## @summary Parses the raw output from the "isdp holdtime" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from isdp_holdtime with any of its options as described in the 
##    corresponding cdef file 'isdp.cdefs'
## @synopsis
##      isdp_holdtime(%args);
## INPUTS TO METHOD CALL 'isdp_holdtime' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'seconds' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub isdp_holdtime {
     my %args = @_;

    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );

  
    return [];
} ## end sub isdp_holdtime


## @name ::isdp_timer
##
## @summary Parses the raw output from the "isdp timer" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from isdp_timer with any of its options as described in the 
##    corresponding cdef file 'isdp.cdefs'
## @synopsis
##      isdp_timer(%args);
## INPUTS TO METHOD CALL 'isdp_timer' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'timer' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub isdp_timer {
     my %args = @_;

    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );

    return [];
} ## end sub isdp_timer
1;

