#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/NetAppNetwork/default/mtu.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary :  mtu command
##
## @description
##    This is a library to parse the raw output of NetAppNetwork switch mtu command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::NetAppNetwork::default::mtu
##
##  mtu(%args)
##
## @author sangavai@netapp.com , dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::NetAppNetwork::default::mtu;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ($error_message,$error);
    my $error_level = $resp->get_execution_status();

    # Check for the error
    if ( $raw_output =~ /(Invalid\s+input\s+detected|Command not found)/i ) {
        $error_message = $raw_output;
        $error         = "InvalidParam";
    } elsif ( $raw_output =~ /(Value is out of range.)/i ) {
        $error_message = $raw_output;
        $error         = "InvalidParamValue";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

## @name ::mtu
##
## @summary Parses the raw output from the "mtu" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from mtu with any of its options as described in the
##    corresponding cdef file 'mtu.cdefs'
## @synopsis
##      mtu(%args);
## INPUTS TO METHOD CALL 'mtu' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'mtu' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub mtu {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );

    return [];
} ## end sub mtu
1;

