#
#Copyright (c) 2013 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse QlogicFC admin and its subcommands
##
## @description
##    This is a library to parse the raw output of QlogicFC switch admin and its subcommands.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::QlogicFC::default::admin
##  
##  admin_start(%args)
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::QlogicFC::default::admin;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error ;

    # Check for the error
    if ($raw_output =~ /(Usage:)/i) {
        $$error_message = $raw_output;
        $error = "InvalidParam";
    }

    return $error;
}
## @name ::admin_start
##
## @summary Parses the raw output from the "admin start" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from admin start with any of its options as described in the 
##    corresponding cdef file 'admin.cdefs'
## @synopsis
##      admin(%args);
## INPUTS TO METHOD CALL 'admin start' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = admin
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub admin_start {
    my %args = @_;

    my ($error_message);

    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_command();
    my ($error, $key);
    $error = _check_error(raw_output    => $output,
                              command       => "admin start",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }

    return [];

}

## @name ::admin_stop
##
## @summary Parses the raw output from the "admin stop" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from admin stop with any of its options as described in the 
##    corresponding cdef file 'admin.cdefs'
## @synopsis
##      admin(%args);
## INPUTS TO METHOD CALL 'admin stop' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = admin
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub admin_stop {
    my %args = @_;

    my ($error_message);

    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_command();
    my ($error, $key);
    $error = _check_error(raw_output    => $output,
                              command       => "admin stop",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }

    return [];

}

1;
