#####################################################################
## $Id: //depot/prod/DOT/dev/test/lib/NACL/APISet/Tools/zapi_xml.pm#1 $
##  
## This code generates XML output for Zapi which is organized by
## API and Type, rather than the original single-file output.
##  
## The primary routine is make_xml()
##  
##  
##   <api_prefix>.xml - where api_prefix comes from the first part of
##                           the api name.  For example lun.xml.
##  
package zapi_xml;

require zcategory;
use Text::Wrap qw($columns &wrap);
use constant MAX_DEPTH => 30;
use constant INDENT_SPACE => 2;

my ($OptionsComment) = 
        "The options tag describes what the user inputs "    .
        "to the command.  A simple tag is just a string "    .
        "more complex options will require an array or "     .
        "a hash. You can refer to the User guide: "          .
        "http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/APILayerUsersGuide/index.html#Passing_options " .
        "for various types of options that can be passed. "  .
        "More examples on this can also be found here: " .
        "//depot/prod/test/main/lib/NACL/UnitTest/APISet/Node/ZAPI/api_cmode_zapi.thpl"; 

##  
## indices of apis & types
##  
my(%apiPackageIndex);
my(%apiIndex);

my(%typePackageIndex);
my(%typeIndex);
my(%typeUsed);

my($debug)                 = 0;

##  
## place to create new doc tree
##  
my($default_directory) = "/tmp/zapixml";

##  
## used to collect & flush web content
##  
my $content;

##
##
##
my ($depth)                  = 1;

##  
## Simple types with their javascript alert definitions
##  
my(%simpleTypes) = ( 

    "string" => 
        "The field is a sequence of characters, UTF8 encoded.",

    "boolean" => "\\'true\\' or \\'false\\'", 

    "integer" => "The field is a number.", 

    "optional" => "The field is optional as an input,\\n" 
        . "or may or may not appear as an output.",

    "encrypted" => "The field value is encrypted.  \\n" 
        . "To access, use routines such as \\n"
        . "na_child_get_string_encrypted() and\\n"
        . "na_child_add_string_encrypted().",

    "nonempty" => "The field, if present, must have a non-empty value.",

    "validate" => "The field is validated.  For example,\\n" 
        . "if it is an integer, the value must contain only\\n"
        . "digits and possibly an initial minus sign.\\n",
);

##
## Set the family as given in -M option
##
my $family;

##  
## Flush content of a single file
##  
sub write_to_file { 

    my($file,$content) = @_;

    if ($debug) { 
        print "writing $file\n";
    } 

    open FILE,">$file" || return 1;
    print FILE $content;
    close FILE;

    return 0;
}

##  
## Add content to a single file
##  
sub append_to_file { 

    my($file,$content) = @_;

    if ($debug) { 
        print "appending $file\n";
    } 

    open FILE,">>$file" || return 1;
    print FILE $content;
    close FILE;

    return 0;
}

##  
## Does the file exist?
##  
sub fileExists {

    my($file) = @_;

    open( FILE,$file ) || return( 0 );
    close FILE;

    return( 1 );
}

sub get_indent {
    my ($total_indent) = @_;
    my $indent_string = "";
    my $spaces = $total_indent * INDENT_SPACE;
    for (my $i = 0 ; $i < $spaces ; $i++) {
        $indent_string .= " ";
    }
    return $indent_string;
}

sub add_short_comment { 
    my($comment) = @_;
    my $indent_string = "";
    my $spaces = $indent * INDENT_SPACE;
    for (my $i = 0 ; $i < $spaces ; $i++) {
        $indent_string .= " ";
    }

    $comment =~ s/\-\-+//g;
    $content .= "$indent_string" . "<!-- $comment -->\n";
} 
sub comment_head { 
    my $indent_string = "";
    my $spaces = $indent * INDENT_SPACE;
    for (my $i = 0 ; $i < $spaces ; $i++) {
        $indent_string .= " ";
    }

    $content .= "$indent_string" . "<!-- *******\n";
} 
sub comment_body { 
    
    my($comment) = @_;
    my $indent_string = "";
    my $spaces = $indent * INDENT_SPACE;
    for (my $i = 0 ; $i < $spaces ; $i++) {
        $indent_string .= " ";
    }
    $comment =~ s/\-\-+//g;
    $content .= $indent_string."  ** $comment\n";
} 
sub comment_tail { 
    my $indent_string = "";
    my $spaces = $indent * INDENT_SPACE;
    for (my $i = 0 ; $i < $spaces ; $i++) {
        $indent_string .= " ";
    }

    $content .= $indent_string."  ********** -->\n";
} 

sub add_line { 
    my($comment) = @_;

    my $indent_string = "";
    my $spaces = $indent * INDENT_SPACE;
    for (my $i = 0 ; $i < $spaces ; $i++) {
        $indent_string .= " ";
    }
    $content .=  $indent_string . "$comment\n";
} 

sub debug_line
{ 
    my($comment) = @_;

    return unless($debug);

    my $indent_string = "";
    my $spaces = $indent * INDENT_SPACE;
    for (my $i = 0 ; $i < $spaces ; $i++)
    {
        $indent_string .= " ";
    }

    $content .= "$indent_string" . "DEBUG: " . "$comment\n";
} 

#  
# See if we should handle api based on namespace settings.  
#  
sub should_include_api { 
    my($zapi) = @_;

    if ( ! $zapi ) {
        print STDERR "Warning, zapi is undefined in should_include_api\n";
        return 0;
    }

    if ( $::INCLUDES_TEST_APIS eq "no" && $zapi->has_status("test") ) {
        return 0;
    }

    my $statusok = 0;
    my $s;
    my $n;

    #
    # Status is OK if either we're printing all statuses or the APIs
    # status appears in the list of requested statuses.
    #
    if ( ($::statusnames eq "all") or ($::statusnames eq "all,test")) {
        $statusok = 1;
    } else {
        foreach $s (split /,/,$::statusnames) { 
            if ( $zapi->has_status($s) ) {
                $statusok = 1;
                last;
            }
        }
    }

    my $nameok = 1;
    if (exists($::zapi{should_handle_namespace})) {
        $nameok   = zapi::should_handle_namespace($zapi->namespace());
    }
    my $familyok = 1;
    # Generate consolidated XML for each of the family specified
    my @families = split(/\s*,\s*/, $family);
    foreach (@families) {
        $zapi::family = $_;
        $familyok = zapi::api_is_member_of_family($zapi);
        last if ($familyok);
    }

    if ( $statusok && $nameok && $familyok ) {
        return 1;
    } 
    return 0;
} # end should_include_api()

#  
# API summary - the comment before the API
#  
sub add_api_summary { 

    my($zapi) = @_;

    my($package) = $zapi->package();
    my($api)     = $zapi->name();

    #
    # If we are listing multiple or all statuses, highlight the status
    # in the API header.
    #
    my($status) =
        ( ( $::statusnames eq "all" ) || ( $::statusnames =~ /,/ ) )
        ? $zapi->status() : undef ;

    comment_head();

    if ( defined($status) ) {
        comment_body("$api ($status)");
    } else {
        comment_body("$api");
    }

    #  
    # add the description
    #  
    add_description($zapi->desc());
    comment_tail();
} # end add_api_summary()

#  
# add description - description of API/type
#  
sub add_description { 
    my($desc) = @_;
    my $indent_string = "";
    my $spaces = $indent * INDENT_SPACE;
    for (my $i = 0 ; $i < $spaces ; $i++) {
        $indent_string .= " ";
    }

    $indent_string .= "  ** ";
    $desc =~ s/\-\-+//g;
    my @input  = split (/\s+/, $desc);
    $columns   = 72;
    my $output = wrap($indent_string, $indent_string, @input);
    $output =~ s/\t/        /g;
    $content  .= $output;
    $content  .= "\n";
} # end add_description()

##  
## Add one field tag (name,type,description) for Input/Output/Contents
##  
sub add_option { 
    my($zapi, $tag) = @_;
    my $line = "";

    comment_head();
    #  
    # name
    #  
    $line .= "Name: " . $tag->name();

    #  
    # range
    #  
    if (exists($::tag{range})) {
        $line .= " Range: " . $tag->range();
    }

    #  
    # type
    #  
    $line .= " Type: " . $tag->type();

    #  
    # description
    #  
    $line .= " Desc: " . $tag->desc();

    add_description($line);
    comment_tail();

    my $type = $tag->type();
    if ($type =~ /(.*), optional/) {
        $type = $1;
    }
    if (is_simple_type($type)) {
        add_line("<" . $tag->name() . " scalar=\"\$" . $tag->name() . "\$\">" .
                 "[\$" . $tag->name() . "\$]" .
                 "</" . $tag->name() . ">");
    } else {  
        my $option_string = add_complex_option($zapi, $tag);
        add_line($option_string);
    }
} # end add_option()

#  
# Get string that describes a non simple type
#  
sub add_complex_option { 
    my ($zapi, $tag) = @_;
    my $option_out = "";

    my ($type, $optional) = split /\s+,/, $tag->type();
    my $tag_name = $tag->name();

    # If the option is of complex type, then opening tag for that option
    # will be added in expand_complex_option()
    $option_out .= expand_complex_option($zapi, $tag, $tag->name());
    # closing tag for the complex option
    $option_out .= "<" . "/$tag_name" . ">";

    debug_line("add_complex_option adding: $option_out");

    return $option_out;
} # end add_complex_option()

#
# Given a non simple tag type, return what we need for an option.
#
sub expand_complex_option {
    my ($zapi, $tag, $rec_tag_name) = @_;
    my (%zapiH) = %$zapi;
    
    my $expanded_option   = "";
    my $tag_name          = $tag->name();
    my $tag_type          = $tag->type();
    my $namespace         = $tag->namespace();
    my $type_is_array     = 0;
    my $type_is_simple    = 0;

    #
    # Check out the tag and finish setting all the info variables
    #
    ($tag_type, $optional) = split /\s*,/, $tag->type();
    if (length($optional)) {
        $optional = 1;
    } else {
        $optional = 0;
    }

    $tag_type =~ s/\[]/.ARRAY/;
    
    #
    # Have a version of the type available without the array braces
    #
    my $short_type        = $tag_type;
    $tag_type   =~ s/\s+$//; # strip trailing whitespace
    $short_type =~ s/\.ARRAY//g;
    $short_type =~ s/\s+$//; # strip trailing whitespace
    if ($tag_type =~ /\.ARRAY/) {
        $type_is_array = 1;
    }

    if (is_simple_type($short_type)) {
        $type_is_simple = 1;
    }

    debug_line("expand_complex_option 1:\n"     .
               "depth     = $depth\n"           .
               "name      = $tag_name\n"        .
               "type      = $tag_type\n"        .
               "optnl     = $optional\n"        .
               "short     = $short_type\n"      .
               "namespace = $namespace\n"       .
               "array     = $type_is_array\n"   .
               "simple    = $type_is_simple"    .
               "rec       = $rec_tag_name");

    #
    # Input option is of simple type (string, integer, boolean). Need not do
    # recursion. Reached leaf node.
    #
    if ($type_is_simple) {
        debug_line("expand_complex_option 2: $tag_name $tag_type is simple\n");
        # 1) Simple scalar input or the scalar element of typedef
        #    Eg: ZAPI: aggr-list-info OPTION: is-cfo
        #    Eg: ZAPI: aggr-create OPTION: aggregate
        # 2) It is an array of simple type (eg: string[]).
        #    Eg: ZAPI: igroup-modify OPTION: initiators
        #    The XML should be generated as
        #    <option>
        #       <string>value1</string>
        #       <string>value1</string>
        #    </option>
        if ($type_is_array) {
            # Case (2) explained above
            $expanded_option .= "<" . $tag_name . ">";
            $expanded_option .= "\n" . get_indent($indent + $depth);
            $expanded_option .= "<$short_type array=\"\@" 
                . $rec_tag_name . "\@\">" 
                . "[\@" . $rec_tag_name . "\@]". "</$short_type>" ;
            $expanded_option .= "\n" . get_indent($indent);
        } else {
            # Case (1) explained above
            $expanded_option .= "<" . $tag_name . " scalar=\"\$"
                . $rec_tag_name . "\$\">" 
                . "[\$" . $rec_tag_name . "\$]" ;
        }
        return $expanded_option;
    }
    
    debug_line("expand_complex_option 3: $expanded_option");
    # For typedef inputs, we proceed by adding option name and
    # tag type tags and then we recursively call the same function
    # to expand typedef elements.
    
    my($ztype) = $::Ztypes{$namespace . ":" . $short_type};
    
    my $inelement_count = 0;
    my $element_item;
    foreach my $item (sort keys(%$ztype)) {  
        if ($item =~ /^ELEMENT/) { 
            debug_line("expand_complex_option 4: $item");
            $element_item = $item;
            $inelement_count++; 
        } 
    }
    # There are 3 different cases. Typedef having one element, more than
    # one element and no element. The following 'if' condition will handle the
    # first two cases and the else part will handle the third case. Typedef
    # having one element can be treated as simple scalar/array and this how
    # it is different from typedef having more than one element where the input
    # will be passed as hash or array of hashes.
    if ($inelement_count) {
        debug_line("expand_complex_option 5: hasContent $tag_name $tag_type");
        
        # open tag for the option or element
        $expanded_option .= "<" . $tag_name    . ">";
        
        # open tag for tag type. Always put the tag type even when element name
        # and type name being same and handle it in ZapiSchema, the exception
        # being typedef having one element in scalar context.
        my $element_type = $ztype->{$element_item}->type();
        my $put_tag_type = 1;
        # Typedef having one element will be treated as a simple scalar or
        # simple array if and only if that one element is of simple type and
        # not an array.
        if (($inelement_count == 1) && (is_simple_type($element_type))) {
            # Whether the value will be passed as scalar/array depends on
            # whether option is of type typedef scalar/array.
            if ($tag_type =~ /ARRAY/) {
                # This input will be passed as a simple array.
                $expanded_option .= "\n" . get_indent($indent + $depth);
                $expanded_option .= "<$short_type array=\"\@".$rec_tag_name."\@\">";
            } else {
                # This input will be passed as a simple scalar.
                $put_tag_type = (trim($tag_name) ne trim($short_type))? 1:0;
                if ($put_tag_type) {
                    $expanded_option .= "\n" . get_indent($indent + $depth);
                    $expanded_option .= "<$short_type>" ;
                }
            }
        } else {
            $expanded_option .= "\n" . get_indent($indent + $depth);
            if ($tag_type =~ /ARRAY/) {
                # This input will be passed as a array of hash structure.
                $expanded_option .= "<$short_type typedefarray=\"\%".
                    $rec_tag_name."\%\">";
            } else {
                # This input will be passed as a hash structure.
                $expanded_option .= "<$short_type typedef=\"\%".
                    $rec_tag_name."\%\">";
            }
        }
        
        if ($depth > MAX_DEPTH) {
            print "\nWARNING: option '$tag_name' of ". $zapi->{"APIname"} .
                " is too deep than the maximum depth of ". MAX_DEPTH .
                " ,suspecting recursive input.\nHence expanded the input only".
                " for ". MAX_DEPTH . " levels.\n";
            return "";
        }        
        
        # Expand the typedef elements
        foreach my $item (sort keys(%$ztype)) {
            my $ret; 
            if (!($item =~ /^ELEMENT/)) { 
                next;
            } 
            my $element = $ztype->{$item};
            debug_line("   element name is " . $element->name());
            debug_line("   element type is " . $element->type());

            $expanded_option .= "\n" . get_indent($indent + $depth + 1);
            $depth++;
            
            if (($inelement_count == 1) && (is_simple_type($element_type))) {
                # This tag is of array type but in ZAPIDOC it is defined as 
                # array of typedef which has only one element in the typedef 
                # definition. This input will be passed as a simple array
                # Eg: ZAPI: aggr-create, OPTION: disks
                my $element_name = $element->name();
                if($type_is_array) {
                    # Eg: ZAPI: aggr-create  OPTION: disks of type disk-type 
                    # which has only one element called "name"
                    $expanded_option .= "<" . $element_name . ">"
                         . "[\@" . $rec_tag_name. "\@]" ;
                    
                } else {
                    # An input need not be a typdef of structure which has only
                    # one element. No input will be designed by introducing
                    # unnecessary extra levels. There is no such case observed.
                    # But still this is taken care to cover all combinations
                    $expanded_option .= "<" . $element_name ." scalar=\"\$".
                        $rec_tag_name."\$\"". ">" ."[\$" .$rec_tag_name ."\$]";
                }
            } else {
                # This tag is of type real typedef (A typedef which has more
                # than one key and the input value will be passed as a hash 
                # structure)
                # Eg: ZAPI: aggr-list-info, OPTION: filter-attrs
                my $send = $rec_tag_name. "_" . $element->name();
                #
                # Recursive call here to expand the option that's an ELEMENT
                #
                $ret = expand_complex_option($zapi, $element, $send);
                $expanded_option .= $ret;
            }           
            
            $depth--;
            # closing tag for the element
            # For recursive inputs, we won't be able to reach leaf element and
            # we will stop expanding inputs when we reach the maximum depth
            # and in that case, there is no leaf element and hence we wouldn't
            # have opened opening tag. Hence closing tag is also not required.           
            $expanded_option .= "</" . $element->name . ">" 
                if ((!defined $ret) || ($ret ne ""));
        } # end foreach key
        $expanded_option .= "\n" . get_indent($indent + $depth);
        
        # closing tag for typedef type
        if ($put_tag_type) {
            $expanded_option .= "</" . $short_type . ">";        
            $expanded_option .= "\n" . get_indent($indent);
        }
    } else {
        # There is no element in typedef. 
        debug_line("expand_complex_option 6: NO inelement $tag_name $tag_type");
        if ($type_is_array) {
            # If typedef is array, then specify the tag type (typedef type)
            # Eg: ZAPI:system-api-get-elements, OPTION:api-list
            $expanded_option .= "<" . $tag_name    . ">";
            $expanded_option .= "\n" . get_indent($indent + $depth);                
            $expanded_option .= "<" . $short_type . " array=\"\@" . $rec_tag_name 
                . "\@\">" . "[\@" . $rec_tag_name . "\@]". "</" 
                . $short_type . ">" ;
            $expanded_option .= "\n" . get_indent($indent);
        } else {
            # If typedef is scalar, then tag type (typedef type) shouldn't be
            # specified
            $expanded_option .= "<" . $tag_name . " scalar=\"\$" . $rec_tag_name 
                . "\$\">" . "[\$" . $rec_tag_name . "\$]" ;
        }        
    }
    foreach my $xitem (%$ztype) {
        debug_line("expand_complex_option 7: xitem is $xitem value is " . 
                   $ztype->{$xitem});
    }

    debug_line("expand_complex_option 8: expanded option is \n" . $expanded_option);
    return $expanded_option;
} # end expand_complex_option()

#  
# add input fields of an API (similar to add_contents)
#  
sub add_input { 
    my($api, $zapi) = @_;
    my(%zapiH) = %$zapi; 

    my($item);
    my($input);
    my($hasInput) = 0;
    
    #  
    # one loop through to see if we have input
    #
    foreach $item (sort keys(%zapiH)) { 
        if ($item =~ /^IN/) { 
            $hasInput = 1; 
        }
    } 
    
    #  
    # input table
    #  
    if ($hasInput) { 
        add_line("<$api>");
        $indent++;

        #  
        # entry for each input
        #  
        foreach $input (sort keys(%zapiH)) { 
            
            if (!($input =~ /^IN/)) { 
                next;
            } 

            add_option($zapi, $zapiH{$input});
        } 
        $indent--;
        add_line("</$api>");
    } else {
        add_short_comment("There are no options for this API");
        add_line("<$api></$api>");
    }
   
    $indent--;
} # end add_input()

#  
# Given the tag type return 1 if it's a simple type, 0 otherwise
#  
sub is_simple_type { 
    my($type) = @_;

    if (($type =~ /boolean|string|integer/) && ($type !~ /\[\]/)) {
        return 1;
    }
    return 0;
} # end is_simple_type()

#  
# Write a file for each package and put in comments that list the apis.
#   
sub start_package_files { 
    my($package);
    my($api);
    my($type);

    #  
    # Loop over all packages, printing API info that appears in the
    # package 
    #  
    foreach $package (sort keys %apiPackageIndex) { 
        undef $content; 

        my($file) = "$theDir/$package.xml";
        print "Creating file $file\n";

        #  
        # create the content
        #  
        comment_head();
        comment_body("APISet CDEF files - $package" . ".xml \$Id\$");
        comment_body("");

        ##  
        ## create API section 
        ##  
        comment_body("APIs in Category: $package");

        #  
        # list all APIs in this package
        #  
        foreach $api (sort keys %apiIndex) { 
            my($zapi) = $::Zapis{$api};

            #  
            # skip apis not in the current package
            #  
            if ($zapi->package() ne $package) { 
                next;
            } 

            if ( $::INCLUDE_TEST_APIS eq "yes" && $zapi->has_status("test") ) {
                $api .= " (in development kernels only)"
            }
            comment_body("   $api");
        } # end foreach api
        

        #
        # I'm not real sure what this information is for
        #
        if (exists($::Zcategorys{$package})) {
            my $z_category= $::Zcategorys{$package};
            if ($z_category->discussion() ne "") {
                comment_body("");
                add_description($z_category->discussion());
            }
        }

        comment_body("");
        comment_body("=========================================");
        comment_body("");
        add_description($OptionsComment);

        if ($typePackageIndex{$package}) { 
            #  
            # create Type section 
            #  
            comment_body("\n======================================");
            comment_body("Types in Category: <b>$package</b><br>");

            #  
            # list all Types in this package
            #  
            foreach $type (sort keys %typeIndex) { 

                my($ztype) = $::Ztypes{$type};
                my($zapi) = $ztype->api();

                #  
                # skip apis not in the current package
                #  
                if ($zapi->package() ne $package) { 
                    next;
                } 

                comment_body("   $type");
            } 
        }
        comment_tail();

        write_to_file($file,$content);
    } # end foreach package
} # end start_package_files()

#  
# Make list of APIs we're going to handle.
#  
sub init_api_list { 
    my($api);

    #  
    # create an api file for each api
    #  
    foreach $api (sort keys(%::Zapis)) { 
        my($zapi) = $::Zapis{$api};

        #  
        # Skip if API characteristics do not match what we've been 
        # asked to generate 
        #  
        if (!should_include_api($zapi)) { 
            next;
        } 

        my($package) = $zapi->package();

        $apiIndex{$api} = $package;
        $apiPackageIndex{$package} = 1;             
    }
} # init_api_list

#  
# Create all the individual API files
#  
sub add_apis_to_files { 
    my($api);
    my($apidir);

    #  
    # We do each category ($pkg) separately, so we can
    # keep track of which types are used in the APIs
    # of that category, and put out that list at the end of the page.
    #  
    foreach $pkg (sort keys %apiPackageIndex) { 

        my($file) = "$theDir/$pkg.xml";
        %typeUsed = ();

        #  
        # top of page - start HTML and top links
        #  
        undef $content;
        $content = "<$pkg>\n";
        $indent = 1;
        append_to_file($file,$content);

        foreach $api (sort keys(%::Zapis)) { 
            undef $content;
    
            if ( ! defined $apiIndex{$api} ) {
                next;
            }
    
            my($zapi) = $::Zapis{$api};
    
            my($package) = $zapi->package();
            if ( $package ne $pkg ) {
                next;
            }

            my($command) = "$api";
            add_api_summary($zapi);
            add_line("<$command>");
            $indent = 2;
            add_line("<command>$api</command>");
    
            #  
            # the sections of the api
            #  
            add_input($api, $zapi);
            # add_expected_output($zapi);
    
            $indent = 1;

            #
            # Close the command tag for the API
            #
            add_line("</$command>\n");

            #
            # Write the content for all the API's to the file
            #
            append_to_file($file,$content);
        } # end for each api

        #
        # Close the top level tag for the package
        #
        $content = "</$pkg>\n";
        append_to_file($file,$content);
    } # end foreach pkg
} # end add_apis_to_files()

#  
# make a directory 
#  
sub make_dir { 
    my($dir) = @_;

    print "Making directory $dir\n";
    mkdir "$dir",0755;
} # end make_dir()

#  
# make xml for all the APIs, Types, and so on 
#  
sub make_xml { 
    #
    # Global variables
    #
    $theDir  = shift;
    $family = shift;
    $indent  = 0;

    #  
    # determine directory to write stuff
    #  
    if (!$theDir) { 
        $theDir = $default_directory;
        print "Using default directory '$theDir'\n";
    } else {
        print "Using directory '$theDir'\n";
    }

    #  
    # make sure top level directory does not exist
    #  
    if (fileExists($theDir)) { 
        zapi::fatalerr("ERROR: directory already exists: '$theDir'");
    }

    #  
    # Prepare the list of APIs we're going to handle.
    #  
    init_api_list();

    #  
    # make the top-level directory
    #  
    make_dir($theDir);

    #  
    # now start writing package summary indices
    #  
    start_package_files();

    ##  
    ## process API files
    ##  
    add_apis_to_files();   
} # end make_xml()

#  
# Trim the string for white spaces 
# 
sub trim {
    my $item = shift;
    $item =~ s/^\s+//;
    $item =~ s/\s+$//;
    return $item;
} # end  trim()

1; 

