#
# Copyright (c) 2001-2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Version class for finding Version attributes specific to 7-Mode transition tool.

## @author rohit4@netapp.com, dl-nacl-dev@netapp.com
##
## @change     2015-03-07 ppranav : Added logic to get 7MTT 2.2 version
## @status Review
## @pod here

package NACL::APISet::Version::Host::SMTT;

#
# Compiler Directives
#
use strict;
use warnings;

#
# Module imports
#
use NACL::APISet::Session::CLI::Windows;
use NACL::APISet::Session::CLI::Linux;
#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use NATE::Exceptions::Argument qw(:try);
use Data::Dumper;

#
# base class
#
use base qw(NACL::APISet::Version::Host);

###############################################################################
# Method:    _find_os_revision
# Objective: Will return 7MTT version
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _find_os_revision {
    my ($self) = @_;
    $Log->enter();
    my ($major, $minor, $session);

    try {
        my $get_windows_session = sub {
            return $self->get_session(
                interface  => "CLI",
                subtype    => "Windows",
            );
        };
        $session = $get_windows_session->();

        my $hostobj = $self->{hostobj};
        my $command='set PATH=%path%;C:\Windows\system32;C:\Windows;'
            .'C:\Program Files\Java\jre7\bin;'
            .'C:\Program Files\Java\jre1.8.0_45\bin;'
            .'C:\Program Files\Java\jre1.8.0_112\bin;'
            .'C:\Program Files\NetApp\7-Mode Transition Tool\bin;';
        $command .= $hostobj->{Path_To_SMTT_Bin};
        $self->execute_CLI_command(
            connection  => $session->{connection},
            command     => $command,
        );
        $self->execute_CLI_command(
            connection  => $session->{connection},
            command     => "export PATH=\$PATH:/root/NetApp_7ModeTransitionTool/bin",
        );

        my $output = $self->execute_CLI_command(
            connection  => $session->{connection},
            command     => "transition version -v",
        );
        
        if( $output =~ /1.3\.\d/ ){
            $major = "V1_3";
        } elsif ( $output =~ /2.0\.\d/ ){
            $major = "V2_0";
        } elsif ( $output =~ /2.1\.\d/ ){
            $major = "V2_1";
        } elsif ( $output =~ /2.2\.\d/ ){
            $major = "V2_2";
        } elsif ( ($output =~ /2\.3\.\d/) ||($output =~ /2\.3/) ){
            $major = "V2_3";
        }elsif ( $output =~ /3.0\.\d/ ){
            $major = "V3_0";
        }elsif ( $output =~ /3.1\.\d/ ){
            $major = "V3_1";
        }else {
            $major = "V3_1";
        }
    } finally {
        # Destroy the session
        $self->destroy_session(session => $session) if (defined $session);
    };

    $self->{os_revision}->{major} = $major;
    $self->{os_revision}->{minor} = $minor;

    $Log->exit();
    return $major;
}

1;

=head1 NAME

NACL::APISet::Version::Host::SMTT - Version library for 7-Mode transition tool

=head1 SYNOPSIS

 use NACL::APISet::Version::Host::SMTT;

=head1 DESCRIPTION

A C<NACL::APISet::Version::Host::SMTT> is used to connect to 7MTT running on Windows
Host and collect version related attributes

=cut
