
#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary Base version class for 'Switch' category
## @author rawat@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Version::Switch;

#
# Compiler Directives
#
use strict;
use warnings;
#
# NATE imports
#
use Data::Dumper;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use NACL::APISet::Response::CLI::ParserUtils;
#
# Base class
#
use base qw(NACL::APISet::Version);
#
# Module imports
#
use NACL::APISet;
#
# Exceptions imported
#
use NACL::APISet::Exceptions::ConnectionFailedException qw(:try);
use NACL::APISet::Exceptions::AttributeDiscoveryFailed;
use NATE::BaseException;

###############################################################################
# Method:    new
# Objective: Constructor to create new Version CiscoFC object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub new {
    $Log->enter() if $may_enter;

    my ( $proto, %args ) = @_;
    my $class = ref($proto) || $proto;

    # Validate parameters and populate member variables
    my $self =  NACL::APISet::validate_params(
        $class,
        params => \%args,
        spec => {
            hostobj  => {isa => "Hostrec", optional => 0},
            category => {type => SCALAR, optional => 0},
            shell_type  => {type => SCALAR, optional => 0},
            conn_overrides => {type => HASHREF, optional => 1},
        }
    );

    bless $self, $class;

    $Log->exit() if $may_exit;
    return $self;
} ## end sub new()

1;

=head1 NAME

NACL::APISet::Version::Switch - Version library for all Switches

=head1 SYNOPSIS

 use NACL::APISet::Version::Switch;

=head1 DESCRIPTION

A C<NACL::APISet::Version::Switch> Base class for all Switch Version related
implementation.

=cut

=head1 EXCEPTIONS

=over

=item NATE::BaseException

=item NACL::APISet::Exceptions::ConnectionFailedException

=item NACL::APISet::Exceptions::AttributeDiscoveryFailed

=back

=cut
