# $Id: //depot/prod/DOT/dev/test/lib/EnGenX/Controller/AggregateWaflIron.pm#1 $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary AggregateWaflIron Controller Layer Module
## @author dl-san-automation-core
## @status shared
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller::AggregateWaflIron

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller::AggregateWaflIron is a derived class of
NACL::AppBuilder::EnGenX::Controller::Task.  It is used to run wafliron on aggregates.
All methods are static.

=cut

package NACL::AppBuilder::EnGenX::Controller::Aggregate::WaflIron;

# Compiler directives.
use strict;
use warnings;

# NATE and Standard Module import.
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Util::DataTypes qw(TRUE FALSE);
use Data::Dumper;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

# NACL and EnGenX Libs
use NACL::MTask::Wafliron;
use NACL::C::Job;
use NACL::AppBuilder::EnGenX::SubProcess;
use NACL::AppBuilder::EnGenX::EnGenXUtil;
use NACL::AppBuilder::EnGenX::Exceptions::WaflIronCorruption;

use NACL::C::Slist;
use NACL::STask::Wreck;
use NACL::C::Options;

# Exceptions
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

# Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
    %DTD_To_NACL_Ignore
    %Delete_Args);

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    name => "aggregate"
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (
    parallel => '',
);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
    command_interface      => '',
    apiset_must            => '',
    apiset_should          => '',
    aggregate              => '',
);

=head1 METHODS

=head2 create

This method currently does nothing

=cut

sub create {
    #NO OP
    return TRUE;
} ## end sub create

=head2 delete

(Static method) Request wafliron to be performed on an aggregate.

  NACL::AppBuilder::EnGenX::Controller::AggregateWaflIron->delete(
            command_interface => $Node,
            aggregate         => $Aggr_name,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::AggregateWaflIron->delete(
            command_interface => $Node,
            aggregate         => $Aggr_name,
            count             => 5
            );

  NACL::AppBuilder::EnGenX::Controller::AggregateWaflIron->delete(
            command_interface => $Node,
            resource_list      => @AggregateWaflIrons,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::AggregateWaflIron->delete(
            command_interface => $Node,
            resource_list      => @AggregateWaflIrons,
            );

  NACL::AppBuilder::EnGenX::Controller::AggregateWaflIron->delete(
            resource_list      => @AggregateWaflIrons,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::AggregateWaflIron->delete(
            resource_list      => @AggregateWaflIrons,
            );

=over

=item Options

Either the resource_list is required or the NACL WaflIron Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Required)
If this is passed in the wafliron will be run on aggrs based on the info in the
resource list. If the resource list is passed in the NACL AggregateWaflIron
Deletion Arguments should NOT be passed in.

=item C<< NACL WaflIron Arguments >>

(Required)
These are arguments that can be passed into NACL::C::AggregateWaflIron->Delete.
If these arguments are passed in the resource_list argument should NOT
be passed in.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method_timeout => $NACL_method_timeout >>

(Optional)
This is the method_timeout that will be used during NACL calls. If a
method_timeout is not passed in then the method_timeout will be defaulted.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the running of WaflIron(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "wafliron";

    # Super::_delete is in Task.pm
    $parent_answer = $pkg->SUPER::_delete(%args);

    return $parent_answer;
} ## end sub delete


#==============================================================================
# Method:     _nacl_create_call
# Objective:  Does nothing right now.
#==============================================================================
sub _nacl_create_call {
    #NO OP
    return TRUE;
} ## end sub _nacl_create_call

#==============================================================================
# Method:     _nacl_delete_call
# Objective:  Makes the raw NACL calls for deletion of a aggregate wafliron.
# Details:    Makes the NACL::STask::AggregateWaflIron->delete() calls. Also works around
#             some common non-fatal errors that can be return from aggregate wafliron
#             deletion and then has a retry mechanism to retry aggregate wafliron deletion.
# Inputs:     NACL::STask::AggregateWaflIron->delete arguments
#             retry => integer, is the current retry the method is on.
#                    This is used to count how many times the _nacl_delete_call
#                    has been retried so that the retries can be limited.
# Outputs:    Returns whether the create was successful or a hash with success
#             if a hash is requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter() if $may_enter;
    my $pkg   = shift;
    my %args = @_;
    # Locals
    my $retry = exists $args{retry} ? delete $args{retry} : 0;
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    my $aggr_exists = TRUE;
    my $wafliron;

    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    try {
        $wafliron = NACL::MTask::Wafliron->new (command_interface => $args{command_interface},
                                                   aggregate         => $args{aggregate});
    } catch NACL::Exceptions::NoElementsFound with {
        # The aggr does exist so we assume that it has already been cleaned up.
        $aggr_exists = FALSE;
        $Log->comment("Aggregate:" . $args{aggregate} . " was already deleted and could not be waflironed");
    };

    # Only try this if the aggr exists because otherwise the wafliron object will not exist.
    if ($aggr_exists) {
        $wafliron->start();
        $wafliron->review();
        if ($wafliron->corruption_found()) {
			NACL::AppBuilder::EnGenX::Exceptions::WaflIronCorruption->throw("Wafliron detected a corruption on aggregate $args{aggregate}");
        }
    }

    $Log->exit() if $may_exit;
    return TRUE;

} ## end sub _nacl_delete_call

#==============================================================================
# Method:     _nacl_wait_call
# Objective:  Makes a NACL call to 'wait_for_creation.
# Details:    Makes the call to NACL::STask::AggregateWaflIron->wait_for_creation();
#             that will wait for the node to finish creating the aggr.
# Inputs:     NACL::STask::AggregateWaflIron->wait arguments
# Outputs:    Returns whether the create was successful TRUE or not FALSE
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_wait_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    $Log->exit() if $may_exit;
    return TRUE;
}

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1;
