# $Id: //depot/prod/test/main/lib/EnGenX/Controller/FPolicyScope.pm#4 $

#
# Copyright (c) 2001-2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary FPolicy Scope Controller Layer Module
## @author dl-san-automation-core
## @status shared
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller::FpolicyScope

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller::FpolicyScope is a derived class of
NACL::AppBuilder::EnGenX::Controller::Task. It is used to Create and
Delete ONTAP Vserver Fpolicie scopes. All methods are static.

=cut

package NACL::AppBuilder::EnGenX::Controller::Fpolicy::FpolicyScope;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Util::DataTypes qw(TRUE FALSE);
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

# NACL and EnGenX Libs
use NACL::STask::VserverFpolicyPolicyScope;

# Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
            %DTD_To_NACL_Ignore
            %Delete_Args);

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    _reuse => 'if_exists',
    fpolicy   => 'policy-name'
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (
    parallel => '',
);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
    command_interface => '',
    vserver           => '',
    'policy-name'     => '',
);

=head1 PUBLIC METHODS

=head2 create

(Static method) Request creation of one or multiple vserver fpolicies
serially or in parallel and return whether the vserver fpolicies
were created successfully. This method passes all user arguments through to
NACL::AppBuilder::EnGenX::Controller::Task and adds specific arguments used by the EnGenX
framework.

  NACL::AppBuilder::EnGenX::Controller::Fpolicy::FpolicyScope->create(command_interface => $Node,
                                                 vserver                       => $Vserver_name,
                                                'polciy-name'                  => $policy_name,);

=over

=item Options

Either the resource_list is required or the NACL Vserver
Quota Policy Creation Arguments are required but both
should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Optional)
If this is passed in the vserver fpolicies in the resource list
will be created based on the information that is supplied in the resource
list. If the resource list is passed in the NACL Vserver Quota Policy
Creation Arguments should NOT be passed in.

=item C<< NACL Vserver Quota Policy Creation Arguments >>

=item C<< vserver => $Vserver_name >>

(Required)
vserver the name of the vserver that the policy relates to.

=item C<< policy-name => $policy_name >>

(Required)
name the name of the  fpolicy

(Optional)
These are arguments that can be passed into NACL::C::VolumeFpolicyScope->Create.
If these arguments are passed in the resource_list argument should NOT
be passed in.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the F Policy scope based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "Fpolicy_Scope";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);
    $Log->exit() if $may_exit;
    return $parent_answer;
} ## end sub create

=head2 delete

(Static method) Request deletion of one or multiple vserver fpolicies
serially or in parallel and return whether the vserver fpolici scopese
were deleted successfully. This method passes all user arguments through
to NACL::AppBuilder::EnGenX::Controller::Task and adds specific arguments used by the EnGenX
framework.

  NACL::AppBuilder::EnGenX::Controller::Fpolicy::FpolicyScope->delete(
            command_interface => $Node,
            vserver           => $Vserver_name,
            polciy-name        => $policy_name);

=over

=item Options

Either the resource_list is required or the NACL Vserver
Quota Policy Deletion Arguments are required but both
should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Required)
If this is passed in the vserver fpolicies in the resource list
will be deleted based on the information that is supplied in the resource
list. If the resource list is passed in the NACL Vserver Quota Policy
Deletion Arguments should NOT be passed in.

=item C<< NACL Vserver Quota Policy Deletion Arguments >>

(Required)
vserver the name of the vserver that the name mapping relates to.

(Required)
policy-name the name of the event for  the fpolicy.

(Optional)
These are arguments that can be passed into NACL::C::VolumeFpolicyScope->Delete.
If these arguments are passed in the resource_list argument should NOT
be passed in.

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the deletion of the Vserver Quota Policy based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "Fpolicy_Scope";

    # Super::_delete is in Task.pm
    $parent_answer = $pkg->SUPER::_delete(%args);
    $Log->exit() if $may_exit;
    return $parent_answer;
} ## end sub delete

=head1 PRIVATE METHODS

=head2 _nacl_create_call

=cut

#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of a  fpolicy.
# Details:    Makes the NACL::STask::VserverFpolicyPolicyScope->create() call.
# Inputs:     NACL::STask::VserverFpolicyPolicyScope->create() arguments
# Outputs:    Returns whether the create was successful or a hash with success
#             and whether the vserver name-mapping was a duplicate if a hash is
#             requested
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $was_created = 0;
    my $if_exists = exists($args{if_exists}) ? delete($args{if_exists}) : 'die';
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    # Create array referneces for list attributes
    $args{'shares-to-include'} = [ split(/,/ , $args{'shares-to-include'}) ] if exists $args{'shares-to-include'};
    $args{'shares-to-exclude'} = [ split(/,/ , $args{'shares-to-exclude'}) ] if exists $args{'shares-to-exclude'};
    $args{'volumes-to-include'} = [ split(/,/ , $args{'volumes-to-include'}) ] if exists $args{'volumes-to-include'};
    $args{'volumes-to-exclude'} = [ split(/,/ , $args{'volumes-to-exclude'}) ] if exists $args{'volumes-to-exclude'};
    $args{'export-policies-to-include'} = [ split(/,/ , $args{'export-policies-to-include'}) ] if exists $args{'export-policies-to-include'};
    $args{'export-policies-to-exclude'} = [ split(/,/ , $args{'export-policies-to-exclude'}) ] if exists $args{'export-policies-to-exclude'};
    $args{'file-extensions-to-include'} = [ split(/,/ , $args{'file-extensions-to-include'}) ] if exists $args{'file-extensions-to-include'};
    $args{'file-extensions-to-exclude'} = [ split(/,/ , $args{'file-extensions-to-exclude'}) ] if exists $args{'file-extensions-to-exclude'};

    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    # Create the FPolicy scope
    NACL::STask::VserverFpolicyPolicyScope->create(%args,
                                        nacltask_if_exists => $if_exists,
                                        _was_created => \$was_created);
    if($was_created) {
        $Log->comment("Created new Fpolicy scope for fpolicy: " . $args{'policy-name'}." on vserver ".$args{'vserver'});
    } else {
        $Log->comment("The fpolicy scope for fpolicy " . $args{'policy-name'} . " already exists on vserver ".$args{'vserver'});
    }

    $Log->exit() if $may_exit;
    # Returning either a list (hash) or scalar TRUE
    return (wantarray()) ? ('success'=>TRUE, 'duplicated'=> !$was_created) : TRUE;

} ## end sub _nacl_create_call

#==============================================================================
# Method:     _nacl_delete_call
# Objective:  Makes the raw NACL calls for deletion of a fpolicy scope.
# Details:    Makes the NACL::STask::VolumeFpolicyPolicyScope->create() calls.
# Inputs:     NACL::STask::VolumeFpolicyPolicyScope->create() arguments
# Outputs:    Returns whether the delete was successful
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_delete_call {
    $Log->enter() if $may_enter;
    my $pkg   = shift;
    my %args = @_;

    # Locals
    my $was_deleted = 0;
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    # Delete the Fpolicy scope
    NACL::STask::VserverFpolicyPolicyScope->purge(%args,
                                        nacltask_if_purged => 'pass',
                                        _was_deleted => \$was_deleted);
    if($was_deleted) {
        $Log->comment("The Fpolicy scope for fpolicy " . $args{'policy-name'} . " was deleted on vserver ".$args{'vserver'});
    } else {
        $Log->comment("The Fpolicy scope for fpolicy " . $args{'policy-name'} . " has already been deleted on vserver ".$args{'vserver'});
    }

    $Log->exit() if $may_exit;
    return TRUE;

} ## end sub _nacl_delete_call

#==============================================================================
# Method:     _nacl_wait_call
# Objective:  Makes a NACL call to 'wait_for_creation.
# Details:    Makes the call to NACL::STask::Vserver->wait_for_creation();
#             that will wait for the node to finish creating the aggr.
# Inputs:     NACL::STask::Vserver->wait arguments
# Outputs:    Returns whether the create was successful TRUE or not FALSE
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
# TODO: Implement this method when needed
#==============================================================================
sub _nacl_wait_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return TRUE;
}

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1
