use strict;
use warnings;

package NACL::AppBuilder::EnGenX::UnitTest::CompositeAggregateConfig;

use parent 'NACL::AppBuilder::Xgen::Config::ONTAP';

use NATE::Log qw(log_global);
use NATE::BaseException;
use NATE::Exceptions::Argument qw(:try);

# resource libs
use NACL::AppBuilder::Xgen::Resource::Aggregate;
use NACL::AppBuilder::Xgen::Resource::Aggregate::ObjectStore;


sub build {
    my ($self, %args) = @_;

    log_global()->comment("Generating Parent Configuration.");
    $self->SUPER::build(%args);

    log_global()->comment("Generating Object Stores.");
    $self->generate_object_store_resources(%args);

    log_global()->comment("Generating Aggregates.");
    $self->generate_aggregates(%args);
}

sub generate_aggregate_resources {
    # Make this a noop since we need to create object stores first.
    # This method gets called in SUPER::build()
}

sub generate_object_store_resources {
    my($self, %args) = @_;
    my $node_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Node");
    my $objectstore_resource = new NACL::AppBuilder::Xgen::Resource::Aggregate::ObjectStore(
                                                        'object-store-name'  => "my-store-1",
                                                        'provider-type'      => "S3EMU",
                                                        'ip-address'         => "scspr0059237188.gdl.englab.netapp.com",
                                                        's3-name'            => "test_bucket",
                                                        'access-key'         => "fake_key",
                                                        'secret-password'    => "netapp1!",
                                                        'ssl-enabled'        => "false",
                                                        'ipspace'            => "Default",
                                                        'port'               => "1234");
    $objectstore_resource->add_parent_resources(resources => [$node_resources->[0]]);
    my @objectstore_resources;
    push (@objectstore_resources, $objectstore_resource);

    $self->add_resources(resources => \@objectstore_resources);
}

sub generate_aggregates {
    my($self, %args) = @_;
    my $node_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Node");
    my $object_store_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Aggregate::ObjectStore");
    my $aggr_resource = new NACL::AppBuilder::Xgen::Resource::Aggregate(aggregate   => "composite_aggregate",
                                                                        diskcount   => 5,
                                                                        nodes       => [$node_resources->[0]->get_attribute('node')],
                                                                        'nacltask_enable_composite' => 1,
                                                                        'nacltask_object_store_name' => $object_store_resources->[0]->get_attribute('object-store-name'),
                                                                        xgen_metadata => {cleanup_policy => 'all'});
    $aggr_resource->add_parent_resources(resources => [$node_resources->[0],$object_store_resources->[0]]);
    my @aggr_resources;
    push (@aggr_resources, $aggr_resource);

    $self->add_resources(resources => \@aggr_resources);
}
1;
