# Copyright (c) 2015 NetApp, Inc.
# All rights reserved.

## @summary Keyspec config class for IceKing
##
## @description
##   It makes sense to have this separated from the Bubblegum code, since
##   it will be modified a lot as support for new object types is added
##
##   Element format is:
##            obj_type => {
##            # normally mapping to a NACL object type
##                alias   => [qw(alias1, alias2)],
##                # optional; these can be used interchangeably in the YAML
##                #  to designate obj_type objects
##                no_scale => 1,
##                # optional; disable scaling for this obj_type
##                #  this would apply to something like a cluster
##                no_obj   => 1,
##                # optional; disable processing this altogether
##                #  for stuff that doesn't need object matching, YAML artifacts
##        # - or -
##                keyspec => {
##                    prefix => 'Prefix_',
##                    # optional; pre/suffixes are there for possible future
##                    # tagging support
##            # START simple key
##                    field_name => 'name',
##                    # name of the field to use as the key
##            # END simple key
##        # - or -
##            # START composite key
##                    value => [
##                        {   search_parents => 1,
##                            # whether to search for this up the tree
##                            #  (in case parent is not of this type but
##                            #  maybe a grandparent++ is)
##                            # or just check the parent
##                            type           => 'vs',
##                            # this has to be an obj_type, not an alias
##                            field_name     => 'name',
##                            # name of the field to use as the key
##                        },
##                        {   search_parents => 0,
##                            type           => 'vol',
##                            field_regex    => qr/dst/,
##                            # a regex can be used as well, since people can't
##                            # follow a simple rule like *_src or src_*
##                            # it works in conjunction with 'type' to filter
##                        },
##                        {   type         => 'vol',
##                            default_name => 1,
##                            # use the 'name' attribute of this object
##                        },
##                    ],
##            # END composite key
##                    suffix => '_Suffix',
##                    # optional
##                },
##            },
##
##   In the future, it could support loading ad-hoc keyspecs in YAML format
##   if specified as an option to the constructor. Of course, code would need to
##   be changed to accept a string where we now have REGEX's.
##   It could look like this:
##
##   adhoc-keyspecs.yml
##   aggr:
##     alias:
##     - aggregate
##     no_scale: 1
##   client:
##     no_scale: 1
##   cluster:
##     alias:
##     - cl
##     keyspec:
##       field_name: name
##     no_scale: 1
##   lif:
##     alias:
##     - vif
##     no_scale: 1
##   lun:
##     keyspec:
##       value:
##       - field_name: name
##         search_parents: 1
##         type: vs
##       - field_name: name
##         search_parents: 1
##         type: vol
##       - default_name: 1
##         type: lun
##   node:
##     no_scale: 1
##   sm:
##     alias:
##     - snapmirror
##     keyspec:
##       value:
##       - field_regex: '.*src.*'
##         search_parents: 0
##         type: vol
##       - field_regex: '.*dst.*'
##         search_parents: 0
##         type: vol
##   task:
##     no_obj: 1
##     no_scale: 1
##   vol:
##     alias:
##     - volume
##     keyspec:
##       value:
##       - field_name: name
##         search_parents: 1
##         type: vs
##       - default_name: 1
##         type: vol
##   vs:
##     alias:
##     - vserver
##     keyspec:
##       value:
##       - field_name: name
##         search_parents: 1
##         type: cluster
##       - default_name: 1
##         type: vs
##
## @notes
##   Ricardio has the heart
##
## @author ng-dpg-smgr-auto
##

package NACL::AppBuilder::IceKing::Ricardio;

use 5.014;
use warnings;
use Moose;

our $VERSION = '3.13';

has 'keyspecs' => (
    is            => 'ro',
    isa           => 'HashRef[Any]',
    documentation => q(These are the supported types v3+ YAML config),
    default       => sub {
        return {
            altavault => {
                no_scale => 1,
                alias    => [qw(ava)],
                #keyspec  => {
                #field_name => 'name',
                #},
            },
            btrfs => {
                no_scale => 1,
                #alias    => [qw(ava)],
                keyspec  => {
                    field_name => 'name',
                },
            },
            solidfire => {
                no_scale => 1,
                #alias    => [qw(ava)],
                keyspec  => {
                    field_name => 'name',
                },
            },
            cluster => {
                no_scale => 1,
                alias    => [qw(cl)],
                keyspec  => {
                    field_name => 'name',
                },
            },
            node   => {no_scale => 1,},
            client => {no_scale => 1,},
            lif    => {
                no_scale => 1,
                alias    => [qw(vif)],
            },
            aggr => {
                no_scale => 1,
                alias    => [qw(aggregate)],
            },
            job_schedule => {
                #alias   => [qw(vserver)],
                keyspec => {
                    value => [
                        {   search_parents => 1,
                            type           => 'cluster',
                            field_name     => 'name',
                        },
                        {   type         => 'job_schedule',
                            default_name => 1,
                        },
                    ],
                },
            },
            share => {
                keyspec => {
                    value => [
                        {   search_parents => 1,
                            type           => 'altavault',
                            field_name     => 'name',
                        },
                        {   type         => 'share',
                            default_name => 1,
                        },
                    ],
                },
            },
            btrfs_volume => {
                keyspec => {
                    value => [
                        {   search_parents => 1,
                            type           => 'btrfs',
                            field_name     => 'name',
                        },
                        {   type         => 'btrfs_volume',
                            default_name => 1,
                        },
                    ],
                },
            },
            solidfire_account => {
                alias   => [qw(sf_account)],
                keyspec => {
                    value => [
                        {   search_parents => 1,
                            type           => 'solidfire',
                            field_name     => 'name',
                        },
                        {   type         => 'solidfire_account',
                            default_name => 1,
                        },
                    ],
                },
            },
            solidfire_volume => {
                keyspec => {
                    value => [
                        {   search_parents => 1,
                            type           => 'solidfire',
                            field_name     => 'name',
                        },
                        {   search_parents => 1,
                            type           => 'solidfire_account',
                            field_name     => 'name',
                        },
                        {   type         => 'solidfire_volume',
                            default_name => 1,
                        },
                    ],
                },
            },
            vol => {
                alias   => [qw(volume)],
                keyspec => {
                    value => [
                        {   search_parents => 1,
                            type           => 'vs',
                            field_name     => 'name',
                        },
                        {   type         => 'vol',
                            default_name => 1,
                        },
                    ],
                },
            },
            vs => {
                alias   => [qw(vserver)],
                keyspec => {
                    value => [
#                        {   search_parents => 1,
#                            type           => 'cluster',
#                            field_name     => 'name',
#                        },
                        {   type         => 'vs',
                            default_name => 1,
                        },
                    ],
                },
            },
            vserver_peer => {
                #alias   => [qw(peer)],
                keyspec => {
                    value => [
                        {   search_parents => 1,
                            type           => 'vs',
                            field_name     => 'name',
                        },
                        {   type         => 'vserver_peer',
                            default_name => 1,
                        },
                    ],
                },
            },
            snapmirror => {
                alias   => [qw(sm)],
                keyspec => {
                    value => [
                        {   search_parents => 0,
                            type           => 'vol',
                            field_regex    => qr/src|source/,
                        },
                        {   search_parents => 0,
                            type           => 'vol',
                            #type           => qr/vol|share/,
                            field_regex    => qr/dst|destination/,
                        },
                    ],
                },
            },
            snapmirror_policy => {
                alias   => [qw(sm_policy)],
                keyspec => {
                    value => [
                        {   search_parents => 1,
                            type           => 'vs',
                            field_name     => 'name',
                        },
                        #{   search_parents => 1,
                        #type           => 'snapmirror_policy',
                        #field_name     => 'policy',
                        #},
                        {   type         => 'snapmirror_policy',
                            default_name => 1,
                        },
                    ],
                },
            },
            snapmirror_policy_rule => {
                #alias   => [qw(rule)],
                keyspec => {
                    value => [
                        {   search_parents => 1,
                            type           => 'vs',
                            field_name     => 'name',
                        },
                        {   search_parents => 1,
                            type           => 'snapmirror_policy',
                            field_name     => 'name',
                        },
                        {   type         => 'snapmirror_policy_rule',
                            default_name => 1,
                        },
                    ],
                },
            },
            snapshot_policy => {
                #alias   => [qw(snap_policy)],
                keyspec => {
                    value => [
                        {   search_parents => 1,
                            type           => 'vs',
                            field_name     => 'name',
                        },
                        #{   search_parents => 1,
                        #type           => 'snapmirror_policy',
                        #field_name     => 'policy',
                        #},
                        {   type         => 'snapshot_policy',
                            default_name => 1,
                        },
                    ],
                },
            },
            lun => {
                keyspec => {
                    value => [
                        {   search_parents => 1,
                            type           => 'vs',
                            field_name     => 'name',
                        },
                        {   search_parents => 1,
                            type           => 'vol',
                            field_name     => 'name',
                        },
#                        {   search_parents => 1,
#                            type           => 'vol',
#                            field_name     => 'name',
#                        },
                        {   type         => 'lun',
                            default_name => 1,
                        },
                    ],
                },
            },
            volume_efficiency => {
                keyspec => {
                    value => [
                        {   search_parents => 1,
                            type           => 'vs',
                            field_name     => 'name',
                        },
                        {   search_parents => 1,
                            type           => 'vol',
                            field_name     => 'name',
                        },
#                        {   search_parents => 1,
#                            type           => 'vol',
#                            field_name     => 'name',
#                        },
                        {   type         => 'volume_efficiency',
                        default_name => 1,
                        },
                    ],
                },
            },
            volume_efficiency_policy => {
                keyspec => {
                    value => [
                        {   search_parents => 1,
                            type           => 'vs',
                            field_name     => 'name',
                        },
                        #{   search_parents => 1,
                        #type           => 'vol',
                        #field_name     => 'name',
                        #},
#                        {   search_parents => 1,
#                            type           => 'vol',
#                            field_name     => 'name',
#                        },
                        {   type         => 'volume_efficiency_policy',
                            default_name => 1,
                        },
                    ],
                },
            },
            vserver_nfs => {
                keyspec => {
                    value => [
                        {   search_parents => 1,
                            type           => 'vs',
                            field_name     => 'name',
                        },
                        #{   type         => 'vserver_nfs',
                        #default_name => 1,
                        #},
                    ],
                },
            },
            task => {
                no_scale => 1,
                no_obj   => 1,
            },
            dummy => {
            },
        };
    },
);

__PACKAGE__->meta->make_immutable;
no Moose;
1;

=head1 NAME

NACL::AppBuilder::IceKing::Ricardio - Keyspec specification class used by IceKing's Bubblegum
and Configurator.

=head1 SYNOPSIS

 use NACL::AppBuilder::IceKing::Ricardio;

=head1 DESCRIPTION

It makes sense to have this separated from the Bubblegum code, since
it will be modified a lot as support for new object types is added to IceKing.
Adding support for 'obj_type_foo' would only require as little as adding an
element to the hash.
Loadable ad-hoc custom keyspecs would require a BUILD (Moose post-constructor)
method. The code would be identical to what is now in C<NACL::AppBuilder::IceKing::Bubblegum>, and
it would make sense to have it moved to C<NACL::AppBuilder::IceKing::Util> shared between the two.

=cut

=head1 AUTHOR/MAINTAINER

=over

=item SMGR Automation Team (ng-dpg-smgr-auto@netapp.com)

=back

 $Id:$
 Last revised $Date:$ by $Author:$

=cut

