# Copyright (c) 2015 NetApp, Inc.
# All rights reserved.

## @summary Utility class for IceKing
##
## @description
##   For now, it's only the Dumper wrapper method IKDumper
##   Going forward, it should have stuff like _walk_tree
##   and other reusable "utility" components.
##
## @notes
##   Naming it Jake would have been too much
##
## @author ng-dpg-smgr-auto
##

package NACL::AppBuilder::IceKing::Util;

use 5.014;
use warnings;
use Readonly;
use Data::Dumper;

our $VERSION = '3.12';

Readonly my $DEFAULT_DUMPER_TERSE      => 1;
Readonly my $DEFAULT_DUMPER_DEEPCOPY   => 0;
Readonly my $DEFAULT_DUMPER_SPARSESEEN => 1;
Readonly my $DEFAULT_DUMPER_PURITY     => 0;
Readonly my $DEFAULT_DUMPER_MAXDEPTH   => 0;
Readonly my $DEFAULT_DUMPER_SORTKEYS   => 1;
Readonly my $DEFAULT_DUMPER_INDENT     => 1;    # or 4
# Excerpt from http://search.cpan.org/~smueller/Data-Dumper-2.154/Dumper.pm
# #
# "Controls the style of indentation. It can be set to 0, 1, 2 or 3. Style 0
# spews output without any newlines, indentation, or spaces between list items.
# It is the most compact format possible that can still be called valid perl.
# Style 1 outputs a readable form with newlines but no fancy indentation (each
# level in the structure is simply indented by a fixed amount of whitespace).
# Style 2 (the default) outputs a very readable form which takes into account
# the length of hash keys (so the hash value lines up). Style 3 is like style 2,
# but also annotates the elements of arrays with their index (but the comment is
# on its own line, so array output consumes twice the number of lines). Style 2
# is the default."

BEGIN {
    require Exporter;
    our @ISA       = qw(Exporter);
    our @EXPORT_OK = qw(IKDumper separator get_abs_file_from_nate);
}

################################################################################
## Objective: Returns a stringified form of the value
## Details:   This method is a wrapper for Data::Dumper with output parameters
##            which avoids poluting the namespace
## Arguments: what to stringify and Data::Dumper parameters as key=>value pairs
################################################################################
sub IKDumper {
    my ($val, %args) = @_;
#<<< because perltidy is so smart
    local $Data::Dumper::Terse      = $args{Terse}      // $DEFAULT_DUMPER_TERSE;
    local $Data::Dumper::Deepcopy   = $args{Deepcopy}   // $DEFAULT_DUMPER_DEEPCOPY;
    local $Data::Dumper::Sparseseen = $args{Sparseseen} // $DEFAULT_DUMPER_SPARSESEEN;
    local $Data::Dumper::Purity     = $args{Purity}     // $DEFAULT_DUMPER_PURITY;
    local $Data::Dumper::Maxdepth   = $args{Maxdepth}   // $DEFAULT_DUMPER_MAXDEPTH;
    local $Data::Dumper::Sortkeys   = $args{Sortkeys}   // $DEFAULT_DUMPER_SORTKEYS;
    local $Data::Dumper::Indent     = $args{Indent}     // $DEFAULT_DUMPER_INDENT;
#>>>
    return Dumper($val);
}

################################################################################
## Objective: Returns a separator line
## Details:   Used by unit tests just to beautify output
## Arguments: none
################################################################################
sub separator {
    my $len = shift || 40;
    return q(-=) x $len . "\n";
}

################################################################################
## Objective: Returns an absolute path using NATE resolution
## Details:   Used by reIgniter
## Arguments: relative path file name
################################################################################
sub get_abs_file_from_nate {
    my $ifn = shift;
    my @paths;
    push @paths, @INC;
    push @paths, split(PATH_SEPARATOR, $ENV{PATH});
#    $log->debug('Search paths are: ' . Dumper(\@paths));
    my $ofn = NATE::Inc::find_file($ifn, search => [@paths]);
    return $ofn || $ifn;
}

1;

=head1 NAME

NACL::AppBuilder::IceKing::Util - Utility class for IceKing

=head1 SYNOPSIS

use NACL::AppBuilder::IceKing::Util qw(IKDumper);

=head1 DESCRIPTION

Utility methods for the IceKing framework are gathered in C<NACL::AppBuilder::IceKing::Util>.
For now, it's only the Dumper wrapper method IKDumper, but going forward, it
should have stuff like _walk_tree and other reusable "utility" components.

=cut

=head1 AUTHOR/MAINTAINER

=over

=item SMGR Automation Team (ng-dpg-smgr-auto@netapp.com)

=back

$Id:$
Last revised $Date:$ by $Author:$

=cut

