#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::Xgen::Config::Asup2Config::Base;

use strict;
use warnings;
use POSIX;
use File::Temp;
use Archive::Extract;
use NATE::Inc qw(find_file);
use Data::Dumper;
use parent qw( NACL::AppBuilder::Xgen::Config::ONTAP );

use NACL::AppBuilder::Xgen::Config::Asup2Config::AggrParser;
use NACL::AppBuilder::Xgen::Resource::Aggregate;
use NACL::AppBuilder::Xgen::Resource::Fcp;
use NACL::AppBuilder::Xgen::Resource::Nfs;
use NACL::AppBuilder::Xgen::Resource::Vserver;
use NACL::AppBuilder::Xgen::Resource::Iscsi;
use NACL::AppBuilder::Xgen::Resource::Volume;
use NACL::AppBuilder::Xgen::Resource::Qtree;
use NACL::AppBuilder::Xgen::Resource::Lun;
use NACL::AppBuilder::Xgen::Config::Asup2Config::XMLParser;
use Clone;
# This config takes in information from asup and builds a config to match the asup data.
sub build {
    my ($self, %args) = @_;

    $self->SUPER::build(%args);

    my $tmp_dir = File::Temp->newdir();
    my $nodes = Clone::clone($self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Node"));
    my $asup_tgz_files = $args{asup_tgz_files};
    my $node_index = 1;
    foreach my $tgz_file (@{$asup_tgz_files}) {
        my $absolume_path = find_file($tgz_file);
        print "\nPHILIP DEBUG tgz_file $absolume_path\n";
        print "\nPHILIP DEBUG tmp_dir  $tmp_dir\n";
        my $ae = Archive::Extract->new( archive => $absolume_path );
        my $ok = $ae->extract( to => $tmp_dir . "/node$node_index") or die $ae->error;
        my $xml_object = NACL::AppBuilder::Xgen::Config::Asup2Config::XMLParser->new(
                                    file => $ae->extract_path() . "/SYSTEM-INFO.XML"
                                    );
        my $rows = $xml_object->get_rows();
        print "\nPHILIP DEBUG SYSTEM-INFO rows" . Dumper($rows) . "\n";
        my $asup_nodename    = $rows->[0]->{'sys-host-name'};
        my $asup_ha_nodename = $rows->[0]->{'partner-host-name'};
        print "\nPhilip Debug asup_nodename " . $asup_nodename . "\n"; 
        if (not defined $args{'nodename_mapping'}->{$asup_nodename}) {
            print "\nPHILIP DEBUG $asup_nodename is not in nodename_mapping\n";
            my $node = undef;
            # First check to see if there is a node with the same name as asup.
            for (my $i = 0; $i < scalar @{$nodes}; $i++) {
                my $current_node = shift(@{$nodes});
                if($current_node->get_attribute('node') eq $asup_nodename) {
                    $node = $current_node;
                    last;
                } else {
                    push(@{$nodes},$current_node);
                }
            }        
            # If there is no node with the same name then use any node.
            $node = shift(@{$nodes}) if (not defined($node));
            my $node_name = $node->get_attribute('node');
            my $nacl_node = NACL::C::Node->new(node => $node_name);
            my $partner_node = NACL::STask::Node->get_partner(command_interface => $nacl_node);
            $args{'nodename_mapping'}->{$asup_nodename} = $node_name;
            $args{'nodename_mapping'}->{$node_name}     = $asup_nodename;
            $args{'nodename_mapping'}->{$asup_ha_nodename} = $partner_node;
            $args{'nodename_mapping'}->{$partner_node}     = $asup_ha_nodename;  
            $args{'file_paths'}->{$node_name}     = $ae->extract_path();
            $args{'file_paths'}->{$asup_nodename} = $ae->extract_path();
            my $found = 0;
            for (my $i = 0; $i < scalar @{$nodes}; $i++) {
                my $current_node = shift(@{$nodes});
                if($current_node->get_attribute('node') eq $partner_node) {
                    $found = 1;
                    last;
                } else {
                    push(@{$nodes},$current_node);
                }
            }
            die "Cound not find a node resource that represents $partner_node but it is an ha partner of $node_name." if(not $found);
        } else {
            print "\nPHILIP DEBUG $asup_nodename was in nodename_mapping.\n";
            my $node_name = $args{'nodename_mapping'}->{$asup_nodename};
            $args{'file_paths'}->{$node_name}     = $ae->extract_path();
            $args{'file_paths'}->{$asup_nodename} = $ae->extract_path();
        }
        $node_index++;
    }

    print "\nPHILIP DEBUG" . Dumper($args{'file_paths'}) . " \n";
    print "\nPHILIP DEBUG" . Dumper($args{'nodename_mapping'}) . " \n";
    
    # Generates the args xml objects for all the generate resources methods.
    %args = $self->generate_xml_objects_from_input_xmls(%args);

    # Call to generate aggregate resources
    $self->generate_aggregate_resources_local(%args);
    
    # Call to generate vserver resources.
    $self->generate_vserver_resources(%args);

    # Call to generate iscsi resources.
    $self->generate_iscsi_resources(%args);

    # Call to generate fcp resources.
    $self->generate_fcp_resources(%args);

    # Call to generate fcp resources.
    $self->generate_nfs_resources(%args);

    # Call to generate volume resources.
    $self->generate_volume_resources(%args);

    # Call to generate qtree resources.
    $self->generate_qtree_resources(%args);
    
    # Call to generate lun resources.
    $self->generate_lun_resources(%args);
}

sub generate_aggregate_resources {
# no op
}

# If there is more disk space available on the destination node than the src node,
# the scaling factor that vols and luns use will create vols and luns larger
# than on the src aggr.
sub generate_aggregate_resources_local {
    my ($self, %args) = @_;

    my $nodes = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Node");
    my @aggr_resources;
    my %dest_node_disks;

    foreach my $node (@{$nodes}) {
        my $node_name = $node->get_attribute('node');
        # There may be more nodes in the destination than the source.
        # Skip extra nodes that dont have a asup file path.
        next if (not defined ($args{file_paths}->{$node_name}));
        my $node_obj = NACL::C::Node->new(node => $node_name);
        my @disks = NACL::CS::StorageDisk->fetch(command_interface => $node_obj,
                                                 filter => {owner => $node_name},
                                                 allow_empty => 1);
        my @spare_disks = grep {$_->container_type() eq "spare"} @disks;
        my $spare_disk_count = scalar(@spare_disks);
        $dest_node_disks{$node_name} = $spare_disk_count;


        my $dir_path = $args{file_paths}->{$node_name};
        my $src_node_disks = 0;
        my $aggr_hash = NACL::AppBuilder::Xgen::Config::Asup2Config::AggrParser->parse_aggr_files(aggr_status_r => $dir_path."/AGGR-STATUS-R",
                                                                                                  aggr_status_v => $dir_path."/AGGR-STATUS-V",
                                                                                                  df_a => $dir_path."/DF-A",);
        
        

        foreach my $aggr (keys %{$aggr_hash}) {
            # Check to see if aggregate exists.
            my $aggregate = NACL::CS::StorageAggregate->fetch(command_interface => $node_obj,
                                          filter => {'nodes'     => [$node_obj->node()],
                                                     'ha-policy' => 'sfo',
                                                     'aggregate'  => $aggr},
                                          allow_empty => 1);
            # If aggregate exists the we want to created the resource and not setup/cleanup the aggr.
            if (defined ($aggregate)) {
                # Get scaling factor
                my $scale_factor = $aggregate->size() / $aggr_hash->{$aggr}->{size};
                $scale_factor = 1 if ($scale_factor > 1);

                my $aggr_resource = new NACL::AppBuilder::Xgen::Resource::Aggregate(aggregate => $aggr,
                                                                                    nodes => [$node_name],
                                                                                    xgen_metadata => {scale_factor => $scale_factor,
                                                                                                      setup => 0,
                                                                                                      cleanup => 0}
                                                                                    );
                $aggr_resource->add_parent_resources(resources => [$node]);
                push (@aggr_resources, $aggr_resource);
                # Also we want to remove the aggr from the disk calculations.
                delete $aggr_hash->{$aggr};
            } else {
                $src_node_disks += $aggr_hash->{$aggr}->{disk_count};
            }
        }

        my $num_aggr_src_node = scalar (keys %{$aggr_hash});

        # If there are enough disks on the destination node
        # to create aggrs of same disk count
        if($src_node_disks <= $dest_node_disks{$node_name}) {
            foreach my $aggr (keys %{$aggr_hash}) {
                my $disklist;
                my $dest_size;
                my $count = $aggr_hash->{$aggr}->{disk_count};
                for (my $i=0; $i<$count; $i++) {
                    my $disk = shift(@spare_disks);
                    push(@$disklist, $disk->disk());
                    # Subtract parity disks (raid dp)
                    if ($i >= 2) {
                        $dest_size += $disk->physical_size();
                    }
                }

                my $scale_factor =  $dest_size / $aggr_hash->{$aggr}->{size};
                $scale_factor = 1 if ($scale_factor > 1);

                my $aggr_resource = new NACL::AppBuilder::Xgen::Resource::Aggregate(aggregate => $aggr,
                                                                                    raidtype => $aggr_hash->{$aggr}->{raid_type},
                                                                                    disklist => $disklist,
                                                                                    xgen_metadata => { scale_factor => $scale_factor },);
                $aggr_resource->add_parent_resources(resources => [$node]);
                push (@aggr_resources, $aggr_resource);
            }
        }
        # If there are not enough disks on the destination node,
        # we need to scale down the aggr disk count
        else {
            if ((3*$num_aggr_src_node) <= $dest_node_disks{$node_name}) {
                my $remaining_disks = $dest_node_disks{$node_name} - (3*$num_aggr_src_node);

                foreach my $aggr (keys %{$aggr_hash}) {
                    my $disk_count = 3;
                    my $ratio_of_disks = floor((($aggr_hash->{$aggr}->{disk_count})/$src_node_disks)*$remaining_disks);
                    $disk_count = $disk_count + $ratio_of_disks;

                    my $disklist;
                    my $dest_size;
                    my $count = $aggr_hash->{$aggr}->{disk_count};
                    for (my $i=0; $i<$count; $i++) {
                        my $disk = shift(@spare_disks);
                        push(@$disklist, $disk->disk());
                        # Subtract parity disks (raid dp)
                        if ($i >= 2) {
                            $dest_size += $disk->physical_size();
                        }
                    }

                    my $scale_factor =  $dest_size / $aggr_hash->{$aggr}->{size};
                    $scale_factor = 1 if ($scale_factor > 1);

                    my $aggr_resource = new NACL::AppBuilder::Xgen::Resource::Aggregate(aggregate => $aggr,
                                                                                    raidtype => $aggr_hash->{$aggr}->{raid_type},
                                                                                    disklist => $disklist,
                                                                                    xgen_metadata => { scale_factor => $scale_factor },);
                    $aggr_resource->add_parent_resources(resources => [$node]);
                    push (@aggr_resources, $aggr_resource);
                }
            } else {
                die "No sufficient disks on the node";
            }
        }
    }

    $self->add_resources(resources => \@aggr_resources);           
}

# From Input XML or ID this will build all the parsed xml file objects and add them to args
# for use by the rest of the config.
sub generate_xml_objects_from_input_xmls {
    my ($self, %args) = @_;
    
    my @nodenames = keys %{$args{file_paths}};
    # For these we only need to look at a single node and not all nodes since this information is
    # clusterwide. If that changes then we can get the node resources and loop through those.
    my $dir_path = $args{file_paths}->{$nodenames[0]};
    $args{vserver_xml_object} = NACL::AppBuilder::Xgen::Config::Asup2Config::XMLParser->new(
                                        file => $dir_path . "/VSERVER-INFO.XML"
                                        );

    $args{iscsi_xml_object} = NACL::AppBuilder::Xgen::Config::Asup2Config::XMLParser->new(
                                        file => $dir_path . "/ISCSI.XML"
                                        );

    $args{fcp_xml_object} = NACL::AppBuilder::Xgen::Config::Asup2Config::XMLParser->new(
                                        file => $dir_path . "/FCP.XML"
                                        );

    $args{nfs_xml_object} = NACL::AppBuilder::Xgen::Config::Asup2Config::XMLParser->new(
                                        file => $dir_path . "/NFS-SERVERS-BYNAME.XML"
                                        );
                                        
    $args{volume_xml_object} = NACL::AppBuilder::Xgen::Config::Asup2Config::XMLParser->new(
                                        file => $dir_path . "/VOLUME.XML"
                                        );

    $args{qtree_xml_object} = NACL::AppBuilder::Xgen::Config::Asup2Config::XMLParser->new(
                                        file => $dir_path . "/QTREE.XML"
                                        );
                                        
    $args{lun_xml_object} = NACL::AppBuilder::Xgen::Config::Asup2Config::XMLParser->new(
                                        file => $dir_path . "/LUN.XML"
                                        );
    return %args;
}

# Generates vserver resources based on the information obtained from asup.
sub generate_vserver_resources {
    my ($self, %args) = @_;

    my $vserver_xml_object = $args{'vserver_xml_object'};
    my $aggr_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Aggregate");

    foreach my $aggr (@{$aggr_resources}) {
        # Get all the vservers from the xml that have "type=data" and aggregate = $aggr->get_attribute(aggregate)
        my $vservers = $vserver_xml_object->get_rows(qualifications => ['create','key'],
                                                     attr_filters     => {aggregate => $aggr->get_attribute('aggregate'),
                                                                          type      => 'data'});

        # Loop through and create each of the vservers.
        foreach my $vserver (@{$vservers}) {
            my $vs_res = NACL::AppBuilder::Xgen::Resource::Vserver->new(%{$vserver});

            # connect the vs resource to the aggr's cluster parent resource
            $vs_res->add_parent_resources(resources => [$aggr]);

            $self->add_resources(resources => [$vs_res]);
        }
    }
}

# Generates fcp service resources based on the data from asup and the vservers that were created during
# generate_vserver_resources.
sub generate_fcp_resources {
    my ($self, %args) = @_;

    my $fcp_xml_object = $args{'fcp_xml_object'};
    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");

    foreach my $vserver (@{$vserver_resources}) {
        # Get all the fcp services from the xml that have use this vserver
        # Basically fcp services vserver matches this vservers name.
        my $fcp = $fcp_xml_object->get_rows(qualifications => ['create','key','write'],
                                            attr_filters     => {vserver   => $vserver->get_attribute('vserver')});
        # Loop through and create each of the fcp services.
        foreach my $fcp (@{$fcp}) {
            # TODO calculate size based on a percentage and change the value of size.
            
            my $fcp_res = NACL::AppBuilder::Xgen::Resource::Fcp->new(%{$fcp});

            # connect the vs resource to the aggr's cluster parent resource
            $fcp_res->add_parent_resources(resources => [$vserver]);

            $self->add_resources(resources => [$fcp_res]);
        }
    }
}

# Generates iscsi service resources based on the data from asup and the vservers that were created during
# generate_vserver_resources.
sub generate_iscsi_resources {
    my ($self, %args) = @_;

    my $iscsi_xml_object = $args{'iscsi_xml_object'};
    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");

    foreach my $vserver (@{$vserver_resources}) {
        # Get all the iscsi services from the xml that have use this vserver
        # Basically iscsi services vserver matches this vservers name.
        my $iscsi = $iscsi_xml_object->get_rows(qualifications => ['create','key','write'],
                                                attr_filters    => {vserver   => $vserver->get_attribute('vserver')});
        # Loop through and create each of the iscsi services.
        foreach my $iscsi (@{$iscsi}) {
            # TODO calculate size based on a percentage and change the value of size.
            
            my $iscsi_res = NACL::AppBuilder::Xgen::Resource::Iscsi->new(%{$iscsi});

            # connect the vs resource to the aggr's cluster parent resource
            $iscsi_res->add_parent_resources(resources => [$vserver]);

            $self->add_resources(resources => [$iscsi_res]);
        }
    }
}

# Generates nfs service resources based on the data from asup and the vservers that were created during
# generate_vserver_resources.
sub generate_nfs_resources {
    my ($self, %args) = @_;

    my $nfs_xml_object = $args{'nfs_xml_object'};
    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");

    foreach my $vserver (@{$vserver_resources}) {

        # Get all the nfs services from the xml that have use this vserver
        # Basically nfs services vserver matches this vservers name.
        my $nfs = $nfs_xml_object->get_rows(qualifications => ['create','key','write'],
                                            attr_filters     => {vserver   => $vserver->get_attribute('vserver')});
        # Loop through and create each of the nfs services.
        foreach my $nfs (@{$nfs}) {
            delete $nfs->{'netgroup-trust-any-ns-switch-no-match'};
            delete $nfs->{'cached-cred-negative-ttl'};
            delete $nfs->{'ignore-nt-acl-for-root'};
            delete $nfs->{'cached-transient-err-ttl'};
            delete $nfs->{'skip-root-owner-write-perm-check'};
            delete $nfs->{'ntacl-display-permissive-perms'};
            delete $nfs->{'cached-cred-positive-ttl'};
            delete $nfs->{'v3-ms-dos-client'};

            my $nfs_res = NACL::AppBuilder::Xgen::Resource::Nfs->new(%{$nfs});

            # connect the vs resource to the aggr's cluster parent resource
            $nfs_res->add_parent_resources(resources => [$vserver]);

            $self->add_resources(resources => [$nfs_res]);
        }
    }
}

# Generates volume resources based on the data from asup and the vservers that were created during
# generate_vserver_resources.
sub generate_volume_resources {
    my ($self, %args) = @_;

    my $volume_xml_object = $args{'volume_xml_object'};
    my $aggr_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Aggregate");
    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");

    foreach my $vserver (@{$vserver_resources}) {
        foreach my $aggr (@{$aggr_resources}) {
            # Get all the volumes from the xml that have use this vserver and aggr
            # Basically volumes vserver matches this vservers name and volumes aggregate matches this aggrs name.
            my $volumes = $volume_xml_object->get_rows(qualifications => ['create','key','write'],
                                                       attr_filters     => {aggregate => $aggr->get_attribute('aggregate'),
                                                                            vserver   => $vserver->get_attribute('vserver')});
            # Loop through and create each of the volumes.
            foreach my $volume (@{$volumes}) {
                # Scale the size according to the scale_factor
                my $scale_factor = $aggr->get_attribute("scale_factor");
                print "PHILLIP DEBUG LOL\nVolume size:  ".$volume->{size}."\nScale factor:  $scale_factor\n\n";
                $volume->{size} = floor($volume->{size} * $scale_factor);
                $volume->{size} = 35651584 if ($volume->{size} < 35651584);
                if (defined $volume->{'min-autosize'}) {
                    $volume->{'min-autosize'} = floor($volume->{'min-autosize'} * $scale_factor);
                    $volume->{'min-autosize'} = 35651584 if ($volume->{'min-autosize'} < 35651584);
                }
                if (defined $volume->{'max-autosize'}) {
                    $volume->{'max-autosize'} = floor($volume->{'max-autosize'} * $scale_factor);
                    $volume->{'max-autosize'} = 35651584 if ($volume->{'max-autosize'} < 35651584);
                }
                if (defined $volume->{'autosize-increment'}) {
                    $volume->{'autosize-increment'} = floor($volume->{'autosize-increment'} * $scale_factor);
                }

                # Take out "write" params that are not allowed
                delete $volume->{'is-autobalance-eligible'};
                delete $volume->{'autosize'};

                my $volume_res = NACL::AppBuilder::Xgen::Resource::Volume->new(%{$volume}, xgen_metadata => {
                                                                                            scale_factor => $scale_factor});

                # connect the vs resource to the aggr's cluster parent resource
                $volume_res->add_parent_resources(resources => [$vserver,$aggr]);

                $self->add_resources(resources => [$volume_res]);
            }
        }
    }
}

# Generates qtree resources based on the data from asup and the volumes that were created during
# generate_volume_resources.
sub generate_qtree_resources {
    my ($self, %args) = @_;

    my $qtree_xml_object = $args{'qtree_xml_object'};
    my $volume_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Volume");  

    foreach my $volume (@{$volume_resources}) {
        # Get all the qtrees from the xml that have use this volume.
        # Basically qtrees volume matches this volumes name and qtrees vserver matches this volumes vserver.
        my $qtrees = $qtree_xml_object->get_rows(qualifications => ['create','key','write'],
                                             attr_filters     => {volume    => $volume->get_attribute('volume'),
                                                                  vserver   => $volume->get_attribute('vserver')});
        # Loop through and create each of the qtrees.
        # Get the aggr from the asup volume information
        foreach my $qtree (@{$qtrees}) {
            delete $qtree->{'export-policy'} if ($qtree->{'export-policy'} =~ /default/i);
            # Skip the qtree that had a name with the ""
            next if (not defined $qtree->{qtree});
            my $qtree_res = NACL::AppBuilder::Xgen::Resource::Qtree->new(%{$qtree});

            # connect the vs resource to the aggr's cluster parent resource
            $qtree_res->add_parent_resources(resources => [$volume]);

            $self->add_resources(resources => [$qtree_res]);
        }
    }
}

# Generates lun resources based on the data from asup and the volumes that were created during
# generate_volume_resources.
sub generate_lun_resources {
    my ($self, %args) = @_;

    my $lun_xml_object = $args{'lun_xml_object'};
    my $volume_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Volume");

    foreach my $volume (@{$volume_resources}) {
        # Get all the luns from the xml that have use this volume.
        # Basically luns volume matches this volumes name and luns vserver matches this volumes vserver.
        my $luns = $lun_xml_object->get_rows(qualifications => ['create','key','write'],
                                             attr_filters     => {volume    => $volume->get_attribute('volume'),
                                                                  vserver   => $volume->get_attribute('vserver')});
        # Loop through and create each of the luns.
        # Get the aggr from the asup volume information
        foreach my $lun (@{$luns}) {
            # Calculate size based on a percentage and change the value of size.
            # Also set to a minimum lun size value if its windows and below a threshhold
            my $scale_factor = $volume->get_attribute("scale_factor");
            $lun->{size} = floor($lun->{size} * $scale_factor);
            $lun->{size} = 32901120 if ($lun->{size} < 32901120 && $lun->{ostype} =~ /windows|hyper/i);
            delete $lun->{path};
            my $lun_res = NACL::AppBuilder::Xgen::Resource::Lun->new(%{$lun});

            # connect the vs resource to the aggr's cluster parent resource
            $lun_res->add_parent_resources(resources => [$volume]);

            $self->add_resources(resources => [$lun_res]);
        }
    }
}
1;

__END__

=head1 NAME

NACL::AppBuilder::Xgen::Config::Asup2Config::Base

=head1 AUTHOR/MAINTAINER

=over

=item PTEFIT AUTOMATION (ng-ptefit-automation-core@netapp.com)

=back

   Last revised $Date: $ by $Author: $

=cut
