#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::Xgen::Config::STE::EQ::Base;

use parent qw(PTEFIT::Xgen::Config::ONTAP::CIFS);

use List::Util 'shuffle';

use Data::Dumper;
use Tharn qw($Log );

# Default Base-line constants
use constant TOTAL_VS_COUNT => 30;

# The layout of VS_NODAL_LAYOUT.
# { node_count => vserver_count,
#   node_count => vserver_count,}
use constant VS_NODAL_LAYOUT => {
    6 => 5,
    4 => 5,
    2 => 5,
    1 => 15

};

use constant DISKS_PER_AGGR => 14;

use constant TOTAL_IPV4_LIFS_PER_VS_PER_NODE_PER_FABRIC => 1;
use constant TOTAL_IPV6_LIFS_PER_VS_PER_NODE_PER_FABRIC => 1;

use constant TOTAL_IPV4_ISCSI_LIFS_PER_VS_PER_NODE_PER_FABRIC => 2;
use constant TOTAL_IPV6_ISCSI_LIFS_PER_VS_PER_NODE_PER_FABRIC => 2;

use constant AGGRS_PER_NODE => 1;
use constant VSERVER_COUNT_CLUSTER_WIDE => 5;

use constant TOTAL_VOLUME_COUNT                  => 3000;
use constant VOL_SIZE                            => '400g';
use constant VOL_MAX_AUTOSIZE_MULTIPLIER         => 3;
use constant VOL_AUTOSIZE_MODE                   => "grow";
use constant VOL_AUTOSIZE_GROW_THRESHOLD_PERCENT => 60;

use constant TOTAL_QTREE_COUNT                => 30000;
use constant CIFS_SECURITY_TRACING_VS_PERCENT => 100;

use constant TOTAL_QTREE_HOMEDIR_COUNT => 3000;
use constant QTREE_HOMEDIR_LAYOUT      => {
    local  => 5,
    domain => 95
};

use constant TOTAL_QOS_POLICY_COUNT => 30;

use constant TOTAL_CIFS_SHARE_COUNT => 6000;
use constant CIFS_SHARE_LAYOUT      => {
    qtree  => 50,
    volume => 50
};

use constant TOTAL_SNAPMIRROR_COUNT => 1000;
use constant SNAPMIRROR_LAYOUT      => {
    "load_sharing"    => 0,
    "vault"           => 30,
    "data_protection" => 70
};

use constant CIFS_SECURITY_AES_ENCRYPTION_VS_PERCENT => 10;
use constant CIFS_SECURITY_SMB_SIGNING_VS_PERCENT    => 25;

use constant AUDIT_VS_PERCENT => 10;
use constant DDNS_VS_PERCENT  => 50;

use constant TOTAL_SYMLINK_COUNT                   => 9000;
use constant CIFS_OPTIONS_NODE_REFERRAL_VS_PERCENT => 10;
use constant CIFS_OPTIONS_DAC_VS_PERCENT           => 25;

use constant SYMLINK_LAYOUT => {
    qtree  => 90,
    volume => 10,
};

use constant TOTAL_LOCAL_USERS    => 300;
use constant TOTAL_DOMAIN_USERS   => 99;
use constant TOTAL_LOCAL_GROUPS   => 3000;
use constant MAX_USER_MEMBERSHIPS => 3000;

use constant TOTAL_DOMAIN_USER_NAME_MAPPINGS => 10;
use constant DOMAIN_USER_NAME_MAPPING_LAYOUT => {
    "local"   => 10,    # domain user will have a mapping to a local user in the vserver (direction: win-unix)
    "default" => 10,    # domain user will have a mapping to 'root' (direction: win-unix)
    "domain"  => 80     # domain user will have a mapping to themselves in both directions (direction: unix-win/win-unix)
};

use constant CIFS_NETBIOS_ALIASES_PER_VS => 10;

use constant DOMAIN_USER_PREFIX => "STaaS1u";

use constant LOCAL_GROUP_MEMBERSHIP_LAYOUT => {
    local  => 50,
    domain => 50
};

# 50% of vservers will have ntfs, 50% will have mixed
use constant VSERVER_ROOT_VOLUME_SECURITY_LAYOUT => {
    ntfs  => 50,
    mixed => 50
};

# all volumes will have vol efficiency enabled
use constant VOL_EFFICIENCY_PERCENT => 100;

use constant VSERVER_FSECURITY_ACCESS_CONTROL_LAYOUT => {
    slag             => 10,
    'file-directory' => 90
};

use constant TOTAL_LOCAL_USER_SACL_PER_NTFS  => 10;
use constant TOTAL_DOMAIN_USER_SACL_PER_NTFS => 10;
use constant TOTAL_LOCAL_USER_DACL_PER_NTFS  => 10;
use constant TOTAL_DOMAIN_USER_DACL_PER_NTFS => 10;

use constant SECURITY_FILTER_USERS_PER_VSERVER => 5;


use constant SAN_VSERVERS_COUNT => 1;
use constant PERCENTAGE_OF_VOLUMES_WITH_LUNS => 100;
use constant TOTAL_LUN_COUNT => 32;
use constant LUN_SIZE => "20m";

use strict;
use warnings;

use NATE::Log qw(log_global);
use NATE::BaseException;
use NATE::Exceptions::Argument qw(:try);

# resource libs
use NACL::AppBuilder::Xgen::Resource::Vserver;
use NACL::AppBuilder::Xgen::Resource::Volume;
use NACL::AppBuilder::Xgen::Resource::Volume::Efficiency;
use NACL::AppBuilder::Xgen::Resource::Qtree;
use NACL::AppBuilder::Xgen::Resource::Dns;
use NACL::AppBuilder::Xgen::Resource::Lif;
use NACL::AppBuilder::Xgen::Resource::NetworkRoute;
use NACL::AppBuilder::Xgen::Resource::Cifs;
use NACL::AppBuilder::Xgen::Resource::Cifs::Share;
use NACL::AppBuilder::Xgen::Resource::Cifs::Privilege;
use NACL::AppBuilder::Xgen::Resource::Cifs::LocalUser;
use NACL::AppBuilder::Xgen::Resource::Cifs::DomainUser;
use NACL::AppBuilder::Xgen::Resource::Cifs::LocalGroup;
use NACL::AppBuilder::Xgen::Resource::Cifs::LocalGroup::Membership;
use NACL::AppBuilder::Xgen::Resource::Cifs::Symlink;
use NACL::AppBuilder::Xgen::Resource::Cifs::Security;
use NACL::AppBuilder::Xgen::Resource::Cifs::Branchcache;
use NACL::AppBuilder::Xgen::Resource::Cifs::HomeDirSearchPath;
use NACL::AppBuilder::Xgen::Resource::Cifs::GroupPolicy;
use NACL::AppBuilder::Xgen::Resource::NameMapping;
use NACL::AppBuilder::Xgen::Resource::Audit;
use NACL::AppBuilder::Xgen::Resource::Security::FileDirectory::Policy;
use NACL::AppBuilder::Xgen::Resource::Security::FileDirectory::Apply;
use NACL::AppBuilder::Xgen::Resource::Security::FileDirectory::Policy::PolicyTask;
use NACL::AppBuilder::Xgen::Resource::Security::FileDirectory::Ntfs;
use NACL::AppBuilder::Xgen::Resource::Security::FileDirectory::Ntfs::Sacl;
use NACL::AppBuilder::Xgen::Resource::Security::FileDirectory::Ntfs::Dacl;
use NACL::AppBuilder::Xgen::Resource::Nfs;
use NACL::AppBuilder::Xgen::Resource::Security::Trace::Filter;
use NACL::AppBuilder::Xgen::Resource::ExportPolicy;
use NACL::AppBuilder::Xgen::Resource::ExportPolicy::Rule;
use NACL::AppBuilder::Xgen::Resource::Cifs::Options;
use NACL::AppBuilder::Xgen::Resource::DynamicDns;
use NACL::AppBuilder::Xgen::Resource::Snapmirror;
use NACL::AppBuilder::Xgen::Resource::SnapmirrorInitialize;
use NACL::AppBuilder::Xgen::Resource::Nis;
use NACL::AppBuilder::Xgen::Resource::VserverPeer;
use NACL::AppBuilder::Xgen::Resource::QosPolicyGroup;

use NACL::AppBuilder::Xgen::Resource::Lun;
use NACL::AppBuilder::Xgen::Resource::LunMap;
use NACL::AppBuilder::Xgen::Resource::Iscsi;
use NACL::AppBuilder::Xgen::Resource::Igroup;
use NACL::AppBuilder::Xgen::Resource::Igroup::Initiator;
use NACL::CS::NetworkSubnet;

use NACL::Service::Xgen::Util qw(convert_percent_layout_to_count_layout
    get_next_layout_element
    dumper
    randomize_array
    positive_filter);

use POSIX qw(ceil floor);

sub build {
    my ($self, %args) = @_;
    log_global()->log("********************* args start *********************");
    log_global()->log(dumper(\%args));
    log_global()->log("********************* args end *********************");
    my $cifs_setup = delete($args{cifs_setup}) // 0;
    $self->get_subnet_info(%args);
    $self->SUPER::build(%args);
} ## end sub build

sub generate_aggregate_resources_new {
    my ($self, %args) = @_;

    my $disks_per_aggr = $self->DISKS_PER_AGGR;
    my $aggrs_per_node = $self->AGGRS_PER_NODE;

    my $nodes = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Node");
    my @aggrs = ();

    foreach my $node (@{$nodes}) {
        my $node_name = $node->get_attribute('node');

        if (!$disks_per_aggr) {

            #find the number of spares on the node, then divide by the number of aggrs
            #ensure leaving enough spares
            $disks_per_aggr = $self->_calculate_disks_per_aggr(
                aggrs_per_node => $aggrs_per_node,
                node           => $node_name
            );
            log_global()->log("\tcalculated $disks_per_aggr disks per aggr");
        }
        if ($disks_per_aggr) {
            for (my $i = 1; $i <= $aggrs_per_node; $i++) {
                my $aggr_name = "aggr_" . $node_name . "_$i";
                $aggr_name =~ s/-/_/g;

                log_global()->log("\tCreating Aggr $aggr_name with $disks_per_aggr disks");
                my $aggr_res = NACL::AppBuilder::Xgen::Resource::Aggregate->new(
                    aggregate   => $aggr_name,
                    node        => $node_name,
                    diskcount   => $disks_per_aggr,
                    maxraidsize => $disks_per_aggr
                );

            #$aggr_res->add_parent_resources(resources => [$node->get_ancestors(type => "NACL::AppBuilder::Xgen::Resource::Cluster")]);
                $aggr_res->add_parent_resources(resources => [$node]);
                push(@aggrs, $aggr_res);
            }
        }
    }

    $self->add_resources(resources => \@aggrs);

} ## end sub generate_aggregate_resources_new

sub generate_dynamic_dns_resources {
    my ($self, %args) = @_;

    my $vservers = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");

    my $ddns_count = ceil((scalar(@{$vservers}) * $self->DDNS_VS_PERCENT) / 100);

    my $ddns_domain = delete($args{ddns_domain}) // NATE::Exceptions::Argument->throw('domain name for ddns is required');

    #create ddns resources
    foreach my $vs (@{$vservers}) {
        last if (!$ddns_count);

        my $ddns_res = NACL::AppBuilder::Xgen::Resource::DynamicDns->new(
            vserver       => $vs->get_attribute('vserver'),
            'is-enabled'  => "true",
            'use-secure'  => "true",
            'domain-name' => $vs->get_attribute('vserver') . "." . $ddns_domain,
        );
        $ddns_count--;
        $ddns_res->add_parent_resources(resources => [$vs]);
        $self->add_resources(resources => [$ddns_res]);
    }
} ## end sub generate_dynamic_dns_resources

sub generate_fsecurity_resources {
    my ($self, %args) = @_;

    my $vservers     = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
    my $domain_users = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Cifs::DomainUser");

    my $layout = convert_percent_layout_to_count_layout(
        layout        => $self->VSERVER_FSECURITY_ACCESS_CONTROL_LAYOUT,
        total_objects => scalar(@{$vservers})
    );

    my $policy_count  = 0;
    my $ntfs_sd_count = 0;

    # create fsecurity resources for vservers.
    foreach my $vs (@{$vservers}) {
        my @local_users = $vs->get_children_resources(type => "NACL::AppBuilder::Xgen::Resource::Cifs::LocalUser");
        my $access_control = get_next_layout_element(layout => $layout);
        my $policy_name = 'policy_fsecurity_';
        if ($access_control eq 'file-directory') {
            $policy_name .= "file_directory_" . $policy_count++;
        } elsif ($access_control eq 'slag') {
            $policy_name .= "slag_" . $policy_count++;
        } else {
            NATE::BaseException->throw("Invalid fsecurity access-control type $access_control");
        }

        # Create 1 policy per vserver
        my $fsecurity_policy = NACL::AppBuilder::Xgen::Resource::Security::FileDirectory::Policy->new(
            vserver       => $vs->get_attribute('vserver'),
            'policy-name' => $policy_name
        );
        $fsecurity_policy->add_parent_resources(resources => [$vs]);
        $self->add_resources(resources => [$fsecurity_policy]);

        # Create 1 apply per policy
        my $fsecurity_policy_apply = NACL::AppBuilder::Xgen::Resource::Security::FileDirectory::Apply->new(
            vserver       => $vs->get_attribute('vserver'),
            'policy-name' => $policy_name
        );
        $fsecurity_policy_apply->add_parent_resources(resources => [$vs]);
        $self->add_resources(resources => [$fsecurity_policy_apply]);

        # Reset the policy_task_index to 1 before we start making policy tasks for each vol.
        my $policy_task_index = 1;

        # Get the volumes that are children of this vserver.
        my @volumes = $vs->get_children_resources(type => "NACL::AppBuilder::Xgen::Resource::Volume");

        foreach my $vol (@volumes) {
            my $ntfs_sd_policy_name = "sd_policy_" . $ntfs_sd_count++;

            my $ntfs_user = shift(@{$domain_users});
            push(@{$domain_users}, $ntfs_user);

            # Create 1 ntfs per volume
            my $ntfs_policy = NACL::AppBuilder::Xgen::Resource::Security::FileDirectory::Ntfs->new(
                vserver   => $vs->get_attribute('vserver'),
                'ntfs-sd' => $ntfs_sd_policy_name,
                owner     => $ntfs_user->get_attribute('domain') . '\\' . $ntfs_user->get_attribute('user-name')
            );
            $ntfs_policy->add_parent_resources(resources => [$ntfs_user, $vol]);
            $self->add_resources(resources => [$ntfs_policy]);

            # Create 1 Policy Task per volume
            my $policy_task = NACL::AppBuilder::Xgen::Resource::Security::FileDirectory::Policy::PolicyTask->new(
                vserver          => $vs->get_attribute('vserver'),
                'ntfs-mode'      => 'propagate',
                'access-control' => $access_control,
                'ntfs-sd'        => [$ntfs_sd_policy_name],
                'policy-name'    => $policy_name,
                'security-type'  => 'ntfs',
                'index'          => $policy_task_index++,
                'path'           => $vol->get_attribute('junction-path')
            );
            $policy_task->add_parent_resources(resources => [$ntfs_policy, $vol]);
            $self->add_resources(resources => [$policy_task]);

            # Make # Local User Sacl per ntfs
            for (my $i = 0; $i < $self->TOTAL_LOCAL_USER_SACL_PER_NTFS; $i++) {
                my $local_user = shift(@local_users);
                push(@local_users, $local_user);
                my $ntfs_sacl = NACL::AppBuilder::Xgen::Resource::Security::FileDirectory::Ntfs::Sacl->new(
                    vserver           => $vs->get_attribute('vserver'),
                    account           => $local_user->get_attribute('user-name'),
                    'ntfs-sd'         => $ntfs_sd_policy_name,
                    'advanced-rights' => ['full-control'],
                    'access-type'     => 'success',
                    'apply-to'        => ['this-folder', 'sub-folders', 'files']
                );
                $ntfs_sacl->add_parent_resources(resources => [$ntfs_policy]);
                $self->add_resources(resources => [$ntfs_sacl]);
            }

            # Make # Domain User Sacl per ntfs
            for (my $i = 0; $i < $self->TOTAL_DOMAIN_USER_SACL_PER_NTFS; $i++) {
                my $domain_user = shift(@{$domain_users});
                push(@{$domain_users}, $domain_user);
                my $ntfs_sacl = NACL::AppBuilder::Xgen::Resource::Security::FileDirectory::Ntfs::Sacl->new(
                    vserver           => $vs->get_attribute('vserver'),
                    account           => $domain_user->get_attribute('domain') . '\\' . $domain_user->get_attribute('user-name'),
                    'ntfs-sd'         => $ntfs_sd_policy_name,
                    'advanced-rights' => ['full-control'],
                    'access-type'     => 'success',
                    'apply-to'        => ['this-folder', 'sub-folders', 'files']
                );
                $ntfs_sacl->add_parent_resources(resources => [$ntfs_policy]);
                $self->add_resources(resources => [$ntfs_sacl]);
            }

            # Make # Local User Dacl per ntfs
            for (my $i = 0; $i < $self->TOTAL_LOCAL_USER_DACL_PER_NTFS; $i++) {
                my $local_user = shift(@local_users);
                push(@local_users, $local_user);
                my $ntfs_dacl = NACL::AppBuilder::Xgen::Resource::Security::FileDirectory::Ntfs::Dacl->new(
                    vserver           => $vs->get_attribute('vserver'),
                    account           => $local_user->get_attribute('user-name'),
                    'ntfs-sd'         => $ntfs_sd_policy_name,
                    'advanced-rights' => ['full-control'],
                    'access-type'     => 'allow',
                    'apply-to'        => ['this-folder', 'sub-folders', 'files']
                );
                $ntfs_dacl->add_parent_resources(resources => [$ntfs_policy]);
                $self->add_resources(resources => [$ntfs_dacl]);
            }

            # Make # Domain User Dacl per ntfs
            for (my $i = 0; $i < $self->TOTAL_DOMAIN_USER_DACL_PER_NTFS; $i++) {
                my $domain_user = shift(@{$domain_users});
                push(@{$domain_users}, $domain_user);
                my $ntfs_dacl = NACL::AppBuilder::Xgen::Resource::Security::FileDirectory::Ntfs::Dacl->new(
                    vserver           => $vs->get_attribute('vserver'),
                    account           => $domain_user->get_attribute('domain') . '\\' . $domain_user->get_attribute('user-name'),
                    'ntfs-sd'         => $ntfs_sd_policy_name,
                    'advanced-rights' => ['full-control'],
                    'access-type'     => 'allow',
                    'apply-to'        => ['this-folder', 'sub-folders', 'files']
                );
                $ntfs_dacl->add_parent_resources(resources => [$ntfs_policy]);
                $self->add_resources(resources => [$ntfs_dacl]);
            }
        }
    }
} ## end sub generate_fsecurity_resources

sub generate_cifs_security_trace_filter_resources {
    my ($self, %args) = @_;

    my $vservers     = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
    my $domain_users = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Cifs::DomainUser");

    my $security_trace_filter_count                  = ceil((scalar(@{$vservers}) * $self->CIFS_SECURITY_TRACING_VS_PERCENT));
    my $cifs_security_trace_filter_users_per_vserver = $self->SECURITY_FILTER_USERS_PER_VSERVER;

    if ($cifs_security_trace_filter_users_per_vserver > 10) {
        NATE::BaseException->throw("Invalid number of Trace Filters, number must be 10 or less");
    }

    # create cifs security trace resources for vservers.
    foreach my $vs (@{$vservers}) {
        last if (!$security_trace_filter_count);
        $security_trace_filter_count--;
        for (my $i = 1; $i < $cifs_security_trace_filter_users_per_vserver; $i++) {
            my $user = shift(@{$domain_users});
            push(@{$domain_users}, $user);
            my $security_trace_filter_res = NACL::AppBuilder::Xgen::Resource::Security::Trace::Filter->new(
                vserver        => $vs->get_attribute('vserver'),
                'trace-allow'  => 'yes',
                index          => $i,
                'windows-name' => $user->get_attribute('domain') . '\\' . $user->get_attribute('user-name'),
            );
            $security_trace_filter_res->add_parent_resources(resources => [$user, $vs]);
            $self->add_resources(resources => [$security_trace_filter_res]);
        }
    }
} ## end sub generate_cifs_security_trace_filter_resources

sub generate_export_policy_resources {
    my ($self, %args) = @_;

    my $vservers = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");

    #create export policy resources
    foreach my $vs (@{$vservers}) {

        my $export_policy_res = NACL::AppBuilder::Xgen::Resource::ExportPolicy->new(
            vserver    => $vs->get_attribute('vserver'),
            policyname => "default",
        );
        $export_policy_res->add_parent_resources(resources => [$vs]);
        $self->add_resources(resources => [$export_policy_res]);
    }
} ## end sub generate_export_policy_resources

sub generate_qos_policy_resources {
    my ($self, %args) = @_;

    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
    my @vservers = @{$vserver_resources};
    
    my $node_resources    = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Node");
    my @nodes = @{$node_resources};
    
    my $layout = $self->VS_NODAL_LAYOUT;
    my $vserver_node_map_resource = create_vserver_to_node_layout(layout => $layout, nodes => \@nodes);
    my @vserver_node_map = @{$vserver_node_map_resource};

    log_global()->log("\tCREATE_QOS");
    
    #create export policy resources
    my $vserver_iter = 0;
    my $i = 0;
    while ($i < $self->TOTAL_QOS_POLICY_COUNT) {
        log_global()->log("CREATE_QOS $i");
        my $vs = shift(@vservers);
        push(@vservers, $vs);
        my @nodes_to_use = @{$vserver_node_map[$vserver_iter % scalar(@vserver_node_map)]};
        $vserver_iter++;
        #make sure qos policies scale as volumes scale per vserver
        for (my $j = 0; ($j < scalar(@nodes_to_use) && $i < $self->TOTAL_QOS_POLICY_COUNT); $j++) {
            my $policyname = "qos_policy_" . $i;
    
            my $qos_policy_res = NACL::AppBuilder::Xgen::Resource::QosPolicyGroup->new(
                vserver        => $vs->get_attribute('vserver'),
                "policy-group" => $policyname,
            );
            $qos_policy_res->add_parent_resources(resources => [$vs]);
            $self->add_resources(resources => [$qos_policy_res]);
            $i++;
        }
    }
} ## end sub generate_qos_policy_resources

sub generate_export_policy_rule_resources {
    my ($self, %args) = @_;

    my $export_policies = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::ExportPolicy");

    #create export policy rule resources
    foreach my $exp (@{$export_policies}) {
        if ($exp->get_attribute('policyname') eq "default") {
            my $export_policy_res = NACL::AppBuilder::Xgen::Resource::ExportPolicy::Rule->new(
                vserver      => $exp->get_attribute('vserver'),
                policyname   => "default",
                protocol     => "any",
                clientmatch  => "0.0.0.0/0",
                rwrule       => "any",
                ruleindex    => "1",
                superuser    => "any",
                "allow-suid" => "true",
                rorule       => "any",
            );
            $export_policy_res->add_parent_resources(resources => [$exp]);
            $self->add_resources(resources => [$export_policy_res]);
        }
    }
} ## end sub generate_export_policy_rule_resources

sub generate_snapmirror_resources {
    my ($self, %args) = @_;
    my $vol_resources  = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Volume");
    my $aggr_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Aggregate");

    my @vols  = @{$vol_resources};
    my @aggrs = @{$aggr_resources};

    my @snapmirror_resources      = ();
    my @snapmirror_init_resources = ();
    my @dest_vol_resources        = ();

    my $layout = convert_percent_layout_to_count_layout(
        layout        => $self->SNAPMIRROR_LAYOUT,
        total_objects => $self->TOTAL_SNAPMIRROR_COUNT
    );

    foreach my $k (keys %{$layout}) {
        my $prefix;
        if ($k eq 'load_sharing') {
            $prefix = "ls_dest_";
        } elsif ($k eq 'vault') {
            $prefix = "xdp_dest_";
        } elsif ($k eq 'data_protection') {
            $prefix = "dp_dest_";
        } else {
            NATE::BaseException->throw("Invalid snapmirror type $k");
        }

        for (my $i = 0; $i < $layout->{$k}; $i++) {
            my $dest_name = $prefix . $i;

            my $vol_src = shift(@vols);
            push(@vols, $vol_src);

            my $dest_aggr = shift(@aggrs);
            push(@aggrs, $dest_aggr);

            my $dest_vol = NACL::AppBuilder::Xgen::Resource::Volume->new(
                vserver           => $vol_src->get_attribute("vserver"),
                volume            => $dest_name,
                size              => $self->VOL_SIZE,
                type              => "DP",
                aggregate         => $dest_aggr->get_attribute("aggregate"),
                "space-guarantee" => "none",
            );
            my ($vs_res) = $vol_src->get_ancestors(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
            $dest_vol->add_parent_resources(resources => [$vs_res]);
            push(@dest_vol_resources, $dest_vol);

            my $snapmirror_res = NACL::AppBuilder::Xgen::Resource::Snapmirror->new(
                "destination-vserver" => $vol_src->get_attribute("vserver"),
                "destination-volume"  => $dest_name,
                "source-vserver"      => $vol_src->get_attribute("vserver"),
                "source-volume"       => $vol_src->get_attribute("volume"),
                "relationship-type"   => $k,
            );
            $snapmirror_res->add_parent_resources(resources => [$dest_vol]);
            push(@snapmirror_resources, $snapmirror_res);

            if ($k !~ /load_sharing/) {
                my $snapmirror_init_res = NACL::AppBuilder::Xgen::Resource::SnapmirrorInitialize->new(
                    "destination-vserver" => $vol_src->get_attribute("vserver"),
                    "destination-volume"  => $dest_name,
                    "source-vserver"      => $vol_src->get_attribute("vserver"),
                    "source-volume"       => $vol_src->get_attribute("volume"),
                );
                $snapmirror_init_res->add_parent_resources(resources => [$snapmirror_res]);
                push(@snapmirror_init_resources, $snapmirror_init_res);
            }

        }
    }
    $self->add_resources(resources => \@dest_vol_resources);
    $self->add_resources(resources => \@snapmirror_resources);
    $self->add_resources(resources => \@snapmirror_init_resources);
} ## end sub generate_snapmirror_resources

sub generate_cifs_branchcache_resources {
    my ($self, %args) = @_;

    my $vservers = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");

    #create branchcache resources
    foreach my $vs (@{$vservers}) {
        my @volumes = $vs->get_children_resources(type => "NACL::AppBuilder::Xgen::Resource::Volume");

        my $branchcache_res = NACL::AppBuilder::Xgen::Resource::Cifs::Branchcache->new(
            vserver           => $vs->get_attribute('vserver'),
            'hash-store-path' => $volumes[0]->get_attribute('junction-path'),
        );
        $branchcache_res->add_parent_resources(resources => [$volumes[0]]);
        $self->add_resources(resources => [$branchcache_res]);
    }
} ## end sub generate_cifs_branchcache_resources

sub generate_volume_efficiency_resources {
    my ($self, %args) = @_;

    my $volumes = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Volume");

    my $eff_count = ceil((scalar(@{$volumes}) * $self->VOL_EFFICIENCY_PERCENT) / 100);

    foreach my $vol (@{$volumes}) {
        last if (!$eff_count);
        $eff_count--;
        my $eff = NACL::AppBuilder::Xgen::Resource::Volume::Efficiency->new(
            vserver            => $vol->get_attribute('vserver'),
            volume             => $vol->get_attribute('volume'),
            compression        => 'true',
            'compression-type' => 'adaptive',

            #'min-blocks-shared' => "16", # causes a bug in the zapi TODO
            'inline-compression' => 'false'
        );
        $eff->add_parent_resources(resources => [$vol]);
        $self->add_resources(resources => [$eff]);
    }
} ## end sub generate_volume_efficiency_resources

sub generate_cifs_homedirsearchpath_resources {
    my ($self, %args) = @_;

    my $volumes = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Volume");

    #create home dir search path resources
    foreach my $vol (@{$volumes}) {
        if ($vol->get_attribute('junction-path')) {
            my $homedirsearchpath_res = NACL::AppBuilder::Xgen::Resource::Cifs::HomeDirSearchPath->new(
                vserver => $vol->get_attribute('vserver'),
                'path'  => $vol->get_attribute('junction-path'),
            );
            $homedirsearchpath_res->add_parent_resources(resources => [$vol]);
            $self->add_resources(resources => [$homedirsearchpath_res]);
        }
    }
} ## end sub generate_cifs_homedirsearchpath_resources

sub generate_cifs_domain_user_name_mapping_resources {
    my ($self, %args) = @_;

    my $vservers     = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
    my $domain_users = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Cifs::DomainUser");

    my $cache = {};

    my $layout = convert_percent_layout_to_count_layout(
        layout        => $self->DOMAIN_USER_NAME_MAPPING_LAYOUT,
        total_objects => $self->TOTAL_DOMAIN_USER_NAME_MAPPINGS
    );

    my $count = 0;
    while (my $type = get_next_layout_element(layout => $layout)) {

        # pick a vserver
        my $vs = shift(@{$vservers});
        push(@{$vservers}, $vs);
        if (!$cache->{$vs}) {
            $cache->{$vs}->{position} = 1;
        }

        # pick a domain user
        my $domain_user = shift(@{$domain_users});
        push(@{$domain_users}, $domain_user);

        # TODO: need to verify that the number of backslashes is proper
        my $pattern    = $domain_user->get_attribute('domain') . "\\" . $domain_user->get_attribute('user-name');
        my $directions = [];
        my $replacement;
        if ($type eq 'local') {

            # (local)
            # find a local user in this vserver to map it to
            my ($local_user) = $vs->get_descendants(type => "NACL::AppBuilder::Xgen::Resource::Cifs::LocalUser");
            if (!$local_user) {

                # pick a different vserver that may have a local user.

                # if we exhausted through all vservers, skip this and go to the next
                if ($count > scalar(@{$vservers})) {

                    # we can't do any more local user name mappings so
                    # clean up the layout for local so that we dont waste time
                    $layout->{local} = 0;
                    $count = 0;
                    next;
                }

                # increment the layout for local so that we redo it.
                $layout->{local}++;

                $count++;

                # redoing the loop will pick a different vserver for us
                redo;
            } else {

                # reset the count
                $count = 0;
            }

            $replacement = $local_user->get_attribute('user-name');
            $directions  = ['win-unix'];
        } elsif ($type eq 'default') {

            # (default)
            $replacement = "root";
            $directions  = ['win-unix'];
        } else {

            # (domain)
            $replacement = $pattern;

            # since we need to make a name mapping for each direction,
            # we'll need to decrement the domain group in the layout
            # an extra time.
            if ($layout->{domain} > 0) {
                $directions = ['win-unix', 'unix-win'];
                $layout->{domain}--;
            } else {

                # we can only make one name mapping resource for this domain user
                # so we have to use a single direction
                $directions = ['win-unix'];
            }
        }

        my @nm_resources = ();
        foreach my $direction (@{$directions}) {
            my $nm_res = NACL::AppBuilder::Xgen::Resource::NameMapping->new(
                vserver     => $vs->get_attribute('vserver'),
                direction   => $direction,
                replacement => $replacement,
                pattern     => $pattern,
                position    => $cache->{$vs}->{position}
            );
            $cache->{$vs}->{position}++;
            $nm_res->add_parent_resources(resources => [$vs]);
            push(@nm_resources, $nm_res);
        }

        $self->add_resources(resources => \@nm_resources);
    }
} ## end sub generate_cifs_domain_user_name_mapping_resources

sub generate_root_user_name_mapping_resources {
    my ($self, %args) = @_;

    my $vserver_obj  = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
    my $domain_users = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Cifs::DomainUser");
    my $replacement = delete($args{root_username}) // NATE::Exceptions::Argument->throw('root_username is required');

    my @vservers = @{$vserver_obj};

    my $count        = 0;
    my @nm_resources = ();
    foreach my $vs (@vservers) {

        my $nm_res = NACL::AppBuilder::Xgen::Resource::NameMapping->new(
            vserver     => $vs->get_attribute('vserver'),
            direction   => "unix-win",
            replacement => $replacement,
            pattern     => "root",
            position    => 1
        );

        $nm_res->add_parent_resources(resources => [$vs]);
        push(@nm_resources, $nm_res);

    }
    $self->add_resources(resources => \@nm_resources);
} ## end sub generate_root_user_name_mapping_resources

sub generate_cifs_group_policy_resources {
    my ($self, %args) = @_;

    my $vservers = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");

    foreach my $vs (@{$vservers}) {
        my $gpo_res = NACL::AppBuilder::Xgen::Resource::Cifs::GroupPolicy->new(
            vserver => $vs->get_attribute('vserver'),
            status  => "enabled"
        );
        $gpo_res->add_parent_resources(resources => [$vs]);
        $self->add_resources(resources => [$gpo_res]);
    }
} ## end sub generate_cifs_group_policy_resources

sub generate_cifs_security_resources {
    my ($self, %args) = @_;

    my $vservers = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");

    my $aes_count         = ceil((scalar(@{$vservers}) * $self->CIFS_SECURITY_AES_ENCRYPTION_VS_PERCENT) / 100);
    my $smb_signing_count = ceil((scalar(@{$vservers}) * $self->CIFS_SECURITY_SMB_SIGNING_VS_PERCENT) / 100);

    # create cifs security resources for vservers.
    foreach my $vs (@{$vservers}) {
        last if (!$aes_count and !$smb_signing_count);

        # skip vservers that do not have cifs children resources
        my ($cifs_res) = $vs->get_children_resources(type => "NACL::AppBuilder::Xgen::Resource::Cifs");
        next if (!$cifs_res);

        my %sec_args = ();
        if ($aes_count > 0) {
            $sec_args{'is-aes-encryption-enabled'} = "true";

            # to change the aes field, we need to specify the username and password to
            # the cifs domain (we can obtain this from the cifs service resource)
            $sec_args{'username'} = $cifs_res->get_attribute('admin-username');
            $sec_args{'password'} = $cifs_res->get_attribute('admin-password');
            $aes_count--;
        }
        if ($smb_signing_count > 0) {
            $sec_args{'is-signing-required'} = "true";
            $smb_signing_count--;
        }
        my $security_res = NACL::AppBuilder::Xgen::Resource::Cifs::Security->new(
            vserver => $vs->get_attribute('vserver'),
            %sec_args
        );
        $security_res->add_parent_resources(resources => [$cifs_res]);
        $self->add_resources(resources => [$security_res]);
    }
} ## end sub generate_cifs_security_resources

sub generate_cifs_options_resources {
    my ($self, %args) = @_;

    my $vservers = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");

    my $node_referral_count = ceil((scalar(@{$vservers}) * $self->CIFS_OPTIONS_NODE_REFERRAL_VS_PERCENT) / 100);
    my $dac_count           = ceil((scalar(@{$vservers}) * $self->CIFS_OPTIONS_DAC_VS_PERCENT) / 100);

    my $wins_servers = delete($args{wins_servers}) // NATE::Exceptions::Argument->throw('wins_servers is required');
    my $wins_servers_count = scalar(@{$vservers});

    # create cifs options resources for vservers.
    foreach my $vs (@{$vservers}) {
        last if (!$node_referral_count and !$dac_count and !$wins_servers_count);

        # skip vservers that do not have cifs children resources
        my ($cifs_res) = $vs->get_children_resources(type => "NACL::AppBuilder::Xgen::Resource::Cifs");
        next if (!$cifs_res);

        my %options_args = ();
        if ($node_referral_count > 0) {
            $options_args{'is-referral-enabled'} = "true";
            $node_referral_count--;
        }
        if ($dac_count > 0) {
            $options_args{'is-dac-enabled'} = "true";
            $dac_count--;
        }
        if ($wins_servers_count > 0) {
            $options_args{'wins-servers'} = $wins_servers;
            $wins_servers_count--;
        }

        my $options_res = NACL::AppBuilder::Xgen::Resource::Cifs::Options->new(
            vserver => $vs->get_attribute('vserver'),
            %options_args
        );
        $options_res->add_parent_resources(resources => [$cifs_res]);
        $self->add_resources(resources => [$options_res]);
    }
} ## end sub generate_cifs_options_resources

sub generate_network_route_resources {
    my ($self, %args) = @_;

    my $eth_fabric_data = delete($args{eth_fabric_data})
        // NATE::Exceptions::Argument->throw('eth_fabric_data is a required parameter for this config');

    # every vserver will have a route for every fabric that contains gateway information.
    my $vservers = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");

    my @network_routes = ();

    foreach my $fabric (keys %{$eth_fabric_data}) {
        my $gateway_v4     = $eth_fabric_data->{$fabric}->{gateway_v4};
        my $destination_v4 = $eth_fabric_data->{$fabric}->{destination_v4};
        my $gateway_v6     = $eth_fabric_data->{$fabric}->{gateway_v6};
        my $destination_v6 = $eth_fabric_data->{$fabric}->{destination_v6};
        if (!$gateway_v4 and !$gateway_v6) {

            #This fabric does not have network route information.
            next;
        }

        foreach my $vserver (@{$vservers}) {
            my $vserver_name = $vserver->get_attribute('vserver');

            if ($gateway_v4) {

                # Add the destination information if it was supplied.
                my %network_route_extra = ();

                $network_route_extra{'destination'} = $destination_v4 if (defined $destination_v4);

                # make a network route for this fabric on this vserver
                my $network_route = NACL::AppBuilder::Xgen::Resource::NetworkRoute->new(
                    vserver   => $vserver_name,
                    'gateway' => $gateway_v4,
                    %network_route_extra
                );
                $network_route->add_parent_resources(resources => [$vserver]);
                push(@network_routes, $network_route);
            }
            if ($gateway_v6) {

                # Add the destination information if it was supplied.
                my %network_route_extra = ();

                $network_route_extra{'destination'} = $destination_v6 if (defined $destination_v6);

                # make a network route for this fabric on this vserver
                my $network_route = NACL::AppBuilder::Xgen::Resource::NetworkRoute->new(
                    vserver   => $vserver_name,
                    'gateway' => $gateway_v6,
                    %network_route_extra
                );
                $network_route->add_parent_resources(resources => [$vserver]);
                push(@network_routes, $network_route);
            }
        }
    }
    $self->add_resources(resources => \@network_routes);
} ## end sub generate_network_route_resources

sub generate_audit_resources {
    my ($self, %args) = @_;

    my $vservers = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");

    my $audit_count = ceil((scalar(@{$vservers}) * $self->AUDIT_VS_PERCENT) / 100);

    #create audit resources
    foreach my $vs (@{$vservers}) {
        last if (!$audit_count);

        my @volumes = $vs->get_children_resources(type => "NACL::AppBuilder::Xgen::Resource::Volume");

        my $audit_res = NACL::AppBuilder::Xgen::Resource::Audit->new(
            vserver        => $vs->get_attribute('vserver'),
            'rotate-limit' => 5,
            destination    => $volumes[0]->get_attribute('junction-path'),
        );
        $audit_count--;
        $audit_res->add_parent_resources(resources => [$volumes[0]]);
        $self->add_resources(resources => [$audit_res]);
    }
} ## end sub generate_audit_resources

sub generate_lif_resources {
    my ($self, %args) = @_;

    my $eth_fabric_data = delete($args{eth_fabric_data}) // NATE::Exceptions::Argument->throw('eth_fabric_data is required');

    # every vserver spans every node
    #
    my $vservers = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
    my $nodes    = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Node");

    my $layout = $self->VS_NODAL_LAYOUT;

    my $vserver_node_map_resource = create_vserver_to_node_layout(layout => $layout, nodes => $nodes);
    my @vserver_node_map = @{$vserver_node_map_resource};

    my @lifs = ();

    my $vserver_iter = 0;

    #foreach my $node (@{$nodes}) {
    foreach my $vserver (@{$vservers}) {
        my $itr_v4        = 0;
        my $itr_v6        = 0;
        my @nodes_to_use = @{$vserver_node_map[$vserver_iter % scalar(@vserver_node_map)]};
        my $vserver_name = $vserver->get_attribute('vserver');
        foreach my $node (@nodes_to_use) {
            my $node_name    = $node->get_attribute('node');
            my $node_id      = $self->get_node_id($node_name);
            my $fabric_table = $self->get_fabric_lookup_table_by_node($node_name, 'eth');
            foreach my $fabric (keys %{$fabric_table}) {
                next if $fabric =~ /used_port_count/;
                my $address_range_v4 = $eth_fabric_data->{$fabric}->{address_range_v4};
                my $netmask_v4       = $eth_fabric_data->{$fabric}->{netmask_v4};
                my $subnet_v4        = $eth_fabric_data->{$fabric}->{subnet_v4};
                my $address_range_v6 = $eth_fabric_data->{$fabric}->{address_range_v6};
                my $netmask_v6       = $eth_fabric_data->{$fabric}->{netmask_v6};
                my $subnet_v6        = $eth_fabric_data->{$fabric}->{subnet_v6};
                if (!$address_range_v4 and !$address_range_v6 and !$subnet_v4 and !$subnet_v6 ) {
                    log_global()->log(dumper($eth_fabric_data));
                    NATE::Exceptions::Argument->throw("No address ranges or subnet were provided for fabric '$fabric' on node $node_name");
                }

                if ($address_range_v4 or $subnet_v4) {
                    for (my $i = 0; $i < $self->TOTAL_IPV4_LIFS_PER_VS_PER_NODE_PER_FABRIC; $i++) {

                        # Check to see if we have used all the ports
                        if (not defined($fabric_table->{used_port_count})
                            or $fabric_table->{used_port_count} < scalar(@{$fabric_table->{$fabric}})) {

                            # If we have not used all the ports then update count and
                            # continue to use next port in list.
                            $fabric_table->{used_port_count}++;
                        } else {

                            # If we have used all the ports then set the count to 0
                            # and rotate the list of ports once so that we do not
                            # always use the same ports.
                            $fabric_table->{used_port_count} = 0;
                            my $rotate_port = shift(@{$fabric_table->{$fabric}});
                            push(@{$fabric_table->{$fabric}}, $rotate_port);
                        }

                        # choose a port on this fabric for this node
                        my $port_name = shift(@{$fabric_table->{$fabric}});
                        push(@{$fabric_table->{$fabric}}, $port_name);

                        my $port_id = $self->get_port_id($node_name, $port_name);

                        # make a lif on this port and node that is ipv4
                        my $lif_name = "lf_v4_" . $itr_v4++ . "_n" . $node_id . "_p" . $port_id;
                        my $lif;
                        if ( $subnet_v4 ) {
                            $lif      = NACL::AppBuilder::Xgen::Resource::Lif->new(
                                vserver         => $vserver_name,
                                'data-protocol' => ['cifs', 'nfs'],
                                'subnet-name'   => $subnet_v4,
                                'auto-configuration' => $subnet_v4,
                                'auto-revert'   => "true",
                                'home-node'     => $node_name,
                                'home-port'     => $port_name,
                                'lif'           => $lif_name
                            );
                        } else {
                            $lif      = NACL::AppBuilder::Xgen::Resource::Lif->new(
                                vserver         => $vserver_name,
                                'data-protocol' => ['cifs', 'nfs'],
                                'address_range' => $address_range_v4,
                                'netmask'       => $netmask_v4,
                                'auto-revert'   => "true",
                                'home-node'     => $node_name,
                                'home-port'     => $port_name,
                                'lif'           => $lif_name
                            );
                        }
                        log_global()->log("CREATE_LIF v4 vserver: $vserver_name node:$node_name fabric:$fabric lif:$lif_name i:$i");
                        $lif->add_parent_resources(resources => [$vserver]);
                        push(@lifs, $lif);
                    }
                }
                if ($address_range_v6 or  $subnet_v6 ) {
                    for (my $i = 0; $i < $self->TOTAL_IPV6_LIFS_PER_VS_PER_NODE_PER_FABRIC; $i++) {

                        # choose a port on this fabric for this node
                        my $port_name = shift(@{$fabric_table->{$fabric}});
                        push(@{$fabric_table->{$fabric}}, $port_name);
                        my $port_id = $self->get_port_id($node_name, $port_name);

                        # make a lif on this port and node that is ipv6
                        my $lif_name = "lf_v6_" . $itr_v6++ . "_n" . $node_id . "_p" . $port_id;
                        my $lif;
                        if ($subnet_v6) {
                            $lif      = NACL::AppBuilder::Xgen::Resource::Lif->new(
                                vserver          => $vserver_name,
                                'data-protocol'  => ['cifs', 'nfs'],
                                'subnet-name'   => $subnet_v6,
                                'auto-configuration' => $subnet_v6,
                                'auto-revert'    => "true",
                                'home-node'      => $node_name,
                                'home-port'      => $port_name,
                                'lif'            => $lif_name
                            );
                        } else {
                            $lif      = NACL::AppBuilder::Xgen::Resource::Lif->new(
                                vserver          => $vserver_name,
                                'data-protocol'  => ['cifs', 'nfs'],
                                'address_range'  => $address_range_v6,
                                'netmask-length' => $netmask_v6,
                                'auto-revert'    => "true",
                                'home-node'      => $node_name,
                                'home-port'      => $port_name,
                                'lif'            => $lif_name
                            );
                        }
                        log_global()->log("CREATE_LIF v6 vserver: $vserver_name node:$node_name fabric:$fabric lif:$lif_name i:$i");
                        $lif->add_parent_resources(resources => [$vserver]);
                        push(@lifs, $lif);
                    }
                }
            }
        }
        $vserver_iter++;
    }
    $self->add_resources(resources => \@lifs);
} ## end sub generate_lif_resources

sub generate_symlink_resources {
    my ($self, %args) = @_;

    my $cifs_share_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Cifs::Share");
    my @cifs_shares          = @{$cifs_share_resources};
    my @symlinks             = ();

    for (my $i = 0; $i < $self->TOTAL_SYMLINK_COUNT; $i++) {
        my $cifs_share = shift(@cifs_shares);
        push(@cifs_shares, $cifs_share);
        my $unix_path = "/symlink_${i}/";
        my $cifs_path = "/symlink_${i}/";

        my ($vs_resource) = $cifs_share->get_ancestors(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
        my @cifs_svc_resources = $vs_resource->get_children_resources(type => "NACL::AppBuilder::Xgen::Resource::Cifs");
        my $cifs_svc_res = $cifs_svc_resources[0];

        my $symlink_res = NACL::AppBuilder::Xgen::Resource::Cifs::Symlink->new(
            'share-name'  => $cifs_share->get_attribute('share-name'),
            'unix-path'   => $unix_path,
            'cifs-path'   => $cifs_path,
            'cifs-server' => $cifs_svc_res->get_attribute('cifs-server'),
            vserver       => $vs_resource->get_attribute("vserver")
        );
        $symlink_res->add_parent_resources(resources => [$cifs_share, $vs_resource]);

        push(@symlinks, $symlink_res);
    }

    $self->add_resources(resources => \@symlinks);

} ## end sub generate_symlink_resources

sub generate_cifs_user_and_group_resources {
    my ($self, %args) = @_;

    my $domain = delete($args{cifs_domain}) // NATE::Exceptions::Argument->throw('cifs_domain is required');

    my $vs_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
    my @vservers = @{$vs_resources};

    # make local users
    my @local_users = ();
    for (my $i = 0; $i < $self->TOTAL_LOCAL_USERS; $i++) {
        my $vs = shift(@vservers);
        push(@vservers, $vs);
        my $user_name = "cfit_lu_" . $i;
        my $full_name = $user_name;
        my $res       = NACL::AppBuilder::Xgen::Resource::Cifs::LocalUser->new(
            'vserver'     => $vs->get_attribute('vserver'),
            'user-name'   => $user_name,
            'full-name'   => $full_name,
            'description' => "User $user_name",
            'password'    => 'netapp1!',
        );
        push(@local_users, $res);
        $res->add_parent_resources(resources => [$vs]);
    }
    $self->add_resources(resources => \@local_users);

    # make domain users
    my @domain_users = ();
    for (my $i = 0; $i < $self->TOTAL_DOMAIN_USERS; $i++) {
        my $user_name = $self->DOMAIN_USER_PREFIX . ($i + 1);
        my $res = NACL::AppBuilder::Xgen::Resource::Cifs::DomainUser->new(
            'user-name' => $user_name,
            'domain'    => $domain,
        );
        push(@domain_users, $res);
    }
    $self->add_resources(resources => \@domain_users);

    my $layout = convert_percent_layout_to_count_layout(
        layout        => $self->LOCAL_GROUP_MEMBERSHIP_LAYOUT,
        total_objects => $self->MAX_USER_MEMBERSHIPS
    );


    # make groups
    my @groups = ();
    my @privs  = ();
    for (my $i = 0; $i < $self->TOTAL_LOCAL_GROUPS; $i++) {
        my $vs = shift(@vservers);
        push(@vservers, $vs);

        my $group_name   = "cfit_lg_" . $i;
        my $vserver_name = $vs->get_attribute('vserver');
        my $group        = NACL::AppBuilder::Xgen::Resource::Cifs::LocalGroup->new(
            xgen_nacl_cs_validation => 0,
            vserver                 => $vserver_name,
            'group-name'            => $group_name,
            description             => "Group $group_name"
        );
        push(@groups, $group);
        $group->add_parent_resources(resources => [$vs]);

        my $priv = NACL::AppBuilder::Xgen::Resource::Cifs::Privilege->new(
            xgen_nacl_cs_validation => 0,
            vserver                 => $vserver_name,
            'user-or-group-name'    => $group_name,
            privileges              => ["SeTcbPrivilege",]
        );
        $priv->add_parent_resources(resources => [$group]);
        push(@privs, $priv);
    }
    $self->add_resources(resources => \@groups);
    $self->add_resources(resources => \@privs);


    # local user memberships
    my $vserver_user_cache      = {};
    my $total_local_memberships = 0;
    my @memberships             = ();
    my @tmp_groups              = @groups;
    for (my $j = 0; $j < $layout->{local}; $j++) {

        # pick a group
        my $group = shift(@tmp_groups);

        my $vserver_name = $group->get_attribute('vserver');

        # initialize the available user cache for this group
        # if its not already cached
        if (not exists $vserver_user_cache->{$vserver_name}) {
            my @vservers = $group->get_ancestors(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
            my @users = $vservers[0]->get_children_resources(type => "NACL::AppBuilder::Xgen::Resource::Cifs::LocalUser");
            $vserver_user_cache->{$vserver_name} = \@users;
        }

        my @existing_memberships
            = $group->get_children_resources(type => "NACL::AppBuilder::Xgen::Resource::Cifs::LocalGroup::Membership");

        # pick a user in this vserver
        my $user;
        my $count = 0;
    USER: while (1) {
            if ($count >= scalar(@{$vserver_user_cache->{$vserver_name}})) {
                log_global()->log("too many users for $vserver_name");

                # no more memberships can be added to this group
                undef $user;
                last;
            }
            $user = shift(@{$vserver_user_cache->{$vserver_name}});
            push(@{$vserver_user_cache->{$vserver_name}}, $user);

            foreach my $m (@existing_memberships) {
                if ($m->get_attribute('member') eq $user->get_attribute('user-name')) {

                    # pick another user
                    $count++;
                    next USER;
                }
            }

            # we found the user
            last;
        }

        if (!$user) {
            log_global()->log("redoing group $j");

            # dont add the group back on the list because we can't add any more users to it
            redo;
        }
        push(@tmp_groups, $group);

        # make the membership
        my $membership_res = NACL::AppBuilder::Xgen::Resource::Cifs::LocalGroup::Membership->new(
            'member'                => $user->get_attribute('user-name'),
            xgen_nacl_cs_validation => 0,
            vserver                 => $user->get_attribute('vserver'),
            'group-name'            => $group->get_attribute('group-name')
        );
        $membership_res->add_parent_resources(resources => [$user, $group]);
        push(@memberships, $membership_res);
        $total_local_memberships++;

        # reset the count since we successfull found a user to assign to this group
        $count = 0;
    }
    if ($total_local_memberships ne $layout->{local}) {
        log_global()
            ->warn("Was only able to create $total_local_memberships local user "
                . "memberships, but was asked to create "
                . $layout->{local});
    }
    undef $vserver_user_cache;


    # domain user memberships
    my $total_domain_memberships = 0;
    for (my $i = 0; $i < $layout->{domain}; $i++) {

        # pick a group
        my $group = shift(@groups);
        if (!$group) {

            # no more groups are left...
            last;
        }

        # check to see if there exists a membership between this user and this group already.
        my @existing_memberships
            = $group->get_children_resources(type => "NACL::AppBuilder::Xgen::Resource::Cifs::LocalGroup::Membership");

        my $user;
        my $count = 0;
    USER: while (1) {
            if ($count >= scalar(@domain_users)) {

                # there are no more domain users that we can add as members to this group
                undef $user;
                last;
            }
            $user = shift(@domain_users);
            push(@domain_users, $user);

            foreach my $member (@existing_memberships) {
                if ($member->get_attribute('member') eq $user->get_attribute('user-name')) {

                    # this user is already a member, pick a different user
                    $count++;
                    next USER;
                }
            }

            # make a membership between this user and group
            last;
        }

        # no more users can be added to this group
        if (!$user) {

            # remove this group from the consideration list
            redo;
        }

        # push this group back on the list
        push(@groups, $group);

        my $membership_res = NACL::AppBuilder::Xgen::Resource::Cifs::LocalGroup::Membership->new(
            'member'                => $user->get_attribute('domain') . "\\" . $user->get_attribute('user-name'),
            xgen_nacl_cs_validation => 0,
            vserver                 => $group->get_attribute('vserver'),
            'group-name'            => $group->get_attribute('group-name')
        );
        $membership_res->add_parent_resources(resources => [$user, $group]);
        push(@memberships, $membership_res);

        $total_domain_memberships++;
    }
    if ($total_domain_memberships ne $layout->{domain}) {
        log_global()
            ->warn("Was only able to create $total_domain_memberships domain user "
                . "memberships, but was asked to create "
                . $layout->{domain});
    }

    $self->add_resources(resources => \@memberships);
} ## end sub generate_cifs_user_and_group_resources

sub generate_cifs_share_resources {
    my ($self, %args) = @_;

    my $qtree_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Qtree");
    my @qtrees = @{$qtree_resources};

    my $volume_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Volume");
    my @volumes = @{$volume_resources};

    my $layout = convert_percent_layout_to_count_layout(
        layout        => $self->CIFS_SHARE_LAYOUT,
        total_objects => $self->TOTAL_CIFS_SHARE_COUNT
    );

    my $arr;
    my @cifs_share_resources = ();
    foreach my $k (keys %{$layout}) {
        my $prefix;
        if ($k eq 'qtree') {
            $arr = \@qtrees;
        } elsif ($k eq 'volume') {
            $arr = \@volumes;
        } else {
            NATE::BaseException->throw("Invalid cifs share layout type $k");
        }

        for (my $i = 0; $i < $layout->{$k}; $i++) {
            my $parent_res = shift(@{$arr});
            push(@{$arr}, $parent_res);
            my $share_name = "s_${k}_${i}";

            my $cifs_share_res = NACL::AppBuilder::Xgen::Resource::Cifs::Share->new(
                'share-name' => $share_name,
                vserver      => $parent_res->get_attribute("vserver")
            );
            $cifs_share_res->add_parent_resources(resources => [$parent_res]);

            my ($vol_res) = $cifs_share_res->get_ancestors(type => "NACL::AppBuilder::Xgen::Resource::Volume");
            my $path = $vol_res->get_attribute('junction-path');
            if ($parent_res->isa("NACL::AppBuilder::Xgen::Resource::Qtree")) {
                $path .= "/" . $parent_res->get_attribute("qtree");
            }
            $cifs_share_res->set_private_attributes(path => $path);
            push(@cifs_share_resources, $cifs_share_res);
        }
    }

    # create a special cifs share on (share-name='~%w',path='%w',share-properties='homedirectory')
    # on each vserver
    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
    foreach my $vs_res (@{$vserver_resources}) {
        my $res = NACL::AppBuilder::Xgen::Resource::Cifs::Share->new(
            'share-name'       => '~%w',
            vserver            => $vs_res->get_attribute('vserver'),
            path               => '%w',
            'share-properties' => 'homedirectory'
        );
        $res->add_parent_resources(resources => [$vs_res]);
        push(@cifs_share_resources, $res);
    }

    $self->add_resources(resources => \@cifs_share_resources);
} ## end sub generate_cifs_share_resources

sub generate_qtree_resources {
    my ($self, %args) = @_;

    my $vol_resources   = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Volume");
    my @vols            = @{$vol_resources};
    my @qtree_resources = ();

    # make regular qtrees

    for (my $i = 0; $i < $self->TOTAL_QTREE_COUNT; $i++) {
        my $vol_res = shift(@vols);
        push(@vols, $vol_res);
        my $qtree_name = "test_qtree_$i";
        my $qt_res     = NACL::AppBuilder::Xgen::Resource::Qtree->new(
            qtree   => $qtree_name,
            volume  => $vol_res->get_attribute("volume"),
            vserver => $vol_res->get_attribute("vserver")
        );
        push(@qtree_resources, $qt_res);
        $qt_res->add_parent_resources(resources => [$vol_res]);
    }

    my $layout = convert_percent_layout_to_count_layout(
        layout        => $self->QTREE_HOMEDIR_LAYOUT,
        total_objects => $self->TOTAL_QTREE_HOMEDIR_COUNT
    );
    foreach my $k (keys %{$layout}) {
        my $prefix;
        if ($k eq 'local') {
            $prefix = "cfit_lu_";
        } elsif ($k eq 'domain') {
            $prefix = "ptefit_";
        } else {
            NATE::BaseException->throw("Invalid qtree type $k");
        }

        for (my $i = 0; $i < $layout->{$k}; $i++) {
            my $qtree_name = $prefix . $i;

            my $vol_res = shift(@vols);
            push(@vols, $vol_res);

            my $qt_res = NACL::AppBuilder::Xgen::Resource::Qtree->new(
                qtree   => $qtree_name,
                volume  => $vol_res->get_attribute("volume"),
                vserver => $vol_res->get_attribute("vserver")
            );
            push(@qtree_resources, $qt_res);
            $qt_res->add_parent_resources(resources => [$vol_res]);
        }
    }

    $self->add_resources(resources => \@qtree_resources);
} ## end sub generate_qtree_resources

sub generate_volume_resources {
    my ($self, %args) = @_;

    #my $aggr_resources    = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Aggregate");
    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
    my $node_resources    = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Node");

    #my @aggrs         = @{$aggr_resources};
    my @vservers = @{$vserver_resources};
    my @nodes    = @{$node_resources};

    my $layout = $self->VS_NODAL_LAYOUT;

    my $vserver_node_map_resource = create_vserver_to_node_layout(layout => $layout, nodes => \@nodes);
    my @vserver_node_map = @{$vserver_node_map_resource};

    my @vol_resources = ();
    my $vserver_iter  = 0;
    my $i             = 0;
    while ($i < $self->TOTAL_VOLUME_COUNT) {

        #for (my $i = 0; $i < $self->TOTAL_VOLUME_COUNT; $i++) {

        my $vs_res       = $vservers[$vserver_iter % scalar(@vservers)];
        my @nodes_to_use = @{$vserver_node_map[$vserver_iter % scalar(@vserver_node_map)]};

        # for all non ntfs vols, open up the unix permissions
        my %extra = ();
        if ($vs_res->get_attribute("rootvolume-security-style") ne 'ntfs') {
            $extra{'unix-permissions'} = "755";
        }

        my @qos_policies = $vs_res->get_children_resources(type => "NACL::AppBuilder::Xgen::Resource::QosPolicyGroup");

        

        my $max_autosize = $self->VOL_SIZE;
        if ($max_autosize =~ /^(\d+)(\D*)/) {
            my $digit = $1;
            my $unit  = $2;
            $max_autosize = ($digit * $self->VOL_MAX_AUTOSIZE_MULTIPLIER);
            if ($unit) {
                $max_autosize .= $unit;
            }
        }
        for (my $k = 0; ($k < scalar(@nodes_to_use)) && ($i < $self->TOTAL_VOLUME_COUNT); $k++) {

            my $node       = $nodes_to_use[$k];
            my $vol_number = floor($vserver_iter / scalar(@vservers)) * scalar(@nodes_to_use) + $k;
            
            
            if (@qos_policies) {
                my $policy_iter = $vol_number % scalar(@qos_policies);
                my $policy = $qos_policies[$policy_iter];
                $extra{'qos-policy-group'} = $policy->get_attribute("policy-group");
            }   

            #log_global()->log(dumper(\$node));
            my @aggrs = $node->get_children_resources(type => "NACL::AppBuilder::Xgen::Resource::Aggregate");

            #log_global()->log(dumper(\@aggrs));

            my $aggr         = $aggrs[$vserver_iter % scalar(@aggrs)];
            my $vserver_name = $vs_res->get_attribute("vserver");
            my $aggr_name    = $aggr->get_attribute("aggregate");
            my $vol_name   = $vserver_name . "_" . $aggr_name . "_vol_$vol_number";
            log_global()->log("CREATE_VOL $vserver_name $aggr_name $vol_name $vserver_iter $k " . scalar(@nodes_to_use));
            my $vol_res = NACL::AppBuilder::Xgen::Resource::Volume->new(
                vserver                           => $vs_res->get_attribute("vserver"),
                volume                            => $vol_name,
                'junction-path'                   => "/" . $vol_name,
                'autosize-mode'                   => $self->VOL_AUTOSIZE_MODE,
                size                              => $self->VOL_SIZE,
                'autosize-grow-threshold-percent' => $self->VOL_AUTOSIZE_GROW_THRESHOLD_PERCENT,
                'max-autosize'                    => $max_autosize,
                aggregate                         => $aggr->get_attribute("aggregate"),
                "space-guarantee"                 => "none",
                %extra
            );
            push(@vol_resources, $vol_res);
            $vol_res->add_parent_resources(resources => [$vs_res]);
            $i++;
        }
        $vserver_iter++;
    }

    $self->add_resources(resources => \@vol_resources);
} ## end sub generate_volume_resources

sub generate_vserver_resources {
    my ($self, %args) = @_;

    my $username           = delete($args{cifs_username})      // NATE::Exceptions::Argument->throw('cifs_username is required');
    my $password           = delete($args{cifs_password})      // NATE::Exceptions::Argument->throw('cifs_password is required');
    my $domain             = delete($args{cifs_domain})        // NATE::Exceptions::Argument->throw('cifs_domain is required');
    my $cifs_server_prefix = delete($args{cifs_server_prefix}) // NATE::Exceptions::Argument->throw('cifs_server_prefix is required');
    my $dns_domains        = delete($args{dns_domains})        // NATE::Exceptions::Argument->throw('dns_domains is required');
    my $name_servers       = delete($args{name_servers})       // NATE::Exceptions::Argument->throw('name_servers is required');
    my $nis_domain         = delete($args{nis_domain})         // NATE::Exceptions::Argument->throw('nis_domain is required');
    my $nis_servers        = delete($args{nis_servers})        // NATE::Exceptions::Argument->throw('nis_servers is required');

    my $aggr_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Aggregate");

    my @aggrs = @{$aggr_resources};

    my $layout = convert_percent_layout_to_count_layout(
        layout        => $self->VSERVER_ROOT_VOLUME_SECURITY_LAYOUT,
        total_objects => $self->TOTAL_VS_COUNT
    );

    my $vserver_layout = "node";
    #**********************************************************
    #* get ipspace
    #**********************************************************
    my $ipspace = 'Default';
    my $eth_fabric_data = $args{eth_fabric_data};
    foreach my $key (%{$eth_fabric_data}) {
        if ( $eth_fabric_data->{$key}->{ipspace}) {
           # use the first ipspace found
           $ipspace = $eth_fabric_data->{$key}->{ipspace};
           last;
        }
    }
    # make vserver resources
    my (@vservers, @cifs_svcs, @dns_svcs, @nfs_svcs, @nis_svcs) = ();
    for (my $i = 0; $i < $self->TOTAL_VS_COUNT; $i++) {
        my $aggr = shift(@aggrs);
        push(@aggrs, $aggr);
        $vserver_layout = "cluster" if $i <= $self->VSERVER_COUNT_CLUSTER_WIDE;

        # get the next security element to use from the layout
        my $sec = get_next_layout_element(
            iterator => $i,
            layout   => $layout
        );

        my $vs_name = "vs_" . $i;

        my $cifs_server_name = $cifs_server_prefix . "_" . $vs_name;

        # convert all '-' characters to '_'
        $cifs_server_name =~ s/-/_/g;
        if (length($cifs_server_name) > 15) {
            NATE::BaseException->throw("CIFS server name '$cifs_server_name' is too long, "
                    . "decrease the length of 'cifs_server_prefix=${cifs_server_prefix}' to be "
                    . "smaller so that the CIFS server name is less than 15 characters.");
        }

        # create net bios aliases for each vserver
        my %netbios_aliases;
        for (my $k = 0; $k < $self->CIFS_NETBIOS_ALIASES_PER_VS; $k++) {
            my $alias = $cifs_server_name . "_" . $k;
            if (length($alias) > 15) {
                NATE::BaseException->throw("CIFS Netbios name '$alias' is too long, decrease the length "
                        . "of 'cifs_server_prefix=${cifs_server_prefix}' to be "
                        . "smaller so that the alias is less than 15 characters.");
            }
            push(@{$netbios_aliases{'netbios-aliases'}}, $cifs_server_name . "_" . $k);
        }

        my $vs_res = NACL::AppBuilder::Xgen::Resource::Vserver->new(
            vserver                     => $vs_name,
            aggregate                   => $aggr->get_attribute("aggregate"),
            'rootvolume-security-style' => $sec,
            ipspace                     => $ipspace,
        );

        my $cifs_svc = NACL::AppBuilder::Xgen::Resource::Cifs->new(
            vserver          => $vs_name,
            domain           => $domain,
            'cifs-server'    => $cifs_server_name,
            'username'       => $username,
            'password'       => $password,
            #'admin-username' => $username,
            #'admin-password' => $password,
            %netbios_aliases,
        );

        #log_global()->log("\tCREATE_CIFS $vs_name $domain" . dumper($cifs_svc));

        my @dns_servers_temp = shuffle(@{$name_servers});
        my @dns_name_servers = @dns_servers_temp[0 .. 1];
        my $dns_svc          = NACL::AppBuilder::Xgen::Resource::Dns->new(
            vserver        => $vs_name,
            domains        => $dns_domains,
            'name-servers' => \@dns_name_servers
        );

        my @nis_servers_temp = shuffle(@{$nis_servers});
        my $nis_servers      = join(",", @nis_servers_temp[0 .. 1]);
        my $nis_svc          = NACL::AppBuilder::Xgen::Resource::Nis->new(
            vserver   => $vs_name,
            "domain"  => $nis_domain,
            'servers' => $nis_servers,
            'active'  => "true"
        );

        my $nfs_svc = NACL::AppBuilder::Xgen::Resource::Nfs->new(
            vserver     => $vs_name,
            "v4.0"      => "enabled",
            "v4.1-acl"  => "enabled",
            "v3"        => "enabled",
            "v4.0-acl"  => "enabled",
            "rquota"    => "enabled",
            "v4.1"      => "enabled",
            "v4.1-pnfs" => "enabled",
        );

        # connect the vs resource to the aggr's cluster parent resource
        $vs_res->add_parent_resources(resources => [$aggr->get_ancestors(type => "NACL::AppBuilder::Xgen::Resource::Cluster")]);

        # connect the cifs, dns, and nfs svc resource to the vs
        $cifs_svc->add_parent_resources(resources => [$vs_res]);
        $dns_svc->add_parent_resources(resources => [$vs_res]);
        $nis_svc->add_parent_resources(resources => [$vs_res]);
        $nfs_svc->add_parent_resources(resources => [$vs_res]);

        push(@vservers,  $vs_res);
        push(@cifs_svcs, $cifs_svc);
        push(@dns_svcs,  $dns_svc);
        push(@nfs_svcs,  $nfs_svc);
        push(@nis_svcs,  $nis_svc);
    }

    $self->add_resources(resources => \@vservers);
    $self->add_resources(resources => \@cifs_svcs);
    $self->add_resources(resources => \@dns_svcs);
    $self->add_resources(resources => \@nfs_svcs);
    $self->add_resources(resources => \@nis_svcs);
} ## end sub generate_vserver_resources

sub _calculate_disks_per_aggr {
    my ($pkg, %args) = @_;
    my $node           = $args{"node"};
    my $aggrs_per_node = $args{"aggrs_per_node"};
    log_global()->log("\t_calculate_disks_per_aggr $node");
    my $nacl_node_obj = NACL::C::Node->new(name => $node);
    my @disks = NACL::CS::StorageDisk->fetch(
        command_interface => $nacl_node_obj,
        filter            => {owner => $node},
        allow_empty       => 1
    );
    my $disk_count       = scalar(@disks);
    my @spare_disks      = grep { $_->container_type() eq "spare" } @disks;
    my $spare_disk_count = scalar(@spare_disks);
    my $disk_per_aggr    = int(($spare_disk_count / $aggrs_per_node) - 1);

    if ($disk_per_aggr >= 14) {
        $disk_per_aggr = 14;
    } elsif ($disk_per_aggr <= 13 && $disk_per_aggr >= 5) {
        $disk_per_aggr = $disk_per_aggr;
    } elsif ($disk_per_aggr < 5) {

        # let the ruleset fail graciously and rollback in this case
        $disk_per_aggr = 0;
        NATE::Exceptions::Argument->throw("$node doesnt have enough disks");
    }
    return $disk_per_aggr;
} ## end sub _calculate_disks_per_aggr

sub convert_vserver_layout_to_percent_layout {
    my (%args) = @_;

    # The layout of layout.
    # { node_count => vserver_count,
    #   node_count => vserver_count,}
    my $count_layout = $args{layout} // NATE::Exceptions::Argument->throw("layout is a required parameter.");
    if (ref($count_layout) ne 'HASH') {
        NATE::Exceptions::Argument->throw("'layout' must be a hash reference");
    }

    my $total = 0;
    foreach my $key (keys %{$count_layout}) {
        $total += ($count_layout->{$key} * $key);
    }

    my %percent_layout;

    foreach my $key (keys %{$count_layout}) {
        $percent_layout{$key} = 100 * ($count_layout->{$key} * $key) / $total;
    }

    return \%percent_layout;
} ## end sub convert_vserver_layout_to_percent_layout

sub create_vserver_to_node_layout {
    my (%args) = @_;

    my $vserver_layout = $args{layout};
    my @nodes_to_use          = @{$args{nodes}};

    if (ref($vserver_layout) ne 'HASH') {
        NATE::Exceptions::Argument->throw("'layout' must be a hash reference");
    }
    log_global()->log("CREATE_LAYOUT " . Dumper($vserver_layout) . @nodes_to_use);

    my @vserver_nodes;
    my $i = 0;
    foreach my $key (sort { $b <=> $a } keys %{$vserver_layout}) {
        #my @nodes_to_use = @nodes;
        #workaround for wierd 24node qos config
        #if ($key == 24){
        #    @nodes_to_use = @nodes;
        #    #log_global()->log("CREATE_LAYOUT $key is 24, using all");
        #} else {
        #    @nodes_to_use = @nodes[12..23];
        #    #log_global()->log("CREATE_LAYOUT $key is not 24, using " . Dumper(@nodes_to_use));
        #}
        for (my $k = 0; $k < $vserver_layout->{$key}; $k++) {
            my @nodes_for_vserver = ();
            for (my $j = 0; $j < $key; $j++) {
                my $internal_iter = ($i) % scalar(@nodes_to_use);
                log_global()->log("CREATE_LAYOUT $key $internal_iter " . $nodes_to_use[$internal_iter]->get_attribute('node'));
                push(@nodes_for_vserver, $nodes_to_use[$internal_iter]);
                $i++;
            }
            push(@vserver_nodes, \@nodes_for_vserver);
        }

    }
    return \@vserver_nodes;

} ## end sub create_vserver_to_node_layout

sub generate_iscsi_services_resources {
    my ($self, %args) = @_;

    # Get the clusters so that we can alternate vserver creation between multiple clusters.
    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
    my @vservers = @{$vserver_resources};

    my @iscsi_services;

    my $i = 0;
    foreach my $vserver (@vservers) {
        while ($i < $self->SAN_VSERVERS_COUNT ) {
            my $iscsi_res = NACL::AppBuilder::Xgen::Resource::Iscsi->new(vserver => $vserver->get_attribute('vserver'));
    
            # Add vserver as parent to iscsi service.
            $iscsi_res->add_parent_resources(resources => [$vserver]);
    
            push(@iscsi_services, $iscsi_res);
            $i++;
            
        }
    }
    $self->add_resources(resources => \@iscsi_services);
}

sub generate_iscsi_lif_resources {
    my ($self, %args) = @_;

    my $iscsi_vservers = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Iscsi");

    foreach my $iscsi_vs (@{$iscsi_vservers}) {
        my $itr_v4        = 0;
        my $itr_v6        = 0;
        my ($vs) = $iscsi_vs->get_ancestors(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
        my $vserver_name = $vs->get_attribute('vserver');
        my $eth_fabric_data = $args{eth_fabric_data};

        # Each vserver should only have one ancestor that is a cluster.
        my ($cluster) = $vs->get_ancestors(type => "NACL::AppBuilder::Xgen::Resource::Cluster");
        # Get all the nodes that are children of the vservers cluster so that we get all the nodes.
        my @nodes = $cluster->get_descendants(type => "NACL::AppBuilder::Xgen::Resource::Node");
        my $already_create_routes;
        # make an iscsi lif on each node (on each fabric)
        foreach my $node (@nodes) {
            my $node_name = $node->get_attribute('node');
            my $node_id = $self->get_node_id($node_name);

            # obtain the ethenet fabric table for this node
            my $fabric_table = $self->get_fabric_lookup_table_by_node($node_name, 'eth');
            foreach my $fabric (keys %{$fabric_table}) {
                print "hit111\n";
                $Log->comment(Dumper($eth_fabric_data->{$fabric}));
                my $address_range_v4 = $eth_fabric_data->{$fabric}->{address_range_v4};
                my $netmask_v4       = $eth_fabric_data->{$fabric}->{netmask_v4};
                my $subnet_v4       = $eth_fabric_data->{$fabric}->{subnet_v4};
                my $address_range_v6 = $eth_fabric_data->{$fabric}->{address_range_v6};
                my $netmask_v6       = $eth_fabric_data->{$fabric}->{netmask_v6};
                my $subnet_v6        = $eth_fabric_data->{$fabric}->{subnet_v6};
                if (!$address_range_v4 and !$address_range_v6 and !$subnet_v4 and !$subnet_v6 ) {
                    log_global()->log(dumper($eth_fabric_data));
                    NATE::Exceptions::Argument->throw("No address ranges or subnet were provided for fabric '$fabric' on node $node_name");
                }
                if ($address_range_v4 or $subnet_v4) {
                    # if this fabric has a gateway defined,
                    # make a route resource on this vserver if we havn't built one yet for this fabric
                    $already_create_routes->{$fabric} //= {};
                    if (!$already_create_routes->{$fabric}->{route_resource} and $eth_fabric_data->{$fabric}->{gateway_v4}) {
                        my $route_resource = NACL::AppBuilder::Xgen::Resource::NetworkRoute->new(vserver => $vs->get_attribute('vserver'),
                                                                                             gateway => $eth_fabric_data->{$fabric}->{gateway_v4});
                        $route_resource->add_parent_resources(resources => [$vs]);
                        $self->add_resources(resources => [$route_resource]);
                        $already_create_routes->{$fabric}->{route_resource} = $route_resource;
                    }

                    for (my $i = 0; $i < $self->TOTAL_IPV4_ISCSI_LIFS_PER_VS_PER_NODE_PER_FABRIC; $i++) {

                        # choose a port on this fabric for this node
                        my $port_name = shift(@{$fabric_table->{$fabric}});
                        push(@{$fabric_table->{$fabric}}, $port_name);
                        my $port_id = $self->get_port_id($node_name, $port_name);
                       
                        # make the lif resource
                        my $lif;
                        my $lif_name = "iscsi_v4_" . $itr_v4++ . "_n" . $node_id . "_p" . $port_id;
                        if ($subnet_v4) {
                            $lif = NACL::AppBuilder::Xgen::Resource::Lif->new(
                                 vserver => $vserver_name,
                                 'data-protocol' => ['iscsi'],
                                 'subnet-name' => $subnet_v4,
                                 'auto-configuration' => $subnet_v4,
                                 lif => $lif_name,
                                 'home-node' => $node_name,
                                 'home-port' => $port_name);
                        } else {
                            $lif = NACL::AppBuilder::Xgen::Resource::Lif->new(
                                 vserver => $vserver_name,
                                 'data-protocol' => ['iscsi'],
                                 address_range => $address_range_v4,
                                 netmask => $netmask_v4,
                                 lif => $lif_name,
                                 'home-node' => $node_name,
                                 'home-port' => $port_name);
                        }
                        log_global()->log("CREATE_LIF v4 vserver: $vserver_name node:$node_name fabric:$fabric lif:$lif_name i:$i");
                        $lif->add_parent_resources(resources => [$vs]);
                        $self->add_resources(resources => [$lif]);
                    }
                }
                if ($address_range_v6 or  $subnet_v6 ) {
                    for (my $i = 0; $i < $self->TOTAL_IPV6_ISCSI_LIFS_PER_VS_PER_NODE_PER_FABRIC; $i++) {

                        # choose a port on this fabric for this node
                        my $port_name = shift(@{$fabric_table->{$fabric}});
                        push(@{$fabric_table->{$fabric}}, $port_name);
                        my $port_id = $self->get_port_id($node_name, $port_name);

                        # make a lif on this port and node that is ipv6
                        my $lif_name = "iscsi_v6_" . $itr_v6++ . "_n" . $node_id . "_p" . $port_id;
                        my $lif;
                        if ($subnet_v6) {
                            $lif      = NACL::AppBuilder::Xgen::Resource::Lif->new(
                                vserver          => $vserver_name,
                                'data-protocol' => ['iscsi'],
                                'subnet-name'   => $subnet_v6,
                                'auto-configuration' => $subnet_v6,
                                'home-node'      => $node_name,
                                'home-port'      => $port_name,
                                'lif'            => $lif_name
                            );
                        } else {
                            $lif      = NACL::AppBuilder::Xgen::Resource::Lif->new(
                                vserver          => $vserver_name,
                                'data-protocol' => ['iscsi'],
                                'address_range'  => $address_range_v6,
                                'netmask-length' => $netmask_v6,
                                'home-node'      => $node_name,
                                'home-port'      => $port_name,
                                'lif'            => $lif_name
                            );
                        }
                        log_global()->log("CREATE_LIF v6 vserver: $vserver_name node:$node_name fabric:$fabric lif:$lif_name i:$i");
                        $lif->add_parent_resources(resources => [$vs]);
                        $self->add_resources(resources => [$lif]);
                    }
                }
            }
        }
    }
}

sub generate_igroup_resources {
    my ($self, %args) = @_;

    # Get the clusters so that we can alternate vserver creation between multiple clusters.
    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Iscsi");
    my @vservers = @{$vserver_resources};

    # Get the clusters so that we can alternate vserver creation between multiple clusters.
    my $client_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Client");
    my @clients = @{$client_resources};

    my @igroup;
    foreach my $iscsi_vserver (@vservers) {
        my ($vserver) = $iscsi_vserver->get_ancestors(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
        foreach my $client (@clients) {
            # skip clients that are not doing iSCSI
            next if (!grep("iscsi", @{$client->get_attribute('protocols')}));

            my $igroup_res = NACL::AppBuilder::Xgen::Resource::Igroup->new(vserver => $vserver->get_attribute('vserver'),
                                                                           igroup => $client->get_attribute('name'),
                                                                           protocol => "iscsi",
                                                                           ostype => $client->get_attribute('ostype') =~ /windows/ ? "windows" : $client->get_attribute('ostype'));

            # We want the vserver to be last.
            $igroup_res->add_parent_resources(resources => [$client,$vserver]);

            push(@igroup, $igroup_res);
        }
    }
    $self->add_resources(resources => \@igroup);
}

sub generate_igroup_initiator_resources {
    my ($self, %args) = @_;

    # Get the clusters so that we can alternate igroup creation between multiple clusters.
    my $igroup_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Igroup");
    my @igroups = @{$igroup_resources};

    my @igroup_initiators;
    foreach my $igroup (@igroups) {
        my $initiator_res = NACL::AppBuilder::Xgen::Resource::Igroup::Initiator->new(vserver => $igroup->get_attribute('vserver'),
                                                                                 igroup => $igroup->get_attribute('igroup'),
                                                                                 protocol => "iscsi",
                                                                                 client_name => $igroup->get_attribute('igroup'));

        # We want the vserver to be last.
        $initiator_res->add_parent_resources(resources => [$igroup]);

        push(@igroup_initiators, $initiator_res);
    }

    $self->add_resources(resources => \@igroup_initiators);
}

sub generate_lun_resources {
    my ($self, %args) = @_;

    # Get the clusters so that we can alternate vserver creation between multiple clusters.
    #my $volume_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Volume");
    my @iscsi_volumes;
    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Iscsi");
    my @vservers = @{$vserver_resources};
    foreach my $iscsi_vserver (@vservers) {
        my ($vserver) = $iscsi_vserver->get_ancestors(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
        my @new_volumes = $vserver->get_descendants(type => "NACL::AppBuilder::Xgen::Resource::Volume");
        push(@iscsi_volumes,@new_volumes);
    }
    # Filter the volumes resources by security style mixed.
    my $matches_hash = positive_filter(resources => \@iscsi_volumes,
                                       filter    => {'type' => 'DP'});

    my @volumes = @{$matches_hash->{'not_matches'}};

    my $volume_count = ceil((scalar(@volumes) * $self->PERCENTAGE_OF_VOLUMES_WITH_LUNS) / 100);
    my @luns;

    return unless ($volume_count > 0);

    for (my $i = 0; $i < $self->TOTAL_LUN_COUNT; $i++) {
        my $volume_index = $i%$volume_count;

        my $volume = $volumes[$volume_index];

        my $lun_res = NACL::AppBuilder::Xgen::Resource::Lun->new(vserver =>$volume->get_attribute('vserver'),
                                                                 volume  =>$volume->get_attribute("volume"),
                                                                 lun     => "lun$i",
                                                                 size    =>$self->LUN_SIZE);

        # connect the vs resource to the aggr's cluster parent resource
        $lun_res->add_parent_resources(resources => [$volume]);

        push(@luns, $lun_res);
    }
    $self->add_resources(resources => \@luns);
}

sub generate_lun_map_resources {
    my ($self, %args) = @_;

    # Get the clusters so that we can alternate vserver creation between multiple clusters.
    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Iscsi");
    my @vservers = @{$vserver_resources};

    my @lun_maps;
    foreach my $iscsi_vserver (@vservers) {
        my ($vserver) = $iscsi_vserver->get_ancestors(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
        my @luns = $vserver->get_descendants(type => "NACL::AppBuilder::Xgen::Resource::Lun");

        next unless (scalar @luns);

        my @igroups = $vserver->get_descendants(type => "NACL::AppBuilder::Xgen::Resource::Igroup");

        next unless (scalar @igroups);

        #(cl1,cl2)  vserver1:vol1:lun1  vserver2:vol1:lun1  might not work in this case so need better logic.
        foreach my $lun (@luns) {
            my $igroup = shift(@igroups);
            push(@igroups,$igroup);

            my $lunmap_res = NACL::AppBuilder::Xgen::Resource::LunMap->new(vserver => $lun->get_attribute('vserver'),
                                                                           lun => $lun->get_attribute('lun'),
                                                                           volume => $lun->get_attribute('volume'),
                                                                           igroup => $igroup->get_attribute('igroup'));

            # We want the vserver to be last.
            $lunmap_res->add_parent_resources(resources => [$igroup,$lun]);

            push(@lun_maps, $lunmap_res);
        }
    }
     $self->add_resources(resources => \@lun_maps);
}

sub get_subnet_info{
    my ($self, %args) = @_;
    my $node = $self->{nacl_node_objs}->[0]->node();
    my $nacl_node_obj = NACL::C::Node->new(name => $node );
    #**********************************************************
    #* 
    #**********************************************************
    my @subnets = NACL::CS::NetworkSubnet->fetch(
        command_interface => $nacl_node_obj,
        allow_empty       => 1
    );
    my $subnet_hash = {};
    foreach my $subnet (@subnets) {
        my @keys = keys(%{$subnet});
        $subnet_hash->{$subnet->{'subnet_name'}} = $subnet;
    }
    #**********************************************************
    #* 
    #**********************************************************
    my $eth_fabric_data = $args{eth_fabric_data};
    foreach my $key (%{$eth_fabric_data}) {
       my $subnet = $eth_fabric_data->{$key}->{subnet_v4};
       if( $subnet ) {
          $eth_fabric_data->{$key}->{ipspace} = $subnet_hash->{$subnet}->{ipspace};
          $eth_fabric_data->{$key}->{gateway_v4} = $subnet_hash->{$subnet}->{gateway};
          $eth_fabric_data->{$key}->{broadcast_domain} = $subnet_hash->{$subnet}->{broadcast_domain};
       }
       $subnet = $eth_fabric_data->{$key}->{subnet_v6};
       if( $subnet ) {
          $eth_fabric_data->{$key}->{ipspace} = $subnet_hash->{$subnet}->{ipspace};
          $eth_fabric_data->{$key}->{gateway_v6} = $subnet_hash->{$subnet}->{gateway};
          $eth_fabric_data->{$key}->{broadcast_domain} = $subnet_hash->{$subnet}->{broadcast_domain};
       }
    }
}

1;

__END__

=head1 NAME

PTEFIT::Xgen::Config::ONTAP::CIFS::Base

=head1 AUTHOR/MAINTAINER

=over

=item PTEFIT AUTOMATION (ng-ptefit-automation-core@netapp.com)

=back

   Last revised $Date: $ by $Author: $

=cut
