#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::Xgen::Resource::Cifs::Security;

use strict;
use warnings;
use Params::Validate qw(validate_with SCALAR ARRAYREF);

use parent qw( NACL::Service::Xgen::Resource );

use constant nacl_cs_class => 'NACL::CS::VserverCifsSecurity';

sub _validate_private_attrs_with_nacl_cs_class {
    my ($self, %args) = @_;

    # remove special args from the validation
    # or else the base class nacl cs class validation will complain
    delete($args{username});
    delete($args{password});

    $self->SUPER::_validate_private_attrs_with_nacl_cs_class(%args);
}

sub _validate_private_attrs {
    my ($self, $args) = @_;

    eval {
        $args = validate_with(
            params => [%{$args}],
            spec => {
                username => {type => SCALAR, optional => 1},
                password => {type => SCALAR, optional => 1}
            },
            allow_extra => 1);
    };
    if (my $err = $@) {
        NACL::Service::Xgen::Exceptions::InvalidAttribute->throw($err);
    }

    $args;
}

1;

__END__

=head1 NAME

NACL::AppBuilder::Xgen::Resource::Cifs::Security

=head1 AUTHOR/MAINTAINER

=over

=item PTEFIT AUTOMATION (ng-ptefit-automation-core@netapp.com)

=back

   Last revised $Date: $ by $Author: $

=cut
