#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::Xgen::Resource::ClientCluster::Node;

use strict;
use warnings;

use Data::Dumper;

use Hostrec;

use parent qw( NACL::AppBuilder::Xgen::Resource::Client );

use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NACL::Service::Xgen::Exceptions::InvalidAttribute;

sub new {
    my ($class, %args) = @_;

    my $self = $class->SUPER::new(%args);

    return $self;
}


sub _validate_private_attrs {
    my ($self, $args) = @_;

    eval {
        $args = validate_with(
            params => [%{$args}],
            spec => {
                name              => {type => SCALAR},
                protocols         => {type => ARRAYREF},
                synthetic_client  => {type => SCALAR, default => 0},
                ostype            => {type => SCALAR, optional => 1},
                virtual_machines  => {type => ARRAYREF, default => []},
                vma_host          => {type => SCALAR, optional => 1},
                is_hypervisor     => {type => SCALAR, optional => 1, default => 0},
                mp_type           => {type => SCALAR, optional => 1},
                is_primary        => {type => SCALAR, optional => 1, default => 0},
                client_cluster    => {type => SCALAR, optional => 1, default => ""},
            });
    };
    if (my $err = $@) {
        NACL::Service::Xgen::Exceptions::InvalidAttribute->throw($err);
    }

    return $args;
}

1;

__END__

=head1 NAME

NACL::AppBuilder::Xgen::Resource::ClientCluster::Node

=head1 AUTHOR/MAINTAINER

=over

=item PTEFIT AUTOMATION (dl-ptefit-automation-core@netapp.com)

=back

   Last revised $Date: $ by $Author: $

=cut
