#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
package NACL::AppBuilder::Xgen::Resource::Port;

use strict;
use warnings;

use Data::Dumper;

use NACL::C::Node;
use NACL::CS::NetworkFcpAdapter;
use NACL::CS::NetworkPort;

use NATE::Log qw( log_global );

use parent qw( NACL::Service::Xgen::Resource );

sub generate_port_info {
    my ($pkg, %args) = @_;

    my $nacl_node_objs = delete($args{nacl_node_objs})
                    // [NACL::C::Node->find()];
    my $ignore_downed_ports = delete($args{ignore_downed_ports}) 
                    // 1;

    my %return = ();

    # fetch ethernet ports
    foreach my $nacl_node_obj (@{$nacl_node_objs}) {
        my @ports = NACL::CS::NetworkPort->fetch(command_interface => $nacl_node_obj,
                                                 filter => {node => $nacl_node_obj->node()},
                                                 allow_empty => 1);

        # removing node mgmt and cluster interconnect ports from list
        @ports = grep {
            my $port = $_;

            # remove node mgmt port
            if ($port->port() eq "e0M") {
                0;
            } else {
                my $type = "";
                eval {
                    $type = $port->ipspace();
                };

                if ($@ or not $type) {
                    eval {
                        $type = $port->role();
                    };
                }

                # remove cluster interconnect port
                if ($type eq "Cluster" or $type eq "cluster") {
                    0;
                } else {
                    if ($port->link() =~ /off|down/i) {
                        if ($ignore_downed_ports) {
                            log_global()->warn("Port '" . $port->node() . " : " . $port->port() . 
                                               "' is down or off, not registering it as a resource.");
                            0;
                        } else {
                            log_global()->warn("Port '" . $port->node() . " : " . $port->port() . 
                                               "' is down or off, but will continue to register it as a resource.");
                            1;
                        }
                    } else {
                        1;
                    }
                }
            }
         } @ports;

        push(@{$return{$nacl_node_obj->node()}->{eth}}, @ports);
    }

    # fetch fibre channel ports
    foreach my $nacl_node_obj (@{$nacl_node_objs}) {
        my @ports = NACL::CS::NetworkFcpAdapter->fetch(command_interface => $nacl_node_obj,
                                                       filter => {node => $nacl_node_obj->node()},
                                                       allow_empty => 1);

        # separate ports into fcoe and fcp
        foreach my $port_obj (@ports) {
            if ($port_obj->status() ne 'online') {
                if ($ignore_downed_ports) {
                    log_global()->warn("Port '" . $port_obj->node() . " : " . $port_obj->adapter() . 
                                       "' is not online, not registering it as a resource");
                    next;
                } else {
                    log_global()->warn("Port '" . $port_obj->node() . " : " . $port_obj->adapter() . 
                                       "' is not online, but will continue to register it as a resource");                    
                }
            }
            if ($port_obj->physical_protocol() eq 'ethernet') {
                push(@{$return{$nacl_node_obj->node()}->{fcoe}}, $port_obj);
            } else {
                push(@{$return{$nacl_node_obj->node()}->{fcp}}, $port_obj);
            }
        }
    }

    return \%return;
}

1;

__END__

=head1 NAME

NACL::AppBuilder::Xgen::Resource::Port

=head1 AUTHOR/MAINTAINER

=over

=item PTEFIT AUTOMATION (dl-ptefit-automation-core@netapp.com)

=back

   Last revised $Date: $ by $Author: $

=cut
