#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::Xgen::Resource::SecurityLogin;

use strict;
use warnings;

use Params::Validate qw(validate_with SCALAR);
use NACL::Service::Xgen::Exceptions::InvalidAttribute;
use NATE::Log qw(log_global);

use parent qw( NACL::Service::Xgen::Resource );

use constant nacl_cs_class => 'NACL::CS::SecurityLogin';

sub _validate_private_attrs_with_nacl_cs_class {
    my ($self, %args) = @_;
    log_global()->enter();

    delete($args{password});

    $self->SUPER::_validate_private_attrs_with_nacl_cs_class(%args);
    log_global()->exit();
}

sub _validate_private_attrs {
    my ($self, $args) = @_;
    log_global()->enter();

    eval {
        $args = validate_with(
            params => [%{$args}],
            spec => {
                password => {type => SCALAR, optional => 1},
            },
            allow_extra => 1);
    };
    if (my $err = $@) {
        NACL::Service::Xgen::Exceptions::InvalidAttribute->throw($err);
    }

    log_global()->exit();
    return $args;
}

1;

__END__

=head1 NAME

NACL::AppBuilder::Xgen::Resource::SecurityLogin

=head1 AUTHOR/MAINTAINER

=over

=item PTEFIT AUTOMATION (ng-ptefit-automation-core@netapp.com)

=back

   Last revised $Date: $ by $Author: $

=cut
