#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::Xgen::Resource::VirtualMachine;

use strict;
use warnings;

use Data::Dumper;

use Hostrec;

use parent qw( NACL::AppBuilder::Xgen::Resource::Client );

use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NACL::Service::Xgen::Exceptions::InvalidAttribute;

sub _validate_private_attrs {
    my ($self, $args) = @_;

    eval {
        $args = validate_with(
            params => [%{$args}],
            spec => {
                name               => {type => SCALAR},
                hypervisor_type    => {type => SCALAR,   default => ""},
                hypervisor_name    => {type => SCALAR,   default => ""},
                is_hypervisor      => {type => SCALAR,   default => 0},
                protocols          => {type => ARRAYREF, default => ["iscsi"]},
                synthetic_client   => {type => SCALAR,   default => 0},
                ostype             => {type => SCALAR,   optional => 1},
                mp_type            => {type => SCALAR,   default => "none"},
                virtual_disks      => {type => SCALAR,   default => 0},
                raw_disks          => {type => SCALAR,   default => 0},
                iscsi_vvol_disks   => {type => SCALAR,   default => 0},
                fcp_vvol_disks     => {type => SCALAR,   default => 0},
                nfs_vvol_disks     => {type => SCALAR,   default => 0},
                raw_lun_path_regex => {type => SCALAR,   default => "qr/.*/"},
                scsi_bus           => {type => SCALAR,   default => "lsilogic"},
                controller_type    => {type => SCALAR,   default => "scsi"},
            });
    };
    if (my $err = $@) {
        NACL::Service::Xgen::Exceptions::InvalidAttribute->throw($err);
    }

    return $args;
}

1;

__END__

=head1 NAME

NACL::AppBuilder::Xgen::Resource::VirtualMachine

=head1 AUTHOR/MAINTAINER

=over

=item PTEFIT AUTOMATION (dl-ptefit-automation-core@netapp.com)

=back

   Last revised $Date: $ by $Author: $

=cut
