
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary AntivirusEngine Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::AntivirusEngine

=head1 DESCRIPTION

C<NACL::C::AntivirusEngine> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. Unlike other components this
package does not model any element/service but is present to provide
component-level support for functionality.
However, since it does not have any primary keys, 'find' and 'state' do not
work for this component. (This component is considered a 'singleton',
i.e. there is only one row shown by the relevant 'show' command).
To access the state, use L<NACL::CS::AntivirusEngine|lib-NACL-CS-AntivirusEngine-pm>.

=cut

package NACL::C::AntivirusEngine;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars Dumper);
use NACL::CS::AntivirusEngine;

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::Job;

=head2 enable

   NACL::C::AntivirusEngine->enable(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to installs a new scan engine.

Uses a CMode CLI/ZAPI APISet.

Invokes 'av-set-engine-info' api with 'is-enabled' option as true for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vendor => $string >>

The antivirus vendor. Can be any of "none", "sophos" or "mcafee".

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: vendor.

=item C<< "method-timeout" => $timeout_in_seconds >>

This method takes more than 60 seconds to run. This option can be used to
specify a larger timeout value. Note that the component automatically
assigns a default of 1200 seconds for this method, so if the user does
not provide a timeout value it will use this default.

=item C<< "job_component"=> \$scalar >>

=item C<< '*' >>

This method takes all other options supported by
"antivirus engine enable" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub enable {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            job_component => {type => SCALARREF, optional => 1},
            _optional_scalars(qw(vendor)),
        },
        ignore_primary_keys => 1,
        api                 => 'antivirus_engine_enable',
    );

    # antivirus engine enable takes more than 60 seconds to run
    # We're setting a default of 1200 seconds here
    $opts{'method-timeout'} ||= 1200;

    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_enable_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_enable_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::AntivirusEngine::enable',
    );

    $Log->exit() if $may_exit;
} ## end sub enable

sub _enable_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $apiset            = delete $opts{apiset};
    my $command_interface = delete $opts{command_interface};
    my $job_component     = delete $opts{job_component};

    my $prompts_answers = ['{yes\|no\|text}:' => "yes",];

    my %cmode_args = $pkg->get_api_options(
        apiset  => $apiset,
        api     => 'antivirus_engine_enable',
        options => \%opts,
    );

    my $response = $apiset->antivirus_engine_enable(%cmode_args,
        'connectrec-match_table' => $prompts_answers);
    my $jobid = $response->get_job_ids();
    if ($jobid) {
        $$job_component = NACL::C::Job->new(
            id                => $jobid,
            command_interface => $command_interface
        );
    } ## end if ($jobid)

    $Log->exit() if $may_exit;
} ## end sub _enable_cmode_cli

sub _enable_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $apiset            = $opts{apiset};
    my $command_interface = $opts{command_interface};

    my $allowed_cmode_options = [qw( vendor )];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $apiset->av_set_engine_info(%cmode_args, 'is-enabled' => 'true');
    $Log->exit() if $may_exit;
} ## end sub _enable_cmode_zapi

=head2 disable

   NACL::C::AntivirusEngine->disable(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to uninstalls the installed scan engine.

Uses a CMode CLI/ZAPI APISet.

Invokes 'av-set-engine-info' api with 'is-enabled' option as false for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< '*' >>

This method takes all other options supported by
"antivirus engine disable" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub disable {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        api    => 'antivirus_engine_disable',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_disable_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_disable_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::AntivirusEngine::disable',
    );

    $Log->exit() if $may_exit;
} ## end sub disable

sub _disable_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %cmode_args = $pkg->get_api_options(
        apiset  => $apiset,
        api     => 'antivirus_engine_disable',
        options => \%opts,
    );

    $apiset->antivirus_engine_disable(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _disable_cmode_cli

sub _disable_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $apiset = $opts{apiset};

    $apiset->av_set_engine_info('is-enabled' => 'false');
    $Log->exit() if $may_exit;
} ## end sub _disable_cmode_zapi

=head2 modify

   NACL::C::AntivirusEngine->modify(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to configures the scan engine.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "mcafee-license-key" => $string >>

McAfee License Key

Applicable for CMode CLI

=item C<< "sophos-http-url-amd64" => $string >>

The URL from which sophos updates are downloaded from for AMD 64-bit machines
(This field appears to only be valid for very early RR builds, and has since
been replaced by sophos_http_url))

Applicable for CMode CLI

=item C<< "sophos-http-url-intel32" => $string >>

The URL from which to download sophos updates.
(This could be a URL or could just be "SOPHOS" which will internally
get automatically mapped into the appropriate URL)

Applicable for CMode CLI

=item C<< "sophos-http-url" => $string >>

The URL from which to download sophos updates.
(This could be a URL or could just be "SOPHOS" which will internally
get automatically mapped into the appropriate URL)

Applicable for CMode CLI

=item C<< "num-license" => $string >>

Number of licenses

Applicable for CMode CLI

=item C<< "sophos-http-user" => $string >>

The Sophos HTTP Username

Applicable for CMode CLI

=item C<< "sophos-public-key" => $string >>

Applicable for CMode CLI

=item C<< "mcafee-http-url" => $string >>

The URL from which to download McAfee updates.

Applicable for CMode CLI

=item C<< mcafee_license_key >>

McAfee License Key

Applicable for CMode CLI

=item C<< "mcafee-license-url" => $string >>

URL from which to get the McAfee license

Applicable for CMode CLI

=item C<< "mcafee-node-id" => $string >>

Applicable for CMode CLI

=item C<< "mcafee-product-id" => $string >>

Applicable for CMode CLI

=item C<< vendor => $string >>

The antivirus vendor. Can be any of "none", "sophos" or "mcafee"

Applicable for CMode CLI

=item C<< password => $string >>

For CLI this answers the prompts "Please enter password:" and
"Please enter password again:".

Applicable for CMode CLI

=item C<< 'trend-http-url' => $string >>

Applicable for CMode CLI

=item C<< 'trend-activation-code' => $string >>

Applicable for CMode CLI

=item C<< 'sophos-http-url-amd64' => $string >>

Applicable for CMode CLI

=item C<< 'sophos-public-key' => $string >>

Applicable for CMode CLI

=item C<< 'sophos-http-url-intel32' => $string >>

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"antivirus engine modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw(mcafee-http-url mcafee-license-key mcafee-license-url
                    mcafee-node-id mcafee-product-id num-license sophos-http-url
                    sophos-http-user password vendor

                    trend-http-url trend-activation-code sophos-http-url-amd64
                    sophos-public-key sophos-http-url-intel32
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'antivirus_engine_modify',
    );

    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::AntivirusEngine::modify',
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset   = delete $opts{apiset};
    my $password = delete $opts{'password'};

    my $prompts_answers = [
        'Please enter password:'       => sub {$password},
        'Please enter password again:' => sub {$password},
        'Enter the password:'          => sub {$password},
        'Enter the password again:'    => sub {$password},
    ];

    my %cmode_args = $pkg->get_api_options(
        apiset  => $apiset,
        api     => 'antivirus_engine_modify',
        options => \%opts,
    );

    $apiset->antivirus_engine_modify(%cmode_args,
        'connectrec-match_table' => $prompts_answers);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

=head2 set

    NACL::C::AntivirusEngine->set(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to set the Target Nodes for Antivirus Engine.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'target-nodes' => [ $target-node1, $target-node2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"antivirus engine set" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub set {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'target-nodes'       => {type => ARRAYREF,  optional => 1},
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
        },
        api => 'antivirus_engine_set',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_set_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::AntivirusEngine::set',
    );

    $Log->exit() if $may_exit;
} ## end sub set

sub _set_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset         = $opts{apiset};
    my $execution_time = $opts{cmd_execution_time};
    my %cmode_args     = $pkg->get_api_options(
        apiset  => $apiset,
        api     => 'antivirus_engine_set',
        options => \%opts,
    );

    my $response = $apiset->antivirus_engine_set(%cmode_args);
    if (defined $execution_time) {
        $$execution_time = $response->get_execution_time();
    }

    $Log->exit() if $may_exit;
} ## end sub _set_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ();
}

1;
