
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary AntivirusRemedy Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::AntivirusRemedy

=head1 DESCRIPTION

C<NACL::C::AntivirusRemedy> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP AntivirusRemedy.  A related class is
L<NACL::CS::AntivirusRemedy|lib-NACL-CS-AntivirusRemedy-pm>, which represents
the past state of an ONTAP AntivirusRemedy.
However, since it does not have any primary keys, 'find' and 'state' do not
work for this component. (This component is considered a 'singleton',
i.e. there is only one row shown by the relevant 'show' command)

=cut

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=cut

package NACL::C::AntivirusRemedy;

use strict;
use warnings;

use NACL::CS::AntivirusRemedy;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $AntivirusRemedy = NACL::C::AntivirusRemedy->new(command_interface=>$ci,
                                  );

Return a new AntivirusRemedy component object with the given attributes.

=cut

=head2 modify

   NACL::C::AntivirusRemedy->modify(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to modifies remedy configuration.
Uses a CMode CLI or CMode ZAPI APISet.
Invokes 'av-set-remedy-info' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "quarantine-action" => $string >>

Applicable for CMode CLI/ZAPI
Maps to:
    CM ZAPI : option

=item C<< "quarantine-ext" => $string >>

Applicable for CMode CLI/ZAPI
Maps to:
    CM ZAPI : extension

=item C<< "failed-repair-action" => $string >>

Applicable for CMode CLI/ZAPI
Maps to:
    CM ZAPI : repair

=item C<< "quarantine-dir" => $string >>

Applicable for CMode CLI/ZAPI
Maps to:
    CM ZAPI : directory

=item C<< "action" => $string >>

Applicable for CMode CLI/ZAPI
Maps to:
    CM ZAPI : action

=item C<< 'extended_query' => $string >>

(Optional)
Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"antivirus remedy modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( quarantine-action quarantine-ext failed-repair-action quarantine-dir
                    action
                    )
            ),
        },
        api => 'antivirus_remedy_modify',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::AntivirusRemedy::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my %cmode_args = $pkg->get_api_options(
        apiset  => $apiset,
        api     => 'antivirus_remedy_modify',
        options => \%opts,
    );

    $apiset->antivirus_remedy_modify(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [qw( action  extended_query )];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        map    => {
            "quarantine-action"    => "option",
            "quarantine-ext"       => "extension",
            "quarantine-dir"       => "directory",
            "failed-repair-action" => "repair",
        },
        target => \%cmode_args,
    );
    %opts = %cmode_args;
    $apiset->av_set_remedy_info(%opts);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi


__PACKAGE__->_load_event_class();

1;
