# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Application Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Application

=head1 DESCRIPTION

C<NACL::C::Application> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP Application.  A related class is
L<NACL::CS::Application|lib-NACL-CS-Application-pm>, which represents
the past state of an ONTAP Application.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

The name of the vserver, used to identify it uniquely.

=head2 application

=cut

package NACL::C::Application;

use strict;
use warnings;

use NACL::CS::Application;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use feature 'state';

=head1 METHODS

=head2 new

  my $Application = NACL::C::Application->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
                                   'application' => $application,
                                  );

Return a new Application component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'application',
];

=head2 find

    my $Application_obj  = NACL::C::Application->find(command_interface => $ci, ...);
    my @Application_objs = NACL::C::Application->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my Application_state = $Application_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::Application would return
a NACL::CS::Application object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 create

    my $Application_obj = NACL::C::Application->create(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'application' => $application,
                                ...);

(Class method)
This method is used to Create an application.
Uses CMode CLI/ZAPI .
Invokes  application-create-async  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required)
Vserver
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'application' => $application >>

(Required)
Application
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'application-template' => $string >>

(Required)
Application Template
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'foreground' => $string >>

Execute in the Foreground
Applicable for CMode CLI, non-iter.

=item C<< 'parameters' => [ $parameter1, $parameter2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "application create" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    $pkg->_frontend_log_debug_opts(@args);
    state $additional_spec = {
        additional_spec => {
            'parameters' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( foreground )
            ),
        }
    };
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        %{$additional_spec},
        cli_cmd => 'application create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_create_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        nacl_exceptions => [qw(AlreadyExists)],
        frontend        => 'NACL::C::Application::create',
    );

    my %obj_opts = (
        command_interface => $opts{command_interface},
        vserver           => $vserver,
        application       => $opts{'application'},
    );
    my $obj = $pkg->new(%obj_opts);
    $Log->exit() if $may_exit;
    return $obj;
}

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'application_create',);
    $Log->exit() if $may_exit;
}

sub _create_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    if (defined($opts{parameters})) {
        my @pairs;
        my @params = @{delete($opts{parameters})};
        foreach my $param (@params) {
            if ($param =~ /(.*):(.*)/) {
                my %pair = (
                    name => $1,
                    value => $2,
                );
                push(@pairs, \%pair);
            }
        }
        $opts{parameters} = \@pairs;
    }
    delete($opts{cli_cmd});
    my $copy = [qw(application application-template parameters)];
    $pkg->_base_backend_cmode_zapi(
        %opts, 
        api => 'application-create-async',
        copy => $copy
    );
    $Log->exit() if $may_exit;
}

=head2 destroy

    NACL::C::Application->destroy(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'application' => $application,
                                ...);

                                 or

    $Application_obj->destroy(...);

(Class or instance method)
This method is used to destroy an application.
Uses CMode CLI/ZAPI .
Invokes  application-destroy  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'application' => $application >>

(Required for class method, Not Applicable for instance method)
Application Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'vserver-str' => $string >>

Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "application destroy" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub destroy {
    $Log->enter() if  $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    state $additional_spec = {
        additional_spec => {
            _optional_scalars(
                qw( vserver-str
                    )
            ),
        }
    };
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        %{$additional_spec},
        cli_cmd => 'application delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_destroy_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_destroy_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_check_destroy_cmode_zapi_non_iter',
            },
        ],
        frontend => 'NACL::C::Application::destroy',
    );

    $Log->exit() if $may_exit;
}

sub _destroy_cmode_cli {
    $Log->enter() if  $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'application_delete',);
    $Log->exit() if $may_exit;
}

sub _destroy_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(application)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'application-delete-async',
        copy => $copy,
    );
    $Log->exit() if $may_exit;;
}

sub _check_destroy_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,

        _primary_keys => [qw(vserver application)],
        _vs_context   => 1

    );
    my @invalid_fields = ();
    ## following are the CLI-only fields
    my @cli_only_fields = qw(vserver-str);
    foreach my $field (@cli_only_fields) {
        push @invalid_fields, $field if (grep (/^$field$/, @opts));
    }
    if (@invalid_fields) {
        my $msg
            = 'The following fields provided '
            . 'in the call to '
            . $pkg
            . ' are not supported by the '
            . 'non-iter ZAPI, so the ZAPI back-end is being '
            . "skipped:
"
            . join(', ', @invalid_fields);
        $Log->comment($msg);
        $Log->exit() if $may_exit;
        NACL::Exceptions::InvalidChoice->throw($msg);
    }
    $Log->exit() if $may_exit;
}

=head2 prepare_to_downgrade

    NACL::C::Application->prepare_to_downgrade(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to clear all application related configuration information..
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< '*' >>

This method takes all other options supported by "application prepare to downgrade" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub prepare_to_downgrade {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        cli_cmd             => 'application prepare-to-downgrade',
    );
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_prepare_to_downgrade_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        frontend => 'NACL::C::Application::prepare_to_downgrade',
    );

    $Log->exit() if $may_exit;
}

sub _prepare_to_downgrade_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'application_prepare_to_downgrade',);
    $Log->exit() if $may_exit;
}

=head2 rename

    NACL::C::Application->rename(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'application' => $application,
                                ...);

                                 or

    $Application_obj->rename(...);

(Class or instance method)
This method is used to rename an application.
Uses CMode CLI/ZAPI .
Invokes  application-rename  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'application' => $application >>

(Required for class method, Not Applicable for instance method)
Application Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'new-name' => $string >>

New application name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'vserver-str' => $string >>

Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "application rename" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub rename {
    $Log->enter() if  $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    state $additional_spec = {
        additional_spec => {
            _optional_scalars(
                qw( new-name vserver-str
                    )
            ),
        }
    };
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        %{$additional_spec},
        cli_cmd => 'application rename',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_rename_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_rename_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::Application::rename',
    );

    $Log->exit() if $may_exit;
}

sub _rename_cmode_cli {
    $Log->enter() if  $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts, api => 'application_rename',);
    $Log->exit() if $may_exit;
}

sub _rename_cmode_zapi_non_iter {
    $Log->enter() if  $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(new-name application)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'application-rename',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}


=head2 show_detail

    my $show_detail_obj = NACL::C::Application->show_detail(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

    my @show_detail_objs = NACL::C::Application->show_detail(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $show_detail_obj = $Application_obj->show_detail();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of
L<fetch|lib-NACL-ComponentState-pm/fetch> (requested_fields, filter, allow_empty)
and has all of the other properties of fetch methods (callable in scalar or list
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-ApplicationShowDetail-pm/ATTRIBUTES>
of NACL::CS::ApplicationShowDetail as input.
This method returns objects of the type L<NACL::CS::ApplicationShowDetail|lib-NACL-CS-ApplicationShowDetail-pm>.

=cut

sub show_detail {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::ApplicationShowDetail;
    my @state_objs = NACL::CS::ApplicationShowDetail->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 modify

    NACL::C::Application->modify(
                                command_interface => $ci,
                                    'vserver' => $vserver,
                                    'application' => $application,
                                ...);

                                                                 or

    $Application_obj->modify(...);

(Class or instance method)
This method is used to (DEPRECATED)-Modify properties of an application.
Uses CMode CLI/ZAPI .
Invokes  application-modify-async  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'application' => $application >>

(Required for class method, Not Applicable for instance method)
Application
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'dry-run' => $string >>

Skip the Job
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'storage-service' => $string >>

Storage Service
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'application-template' => $string >>

Application Modify Template
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'foreground' => $string >>

Execute in the Foreground possible value(s) are, true,false
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'parameters' => [ $parameter1, $parameter2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'application-components' => [ $application-component1, $application-component2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "application modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    state $additional_spec = {
        additional_spec => {
            'parameters'             => {type => ARRAYREF, optional => 1},
            'application-components' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( dry-run storage-service application-template foreground 
                 )
            ),
        }
    };
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        %{$additional_spec},
        cli_cmd => 'application modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::Application::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts, api => 'application_modify',);
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $copy = [qw(storage-service application application-components application-template)];
    my $api = 'application-modify-async';

    if($opts{'application-template'}) {
        $api = 'application-modify-via-template-async';
        push @$copy, ('application-template');
        if (defined($opts{parameters})) {
        my @pairs;
        my @params = @{delete($opts{parameters})};
        foreach my $param (@params) {
            if ($param =~ /(.*):(.*)/) {
                my %pair = (
                    name => $1,
                    value => $2,
                );
                push(@pairs, \%pair);
            }
        }
        $opts{parameters} = \@pairs;
        push @$copy, ('parameters');
        }       
    }

    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api  => $api,
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver'     => {type => SCALAR},
        'application' => {type => SCALAR},
    );
}

1;
