# $Id$

#
# Copyright (c) 2001-2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ApplicationVolume Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::ApplicationVolume

=head1 DESCRIPTION

C<NACL::C::ApplicationVolume> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP ApplicationVolume.  A related class is
L<NACL::CS::ApplicationVolume|lib-NACL-CS-ApplicationVolume-pm>, which represents
the past state of an ONTAP ApplicationVolume.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 volume

=head2 vserver

=head2 application

=cut

package NACL::C::ApplicationVolume;

use strict;
use warnings;

use NACL::CS::ApplicationVolume;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use feature 'state';

=head1 METHODS

=head2 new

  my $ApplicationVolume = NACL::C::ApplicationVolume->new(
                                   command_interface => $ci,
                                   'volume' => $volume,
                                   'vserver' => $vserver,
                                   'application' => $application,
								  );

Return a new ApplicationVolume component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'volume',
    scalar => 'vserver',
    scalar => 'application',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $ApplicationVolume_obj  = NACL::C::ApplicationVolume->find(command_interface => $ci, ...);
	my @ApplicationVolume_objs = NACL::C::ApplicationVolume->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my ApplicationVolume_state = $ApplicationVolume_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::ApplicationVolume would return 
a NACL::CS::ApplicationVolume object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 add

    NACL::C::ApplicationVolume->add(
                                command_interface => $ci,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
	                            'application' => $application,
                                ...);
	                             
								 or

    $ApplicationVolume_obj->add(...);

(Class or instance method)
This method is used to add a volume to an application.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)
Volume Name
Applicable for CMode CLI

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI

=item C<< 'application' => $application >>

(Required for class method, Not Applicable for instance method)
Application Name
Applicable for CMode CLI

=item C<< 'application-str' => $string >>

Applicable for CMode CLI

=item C<< 'vserver-str' => $string >>

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "application volume add" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    state $additional_spec = {
        additional_spec => {
            _optional_scalars(
                qw( application-str vserver-str
                    )
            ),
        }
    };
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        %{$additional_spec},
        cli_cmd => 'application volume add',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_add_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        frontend => 'NACL::C::ApplicationVolume::add',
    );

    $Log->exit() if $may_exit;
}

sub _add_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'application_volume_add',);
    $Log->exit() if $may_exit;
}

=head2 remove

    NACL::C::ApplicationVolume->remove(
                                command_interface => $ci,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
	                            'application' => $application,
                                ...);
	                             
								 or

    $ApplicationVolume_obj->remove(...);

(Class or instance method)
This method is used to remove a volume from an application.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)
Volume Name
Applicable for CMode CLI

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI

=item C<< 'application' => $application >>

(Required for class method, Not Applicable for instance method)
Application Name
Applicable for CMode CLI

=item C<< 'application-str' => $string >>

Applicable for CMode CLI

=item C<< 'vserver-str' => $string >>

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "application volume remove" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub remove {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    state $additional_spec = {
        additional_spec => {
            _optional_scalars(
                qw( application-str vserver-str
                    )
            ),
        }
    };
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        %{$additional_spec},
        cli_cmd => 'application volume remove',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_remove_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        frontend => 'NACL::C::ApplicationVolume::remove',
    );

    $Log->exit() if $may_exit;
}

sub _remove_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'application_volume_remove',);
    $Log->exit() if $may_exit;
}
__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'volume'      => {type => SCALAR},
        'vserver'     => {type => SCALAR},
        'application' => {type => SCALAR},
    );
}

1;
