#
#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary Upgrade Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here
################################################################################

=pod

=head NAME

NACL::C::Client::ONTAPSelectDeploy::Upgrade

=for html <hr>

=head DESCRIPTION

This library's purpose is to aid automation of workflows around the ONTAP Select Deploy Utility
by providing methods for upgrade operations, such as upgrade,

=for html <hr>

=head SYNOPSIS

Upgrade ONTAP Select Deploy:
NACL::C::Client::ONTAPSelectDeploy::Upgrade->upgrade(
    command_interface => $ci,
    package_path => "package_path",
    ignore_degraded_clusters => "ignore_degraded_clusters",
);

=cut

package NACL::C::Client::ONTAPSelectDeploy::Upgrade;

use Exporter qw(import);
@EXPORT_OK = qw();

use strict;
use warnings;

use Options;
use Params::Validate qw(:all);
use NATE::BaseException;
use base 'NACL::C::Component::Client';
use NACL::ComponentUtils qw(_optional_scalars);

use Class::MethodMaker [
];

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
###############################################################################
#           Upgrade Operation methods
###############################################################################

=head upgrade

Upgrade ONTAP Select Deploy

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)

=item C<< package_path => SCALAR >>

 (Required) Package Path for Upgrade

=item C<< ignore_degraded_clusters => SCALAR >>

 (Optional) Force upgrade Deploy

=back

=item Returns

None for success, throws an exception on failure.

=item Examples

# Upgrade ONTAP Select Deploy
NACL::C::Client::ONTAPSelectDeploy::Upgrade->upgrade(
    command_interface => $ci,
    package_path => "package_path",
    ignore_degraded_clusters => "ignore_degraded_clusters",
);

=back

=cut

sub upgrade {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
	additional_spec => {
	    package_path	=> { type => SCALAR },
            _optional_scalars(
		qw( ignore_degraded_clusters )
	    ),
	    help		=> { type => BOOLEAN, optional => 1 },
	},
        allow_extra => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_upgrade",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub upgrade

sub _upgrade {
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
	api => 'upgrade' );
    $Log->exit() if $may_exit;
} ## end sub _upgrade

1;
