# Hey emacs, this is basically -* perl -*-
#
# $Id: //depot/prod/test/nacldev/lib/NACL/C/Client/Ping.pm#1 $
#
#
# Copyright (c) 2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary Ping Component Module
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here
################################################################################

=head1 NAME

NACL::C::Client::Ping

=head1 DESCRIPTION

C<NACL::C::Client::Ping> Is a derived class of
L<NACL::C::Component|lib-NACL-C-Component-pm>. It represents ping 
& its attributes.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the Ping
element are the attributes of the Ping ComponentState.

=over

=back

=head2 destination  

The destination can be hostname or ipaddress.

=head2 command_interface

See L<NACL::C::Client::command_interface|lib-NACL-C-Client-pm/command_interface>

=cut

###################
# Package
package NACL::C::Client::Ping;

###################
# Everytime
use strict;
use warnings;

###################
# Module includes
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::Client';
use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NACL::APISet::Exceptions::ResponseException qw/:try/;
use NACL::CS::Client::Ping;
use NACL::C::UnitNormalization;
use NACL::CS::ComponentState::Client;
use Net::IP qw(ip_is_ipv6);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

    my $state = NACL::C::Client::Ping->new( command_interface => $ci );

Return a new ping component state object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'destination',
];

=head2 ping

    $ping_CS_obj = NACL::C::Client::Ping->ping(command_interface=>$ci,
                                destination => $destination,
                                count => '5',
                                );

(Class method)
This method is used to ping and returns a CS like object. Throws a
verifyfailure exception when the host is not alive.
Uses a Windows or Linux or Solaris CLI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< destination => $string >>

(Required) The destination can be hostname or ipaddress.

=item C<< count  => $string >>

(Optional) The no of packets

=item C<< tos  => $string >>

(Optional) Type of service
For Windows & Linux, IPV4 only.

=item C<< time-to-live => $string >>

(Optional) Set the IP Time to Live.

=item C<< audible => $boolean >>

(Optional) For each packet sent, the beep sound will be heared. 
Used for Linux only.

=item C<< adaptive => $boolean >>

(Optional) Interpacket interval adapts to round-trip time,
so that effectively not more than one unanswered
probes present in the network
Used for Linux only. 

=item C<< allow-broadcast => $boolean >>

(Optional) Allow pinging a broadcast address.
Used for Linux only.

=item C<< bind-source => $boolean >>

(Optional) Do not allow ping to change source address of probes.
Used for Linux only.

=item C<< so-debug => $boolean >>

(Optional) Set the SO_DEBUG option on the socket being used.
Used for Linux & Solaris.

=item C<< flow-label => $boolean >>

(Optional) Allocate and set flow label on echo request packets.
Used for ipv6 only.
Used for Linux & Solaris.

=item C<< record-route => $boolean >>

(Optional) Includes the RECORD_ROUTE option in the ECHO_REQUEST
packet and displays the route buffer on returned  packets.
For solaris, -s & -v options should be used along with this option. 
Used for ipv4 only.

=item C<< packet-size => $string|$boolean >>

(Optional) 
For Linux, Specifies  the  number  of  data  bytes to be sent. 
For Solaris, Send one datagram per second.
Used for Linux & Solaris.

=item C<< flood-ping => $boolean >>

(Optional) For every ECHO_REQUEST sent a period "." is printed,
while for ever ECHO_REPLY received a backspace is printed.
Used for Linux only.

=item C<< supress-loopback-multicast => $boolean >>

(Optional) Suppress loopback of multicast packets.
This applies if the ping destination is a multicast address.
Used for Linux & Solaris.

=item C<< numeric => $boolean >> 

(Optional) Numeric output only.
Used for Linux & Solaris.

=item C<< quiet => $boolean >> 

(Optional) Quiet output.
Nothing is displayed except the summary lines
at startup time and when finished.
Used for Linux only.

=item C<< bypass-routing-tables => $boolean >>

(Optional) Bypass  the  normal  routing tables and
send directly to a host on an attached interface.
If the host is not on a directly-attached network,
an error is returned.
Used for Linux & Solaris.

=item C<< user-to-user-latency => $boolean >>

(Optional) Print full user-to-user latency
Used for Linux only.

=item C<< verbose => $boolean >> 

(Optional) Verbose  output.
List any ICMP(ICMP6)  packets.
Used for Linux & Solaris.

=item C<< preload => $string >>

(Optional) ping sends that many packets not waiting for reply.
Only the super-user may select preload more than 3.
Used for Linux only.

=item C<< pattern => $string >>

(Optional) You  may specify up to 16 bytes to fill out the packet you send.
-p ff will cause the sent packet to be filled with all ones.
Used for Linux only.

=item C<< hint => $string >>

(Optional) Select  Path  MTU  Discovery strategy.
The value for hint may be either do,want or dont
Used for Linux only.

=item C<< sndbuf => $string >>

(Optional) Set socket sndbuf.
If not specified, it is selected to buffer not more than one packet.
Used for Linux only.

=item C<< timestamp => $string >>

(Optional) 
For Linux, set special IP timestamp options.
timestamp option may be either tsonly, tsandaddr or tsprespec
For Windows, timestamp for count hops. 
The value from 1 to 4.
Used for Linux & Windows only.

=item C<< version => $string >>

(Optional) Show version and exit.
Used for Linux only.

=item C<< deadline => $string >>

(Optional) Specify a timeout, in seconds,
before ping exits regardless of how many packets have been sent or received.
ping does not stop after count packet are sent.
Used for Linux only.

=item C<< timeout => $string >>

(Optional) Time  to wait for a response, in seconds.
The option affects only timeout in absense of any responses,
otherwise ping waits for two RTTs.
Used for Linux & Windows only.

=item C<< hop => $string >>

(Optional) The hops to be specified to reach the destination.
Used for Linux only.

=item C<< udp-packets => $string >>

(Optional) Send UDP  packets  instead  of  ICMP packets
Used for Solaris only.

=item C<< all-addresses => $string >>

(Optional) ping all addresses, both IPv4 and IPv6,
of the multihomed destination.
Used for Solaris only.

=item C<< traffic-class => $string >>

(Optional) Specify the traffic class of probe packets.
The value must be an integer in the range from 0 to 255
Used for Solaris only.

=item C<< gateway => $string >>

(Optional) Specify a loose source route gateway  
so that the probe packet goes through the specified host along the
path to the target host.
The maximum number of gateways is 8 for IPv4 and 127 for IPv6.
Usedfor Solaris only.

=item C<< loose-source => $string|$boolean >>

(Optional) Use to send the probe packet to the given host and back again
using loose source routing.
For Solaris, can be used along with the -g option 
For Windows, hostlist is specified followed by this option ,
& finally destination follows the hostlist. 
Used for Windows & Solaris.

=item C<< port => $string >>

(Optional) Set the base UDP port number used in probes.
The  default  base  port number  is  33434.
Used for Solaris only.

=item C<< interval => $string >>

(Optional) Wait interval seconds between sending each packet.
Default is one second.
Used for Linux & Solaris only.

=item C<< interface-address => $string >>

(Optional) Set source address to specified interface address.
Used for Linux & Solaris only.

=item C<< data-size => $string >>

(Optional) Data size of the packet in bytes.
Used for Solaris only.

=item C<< hostname => $boolean >>

(Optional) Resolve addresses to hostnames.
Used for Windows only.

=item C<< fragment => $boolean >>

(Optional) Set Don't Fragment flag in packet (IPv4-only).
Used for Windows only.

=item C<< strict-source => $string >>

(Optional) Strict source route along host-list (IPv4-only).
Used for Windows only.

=item C<< reverse-route => $boolean >>

(Optional) Use routing header to test reverse route also (IPv6-only).
Used for Windows only.

=item C<< source-addr => $string >>

(Optional) Source address to use. 
Used for Windows only.

=back

=back

=cut

sub ping {
    my $pkg = shift;
    $Log->enter() if $may_enter;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw (audible adaptive allow-broadcast bind-source so-debug
                    flow-label flood-ping supress-loopback-multicast numeric
                    quiet bypass-routing-tables user-to-user-latency verbose
                    interval interface-address preload pattern
                    packet-size time-to-live hint tos record-route sndbuf
                    timestamp verbose version deadline timeout hop destination
                    addr-family udp-packets all-addresses traffic-class
                    gateway loose-source port data-size hostname fragment
                    reverse-route source-addr ipv4 ipv6)
            ),
            count => { type => SCALAR, default => 5},
        },
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_ping_windows_cli",
                interface => "CLI",
                set       => "Windows"
            },
            {   method    => "_ping_linux_cli",
                interface => "CLI",
                set       => "Linux"
            },
            {   method    => "_ping_solaris_cli",
                interface => "CLI",
                set       => "Solaris"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
}

sub _ping_windows_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $destination       = $opts{destination};
    my %api_opts;
    my $allowed_options
        = [qw( hostname count size fragment ttl tos record-route timestamp
              loose-source strict-source reverse-route timeout source-addr ipv4
              ipv6 destination)];

    if(Net::IP::ip_is_ipv6($destination)) {
        $opts{ipv6} = 1;
    } else {
        $opts{ipv4} = 1;
    }
    if(( defined $opts{timestamp}) && ($opts{timestamp} =~ /ts/)) {
        delete $opts{timestamp};
    }
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%api_opts,
        map    => { "packet-size"  => "size",
                    "time-to-live" => "ttl",
        },
    );
    $pkg->_api_call_log_debug_opts(
            api_opts => \%api_opts,
            api      => 'ping'
    );
    my $response = $apiset->ping(%api_opts);
    my $output   = $response->get_parsed_output();
    my $final_attributes;
    $final_attributes = $pkg->_hash_copy(
        source => $output->[0],
        copy   => [qw( bytes avg min ip max host packets_transmitted loss received stats loss_percent) ],
    );
    my $obj = NACL::CS::Client::Ping->new( command_interface => $command_interface ,destination => $destination);
    $obj->_set_fields( row => $final_attributes );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _ping_linux_cli {
    my $pkg = shift;
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $destination       = $opts{destination}; 
    my %api_opts;
    my $status;
    my $check  = 0;
    my $allowed_options;
    my $cmd ;

    if(Net::IP::ip_is_ipv6($destination)) {
        $allowed_options 
            = [qw ( audible adaptive count so-debug flow-label flood-ping
                   interval interface-address preload supress-loopback-multicast
                   numeric pattern tos quiet bypass-routing-tables packet-size
                   sndbuf time-to-live hint user-to-user-latency verbose
                   version deadline timeout hop destination)];
        $cmd = 'ping6';
    } else {
        $allowed_options
            = [qw( audible adaptive allow-broadcast bind-source so-debug
                  flood-ping supress-loopback-multicast numeric quiet
                  bypass-routing-tables user-to-user-latency verbose
                  record-route version count interval preload pattern
                  packet-size time-to-live deadline interface-address hint tos
                  sndbuf timestamp timeout hop destination)];
        $cmd = 'ping';
    }
    if(( defined $opts{timestamp}) && ($opts{timestamp} !~ /ts/)) {
        delete $opts{timestamp};
    }
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%api_opts,
    );

    $pkg->_api_call_log_debug_opts(
        api_opts => \%api_opts,
        api      => '$cmd'
    );
    
    my $response = $apiset->$cmd(%api_opts);
    my $output   = $response->get_parsed_output();
    my $final_attributes;
    $final_attributes = $pkg->_hash_copy(
        source => $output->[0],
        copy   => [qw( bytes avg min time ip mdev max host packets_transmitted received stats) ],
        map    => { "loss" => "loss_percent", }, 
    );
    my $obj = NACL::CS::Client::Ping->new( command_interface => $command_interface ,destination => $destination);
    $obj->_set_fields( row => $final_attributes );
    $Log->exit() if $may_exit;
    return $obj;
} 

sub _ping_solaris_cli {
    my $pkg = shift;
    $Log->enter() if $may_enter;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $destination       = $opts{destination};
    my %api_opts;
    my $allowed_options
            = [qw( addr-family flow-label interval supress-loopback-multicast
                  tos record-route udp-packets all-addresses traffic-class
                  so-debug gateway interface-address loose-source numeric port
                  bypass-routing-tables packet-size ttl verbose destination
                  data-size count)];

    if(defined $opts{'packet-size'}) {
        $opts{'packet-size'} = 1;
    }

    if(defined $opts{'loose-source'}) { 
        $opts{'loose-source'} = 1;
    }

    if(Net::IP::ip_is_ipv6($destination)) {
        $opts{'addr-family'} = "inet6";
    } else {
        $opts{'addr-family'} = "inet";
    }
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%api_opts,
        map    => { "time-to-live" => "ttl" },
    );

    $pkg->_api_call_log_debug_opts(
        api_opts => \%api_opts,
        api      => 'ping'
    );

    my $response = $apiset->ping(%api_opts);
    my $output   = $response->get_parsed_output();
    my $final_attributes;
    $final_attributes = $pkg->_hash_copy(
        source => $output->[0],
        copy   => [qw( state) ],
        map    => {'host' => 'destination', },
    );
    my $obj = NACL::CS::Client::Ping->new( command_interface => $command_interface , destination => $destination);
    $obj->_set_fields( row => $final_attributes );
    $Log->exit() if $may_exit;
    return $obj;

}
        
sub _primary_keys_validate_spec {
    return ( destination => { type => SCALAR },);
}

1;
