# $Id$

#
# Copyright (c) 2001-2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ClusterImage Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::ClusterImage

=head1 DESCRIPTION

C<NACL::C::ClusterImage> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP ClusterImage.  A related class is
L<NACL::CS::ClusterImage|lib-NACL-CS-ClusterImage-pm>, which represents
the past state of an ONTAP ClusterImage.
However, since it does not have any primary keys, 'find' and 'state' do not
work for this component. (This component is considered a 'singleton',
i.e. there is only one row shown by the relevant 'show' command)

=cut

=head1 ATTRIBUTES

None.

=cut

package NACL::C::ClusterImage;

use strict;
use warnings;

use NACL::CS::ClusterImage;
use NACL::CS::ClusterImageUpdate;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use NATE::BaseException qw(:try);
use NACL::C::Exceptions::ClusterImage::VerifySignaturesImage;

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(SCALAR SCALARREF ARRAYREF validate_with);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $ClusterImage = NACL::C::ClusterImage->new(
                                   command_interface => $ci,
                                  );

Return a new ClusterImage component object with the given attributes.

=cut

=head2 find

    my $ClusterImage_obj  = NACL::C::ClusterImage->find(command_interface => $ci, ...);
    my @ClusterImage_objs = NACL::C::ClusterImage->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.
This method invokes cluster-image-get-iter for CMode ZAPI

=head2 state

    my ClusterImage_state = $ClusterImage_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::ClusterImage would return
a NACL::CS::ClusterImage object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.
This method invokes cluster-image-get-iter for CMode ZAPI

=cut

=head2 show_update_history

    my $show_update_history_obj = NACL::C::ClusterImage->show_update_history(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

    my @show_update_history_objs = NACL::C::ClusterImage->show_update_history(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $show_update_history_obj = $ClusterImage_obj->show_update_history();

(Class or Instance method)
This method returns objects of the type L<NACL::CS::ClusterImageShowUpdateHistory|lib-NACL-CS-ClusterImageShowUpdateHistory-pm>.

This method accepts all the parameters accepted by L<fetch|lib-NACL-CS-ComponentState-pm/fetch>
(requested_fields, filter, allow_empty) and has all of the other properties of
fetch methods (callable in scalar or list context, throws a NoElementsFound
exception if no matching elements are found)

Uses a CMode CLI/ZAPI APISet.
The method invokes 'cluster-image-update-history-get-iter' for CMode ZAPI

=cut

sub show_update_history {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::ClusterImageShowUpdateHistory;
    my @state_objs = NACL::CS::ClusterImageShowUpdateHistory->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 update

    NACL::C::ClusterImage->update(
                                command_interface => $ci,
                                ...);

    my $cluster_image_update_obj = NACL::C::ClusterImage->update(...);

    my @cluster_image_update_objs = NACL::C::ClusterImage->update(...);

(Class method)
This method is used to perform operations associated with a non-disruptive update.

Uses a CMode CLI APIset.

This method returns objects of type
L<NACL::CS::ClusterImageUpdate|lib-NACL-CS-ClusterImageUpdate-pm>.

This method defaults to invalidating the version manager cache.
See L<NACL::C::CommandInterface::ONTAP|lib-NACL-C-CommandInterface-ONTAP-pm>.
Use C<< nacl_invalidate_version_manager_cache => 0 >> to prevent the version
manager cache invalidation.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'ignore-validation-warning' => $string >>

Applicable for CMode CLI.

=item C<< 'auto-backout' => $string >>

Applicable for CMode CLI.

=item C<< 'report-only' => $string >>

Applicable for CMode CLI.

=item C<< 'version' => $string >>

Applicable for CMode CLI.

=item C<< 'node-retry-threshold' => $string >>

Applicable for CMode CLI.

=item C<< 'cluster-retry-threshold' => $string >>

Applicable for CMode CLI.

=item C<< 'pause-after' => $string >>

Applicable for CMode CLI.

=item C<< 'nodes' => [ $node1, $node2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'skip-failover' => $string >>

*Skip Failover, possible value(s) are true,false

Applicable for CMode CLI.

=item C<< 'skip-validation' => $string >>

*Skip Validation, possible value(s) are true,false

Applicable for CMode CLI.

=item C<< 'skip-download' => $string >>

*Skip Download, possible value(s) are true,false

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< 'skip-confirmation' => $string >>

Skip Confirmation possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'force-serial' => $string >>

Force Serial Update
Applicable for CMode CLI

=item C<< 'stabilize-minutes' => $string >>

Minutes to stabilize ranges from 1 to 60
Applicable for CMode CLI

=item C<< 'force-rolling' => $string >>

Force Rolling Update
Applicable for CMode CLI

=item C<< 'proceed-with-update' => $answer >>

Optional
if 'y', then proceed with update
if 'n', then don't  proceed with update

=item C<< '*' >>

This method takes all other options supported by
"cluster image update" command in ONTAP.

Applicable for CMode CLI.

=item C<< nacl_invalidate_version_manager_cache => 0|1 >>

(Defaults to 1)
Specifies whether to invalidate the version manager cache.

=item C<NACL::C::Exceptions::ClusterImage::VerifySignaturesImage>

This exception might be thrown during an upgrade process.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub update {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'nodes' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( ignore-validation-warning auto-backout report-only version
                    node-retry-threshold cluster-retry-threshold pause-after

                    skip-failover skip-validation skip-download

                    skip-confirmation

                    force-serial stabilize-minutes use-batch-update

                    force-rolling

                    proceed-with-update
                    )
            ),
            nacl_invalidate_version_manager_cache => {type => SCALAR, default => 1},
        },
        cli_cmd  => 'cluster image update',
    );
    my $invalidate_cache = delete $opts{nacl_invalidate_version_manager_cache};
    my @ret;
    try {
	@ret = $pkg->call_on_apiset(
             %opts,
             choices => [
		 {   method    => '_update_cmode_cli',
                     interface => 'CLI',
                     set       => 'CMode',
		 },
        	],
        	frontend => 'NACL::C::ClusterImage::update',
    	);
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        foreach my $type (qw(VerifySignaturesImage)) {
            my $full_type = "NACL::C::Exceptions::ClusterImage::$type";
            $full_type->detect_convert_and_throw(exception => $exception);
        }
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    # Version details would have changed, so invalidate existing cache.
    NACL::C::CommandInterface::ONTAP->invalidate_version_manager_cache() if $invalidate_cache;

    $Log->exit() if $may_exit;
    return wantarray ? @ret : $ret[0];
} ## end sub update

sub _update_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{'command_interface'};
    my %additional_args;
    if (exists $opts{'proceed-with-update'}) {
        $additional_args{'connectrec-match_table'}
            = ['Would you like to proceed with update' =>
                $opts{'proceed-with-update'}
            ];
        delete $opts{'proceed-with-update'};
    }

    if ( exists $opts{'use-batch-update'}) {
         $additional_args{'connectrec-match_table'}
           = ['Do you want to continue?' =>
                $opts{'use-batch-update'}
            ];
        delete $opts{'use-batch-update'};
    }

    my $response;
    $pkg->_base_backend_cmode_cli(
        %opts,
        api                 => 'cluster_image_update',
        apiset_response_obj => \$response,
        additional_args     => \%additional_args,
    );

    my $output = $response->get_parsed_output();
    my @state_objs;
    foreach my $row (@$output) {
        my $obj = NACL::CS::ClusterImageUpdate->new(
            command_interface => $command_interface);
        $obj->_set_fields(row => $row);
        push @state_objs, $obj;
    }

    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _update_cmode_cli

=head2 show_update_log

    my $show_update_log_obj = NACL::C::ClusterImage->show_update_log(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

    my @show_update_log_objs = NACL::C::ClusterImage->show_update_log(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $show_update_log_obj = $ClusterImage_obj->show_update_log();

(Class or Instance method)
This method returns objects of the type L<NACL::CS::ClusterImageShowUpdateLog|lib-NACL-CS-ClusterImageShowUpdateLog-pm>.

This method accepts all the parameters accepted by L<fetch|lib-NACL-CS-ComponentState-pm/fetch>
(requested_fields, filter, allow_empty) and has all of the other properties of
fetch methods (callable in scalar or list context, throws a NoElementsFound
exception if no matching elements are found)

Uses a CMode CLI/ZAPI APISet.
The method invokes 'cluster-image-update-log-get-iter' for CMode ZAPI

=cut

sub show_update_log {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::ClusterImageShowUpdateLog;
    my @state_objs = NACL::CS::ClusterImageShowUpdateLog->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 validate

    my $validate_obj = NACL::C::ClusterImage->validate(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

    my @validate_objs = NACL::C::ClusterImage->validate(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $validate_obj = $ClusterImage_obj->validate();

(Class or Instance method)
This method returns objects of the type L<NACL::CS::ClusterImageValidate|lib-NACL-CS-ClusterImageValidate-pm>.

This method accepts all the parameters accepted by L<fetch|lib-NACL-CS-ComponentState-pm/fetch>
(requested_fields, filter, allow_empty) and has all of the other properties of
fetch methods (callable in scalar or list context, throws a NoElementsFound
exception if no matching elements are found)

=cut

sub validate {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::ClusterImageValidate;
    my @state_objs = NACL::CS::ClusterImageValidate->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 start_failover_job

    NACL::C::ClusterImage->start_failover_job(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to start NDU Failover job.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'version' => $string >>

Applicable for CMode CLI.

=item C<< 'task-name' => $string >>

Applicable for CMode CLI.

=item C<< 'task-id' => $string >>

Applicable for CMode CLI.

=item C<< 'nodes' => [ $node1, $node2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< 'update-timestamp' => $string >>

Update Timestamp
Applicable for CMode CLI

=item C<< 'stabilize-minutes' => $string >>

Stabilize Minutes
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"cluster image start-failover-job" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start_failover_job {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'nodes' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( version task-name task-id

                    update-timestamp

                    stabilize-minutes
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd                 => 'cluster image start-failover-job',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_start_failover_job_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterImage::start_failover_job',
    );

    $Log->exit() if $may_exit;
} ## end sub start_failover_job

sub _start_failover_job_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args, api => 'cluster_image_start_failover_job');
    $Log->exit() if $may_exit;
} ## end sub _start_failover_job_cmode_cli

=head2 show_update_progress

    my $show_update_progress_obj = NACL::C::ClusterImage->show_update_progress(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

    my @show_update_progress_objs = NACL::C::ClusterImage->show_update_progress(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $show_update_progress_obj = $ClusterImage_obj->show_update_progress();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty)
and has all of the other properties of fetch methods (callable in scalar or list
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-ClusterImageShowUpdateProgress-pm/ATTRIBUTES>
of NACL::CS::ClusterImageShowUpdateProgress as input.
This method returns objects of the type L<NACL::CS::ClusterImageShowUpdateProgress|lib-NACL-CS-ClusterImageShowUpdateProgress-pm>.

=cut

sub show_update_progress {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::ClusterImageShowUpdateProgress;
    my @state_objs = NACL::CS::ClusterImageShowUpdateProgress->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 start_download

    NACL::C::ClusterImage->start_download(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to start cluster download image daemon job.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'version' => $string >>

*Download Version

Applicable for CMode CLI.

=item C<< 'task-name' => $string >>

*Task Name, possible value(s) are
initialize,mount-image,restart-hm,get-health,run-scripts,unmount-image,clear-alert,post-restart-hm,cleanup-rd,synch-image,do-download-job,do-failover-job,check-progress,complete-validation,invalid-task

Applicable for CMode CLI.

=item C<< 'task-id' => $string >>

*Task Id

Applicable for CMode CLI.

=item C<< 'nodes' => [ $node1, $node2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< 'update-timestamp' => $string >>

Update Timestamp
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"cluster image start-download" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start_download {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'nodes' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( version task-name task-id

                    update-timestamp
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'cluster image start-download',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_start_download_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterImage::start_download',
    );

    $Log->exit() if $may_exit;
} ## end sub start_download

sub _start_download_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args, api => 'cluster_image_start_download');
    $Log->exit() if $may_exit;
} ## end sub _start_download_cmode_cli

=head2 check_script

    NACL::C::ClusterImage->check_script(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
	                             
								 or

    $ClusterImage_obj->check_script(...);

(Class or instance method)
This method is used to check validation scripts.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node possible value(s) are, <nodename>,local

Applicable for CMode CLI.

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'node' => $string >>

Node possible value(s) are, <nodename>,local

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"cluster image check-script" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub check_script {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( node
                    )
            ),
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
        },
        cli_cmd => 'cluster image check-script',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_check_script_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterImage::check_script',
    );

    $Log->exit() if $may_exit;
} ## end sub check_script

sub _check_script_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts, api => 'cluster_image_check_script');

    $Log->exit() if $may_exit;
} ## end sub _check_script_cmode_cli

=head2 complete_ndu_validation

    NACL::C::ClusterImage->complete_ndu_validation(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to completes the ndu validation process.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"cluster image complete-ndu-validation" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub complete_ndu_validation {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        additional_spec =>
            {'cmd_execution_time' => {type => SCALARREF, optional => 1},},
        ignore_primary_keys => 1,
        cli_cmd             => 'cluster image complete-ndu-validation',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_complete_ndu_validation_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterImage::complete_ndu_validation',
    );

    $Log->exit() if $may_exit;
} ## end sub complete_ndu_validation

sub _complete_ndu_validation_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;

    $pkg->_base_backend_cmode_cli(@opts,api => 'cluster_image_complete_ndu_validation');

    $Log->exit() if $may_exit;
} ## end sub _complete_ndu_validation_cmode_cli

=head2 package_details

    my $package_details_obj = NACL::C::ClusterImage->package_details(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @package_details_objs = NACL::C::ClusterImage->package_details(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $package_details_obj = $ClusterImage_obj->package_details(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-ClusterImagePackageDetails-pm/ATTRIBUTES>
of NACL::CS::ClusterImagePackageDetails as input. 
This method returns objects of the type L<NACL::CS::ClusterImagePackageDetails|lib-NACL-CS-ClusterImagePackageDetails-pm>.

=cut

sub package_details {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::ClusterImagePackageDetails;
    my @state_objs = NACL::CS::ClusterImagePackageDetails->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 restart_local_health_monitors

    NACL::C::ClusterImage->restart_local_health_monitors(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to restarts the local health monitors.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"cluster image restart-local-health-monitors" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub restart_local_health_monitors {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        additional_spec =>
            {'cmd_execution_time' => {type => SCALARREF, optional => 1},},
        ignore_primary_keys => 1,
        cli_cmd             => 'cluster image restart-local-health-monitors',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_restart_local_health_monitors_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterImage::restart_local_health_monitors',
    );

    $Log->exit() if $may_exit;
} ## end sub restart_local_health_monitors

sub _restart_local_health_monitors_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'cluster_image_restart_local_health_monitors');

    $Log->exit() if $may_exit;
} ## end sub _restart_local_health_monitors_cmode_cli

=head2 start_ndu_validation

    NACL::C::ClusterImage->start_ndu_validation(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to starts the ndu validation process.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'name' => $string >>

Name of The Image possible value(s) are, text

Applicable for CMode CLI.

=item C<< 'nodes' => [ $node1, $node2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI.

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"cluster image start-ndu-validation" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start_ndu_validation {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'nodes'              => {type => ARRAYREF,  optional => 1},
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( name
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'cluster image start-ndu-validation',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_start_ndu_validation_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterImage::start_ndu_validation',
    );

    $Log->exit() if $may_exit;
} ## end sub start_ndu_validation

sub _start_ndu_validation_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,api => 'cluster_image_start_ndu_validation');

    $Log->exit() if $may_exit;
} ## end sub _start_ndu_validation_cmode_cli

=head2 start_package_check

    NACL::C::ClusterImage->start_package_check(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
	                             
								 or

    $ClusterImage_obj->start_package_check(...);

(Class or instance method)
This method is used to start cluster package check daemon job.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node possible value(s) are, <nodename>,local

Applicable for CMode CLI.

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'node' => $string >>

Node possible value(s) are, <nodename>,local

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"cluster image start-package-check" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start_package_check {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( node
                    )
            ),
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
        },
        cli_cmd  => 'cluster image start-package-check',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_start_package_check_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterImage::start_package_check',
    );

    $Log->exit() if $may_exit;
} ## end sub start_package_check

sub _start_package_check_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,api => 'cluster_image_start_package_check');

    $Log->exit() if $may_exit;
} ## end sub _start_package_check_cmode_cli

=head2 start_package_synchronize

    NACL::C::ClusterImage->start_package_synchronize(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
	                             
								 or

    $ClusterImage_obj->start_package_synchronize(...);

(Class or instance method)
This method is used to start cluster package Synchronize daemon job.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node possible value(s) are, <nodename>,local

Applicable for CMode CLI.

=item C<< 'ndu-phase' => $string >>

Update phase possible value(s) are, Cluster update phase

Applicable for CMode CLI.

=item C<< 'task-name' => $string >>


Task Name possible value(s) are, initialize,mount-image,restart-hm,get-health,run-scripts,unmount-image,clear-alert,post-restart-hm,cleanup-rd,synch-image,do-download-job,do-failover-job,check-progress,complete-validation,invalid-task 

Applicable for CMode CLI.

=item C<< 'task-id' => $string >>

Task Id possible value(s) are, integer

Applicable for CMode CLI.

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'node' => $string >>

Node possible value(s) are, <nodename>,local

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"cluster image start-package-synchronize" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start_package_synchronize {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( ndu-phase task-name task-id

                    node
                    )
            ),
        },
        cli_cmd => 'cluster image start-package-synchronize',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_start_package_synchronize_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterImage::start_package_synchronize',
    );

    $Log->exit() if $may_exit;
} ## end sub start_package_synchronize

sub _start_package_synchronize_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;

    $pkg_or_obj->_base_backend_cmode_cli( @opts, api => 'cluster_image_start_package_synchronize');

    $Log->exit() if $may_exit;
} ## end sub _start_package_synchronize_cmode_cli

=head2 show_update_detailed_report

    my $show_update_detailed_report_obj = NACL::C::ClusterImage->show_update_detailed_report(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_update_detailed_report_objs = NACL::C::ClusterImage->show_update_detailed_report(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_update_detailed_report_obj = $ClusterImage_obj->show_update_detailed_report(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-ClusterImageShowUpdateDetailedReport-pm/ATTRIBUTES>
of NACL::CS::ClusterImageShowUpdateDetailedReport as input. 
This method returns objects of the type L<NACL::CS::ClusterImageShowUpdateDetailedReport|lib-NACL-CS-ClusterImageShowUpdateDetailedReport-pm>.

=cut

sub show_update_detailed_report {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::ClusterImageShowUpdateDetailedReport;
    my @state_objs
        = NACL::CS::ClusterImageShowUpdateDetailedReport->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 show_update_summary_report

    my $show_update_summary_report_obj = NACL::C::ClusterImage->show_update_summary_report(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_update_summary_report_objs = NACL::C::ClusterImage->show_update_summary_report(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_update_summary_report_obj = $ClusterImage_obj->show_update_summary_report(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-ClusterImageShowUpdateSummaryReport-pm/ATTRIBUTES>
of NACL::CS::ClusterImageShowUpdateSummaryReport as input. 
This method returns objects of the type L<NACL::CS::ClusterImageShowUpdateSummaryReport|lib-NACL-CS-ClusterImageShowUpdateSummaryReport-pm>.

=cut

sub show_update_summary_report {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::ClusterImageShowUpdateSummaryReport;
    my @state_objs
        = NACL::CS::ClusterImageShowUpdateSummaryReport->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 show_update_log_detail_test

    my $show_update_log_detail_test_obj = NACL::C::ClusterImage->show_update_log_detail_test(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_update_log_detail_test_objs = NACL::C::ClusterImage->show_update_log_detail_test(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_update_log_detail_test_obj = $ClusterImage_obj->show_update_log_detail_test(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-ClusterImageShowUpdateLogDetailTest-pm/ATTRIBUTES>
of NACL::CS::ClusterImageShowUpdateLogDetailTest as input. 
This method returns objects of the type L<NACL::CS::ClusterImageShowUpdateLogDetailTest|lib-NACL-CS-ClusterImageShowUpdateLogDetailTest-pm>.

=cut

sub show_update_log_detail_test {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::ClusterImageShowUpdateLogDetailTest;
    my @state_objs
        = NACL::CS::ClusterImageShowUpdateLogDetailTest->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 show_update_log_detail

    my $show_update_log_detail_obj = NACL::C::ClusterImage->show_update_log_detail(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_update_log_detail_objs = NACL::C::ClusterImage->show_update_log_detail(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_update_log_detail_obj = $ClusterImage_obj->show_update_log_detail(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-ClusterImageShowUpdateLogDetail-pm/ATTRIBUTES>
of NACL::CS::ClusterImageShowUpdateLogDetail as input. 
This method returns objects of the type L<NACL::CS::ClusterImageShowUpdateLogDetail|lib-NACL-CS-ClusterImageShowUpdateLogDetail-pm>.

=cut

sub show_update_log_detail {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::ClusterImageShowUpdateLogDetail;
    my @state_objs = NACL::CS::ClusterImageShowUpdateLogDetail->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 start_local_health_monitors

    NACL::C::ClusterImage->start_local_health_monitors(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to starts the local health monitors.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'monitor' => $string >>

Health Monitor Service
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional)
Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< "job_component" => \$scalar >>

(Optional)
Scalar reference, If "cluster image start local health monitors" command returns job_id
then \$scalar gets updated with the corresponding job component

Applicable for CMode CLI

=item C<< "job_components" => \@array >>

(Optional)
Array reference , If "cluster image start local health monitors" command returns multiple job_ids
then \@array gets updated with an array of corresponding job components

Applicable for CMode CLI

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by "cluster image start local health monitors" command in ONTAP

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start_local_health_monitors {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( monitor
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'cluster image start-local-health-monitors',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_start_local_health_monitors_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterImage::start_local_health_monitors',
    );

}

sub _start_local_health_monitors_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'cluster_image_start_local_health_monitors',);
    $Log->exit() if $may_exit;
}

=head2 stop_local_health_monitors

    NACL::C::ClusterImage->stop_local_health_monitors(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to stops the local health monitors.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'monitor' => $string >>

Health Monitor Service
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional)
Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< "job_component" => \$scalar >>

(Optional)
Scalar reference, If "cluster image stop local health monitors" command returns job_id
then \$scalar gets updated with the corresponding job component

Applicable for CMode CLI

=item C<< "job_components" => \@array >>

(Optional)
Array reference , If "cluster image stop local health monitors" command returns multiple job_ids
then \@array gets updated with an array of corresponding job components

Applicable for CMode CLI

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by "cluster image stop local health monitors" command in ONTAP

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub stop_local_health_monitors {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( monitor
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'cluster image stop-local-health-monitors',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_stop_local_health_monitors_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterImage::stop_local_health_monitors',
    );

}

sub _stop_local_health_monitors_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'cluster_image_stop_local_health_monitors',);
    $Log->exit() if $may_exit;
}

=head2 find_update_job

    NACL::C::ClusterImage->find_update_job(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to finds NDU job with a given name.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'name' => $string >>

NDU Job Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "cluster image find update job" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub find_update_job {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( name
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'cluster image find-update-job',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_find_update_job_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterImage::find_update_job',
    );

}

sub _find_update_job_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'cluster_image_find_update_job',);
    $Log->exit() if $may_exit;
}

=head2 resume_update_job

    NACL::C::ClusterImage->resume_update_job(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to resumes NDU job with a given name.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'name' => $string >>

NDU Job Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "cluster image resume update job" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub resume_update_job {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( name
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd                 => 'cluster image resume-update-job',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_resume_update_job_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterImage::resume_update_job',
    );

}

sub _resume_update_job_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'cluster_image_resume_update_job',);
    $Log->exit() if $may_exit;
}

=head2 start_giveback_job

    NACL::C::ClusterImage->start_giveback_job(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to start NDU Giveback job.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'version' => $string >>

Update Version
Applicable for CMode CLI

=item C<< 'task-name' => $string >>


Task Name possible value(s) are, initialize,mount-image,restart-hm,get-health,run-scripts,unmount-image,clear-alert,post-restart-hm,cleanup-rd,synch-image,do-download-job,do-failover-job,do-giveback-job,check-progress,complete-validation,invalid-task,default-task 
Applicable for CMode CLI

=item C<< 'task-id' => $string >>

Task Id
Applicable for CMode CLI

=item C<< 'nodes' => [ $node1, $node2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< 'update-timestamp' => $string >>

Update Timestamp
Applicable for CMode CLI

=item C<< 'stabilize-minutes' => $string >>

Stabilize Minutes
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "cluster image start giveback job" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start_giveback_job {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'nodes' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( version task-name task-id

                    update-timestamp

                    stabilize-minutes
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'cluster image start giveback-job',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_start_giveback_job_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterImage::start_giveback_job',
    );

}

sub _start_giveback_job_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'cluster_image_start_giveback_job',);
    $Log->exit() if $may_exit;
}

=head2 update_pause

Usage of this method is deprecated. It is recommended to use
   NACL::C::ClusterImage->pause_update(...); instead.

=cut

sub update_pause {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_redirect_call(_method => 'pause', %opts);
    $Log->exit() if $may_exit;
}

=head2 update_resume

Usage of this method is deprecated. It is recommended to use
   NACL::C::ClusterImage->resume_update(...); instead.

=cut

sub update_resume {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_redirect_call(_method => 'resume', %opts);
    $Log->exit() if $may_exit;
}

=head2 update_cancel

Usage of this method is deprecated. It is recommended to use
   NACL::C::ClusterImage->cancel_update(...); instead.

=cut

sub update_cancel {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_redirect_call(_method => 'cancel', %opts);
    $Log->exit() if $may_exit;
}

=head2 update_abort

Usage of this method is deprecated. It is recommended to use
   NACL::C::ClusterImage->abort_update(...); instead.

=cut

sub update_abort {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_redirect_call(_method => 'abort', %opts);
    $Log->exit() if $may_exit;
}

=head2 show_upgrade_manager

    NACL::C::ClusterImage->show_upgrade_manager(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to displays the master upgrade manager.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'mum-node' => $string >>

Current Master Upgrade Manager
Applicable for CMode CLI

=item C<< 'mum-nodename' => $string >>

Current Master Upgrade Manager possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "cluster image show upgrade manager" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub show_upgrade_manager {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( mum-node

                    mum-nodename
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'cluster image show-upgrade-manager',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_show_upgrade_manager_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterImage::show_upgrade_manager',
    );

    $Log->exit() if $may_exit;
}

sub _show_upgrade_manager_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'cluster_image_show_upgrade_manager',);
    $Log->exit() if $may_exit;
}

=head2 check_download_progress

    NACL::C::ClusterImage->check_download_progress(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to check image download progress.
Uses CMode CLI/ZAPI .
Invokes  cluster-image-get-download-progress for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'version' => $string >>

Package Version

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "cluster image check download progress" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub check_download_progress {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( version
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'cluster_image_check_download_progress',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_check_download_progress_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_check_download_progress_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterImage::check_download_progress',
    );

    $Log->exit() if $may_exit;
}

sub _check_download_progress_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'cluster_image_check_download_progress',);
    $Log->exit() if $may_exit;
}

sub _check_download_progress_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(@opts,
        api => 'cluster-image-get-download-progress',);
    $Log->exit() if $may_exit;
}

sub _redirect_call {
    $Log->enter() if $may_enter;

    my ($pkg_or_obj, %opts) = @_;
    my $command = delete $opts{_method};
    my $new_name = $command.'_update';
    $Log->warn("The command 'cluster image update-$command' got changed to "
                . "'cluster image $command-update'. Please use NACL::C::ClusterImage->$new_name ");
    $pkg_or_obj->$new_name(%opts);
    $Log->exit() if $may_exit;
}

__PACKAGE__->_load_event_class();

1;
