# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ClusterTimeServiceNtpServer Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::ClusterTimeServiceNtpServer

=head1 DESCRIPTION

C<NACL::C::ClusterTimeServiceNtpServer> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP ClusterTimeServiceNtpServer.  A related class is
L<NACL::CS::ClusterTimeServiceNtpServer|lib-NACL-CS-ClusterTimeServiceNtpServer-pm>, which represents
the past state of an ONTAP ClusterTimeServiceNtpServer.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 server

=cut

package NACL::C::ClusterTimeServiceNtpServer;

use strict;
use warnings;

use NACL::CS::ClusterTimeServiceNtpServer;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $ClusterTimeServiceNtpServer = NACL::C::ClusterTimeServiceNtpServer->new(
                                   command_interface => $ci,
                                   'server' => $server,
								  );

Return a new ClusterTimeServiceNtpServer component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'server',];

=head2 find

    my $ClusterTimeServiceNtpServer_obj  = NACL::C::ClusterTimeServiceNtpServer->find(command_interface => $ci, ...);
	my @ClusterTimeServiceNtpServer_objs = NACL::C::ClusterTimeServiceNtpServer->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my ClusterTimeServiceNtpServer_state = $ClusterTimeServiceNtpServer_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::ClusterTimeServiceNtpServer would return 
a NACL::CS::ClusterTimeServiceNtpServer object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 create

    my $ClusterTimeServiceNtpServer_obj	= NACL::C::ClusterTimeServiceNtpServer->create(
                                command_interface => $ci,
	                            'server' => $server,
                                ...);

(Class method)
This method is used to add a NTP Server.
Uses CMode  CLI/ZAPI APISet.

Invokes 'ntp-server-create' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'server' => $server >>

(Required)
NTP Server Host Name, IPv4 or IPv6 Address
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : server-name

=item C<< 'is-preferred' => $string >>


Is Preferred NTP Server (default: false) possible value(s) are, true,false
Applicable for CMode CLI/ZAPI

=item C<< 'version' => $string >>

NTP Version For Server (default: auto) possible value(s) are, 3,4,auto
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "cluster time service ntp server create" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( is-preferred version
                    )
            ),
        },
        api => 'cluster_time_service_ntp_server_create',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterTimeServiceNtpServer::create',
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        server            => $opts{'server'},
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'cluster_time_service_ntp_server_create',);
    $Log->exit() if $may_exit;
}

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'ntp-server-create',
        copy            => [qw( is-preferred version )],
        map             => {server => 'server-name'},
        frontend_method => 'create'
    );
    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::ClusterTimeServiceNtpServer->delete(
                                command_interface => $ci,
	                            'server' => $server,
                                ...);

								 or

    $ClusterTimeServiceNtpServer_obj->delete(...);

(Class or instance method)
This method is used to delete a NTP Server.
Uses CMode CLI/ZAPI APISet.

Invokes 'ntp-server-delete' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'server' => $server >>

(Required for class method, Not Applicable for instance method)
NTP Server Host Name, IPv4 or IPv6 Address
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: server-name

=item C<< '*' >>

This method takes all other options supported by "cluster time service ntp server delete" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        api    => 'cluster_time_service_ntp_server_delete',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterTimeServiceNtpServer::delete',
    );

    $Log->exit() if $may_exit;
}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'cluster_time_service_ntp_server_delete',);
    $Log->exit() if $may_exit;
}

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'ntp-server-delete',
        map             => {server => 'server-name'},
        frontend_method => 'delete',
    );
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::ClusterTimeServiceNtpServer->modify(
                                command_interface => $ci,
	                            'server' => $server,
                                ...);
	                             
								 or

    $ClusterTimeServiceNtpServer_obj->modify(...);

(Class or instance method)
This method is used to modify NTP Server Options.
Uses CMode CLI/ZAPI APIAet.

Invokes 'ntp-server-modify' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'server' => $server >>

(Required for class method, Not Applicable for instance method)
NTP Server Host Name, IPv4 or IPv6 Address
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : server-name

=item C<< 'is-preferred' => $string >>


Is Preferred NTP Server (default: false) possible value(s) are, true,false
Applicable for CMode CLI/ZAPI

=item C<< 'version' => $string >>

NTP Version For Server (default: auto) possible value(s) are, 3,4,auto
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "cluster time service ntp server modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( is-preferred version
                    )
            ),
        },
        api => 'cluster_time_service_ntp_server_modify',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterTimeServiceNtpServer::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'cluster_time_service_ntp_server_modify',);
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'ntp-server-modify',
        copy            => [qw( is-preferred version )],
        map             => {server => 'server-name'},
        frontend_method => 'modify',
    );
    $Log->exit() if $may_exit;
}

=head2 validate

    NACL::C::ClusterTimeServiceNtpServer->validate(
                                command_interface => $ci,
	                            'server' => $server,
                                ...);
	                             
								 or

    $ClusterTimeServiceNtpServer_obj->validate(...);

(Class or instance method)
This method is used to validate NTP server settings and report the result.
Uses CMode CLI/ZAPI APISet.

Invokes 'ntp-server-validate' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'server' => $server >>

(Required for class method, Not Applicable for instance method)
NTP Server Host Name, IPv4 or IPv6 Address
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : server-name

=item C<< '*' >>

This method takes all other options supported by "cluster time service ntp server validate" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub validate {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        api    => 'cluster_time_service_ntp_server_validate',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_validate_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_validate_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterTimeServiceNtpServer::validate',
    );

    $Log->exit() if $may_exit;
}

sub _validate_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'cluster_time_service_ntp_server_validate',);
    $Log->exit() if $may_exit;
}

sub _validate_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'ntp-server-validate',
        map             => {server => 'server-name'},
        frontend_method => 'validate',
    );
    $Log->exit() if $may_exit;
}

=head2 reset

    NACL::C::ClusterTimeServiceNtpServer->reset(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to reset NTP server list to a default selection.
Uses CMode CLI/ZAPI APISet.

Invokes 'ntp-server-reset' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'use-public' => $string >>


Reset Server List to Public Identified Defaults (default: false) possible value(s) are, true,false
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "cluster time service ntp server reset" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub reset {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( use-public
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'cluster_time_service_ntp_server_reset',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_reset_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_reset_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            }
        ],
        frontend => 'NACL::C::ClusterTimeServiceNtpServer::reset',
    );

    $Log->exit() if $may_exit;
}

sub _reset_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'cluster_time_service_ntp_server_reset',);
    $Log->exit() if $may_exit;
}

sub _reset_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'ntp-server-reset',
        copy            => [qw( use-public )],
        frontend_method => 'reset',
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('server' => {type => SCALAR},);
}

1;
