# $Id //depot/prod/test/UM6.0_integ/lib/NACL/C/DFM/Cluster.pm#1 $

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Cluster Component Module (auto-generated by NCAGE)
## @author vinothg,dl-mpo-nacl-dev@netapp.com
## @status Review
## @pod here

=head1 NAME

NACL::C::DFM::Cluster

=head1 DESCRIPTION

C<NACL::C::DFM::Cluster> is a derived class of
L<NACL::C::Component|lib-NACL-C-Component-pm>. It represents access
to an DFM Cluster.  A related class is
L<NACL::CS::DFM::Cluster|lib-NACL-CS-DFM-Cluster-pm>, which represents
the State Info of an DFM Cluster.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 object_name_or_ids

=head2 command

=cut

package NACL::C::DFM::Cluster;

use strict;
use warnings;

use NACL::CS::DFM::Cluster;
use NACL::CS::DFM::ClusterNode;
use NACL::CS::DFM::ClusterSfo;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $Cluster = NACL::C::DFM::Cluster->new( command_interface => $ci,
                                   'object-name-or-ids' => $object_name_or_ids,
                                   'command' => $command,
                            );

Return a new Cluster component object with the given attributes.

=cut

use Class::MethodMaker [
    new    => [ '-hash', '-init', 'new' ],
    scalar => 'object_name_or_ids',
    scalar => 'command',
    scalar => [ { -type => 'NACL::C::DFM::DFMClient' }, 'command_interface', ],
];

=head2 find

    my $Cluster  = NACL::C::DFM::Cluster->find(command_interface => $ci, ...);
    my @Clusters = NACL::C::DFM::Cluster->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my Cluster_state = $NACL::C::DFM::Cluster->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DFM::Cluster would return
a NACL::CS::DFM::Cluster object)

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 command_run

    NACL::C::DFM::Cluster->command_run(  command_interface => $ci,
                                'command' => $command,
                                'object-name-or-ids' => $object_name_or_ids,
                                  ...);
                                      or

    $Cluster_object->command_run(...);

=for html (Class or instance method)
This method is used to  submit a job for execution on a remote cluster or clusters.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'command' => $command >>

(Required for class method, Not Applicable for instance method)

=item C<< 'object-name-or-ids' => $object_name_or_ids >>

(Required for class method, Not Applicable for instance method)

=item C<< 'timeout' => $string >>

=item C<< 'max-retries' => $string >>

=item C<< 'object-name-or-ids' => [ $object-name-or-id1, $object-name-or-id2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
=item C<< 'command' => [ $command1, $command2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub command_run {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'object-name-or-ids' => { type => ARRAYREF, optional => 1 },
            'command'            => { type => ARRAYREF, optional => 1 },
            _optional_scalars(
                qw( timeout max-retries
                  )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_command_run_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _command_run_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( timeout max-retries object-name-or-ids command
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    $pkg_or_obj->_api_call_log_debug_opts(
        api      => 'cluster_command_run',
        api_opts => \%args,
    );

    # Making the API Call
    $apiset->cluster_command_run(%args);

    $Log->exit() if $may_exit;

}

sub _primary_keys_validate_spec {
    return (
        'object-name-or-ids' => { type => SCALAR },
        'command'            => { type => SCALAR },
    );
}

=head2 node_list

    NACL::C::DFM::Cluster->node_list(  command_interface => $ci,
                                   'resource-filter' => $resource_key,
                                  ...);
                                      or

    $Cluster_object->node_list(...);

=for html (Class or instance method)
This method is used to  modify attributes of a managed cluster.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'resource-filter' => $resource_key >>

(Optional)


=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub node_list {

    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);

    my %opts = $pkg->_common_validate_with(
        params              => \@_,
        additional_spec     => { _optional_scalars(qw( resource-filter )), },
        ignore_primary_keys => 1,
    );
    my @state_objs = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_node_list_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _node_list_zapi {

    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $allowed_options = [
        qw( resource-filter
          )
    ];

    my %filter_opts;

    $pkg->_hash_move(
        source => \%opts,
        target => \%filter_opts,
        move   => $allowed_options,
    );

    my @state_objs =
      NACL::CS::DFM::ClusterNode->_fetch_zapi( %opts, filter => \%filter_opts );

    $Log->exit() if $may_exit;
    return @state_objs;
}

=head2 sfo_list

    NACL::C::DFM::Cluster->sfo_list(  command_interface => $ci,
                                   'resource-filter' => $resource_key,
                                  ...);
                                      or

    $Cluster_object->sfo_list(...);

=for html (Class or instance method)
This method is used to  modify attributes of a managed cluster.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'resource-filter' => $resource_key >>

(Optional)


The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub sfo_list {

    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);

    my %opts = $pkg->_common_validate_with(
        params              => \@_,
        additional_spec     => { _optional_scalars(qw( resource-filter )), },
        ignore_primary_keys => 1,
    );
    my @state_objs = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_sfo_list_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _sfo_list_zapi {

    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $allowed_options = [
        qw( resource-filter
          )
    ];

    my %filter_opts;

    $pkg->_hash_move(
        source => \%opts,
        target => \%filter_opts,
        move   => $allowed_options,
    );

    my @state_objs =
      NACL::CS::DFM::ClusterSfo->_fetch_zapi( %opts, filter => \%filter_opts );

    $Log->exit() if $may_exit;
    return @state_objs;

}

1;
