# $Id //depot/prod/test/UM6.0_integ/lib/NACL/C/DFM/Datasource.pm#1 $

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Datasource Component Module (auto-generated by NCAGE)
## @author revathi,dl-mpo-nacl-dev@netapp.com
## @status Review
## @pod here

=head1 NAME

NACL::C::DFM::Datasource

=head1 DESCRIPTION

C<NACL::C::DFM::Datasource> is a derived class of
L<NACL::C::Component|lib-NACL-C-Component-pm>. It represents access
to a DFM Datasource.  A related class is
L<NACL::CS::DFM::Datasource|lib-NACL-CS-DFM-Datasource-pm>, which represents
the State Info of a DFM Datasource.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 datasource_id

=cut

package NACL::C::DFM::Datasource;

use strict;
use warnings;

use NACL::CS::DFM::Datasource;
use NACL::C::DFM::Job;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $Datasource = NACL::C::DFM::Datasource->new( command_interface => $ci,
                                   'datasource_id' => $resource_key,
                            );

Return a new Datasource component object with the given attributes.

=cut

use Class::MethodMaker [
    new    => [ '-hash', '-init', 'new' ],
    scalar => 'datasource_id',
    scalar => [ { -type => 'NACL::C::DFM::DFMClient' }, 'command_interface', ],
];

=head2 find

    my $Datasource  = NACL::C::DFM::Datasource->find(command_interface => $ci, ...);
    my @Datasources = NACL::C::DFM::Datasource->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my Datasource_state = $NACL::C::DFM::Datasource->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DFM::Datasource would return
a NACL::CS::DFM::Datasource object)

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 delete

    NACL::C::DFM::Datasource->delete(  command_interface => $ci,
                                'datasource-id' => $datasource_id,
                                  ...);
                                      or

    $Datasource_object->delete(...);

=for html (Class or instance method)
This method is used to  irrecoverably deletes a datasource and all information collected from it. <p> A
successful deletion of a datasource will have the following impacts: <ul> <li>
All monitored objects contained by the cluster will be removed. <li> Monitoring
of the cluster will be stopped. <li> Resource groups and other objects with
references to the cluster or its contents will be updated based on its removal.
</ul> <p> You cannot undo this operation.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'datasource-id' => $datasource_id >>

(Required for class method, Not Applicable for instance method)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => { _optional_scalars(qw( datasource-id )), },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_delete_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _delete_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( datasource-id
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    $pkg_or_obj->_api_call_log_debug_opts(
        api      => 'datasource_remove',
        api_opts => \%args,
    );

    # Making the API Call
    $apiset->datasource_remove(%args);

    $Log->exit() if $may_exit;

}

=head2 add

    NACL::C::DFM::Datasource->add(  command_interface => $ci,
                                'password' => $password,
                                'hostname-or-ip' => $hostname_or_ip,
                                'username' => $username,
                                  ...);

=for html (Class method)
This method is used to  add a new datasource for connecting to a cluster.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'password' => $password >>

(Required)

=item C<< 'hostname-or-ip' => $hostname_or_ip >>

(Required)

=item C<< 'username' => $username >>

(Required)

=item C<< 'protocol' => $string >>

=item C<< 'port' => $string >>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub add {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params              => \@_,
        ignore_primary_keys => 1,
        additional_spec     => {
            _optional_scalars(
                qw( hostname-or-ip username password protocol port
                  )
            ),
        },
    );

    my ( $job, $ret ) = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_add_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return ( $job, $ret );
}

sub _add_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( protocol password hostname-or-ip port username
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    $pkg_or_obj->_api_call_log_debug_opts(
        api      => 'datasource_add',
        api_opts => \%args,
    );
	
    # Making the API Call
    my $response      = $apiset->datasource_add(%args);
    my $parsed_output = $response->get_parsed_output();
    my $id            = $parsed_output->[0]->{'datasource-id'};
	my $job_id        = $parsed_output->[0]->{'job-id'};
	my $job = NACL::C::DFM::Job->new(
        'command_interface' => $command_interface,
        'job-id'            => $job_id,
    );
	my $datasource_obj = $pkg_or_obj->new(
		'command_interface' => $command_interface,
		'datasource_id'     => $id,
	);
    $Log->exit() if $may_exit;
	return ( $job, $datasource_obj );
}

=head2 object_discover

    NACL::C::DFM::Datasource->object_discover(  command_interface => $ci,
                                'object-type' => $object_type,
                                'datasource-id' => $datasource_id,
                                'object-name' => $object_name,
                                  ...);
                                      or

    $Datasource_object->object_discover(...);

=for html (Class or instance method)
This method is used to  attempts to discover a storage object with a given name and of a specified type on an
already monitored cluster.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'object-type' => $object_type >>

(Required)

=item C<< 'datasource-id' => $datasource_id >>

(Required for class method, Not Applicable for instance method)

=item C<< 'object-name' => $object_name >>

(Required)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub object_discover {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params              => \@_,
        additional_spec     => {
            _optional_scalars(
                qw( object-name object-type datasource-id
                  )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_object_discover_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _object_discover_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( object-type object-name datasource-id
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response      = $apiset->datasource_object_discover(%args);
    my $parsed_output = $response->get_parsed_output();
    my $job_id        = $parsed_output->[0]->{'job-id'};

    $Log->exit() if $may_exit;
    return NACL::C::DFM::Job->new(
        'command_interface' => $command_interface,
        'job-id'            => $job_id,
    );

}

=head2 object_refresh

    NACL::C::DFM::Datasource->object_refresh(  command_interface => $ci,
                                'datasource-id' => $datasource_id,
                                  ...);
                                      or

    $Datasource_object->object_refresh(...);

=for html (Class or instance method)
This method is used to  request that the monitored attributes of the given object (and optionally child
objects) be "refreshed" by re-reading their current values from the
corresponding cluster.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'datasource-id' => $datasource_id >>

(Required for class method, Not Applicable for instance method)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub object_refresh {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => { _optional_scalars(qw( datasource-id)), },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_object_refresh_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _object_refresh_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( datasource-id
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response      = $apiset->datasource_object_refresh(%args);
    my $parsed_output = $response->get_parsed_output();
    my $job_id        = $parsed_output->[0]->{'job-id'};

    $Log->exit() if $may_exit;
    return NACL::C::DFM::Job->new(
        'command_interface' => $command_interface,
        'job-id'            => $job_id,
    );

}

=head2 modify

    NACL::C::DFM::Datasource->modify(  command_interface => $ci,
                                'datasource-id' => $datasource_id,
                                  ...);
                                      or

    $Datasource_object->modify(...);

=for html (Class or instance method)
This method is used to  modify attributes of a datasource for connecting to a cluster.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'datasource-id' => $datasource_id >>

(Required for class method, Not Applicable for instance method)

=item C<< 'protocol' => $string >>

=item C<< 'password' => $string >>

=item C<< 'hostname-or-ip' => $string >>

=item C<< 'port' => $string >>

=item C<< 'username' => $string >>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( protocol password hostname-or-ip port username
                  )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_modify_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _modify_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( protocol password hostname-or-ip datasource-id port username
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    $pkg_or_obj->_api_call_log_debug_opts(
        api      => 'datasource_modify',
        api_opts => \%args,
    );

    # Making the API Call
    $apiset->datasource_modify(%args);

    $Log->exit() if $may_exit;

}

sub _primary_keys_validate_spec {
    return ( 'datasource-id' => { type => SCALAR }, );
}

1;
