# $Id: //depot/prod/test/UM6.0_integ/lib/NACL/C/DFM/Disk.pm#1 $

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Disk Component Module (auto-generated by NCAGE)
## @author vinothg, dl-mei-nacl-dev@netapp.com
## @status Review
## @pod here

=head1 NAME

NACL::C::DFM::Disk

=head1 DESCRIPTION

C<NACL::C::DFM::Disk> is a derived class of
L<NACL::C::Component::DFM|lib-NACL-C-Component-DFM-pm>. It represents access
to an DFM Disk.  A related class is
L<NACL::CS::DFM::Disk|lib-NACL-CS-DFM-Disk-pm>, which represents
the State Info of an DFM Disk.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 resource_key

=cut

package NACL::C::DFM::Disk;

use strict;
use warnings;

use NACL::CS::DFM::Disk;
use NACL::CS::DFM::DiskPhysicalSummary;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::DFM';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $Disk = NACL::C::DFM::Disk->new( command_interface => $ci,
                                   'resource_key' => $resource_key,
                            );

Return a new Disk component object with the given attributes.

=cut

use Class::MethodMaker [ scalar => 'resource_key', ];

=head2 find

    my $Disk  = NACL::C::DFM::Disk->find(command_interface => $ci, ...);
    my @Disks = NACL::C::DFM::Disk->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my Disk_state = $NACL::C::DFM::Disk->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DFM::Disk would return
a NACL::CS::DFM::Disk object)

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 physical_summary_get

    NACL::C::DFM::Disk->physical_summary_get(  command_interface => $ci,
                                'resource_filter' => $resource_key,
                                  ...);
                                      or

    $Disk_object->physical_summary_get(...);

=for html (Class or instance method)
This method is used to  retrieve data, parity, spare, and total space usage (by number of disks and bytes)
for the specified aggregate, cluster, or cluster node. For a V-Series storage
system, data LUNs, spare LUNs and total LUNs are also reported, if array LUNs
exist. For an aggregate, spares and spare space is not reported. <br> Error
Conditions: <ul> <li> EINVALIDINPUT - When invalid input specified. <li>
EOBJECTNOTFOUND - When the resource-key does not correspond to an
appropriate object. <li> ENODISKSFOUND - When no disk information has been
retrieved yet for the resource-key. </ul>

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'resource-filter' => $resource_key >>

(Required for class method, Not Applicable for instance method)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub physical_summary_get {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@_,
        additional_spec =>
          { 'resource-filter' => { type => SCALAR, optional => 1 }, }
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_physical_summary_get_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _physical_summary_get_zapi {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [qw( resource-filter )];

    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response = $apiset->disk_physical_summary_get(%args);

    my $parsed_output = $response->get_parsed_output();
    my $disk_info =
      $parsed_output->[0]->{'disk-info'}->[0]
      ->{'disk-physical-summary-disk-info'}->[0];

    my $lun_info =
      $parsed_output->[0]->{'lun-info'}->[0]->{'disk-physical-summary-lun-info'}
      ->[0];

    my $row = {};
    if ( defined($disk_info) ) {
        my $disk_obj = NACL::CS::DFM::DiskPhysicalSummaryDiskInfo->new();
        $disk_obj->_set_fields( row => $disk_info );
        %$row = ( 'disk-info' => $disk_obj );
    }
    my $lun_obj;
    if ( defined($lun_info) ) {
        $lun_obj = NACL::CS::DFM::DiskPhysicalSummaryLunInfo->new();
        $lun_obj->_set_fields( row => $lun_info );
        %$row = ( 'lun-info' => $lun_obj );
    }

    my $obj =
      NACL::CS::DFM::DiskPhysicalSummary->new(
        command_interface => $command_interface );
    $obj->_set_fields( row => $row );

    return $obj;

}

sub _primary_keys_validate_spec {
    return ( 'resource-key' => { type => SCALAR, optional => 1 } );
}

1;
