# $Id: //depot/prod/test/UM6.0_integ/lib/NACL/C/DFM/Event.pm#1 $

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Event Component Module (auto-generated by NCAGE)
## @author dl-mpo-nacl-dev@netapp.com
## @change 2011-10-14 deepthis: Modified script to make it work and added CLI part
## @change 2012-02-28 ramyar: Modified the component to accomodate new ZAPIs
## @change 2012-08-28 vinothg:  Modified the component to accomodate new ZAPIs
#
## @status Review
## @pod here

=head1 NAME

NACL::C::DFM::Event

=head1 DESCRIPTION

C<NACL::C::DFM::Event> is a derived class of
L<NACL::C::Component::DFM|lib-NACL-C-Component-DFM-pm>. It represents access
to an DFM Event.  A related class is
L<NACL::CS::DFM::Event|lib-NACL-CS-DFM-Event-pm>, which represents
the State Info of an DFM Event.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 event_id

=cut

package NACL::C::DFM::Event;

use strict;
use warnings;

use NACL::CS::DFM::Event;
use NACL::ComponentUtils qw (_optional_scalars);
use NACL::CS::DFM::EventAction;
use base 'NACL::C::Component::DFM';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $Event = NACL::C::DFM::Event->new( command_interface => $ci,
                                   'event-id' => $event_id,
                            );

Return a new Event component object with the given attributes.

=cut

use Class::MethodMaker [ scalar => 'event_id', ];

=head2 find

    my $Event  = NACL::C::DFM::Event->find(command_interface => $ci, ...);
    my @Events = NACL::C::DFM::Event->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my Event_state = $Event->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DFM::Event would return
a NACL::CS::DFM::Event object)

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 acknowledge

    NACL::C::DFM::Event->acknowledge(  
        command_interface => $ci,
        'event-id' => $event_id,
        ...
    );

	or

    $Event_object->acknowledge( ... ) ;  
   
        or

    NACL::C::DFM::Event->acknowledge( 'event-id-list' => \@list, ... ); 
 
(Class or Instance method)
This method is used to  acknowledge events. This terminates repeated notifications due to that event.
It returns an object of the class NACL::C::DFM::EventAction.
see  L<NACL::CS::DFM::EventAction|lib-NACL-CS-DFM-EventAction-pm/events-acknowledged-list>
Uses both DFM ZAPI and CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'event-id-list' => [ $event-id-1, $event-id-2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub acknowledge {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@_,
        additional_spec =>
          { 'event-id-list' => { type => ARRAYREF, optional => 1 }, },
        ignore_primary_keys => 1,
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_acknowledge_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },

        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub acknowledge

sub _acknowledge_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( event-id-list
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    my $response      = $apiset->event_acknowledge(%args);
    my $parsed_output = $response->get_parsed_output();
    my $output =
      $parsed_output->[0]->{'events-acknowledged-list'}->[0]
      ->{'event-action-info'};
    my @state_objs = ();
    foreach my $row ( @{$output} ) {
        my $obj = NACL::CS::DFM::EventAction->new();
        $obj->_set_fields( row => $row );
        push( @state_objs, $obj );
    }
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];

} ## end sub _acknowledge_zapi

=head2 generate

    $Event_object = NACL::C::DFM::Event->generate(  command_interface => $ci,
                                'source' => $source,
                                'event-name' => $event_name,
                                  ...);
                                      

=for html (Class method)
This method is used to  the event-generate API helps clients to generate events.
It returns an object of the class NACL::C::DFM::Event.  

Uses both DFM ZAPI and CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'source' => $source >>

(Required for class method, Not Applicable for instance method)

=item C<< 'event-name' => $event_name >>

(Required for class method, Not Applicable for instance method)

=item C<< 'event-message' => $string >>

=item C<< 'timestamp' => $string >>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub generate {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( event-message event-name source timestamp
                  )
            ),
        },
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_generate_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },

        ],
    );
    $Log->comment("After calling zapi ");
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub generate

sub _generate_zapi {
    $Log->enter() if $may_enter;

    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( event-message event-name source timestamp
          )
    ];
    my %args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );
    $Log->comment(Dumper(%opts));
    $Log->comment(Dumper(%args));
    # Making the API Call
    $apiset->event_generate(%args);

    $Log->exit() if $may_exit;

    # The event-generate does not return an ID. Hence use find()
    # to find event with given event-name and event-source-name
    # And find() anyway returns a Component object.
    # And this is automatically returned by the calling function.
    return $pkg->find(
        command_interface => $command_interface,
        filter            => {
            'source'     => $args{'source'},
            'event-name' => $args{'event-name'},
        }
    );

} ## end sub _generate_zapi

=head2 purge

    NACL::C::DFM::Event->purge(  command_interface => $ci,
                                'interval' => $interval,
                                  ...);

=for html (Class method)
This method is used to  purges the events which are older than that given interval.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'interval' => $interval >>

(Required for class method, Not Applicable for instance method)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub purge {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);

    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( interval
                  )
            ),
        },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_purge_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub purge

sub _purge_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( interval
          )
    ];
    my %args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    my $response            = $apiset->event_purge(%args);
    my $parsed_output       = $response->get_parsed_output();
    my $cutoff_timestamp    = $parsed_output->[0]->{'cutoff-timestamp'};
    my $purged_events_count = $parsed_output->[0]->{'purged-events-count'};
    my $output;
    $output->{'cutoff-timestamp'}    = $cutoff_timestamp;
    $output->{'purged-events-count'} = $purged_events_count;

    $Log->exit() if $may_exit;
    return $output;

} ## end sub _purge_zapi

=head2 assign

    NACL::C::DFM::Event->assign(  
	command_interface => $ci,
        'event-assignee-name' => $event_assignee_name,
	'event-id-list' => \@event_id_list,
	...
    );  
	

(Class method)
This method is used to assign events to a specific user.
This method returns object of type NACL::CS::DFM::EventAction.
see  L<NACL::CS::DFM::EventAction|lib-NACL-CS-DFM-EventAction-pm/assigned-events-list>

Uses both DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'event-assignee-name' => $string >>

=item C<< 'event-id-list' => [ $event-id-1, $event-id-2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub assign {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'event-id-list' => { type => ARRAYREF, optional => 1 },
            _optional_scalars(
                qw( event-assignee-name
                  )
            ),
        },
        ignore_primary_keys => 1,
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_assign_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },

        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub assign

sub _assign_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( event-assignee-name event-id-list
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    my $response      = $apiset->event_assign(%args);
    my $parsed_output = $response->get_parsed_output();
    my $output =
      $parsed_output->[0]->{'assigned-events-list'}->[0]->{'event-action-info'};
    my @state_objs = ();
    foreach my $row ( @{$output} ) {
        my $obj = NACL::CS::DFM::EventAction->new();
        $obj->_set_fields( row => $row );
        push( @state_objs, $obj );
    }
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];

} ## end sub _assign_zapi

=head2 resolve

    NACL::C::DFM::Event->resolve(  
	command_interface => $ci,
	'event-id-list' => \@event_id_list,
	...
    );  
	

(Class method)
This method is used to resolve events

Uses DFM ZAPI APISet.

This method returns an object of type NACL::CS::DFM::EventAction 
see  L<NACL::CS::DFM::EventAction|lib-NACL-CS-DFM-EventAction-pm/events-resolved-list>

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'event-id-list' => [ $event-id-1, $event-id-2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub resolve {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params              => \@_,
        ignore_primary_keys => 1,
        additional_spec =>
          { 'event-id-list' => { type => ARRAYREF, optional => 1 }, },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_resolve_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },

        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub resolve

sub _resolve_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( event-id-list
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    my $response      = $apiset->event_resolve(%args);
    my $parsed_output = $response->get_parsed_output();
    my $output =
      $parsed_output->[0]->{'events-resolved-list'}->[0]->{'event-action-info'};
    my @state_objs = ();
    foreach my $row ( @{$output} ) {
        my $obj = NACL::CS::DFM::EventAction->new();
        $obj->_set_fields( row => $row );
        push( @state_objs, $obj );
    }
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];

} ## end sub _resolve_zapi

=head2 find_or_generate

    $Event= NACL::C::DFM::Event->find_or_generate(
        command_interface       => $client,
        find_specific_options   => { 'event-name' => $event_pretty_name, },
        create_specific_options => {
            'event-name '    => $event_name,
	    'source'          => $resource,
	    'event-message'   => $event_message,
	    'timestamp'       => $timestamp,
        },
    );
        
This convenience method finds a particular Event entry, else generates the
event. This is a wrapper around
L<find_or_create|lib-NACL-C-Component-pm/find_or_create> (see that page
for a detailed explanation of the options accepted)

The example code shown above can be used instead of having lengthier
code such as this:

    try {
        $Event = NACL::C::DFM::Event->find(
            'command_interface' => $client,
            filter              => { 'event-name' => $event_pretty_name, }
        );
    } catch NACL::Exceptions::NoElementsFound with {
        $Event = NACL::C::DFM::Event->(
            command_interface => $client,
			'event-name'      => $event_name,
		        'source'          => $resource,
			'event-message'   => $event_message,
			'timestamp'       => $timestamp,
        );
    };

=cut

sub find_or_generate {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @objs = $pkg->find_or_create( @_, custom_create_command => 'generate' );
    $Log->exit() if $may_exit;
    return wantarray ? @objs : $objs[0];
}

sub _primary_keys_validate_spec {
    return ( 'event-id' => { type => SCALAR, optional => 1 }, );
}

# Overriding the init function
sub init {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => { 'event_id' => { type => SCALAR } },
        allow_extra => 1  # Let the other options be validated by the base class
    );

    $self->SUPER::init(%opts);
}

1;
