#
# Copyright (c) 2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Eventclass Component Module (auto-generated by NCAGE)
## @author dl-mei-nacl-dev@netapp.com
## @status Review
## @pod here

=head1 NAME

NACL::C::DFM::Eventclass

=head1 DESCRIPTION

C<NACL::C::DFM::Eventclass> is a derived class of
L<NACL::C::Component::DFM|lib-NACL-C-Component-DFM-pm>. It represents access
to an DFM Eventclass.  A related class is
L<NACL::CS::DFM::Eventclass|lib-NACL-CS-DFM-Eventclass-pm>, which represents
the State Info of an DFM Eventclass.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 event_class_name

=cut

package NACL::C::DFM::Eventclass;

use strict;
use warnings;

use NACL::CS::DFM::Eventclass;
use NACL::ComponentUtils qw (_optional_scalars);

use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

use base 'NACL::C::Component::DFM';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

# Component exceptions
use NACL::C::Exceptions::DFM::Eventclass::AlreadyExists();
use NACL::C::Exceptions::DFM::Eventclass::DoesNotExist();

=head1 METHODS

=head2 new

  my $Eventclass = NACL::C::DFM::Eventclass->new( command_interface => $ci,
                                   'event_class_name_or_id' => $event_class_name,
                            );

Return a new Eventclass component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'event_class_name_or_id',
];

=head2 find

    my $Eventclass  = NACL::C::DFM::Eventclass->find(command_interface => $ci, ...);
    my @Eventclasss = NACL::C::DFM::Eventclass->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my Eventclass_state = $Eventclass->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DFM::Eventclass would return
a NACL::CS::DFM::Eventclass object)

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 delete_custom

    NACL::C::DFM::Eventclass->delete_custom(  command_interface => $ci,
                                'event-class-name' => $event_class_name,
                                  ...);
                                      or

    $Eventclass_object->delete_custom(...);

=for html (Class or instance method)
This method is used to  supports deletion of custom event classes.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'event-class-name' => $event_class_name >>

(Required for class method, Not Applicable for instance method)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub delete_custom {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( event-class-name event-class-name-or-id
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_custom_zapi',
                    interface => 'ZAPI',
                    set       => 'DFM',
                },
            ],
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        my $text      = $exception->text();
        if ( $pkg_or_obj->_eventclass_does_not_exist_error( text => $text ) )
        {
            $Log->exit() if $may_exit;
            $exception->convert_and_throw( type =>
                    'NACL::C::Exceptions::DFM::Eventclass::DoesNotExist' );
        } else {
            $Log->exit() if $may_exit;
            $exception->throw();
        }
    };
    $Log->exit() if $may_exit;
    return $ret;
}

sub _delete_custom_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( event-class-name event-class-name-or-id
            )
    ];
    my %args;
    ## If the method is being invoked as an instance method, the primary key "event-class-name-or-id" needs to be mapped to "event-class-name"
    if ( defined( $opts{'event-class-name-or-id'} ) ) {
        $pkg_or_obj->_hash_copy(
            source => \%opts,
            copy   => $allowed_options,
            map    => { 'event-class-name-or-id' => 'event-class-name' },
            target => \%args,
        );
        ## deleting the key "event-class-name-or-id" as it is no longer required for the execution of the ZAPI
        delete $args{'event-class-name-or-id'};
    }
    ## If the method is being invoked as a class method , then "event-class-name" is directly provided
    else {
        $pkg_or_obj->_hash_copy(
            source => \%opts,
            copy   => $allowed_options,
            target => \%args,
        );
    }

    # Making the API Call
    $apiset->eventclass_delete_custom(%args);

    $Log->exit() if $may_exit;

}

=head2 add_custom

    NACL::C::DFM::Eventclass->add_custom(  command_interface => $ci,
                                'event-namespace-name' => $event_namespace_name,
                                'about-message' => $about_message,
                                'event-names' => $event_names,
                                'event-class-name' => $event_class_name,
                                  ...);
                                      or

    $Eventclass_object->add_custom(...);

=for html (Class or instance method)
This method is used to  supports adding custom event classes.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'event-namespace-name' => $event_namespace_name >>

(Required)

=item C<< 'about-message' => $about_message >>

(Required)

=item C<< 'event-names' => $event_names >>

(Required)

=item C<< 'event-class-name' => $event_class_name >>

(Required for class method, Not Applicable for instance method)

=item C<< 'is-multi-current' => $string >>

=item C<< 'is-allow-duplicates' => $string >>

=item C<< 'event-names' => [ $event-name1, $event-name2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub add_custom {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'event-names' => { type => ARRAYREF, optional => 1 },
            _optional_scalars(
                qw( is-multi-current is-allow-duplicates event-namespace-name about-message event-class-name event-name-short event-class-name-or-id
                    )
            ),
        },
    );
    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_add_custom_zapi',
                    interface => 'ZAPI',
                    set       => 'DFM',
                },
            ],
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        my $text      = $exception->text();
        if ( $pkg_or_obj->_eventclass_already_exists_error( text => $text ) )
        {
            $Log->exit() if $may_exit;
            NACL::C::Exceptions::DFM::Eventclass::AlreadyExists->convert_and_throw(
                exception => $exception );
        } else {
            $Log->exit() if $may_exit;
            $exception->throw();
        }
    };
    $Log->exit() if $may_exit;
    return $ret;
}

sub _add_custom_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( is-multi-current event-namespace-name event-class-name event-names about-message
            about-message is-allow-duplicates event-class-name-or-id
            )
    ];
    my %args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call

    my $response       = $apiset->eventclass_add_custom(%args);
    my $parsed_output  = $response->get_parsed_output();
    my $event_class_id = $parsed_output->[0]->{'event-class-id'};
    return $pkg->new(
        command_interface      => $command_interface,
        event_class_name_or_id => $event_class_id,
    );
    $Log->exit() if $may_exit;

}

=head2 modify

    NACL::C::DFM::Eventclass->modify(  command_interface => $ci,
                                'event-name' => $event_class_name,
                                'severity' => $severity,
                                  ...);
                                      or

    $Eventclass_object->modify(...);

=for html (Class or instance method)
This method is used to  modifies the severity of the event class.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'event-name' => $event_class_name >>

(Required)

=item C<< 'severity' => $severity >>

(Required)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw(  severity event-name event-class-name-or-id )),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _modify_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw(  severity event-name
            )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->eventclass_modify(%args);

    $Log->exit() if $may_exit;

}

# Return 1 if the exception text matches those messages which would indicate
# the eventclass already exists
sub _eventclass_already_exists_error {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate( @_, { text => { type => SCALAR } } );
    my $text       = $opts{text};
    if ( $text =~ /Event Class.*already exists in namespace.*/ ) {
        $Log->exit() if $may_exit;
        return 1;
    } else {
        $Log->exit() if $may_exit;
        return 0;
    }
}

# Return 1 if the exception text matches those messages which would indicate
# the eventclass does not exist
sub _eventclass_does_not_exist_error {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate( @_, { text => { type => SCALAR } } );
    my $text       = $opts{text};
    if ( $text =~ /Event Class.*does not exist/ ) {
        $Log->exit() if $may_exit;
        return 1;
    } else {
        $Log->exit() if $may_exit;
        return 0;
    }
}

sub _primary_keys_validate_spec {
    return ( 'event-class-name-or-id' => { type => SCALAR }, );
}

sub _create_to_pk_mapping {

    return { 'event-class-name' => 'event-class-name-or-id' };

}
1;
;
