# $Id//depot/prod/test/UM6.0_integ/lib/NACL/C/DFM/Group.pm$

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Group Component Module (auto-generated by NCAGE)
## @author dl-mpo-nacl-dev@netapp.com
## @status Review
## @pod here

=head1 NAME

NACL::C::DFM::Group

=head1 DESCRIPTION

C<NACL::C::DFM::Group> is a derived class of
L<NACL::C::Component::DFM|lib-NACL-C-Component-DFM-pm>. It represents access
to an DFM Group.  A related class is
L<NACL::CS::DFM::Group|lib-NACL-CS-DFM-Group-pm>, which represents
the State Info of an DFM Group.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 resource_key

=cut

package NACL::C::DFM::Group;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::DFM';
use NACL::CS::DFM::GroupMember;

use Params::Validate qw(validate validate_with SCALAR ARRAYREF HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $Group = NACL::C::DFM::Group->new( command_interface => $ci,
                                        'resource-key'    => $group_resource_key,
                            );
Return a new Group component object with the given attributes.

=cut

use Class::MethodMaker [
    new    => [ '-hash', '-init', 'new' ],
    scalar => 'resource_key',
];

=head2 find

    my $Group  = NACL::C::DFM::Group->find(command_interface => $ci, ...);
    my @Groups = NACL::C::DFM::Group->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my Group_state = $Group->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DFM::Group would return
a NACL::CS::DFM::Group object)

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 delete_member

    NACL::C::DFM::Group->delete_member(  command_interface => $ci,
                                'member-resource-key' => $member_resource_key,
                                'group-resource-key' => $group_resource_key,
                                  ...);
                                      or

    $Group_object->delete_member(...);

=for html (Class or instance method)
This method is used to  remove one member from a group. <hr> Error conditions: <ul> <li>
EGROUPDOESNOTEXIST - The group name or ID was not found. <li> EMEMBERNOTINGROUP -
The object specified is not a member in the group. <li> EOBJECTNOTFOUND - The
object specified was not found. </ul>

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'member-resource-key' => $member_resource_key >>

(Required)

=item C<< 'group-resource-key' => $group_resource_key >>

(Required for class method, Not Applicable for instance method)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub delete_member {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( member-resource-key group-resource-key
                    resource-key )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_member_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _delete_member_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( member-resource-key group-resource-key
         resource-key   )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    ## If the method is being invoked as an instance method, the primary key 
    ## 'resource-key' needs to be mapped to 'group-resource-key'
    if( defined $args{'resource-key'} ) { 
	$args{'group-resource-key'} = delete $args{'resource-key'};
    } 

    # Making the API Call
    $apiset->group_delete_member(%args);

    $Log->exit() if $may_exit;

}

=head2 add_member

    NACL::C::DFM::Group->add_member(  command_interface => $ci,
                                'member-resource-key' => $member_resource_key,
                                'group-resource-key' => $group_resource_key,
                                  ...);
                                      or

    $Group_object->add_member(...);

=for html (Class or instance method)
This method is used to  add a member to a group. <hr> Error conditions: <ul> <li> EGROUPDOESNOTEXIST - The
group name or ID was not found. <li> EMEMBERALREADYINGROUP - The proposed new
member is already in the group. <li> EOBJECTNOTFOUND - The proposed member name or
ID was not found. </ul>

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'member-resource-key' => $member_resource_key >>

(Required)

=item C<< 'group-resource-key' => $group_resource_key >>

(Required for class method, Not Applicable for instance method)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub add_member {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( member-resource-key group-resource-key
                    resource-key )
            ),

        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_add_member_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _add_member_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( member-resource-key group-resource-key
            resource-key )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    ## If the method is being invoked as an instance method, the primary key
    ## 'resource-key' needs to be mapped to 'group-resource-key'
    if( defined $args{'resource-key'} ) {
        $args{'group-resource-key'} = delete $args{'resource-key'};
    }

    # Making the API Call
    $apiset->group_add_member(%args);

    $Log->exit() if $may_exit;

}

=head2 create

    my $Group_object	= NACL::C::DFM::Group->create(
				command_interface => $ci,
                                'group-name' => $group_name,
                                  ...);

=for html (Class method)
This method is used to  create a new group. <hr> Error conditions: <ul> <li> EGROUPEXISTS - An attempt was
made to create a new group with the same name as an already existing group. Try again
with a different name. <li> EINVALIDGROUPNAME - The proposed group name was not
valid. <li> EPARENTGROUPDOESNOTEXIST - The parent group name or ID was not found.
<li> EPARENTGROUPNAMEAMBIGUOUS - The specified parent group name could refer to
2 or more groups. Try again with a parent group ID or a fully qualified parent group
name. </ul>

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'group-name' => $group_name >>

(Required)

=item C<< 'parent-resource-key' => $resource_key >>

(Optional) Resource key of the parent group.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);

    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {

            _optional_scalars(qw(group-name parent-resource-key )),
        },
        ignore_primary_keys => 1,
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _create_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( group-name parent-resource-key
            )
    ];
    my %args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response      = $apiset->group_create(%args);
    my $parsed_output = $response->get_parsed_output();
    my $resource_key = $parsed_output->[0]->{'resource-key'};

    $Log->exit() if $may_exit;

    # Create a new component object and return
    return $pkg->new(
        command_interface => $command_interface,
        resource_key      => $resource_key,
    );
}

=head2 destroy

    NACL::C::DFM::Group->destroy(  command_interface => $ci,
                                'resource-key' => $resource_key,
                                  ...);
                                      or

    $Group_object->destroy(...);

=for html (Class or instance method)
This method is used to  destroy an existing group. Child groups are destroyed recursively. If there is
any error, then no groups are destroyed. <hr> Error conditions: <ul> <li>
EGROUPDOESNOTEXIST - The group name or ID was not found. <li> EOBJECTAMBIGUOUS -
The specified group name could refer to 2 or more groups. Try again with a group ID or
a fully qualified group name. </ul>

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'resource-key' => $resource_key >>

(Required for class method, Not Applicable for instance method)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub destroy {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@_,
        additional_spec => {
	    _optional_scalars( qw( resource-key ) ),
                            },
              );

    my $ret =
      $pkg_or_obj->call_on_apiset(
        %opts,
            choices => [
                {   method => '_destroy_zapi',
                    interface => 'ZAPI',
                    set => 'DFM',
                },
            ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _destroy_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( resource-key
 )
        ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->group_destroy(%args);

    $Log->exit() if $may_exit;

}

sub _create_to_pk_mapping {

    return { 'group-resource-key' => 'resource-key' };

}

sub _primary_keys_validate_spec {
    return ( 'resource-key' => { type => SCALAR }, );
}

1;
