# $Id //depot/prod/test/UM6.0_integ/lib/NACL/C/DFM/Option.pm$

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Option Component Module (auto-generated by NCAGE)
## @author vinothg,dl-mpo-nacl-dev@netapp.com
## @status Review
## @pod here

=head1 NAME

NACL::C::DFM::Option

=head1 DESCRIPTION

C<NACL::C::DFM::Option> is a derived class of
L<NACL::C::Component|lib-NACL-C-Component-pm>. It represents access
to an DFM Option.  A related class is
L<NACL::CS::DFM::Option|lib-NACL-CS-DFM-Option-pm>, which represents
the State Info of an DFM Option.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 option_name

=cut

package NACL::C::DFM::Option;

use strict;
use warnings;

use NACL::CS::DFM::Option;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $Option = NACL::C::DFM::Option->new( command_interface => $ci,
                                   'option-name' => $option_name,
                            );

Return a new Option component object with the given attributes.

=cut

use Class::MethodMaker [
    new    => [ '-hash', '-init', 'new' ],
    scalar => 'option_name',
    scalar => [ { -type => 'NACL::C::DFM::DFMClient' }, 'command_interface', ],
];

=head2 find

    my $Option  = NACL::C::DFM::Option->find(command_interface => $ci, ...);
    my @Options = NACL::C::DFM::Option->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my Option_state = $NACL::C::DFM::Option->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DFM::Option would return
a NACL::CS::DFM::Option object)

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 set

    NACL::C::DFM::Option->set(  command_interface => $ci,
                                'option-name' => $option_name,
                                  ...);
                                      or

    $Option_object->set(...);

=for html (Class or instance method)
This method is used to  sets the value of the specified option either globally or for a specific object. To
set a value for a specific object, specify either object-name-or-id and
object-type.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'option-name' => $option_name >>

(Required for class method, Not Applicable for instance method)

=item C<< 'option-value' => $string >>

=item C<< 'object-name-or-id' => $string >>

=item C<< 'object-type' => $hashref >>

(Hashref, can be a reference to a hash or hashofarrays or hashofhashes)

=item C<< 'option-name' => $hashref >>

(Hashref, can be a reference to a hash or hashofarrays or hashofhashes)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub set {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( option-value resource-key option-name
                  )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_set_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _set_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( option-value option-name resource-key
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    $pkg_or_obj->_api_call_log_debug_opts(
        api      => 'option_set',
        api_opts => \%args,
    );

    # Making the API Call
    my $response      = $apiset->option_set(%args);
    my $parsed_output = $response->get_parsed_output();

    $Log->exit() if $may_exit;
    return $parsed_output->[0];
}

=head2 list

    NACL::C::DFM::Option->list(
                command_interface => $ci,
                filter            => {
                        'object-name-or-id' => $id,
                        'object-type' => $type,
                        'option-names' => [ $names ],
                        ...},
    );
        or

    $option->list( ... );

=for html (Class or instance method)
This method provides the same functionality a fetch/state of NACL::CS::DFM::Option.

This method is used to  lists dfm options and their current values. This will only return values for
hidden options if they are not the default value or are explicitly requested by
name. <p> Options can be listed globally, or for a specific object. To list option
values for a specific object, specify object-name-or-id and object-type. To
list available options for a specific type of object, specify only object-type
(output option values will not be returned in this case) <p> If you do not specify an
object, then the option values returned will be the global option values, and the
default values returned will be the global option defaults. <p> If you do specify
an object, then the option values returned will be the values as applied to the
specified object, and the default values returned will be the global option
values. <p> For example, thresholds are defined globally, but can be overridden
for specific objects. <p> To determine if the option value is overridden for a
specific object, you can compare the value to the default value.

Uses a DFM ZAPI APISet.

Returns one or more Component State objects of type NACL::CS::DFM::Option.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm>

=item C<< filter => HASHREF >>

(Required for class method, Not Applicable for instance method)
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm>

=back

=back

=cut

sub list {

    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    $Log->exit() if $may_exit;

    if ( ref $pkg_or_obj ) {

        return $pkg_or_obj->state();

    } else {

        return NACL::CS::DFM::Option->fetch(@_);
    }
}

sub _primary_keys_validate_spec {
    return ( 'option-name' => { type => SCALAR }, );
}

1;
