# $Id: //depot/prod/test/nacldev/lib/NACL/C/DFM/Quota.pm#1 $

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Quota Component Module (auto-generated by NCAGE)
## @author deepthis, dl-mpo-nacl-dev@netapp.com
## @status Review
## @pod here

=head1 NAME

NACL::C::DFM::Quota

=head1 DESCRIPTION

C<NACL::C::DFM::Quota> is a derived class of
L<NACL::C::Component::DFM|lib-NACL-C-Component-DFM-pm>. It represents access
to an DFM Quota.  A related class is
L<NACL::CS::DFM::Quota|lib-NACL-CS-DFM-Quota-pm>, which represents
the State Info of an DFM Quota.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=cut

package NACL::C::DFM::Quota;

use strict;
use warnings;

use NACL::CS::DFM::QuotaMailmap;
use NACL::CS::DFM::QuotaUser;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::DFM';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $Quota = NACL::C::DFM::Quota->new( command_interface => $ci,
                            );

Return a new Quota component object with the given attributes.

=cut

use Class::MethodMaker [
];

=head2 find

    my $Quota  = NACL::C::DFM::Quota->find(command_interface => $ci, ...);
    my @Quotas = NACL::C::DFM::Quota->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my Quota_state = $NACL::C::DFM::Quota->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DFM::Quota would return
a NACL::CS::DFM::Quota object)

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 mailmap_reset

    NACL::C::DFM::Quota->mailmap_reset(  command_interface => $ci,
                                  ...);
	or 

    $quota->mailmap_reset(...); 

(Class or Instance method)
This method is used to  deletes unused mailmap entries. The mailmaps of users for which the server is
tracking quotas are not removed.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub mailmap_reset {

    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);

    my %opts = $pkg->_common_validate_with( params => \@_, );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_mailmap_reset_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _mailmap_reset_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    # Making the API Call
    $apiset->quota_mailmap_reset();

    $Log->exit() if $may_exit;

}

=head2 mailformat_reset

    NACL::C::DFM::Quota->mailformat_reset(  command_interface => $ci,
                                  ...);
        or 
    $quota->mailformat_reset(...); 

(Class or Instance method)
This method is used to  restores the user quota alert email format to the system default.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub mailformat_reset {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);

    my %opts = $pkg->_common_validate_with( params => \@_, );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_mailformat_reset_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _mailformat_reset_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    # Making the API Call
    $apiset->quota_mailformat_reset();

    $Log->exit() if $may_exit;

}

=head2 mailformat_get

    NACL::C::DFM::Quota->mailformat_get(  command_interface => $ci,
                                  ...);
	or 
  
    $quota->mailformat_get(...); 

(Class or Instance method)
This method is used to  gets the current user quota alert email format.

Uses a DFM ZAPI APISet.
Returns the raw output (type: quota-mail-format, cdata) 

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, not for instance method) 
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub mailformat_get {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);

    my %opts = $pkg->_common_validate_with( params => \@_, );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_mailformat_get_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _mailformat_get_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    # Making the API Call
    my $response      = $apiset->quota_mailformat_get();
    my $parsed_output = $response->get_parsed_output();

    $Log->exit() if $may_exit;

    # Rerutn output
    return $parsed_output->[0]->{'quota-mail-format'};

}

=head2 mailformat_set

    NACL::C::DFM::Quota->mailformat_set(  command_interface => $ci,
                                'quota-mail-format' => $quota_mail_format,
                                  ...);
                                      or

    $Quota_object->mailformat_set( 'quota-mail-format' => $quota_mail_format,
				   ...);

=for html (Class or instance method)
This method is used to  sets a user quota alert email format. Overwrites the existing email format.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'quota-mail-format' => $quota_mail_format >>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub mailformat_set {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@_,
        additional_spec =>
          { 'quota-mail-format' => { type => SCALAR, optional => 1 }, },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_mailformat_set_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _mailformat_set_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( quota-mail-format
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->quota_mailformat_set(%args);

    $Log->exit() if $may_exit;

}

=head2 modify

    NACL::C::DFM::Quota->modify(  command_interface => $ci,
                                'object-name-or-id' => $object_name_or_id,
                                  ...);
                                      or

    $Quota_object->modify(...);

=for html (Class or instance method)
This method is used to  modify settings for user and group quotas on a particular volume or qtree. Tree
quotas are not supported. Note that the settings are modified on the Data ONTAP
storage system. Error conditions: <ul> <li> EACCESSDENIED - When the user does
not have DFM.Quota.FullControl capability on the specified volume or
qtree.</li> <li> EAPIERROR - An error occurred while changing the settings on the
Data ONTAP storage system.</li> <li> EOBJECTNOTFOUND - When volume or qtree
cannot be found.</li> <li> EINVALIDINPUT - When no changes to the limits are
specified or one or more of the limits contains an invalid value.</li> </ul>

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'object-name-or-id' => $object_name_or_id >>

=item C<< 'disk-space-soft-limit' => $string >>

=item C<< 'files-hard-limit' => $string >>

=item C<< 'disk-space-hard-limit' => $string >>

=item C<< 'files-soft-limit' => $string >>

=item C<< 'disk-space-threshold' => $string >>

=item C<< 'user-or-group-name' => $string >>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( disk-space-soft-limit files-hard-limit disk-space-hard-limit
                  files-soft-limit disk-space-threshold user-or-group-name
                  object-name-or-id
                  )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_modify_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _modify_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( disk-space-soft-limit files-hard-limit object-name-or-id
          disk-space-hard-limit files-soft-limit disk-space-threshold
          user-or-group-name
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->quota_modify(%args);

    $Log->exit() if $may_exit;

}

=head2 mailmap_set

    NACL::C::DFM::Quota->mailmap_set(  command_interface => $ci,
                                'quota-mailmap-infos' => $quota_mailmap_infos,
                                  ...);
                                      or

    $Quota_object->mailmap_set('quota-mailmap-infos' => $quota_mailmap_infos,
				...);

=for html (Class or instance method)
This method is used to  create a new mailmap for a quota-user or quota-group. This is a mapping from a
quota-user or quota-group name to an email address

Uses a DFM ZAPI APISet.

Output is a hash element like:
{
    'new-entries-count' => int1,
    'updated-entries-count' => int2
} 

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'validate-address' => $string >>

=item C<< 'quota-mailmap-infos' => [ $quota-mailmap-info1, $quota-mailmap-info2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub mailmap_set {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'quota-mailmap-infos' => { type => ARRAYREF, optional => 1 },
            _optional_scalars(
                qw( validate-address
                  )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_mailmap_set_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _mailmap_set_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( quota-mailmap-infos validate-address
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response      = $apiset->quota_mailmap_set(%args);
    my $parsed_output = $response->get_parsed_output();

    $Log->exit() if $may_exit;

    # Return
    return $parsed_output->[0];

}

1;
;
