# $Id//depot/prod/test/nacldev/lib/NACL/C/DFM/Rbac.pm$

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Rbac Component Module (auto-generated by NCAGE)
## @author dl-mpo-nacl-dev@netapp.com
## @status Review
## @pod here

=head1 NAME

NACL::C::DFM::Rbac

=head1 DESCRIPTION

C<NACL::C::DFM::Rbac> is a derived class of
L<NACL::C::Component::DFM|lib-NACL-C-Component-DFM-pm>. It represents access
to an DFM Rbac.  A related class is
L<NACL::CS::DFM::Rbac|lib-NACL-CS-DFM-Rbac-pm>, which represents
the State Info of an DFM Rbac.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 role_name_or_id

=cut

package NACL::C::DFM::Rbac;

use strict;
use warnings;

use NACL::CS::DFM::Rbac;
use NACL::C::DFM::RbacUser;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::DFM';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $Rbac = NACL::C::DFM::Rbac->new( command_interface => $ci,
                                   'role-name-or-id' => $role_name_or_id,
                            );

Return a new Rbac component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'role_name_or_id',
];

=head2 find

    my $Rbac  = NACL::C::DFM::Rbac->find(command_interface => $ci, ...);
    my @Rbacs = NACL::C::DFM::Rbac->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my Rbac_state = $NACL::C::DFM::Rbac->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DFM::Rbac would return
a NACL::CS::DFM::Rbac object)

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 modify

    NACL::C::DFM::Rbac->modify(  command_interface => $ci,
                                'role-name-or-id' => $role_name_or_id,
                                  ...);
                                      or

    $Rbac_object->modify(...);

=for html (Class or instance method)
This method is used to  modify an existing role name and/or its description.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'role-name-or-id' => $role_name_or_id >>

(Required for class method, Not Applicable for instance method)

=item C<< 'role-name-new' => $string >>

=item C<< 'role-description-new' => $string >>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( role-name-new role-description-new
                    )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_role_modify_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _role_modify_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( role-name-new role-name-or-id role-description-new
            )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->rbac_role_modify(%args);

#In Case, Rbac Object is created by the user using the "new" method specifying "role-name"
# for the primary key "role-name-or-id', then on calling "role_modify",the primary key which the role name needs to be updated

    if ( ( $args{'role-name-or-id'} !~ /^\d+$/ )
        && defined $args{'role-name-new'} )
    {
        $pkg_or_obj->role_name_or_id( $args{'role-name-new'} )
            if ref($pkg_or_obj);
    }

    $Log->exit() if $may_exit;

}

=head2 disinherit

    NACL::C::DFM::Rbac->disinherit(  command_interface => $ci,
                                'role-name-or-id' => $role_name_or_id,
                                  ...);
                                      or

    $Rbac_object->disinherit(...);

=for html (Class or instance method)
This method is used to  disinherit one or more roles. The effect is that the affected role will no longer
have the capabilities gained from the disinherited role(s). If disinherit-all
is not specified or is false, then disinherited-role-name-or-id must be
specified. If both disinherit-all and inherited-role-name-or-id are
specified, inherited-role-name-or-id is ignored.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'role-name-or-id' => $role_name_or_id >>

(Required for class method, Not Applicable for instance method)

=item C<< 'disinherited-role-name-or-id' => $string >>

=item C<< 'disinherit-all' => $string >>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub disinherit {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( disinherited-role-name-or-id disinherit-all
                    )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_role_disinherit_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _role_disinherit_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( disinherited-role-name-or-id role-name-or-id disinherit-all
            )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->rbac_role_disinherit(%args);

    $Log->exit() if $may_exit;

}

=head2 capability_remove

    NACL::C::DFM::Rbac->capability_remove(  command_interface => $ci,
                                'role-name-or-id' => $role_name_or_id,
                                  ...);
                                      or

    $Rbac_object->capability_remove(...);

=for html (Class or instance method)
This method is used to  remove one or more capabilities (resource/operation pair) from an existing
role. If delete-all is true, it removes all capabilities from given role.
Otherwise, it removes only the given capability (resource/operation pair). If
delete-all is not specified or is false, then operation and resource must be
specified.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'role-name-or-id' => $role_name_or_id >>

(Required for class method, Not Applicable for instance method)

=item C<< 'delete-all' => $string >>

=item C<< 'operation' => $string >>

=item C<< 'resource-name-or-id' => $string >>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub capability_remove {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( delete-all operation resource-name-or-id
                    )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_role_capability_remove_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _role_capability_remove_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( delete-all operation role-name-or-id resource-name-or-id
            )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->rbac_role_capability_remove(%args);

    $Log->exit() if $may_exit;

}

=head2 create

    my $Rbac_object	=
      NACL::C::DFM::Rbac->create(command_interface => $ci,
                                'role-name' => $role_name,
                                  ...);
                                      or

=for html (Class Method )
This method is used to  create a new role to the RBAC system

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'role-name' => $role_name >>

=item C<< 'owner-role-name-or-id' => $string >>

=item C<< 'description' => $string >>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);

    my %opts = $pkg->_common_validate_with(
        params              => \@_,
        ignore_primary_keys => 1,
        additional_spec     => {
            _optional_scalars(
                qw( role-name owner-role-name-or-id description
                    )
            ),
        },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_role_create_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _role_create_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( role-name owner-role-name-or-id description
            )
    ];
    my %args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response        = $apiset->rbac_role_create(%args);
    my $parsed_output   = $response->get_parsed_output();
    my $role_name_or_id = $parsed_output->[0]->{"role-id"};
    $Log->exit() if $may_exit;

    return $pkg->new(
        command_interface => $command_interface,
        role_name_or_id   => $role_name_or_id,
    );

}

=head2 destroy

    NACL::C::DFM::Rbac->destroy(  command_interface => $ci,
                                'role-name-or-id' => $role_name_or_id,
                                  ...);
                                      or

    $Rbac_object->destroy(...);

=for html (Class or instance method)
This method is used to  destroy an existing role from the RBAC system

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'role-name-or-id' => $role_name_or_id >>

(Required for class method, Not Applicable for instance method)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub destroy {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( role-name-or-id
                    )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_role_destroy_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _role_destroy_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( role-name-or-id
            )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->rbac_role_destroy(%args);

    $Log->exit() if $may_exit;

}

=head2 add_role_to_user

	 $Rbac_object->add_role_to_user('user-name-or-id' => $user_name_or_id,
					...);

=for html (Instance Method )
This method is used to  add a new role to the existing Rbac user

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'user-name-or-id' => $user_name_or_id >>

(Required, Name or Id of the user to whom the role must be added)


=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub add_role_to_user {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);

 
    # Determine whether to redirect to the Component or STask
    my $is_c_or_t     = $pkg->is_c_or_t();
    my $rbac_user_pkg = "NACL::${is_c_or_t}::DFM::RbacUser";
	my %opts = $pkg->_common_validate_with(
        params              => \@_,
		allow_extra         => 1,
    );
    my $ret           = $rbac_user_pkg->role_add(%opts);
    $Log->exit() if $may_exit;
    return $ret;
}

=head2 remove_role_from_user

   
    $Rbac_object->remove_role_from_user(...);

=for html (Instance method)
This method is used to  remove an existing role from the Rbac user

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'user-name-or-id' => $user_name_or_id >>

(Required, name or id of the user)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub remove_role_from_user {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
		allow_extra     => 1,
    );

    # Determine whether to redirect to the Component or STask
    my $is_c_or_t     = $pkg_or_obj->is_c_or_t();
    my $rbac_user_pkg = "NACL::${is_c_or_t}::DFM::RbacUser";
    my $ret           = $rbac_user_pkg->role_remove(%opts);
    $Log->exit() if $may_exit;
    return $ret;
}

=head2 capability_add

    NACL::C::DFM::Rbac->capability_add(  command_interface => $ci,
                                'operation' => $operation,
                                'role-name-or-id' => $role_name_or_id,
                                'resource-name-or-id' => $resource_name_or_id,
                                  ...);
                                      or

    $Rbac_object->capability_add(...);

=for html (Class or instance method)
This method is used to  add a resource/operation capability to a role.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'operation' => $operation >>

(Required)

=item C<< 'role-name-or-id' => $role_name_or_id >>

(Required for class method, Not Applicable for instance method)

=item C<< 'resource-name-or-id' => $resource_name_or_id >>

(Required)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub capability_add {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@_,
        additional_spec =>
            { _optional_scalars(qw( operation resource-name-or-id )), },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_role_capability_add_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _role_capability_add_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( role-name-or-id operation resource-name-or-id
            )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->rbac_role_capability_add(%args);

    $Log->exit() if $may_exit;

}

=head2 inherit

    NACL::C::DFM::Rbac->inherit(  command_interface => $ci,
                                'role-name-or-id' => $role_name_or_id,
                                'inherited-role-name-or-id' => $inherited_role_name_or_id,
                                  ...);
                                      or

    $Rbac_object->inherit(...);

=for html (Class or instance method)
This method is used to  inherit from a role. The effect is that the affected role will gain the
capabilities from the inherited role.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'role-name-or-id' => $role_name_or_id >>

(Required for class method, Not Applicable for instance method)

=item C<< 'inherited-role-name-or-id' => $inherited_role_name_or_id >>

(Required)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub inherit {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@_,
        additional_spec =>
            { _optional_scalars(qw( inherited-role-name-or-id )), },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_role_inherit_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _role_inherit_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( inherited-role-name-or-id role-name-or-id
            )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->rbac_role_inherit(%args);

    $Log->exit() if $may_exit;

}

=head2 operation_info_list

    NACL::C::DFM::Rbac->operation_info_list(  command_interface => $ci,
                                  ...);
                                        or

    $Rbac_object->operation_info_list(...);

=for html (Class or instance method)
This method is used to  get information about an existing operation or all operations in the system.

Uses a DFM ZAPI APISet.

This method returns the list of all operations or a specific operation depending on the arguments passed;

$VAR1 = [
			{
            'operation-description' => ..
            'operation-synopsis' => ..                                                   
            'operation-name' => ..
			}
			{
            'operation-description' => ..
            'operation-synopsis' => ..                                                   
            'operation-name' => ..
			}
			.
			.
			.
			{
            'operation-description' => ..
            'operation-synopsis' => ..                                                   
            'operation-name' => ..
			}
		  
        ];

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'operation-name' => $string >>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub operation_info_list {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $Log->debug(
        "Opts to 'operation_info_list' front end are:\n" . Dumper( \@_ ) );

    my %opts = $pkg_or_obj->_common_validate_with(
        params              => \@_,
        ignore_primary_keys => 1,
        additional_spec     => {
            _optional_scalars(
                qw( operation-name
                    )
            ),
        },
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_operation_info_list_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _operation_info_list_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( operation-name
            )
    ];

    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    my $response      = $apiset->rbac_operation_info_list(%args);
    my $parsed_output = $response->get_parsed_output();
    my $list
        = $parsed_output->[0]->{'operation-list'}->[0]->{'rbac-operation'};
    my @arr;

    foreach my $row ( @{$list} ) {
        my $newrow;
        $newrow->{'operation-description'}
            = delete $row->{'operation-details'}->[0]
            ->{'rbac-operation-details'}->[0]->{'operation-description'};
        $newrow->{'operation-synopsis'}
            = delete $row->{'operation-details'}->[0]
            ->{'rbac-operation-details'}->[0]->{'operation-synopsis'};
        $newrow->{'operation-name'} = delete $row->{'operation-name'};
        push( @arr, $newrow );
    }
    $Log->exit() if $may_exit;
    return \@arr;
}

=head2 operation_create

   
      NACL::C::DFM::Rbac->operation_create(command_interface => $ci,
                                'operation' => $operation,
                                  ...);
                                      or

    $Rbac_object->operation_create(...);

=for html (Class or instance method)
This method is used to  create a new operation in the RBAC system. An operation is an ability to perform an
action on a particular resource type. An operation is tied to a specific
application so that different applications are able to manage access control
that are specific to them.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'operation' => $hashref >>

(Hashref, can be a reference to a hash or hashofarrays or hashofhashes)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub operation_create {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);

    my %opts = $pkg->_common_validate_with(
        params              => \@_,
        ignore_primary_keys => 1,
        additional_spec =>
            { 'operation' => { type => HASHREF, optional => 1 }, },
    );

    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_operation_create_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;

}

sub _operation_create_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( operation
            )
    ];
    my %args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->rbac_operation_create(%args);
    $Log->exit() if $may_exit;

}

=head2 operation_destroy

    NACL::C::DFM::Rbac->operation_destroy(  command_interface => $ci,
                                'operation-name' => $operation_name,
                                  ...);
                                      or

    $Rbac_object->operation_destroy(...);

=for html (Class or instance method)
This method is used to  permanently destroy the operation from RBAC system.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'operation-name' => $operation_name >>

(Required)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub operation_destroy {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        ignore_primary_keys => 1,
        params              => \@_,
        additional_spec     => {
            _optional_scalars(
                qw( operation-name
                    )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_operation_destroy_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _operation_destroy_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [ qw( operation-name ) ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->rbac_operation_destroy(%args);

    $Log->exit() if $may_exit;

}

=head2 find_or_add

    $Rbac= NACL::C::DFM::Rbac->find_or_add(
        command_interface       => $client,
        find_specific_options   => { 'role-name-or-id' => $role_name_or_id, },
        create_specific_options => {
            'role-name '                => $role_name,
			'owner-role-name-or-id'     => $owner_role_name_or_id,
			'description'               => $description,
        },
        _was_created => \$role_added_flag,
    );
        
This convenience method finds a particular Rbac entry, else creates a new
role entry. This is a wrapper around
L<find_or_create|lib-NACL-C-Component-pm/find_or_create> (see that page
for a detailed explanation of the options accepted)

The example code shown above can be used instead of having lengthier
code such as this:

    try {
        $Rbac = NACL::C::DFM::Rbac->find(
            'command_interface' => $client,
            filter              => { 'role-name-or-id' => $role_name_or_id, }
        );
    } catch NACL::Exceptions::NoElementsFound with {
        $Rbac = NACL::C::DFM::Rbac->(
            command_interface        => $client,
			'role-name'                => $role_name,
		    'owner-role-name-or-id'     => $owner_role_name_or_id,
			'description'               => $description,    
        );
        $user_added_flag = 1;
    };

=cut

sub find_or_add {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @objs = $pkg->find_or_create( @_, custom_create_command => 'create' );
    $Log->exit() if $may_exit;
    return wantarray ? @objs : $objs[0];
}

sub _create_to_pk_mapping {

    return { 'role-name' => 'role-name-or-id' };

}

sub _primary_keys_validate_spec {
    return ( 'role-name-or-id' => { type => SCALAR }, );
}

1;
;
