# $Id//depot/prod/test/nacldev/lib/NACL/C/DFM/RbacUser.pm$

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary RbacUser Component Module (auto-generated by NCAGE)
## @author dl-mpo-nacl-dev@netapp.com
## @status Review
## @pod here

=head1 NAME

NACL::C::DFM::RbacUser

=head1 DESCRIPTION

C<NACL::C::DFM::RbacUser> is a derived class of
L<NACL::C::Component::DFM|lib-NACL-C-Component-DFM-pm>. It represents access
to an DFM RbacUser.  A related class is
L<NACL::CS::DFM::RbacUser|lib-NACL-CS-DFM-RbacUser-pm>, which represents
the State Info of an DFM RbacUser.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 user_name

=cut

package NACL::C::DFM::RbacUser;

use strict;
use warnings;

use NACL::CS::DFM::RbacUser;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::DFM';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $RbacUser = NACL::C::DFM::RbacUser->new( command_interface => $ci,
                                   'user-name' => $user_name,
                            );

Return a new RbacUser component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'user_name',
];

=head2 find

    my $RbacUser  = NACL::C::DFM::RbacUser->find(command_interface => $ci, ...);
    my @RbacUsers = NACL::C::DFM::RbacUser->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my RbacUser_state = $NACL::C::DFM::RbacUser->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DFM::RbacUser would return
a NACL::CS::DFM::RbacUser object)

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 add

    NACL::C::DFM::RbacUser->add(  command_interface => $ci,
                                'user-name' => $user_name,
                                  ...);
                                      or


=for html (Class method)
This method is used to  add a new user.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'user-name' => $user_name >>

=item C<< 'role' => $role >>

=item C<< 'user-type' => $user_type >>

(Required)

=item C<< 'email-address' => $string >>

=item C<< 'password' => $password >>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub add {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params              => \@_,
        ignore_primary_keys => 1,
        additional_spec     => {
            _optional_scalars(
                qw( email-address user-name password role user-type
                    )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_user_add_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _user_add_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $user_name         = $opts{'user-name'};

    my $allowed_options = [
        qw( email-address user-name password role user-type
            )
    ];
    my %args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response      = $apiset->rbac_user_add(%args);    

    $Log->exit() if $may_exit;
    return $pkg->new(
        command_interface => $command_interface,
        'user_name' => $user_name,
    );

}

=head2 remove

    NACL::C::DFM::RbacUser->remove(  command_interface => $ci,
                                'user-name' => $user_name,
                                  ...);
                                      or

    $RbacUser_object->remove(...);

=for html (Class or instance method)
This method is used to  remove an existing user.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'user-name' => $user-name >>

(Required only for Class method)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub remove {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@_, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_user_remove_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _user_remove_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( user-name
            )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    $pkg_or_obj->_api_call_log_debug_opts(
        api      => 'rbac_user_remove',
        api_opts => \%args,
    );

    # Making the API Call
    $apiset->rbac_user_remove(%args);

    $Log->exit() if $may_exit;

}

=head2 modify

    NACL::C::DFM::RbacUser->modify(  command_interface => $ci,
                                'user-name' => $user_name,
                                  ...);
                                      or

    $RbacUser_object->modify(...);

=for html (Class or instance method)
This method is used to  modify an existing user's email address.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'user-name' => $user_name >>

(Required for Class Method)

=item C<< 'email-address-new' => $string >>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( email-address-new role-new user-name password-new
                    )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_user_modify_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _user_modify_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( email-address-new role-new user-name password-new
            )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    $pkg_or_obj->_api_call_log_debug_opts(
        api      => 'rbac_user_modify',
        api_opts => \%args,
    );

    # Making the API Call
    $apiset->rbac_user_modify(%args);

    $Log->exit() if $may_exit;

}
=head2 role_add

    NACL::C::DFM::RbacUser->role_add(  command_interface => $ci,
                                'user-name' => $user_name,
                                'role-name-or-id' => $role_name_or_id,
                                  ...);
                                      or

     $RbacUser_object->role_add(...);

=for html (Class or instance method)
This method is used to  add an existing role to an existing user or usergroup. The user effectively gains
the capabilities from the role and its inherited roles. As for a usergroup, all
members of the usergroup will gain the capabilities assigned to that role and its
inherited roles.

Uses a DFM ZAPI APISet.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'user-name' => $user_name >>

(Required for Class Method)

=item C<< 'role-name-or-id' => $role_name_or_id >>

(Required)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub role_add {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@_,
        additional_spec =>
            { _optional_scalars(qw( user-name role-name-or-id )), },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_user_role_add_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _user_role_add_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( user-name role-name-or-id
            )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response      = $apiset->rbac_user_role_add(%args);
    my $parsed_output = $response->get_parsed_output();

    $Log->exit() if $may_exit;

}

=head2 find_or_add

    $RbacUser = NACL::C::DFM::RbacUser->find_or_add(
        command_interface       => $client,
        find_specific_options   => { 'user-name' => $user_name, },
        create_specific_options => {
            user-name                 => $user_name,
			email-address             => $email_address,
        },
        _was_created => \$user_added_flag,
    );

This convenience method finds a particular Rbac User entry, else adds a new
user entry. This is a wrapper around
L<find_or_create|lib-NACL-C-Component-pm/find_or_create> (see that page
for a detailed explanation of the options accepted)

The example code shown above can be used instead of having lengthier
code such as this:

    try {
        $RbacUser = NACL::C::DFM::RbacUser->find(
            'command_interface' => $client,
            filter              => { 'user-name' => $user_name, }
        );
    } catch NACL::Exceptions::NoElementsFound with {
        $RbacUser = NACL::C::DFM::RbacUser->add(
            command_interface   => $client,
			user-name           => $user_name,
            email-address       => $email_address,
        );
        $user_added_flag = 1;
    };

=cut

sub find_or_add {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @objs = $pkg->find_or_create( @_, custom_create_command => 'add' );
    $Log->exit() if $may_exit;
    return wantarray ? @objs : $objs[0];
}

sub _create_to_pk_mapping {

    return { 'user-name' => 'user-name' };

}

sub _primary_keys_validate_spec {
    return ( 'user-name' => { type => SCALAR }, );
}

1;
;
