# $Id//depot/prod/test/UM6.0_integ/lib/NACL/C/DFM/Resource.pm$ 

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Resource Component Module (auto-generated by NCAGE)
## @author dl-mpo-nacl-dev@netapp.com
## @status Review
## @pod here

=head1 NAME

NACL::C::DFM::Resource

=head1 DESCRIPTION

C<NACL::C::DFM::Resource> is a derived class of
L<NACL::C::Component|lib-NACL-C-Component-pm>. It represents access
to an DFM Resource.  A related class is
L<NACL::CS::DFM::Resource|lib-NACL-CS-DFM-Resource-pm>, which represents
the State Info of an DFM Resource.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 qualified_name

=head2 type

=cut

package NACL::C::DFM::Resource;

use strict;
use warnings;

#use NACL::CS::DFM::Resource;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF HASHREF); 
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $Resource = NACL::C::DFM::Resource->new( command_interface => $ci,
                                   'qualified-name' => $qualified_name,
                                   'type' => $type,
                            );

Return a new Resource component object with the given attributes.

=cut

use Class::MethodMaker [
    new    => [ '-hash', '-init', 'new' ],
    scalar => 'resource_key', 
    scalar => [ { -type => 'NACL::C::DFM::DFMClient' }, 'command_interface', ],
];

=head2 find

    my $Resource  = NACL::C::DFM::Resource->find(command_interface => $ci, ...);
    my @Resources = NACL::C::DFM::Resource->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my Resource_state = $NACL::C::DFM::Resource->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DFM::Resource would return
a NACL::CS::DFM::Resource object)

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 lookup

    NACL::C::DFM::Resource->lookup(  command_interface => $ci,
                                'type' => $type,
                                'qualified-name' => $qualified_name,
                                  ...);
                                      or

    $Resource_object->lookup(...);

=for html (Class or instance method)
This method is used to  lookup a managed resource by its fully qualified name. Returns the resource key
for the managed resource.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'type' => $type >>

(Required for class method, Not Applicable for instance method)

=item C<< 'qualified-name' => $qualified_name >>

(Required for class method, Not Applicable for instance method)

The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub lookup {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
	params => \@_,
        additional_spec => {
           _optional_scalars(
                qw( qualified-name type) 
            ), 
                            },
              );

    my $ret =
      $pkg_or_obj->call_on_apiset(
        %opts,
	    choices => [
		{   method => '_lookup_zapi',
                    interface => 'ZAPI',
                    set => 'DFM',
		},
	    ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}
sub _lookup_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts = $pkg_or_obj->_validate_backend_opts(@_); 

    my $command_interface = delete $opts{command_interface}; 
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( qualified-name type
 )
        ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
	copy => $allowed_options,
	target => \%args,
    );

    # Making the API Call 
    my $response = $apiset->resource_lookup(%args);
    my $parsed_output = $response->get_parsed_output();
    my $resource_key            = $parsed_output->[0]->{'resource-key'};

    $Log->exit() if $may_exit;
   
    return $resource_key;

}

1;
