# $Id: //depot/prod/test/main/lib/NACL/C/DFM/ResourcePool.pm$

#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ResourcePool Component Module (auto-generated by NCAGE)
## @author deepthis,dl-mpo-nacl-dev@netapp.com
## @status Review
## @pod here

=head1 NAME

NACL::C::DFM::ResourcePool

=head1 DESCRIPTION

C<NACL::C::DFM::ResourcePool> is a derived class of
L<NACL::C::Component|lib-NACL-C-Component-pm>. It represents access
to an DFM ResourcePool.  A related class is
L<NACL::CS::DFM::ResourcePool|lib-NACL-CS-DFM-ResourcePool-pm>, which represents
the State Info of an DFM ResourcePool.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 resource_pool

=head2 resource_pool

=cut

package NACL::C::DFM::ResourcePool;

use strict;
use warnings;

use NACL::CS::DFM::ResourcePool;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $ResourcePool = NACL::C::DFM::ResourcePool->new( command_interface => $ci,
                                   'resource-key' => $resource_pool,
                            );

Return a new ResourcePool component object with the given attributes.

=cut

use Class::MethodMaker [
    new    => [ '-hash', '-init', 'new' ],
    scalar => 'resource_key',
    scalar => 'resource_pool',
    scalar => [ { -type => 'NACL::C::DFM::DFMClient' }, 'command_interface', ],
];

=head2 find

    my $ResourcePool  = NACL::C::DFM::ResourcePool->find(command_interface => $ci, ...);
    my @ResourcePools = NACL::C::DFM::ResourcePool->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my ResourcePool_state = $NACL::C::DFM::ResourcePool->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DFM::ResourcePool would return
a NACL::CS::DFM::ResourcePool object)

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 aggregate_add

    NACL::C::DFM::ResourcePool->aggregate_add(  command_interface => $ci,
                                'aggregate' => $aggregate,
                                'resource-pool' => $resource_pool,
                                  ...);
                                      or

    $ResourcePool_object->aggregate_add(...);

=for html (Class or instance method)
This method is used to  add an aggregate to a resource pool. An aggregate may only belong to a single
resource pool.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'aggregate' => $aggregate >>
(Required)

=item C<< 'resource-pool' => $resource_pool >>
(Required for class method, not applicable for instance method)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub aggregate_add {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
    params => \@_,
        additional_spec => {
        _optional_scalars( qw( aggregate resource-pool) ),
                            },
              );

    my $ret =
      $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
        {   method => '_aggregate_add_zapi',
                    interface => 'ZAPI',
                    set => 'DFM',
        },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}
sub _aggregate_add_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( resource-key resource-pool aggregate
 )
        ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
    copy => $allowed_options,
    target => \%args,
    );

    if( defined $args{'resource-key'} ){
         $args{'resource-pool'} = delete $args{'resource-key'};
    }

    # Making the API Call
    $apiset->resource_pool_aggregate_add(%args);

    $Log->exit() if $may_exit;

}

=head2 destroy

    NACL::C::DFM::ResourcePool->destroy(  command_interface => $ci,
                                'resource-pool' => $resource_pool,
                                  ...);
                                      or

    $ResourcePool_object->destroy(...);

=for html (Class or instance method)
This method is used to  destroy an existing resource pool.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'resource-pool' => $resource_pool >>

(Required for class method, Not Applicable for instance method)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub destroy {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
    params => \@_,
        additional_spec => {
        _optional_scalars( qw( resource-pool)),
                            },
              );

    my $ret =
      $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
        {   method => '_destroy_zapi',
                    interface => 'ZAPI',
                    set => 'DFM',
        },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}
sub _destroy_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( resource-key resource-pool
 )
        ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
    copy => $allowed_options,
    target => \%args,
    );

    if( defined $args{'resource-key'} ){
         $args{'resource-pool'} = delete $args{'resource-key'};
    }

    # Making the API Call
    $apiset->resource_pool_destroy(%args);

    $Log->exit() if $may_exit;

}

=head2 create

    my $ResourcePool_object	=
      NACL::C::DFM::ResourcePool->create(command_interface => $ci,
                                'name' => $name,
                'description' => $description,
                                  ...);

=for html (Class method)
This method is used to  create a new resource pool.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'name' => $name >>
(Required)

=item C<< 'description' => $string >>
(Optional)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);

    my %opts = $pkg->_common_validate_with(
    params => \@_,
        additional_spec => {
            _optional_scalars( qw( name description
) ),                            },
    ignore_primary_keys => 1,
              );

    my $ret =
      $pkg->call_on_apiset(
        %opts,
        choices => [
        {   method => '_create_zapi',
                    interface => 'ZAPI',
                    set => 'DFM',
        },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}
sub _create_zapi {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

        my $command_interface = delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( name description
 )
        ];
    my %args;
    $pkg->_hash_copy(
        source => \%opts,
    copy => $allowed_options,
    target => \%args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'resource_pool_create',
        api_opts => \%args,
    );

    # Making the API Call
    my $response = $apiset->resource_pool_create(%args);
    my $parsed_output = $response->get_parsed_output();
    my $resource_pool_key = $parsed_output->[0]->{'resource-key'};

    $Log->exit() if $may_exit;
    return $pkg->new(
        command_interface => $command_interface,
        resource_key      => $resource_pool_key,
        resource_pool     => $args{name}
    );

}

=head2 aggregate_remove

    NACL::C::DFM::ResourcePool->aggregate_remove(  command_interface => $ci,
                                'aggregate' => $aggregate,
                                'resource-pool' => $resource_pool,
                                  ...);
                                      or

    $ResourcePool_object->aggregate_remove(...);

=for html (Class or instance method)
This method is used to  remove an aggregate from a resource pool.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'aggregate' => $aggregate >>

(Required)

=item C<< 'resource-pool' => $resource_pool >>

(Required for class method, Not Applicable for instance method)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub aggregate_remove {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
    params => \@_,
        additional_spec => {
            _optional_scalars( qw( aggregate resource-pool) ),
                            },
              );

    my $ret =
      $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
        {   method => '_aggregate_remove_zapi',
                    interface => 'ZAPI',
                    set => 'DFM',
        },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}
sub _aggregate_remove_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( resource-key resource-pool aggregate
 )
        ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
    copy => $allowed_options,
    target => \%args,
    );

    if( defined $args{'resource-key'} ){
         $args{'resource-pool'} = delete $args{'resource-key'};
    }

    # Making the API Call
    $apiset->resource_pool_aggregate_remove(%args);

    $Log->exit() if $may_exit;

}

=head2 modify

    NACL::C::DFM::ResourcePool->modify(  command_interface => $ci,
                                'resource-pool' => $resource_pool,
                                  ...);
                                      or

    $ResourcePool_object->modify(...);

=for html (Class or instance method)
This method is used to  modify an existing resource pool.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'resource-pool' => $resource_pool >>

(Required for class method, Not Applicable for instance method)

=item C<< 'name' => $string >>

=item C<< 'description' => $string >>

=item C<< 'resource-pool' => $hashref >>

(Hashref, can be a reference to a hash or hashofarrays or hashofhashes)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
    params => \@_,
        additional_spec => {
            'resource-pool' => { type => HASHREF, optional => 1 },
            _optional_scalars( qw( name description
) ),                            },
              );

    my $ret =
      $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
        {   method => '_modify_zapi',
                    interface => 'ZAPI',
                    set => 'DFM',
        },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}
sub _modify_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( resource-key resource-pool name description
 )
        ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
    copy => $allowed_options,
    target => \%args,
    );

    if( defined $args{'resource-key'} ){
         $args{'resource-pool'} = delete $args{'resource-key'};
    }

    # Making the API Call
    $apiset->resource_pool_modify(%args);

    $Log->exit() if $may_exit;

}

sub _primary_keys_validate_spec {
    return (
        'resource-key' => { type => SCALAR },
    );
}

# Overriding the init function
sub init {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => { 'resource_key' => { type => SCALAR } },
        allow_extra => 1  # Let the other options be validated by the base class
    );

    $self->SUPER::init(%opts);
}

1;
