# $Id  //depot/prod/test/UM6.0_integ/lib/NACL/C/DFM/Snapshot.pm#1$

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Snapshot Component Module (auto-generated by NCAGE)
## @author vinothg,dl-mpo-nacl-dev@netapp.com
## @status Review
## @pod here

=head1 NAME

NACL::C::DFM::Snapshot

=head1 DESCRIPTION

C<NACL::C::DFM::Snapshot> is a derived class of
L<NACL::C::Component|lib-NACL-C-Component-pm>. It represents access
to an DFM Snapshot.  A related class is
L<NACL::CS::DFM::Snapshot|lib-NACL-CS-DFM-Snapshot-pm>, which represents
the State Info of an DFM Snapshot.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 snapshot_identifiers

=cut

package NACL::C::DFM::Snapshot;

use strict;
use warnings;

use NACL::CS::DFM::Snapshot;
use NACL::CS::DFM::SnapshotLocationResultInfo;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $Snapshot = NACL::C::DFM::Snapshot->new( command_interface => $ci,
                                   'snapshot-name' => $snapshot_name,
                                   'volume-resource-key' => $volume_resource_key,
                            );

Return a new Snapshot component object with the given attributes.

=cut

use Class::MethodMaker [
    new    => [ '-hash', '-init', 'new' ],
    scalar => 'snapshot_name',
    scalar => 'volume_resource_key',
    scalar => [ { -type => 'NACL::C::DFM::DFMClient' }, 'command_interface', ],
];

=head2 find

    my $Snapshot  = NACL::C::DFM::Snapshot->find(command_interface => $ci, ...);
    my @Snapshots = NACL::C::DFM::Snapshot->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my Snapshot_state = $Snapshot->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DFM::Snapshot would return
a NACL::CS::DFM::Snapshot object)

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 get_location

    NACL::C::DFM::Snapshot->get_location(  command_interface => $ci,
                                'snapshot-identifiers' => \@snapshot_identifiers,
                                  ...);
                                      or

    $Snapshot_object->get_location( 'snapshot-identifiers' => \@snapshot_identifiers);

=for html (Class or instance method)
This method is used to  gets location of the replicas of the requested Snapshot copies.
It returns an object of the class NACL::C::DFM::SnapshotLocationResultInfo

 L<NACL::CS::DFM::SnapshotLocationResultInfo|lib-NACL-CS-DFM-SnapshotLocationResultInfo-pm/snapshot-locations>
Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'snapshot-identifiers' => [ $snapshot-identifier1, $snapshot-identifier2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

=item C<< 'method-timeout' => $time_in_seconds >>
(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub get_location {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@_,
        additional_spec =>
          { 'snapshot-identifiers' => { type => ARRAYREF, optional => 1 }, }
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_get_location_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _get_location_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( snapshot-identifiers
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response      = $apiset->snapshot_get_location(%args);
    my $parsed_output = $response->get_parsed_output();
    my $output =
      $parsed_output->[0]->{'snapshot-location-results'}->[0]
      ->{'snapshot-location-result-info'}->[0]->{'snapshot-locations'}->[0]
      ->{'snapshot-location-info'};

    my @state_objs = ();
    foreach my $row ( @{$output} ) {
        my $obj = NACL::CS::DFM::SnapshotLocationResultInfo->new();
        $obj->_set_fields( row => $row );
        push( @state_objs, $obj );
    }
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];

}

sub _primary_keys_validate_spec {
    return (
        'snapshot-name'       => { type => SCALAR, optional => 1 },
        'volume-resource-key' => { type => SCALAR, optional => 1 },
    );
}

1;
