# $Id //depot/prod/test/UM6.0_integ/lib/NACL/C/DFM/StorageService.pm #1 $

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StorageService Component Module (auto-generated by NCAGE)
## @author vinothg, dl-mpo-nacl-dev@netapp.com
#
## @status Review
## @pod here

=head1 NAME

NACL::C::DFM::StorageService

=head1 DESCRIPTION

C<NACL::C::DFM::StorageService> is a derived class of
L<NACL::C::Component|lib-NACL-C-Component-pm>. It represents access
to a DFM StorageService.  A related class is
L<NACL::CS::DFM::StorageService|lib-NACL-CS-DFM-StorageService-pm>, which represents
the State Info of a DFM StorageService.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 storage_service_resource-key

=cut

package NACL::C::DFM::StorageService;

use strict;
use warnings;
use NACL::C::DFM::Job;

use NACL::CS::DFM::StorageService;
use NACL::CS::DFM::StorageServiceMember;
use NACL::CS::DFM::StorageServiceSubscription;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::DFM';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF HASHREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $StorageService = NACL::C::DFM::StorageService->new( command_interface => $ci,
                                   'storage-service-resource-key' => $storage_service_resource_key,
                            );

Return a new StorageService component object with the given attributes.

=cut

use Class::MethodMaker [ scalar => 'storage_service_resource_key', ];

=head2 find

    my $StorageService  = NACL::C::DFM::StorageService->find(command_interface => $ci, ...);
    my @StorageServices = NACL::C::DFM::StorageService->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my StorageService_state = $StorageService->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DFM::StorageService would return
a NACL::CS::DFM::StorageService object)

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 subscribe

    NACL::C::DFM::StorageService->subscribe(  command_interface => $ci,
                                'subscription-members' => $subscription_members,
                                'storage-service-resource-key' => $storage_service_resource_key,
                                'subscription-context' => $subscription_context,
                                  ...);
                                      or

    $StorageService_object->subscribe(...);

=for html (Class or instance method)
This method is used to  subscribes objects to a storage service. Currently, volumes are subscribable
objects.It returns a Job Component object.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'subscription-members' => $subscription_members >>

(Required)

=item C<< 'storage-service-resource-key' => $storage_service_resource_key >>

(Required for class method, Not Applicable for instance method)

=item C<< 'subscription-context' => $subscription_context >>

(Required)

=item C<< 'dry-run' => $string >>

=item C<< 'perform-inline-discovery' => $string >>

=item C<< 'subscription-members' => [ $subscription-member1, $subscription-member2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub subscribe {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'subscription-members' => { type => ARRAYREF, optional => 1 },
            _optional_scalars(
                qw( dry-run perform-inline-discovery subscription-context
                  )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_subscribe_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _subscribe_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( dry-run subscription-members perform-inline-discovery
          subscription-context storage-service-resource-key
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response      = $apiset->storage_service_subscribe(%args);
    my $parsed_output = $response->get_parsed_output();
    my $job_id        = $parsed_output->[0]->{'job-id'};

    $Log->exit() if $may_exit;
    return NACL::C::DFM::Job->new(
        'command_interface' => $command_interface,
        'job-id'            => $job_id,
    );

}

=head2 protection_update_start

    NACL::C::DFM::StorageService->protection_update_start(  command_interface => $ci,
                                'storage-service-resource-key' => $storage_service_resource_key,
                                  ...);
                                      or

    $StorageService_object->protection_update_start(...);

=for html (Class or instance method)
This method is used to update data protection relationships for one or more members of the storage
service. Storage service has a built-in topology relationship defined between
the nodes. When the storage-service-subscribe API is called, protection
relationships are created but not initialized. The first call to the
storage-service-protection-update-start API triggers initialization
(baseline data transfer). Subsequent calls trigger protection relationship
updates.It returns a Job Component object.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'storage-service-resource-key' => $storage_service_resource_key >>

(Required for class method, Not Applicable for instance method)

=item C<< 'subscription-context' => $string >>

=item C<< 'destination-node-resource-key' => $destination_node_resource_key >>

=item C<< 'source-node-resource-key' => $source_node_resource_key >>

=item C<< 'members' => [ $member1, $member2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

=item C<< 'dry-run' => true|false >>

(Optional)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub protection_update_start {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'members' => { type => ARRAYREF, optional => 1 },
            'member-snapshots' => { type => ARRAYREF, optional => 1 },
            _optional_scalars(
                qw( dry-run subscription-context destination-node-resource-key
                    source-node-resource-key )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_protection_update_start_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _protection_update_start_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( member-snapshots members subscription-context storage-service-resource-key
            destination-node-resource-key source-node-resource-key dry-run )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response      = $apiset->storage_service_protection_update_start(%args);
    my $parsed_output = $response->get_parsed_output();
    my $job_id        = $parsed_output->[0]->{'job-id'};

    $Log->exit() if $may_exit;
    return NACL::C::DFM::Job->new(
        'command_interface' => $command_interface,
        'job-id'            => $job_id,
    );

}

=head2 modify

    NACL::C::DFM::StorageService->modify(  command_interface => $ci,
                                'storage-service-resource-key' => $storage_service_resource_key,
                                  ...);
                                      or

    $StorageService_object->modify(...);

=for html (Class or instance method)
This method is used to  modify the attributes of an existing storage service.
It returns a Job Component object.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'storage-service-resource-key' => $storage_service_resource_key >>

(Required for class method, Not Applicable for instance method)

=item C<< 'dry-run' => $string >>

=item C<< 'storage-service-description' => $string >>

=item C<< 'storage-service-client-tag' => $string >>

=item C<< 'storage-service-name' => $string >>

=item C<< 'storage-service-owner' => $string >>

=item C<< 'storage-service-contact-list' => $arryref >>
(Arrayref, containing a list of email-addresses )

=item C<< 'storage-service-topology-info' => $hashref >>

(Hashref, can be a reference to a hash or hashofarrays or hashofhashes)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'storage-service-contact-list' =>
              { type => ARRAYREF, optional => 1 },
            'storage-service-topology-info' =>
              { type => HASHREF, optional => 1 },
            _optional_scalars(
                qw( dry-run storage-service-description storage-service-client-tag
                  storage-service-name storage-service-owner
                  )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_modify_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _modify_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( dry-run storage-service-contact-list storage-service-description
          storage-service-client-tag storage-service-name
          storage-service-topology-info storage-service-owner
          storage-service-resource-key
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response      = $apiset->storage_service_modify(%args);
    my $parsed_output = $response->get_parsed_output();
    my $job_id        = $parsed_output->[0]->{'job-id'};

    $Log->exit() if $may_exit;
    return NACL::C::DFM::Job->new(
        'command_interface' => $command_interface,
        'job-id'            => $job_id,
    );

}

=head2 import_relationships

    NACL::C::DFM::StorageService->import_relationships(  command_interface => $ci,
                                'storage-service-resource-key' => $storage_service_resource_key,
                                'import-info' => $import_info,
                                  ...);
                                      or

    $StorageService_object->import_relationships(...);

=for html (Class or instance method)
This method is used to  imports non-managed storage objects into an existing storage service. When you
use the storage-service-import API, you must adhere to the following rules. If a
rule is violated, the error code EINVALIDINPUT is returned, unless ootherwise
specified. <ul> <li> Each relationship must satisfy one of the following three
conditions: <ul> <li> The source volume is being imported into the root node.
</li> <li> The source volume already exists in the connection's source node.
</li> <li> The preceding relationship is also being imported. </li> </ul> </li>
<li> A relationship can only be imported into one storage service. If the same
relationship exists in this or any storage service, the error code
E_RELATIONSHIP_ALREADY_MANAGED is returned. </li> <li> If the subscription
for the specified subscription context does not exist, a new subscription is
created by the API. </li> <li> Source and destination volumes of the relationship
can be associated with only a single node of the same storage service. </li> <li>
Source or destination volume being imported into a non-root node of the storage
service cannot be in the non-root node of any other storage service. </li> <li> If
multiple relationships have the same source volume, each of these relationships
must be imported into a different connection. </li> </ul>
It returns a Job Component object.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'storage-service-resource-key' => $storage_service_resource_key >>

(Required for class method, Not Applicable for instance method)

=item C<< 'dry-run' => $string >>

=item C<< 'subscription-context' => $string >>

=item C<< 'import-info' => [ $import-info1, $import-info2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

=item C<< 'perform-inline-discovery' => true|false >>

(Optional) A boolean value.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub import_relationships {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'import-info' => { type => ARRAYREF, optional => 1 },
            _optional_scalars(
                qw( dry-run subscription-context storage-service-resource-key )
            ),
        },
    );

    $Log->exit() if $may_exit;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_import_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    return $ret;
}

sub _import_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( dry-run subscription-context storage-service-resource-key import-info)
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response      = $apiset->storage_service_import(%args);
    my $parsed_output = $response->get_parsed_output();
    my $job_id        = $parsed_output->[0]->{'job-id'};

    $Log->exit() if $may_exit;
    return NACL::C::DFM::Job->new(
        'command_interface' => $command_interface,
        'job-id'            => $job_id,
    );
}

=head2 create

    my $StorageService_object	=
      NACL::C::DFM::StorageService->create(command_interface => $ci,
                                'storage-service-topology-info' => $storage_service_topology_info,
                                'storage-service-name' => $storage_service_name,
                                  ...);
                                      or

=for html (Class method)
This method is used to  creates a new storage service by specifying protection and provisioning
strategy.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'storage-service-name' => $storage_service_name >>

(Required for class method, Not Applicable for instance method)

=item C<< 'storage-service-description' => $string >>

=item C<< 'storage-service-client-tag' => $string >>

=item C<< 'storage-service-owner' => $string >>

=item C<< 'storage-service-contact-list' => $arryref >>
(Arrayref, containing a list of email-addresses )

=item C<< 'storage-service-topology-info' => $hashref >>

(Hashref, can be a reference to a hash or hashofarrays or hashofhashes)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'storage-service-topology-info' =>
              { type => HASHREF, optional => 1 },
            'storage-service-contact-list' =>
              { type => ARRAYREF, optional => 1 },
            _optional_scalars(
                qw( storage-service-description storage-service-client-tag
                  storage-service-owner storage-service-name storage-service-resource-key
                  )
            ),
        },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_create_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _create_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( storage-service-contact-list storage-service-description
          storage-service-client-tag storage-service-owner storage-service-name
          storage-service-topology-info
          )
    ];
    my %args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response = $apiset->storage_service_create(%args);

    my $parsed_output = $response->get_parsed_output();
    my $storage_service_resource_key =
      $parsed_output->[0]->{'storage-service-resource-key'};

    $Log->exit() if $may_exit;

    return $pkg->new(
        'command_interface'            => $command_interface,
        'storage-service-resource-key' => $storage_service_resource_key,
    );
}

=head2 conform

    NACL::C::DFM::StorageService->conform(  command_interface => $ci,
                                'storage-service-resource-key' => $storage_service_resource_key,
                                  ...);
                                      or

    $StorageService_object->conform(...);

=for html (Class or instance method)
This method is used to  checks configuration on members of a storage service. This API runs an on-demand
check on one or more members of the storage service to bring the storage system
configuration into conformance with the strategy defined by the storage
service. A configuration check determines the volumes' current conformance
status and the set of "actions" needed to make them conformant. Then this API
starts the actions to bring the storage system configuration into conformance.
Successful completion of this API indicates that "corrective" conformance
actions are started though not necessarily completed.
It returns a Job Component object.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'storage-service-resource-key' => $storage_service_resource_key >>

(Required for class method, Not Applicable for instance method)

=item C<< 'dry-run' => $string >>

=item C<< 'subscription-context' => $string >>

=item C<< 'members' => [ $member1, $member2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub conform {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'members' => { type => ARRAYREF, optional => 1 },
            _optional_scalars(
                qw( dry-run subscription-context
                  )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_conform_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _conform_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( dry-run members subscription-context storage-service-resource-key
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response      = $apiset->storage_service_conform(%args);
    my $parsed_output = $response->get_parsed_output();
    my $job_id        = $parsed_output->[0]->{'job-id'};

    $Log->exit() if $may_exit;
    return NACL::C::DFM::Job->new(
        'command_interface' => $command_interface,
        'job-id'            => $job_id,
    );

}

=head2 cleanup

    NACL::C::DFM::StorageService->cleanup(  command_interface => $ci,
                                'storage-service-resource-key' => $storage_service_resource_key,
                                  ...);
                                      or

    $StorageService_object->cleanup(...);

=for html (Class or instance method)
This method is used to  users can invoke this API when they feel no more need to maintain the member
artifacts for any kind of restore purposes. Specified members must not have any
active subscriptions. For every specified member, all the relevant data
protection relationships and relevant provisioned storage objects are removed
from the storage service. Users may also opt to destroy these artifacts from the
storage system.It returns a Job Component object.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'storage-service-resource-key' => $storage_service_resource_key >>

(Required for class method, Not Applicable for instance method)

=item C<< 'dry-run' => $string >>

=item C<< 'keep-storage-artifacts' => $string >>

=item C<< 'members' => [ $member1, $member2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

=item C<< 'storage-service-nodes' => [ $node1, $node2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub cleanup {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'members' => { type => ARRAYREF, optional => 1 },
            'storage-service-nodes'  => { type => ARRAYREF, optional => 1 },
            _optional_scalars(
                qw( dry-run keep-storage-artifacts
                  )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_cleanup_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _cleanup_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( dry-run members keep-storage-artifacts storage-service-resource-key
           storage-service-nodes )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response      = $apiset->storage_service_cleanup(%args);
    my $parsed_output = $response->get_parsed_output();
    my $job_id        = $parsed_output->[0]->{'job-id'};

    $Log->exit() if $may_exit;
    return NACL::C::DFM::Job->new(
        'command_interface' => $command_interface,
        'job-id'            => $job_id,
    );

}

=head2 unsubscribe

    NACL::C::DFM::StorageService->unsubscribe(  command_interface => $ci,
                                'storage-service-resource-key' => $storage_service_resource_key,
                                'subscription-context' => $subscription_context,
                                  ...);
                                      or

    $StorageService_object->unsubscribe(...);

=for html (Class or instance method)
This method is used to  unsubscribes members from an existing storage service. If member is
unsubscribed, it remains in the storage service but is marked disabled. When a
member is marked disabled, all of its protection artifacts, such as
relationships and storage objects, are preserved; however, it does not actively
participate in future replication updates or conformance checks. <p> NOTE: To
relinquish a member, or relinquish or destroy its related protection artifact
members you can use the storage-service-cleanup API.
It returns a Job Component object.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'storage-service-resource-key' => $storage_service_resource_key >>

(Required for class method, Not Applicable for instance method)

=item C<< 'subscription-context' => $subscription_context >>

(Required)

=item C<< 'dry-run' => $string >>

=item C<< 'subscription-members' => [ $subscription-member1, $subscription-member2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub unsubscribe {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'subscription-members' => { type => ARRAYREF, optional => 1 },
            _optional_scalars(
                qw( dry-run subscription-context
                  )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_unsubscribe_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _unsubscribe_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( dry-run subscription-members subscription-context
          storage-service-resource-key
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response      = $apiset->storage_service_unsubscribe(%args);
    my $parsed_output = $response->get_parsed_output();
    my $job_id        = $parsed_output->[0]->{'job-id'};

    $Log->exit() if $may_exit;
    return NACL::C::DFM::Job->new(
        'command_interface' => $command_interface,
        'job-id'            => $job_id,
    );

}

=head2 destroy

    NACL::C::DFM::StorageService->destroy(  command_interface => $ci,
                                'storage-service-resource-key' => $storage_service_resource_key,
                                  ...);
                                      or

    $StorageService_object->destroy(...);

=for html (Class or instance method)
This method is used to  destroys a storage service. This API destroys all protection artifacts, such as
volumes and relationships, that were created by this storage service.
It returns a Job Component object.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'storage-service-resource-key' => $storage_service_resource_key >>

(Required for class method, Not Applicable for instance method)

=item C<< 'dry-run' => $string >>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub destroy {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( dry-run
                  )
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_destroy_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _destroy_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( dry-run storage-service-resource-key
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response      = $apiset->storage_service_destroy(%args);
    my $parsed_output = $response->get_parsed_output();
    my $job_id        = $parsed_output->[0]->{'job-id'};

    $Log->exit() if $may_exit;
    return NACL::C::DFM::Job->new(
        'command_interface' => $command_interface,
        'job-id'            => $job_id,
    );

}

=head2 member_list

    NACL::C::DFM::StorageService->member_list(  command_interface => $ci,
						'storage-service-resource-key'    => $storge_service_resource_key,
						'node-resource-key'      => $node_resource_key,
						'subscription-context'   => $string,
                                  ...);
                                      or

    $Service_object->member_list(...);

=for html (Class or instance method)
Starts iteration to list members of the storage service
It returns a state object of the Component State NACL::CS::StorageServiceMember

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< 'storage-service-resource-key' => $storage_service_resource_key >>

(Required for class method, Not Applicable for instance method)

=item C<< 'subscription-context' => $context >>

(Optional)

=item C<< 'node-resource-key'  => $node_resource_key >>

(Optional)

=back

=back

=cut

sub member_list {

    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);

    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( storage-service-resource-key  node-resource-key subscription-context )
            ),
        },
        ignore_primary_keys => 1,
    );
    my @state_objs = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_member_list_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _member_list_zapi {

    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $allowed_options = [
        qw( subscription-context node-resource-key storage-service-resource-key )
    ];

    my %filter_opts;

    $pkg->_hash_move(
        source => \%opts,
        target => \%filter_opts,
        move   => $allowed_options,
    );

    my @state_objs =
      NACL::CS::DFM::StorageServiceMember->_fetch_zapi( %opts,
        filter => \%filter_opts );

    $Log->exit() if $may_exit;
    return @state_objs;
}

=head2 subscription_list

    NACL::C::DFM::StorageService->subscription_list(  command_interface => $ci,
                                       'storage-service-resource-key' => $storage_service_resource_key,
                                       'subscription-context' => $context,
                                       'subscription-resource-key' => $subscription_resource_key,  
                                  ...);
                                      or

    $storageservice_obj->subscription_list(...); 

=for html (Class or instance method)
Starts to list subscriptions of a storage service
It returns a state object of the Component State NACL::CS::StorageServiceSubscription

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< 'storage-service-resource-key' => $storage_service_resource_key >>

(Required for class method, Not Applicable for instance method)

=item C<< 'subscription-context' => $context >>
(Optional)

=item C<< 'subscription-resource-key' => $subscription_resource_key >>
(Optional)

=back

=back

=cut

sub subscription_list {

    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);

    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw(storage-service-resource-key subscription-resource-key subscription-context )
            ),
        },
        ignore_primary_keys => 1,
    );
    my @state_objs = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_subscription_list_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _subscription_list_zapi {

    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $allowed_options = [
        qw( storage-service-resource-key subscription-resource-key  subscription-context
          )
    ];

    my %filter_opts;

    $pkg->_hash_move(
        source => \%opts,
        target => \%filter_opts,
        move   => $allowed_options,
    );

    my @state_objs =
      NACL::CS::DFM::StorageServiceSubscription->_fetch_zapi( %opts,
        filter => \%filter_opts );

    $Log->exit() if $may_exit;
    return @state_objs;
}

=head2 get_metadata 

    NACL::C::DFM::StorageService->get_metadata(  command_interface => $ci,
                                'storage-service-resource-key' => $storage_service_resource_key,
                                  ...);
                                      or

    $StorageService_object->get_metadata(...);

=for html (Class or instance method)
Returns metadata of a subscription context set by storage-service-subscription-context-set-metadata API

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< 'storage-service-resource-key' => $storage_service_resource_key >>

(Required for class method, Not Applicable for instance method)

=item C<< 'subscription-context' => $string >>

=back

=back

=cut

sub get_metadata {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( storage-service-resource-key subscription-context )),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_get_metadata_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _get_metadata_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options =
      [qw( storage-service-resource-key subscription-context  )];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->storage_service_subscription_context_get_metadata(%args);

    $Log->exit() if $may_exit;

}

=head2 set_metadata

    NACL::C::DFM::StorageService->set_metadata(  command_interface => $ci,
                                'storage-service-resource-key' => $storage_service_resource_key,
                                  ...);
                                      or

    $StorageService_object->set_metadata(...);

=for html (Class or instance method)
Sets metadata for an existing subscription context for a given storage service.
The metadata is automatically deleted if the subscription context is removed for the storage service

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< 'storage-service-resource-key' => $storage_service_resource_key >>

(Required for class method, Not Applicable for instance method)

=item C<< 'subscription-context' => $string >>

=item C<< 'metadata'   => @keyvalue  >> 

=back

=back

=cut

sub set_metadata {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'metadata' => { type => ARRAYREF, optional => 1 },
            _optional_scalars(
                qw( storage-service-resource-key subscription-context )),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_set_metadata_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _set_metadata_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options =
      [qw( storage-service-resource-key subscription-context metadata  )];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->storage_service_subscription_context_set_metadata(%args);

    $Log->exit() if $may_exit;

}

sub _primary_keys_validate_spec {
    return ( 'storage-service-resource-key' => { type => SCALAR }, );
}

1;
