# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Debug Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Debug

=head1 DESCRIPTION

C<NACL::C::Debug> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. Unlike other components this
package does not model any element/service but is present to provide 
component-level support for functionality. Since it does not model any 
element/service there is no ComponentState module for this component.
For the same reason, 'find' and 'state' will not work for this component.
Instead NACL::CS::Debug->fetch() should be used.
(See L<fetch|lib-NACL-CS-ComponentState-pm/fetch> for a full description)

=cut

=head1 ATTRIBUTES

None.

=cut

package NACL::C::Debug;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR SCALARREF ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $Debug = NACL::C::Debug->new(
                                   command_interface => $ci,
								  );

Return a new Debug component object with the given attributes.

=cut

=head2 vol_ops_diag_test

    NACL::C::Debug->vol_ops_diag_test(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to run the volume ops diag test.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $string >>

Applicable for CMode CLI.

=item C<< 'aggregate' => $string >>

Applicable for CMode CLI.

=item C<< 'foreground' => $string >>

Applicable for CMode CLI.

=item C<< 'node' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub vol_ops_diag_test {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( vserver aggregate foreground node
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_vol_ops_diag_test_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub vol_ops_diag_test

sub _vol_ops_diag_test_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( vserver aggregate foreground node
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'debug_vol_ops_diag_test',
        api_opts => \%cmode_args
    );

    $apiset->debug_vol_ops_diag_test(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _vol_ops_diag_test_cmode_cli

=head2 profile

    NACL::C::Debug->profile(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to profile CPU usage of a process.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'action' => $string >>

Applicable for CMode CLI.

=item C<< 'process' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub profile {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( action process
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_profile_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub profile

sub _profile_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( action process
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'debug_profile',
        api_opts => \%cmode_args
    );

    $apiset->debug_profile(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _profile_cmode_cli

=head2 result

    my $result_obj = NACL::C::Debug->result(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @result_objs = NACL::C::Debug->result(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $result_obj = $Debug_obj->result();
 
(Class or Instance method)
This method returns objects of the type L<NACL::CS::DebugResult|lib-NACL-CS-DebugResult-pm>.

This method accepts all the parameters accepted by L<fetch|lib-NACL-CS-ComponentState-pm/fetch> 
(requested_fields, filter, allow_empty) and has all of the other properties of 
fetch methods (callable in scalar or list context, throws a NoElementsFound 
exception if no matching elements are found)

=cut

sub result {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::DebugResult;
    my @state_objs = NACL::CS::DebugResult->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub result

=head2 vol_ops_xc_test

    NACL::C::Debug->vol_ops_xc_test(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to run the volume snapshot ops test cross cluster.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'cluster-uuid' => $string >>

Applicable for CMode CLI.

=item C<< 'vserver' => $string >>

Applicable for CMode CLI.

=item C<< 'msid' => $string >>

Applicable for CMode CLI.

=item C<< 'dsid' => $string >>

Applicable for CMode CLI.

=item C<< 'foreground' => $string >>

Applicable for CMode CLI.

=item C<< 'clusterId' => $string >>

Applicable for CMode CLI.

=item C<< 'node' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub vol_ops_xc_test {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( cluster-uuid vserver msid dsid foreground clusterId node
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_vol_ops_xc_test_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub vol_ops_xc_test

sub _vol_ops_xc_test_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( cluster-uuid vserver msid dsid foreground clusterId node
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'debug_vol_ops_xc_test',
        api_opts => \%cmode_args
    );

    $apiset->debug_vol_ops_xc_test(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _vol_ops_xc_test_cmode_cli

=head2 blastout

    NACL::C::Debug->blastout(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to blast out data to stdout.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub blastout {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        additional_spec =>
            {'cmd_execution_time' => {type => SCALARREF, optional => 1},},
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_blastout_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Debug::blastout',
    );

    $Log->exit() if $may_exit;
}

sub _blastout_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset                = $opts{apiset};
    my $execution_time        = $opts{cmd_execution_time};
    my $allowed_cmode_options = [
        qw( extended_query
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    my $response = $apiset->debug_blastout(%cmode_args);
    if (defined $execution_time) {
        $$execution_time = $response->get_execution_time();
    }

    $Log->exit() if $may_exit;
}

=head2 dumpcli

    NACL::C::Debug->dumpcli(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to dump the CLI tree.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'outputxml' => $string >>

output CLI commands as XML possible value(s) are, true,false

Applicable for CMode CLI.

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub dumpcli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( outputxml
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_dumpcli_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Debug::dumpcli',
    );

    $Log->exit() if $may_exit;
}

sub _dumpcli_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset                = $opts{apiset};
    my $execution_time        = $opts{cmd_execution_time};
    my $allowed_cmode_options = [
        qw( outputxml extended_query
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    my $response = $apiset->debug_dumpcli(%cmode_args);
    if (defined $execution_time) {
        $$execution_time = $response->get_execution_time();
    }

    $Log->exit() if $may_exit;
}

=head2 evaluate

    NACL::C::Debug->evaluate(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to evaluate an expression.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'expression' => $string >>

Expression possible value(s) are, Arithmetic Expression

Applicable for CMode CLI.

=item C<< 'operands-of' => $string >>

Operator possible value(s) are, text

Applicable for CMode CLI.

=item C<< 'values' => [ $value1, $value2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI.

=item C<< 'variables' => [ $variable1, $variable2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI.

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub evaluate {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'values'             => {type => ARRAYREF,  optional => 1},
            'variables'          => {type => ARRAYREF,  optional => 1},
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( expression operands-of
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_evaluate_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Debug::evaluate',
    );

    $Log->exit() if $may_exit;
}

sub _evaluate_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset                = $opts{apiset};
    my $execution_time        = $opts{cmd_execution_time};
    my $allowed_cmode_options = [
        qw( expression values variables operands-of extended_query
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    foreach my $array_element (qw(values variables)) {
        if (defined $cmode_args{$array_element}) {
            $cmode_args{$array_element} = join ',',
                @{$cmode_args{$array_element}};
        }
    }

    my $response = $apiset->debug_evaluate(%cmode_args);
    if (defined $execution_time) {
        $$execution_time = $response->get_execution_time();
    }

    $Log->exit() if $may_exit;
}

=head2 ksmf_action

    NACL::C::Debug->ksmf_action(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to run the ksmf action test.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'requeue_cnt' => $string >>

Requeue count possible value(s) are, integer

Applicable for CMode CLI.

=item C<< 'iterations' => $string >>

Number of kSMF calls possible value(s) are, integer

Applicable for CMode CLI.

=item C<< 'exit_queue' => $string >>

Use the exit queue possible value(s) are, true,false

Applicable for CMode CLI.

=item C<< 'dedicated_q' => $string >>

Dedicate a kSMF queue possible value(s) are, integer

Applicable for CMode CLI.

=item C<< 'sleep' => $string >>

sleep length possible value(s) are, integer

Applicable for CMode CLI.

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub ksmf_action {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( requeue_cnt iterations exit_queue dedicated_q sleep
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_ksmf_action_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Debug::ksmf_action',
    );

    $Log->exit() if $may_exit;
}

sub _ksmf_action_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset                = $opts{apiset};
    my $execution_time        = $opts{cmd_execution_time};
    my $allowed_cmode_options = [
        qw( requeue_cnt iterations exit_queue dedicated_q sleep extended_query
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    my $response = $apiset->debug_ksmf_action(%cmode_args);
    if (defined $execution_time) {
        $$execution_time = $response->get_execution_time();
    }

    $Log->exit() if $may_exit;
}

=head2 nettest

    NACL::C::Debug->nettest(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to nettest.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'process' => $string >>

Process to profile possible value(s) are, mgwd

Applicable for CMode CLI.

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub nettest {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( process
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_nettest_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Debug::nettest',
    );

    $Log->exit() if $may_exit;
}

sub _nettest_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset                = $opts{apiset};
    my $execution_time        = $opts{cmd_execution_time};
    my $allowed_cmode_options = [
        qw( process extended_query
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    my $response = $apiset->debug_nettest(%cmode_args);
    if (defined $execution_time) {
        $$execution_time = $response->get_execution_time();
    }

    $Log->exit() if $may_exit;
}

=head2 message_fmt

    NACL::C::Debug->message_fmt(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to test message_fmt.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'function' => $string >>

Function Name
Applicable for CMode CLI

=item C<< 'privilege' => $string >>

Privilege
Applicable for CMode CLI

=item C<< 'message' => $string >>

Message
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by "debug message fmt" command in ONTAP

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub message_fmt {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( function privilege message
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'debug message_fmt',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_message_fmt_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Debug::message_fmt',
    );

    $Log->exit() if $may_exit;
}

sub _message_fmt_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;

    $pkg->_base_backend_cmode_cli(
        @opts,
        api => 'debug_message_fmt',
    );

    $Log->exit() if $may_exit;
}

=head2 process_leak

    NACL::C::Debug->process_leak(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to cause given process to leak memory.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'amount' => $string >>

Amount to leak
Applicable for CMode CLI

=item C<< 'process' => $string >>

Process Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug process leak" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub process_leak {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( amount process
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'debug_process_leak',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_process_leak_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Debug::process_leak',
    );

}

sub _process_leak_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'debug_process_leak',);
    $Log->exit() if $may_exit;
}

=head2 process_rss

    NACL::C::Debug->process_rss(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to change the RSS limit of a proces.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'rss' => $string >>

RSS value to set
Applicable for CMode CLI

=item C<< 'process' => $string >>

Process Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug process rss" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub process_rss {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( rss process
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'debug_process_rss',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_process_rss_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Debug::process_rss',
    );

}

sub _process_rss_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'debug_process_rss',);
    $Log->exit() if $may_exit;
}

=head2 process_vsz

    NACL::C::Debug->process_vsz(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to change the VSZ limit of a proces.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vsz' => $string >>

VSZ value to set
Applicable for CMode CLI

=item C<< 'process' => $string >>

Process Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug process vsz" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub process_vsz {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vsz process
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'debug_process_vsz',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_process_vsz_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Debug::process_vsz',
    );

}

sub _process_vsz_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'debug_process_vsz',);
    $Log->exit() if $may_exit;
}

__PACKAGE__->_load_event_class();

1;
