
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary DiagSecdAuthentication Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::DiagSecdAuthentication

=head1 DESCRIPTION

C<NACL::C::DiagSecdAuthentication> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. Unlike other components this
package does not model any element/service but is present to provide 
component-level support for functionality. Since it does not model any 
element/service there is no ComponentState module for this component.
For the same reason, 'find' and 'state' will not work for this component.

=cut

package NACL::C::DiagSecdAuthentication;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with BOOLEAN SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::ComponentUtils qw(Dumper);
use NACL::CS::DiagSecdAuthenticationSidToUnixName;
use NACL::CS::DiagSecdAuthenticationTranslate;
use NACL::CS::DiagSecdAuthenticationShowOntapAdminUnixCreds;
use NACL::C::Exceptions::DiagSecd::AuthenticationRejected ;
=head2 sid_to_uid

   NACL::C::DiagSecdAuthentication->sid_to_uid(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to translates a Windows SID to a Unix ID and it returns UID.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "sid" => $string >>

Applicable for CMode CLI.

=item C<< "vserver" => $string >>

Applicable for CMode CLI.

=item C<< 'node' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"diag secd authentication sid-to-uid" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub sid_to_uid {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( sid vserver
                    node
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'diag secd authentication sid-to-uid',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_sid_to_uid_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::DiagSecdAuthentication::sid_to_uid',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub sid_to_uid

sub _sid_to_uid_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $response ;
    $pkg->_base_backend_cmode_cli(@args,
            api => 'diag_secd_authentication_sid_to_uid',
            apiset_response_obj => \$response,);

    my ($uid) = $response->get_raw_output() =~ /UID:\s+(.*)/;
    $Log->exit() if $may_exit;
    return $uid ;
} ## end sub _sid_to_uid_cmode_cli

=head2 show_creds

   NACL::C::DiagSecdAuthentication->show_creds(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to show user's creds based on a Unix UID or CIFS SID.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "win-name" => $string >>

Applicable for CMode CLI.

=item C<< "sid" => $string >>

Applicable for CMode CLI.

=item C<< "uid" => $string >>

Applicable for CMode CLI.

=item C<< "vserver" => $string >>

Applicable for CMode CLI.

=item C<< "unix-user-name" => $string >>

Applicable for CMode CLI.

=item C<< 'node' => $string >>

Applicable for CMode CLI.

=item C<< 'list-name' => $string >>

*Display Translated Names, possible value(s) are true,false

Applicable for CMode CLI.

=item C<< 'list-id' => $string >>

*Display IDs, possible value(s) are true,false

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"diag secd authentication show-creds" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub show_creds {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( win-name sid uid vserver unix-user-name
                    node list-name list-id
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'diag secd authentication show-creds',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_show_creds_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::DiagSecdAuthentication::show_creds',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub show_creds

sub _show_creds_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    $pkg->_base_backend_cmode_cli(
        @_,
        api => 'diag_secd_authentication_show_creds'
    );

    $Log->exit() if $may_exit;
} ## end sub _show_creds_cmode_cli

=head2 translate

   my $state_obj = NACL::C::DiagSecdAuthentication->translate(
                                command_interface => $ci,
                                  ...);

Returns a L<NACL::CS::DiagSecdAuthenticationTranslate |lib-NACL-CS-DiagSecdAuthenticationTranslate-pm> object which has various names
or identifiers value.

(Class method)
This method is used to translates between various names and their identifiers.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "win-name" => $string >>

Applicable for CMode CLI.

=item C<< "sid" => $string >>

Applicable for CMode CLI.

=item C<< "unix-group-name" => $string >>

Applicable for CMode CLI.

=item C<< "uid" => $string >>

Applicable for CMode CLI.

=item C<< "vserver" => $string >>

Applicable for CMode CLI.

=item C<< "unix-user-name" => $string >>

Applicable for CMode CLI.

=item C<< "gid" => $string >>

Applicable for CMode CLI.

=item C<< 'node' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"diag secd authentication translate" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub translate {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( win-name sid unix-group-name uid vserver unix-user-name gid
                    node
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'diag secd authentication translate',
    );
    my @ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_translate_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::DiagSecdAuthentication::translate',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @ret : $ret[0];
} ## end sub translate

sub _translate_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    my $response;
    $pkg->_base_backend_cmode_cli(%opts,
            api => 'diag_secd_authentication_translate',
            apiset_response_obj => \$response);

    my $output   = $response->get_parsed_output();
    my @state_objs;
    my $obj = NACL::CS::DiagSecdAuthenticationTranslate->new(
       command_interface => $opts{command_interface});
    $obj->_set_fields(row => $output->[0]);
    push(@state_objs, $obj);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _translate_cmode_cli

=head2 ontap_admin_login_cifs

   NACL::C::DiagSecdAuthentication->ontap_admin_login_cifs(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to to authenticate Ontap Admin CIFS user.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "passwd" => $string >>

Applicable for CMode CLI.

=item C<< "vserver" => $string >>

Applicable for CMode CLI.

=item C<< "user" => $string >>

Applicable for CMode CLI.

=item C<< 'node' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"diag secd authentication ontap-admin-login-cifs" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub ontap_admin_login_cifs {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( passwd vserver user

                    node
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd => 'diag secd authentication ontap-admin-login-cifs',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_ontap_admin_login_cifs_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::DiagSecdAuthentication::ontap_admin_login_cifs',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub ontap_admin_login_cifs

sub _ontap_admin_login_cifs_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $password        = $opts{passwd};
    my $prompts_answers = [
        "Enter the password: " => $password,
    ];
    $pkg->_base_backend_cmode_cli(
        %opts,
        api             => 'diag_secd_authentication_ontap_admin_login_cifs',
        additional_args => {'connectrec-match_table' => $prompts_answers},
        frontend_method => 'ontap_admin_login_cifs',
    );

    $Log->exit() if $may_exit;
} ## end sub _ontap_admin_login_cifs_cmode_cli

=head2 login_cifs

   NACL::C::DiagSecdAuthentication->login_cifs(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to to authenticate a CIFS user.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "passwd" => $string >>

provides the user password for the login_cifs command  

Applicable for CMode CLI.


=item C<< "vserver" => $string >>

Applicable for CMode CLI.

=item C<< "user" => $string >>

Applicable for CMode CLI.

=item C<< 'node' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

<< nacl_exception_on_authentication_failure => 0 | 1 >>

(Optional, defaults to 0)

If this option is set to 1, NACL::C::Exceptions::DiagSecd::AuthenticationRejected exception is thrown, indicating wrong username/password.

Applicable for CMode CLI

This method takes all other options supported by
"diag secd authentication login-cifs" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item NACL::C::Exceptions::DiagSecd::AuthenticationRejected

This type of exception is thrown when an attempt is made to login with wrong credentials
in cifs server

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub login_cifs {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( passwd vserver user

                    node
                    )
            ),
            nacl_exception_on_authentication_failure => {type => BOOLEAN, default => 0},
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'diag secd authentication login-cifs',
    );
    my $nacl_exception_on_authentication_failure = delete $opts{'nacl_exception_on_authentication_failure'};
    my $ret = $pkg->call_on_apiset(
       %opts,
       choices => [
           {   method    => "_login_cifs_cmode_cli",
               interface => "CLI",
               set       => "CMode",
           },
       ],
       frontend => 'NACL::C::DiagSecdAuthentication::login_cifs',
      );
   my $dummy_response;
    if( $nacl_exception_on_authentication_failure ){
        if( $ret =~ /Authentication rejected/) {
              $dummy_response = NACL::APISet::Response::CLI::CMode->new(
                        output              => 'Authentication Rejected. DC Returned',
                        command             => '<diag_secd_authentication_login-cifs>',
                        execution_time      => '0',
                        output_compat_mirror =>{},
                        CLI_separator        => '',
                    );
          }

    NACL::C::Exceptions::DiagSecd::AuthenticationRejected->throw(text=>"Authentication rejected.please use correct username and password. Vserver =>$opts{vserver}|username=>$opts{user}|password=>$opts{passwd}",response_object=>$dummy_response);
    }

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub login_cifs

sub _login_cifs_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    my $password = delete $opts{'passwd'};

    my $prompts_answers = [
        'Enter the password:'       => sub {$password},
        'Enter the password again:' => sub {$password},
    ];
    my $response ;

    $pkg->_base_backend_cmode_cli(%opts,
            api => 'diag_secd_authentication_login_cifs',
            apiset_response_obj => \$response,
            additional_args => { 'connectrec-match_table' => $prompts_answers, },);
    my $raw_output = $response->get_raw_output();

    $Log->exit() if $may_exit;
    return $raw_output;
} ## end sub _login_cifs_cmode_cli

=head2 uid_to_sid

  my $sid = NACL::C::DiagSecdAuthentication->uid_to_sid(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to translates a Unix ID to a Windows SID and it returns SID.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "uid" => $string >>

Applicable for CMode CLI.

=item C<< "vserver" => $string >>

Applicable for CMode CLI.

=item C<< 'node' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"diag secd authentication uid-to-sid" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub uid_to_sid {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( uid vserver
                    node
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'diag secd authentication uid-to-sid',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_uid_to_sid_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::DiagSecdAuthentication::uid_to_sid',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub uid_to_sid

sub _uid_to_sid_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $response;
    $pkg->_base_backend_cmode_cli(@args, api  => 'diag_secd_authentication_uid_to_sid',
              apiset_response_obj => \$response,);   
    my ($sid) = $response->get_raw_output() =~ /SID:\s+(.*)/;
    $Log->exit() if $may_exit;
    return $sid;
} ## end sub _uid_to_sid_cmode_cli

=head2 show_ontap_admin_unix_creds

   my $state_obj = NACL::C::DiagSecdAuthentication->show_ontap_admin_unix_creds(
                                command_interface => $ci,
                                  ...);

Returns a L<NACL::CS::DiagSecdAuthenticationShowOntapAdminUnixCreds |lib-NACL-CS-DiagSecdAuthenticationShowOntapAdminUnixCreds-pm>
object which has various names or identifiers value.

(Class method)
This method is used to show Ontap Admin's Unix Credentials based on User Name or User Id.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "uid" => $string >>

Applicable for CMode CLI.

=item C<< "vserver" => $string >>

Applicable for CMode CLI.

=item C<< "unix-user-name" => $string >>

Applicable for CMode CLI.

=item C<< 'node' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"diag secd authentication show-ontap-admin-unix-creds" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub show_ontap_admin_unix_creds {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( uid vserver unix-user-name
                    node
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd => 'diag secd authentication show-ontap-admin-unix-creds',
    );
    my @ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_show_ontap_admin_unix_creds_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend =>
            'NACL::C::DiagSecdAuthentication::show_ontap_admin_unix_creds',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @ret : $ret[0];
} ## end sub show_ontap_admin_unix_creds

sub _show_ontap_admin_unix_creds_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = $opts{command_interface};
    my $response;
    $pkg->_base_backend_cmode_cli(
        @args,
        api => 'diag_secd_authentication_show_ontap_admin_unix_creds',
        apiset_response_obj => \$response
    );
    my $output = $response->get_parsed_output();
    my @state_objs;
    my $obj = NACL::CS::DiagSecdAuthenticationShowOntapAdminUnixCreds->new(
        command_interface => $command_interface);
    $obj->_set_fields(row => $output->[0]);
    push(@state_objs, $obj);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _show_ontap_admin_unix_creds_cmode_cli

=head2 sid_to_unix_name

    my $state_obj = NACL::C::DiagSecdAuthentication->sid_to_unix_name(
                                command_interface => $ci,
                                ...);

Returns a NACL::CS::DiagSecdAuthenticationSidToUnixName object which has the sid-to-unix-name details

(Class method)
This method is used to translates a SID to a Unix Name.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'sid' => $string >>

Applicable for CMode CLI.

=item C<< 'vserver' => $string >>

Applicable for CMode CLI.

=item C<< 'node' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"diag secd authentication sid-to-unix-name" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub sid_to_unix_name {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( sid vserver node
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'diag secd authentication sid-to-unix-name',
    );
    my @objs = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_sid_to_unix_name_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DiagSecdAuthentication::sid_to_unix_name',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @objs : $objs[0];
} ## end sub sid_to_unix_name

sub _sid_to_unix_name_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $response;
    $pkg->_base_backend_cmode_cli(%opts,
            api   => 'diag_secd_authentication_sid_to_unix_name',
            apiset_response_obj => \$response,);

    my $output = $response->get_parsed_output();
    my @state_objs;
    my $obj = NACL::CS::DiagSecdAuthenticationSidToUnixName->new(
        command_interface => $opts{command_interface});
    $obj->_set_fields(row => $output->[0]);
    push(@state_objs, $obj);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _sid_to_unix_name_cmode_cli

=head2 get_claim_name

    NACL::C::DiagSecdAuthentication->get_claim_name(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to get the name of a claim given its CN.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'claim-cn' => $string >>

Claim CN
Applicable for CMode CLI

=item C<< 'vserver' => $string >>

Vserver
Applicable for CMode CLI

=item C<< 'node' => $string >>

Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "diag secd authentication get claim name" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub get_claim_name {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( claim-cn vserver node
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'diag secd authentication get-claim-name',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_get_claim_name_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DiagSecdAuthentication::get_claim_name',
    );

    $Log->exit() if $may_exit;
}

sub _get_claim_name_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'diag_secd_authentication_get_claim_name',);
    $Log->exit() if $may_exit;
}

=head2 get_dc_info

    NACL::C::DiagSecdAuthentication->get_dc_info(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to get information about one of the DCs for the domain of which the CIFS server is a member.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $string >>

Vserver
Applicable for CMode CLI

=item C<< 'node' => $string >>

Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "diag secd authentication get dc info" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub get_dc_info {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vserver node
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'diag secd authentication get-dc-info',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_get_dc_info_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DiagSecdAuthentication::get_dc_info',
    );

    $Log->exit() if $may_exit;
}

sub _get_dc_info_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'diag_secd_authentication_get_dc_info',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ();
}
1;
