# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary DiagSecdDns Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::DiagSecdDns

=head1 DESCRIPTION

C<NACL::C::DiagSecdDns> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. 
Unlike other components this package does not model any element/service 
but is present to provide component-level support for functionality. 
Since it does not model any element/service there is no ComponentState 
module for this component. For the same reason, 'find' and 'state' will
not work for this component.
Instead NACL::CS::DiagSecdDns->fetch() should be used.
(See L<fetch|lib-NACL-CS-ComponentState-pm/fetch> for a full description)

=cut

=head1 ATTRIBUTES

None.

=cut

package NACL::C::DiagSecdDns;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $DiagSecdDns = NACL::C::DiagSecdDns->new(
                                   command_interface => $ci,
								  );

Return a new DiagSecdDns component object with the given attributes.

=cut

=head2 srv_lookup

    NACL::C::DiagSecdDns->srv_lookup(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to perform DNS lookup for SRV records.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $string >>

*Vserver

Applicable for CMode CLI.

=item C<< 'lookup-type' => $string >>

*Lookup Type, possible value(s) are ipv4,ipv6,ipv4_ipv6

Applicable for CMode CLI.

=item C<< 'lookup-string' => $string >>

*Name to Lookup For

Applicable for CMode CLI.

=item C<< 'node' => $string >>

*Node

Applicable for CMode CLI.

=item C<< 'name-servers' => [ $name-server1, $name-server2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub srv_lookup {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'name-servers' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( vserver lookup-type lookup-string node
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    my $ret =  $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_srv_lookup_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _srv_lookup_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $apiset                = $opts{apiset};
    my $allowed_cmode_options = [
        qw( vserver name-servers lookup-type lookup-string node
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    foreach my $array_element (qw(name-servers)) {
        if (defined $cmode_args{$array_element}) {
            $cmode_args{$array_element} = join ',',
                @{$cmode_args{$array_element}};
        }
    }

    my $response = $apiset->diag_secd_dns_srv_lookup(%cmode_args);
    my $output  = $response->get_parsed_output();
    $Log->exit() if $may_exit;
    return $output->[0]->{'ip_address'};
}

=head2 forward_lookup

    NACL::C::DiagSecdDns->forward_lookup(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to perform DNS forward lookup.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $string >>

*Vserver

Applicable for CMode CLI.

=item C<< 'hostname' => $string >>

*Hostname

Applicable for CMode CLI.

=item C<< 'lookup-type' => $string >>

*Lookup Type, possible value(s) are ipv4,ipv6,ipv4_ipv6

Applicable for CMode CLI.

=item C<< 'node' => $string >>

*Node

Applicable for CMode CLI.

=item C<< 'domains' => [ $domain1, $domain2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI.

=item C<< 'name-servers' => [ $name-server1, $name-server2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub forward_lookup {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'domains'      => {type => ARRAYREF, optional => 1},
            'name-servers' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( vserver hostname lookup-type node
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_forward_lookup_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
}

sub _forward_lookup_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $apiset                = $opts{apiset};
    my $allowed_cmode_options = [
        qw( domains vserver name-servers hostname lookup-type node
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    foreach my $array_element (qw(domains name-servers)) {
        if (defined $cmode_args{$array_element}) {
            $cmode_args{$array_element} = join ',',
                @{$cmode_args{$array_element}};
        }
    }

    $apiset->diag_secd_dns_forward_lookup(%cmode_args);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

1;
