#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a AntivirusOnDemand has already been scheduled
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::AntivirusOnDemand::AlreadyScheduled

=head1 DESCRIPTION

This exception is thrown when we attempt to schedule an antivirus on-demand
scan that has already been scheduled.

=head1 EXAMPLE

   use NACL::C::Exceptions::AntivirusOnDemand::AlreadyScheduled qw(:try);

   try {
       NACL::C::AntivirusOnDemand->schedule(%opts);
   } catch NACL::C::Exceptions::AntivirusOnDemand::AlreadyScheduled with {
        # AntivirusOnDemand has already been scheduled
   };

=cut

package NACL::C::Exceptions::AntivirusOnDemand::AlreadyScheduled;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/failed to queue.*exists/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
