#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a AntivirusOnDemandCommandScanDir is in use
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::AntivirusOnDemandCommandScanDir::InUse

=head1 DESCRIPTION

This exception is thrown when we attempt to delete an antivirus on-demand
directory scan command which is in use.

=head1 EXAMPLE

   use NACL::C::Exceptions::AntivirusOnDemandCommandScanDir::InUse qw(:try);

   try {
       NACL::C::AntivirusOnDemandCommandScanDir->delete(%opts);
   } catch NACL::C::Exceptions::AntivirusOnDemandCommand::InUse with {
        # AntivirusOnDemandCommandScanDir is in use.
   };

=cut

package NACL::C::Exceptions::AntivirusOnDemandCommandScanDir::InUse;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/in use/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
