#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for NetworkRoute
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Client::Vlan::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a Vlan
which already exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::Client::Vlan::AlreadyExists qw(:try);

   try {
       NACL::C::Client::Vlan->create(
      'command_interface'        =>  $Client,
      'vlan-id'                  =>  $Vlan_Id,
      'interface'                =>  $Interface,
      %other_opts);
   } catch NACL::C::Exceptions::Client::Vlan::AlreadyExists {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::Client::Vlan::AlreadyExists;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::ComponentUtils qw (Dumper);
use base qw(
    NACL::C::Exceptions::AlreadyExists
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
          qr/ERROR:\s+trying\s+to\s+add\s+VLAN\s+(.*)error:\s+(File\s+exists|Invalid\s+argument)/i,
          qr/already\s+exists/i,
        ]
    );
    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect

1;
