#
# Copyright (c) 2009-2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a attempt is made to delete a cluster peer
##          relationship which is non-existent.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::ClusterPeer::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when a attempt is made to delete a cluster peer
relationship which is non-existent.

=head1 EXAMPLE

   use NACL::C::Exceptions::ClusterPeer::DoesNotExist qw(:try);

   try {
       NACL::C::ClusterPeer->delete( cluster => $cluster );
   } catch NACL::C::Exceptions::ClusterPeer::DoesNotExist with {
        # ClusterPeer does not exist any more
   };

=cut

package NACL::C::Exceptions::ClusterPeer::DoesNotExist;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::DoesNotExist
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/entry doesn't exist/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
