#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary A base class for the DoesNotExist exceptions
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::DoesNotExist

=head1 DESCRIPTION

This is an abstract base class for the exceptions that get thrown
when we attempt to operate on an element that no longer exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::DoesNotExist qw(:try);

   # $comp_obj could be any component object
   try {
       $comp_obj->delete(%opts);
   } catch NACL::C::Exceptions::DoesNotExist with {
        # Element no longer exists
   };

=cut


package NACL::C::Exceptions::DoesNotExist;

# These are the exceptions typically thrown by the APISet
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate validate_with HASHREF ARRAYREF);
# Handle the typical error message
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {additional_regexes => {type => ARRAYREF, default => []}},
        allow_extra => 1
    );
    my $additional_regexes= delete $opts{additional_regexes};
    my $detected = $pkg->_detect_helper(
        %opts,
        regexes => [qr/.*entry\s+doesn't\s+exist*/i, qr/There\s+are\s+no\s+entries.*/i,@$additional_regexes],
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
