# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/JobLog/DoesNotExist.pm#1 $
#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when a JobLog operation is performed on a
##          coredump that does not exist.
##
## @author raghaveb@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::JobLog::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when a JobLog operation is performed on a
core that does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::JobLog::DoesNotExist qw(:try);

   try {
       $non_existing_job_log_obj->modify(level=>'emerg');
   } catch NACL::C::Exceptions::JobLog::DoesNotExist with {
        # JobLog does not exist any more
   };

=cut

package NACL::C::Exceptions::JobLog::DoesNotExist;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException
    NACL::C::Exceptions::DoesNotExist
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected =
      $pkg->_detect_helper( @args, regexes => [qr/.*entry\s+doesn't\s+exist*/] );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
