# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/JobPrivate/DoesNotExist.pm#3 $
#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a JobPrivate operation is performed on a
##          volume that does not exist.
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::JobPrivate::DoesNotExist

=head1 DESCRIPTION

This exception is thrown hen a JobPrivate operation is performed on a
volume that does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::JobPrivate::DoesNotExist qw(:try);

   try {
       $non_existing_vol_obj->modify(state => 'offline');
   } catch NACL::C::Exceptions::JobPrivate::DoesNotExist with {
        # JobPrivate does not exist any more
   };

=cut

package NACL::C::Exceptions::JobPrivate::DoesNotExist;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::InvalidParamValueException
     NACL::C::Exceptions::DoesNotExist
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/entry doesn't exist/, qr/no job named/, qr/is an invalid value for field/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
