# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/JobScheduleInterval/DoesNotExist.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when entry doesn't exists for JobScheduleInterval
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::JobScheduleInterval::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a job schedule interval
when entry does not exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::JobScheduleInterval::DoesNotExist qw(:try);

   try {
       NACL::C::JobScheduleInterval->delete(
           command_interface  => $Node,
           name               => $shedule,
           %other_opts);
   } catch NACL::C::Exceptions::JobScheduleInterval::DoesNotExist with {
        # Entry Does not exists
   };

=cut

package NACL::C::Exceptions::JobScheduleInterval::DoesNotExist;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::DoesNotExist
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/entry doesn't exist/i, qr/There\s+are\s+no\s+entries.*/i, ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
