#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when we attempt to map the lun id which is
## already used
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Lun::LunIdAlreadyUsed

=head1 DESCRIPTION

This exception is thrown when we attempt to create a LunMap which used an already
used lun-id.

=head1 EXAMPLE

   use NACL::C::Exceptions::Lun::LunIdAlreadyUsed qw(:try);

   try {
       NACL::C::Lun->map(path => $lun_path, %other_opts);
   } catch NACL::C::Exceptions::Lun::LunIdAlreadyUsed with {
        # Lun already mapped.
   };

=cut

package NACL::C::Exceptions::Lun::LunIdAlreadyUsed;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/BCOMKA contains a duplicate entry|Another LUN mapped at this number/i,]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
