# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/Lun/NotEnoughSpaceOnVolume.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to create a lun
##          of space which is not available
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Lun::NotEnoughSpaceOnVolume

=head1 DESCRIPTION

This exception is thrown when we attempt is made to create a lun
of space which is not available.

=head1 EXAMPLE

   use NACL::C::Exceptions::Lun::NotEnoughSpaceOnVolume qw(:try);

   try {
	NACL::C::Lun->create(
		command_interface => $ci,
		path	=> $path,
		size	=> $size #size which is not enough
	);
   } catch NACL::C::Exceptions::Lun::NotEnoughSpaceOnVolume with {
      #	size which is more than what's available on the volume
   };

=cut

package NACL::C::Exceptions::Lun::NotEnoughSpaceOnVolume;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/No\s+space\s+left/i ],
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
