# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/Lun/AlreadyExists.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when we attempt to map the lun which is
## already mapped
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::LunMapping::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a igroup which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::LunMapping::AlreadyExists qw(:try);

   try {
       NACL::C::Lun->map(path => $lun_path, %other_opts);
   } catch NACL::C::Exceptions::LunMapping::AlreadyExists with {
        # Lun already mapped.
   };

=cut

package NACL::C::Exceptions::LunMapping::AlreadyExists;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/already mapped/i,]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
