# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/NetworkInterface/DoesNotExist.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when entry doesn't exists for NetworkInterface
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Public
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkInterface::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a network interface
when entry does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkInterface::DoesNotExist qw(:try);

   try {
       NACL::C::NetworkInterface->delete(
           command_interface  => $Node,
	   node              => $node,
           lif               => $interface_name,
           %other_opts);
   } catch NACL::C::Exceptions::NetworkInterface::DoesNotExist with {
        # Entry Does not exists
   };

=cut

package NACL::C::Exceptions::NetworkInterface::DoesNotExist;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException
    NACL::C::Exceptions::DoesNotExist
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/entry doesn't exist/i, /no entries matching your query/i]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
