#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception when an IP is already bound to a lif 
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkInterface::IpAlreadyBound

=head1 DESCRIPTION

This exception is thrown when we attempt to create a lif using an IP which
is already bound to another lif

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkInterface::IpAlreadyBound qw(:try);

   try {
       NACL::C::NetworkInterface->create(
           command_interface  => $Node,
	   node              => $node,
           lif               => $interface_name,
           %other_opts);
   } catch NACL::C::Exceptions::NetworkInterface::IpAlreadyBound with {
        # Ip already in use
   };

=cut

package NACL::C::Exceptions::NetworkInterface::IpAlreadyBound;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
        NACL::C::Exceptions::Component
        NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Duplicate ip address/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
