# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/NetworkInterfaceFailoverGroups/DoesNotExist.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when entry doesn't exists for NetworkInterfaceFailoverGroups
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkInterfaceFailoverGroups::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a network interface failover-group
when entry does not exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkInterfaceFailoverGroups::DoesNotExist qw(:try);

   try {
       NACL::C::NetworkInterfaceFailoverGroups->delete(
           command_interface  => $Node,
           'failover-group' => $Failover_group,
           node => $Node_name,
           port => $Port,
           %other_opts);
   } catch NACL::C::Exceptions::NetworkInterfaceFailoverGroups::DoesNotExist with {
        # Entry Does not exists
   };

=cut

package NACL::C::Exceptions::NetworkInterfaceFailoverGroups::DoesNotExist;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::DoesNotExist
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/entry\s+doesn't\s+exist/, # Error if failover group does not exist
                     qr/invalid\s+value\s+for\s+field\s+"-vserver\s+<vserver>"/, # CLI error if vserver does not exist
                     qr/Invalid\s+value\s+specified\s+for\s+"vserver"/] # ZAPI error if vserver does not exist
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
