#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception when we attempt to create a Ipspace with invalid IPspace name.
##          InvalidIPspace could be either IPspace name is already in use by a cluster node,
##          Vserver, or is the name of the local cluster.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkIpspace::InvalidIPspace

=head1 DESCRIPTION

This exception is thrown when we attempt to create a Ipspace with invalid
IPspace name. InvalidIPspace could be either IPspace name is already in use by a cluster node,
Vserver, or is the name of the local cluster.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkIpspace::InvalidIPspace qw(:try);

   try {
       NACL::C::NetworkIpspace->create(Ipspace => $bd_name, %other_opts);
   } catch NACL::C::Exceptions::NetworkIpspace::InvalidIPspace with {
        # Ipspace does not exist
   };

=cut

package NACL::C::Exceptions::NetworkIpspace::InvalidIPspace;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/Invalid IPspace name/i],
    );

    $Log->exit();
    return $detected;
}

1;
