# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/NetworkPortIfgrp/HomePortOrFailoverTarget.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when we attempt to delete as ifgrp which is either home-port
## or failover target of lif.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkPortIfgrp::HomePortOrFailoverTarget

=head1 DESCRIPTION

This exception is thrown when we attempt to delete as ifgrp which is either home-port
or failover target of lif.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkPortIfgrp::HomePortOrFailoverTarget qw(:try);

   try {
    NACL::C::NetworkPortIfgrp->delete(
                command_interface => $ci,
		ifgrp             => $ifgrp,
		%other_opts,
    );
   } catch NACL::C::Exceptions::NetworkPortIfgrp::HomePortOrFailoverTarget with {
        # cannot be deleted as it is either home-port or failover target of lif
   };

=cut

package NACL::C::Exceptions::NetworkPortIfgrp::HomePortOrFailoverTarget;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/is\s+the\s+home\s+port\s+or\s+current\s+port\s+of\s+a\s+LIF/]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
