# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/NetworkPortIfgrp/PortAlreadyAssigned#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when we attempt to add port which is already assigned to broadcast domain.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkPortIfgrp::PortAlreadyAssigned

=head1 DESCRIPTION

This exception is thrown when we attempt to add port which is already assigned to broadcast domain.

=head1 EXAMPLE


   use NACL::C::Exceptions::NetworkPortIfgrp::PortAlreadyAssigned qw(:try);

   try {
    NACL::C::NetworkPortIfgrp->add_port(
        command_interface => $ci,
        ifgrp             => $ifgrp,
        port              => $port,
    );
   } catch NACL::C::Exceptions::NetworkPortIfgrp::PortAlreadyAssigned with {
        # cannot add port as it is assigned with Braodcast Domain
   };

=cut

package NACL::C::Exceptions::NetworkPortIfgrp::PortAlreadyAssigned;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/is already assigned to broadcast domain/]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
