#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for NetworkRoute
##
## @author anupamap@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkRoute::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a network route
which already exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkRoute::AlreadyExists qw(:try);

   try {
       NACL::C::NetworkRoute->delete(
           command_interface  => $Node,
           destination       => $Destination,
           gateway           => $Gateway,
           %other_opts);
   } catch NACL::C::Exceptions::NetworkRoute::AlreadyExists with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::NetworkRoute::AlreadyExists;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::AlreadyExists
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
	    regexes => [ qr/duplicate route exists|duplicate entry/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
