#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a Aggregate is run out of space
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NotEnoughSpaceOnAggregate

=head1 DESCRIPTION

This exception is thrown when we attempt to execute a operation on aggregate which
is out of space.

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageAggregate::NotEnoughSpaceOnAggregate qw(:try);

   try {
       NACL::C::Volume->create(aggregate => $aggregate_size_lesser_than_volume, %other_opts);
   } catch NACL::C::Exceptions::StorageAggregate::NotEnoughSpaceOnAggregate with {
        # Aggregate run out of space for volume creation
   };

=cut

package NACL::C::Exceptions::NotEnoughSpaceOnAggregate;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected = $pkg->_detect_helper( @args,
        regexes => [qr/not enough space in aggregate/i] );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
