#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a QosPolicyGroup of the same name already exists
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::QosPolicyGroup::WorkloadsAssociated

=head1 DESCRIPTION

This exception is thrown when we attempt to create a policy group which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::QosPolicyGroup::WorkloadsAssociated qw(:try);

   try {
       NACL::C::QosPolicyGroup->delete('policy-group' => $policy_group,%other_opts);
   } catch NACL::C::Exceptions::QosPolicyGroup::WorkloadsAssociated with {
        # QosPolicyGroup still has workloads associated with it.
   };

=cut

package NACL::C::Exceptions::QosPolicyGroup::WorkloadsAssociated;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Policy group could not be removed because there are still workloads associated with it/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
