#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception when entry doesn't exists for QosSettingsCache
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::QosSettingsCache::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to delete/modify a qos settings cache
when entry does not exist.

=head1 EXAMPLE

use NACL::C::Exceptions::QosSettingsCache::DoesNotExist qw(:try);

try {
    NACL::C::QosSettingsCache->modify(
           command_interface    => $Node,
           'read-ahead-setting' => $Name,
           'max-blocks'         => '172',
           'overshoot'          => '150',
    );
} catch NACL::C::Exceptions::QosSettingsCache::DoesNotExist with {
      # Entry Does not exist
};


=cut

package NACL::C::Exceptions::QosSettingsCache::DoesNotExist;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::DoesNotExist
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
          @args,
          regexes => [ qr/entry doesn't exist/, qr/no entries/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;

