# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/SecurityCertificate/DoesNotExist.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when entry doesn't exists for SecurityCertificate
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::SecurityCertificate::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a non-existing security
certificate.

=head1 EXAMPLE

   use NACL::C::Exceptions::SecurityCertificate::DoesNotExist qw(:try);

   try {
       NACL::C::SecurityCertificate->delete(
           command_interface  => $Node,
	    ca     => $ca,
            serial => $key,
           %other_opts);
   } catch NACL::C::Exceptions::SecurityCertificate::DoesNotExist with {
        # Entry Does not exists
   };

=cut

package NACL::C::Exceptions::SecurityCertificate::DoesNotExist;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::InvalidParamValueException
    NACL::C::Exceptions::DoesNotExist
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/does\s+not\s+exist/, qr/entry\s+doesn't\s+exist/, qr/invalid\s+value\s+for\s+field/, qr/There\s+are\s+no\s+entries\s+matching\s+your\s+query/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
