# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/SecurityLogin/AlreadyExists.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for SecurityLogin
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::SecurityLogin::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a security login which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::SecurityLogin::AlreadyExists qw(:try);

   try {
       NACL::C::SecurityLogin->create(
           command_interface  => $Node,
           vserver            => $Vserver,
           username           => $Username,
           application        => $Application,
           authmethod         => $Authmethod,
           %other_opts);
   } catch NACL::C::Exceptions::SecurityLogin::AlreadyExists with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::SecurityLogin::AlreadyExists;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
    NACL::C::Exceptions::SecurityLogin::DuplicateEntry
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/duplicate\s+entry/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
