# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/SecurityLogin/DoesNotExist.pm#1 $

# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when entry doesn't exists for SecurityLogin
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::SecurityLogin::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to purge a security login
when entry does not exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::SecurityLogin::DoesNotExist qw(:try);

   try {
       NACL::C::SecurityLogin->delete(
           command_interface  => $Node,
           vserver            => $Vserver,
           username           => $Username,
           application        => $Application,
           authmethod         => $Authmethod,
           %other_opts);
   } catch NACL::C::Exceptions::SecurityLogin::DoesNotExist with {
        # Entry Does not exists
   };

=cut

package NACL::C::Exceptions::SecurityLogin::DoesNotExist;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::DoesNotExist
    NACL::C::Exceptions::SecurityLogin::EntryDoesNotExists
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/entry doesn't exist/i, qr/User not found/, qr/no entries matching your query/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
