#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary
## This exception is thrown when a security login
## operation is performed on invalid protocol/methods.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::SecurityLogin::InvalidOperationOrProtocol

=head1 DESCRIPTION

This exception is thrown when a security login operation is performed
on invalid methods/protocols.

=head1 EXAMPLE

   use NACL::C::Exceptions::SecurityLogin::InvalidOperationOrProtocol qw(:try);

   try {
       NACL::C::SecurityLogin->create(
           command_interface  => $Node,
           vserver            => $Vserver,
           username           => $Username,
           application        => $Application,
           authmethod         => $Authmethod,
           %other_opts);
   } catch NACL::C::Exceptions::SecurityLogin::InvalidOperationOrProtocol with {
        # Cannot use this Protocol
   };

=cut

package NACL::C::Exceptions::SecurityLogin::InvalidOperationOrProtocol;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
            qr/Cannot use .* application on a data Vserver/i,
            qr/Cannot use .* to authenticate/i,
            qr/invalid operation/i,
         ],
    );
    $Log->exit();
    return $detected;
}
1;
