#
# Copyright (c) 2015 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for SecurityLogin
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::SecurityLogin::InvalidPassword

=head1 DESCRIPTION

This exception is thrown when we attempt to create a security login with a
password that does not conform to the filer conventions.

=head1 EXAMPLE

   use NACL::C::Exceptions::SecurityLogin::InvalidPassword qw(:try);

   try {
       NACL::C::SecurityLogin->create(
           command_interface  => $Node,
           vserver            => $Vserver,
           username           => $Username,
           application        => $Application,
           authmethod         => $Authmethod,
           password           => $invalid_pwd,
           %other_opts
       );
   } catch NACL::C::Exceptions::SecurityLogin::InvalidPassword with {
        # Invalid password was supplied
   };

=cut

package NACL::C::Exceptions::SecurityLogin::InvalidPassword;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Password does not conform to filer conventions/i ]
    );
    $Log->exit();
    return $detected;
}

1;
