#
# Copyright (c) 2001-2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a Snaplock clock is already initialized
##
## @author msmeena@netapp.com, dl-snaplock-qa@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::SnaplockComplianceClock::AlreadyInitialized

=head1 DESCRIPTION

This exception is thrown when we attempt to initialize snaplock clock which is
already initialized.

=head1 EXAMPLE

   use NACL::C::Exceptions::SnaplockComplianceClock::AlreadyInitialized qw(:try);

   try {
       NACL::C::SnaplockClock->initialize(%opts);
   } catch NACL::C::Exceptions::SnaplockComplianceClock::AlreadyInitialized with {
        # Clock already initialized.
   };

=cut

package NACL::C::Exceptions::SnaplockComplianceClock::AlreadyInitialized;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
            qr/ComplianceClock already initialized/,
        ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
