#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception is raised when we attempt to initialize snapmirror
## without vserver peer done for the source & destination vservers.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Snapmirror::NoDrRelationship

=head1 DESCRIPTION

This exception is thrown when we attempt to initialize snapmirror without
vserver peer done for the source & destination vservers.

=head1 EXAMPLE

   use NACL::C::Exceptions::Snapmirror::NoDrRelationship qw(:try);

   try {
       NACL::STask::Snapmirror->initialize(
           command_interface  => $Node,
           'source-path'      => $src_vs.":",
           'destination-path' => $dest_vs.":",
           %other_opts);
   } catch NACL::C::Exceptions::Snapmirror::NoDrRelationship with {
       # Create the vserver peer relationship
   };

=cut

package NACL::C::Exceptions::Snapmirror::NoDrRelationship;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);


sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/.*Cannot find (the|any) Vserver DR relationship for Vserver.*/i],
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
