# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/Snapmirror/NoTransferToAbort.pm
#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when we attempt to abort a transfer when there is no transfer
## to abort. 
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Snapmirror::NoTransferToAbort

=head1 DESCRIPTION

This exception is thrown when we attempt to abort a transfer when there is no transfer
to abort.

=head1 EXAMPLE

   use NACL::C::Exceptions::Snapmirror::NoTransferToAbort qw(:try);

   try {
       NACL::C::Snapmirror->abort(
           command_interface  => $Node,
           'destination-path' => $Path,
           %other_opts);
   } catch NACL::C::Exceptions::Snapmirror::NoTransferToAbort with {
       # No transfer to abort
   };

=cut

package NACL::C::Exceptions::Snapmirror::NoTransferToAbort;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
	# first and third regex is for CMode , second is for 7Mode
        regexes => [
	        qr/No\s+transfer\s+to\s+abort/
	        ,qr/destination\s+is\s+already\s+idle/
	        ,qr/SnapMirror\s+operation\s+status\s+is.+Idle/
        ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
