# Copyright (c) 2015 NetApp Inc.
# All rights reserved.
#
## @summary A class for the NotGranular exceptions
##
## @author rjray@netapp.com
## @status Shared
## @pod here

package NACL::C::Exceptions::Snapmirror::NotGranular;

use strict;
use warnings;
use parent 'NACL::C::Exceptions::Component';

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 NAME

NACL::C::Exceptions::Snapmirror::NotGranular

=head1 DESCRIPTION

This is a class for exceptions that get thrown when we attempt to do an
operation that is specific to granular relationships, on a snapmirror that is
not a granular relationship.

=head1 EXAMPLE

    use NACL::C::Exceptions::Snapmirror::NotGranular qw(:try);

    # $sm is a NACL Snapmirror object (component or STask)
    # that represents a relationship that is NOT granular:
    try {
        # Get the data for the consistency group:
        $cg_data = $sm->get_consistency_group_data;
    } catch NACL::C::Exceptions::Snapmirror::NotGranular with {
         # Operation failed because the relationship is not granular
    };

=cut

sub detect {
    $Log->enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        # This is a placeholder for future when we know what the errors look
        # like. THIS WILL NEED TO BE UPDATED.
        regexes => [ qr/NOT READY FOR AUTO-DETECT YET/ ]
    );

    $Log->exit;
    return $detected;
}

1;
