# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/Snapmirror/PolicyLookupFailed.pm#1 $
#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception when policy lookup fails for a snapmirror operation
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Snapmirror::PolicyLookupFailed

=head1 DESCRIPTION

This exception is thrown when we attempt to perform any snapmirror operation
for an invalid policy.

=head1 EXAMPLE

   use NACL::C::Exceptions::Snapmirror::PolicyLookupFailed qw(:try);

   try {
       NACL::C::Snapmirror->create(
           command_interface  => $Node,
           'destination-path'       => $Path,
           %other_opts);
   } catch NACL::C::Exceptions::Snapmirror::PolicyLookupFailed with {
        # Policy Lookup for .. failed
   };

=cut

package NACL::C::Exceptions::Snapmirror::PolicyLookupFailed;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::ResponseException
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
            qr/Policy lookup for.*failed/, 
        ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
