#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a relationship information
## is being propagated
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Snapmirror::RelationshipInfoBeingPropagated

=head1 DESCRIPTION

This exception is thrown when we attempt to resume a snapmirror
relation when the relationship information is being propagated.

=head1 EXAMPLE

    use NACL::C::Exceptions::Snapmirror::RelationshipInfoBeingPropagated qw(:try);

    try {
        NACL::STask::Snapmirror->resume(%opts);
    } catch NACL::C::Exceptions::Snapmirror::RelationshipInfoBeingPropagated with {
        # Snapmirror relation is being updated
    };

    # An example of how to retry the delete() call if it fails for this reason
    use NACL::GeneralUtils qw(nacl_method_retry);
    nacl_method_retry(
        code       => sub { NACL::STask::Snapmirror->resume(%opts); },
        exceptions => 'NACL::C::Exceptions::Snapmirror::RelationshipInfoBeingPropagated',
        try_count  => 3,
        sleep_time => 10
    );

=cut

package NACL::C::Exceptions::Snapmirror::RelationshipInfoBeingPropagated;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected =
        $pkg->_detect_helper( @args, regexes => [qr/relationship\s+with.*being\s+propagated\.\s+Please\s+try\s+again\s+after\s+a\s+minute/i] );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
