#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when a stale cache entry during snapmirror release command
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Snapmirror::StaleCache

=head1 DESCRIPTION

This exception is thrown when we have a stale cache entry while attempting snapmirror release.

=head1 EXAMPLE

    use NACL::C::Exceptions::Snapmirror::StaleCache qw(:try);

    try {
        NACL::STask::Snapmirror->release(%opts);
    } catch NACL::C::Exceptions::Snapmirror::StaleCache with {
        # stale cache entry.
    };

    # An example of how to retry the delete() call if it fails for this reason
    use NACL::GeneralUtils qw(nacl_method_retry);
    nacl_method_retry(
        code       => sub { NACL::STask::Snapmirror->release(%opts); },
        exceptions => 'NACL::C::Exceptions::Snapmirror::StaleCache',
        try_count  => 3,
        sleep_time => 10
    );

=cut

package NACL::C::Exceptions::Snapmirror::StaleCache;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected =
      $pkg->_detect_helper( @args, regexes => [qr/Failed\s+to\s+fetch\s+status\s+information\s+for\s+volume/i] );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
