# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/Snapmirror/TransferInProgress.pm# $
#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to perform snapmirror operation
##          when another operation is in progress.
##
## @author msmeena@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Snapmirror::TransferInProgress

=head1 DESCRIPTION

This exception is thrown when an attempt is made to perform snapmirror
operation when another operation is in progress.

=head1 EXAMPLE

   use NACL::C::Exceptions::Snapmirror::TransferInProgress qw(:try);

   try {
       my $sm = NACL::C::Snapmirror->initialize(%opts);
       $sm->break();
   } catch NACL::C::Exceptions::Snapmirror::TransferInProgress with {
        # Failed because transfer is in progress
   };

=cut

package NACL::C::Exceptions::Snapmirror::TransferInProgress;

use base qw(
	NACL::APISet::Exceptions::CommandFailedException
	NACL::C::Exceptions::Component
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected =
      $pkg->_detect_helper( @args, regexes => [qr/is currently involved in another operation/i,
        qr/Transfer is in progress/i, qr/SnapMirror operation is in progress/i] );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
