#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception gets thrown when we attempt to 
## break an uninitalized volume snapmirror.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Snapmirror::VolumeNotInitializedorRestricted

=head1 DESCRIPTION

An exception gets thrown snapmirror when we attempt
to break an uninitalized volume snapmirror.

=head1 EXAMPLE

   use NACL::C::Exceptions::Snapmirror::VolumeNotInitializedorRestricted qw(:try);

   try {
       NACL::C::Snapmirror->break(
           command_interface  => $Node,
           'destination-path'       => $Path,
           %other_opts);
   } catch NACL::C::Exceptions::Snapmirror::VolumeNotInitializedorRestricted with {
   };

=cut

package NACL::C::Exceptions::Snapmirror::VolumeNotInitializedorRestricted;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/Volume\s+\S+\s+is\s+(not initialized|restricted)/i]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
