#
# Copyright (c) 2009-2011 NetApp Inc.
# All rights reserved.
#
## @summary An exception which is thrown when the aggregate is being modified to offline state
##          In this state of an aggregate no modification to the aggregate is allowed.
##
## @author  dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageAggregate::AggregateHasMountedVolumes;

=head1 DESCRIPTION

This exception is thrown when the aggregate is modified to offline state.

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageAggregate::AggregateHasMountedVolumes; qw(:try);

   try {
       $offline_aggr_obj->modify(state => 'offline');
   } catch NACL::C::Exceptions::StorageAggregate::AggregateHasMountedVolumes with {
        # Failed to take aggregate  offline . Reason:
        # Aggregate has mounted volumes which could not be unmounted.

   };

=cut

package NACL::C::Exceptions::StorageAggregate::AggregateHasMountedVolumes;

use base qw(
	NACL::APISet::Exceptions::CommandFailedException
        NACL::C::Exceptions::Component
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/has mounted volumes which could not be unmounted/i ],
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
