#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a Aggregate of the same name already exists
##
## @author chethanm@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageAggregate::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a aggregate which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageAggregate::AlreadyExists qw(:try);

   try {
       NACL::C::Aggregate->create(aggregate => $already_existing_aggr, %other_opts);
   } catch NACL::C::Exceptions::StorageAggregate::AlreadyExists with {
        # Aggregate already exists.
   };

=cut

package NACL::C::Exceptions::StorageAggregate::AlreadyExists;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::InvalidParamValueException
    NACL::C::Exceptions::AlreadyExists
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
            qr/already uses .* as name/, # CMode
            qr/aggregate .* already exists/i, # 7Mode
            qr/volume .* already exists/i # 7Mode trad vols
        ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
