#
# Copyright (c) 2009-2016 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a delete/purge operation is performed on a
##          aggregate that does not exist.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageAggregate::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when a delete/purge operation is performed on a
aggregate that does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageAggregate::DoesNotExist qw(:try);

   try {
       $aggr_obj->delete();
   } catch NACL::C::Exceptions::StorageAggregate::DoesNotExist with {
        # Aggregate does not exist any more
   };

=cut

package NACL::C::Exceptions::StorageAggregate::DoesNotExist;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::InvalidParamValueException
    NACL::C::Exceptions::DoesNotExist
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/entry doesn't exist/i,
                     qr/No aggregate named .* exists/i,    #7mode
                     qr/Aggregate .* does not exist/i,
        ]
    );
 
    $Log->exit() if $may_exit;
    return $detected;
}

1;
