# $Id$
#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a Aggregate of the same name already exists
##
## @author chethanm@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageAggregate::InsufficientDisksToStartNewRaidGroup

=head1 DESCRIPTION

This exception is thrown when we attempt to add disks to an aggregate and it fails 
because the count of disks is not sufficient to create a new raidgroup.

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageAggregate::InsufficientDisksToStartNewRaidGroup qw(:try);

   try {
       NACL::C::Aggregate->add_disks(aggregate => $already_existing_aggr, diskcount => 1,  %other_opts);
   } catch NACL::C::Exceptions::StorageAggregate::InsufficientDisksToStartNewRaidGroup with {
        # The number of disks specified must be at least three times the number of plexes 
        # in the target aggregate.
   };

=cut

package NACL::C::Exceptions::StorageAggregate::InsufficientDisksToStartNewRaidGroup;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
            qr/Starting a new raid(4|_dp) RAID group/i, # CMode
        ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
