# $Id:$
#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when there are no matching disks available
##
## @author chethanm@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageAggregate::NoMatchingDisksAvailable

=head1 DESCRIPTION

This exception is thrown when there are no matching disks available

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageAggregate::NoMatchingDisksAvailable qw(:try);

   try {
       NACL::C::Aggregate->add_disk(aggregate => $already_existing_aggr, diskcount => 1, %other_opts);
   } catch NACL::C::Exceptions::StorageAggregate::NoMatchingDisksAvailable with {
        # no matching disks available
   };

=cut

package NACL::C::Exceptions::StorageAggregate::NoMatchingDisksAvailable;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected = $pkg->_detect_helper( @args,
        regexes => [qr/no matching disks are available/i] );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
