#
# Copyright (c) 2009-2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when attempting to fail a disk that is not
##          a file-system disk.
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageDisk::NotFileSystemDisk

=head1 DESCRIPTION

This exception is thrown when attempting to fail a disk that is not a
file-system disk.

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::AlreadyExists qw(:try);

   try {
       NACL::C::StorageDisk->fail(disk => $disk, %other_opts);
   } catch NACL::C::Exceptions::StorageDisk::NotFileSystemDisk with {
        # Disk is not a file-system disk.
   };

=cut

package NACL::C::Exceptions::StorageDisk::NotFileSystemDisk;

use strict;
use warnings;

# Sub-classing UnexpectedState because the STask used to throw this
# exception for this scenario; so the sub-classing provides backwards
# compatibility for anyone who might have been catching an UnexpectedState
# exception. 
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
    NACL::Exceptions::UnexpectedState
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/not a file system disk/i]
    );
    $Log->exit();
    return $detected;
}

1;
