# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/StorageFailover/GivebackAlreadyInProgress.pm#1 $
#
# Copyright (c) 2017 NetApp Inc.
# All rights reserved.
#
## @summary An exception when giveback fails as a giveback is already in progress.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageFailover::GivebackAlreadyInProgress

=head1 DESCRIPTION

This exception is thrown when giveback fails as a giveback is already in progress.

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageFailover::GivebackAlreadyInProgress qw(:try);

   try {
    NACL::STask::StorageFailover->giveback(
                command_interface => $ci,
		fromnode          => "$Filer2"
    );
   } catch NACL::C::Exceptions::StorageFailover::GivebackAlreadyInProgress with {
        # Failed to initiate giveback. Reason: A giveback is already in progress.
   };

=cut

package NACL::C::Exceptions::StorageFailover::GivebackAlreadyInProgress;

use NATE::Log qw(log_global);
my $Log       = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter()if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected = $pkg->_detect_helper( @args,
        regexes => [qr/Failed\s+to\s+initiate\s+giveback\.+\s+Reason:\s+A\s+giveback\s+is\s+already\s+in\s+progress\./i] );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
