# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/StorageFailover/PartnerNotFullyBooted.pm#1 $
#
# Copyright (c) 2016 NetApp Inc.
# All rights reserved.
#
## @summary An exception when giveback operation is performed
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageFailover::PartnerNotFullyBooted

=head1 DESCRIPTION

This exception is thrown when giveback operation is performed.

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageFailover::PartnerNotFullyBooted;

   try {
    NACL::C::StorageFailover->giveback(
                command_interface => $node,
                ofnode            => $partner_name,
    );
   } catch NACL::C::Exceptions::StorageFailover::PartnerNotFullyBooted with {
        # Failed to initiate giveback. Reason: Partner has not fully booted for giveback of data aggregates
   };

=cut

package NACL::C::Exceptions::StorageFailover::PartnerNotFullyBooted;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/Partner has not fully booted for giveback of data aggregates/i]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
