#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a Storage Pool of the same name already exists
##
## @author vidyat@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StoragePool::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a Storage Pool which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::StoragePool::AlreadyExists qw(:try);

   try {
       NACL::C::StoragePool->create(
           'storage-pool' => $already_existing_name,
           %other_opts
       );
   } catch NACL::C::Exceptions::StoragePool::AlreadyExists with {
        # Storage Pool already exists.
   };

=cut

package NACL::C::Exceptions::StoragePool::AlreadyExists;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::AlreadyExists
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::InvalidParamValueException
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/storage pool already uses/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
