#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a Switch operation is performed on a
##          Switch that does not exist.
##
## @author
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Switch::DoesNotExists

=head1 DESCRIPTION

This exception is thrown when a operation is performed on a
Switch that does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::Switch::DoesNotExists qw(:try);

   try {
       $zone_obj->delete( 'zone-name' => $Zone);
   } catch NACL::C::Exceptions::Switch::DoesNotExists with {
        # Zone does not exists
   };

=cut

package NACL::C::Exceptions::Switch::DoesNotExists;

use base qw(
    NATE::BaseException
    NACL::APISet::Exceptions::InvalidParamValueException
    NACL::APISet::Exceptions::MissingRequiredParamException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
                    qr/not found/,          # Brocade FC
                    qr/Incomplete command/, # Cisco FC
                   ]
    );
    $Log->exit();
    return $detected;
}

1;
