#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a Switch operation is performed on a
##          Switch that does not exist.
##
## @author
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Switch::ElementAlreadyExists

=head1 DESCRIPTION

This exception is thrown when a Switch operation is performed on a
Switch that does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::Switch::ElementAlreadyExists qw(:try);

   try {
       $zone_obj->create( 'zone-name' => $Zone);
   } catch NACL::C::Exceptions::Switch::ElementAlreadyExists with {
        # Zone already exists
   };

=cut

package NACL::C::Exceptions::Switch::ElementAlreadyExists;

use base qw(
    NATE::BaseException
    NACL::APISet::Exceptions::ResponseException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/duplicate name/ ]
    );
    $Log->exit();
    return $detected;
}

1;
