#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary This exception is thrown when we try to remove a
##          non-exixting license.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::SystemLicense::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we try to remove a non-existing license. 

=head1 EXAMPLE

   use NACL::C::Exceptions::SystemLicense::DoesNotExist qw(:try);

   try {
       $non_existing_license_obj->delete(%opts);
   } catch NACL::C::Exceptions::SystemLicense::DoesNotExist with {
       # Entry doesn't exist/license not found
   };

=cut

package NACL::C::Exceptions::SystemLicense::DoesNotExist;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::DoesNotExist
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
            qr/entry doesn't exist/,
            qr/license not found/,
            qr/There are no entries matching your query/
        ]
    );

    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect

1;
