# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/SystemNodeCoredumpSegment/NotSupportedForThisPlatform.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when attempt to coredump segments when command is
## not supported in the current platform.
##
## @author anupamap@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::SystemNodeCoredumpSegment::NotSupportedForThisPlatform

=head1 DESCRIPTION

This exception is thrown when we attempt to do coredump segment when command is
not supported in the current platform.

=head1 EXAMPLE

   use NACL::C::Exceptions::SystemNodeCoredumpSegment::NotSupportedForThisPlatform qw(:try);

   try {
       NACL::C::SystemNodeCoredumpSegment->find(
			    command_interface => $command_interface
			    filter            => { node => $opts{'node'}, },
                            );
   } catch NACL::C::Exceptions::SystemNodeCoredumpSegment::NotSupportedForThisPlatform with {
	# command is not supported for this platform
   };

=cut

package NACL::C::Exceptions::SystemNodeCoredumpSegment::NotSupportedForThisPlatform;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/not\s+supported\s+for\s+this\s+platform/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
