#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception can be thrown during software image upgrade.
##          relationship.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::SystemNodeImage::DisableNFSv3

=head1 DESCRIPTION

Nfs dos is not supported in LB and FS filers. But they are
supported in SN versions(SN.3). This exception will be thrown if there
is an attempt to upgrade the image from lower version(like SN.3)
to higher version(LB or FS).

=head1 EXAMPLE

   use NACL::C::Exceptions::SystemNodeImage::DisableNFSv3;

   try {
       NACL::C::SystemNodeImage->update(%opts);
   } catch NACL::C::Exceptions::SystemNodeImage::DisableNFSv3 with {};

=cut

package NACL::C::Exceptions::SystemNodeImage::DisableNFSv3;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
            qr/nfsv3.dos.support: Disable NFSv3 MS-DOS Client.*vserver nfs prepare-for-v3-ms-dos-client-downgrade/i
        ],
    );
    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect

1;
