# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/SystemServicesFirewallPolicy/AlreadyExists.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for SystemServicesFirewallPolicy
##
## @author anupamap@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::SystemServicesFirewallPolicy::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a firewall policy which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::SystemServicesFirewallPolicy::AlreadyExists qw(:try);

   try {
       NACL::C::SystemServicesFirewallPolicy->create(
           command_interface  => $Node,
           vserver            => $Vserver,
           policy             => $Policy,
           action             => $Action,
           service            => $Service,
           'ip-list'          => [ $Ip-List ],
           %other_opts);
   } catch NACL::C::Exceptions::SystemServicesFirewallPolicy::AlreadyExists with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::SystemServicesFirewallPolicy::AlreadyExists;
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
    NACL::C::Exceptions::SystemServicesFirewallPolicy::DuplicateEntry
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/duplicate\s+entry/, qr/with\s+the\s+same\s+name/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
