#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary A base class for the NoRouteToHost exceptions
##
## @author garveyj@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NoRouteToHost

=head1 DESCRIPTION

This is an abstract base class for the exceptions that get thrown
when we attempt to mount a vvol datastore, using a container that has no route to the host.

=head1 EXAMPLE

   use NACL::C::Exceptions::VPserver::NoRouteToHost qw(:try);

   try {
       NACL::C::VPserver::VVolDatastore->add_storage(command_interface => ...,
                                                     host_moref => ...,
                                                     container_id => ...,
                                                     datastore => ...);
   } catch NACL::C::Exceptions::VPserver::NoRouteToHost with {
        # No route to the host exists
   };

=cut


package NACL::C::Exceptions::VPserver::NoRouteToHost;
use base 'NACL::C::Exceptions::Component';

use Params::Validate qw(validate_with OBJECT SCALAR);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/no route to host/i ]
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
