# $Id: $ 
#
# Copyright (c) 2015 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for a VVOL Datastore
##
## @author garveyj@netapp.com
## @status Public
## @pod here

=head1 NAME

NACL::C::Exceptions::VPserver::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to mount/create a vvol datastore that already exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VPserver::VVolDatastore::AlreadyExists qw(:try);

   try {
       NACL::C::VPserver::VVolDatastore->mount_vvol_datastore_on_host(command_interface => $vp_ci,
                                                                      host_moref => $host_moref,
                                                                      container_id => $container_id,
                                                                      datastore => $datastore,
                                                                      %other_opts);
   } catch NACL::C::Exceptions::VPserver::VVolDatastore::AlreadyExists with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::VPserver::VVolDatastore::AlreadyExists;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/cannot mount datastore with duplicate name/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
