#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a Vfiler has aready stopped
##
## @author smittal@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

    NACL::C::Exceptions::Vfiler::AlreadyStopped

=head1 DESCRIPTION

    This exception is thrown when we attempt to stop a vfiler which is already
    stopped.

=head1 EXAMPLE

    use NACL::C::Exceptions::Vfiler::AlreadyStopped qw(:try);

    try {
        NACL::C::Vfiler->stop( vfiler => $already_existing_vol, %other_opts );
    }
    catch NACL::C::Exceptions::Vfiler::AlreadyStopped with {

        # Vfiler already stopped.
    };

=cut

package NACL::C::Exceptions::Vfiler::AlreadyStopped;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::InvalidParamValueException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper( @args,
        regexes => [qr/[vV]filer \S+ already stopped/] );
    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect
1;

