#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a Vfiler operation is performed on a
##          Vfiler that does not exist.
##
## @author smittal@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Vfiler::DoesNotExist

=head1 DESCRIPTION

This exception is thrown hen a Vfiler operation is performed on a
Vfiler that does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::Vfiler::DoesNotExist qw(:try);

   try {
       NACL::C::Vfiler->delete(vfiler => $not_existing_vfiler);
   } catch NACL::C::Exceptions::Vfiler::DoesNotExist with {
        # Vfiler does not exist any more
   };

=cut

package NACL::C::Exceptions::Vfiler::DoesNotExist;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::NoMatchingEntriesException
    NACL::C::Exceptions::DoesNotExist
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected
        = $pkg->_detect_helper( @args, regexes => [qr/does\s+not\s+exist/] );
    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect
1;
