# $Id: //depot/prod/DOT/dev/test/lib/NACL/C/Exceptions/Volume/AggregateRequiredForCreate.pm#1 $
#
# Copyright (c) 2009-2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to create a volume without
##          providing the aggregate name.
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::AggregateRequiredForCreate

=head1 DESCRIPTION

This exception is thrown when an attempt is made to create a volume without
providing the aggregate name.

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::AggregateRequiredForCreate qw(:try);

   try {
       NACL::C::Volume->create(
           command_interface => $ci,
           volume            => $vol_name,
           vserver           => $vs_name
       );
   } catch NACL::C::Exceptions::Volume::AggregateRequiredForCreate with {
        # Failed because "aggregate" is required for volume creation
   };

=cut

package NACL::C::Exceptions::Volume::AggregateRequiredForCreate;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(@args,
        regexes => [qr/"aggregate" is a required field/i,
                    qr/One of the following parameters is required: -aggregate, -aggr-list/i]);

    $Log->exit() if $may_exit;
    return $detected;
}

1;
