# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/Volume/AlreadyRestricted.pm
#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when we attempt to restrict a volume which is
## already restricted.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::AlreadyRestricted

=head1 DESCRIPTION

This exception is thrown when we attempt to restrict a volume which is already restricted.

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::AlreadyRestricted qw(:try);

   try {
       NACL::C::Volume->restrict(
           command_interface  => $Node,
	   volume             => $volume
           %other_opts);
   } catch NACL::C::Exceptions::Volume::AlreadyRestricted with {
        # volume already restricted
   };

=cut

package NACL::C::Exceptions::Volume::AlreadyRestricted;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/already\s+restricted/i]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

=head2 is_in_snapmirror_relationship

    # An example showing the call failing because the volume is already
    # restricted and to check whether it is snapmirror relationship.
        use warnings;
        try {
            NACL::C::Volume->restrict(command_interface => $ci, %other_opts);
        } catch NACL::C::Exceptions::Volume::AlreadyRestricted with {
            my $exception   = shift;
            $value = $exception->is_in_snapmirror_relationship(); #returns boolean value
        };

(Instance Method Call)
This method can be used to check whether volume is in snapmirror relationship,
which is preventing volume to get restricted.

=cut

sub is_in_snapmirror_relationship {
   $Log->enter() if $may_enter;
   my ($self) = @_;
   if ( $self->text()
        !~ /The following prevents restrict.*snapmirror destination/s ) {
      $Log->exit() if $may_exit;
      return 1;
   } else {
      $Log->exit() if $may_exit;
      return 0;
   }
}
1;
