#
# Copyright (c) 2009-2011 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a volume operation is performed on a
##          volume that does not exist.
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::DoesNotExist

=head1 DESCRIPTION

This exception is thrown hen a volume operation is performed on a
volume that does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::DoesNotExist qw(:try);

   try {
       $non_existing_vol_obj->modify(state => 'offline');
   } catch NACL::C::Exceptions::Volume::DoesNotExist with {
        # Volume does not exist any more
   };

=cut

package NACL::C::Exceptions::Volume::DoesNotExist;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::InvalidParamValueException
    NACL::C::Exceptions::DoesNotExist
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    
    my $detected = $pkg->_detect_helper(
        @args,
        
        # First regex is for CMode, second is for 7Mode
        regexes => [ qr/entry doesn't exist/, qr/No volume named .* [exists|found]/ ]
    );
    
    $Log->exit() if $may_exit;
    return $detected;
}

1;
