#
# Copyright (c) 2009-2011 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a volume operation is performed on a
##          volume in an unknown state.
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::StateUnknown

=head1 DESCRIPTION

This exception is thrown when a volume operation is performed on a volume
whose state is unknown ("-").

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::StateUnknown qw(:try);

   try {
       $unknown_state_vol_obj->modify(state => 'offline');
   } catch NACL::C::Exceptions::Volume::StateUnknown with {
        # Volume state is "-"
   };

=cut

package NACL::C::Exceptions::Volume::StateUnknown;

use base qw(
	NACL::APISet::Exceptions::CommandFailedException
	NACL::C::Exceptions::Component
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Unable to set volume attributes.*The volume does not exist/,qr/.*Unable to determine the state of the volume.*/ ]
    );
    
    $Log->exit() if $may_exit;
    return $detected;
}

1;
