#
# Copyright (c) 2009-2011 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to unmount a vserver's
##          root volume
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::UnmountRoot

=head1 DESCRIPTION

This exception is thrown when we attempt to unmount a vserver's root volume.

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::UnmountRoot qw(:try);

   try {
       $root_vol->unmount();
   } catch NACL::C::Exceptions::Volume::AlreadyExists with {
        # Failed because it is a root
   };

=cut

package NACL::C::Exceptions::Volume::UnmountRoot;

use base qw(
	NACL::APISet::Exceptions::CommandFailedException
	NACL::C::Exceptions::Component
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Cannot unmount.*root volume/i ]
    );
    
    $Log->exit() if $may_exit;
    return $detected;
}

1;
