#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to modify volume
##          which if offline
##
## @author raghaveb@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::VolumeOffline

=head1 DESCRIPTION

This exception is thrown when we attempt is made to modify volume
	 which if offline 

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::VolumeOffline qw(:try);

   try {
       $vol_obj->offline();
       $vol_obj->modify(size => '21m');
   } catch NACL::C::Exceptions::Volume::VolumeOffline with {
        # Failed because volume offline 
   };

=cut

package NACL::C::Exceptions::Volume::VolumeOffline;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper(
        @args,

        # first regex is for CMode second is for 7Mode
        regexes => [
            qr/volume\s+(.*?\s)*is offline/i,
            qr/is\s+not\s+online/i,
        ]
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
