#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to modify the
##          attributes of a volume when its vserver is locked (either
##          config-locked or baseline is being generated. See burt 637356)
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::VserverLocked

=head1 DESCRIPTION

This exception is thrown when we attempt is made to modify the
attributes of a volume when its vserver is locked (either config-locked
or baseline is being generated. See burt 637356)

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::VserverLocked qw(:try);

   try {
       $vol_obj->modify(%opts);
   } catch NACL::C::Exceptions::Volume::VserverLocked with {
        # Failed because the vserver is locked
   };

=cut

package NACL::C::Exceptions::Volume::VserverLocked;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter();
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Vserver is temporarily locked/i ],
    );

    $Log->exit();
    return $detected;
}

1;
