#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to create a  volume
##          on an aggregate which is not added to the vserver aggr-list 
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::VserverNotInAggrList

=head1 DESCRIPTION

This exception is thrown when we attempt to create a volume on an 
aggregate which is not added to the vserver aggr-list.

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::VserverNotInAggrList qw(:try);

   try {
       NACL::C::Volume->create(command_interface => $ci,
                               volume => $volume,
                               aggregate => $aggr,
                               vserver => $vs);
   } catch NACL::C::Exceptions::Volume::VserverNotInAggrList with {
        # Failed because aggr is not added to the vserver aggr-list 
   };

=cut

package NACL::C::Exceptions::Volume::VserverNotInAggrList;

use base qw(
	NACL::C::Exceptions::Component
	NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,

        # Was "not in aggrlist"; changed to "not in aggr-list" in LB (burt
        # 882089). Need to handle both so that FS also works. Once we reach
        # HN, then this can just be "aggr-list"
        regexes => [ qr/not in aggr[-]?list/i ]
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
