# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VolumeCopy/DestinationVolumeDoesNotExist.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when a volume copy operation is performed from a
##          source volume to non-existing destination volume.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeCopy::DestinationVolumeDoesNotExist

=head1 DESCRIPTION

This exception is thrown when a volume operation is performed from a source volume
to a non-existing volume.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeCopy::DestinationVolumeDoesNotExist qw(:try);

   try {
       NACL::C::VolumeCopy->start(
             command_interface    => $ci,
             'destination-volume' => $non_existing_dest_vol,
             'volume'             => $src_vol,
             %other_opts,
       );
   } catch NACL::C::Exceptions::VolumeCopy::DestinationVolumeDoesNotExist with {
        # Destination volume doesn't exist
   };

=cut

package NACL::C::Exceptions::VolumeCopy::DestinationVolumeDoesNotExist;

use base qw(
    NACL::C::Exceptions::DoesNotExist
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    
    my $detected = $pkg->_detect_helper(
        @args,
        
        regexes => [ qr/Volume doesn't exist/i ]
    );
    
    $Log->exit() if $may_exit;
    return $detected;
}

1;
