#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary
#  This exception is thrown when we try to disable compression when inline compression
#  or incompressible data detection is enabled on the volume.
##
## @author anupamap@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeEfficiency::DisableInlineCompression

=head1 DESCRIPTION

This exception is thrown when we try to disable compression when inline compression
or incompressible data detection is enabled on the volume.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeEfficiency::DisableInlineCompression qw(:try);

   try {
       NACL::C::VolumeEfficiency->config(
           command_interface  => $Node,
           vserver            => $Vserver,
           %other_opts);
   } catch NACL::C::Exceptions::VolumeEfficiency::DisableInlineCompression with {
        # Disable inline compression and retry the command
   };

=cut

package NACL::C::Exceptions::VolumeEfficiency::DisableInlineCompression;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/.*Compression cannot be disabled when inline compression.*Disable inline compression.*/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;

