#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a VolumeEfficiency is enable on a volume failed
##          due to not enough disk space for the operation
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeEfficiency::NotEnoughDiskSpace

=head1 DESCRIPTION

This exception is thrown when we attempt to enable volume efficiency
on a volume that has not enough disk space

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeEfficiency::NotEnoughDiskSpace qw(:try);

   try {
       NACL::C::VolumeEfficiency->on(volume => $already_existing_vol, %other_opts);
   } catch NACL::C::Exceptions::VolumeEfficiency::NotEnoughDiskSpace with {
        # Volume already exists.
   };

=cut

package NACL::C::Exceptions::VolumeEfficiency::NotEnoughDiskSpace;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::InvalidParamValueException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Not enough disk space for the operation/i,  ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;

