#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown while performing volume efficiency operation on a volume
##          which is offline
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeEfficiency::OfflineVolume

=head1 DESCRIPTION

This exception is thrown while performing volume efficiency operation
on a volume which is currently offline

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeEfficiency::OfflineVolume qw(:try);

   try {
        NACL::C::VolumeEfficiency->stop(
        command_interface  => $Node,
        vserver            => $Vserver,
        %other_opts);
   } catch NACL::C::Exceptions::VolumeEfficiency::OfflineVolume with {
      # Volume is currently active.
  };

=cut

package NACL::C::Exceptions::VolumeEfficiency::OfflineVolume;

use base qw(
  NACL::C::Exceptions::Component
  NACL::APISet::Exceptions::CommandFailedException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Volume is offline/i , qr/Failed\s+to\s+enable\s+efficiency\s+on\s+volume.*of\s+Vserver.*Volume\s+is\s+offline/i,]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;

