#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a VolumeEfficiency is already enabled
##          on a volume
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeEfficiency::OperationAlreadyEnabled

=head1 DESCRIPTION

This exception is thrown when we attempt to enable volume efficiency
on a volume that has efficiency already enabled 

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeEfficiency::OperationAlreadyEnabled qw(:try);

   try {
       NACL::C::VolumeEfficiency->on(volume => $already_existing_vol, %other_opts);
   } catch NACL::C::Exceptions::VolumeEfficiency::OperationAlreadyEnabled with {
        # Volume already exists.
   };

=cut

package NACL::C::Exceptions::VolumeEfficiency::OperationAlreadyEnabled;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::InvalidParamValueException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/Operation has already been enabled/i, qr/Failed\s+to\s+enable\s+efficiency\s+on\s+volume.*of\s+Vserver.*Operation\s+has\s+already\s+been\s+enabled/i, qr/operation already enabled/, qr/.*Already\s+existing\s+data\s+could\s+be\s+processed\s+by\s+running.*/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
