#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown while doing volume efficiency off operation
##          when it is active on another volume which belongs to same vserver.
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeEfficiency::OperationCurrentlyActive

=head1 DESCRIPTION

This exception is thrown while doing volume efficiency off operation
when it is active on another volume which belongs to same vserver. 

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeEfficiency::OperationCurrentlyActive qw(:try);

   try {
       NACL::C::VolumeEfficiency->stop(
           command_interface  => $Node,
           vserver            => $Vserver,
           %other_opts);
   } catch NACL::C::Exceptions::VolumeEfficiency::OperationCurrentlyActive with {
        # Volume is currently active.
   };

=cut

package NACL::C::Exceptions::VolumeEfficiency::OperationCurrentlyActive;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/operation currently active/i , qr/Another\s+sis\s+operation\s+is\s+currently\s+active.*/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;

