#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown while trying to perform volume efficiency stop
##          operation when it is already idle. 
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeEfficiency::OperationCurrentlyIdle

=head1 DESCRIPTION

This exception is thrown while trying to perform volume efficiency stop 
operation when it is already idle.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeEfficiency::OperationCurrentlyIdle qw(:try);

   try {
       NACL::C::VolumeEfficiency->stop(
           command_interface  => $Node,
           vserver            => $Vserver,
           %other_opts);
   } catch NACL::C::Exceptions::VolumeEfficiency::OperationCurrentlyIdle with {
        # Volume is currently idle.
   };

=cut

package NACL::C::Exceptions::VolumeEfficiency::OperationCurrentlyIdle;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/operation currently idle/i , qr/Failed to stop efficiency on volume.*Operation is currently idle.*/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;

