#
# Copyright (c) 2017 NetApp Inc.
# All rights reserved.
#
## @summary
#  This exception is thrown when we try to perform volume efficiency "prepare-to-downgrade" on a deprioritized volume
#  before enabling efficiency operations using "volume efficiency promote".
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeEfficiency::PromoteDeprioritizedVolume

=head1 DESCRIPTION

This exception is thrown when we try to perform volume efficiency "prepare-to-downgrade" on a volume before deleting Snapshot copies.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeEfficiency::PromoteDeprioritizedVolume qw(:try);

   try {
       NACL::C::VolumeEfficiency->prepare_to_downgrade(
           command_interface  => $Node,
           %other_opts);
   } catch NACL::C::Exceptions::VolumeEfficiency::PromoteDeprioritizedVolume  with {
        #  enable efficiency operations and retry the command
   };

=cut

package NACL::C::Exceptions::VolumeEfficiency::PromoteDeprioritizedVolume;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
	    qr/.*Failed to modify efficiency configuration for volume .* of Vserver .*Cannot perform efficiency operations on deprioritized volume.*/i,
        ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

sub get_vol_vs_to_promote {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %vol_vs_details = ();
    if ( $pkg->text()
	=~ qr/Failed to modify efficiency configuration for volume \"(.*?)\" of Vserver \"(.*?)\":/i
        )
    {
        $vol_vs_details{'volume'}  = $1;
        $vol_vs_details{'vserver'} = $2;
    }

    $Log->exit() if $may_exit;
    return \%vol_vs_details;
}

1;

