#
# Copyright (c) 2016 NetApp Inc.
# All rights reserved.
#
## @summary
#  This exception is thrown when we try to perform volume efficiency "prepare-to-downgrade" on a volume
#  before deleting Snapshot copies.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeEfficiency::RevertToFailedSnapshotCopied

=head1 DESCRIPTION

This exception is thrown when we try to perform volume efficiency "prepare-to-downgrade" on a volume before deleting Snapshot copies.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeEfficiency::RevertToFailedSnapshotCopied qw(:try);

   try {
       NACL::C::VolumeEfficiency->prepare_to_downgrade(
           command_interface  => $Node,
           %other_opts);
   } catch NACL::C::Exceptions::VolumeEfficiency::RevertToFailedSnapshotCopied  with {
        #  delete snapshot copies and retry the command
   };

=cut

package NACL::C::Exceptions::VolumeEfficiency::RevertToFailedSnapshotCopied;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
            qr/.*Failed to start \"efficiency revert-to\" operation on Volume .* of Vserver .*Use the \"volume snapshot delete\".*Snapshot copies\:.*/i,
        ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

sub get_vol_vs_snapshot_list {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %vol_vs_snapshot_details = ();
    if ( $pkg->text()
        =~ qr/Failed to start \"efficiency revert-to\" operation on Volume (.*) of Vserver (.*)Use the \"volume snapshot delete\".*Snapshot copies\:(.*)/i
        )
    {
        $vol_vs_snapshot_details{'volume'}  = $1;
        $vol_vs_snapshot_details{'vserver'} = $2;

        my $snapshot_list = $3;
        $snapshot_list =~ s/^\s+//;
        $snapshot_list =~ s/.\s+$//;
        my @snapshots = split /\s+/, $snapshot_list;
        $vol_vs_snapshot_details{'snapshot_list'} = \@snapshots;
    }

    $Log->exit() if $may_exit;
    return \%vol_vs_snapshot_details;
}

1;

