#
# Copyright (c) 2016 NetApp Inc.
# All rights reserved.
#
## @summary
#  This exception is thrown when we try to do volume efficiency "prepare-to-downgrade" on a volume
#  before undo compression to "true".
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeEfficiency::RevertToFailedUndoCompression

=head1 DESCRIPTION

This exception is thrown when we try to do volume efficiency config on a volume before starting compression.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeEfficiency::RevertToFailedUndoCompressionqw(:try);

   try {
       NACL::C::VolumeEfficiency->prepare_to_downgrade(
           command_interface  => $Node,
           %other_opts);
   } catch NACL::C::Exceptions::VolumeEfficiency::RevertToFailedUndoCompressionwith {
        #  first disable compression and idd followed by running "volume efficiency undo -compression true"
	#  and then retry the command
   };

=cut

package NACL::C::Exceptions::VolumeEfficiency::RevertToFailedUndoCompression;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/.*Failed to start efficiency revert-to operation on volume .* of Vserver .* Run the \"volume efficiency undo -compression true\".*/i,]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

sub get_vol_vs_list {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %vol_vs_details = ();

    if ( $pkg->text() =~ qr/Failed to start efficiency revert-to operation on volume (.*) of Vserver (.*) Run the \"volume efficiency undo -compression true\"/i ){
	$vol_vs_details{'volume'} = $1;
	$vol_vs_details{'vserver'} = $2;
    }

    $Log->exit() if $may_exit;
    return \%vol_vs_details;
}

1;
