#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a volume efficiency operation is performed on a
##          vserver that does not exist.
##
## @author anupamap@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeEfficiency::VserverDoesNotExist

=head1 DESCRIPTION

This exception is thrown hen a vserver operation is performed on a
vserver that does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeEfficiency::VserverDoesNotExist qw(:try);

   try {
       NACL::C::Vserver->delete(vserver => $not_existing_vserver);
   } catch NACL::C::Exceptions::VolumeEfficiency::VserverDoesNotExist with {
        # Vserver does not exist any more
   };

=cut

package NACL::C::Exceptions::VolumeEfficiency::VserverDoesNotExist;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::DoesNotExist
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/entry\s+doesn't\s+exist/,qr/Vserver.*does\s+not\s+exist/i, qr/Specified\s+vserver\s+not\s+found/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
