#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to get the block
##          information for a block that is a hole.
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeFile::BlockIsAHole

=head1 DESCRIPTION

This exception is thrown when an attempt is made to get the block information
for a block that is a hole.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeFile::BlockIsAHole qw(:try);

   try {
       $file_obj->dumblock_info(blockno => $block, %opts);
   } catch NACL::C::Exceptions::VolumeFile::BlockIsAHole with {
        # Block is a hole
   };

=cut

package NACL::C::Exceptions::VolumeFile::BlockIsAHole;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/block is a hole|block not found/ ],
    );
    $Log->exit();
    return $detected;
}

1;
