# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VolumeFileClone/SourceFileDoesNotExist.pm#4 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when a clone operation is failed as
##          source path does not exist.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeFileClone::SourceFileDoesNotExist

=head1 DESCRIPTION

An exception is thrown when a clone operation is failed as source path does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeFileClone::SourceFileDoesNotExist qw(:try);

   try {
       NACL::C::VolumeFileClone->create(
            'source-path' => $non_existing_source_path,
            %other_opts
       );
   } catch NACL::C::Exceptions::VolumeFileClone::SourceFileDoesNotExist with {
        # Source file does not exist
   };

=cut

package NACL::C::Exceptions::VolumeFileClone::SourceFileDoesNotExist;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::DoesNotExist
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    
    my $detected = $pkg->_detect_helper(
        @args,
        # First regex is for CMode, second is for 7Mode 
        regexes => [ qr/Source file does not exist/i,qr/No such LUN exists/i ]
    );
    
    $Log->exit() if $may_exit;
    return $detected;
}

1;
