# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VolumeQtree/DoesNotExist.pm#1 $
#
# Copyright (c) 2009-2011 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a delete operation is performed on a
##          qtree that does not exist.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeQtree::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when a qtree operation is performed on a
qtree that does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeQtree::DoesNotExist qw(:try);

   try {
       $non_existing_qtree_obj->delete(state => 'offline');
   } catch NACL::C::Exceptions::VolumeQtree::DoesNotExist with {
        # Qtree does not exist any more
   };

=cut

package NACL::C::Exceptions::VolumeQtree::DoesNotExist;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::InvalidParamValueException
    NACL::C::Exceptions::DoesNotExist
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        # First regex is for CMode, second is for 7Mode
        regexes => [ qr/does not exist/, qr/can't find path/ ]
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
