# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VolumeSnapshot/AlreadyExists.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a VolumeSnapshot of the same name already exists
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeSnapshot::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a snapshot which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeSnapshot::AlreadyExists qw(:try);

   try {
       NACL::C::VolumeSnapshot->create(snapshot => $already_existing_snap, %other_opts);
   } catch NACL::C::Exceptions::VolumeSnapshot::AlreadyExists with {
        # VolumeSnapshot already exists.
   };

=cut

package NACL::C::Exceptions::VolumeSnapshot::AlreadyExists;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::InvalidParamValueException
    NACL::C::Exceptions::AlreadyExists
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper( @args,
        regexes => [ qr/.*already exists.*/, qr/.*name exists.*/ ] );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
