#
## Copyright (c) 2009-2015 NetApp Inc.
# All rights reserved.
#
## @summary An exception whrn volume snapshot fails in prepare for revert
##
## @author vanditav@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeSnapshot::DisableSnapshotPolicies

=head1 DESCRIPTION

This exception is thrown when volume snapshot fails in prepare for revert

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeSnapshot::DisableSnapshotPolicies

   try {
       $snapshot->restore();
   } catch NACL::C::Exceptions::VolumeSnapshot::DisableSnapshotPolicies {
       disable snapshot policies and re-run the volume snapshot prepare-for-revert
   };

=cut

package NACL::C::Exceptions::VolumeSnapshot::DisableSnapshotPolicies;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
        NACL::C::Exceptions::Component
        NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg,@args) = @_;
    my $detected = $pkg->_detect_helper(
            @args,
            regexes => [ qr/Disable\s+snapshot\s+policies.*/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
