# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VolumeSnapshot/DoesNotExist.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when entry doesn't exists for VolumeSnapshot
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeSnapshot::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a snapshot
when entry does not exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeSnapshot::DoesNotExist qw(:try);

   try {
       NACL::C::VolumeSnapshot->delete(
           command_interface => $Node,
           volume            => $Volume,
           snapshot          => $Snapshot,
           %other_opts);
   } catch NACL::C::Exceptions::VolumeSnapshot::DoesNotExist with {
        # Entry Does not exists
   };

=cut

package NACL::C::Exceptions::VolumeSnapshot::DoesNotExist;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::DoesNotExist
    NACL::APISet::Exceptions::NoMatchingEntriesException
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected =
      $pkg->_detect_helper( @args,
	        # First regex is for CMode ZAPI, second is for CMode CLI,
                # third is for 7Mode CLI
		regexes => [ qr/.*entry\s+doesn't\s+exist*/i ,
                             qr/.*There\s+are\s+no\s+entries.*/i,
		 	     qr/No\s+such\s+snapshot/i]
      );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
