#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception when entry doesn't exists for VolumeSnapshot
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeSnapshot::RetryCmd

=head1 DESCRIPTION

This exception is thrown when we attempt to downgrade the snapshot 
via prepare_for_revert and its failed with same command suggestion.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeSnapshot::RetryCmd qw(:try);

   try {
       NACL::C::VolumeSnapshot->prepare_for_revert(
           command_interface => $Node,
           node              => $nodename,
           %other_opts);
   } catch NACL::C::Exceptions::VolumeSnapshot::RetryCmd with {
        # Entry Does not exists
   };

=cut

package NACL::C::Exceptions::VolumeSnapshot::RetryCmd;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected =
      $pkg->_detect_helper( @args,
		regexes => [ qr/.*Command\s+to\s+delete\s+snapshots:\s+"snapshot\s+prepare-for-revert\s+\-node\s+<nodename>/i 
		 	     ]
      );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
