#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made the operation on Snapshot
##          which is fenced temporarily.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeSnapshot::SnapshotFenced

=head1 DESCRIPTION

This exception is thrown when an attempt is made the operation on snapshot
which is fenced temporarily.
or
When the Snapshot is used as a reference for Snapmirror relationships.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeSnapshot::SnapshotFenced qw(:try);

   try {
       $snapshot_obj->restore(%opts);
   } catch NACL::C::Exceptions::VolumeSnapshot::SnapshotFenced with {
        # Snapshot is fenced
   };

=cut

package NACL::C::Exceptions::VolumeSnapshot::SnapshotFenced;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Snapshot\s+operation(\s+is)?\s+not\s+allowed/i,
                     qr/Snapshot operation not supported/i,
                     qr/Snapshot.*currently used as a reference.*SnapMirror relationships/i,
                     qr/Snapshot.*currently used as a reference.*Snapshot copy by Job IDs/i,
                   ],
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
