#
# Copyright (c) 2009-2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a VolumeSnapshot is in use
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeSnapshot::SnapshotHasOwner

=head1 DESCRIPTION

This exception is thrown when we attempt to prepare_for_revert
and snapshot is in use and owned by another OnTap subsystem.

=head1 EXAMPLE

    use NACL::C::Exceptions::VolumeSnapshot::SnapshotHasOwner qw(:try);

    try {
        NACL::C::VolumeSnapshot->prepare_for_revert(%opts);
    } catch NACL::C::Exceptions::VolumeSnapshot::SnapshotHasOwner with {
        # VolumeSnapshot is in use.
    };
    
=cut

package NACL::C::Exceptions::VolumeSnapshot::SnapshotHasOwner;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter();
    my ( $pkg, @args ) = @_;

    my $detected =
      $pkg->_detect_helper( @args, regexes => [qr/Cannot remove Snapshot copies that have an owner/i] );
    $Log->exit();
    return $detected;
}

1;
