#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a Vserver of the same name already exists
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Vserver::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a vserver which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::Vserver::AlreadyExists qw(:try);

   try {
       NACL::C::Vserver->create(vserver => $already_existing_vs, %other_opts);
   } catch NACL::C::Exceptions::Vserver::AlreadyExists with {
        # Vserver already exists.
   };

=cut

package NACL::C::Exceptions::Vserver::AlreadyExists;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
                    qr/is\s+already\s+used/i, # CMode CLI/ZAPI
                    qr/name\s+already\s+exists/i, # CMode GUI
                ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
