#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a Vserver of the same name already Stopped
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Vserver::AlreadyStopped

=head1 DESCRIPTION

This exception is thrown when we attempt to create a vserver which already
Stopped.

=head1 EXAMPLE

   use NACL::C::Exceptions::Vserver::AlreadyStopped qw(:try);

   try {
       NACL::C::Vserver->start(vserver => $already_Stopped_vs, %other_opts);
   } catch NACL::C::Exceptions::Vserver::AlreadyStopped with {
        # Vserver already Stopped.
   };

=cut

package NACL::C::Exceptions::Vserver::AlreadyStopped;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Vserver stop requires admin-state of the Vserver to be running./i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
