# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when deletion of vserver fail due to existance of
##  one or more DP volumes on vserver
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Vserver::CannotDeleteDPVserver

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a vserver and it will not
happen due to the existence of DP volumes. This is because the vserver which
is about to delete, is a destination is vserver DR setup.

=head1 EXAMPLE

   use NACL::C::Exceptions::Vserver::CannotDeleteDPVserver qw(:try);

   try {
       NACL::STask::Vserver->purge('vserver' => $vserver, %other_opts);
   } catch NACL::C::Exceptions::Vserver::CannotDeleteDPVserver with {
        #The source vserver needs to be deleted first 
   };

=cut

package NACL::C::Exceptions::Vserver::CannotDeleteDPVserver;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected = $pkg->_detect_helper( @args,
        regexes => [ qr/Vserver that is configured as the destination for Vserver DR/ ] );

    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect

1;
