#
# Copyright (c) 2001-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when deletion of vserver fail due to existance of
##  vserver peer relationship
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Vserver::PeerRelationshipExist

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a vserver and it is
in peer relationship with vserver on other node.

=head1 EXAMPLE

   use NACL::C::Exceptions::Vserver::PeerRelationshipExist qw(:try);

   try {
       NACL::C::Vserver->delete('vserver' => $vserver, %other_opts);
   } catch NACL::C::Exceptions::Vserver::PeerRelationshipExist with {
        # delete vserver peer relationship to delete the vserver
   };

=cut

package NACL::C::Exceptions::Vserver::PeerRelationshipExist;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $error1
         = qr/Cannot delete a Vserver that is part of (a\s)?Vserver peer(ing)?\s?(or transition peer(ing)?)?\s?relationship/i;

    my $detected = $pkg->_detect_helper( @args, regexes => [$error1] );
    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect

1;
