#
# Copyright (c) 2001-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when deletion of vserver fail due to existance of
##  volumes on vserver
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Vserver::VolumesExist

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a vserver and it will not
happen due to the volumes.

=head1 EXAMPLE

   use NACL::C::Exceptions::Vserver::VolumesExist qw(:try);

   try {
       NACL::C::Vserver->delete('vserver' => $vserver, %other_opts);
   } catch NACL::C::Exceptions::Vserver::VolumesExist with {
        # delete volumes to delete the vserver
   };

=cut

package NACL::C::Exceptions::Vserver::VolumesExist;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper( @args,
        regexes => [ qr/Failed to delete Vserver.*[1-9][0-9]* volume/,
                     qr/Failed to delete Vserver.* Volumes: [1-9][0-9]*/is ] );
    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect

1;
