#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is mad eto enable a vserver audit
##          for a vserver which already has audit enabled
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverAudit::AlreadyEnabled

=head1 DESCRIPTION

This exception is thrown when we attempt to enable a vserver audit for a
vserver which already has audit enabled

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverAudit::AlreadyEnabled qw(:try);

   try {
       NACL::C::VserverAudit->enable(vserver => $already_enabled_vserver, %other_opts);
   } catch NACL::C::Exceptions::VserverAudit::AlreadyEnabled with {
        # VserverAudit already enabled.
   };

=cut

package NACL::C::Exceptions::VserverAudit::AlreadyEnabled;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/already enabled/ ],
    );
    $Log->exit();
    return $detected;
}

1;
