#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a VserverAudit of the same name already exists
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverAudit::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a vserver audit for a
vserver which already has an audit configuration existing.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverAudit::AlreadyExists qw(:try);

   try {
       NACL::C::VserverAudit->create(vserver => $already_existing_audit, %other_opts);
   } catch NACL::C::Exceptions::VserverAudit::AlreadyExists with {
        # VserverAudit already exists.
   };

=cut

package NACL::C::Exceptions::VserverAudit::AlreadyExists;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::AlreadyExists
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/already exists/ ],
    );
    $Log->exit();
    return $detected;
}

1;
