#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when attempting to delete an audit configuration
##          for a vserver that does not have an audit configuration.
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverAudit::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when attempting to delete an audit configuration
for a vserver that does not have an audit configuration.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverAudit::DoesNotExist qw(:try);

   try {
       NACL::C::VserverAudit->delete(%opts, vserver => $vserver_without_audit);
   } catch NACL::C::Exceptions::VserverAudit::DoesNotExist with {
        # VserverAudit does not exist any more
   };

=cut

package NACL::C::Exceptions::VserverAudit::DoesNotExist;

use strict;
use warnings;

use base qw(
    NACL::C::Exceptions::DoesNotExist
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/entry doesn't exist/,
                     qr/no entries matching your query/,
                     qr/.*Audit configuration doesn't exist.*/i,
                   ]
    );

    $Log->exit();
    return $detected;
}

1;
