#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for VserverCifs
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverCifs::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a cifs which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverCifs::AlreadyExists qw(:try);

   try {
       NACL::C::VserverCifs->create(
           command_interface  => $Node,
           vserver            => $Vserver,
           'share-name'       => $Share_name,
           %other_opts);
   } catch NACL::C::Exceptions::VserverCifs::AlreadyExists with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::VserverCifs::AlreadyExists;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ 
            qr/Only\sone\sCIFS\sserver\sis\sallowed/i,
            qr/duplicate\sentry/i,
            qr/CIFS\sserver\sis\ssupported\sper/i
        ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
