# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a character map entry already exists
##
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverCifsCharacterMapping::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a character map which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverCifsCharacterMapping::AlreadyExists qw(:try);

   try {
       NACL::C::VserverCifsCharacterMapping->create(
           command_interface => $ci,
           'volume'          => $volume,
           'vserver'         => $vserver,
           %other_opts,
       );
   } catch NACL::C::Exceptions::VserverCifsCharacterMapping::AlreadyExists with{
        # Character map already exists.
   };

=cut

package NACL::C::Exceptions::VserverCifsCharacterMapping::AlreadyExists;

use strict;
use warnings;

use base qw(
    NACL::C::Exceptions::AlreadyExists
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::InvalidParamValueException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected
        = $pkg->_detect_helper( @args, regexes => [qr/duplicate\s+entry/] );
    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect

1;
