# Copyright (c) 2015 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for Symlink
##
## @author ng-ptefit-automation-core
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverCifsSymlink::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a symlink which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverCifsSymlink::AlreadyExists qw(:try);

   try {
       $sched = NACL::C::VserverCifsSymlink->create(
                    command_interface => $Filer_Dst,
                    vserver => $vserver,
                    'unix-path' => $unix_path,
                    'share-name' => $share_name,
                    'cifs-path' => $cifs_path,
                    %other_opts);
   } catch NACL::C::Exceptions::VserverCifsSymlink::AlreadyExists with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::VserverCifsSymlink::AlreadyExists;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/duplicate\s+entry/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
