#
# Copyright (c) 2015 NetApp Inc.
# All rights reserved.
#
## @summary An exception when entry doesn't exists for VserverCifsUsersAndGroupsLocalGroup
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverCifsUsersAndGroupsLocalGroup::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when an attempt is made to delete a 
non-existent local cifs Group

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverCifsUsersAndGroupsLocalGroup::DoesNotExist qw(:try);

   try {
       NACL::C::VserverCifsUsersAndGroupsLocalGroup->delete(
           command_interface => $Node,
           vserver           => $vserver,
           %other_opts);
   } catch NACL::C::Exceptions::VserverCifsUsersAndGroupsLocalGroup::DoesNotExist with {
        # Entry Does not exists
   };

=cut

package NACL::C::Exceptions::VserverCifsUsersAndGroupsLocalGroup::DoesNotExist;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(NACL::C::Exceptions::DoesNotExist);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected = $pkg->SUPER::detect(@args);
    if (!$detected) {
        $detected = $pkg->_detect_helper(@args, regexes => [qr/.*local\s+group\s+\"\S+\"\s+not\s+found.*/i] );
    }
    $Log->exit() if $may_exit;
    return $detected;
}

1;
