# Copyright (c) 2012-2015 NetApp Inc.
# All rights reserved.
#
## @summary An exception when entry doesn't exists for VserverExportPolicyRule
##
## @author ng-ptefit-automation-core
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverExportPolicyRule::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to create a export policy rule
when entry does not exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverExportPolicyRule::DoesNotExist qw(:try);

   try {
       NACL::C::VserverExportPolicyRule->delete(
           command_interface  => $Node,
           vserver            => $Vserver,
           %other_opts);
   } catch NACL::C::Exceptions::VserverExportPolicyRule::DoesNotExist with {
        # Entry Does not exists
   };

=cut

package NACL::C::Exceptions::VserverExportPolicyRule::DoesNotExist;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base 'NACL::C::Exceptions::DoesNotExist';

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected = $pkg->SUPER::detect(@args);
    if (!$detected) {
        $detected = $pkg->_detect_helper(@args, regexes => [qr/.*failed\s+to\s+set\s+field\s+\"policyname\".*/i] );
    }
    $Log->exit() if $may_exit;
    return $detected;
}

1;
