#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception when the FCP service cannot be deleted, while FCP is enabled.
##
## @author rahula@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverFcp::CannotRemoveServiceWhileEnabled

=head1 DESCRIPTION

This exception is thrown when the FCP service cannot be deleted, while FCP is enabled.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverFcp::CannotRemoveServiceWhileEnabled qw(:try);

   try {
       NACL::C::VserverFcp->delete(
           command_interface  => $Node,
           vserver            => $Vserver,
           %other_opts);
   } catch NACL::C::Exceptions::VserverFcp::CannotRemoveServiceWhileEnabled with {
        # FCP service is enabled, so cannot delete the FCP.
   };

=cut

package NACL::C::Exceptions::VserverFcp::CannotRemoveServiceWhileEnabled;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(@args,
        regexes => [qr/Cannot remove service while enabled/i],);
    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect
1;
