#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception when the FCP service already started.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverFcp::ServiceAlreadyStarted

=head1 DESCRIPTION

This exception is thrown when the FCP service already started.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverFcp::ServiceAlreadyStarted qw(:try);

   try {
       NACL::C::VserverFcp->start(
           command_interface  => $Node,
           vserver            => $Vserver,
           %other_opts);
   } catch NACL::C::Exceptions::VserverFcp::ServiceAlreadyStarted with {
        # FCP service already Started.
   };

=cut

package NACL::C::Exceptions::VserverFcp::ServiceAlreadyStarted;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(@args,
        regexes => [qr/Service,\s+adapter,\s+or\s+operation\s+already\s+started/i],);
    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect
1;
