# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VserverFpolicy/AlreadyEnabled.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when we attempt to enable a vserver fpolicy
## which is already enabled.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverFpolicy::AlreadyEnabled

=head1 DESCRIPTION

This exception is thrown when we attempt to enable a vserver fpolicy which is already
enabled.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverFpolicy::AlreadyEnabled qw(:try);

   try {
	 NACL::C::VserverFpolicy->enable(
             'command_interface' => $node,
             'policy-name'       => 'test_policy2',
             'sequence-number'   => '8',
             'vserver'           => $TestVserver,
        );
   } catch NACL::C::Exceptions::VserverFpolicy::AlreadyEnabled with {
        # Already enabled
   };

=cut

package NACL::C::Exceptions::VserverFpolicy::AlreadyEnabled;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/already enabled/i]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
