#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when vserver iSCSI service creation
## fail as it already exist.
##
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverIscsi::IscsiServiceExists

=head1 DESCRIPTION

This exception is thrown when attempt is made to create iSCSI service
that already exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverIscsi::IscsiServiceExists qw(:try);

   try {
       NACL::C::VserverIscsi->create(...);
   } catch NACL::C::Exceptions::VserverIscsi::IscsiServiceExists with {
        # take approriate action or ignore.
   };

=cut

package NACL::C::Exceptions::VserverIscsi::IscsiServiceExists;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper( @args,
        regexes => [ qr/iSCSI service already exists/i ] );
    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect

1;
