# Copyright (c) 2015 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for
#   VserverIscsiSecurity
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverIscsiSecurity::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a local cifs user which
already exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverIscsiSecurity::AlreadyExists qw(:try);

   try {
       $cifs_user = NACL::C::VserverIscsiSecurity->create(
                command_interface => $command_interface,
                'vserver'   => $vserver,
                 %other_opts);
   } catch NACL::C::Exceptions::VserverIscsiSecurity::AlreadyExists with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::VserverIscsiSecurity::AlreadyExists;
use base 'NACL::C::Exceptions::AlreadyExists';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/already\s+has\s+a\s+security\s+policy/,
                    qr/duplicate\sentry/]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
