# Copyright (c) 2015 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there are no nodes  for
#   VserverIscsiSecurity
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverIscsiSecurity::NoNodesAvailable

=head1 DESCRIPTION

This exception is thrown when we attempt to create or delete an iscsi security

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverIscsiSecurity::NoNodesAvailable qw(:try);

   try {
       $cifs_user = NACL::C::VserverIscsiSecurity->create(
                command_interface => $command_interface,
                'vserver'   => $vserver,
                 %other_opts);
   } catch NACL::C::Exceptions::VserverIscsiSecurity::NoNodesAvailable with {
   };

=cut

package NACL::C::Exceptions::VserverIscsiSecurity::NoNodesAvailable;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/No\snodes\sare\savailable\sto\sprocess/,
                    ] 
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
