# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VserverNfs/AllowedProtocolsNotAdded.pm#1 $
#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when NFS is not added to allowed-protocols
##
## @author  dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverNfs::AllowedProtocolsNotAdded

=head1 DESCRIPTION

This exception is thrown when NFS is not added to the allowed-protcols list of 
the vserver.

This exception is a sub-class of
L<NACL::C::Exceptions::AllowedProtocolsNotAdded|lib-NACL-C-Exceptions-AllowedProtocolsNotAdded-pm>,
hence offers the same functionality and has the same attributes.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverNfs::AllowedProtocolsNotAdded qw(:try);

   try {
       NACL::C::VserverNfs->create(vserver => $vserver);
   } catch NACL::C::Exceptions::VserverNfs::AllowedProtocolsNotAdded with {
        my $exception = shift;
        $exception->add_allowed_protocols();
   };

=cut

package NACL::C::Exceptions::VserverNfs::AllowedProtocolsNotAdded;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::AllowedProtocolsNotAdded
    NACL::APISet::Exceptions::CommandFailedException
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/The\s+protocol\s+.*\s+is\s+not\s+allowed\s+to\s+run\s+on\s+Vserver/i]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
