#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an NFS share already exists for a Vserver
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverNfs::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create an NFS share for a
vserver which already has one.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverNfs::AlreadyExists qw(:try);

   try {
       NACL::C::VserverNfs->create(vserver => $vserver_with_nfs_share);
   } catch NACL::C::Exceptions::VserverNfs::AlreadyExists with {
        # NFS share already exists.
   };

=cut

package NACL::C::Exceptions::VserverNfs::AlreadyExists;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/.*duplicate\s+entry.*/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}



1;
