#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a vserver nfs operation is performed on a
##          vserver for which there is no nfs share.
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverNfs::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when a vserver nfs operation is performed on a
vserver for which there is no nfs share.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverNfs::DoesNotExist qw(:try);

   try {
       NACL::C::VserverNfs->delete(%opts);
   } catch NACL::C::Exceptions::VserverNfs::DoesNotExist with {
        # Vserver exists, but it does not have an nfs share
   };

=cut

package NACL::C::Exceptions::VserverNfs::DoesNotExist;


use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::DoesNotExist
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/.*entry\s+doesn\'t\s+exist.*/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
