# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VserverPeer/AlreadyExists.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for VserverPeer
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverPeer::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a vserver peering relationship
which already exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverPeer::AlreadyExists qw(:try);

   try {
       NACL::C::VserverPeer->create(
           command_interface  => $Node,
           vserver            => $Vserver,
	   'peer-vserver'     => $PeerVserver,
           'applications'     => [$Applications],
           %other_opts);
   } catch NACL::C::Exceptions::VserverPeer::AlreadyExists with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::VserverPeer::AlreadyExists;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/already\s+exists/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
