#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when we attempt to delete a VserverPeer relationship which is in use by snapmirror.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverPeer::RelationshipIsInUseBySnapmirror

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a Vserver peering relationship
which is in use by snapmirror.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverPeer::RelationshipIsInUseBySnapmirror qw(:try);

   try {
       NACL::C::VserverPeer->delete(
           command_interface  => $Node,
           vserver            => $Vserver,
	       'peer-vserver'     => $PeerVserver,
           %other_opts);
   } catch NACL::C::Exceptions::VserverPeer::RelationshipIsInUseBySnapmirror with {
        # Relationship is in use by SnapMirror
   };

=cut

package NACL::C::Exceptions::VserverPeer::RelationshipIsInUseBySnapmirror;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Relationship is in use by SnapMirror/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
