# Copyright (c) 2015 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for VserverPeerTransition
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverPeerTransition::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a quota policy rule which
already exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverPeerTransition::AlreadyExists qw(:try);

   try {
       $vserver_peer = NACL::C::VserverPeerTransition->create(
                command_interface => $command_interface,
                'src-filer-name'  => $ip_7mode,
                'local-vserver'   => $local_vserver,
                 %other_opts);
   } catch NACL::C::Exceptions::VserverPeerTransition::AlreadyExists with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::VserverPeerTransition::AlreadyExists;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base 'NACL::C::Exceptions::AlreadyExists';

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected = $pkg->SUPER::detect(@args);
    if (!$detected) {
        $detected = $pkg->_detect_helper(@args, regexes => [qr/.*The pattern (\S+) already exists.*/i] );
    }
    $Log->exit() if $may_exit;
    return $detected;
}
1;
