# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VserverPeerTransition/DoesNotExist.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when entry doesn't exists for VserverPeerTransition
##
## @author raghaveb@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverPeerTransition::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a Vserver peer Transition relationship
when entry does not exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverPeerTransition::DoesNotExist qw(:try);

   try {
       NACL::C::VserverPeerTransition->delete(
           command_interface  => $Node,
           vserver            => $Vserver,
	   'peer-vserver'     => $PeerVserver,
           vserver            => $TestVserver,
           %other_opts);
   } catch NACL::C::Exceptions::VserverPeerTransition::DoesNotExist with {
        # Entry Does not exists
   };

=cut

package NACL::C::Exceptions::VserverPeerTransition::DoesNotExist;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::DoesNotExist
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/doesn\'t\s+exist/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
