# Copyright (c) 2012-2015 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for VserverSecurityTraceFilter
##
## @author ng-ptefit-automation-core
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverSecurityTraceFilter::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a name mapping which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverSecurityTraceFilter::AlreadyExists qw(:try);

   try {
       NACL::C::VserverSecurityTraceFilter->create(
           command_interface  => $Node,
           vserver            => $Vserver,
           %other_opts);
   } catch NACL::C::Exceptions::VserverSecurityTraceFilter::AlreadyExists with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::VserverSecurityTraceFilter::AlreadyExists;
use base 'NACL::C::Exceptions::AlreadyExists';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected = $pkg->SUPER::detect(@args);
    if (!$detected) {
        $detected = $pkg->_detect_helper(@args, regexes => [qr/.*The pattern (\S+) already exists.*/i] );
    }
    $Log->exit() if $may_exit;
    return $detected;
}

1;
