# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VserverServicesDns/DnsServerException.pm#1 $
#
# Copyright (c) 2016 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for VserverServicesDns
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverServicesDns::DnsServerException

=head1 DESCRIPTION

This exception is thrown when we attempt to create vserver services dns and creation fails
with configuration error. 

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverServicesDns::DnsServerException qw(:try);

   try {
       NACL::C::VserverServicesDns->create(
           command_interface  => $Node,
           vserver            => $Vserver,
           name-servers       => $Name_Servers,
           domains            => $Domains,
           state              => $State,
           %other_opts);
   } catch NACL::C::Exceptions::VserverServicesDns::DnsServerException with {
        # Add " 'skip-congif-validation' => 'true' " parameter to bypass this error
   };

=cut

package NACL::C::Exceptions::VserverServicesDns::DnsServerException;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Verify that the network configuration is correct and that DNS servers are available.+\-skip\-config\-validation.+/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
