# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VserverServicesDns/DoesNotExist.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for VserverServicesDns
##
## @author rahula@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverServicesDns::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to create vserver services dns which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverServicesDns::DoesNotExist qw(:try);

   try {
       NACL::C::VserverServicesDns->delete(
           command_interface  => $Node,
            vserver           => $TestVserver,
           %other_opts);
   } catch NACL::C::Exceptions::VserverServicesDns::DoesNotExist with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::VserverServicesDns::DoesNotExist;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException
    NACL::C::Exceptions::DoesNotExist
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/entry\s+doesn't\s+exist/]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
