# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VserverServicesLdap/AlreadyExists.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for VserverServicesLdap
##
## @author rahula@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverServicesLdap::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a vserver services ldap which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverServicesLdap::AlreadyExists qw(:try);

   try {
    NACL::C::VserverServicesLdap->create(
                command_interface => $ci,
                'client-config'   => $client_config,
                'client-enabled'  => $enabled,
                vserver           => $vserver,
    );
   } catch NACL::C::Exceptions::VserverServicesLdap::AlreadyExists with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::VserverServicesLdap::AlreadyExists;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/duplicate\s+entry/i]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
