#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when the provided LDAP configuration does not exist
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverServicesLdap::ClientConfigDoesNotExist

=head1 DESCRIPTION

This exception is thrown when the provided LDAP configuration does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverServicesLdap::ClientConfigDoesNotExist qw(:try);

   try {
       NACL::C::VserverServicesLdap->create(
           'client-config' => 'non-existing-ldap-config', ...);
   } catch NACL::C::Exceptions::VserverServicesLdap::ClientConfigDoesNotExist with {
        # LDAP config does not exist.
   };

=cut

package NACL::C::Exceptions::VserverServicesLdap::ClientConfigDoesNotExist;

use strict;
use warnings;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::NoMatchingEntriesException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;
    
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/LDAP client configuration.*does not exist/i],
    );
    
    $Log->exit();
    return $detected;
}

1;
