# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VserverServicesLdapClientClient/DoesNotExist.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for VserverServicesLdapClient
##
## @author rahula@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverServicesLdapClient::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to create a vserver services ldap client which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverServicesLdapClient::DoesNotExist qw(:try);

   try {
    NACL::C::VserverServicesLdapClient->delete(
                command_interface => $ci,
                'client-config'   => $client_config,
    );
   } catch NACL::C::Exceptions::VserverServicesLdapClient::DoesNotExist with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::VserverServicesLdapClient::DoesNotExist;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException
    NACL::C::Exceptions::DoesNotExist
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/.*entry\s+doesn\'t\s+exist.*/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
