# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VserverServicesNisDomain/AlreadyExists.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for VserverServicesNisDomain
##
## @author rahula@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverServicesNisDomain::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create vserver services nis which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverServicesNisDomain::AlreadyExists qw(:try);

   try {
       NACL::C::VserverServicesNisDomain->create(
           command_interface  => $Node,
           vserver            => $Vserver,
           servers            => [$Servers],
           domain             => $Domain,
           active             => $Active,
           %other_opts);
   } catch NACL::C::Exceptions::VserverServicesNisDomain::AlreadyExists with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::VserverServicesNisDomain::AlreadyExists;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/duplicate\s+entry/, qr/already exists/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
