# $Id:$
#
# Copyright (c) 2001-2012 NetApp Inc.
# All rights reserved.
#
## @summary Exception to be thrown when there is a duplicate unix user exist
##
## @author chethanm@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverServicesUnixUser::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when the unix user already exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverServicesUnixUser::AlreadyExists qw(:try);

   try {
       $unix_group->create(%opts);
   } catch NACL::C::Exceptions::VserverServicesUnixUser::AlreadyExists with {
        my $ex = shift;
        my $event_objs = $ex->state_objects();
        $Test->nlog("Got the error " . $ex->text());
   };

=cut

package NACL::C::Exceptions::VserverServicesUnixUser::AlreadyExists;

use strict;
use warnings;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected
        = $pkg->_detect_helper( @args, regexes => [qr/duplicate\s+entry/] );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
