
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary JobHistory Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::JobHistory

=head1 DESCRIPTION

C<NACL::C::JobHistory> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP JobHistory.  A related class is
L<NACL::CS::JobHistory|lib-NACL-CS-JobHistory-pm>, which represents
the past state of an ONTAP JobHistory.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 record

=head2 node

=cut

package NACL::C::JobHistory;

use strict;
use warnings;

use NACL::CS::JobHistory;
use NACL::ComponentUtils qw (_optional_scalars _retry_spec _retry_logic);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF OBJECT);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $JobHistory = NACL::C::JobHistory->new(command_interface=>$ci,
                                   record =>$record,
                                   node =>$node,
                                   vserver =>$vserver,
								  );

Return a new JobHistory component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'record',
    scalar => 'node',
    scalar => 'vserver',
];

=head2 find

    my $JobHistory  = NACL::C::JobHistory->find(command_interface => $ci, ...);
	my @JobHistorys = NACL::C::JobHistory->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $JobHistory_state = $NACL::C::JobHistory->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::JobHistory would return 
a NACL::CS::JobHistory object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 test

   NACL::C::JobHistory->test(
                                command_interface => $ci,
	                             record => $record,
	                             node => $node,
                                     vserver =>$vserver,
                                  ...);

(Class method)
This method is used to tests job history iterator.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "record" => $record >>

(Required)

Applicable for CMode CLI.

=item C<< "node" => $node >>

(Required)

Applicable for CMode CLI.

=item C<< "vserver" => $vserver >>

(Required)

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"job history test" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub test {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        api    => 'job_history_test',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_test_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::JobHistory::test',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub test

sub _test_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    $pkg->_base_backend_cmode_cli(@args,
            api => 'job_history_test');

    $Log->exit() if $may_exit;
} ## end sub _test_cmode_cli

sub _primary_keys_validate_spec {
    return (
        'record'  => {type => SCALAR},
        'node'    => {type => SCALAR | OBJECT},
        'vserver' => {type => SCALAR | OBJECT},
    );
} ## end sub _primary_keys_validate_spec

# overriden call_on_apiset to handle product burt 517892
# ONTAP complains that job is not found for an existing job id
# because of the timing issues. This is a rare case hit only when
# executing commands in a fast fashion using the automated scripts.
sub call_on_apiset {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $retry    = delete $opts{nacl_retry_count};
    my $response = $pkg_or_obj->_retry_logic(
        method => 'SUPER::call_on_apiset',
        args   => \%opts,
        retry  => $retry,
        exception_type =>
            'NACL::APISet::Exceptions::NoMatchingEntriesException'
    );
    $Log->exit() if $may_exit;
    return $response;
}

# overriden _common_validate_with to accept nacl_retry_count as the valid
# option
sub _common_validate_with {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

   # Add nacl_retry_count to the additional spec as an optional scalar
   # Default is to try 3 times. This is not applicable for all the
   # methods of this job component (say db, bulk_remove, etc.,) but we
   # are still doing it. So if a user tries to pass this option for the
   # commands which doesn't take job id in 'id', validation will pass as we
   # have overriden _common_validate_with and so this code is applicable
   # for all the methods of this component and its inherited classes
   # as well. Since passing this option has no other harmful effects, we
   # are doing it this way as this makes the implementation easy and it also
   # handles any new commands added in job directory which will take job id in
   # 'id' parameter automatically - merits have overcome the demerits!
    $opts{additional_spec}->{'nacl_retry_count'} = $pkg_or_obj->_retry_spec();
    my %args = $pkg_or_obj->SUPER::_common_validate_with(%opts);
    $Log->exit() if $may_exit;
    return %args;
}

sub _find_validate_spec {
    $Log->enter() if $may_enter;
    my $pkg_or_obj         = shift;
    my $find_validate_spec = $pkg_or_obj->SUPER::_find_validate_spec();
    $find_validate_spec->{nacl_retry_count} = $pkg_or_obj->_retry_spec;

    $Log->exit() if $may_exit;
    return $find_validate_spec;
} ## end sub _find_validate_spec

1;
