#//depot/prod/test/nacldev/lib/NACL/C/JobScheduleCron.pm#17 - edit change 1751629 (ktext)
# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary JobScheduleCron Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::JobScheduleCron

=head1 DESCRIPTION

C<NACL::C::JobScheduleCron> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP JobScheduleCron.  A related class is
L<NACL::CS::JobScheduleCron|lib-NACL-CS-JobScheduleCron-pm>, which represents
the past state of an ONTAP JobScheduleCron.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 name

=cut

package NACL::C::JobScheduleCron;

use strict;
use warnings;

use NACL::CS::JobScheduleCron;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use NACL::C::Exceptions::Component qw(:try);
use NACL::C::Exceptions::JobScheduleCron::DoesNotExist ();
use NACL::C::Exceptions::JobScheduleCron::AlreadyExists;
use NACL::APISet::Exceptions::ResponseException qw(:try);

=head1 METHODS

=head2 new

  my $JobScheduleCron = NACL::C::JobScheduleCron->new(command_interface=>$ci,
                                   name =>$name,
                                  );

Return a new JobScheduleCron component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'name',];

=head2 find

    my $JobScheduleCron  = NACL::C::JobScheduleCron->find(command_interface => $ci, ...);
    my @JobScheduleCrons = NACL::C::JobScheduleCron->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $JobScheduleCron_state = $NACL::C::JobScheduleCron->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::JobScheduleCron would return
a NACL::CS::JobScheduleCron object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 delete

   NACL::C::JobScheduleCron->delete(
                                command_interface => $ci,
                                 name => $name,
                                  ...);

                                 or

    $JobScheduleCron_object->delete(...);

(Class or instance method)
This method is used to delete a cron schedule.

Uses a CMode CLI/ZAPI APISet.

Invokes "job-schedule-cron-destroy-iter" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "name" => $name >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-name

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"job schedule cron delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        cli_cmd  => 'job schedule cron delete',
    );
    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_delete_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::JobScheduleCron::delete',
        );
    }
    catch NACL::C::Exceptions::Component with {
        $Log->exit() if $may_exit;
        $_[0]->throw();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::JobScheduleCron::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    $pkg_or_obj->_base_backend_cmode_cli(@args,
            api => 'job_schedule_cron_delete');

    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    $pkg_or_obj->_iter_helper(
        @args,
        api                  => 'job-schedule-cron-destroy-iter',
        show_cmd             => 'job schedule cron show',
        map                  => {"name" => "job-schedule-name",},
        does_not_exist_error => "entry doesn't exist",
        does_not_exist_exception =>
            'NACL::C::Exceptions::JobScheduleCron::DoesNotExist',
        no_attributes => 1,
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 create

    my $JobScheduleCron_object    = NACL::C::JobScheduleCron->create(
                                command_interface => $ci,
                                 name => $name,
                                  ...);

(Class method)
This method is used to create a cron schedule.

Uses a CMode CLI/ZAPI APISet.

Invokes "job-schedule-cron-create" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "name" => $name >>

(Required)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-name

=item C<< "hour" => [ $hour1, $hour2, ... ] >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-cron-hour

=item C<< "minute" => [ $minute1, $minute2, ... ] >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-cron-minute

=item C<< "month" => [ $month1, $month2, ... ] >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-cron-month

=item C<< "dayofweek" => [ $dayofweek1, $dayofweek2, ... ] >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-cron-day-of-week

=item C<< "day" => [ $day1, $day2, ... ] >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-cron-day

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"job schedule cron create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::JobScheduleCron::AlreadyExists>

This type of exception gets thrown if an attempt is made to schedule a cron job
that already exists.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "hour"      => {type => ARRAYREF, optional => 1},
            "minute"    => {type => ARRAYREF, optional => 1},
            "month"     => {type => ARRAYREF, optional => 1},
            "dayofweek" => {type => ARRAYREF, optional => 1},
            "day"       => {type => ARRAYREF, optional => 1},
        },
        cli_cmd => 'job schedule cron create',
    );
    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_create_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_create_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::JobScheduleCron::create',
        );
     }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::JobScheduleCron::AlreadyExists
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    
    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        name              => $opts{'name'},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,
        api     => 'job_schedule_cron_create',
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    %opts = $pkg->_handle_job_options(%opts);
    $pkg->_base_backend_cmode_zapi(%opts,
        api    => 'job_schedule_cron_create',
        map    => {
            "name"      => "job-schedule-name",
            "month"     => "job-schedule-cron-month",
            "dayofweek" => "job-schedule-cron-day-of-week",
            "day"       => "job-schedule-cron-day",
            "hour"      => "job-schedule-cron-hour",
            "minute"    => "job-schedule-cron-minute",
        },
        zapi_field_translations => {
            cron_time_integer => [qw(hour minute month day dayofweek)],
        },
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

=head2 modify

   NACL::C::JobScheduleCron->modify(
                                command_interface => $ci,
                                 name => $name,
                                  ...);

                                 or

    $JobScheduleCron_object->modify(...);

(Class or instance method)
This method is used to modify a cron schedule.

Uses a CMode CLI/ZAPI APISet.

Invokes "job-schedule-cron-modify" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "name" => $name >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-name

=item C<< "hour" => [ $hour1, $hour2, ... ] >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-cron-hour

=item C<< "minute" => [ $minute1, $minute2, ... ] >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-cron-minute

=item C<< "month" => [ $month1, $month2, ... ] >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-cron-month

=item C<< "dayofweek" => [ $dayofweek1, $dayofweek2, ... ] >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-cron-day-of-week

=item C<< "day" => [ $day1, $day2, ... ] >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-schedule-cron-day

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"job schedule cron modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "hour"      => {type => ARRAYREF, optional => 1},
            "minute"    => {type => ARRAYREF, optional => 1},
            "month"     => {type => ARRAYREF, optional => 1},
            "dayofweek" => {type => ARRAYREF, optional => 1},
            "day"       => {type => ARRAYREF, optional => 1},
        },
        cli_cmd  => 'job schedule cron modify',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::JobScheduleCron::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'job_schedule_cron_modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    %opts = $pkg_or_obj->_handle_job_options(%opts);
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
        api    => 'job_schedule_cron_modify',
        map    => {
            "name"      => "job-schedule-name",
            "month"     => "job-schedule-cron-month",
            "dayofweek" => "job-schedule-cron-day-of-week",
            "day"       => "job-schedule-cron-day",
            "hour"      => "job-schedule-cron-hour",
            "minute"    => "job-schedule-cron-minute",
        },
        zapi_field_translations => {
            cron_time_integer => [qw(hour minute month day dayofweek)],
        },
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

sub _handle_job_options {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    my %month_map = (
        all       => -1,
        january   => 0,
        february  => 1,
        march     => 2,
        april     => 3,
        may       => 4,
        june      => 5,
        july      => 6,
        august    => 7,
        september => 8,
        october   => 9,
        november  => 10,
        december  => 11
    );
    my %dayofweek_map = (
        all       => -1,
        sunday    => 0,
        monday    => 1,
        tuesday   => 2,
        wednesday => 3,
        thursday  => 4,
        friday    => 5,
        saturday  => 6
    );

    if (defined $opts{month}) {
        my $handle_month = sub {
            my $actual_val = $_;
            my $month      = lc($actual_val);
            $actual_val
                = (defined $month_map{$month})
                ? $month_map{$month}
                : $actual_val;
            return $actual_val;
        };
        my @months = map { $handle_month->($_) } @{$opts{month}};
        $opts{month} = \@months;
    }
    if (defined $opts{dayofweek}) {
        my $handle_dayofweek = sub {
            my $actual_val = $_;
            my $dayofweek  = lc($actual_val);
            $actual_val
                = (defined $dayofweek_map{$dayofweek})
                ? $dayofweek_map{$dayofweek}
                : $actual_val;
            return $actual_val;
        };
        my @dayofweeks = map { $handle_dayofweek->($_) } @{$opts{dayofweek}};
        $opts{dayofweek} = \@dayofweeks;
    }
    if (defined $opts{day}) {

        # No mapping required
    }
    if (defined $opts{hour}) {
        my $handle_hour = sub {
            my $actual_val = $_;
            $actual_val = ($actual_val eq 'all') ? '-1' : $actual_val;
            return $actual_val;
        };
        my @hours = map { $handle_hour->($_) } @{$opts{hour}};
        $opts{hour} = \@hours;
    }
    if (defined $opts{minute}) {

        # No mapping required
    }

    $Log->exit() if $may_exit;
    return %opts;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('name' => {type => SCALAR},);
}

1;
