# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary LeagueApplication Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::LeagueApplication

=head1 DESCRIPTION

C<NACL::C::LeagueApplication> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP LeagueApplication.  A related class is
L<NACL::CS::LeagueApplication|lib-NACL-CS-LeagueApplication-pm>, which represents
the past state of an ONTAP LeagueApplication.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 application_number

=head2 league_name

=cut

package NACL::C::LeagueApplication;

use strict;
use warnings;

use NACL::CS::LeagueApplication;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $LeagueApplication = NACL::C::LeagueApplication->new(
                                   command_interface => $ci,
                                   'application-number' => $application_number,
                                   'league-name' => $league_name,
								  );

Return a new LeagueApplication component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'application_number',
    scalar => 'league_name',
];

=head2 find

    my $LeagueApplication_obj  = NACL::C::LeagueApplication->find(command_interface => $ci, ...);
	my @LeagueApplication_objs = NACL::C::LeagueApplication->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my LeagueApplication_state = $LeagueApplication_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::LeagueApplication would return 
a NACL::CS::LeagueApplication object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 accept

    NACL::C::LeagueApplication->accept(
                                command_interface => $ci,
	                            'application-number' => $application_number,
	                            'league-name' => $league_name,
                                ...);
	                             
								 or

    $LeagueApplication_obj->accept(...);

(Class or instance method)
This method is used to accept a pending application for membership.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'application-number' => $application_number >>

(Required for class method, Not Applicable for instance method)
Application Serial Number
Applicable for CMode CLI

=item C<< 'league-name' => $league_name >>

(Required for class method, Not Applicable for instance method)
League Name
Applicable for CMode CLI

=item C<< 'keyphrase' => $string >>

Matching Keyphrase for Shared Secret
Applicable for CMode CLI

=item C<< 'passphrase' => $string >>

Matching Passphrase for Identification
Applicable for CMode CLI

=item C<< 'remote-cluster-name' => $string >>

Applicant Cluster Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "league application accept" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub accept {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( keyphrase passphrase remote-cluster-name
                    )
            ),
        },
        api => 'league_application_accept',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_accept_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LeagueApplication::accept',
    );

}

sub _accept_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'league_application_accept',);
    $Log->exit() if $may_exit;
}

=head2 reject

    NACL::C::LeagueApplication->reject(
                                command_interface => $ci,
	                            'application-number' => $application_number,
	                            'league-name' => $league_name,
                                ...);
	                             
								 or

    $LeagueApplication_obj->reject(...);

(Class or instance method)
This method is used to reject a pending application for membership.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'application-number' => $application_number >>

(Required for class method, Not Applicable for instance method)
Application Serial Number
Applicable for CMode CLI

=item C<< 'league-name' => $league_name >>

(Required for class method, Not Applicable for instance method)
League Name
Applicable for CMode CLI

=item C<< 'reason' => $string >>

Reason for Rejection
Applicable for CMode CLI

=item C<< 'remote-cluster-name' => $string >>

Applicant Cluster Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "league application reject" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub reject {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( reason remote-cluster-name
                    )
            ),
        },
        api => 'league_application_reject',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_reject_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LeagueApplication::reject',
    );

}

sub _reject_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'league_application_reject',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'application-number' => {type => SCALAR},
        'league-name'        => {type => SCALAR},
    );
}

1;
