#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary LunCopy Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::LunCopy

=head1 DESCRIPTION

C<NACL::C::LunCopy> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP LunCopy.  A related class is
L<NACL::CS::LunCopy|lib-NACL-CS-LunCopy-pm>, which represents
the past state of an ONTAP LunCopy.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 destination_path

=head2 vserver

=cut

package NACL::C::LunCopy;

use strict;
use warnings;

use NACL::CS::LunCopy;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate
    qw( validate validate_with SCALAR ARRAYREF OBJECT HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::Exceptions::InvalidChoice qw(:try);
use NACL::C::_Mixins::LunCopyRelocation qw(:all);
use NACL::C::Exceptions::LunCopy::MaximumSimultaneousOperation ();
use NACL::APISet::Exceptions::ResponseException;

=head1 METHODS

=head2 new

  my $LunCopy = NACL::C::LunCopy->new(
                                   command_interface  => $ci,
                                   'destination-path' => $destination_path,
                                   'vserver'          => $vserver,
                                   'job-uuid'         => $uuid,
                                   'lun-index'        => $lun_index
                                  );

Return a new LunCopy component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'destination_path',
    scalar => 'vserver',
    scalar => 'job_uuid',
    scalar => 'lun_index'

];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $LunCopy_obj  = NACL::C::LunCopy->find(command_interface => $ci, ...);
    my @LunCopy_objs = NACL::C::LunCopy->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my LunCopy_state = $LunCopy_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::LunCopy would return
a NACL::CS::LunCopy object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 resume

    NACL::C::LunCopy->resume(
                                command_interface => $ci,
                                'destination-path' => $destination_path,
                                'vserver' => $vserver,
                                'job-uuid => $uuid,
                                ...);

                                 or

    $LunCopy_obj->resume(...);

(Class or instance method)
This method is used to resume a paused LUN copy operation..

Uses a CMode CLI/ZAPI APISet.

Invokes "lun-copy-resume" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'destination-path' => $destination_path >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : destination-path

=item C<< 'job-uuid' => $job_uuid >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-uuid'=

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'lun-index' => $string >>

LUN Index
Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub resume {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( lun-index vserver destination-path job-uuid
                    )
            )
        },
    );

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_resume_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_resume_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub resume

sub _resume_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(api => 'lun_copy_resume', %opts);
    $Log->exit() if $may_exit;
} ## end sub _resume_cmode_cli

sub _resume_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    $pkg_or_obj->_base_backend_cmode_zapi(api => 'lun-copy-resume', %opts);
    $Log->exit() if $may_exit;
} ## end sub _resume_cmode_zapi

=head2 modify

    NACL::C::LunCopy->modify(
                                command_interface => $ci,
                                'destination-path' => $destination_path,
                                'vserver' => $vserver,
                                'job-uuid' => $uuid,
                                ...);

                                 or

    $LunCopy_obj->modify(...);

(Class or instance method)
This method is used to modify an ongoing LUN copy operation..

Uses a CMode CLI/ZAPI APISet.

Invokes "lun-copy-modify" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'destination-path' => $destination_path >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : destination-path

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< 'job-uuid' => $job_uuid >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-uuid

=item C<< 'max-cutover-time' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : max-cutover-time

=item C<< 'max-throughput' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : max-throughput

=item C<< 'scanner-paused' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : scanner-paused

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'lun-index' => $string >>

LUN Index
Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( max-cutover-time max-throughput scanner-paused
                    lun-index destination-path job-uuid
                    )
            ),
        },
    );

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(%opts, api => 'lun_copy_modify');
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_modify_cmode_zapi_helper(%opts, api => 'lun-copy-modify');
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

=head2 pause

    NACL::C::LunCopy->pause(
                                command_interface => $ci,
                                'destination-path' => $destination_path,
                                'vserver' => $vserver,
                                'job-uuid' => $uuid,
                                ...);

                                 or

    $LunCopy_obj->pause(...);

(Class or instance method)
This method is used to pause an ongoing LUN copy operation..

Uses a CMode CLI/ZAPI APISet.

Invokes "lun-copy-pause" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'destination-path' => $destination_path >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : destination-path

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< 'job-uuid' => $job_uuid >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-uuid

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'lun-index' => $string >>

LUN Index
Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub pause {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( lun-index vserver destination-path job-uuid
                    )
            )
        },
    );

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_pause_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_pause_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub pause

sub _pause_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(api => 'lun_copy_pause', %opts);
    $Log->exit() if $may_exit;
} ## end sub _pause_cmode_cli

sub _pause_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(%opts, api => 'lun-copy-pause');
    $Log->exit() if $may_exit;
} ## end sub _pause_cmode_zapi

=head2 start

    my $LunCopy_obj    = NACL::C::LunCopy->start(
                                command_interface => $ci,
                                'destination-path' => $destination_path,
                                'vserver' => $vserver,
                                ...);

(Class method)
This method is used to start copying a LUN from one volume to another within a vserver.

Uses a CMode CLI/ZAPI APISet.

Invokes "lun-copy-start" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'destination-path' => $destination_path >>

(Required)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI :
<paths>
<lun-path-pair>
<destination-path>$scalar</destination-path>
</lun-path-pair>
</paths>

=item C<< 'vserver' => $vserver >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< 'max-cutover-time' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : max-cutover-time

=item C<< 'max-throughput' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : max-throughput

=item C<< 'scanner-paused' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : scanner-paused

=item C<< 'source-path' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI :
<paths>
<lun-path-pair>
<source-path>$scalar</source-path>
</lun-path-pair>
</paths>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'promote-early' => $string >>

*Promote Early, possible value(s) are true,false

Applicable for CMode CLI

=item C<< 'job-uuid' => $string >>

Applicable for CMode CLI/ZAPI

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'source-paths' => [ $source-path1, $source-path2, ... ] >>

Source Paths
(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI:

<paths>
<lun-path-pair>
<source-path>$scalar</source-path>
</lun-path-pair>
</paths>

=item C<< 'destination-paths' => [ $destination-path1, $destination-path2, ... ] >>

Destination Paths
(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI

Maps to;

CM ZAPI:
<paths>
<lun-path-pair>
<destination-path>$scalar</destination-path>
</lun-path-pair>
</paths>

=item C<< 'source-vserver' => $string >>

Source Vserver Name
Applicable for CMode CLI/ZAPI

=item C<< 'pause-transfer' => $string >>

Pause Transfer
Applicable for CMode CLI/ZAPI

=item C<< 'reference-path' => $string >>

Source Reference Path
Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::LunCopy::MaximumSimultaneousOperation>

This exception gets thrown, when we attempt for
Lun copy operation and maximum simultaneous file operations reached.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "source-paths"      => {type => ARRAYREF, optional => 1},
            "destination-paths" => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( max-cutover-time max-throughput scanner-paused source-path
                    promote-early job-uuid vserver destination-path
                    source-vserver pause-transfer

                    reference-path
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    my $job_uuid;
    $opts{_job_uuid} = \$job_uuid;

    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_start_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_start_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                    check     => '_check_zapi_option',
                },
            ],
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::LunCopy::MaximumSimultaneousOperation
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    my (@objs, $index);
    $opts{'destination-paths'} ||= [$opts{'destination-path'}];
    foreach my $dest_path (@{$opts{'destination-paths'}}) {
        my $obj = $pkg->new(
            command_interface  => $opts{command_interface},
            vserver            => $opts{vserver},
            'destination-path' => $dest_path,
            );
        if (defined $job_uuid) {
            $obj->job_uuid($job_uuid);
            $obj->lun_index($index++);
        }
        push @objs, $obj;
    }
    $Log->exit() if $may_exit;
    return wantarray ? @objs : $objs[0];
} ## end sub start

sub _start_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);
    my $response;
    $pkg->_base_backend_cmode_cli(
			%opts,
			api => 'lun_copy_start',
			apiset_response_obj => \$response
    );
    $Log->exit() if $may_exit;

} ## end sub _start_cmode_cli

sub _start_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my (%map, %additional_args, $response);
    my $copy = $pkg->_start_common_copy();

    if (!$opts{command_interface}->isa('NACL::C::Vserver')) {
        push(@$copy, 'source-vserver');
        $map{vserver} = 'destination-vserver';
    }
    $additional_args{'paths'} = $pkg->_construct_lun_path_pairs(%opts);
    delete $opts{'destination-path'};
    delete $opts{'source-path'};

    $pkg->SUPER::_base_backend_cmode_zapi(
        %opts,
        copy                => $copy,
        map                 => \%map,
        apiset_response_obj => \$response,
        api                 => 'lun-copy-start',
        frontend_method     => 'start',
        additional_args     => \%additional_args,
    );
    my $job_uuid = $response->get_parsed_output()->[0]{'job-uuid'};

    ${$opts{_job_uuid}} = $job_uuid;
    $Log->exit() if $may_exit;
} ## end sub _start_cmode_zapi

=head2 destroy

    NACL::C::LunCopy->destroy(
                                command_interface => $ci,
                                'destination-path' => $destination_path,
                                'vserver' => $vserver,
                                'job-uuid' => $uuid,
                                ...);

                                 or

    $LunCopy_obj->destroy(...);

(Class or instance method)
This method is used to abort an ongoing LUN copy operation..

Uses a CMode CLI/ZAPI APISet.

Invokes "lun-copy-destroy" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'destination-path' => $destination_path >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : destination-path

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< 'job-uuid' => $job_uuid >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : job-uuid

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'force' => $string >>

*Force destroy

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : force

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub destroy {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw(force vserver destination-path job-uuid lun-index))
        },
    );

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_destroy_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_destroy_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub destroy

sub _destroy_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(%opts, api => 'lun_copy_destroy');
    $Log->exit() if $may_exit;
} ## end sub _destroy_cmode_cli

sub _destroy_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    $pkg_or_obj->_base_backend_cmode_zapi(api => 'lun-copy-destroy', %opts);
    $Log->exit() if $may_exit;
} ## end sub _destroy_cmode_zapi

=head2 cancel

    NACL::C::LunCopy->cancel(
                                command_interface => $ci,
	                            'destination-path' => $destination_path,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $LunCopy_obj->cancel(...);

(Class or instance method)
This method is used to cancel a LUN copy operation before the new LUN has been created.
Uses CMode CLI/ZAPI .

Invokes 'lun-copy-cancel' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'destination-path' => $destination_path >>

(Required for class method, Not Applicable for instance method)
Destination Path
Applicable for CMode CLI

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name
Applicable for CMode CLI

=item C<< 'job-uuid' => $string >>

LUN Copy Job UUID
Applicable for CMode CLI/ZAPI

=item C<< 'lun-index' => $string >>

LUN Index
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "lun copy cancel" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub cancel {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( job-uuid lun-index
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'lun_copy_cancel',
    );

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_cancel_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_cancel_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LunCopy::cancel',
    );

    $Log->exit() if $may_exit;
}

sub _cancel_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts, api => 'lun_copy_cancel',);
    $Log->exit() if $may_exit;
}

# The object could store all of vserver + destination-path + job-uuid +
# lun-index. However, the CLI command does not allow of them to be passed
# together. So if either vserver/destination-path exists, remove job-uuid and
# lun-index from the list of options to be passed to the command.
sub get_api_options {
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => {options => {type => HASHREF, default => {}}},
        allow_extra => 1
    );
    my %hash = $pkg_or_obj->SUPER::get_api_options(%opts);
    if (exists $hash{vserver} || exists $hash{'destination-path'}) {
        delete $hash{'job-uuid'};
        delete $hash{'lun-index'};
    }
    return %hash;
}

sub _base_backend_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => {copy => {type => ARRAYREF, default => []}},
        allow_extra => 1
    );
    $pkg_or_obj->_get_uuid_lun_index(\%opts);
    $opts{copy} = [@{$opts{copy}}, (qw(job-uuid lun-index))];
    $pkg_or_obj->SUPER::_base_backend_cmode_zapi(%opts,);
    $Log->exit() if $may_exit;
} ## end sub _base_backend_cmode_zapi

sub _cancel_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(api => 'lun-copy-cancel', %opts);
    $Log->exit() if $may_exit;
} ## end sub _cancel_cmode_zapi

=head2 show_by_job_info

    my $show_by_job_info_obj = NACL::C::LunCopy->show_by_job_info(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_by_job_info_objs = NACL::C::LunCopy->show_by_job_info(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_by_job_info_obj = $LunCopy_obj->show_by_job_info(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-LunCopyShowByJobInfo-pm/ATTRIBUTES>
of NACL::CS::LunCopyShowByJobInfo as input. 
This method returns objects of the type L<NACL::CS::LunCopyShowByJobInfo|lib-NACL-CS-LunCopyShowByJobInfo-pm>.

=cut

sub show_by_job_info {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::LunCopyShowByJobInfo;
    my @state_objs = NACL::CS::LunCopyShowByJobInfo->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'destination-path' => {type => SCALAR},
        'vserver'          => {type => SCALAR},
        'job-uuid'         => {type => SCALAR, optional => 1},
        'lun-index'        => {type => SCALAR, optional => 1}
    );

} ## end sub _primary_keys_validate_spec


1;
