
#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary LunImport Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::LunImport

=head1 DESCRIPTION

C<NACL::C::LunImport> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP LunImport.  A related class is
L<NACL::CS::LunImport|lib-NACL-CS-LunImport-pm>, which represents
the past state of an ONTAP LunImport.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=head2 path

=cut

package NACL::C::LunImport;

use strict;
use warnings;

use NACL::CS::LunImport;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $LunImport = NACL::C::LunImport->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
                                    'path'    => $path
								  );

Return a new LunImport component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'path'
];

=head2 find

    my $LunImport_obj  = NACL::C::LunImport->find(command_interface => $ci, ...);
	my @LunImport_objs = NACL::C::LunImport->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

Invokes "lun-import-get-iter" api for CMode ZAPI.

=head2 state

    my LunImport_state = $LunImport_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::LunImport would return
a NACL::CS::LunImport object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 create

    my $LunImport_obj	= NACL::C::LunImport->create(
                                command_interface => $ci,
	                            'foreign-disk' => $foreign_disk,
	                            'vserver' => $vserver,
                                ...);

(Class method)
This method is used to create an import relationship.
Uses CMode CLI/ZAPI.

Invokes "lun-import-create" api for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'foreign-disk' => $foreign_disk >>

(Required)
Foreign Disk Serial Number
Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: foreign-disk

=item C<< 'vserver' => $vserver >>

(Required)
Vserver Name
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: vserver

=item C<< 'path' => $string >>

(Required)
LUN Path
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: path

=item C<< 'path' => $string >>

LUN Path
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "lun import create" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        api    => 'lun_import_create',
        ,
        additional_spec => {
            _optional_scalars(
                qw( path
                    )
            )
        },
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LunImport::create',
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $opts{'vserver'},
        path              => $opts{'path'}
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'lun_import_create',);
    $Log->exit() if $may_exit;
}

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(vserver path foreign-disk)],
        api  => 'lun_import_create',
    );
    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::LunImport->delete(
                                command_interface => $ci,
	                            'foreign-disk' => $foreign_disk,
                                ...);

								 or

    $LunImport_obj->delete(...);

                                                                or
    NACL::C::LunImport->delete(
                                command_interface => $ci,
	                            'path'=> $path,
	                            'vserver' => $vserver,
                                ...);

(Class or instance method)
This method is used to deletes the import relationship of the specified LUN or the specified foreign disk.
Uses CMode CLI/ZAPI.

Invokes "lun-import-delete" api for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'foreign-disk' => $foreign_disk >>

(Either path or foreign-disk is required for class method, Not Applicable for instance method)
Foreign Disk Serial Number
Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: foreign-disk

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name
Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: vserver

=item C<< 'path' => $string >>

LUN Path
(Either path or foreign-disk is required for class method)
Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: path

=item C<< 'force' => $string >>

Force Delete possible value(s) are, true,false
Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: force

=item C<< '*' >>

This method takes all other options supported by "lun import delete" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( force vserver path foreign-disk
                    )
            ),
        },
        api => 'lun_import_delete',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LunImport::delete',
    );

}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts, api => 'lun_import_delete',);
    $Log->exit() if $may_exit;
}

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        copy => [qw(vserver path force foreign-disk)],
        api  => 'lun_import_delete',
    );
    $Log->exit() if $may_exit;
}

=head2 pause

    NACL::C::LunImport->pause(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);

(Class method)
This method is used to pause the import for the specified LUN.
Uses CMode CLI .

Invokes "lun-import-pause" api for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required)
Vserver Name
Applicable for CMode CLI

Maps to:

CM ZAPI: vserver

=item C<< 'path' => $string >>

(Required)
LUN Path
Applicable for CMode CLI

Maps to:

CM ZAPI: path

=item C<< '*' >>

This method takes all other options supported by "lun import pause" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub pause {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vserver path
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'lun_import_pause',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_pause_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_pause_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LunImport::pause',
    );

}

sub _pause_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'lun_import_pause',);
    $Log->exit() if $may_exit;
}

sub _pause_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(vserver path)],
        api  => 'lun_import_pause',
    );
    $Log->exit() if $may_exit;
}

=head2 resume

    NACL::C::LunImport->resume(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);

(Class method)
This method is used to resume the import for the specified LUN.
Uses CMode CLI/ZAPI.

Invokes "lun-import-resume" api for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required)
Vserver Name
Applicable for CMode CLI

Maps to:

CM ZAPI: vserver

=item C<< 'path' => $string >>

(Required)
LUN Path
Applicable for CMode CLI

Maps to:

CM ZAPI: path

=item C<< '*' >>

This method takes all other options supported by "lun import resume" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub resume {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vserver path
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'lun_import_resume',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_resume_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_resume_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LunImport::resume',
    );

}

sub _resume_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'lun_import_resume',);
    $Log->exit() if $may_exit;
}

sub _resume_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(vserver path)],
        api  => 'lun_import_resume',
    );
    $Log->exit() if $may_exit;
}

=head2 start

    NACL::C::LunImport->start(
        command_interface => $ci,
        'vserver' => $vserver,
      ...);

(Class method)
This method is used to start the import for the specified LUN.
Uses CMode CLI/ZAPI.

Invokes "lun-import-start" api for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required)
Vserver Name
Applicable for CMode CLI

Maps to:

CM ZAPI: vserver

=item C<< 'path' => $string >>

(Required)
LUN Path
Applicable for CMode CLI

Maps to:

CM ZAPI: path

=item C<< '*' >>

This method takes all other options supported by "lun import start" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vserver path
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'lun_import_start',
    );

    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_start_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_start_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LunImport::start',
    );

    $Log->exit() if $may_exit;
}

sub _start_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'lun_import_start',);
    $Log->exit() if $may_exit;
}

sub _start_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(vserver path)],
        api  => 'lun_import_start',
    );
    $Log->exit() if $may_exit;
}

=head2 stop

    NACL::C::LunImport->stop(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);

(Class method)
This method is used to stop the import for the specified LUN.
Uses CMode CLI/ZAPI.

Invokes "lun-import-stop" api for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required)
Vserver Name
Applicable for CMode CLI

Maps to:

CM ZAPI: vserver

=item C<< 'path' => $string >>

(Required)
LUN Path
Applicable for CMode CLI

Maps to:

CM ZAPI: path

=item C<< '*' >>

This method takes all other options supported by "lun import stop" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub stop {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vserver path
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'lun_import_stop',
    );

    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_stop_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_stop_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LunImport::stop',
    );

}

sub _stop_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'lun_import_stop',);
    $Log->exit() if $may_exit;
}

sub _stop_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(vserver path )],
        api  => 'lun_import_stop',
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver' => {type => SCALAR},
        'path'    => {type => SCALAR}
    );
}

1;
