#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary LunMapping Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::LunMapping

=head1 DESCRIPTION

C<NACL::C::LunMapping> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP LunMapping.  A related class is
L<NACL::CS::LunMapping|lib-NACL-CS-LunMapping-pm>, which represents
the past state of an ONTAP LunMapping.

=head1 ATTRIBUTES

=head1 Forms of specifying the lun for static method calls

All methods in this package support specifying the lun either through
the C<path> or by specifying C<volume>+C<lun> (and C<qtree>, optionally).
Taking the C<delete> method as example, both of these forms are supported:

    # Providing the lun path
    NACL::C::LunMapping->delete(path => $path, %other_opts);

    # Providing volume+lun
    NACL::C::LunMapping->delete(volume => $volume, lun => $lun, %other_opts);

    # If the lun within a qtree,
    NACL::C::LunMapping->delete(volume => $volume, qtree => $qtree, lun => $lun, %other_opts);

While the POD of the individual methods might not list all of these forms,
please note that these forms are applicable for all methods.

=cut

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>


=head2 igroup

=head2 vserver

=head2 path

=cut

package NACL::C::LunMapping;

use strict;
use warnings;

use NACL::CS::LunMapping;
use NACL::C::_Mixins::Lun qw(:all);
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use NACL::C::Exceptions::LunMapping::AlreadyExists;
use NACL::C::Exceptions::LunMapping::LunIdAlreadyUsed;
use NACL::C::Exceptions::LunMapping::DoesNotExist;
use NATE::BaseException qw(:try);

=head1 METHODS

=head2 new

  my $LunMapping = NACL::C::LunMapping->new(
                                   command_interface => $ci,
                                   'igroup' => $igroup,
                                   'vserver' => $vserver,
                                   'path' => $path,
                                  );

Return a new LunMapping component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'igroup',
    scalar => 'vserver',
    scalar => 'path',
];

=head2 find

    my $LunMapping_obj  = NACL::C::LunMapping->find(command_interface => $ci, ...);
    my @LunMapping_objs = NACL::C::LunMapping->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my LunMapping_state = $LunMapping_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::LunMapping would return
a NACL::CS::LunMapping object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 lun

   $lun_name = $lun_map_obj->lun();

(Instance method)
This method is used to get the name of the Lun through the LunMapping Component object
Returns the name of the lun as a scalar

=cut

sub lun {
    $Log->enter() if $may_enter;
    my ($self) = @_;
    my ($volume, $qtree, $lun)
        = $self->_extract_volume_qtree_lun_from_path(path => $self->path());
    $Log->exit() if $may_exit;
    return $lun;
}

=head2 show_initiator

    my $show_initiator_obj = NACL::C::LunMapping->show_initiator(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

    my @show_initiator_objs = NACL::C::LunMapping->show_initiator(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $show_initiator_obj = $LunMapping_obj->show_initiator();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty)
and has all of the other properties of fetch methods (callable in scalar or list
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-LunMappingShowInitiator-pm/ATTRIBUTES>
of NACL::CS::LunMappingShowInitiator as input.
This method returns objects of the type L<NACL::CS::LunMappingShowInitiator|lib-NACL-CS-LunMappingShowInitiator-pm>.

=cut

sub show_initiator {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::LunMappingShowInitiator;
    my @state_objs = NACL::CS::LunMappingShowInitiator->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 create

    my $LunMapping_obj    = NACL::C::LunMapping->create(
                                command_interface => $ci,
                                'igroup' => $igroup,
                                'vserver' => $vserver,
                                'path' => $path,
                                ...);

(Class method)
This method is used to map a LUN to an initiator group.

Uses CMode CLI/ZAPI .

Invokes  lun-map for CMode ZAPI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'igroup' => $igroup >>

(Required)
Igroup Name

Applicable for CMode CLI/ZAPI

Maps to:

CMode ZAPI :
<lun-map>
<initiator-group>$value</initiator-group>
</lun-map>

=item C<< 'vserver' => $vserver >>

(Required)
Vserver Name

Applicable for CMode CLI/ZAPI

=item C<< 'path' => $path >>

(Required)
LUN Path

Applicable for CMode CLI/ZAPI

=item C<< 'lun-id' => $string >>

LUN ID ranges from 0 to 4095

Applicable for CMode CLI/ZAPI

=item C<< 'force' => $string >>

Force possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'additional-reporting-node' => $string >>

Additional Reporting Node
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "lun mapping create" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::Lun::AlreadyExists>

This type of exception gets thrown if an attempt is made to create
a lun mapping which already exists.

=item C<NACL::C::Exceptions::Lun::LunIdAlreadyUsed>

This type of exception gets thrown if an attempt is made to create
a lun mapping which the id is already used.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( lun-id

                    force

                    additional-reporting-node
                    )
            ),
        },
        cli_cmd => 'lun mapping create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_create_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_create_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::LunMapping::create',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::LunMapping::AlreadyExists
            ->detect_convert_and_throw(exception => $exception);
        NACL::C::Exceptions::LunMapping::LunIdAlreadyUsed
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        igroup            => $opts{'igroup'},
        vserver           => $vserver,
        path              => $opts{'path'},
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'lun_mapping_create',);
    $Log->exit() if $may_exit;
}

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(path lun-id additional-reporting-node )];
    my $map = {'igroup' => 'initiator-group',};
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'lun-map',
        copy => $copy,
        map  => $map,
    );
    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::LunMapping->delete(
                                command_interface => $ci,
                                'igroup' => $igroup,
                                'vserver' => $vserver,
                                'path' => $path,
                                ...);

                                 or

    $LunMapping_obj->delete(...);

(Class or instance method)
This method is used to unmap a LUN from an initiator group.

Uses CMode CLI/ZAPI .

Invokes  lun-unmap for CMode ZAPI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'igroup' => $igroup >>

(Required for class method, Not Applicable for instance method)
Igroup Name

Applicable for CMode CLI/ZAPI


Maps to:

CMode ZAPI :

CMode ZAPI :
<lun-unmap>
<initiator-group>$value</initiator-group>
</lun-unmap>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name

Applicable for CMode CLI/ZAPI

=item C<< 'path' => $path >>

(Required for class method, Not Applicable for instance method)
LUN Path

Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "lun mapping delete" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::Lun::DoesNotExist>

This type of exception gets thrown if an attempt is made delete
a lun mapping that does not exist.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params   => \@args,
        cli_cmd  => 'lun mapping delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_delete_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::LunMapping::delete',
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::LunMapping::DoesNotExist->detect_convert_and_throw(
            exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts, api => 'lun_mapping_delete',);
    $Log->exit() if $may_exit;
}

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(path)];
    my $map = {'igroup' => 'initiator-group',};
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'lun-unmap',
        copy => $copy,
        map  => $map,
    );
    $Log->exit() if $may_exit;
}

=head2 add_reporting_nodes

    NACL::C::LunMapping->add_reporting_nodes(
                                command_interface => $ci,
                                'igroup' => $igroup,
                                'vserver' => $vserver,
                                'path' => $path,
                                ...);

                                 or

    $LunMapping_obj->add_reporting_nodes(...);

(Class or instance method)
This method is used to add Reporting Nodes.

Uses CMode CLI/ZAPI .

Invokes  lun-map-add-reporting-nodes for CMode ZAPI


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'igroup' => $igroup >>

(Required for class method, Not Applicable for instance method)
Igroup Name

Applicable for CMode CLI/ZAPI

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name

Applicable for CMode CLI/ZAPI

=item C<< 'path' => $path >>

(Required for class method, Not Applicable for instance method)
LUN Path

Applicable for CMode CLI/ZAPI

=item C<< 'destination-volume' => $string >>

Add Nodes for Volume

Applicable for CMode CLI/ZAPI

=item C<< 'local-nodes' => $string >>

Add Nodes for Current LUN Location

Applicable for CMode CLI

=item C<< 'all' => $string >>

Add All Nodes

Applicable for CMode CLI/ZAPI

=item C<< 'destination-aggregate' => $string >>

Add Nodes for Aggregate

Applicable for CMode CLI/ZAPI

=item C<< 'nodes' => [ $node1, $node2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)


Applicable for CMode CLI/ZAPI

=item C<< 'no-ha-relationships' => $string >>

Do Not Expand HA Relationships
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "lun mapping add reporting nodes" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add_reporting_nodes {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'nodes' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( destination-volume local-nodes all destination-aggregate

                    no-ha-relationships
                    )
            ),
        },
        cli_cmd => 'lun mapping add-reporting-nodes',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_add_reporting_nodes_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_add_reporting_nodes_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LunMapping::add_reporting_nodes',
    );

    $Log->exit() if $may_exit;
}

sub _add_reporting_nodes_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'lun_mapping_add_reporting_nodes',);
    $Log->exit() if $may_exit;
}

sub _add_reporting_nodes_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [
        qw(all  destination-aggregate destination-volume igroup
            nodes path)
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'lun-map-add-reporting-nodes',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

=head2 remove_reporting_nodes

    NACL::C::LunMapping->remove_reporting_nodes(
                                command_interface => $ci,
                                'igroup' => $igroup,
                                'vserver' => $vserver,
                                'path' => $path,
                                ...);

                                 or

    $LunMapping_obj->remove_reporting_nodes(...);

(Class or instance method)
This method is used to remove Reporting Nodes.

Uses CMode CLI/ZAPI .

Invokes  lun-map-remove-reporting-nodes for CMode ZAPI


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'igroup' => $igroup >>

(Required for class method, Not Applicable for instance method)
Igroup Name

Applicable for CMode CLI/ZAPI

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name

Applicable for CMode CLI/ZAPI

=item C<< 'path' => $path >>

(Required for class method, Not Applicable for instance method)
LUN Path

Applicable for CMode CLI/ZAPI

=item C<< 'remote-nodes' => $string >>

Remove Remote Nodes for LUN Location

Applicable for CMode CLI

=item C<< 'nodes' => [ $node1, $node2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)


Applicable for CMode CLI/ZAPI

=item C<< 'no-ha-relationships' => $string >>

Do Not Expand HA Relationships
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "lun mapping remove reporting nodes" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub remove_reporting_nodes {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'nodes' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( remote-nodes

                    no-ha-relationships
                    )
            ),
        },
        cli_cmd => 'lun mapping remove-reporting-nodes',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_remove_reporting_nodes_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_remove_reporting_nodes_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LunMapping::remove_reporting_nodes',
    );

    $Log->exit() if $may_exit;
}

sub _remove_reporting_nodes_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'lun_mapping_remove_reporting_nodes',);
    $Log->exit() if $may_exit;
}

sub _remove_reporting_nodes_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(igroup nodes path)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'lun-map-remove-reporting-nodes',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'igroup'  => {type => SCALAR},
        'vserver' => {type => SCALAR},
        'path'    => {type => SCALAR},
    );
}

sub _super_common_validate_with {
    my ($pkg_or_obj, @opts) = @_;
    return $pkg_or_obj->SUPER::_common_validate_with(@opts);
}

1;
