#
# Copyright (c) 2011-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MetroclusterConfigReplication Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::MetroclusterConfigReplication

=head1 DESCRIPTION

C<NACL::C::MetroclusterConfigReplication> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP MetroclusterConfigReplication.  A related class is
L<NACL::CS::MetroclusterConfigReplication|lib-NACL-CS-MetroclusterConfigReplication-pm>, which represents
the past state of an ONTAP MetroclusterConfigReplication.
However, since it does not have any primary keys, 'find' and 'state' do not
work for this component. (This component is considered a 'singleton',
i.e. there is only one row shown by the relevant 'show' command)

=cut

=head1 ATTRIBUTES

None.

=cut

package NACL::C::MetroclusterConfigReplication;

use strict;
use warnings;

use NACL::CS::MetroclusterConfigReplication;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $MetroclusterConfigReplication = NACL::C::MetroclusterConfigReplication->new(
                                   command_interface => $ci,
								  );

Return a new MetroclusterConfigReplication component object with the given attributes.

=cut

use Class::MethodMaker [];

=head2 find

    my $MetroclusterConfigReplication_obj  = NACL::C::MetroclusterConfigReplication->find(command_interface => $ci, ...);
	my @MetroclusterConfigReplication_objs = NACL::C::MetroclusterConfigReplication->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my MetroclusterConfigReplication_state = $MetroclusterConfigReplication_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::MetroclusterConfigReplication would return 
a NACL::CS::MetroclusterConfigReplication object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 modify

    NACL::C::MetroclusterConfigReplication->modify(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to modify MetroCluster config-replication storage information.
Uses CMode CLI/ZAPI.

Invokes "metrocluster-config-replication-modify" API  for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'auto-recreate' => $string >>

Auto-Recreate possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'auto-repair' => $string >>

Auto-Repair possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'disallowed-aggregates' => [ $disallowed-aggregate1, $disallowed-aggregate2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "metrocluster config replication modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'disallowed-aggregates' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( auto-recreate auto-repair
                    )
            ),
        },
        api => 'metrocluster_config_replication_modify',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::MetroclusterConfigReplication::modify',
    );

}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'metrocluster_config_replication_modify',);
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(disallowed-aggregates)],
        api  => 'metrocluster-config-replication-modify',
    );
    $Log->exit() if $may_exit;
}

=head2 pre_enable_cps

    NACL::C::MetroclusterConfigReplication->pre_enable_cps(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to pre-enables Cluster Persistent Storage after metrocluster has been configured.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< '*' >>

This method takes all other options supported by "metrocluster config replication pre enable cps" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub pre_enable_cps {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        cli_cmd    => 'metrocluster config-replication pre-enable-cps',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_pre_enable_cps_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::MetroclusterConfigReplication::pre_enable_cps',
    );

    $Log->exit() if $may_exit;
}

sub _pre_enable_cps_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'metrocluster_config_replication_pre_enable_cps',);
    $Log->exit() if $may_exit;
}

__PACKAGE__->_load_event_class();

1;
