# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MetroclusterInterconnectAdapter Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::MetroclusterInterconnectAdapter

=head1 DESCRIPTION

C<NACL::C::MetroclusterInterconnectAdapter> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP MetroclusterInterconnectAdapter.  A related class is
L<NACL::CS::MetroclusterInterconnectAdapter|lib-NACL-CS-MetroclusterInterconnectAdapter-pm>, which represents
the past state of an ONTAP MetroclusterInterconnectAdapter.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 portno

=head2 port_name

=head2 next_adapter

=head2 partner_type

=head2 adapter

=head2 next_partner

=head2 node

=head2 conn_index

=head2 partner_sysid

=head2 adapter_index

=head2 index

=cut

package NACL::C::MetroclusterInterconnectAdapter;

use strict;
use warnings;

use NACL::CS::MetroclusterInterconnectAdapter;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $MetroclusterInterconnectAdapter = NACL::C::MetroclusterInterconnectAdapter->new(
                                   command_interface => $ci,
                                   'port-name' => $port_name,
                                   'next-adapter' => $next_adapter,
                                   'partner-type' => $partner_type,
                                   'adapter' => $adapter,
                                   'next-partner' => $next_partner,
                                   'node' => $node,
                                   'conn-index' => $conn_index,
                                   'partner-sysid' => $partner_sysid,
                                   'adapter-index' => $adapter_index,
                                   'index' => $index,
								  );

Return a new MetroclusterInterconnectAdapter component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'port_name',
    scalar => 'portno',
    scalar => 'next_adapter',
    scalar => 'partner_type',
    scalar => 'adapter',
    scalar => 'next_partner',
    scalar => 'node',
    scalar => 'conn_index',
    scalar => 'partner_sysid',
    scalar => 'adapter_index',
    scalar => 'index',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $MetroclusterInterconnectAdapter_obj  = NACL::C::MetroclusterInterconnectAdapter->find(command_interface => $ci, ...);
	my @MetroclusterInterconnectAdapter_objs = NACL::C::MetroclusterInterconnectAdapter->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my MetroclusterInterconnectAdapter_state = $MetroclusterInterconnectAdapter_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::MetroclusterInterconnectAdapter would return 
a NACL::CS::MetroclusterInterconnectAdapter object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 modify

    NACL::C::MetroclusterInterconnectAdapter->modify(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
   
(Class method)
This method is used to modify MetroCluster interconnect adapter settings.
Uses CMode CLI/ZAPI .
Invokes  metrocluster-interconnect-adapter-modify  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'node' => $node >>

(Required)
Node Name possible value(s) are, <nodename>,local
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'adapter-port-name' => $string >>

Adapter Port Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'is-ood-enabled' => $string >>

Is Out-of-Order Delivery Enabled? possible value(s) are, true,false
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "metrocluster interconnect adapter modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( adapter-port-name node is-ood-enabled
                    )
            ),
        },
        cli_cmd             => 'metrocluster interconnect adapter modify',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_modify_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::MetroclusterInterconnectAdapter::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'metrocluster_interconnect_adapter_modify',);
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(adapter-port-name is-ood-enabled node)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'metrocluster-interconnect-adapter-modify',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [
            qw(port-name next-adapter partner-type adapter next-partner node conn-index partner-sysid adapter-index index)
        ],
    );
    $Log->exit() if $may_exit;
}

=head2 reset

    NACL::C::MetroclusterInterconnectAdapter->reset(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
   
(Class method)
This method is used to reset MetroCluster interconnect adapter.
Uses CMode CLI/ZAPI .
Invokes  metrocluster-interconnect-adapter-reset  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'node' => $node >>

(Required)
Node Name possible value(s) are, <nodename>,local
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'adapter-port-name' => $string >>

Adapter Port Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "metrocluster interconnect adapter reset" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub reset {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( adapter-port-name node
                    )
            ),
        },
        cli_cmd             => 'metrocluster interconnect adapter reset',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_reset_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_reset_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::MetroclusterInterconnectAdapter::reset',
    );

    $Log->exit() if $may_exit;
}

sub _reset_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'metrocluster_interconnect_adapter_reset',);
    $Log->exit() if $may_exit;
}

sub _reset_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(adapter-port-name node)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'metrocluster-interconnect-adapter-reset',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'portno'        => {type => SCALAR},
        'adapter'       => {type => SCALAR},
        'node'          => {type => SCALAR},
        'port-name'     => {type => SCALAR, optional => 1},
        'next-adapter'  => {type => SCALAR, optional => 1},
        'partner-type'  => {type => SCALAR, optional => 1},
        'next-partner'  => {type => SCALAR, optional => 1},
        'conn-index'    => {type => SCALAR, optional => 1},
        'partner-sysid' => {type => SCALAR, optional => 1},
        'adapter-index' => {type => SCALAR, optional => 1},
        'index'         => {type => SCALAR, optional => 1},
    );
}

1;
