# $Id: //depot/prod/test/nacldev/lib/NACL/C/NetworkInterfacePlacement.pm#3 $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkInterfacePlacement Component Module (auto-generated by CGT)
## @author pbala@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::NetworkInterfacePlacement

=head1 DESCRIPTION

C<NACL::C::NetworkInterfacePlacement> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP NetworkInterfacePlacement.  A related class is
L<NACL::CS::NetworkInterfacePlacement|lib-NACL-CS-NetworkInterfacePlacement-pm>, which represents
the past state of an ONTAP NetworkInterfacePlacement.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 netmask_length

=head2 vserver

=head2 address

=head2 port

=head2 node

=cut

package NACL::C::NetworkInterfacePlacement;

use strict;
use warnings;

use NACL::CS::NetworkInterfacePlacement;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $NetworkInterfacePlacement = NACL::C::NetworkInterfacePlacement->new(
                                   command_interface => $ci,
                                   'netmask-length' => $netmask_length,
                                   'vserver' => $vserver,
                                   'address' => $address,
                                   'port' => $port,
                                   'node' => $node,
                                  );

Return a new NetworkInterfacePlacement component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'netmask_length',
    scalar => 'vserver',
    scalar => 'address',
    scalar => 'port',
    scalar => 'node',
];

=head2 find

    my $NetworkInterfacePlacement_obj  = NACL::C::NetworkInterfacePlacement->find(command_interface => $ci, ...);
    my @NetworkInterfacePlacement_objs = NACL::C::NetworkInterfacePlacement->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my NetworkInterfacePlacement_state = $NetworkInterfacePlacement_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::NetworkInterfacePlacement would return
a NACL::CS::NetworkInterfacePlacement object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 discover

    NACL::C::NetworkInterfacePlacement->discover(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'address' => $address,
                                ...);

(Class method)
This method is used to trigger a port discovery for a subnet.

Uses a CMode CLI/ZAPI APISet.

Invokes "net-placement-discover" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI : vserver

=item C<< 'address' => $address >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI : address

=item C<< 'netmask_length' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : netmask-length

=item C<< 'mac' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI : mac

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'candidate' => $string >>

Applicable for CMode CLI

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"network interface placement discover" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub discover {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( netmask_length vserver address mac

                    candidate
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'network interface placement discover',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_discover_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_discover_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkInterfacePlacement::discover',
    );

    $Log->exit() if $may_exit;
} ## end sub discover

sub _discover_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'network_interface_placement_discover',);
    $Log->exit() if $may_exit;
} ## end sub _discover_cmode_cli

sub _discover_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $allowed_cmode_options = [
         qw( vserver address mac
             )
    ];

    $pkg->_base_backend_cmode_zapi(%opts,
        api => 'net_placement_discover',
        copy   => $allowed_cmode_options,
        map    => {'netmask_length' => 'netmask-length'},
    );
    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::NetworkInterfacePlacement->delete(
                                command_interface => $ci,
                                'netmask-length' => $netmask_length,
                                'vserver' => $vserver,
                                'address' => $address,
                                'port' => $port,
                                'node' => $node,
                                ...);

                                 or

    $NetworkInterfacePlacement_obj->delete(...);

(Class or instance method)
This method is used to delete the cache for a subnet or port.

Uses a CMode CLI/ZAPI APISet.

Invokes "net-placement-cache-delete" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'netmask-length' => $netmask_length >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : netmask-length

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : vserver

=item C<< 'address' => $address >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI : address

(Required for class method, Not Applicable for instance method)

=item C<< 'port' => $port >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI : port

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI : node

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"network interface placement delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        cli_cmd    => 'network interface placement delete',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkInterfacePlacement::delete',
    );

    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'network_interface_placement_delete',);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( vserver netmask-length address port node
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
        api    => 'net_placement_cache_delete',
        copy   => $allowed_cmode_options,
    );
    $Log->exit() if $may_exit;
}

=head2 create

    my $NetworkInterfacePlacement_obj    = NACL::C::NetworkInterfacePlacement->create(
                                command_interface => $ci,
                                'netmask-length' => $netmask_length,
                                'vserver' => $vserver,
                                'address' => $address,
                                'port' => $port,
                                'node' => $node,
                                ...);

(Class method)

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'netmask-length' => $netmask_length >>

(Required)

Applicable for CMode CLI.

=item C<< 'vserver' => $vserver >>

(Required)

Applicable for CMode CLI.

=item C<< 'address' => $address >>

(Required)

Applicable for CMode CLI.

=item C<< 'port' => $port >>

(Required)

Applicable for CMode CLI.

=item C<< 'node' => $node >>

(Required)

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"network interface placement create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        api    => 'network_interface_placement_create',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkInterfacePlacement::create',
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        netmask_length    => $opts{'netmask-length'},
        vserver           => $opts{'vserver'},
        address           => $opts{'address'},
        port              => $opts{'port'},
        node              => $opts{'node'},
    );

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'network_interface_placement_create',);
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'netmask-length' => {type => SCALAR},
        'vserver'        => {type => SCALAR},
        'address'        => {type => SCALAR},
        'port'           => {type => SCALAR},
        'node'           => {type => SCALAR},
    );
}

1;
