#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkSubnet Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::NetworkSubnet

=head1 DESCRIPTION

C<NACL::C::NetworkSubnet> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP NetworkSubnet.  A related class is
L<NACL::CS::NetworkSubnet|lib-NACL-CS-NetworkSubnet-pm>, which represents
the past state of an ONTAP NetworkSubnet.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 subnet_name

=head2 ipspace

=cut

package NACL::C::NetworkSubnet;

use strict;
use warnings;

use NACL::CS::NetworkSubnet;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $NetworkSubnet = NACL::C::NetworkSubnet->new(
                                   command_interface => $ci,
                                   'subnet-name' => $subnet_name,
                                   'ipspace' => $ipspace,
								  );

Return a new NetworkSubnet component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'subnet_name',
    scalar => 'ipspace',
];

=head2 find

    my $NetworkSubnet_obj  = NACL::C::NetworkSubnet->find(command_interface => $ci, ...);
	my @NetworkSubnet_objs = NACL::C::NetworkSubnet->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my NetworkSubnet_state = $NetworkSubnet_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::NetworkSubnet would return 
a NACL::CS::NetworkSubnet object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 create

    my $NetworkSubnet_obj	= NACL::C::NetworkSubnet->create(
                                command_interface => $ci,
	                            'subnet-name' => $subnet_name,
	                            'ipspace' => $ipspace,
                                ...);

(Class method)
This method is used to create a new layer 3 subnet.
Uses CMode CLI/ZAPI .

Invokes API 'net-subnet-create' for Cmode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'subnet-name' => $subnet_name >>

(Required)
Subnet Name
Applicable for CMode CLI

=item C<< 'ipspace' => $ipspace >>

(Required)
IPspace Name
Applicable for CMode CLI/ZAPI

=item C<< 'gateway' => $string >>

Gateway
Applicable for CMode CLI/ZAPI

=item C<< 'broadcast-domain' => $string >>

Broadcast Domain
Applicable for CMode CLI/ZAPI

=item C<< 'subnet' => $string >>

Layer 3 Subnet
Applicable for CMode CLI/ZAPI

=item C<< 'force-update-lif-associations' => $string >>

Change the subnet association
Applicable for CMode CLI/ZAPI

=item C<< 'ip-ranges' => [ $ip-range1, $ip-range2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "network subnet create" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'ip-ranges' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( gateway broadcast-domain subnet force-update-lif-associations
                    )
            ),
        },
        cli_cmd  => 'network subnet create',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkSubnet::create',
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        subnet_name       => $opts{'subnet-name'},
        ipspace           => $opts{'ipspace'},
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'network_subnet_create',);
    $Log->exit() if $may_exit;
}

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->_base_backend_cmode_zapi(
        %opts,
        api  => 'net-subnet-create',
        copy => [
            qw( broadcast-domain gateway ip-ranges
                ipspace subnet subnet-name )
        ],
        frontend_method => 'create'
    );
    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::NetworkSubnet->delete(
                                command_interface => $ci,
	                            'subnet-name' => $subnet_name,
	                            'ipspace' => $ipspace,
                                ...);
	                             
								 or

    $NetworkSubnet_obj->delete(...);

(Class or instance method)
This method is used to delete a layer 3 subnet.
Uses CMode CLI/ZAPI .

Invokes API 'net-subnet-destroy' for Cmode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'subnet-name' => $subnet_name >>

(Required for class method, Not Applicable for instance method)
Subnet Name
Applicable for CMode CLI/ZAPI

=item C<< 'ipspace' => $ipspace >>

(Required for class method, Not Applicable for instance method)
IPspace Name
Applicable for CMode CLI/ZAPI

=item C<< 'force-update-lif-associations' => $string >>

Change the subnet association
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "network subnet delete" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        ,
        additional_spec => {
            _optional_scalars(
                qw( force-update-lif-associations
                    )
            )
        },
        cli_cmd             => 'network subnet delete',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkSubnet::delete',
    );

    $Log->exit() if $may_exit;
}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'network_subnet_delete',);
    $Log->exit() if $may_exit;
}

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api             => 'net-subnet-destroy',
        copy            => [qw( ipspace subnet-name )],
        frontend_method => 'delete',
    );
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::NetworkSubnet->modify(
                                command_interface => $ci,
	                            'subnet-name' => $subnet_name,
	                            'ipspace' => $ipspace,
                                ...);
	                             
								 or

    $NetworkSubnet_obj->modify(...);

(Class or instance method)
This method is used to modify a layer 3 subnet.
Uses CMode CLI/ZAPI .

Invokes API 'net-subnet-modify' for Cmode ZAPI

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'subnet-name' => $subnet_name >>

(Required for class method, Not Applicable for instance method)
Subnet Name
Applicable for CMode CLI/ZAPI

=item C<< 'ipspace' => $ipspace >>

(Required for class method, Not Applicable for instance method)
IPspace Name
Applicable for CMode CLI/ZAPI

=item C<< 'gateway' => $string >>

Gateway
Applicable for CMode CLI/ZAPI

=item C<< 'subnet' => $string >>

Layer 3 Subnet
Applicable for CMode CLI

=item C<< 'force-update-lif-associations' => $string >>

Change the subnet association
Applicable for CMode CLI/ZAPI

=item C<< 'ip-ranges' => [ $ip-range1, $ip-range2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "network subnet modify" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'ip-ranges' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( gateway subnet force-update-lif-associations
                    )
            ),
        },
        cli_cmd => 'network subnet modify',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkSubnet::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'network_subnet_modify',);
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api  => 'net-subnet-modify',
        copy => [qw( gateway ip-ranges ipspace subnet subnet-name )],
        frontend_method => 'modify',
    );
    $Log->exit;
}

=head2 rename

    NACL::C::NetworkSubnet->rename(
                                command_interface => $ci,
	                            'subnet-name' => $subnet_name,
	                            'ipspace' => $ipspace,
                                ...);
	                             
								 or

    $NetworkSubnet_obj->rename(...);

(Class or instance method)
This method is used to rename a layer 3 subnet.
Uses CMode CLI/ZAPI .

Invokes API 'net-subnet-rename' for Cmode ZAPI

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'subnet-name' => $subnet_name >>

(Required for class method, Not Applicable for instance method)
Subnet Name
Applicable for CMode CLI/ZAPI

=item C<< 'ipspace' => $ipspace >>

(Required for class method, Not Applicable for instance method)
IPspace Name
Applicable for CMode CLI/ZAPI

=item C<< 'new-name' => $string >>

New Name
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "network subnet rename" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub rename {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( new-name
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'network subnet rename',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_rename_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_rename_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkSubnet::rename',
    );

    $Log->exit() if $may_exit;
}

sub _rename_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'network_subnet_rename',);
    $Log->exit() if $may_exit;
}

sub _rename_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api             => 'net-subnet-rename',
        copy            => [qw( ipspace new-name subnet-name )],
        frontend_method => 'rename',
    );
    $Log->exit() if $may_exit;
}

=head2 add_ranges

    NACL::C::NetworkSubnet->add_ranges(
                                command_interface => $ci,
	                            'subnet-name' => $subnet_name,
	                            'ipspace' => $ipspace,
                                ...);
	                             
								 or

    $NetworkSubnet_obj->add_ranges(...);

(Class or instance method)
This method is used to add new address ranges to a subnet.
Uses CMode CLI/ZAPI .

Invokes API 'net-subnet-add-ranges' for Cmode ZAPI

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'subnet-name' => $subnet_name >>

(Required for class method, Not Applicable for instance method)
Subnet Name
Applicable for CMode CLI/ZAPI

=item C<< 'ipspace' => $ipspace >>

(Required for class method, Not Applicable for instance method)
IPspace Name
Applicable for CMode CLI/ZAPI

=item C<< 'force-update-lif-associations' => $string >>

Force Update LIF Associations
Applicable for CMode CLI/ZAPI

=item C<< 'ip-ranges' => [ $ip-range1, $ip-range2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "network subnet add ranges" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add_ranges {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'ip-ranges' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( force-update-lif-associations
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'network subnet add ranges',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_add_ranges_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_add_ranges_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkSubnet::add_ranges',
    );

    $Log->exit() if $may_exit;
}

sub _add_ranges_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'network_subnet_add_ranges',);
    $Log->exit() if $may_exit;
}

sub _add_ranges_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api  => 'net-subnet-add-ranges',
        copy => [
            qw( force-update-lif-associations
                ip-ranges ipspace subnet-name )
        ],
        frontend_method => 'add_ranges',
    );
    $Log->exit() if $may_exit;
}

=head2 remove_ranges

    NACL::C::NetworkSubnet->remove_ranges(
                                command_interface => $ci,
	                            'subnet-name' => $subnet_name,
	                            'ipspace' => $ipspace,
                                ...);
	                             
								 or

    $NetworkSubnet_obj->remove_ranges(...);

(Class or instance method)
This method is used to remove address ranges from a subnet.
Uses CMode CLI/ZAPI .

Invoke API 'net-subnet-remove-ranges' for Cmode ZAPI

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'subnet-name' => $subnet_name >>

(Required for class method, Not Applicable for instance method)
Subnet Name
Applicable for CMode CLI/ZAPI

=item C<< 'ipspace' => $ipspace >>

(Required for class method, Not Applicable for instance method)
IPspace Name
Applicable for CMode CLI/ZAPI

=item C<< 'force-update-lif-associations' => $string >>

Force Update LIF Associations
Applicable for CMode CLI/ZAPI

=item C<< 'ip-ranges' => [ $ip-range1, $ip-range2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "network subnet remove ranges" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub remove_ranges {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'ip-ranges' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( force-update-lif-associations
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'network subnet remove ranges',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_remove_ranges_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_remove_ranges_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkSubnet::remove_ranges',
    );

    $Log->exit() if $may_exit;
}

sub _remove_ranges_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'network_subnet_remove_ranges',);
    $Log->exit() if $may_exit;
}

sub _remove_ranges_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api  => 'net-subnet-remove-ranges',
        copy => [
            qw( force-update-lif-associations ip-ranges
                ipspace subnet-name )
        ],
        frontend_method => 'remove_ranges',
    );
    $Log->exit() if $may_exit;
}

=head2 api

    NACL::C::NetworkSubnet->api(
                                command_interface => $ci,
	                            'subnet-name' => $subnet_name,
	                            'ipspace' => $ipspace,
                                ...);
	                             
								 or

    $NetworkSubnet_obj->api(...);

(Class or instance method)
This method is used to run a network subnet api.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'subnet-name' => $subnet_name >>

(Required for class method, Not Applicable for instance method)
Subnet Name
Applicable for CMode CLI

=item C<< 'ipspace' => $ipspace >>

(Required for class method, Not Applicable for instance method)
IPSpace Name
Applicable for CMode CLI

=item C<< 'api' => $string >>


subnet api possible value(s) are, get,acquire_address_from_ranges,acquire_specific_address_from_ranges,release_address,contains_address
Applicable for CMode CLI

=item C<< 'ipspace-id' => $string >>

IPSpace id
Applicable for CMode CLI

=item C<< 'ipspace-uuid' => $string >>

IPspace UUID
Applicable for CMode CLI

=item C<< 'subnet-id' => $string >>

Subnet ID
Applicable for CMode CLI

=item C<< 'netmask' => $string >>

Netmask/Prefix
Applicable for CMode CLI

=item C<< 'address' => $string >>

IP Address
Applicable for CMode CLI

=item C<< 'subnet' => $string >>

Layer 3 Subnet
Applicable for CMode CLI

=item C<< 'netmask-length' => $string >>

Netmask/Prefix Length
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "network subnet api" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub api {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( api ipspace-id ipspace-uuid subnet-id netmask address subnet

                    netmask-length
                    )
            ),
        },
        cli_cmd => 'network subnet api',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_api_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkSubnet::api',
    );

    $Log->exit() if $may_exit;
}

sub _api_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts, api => 'network_subnet_api',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'subnet-name' => {type => SCALAR},
        'ipspace'     => {type => SCALAR},
    );
}

1;
