# $Id$

#
# Copyright (c) 2001-2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary QosSettingsCache Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::QosSettingsCache

=head1 DESCRIPTION

C<NACL::C::QosSettingsCache> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP QosSettingsCache.  A related class is
L<NACL::CS::QosSettingsCache|lib-NACL-CS-QosSettingsCache-pm>, which represents
the past state of an ONTAP QosSettingsCache.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 cache_setting

=cut

package NACL::C::QosSettingsCache;

use strict;
use warnings;

use NACL::CS::QosSettingsCache;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $QosSettingsCache = NACL::C::QosSettingsCache->new(
                                   command_interface => $ci,
                                   'cache-setting' => $cache_setting,
								  );

Return a new QosSettingsCache component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'cache_setting',];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $QosSettingsCache_obj  = NACL::C::QosSettingsCache->find(command_interface => $ci, ...);
	my @QosSettingsCache_objs = NACL::C::QosSettingsCache->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my QosSettingsCache_state = $QosSettingsCache_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::QosSettingsCache would return 
a NACL::CS::QosSettingsCache object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 modify

    NACL::C::QosSettingsCache->modify(
                                command_interface => $ci,
	                            'cache-setting' => $cache_setting,
                                ...);

								 or

    $QosSettingsCache_obj->modify(...);

(Class or instance method)
This method is used to modify the cache policy.
Uses CMode CLI/ZAPI .
Invokes  cache-policy-modify-iter for CMode Zapi iter
Invokes  cache-policy-modify  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'cache-setting' => $cache_setting >>

(Required for class method, Not Applicable for instance method)
Cache Policy Name
Applicable for CMode CLI/ZAPI iter, non-iter.

Mapping for iter-zapi: <cache-info cache-policy-name="$value" />

Mapping for non-iter: cache-policy-name

=item C<< 'cache-setting-id' => $string >>

Cache Setting Id
Applicable for CMode CLI/ZAPI iter, non-iter.

=item C<< 'default' => $string >>

Is Default? possible value(s) are, true,false
Applicable for CMode CLI/ZAPI iter, non-iter.

Mapping for iter-zapi: <cache-info is-default="$value" />

Mapping for non-iter: is-default

=item C<< '*' >>

This method takes all other options supported by "qos settings cache modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( cache-setting-id default
                    )
            ),
        },
        cli_cmd => 'qos settings cache modify',
    );
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },
            {   method    => '_modify_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_check_non_iter'
            },
        ],
        frontend => 'NACL::C::QosSettingsCache::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'qos_settings_cache_modify',);
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    require NACL::C::Exceptions::QosSettingsCache::DoesNotExist;
    my $copy = [qw(num-workloads)];
    my $map  = {
        'normal'        => 'is-normal-data',
        'default'       => 'is-default',
        'randomwrite'   => 'is-randomwrite-data',
        'readahead'     => 'is-readahead-data',
        'meta'          => 'is-meta-data',
        'lopri'         => 'is-lopri-data',
        'cache-setting' => 'cache-policy-name',
        'class'         => 'cache-class',
    };
    $pkg_or_obj->_iter_helper(
        @opts,
        api      => 'cache-policy-modify-iter',
        show_cmd => 'qos settings cache show',
        copy     => $copy,
        map      => $map,
        does_not_exist_exception =>
            'NACL::C::Exceptions::QosSettingsCache::DoesNotExist',
    );
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $map = {
        'cache-setting' => 'cache-policy-name',
        'default'       => 'is-default',
    };
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api => 'cache-policy-modify',
        map => $map,
    );
    $Log->exit() if $may_exit;
}

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(@opts, _primary_keys => [qw(cache-setting)],);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('cache-setting' => {type => SCALAR},);
}

1;
