# $Id$

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Reallocate Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, kathar.hidayath@netapp.com
## @status
## @pod here

=head1 NAME

NACL::C::Reallocate

=head1 DESCRIPTION

C<NACL::C::Reallocate> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP Reallocate.  A related class is
L<NACL::CS::Reallocate|lib-NACL-CS-Reallocate-pm>, which represents
the past state of an ONTAP Reallocate.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 once

=head2 vserver

=head2 path

=cut

package NACL::C::Reallocate;

use strict;
use warnings;

use NACL::CS::Reallocate;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $Reallocate = NACL::C::Reallocate->new(
                                   command_interface => $ci,
                                   'once' => $once,
                                   'vserver' => $vserver,
                                   'path' => $path,
                                );

Return a new Reallocate component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'once',
    scalar => 'vserver',
    scalar => 'path',
];

=head2 find

    my $Reallocate_obj  = NACL::C::Reallocate->find(command_interface => $ci, ...);
    my @Reallocate_objs = NACL::C::Reallocate->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode or 7Mode CLI APISet.

=head2 state

    my Reallocate_state = $Reallocate_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::Reallocate would return
a NACL::CS::Reallocate object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode or 7Mode CLI APISet.

=cut

=head2 quiesce

    NACL::C::Reallocate->quiesce(
                                command_interface => $ci,
                                'once' => $once,
                                'vserver' => $vserver,
                                'path' => $path,
                                ...);

(Class method)
This method is used to quiesce reallocate job.

Uses a CMode or 7Mode/Nodescope CLI APISet.
  Invokes 'reallocate quiesce' command for 7Mode/Nodescope CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'once' => $once >>

(Required)
  Applicable only for CMode CLI

=item C<< 'vserver' => $vserver >>

(Required)
  Applicable only for CMode CLI

=item C<< 'path' => $path >>

(Required)
  Applicable for CMode CLI, 7Mode/Nodescope CLI
  Maps to:
    7M CLI : path name (Programmatic name: 'path')

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"reallocate quiesce" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)
  Applicable only for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub quiesce {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( once vserver path
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd => 'reallocate quiesce',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_quiesce_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => "_quiesce_7mode_cli",
                interface => "CLI",
                set       => "7Mode|Nodescope",
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub quiesce

sub _quiesce_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( once vserver path
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'reallocate_quiesce',
        api_opts => \%cmode_args
    );

    $apiset->reallocate_quiesce(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _quiesce_cmode_cli

sub _quiesce_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface     = delete $opts{command_interface};
    my $apiset                = delete $opts{apiset};
    my $allowed_7mode_options = [qw( path )];
    my %valid_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%valid_args,
    );
    %opts = %valid_args;

    $apiset->reallocate_quiesce(%opts);
    $Log->exit() if $may_exit;
} ## end sub _quiesce_7mode_cli

=head2 start

    my $Reallocate_obj = NACL::C::Reallocate->start(
                                command_interface => $ci,
                                'once' => $once,
                                'vserver' => $vserver,
                                'path' => $path,
                                ...);

(Class method)
This method is used to start reallocate job.

Uses a CMode or 7Mode CLI APISet.
  Invokes 'reallocate start' command for 7Mode/Nodescope CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'once' => $once >>

(Required)
  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7M CLI : -o (Programmatic name: 'once')

=item C<< 'vserver' => $vserver >>

(Required)
  Applicable only for CMode CLI

=item C<< 'path' => $path >>

(Required)
  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7M CLI : path name (Programmatic name: 'path')

=item C<< 'no-check' => $string >>

  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7M CLI : -n (Programmatic name: 'no-check')

=item C<< 'threshold' => $string >>

  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7M CLI : -t (Programmatic name: 'threshold')

=item C<< 'interval' => $string >>

  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7M CLI : -i <value> (Programmatic name: 'interval')

=item C<< 'force' => $string >>

  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7M CLI : -f (Programmatic name: 'force')

=item C<< 'space-optimized' => $string >>

  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7M CLI : -p (Programmatic name: 'physical-blocks-alone')

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"reallocate start" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< 'unshare' => $string >>

Unshare Deduplicated Blocks possible value(s) are, true
Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( once no-check threshold path interval force space-optimized
                    unshare
                    )
            ),
        },
        cli_cmd => 'reallocate start',
    );
    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_start_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => "_start_7mode_cli",
                interface => "CLI",
                set       => "7Mode|Nodescope",
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub start

sub _start_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( once no-check threshold vserver path interval force space-optimized
            extended_query
            unshare
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'reallocate_start',
        api_opts => \%cmode_args
    );

    $apiset->reallocate_start(%cmode_args);
    $Log->exit() if $may_exit;
    return $pkg->new(
        command_interface => $command_interface,
        vserver           => $cmode_args{'vserver'},
        path              => $cmode_args{'path'},
    );
} ## end sub _start_cmode_cli

sub _start_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $allowed_7mode_options
        = [qw( path interval force threshold once no-check )];
    my %valid_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        map    => {"space-optimized" => "physical-blocks-alone",},
        target => \%valid_args,
    );
    %opts = %valid_args;

    $apiset->reallocate_start(%opts);
    $Log->exit() if $may_exit;
    return $pkg->new(
        command_interface => $command_interface,
        path              => $opts{'path'},
    );
} ## end sub _start_7mode_cli

=head2 restart

    NACL::C::Reallocate->restart(
                                command_interface => $ci,
                                    'once' => $once,
                                    'vserver' => $vserver,
                                    'path' => $path,
                                ...);

(Class method)
This method is used to restart reallocate job.

Uses a CMode or 7Mode CLI APISet.
  Invokes 'reallocate restart' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'once' => $once >>

(Required)
  Applicable only for CMode CLI

=item C<< 'vserver' => $vserver >>

(Required)
  Applicable only for CMode CLI

=item C<< 'path' => $path >>

(Required)
  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7M CLI : path name(Programmatic name: 'path')

=item C<< 'ignore-checkpoint' => $string >>

  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7M CLI : -i (Programmatic name: 'ignore-checkpoints')

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"reallocate restart" command in ONTAP.

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub restart {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( once ignore-checkpoint vserver path
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd => 'reallocate restart',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_restart_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => "_restart_7mode_cli",
                interface => "CLI",
                set       => "7Mode|Nodescope",
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub restart

sub _restart_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( once ignore-checkpoint vserver path
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'reallocate_restart',
        api_opts => \%cmode_args
    );

    $apiset->reallocate_restart(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _restart_cmode_cli

sub _restart_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface     = delete $opts{command_interface};
    my $apiset                = delete $opts{apiset};
    my $allowed_7mode_options = [qw( path )];
    my %valid_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        map    => {"ignore-checkpoint" => "ignore-checkpoints"},
        target => \%valid_args,
    );
    %opts = %valid_args;

    $apiset->reallocate_restart(%opts);
    $Log->exit() if $may_exit;
} ## end sub _restart_7mode_cli

=head2 off

    NACL::C::Reallocate->off(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to disable reallocate jobs.

Uses a CMode or 7Mode CLI APISet.
  Invokes 'reallocate off' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.


=item C<< '*' >>

This method takes all other options supported by
"reallocate off" command in ONTAP.

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub off {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
                    params => \@_,
                    cli_cmd => 'reallocate off',
                     );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_off_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_off_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub off

sub _off_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw(
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'reallocate_off',
        api_opts => \%cmode_args
    );

    $apiset->reallocate_off(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _off_cmode_cli

sub _off_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    $apiset->reallocate_off();
    $Log->exit() if $may_exit;
} ## end sub _off_7mode_cli

=head2 measure

    NACL::C::Reallocate->measure(
                                command_interface => $ci,
                                    'once' => $once,
                                    'vserver' => $vserver,
                                    'path' => $path,
                                ...);

(Class method)
This method is used to start reallocate measure job.

Uses a CMode or 7Mode CLI APISet.
  Invokes 'reallocate measure' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'once' => $once >>

(Required)
  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7M CLI : -o (Programmatic name: 'once')

=item C<< 'vserver' => $vserver >>

(Required)
  Applicable only for CMode CLI

=item C<< 'path' => $path >>

(Required)
  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7M CLI : path (Programmatic name: 'path')

=item C<< 'threshold' => $string >>

  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7M CLI : threshold <value> (Programmatic name: 'threshold')

=item C<< 'interval' => $string >>

  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7M CLI : -i <value> (Programmatic name: 'interval')

=item C<< 'logpath' => $string >>

  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7M CLI : -l <value> (Programmatic name: 'logfile')

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"reallocate measure" command in ONTAP.

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub measure {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( once threshold vserver path interval logpath
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd => 'reallocate measure',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_measure_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_measure_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub measure

sub _measure_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( once threshold vserver path interval logpath
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'reallocate_measure',
        api_opts => \%cmode_args
    );

    $apiset->reallocate_measure(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _measure_cmode_cli

sub _measure_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface     = delete $opts{command_interface};
    my $apiset                = delete $opts{apiset};
    my $allowed_7mode_options = [qw( threshold once interval path )];
    my %valid_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        map    => {"logpath" => "logfile",},
        target => \%valid_args,
    );
    %opts = %valid_args;

    $apiset->reallocate_measure(%opts);
    $Log->exit() if $may_exit;
} ## end sub _measure_7mode_cli

=head2 stop

    NACL::C::Reallocate->stop(
                                command_interface => $ci,
                                    'once' => $once,
                                    'vserver' => $vserver,
                                    'path' => $path,
                                ...);

(Class method)
This method is used to stop reallocate job.

Uses a CMode or 7Mode CLI APISet.
  Invokes 'reallocate stop' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'once' => $once >>

(Required)
  Applicable only for CMode CLI

=item C<< 'vserver' => $vserver >>

(Required)
  Applicable only for CMode CLI

=item C<< 'path' => $path >>

(Required)
  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7M CLI : path name (Programmatic name: 'path')

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"reallocate stop" command in ONTAP.

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub stop {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( once vserver path
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd => 'reallocate stop',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_stop_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_stop_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub stop

sub _stop_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( once vserver path
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'reallocate_stop',
        api_opts => \%cmode_args
    );

    $apiset->reallocate_stop(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _stop_cmode_cli

sub _stop_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface     = delete $opts{command_interface};
    my $apiset                = delete $opts{apiset};
    my $allowed_7mode_options = [qw( path )];

    my %valid_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%valid_args,
    );
    %opts = %valid_args;
    $pkg->_api_call_log_debug_opts(
        api      => 'reallocate_stop',
        api_opts => \%opts
    );
    $apiset->reallocate_stop(%opts);
    $Log->exit() if $may_exit;
} ## end sub _stop_7mode_cli

=head2 on

    NACL::C::Reallocate->on(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to enable reallocate jobs.

Uses a CMode or 7Mode CLI APISet.
  Invokes 'reallocate on' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"reallocate on" command in ONTAP.

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub on {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
                       params => \@_,
                       cli_cmd => 'reallocate on',);
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_on_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_on_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub on

sub _on_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw(
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'reallocate_on',
        api_opts => \%cmode_args
    );

    $apiset->reallocate_on(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _on_cmode_cli

sub _on_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    $apiset->reallocate_on();
    $Log->exit() if $may_exit;
} ## end sub _on_7mode_cli

=head2 schedule

    NACL::C::Reallocate->schedule(
                                command_interface => $ci,
                                    'once' => $once,
                                    'vserver' => $vserver,
                                    'path' => $path,
                                ...);

(Class method)
This method is used to modify schedule of reallocate job.

Uses a CMode or 7Mode CLI APISet.
  Invokes 'reallocate schedule' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'once' => $once >>

(Required)
  Applicable only for CMode CLI

=item C<< 'vserver' => $vserver >>

(Required)
  Applicable only for CMode CLI

=item C<< 'path' => $path >>

(Required)
  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7M CLI : path name (Programmatic name: 'path')

=item C<< 'cron' => $string >>

  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7M CLI : -s (Programmatic name: 'schedule')

=item C<< 'del' => $string >>

  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7M CLI : -d (Programmatic name: 'delete-schedule')

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"reallocate schedule" command in ONTAP.

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub schedule {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( once cron del vserver path
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd => 'reallocate schedule',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_schedule_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_schedule_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub schedule

sub _schedule_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( once del vserver path
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'reallocate_schedule',
        api_opts => \%cmode_args
    );

    $apiset->reallocate_schedule(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _schedule_cmode_cli

sub _schedule_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface     = delete $opts{command_interface};
    my $apiset                = delete $opts{apiset};
    my $allowed_7mode_options = [qw( path )];
    my %valid_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        map    => {
            "del"  => "delete-schedule",
            "cron" => "schedule",
        },
        target => \%valid_args,
    );
    %opts = %valid_args;

    $apiset->reallocate_schedule(%opts);
    $Log->exit() if $may_exit;
} ## end sub _schedule_7mode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver' => {type => SCALAR | UNDEF, optional => 1},
        'path'    => {type => SCALAR,         optional => 1},
    );
} ## end sub _primary_keys_validate_spec

1;
