
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary RepositoryVolume Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::RepositoryVolume

=head1 DESCRIPTION

C<NACL::C::RepositoryVolume> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP RepositoryVolume.  A related class is
L<NACL::CS::RepositoryVolume|lib-NACL-CS-RepositoryVolume-pm>, which represents
the past state of an ONTAP RepositoryVolume.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=cut

package NACL::C::RepositoryVolume;

use strict;
use warnings;

use NACL::CS::RepositoryVolume;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $RepositoryVolume = NACL::C::RepositoryVolume->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
								  );

Return a new RepositoryVolume component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'volume',
];

=head2 find

    my $RepositoryVolume  = NACL::C::RepositoryVolume->find(command_interface => $ci, ...);
	my @RepositoryVolumes = NACL::C::RepositoryVolume->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my RepositoryVolume_state = $NACL::C::RepositoryVolume->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::RepositoryVolume would return 
a NACL::CS::RepositoryVolume object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 mount

    NACL::C::RepositoryVolume->mount(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $RepositoryVolume_object->mount(...);

(Class or instance method)
This method is used to mount a repository volume on a junction-path.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'junction-active' => $string >>

Applicable for CMode CLI.

=item C<< 'volume' => $string >>

Applicable for CMode CLI.

=item C<< 'junction-path' => $string >>

Applicable for CMode CLI.

=item C<< 'foreground' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'active' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"repository volume mount" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub mount {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( junction-active volume junction-path foreground
                    active
                    vserver
                    )
            ),
        },
        cli_cmd => 'repository volume mount',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_mount_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::RepositoryVolume::mount',
    );

    $Log->exit() if $may_exit;
} ## end sub mount

sub _mount_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'repository_volume_mount',);
    $Log->exit() if $may_exit;
} ## end sub _mount_cmode_cli

=head2 modify

    NACL::C::RepositoryVolume->modify(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $RepositoryVolume_object->modify(...);

(Class or instance method)
This method is used to modify a repository server.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'volume' => $string >>

Applicable for CMode CLI.

=item C<< 'size' => $string >>

Applicable for CMode CLI.

=item C<< 'junction-active' => $string >>

Applicable for CMode CLI.

=item C<< 'group' => $string >>

Applicable for CMode CLI.

=item C<< 'snapshot-policy' => $string >>

Applicable for CMode CLI.

=item C<< 'unix-permissions' => $string >>

Applicable for CMode CLI.

=item C<< 'policy' => $string >>

Applicable for CMode CLI.

=item C<< 'user' => $string >>

Applicable for CMode CLI.

=item C<< 'foreground' => $string >>

Applicable for CMode CLI.

=item C<< 'nvfail' => $string >>

Applicable for CMode CLI.

=item C<< 'sys-aggr-list' => [ $sys-aggr-1, $sys-aggr-2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'read-realloc' => $string >>

Applicable for CMode CLI.

=item C<< 'snapdir-access' => $string >>

Applicable for CMode CLI.

=item C<< 'files' => $string >>

Applicable for CMode CLI.

=item C<< 'filesys-size-fixed' => $string >>

Applicable for CMode CLI.

=item C<< 'percent-snapshot-space' => $string >>

Applicable for CMode CLI.

=item C<< 'atime-update' => $string >>

Applicable for CMode CLI.

=item C<< 'security-style' => $string >>

Applicable for CMode CLI.

=item C<< 'extent-enabled' => $string >>

Applicable for CMode CLI.

=item C<< 'comment' => $string >>

Applicable for CMode CLI.

=item C<< 'min-readahead' => $string >>

Applicable for CMode CLI.

=item C<< 'ols-aggr-list' => [ $ols-aggr-1, $ols-aggr-2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI.

=item C<< 'object-write-sync-period' => $string >>

Applicable for CMode CLI.

=item C<< 'enable-object-store' => $string >>

Applicable for CMode CLI.

=item C<< 'ols-constituent-count' => $string >>

Applicable for CMode CLI.

=item C<< 'ols-constituent-size' => $string >>

Applicable for CMode CLI.

=item C<< 'storage-service' => $string >>

Applicable for CMode CLI.

=item C<< 'use-new-algorithm' => $string >>

Applicable for CMode CLI.

=item C<< 'is-managed-by-service' => $string >>

Applicable for CMode CLI.

=item C<< 'max-data-constituent-size' => $string >>

Applicable for CMode CLI.

=item C<< 'ns-mirror-aggr-list' => [ $ns-mirror-aggr-1, $ns-mirror-aggr-2, ... ] >>

*List of Aggregates for Namespace Mirrors
(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI.

=item C<< 'max-namespace-constituent-size' => $string >>

*Maximum Size of Namespace Constituent, possible value(s) are
<integer>[KB,MB,GB,TB,PB]

Applicable for CMode CLI.

=item C<< 'space-guarantee' => $string >>

*Space Guarantee, possible value(s) are none,volume,file

Applicable for CMode CLI.

=item C<< 'enable-snapdiff' => $string >>

*Create Namespace Mirrors For Snapdiff Use, possible value(s) are true,false

Applicable for CMode CLI.

=item C<< 'percent-reserve-constituent' => $string >>

*Percent Space For Reserve Data Constituent

Applicable for CMode CLI.

=item C<< 'in-nvfailed-state' => $string >>

*Volume's Nvfailed State, possible value(s) are true,false

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< 'unreachable-attr-action' => $string >>


Action When Attributes Are Not Reachable possible value(s) are, return-generated,wait
Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< 'excluded-from-balancer' => $string >>

Is Capacity Balancer Disabled possible value(s) are, true,false
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"repository volume modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "ns-mirror-aggr-list" => {type => ARRAYREF, optional => 1},
            "ols-aggr-list"       => {type => ARRAYREF, optional => 1},
            'sys-aggr-list'       => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( volume size junction-active group snapshot-policy unix-permissions policy
                    user foreground nvfail

                    read-realloc snapdir-access files filesys-size-fixed
                    percent-snapshot-space atime-update security-style extent-enabled comment
                    min-readahead

                    vserver

                    object-write-sync-period enable-object-store ols-constituent-count
                    ols-constituent-size

                    storage-service use-new-algorithm is-managed-by-service
                    max-data-constituent-size

                    max-namespace-constituent-size space-guarantee enable-snapdiff
                    percent-reserve-constituent

                    in-nvfailed-state

                    unreachable-attr-action

                    excluded-from-balancer
                    )
            ),
        },
        cli_cmd => 'repository volume modify',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::RepositoryVolume::modify',
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'repository_volume_modify',);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

=head2 offline

    NACL::C::RepositoryVolume->offline(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $RepositoryVolume_object->offline(...);

(Class or instance method)
This method is used to offline a repository volume.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'volume' => $string >>

Applicable for CMode CLI.

=item C<< 'foreground' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'force' => $string >>

*Force Unmount

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"repository volume offline" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub offline {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volume foreground
                    vserver
                    force
                    )
            ),
        },
        cli_cmd => 'repository volume offline',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_offline_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::RepositoryVolume::offline',
    );

    $Log->exit() if $may_exit;
} ## end sub offline

sub _offline_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'repository_volume_offline',);
    $Log->exit() if $may_exit;
} ## end sub _offline_cmode_cli

=head2 unmount

    NACL::C::RepositoryVolume->unmount(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $RepositoryVolume_object->unmount(...);

(Class or instance method)
This method is used to unmount a repository volume from a junction-path.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'volume' => $string >>

Applicable for CMode CLI.

=item C<< 'foreground' => $string >>

Applicable for CMode CLI.

=item C<< 'force' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"repository volume unmount" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub unmount {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volume foreground force
                    vserver
                    )
            ),
        },
        cli_cmd => 'repository volume unmount',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_unmount_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::RepositoryVolume::unmount',
    );

    $Log->exit() if $may_exit;
} ## end sub unmount

sub _unmount_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'repository_volume_unmount',);
    $Log->exit() if $may_exit;
} ## end sub _unmount_cmode_cli

=head2 restrict

    NACL::C::RepositoryVolume->restrict(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $RepositoryVolume_object->restrict(...);

(Class or instance method)
This method is used to restrict a repository volume.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'volume' => $string >>

Applicable for CMode CLI.

=item C<< 'foreground' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"repository volume restrict" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub restrict {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volume foreground
                    vserver
                    )
            ),
        },
        cli_cmd => 'repository volume restrict',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_restrict_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::RepositoryVolume::restrict',
    );

    $Log->exit() if $may_exit;
} ## end sub restrict

sub _restrict_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'repository_volume_restrict',);
    $Log->exit() if $may_exit;
} ## end sub _restrict_cmode_cli

=head2 rename

    NACL::C::RepositoryVolume->rename(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $RepositoryVolume_object->rename(...);

(Class or instance method)
This method is used to rename a repository volume.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'volume' => $string >>

Applicable for CMode CLI.

=item C<< 'newname' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'foreground' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"repository volume rename" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub rename {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volume newname
                    foreground
                    vserver
                    )
            ),
        },
        cli_cmd => 'repository volume rename',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_rename_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::RepositoryVolume::rename',
    );

    $Log->exit() if $may_exit;
} ## end sub rename

sub _rename_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'repository_volume_rename',);
    $Log->exit() if $may_exit;
} ## end sub _rename_cmode_cli

=head2 assign

    NACL::C::RepositoryVolume->assign(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $RepositoryVolume_object->assign(...);

(Class or instance method)
This method is used to assign a role to a list of volumes.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'obj-volume-group' => $string >>

Applicable for CMode CLI.

=item C<< 'role' => $string >>

Applicable for CMode CLI.

=item C<< 'volumes' => [ $volume1, $volume2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"repository volume assign" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub assign {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'volumes' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( obj-volume-group role
                    )
            ),
        },
        api => 'repository_volume_assign',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_assign_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::RepositoryVolume::assign',
    );

    $Log->exit() if $may_exit;
} ## end sub assign

sub _assign_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'repository_volume_assign',);
    $Log->exit() if $may_exit;
} ## end sub _assign_cmode_cli

=head2 online

    NACL::C::RepositoryVolume->online(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $RepositoryVolume_object->online(...);

(Class or instance method)
This method is used to online a repository volume.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'volume' => $string >>

Applicable for CMode CLI.

=item C<< 'foreground' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"repository volume online" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub online {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volume foreground
                    vserver
                    )
            ),
        },
        cli_cmd => 'repository volume online',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_online_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::RepositoryVolume::online',
    );

    $Log->exit() if $may_exit;
} ## end sub online

sub _online_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'repository_volume_online',);
    $Log->exit() if $may_exit;
} ## end sub _online_cmode_cli

=head2 create

    my $RepositoryVolume_object	= NACL::C::RepositoryVolume->create(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);

(Class method)
This method is used to create a repository server.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required)

Applicable for CMode CLI.

=item C<< 'junction-path' => $string >>

Applicable for CMode CLI.

=item C<< 'size' => $string >>

Applicable for CMode CLI.

=item C<< 'namespace-aggregate' => $string >>

Applicable for CMode CLI.

=item C<< 'junction-active' => $string >>

Applicable for CMode CLI.

=item C<< 'group' => $string >>

Applicable for CMode CLI.

=item C<< 'user' => $string >>

Applicable for CMode CLI.

=item C<< 'policy' => $string >>

Applicable for CMode CLI.

=item C<< 'default-object-volume-group' => $string >>

Applicable for CMode CLI.

=item C<< 'directory-bitwidth' => $string >>

Applicable for CMode CLI.

=item C<< 'volume' => $string >>

Applicable for CMode CLI.

=item C<< 'space-guarantee' => $string >>

Applicable for CMode CLI.

=item C<< 'snapshot-policy' => $string >>

Applicable for CMode CLI.

=item C<< 'unix-permissions' => $string >>

Applicable for CMode CLI.

=item C<< 'foreground' => $string >>

Applicable for CMode CLI.

=item C<< 'type' => $string >>

Applicable for CMode CLI.

=item C<< 'nvfail' => $string >>

Applicable for CMode CLI.

=item C<< 'sys-aggr-list' => [ $sys-aggr-1, $sys-aggr-2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'state' => $string >>

Applicable for CMode CLI.

=item C<< 'default-data-constituent-group' => $string >>

Applicable for CMode CLI.

=item C<< 'security-style' => $string >>

Applicable for CMode CLI.

=item C<< 'comment' => $string >>

Applicable for CMode CLI.

=item C<< 'percent-snapshot-space' => $string >>

Applicable for CMode CLI.

=item C<< 'ols-aggr-list' => [ $ols-aggr-1, $ols-aggr-2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI.

=item C<< 'object-write-sync-period' => $string >>

Applicable for CMode CLI.

=item C<< 'enable-object-store' => $string >>

Applicable for CMode CLI.

=item C<< 'ols-constituent-count' => $string >>

Applicable for CMode CLI.

=item C<< 'ols-constituent-size' => $string >>

Applicable for CMode CLI.

=item C<< 'storage-service' => $string >>

Applicable for CMode CLI.

=item C<< 'use-new-algorithm' => $string >>

Applicable for CMode CLI.

=item C<< 'is-managed-by-service' => $string >>

Applicable for CMode CLI.

=item C<< 'max-data-constituent-size' => $string >>

Applicable for CMode CLI.

=item C<< 'data-aggr-list' => [ $data-aggr-1, $data-aggr-2, ... ] >>

*List of Aggregates for Data Constituents
(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI.

=item C<< 'ns-mirror-aggr-list' => [ $ns-mirror-aggr-1, $ns-mirror-aggr-2, ... ] >>

*List of Aggregates for Namespace Mirrors
(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI.

=item C<< 'max-namespace-constituent-size' => $string >>

*Maximum Size of Namespace Constituent, possible value(s) are
<integer>[KB,MB,GB,TB,PB]

Applicable for CMode CLI.


=item C<< 'enable-snapdiff' => $string >>

*Create Namespace Mirrors For Snapdiff Use, possible value(s) are true,false

Applicable for CMode CLI.

=item C<< 'percent-reserve-constituent' => $string >>

*Percent Space For Reserve Data Constituent

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< 'unreachable-attr-action' => $string >>


Action When Attributes Are Not Reachable possible value(s) are, return-generated,wait
Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< 'excluded-from-balancer' => $string >>

Is Capacity Balancer Disabled possible value(s) are, true,false
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"repository volume create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "data-aggr-list"      => {type => ARRAYREF, optional => 1},
            "ns-mirror-aggr-list" => {type => ARRAYREF, optional => 1},
            "ols-aggr-list"       => {type => ARRAYREF, optional => 1},
            'sys-aggr-list'       => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( junction-path size namespace-aggregate junction-active group user policy
                    default-object-volume-group directory-bitwidth volume space-guarantee
                    snapshot-policy unix-permissions foreground type nvfail

                    state default-data-constituent-group security-style comment

                    percent-snapshot-space

                    vserver

                    object-write-sync-period enable-object-store ols-constituent-count
                    ols-constituent-size

                    storage-service use-new-algorithm is-managed-by-service
                    max-data-constituent-size

                    max-namespace-constituent-size enable-snapdiff
                    percent-reserve-constituent

                    unreachable-attr-action

                    excluded-from-balancer
                    )
            ),
        },
        cli_cmd => 'repository volume create',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::RepositoryVolume::create',
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $opts{'vserver'},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'repository_volume_create',);
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

=head2 size

    NACL::C::RepositoryVolume->size(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $RepositoryVolume_object->size(...);

(Class or instance method)
This method is used to set/Display the size of repository volume.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'volume' => $string >>

Applicable for CMode CLI.

=item C<< 'new-size' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'foreground' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"repository volume size" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub size {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volume new-size
                    foreground
                    vserver
                    )
            ),
        },
        cli_cmd => 'repository volume size',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_size_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::RepositoryVolume::size',
    );

    $Log->exit() if $may_exit;
} ## end sub size

sub _size_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'repository_volume_size',);
    $Log->exit() if $may_exit;
} ## end sub _size_cmode_cli

=head2 delete

    NACL::C::RepositoryVolume->delete(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $RepositoryVolume_object->delete(...);

(Class or instance method)
This method is used to delete a repository server.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'disable-offline-check' => $string >>

*Disable Check for Volume Offline

Applicable for CMode CLI.

=item C<< 'foreground' => $string >>

*Foreground Process, possible value(s) are true,false

Applicable for CMode CLI.

=item C<< 'force' => $string >>

*Force Delete

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"repository volume delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vserver
                    disable-offline-check foreground force
                    )
            )
        },
        cli_cmd => 'repository volume delete',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::RepositoryVolume::delete',
    );

    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'repository_volume_delete',);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

=head2 sdiftest

    NACL::C::RepositoryVolume->sdiftest(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $RepositoryVolume_obj->sdiftest(...);

(Class or instance method)
This method is used to test the Snapdiff RPC interface (check log for results).

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'volume' => $string >>

Applicable for CMode CLI.

=item C<< 'snapshot' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"repository volume sdiftest" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub sdiftest {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volume snapshot
                    vserver
                    )
            ),
        },
        cli_cmd => 'repository volume sdiftest',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_sdiftest_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::RepositoryVolume::sdiftest',
    );

    $Log->exit() if $may_exit;
} ## end sub sdiftest

sub _sdiftest_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'repository_volume_sdiftest',);
    $Log->exit() if $may_exit;
} ## end sub _sdiftest_cmode_cli

=head2 check_And_Get_RepositoryId

    NACL::C::RepositoryVolume->check_And_Get_RepositoryId(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $RepositoryVolume_obj->check_And_Get_RepositoryId(...);

(Class or instance method)
This method is used to get a Repository ID of repository volume.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'volume' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub check_And_Get_RepositoryId {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( volume

                    vserver
                    )
            ),
        },
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_check_And_Get_RepositoryId_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
}

sub _check_And_Get_RepositoryId_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( volume vserver

            vserver
            extended_query )
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg_or_obj->_api_call_log_debug_opts(
        api      => 'repository_volume_check_And_Get_RepositoryId',
        api_opts => \%cmode_args
    );

    $apiset->repository_volume_check_And_Get_RepositoryId(%cmode_args);
    $Log->exit() if $may_exit;
}

=head2 locate_file_for_storage_service

    NACL::C::RepositoryVolume->locate_file_for_storage_service(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $RepositoryVolume_obj->locate_file_for_storage_service(...);

(Class or instance method)
This method is used to returns the storage service a file is stored in..

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
*Virtual Server

Applicable for CMode CLI.


=item C<< 'volume' => $string >>

*Volume name

Applicable for CMode CLI.

=item C<< 'filename' => $string >>

*The name of the file to locate

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"repository volume locate-file-for-storage-service" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub locate_file_for_storage_service {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volume filename
                    )
            ),
        },
        cli_cmd => 'repository volume locate-file-for-storage-service',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_locate_file_for_storage_service_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend =>
            'NACL::C::RepositoryVolume::locate_file_for_storage_service',
    );

    $Log->exit() if $may_exit;
} ## end sub locate_file_for_storage_service

sub _locate_file_for_storage_service_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'repository_volume_locate_file_for_storage_service',);
    $Log->exit() if $may_exit;
} ## end sub _locate_file_for_storage_service_cmode_cli

=head2 recover_namespace

    NACL::C::RepositoryVolume->recover_namespace(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $RepositoryVolume_obj->recover_namespace(...);

(Class or instance method)
This method is used to recovers the namespace constituent of an Infinite Volume.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI

=item C<< 'volume' => $string >>

Volume name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "repository volume recover namespace" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub recover_namespace {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volume
                    )
            ),
        },
        cli_cmd => 'repository volume recover-namespace',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_recover_namespace_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::RepositoryVolume::recover_namespace',
    );

}

sub _recover_namespace_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'repository_volume_recover_namespace',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver' => {type => SCALAR | UNDEF, optional => 1},
        'volume'  => {type => SCALAR | UNDEF, optional => 1},
    );
} ## end sub _primary_keys_validate_spec

1;
