# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary RepositoryVolumeConstituent Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::RepositoryVolumeConstituent

=head1 DESCRIPTION

C<NACL::C::RepositoryVolumeConstituent> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP RepositoryVolumeConstituent.  A related class is
L<NACL::CS::RepositoryVolumeConstituent|lib-NACL-CS-RepositoryVolumeConstituent-pm>, which represents
the past state of an ONTAP RepositoryVolumeConstituent.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 volume

=head2 vserver

=cut

package NACL::C::RepositoryVolumeConstituent;

use strict;
use warnings;

use NACL::CS::RepositoryVolumeConstituent;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $RepositoryVolumeConstituent = NACL::C::RepositoryVolumeConstituent->new(
                                   command_interface => $ci,
                                   'volume' => $volume,
                                   'vserver' => $vserver,
								  );

Return a new RepositoryVolumeConstituent component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'volume',
    scalar => 'vserver',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $RepositoryVolumeConstituent  = NACL::C::RepositoryVolumeConstituent->find(command_interface => $ci, ...);
	my @RepositoryVolumeConstituents = NACL::C::RepositoryVolumeConstituent->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my RepositoryVolumeConstituent_state = $NACL::C::RepositoryVolumeConstituent->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::RepositoryVolumeConstituent would return 
a NACL::CS::RepositoryVolumeConstituent object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 unassign

    NACL::C::RepositoryVolumeConstituent->unassign(
                                command_interface => $ci,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $RepositoryVolumeConstituent_obj->unassign(...);

(Class or instance method)
This method is used to unassign a role from a volume.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)
*Constituent Volume Name

Applicable for CMode CLI.

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
*Virtual Server

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'volumes' => $string >>

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"repository volume constituent unassign" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub unassign {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volumes
                    )
            )
        },
        cli_cmd => 'repository volume constituent unassign"',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_unassign_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::RepositoryVolumeConstituent::unassign',
    );

    $Log->exit() if $may_exit;
} ## end sub unassign

sub _unassign_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'repository_volume_constituent_unassign',);
    $Log->exit() if $may_exit;
} ## end sub _unassign_cmode_cli

=head2 assign

    NACL::C::RepositoryVolumeConstituent->assign(
                                command_interface => $ci,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $RepositoryVolumeConstituent_obj->assign(...);

(Class or instance method)
This method is used to assign a role to a volume.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)
*Constituent Volume Name

Applicable for CMode CLI.

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
*Virtual Server

Applicable for CMode CLI.

=item C<< 'origMsid' => $string >>

*Origin Volume MSID

Applicable for CMode CLI.

=item C<< 'dcg_name' => $string >>

*Data Constituent Group

Applicable for CMode CLI.

=item C<< 'role' => $string >>

*Constituent Volume Role

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'storage-service' => $string >>

*Storage Service Name

Applicable for CMode CLI.

=item C<< 'volumes' => $string >>

Applicable for CMode CLI.

=item C<< 'obj-volume-group' => $string >>

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"repository volume constituent assign" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub assign {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( origMsid dcg_name role

                    storage-service

                    volumes obj-volume-group
                    )
            ),
        },
        cli_cmd => 'repository volume constituent assign',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_assign_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::RepositoryVolumeConstituent::assign',
    );

    $Log->exit() if $may_exit;
} ## end sub assign

sub _assign_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'repository_volume_constituent_assign',);
    $Log->exit() if $may_exit;
} ## end sub _assign_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'volume'  => {type => SCALAR},
        'vserver' => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
