
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary RepositoryVolumeSnapshot Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::RepositoryVolumeSnapshot

=head1 DESCRIPTION

C<NACL::C::RepositoryVolumeSnapshot> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP RepositoryVolumeSnapshot.  A related class is
L<NACL::CS::RepositoryVolumeSnapshot|lib-NACL-CS-RepositoryVolumeSnapshot-pm>, which represents
the past state of an ONTAP RepositoryVolumeSnapshot.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 volume

=head2 vserver

=head2 snapshot

=cut

package NACL::C::RepositoryVolumeSnapshot;

use strict;
use warnings;

use NACL::CS::RepositoryVolumeSnapshot;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::CS::RepositoryVolumeSnapshotGetSnapshotUUID;

=head1 METHODS

=head2 new

  my $RepositoryVolumeSnapshot = NACL::C::RepositoryVolumeSnapshot->new(
                                   command_interface => $ci,
                                   'volume' => $volume,
                                   'vserver' => $vserver,
                                   'snapshot' => $snapshot,
								  );

Return a new RepositoryVolumeSnapshot component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'volume',
    scalar => 'vserver',
    scalar => 'snapshot',
];

=head2 find

    my $RepositoryVolumeSnapshot  = NACL::C::RepositoryVolumeSnapshot->find(command_interface => $ci, ...);
	my @RepositoryVolumeSnapshots = NACL::C::RepositoryVolumeSnapshot->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my RepositoryVolumeSnapshot_state = $NACL::C::RepositoryVolumeSnapshot->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::RepositoryVolumeSnapshot would return 
a NACL::CS::RepositoryVolumeSnapshot object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 rename

    NACL::C::RepositoryVolumeSnapshot->rename(
                                command_interface => $ci,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
	                            'snapshot' => $snapshot,
                                ...);
	                             
								 or

    $RepositoryVolumeSnapshot_object->rename(...);

(Class or instance method)
This method is used to rename a repository snapshot(Not Implemented).

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'snapshot' => $snapshot >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'new-name' => $string >>

Applicable for CMode CLI.

=item C<< 'instance-uuid' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'timestamp-sort' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub rename {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@_,
        additional_spec =>
            {_optional_scalars(qw( new-name instance-uuid timestamp-sort )),},
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_rename_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub rename

sub _rename_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $apiset = $opts{apiset};

    my $allowed_cmode_options = [
        qw( volume vserver new-name instance-uuid snapshot timestamp-sort
            extended_query)
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $apiset->repository_volume_snapshot_rename(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _rename_cmode_cli

=head2 sched_create

    NACL::C::RepositoryVolumeSnapshot->sched_create(
                                command_interface => $ci,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
	                            'snapshot' => $snapshot,
                                ...);
                                
								 or

    $RepositoryVolumeSnapshot_object->sched_create(...);

(Class or instance method)
This method is used to kick off one run of a scheduled repository snapshot job.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $volume >>

(Required)

Applicable for CMode CLI.

=item C<< 'vserver' => $vserver >>

(Required)

Applicable for CMode CLI.

=item C<< 'snapshot' => $snapshot >>

(Required)

Applicable for CMode CLI.

=item C<< 'foreground' => $string >>

Applicable for CMode CLI.

=item C<< 'schedule' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'timestamp-sort' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub sched_create {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@_,
        additional_spec =>
            {_optional_scalars(qw( foreground schedule timestamp-sort )),},
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_sched_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub sched_create

sub _sched_create_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $apiset = $opts{apiset};

    my $allowed_cmode_options = [
        qw( volume vserver snapshot foreground schedule timestamp-sort
            extended_query )
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $apiset->repository_volume_snapshot_sched_create(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _sched_create_cmode_cli

=head2 remove_owner

    NACL::C::RepositoryVolumeSnapshot->remove_owner(
                                command_interface => $ci,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
	                            'snapshot' => $snapshot,
                                ...);
	                             
								 or

    $RepositoryVolumeSnapshot_object->remove_owner(...);

(Class or instance method)
This method is used to remove owner from the ownership list.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'snapshot' => $snapshot >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'owner' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'timestamp-sort' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub remove_owner {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( owner

                    volume vserver snapshot timestamp-sort
                    )
            ),
        },
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_remove_owner_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub remove_owner

sub _remove_owner_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $apiset = $opts{apiset};

    my $allowed_cmode_options = [
        qw( owner volume vserver snapshot timestamp-sort
            extended_query )
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $apiset->repository_volume_snapshot_remove_owner(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _remove_owner_cmode_cli

=head2 restore

    NACL::C::RepositoryVolumeSnapshot->restore(
                                command_interface => $ci,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
	                            'snapshot' => $snapshot,
                                ...);
	                             
								 or

    $RepositoryVolumeSnapshot_object->restore(...);

(Class or instance method)
This method is used to restore a repository snapshot.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'snapshot' => $snapshot >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'instance-uuid' => $string >>

Applicable for CMode CLI.

=item C<< 'foreground' => $string >>

Applicable for CMode CLI.

=item C<< 'force' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'timestamp-sort' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub restore {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( instance-uuid foreground force timestamp-sort )),
        },
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_restore_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub restore

sub _restore_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $apiset = $opts{apiset};

    my $allowed_cmode_options = [
        qw( volume vserver instance-uuid snapshot foreground force
            timestamp-sort extended_query )
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $apiset->repository_volume_snapshot_restore(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _restore_cmode_cli

=head2 delete

    NACL::C::RepositoryVolumeSnapshot->delete(
                                command_interface => $ci,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
	                            'snapshot' => $snapshot,
                                ...);
	                             
								 or

    $RepositoryVolumeSnapshot_object->delete(...);

(Class or instance method)
This method is used to delete a repository snapshot.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'snapshot' => $snapshot >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'ignore-owners' => $string >>

Applicable for CMode CLI.

=item C<< 'instance-uuid' => $string >>

Applicable for CMode CLI.

=item C<< 'foreground' => $string >>

Applicable for CMode CLI.

=item C<< 'force' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'timestamp-sort' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( ignore-owners instance-uuid foreground force
                    timestamp-sort )
            ),
        },
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $apiset = $opts{apiset};

    my $allowed_cmode_options = [
        qw( volume ignore-owners vserver instance-uuid snapshot foreground force
            timestamp-sort extended_query
            )
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $apiset->repository_volume_snapshot_delete(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

=head2 old_create

    NACL::C::RepositoryVolumeSnapshot->old_create(
                                command_interface => $ci,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
	                            'snapshot' => $snapshot,
                                ...);

(Class method)
This method is used to tEMPORARY: Create a repository snapshot using VOPL.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $volume >>

(Required)

Applicable for CMode CLI.

=item C<< 'vserver' => $vserver >>

(Required)

Applicable for CMode CLI.

=item C<< 'snapshot' => $snapshot >>

(Required)

Applicable for CMode CLI.

=item C<< 'comment' => $string >>

Applicable for CMode CLI.

=item C<< 'foreground' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub old_create {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( comment foreground
                    )
            ),
        },
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_old_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub old_create

sub _old_create_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $apiset = $opts{apiset};

    my $allowed_cmode_options = [
        qw( comment volume vserver snapshot foreground extended_query
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $apiset->repository_volume_snapshot_old_create(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _old_create_cmode_cli

=head2 modify

    NACL::C::RepositoryVolumeSnapshot->modify(
                                command_interface => $ci,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
	                            'snapshot' => $snapshot,
                                ...);
	                             
								 or

    $RepositoryVolumeSnapshot_object->modify(...);

(Class or instance method)
This method is used to modify repository snapshot(Not Implemented).

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'snapshot' => $snapshot >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'comment' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'timestamp-sort' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {_optional_scalars(qw(comment)),},
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $apiset = $opts{apiset};

    my $allowed_cmode_options = [
        qw( comment volume vserver snapshot timestamp-sort extended_query
            )
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $apiset->repository_volume_snapshot_modify(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

=head2 add_test_owners

    NACL::C::RepositoryVolumeSnapshot->add_test_owners(
                                command_interface => $ci,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
	                            'snapshot' => $snapshot,
                                ...);
	                             
								 or

    $RepositoryVolumeSnapshot_object->add_test_owners(...);

(Class or instance method)
This method is used to add test owners to a repository snapshot.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'snapshot' => $snapshot >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'timestamp-sort' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add_test_owners {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {_optional_scalars(qw( timestamp-sort ))},
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_add_test_owners_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub add_test_owners

sub _add_test_owners_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $apiset = $opts{apiset};

    my $allowed_cmode_options
        = [qw( volume vserver snapshot timestamp-sort extended_query )];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $apiset->repository_volume_snapshot_add_test_owners(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _add_test_owners_cmode_cli

=head2 old_delete

    NACL::C::RepositoryVolumeSnapshot->old_delete(
                                command_interface => $ci,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
	                            'snapshot' => $snapshot,
                                ...);
	                             
								 or

    $RepositoryVolumeSnapshot_object->old_delete(...);

(Class or instance method)
This method is used to tEMPORARY: Delete a repository snapshot using VOPL.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'snapshot' => $snapshot >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'ignore-owners' => $string >>

Applicable for CMode CLI.

=item C<< 'instance-uuid' => $string >>

Applicable for CMode CLI.

=item C<< 'foreground' => $string >>

Applicable for CMode CLI.

=item C<< 'force' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub old_delete {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( ignore-owners instance-uuid foreground force
                    )
            ),
        },
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_old_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub old_delete

sub _old_delete_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $apiset = $opts{apiset};

    my $allowed_cmode_options = [
        qw( volume ignore-owners vserver instance-uuid snapshot foreground force
            extended_query )
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $apiset->repository_volume_snapshot_old_delete(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _old_delete_cmode_cli

=head2 create

    my $RepositoryVolumeSnapshot_object	= NACL::C::RepositoryVolumeSnapshot->create(
                                command_interface => $ci,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
	                            'snapshot' => $snapshot,
                                ...);

(Class method)
This method is used to create repository snapshot.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $volume >>

(Required)

Applicable for CMode CLI.

=item C<< 'vserver' => $vserver >>

(Required)

Applicable for CMode CLI.

=item C<< 'snapshot' => $snapshot >>

(Required)

Applicable for CMode CLI.

=item C<< 'comment' => $string >>

Applicable for CMode CLI.

=item C<< 'foreground' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'timestamp-sort' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        additional_spec =>
            {_optional_scalars(qw( comment foreground timestamp-sort )),},
    );
    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $allowed_cmode_options = [
        qw( comment volume vserver snapshot foreground timestamp-sort
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $apiset->repository_volume_snapshot_create(%cmode_args);
    $Log->exit() if $may_exit;
    return $pkg->new(
        command_interface => $command_interface,
        volume            => $cmode_args{'volume'},
        vserver           => $cmode_args{'vserver'},
        snapshot          => $cmode_args{'snapshot'},
    );
} ## end sub _create_cmode_cli

=head2 get_snapshot_UUID

    my $cs_obj = NACL::C::RepositoryVolumeSnapshot->get_snapshot_UUID(
                                command_interface => $ci,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
	                            'snapshot' => $snapshot,
                                ...);
	                             
								 or

    my $cs_obj = $RepositoryVolumeSnapshot_obj->get_snapshot_UUID(...);

(Class or instance method)
This method is used to get UUID of given snapshot. It returns a CS-like
object of type
L<NACL::CS::RepositoryVolumeSnapshotGetSnapshotUUID|lib-NACL-CS-RepositoryVolumeSnapshotGetSnapshotUUID-pm>.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'snapshot' => $snapshot >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'timestamp-sort' => $timestamp_sort >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'reposId' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'timestamp-sort' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub get_snapshot_UUID {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {_optional_scalars(qw( reposId timestamp-sort )),},
    );
    my $cs_obj = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_get_snapshot_UUID_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $cs_obj;
}

sub _get_snapshot_UUID_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $allowed_cmode_options = [
        qw( reposId volume vserver snapshot timestamp-sort extended_query)];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    my $response
        = $apiset->repository_volume_snapshot_get_snapshot_UUID(%cmode_args);
    my $row    = $response->get_parsed_output()->[0];
    my $cs_obj = NACL::CS::RepositoryVolumeSnapshotGetSnapshotUUID->new();
    $cs_obj->command_interface($command_interface);
    $cs_obj->_set_fields(row => $row);
    $Log->exit() if $may_exit;
    return $cs_obj;
}

=head2 get_Constituent_SnapShot_ID

    NACL::C::RepositoryVolumeSnapshot->get_Constituent_SnapShot_ID(
                                command_interface => $ci,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
	                            'snapshot' => $snapshot,
	                            'timestamp-sort' => $timestamp_sort,
                                ...);
	                             
								 or

    $RepositoryVolumeSnapshot_obj->get_Constituent_SnapShot_ID(...);

(Class or instance method)
This method is used to get snapID of Repository Data Constituent.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'snapshot' => $snapshot >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'timestamp-sort' => $timestamp_sort >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'constituentMsid' => $string >>

Applicable for CMode CLI.

=item C<< 'reposId' => $string >>

Applicable for CMode CLI.

=item C<< 'instance-uuid' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'timestamp-sort' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub get_Constituent_SnapShot_ID {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( constituentMsid reposId instance-uuid timestamp-sort
                    )
            ),
        },
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_get_Constituent_SnapShot_ID_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
}

sub _get_Constituent_SnapShot_ID_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $apiset = $opts{apiset};

    my $allowed_cmode_options = [
        qw( constituentMsid reposId volume vserver instance-uuid snapshot timestamp-sort
            extended_query
            )
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $apiset->repository_volume_snapshot_get_Constituent_SnapShot_ID(
        %cmode_args);
    $Log->exit() if $may_exit;
}

=head2 get_NS_Constituent

    NACL::C::RepositoryVolumeSnapshot->get_NS_Constituent(
                                command_interface => $ci,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
	                            'snapshot' => $snapshot,
	                            'timestamp-sort' => $timestamp_sort,
                                ...);
	                             
								 or

    $RepositoryVolumeSnapshot_obj->get_NS_Constituent(...);

(Class or instance method)
This method is used to get snapMsid of Repository NS Constituent.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'snapshot' => $snapshot >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'timestamp-sort' => $timestamp_sort >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'reposId' => $string >>

Applicable for CMode CLI.

=item C<< 'instance-uuid' => $string >>

Applicable for CMode CLI.

=item C<< 'needMirror' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'timestamp-sort' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub get_NS_Constituent {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( reposId instance-uuid needMirror timestamp-sort )),
        },
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_get_NS_Constituent_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
}

sub _get_NS_Constituent_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $apiset = $opts{apiset};

    my $allowed_cmode_options = [
        qw( reposId volume vserver instance-uuid needMirror snapshot timestamp-sort
            extended_query
            )
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $apiset->repository_volume_snapshot_get_NS_Constituent(%cmode_args);
    $Log->exit() if $may_exit;
}

=head2 get_Data_Constituent

    NACL::C::RepositoryVolumeSnapshot->get_Data_Constituent(
                                command_interface => $ci,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
	                            'snapshot' => $snapshot,
	                            'timestamp-sort' => $timestamp_sort,
                                ...);
	                             
								 or

    $RepositoryVolumeSnapshot_obj->get_Data_Constituent(...);

(Class or instance method)
This method is used to get snapMsid of Repository Data Constituent.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'snapshot' => $snapshot >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'timestamp-sort' => $timestamp_sort >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'reposId' => $string >>

Applicable for CMode CLI.

=item C<< 'instance-uuid' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'timestamp-sort' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub get_Data_Constituent {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@_,
        additional_spec =>
            {_optional_scalars(qw( reposId instance-uuid timestamp-sort )),},
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_get_Data_Constituent_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
}

sub _get_Data_Constituent_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $apiset = $opts{apiset};

    my $allowed_cmode_options = [
        qw( reposId volume vserver instance-uuid snapshot timestamp-sort
            extended_query
            )
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $apiset->repository_volume_snapshot_get_Data_Constituent(%cmode_args);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'volume'   => {type => SCALAR},
        'vserver'  => {type => SCALAR},
        'snapshot' => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
