#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SecurityAudit Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@natapp.com
## @status public
## @pod here

=head1 NAME

NACL::C::SecurityAudit

=head1 DESCRIPTION

C<NACL::C::SecurityAudit> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SecurityAudit.  A related class is
L<NACL::CS::SecurityAudit|lib-NACL-CS-SecurityAudit-pm>, which represents
the past state of an ONTAP SecurityAudit.
However, since it does not have any primary keys, 'find' and 'state' do not
work for this component. (This component is considered a 'singleton',
i.e. there is only one row shown by the relevant 'show' command)

=cut

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=cut

package NACL::C::SecurityAudit;

use strict;
use warnings;

use NACL::CS::SecurityAudit;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::_Mixins::SecurityAudit qw(:all);

=head1 METHODS

=head2 new

  my $SecurityAudit = NACL::C::SecurityAudit->new(command_interface=>$ci,
								  );

Return a new SecurityAudit component object with the given attributes.

=cut

=head2 modify

   NACL::C::SecurityAudit->modify(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to set audit logging.

Uses a CMode CLI/ZAPI APISet.

Invokes "audit-modify" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "httpset" => $string >>

Possible values are "on", "off".

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: http-set

=item C<< "httpget" => $string >>

Possible values are "on", "off".

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: http-get

=item C<< "snmpset" => $string >>

Possible values are "on", "off".

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: snmp-set

=item C<< "cliset" => $string >>

Possible values are "on", "off".

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: cli-set

=item C<< "snmpget" => $string >>

Possible values are "on", "off".

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: snmp-get

=item C<< "cliget" => $string >>

Possible values are "on", "off".

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: cli-get

=item C<< "oldsnmpget" => $string >>

Possible values are "on", "off".

Applicable for CMode CLI.

=item C<< 'ontapiset' => $string >>

Possible values are "on", "off".

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ontapi-set

=item C<< 'ontapiget' => $string >>

Possible values are "on", "off".

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ontapi-get

=item C<< '*' >>

This method takes all other options supported by
"security audit modify" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( httpset httpget snmpset cliset snmpget cliget oldsnmpget
                    ontapiset ontapiget
                    )
            ),
        },
        cli_cmd => 'security audit modify',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_modify_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SecurityAudit::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,
        api     => 'security_audit_modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    my %on_off_true_false_map = ('on' => 'true', 'off' => 'false');
    while (my ($key, $value) = each %opts) {
        if (exists $on_off_true_false_map{$value}) {
            $opts{$key} = $on_off_true_false_map{$value};
        }
    }

    $pkg->_base_backend_cmode_zapi(%opts,
        api    => 'audit_modify',
        map    => $pkg->_cli_to_zapi_map(),
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

=head2 delete

    NACL::C::SecurityAudit->delete(
                                command_interface => $ci,
                                ...);

(Class method)

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"security audit delete" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        api    => 'security_audit_delete',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SecurityAudit::delete',
    );

    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->_base_backend_cmode_cli(
        %opts,
        api => 'security_audit_delete',
    );

    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli 

=head2 create

    my $SecurityAudit_obj	= NACL::C::SecurityAudit->create(
                                command_interface => $ci,
                                ...);

(Class method)

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'ontapiset' => $string >>

Applicable for CMode CLI

=item C<< 'httpset' => $string >>

Applicable for CMode CLI

=item C<< 'httpget' => $string >>

Applicable for CMode CLI

=item C<< 'ontapiget' => $string >>

Applicable for CMode CLI

=item C<< 'snmpset' => $string >>

Applicable for CMode CLI

=item C<< 'cliset' => $string >>

Applicable for CMode CLI

=item C<< 'snmpget' => $string >>

Applicable for CMode CLI

=item C<< 'cliget' => $string >>

Applicable for CMode CLI

=item C<< 'oldsnmpget' => $string >>

Applicable for CMode CLI

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"security audit create" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( ontapiset httpset httpget ontapiget snmpset cliset snmpget cliget oldsnmpget
                    )
            ),
        },
        api => 'security_audit_create',
    );
    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SecurityAudit::create',
    );

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->_base_backend_cmode_cli(
        %opts,
        api => 'security_audit_create',
    );

    $Log->exit() if $may_exit;
    return $pkg->new(command_interface => $opts{command_interface});
}## end sub _create_cmode_cli


__PACKAGE__->_load_event_class();

1;
