# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SecurityConfig Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SecurityConfig

=head1 DESCRIPTION

C<NACL::C::SecurityConfig> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SecurityConfig.  A related class is
L<NACL::CS::SecurityConfig|lib-NACL-CS-SecurityConfig-pm>, which represents
the past state of an ONTAP SecurityConfig.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 interface

=cut

package NACL::C::SecurityConfig;

use strict;
use warnings;

use NACL::CS::SecurityConfig;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $SecurityConfig = NACL::C::SecurityConfig->new(
                                   command_interface => $ci,
                                   'interface' => $interface,
								  );

Return a new SecurityConfig component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'interface',];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $SecurityConfig_obj  = NACL::C::SecurityConfig->find(command_interface => $ci, ...);
	my @SecurityConfig_objs = NACL::C::SecurityConfig->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my SecurityConfig_state = $SecurityConfig_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SecurityConfig would return 
a NACL::CS::SecurityConfig object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 modify

    NACL::C::SecurityConfig->modify(
                                command_interface => $ci,
	                            'interface' => $interface,
                                ...);
	                             
								 or

    $SecurityConfig_obj->modify(...);

(Class or instance method)
This method is used to modify Security Configuration Options.
Uses CMode CLI/ZAPI .
Invokes  security-config-modify  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'interface' => $interface >>

(Required for class method, Not Applicable for instance method)
FIPS Compliant Interface
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'supported-ciphers' => $string >>

Supported Ciphers
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'is-fips-enabled' => $string >>

FIPS Mode possible value(s) are, true,false
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'supported-protocols' => [ $supported-protocol1, $supported-protocol2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "security config modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'supported-protocols' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( supported-ciphers is-fips-enabled
                    )
            ),
        },
        cli_cmd => 'security config modify',
    );
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_modify_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::SecurityConfig::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'security_config_modify',);
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [
        qw(supported-ciphers is-fips-enabled interface supported-protocols)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'security-config-modify',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(@opts, _primary_keys => [qw(interface)],);
    $Log->exit() if $may_exit;
}

=head2 prepare_to_downgrade

    NACL::C::SecurityConfig->prepare_to_downgrade(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to restore security configuration to releases earlier than Data ONTAP 8.4.0..
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< '*' >>

This method takes all other options supported by "security config prepare to downgrade" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub prepare_to_downgrade {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        cli_cmd             => 'security config prepare-to-downgrade',
    );
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_prepare_to_downgrade_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        frontend => 'NACL::C::SecurityConfig::prepare_to_downgrade',
    );

    $Log->exit() if $may_exit;
}

sub _prepare_to_downgrade_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'security_config_prepare_to_downgrade',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('interface' => {type => SCALAR},);
}

1;
