# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SecurityLoginBanner Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SecurityLoginBanner

=head1 DESCRIPTION

C<NACL::C::SecurityLoginBanner> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SecurityLoginBanner.  A related class is
L<NACL::CS::SecurityLoginBanner|lib-NACL-CS-SecurityLoginBanner-pm>, which represents
the past state of an ONTAP SecurityLoginBanner.
However, since it does not have any primary keys, 'find' and 'state' do not
work for this component. (This component is considered a 'singleton',
i.e. there is only one row shown by the relevant 'show' command)

=cut

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=cut

package NACL::C::SecurityLoginBanner;

use strict;
use warnings;

use NACL::CS::SecurityLoginBanner;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use NACL::C::_Mixins::SecurityLogin qw(_check_for_zapi);

=head1 METHODS

=head2 new

  my $SecurityLoginBanner = NACL::C::SecurityLoginBanner->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
								  );

Return a new SecurityLoginBanner component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'vserver',];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $SecurityLoginBanner_obj  = NACL::C::SecurityLoginBanner->find(command_interface => $ci, ...);
    my @SecurityLoginBanner_objs = NACL::C::SecurityLoginBanner->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my SecurityLoginBanner_state = $SecurityLoginBanner_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SecurityLoginBanner would return
a NACL::CS::SecurityLoginBanner object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::SecurityLoginBanner->modify(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
		 or

    $SecurityLoginBanner_obj->modify(...);

(Class or instance method)
This method is used to modify the login banner message.
Uses CMode CLI/ZAPI .
Invokes  vserver-login-banner-modify-iter for CMode Zapi iter
Invokes  vserver-login-banner-modify  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name
Applicable for CMode CLI/ZAPI iter, non-iter.

Mapping for iter-zapi: <vserver-login-banner-info vserver="$value" />

=item C<< 'message' => $string >>

Login Banner Message
Applicable for CMode CLI/ZAPI iter, non-iter.

Mapping for iter-zapi: <vserver-login-banner-info message="$value" />

=item C<< 'uri' => $string >>


Download URI for the Banner Message possible value(s) are, (ftp,http)://(hostname,IPv4 Address,'['IPv6 Address']')...
Applicable for CMode CLI/ZAPI iter, non-iter.

=item C<< '*' >>

This method takes all other options supported by "security login banner modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( message uri
                    )
            ),
        },
        cli_cmd => 'security login banner modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },
            {   method    => '_modify_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_check_non_iter'
            },
        ],
        frontend => 'NACL::C::SecurityLoginBanner::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'security_login_banner_modify',);
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    require NACL::C::Exceptions::SecurityLoginBanner::DoesNotExist;
    my $copy = [qw(vserver message)];
    $pkg_or_obj->_iter_helper(
        @opts,
        api      => 'vserver-login-banner-modify-iter',
        show_cmd => 'security login banner show',
        copy     => $copy,
        does_not_exist_exception =>
            'NACL::C::Exceptions::SecurityLoginBanner::DoesNotExist',
        fix_api_args => sub {
            my %opts_in_sub = @_;
            my $attributes  = delete $opts_in_sub{attributes};
            $pkg_or_obj->_hash_copy(
                source => $attributes,
                copy   => $copy,
                target => \%opts_in_sub,
            );
            return %opts_in_sub;
        },
    );
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(message)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'vserver-login-banner-modify',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [qw(vserver)],
        _vs_context   => 1
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('vserver' => {type => SCALAR},);
}

1;
