# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SecurityLoginMotd Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SecurityLoginMotd

=head1 DESCRIPTION

C<NACL::C::SecurityLoginMotd> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SecurityLoginMotd.  A related class is
L<NACL::CS::SecurityLoginMotd|lib-NACL-CS-SecurityLoginMotd-pm>, which represents
the past state of an ONTAP SecurityLoginMotd.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=cut

package NACL::C::SecurityLoginMotd;

use strict;
use warnings;

use NACL::CS::SecurityLoginMotd;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF );
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use NACL::Exceptions::InvalidChoice qw(:try);
use NACL::C::_Mixins::SecurityLogin qw(_check_for_zapi);

=head1 METHODS

=head2 new

  my $SecurityLoginMotd = NACL::C::SecurityLoginMotd->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
								  );

Return a new SecurityLoginMotd component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'vserver',];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $SecurityLoginMotd_obj  = NACL::C::SecurityLoginMotd->find(command_interface => $ci, ...);
	my @SecurityLoginMotd_objs = NACL::C::SecurityLoginMotd->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my SecurityLoginMotd_state = $SecurityLoginMotd_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SecurityLoginMotd would return
a NACL::CS::SecurityLoginMotd object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::SecurityLoginMotd->modify(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);

								 or

    $SecurityLoginMotd_obj->modify(...);

(Class or instance method)
This method is used to modify the message of the day.
Uses CMode CLI/ZAPI .
Invokes  vserver-motd-modify-iter for CMode Zapi iter
Invokes  vserver-motd-modify  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name
Applicable for CMode CLI/ZAPI iter, non-iter.

Mapping for iter-zapi: <vserver-motd-info vserver="$value" />

=item C<< 'is-cluster-message-enabled' => $string >>

Is Cluster-level Message Enabled? possible value(s) are, true,false
Applicable for CMode CLI/ZAPI iter, non-iter.

Mapping for iter-zapi: <vserver-motd-info is-cluster-message-enabled="$value" />

=item C<< 'message' => $string >>

Message of the Day (MOTD)
Applicable for CMode CLI/ZAPI iter, non-iter.

Mapping for iter-zapi: <vserver-motd-info message="$value" />

=item C<< 'uri' => $string >>


Download URI for the MOTD possible value(s) are, (ftp,http)://(hostname,IPv4 Address,'['IPv6 Address']')...
Applicable for CMode CLI/ZAPI iter, non-iter.

=item C<< '*' >>

This method takes all other options supported by "security login motd modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( is-cluster-message-enabled message uri
                    )
            ),
        },
        api => 'security_login_motd_modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter',
                check     => '_check_for_zapi',
            },
            {   method    => '_modify_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_modify_non_iter_check',
            },
        ],
        frontend => 'NACL::C::SecurityLoginMotd::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api => 'security_login_motd_modify',);
    $Log->exit() if $may_exit;
} #end of _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    require NACL::C::Exceptions::SecurityLogin::DoesNotExist;
    my $copy = [qw(vserver is-cluster-message-enabled message)];

    $pkg_or_obj->_iter_helper(
        %opts,
        api      => 'vserver-motd-modify-iter',
        show_cmd => 'security login motd show',
        copy     => $copy,
        does_not_exist_exception =>
            'NACL::C::Exceptions::SecurityLogin::DoesNotExist',
        fix_api_args => sub {
            my %opts_in_sub = @_;
            my $attributes  = delete $opts_in_sub{attributes};
            $pkg_or_obj->_hash_copy(
                source => $attributes,
                copy   => $copy,
                target => \%opts_in_sub,
            );
            return %opts_in_sub;
        },
    );
    $Log->exit() if $may_exit;
} #end of _modify_cmode_zapi

sub _modify_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $copy = [qw(is-cluster-message-enabled message)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api  => 'vserver-motd-modify',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
} #end of _modify_cmode_zapi_non_iter

sub _modify_non_iter_check {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    # First call the common ZAPI check
    $pkg_or_obj->_check_for_zapi(%opts);

    # Since is-cluster-message-enabled cannot be modified on a data-vserver,
    # check if the specified vserver is a data-vserver if the option is
    # provided.
    if (defined $opts{'is-cluster-message-enabled'}) {
         my $cs = NACL::CS::Vserver->fetch(
            command_interface => $opts{command_interface},
            apiset_should     => {interface => 'ZAPI'},
            filter            => {vserver => $opts{vserver}},
            requested_fields  => [qw(type)],
            allow_empty       => 1
         );
         if (defined $cs && $cs->type() eq 'data') {
             my $msg = "The 'is-cluster-message-enabled' field cannot "
                 . 'be modified for the data-vserver using the non-iter '
                 . 'ZAPI, so skipping the non-iter back-end';
             $Log->exit() if $may_exit;
             NACL::Exceptions::InvalidChoice->throw($msg);
         }
    }
    $Log->exit() if $may_exit;
}

=head2 motd_and_banner_publish

    NACL::C::SecurityLoginMotd->motd_and_banner_publish(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to publish processing for the message of the day and SSH banner.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< '*' >>

This method takes all other options supported by "security login motd motd and banner publish" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub motd_and_banner_publish {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        api                 => 'security_login_motd_motd_and_banner_publish',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_motd_and_banner_publish_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        frontend => 'NACL::C::SecurityLoginMotd::motd_and_banner_publish',
    );

    $Log->exit() if $may_exit;
}

sub _motd_and_banner_publish_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'security_login_motd_motd_and_banner_publish',);
    $Log->exit() if $may_exit;
}

=head2 motd_and_banner_subscribe

    NACL::C::SecurityLoginMotd->motd_and_banner_subscribe(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to subscribe processing for the message of the day and SSH banner.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< '*' >>

This method takes all other options supported by "security login motd motd and banner subscribe" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub motd_and_banner_subscribe {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        api => 'security_login_motd_motd_and_banner_subscribe',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_motd_and_banner_subscribe_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        frontend => 'NACL::C::SecurityLoginMotd::motd_and_banner_subscribe',
    );

    $Log->exit() if $may_exit;
}

sub _motd_and_banner_subscribe_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'security_login_motd_motd_and_banner_subscribe',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('vserver' => {type => SCALAR},);
}

1;
