#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SecurityLoginRoleConfig Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SecurityLoginRoleConfig

=head1 DESCRIPTION

C<NACL::C::SecurityLoginRoleConfig> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SecurityLoginRoleConfig.  A related class is
L<NACL::CS::SecurityLoginRoleConfig|lib-NACL-CS-SecurityLoginRoleConfig-pm>, which represents
the past state of an ONTAP SecurityLoginRoleConfig.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 profile

=head2 vserver

=head2 role

=cut

package NACL::C::SecurityLoginRoleConfig;

use strict;
use warnings;

use NACL::CS::SecurityLoginRoleConfig;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF UNDEF);
use NACL::C::Exceptions::SecurityLoginRoleConfig::DoesNotExist qw(:try);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use NACL::APISet::Exceptions::ResponseException qw(:try);

=head1 METHODS

=head2 new

  my $SecurityLoginRoleConfig = NACL::C::SecurityLoginRoleConfig->new(command_interface=>$ci,
                                   profile =>$profile,
                                   vserver =>$vserver,
                                   role =>$role,
                                  );

Return a new SecurityLoginRoleConfig component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'profile',
    scalar => 'vserver',
    scalar => 'role',
];

=head2 find

    my $SecurityLoginRoleConfig  = NACL::C::SecurityLoginRoleConfig->find(command_interface => $ci, ...);
    my @SecurityLoginRoleConfigs = NACL::C::SecurityLoginRoleConfig->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $SecurityLoginRoleConfig_state = $NACL::C::SecurityLoginRoleConfig->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SecurityLoginRoleConfig would return
a NACL::CS::SecurityLoginRoleConfig object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 modify

   NACL::C::SecurityLoginRoleConfig->modify(
                                command_interface => $ci,
                                 profile => $profile,
                                 vserver => $vserver,
                                 role => $role,
                                  ...);

                                 or

    $SecurityLoginRoleConfig_object->modify(...);

(Class or instance method)
This method is used to modify local user account restrictions.

Uses a CMode CLI/ZAPI, 7Mode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "profile" => $profile >>

(Required for class method, Not Applicable for instance method)

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: vserver

=item C<< "role" => $role >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: role-name

=item C<< "passwd-alphanum" => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI: require-password-alpha-numeric

=item C<< "passwd-alphanum-zapi" => $string >>

Applicable for CMode CLI

=item C<< "disallowed-reuse" => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI: last-passwords-disallowed-count

=item C<< "locktime" => $string >>

Applicable for CMode CLI

=item C<< "passwd-minsize" => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI: min-password-length

=item C<< "passwd-expwarntime" => $string >>

Applicable for CMode CLI

=item C<< "account-exptime" => $string >>

Applicable for CMode CLI

=item C<< "username-alphanum" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: require-username-alpha-numeric

=item C<< "username-minsize" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: min-username-length

=item C<< "passwd-exptime" => $string >>

Applicable for CMode CLI

=item C<< "maxattempts" => $string >>

Applicable for CMode CLI

=item C<< "require-update" => $string >>

Applicable for CMode CLI

=item C<< "username-alphanum-zapi" => $string >>

Applicable for CMode CLI

=item C<< "change-delay" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: change-password-duration-in-days

=item C<< 'passwd-expiry-time' => $string >>

Applicable for CMode CLI

=item C<< 'username-minlength' => $string >>

Applicable for CMode CLI

=item C<< 'passwd-min-special-chars' => $string >>

Applicable for CMode CLI

=item C<< 'max-failed-login-attempts' => $string >>

Applicable for CMode CLI

=item C<< 'passwd-minlength' => $string >>

Applicable for CMode CLI

=item C<< 'lockout-duration' => $string >>

Applicable for CMode CLI

=item C<< 'require-initial-passwd-update' => $string >>

Applicable for CMode CLI

=item C<< 'delay-login-attempts' => $string >>

Applicable for CMode CLI

=item C<< 'require-initial-passwd-update-zapi' => $string >>

=item C<< '*' >>

This method takes all other options supported by
"security login role config modify" command in ONTAP.

Applicable for CMode CLI.

Applicable for CMode CLI

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( passwd-alphanum passwd-alphanum-zapi disallowed-reuse locktime
                    passwd-expwarntime account-exptime username-alphanum
                    username-minsize passwd-exptime maxattempts require-update
                    username-alphanum-zapi change-delay passwd-minsize
                    passwd-expiry-time username-minlength passwd-min-special-chars
                    max-failed-login-attempts passwd-minlength lockout-duration
                    require-initial-passwd-update delay-login-attempts
                    require-initial-passwd-update-zapi
                    )
            ),
        },
        cli_cmd => 'security login role config modify',
    );
    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_modify_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_modify_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                },
                {   method    => "_modify_cmode_zapi",
                    interface => "ZAPI",
                    zapi_type => "iter",
                    set       => "CMode",
                },
                {   method    => "_modify_cmode_zapi_non_iter",
                    interface => "ZAPI",
                    zapi_type => "non-iter",
                    set       => "CMode",
                    check     => "_base_check_non_iter",
                },

            ],
            frontend => 'NACL::C::SecurityLoginRoleConfig::modify',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        NACL::C::Exceptions::SecurityLoginRoleConfig::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj,%opts) = @_;

    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api => 'security_login_role_config_modify',
    );

    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my @names;
    my @values;
    my %security_opts;
    $security_opts{'security.passwd.rules.enable'} = 'on';

    if (exists($opts{'passwd-minsize'})) {
        $security_opts{'security.passwd.rules.minimum'}
            = $opts{'passwd-minsize'};
    }
    if (exists($opts{'disallowed-reuse'})) {
        $security_opts{'security.passwd.rules.history'}
            = $opts{'disallowed-reuse'};
    }
    if (exists($opts{'passwd-alphanum'})) {
        if ($opts{'passwd-alphanum'} eq "enabled") {
            $security_opts{'security.passwd.rules.minimum.alphabetic'} = '2';
            $security_opts{'security.passwd.rules.minimum.digit'}      = '1';
        }

        if ($opts{'passwd-alphanum'} eq "disabled") {
            $security_opts{'security.passwd.rules.minimum.alphabetic'} = 0;
            $security_opts{'security.passwd.rules.minimum.digit'}      = 0;

        }
    } ## end if ( exists( $opts{'passwd-alphanum'...
    $pkg_or_obj->_api_call_log_debug_opts(
        api      => 'options security',
        api_opts => \%opts
    );
    $apiset->options(%security_opts);
    $Log->exit() if $may_exit;
} ## end sub _modify_7mode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(max-failed-login-attempts vserver lockout-duration)];
    my $map  = {
        'passwd-expiry-time'            => 'password-expiration-duration',
        'username-minlength'            => 'min-username-size',
        'passwd-alphanum'               => 'require-password-alpha-numeric',
        'disallowed-reuse'              => 'last-passwords-disallowed-count',
        'passwd-min-special-chars'      => 'min-passwd-specialchar',
        'passwd-minlength'              => 'min-password-size',
        'require-initial-passwd-update' => 'require-initial-password-update',
        'username-alphanum'             => 'require-username-alpha-numeric',
        'change-delay'                  => 'change-password-duration-in-days',
        'role'                          => 'role-name',
    };
    $pkg_or_obj->_iter_helper(
        @opts,
        api      => 'security-login-role-config-modify-iter',
        show_cmd => 'security login role config show',
        copy     => $copy,
        map      => $map,
        does_not_exist_exception =>
            'NACL::C::Exceptions::SecurityLoginRoleConfig::DoesNotExist'
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

sub _modify_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw( vserver lockout-duration max-failed-login-attempts)];
    my $map  = {
        "role"              => "role-name",
        "username-alphanum" => "require-username-alpha-numeric",
        "passwd-alphanum"   => "require-password-alpha-numeric",
        "disallowed-reuse"  => "last-passwords-disallowed-count",
        "change-delay"      => "change-password-duration-in-days",
        'passwd-expiry-time'        => 'password-expiration-duration',
        'passwd-min-special-chars'  => 'min-passwd-specialchar',
        'passwd-expiry-time'        => 'password-expiration-duration',
        'passwd-minsize'            => 'min-password-size',
        'username-minsize'          => 'min-username-size',
        'require-initial-passwd-update' =>
            'require-initial-password-update',
        'passwd-minlength'         => 'min-password-size',
        'username-minlength'       => 'min-username-size',
    };
    
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'security-login-role-config-modify',
        copy => $copy,
        map  => $map,
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi_non_iter

=head2 reset

    NACL::C::SecurityLoginRoleConfig->reset(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to reset new RBAC characteristics.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"security login role config reset" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string  >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub reset {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        cli_cmd             => 'security login role config reset',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_reset_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SecurityLoginRoleConfig::reset',
    );

    $Log->exit() if $may_exit;
} ## end sub reset

sub _reset_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg,%opts) = @_;

    $pkg->_base_backend_cmode_cli(
        %opts,
        api => 'security_login_role_config_reset',
    );

    $Log->exit() if $may_exit;
} ## end sub _reset_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver' => {type => SCALAR, optional => 1},
        'role'    => {type => SCALAR, optional => 1},
    );
} ## end sub _primary_keys_validate_spec

1;
