# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SecuritySsh Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SecuritySsh

=head1 DESCRIPTION

C<NACL::C::SecuritySsh> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SecuritySsh.  A related class is
L<NACL::CS::SecuritySsh|lib-NACL-CS-SecuritySsh-pm>, which represents
the past state of an ONTAP SecuritySsh.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=cut

package NACL::C::SecuritySsh;

use strict;
use warnings;

use NACL::CS::SecuritySsh;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $SecuritySsh = NACL::C::SecuritySsh->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
								  );

Return a new SecuritySsh component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'vserver',];

=head2 find

    my $SecuritySsh_obj  = NACL::C::SecuritySsh->find(command_interface => $ci, ...);
	my @SecuritySsh_objs = NACL::C::SecuritySsh->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my SecuritySsh_state = $SecuritySsh_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SecuritySsh would return 
a NACL::CS::SecuritySsh object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::SecuritySsh->modify(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $SecuritySsh_obj->modify(...);

(Class or instance method)
This method is used to modify SSH configuration options.

Uses a CMode CLI/ZAPI APISet.

Invokes "security-ssh-reset" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : vserver

=item C<< 'ciphers' => [ $cipher1, $cipher2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : ciphers

=item C<< 'key-exchange-algorithms' => [ $key-exchange-algorithm1, $key-exchange-algorithm2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : key-exchange-algorithms

=item C<< '*' >>

This method takes all other options supported by "security ssh modify" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'ciphers'                 => {type => ARRAYREF, optional => 1},
            'key-exchange-algorithms' => {type => ARRAYREF, optional => 1},
        },
        cli_cmd => 'security ssh modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SecuritySsh::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts, api => 'security_ssh_modify',
    );
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'security-ssh-reset',
        copy            => [qw( ciphers key-exchange-algorithms)],
        frontend_method => 'modify'
    );
    $Log->exit() if $may_exit;
}

=head2 add

    NACL::C::SecuritySsh->add(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $SecuritySsh_obj->add(...);

(Class or instance method)
This method is used to add SSH configuration options..

Uses a CMode CLI/ZAPI APISet.

Invokes "security-ssh-add" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : vserver

=item C<< 'ciphers' => [ $cipher1, $cipher2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : ciphers

=item C<< 'key-exchange-algorithms' => [ $key-exchange-algorithm1, $key-exchange-algorithm2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : key-exchange-algorithms

=item C<< '*' >>

This method takes all other options supported by "security ssh add" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'ciphers'                 => {type => ARRAYREF, optional => 1},
            'key-exchange-algorithms' => {type => ARRAYREF, optional => 1},
        },
        cli_cmd => 'security ssh add',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_add_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_add_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SecuritySsh::add',
    );

    $Log->exit() if $may_exit;
}

sub _add_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts, api => 'security_ssh_add',);
    $Log->exit() if $may_exit;
}

sub _add_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'security-ssh-add',
        copy            => [qw( ciphers key-exchange-algorithms)],
        frontend_method => 'add'
    );
    $Log->exit() if $may_exit;
}

=head2 remove

    NACL::C::SecuritySsh->remove(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $SecuritySsh_obj->remove(...);

(Class or instance method)
This method is used to remove SSH configuration options..

Uses a CMode CLI/ZAPI APISet.

Invokes "security-ssh-remove" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : vserver

=item C<< 'ciphers' => [ $cipher1, $cipher2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : ciphers

=item C<< 'key-exchange-algorithms' => [ $key-exchange-algorithm1, $key-exchange-algorithm2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : key-exchange-algorithms

=item C<< '*' >>

This method takes all other options supported by "security ssh remove" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub remove {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'ciphers'                 => {type => ARRAYREF, optional => 1},
            'key-exchange-algorithms' => {type => ARRAYREF, optional => 1},
        },
        cli_cmd => 'security ssh remove',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_remove_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_remove_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SecuritySsh::remove',
    );

    $Log->exit() if $may_exit;
}

sub _remove_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts, api => 'security_ssh_remove',
    );
    $Log->exit() if $may_exit;
}

sub _remove_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'security-ssh-remove',
        copy            => [qw( ciphers key-exchange-algorithms)],
        frontend_method => 'remove'
    );
    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::SecuritySsh->delete(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $SecuritySsh_obj->delete(...);

(Class or instance method)
This method is used to restore SSH configuration to prior release..

Uses a CMode CLI/ZAPI APISet.

Invokes "ssh-delete" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : vserver

=item C<< '*' >>

This method takes all other options supported by "security ssh delete" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        cli_cmd    => 'security ssh delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SecuritySsh::delete',
    );

    $Log->exit() if $may_exit;
}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts, api => 'security_ssh_delete',
    );
    $Log->exit() if $may_exit;
}

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'ssh-delete',
        frontend_method => 'delete'
    );
    $Log->exit() if $may_exit;
}

=head2 prepare_to_downgrade

    NACL::C::SecuritySsh->prepare_to_downgrade(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $SecuritySsh_obj->prepare_to_downgrade(...);

(Class or instance method)
This method is used to restore SSH configuration to releases earlier than Data ONTAP 8.2.1..
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "security ssh prepare to downgrade" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub prepare_to_downgrade {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {'vserver' => {type => SCALAR, optional => 1}},
        cli_cmd         => 'security ssh prepare-to-downgrade',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_prepare_to_downgrade_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_prepare_to_downgrade_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SecuritySsh::prepare_to_downgrade',
    );

    $Log->exit() if $may_exit;
}

sub _prepare_to_downgrade_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'security_ssh_prepare_to_downgrade',);
    $Log->exit() if $may_exit;
}

sub _prepare_to_downgrade_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api             => 'ssh-prepare-to-downgrade',
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('vserver' => {type => SCALAR},);
}

1;
