# $Id: //depot/prod/test/nacldev/lib/NACL/C/Slist.pm#6 $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Slist Component Module
## @author anbumozh, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Slist

=head1 DESCRIPTION

C<NACL::C::Slist> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.

=cut

package NACL::C::Slist;

use strict;
use warnings;

use Params::Validate qw(validate_with);
use NACL::ComponentUtils qw (_optional_scalars );
use base 'NACL::C::Component::ONTAP';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 slist

  NACL::C::Slist->slist(command_interface=>$ci,path=>$file);

(Class method) Request todo a slist on a file. Return apiset response object
on completion.

Uses a 7Mode-CLI or Nodescope APISet.
  Invokes 'slist' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< path=>$file >>

(Required) Name of a file or directory>
  Applicable only for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut


sub slist {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params => \@_,
        additional_spec => {
               _optional_scalars( qw(path) ) }
    );
    $Log->exit() if $may_exit;
    return $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_slist_7mode_cli",
                interface => "CLI",
                set       => "7Mode|Nodescope",
            }
        ]
    );
} ## end sub create

sub _slist_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %args;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $apiset = delete $opts{apiset};
    delete $opts{command_interface};
    my $response = $apiset->slist(%opts);
    $Log->exit() if $may_exit;
    return $response;
} ## end sub _slist_7mode_cli



__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ();
}

1;
