# $Id: //depot/prod/test/main/lib/NACL/C/SnapvaultSnap.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#

## @summary snapvault snap Component Module
## @author dl-dpg-nb-automation, dl-nacl-dev;
## @status shared
## @pod here
# Supports 7Mode CLI

=head1 NAME

NACL::C::SnapvaultSnap

=head1 DESCRIPTION

C<NACL::C::SnapvaultSnap> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.
It represents snapvault command on a filer.

=cut

package NACL::C::SnapvaultSnap;

use strict;
use warnings;
use NACL::ComponentUtils qw (Dumper _optional_scalars);
use NACL::CS::SnapvaultSnap;
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();


=head1 METHODS

=head2 new

    my $session = NACL::C::SnapvaultSnap->new(
                                   command_interface=>$ci,
                                   'secondary-path'=>$secondary_path,
                              );

Return a new SnapvaultSnap component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vol_name',
    scalar => 'snapname',
];

=head2 snap_sched

   NACL::C::snapvault->snap_sched(
                   command_interface => $ci,
                   'secondary-path'   => $secondary-path
                  ...);

 (Class method or Instance Method)
This method is used to schedule snapshot.

Uses a 7Mode CLI.
  Invokes 'snapvault snap sched' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "secondary-path" => $secondary_path >>

(Required for class method) secondary location path
  Applicable only for 7Mode CLI.

<< "vol-name" => $scalar >>

(Required) Volume name on which snapshot have to be taken.
  Applicable only for 7Mode CLI.

<< "snap-name" => $scalar >>

(Required) Name of snapshot.
  Applicable only for 7Mode CLI.

<< "schedule" => $scalar >>

(Required)Schedule for taking snapshot.
  Applicable only for 7Mode CLI.

<< "force" => $Boolean >>

(Optional)forceful start.
  Applicable only for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut


sub snap_sched {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars qw(
                 secondary-path force options start-transfer vol-name snap-name schedule
            ),
        },
       ignore_primary_keys => 1,
    );
    my $ret = $pkg->call_on_apiset(
          %opts,
          choices => [
              {
                  method    => '_snap_sched_7mode_cli',
                  interface => 'CLI',
                  set       => '7Mode',
              },
          ],
    );
    $Log->exit() if $may_exit;
    return $ret;
}
sub _snap_sched_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{'secondary-path'};
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    my @prompts_answers = ('.*Are you sure you want this setting.*'=> 'y');
    delete $opts{'secondary-path'};
    $pkg->_handle_boolean_options(
            api_options => \%opts,
            boolean_options => [ qw(force) ],
            is_7m_cli       => 1
    );
    $opts{'connectrec-match_table'} = \@prompts_answers;
    $pkg->_api_call_log_debug_opts(
        api      => 'snapvault snap sched',
        api_opts => \%opts
    );
    $apiset->snapvault_snap_sched(%opts);
    $Log->exit() if $may_exit;
}

=head2 snap_unsched

   NACL::C::snapvault->snap_unsched(
                   command_interface => $ci,
                   'secondary-path'   => $secondary-path
                  ...);

 (Class method or Instance Method)
This method is used to unschedule taking snap.

Uses a 7Mode CLI.
  Invokes 'snapvault snap unsched' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "secondary-path" => $secondary_path >>

(Required for class method) secondary location path
  Applicable only for 7Mode CLI.

<< "vol-name" => $scalar >>

(Required) Volume name on which snapshot have to be taken.
  Applicable only for 7Mode CLI.

<< "snap-name" => $scalar >>

(Required) Name of snapshot.
  Applicable only for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut


sub snap_unsched {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars qw(
                secondary-path force vol-name snap-name
            ),
        },
        ignore_primary_keys => 1,
    );
    my $ret = $pkg->call_on_apiset(
          %opts,
          choices => [
                {
                  method    => '_snap_unsched_7mode_cli',
                  interface => 'CLI',
                  set       => '7Mode',
                },
          ],
    );
    $Log->exit() if $may_exit;
    return $ret;
}
sub _snap_unsched_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{'secondary-path'};
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    my @prompts_answers = ('.*Unconfigure these snapshot schedule.*'=> 'y');
    $pkg->_handle_boolean_options(
            api_options => \%opts,
            boolean_options => [ qw(force) ],
            is_7m_cli       => 1
    );
    $opts{'connectrec-match_table'} = \@prompts_answers;
    $pkg->_api_call_log_debug_opts(
        api      => 'snapvault snap unsched',
        api_opts => \%opts
    );
    $apiset->snapvault_snap_unsched(%opts);
    $Log->exit() if $may_exit;
}


=head2 snap_create

   NACL::C::snapvault->snap_create(
                   command_interface => $ci,
                   'secondary-path'   => $secondary-path
                  ...);

 (Class method )
This method is used create a snap.

Uses a 7Mode CLI.
  Invokes 'snapvault snap create' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "secondary-path" => $secondary_path >>

(Required for class method) secondary location path
  Applicable only for 7Mode CLI.

<< "vol-name" => $scalar >>

(Required) Volume name on which snapshot have to be taken.
  Applicable only for 7Mode CLI.

<< "snap-name" => $scalar >>

(Required) Name of snapshot.
  Applicable only for 7Mode CLI.

<< "wait" => $Boolean >>

(Optional)option causes the command not to return once the baseline transfer starts..
  Applicable only for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub snap_create {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars qw(
               secondary-path options wait snap-name vol-name
            ),
        },
        ignore_primary_keys => 1,
    );
    my $ret = $pkg->call_on_apiset(
          %opts,
          choices => [
                {
                  method    => '_snap_create_7mode_cli',
                  interface => 'CLI',
                  set       => '7Mode',
                },
          ],
    );
    $Log->exit() if $may_exit;
    return $ret;
}
sub _snap_create_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    my $secondarypath = delete $opts{'secondary-path'};
    $pkg->_api_call_log_debug_opts(
        api      => 'snapvault snap create',
        api_opts => \%opts
    );
    $apiset->snapvault_snap_create(%opts);
    $Log->exit() if $may_exit;
    return $pkg->new(
        snapname          => $opts{'snap-name'},
        command_interface => $command_interface
    );
}


=head2 snap_preserve

   NACL::C::snapvault->snap_preserve(
                   command_interface => $ci,
                   'secondary-path'   => $secondary-path
                  ...);

 (Class method or Instance Method)
This method is used preserve snapshot.

Uses a 7Mode CLI.
  Invokes 'snapvault snap preserve' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "secondary-path" => $secondary_path >>

(Required for class method) secondary location path
  Applicable only for 7Mode CLI.

<< "vol-name" => $scalar >>

(Required) Volume name on which snapshot have to be taken.
  Applicable only for 7Mode CLI.

<< "snap-name" => $scalar >>

(Required) Name of snapshot.
  Applicable only for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut


sub snap_preserve {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
                        _optional_scalars qw(
                        secondary-path vol-name snap-name tag-name
                        ),
                        ignore_primary_keys => 1,
        },
        ignore_primary_keys => 1,
    );
    my $ret = $pkg->call_on_apiset(
          %opts,
          choices => [
              {
                  method    => '_snap_preserve_7mode_cli',
                  interface => 'CLI',
                  set       => '7Mode',
              },
          ],
    );
    $Log->exit() if $may_exit;
    return $ret;
}
sub _snap_preserve_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    delete $opts{'secondary-path'};
    $pkg->_api_call_log_debug_opts(
        api      => 'snapvault snap preserve',
        api_opts => \%opts
    );
    $apiset->snapvault_snap_preserve(%opts);
    $Log->exit() if $may_exit;
}

=head2 snap_unpreserve

   NACL::C::snapvault->snap_unpreserve(
                   command_interface => $ci,
                   'secondary-path'   => $secondary-path
                  ...);

 (Class method or Instance Method)
This method is used to unpreserve the preserved snap.

Uses a 7Mode CLI.
  Invokes 'snapvault snap unpreserve' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "secondary-path" => $secondary_path >>

(Required for class method) secondary location path
  Applicable only for 7Mode CLI.

<< "vol-name" => $scalar >>

(Required) Volume name on which snapshot have to be taken.
  Applicable only for 7Mode CLI.

<< "snap-name" => $scalar >>

(Required) Name of snapshot.
  Applicable only for 7Mode CLI.

<< "snap_unpreserve" => $string >>

(Optional) all will preserve all snapshots
  Applicable only for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut


sub snap_unpreserve {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars qw(
            secondary-path vol-name snap-name tag-name preserve-all
            ),
        },
        ignore_primary_keys => 1,
    );
    my $ret = $pkg->call_on_apiset(
          %opts,
          choices => [
              {
                  method    => '_snap_unpreserve_7mode_cli',
                  interface => 'CLI',
                  set       => '7Mode',
              },
          ],
    );
    $Log->exit() if $may_exit;
    return $ret;
}
sub _snap_unpreserve_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts        = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    if ( $opts{'tag-name'} ne "" ) {
        delete $opts{'preserve-all'};
        delete $opts{'secondary-path'};
    } else {
        delete $opts{'tag-name'};
        delete $opts{'secondary-path'};
    }
    $pkg->_api_call_log_debug_opts(
        api      => 'snapvault snap unpreserve',
        api_opts => \%opts
    );
    $apiset->snapvault_snap_unpreserve(%opts);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

 sub _primary_keys_validate_spec {
       return (
                'snapname' => { type => SCALAR },
              );
}
1;
