# $Id: //depot/prod/test/nacldev/lib/NACL/C/StatisticsArchive.pm#16 $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StatisticsArchive Component Module (auto-generated by CGT)
## @author Akshay Mohit(makshay@netapp.com), dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::StatisticsArchive

=head1 DESCRIPTION

C<NACL::C::StatisticsArchive> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP StatisticsArchive.  A related class is
L<NACL::CS::StatisticsArchive|lib-NACL-CS-StatisticsArchive-pm>, which represents
the past state of an ONTAP StatisticsArchive.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 archive

=cut

package NACL::C::StatisticsArchive;

use strict;
use warnings;

use NACL::CS::StatisticsArchive;
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use base 'NACL::C::Component::ONTAP';
use NACL::ComponentUtils qw(_optional_scalars convert_to_unix_time);
use Params::Validate qw(validate validate_with SCALAR ARRAYREF OBJECT);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::APISet::Exceptions::CommandNotFoundException qw(:try);

=head1 METHODS

=head2 new

  my $StatisticsArchive = NACL::C::StatisticsArchive->new(
                                   command_interface => $ci,
                                   'archive' => $archive,
          );

Return a new StatisticsArchive component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'archive',

    # supported for backwards compatibility
    scalar => [
        {   '-read_cb' => sub {
                return (defined $_[1]) ? $_[1] : $_[0]->archive;
                }
        },
        'name'
    ],
];

=head2 find

    my $StatisticsArchive_obj  = NACL::C::StatisticsArchive->find(command_interface => $ci, ...);
  my @StatisticsArchive_objs = NACL::C::StatisticsArchive->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my StatisticsArchive_state = $StatisticsArchive_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::StatisticsArchive would return
a NACL::CS::StatisticsArchive object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 create

    my $StatisticsArchive_obj  = NACL::C::StatisticsArchive->create(
                                command_interface => $ci,
                                'name' => $name,
                                ...);

(Class method)
This method is used to save new Performance Archive.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'perf-archive-save' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'case-number' => $string >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: case-number

=item C<< 'end-time' => $string >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: end-time

=item C<< 'upload-uri' => $string >>

  Applicable only for CMode CLI.

=item C<< 'start-time' => $string >>

  Applicable for CMode CLI.
  Maps to:
    CM ZAPI: start-time

=item C<< 'comment' => $string >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: comment

=item C<< 'host-node' => $string >>

  Applicable only for CMode CLI.

=item C<< 'source-nodes' => [ $source-node1, $source-node2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: source-nodes

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'archive' => $string >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: archive

=item C<< 'extended_query' => $string >>

(Optional)
Applicable only for CMode CLI

=item C<< 'foreground' => $string >>

*Execute Operation in the Foreground?, possible value(s) are true,false
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"statistics archive create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'source-nodes' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( case-number end-time upload-uri start-time comment archive host-node

                    foreground
                    )
            ),
        },
        cli_cmd => 'statistics archive create',
    );
    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                check     => '_foreground_zapi_check',
            },
        ],
        frontend => 'NACL::C::StatisticsArchive::create',
    );

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_cli(%opts,
        api     => 'statistics_archive_create',
    );

    $Log->exit() if $may_exit;
    if (defined $opts{'name'}) {
        return $pkg->new(
            command_interface => $opts{command_interface},
            name              => $opts{'name'},
        );
    } else {
        return $pkg->new(
            command_interface => $opts{command_interface},
            archive           => $opts{'archive'}
        );
    } ## end else [ if ( defined $cmode_args...)]
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $command_interface     = delete $opts{command_interface};
    my $apiset                = delete $opts{apiset};
    my $allowed_cmode_options = [
        qw(case-number comment archive
            source-nodes start-time end-time
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        map    => {'name' => 'archive'},
        target => \%cmode_args,
    );

    foreach my $time (qw( start-time end-time )) {
        if (defined $opts{$time}
            && $opts{$time} =~ /(\d+)\/(\d+) (\d+):(\d+):(\d+)/)
        {
            my ($mm, $dd, $hh, $min, $sec, $year);
            $mm  = $1;
            $dd  = $2;
            $hh  = $3;
            $min = $4;
            $sec = $5;
            my @array = localtime(time);
            $year = $array[5] + 1900;
            $cmode_args{$time} = $pkg->convert_to_unix_time(
                command_interface => $command_interface,
                interface         => 'ZAPI',
                year              => $year,
                month             => $mm,
                day               => $dd,
                hour              => $hh,
                minute            => $min,
                second            => $sec,
            );
        } elsif ($opts{$time}) {
            $cmode_args{$time} = $opts{$time};
        }
    }
    try {
        $apiset->perf_archive_save(%cmode_args);
    }
    catch NACL::APISet::Exceptions::CommandNotFoundException with {

        #trying to create using perf_archive_create api
        $apiset->perf_archive_create(%cmode_args);
    };
    $Log->exit() if $may_exit;
    return $pkg->new(
        command_interface => $command_interface,
        archive           => $cmode_args{'archive'}
    );
} ## end sub _create_cmode_zapi

=head2 delete

    NACL::C::StatisticsArchive->delete(
                                command_interface => $ci,
                                'name' => $name,
                                ...);
         or

    $StatisticsArchive_obj->delete(...);

(Class or instance method)
This method is used to destroy existing Performance Archive.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'perf-archive-destroy' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'archive' => $string >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.
Maps to:
    CM ZAPI: archive

=item C<< 'host-node' => $string >>

  Applicable only for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"statistics archive delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {_optional_scalars(qw( archive host-node )),},
        cli_cmd         => 'statistics archive delete',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::StatisticsArchive::delete',
    );

    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'statistics_archive_delete',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $allowed_cmode_options = [
        qw( archive
            )
    ];

    $pkg->_base_backend_cmode_zapi(@args,
        api    => 'perf_archive_destroy',
        copy   => $allowed_cmode_options,
        map    => {'name' => 'archive'},
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 upload

    NACL::C::StatisticsArchive->upload(
                                command_interface => $ci,
                              'name' => $name,
                                ...);
         or

    $StatisticsArchive_obj->upload(...);

(Class or instance method)
This method is used to upload Performance Archive.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'name' => $name >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI.

=item C<< 'upload-uri' => $string >>

  Applicable only for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.
  Applicable only for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)
  Applicable only for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"statistics archive upload" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub upload {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            {_optional_scalars(qw( upload-uri archive host-node )),},
        api => 'statistics_archive_upload',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_upload_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::StatisticsArchive::upload',
    );

    $Log->exit() if $may_exit;
} ## end sub upload

sub _upload_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj,%opts) = @_;

    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api => 'statistics_archive_upload',
    );

    $Log->exit() if $may_exit;
} ## end sub _upload_cmode_cli

=head2 modify

    NACL::C::StatisticsArchive->modify(
                                command_interface => $ci,
                              'name' => $name,
                                ...);
         or

    $StatisticsArchive_obj->modify(...);

(Class or instance method)
This method is used to modify properties of Performance Archive.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'perf-archive-modify' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'comment' => $string >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: comment

=item C<< 'case-number' => $string >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: case-number

=item C<< 'new-name' => $string >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: new-name

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'archive' => $string >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: archive

=item C<< 'host-node' => $string >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: host-node

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"statistics archive modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( comment case-number new-name archive host-node
                    )
            ),
        },
        cli_cmd => 'statistics archive modify',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::StatisticsArchive::modify',
    );
    if (defined $opts{'new-name'} && ref $pkg_or_obj) {
        $pkg_or_obj->name($opts{'new-name'});
    }
    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'statistics_archive_modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( comment case-number new-name name
            host-node archive
            )
    ];

    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
        api    => 'perf_archive_modify',
        copy   => $allowed_cmode_options,
        map    => {name => 'archive'},
    );

    $Log->exit() if $may_exit;

} ## end sub _modify_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate(
        @args,
        {   command_interface => {
                type     => OBJECT,
                isa      => 'NACL::C::CommandInterface::ONTAP',
                optional => 1
            }
        }
    );
    my $command_interface = $opts{command_interface};

    my %default_spec = (
        'archive' => {type => SCALAR, optional => 1},
        'name'    => {type => SCALAR, optional => 1},
    );

    if (defined $command_interface) {
        return ('archive' => {type => SCALAR});
    } else {
        $Log->exit() if $may_exit;
        return %default_spec;
    }
} ## end sub _primary_keys_validate_spec

1;
