# $Id: //depot/prod/test/nacldev/lib/NACL/C/StatisticsPreset.pm#16 $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StatisticsPreset Component Module (auto-generated by CGT)
## @author Akshay mohit(makshay@netapp.com), pkj@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::StatisticsPreset

=head1 DESCRIPTION

C<NACL::C::StatisticsPreset> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP StatisticsPreset.  A related class is
L<NACL::CS::StatisticsPreset|lib-NACL-CS-StatisticsPreset-pm>, which represents
the past state of an ONTAP StatisticsPreset.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 preset

=cut

package NACL::C::StatisticsPreset;

use strict;
use warnings;

use NACL::CS::StatisticsPreset;
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $StatisticsPreset = NACL::C::StatisticsPreset->new(
                                   command_interface => $ci,
                                   'preset' => $preset,
                                  );

Return a new StatisticsPreset component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'preset',
];

=head2 find

    my $StatisticsPreset_obj  = NACL::C::StatisticsPreset->find(command_interface => $ci, ...);
    my @StatisticsPreset_objs = NACL::C::StatisticsPreset->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my StatisticsPreset_state = $StatisticsPreset_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::StatisticsPreset would return
a NACL::CS::StatisticsPreset object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

Invokes 'perf-preset-create' API for CMode ZAPI.

=cut

=head2 create

    my $StatisticsPreset_obj    = NACL::C::StatisticsPreset->create(
                                command_interface => $ci,
                                'preset' => $preset,
                                ...);

(Class method)
This method is used to create a Performance Preset.

Uses a CMode ZAPI APISet.
CMode CLI support is not provided as ONTAP allows to set presets only through
ZAPI.

Invokes 'perf-preset-create' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'preset' => $preset >>

(Required)
Applicable for CMode ZAPI.

Maps to:

CM ZAPI:

<perf-preset-create>
<preset>$value</preset>
</perf-preset-create>

=item C<< 'is-vserver-visible' => $string >>

Applicable for CMode ZAPI.

Maps to:

CM ZAPI:

<perf-preset-create>
<is-vserver-visible>$value</is-vserver-visible>
</perf-preset-create>

=item C<< 'comment' => $string >>

Applicable for CMode ZAPI.

Maps to:

CM ZAPI:

<perf-preset-create>
<comment>$value</comment>
</perf-preset-create>

=item C<< 'privilege' => $string >>

Applicable for CMode ZAPI.

Maps to:

CM ZAPI:

<perf-preset-create>
<privilege>$value</privilege>
</perf-preset-create>

=item C<< 'expiration-length' => $string >>

Applicable for CMode ZAPI.

Maps to:

CM ZAPI:

<perf-preset-create>
<expiration-length>$value</expiration-length>
</perf-preset-create>

=item C<< 'is-archive-enabled' => $string >>

Applicable for CMode ZAPI.

Maps to:

CM ZAPI:

<perf-preset-create>
<is-archive-enabled>$value</is-archive-enabled>
</perf-preset-create>

=item C<< "preset-detail-infos" => @array >>

This is an array of typedefs, which in turn contains other elements which are arrays.

For Example:
'preset-detail-infos' => [
    {
        'counter-set'      => [ $val1, $val2 ],
        'instance-filters' => [ $v1, $v2 ],
    },
    {
        'counter-set' => [ $val3, $val4 ],
    },
 ]

=item C<< 'preset-details' => @array >>

This is an array of typedefs, which in turn contains other elements which are arrays.

For Example:
'preset-details' => [
    {
        'counter-set'      => [ $val1, $val2 ],
        'instance-filters' => [ $v1, $v2 ],
    }
 ]

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'preset-detail-infos' => { type => ARRAYREF, optional => 1 },
            _optional_scalars(
                qw( is-vserver-visible comment privilege preset vserver
                    expiration-length is-archive-enabled
                    preset-details
                    )
            ),
        },
    );
    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $copy = [
        qw( is-vserver-visible comment vserver privilege preset
            expiration-length is-archive-enabled preset-detail-infos
            preset-details
            )
    ];
    $pkg->_base_backend_cmode_zapi(
        %opts,
        copy => $copy,
        api  => 'perf-preset-create',
    );
    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        preset            => $opts{'preset'},
        vserver           => $opts{'vserver'},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub _create_cmode_zapi

=head2 modify

    NACL::C::StatisticsPreset->modify(
                                command_interface => $ci,
                                'preset' => $preset,
                                ...);

                                 or

    $StatisticsPreset_obj->modify(...);

(Class or instance method)
This method is used to modify a Performance Preset.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'perf-preset-modify' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'preset' => $preset >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: preset

(Required for class method, Not Applicable for instance method)

=item C<< 'newname' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: newname

=item C<< 'is-vserver-visible' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: is-vserver-visible

=item C<< 'comment' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: comment

=item C<< 'is-archiver-enabled' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: is-archiver-enabled

=item C<< 'privilege' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: privilege

=item C<< 'archiver-expiration' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: archiver-expiration

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'is-archive-enabled' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: is-archive-enabled

=item C<< 'new-name' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: new-name

=item C<< 'expiration-length' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: expiration-length

=item C<< 'extended_query' => $string >>

(Optional)
Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"statistics preset modify" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( newname is-vserver-visible comment is-archiver-enabled
                    privilege archiver-expiration is-archive-enabled new-name
                    expiration-length
                    )
            ),
        },
        cli_cmd => 'statistics preset modify',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::StatisticsPreset::modify',
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'statistics_preset_modify',);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [
        qw( archiver-expiration comment expiration-length is-archive-enabled
            is-archiver-enabled is-vserver-visible new-name newname preset
            privilege)
    ];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => $copy,
        api  => 'perf-preset-modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

=head2 do_import

    NACL::C::StatisticsPreset->do_import(
                                command_interface => $ci,
                                'preset' => $preset,
                                ...);

                                 or

    $StatisticsPreset_obj->do_import(...);

(Class or instance method)
This method is used to import Performance Preset configuration from source URI.
Note that the name of this method does not match the CMode CLI command.
This is because the "import" method has special meaning in Perl.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'preset' => $preset >>

(Required for class method, Not Applicable for instance method)
Applicable only for CMode CLI.

=item C<< 'uri-password' => $string >>

(Optional)

Applicable only for CMode CLI.

=item C<< 'uri-username' => $string >>

(Optional)

Applicable only for CMode CLI.

=item C<< 'source-uri' => $string >>

(Required)

Applicable only for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)
Applicable only for CMode CLI.

=item C<< 'is-archive-enabled' => $string >>

Is Preset Archive-Enabled? possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'comment' => $string >>

Preset Description
Applicable for CMode CLI

=item C<< 'privilege' => $string >>

Preset Privilege Level
Applicable for CMode CLI

=item C<< 'expiration-length' => $string >>

Default Expiration Length (Minutes) ranges from 0 to 1440
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"statistics preset import" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub do_import {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( uri-password uri-username source-uri

                    is-archive-enabled comment privilege expiration-length
                    )
            ),
        },
        cli_cmd => 'statistics preset import',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_do_import_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_do_import_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::StatisticsPreset::do_import',
    );

    $Log->exit() if $may_exit;
} ## end sub do_import

sub _do_import_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api => 'statistics_preset_import',);
    $Log->exit() if $may_exit;
} ## end sub _do_import_cmode_cli

sub _do_import_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [
        qw(uri-password uri-username source-uri is-archive-enabled comment privilege
            preset expiration-length)
    ];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'perf-preset-import',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}


=head2 delete

    NACL::C::StatisticsPreset->delete(
                                command_interface => $ci,
                                'preset' => $preset,
                                ...);

                                 or

    $StatisticsPreset_obj->delete(...);

(Class or instance method)
This method is used to delete a Performance Preset.

Uses a CMode CLI/ZAPI APISet.

Invokes 'perf-preset-delete' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'preset' => $preset >>

(Required for class method, Not Applicable for instance method)
Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<perf-preset-delete>
<expiration-length>$value</expiration-length>
</perf-preset-delete>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable only for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"statistics preset delete" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params   => \@args,
        cli_cmd  => 'statistics preset delete',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        frontend => 'NACL::C::StatisticsPreset::delete',
    );

    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'statistics_preset_delete',);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(preset)],
        api  => 'perf-preset-delete',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver' => {type => SCALAR | UNDEF, optional => 1},
        'preset'  => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
