#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StatisticsSettings Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::StatisticsSettings

=head1 DESCRIPTION

C<NACL::C::StatisticsSettings> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP StatisticsSettings.  A related class is
L<NACL::CS::StatisticsSettings|lib-NACL-CS-StatisticsSettings-pm>, which represents
the past state of an ONTAP StatisticsSettings.
However, since it does not have any primary keys, 'find' and 'state' do not
work for this component. (This component is considered a 'singleton',
i.e. there is only one row shown by the relevant 'show' command)

=cut

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=cut

package NACL::C::StatisticsSettings;

use strict;
use warnings;

use NACL::CS::StatisticsSettings;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $StatisticsSettings = NACL::C::StatisticsSettings->new(command_interface=>$ci,
								  );

Return a new StatisticsSettings component object with the given attributes.

=cut

=head2 modify

   NACL::C::StatisticsSettings->modify(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to specify whether to update base values on displayed statistics and client statistics setting.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "max-clients-to-view" => $string >>

Applicable for CMode CLI.

=item C<< "display-rates" => $string >>

Applicable for CMode CLI.

=item C<< "max-clients-to-collect" => $string >>

Applicable for CMode CLI.

=item C<< "client-stats" => $string >>

Applicable for CMode CLI.

=item C<< "update-delta-base" => $string >>

Applicable for CMode CLI.

=item C<< 'counter-display' => $string >>

*Counter Display Value, possible value(s) are Counter Display
full,non_zero,none,all

Applicable for CMode CLI.


=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< 'display-count-exponent' => $string >>

Display Count Exponent ranges from 1 to 10
Applicable for CMode CLI

=item C<< 'counter-display-units' => $string >>

Counter Display Units possible value(s) are, Counter Units B,KB,MB,GB
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"statistics settings modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( max-clients-to-view display-rates max-clients-to-collect client-stats
                    update-delta-base

                    counter-display

                    display-count-exponent counter-display-units
                    )
            ),
        },
        api => 'statistics_settings_modify',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::StatisticsSettings::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'statistics_settings_modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ();
}

1;
