# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StatisticsV1 Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::StatisticsV1

=head1 DESCRIPTION

C<NACL::C::StatisticsV1> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP StatisticsV1.  A related class is
L<NACL::CS::StatisticsV1|lib-NACL-CS-StatisticsV1-pm>, which represents
the past state of an ONTAP StatisticsV1.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 object

=head2 counter

=head2 node

=cut

package NACL::C::StatisticsV1;

use strict;
use warnings;

use NACL::CS::StatisticsV1;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $StatisticsV1 = NACL::C::StatisticsV1->new(
                                   command_interface => $ci,
                                   'object' => $object,
                                   'counter' => $counter,
                                   'node' => $node,
                                );

Return a new StatisticsV1 component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'object',
    scalar => 'counter',
    scalar => 'node',
];

=head2 find

    my $StatisticsV1_obj  = NACL::C::StatisticsV1->find(command_interface => $ci, ...);
    my @StatisticsV1_objs = NACL::C::StatisticsV1->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my StatisticsV1_state = $StatisticsV1_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::StatisticsV1 would return
a NACL::CS::StatisticsV1 object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 show_periodic

    NACL::C::StatisticsV1->show_periodic(
                                command_interface => $ci,
	                            'object' => $object,
	                            'counter' => $counter,
	                            'node' => $node,
                                ...);

(Class method)
This method is used to continuously display performance statistics at regular intervals.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'object' => $object >>

(Applicable for class method call & not required for instance method)
Object
Applicable for CMode CLI

=item C<< 'counter' => $counter >>

(Applicable for class method call & not required for instance method)
Counter
Applicable for CMode CLI

=item C<< 'node' => $node >>

(Applicable for class method call & not required for instance method)
Node
Applicable for CMode CLI

=item C<< 'vserver' => $string >>

Vserver
Applicable for CMode CLI

=item C<< 'iterations' => $string >>

Number of iterations
Applicable for CMode CLI

=item C<< 'interval' => $string >>

Interval in seconds
Applicable for CMode CLI

=item C<< 'instance' => $string >>

Instance
Applicable for CMode CLI

=item C<< 'summary' => $string >>

Print Summary, possible value(s) are true,false
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub show_periodic {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( object vserver iterations interval node instance summary counter
                    )
            ),
        },
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_show_periodic_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
}

sub _show_periodic_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my %opts = $pkg_or_obj->_validate_backend_opts(@opts);

    my $apiset                = $opts{apiset};
    my $execution_time        = $opts{cmd_execution_time};
    my $allowed_cmode_options = [
        qw( object vserver iterations interval node instance summary counter
            extended_query
            )
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    my $response = $apiset->statistics_v1_show_periodic(%cmode_args);
    if (defined $execution_time) {
        $$execution_time = $response->get_execution_time();
    }

    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'object'  => {type => SCALAR},
        'counter' => {type => SCALAR},
        'node'    => {type => SCALAR},
    );
}

1;
