# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StorageAggregatePlex Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::StorageAggregatePlex

=head1 DESCRIPTION

C<NACL::C::StorageAggregatePlex> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. Unlike other components this
package does not model any element/service but is present to provide
component-level support for functionality. Since it does not model any
element/service there is no ComponentState module for this component.
For the same reason, 'find' and 'state' will not work for this component.
Instead NACL::CS::StorageAggregatePlex->fetch() should be used.
(See L<fetch|lib-NACL-CS-ComponentState-pm/fetch> for a full description)

=cut

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 aggregate

=head2 plex

=cut

package NACL::C::StorageAggregatePlex;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use NACL::CS::StorageAggregatePlex;
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $aggregateplex = NACL::C::StorageAggregatePlex->new(command_interface=>$ci,
                                         aggregate =>$aggr_name,
                                          plex     => $plex);

Return a new StorageAggregate component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'aggregate',
    scalar => 'plex',
    scalar => [
        {'-read_cb' => sub { $_[0]->get_plex_path() if (ref $_[0]) },},
        'plex_path'
    ],
];

=head2 offline

    NACL::C::StorageAggregatePlex->offline(
                                command_interface => $ci,
                                aggregate =>$aggr_name,
                                 plex     => $plex,
                                ...);

    $aggregateplex->offline(...);

(Class or Instance method)
This method is used to offline a plex.

Uses a CMode CLI  and 7Mode CLI APISet.

Invokes 'aggr offline' command for 7Mode CLI.

Invokes "aggr-offline" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'plex' => $string >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI/ZAPI

=item C<< 'unmount-volumes' => $string >>

  Applicable only for CMode CLI

=item C<< 'aggregate' => $string >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI/ZAPI

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"storage aggregate plex offline" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub offline {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( unmount-volumes
                    )
            ),
        },
        cli_cmd => 'storage aggregate plex offline',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_offline_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_offline_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_offline_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::StorageAggregatePlex::offline',
    );

    $Log->exit() if $may_exit;
} ## end sub offline

sub _offline_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    $pkg_or_obj->_base_backend_cmode_cli (
        @args,
        api => 'storage_aggregate_plex_offline',
    );

    $Log->exit() if $may_exit;
} ## end sub _offline_cmode_cli

sub _offline_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $plex = $pkg_or_obj->get_plex_path(
        aggregate => $opts{aggregate},
        plex      => $opts{plex}
    );

    $apiset->aggr_offline('plex-name' => $plex);
    $Log->exit() if $may_exit;
} ## end sub _offline_7mode_cli

sub _offline_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(aggregate plex)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'aggr-offline',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
} ## end sub _offline_cmode_zapi

=head2 delete

    NACL::C::StorageAggregatePlex->delete(
                                command_interface => $ci,
                                aggregate =>$aggr_name,
                                 plex     => $plex,
                                ...);
    $aggregateplex->delete(...);

(Class or Instance method)
This method is used to delete a plex.

Uses a CMode CLI and 7Mode CLI APISet.

Invokes 'aggr destroy' command for 7Mode CLI.

Invokes "aggr-destroy" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'plex' => $string >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI/ZAPI.

=item C<< 'aggregate' => $string >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI/ZAPI.

=item C<< 'force' => $string >>

  Applicable only for CMode CLI

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"storage aggregate plex delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( force
                    )
            ),
        },
        cli_cmd => 'storage aggregate plex delete',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::StorageAggregatePlex::delete',
    );

    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    $pkg_or_obj->_base_backend_cmode_cli(
        @args,
        api => 'storage_aggregate_plex_delete',
    );

    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %args;

    my $plex = $pkg_or_obj->get_plex_path(
        aggregate => $opts{aggregate},
        plex      => $opts{plex}
    );
    $args{'plex-name'} = $plex;

    $pkg_or_obj->_handle_boolean_options(
        api_options     => \%opts,
        boolean_options => ['force'],
        is_7m_cli       => 1,
    );
    if (defined $opts{'force'}) {
        $args{'force'} = $opts{'force'};
    } else {
        $args{'force'} = 1;
    }
    $apiset->aggr_destroy(%args);
    $Log->exit() if $may_exit;
} ## end sub _delete_7mode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    delete $opts{force} if (defined $opts{force});
    my $copy = [qw(aggregate plex)];
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api  => 'aggr-destroy',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 online

    NACL::C::StorageAggregatePlex->online(
                                command_interface => $ci,
                                aggregate =>$aggr_name,
                                 plex     => $plex,
                                ...);
    $aggregateplex->online(...);

(Class or Instance method)
This method is used to online a plex.

Uses a CMode CLI  and 7Mode CLI APISet.

Invokes 'aggr online' command for 7Mode CLI.

Invokes "aggr-online" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'plex' => $string >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI/ZAPI.

=item C<< 'aggregate' => $string >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI/ZAPI.

=item C<< 'force' => $string >>

  Applicable only for CMode CLI.

=item C<< 'force-online' => $string >>

  Applicable only for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"storage aggregate plex online" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub online {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( force force-online
                    )
            ),
        },
        cli_cmd => 'storage aggregate plex online',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_online_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_online_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_online_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::StorageAggregatePlex::online',
    );

    $Log->exit() if $may_exit;
} ## end sub online

sub _online_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    $pkg_or_obj->_base_backend_cmode_cli(
        @args,
        api => 'storage_aggregate_plex_online',
    );

    $Log->exit() if $may_exit;
} ## end sub _online_cmode_cli

sub _online_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $plex = $pkg_or_obj->get_plex_path(
        aggregate => $opts{aggregate},
        plex      => $opts{plex}
    );

    $apiset->aggr_online('plex-name' => $plex);
    $Log->exit() if $may_exit;
} ## end sub _online_7mode_cli

sub _online_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    delete $opts{force} if (defined $opts{force});

    my $copy = [qw(aggregate plex)];
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api  => 'aggr-online',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
} ## end sub _online_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        aggregate => {type => SCALAR},
        plex      => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

=head2 get_plex_path

    my $plex_path = NACL::C::StorageAggregatePlex->get_plex_path(
                                aggregate =>$aggr_name,
                                 plex     => $plex
                    );

    my $plex_path = $StorageAggregatePlexObject->get_plex_path();

    Note that, you can also perform,
    my $plex_path = $StorageAggregatePlexObject->plex_path();

(Class or Instance method)
This method returns the plex path in the format /<aggr_name>/<plex_name>
Eg: /aggr1/plex0

=over

=item Options

=over

=item C<< 'plex' => $string >>
Name of the plex. Eg: 'plex0'

=item C<< 'aggregate' => $string >>
Name of the aggregate

=back

=back

=cut

sub get_plex_path {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift @_;
    my %opts       = $pkg_or_obj->_common_validate_with(
        params               => \@_,
        no_command_interface => 1,
    );
    my $path
        = NACL::CS::StorageAggregatePlex->_get_plex_path($opts{aggregate},
        $opts{plex});
    $Log->debug("Plex path is: $path");
    $Log->exit() if $may_exit;
    return $path;
}
1;
