# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StorageAggregateReallocation Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::StorageAggregateReallocation

=head1 DESCRIPTION

C<NACL::C::StorageAggregateReallocation> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP StorageAggregateReallocation.  A related class is
L<NACL::CS::StorageAggregateReallocation|lib-NACL-CS-StorageAggregateReallocation-pm>, which represents
the past state of an ONTAP StorageAggregateReallocation.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 id

=cut

package NACL::C::StorageAggregateReallocation;

use strict;
use warnings;

use NACL::CS::StorageAggregateReallocation;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF UNDEF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $StorageAggregateReallocation = NACL::C::StorageAggregateReallocation->new(
                                   command_interface => $ci,
                                   'aggregate' => $aggregate,
								  );

Return a new StorageAggregateReallocation component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'aggregate',];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $StorageAggregateReallocation_obj  = NACL::C::StorageAggregateReallocation->find(command_interface => $ci, ...);
	my @StorageAggregateReallocation_objs = NACL::C::StorageAggregateReallocation->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my StorageAggregateReallocation_state = $StorageAggregateReallocation_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::StorageAggregateReallocation would return 
a NACL::CS::StorageAggregateReallocation object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 quiesce

    NACL::C::StorageAggregateReallocation->quiesce(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to quiesce reallocate job on aggregate.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'aggregate' => $string >>

Aggregate Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "storage aggregate reallocation quiesce" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub quiesce {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( aggregate
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'storage aggregate reallocation quiesce',
    );
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_quiesce_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        frontend => 'NACL::C::StorageAggregateReallocation::quiesce',
    );

    $Log->exit() if $may_exit;
}

sub _quiesce_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'storage_aggregate_reallocation_quiesce',);
    $Log->exit() if $may_exit;
}

=head2 restart

    NACL::C::StorageAggregateReallocation->restart(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to restart reallocate job on aggregate.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'ignore-checkpoint' => $string >>

Ignore Checkpoint
Applicable for CMode CLI

=item C<< 'aggregate' => $string >>

Aggregate Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "storage aggregate reallocation restart" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub restart {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( ignore-checkpoint aggregate
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'storage aggregate reallocation restart',
    );
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_restart_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        frontend => 'NACL::C::StorageAggregateReallocation::restart',
    );

    $Log->exit() if $may_exit;
}

sub _restart_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'storage_aggregate_reallocation_restart',);
    $Log->exit() if $may_exit;
}

=head2 schedule

    NACL::C::StorageAggregateReallocation->schedule(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to modify schedule of reallocate job on aggregate.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'cron' => $string >>

Cron Schedule
Applicable for CMode CLI

=item C<< 'del' => $string >>

Delete
Applicable for CMode CLI

=item C<< 'aggregate' => $string >>

Aggregate Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "storage aggregate reallocation schedule" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub schedule {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( cron del aggregate
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'storage aggregate reallocation schedule',
    );
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_schedule_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        frontend => 'NACL::C::StorageAggregateReallocation::schedule',
    );

    $Log->exit() if $may_exit;
}

sub _schedule_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'storage_aggregate_reallocation_schedule',);
    $Log->exit() if $may_exit;
}

=head2 start

    my $StorageAggregateReallocation_obj	= NACL::C::StorageAggregateReallocation->start(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to start reallocate job on aggregate.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'once' => $string >>

Once
Applicable for CMode CLI

=item C<< 'aggregate' => $string >>

Aggregate Name
Applicable for CMode CLI

=item C<< 'interval' => $string >>

Interval Schedule
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "storage aggregate reallocation start" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( once aggregate interval
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'storage aggregate reallocation start',
    );
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_start_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        frontend => 'NACL::C::StorageAggregateReallocation::start',
    );
    my @objs;
    my @aggregates = $opts{aggregate} // @{$opts{'aggregate-list'}};
    foreach my $aggr (@aggregates) {
        $DB::single=2;
        push @objs, $pkg->new(command_interface => $opts{command_interface}, aggregate => $aggr);
    }
    $Log->exit() if $may_exit;
    return wantarray ? @objs : $objs[0];
}

sub _start_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'storage_aggregate_reallocation_start',);
    $Log->exit() if $may_exit;
}

=head2 stop

    NACL::C::StorageAggregateReallocation->stop(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to stop reallocate job on aggregate.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'aggregate' => $string >>

Aggregate Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "storage aggregate reallocation stop" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub stop {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( aggregate
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'storage aggregate reallocation stop',
    );
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_stop_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        frontend => 'NACL::C::StorageAggregateReallocation::stop',
    );

    $Log->exit() if $may_exit;
}

sub _stop_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'storage_aggregate_reallocation_stop',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (aggregate => {type => SCALAR},);
}

1;
