# $Id: //depot/prod/test/nacldev/lib/NACL/C/StorageAggregateWafliron.pm#17 $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StorageAggregateWafliron Component Module
## @author Sumit.Mittal@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::StorageAggregateWafliron

=head1 DESCRIPTION

C<NACL::C::StorageAggregateWafliron> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to an ONTAP StorageAggregateWafliron on an aggregate.  A related class is
L<NACL::CS::StorageAggregateWafliron|lib-NACL-CS-StorageAggregateWafliron-pm>,
which represents the past state of an ONTAP StorageAggregateWafliron.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 aggregate

The name of the aggregate on which wafliron is done, used to identify it
uniquely.  (Aggregate names are assumed to be unique within a node or
within a cluster)

=cut

package NACL::C::StorageAggregateWafliron;

use strict;
use warnings;

use NACL::CS::StorageAggregateWafliron;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR BOOLEAN);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::C::Exceptions::StorageAggregateWafliron::IsFinishing qw(:try);

=head1 METHODS

=head2 new

  my $wafliron = NACL::C::StorageAggregateWafliron->new(command_interface=>$ci,
                                         aggregate =>$aggr_name);

Return a new StorageAggregateWafliron component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'aggregate',];

sub isa {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, $kind) = @_;
    my $isa = $pkg_or_obj->_build_isa(
        kind  => $kind,
        alias => 'NACL::C::Wafliron'
    );
    $Log->exit() if $may_exit;
    return $isa;
} ## end sub isa

=head2 start

  my $wafliron = NACL::C::StorageAggregateWafliron->start(command_interface=>$ci,
                                              aggregate=>$aggr_name,...);

(Class method) Request creation of a new wafliron element, and  return a new
wafliron component object that refers to it.  Does not wait for wafliron
to complete.

Uses a CMode-CLI, 7Mode-CLI APISet.
  In 7Mode-CLI the command is issued in 'diag' privilege level.
  Invokes 'aggr wafliron start' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional) Time in sec for the command to complete. Default timeout is 60 sec.

=item C<< aggregate=>$aggr_name >>

(Required) Name of aggregate which has to be waflironed.
  Applicable for CMode CLI, 7Mode CLI.
  Maps to:
    7M CLI : aggregate name (Programmatic name: "aggregate")

=item C<< include-mirrors => $boolean >>

(Optional) option to indicate if snapmirrors need to be included for checking
  Applicable only for CMode CLI

=item C<< ignore-member-start-errors => $boolean >>

(Optional) option to indicate whether to ignore start-errors on member
aggregates, only in case of striped aggregates
  Applicable only for CMode CLI

=item C<< optional-commit => $boolean >>

(Optional) option to indicate whether the changes to the filesystem need to be
committed to the disk or not.
  Applicable for CMode CLI, 7Mode CLI.
  Maps to:
    7M CLI : -o (Programmatic name: "optional_commit")

=item C<< previous-cp => $text >>

(Optional) Wafliron's previous checkpoint.  This library does not support previous-cp
interactive, non-interactive and semi-interactive mode. Please use NACL::MTask::Wafliron
for previous-cp related operations. See L<NACL::MTask::Wafliron|lib-NACL-MTask-Wafliron-pm>
  Applicable for CMode CLI, 7Mode CLI.
  Maps to:
    7M CLI : -prev_cp <value> (Programmatic name: "prev_cp")

=item C<< "volume-to-iron-first" => $string >>

  Applicable only for 7Mode CLI.

=item C<< "check-compressed-data-blocks" => $string >>

  Applicable only for 7Mode CLI.

=item C<< "ignore-mirrors" => $string >>

=item C<< 'backup' => $string >>

  Applicable only for CMode CLI

=item C<< 'use-backup' => $string >>

  Applicable only for CMode CLI

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            force => {type => BOOLEAN, optional => 1},
            _optional_scalars(
                qw(include-mirrors ignore-member-start-errors
                    optional-commit previous-cp volume-to-iron-first
                    check-compressed-data-blocks
                    backup use-backup
                    )
            )
        }
    );
    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_start_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_start_7mode_cli",
                interface => "CLI",
                set       => "7Mode|Nodescope",
            }
        ],
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub start

sub _start_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg               = shift;
    my %opts              = $pkg->_validate_backend_opts(@_);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw(aggregate include-mirrors
            ignore-member-start-errors optional-commit previous-cp
            backup use-backup
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'storage_aggregate_wafliron_start',
        api_opts => \%cmode_args
    );

    $apiset->storage_aggregate_wafliron_start(%cmode_args);

    $Log->exit() if $may_exit;
    return $pkg->new(
        aggregate         => $cmode_args{aggregate},
        command_interface => $command_interface
    );

} ## end sub _start_cmode_cli

sub _start_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg               = shift;
    my %opts              = $pkg->_validate_backend_opts(@_);
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw(aggregate optional-commit force
            volume-to-iron-first check-compressed-data-blocks)
    ];

    my %wafliron_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        map    => {'previous-cp' => 'prev_cp',},
        target => \%wafliron_args,
    );

    $apiset->aggr_wafliron_start(%wafliron_args);

    $Log->exit() if $may_exit;
    return $pkg->new(
        aggregate         => $wafliron_args{aggregate},
        command_interface => $command_interface
    );

} ## end sub _start_7mode_cli

=head2 stop

  NACL::C::StorageAggregateWafliron->stop(command_interface=>$ci,
                          aggregate=>$aggr_name,...);

 or

 $wafliron_object->stop(...)

(Class or instance method) The method stops an active wafliron on an aggregate.

Uses a CMode CLI or 7Mode CLI/ZAPI APISet.
  Invokes 'aggr wafliron stop' command for 7Mode CLI.
  Invokes 'aggr-wafliron-stop' command for 7Mode ZAPI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional) Time in sec for the command to complete. Default timeout is 60 sec.

=item C<< aggregate=>$aggr_name >>

(Required for class method, Not Applicable for instance method)
Name of aggregate in which the wafliron needs to be stopped.
  Applicable for 7Mode CLI/ZAPI.
  Maps to:
    7M CLI : aggregate
    7M CLI : aggregate name (Programmatic name: "aggregate")

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::StorageAggregateWafliron::IsFinishing>

This type of exception is thrown if we try to stop a wafliron operation that
is finishing.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub stop {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(params => \@args,);

    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_stop_cmode_cli",
                    interface => "CLI",
                    set       => "CMode",
                },
                {   method    => "_stop_7mode_cli",
                    interface => "CLI",
                    set       => "7Mode|Nodescope",
                },
                {   method    => '_wafliron_stop_7mode_zapi',
                    interface => 'ZAPI',
                    set       => '7Mode',
                },
            ],
            frontend => 'NACL::C::StorageAggregateWafliron::stop'
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::StorageAggregateWafliron::IsFinishing
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
} ## end sub stop

sub _stop_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [qw(aggregate extended_query )];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg_or_obj->_api_call_log_debug_opts(
        api      => 'storage_aggregate_wafliron_stop',
        api_opts => \%cmode_args
    );

    $apiset->storage_aggregate_wafliron_stop(%opts);

    $Log->exit() if $may_exit;

} ## end sub _stop_cmode_cli

sub _stop_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [qw(aggregate)];
    my %wafliron_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%wafliron_args,
    );

    $apiset->aggr_wafliron_stop(%wafliron_args);
    $Log->exit() if $may_exit;
} ## end sub _stop_7mode_cli

sub _wafliron_stop_7mode_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $aggr_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(aggregate)],
    );

    $apiset->aggr_wafliron_stop(%$aggr_args);
    $Log->exit() if $may_exit;
} ## end sub _wafliron_stop_7mode_zapi

=head2 review

 my $wafliron_review = NACL::C::StorageAggregateWafliron->review(command_interface=>$ci,
                          aggregate=>$aggr_name,...);

 or

 my $wafliron_review = $wafliron_object->review(...)

(Class or instance method) Show changes of Optional Commit wafliron to an
aggregate and  returns a component object that refers to review changes
Uses a CMode-CLI, Nodescope-CLI, 7Mode-CLI APISet.

Invokes "aggr wafliron review" api for 7Mode CLI

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional) Time in sec for the command to complete. Default timeout is 60 sec.

=item C<< aggregate=>$aggr_name >>

(Required for class method, Not Applicable for instance method)
Name of aggregate whose changes of optional Commit wafliron needs
to be reviewed.
  Applicable for CMode CLI, 7Mode CLI.
  Maps to:
    7M CLI : aggregate name (Programmatic name: "aggregate")

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub review {
    my $pkg_or_obj = shift;
    my %opts = $pkg_or_obj->_common_validate_with(params => \@_,);
    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_review_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_review_7mode_cli",
                interface => "CLI",
                set       => "7Mode|Nodescope",
            }
        ],
    );
} ## end sub review

sub _review_cmode_cli {

    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my $ci     = $opts{command_interface};
    my $apiset = $opts{apiset};

    my $allowed_cmode_options = [qw(aggregate extended_query )];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    my $response = $apiset->storage_aggregate_wafliron_review(%cmode_args);
    $Log->exit() if $may_exit;

    return $response;

} ## end sub _review_cmode_cli

sub _review_7mode_cli {

    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [qw(aggregate)];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    my $response = $apiset->aggr_wafliron_review(%opts);

    return $response;

} ## end sub _review_7mode_cli

=head2 commit

  NACL::C::StorageAggregateWafliron->commit(command_interface=>$ci,
                          aggregate=>$aggr_name,...);

 or

 $wafliron_object->commit(...)

(Class or instance method) write changes of Optional Commit wafliron to an
aggregate.

Uses a CMode-CLI, 7Mode-CLI APISet.
  In 7Mode-CLI the command is issued in 'diag' privilege level.
  Invokes 'aggr wafliron commit' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>


=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional) Time in sec for the command to complete. Default timeout is 60 sec.

=item C<< aggregate=>$aggr_name >>

(Required for class method, Not Applicable for instance method)
Name of aggregate whose changes of optional Commit wafliron needs
to be committed.
  Applicable for CMode CLI, 7Mode CLI.
  Maps to:
    7M CLI : aggregate name (Programmatic name: "aggregate")

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub commit {
    my $pkg_or_obj = shift;
    my %opts = $pkg_or_obj->_common_validate_with(params => \@_,);
    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_commit_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_commit_7mode_cli",
                interface => "CLI",
                set       => "7Mode|Nodescope",
            }
        ],
    );
} ## end sub commit

sub _commit_cmode_cli {

    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [qw(aggregate extended_query )];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    $apiset->storage_aggregate_wafliron_commit(%opts);

} ## end sub _commit_cmode_cli

sub _commit_7mode_cli {

    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [qw(aggregate)];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    $apiset->aggr_wafliron_commit(%opts);

} ## end sub _commit_7mode_cli

=head2 reject

  NACL::C::StorageAggregateWafliron->reject(command_interface=>$ci,
                          aggregate=>$aggr_name,...);

 or

 $wafliron_object->reject(...)

(Class or instance method) discard changes of Optional Commit wafliron to an
aggregate.

Uses a CMode-CLI, 7Mode-CLI APISet.
  In 7Mode-CLI the command is issued in 'diag' privilege level.
  Invokes 'aggr wafliron reject' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional) Time in sec for the command to complete. Default timeout is 60 sec.

=item C<< aggregate=>$aggr_name >>

(Required for class method, Not Applicable for instance method)
Name of aggregate whose changes of optional Commit wafliron needs
to be rejected.
  Applicable for CMode CLI, 7Mode CLI.
  Maps to:
    7M CLI : aggregate name (Programmatic name: "aggregate")

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub reject {
    my $pkg_or_obj = shift;
    my %opts = $pkg_or_obj->_common_validate_with(params => \@_,);
    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_reject_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_reject_7mode_cli",
                interface => "CLI",
                set       => "7Mode|Nodescope",
            }
        ],
    );
} ## end sub reject

sub _reject_cmode_cli {

    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [qw(aggregate extended_query )];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    $apiset->storage_aggregate_wafliron_reject(%opts);

} ## end sub _reject_cmode_cli

sub _reject_7mode_cli {

    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [qw(aggregate)];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    $apiset->aggr_wafliron_reject(%opts);

} ## end sub _reject_7mode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (aggregate => {type => SCALAR});
}

1;
