#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StorageArrayPort Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::StorageArrayPort

=head1 DESCRIPTION

C<NACL::C::StorageArrayPort> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP StorageArrayPort.  A related class is
L<NACL::CS::StorageArrayPort|lib-NACL-CS-StorageArrayPort-pm>, which represents
the past state of an ONTAP StorageArrayPort.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 wwnn

=head2 wwpn

=head2 name

=cut

package NACL::C::StorageArrayPort;

use strict;
use warnings;

use NACL::CS::StorageArrayPort;
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $StorageArrayPort = NACL::C::StorageArrayPort->new(command_interface=>$ci,
                                   wwnn =>$wwnn,
                                   wwpn =>$wwpn,
                                   name =>$name,
                        );

Return a new StorageArrayPort component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'wwnn',
    scalar => 'wwpn',
    scalar => 'name',
];

=head2 find

    my $StorageArrayPort  = NACL::C::StorageArrayPort->find(command_interface => $ci, ...);
    my @StorageArrayPorts = NACL::C::StorageArrayPort->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $StorageArrayPort_state = $NACL::C::StorageArrayPort->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::StorageArrayPort would return
a NACL::CS::StorageArrayPort object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 remove

   NACL::C::StorageArrayPort->remove(
                                command_interface => $ci,
                                wwnn => $wwnn,
                                wwpn => $wwpn,
                                name => $name,
                                  ...);
                        or

    $StorageArrayPort_object->remove(...);

(Class or instance method)
This method is used to remove a port record from an array profile..

Uses a CMode or 7Mode CLI APISet.
  Invokes 'storage array remove port' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "wwnn" => $wwnn >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI

=item C<< "wwpn" => $wwpn >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI, 7Mode CLI.
  Maps to:
    7M CLI : -p <value> (Programmatic name: wwpn-num)

=item C<< "name" => $name >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI, 7Mode CLI.
  Maps to:
    7M CLI : array name (Programmatic name: "array-name")

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"storage array port remove" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub remove {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {_optional_scalars(qw( wwnn )),},
        api             => 'storage_array_port_remove',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_remove_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_remove_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
        frontend => 'NACL::C::StorageArrayPort::remove',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub remove

sub _remove_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'storage_array_port_remove',);
    $Log->exit() if $may_exit;
} ## end sub _remove_cmode_cli

sub _remove_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    my %valid_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            name => 'array-name',
            wwpn => 'wwpn-num'
        },
        target => \%valid_args,
    );

    $apiset->storage_array_remove_port(%valid_args);
    $Log->exit() if $may_exit;
} ## end sub _remove_7mode_cli

=head2 modify

    NACL::C::StorageArrayPort->modify(
                                command_interface => $ci,
                                'wwnn' => $wwnn,
                                'wwpn' => $wwpn,
                                'name' => $name,
                                ...);

                                 or

    $StorageArrayPort_obj->modify(...);

(Class or instance method)
This method is used to make changes to a target port record..

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'wwnn' => $wwnn >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI.

=item C<< 'wwpn' => $wwpn >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI.

=item C<< 'name' => $name >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI.

=item C<< 'max-queue-depth' => $string >>

  Applicable only for CMode CLI.

=item C<< 'lun-queue-depth' => $string >>

  Applicable only for CMode CLI.

=item C<< 'id' => $string >>

  Applicable only for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'utilization-policy' => $string >>

  Applicable only for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)
  Applicable only for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"storage array port modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( max-queue-depth lun-queue-depth id

                    utilization-policy
                    )
            ),
        },
        api => 'storage_array_port_modify',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::StorageArrayPort::modify',
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'storage_array_port_modify',);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'wwnn' => {type => SCALAR},
        'wwpn' => {type => SCALAR},
        'name' => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
