
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StorageDiskOption Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::StorageDiskOption

=head1 DESCRIPTION

C<NACL::C::StorageDiskOption> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP StorageDiskOption.  A related class is
L<NACL::CS::StorageDiskOption|lib-NACL-CS-StorageDiskOption-pm>, which represents
the past state of an ONTAP StorageDiskOption.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 node

=cut

package NACL::C::StorageDiskOption;

use strict;
use warnings;

use NACL::CS::StorageDiskOption;
use NACL::ComponentUtils qw (_optional_scalars);

use NACL::C::Options;
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $StorageDiskOption = NACL::C::StorageDiskOption->new(command_interface=>$ci,
                                   node =>$node,
                                  );

Return a new StorageDiskOption component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'node',];

=head2 find

    my $StorageDiskOption  = NACL::C::StorageDiskOption->find(command_interface => $ci, ...);
    my @StorageDiskOptions = NACL::C::StorageDiskOption->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $StorageDiskOption_state = $NACL::C::StorageDiskOption->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::StorageDiskOption would return
a NACL::CS::StorageDiskOption object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 modify

   NACL::C::StorageDiskOption->modify(
                                command_interface => $ci,
                                 node => $node,
                                  ...);

                                 or

    $StorageDiskOption_object->modify(...);

(Class or instance method)
This method is used to modify disk options.

Uses a CMode CLI APISet.
  Invokes 'options' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $node >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI.

=item C<< "bkg-firmware-update" => $string >>

  Applicable only for CMode CLI.

=item C<< "autocopy" => $string >>

  Applicable only for CMode CLI.

=item C<< "autoassign" => $string >>

  Applicable only for CMode CLI.

=item C<< 'autoassign-shelf' => $string >>

  Applicable only for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< 'autoassign-policy' => $string >>

Auto Assignment Policy possible value(s) are, default,bay,shelf,stack
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"storage disk option modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( bkg-firmware-update autocopy autoassign

                    autoassign-shelf

                    autoassign-policy
                    )
            ),
        },
        api => 'storage_disk_option_modify',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_modify_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
        frontend => 'NACL::C::StorageDiskOption::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'storage_disk_option_modify',);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    if (defined $opts{autoassign}) {
        $apiset->options('disk.auto_assign' => lc $opts{autoassign});
    } else {
        $Log->trace('Command is not executed');
    }

    $Log->exit() if $may_exit;
} ## end sub _modify_7mode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('node' => {type => SCALAR, optional => 1},);
}

1;
