# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StorageDiskPartition Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::StorageDiskPartition

=head1 DESCRIPTION

C<NACL::C::StorageDiskPartition> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP StorageDiskPartition.  A related class is
L<NACL::CS::StorageDiskPartition|lib-NACL-CS-StorageDiskPartition-pm>, which represents
the past state of an ONTAP StorageDiskPartition.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 partition

=cut

package NACL::C::StorageDiskPartition;

use strict;
use warnings;

use NACL::CS::StorageDiskPartition;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $StorageDiskPartition = NACL::C::StorageDiskPartition->new(
                                   command_interface => $ci,
                                   'partition' => $partition,
								  );

Return a new StorageDiskPartition component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'partition',];

=head2 find

    my $StorageDiskPartition_obj  = NACL::C::StorageDiskPartition->find(command_interface => $ci, ...);
	my @StorageDiskPartition_objs = NACL::C::StorageDiskPartition->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my StorageDiskPartition_state = $StorageDiskPartition_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::StorageDiskPartition would return 
a NACL::CS::StorageDiskPartition object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 show_node_level

    my $show_node_level_obj = NACL::C::StorageDiskPartition->show_node_level(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_node_level_objs = NACL::C::StorageDiskPartition->show_node_level(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_node_level_obj = $StorageDiskPartition_obj->show_node_level(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-StorageDiskPartitionShowNodeLevel-pm/ATTRIBUTES>
of NACL::CS::StorageDiskPartitionShowNodeLevel as input. 
This method returns objects of the type L<NACL::CS::StorageDiskPartitionShowNodeLevel|lib-NACL-CS-StorageDiskPartitionShowNodeLevel-pm>.

=cut

sub show_node_level {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::StorageDiskPartitionShowNodeLevel;
    my @state_objs
        = NACL::CS::StorageDiskPartitionShowNodeLevel->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 replace

    NACL::C::StorageDiskPartition->replace(
                                command_interface => $ci,
	                            'partition' => $partition,
                                ...);
	                             
								 or

    $StorageDiskPartition_obj->replace(...);

(Class or instance method)
This method is used to initiate or stop replacing a file-system partition.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'partition' => $partition >>

(Required for class method, Not Applicable for instance method)
Partition Name
Applicable for CMode CLI

=item C<< 'allow-same-carrier' => $string >>

Allow same RAID group within carrier
Applicable for CMode CLI

=item C<< 'allow-mixing' => $string >>

Allow mixing of partitions of different RPM or pool
Applicable for CMode CLI

=item C<< 'action' => $string >>

Action possible value(s) are, start , stop
Applicable for CMode CLI

=item C<< 'replacement' => $string >>

Replacement
Applicable for CMode CLI

=item C<< 'reason' => $string >>

Reason for replacement
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "storage disk partition replace" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub replace {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( allow-same-carrier allow-mixing action replacement

                    reason
                    )
            ),
        },
        api => 'storage_disk_partition_replace',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_replace_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::StorageDiskPartition::replace',
    );

    $Log->exit() if $may_exit;
}

sub _replace_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'storage_disk_partition_replace',);
    $Log->exit() if $may_exit;
}

=head2 assign

    NACL::C::StorageDiskPartition->assign(
                                command_interface => $ci,
	                            'partition' => $partition,
                                ...);
	                             
								 or

    $StorageDiskPartition_obj->assign(...);

(Class or instance method)
This method is used to assigns ownership of a partition to a system.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'partition' => $partition >>

(Required for class method, Not Applicable for instance method)
Partition Name
Applicable for CMode CLI

=item C<< 'owner' => $string >>

Owner Name
Applicable for CMode CLI

=item C<< 'pool' => $string >>

Pool
Applicable for CMode CLI

=item C<< 'force' => $string >>

Force Flag
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "storage disk partition assign" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub assign {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( owner pool force
                    )
            ),
        },
        api => 'storage_disk_partition_assign',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_assign_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::StorageDiskPartition::assign',
    );

    $Log->exit() if $may_exit;
}

sub _assign_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'storage_disk_partition_assign',);
    $Log->exit() if $may_exit;
}

=head2 removeowner

    NACL::C::StorageDiskPartition->removeowner(
                                command_interface => $ci,
	                            'partition' => $partition,
                                ...);
	                             
								 or

    $StorageDiskPartition_obj->removeowner(...);

(Class or instance method)
This method is used to removes partition ownership.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'partition' => $partition >>

(Required for class method, Not Applicable for instance method)
Partition Name
Applicable for CMode CLI

=item C<< 'force' => $string >>

Force the Ownership Removal
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "storage disk partition removeowner" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub removeowner {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( force
                    )
            ),
        },
        api => 'storage_disk_partition_removeowner',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_removeowner_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::StorageDiskPartition::removeowner',
    );

    $Log->exit() if $may_exit;
}

sub _removeowner_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'storage_disk_partition_removeowner',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('partition' => {type => SCALAR},);
}

1;
