# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StoragePool Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::StoragePool

=head1 DESCRIPTION

C<NACL::C::StoragePool> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP StoragePool.  A related class is
L<NACL::CS::StoragePool|lib-NACL-CS-StoragePool-pm>, which represents
the past state of an ONTAP StoragePool.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 storage_pool

=cut

package NACL::C::StoragePool;

use strict;
use warnings;

use NACL::CS::StoragePool;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use NACL::C::Exceptions::StoragePool::AlreadyExists qw(:try);
use NACL::C::Exceptions::StoragePool::DoesNotExist qw(:try);

=head1 METHODS

=head2 new

  my $StoragePool = NACL::C::StoragePool->new(
                                   command_interface => $ci,
                                   'storage-pool' => $storage_pool,
								  );

Return a new StoragePool component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'storage_pool',];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $StoragePool_obj  = NACL::C::StoragePool->find(command_interface => $ci, ...);
	my @StoragePool_objs = NACL::C::StoragePool->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one 
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my StoragePool_state = $StoragePool_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::StoragePool would return
a NACL::CS::StoragePool object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 create

    my $StoragePool_obj	= NACL::C::StoragePool->create(
                                command_interface => $ci,
	                            'storage-pool' => $storage_pool,
                                ...);

(Class method)
This method is used to create a new storage pool.
Uses CMode CLI/ZAPI .
Invokes  storage-pool-create  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'storage-pool' => $storage_pool >>

(Required)
Storage Pool Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'allocation-unit-depth' => $string >>

Allocation Unit Depth
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'simulate' => $string >>

Simulate Storage Pool Creation
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'disk-list' => [ $disk-1, $disk-2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified 
as an arrayref containing that single value) 
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "storage pool create" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::StoragePool::AlreadyExists>

This type of exception gets thrown if an attempt is made to create a
storage pool that already exists.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'disk-list' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( allocation-unit-depth simulate
                    )
            ),
        },
        api => 'storage_pool_create',
    );
    try {
	  $pkg->call_on_apiset(
		%opts,
		choices => [
			{   method    => '_create_cmode_cli',
			    interface => 'CLI',
                            set       => 'CMode',
			    zapi_type => 'none',
			},
			{   method    => '_create_cmode_zapi_non_iter',
			    interface => 'ZAPI',
			    set       => 'CMode',
			    zapi_type => 'non-iter',
			},
		],
		frontend => 'NACL::C::StoragePool::create',
	    );
	}
	catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::StoragePool::AlreadyExists
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        storage_pool      => $opts{'storage-pool'},
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'storage_pool_create',);
    $Log->exit() if $may_exit;
}

sub _create_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(disk-list simulate storage-pool)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'storage-pool-create',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

=head2 add

    NACL::C::StoragePool->add(
                                command_interface => $ci,
                                    'storage-pool' => $storage_pool,
                                ...);
                 or

    $StoragePool_obj->add(...);

(Class or instance method)
This method is used to add disks to a storage pool.
Uses CMode CLI/ZAPI .
Invokes  storage-pool-add  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'storage-pool' => $storage_pool >>

(Required for class method, Not Applicable for instance method) 
Storage Pool Name
Applicable for CMode CLI/ZAPI, non-iter

=item C<< 'quiet' => $string >>

Confirmations off
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'auto-grow-aggregates' => $string >>

Automatically Grow Existing Aggregates possible value(s) are, true,false
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'new-spare-capacity' => $string >>

Spare Capacity of the Pool after Addition
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'spare-capacity' => $string >>

Spare Capacity of the Pool before Addition
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'new-available-capacity' => $string >>

Available Capacity of the Pool after Addition
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'available-capacity' => $string >>

Available Capacity of the Pool before Addition
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'simulate' => $string >>

Simulate Storage Pool Addition
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'foreground' => $string >>

Run Complete Job in Foreground
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'disk-list' => [ $disk-1, $disk-2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified 
as an arrayref containing that single value) 
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'aggregate-names' => [ $aggregate-name1, $aggregate-name2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'aggregate-sizes' => [ $aggregate-size1, $aggregate-size2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'aggregate-new-sizes' => [ $aggregate-new-size1, $aggregate-new-size2, ... ] >>
(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "storage pool add" command in ONTAP

=item C<< other_options >>

This method takes other common options described in
L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'disk-list'           => {type => ARRAYREF, optional => 1},
            'aggregate-names'     => {type => ARRAYREF, optional => 1},
            'aggregate-sizes'     => {type => ARRAYREF, optional => 1},
            'aggregate-new-sizes' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( quiet auto-grow-aggregates new-spare-capacity
                    new-available-capacity spare-capacity available-capacity 
                    simulate foreground
                    )
            ),
        },
        api => 'storage_pool_add',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_add_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_add_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_foreground_zapi_check',
            },
        ],
        frontend => 'NACL::C::StoragePool::add',
    );

    $Log->exit() if $may_exit;
}

sub _add_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
        my %additional_args;
    my $prompt_answers
        = [
		'Are you sure you want to continue with this operation.*' => "y",
        ];

    $additional_args{'connectrec-match_table'} = $prompt_answers;
    $opts{additional_args} = \%additional_args;
    $pkg_or_obj->_base_backend_cmode_cli(%opts, api => 'storage_pool_add',);
    $Log->exit() if $may_exit;
}

sub _add_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $copy = [
        qw(auto-grow-aggregates disk-list new-available-capacity available-capacity
            new-spare-capacity spare-capacity aggregate-names simulate aggregate-sizes aggregate-new-sizes storage-pool)
    ];
    my %map = ();
    my $command_interface = $opts{'command_interface'};
    if (!$command_interface->has_uichange(
           uichange => 'storage-pool-spare-ui-update')) {
           %map =
                   ( 'spare-capacity'     => 'available-capacity',
                     'new-spare-capacity' => 'new-available-capacity',
                   );
        }
        $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api  => 'storage-pool-add',
        copy => $copy,
        map  => \%map,
    );
    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::StoragePool->delete(
                                command_interface => $ci,
	                            'storage-pool' => $storage_pool,
                                ...);
	                             
								 or

    $StoragePool_obj->delete(...);

(Class or instance method)
This method is used to delete an existing storage pool.
Uses CMode CLI/ZAPI .
Invokes  storage-pool-delete  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method) See 
L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'storage-pool' => $storage_pool >>

(Required for class method, Not Applicable for instance method)
Storage Pool Name Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'force' => $string >>

Force Delete
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'quiet' => $string >>

Quite Delete
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "storage pool delete" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over


=item C<NACL::C::Exceptions::StoragePool::DoesNotExist>

This type of exception gets thrown if an attempt is made to delete
storage pool which does not exists.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from underlying API layer and the details of those exceptions are described in the L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( force quiet
                    )
            ),
        },
        api => 'storage_pool_delete',
    );
    try {
          $pkg_or_obj->call_on_apiset(
		%opts,
		choices => [
			{   method    => '_delete_cmode_cli',
			    interface => 'CLI',
			    set       => 'CMode',
			    zapi_type => 'none',
			},
			{   method    => '_delete_cmode_zapi_non_iter',
			    interface => 'ZAPI',
			    set       => 'CMode',
			    zapi_type => 'non-iter',
			},
		],
		frontend => 'NACL::C::StoragePool::delete',
	    );
	}
	catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::StoragePool::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api => 'storage_pool_delete',);
    $Log->exit() if $may_exit;
}

sub _delete_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $copy = [qw(force storage-pool)];
    my $command_interface = $opts{'command_interface'};
    my %map = ();
    if (!$command_interface->has_uichange(
           uichange => 'storage-pool-ui-update')) {
            %map =
                   ('force' => 'quiet');
        }
     $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api  => 'storage-pool-delete',
        copy => $copy,
        map  => \%map,
    );
    $Log->exit() if $may_exit;
}

=head2 reassign

    NACL::C::StoragePool->reassign(
                                command_interface => $ci,
	                            'storage-pool' => $storage_pool,
                                ...);
	                             
								 or

    $StoragePool_obj->reassign(...);

(Class or instance method)
This method is used to reassign capacity from one node to another node in storage pool.
Uses CMode CLI/ZAPI .
Invokes  storage-pool-reassign  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method) See 
L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'storage-pool' => $storage_pool >>

(Required for class method, Not Applicable for instance method)
 Storage Pool Name 
 Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'from-node' => $string >>

Reassign Spare Capacity from This Node possible value(s) are, <nodename>,local
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'to-node' => $string >>

Reassign Spare Capacity to This Node possible value(s) are, <nodename>,local
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'allocation-units' => $string >>

Allocation Units ranges from 1 to 4
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "storage pool reassign" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from underlying API layer and the details of those exceptions are described in the L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub reassign {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( from-node to-node allocation-units
                    )
            ),
        },
        api => 'storage_pool_reassign',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_reassign_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_reassign_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::StoragePool::reassign',
    );

    $Log->exit() if $may_exit;
}

sub _reassign_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'storage_pool_reassign',);
    $Log->exit() if $may_exit;
}

sub _reassign_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(from-node to-node allocation-units storage-pool)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'storage-pool-reassign',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

=head2 show_aggregate

    my $show_aggregate_obj = NACL::C::StoragePool->show_aggregate(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_aggregate_objs = NACL::C::StoragePool->show_aggregate(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or

   my $show_aggregate_obj = $StoragePool_obj->show_aggregate();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-StoragePoolShowAggregate-pm/ATTRIBUTES>
of NACL::CS::StoragePoolShowAggregate as input.
This method returns objects of the type
L<NACL::CS::StoragePoolShowAggregate|lib-NACL-CS-StoragePoolShowAggregate-pm>.

=cut

sub show_aggregate {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::StoragePoolShowAggregate;
    my @state_objs = NACL::CS::StoragePoolShowAggregate->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 show_disks

    my $show_disks_obj = NACL::C::StoragePool->show_disks(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or

    my @show_disks_objs = NACL::C::StoragePool->show_disks(command_interface => $ci,
                          [ filter => { %filter } ,]
                          [ requested_fields => [ @req_fields ]);

                    or

   my $show_disks_obj = $StoragePool_obj->show_disks();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) and has all of the other properties of fetch methods (callable in scalar or list context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-StoragePoolShowDisks-pm/ATTRIBUTES>
of NACL::CS::StoragePoolShowDisks as input.
This method returns objects of the type L<NACL::CS::StoragePoolShowDisks|lib-NACL-CS-StoragePoolShowDisks-pm>.

=cut

sub show_disks {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::StoragePoolShowDisks;
    my @state_objs = NACL::CS::StoragePoolShowDisks->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0]; }

=head2 show_spare_capacity

    my $show_spare_capacity_obj = NACL::C::StoragePool->show_spare_capacity(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or

    my @show_spare_capacity_objs = NACL::C::StoragePool->show_spare_capacity(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or

   my $show_spare_capacity_obj = $StoragePool_obj->show_spare_capacity(); 

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) and has all of the other properties of fetch methods (callable in scalar or list context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-StoragePoolShowSpareCapacity-pm/ATTRIBUTES>
of NACL::CS::StoragePoolShowSpareCapacity as input.
This method returns objects of the type
L<NACL::CS::StoragePoolShowSpareCapacity|lib-NACL-CS-StoragePoolShowSpareCapacity-pm>.

=cut

sub show_spare_capacity {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my @state_objs;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    if($opts{command_interface}
       ->has_uichange(uichange => 'storage-pool-spare-ui-update')) {
        require NACL::CS::StoragePoolShowAvailableCapacity;
        @state_objs = NACL::CS::StoragePoolShowAvailableCapacity->fetch(%opts);
    } else {
        require NACL::CS::StoragePoolShowSpareCapacity;
        @state_objs = NACL::CS::StoragePoolShowSpareCapacity->fetch(%opts);
    }
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('storage-pool' => {type => SCALAR},);
}

1;
