# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StorageShelfHardware Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::StorageShelfHardware

=head1 DESCRIPTION

C<NACL::C::StorageShelfHardware> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.
Unlike other components this package does not model any element/service
but is present to provide component-level support for functionality.
Since it does not model any element/service there is no ComponentState
module for this component. For the same reason, 'find' and 'state' will
not work for this component.
Instead NACL::CS::StorageShelfHardware->fetch() should be used.
(See L<fetch|lib-NACL-CS-ComponentState-pm/fetch> for a full description)

=cut

=head1 ATTRIBUTES

None.

=cut

package NACL::C::StorageShelfHardware;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $StorageShelfHardware = NACL::C::StorageShelfHardware->new(
                                   command_interface => $ci,
								  );

Return a new StorageShelfHardware component object with the given attributes.

=cut

use Class::MethodMaker [];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 identify

    NACL::C::StorageShelfHardware->identify(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to identify shelf by turning on the identification LED.
Uses CMode CLI/ZAPI .
Invokes  storage-shelf-identify  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'shelf' => $string >>

Shelf Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'is-turn-led-on' => $string >>


true turns identification LED on, false turns it off possible value(s) are, true,false
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'gubed' => $string >>


true will allow debug messages to be shown on the console possible value(s) are, true,false
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "storage shelf hardware identify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub identify {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( shelf is-turn-led-on gubed
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'storage shelf hardware identify',
    );
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_identify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_identify_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::StorageShelfHardware::identify',
    );

    $Log->exit() if $may_exit;
}

sub _identify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'storage_shelf_hardware_identify',);
    $Log->exit() if $may_exit;
}

sub _identify_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(shelf is-turn-led-on)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'storage-shelf-identify',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

__PACKAGE__->_load_event_class();

1;
