# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#  Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary Snapmirror Component Module
## @author Neethu.George@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::Snapmirror

=head1 DESCRIPTION

C<NACL::C::SysManager::Snapmirror> is a derived class of
L<NACL::C::Component::SysManager|lib-NACL-C-Component-SysManager-pm>.  It represents access
to an ONTAP Snapmirror present on the System Manger GUI Screen.  A related class is
L<NACL::CS::SysManager::Snapmirror|lib-NACL-CS-Snapmirror-pm>, which represents
the State Info of an ONTAP snapmirror present on the System Manager GUI Screen.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 targetfiler

The name of the Target Filer.

=head2 source

The Source name

=head2 destination

The Destination name

=head2 vserver

(Required for CMode, Not Applicable for 7Mode) The name of the vserver
in which the snapmirror relation is present.

=cut

package NACL::C::SysManager::Snapmirror;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

use NACL::CS::SysManager::Snapmirror;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component::SysManager';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

	my $snapmirror = 
		NACL::C::SysManager::Snapmirror->new(command_interface => $ci,
											 targetfiler       => $targetfiler,
											 source            => $source_name,
											 destination       => $destination_name,
											 vserver           => $vserver);

Return a new Snapmirror component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'targetfiler',
    scalar => 'source',
    scalar => 'destination',
    scalar => 'vserver',
];

=head2 create

	my $snapmirror  = 
		NACL::C::SysManager::Snapmirror->create(command_interface => $ci,
												"targetfiler"     => $filer,
												"vserver"         => $vserver,
												"source"          => $srcvolume,
												"destination"     => $dstvolume,
												type              => $type, ...);

(Class method) Request creation of a new snapmirror element, and  return a new
snapmirror component object that refers to it.
Uses a CMode-GUI or CMode-GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "source_cluster" => $string >>

(Required) Name of filer where the source volume resides

=item C<< "source" => $string >>

(Required) Name of the source volume for which the snapmirror relation is created

=item C<< type => $string >>

(Required) The type of snapmirror relation either by creating new volume or
using the existing one

=item C<< "volume-option" => $string >>

(Optional) Option for creating new volume or use existing volume 

=item C<< "destination" => $string >>

(Required) Name of the destination volume in snapmirror relation

=item C<< vserver => $string >>

(Required) Name of the vserver .Applicable only in Cmode.

=item C<< source-vserver => $string >>

(Optional) Name of the source vserver .Applicable only in Cmode.

=item C<< destination-vserver => $string >>

(Optional) Name of the destination vserver .Applicable only in Cmode.

=item C<< username => $string >>

(Optional) Username of the filer

=item C<< password => $string >>

(Optional) Password of the filer

(Optional) Name of the aggregate in which destination volume resides.
Needed param when  value of type param  is "existing". 

=item C<< schedule => $string >>

(Optional) Schedule of the snapmirror relation

=item C<< aggregate => $string >>

=item C<< size => $string >>

(Optional) Size of the new volume to be created incase the type param is 
"new"

=item C<< unit => $string >>

(Optional) Unit of size of the new volume to be created incase the type param is 
"new"

=item C<< space-guarantee => $string >>

(Optional)Space Guarantee on the Volume to be created.

=item C<< initialize-mirror=>$string >>

(Optional) To initialize mirror after creatign snapmirror relation

=item C<< schedule-type =>$string >>

(Optional) Type of schedule.

=item C<<  minute = $string >>

(Optional) Minute option for the Schedule

=item C<< recurring-schedule-time = $string>>

(Optional) Recurring the Schedule time

=item C<<  weekday = $string >>

(Optional) Enable Weekday option for Schedule

=item C<< data-transfer=>$string >>

(Optional) Data transfer option

=item C<< limit =>$string >>

(Optional) Limit option for the Schedule

=item C<< qtree =>$string >>

(Optional) Name of the destination qtree in case the destination is 
qtree


=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    $pkg->_frontend_log_debug_opts(@args);

    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            type => { type => SCALAR },
            _optional_scalars(
                qw (username password source-volume volume-option
                    aggregate destination-volume size unit space-guarantee schedule-type
                    schedule weekdaysun weekdaymon weekdaytue weekdaywed weekdaythu weekdayfri
                    weekdaysat recurring-schedule-time minutes hours initialize-mirror data-transfer
                    limitdatatransferrate qtree source-vserver destination-vserver schedule-name)
            )
        }
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method => '_create_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_create_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub create

####################### GUI 7Mode Create ###############################
sub _create_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $snapmirror_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "type"                    => "-mirrortype",
            "targetfiler"             => "-targetfiler",
            "username"                => "-targetusername",
            "password"                => "-targetpassword",
            "source"                  => "-sourcevol",
            "volume-option"           => "-create_newvolume",
            "aggregate"               => "-aggrname",
            "destination"             => "-destvol",
            "size"                    => "-size",
            "unit"                    => "-sizeunit",
            "space-guarantee"         => "-spaceguarantee",
            "schedule-type"           => "-scheduletype",
            "schedule"                => "-schedule",
            "weekdaysun"              => "-weekdaysun",
            "weekdaymon"              => "-weekdaymon",
            "weekdaytue"              => "-weekdaytue",
            "weekdaywed"              => "-weekdaywed",
            "weekdaythu"              => "-weekdaythu",
            "weekdayfri"              => "-weekdayfri",
            "weekdaysat"              => "-weekdaysat",
            "recurring-schedule-time" => "-recurringschedule",
            "minutes"                 => "-minutes",
            "hours"                   => "-hours",
            "initialize-mirror"       => "-initializemirror",
            "data-transfer"           => "-datatransferrate",
            "limitdatatransferrate"   => "-limitdatatransferrate",
            "qtree"                   => "-qtreename"
        }
    );

    $apiset->snapmirror_create(%$snapmirror_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        "targetfiler"     => $snapmirror_args->{-targetfiler},
        "source"          => $snapmirror_args->{-sourcevol},
        "destination"     => $snapmirror_args->{-destvol},
        command_interface => $command_interface
    );
} ## end sub _create_7mode_gui

####################### GUI CMode Create ###############################
sub _create_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $snapmirror_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "type"                => "-mirrortype",
            "targetfiler"         => "-srcfiler",
            "vserver"             => "-vservername",
            "username"            => "-srcusername",
            "password"            => "-srcpassword",
            "source-vserver"      => "-sourcevsr",
            "source"              => "-sourcevol",
            "destination-vserver" => "-dstvserver",
            "volume-option"       => "-optionvol",
            "destination"         => "-createvolname",
            "aggregate"           => "-aggrname",
            "size"                => "-volsize",
            "unit"                => "-volsizeunit",
            "space-guarantee"     => "-thinprov",
            "schedule-type"       => "-schedule",
            "schedule-name"       => "-schedulename",
            "initialize-mirror"   => "-initialize"
        }
    );

    $apiset->snapmirror_create(%$snapmirror_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        "targetfiler"     => $snapmirror_args->{-srcfiler},
        vserver           => $snapmirror_args->{-vservername},
        "source"          => $snapmirror_args->{-sourcevol},
        "destination"     => $snapmirror_args->{-createvolname},
        command_interface => $command_interface
    );
} ## end sub _create_cmode_gui

=head2 edit
  
	NACL::C::SysManager::Snapmirror->edit("vserver"          => $vserver,
										  "source"      => $src,
										  "destination" => $dst);
  
						(or)

	$snapmirror_object->edit(...);

(Class or instance method) The method edits snapmirror relation
Uses a CMode-GUI or 7mode-GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "destination"=>$string >>

(Optional for class method, Not Applicable for instance method)
Name of the destination in the snapmirror relation

=item C<< "source"=>$string >>

(Optional for class method, Not Applicable for instance method)
Name of the source in the snapmirror relation

=item C<< type => $string >>

(Required) The type of snapmirror relation either by creating new volume or
using the existing one

=item C<< "volume-option" => $string >>

(Optional) Option for creating new volume or use existing volume 

=item C<< "destination_volume" => $string >>

(Required) Name of the destination volume in snapmirror relation

=item C<< vserver => $string >>

(Required) Name of the vserver .Applicable only in Cmode.

=item C<< source-vserver => $string >>

(Optional) Name of the source vserver .Applicable only in Cmode.

=item C<< destination-vserver => $string >>

(Optional) Name of the destination vserver .Applicable only in Cmode.

=item C<< username => $string >>

(Optional) Username of the filer

=item C<< password => $string >>

(Optional) Password of the filer

(Optional) Name of the aggregate in which destination volume resides.
Needed param when  value of type param  is "existing". 

=item C<< schedule => $string >>

(Optional) Schedule of the snapmirror relation

=item C<< aggregate => $string >>

=item C<< size => $string >>

(Optional) Size of the new volume to be created incase the type param is 
"new"

=item C<< unit => $string >>

(Optional) Unit of size of the new volume to be created incase the type param is 
"new"

=item C<< space-guarantee => $string >>

(Optional)Space Guarantee on the Volume to be created.

=item C<< initialize-mirror=>$string >>

(Optional) To initialize mirror after creatign snapmirror relation

=item C<< schedule-type =>$string >>

(Optional) Type of schedule.

=item C<<  minute = $string >>

(Optional) Minute option for the Schedule

=item C<< recurring-schedule-time = $string>>

(Optional) Recurring the Schedule time

=item C<<  weekday = $string >>

(Optional) Enable Weekday option for Schedule

=item C<< data-transfer=>$string >>

(Optional) Data transfer option

=item C<< limit =>$string >>

(Optional) Limit option for the Schedule

=item C<< qtree =>$string >>

(Optional) Name of the destination qtree in case the destination is 
qtree

=back

=back

=cut

sub edit {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            #source        => { type => SCALAR },
            #"destination" => { type => SCALAR },
            _optional_scalars(
                qw (schedule schedule-name schedule-type schedule-settings recurring-days
                    select-days recurring-schedule recurring-schedule-time minutes hours datatransfer-rate
                    limittransfer-rate limitdatatransferrate data-transfer)
            )
        }
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_edit_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_edit_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub edit

####################### GUI 7Mode Edit ###############################
sub _edit_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"                  => "-sourcename",
            "destination"             => "-dstname",
            "schedule-type"           => "-scheduletype",
            "schedule"                => "-schedule",
            "weekdaysun"              => "-weekdaysun",
            "weekdaymon"              => "-weekdaymon",
            "weekdaytue"              => "-weekdaytue",
            "weekdaywed"              => "-weekdaywed",
            "weekdaythu"              => "-weekdaythu",
            "weekdayfri"              => "-weekdayfri",
            "weekdaysat"              => "-weekdaysat",
            "recurring-schedule-time" => "-recurringschedule",
            "minutes"                 => "-minutes",
            "hours"                   => "-hours",
            "initialize-mirror"       => "-initializemirror",
            "data-transfer"           => "-datatransferrate",
            "limitdatatransferrate"   => "-limitdatatransferrate",
            "qtree"                   => "-qtreename"
        }
    );

    $apiset->snapmirror_edit(%$snapmirror_args);
    $Log->exit() if $may_exit;

} ## end sub _edit_7mode_gui

####################### GUI CMode Edit ###############################
sub _edit_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"        => "-sourcename",
            "destination"   => "-dstname",
            "vserver"       => "-vservername",
            "schedule"      => "-schedule",
            "schedule-name" => "-schedulename"
        }
    );

    $apiset->snapmirror_edit(%$snapmirror_args);
    $Log->exit() if $may_exit;

} ## end sub _edit_cmode_gui

=head2 delete
  
	NACL::C::SysManager::Snapmirror->delete("vserver"        => $vserver,
											"source"         => $src,
											"destination"    => $dst);
  
				(or)

	$snapmirror_object->delete(...);

(Class or instance method) The method deletes a snapmirror relation
Uses a CMode-GUI or 7mode-GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>


=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "destination"=>$string >>

(Optional for class method, Not Applicable for instance method)
Name of the destination in the snapmirror relation

=item C<< "source"=>$string >>

(Optional for class method, Not Applicable for instance method)
Name of the source in the snapmirror relation

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_delete_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_delete_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

####################### GUI 7Mode Delete ###############################
sub _delete_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"      => "-sourcename",
            "destination" => "-dstname"
        }
    );

    $apiset->snapmirror_delete(%$snapmirror_args);
    $Log->exit() if $may_exit;

} ## end sub _delete_7mode_gui

####################### GUI CMode Delete ###############################
sub _delete_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"      => "-sourcename",
            "destination" => "-dstname",
            "vserver"     => "-vservername"
        }
    );

    $apiset->snapmirror_delete(%$snapmirror_args);
    $Log->exit() if $may_exit;

} ## end sub _delete_cmode_gui

=head2 quiesce

	NACL::C::SysManager::Snapmirror->quiesce(command_interface => $ci,
											 "source"          => $src
											 "destination"     => $dest);
	
						(or)

	$snapmirror_object->quiesce(...);

(Class or instnace method)This method is used to quiesce snapmirror
relation.
Uses a CMode GUI or 7Mode-GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "source" => $string >>

(Optional for class method, Not Applicable for instance method)
Name of the source in which the snapmirror relation is created

=item C<< "destination" => $string >>

(Optional for class method, Not Applicable for instance method)
Name of the destination in which the snapmirror relation is created

=back

=back

=cut

sub quiesce {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_quiesce_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_quiesce_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub quiesce

####################### GUI 7Mode quiesce ###############################
sub _quiesce_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"      => "-sourcename",
            "destination" => "-dstname",
        }
    );

    $apiset->snapmirror_operation(
        -operation => "quiesce",
        %$snapmirror_args
    );
    $Log->exit() if $may_exit;

} ## end sub _quiesce_7mode_gui

####################### GUI CMode quiesce ###############################
sub _quiesce_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"      => "-sourcename",
            "destination" => "-dstname",
            "vserver"     => "-vservername"
        }
    );

    $apiset->snapmirror_operation(
        -operation => "quiesce",
        %$snapmirror_args
    );
    $Log->exit() if $may_exit;

} ## end sub _quiesce_cmode_gui

=head2 resume

	NACL::C::SysManager::Snapmirror->resume(command_interface => $ci,
											"source"          => $src,
											"destination"     => $dest,);
 
					(or)

	$snapmirror_object->resume(...);

(Class or instnace method)This method is used to 
resume the snapmirror relation.
Uses a CMode GUI or 7Mode-GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "source" => $string >>

(Optional for class method, Not Applicable for instance method)
Name of the source volume for which the snapmirror relation is created

=item C<< "destination" => $string >>

(Optional for class method, Not Applicable for instance method)
Name of the destination in which the snapmirror relation is created

=back

=back

=cut

sub resume {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_resume_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_resume_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub resume

####################### GUI 7Mode resume ###############################
sub _resume_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"      => "-sourcename",
            "destination" => "-dstname"
        }
    );

    $apiset->snapmirror_operation(
        -operation => "resume",
        %$snapmirror_args
    );
    $Log->exit() if $may_exit;

} ## end sub _resume_7mode_gui

####################### GUI CMode resume ###############################
sub _resume_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"      => "-sourcename",
            "destination" => "-dstname",
            vserver       => "-vservername"
        }
    );

    $apiset->snapmirror_operation(
        -operation => "resume",
        %$snapmirror_args
    );
    $Log->exit() if $may_exit;

} ## end sub _resume_cmode_gui

=head2 break

	NACL::C::SysManager::Snapmirror->break(command_interface => $ci,
										   "source"          => $src,
										   "destination"     => $dest,);
 
				(or)

	$snapmirror_object->break(...);

(Class or instnace method)This method is used to 
break snapmirror relation.
Uses a CMode GUI or 7Mode-GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "source" => $string >>

(Optional for class method, Not Applicable for instance method)
Name of the source volume for which the snapmirror relation is created

=item C<< "destination" => $string >>

(Optional for class method, Not Applicable for instance method)
Name of the destination in which the snapmirror relation is created

=back

=back

=cut

sub break {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_break_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_break_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub break

####################### GUI 7Mode break ###############################
sub _break_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"      => "-sourcename",
            "destination" => "-dstname"
        }
    );

    $apiset->snapmirror_operation( -operation => "break", %$snapmirror_args );
    $Log->exit() if $may_exit;

} ## end sub _break_7mode_gui

####################### GUI CMode break ###############################
sub _break_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"      => "-sourcename",
            "destination" => "-dstname",
            vserver       => "-vservername"
        }
    );

    $apiset->snapmirror_operation( -operation => "break", %$snapmirror_args );
    $Log->exit() if $may_exit;

} ## end sub _break_cmode_gui

=head2 resync

	NACL::C::SysManager::Snapmirror->resync(command_interface => $ci,
											"source"          => $src,
											"destination"     => $dest,);
 
				(or)

	$snapmirror_object->resync(...);

(Class or instnace method)This method is used to 
resync the snapmirror relation.
Uses a CMode GUI or 7Mode-GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "source" => $string >>

(Optional for class method, Not Applicable for instance method)
Name of the source volume for which the snapmirror relation is created

=item C<< "destination" => $string >>

(Optional for class method, Not Applicable for instance method)
Name of the destination in which the snapmirror relation is created

=back

=back

=cut

sub resync {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_resync_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_resync_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub resync

####################### GUI 7Mode resync ###############################
sub _resync_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"      => "-sourcename",
            "destination" => "-dstname",
        }
    );

    $apiset->snapmirror_operation(
        -operation => "resync",
        %$snapmirror_args
    );
    $Log->exit() if $may_exit;

} ## end sub _resync_7mode_gui

####################### GUI CMode resync ###############################
sub _resync_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"      => "-sourcename",
            "destination" => "-dstname",
            vserver       => "-vservername"
        }
    );

    $apiset->snapmirror_operation(
        -operation => "resync",
        %$snapmirror_args
    );
    $Log->exit() if $may_exit;

} ## end sub _resync_cmode_gui

=head2 reverseresync

	NACL::C::SysManager::Snapmirror->reverseresync(command_interface => $ci,
												   "source"          => $src,
												   "destination"     => $dest,);
 
					(or)

	$snapmirror_object->reverseresync(...);

(Class or instnace method)This method is used to 
reverse resync the snapmirror relation
Uses a CMode GUI or 7Mode-GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "source" => $string >>

(Optional for class method, Not Applicable for instance method)
Name of the source volume for which the snapmirror relation is created

=item C<< "destination" => $string >>

(Optional for class method, Not Applicable for instance method)
Name of the destination in which the snapmirror relation is created

=back

=back

=cut

sub reverseresync {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_reverseresync_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_reverseresync_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub reverseresync

####################### GUI 7Mode reverseresync ###############################
sub _reverseresync_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"      => "-sourcename",
            "destination" => "-dstname",
        }
    );

    $apiset->snapmirror_operation(
        -operation => "reverseresync",
        %$snapmirror_args
    );
    $Log->exit() if $may_exit;

} ## end sub _reverseresync_7mode_gui

####################### GUI CMode reverseresync ###############################
sub _reverseresync_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"      => "-sourcename",
            "destination" => "-dstname",
            vserver       => "-vservername"
        }
    );

    $apiset->snapmirror_operation(
        -operation => "reverseresync",
        %$snapmirror_args
    );
    $Log->exit() if $may_exit;

} ## end sub _reverseresync_cmode_gui

=head2 initialize

	NACL::C::SysManager::Snapmirror->initialize(command_interface => $ci,
												"source"          => $src,
												"destination"     => $dest,);
 
					(or)

	$snapmirror_object->initialize(...);

(Class or instnace method)This method is used to 
initialize the snapmirror relation.
Uses a CMode GUI or 7Mode-GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "source" => $string >>

(Optional for class method, Not Applicable for instance method)
Name of the source volume for which the snapmirror relation is created

=item C<< "destination" => $string >>

(Optional for class method, Not Applicable for instance method)
Name of the destination in which the snapmirror relation is created

=back

=back

=cut

sub initialize {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_initialize_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_initialize_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub initialize

####################### GUI 7Mode initialize ###############################
sub _initialize_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"      => "-sourcename",
            "destination" => "-dstname"
        }
    );

    $apiset->snapmirror_operation(
        -operation => "initialize",
        %$snapmirror_args
    );
    $Log->exit() if $may_exit;

} ## end sub _initialize_7mode_gui

####################### GUI CMode initialize ###############################
sub _initialize_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"      => "-sourcename",
            "destination" => "-dstname",
            vserver       => "-vservername"
        }
    );

    $apiset->snapmirror_operation(
        -operation => "initialize",
        %$snapmirror_args
    );
    $Log->exit() if $may_exit;

} ## end sub _initialize_cmode_gui

=head2 update

	NACL::C::SysManager::Snapmirror->update(command_interface => $ci,
											"source"          => $src,
											"destination"     => $dest,);
 
				(or)

	$snapmirror_object->update(...);

(Class or instnace method)This method is used to
update snapmirror relation
Uses a CMode GUI or 7Mode-GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "source" => $string >>

(Optional for class method, Not Applicable for instance method)
Name of the source volume for which the snapmirror relation is created

=item C<< "destination" => $string >>

(Optional for class method, Not Applicable for instance method)
Name of the destination in which the snapmirror relation is created

=back

=back

=cut

sub update {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_update_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_update_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub update

####################### GUI 7Mode update ###############################
sub _update_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"      => "-sourcename",
            "destination" => "-dstname",
        }
    );

    $apiset->snapmirror_operation(
        -operation => "update",
        %$snapmirror_args
    );
    $Log->exit() if $may_exit;

} ## end sub _update_7mode_gui

####################### GUI CMode update ###############################
sub _update_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"      => "-sourcename",
            "destination" => "-dstname",
            vserver       => "-vservername"
        }
    );

    $apiset->snapmirror_operation(
        -operation => "update",
        %$snapmirror_args
    );
    $Log->exit() if $may_exit;

} ## end sub _update_cmode_gui

=head2 abort

	NACL::C::SysManager::Snapmirror->abort(command_interface => $ci,
										   "source"          => $src,
										   "destination"     => $dest,);
 
					(or)

	$snapmirror_object->abort(...);

(Class or instnace method)This method is used to 
abort snapmirror relation
Uses a CMode GUI or 7Mode-GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "source" => $string >>

(Optional for class method, Not Applicable for instance method)
Name of the source volume for which the snapmirror relation is created

=item C<< "destination" => $string >>

(Optional for class method, Not Applicable for instance method)
Name of the destination in which the snapmirror relation is created

=back

=back

=cut

sub abort {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_abort_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_abort_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub abort

####################### GUI 7Mode abort ###############################
sub _abort_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"      => "-sourcename",
            "destination" => "-dstname"
        }
    );

    $apiset->snapmirror_operation( -operation => "abort", %$snapmirror_args );
    $Log->exit() if $may_exit;

} ## end sub _abort_7mode_gui

####################### GUI CMode abort ###############################
sub _abort_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "source"      => "-sourcename",
            "destination" => "-dstname",
            vserver       => "-vservername"
        }
    );

    $apiset->snapmirror_operation( -operation => "abort", %$snapmirror_args );
    $Log->exit() if $may_exit;

} ## end sub _abort_cmode_gui

=head2 remote_access

	NACL::C::SysManager::Snapmirror->remote_access(command_interface => $ci,
												   "source"          => $src,
												   "destination"     => $dest,);
 
							(or)

	$snapmirror_object->remote_access(...);

(Class or instnace method)This method is used to 
remote access the Snapmirror relation
Uses a 7Mode-GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "source" => $string >>

(Optional for class method, Not Applicable for instance method)
Name of the source volume for which the snapmirror relation is created

=item C<< "destination" => $string >>

(Optional for class method, Not Applicable for instance method)
Name of the destination in which the snapmirror relation is created

=back

=back

=cut

sub remote_access {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            volume => { type => SCALAR },
            option => { type => SCALAR },
            _optional_scalars(qw(ip new-volume))
        }
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_remote_access_7mode_gui',
                set    => 'SysManager_7Mode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub remote_access

####################### GUI 7Mode remote_access ###############################
sub _remote_access_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "volume"     => "-volumename",
            "ip"         => "-remotesystemip",
            "new-volume" => "-newvolname",
            "option"     => "-option"
        }
    );

    $apiset->snapmirror_remoteaccess(%$snapmirror_args);
    $Log->exit() if $may_exit;

} ## end sub _remote_access_7mode_gui

=head2 refresh

	NACL::C::SysManager::Snapmirror->refresh(command_interface => $ci,);
 
							(or)

	$snapmirror_object->refresh(...);

(Class or instnace method)This method is used to 
refreshs the Snapmirror page.
Uses a 7Mode-GUI or CMode-GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub refresh {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_refresh_7mode_gui',
                set    => 'SysManager_7Mode',
            },
            {   method => '_refresh_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub refresh

####################### GUI 7Mode Refresh ###############################
sub _refresh_7mode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy( source => \%opts );

    $apiset->snapmirror_refresh(%$snapmirror_args);
    $Log->exit() if $may_exit;

} ## end sub _refresh_7mode_gui

####################### GUI CMode Refresh ###############################
sub _refresh_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $snapmirror_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            vserver => "-vservername"
        }
    );
    
    $apiset->snapmirror_refresh(%$snapmirror_args);
    $Log->exit() if $may_exit;

} ## end sub _refresh_cmode_gui

####################### Primary Keys Validate ###############################
sub _primary_keys_validate_spec {
    return (
        "targetfiler" => { type => SCALAR },
        "source"      => { type => SCALAR },
        "destination" => { type => SCALAR },
        vserver       => { type => SCALAR | UNDEF, optional => 1 },
    );
} ## end sub _primary_keys_validate_spec

1
