#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemConfigurationRecoveryCluster Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemConfigurationRecoveryCluster

=head1 DESCRIPTION

C<NACL::C::SystemConfigurationRecoveryCluster> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. Unlike other components this
package does not model any element/service but is present to provide 
component-level support for functionality. Since it does not model any 
element/service there is no ComponentState module for this component.
For the same reason, 'find' and 'state' will not work for this component.

=cut

package NACL::C::SystemConfigurationRecoveryCluster;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 recreate

   NACL::C::SystemConfigurationRecoveryCluster->recreate(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to recreate cluster.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "from" => $string >>

Applicable for CMode CLI.

=item C<< "backup" => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub recreate {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( from backup
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_recreate_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub recreate

sub _recreate_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( from backup
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    $Log->debug(
        "Opts to API call 'system_configuration_recovery_cluster_recreate' are: \n"
            . Dumper(\%opts));

    $apiset->system_configuration_recovery_cluster_recreate(%opts);
    $Log->exit() if $may_exit;
} ## end sub _recreate_cmode_cli

=head2 rejoin

   NACL::C::SystemConfigurationRecoveryCluster->rejoin(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to rejoin a cluster.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub rejoin {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( node
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_rejoin_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub rejoin

sub _rejoin_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( node
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    $Log->debug(
        "Opts to API call 'system_configuration_recovery_cluster_rejoin' are: \n"
            . Dumper(\%opts));

    $apiset->system_configuration_recovery_cluster_rejoin(%opts);
    $Log->exit() if $may_exit;
} ## end sub _rejoin_cmode_cli

=head2 sync

   NACL::C::SystemConfigurationRecoveryCluster->sync(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to sync a node with cluster configuration.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub sync {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( node
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_sync_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub sync

sub _sync_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( node
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    $Log->debug(
        "Opts to API call 'system_configuration_recovery_cluster_sync' are: \n"
            . Dumper(\%opts));

    $apiset->system_configuration_recovery_cluster_sync(%opts);
    $Log->exit() if $may_exit;
} ## end sub _sync_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ();
}
1;
