#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemNodeCoredumpReports Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemNodeCoredumpReports

=head1 DESCRIPTION

C<NACL::C::SystemNodeCoredumpReports> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SystemNodeCoredumpReports.  A related class is
L<NACL::CS::SystemNodeCoredumpReports|lib-NACL-CS-SystemNodeCoredumpReports-pm>, which represents
the past state of an ONTAP SystemNodeCoredumpReports.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 reportname

=head2 node

=cut

package NACL::C::SystemNodeCoredumpReports;

use strict;
use warnings;

use NACL::CS::SystemNodeCoredumpReports;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $SystemNodeCoredumpReports = NACL::C::SystemNodeCoredumpReports->new(command_interface=>$ci,
                                   reportname =>$reportname,
                                   node =>$node,
                                  );

Return a new SystemNodeCoredumpReports component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'reportname',
    scalar => 'node',
];

=head2 find

    my $SystemNodeCoredumpReports  = NACL::C::SystemNodeCoredumpReports->find(command_interface => $ci, ...);
    my @SystemNodeCoredumpReportss = NACL::C::SystemNodeCoredumpReports->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $SystemNodeCoredumpReports_state = $NACL::C::SystemNodeCoredumpReports->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SystemNodeCoredumpReports would return 
a NACL::CS::SystemNodeCoredumpReports object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 upload

   NACL::C::SystemNodeCoredumpReports->upload(
                                command_interface => $ci,
                                 reportname => $reportname,                              
                                 node => $node,
                                  ...);
                                 
                                 or

    $SystemNodeCoredumpReports_object->upload(...);

(Class or instance method)
This method is used to upload an application core report to a remote site.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "reportname" => $reportname >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "node" => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "location" => $string >>

Applicable for CMode CLI.

=item C<< "sequence" => $string >>

Applicable for CMode CLI.

=item C<< "casenum" => $string >>

Applicable for CMode CLI.

=item C<< "user-name" => $string >>

(Required) User name for ftp login.
  Applicable for CMode CLI.

=item C<< "password" => $string >>

(Required) password for ftp login.
  Applicable for CMode CLI.

=item C<< 'coreid' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub upload {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $Log->debug("Opts to 'upload' front end are:\n" . Dumper(\@_));
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( location sequence casenum
                    user-name password
                    coreid
                    )
            ),
        },
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_upload_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub upload

sub _upload_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $opts{additional_args} = {
        'connectrec-match_table' => [
            qr/user:/i     => delete $opts{'user-name'},
            qr/password:/i => delete $opts{password}
        ]
    };
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api => 'system_node_coredump_reports_upload',);
    $Log->exit() if $may_exit;
} ## end sub _upload_cmode_cli

=head2 delete

   NACL::C::SystemNodeCoredumpReports->delete(
                                command_interface => $ci,
                                 reportname => $reportname,                              
                                 node => $node,
                                  ...);
                                 
                                 or

    $SystemNodeCoredumpReports_object->delete(...);

(Class or instance method)
This method is used to delete an application core report.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "reportname" => $reportname >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "node" => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'coreid' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $Log->debug("Opts to 'delete' front end are:\n" . Dumper(\@_));
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@_,
        ,
        additional_spec => {
            _optional_scalars(
                qw( coreid
                    )
            )
        },
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_delete_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( reportname node
            extended_query
            coreid
            )
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    $apiset->system_node_coredump_reports_delete(%opts);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'reportname' => {type => SCALAR},
        'node'       => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
