# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemNodeDate Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemNodeDate

=head1 DESCRIPTION

B<For CMode, the time is now set cluster-wide rather than on a per-node basis,
so switch to using NACL::CS::ClusterDate instead.>
(See https://wikid.netapp.com/w/Cluster_QoS/SN_QOS_NTP_FS.)

C<NACL::C::SystemNodeDate> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SystemNodeDate.  A related class is
L<NACL::CS::SystemNodeDate|lib-NACL-CS-SystemNodeDate-pm>, which represents
the past state of an ONTAP SystemNodeDate.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 node

=cut

package NACL::C::SystemNodeDate;

use strict;
use warnings;

use NACL::CS::SystemNodeDate;
use NACL::C::ClusterDate;
use NACL::ComponentUtils qw (_optional_scalars);
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::ComponentUtils qw(Dumper);
use NATE::Exceptions::Argument qw(:try);

=head1 METHODS

=head2 new

  my $SystemNodeDate = NACL::C::SystemNodeDate->new(command_interface=>$ci,
                                   node =>$node,
                                    );

Return a new SystemNodeDate component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'node',];

=head2 find

    my $SystemNodeDate  = NACL::C::SystemNodeDate->find(command_interface => $ci, ...);
    my @SystemNodeDates = NACL::C::SystemNodeDate->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $SystemNodeDate_state = $NACL::C::SystemNodeDate->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SystemNodeDate would return
a NACL::CS::SystemNodeDate object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 modify

   NACL::C::SystemNodeDate->modify(
                                command_interface => $ci,
                                node => $node,
                                  ...);

or

    $SystemNodeDate_object->modify(...);

(Class or instance method)
This method is used to modify node current date and time.

Uses a CMode CLI and 7Mode CLI APISet.

Invokes 'date' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $node >>

(Optional for Class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "timezone" => $string >>

Applicable for CMode CLI.

=item C<< "utcdateandtime" => $string >>

Applicable for CMode CLI.

=item C<< "date" => $string >>

Applicable for CMode CLI.

=item C<< "dateandtime" => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system node date modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( timezone utcdateandtime date dateandtime node )),
        },
        api => 'system_node_date_modify',
    );
    my $ci = $opts{command_interface};
    if ($ci->is_cmode) {
        $Log->warn("Your build has cluster QoS changes for NTP.\n"
            . 'Date and time should be set cluster-wide. Switch to '
            . 'using NACL::C::ClusterDate->modify()');
        delete $opts{node};
        NACL::C::ClusterDate->modify(%opts);
        $Log->exit() if $may_exit;
        return;
    }
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_modify_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
        frontend => 'NACL::C::SystemNodeDate::modify',
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api     => 'system_node_date_modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    if ($opts{dateandtime}) {
        $apiset->date('set-date' => $opts{dateandtime});
    } elsif ($opts{utcdateandtime}) {
        $apiset->date(
            'universal-time' => 1,
            'set-date'       => $opts{utcdateandtime}
        );
    } elsif ($opts{date}) {
        if ($opts{date} =~ /(\d+)\/(\d+)\/(\d+)\s+(\d+):(\d+):(\d+)/) {

            # For CMode CLI, user can pass "4/3/2012 13:5:2"
            # But passing two digit value for the year "4/3/12 13:5:2" is
            # not accepted
            my $time
                = sprintf("%d%02d%02d%02d%02d.%02d", $3, $1, $2, $4, $5, $6);
            $apiset->date('set-date' => $time);
        } else {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw("Invalid date has passed");
        }
    } else {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "No options are specified for modify()");
    }
    $Log->exit() if $may_exit;
} ## end sub _modify_7mode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('node' => {type => SCALAR},);
}
1;
