# $Id$
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemNodeServiceProcessor Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com,skotagir@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemNodeServiceProcessor

=head1 DESCRIPTION

C<NACL::C::SystemNodeServiceProcessor> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SystemNodeServiceProcessor.  A related class is
L<NACL::CS::SystemNodeServiceProcessor|lib-NACL-CS-SystemNodeServiceProcessor-pm>, which represents
the past state of an ONTAP SystemNodeServiceProcessor.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 node

=cut

package NACL::C::SystemNodeServiceProcessor;

use strict;
use warnings;

use NACL::CS::SystemNodeServiceProcessor;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $SystemNodeServiceProcessor = NACL::C::SystemNodeServiceProcessor->new(
                                   command_interface => $ci,
                                   'node' => $node,
                                  );

Return a new SystemNodeServiceProcessor component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'node',];

=head2 find

    my $SystemNodeServiceProcessor_obj  = NACL::C::SystemNodeServiceProcessor->find(command_interface => $ci, ...);
    my @SystemNodeServiceProcessor_objs = NACL::C::SystemNodeServiceProcessor->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI , 7Mode CLI APISet.

Invokes "system node service-processor show" command for 7Mode CLI.

=head2 state

    my SystemNodeServiceProcessor_state = $SystemNodeServiceProcessor_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SystemNodeServiceProcessor would return
a NACL::CS::SystemNodeServiceProcessor object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI,7Mode CLI APISet.

Invokes "system node service-processor show" command for 7Mode CLI.

=cut

=head2 test_autosupport

    NACL::C::SystemNodeServiceProcessor->test_autosupport(
                                command_interface => $ci,
                                'node' => $node,
                                ...);

                                 or

    $SystemNodeServiceProcessor_obj->test_autosupport(...);

(Class or instance method)
This method is used to generate and send a test AutoSupport message from the service processor.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'node' => $string >>

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"system node service-processor test-autosupport" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub test_autosupport {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( node
                    )
            )
        },
        api => 'system_node_service_processor_test_autosupport',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_test_autosupport_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemNodeServiceProcessor::test_autosupport',
    );

    $Log->exit() if $may_exit;
} ## end sub test_autosupport

sub _test_autosupport_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_node_service_processor_test_autosupport',);
    $Log->exit() if $may_exit;
} ## end sub _test_autosupport_cmode_cli

=head2 reboot_sp

    NACL::C::SystemNodeServiceProcessor->reboot_sp(
                                command_interface => $ci,
                                'node' => $node,
                                ...);

                                 or

    $SystemNodeServiceProcessor_obj->reboot_sp(...);

(Class or instance method)
This method is used to reboot the service processor.

Uses a CMode and 7Mode CLI APISet.

Invokes "system node service-processor reboot sp" command for 7Mode CLI.

Invokes "service-processor-reboot" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'node' => $string >>

The name of the node on which you want to reboot the service processor.

Applicable for CMode CLI/ZAPI and 7Mode CLI

Maps to:

7M CLI: -node <value> (Programmatic name: "nodename")

CM ZAPI: node

=item C<< 'image' => $string >>

Image to Boot with After Reboot, possible value(s) are primary,backup

Applicable for CMode CLI/ZAPI and 7Mode CLI

Maps to:

7M CLI: -image <value> (Programmatic name: "image")

CM ZAPI: image

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"system node service-processor reboot-sp" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub reboot_sp {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( node

                    image
                    )
            )
        },
        cli_cmd => 'system service-processor reboot-sp',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_reboot_sp_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_reboot_sp_7mode_cli',
                interface => 'CLI',
                set       => '7Mode'
            },
            {   method    => '_reboot_sp_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemNodeServiceProcessor::reboot_sp',
    );

    $Log->exit() if $may_exit;
} ## end sub reboot_sp

sub _reboot_sp_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_node_service_processor_reboot_sp',);
    $Log->exit() if $may_exit;
} ## end sub _reboot_sp_cmode_cli

sub _reboot_sp_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts   = $pkg->_validate_backend_opts(@args);
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [qw(image)];

    my %reboot_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        map    => {'node' => 'nodename',},
        target => \%reboot_args,
    );

    my $prompts_answers = ['.* Do you want to reboot the SP.*' => 'y'];
    $reboot_args{'connectrec-match_table'} = $prompts_answers;

    $apiset->system_node_service_processor_reboot_sp(%reboot_args);
    $Log->exit() if $may_exit;

}

sub _reboot_sp_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
            api    => 'service_processor_reboot',
            copy   => [qw( node )],
            map    => {image => 'firmware-image'},
	);
    $Log->exit() if $may_exit;
} ## end sub _reboot_sp_cmode_zapi

=head2 modify

    NACL::C::SystemNodeServiceProcessor->modify(
                                command_interface => $ci,
                                'node' => $node,
                                ...);

                                 or

    $SystemNodeServiceProcessor_obj->modify(...);

(Class or instance method)
This method is used to modify the network configuration of the service processor.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'gateway' => $string >>

Applicable for CMode CLI.

=item C<< 'arp-interval' => $string >>

Applicable for CMode CLI.

=item C<< 'netmask' => $string >>

Applicable for CMode CLI.

=item C<< 'address' => $string >>

Applicable for CMode CLI.

=item C<< 'dhcp' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"system node service-processor modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "address" => {type => SCALAR | ARRAYREF, optional => 1},
            _optional_scalars(
                qw( gateway arp-interval netmask dhcp

                    node
                    )
            ),
        },
        api => 'system_node_service_processor_modify',
    );

    $pkg_or_obj->_scalar_to_array_deprication_warning(
        params  => ['address'],
        options => \%opts
    );

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemNodeServiceProcessor::modify',
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_node_service_processor_modify',);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('node' => {type => SCALAR},);
}

1;
