# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemServiceProcessorImage Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemServiceProcessorImage

=head1 DESCRIPTION

C<NACL::C::SystemServiceProcessorImage> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SystemServiceProcessorImage.  A related class is
L<NACL::CS::SystemServiceProcessorImage|lib-NACL-CS-SystemServiceProcessorImage-pm>, which represents
the past state of an ONTAP SystemServiceProcessorImage.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 node

=head2 image

=cut

package NACL::C::SystemServiceProcessorImage;

use strict;
use warnings;

use NACL::CS::SystemServiceProcessorImage;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $SystemServiceProcessorImage = NACL::C::SystemServiceProcessorImage->new(
                                   command_interface => $ci,
                                   'node' => $node,
                                   'image' => $image,
								  );

Return a new SystemServiceProcessorImage component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'node',
    scalar => 'image',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $SystemServiceProcessorImage_obj  = NACL::C::SystemServiceProcessorImage->find(command_interface => $ci, ...);
	my @SystemServiceProcessorImage_objs = NACL::C::SystemServiceProcessorImage->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my SystemServiceProcessorImage_state = $SystemServiceProcessorImage_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SystemServiceProcessorImage would return 
a NACL::CS::SystemServiceProcessorImage object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 modify

    NACL::C::SystemServiceProcessorImage->modify(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
   
(Class method)
This method is used to enable/Disable automatic firmware update.
Uses CMode CLI/ZAPI .
Invokes  service-processor-image-modify  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'node' => $node >>

(Required)
Node possible value(s) are, <nodename>,local
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'autoupdate' => $string >>

Firmware Autoupdate possible value(s) are, true,false
Applicable for CMode CLI/ZAPI, non-iter.

Mapping for non-iter: is-autoupdate-enabled

=item C<< '*' >>

This method takes all other options supported by "system service processor image modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( node autoupdate
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'system_service_processor_image_modify',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_modify_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::SystemServiceProcessorImage::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'system_service_processor_image_modify',);
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(node)];
    my $map = {'autoupdate' => 'is-autoupdate-enabled',};
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'service-processor-image-modify',
        copy => $copy,
        map  => $map,
    );
    $Log->exit() if $may_exit;
}

=head2 update

    NACL::C::SystemServiceProcessorImage->update(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
   
(Class method)
This method is used to update Service Processor firmware.
Uses CMode CLI/ZAPI .
Invokes  service-processor-image-update  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'node' => $node >>

(Required)
Node possible value(s) are, <nodename>,local
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'baseline' => $string >>

Install Baseline possible value(s) are, true,false
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'clear-logs' => $string >>

Clear Logs After Update possible value(s) are, true,false
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'update-type' => $string >>


Type possible value(s) are, serial-full,serial-differential,network-full
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'package' => $string >>

Firmware Package
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "system service processor image update" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub update {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( baseline clear-logs node update-type package
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'system_service_processor_image_update',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_update_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_update_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::SystemServiceProcessorImage::update',
    );

    $Log->exit() if $may_exit;
}

sub _update_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'system_service_processor_image_update',);
    $Log->exit() if $may_exit;
}

sub _update_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(update-type node package clear-logs)];
    my $map = {'baseline' => 'install-baseline-image',};
	$pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'service-processor-image-update',
        copy => $copy,
        map  => $map,
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'node'  => {type => SCALAR},
        'image' => {type => SCALAR},
    );
}

1;
