# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemSnmp Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemSnmp

=head1 DESCRIPTION

C<NACL::C::SystemSnmp> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SystemSnmp.  A related class is
L<NACL::CS::SystemSnmp|lib-NACL-CS-SystemSnmp-pm>, which represents
the past state of an ONTAP SystemSnmp.
However, since it does not have any primary keys, 'find' and 'state' do not
work for this component. (This component is considered a 'singleton',
i.e. there is only one row shown by the relevant 'show' command)

=cut

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=cut

package NACL::C::SystemSnmp;

use strict;
use warnings;

use NACL::CS::SystemSnmp;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $SystemSnmp = NACL::C::SystemSnmp->new(command_interface=>$ci,
                            );

Return a new SystemSnmp component object with the given attributes.

=cut

=head2 init

   NACL::C::SystemSnmp->init(
                                command_interface => $ci,
                                  ...);

(Class method)

Uses a CMode CLI/ZAPI or 7Mode CLI APISet.

Invokes 'snmp-enable' or 'snmp-disable' API for CMode ZAPI.

Invokes 'snmp init' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "init" => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : init

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system snmp init" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( init
                    )
            ),
        },
        cli_cmd => 'system snmp init',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_init_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_init_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
            {   method    => "_init_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SystemSnmp::init',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub init

sub _init_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    $pkg->_base_backend_cmode_cli(
        @args,
        api => 'system_snmp_init',
    );

    $Log->exit() if $may_exit;
} ## end sub _init_cmode_cli

sub _init_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [
        qw( init
            )
    ];
    my %args_7mode;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%args_7mode,
    );

    $apiset->snmp_init(%args_7mode);
    $Log->exit() if $may_exit;
} ## end sub _init_7mode_cli

sub _init_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    if ($opts{'init'} == 1) {
        $apiset->snmp_enable();
    } elsif ($opts{'init'} == 0) {
        $apiset->snmp_disable();
    } else {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            'Invalid value' . $opts{init} . "passed for init method\n");
    }

} ## end sub _init_cmode_zapi

=head2 contact

   NACL::C::SystemSnmp->contact(
                                command_interface => $ci,
                                  ...);

(Class method)

Uses a CMode CLI or 7Mode CLI APISet.

Invokes 'snmp contact' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "contact" => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : contact

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system snmp contact" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub contact {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( contact
                    )
            ),
        },
        cli_cmd => 'system snmp contact',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_contact_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_contact_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
        frontend => 'NACL::C::SystemSnmp::contact',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub contact

sub _contact_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,  api  => 'system_snmp_contact');
    $Log->exit() if $may_exit;
} ## end sub _contact_cmode_cli

sub _contact_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [
        qw( contact
            )
    ];
    my %args_7mode;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%args_7mode,
    );

    $apiset->snmp_contact(%args_7mode);
    $Log->exit() if $may_exit;
} ## end sub _contact_7mode_cli

=head2 location

   NACL::C::SystemSnmp->location(
                                command_interface => $ci,
                                  ...);

(Class method)

Uses a CMode CLI or 7Mode CLI APISet.

Invokes 'snmp location' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "location" => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : location

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system snmp location" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub location {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( location
                    )
            ),
        },
        cli_cmd => 'system snmp location',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_location_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_location_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
        frontend => 'NACL::C::SystemSnmp::location',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub location

sub _location_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,  api  => 'system_snmp_location');
    $Log->exit() if $may_exit;
} ## end sub _location_cmode_cli

sub _location_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [
        qw( location
            )
    ];
    my %args_7mode;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%args_7mode,
    );

    $apiset->snmp_location(%args_7mode);
    $Log->exit() if $may_exit;
} ## end sub _location_7mode_cli

=head2 authtrap

   NACL::C::SystemSnmp->authtrap(
                                command_interface => $ci,
                                  ...);

(Class method)

Uses a CMode CLI or 7Mode CLI/ZAPI APISet.

Invokes 'snmp-trap-enable' and 'snmp-trap-disable' for CMode ZAPI.

Invokes 'snmp authtrap' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "authtrap" => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : authtrap

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system snmp authtrap" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub authtrap {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( authtrap
                    )
            ),
        },
        cli_cmd => 'system snmp authtrap',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_authtrap_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_authtrap_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
            {   method    => "_authtrap_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },

        ],
        frontend => 'NACL::C::SystemSnmp::authtrap',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub authtrap

sub _authtrap_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,  api  => 'system_snmp_authtrap');
    $Log->exit() if $may_exit;
} ## end sub _authtrap_cmode_cli

sub _authtrap_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [
        qw( authtrap
            )
    ];
    my %args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%args,
    );

    $apiset->snmp_authtrap(%args);
    $Log->exit() if $may_exit;
} ## end sub _authtrap_7mode_cli

sub _authtrap_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    if ($opts{authtrap} eq '1') {
        $apiset->snmp_trap_enable();
    } elsif ($opts{authtrap} eq '0') {
        $apiset->snmp_trap_disable();
    } else {
        NATE::Exceptions::Argument->throw(
            "'authtrap' can be any of " . "'1' or '0' ");
    }
    $Log->exit() if $may_exit;
} ## end sub _authtrap_cmode_zapi

=head2 prepare_to_downgrade

    NACL::C::SystemSnmp->prepare_to_downgrade(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to change SNMP configuration to the default settings for releases earlier than Data ONTAP 8.2.1.
Uses CMode CLI/ZAPI .
Invokes  snmp-prepare-to-downgrade for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< '*' >>

This method takes all other options supported by "system snmp prepare to downgrade" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub prepare_to_downgrade {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        api    => 'system_snmp_prepare_to_downgrade',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_prepare_to_downgrade_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_prepare_to_downgrade_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemSnmp::prepare_to_downgrade',
    );

    $Log->exit() if $may_exit;
}

sub _prepare_to_downgrade_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'system_snmp_prepare_to_downgrade',);
    $Log->exit() if $may_exit;
}

sub _prepare_to_downgrade_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(@opts,
        api => 'snmp-prepare-to-downgrade',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ();
}
1;
