# Copyright (c) 2010 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#

## @summary treecompare Component Module
## @author dl-dpg-nb-automation, dl-nacl-dev;
## @status shared
## @pod here

=head1 NAME

NACL::C::Treecompare

=head1 DESCRIPTION

C<NACL::C::Treecompare> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.
It represents Treecompare command on a filer.

=cut

package NACL::C::Treecompare;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

sub start {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $Log->debug( "Opts to 'treecompare' front end are:\n" . Dumper( \@_ ) );

    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars qw(
            dc-ACLs dc-ctime dc-DOS-bits c-inode-generation-treeid dc-permissions-uid
            dc-permissions-uid quick-mode dc-crtime dc-atime dc-cifs-bit dc-vdisk-metadata remote
            treepath1 treepath2 s-throttle-value s-check-level method-timeout privilege-level 
            ),
        },
    );

    my $ret = $pkg->call_on_apiset(
          %opts,
          choices => [
              {
                  method    => '_start_7mode_cli',
                  interface => 'CLI',
                  set       => '7Mode|Nodescope',
              },
          ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _start_7mode_cli {
      $Log->enter() if $may_enter;
      my $pkg = shift;
      my %opts       = validate_with(
          params      => \@_,
          spec        => $pkg->_backend_validate_spec(),
          allow_extra => 1
      );

      delete $opts{command_interface};
      my $apiset = delete $opts{apiset};

      $pkg->_hash_move(
          source => \%opts,
          target => \%opts,
          map => { 'method-timeout' => 'connectrec-timeout' }
      );

      my $response = $apiset->treecompare_start(%opts);
      $Log->exit() if $may_exit;
      return $response;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
      return ();
}

=head1 METHODS

=head2 Treecompare

   NACL::C::Treecompare->treecompare_start(
                   command_interface => $ci,
                   treepath1            => $treepath1,
                   treepath2       => $treepath2,
                  ...);

(Class method)
This method is used to compare two snapshots.

Applicable for 7Mode CLI

Invokes "treecompare start" command for 7Mode CLI

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "treepath1" => $treepath1 >>

(Required for class method) tree1  path

Applicable for 7Mode CLI

Maps to:

7M CLI : path of the first tree ( programmatic name: treepath1)

=item C<< "treepath2" => $treepath2 >>

(Required for class method) tree2 path

Applicable for 7Mode CLI

Maps to:

7M CLI : path of the second tree (programmatic name: treepath2)

<< "method-timeout" => $scalar >>

(Optional)The new timeout value.

=back

=back

=cut

1;
