# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeConstituent Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VolumeConstituent

=head1 DESCRIPTION

C<NACL::C::VolumeConstituent> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.
Unlike other components this package does not model any element/service
but is present to provide component-level support for functionality.
Since it does not model any element/service there is no ComponentState
module for this component. For the same reason, 'find' and 'state' will
not work for this component.
Instead NACL::CS::VolumeConstituent->fetch() should be used.
(See L<fetch|lib-NACL-CS-ComponentState-pm/fetch> for a full description)

=cut

=head1 ATTRIBUTES

None.

=cut

package NACL::C::VolumeConstituent;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $VolumeConstituent = NACL::C::VolumeConstituent->new(
                                   command_interface => $ci,
								  );

Return a new VolumeConstituent component object with the given attributes.

=cut

use Class::MethodMaker [];

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 assign

    NACL::C::VolumeConstituent->assign(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to assign a role to a volume.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'origMsid' => $string >>

Origin Volume MSID

Applicable for CMode CLI

=item C<< 'volume' => $string >>

Constituent Volume Name

Applicable for CMode CLI

=item C<< 'storage-service' => $string >>

Storage Service Name

Applicable for CMode CLI

=item C<< 'vserver' => $string >>

Vserver

Applicable for CMode CLI

=item C<< 'role' => $string >>

Constituent Volume Role

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "volume constituent assign" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub assign {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( origMsid volume storage-service vserver role
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'volume constituent assign',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_assign_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VolumeConstituent::assign',
    );

    $Log->exit() if $may_exit;
}

sub _assign_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'volume_constituent_assign',);
    $Log->exit() if $may_exit;
}

=head2 unassign

    NACL::C::VolumeConstituent->unassign(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to unassign a role from a volume.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $string >>

Constituent Volume Name

Applicable for CMode CLI

=item C<< 'vserver' => $string >>

Vserver

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "volume constituent unassign" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub unassign {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volume vserver
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'volume constituent unassign',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_unassign_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VolumeConstituent::unassign',
    );

    $Log->exit() if $may_exit;
}

sub _unassign_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'volume_constituent_unassign',);
    $Log->exit() if $may_exit;
}

=head2 recover_namespace

    NACL::C::VolumeConstituent->recover_namespace(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to recovers the namespace constituent of an Infinite Volume.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $string >>

Volume name

Applicable for CMode CLI

=item C<< 'vserver' => $string >>

Vserver

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "volume constituent recover namespace" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub recover_namespace {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volume vserver
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'volume constituent recover-namespace',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_recover_namespace_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VolumeConstituent::recover_namespace',
    );

    $Log->exit() if $may_exit;
}

sub _recover_namespace_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'volume_constituent_recover_namespace',);
    $Log->exit() if $may_exit;
}

=head2 trigger_metatype_scan

    NACL::C::VolumeConstituent->trigger_metatype_scan(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to trigger the metatype inode flag scan on the namespace constituent of an Infinite Volume.
Uses CMode CLI/ZAPI .
Invokes  volume-constituent-trigger-metatype-scan for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $string >>

Volume Name

Applicable for CMode CLI/ZAPI


Maps to:

CMode ZAPI : name

=item C<< 'vserver' => $string >>

Vserver Name

Applicable for CMode CLI/ZAPI


Maps to:

CMode ZAPI : owning-vserver-name

=item C<< '*' >>

This method takes all other options supported by "volume constituent trigger metatype scan" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub trigger_metatype_scan {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volume vserver
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'volume constituent trigger-metatype-scan',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_trigger_metatype_scan_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_trigger_metatype_scan_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VolumeConstituent::trigger_metatype_scan',
    );

    $Log->exit() if $may_exit;
}

sub _trigger_metatype_scan_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'volume_constituent_trigger_metatype_scan',);
    $Log->exit() if $may_exit;
}

sub _trigger_metatype_scan_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $map = {'volume' => 'name',};
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api => 'volume-constituent-trigger-metatype-scan',
        map => $map,
    );
    $Log->exit() if $may_exit;
}

__PACKAGE__->_load_event_class();

1;
