# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeFileFingerprint Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VolumeFileFingerprint

=head1 DESCRIPTION

C<NACL::C::VolumeFileFingerprint> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VolumeFileFingerprint.  A related class is
L<NACL::CS::VolumeFileFingerprint|lib-NACL-CS-VolumeFileFingerprint-pm>, which represents
the past state of an ONTAP VolumeFileFingerprint.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 session_id

=cut

package NACL::C::VolumeFileFingerprint;

use strict;
use warnings;

use NACL::CS::VolumeFileFingerprint;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $VolumeFileFingerprint = NACL::C::VolumeFileFingerprint->new(
                                   command_interface => $ci,
                                   'session-id' => $session_id,
								  );

Return a new VolumeFileFingerprint component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'session_id',];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $VolumeFileFingerprint_obj  = NACL::C::VolumeFileFingerprint->find(command_interface => $ci, ...);
	my @VolumeFileFingerprint_objs = NACL::C::VolumeFileFingerprint->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my VolumeFileFingerprint_state = $VolumeFileFingerprint_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VolumeFileFingerprint would return
a NACL::CS::VolumeFileFingerprint object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 abort

    NACL::C::VolumeFileFingerprint->abort(
                                command_interface => $ci,
	                            'session-id' => $session_id,
                                ...);

								 or

    $VolumeFileFingerprint_obj->abort(...);

(Class or instance method)
This method is used to abort a file fingerprint operation.
Uses CMode CLI/ZAPI .
Invokes  file-fingerprint-abort  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'session-id' => $session_id >>

(Required for class method, Not Applicable for instance method)
Session-id
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "volume file fingerprint abort" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub abort {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'volume file fingerprint abort',
    );

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_abort_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_abort_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::VolumeFileFingerprint::abort',
    );

    $Log->exit() if $may_exit;
}

sub _abort_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'volume_file_fingerprint_abort',);
    $Log->exit() if $may_exit;
}

sub _abort_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(session-id)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'file-fingerprint-abort',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [qw(session-id)],
        _vs_context   => 1
    );
    $Log->exit() if $may_exit;
}

=head2 dump

This method queries for the fingerprint details and returns an object of type
L<NACL::CS::VolumeFileFingerprintDump|lib-NACL-CS-VolumeFileFingerprintDump-pm>.
There are differences in the input for CMode and 7Mode, which are described below.
Note that
L<< NACL::STask::VolumeFileFingerprint->start_and_dump|lib-NACL-STask-VolumeFileFingerprint-pm/start_and_dump >>
provides an interface that is mode-agnostic (i.e. works for both CMode and 7Mode).

B<For CMode:>

This method only queries for the fingerprint details. It requires that
the file fingerprint operation has already been started and has completed.
If this is invoked before the operation has completed, then a
L<NACL::C::Exceptions::VolumeFileFingerprint::>
exception is thrown.

The C<requested_fields> option can be used to limit the fields that show
up in the output.

    # Package call
    my $dump_obj = NACL::C::VolumeFileFingerprint->dump(
        command_interface => $ci,

        # Mandatory
        filter            => {'session-id' => $session_id},

        # This is optional
        requested_fields => [ ... list of fields ... ],
    );

    # Instance call
    my $dump_obj = $fingerprint_obj->dump(
        # This is optional
        requested_fields => [ ... list of fields ... ],
    );

B<For 7Mode:>

In 7Mode, the file fingerprint operation is synchronous and there is no
concept of C<session-id>. For this reason, the input required is the
file path (to be specified through C<file>). Specifying extra options,
such as the algorithm and scope of the fingerprint operation can be
specified through the filter. Examples:

    # Only package call supported for 7Mode
    my $dump_obj = NACL::C::VolumeFileFingerprint->dump(
        command_interface => $ci,

        # Mandatory
        filter            => {
            file => $file_path,

            # Can only specify other inputs like algorithm, scope
            algorithm => 'MD5',
            scope     => 'Data_only',
        },
    );

The values accepted by C<algorithm> are: "MD5", "SHA256"
The values accepted by C<scope> are: "Data_and_Metadata", "Data_only", "Metadata_only"

=cut

sub dump {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::VolumeFileFingerprintDump;
    my @state_objs = NACL::CS::VolumeFileFingerprintDump->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 start

    my $VolumeFileFingerprint_obj	= NACL::C::VolumeFileFingerprint->start(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to start a file fingerprint computation on a file.
Uses CMode CLI/ZAPI .
Invokes  file-fingerprint-start  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'algorithm' => $string >>

Fingerprint  Algorithm possible value(s) are, MD5,SHA256
Applicable for CMode CLI/ZAPI, non-iter.

Mapping for non-iter: fingerprint-algorithm

=item C<< 'result-session-id' => $string >>

Session ID of fingerprint operation
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'vserver' => $string >>

Vserver
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'file' => $string >>

Path
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'scope' => $string >>


Fingerprint Scope possible value(s) are, Data_and_Metadata,Data_only,Metadata_only
Applicable for CMode CLI/ZAPI, non-iter.

Mapping for non-iter: fingerprint-scope

=item C<< '*' >>

This method takes all other options supported by "volume file fingerprint start" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( algorithm result-session-id vserver file scope
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'volume file fingerprint start',
    );

    # session-id is returned by the object, so get it from the parsed output
    # in the back-end methods.
    my $vserver;                            # With burt932785, this ZAPIs will be supported only in ontap-vserver family
    $opts{vserver_operated_on} = \$vserver; # and not in ontap-cluster family
    my $session_id;
    $pkg->call_on_apiset(
        %opts,
        _session_id => \$session_id,
        choices => [
            {   method    => '_start_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_start_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::VolumeFileFingerprint::start',
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        session_id        => $session_id,
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _start_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $response;
    $pkg->_base_backend_cmode_cli(
        %opts,
        api => 'volume_file_fingerprint_start',
        apiset_response_obj => \$response
    );
    ${$opts{_session_id}} = $response->get_parsed_output()->[0]{session_id};
    $Log->exit() if $may_exit;
}

sub _start_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $copy = [qw(file algorithm scope)];
    my $response;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api  => 'file-fingerprint-start',
        copy => $copy,
        apiset_response_obj => \$response
    );
    ${$opts{_session_id}} = $response->get_parsed_output()->[0]{'result-session-id'};
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('session-id' => {type => SCALAR},);
}

1;
