# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverAudit Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverAudit

=head1 DESCRIPTION

C<NACL::C::VserverAudit> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverAudit.  A related class is
L<NACL::CS::VserverAudit|lib-NACL-CS-VserverAudit-pm>, which represents
the past state of an ONTAP VserverAudit.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=cut

package NACL::C::VserverAudit;

use strict;
use warnings;

use NACL::CS::VserverAudit;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use NACL::C::Exceptions::VserverAudit::AlreadyExists qw(:try);
use NACL::C::Exceptions::VserverAudit::DoesNotExist;
use NACL::C::Exceptions::VserverAudit::AlreadyEnabled;
use NACL::C::Exceptions::VserverAudit::AlreadyDisabled;

=head1 METHODS

=head2 new

  my $VserverAudit = NACL::C::VserverAudit->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
								  );

Return a new VserverAudit component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'vserver',];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $VserverAudit_obj  = NACL::C::VserverAudit->find(command_interface => $ci, ...);
	my @VserverAudit_objs = NACL::C::VserverAudit->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my VserverAudit_state = $VserverAudit_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverAudit would return 
a NACL::CS::VserverAudit object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::VserverAudit->delete(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $VserverAudit_obj->delete(...);

(Class or instance method)
This method is used to delete audit configuration.
Uses CMode CLI/ZAPI .
Invokes  fileservice-audit-remove  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'force' => $string >>

Force Delete
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "vserver audit delete" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( force
                    )
            ),
        },
        cli_cmd => 'vserver audit delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_delete_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::VserverAudit::delete',
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];

        NACL::C::Exceptions::VserverAudit::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);

        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_audit_delete',);
    $Log->exit() if $may_exit;
}

sub _delete_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(force)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'fileservice-audit-remove',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [qw(vserver)],
        _vs_context   => 1
    );
    $Log->exit() if $may_exit;
}

=head2 disable

    NACL::C::VserverAudit->disable(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $VserverAudit_obj->disable(...);

(Class or instance method)
This method is used to disable auditing.
Uses CMode CLI/ZAPI .
Invokes  fileservice-audit-disable  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "vserver audit disable" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub disable {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        api    => 'vserver_audit_disable',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_disable_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_disable_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::VserverAudit::disable',
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];

        NACL::C::Exceptions::VserverAudit::AlreadyDisabled
            ->detect_convert_and_throw(exception => $exception);
        NACL::C::Exceptions::VserverAudit::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    }; 

    $Log->exit() if $may_exit;
}

sub _disable_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_audit_disable',);
    $Log->exit() if $may_exit;
}

sub _disable_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@opts,
        api => 'fileservice-audit-disable',);
    $Log->exit() if $may_exit;
}

=head2 create

    my $VserverAudit_obj	= NACL::C::VserverAudit->create(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
   
(Class method)
This method is used to create an audit configuration.
Uses CMode CLI/ZAPI .
Invokes  fileservice-audit-config-create  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'vserver' => $vserver >>

(Required)
Vserver
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'destination' => $string >>

Log Destination Path
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'rotate-size' => $string >>

Log File Size Limit
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'format' => $string >>

Log Format possible value(s) are, xml,evtx
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'rotate-limit' => $string >>

Log Files Rotation Limit
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'rotate-schedule-minute' => [ $rotate-schedule-minute1, $rotate-schedule-minute2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'rotate-schedule-dayofweek' => [ $rotate-schedule-dayofweek1, $rotate-schedule-dayofweek2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'rotate-schedule-day' => [ $rotate-schedule-day1, $rotate-schedule-day2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'events' => [ $event1, $event2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'rotate-schedule-month' => [ $rotate-schedule-month1, $rotate-schedule-month2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'rotate-schedule-hour' => [ $rotate-schedule-hour1, $rotate-schedule-hour2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "vserver audit create" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'rotate-schedule-minute'    => {type => ARRAYREF, optional => 1},
            'rotate-schedule-dayofweek' => {type => ARRAYREF, optional => 1},
            'rotate-schedule-day'       => {type => ARRAYREF, optional => 1},
            'events'                    => {type => ARRAYREF, optional => 1},
            'rotate-schedule-month'     => {type => ARRAYREF, optional => 1},
            'rotate-schedule-hour'      => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( destination rotate-size format rotate-limit
                    )
            ),
        },
        cli_cmd => 'vserver audit create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_create_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::VserverAudit::create',
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VserverAudit::AlreadyExists
            ->detect_convert_and_throw(exception => $exception);

        $Log->exit() if $may_exit;
        $exception->throw();
    };

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $vserver,
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'vserver_audit_create',);
    $Log->exit() if $may_exit;
}

sub _create_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [
        qw(rotate-schedule-minute rotate-schedule-day rotate-schedule-dayofweek
            destination rotate-size format events rotate-limit rotate-schedule-month
            rotate-schedule-hour)
    ];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api                     => 'fileservice-audit-config-create',
        copy                    => $copy,
        zapi_field_translations => {
            cron_time_integer => [
                qw(rotate-schedule-minute rotate-schedule-dayofweek rotate-schedule-day
                    rotate-schedule-month rotate-schedule-hour)
            ],
            convert_to_bytes  => [qw(rotate-size)],
        },
    );
    $Log->exit() if $may_exit;
}

=head2 enable

    NACL::C::VserverAudit->enable(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $VserverAudit_obj->enable(...);

(Class or instance method)
This method is used to enable auditing.
Uses CMode CLI/ZAPI .
Invokes  fileservice-audit-enable  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'force' => $string >>

Force Enable
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "vserver audit enable" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub enable {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( force
                    )
            ),
        },
        cli_cmd => 'vserver audit enable',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_enable_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_enable_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::VserverAudit::enable',
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];

        NACL::C::Exceptions::VserverAudit::AlreadyEnabled
            ->detect_convert_and_throw(exception => $exception);

        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
}

sub _enable_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_audit_enable',);
    $Log->exit() if $may_exit;
}

sub _enable_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(force)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'fileservice-audit-enable',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::VserverAudit->modify(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $VserverAudit_obj->modify(...);

(Class or instance method)
This method is used to modify the audit configuration.
Uses CMode CLI/ZAPI .
Invokes  fileservice-audit-config-modify  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'destination' => $string >>

Log Destination Path
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'audit-guarantee' => $string >>

Strict Guarantee of Auditing possible value(s) are, true,false
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'rotate-size' => $string >>

Log File Size Limit
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'format' => $string >>

Log Format possible value(s) are, xml,evtx
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'rotate-limit' => $string >>

Log Files Rotation Limit
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'rotate-schedule-minute' => [ $rotate-schedule-minute1, $rotate-schedule-minute2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'rotate-schedule-dayofweek' => [ $rotate-schedule-dayofweek1, $rotate-schedule-dayofweek2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'rotate-schedule-day' => [ $rotate-schedule-day1, $rotate-schedule-day2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'events' => [ $event1, $event2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'rotate-schedule-month' => [ $rotate-schedule-month1, $rotate-schedule-month2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'rotate-schedule-hour' => [ $rotate-schedule-hour1, $rotate-schedule-hour2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "vserver audit modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'rotate-schedule-minute'    => {type => ARRAYREF, optional => 1},
            'rotate-schedule-dayofweek' => {type => ARRAYREF, optional => 1},
            'rotate-schedule-day'       => {type => ARRAYREF, optional => 1},
            'events'                    => {type => ARRAYREF, optional => 1},
            'rotate-schedule-month'     => {type => ARRAYREF, optional => 1},
            'rotate-schedule-hour'      => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( destination audit-guarantee rotate-size format rotate-limit
                    )
            ),
        },
        cli_cmd => 'vserver audit modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_modify_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::VserverAudit::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_audit_modify',);
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [
        qw(rotate-schedule-minute audit-guarantee rotate-schedule-day
            rotate-schedule-dayofweek destination rotate-size format events
            rotate-limit rotate-schedule-month rotate-schedule-hour)
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api                     => 'fileservice-audit-config-modify',
        copy                    => $copy,
        zapi_field_translations => {
            cron_time_integer => [
                qw(rotate-schedule-minute rotate-schedule-dayofweek rotate-schedule-day
                    rotate-schedule-month rotate-schedule-hour)],
            convert_to_bytes  => [qw(rotate-size)], 
        },
    );
    $Log->exit() if $may_exit;
}

=head2 rotate_log

    NACL::C::VserverAudit->rotate_log(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $VserverAudit_obj->rotate_log(...);

(Class or instance method)
This method is used to rotate audit log.
Uses CMode CLI/ZAPI .
Invokes  fileservice-audit-rotate-log  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "vserver audit rotate log" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub rotate_log {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        api    => 'vserver_audit_rotate_log',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_rotate_log_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_rotate_log_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::VserverAudit::rotate_log',
    );

    $Log->exit() if $may_exit;
}

sub _rotate_log_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_audit_rotate_log',);
    $Log->exit() if $may_exit;
}

sub _rotate_log_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@opts,
        api => 'fileservice-audit-rotate-log',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('vserver' => {type => SCALAR},);
}

1;
