# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifsSessionFile Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverCifsSessionFile

=head1 DESCRIPTION

C<NACL::C::VserverCifsSessionFile> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverCifsSessionFile.  A related class is
L<NACL::CS::VserverCifsSessionFile|lib-NACL-CS-VserverCifsSessionFile-pm>, which represents
the past state of an ONTAP VserverCifsSessionFile.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 file_id

=head2 connection_id

=head2 vserver

=head2 node

=cut

package NACL::C::VserverCifsSessionFile;

use strict;
use warnings;

use NACL::CS::VserverCifsSessionFile;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);

=head1 METHODS

=head2 new

  my $VserverCifsSessionFile = NACL::C::VserverCifsSessionFile->new(
                                   command_interface => $ci,
                                   'file-id' => $file_id,
                                   'connection-id' => $connection_id,
                                   'vserver' => $vserver,
                                   'node' => $node,
								  );

Return a new VserverCifsSessionFile component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'file_id',
    scalar => 'connection_id',
    scalar => 'vserver',
    scalar => 'node',
];

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $VserverCifsSessionFile_obj  = NACL::C::VserverCifsSessionFile->find(command_interface => $ci, ...);
	my @VserverCifsSessionFile_objs = NACL::C::VserverCifsSessionFile->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my VserverCifsSessionFile_state = $VserverCifsSessionFile_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverCifsSessionFile would return 
a NACL::CS::VserverCifsSessionFile object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 close

    NACL::C::VserverCifsSessionFile->close(
                                command_interface => $ci,
	                            'node' => $node,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $VserverCifsSessionFile_obj->close(...);

(Class or instance method)
This method is used to close CIFS file.
Uses CMode CLI/ZAPI .
Invokes  cifs-session-file-close for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node possible value(s) are, <nodename>,local

Applicable for CMode CLI/ZAPI

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver

Applicable for CMode CLI/ZAPI

=item C<< 'connection-id' => $string >>

Connection ID

Applicable for CMode CLI/ZAPI

=item C<< 'file-id' => $string >>

File ID

Applicable for CMode CLI/ZAPI

=item C<< 'session-id' => $string >>

Session ID

Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "vserver cifs session file close" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub close {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( connection-id file-id session-id
                    )
            ),
        },
        api => 'vserver_cifs_session_file_close',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_close_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_close_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifsSessionFile::close',
    );

    $Log->exit() if $may_exit;
}

sub _close_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_cifs_session_file_close',);
    $Log->exit() if $may_exit;
}

sub _close_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(connection-id file-id session-id node)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'cifs-session-file-close',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'file-id'       => {type => SCALAR},
        'vserver'       => {type => SCALAR},
        'node'          => {type => SCALAR},
        'connection-id' => {type => SCALAR},
    );
}

1;
