# Copyright (c) 2001-2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifsShareAccessControl Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status public
## @pod here

=head1 NAME

NACL::C::VserverCifsShareAccessControl

=head1 DESCRIPTION

C<NACL::C::VserverCifsShareAccessControl> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverCifsShareAccessControl.  A related class is
L<NACL::CS::VserverCifsShareAccessControl|lib-NACL-CS-VserverCifsShareAccessControl-pm>, which represents
the past state of an ONTAP VserverCifsShareAccessControl.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 user_or_group

=head2 share

=head2 vserver

=cut

package NACL::C::VserverCifsShareAccessControl;

use strict;
use warnings;

use NACL::CS::VserverCifsShareAccessControl;
use NACL::C::Exceptions::VserverCifsShareAccessControl::AlreadyExists;
use NACL::C::Exceptions::VserverCifsShareAccessControl::DoesNotExist ();
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::C::Exceptions::Component qw(:try);
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $VserverCifsShareAccessControl = NACL::C::VserverCifsShareAccessControl->new(command_interface=>$ci,
                                   user_or_group =>$user_or_group,
                                   share =>$share,
                                   vserver =>$vserver,
                                  );

Return a new VserverCifsShareAccessControl component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'user_or_group',
    scalar => 'share',
    scalar => 'vserver',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $VserverCifsShareAccessControl  = NACL::C::VserverCifsShareAccessControl->find(command_interface => $ci, ...);
    my @VserverCifsShareAccessControls = NACL::C::VserverCifsShareAccessControl->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $VserverCifsShareAccessControl_state = $NACL::C::VserverCifsShareAccessControl->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverCifsShareAccessControl would return
a NACL::CS::VserverCifsShareAccessControl object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 delete

   NACL::C::VserverCifsShareAccessControl->delete(
                                command_interface => $ci,
                                 user_or_group => $user_or_group,
                                 share => $share,
                                 vserver => $vserver,
                                  ...);

                                 or

    $VserverCifsShareAccessControl_object->delete(...);

(Class or instance method)
This method is used to delete an access control list.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "cifs-share-access-control-delete" API for CMode ZAPI.

Invokes "cifs access" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "user-or-group" => $user_or_group >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: user-or-group

7M CLI : user or group name (Programmatic name: "user-or-group")

=item C<< "group" => $group >>

Applicable for 7Mode CLI.

=item C<< "share" => $share >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: share

7M CLI : share name (Programmatic name: "sharename")

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs share access-control delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverCifsShareAccessControl::DoesNotExist>

This type of exception gets thrown when an operation is performed on share access control that doesn't exist.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            {_optional_scalars(qw(delete group user-or-group))},
        cli_cmd => 'vserver cifs share access-control delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret;
        $ret = $pkg_or_obj->call_on_apiset_with_exceptions(
            %opts,
            choices => [
                {   method    => "_delete_cmode_cli",
                    interface => "CLI",
                    set       => "CMode",
                },
                {   method    => "_delete_cmode_zapi",
                    interface => "ZAPI",
                    set       => "CMode",
                },
                {   method    => "_delete_7mode_cli",
                    interface => "CLI",
                    set       => "7Mode",
                },
            ],
            frontend => 'NACL::C::VserverCifsShareAccessControl::delete',
            nacl_exceptions => [qw(DoesNotExist)]
        );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_cifs_share_access_control_delete',);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(user-or-group share)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'cifs-share-access-control-delete',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

sub _delete_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [qw( group user-or-group )];

    my %mode_7args;

    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        map    => {'share' => 'sharename',},
        target => \%mode_7args,
    );

    $mode_7args{'delete'} = 1;
    $apiset->cifs_access(%mode_7args);
    $Log->exit() if $may_exit;
} ## end sub _delete_7mode_cli

=head2 modify

   NACL::C::VserverCifsShareAccessControl->modify(
                                command_interface => $ci,
                                 user_or_group => $user_or_group,
                                 share => $share,
                                 vserver => $vserver,
                                  ...);

                                 or

    $VserverCifsShareAccessControl_object->modify(...);

(Class or instance method)
This method is used to modify an access control list.

Uses a CMode CLI/ZAPI APISet.

Invokes "cifs-share-access-control-modify" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "user-or-group" => $user_or_group >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: user-or-group

=item C<< "share" => $share >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: share

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "permission" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: permission

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs share access-control modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( permission
                    )
            ),
        },
        cli_cmd => 'vserver cifs share access-control modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_modify_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::VserverCifsShareAccessControl::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_cifs_share_access_control_modify',);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(user-or-group share permission)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'cifs-share-access-control-modify',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

=head2 create

    my $VserverCifsShareAccessControl_object    = NACL::C::VserverCifsShareAccessControl->create(
                                command_interface => $ci,
                                 user_or_group => $user_or_group,
                                 share => $share,
                                 vserver => $vserver,
                                  ...);

(Class method)
This method is used to create an access control list.

Uses a CMode CLI/ZAPI APISet, 7Mode CLI APISet.

Invokes "cifs-share-access-control-create" API for CMode ZAPI.

Invokes "cifs access" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "user-or-group" => $user_or_group >>

(Required)

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: user-or-group

7M CLI : user or group name (Programmatic name: "user-or-group")

=item C<< "group" => $group >>

Applcable for 7Mode CLI.

=item C<< "share" => $share >>

(Required)

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: share

7M CLI : share name (Programmatic name: "sharename")

=item C<< "vserver" => $vserver >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "permission" => $string >>

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: permission

7M CLI : permission (Programmatic name: "permissions")

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs share access-control create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverCifsShareAccessControl::AlreadyExists>

This type of exception gets thrown if an attempt is made to create a share access control which already exists.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        additional_spec =>
            {_optional_scalars(qw(  group user-or-group permission )),},
        cli_cmd => 'vserver cifs share access-control create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
        $pkg->call_on_apiset_with_exceptions(
            %opts,
            choices => [
                {   method    => "_create_cmode_cli",
                    interface => "CLI",
                    set       => "CMode",
                },
                {   method    => "_create_7mode_cli",
                    interface => "CLI",
                    set       => "7Mode",
                },
                {   method    => "_create_cmode_zapi",
                    interface => "ZAPI",
                    set       => "CMode",
                }
            ],
            frontend => 'NACL::C::VserverCifsShareAccessControl::create',
            nacl_exceptions => [qw(AlreadyExists)]
        );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        share             => $opts{'share'},
        user_or_group     => $opts{'user-or-group'},
        vserver           => $vserver,
    );

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'vserver_cifs_share_access_control_create',);
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(user-or-group share permission)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'cifs-share-access-control-create',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

sub _create_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_7mode_options = [qw( group user-or-group )];
    my %mode_7args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        map    => {
            'share'      => 'sharename',
            'permission' => 'permissions',
        },
        target => \%mode_7args,
    );

    $apiset->cifs_access(%mode_7args);
    $Log->exit() if $may_exit;
    return $pkg->new(
        command_interface => $command_interface,
        user_or_group     => $opts{'user-or-group'},
        share             => $opts{'share'},
    );
} ## end sub _create_7mode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'user-or-group' => {type => SCALAR},
        'share'         => {type => SCALAR},
        'vserver'       => {type => SCALAR | UNDEF, optional => 1},
    );
} ## end sub _primary_keys_validate_spec

1;
