
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverDataPolicy Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverDataPolicy

=head1 DESCRIPTION

C<NACL::C::VserverDataPolicy> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. Unlike other components this
package does not model any element/service but is present to provide
component-level support for functionality. Since it does not model any
element/service there is no ComponentState module for this component.
For the same reason, 'find' and 'state' will not work for this component.
Instead NACL::CS::VserverDataPolicy->fetch() should be used.
(See L<fetch|lib-NACL-CS-ComponentState-pm/fetch> for a full description)

=cut

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=cut

package NACL::C::VserverDataPolicy;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::BaseException qw(:try);

=head1 METHODS

=head2 new

  my $VserverDataPolicy = NACL::C::VserverDataPolicy->new(
                                   command_interface => $ci,
                                  );

Return a new VserverDataPolicy component object with the given attributes.

=cut

=head2 do_import

    NACL::C::VserverDataPolicy->do_import(
                                command_interface => $ci,
                                file_data_policy => $str,
                                ...);

(Class method)
This method is used to import a data policy from stdin.
Note that the name of this method does not match the CMode CLI command.
This is because the "import" method has special meaning in Perl.

Uses a CMode CLI APISet or CMode ZAPI APISet.
Invokes 'vserver-data-policy-import' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< file_data_policy => $string >>

(Required) For CMode CLI, passed to following prompt as it appears.
"Enter the contents of the file data policy for Vserver "vs0": Press <Enter> when done"

Applicable for CMode CLI/ZAPI.

Maps to:

 CM ZAPI: data-policy-expression

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'domain' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

 CM ZAPI:
	<vserver-data-policy-import>
	    <domain-name>$value</domain-name>
	</vserver-data-policy-import>

=item C<< 'vserver' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'data-policy-expression' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

 CM ZAPI:
	<vserver-data-policy-import>
        <data-policy-expression>$value</data-policy-expression>
    </vserver-data-policy-import>

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver data-policy import" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub do_import {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( domain vserver file_data_policy

                    data-policy-expression
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'vserver data-policy import',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_do_import_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_do_import_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },

        ],
        frontend => 'NACL::C::VserverDataPolicy::do_import',
    );

    $Log->exit() if $may_exit;
} ## end sub do_import

sub _do_import_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    # Convert 'file_data_policy' argument into answering prompts
    # Note that the prompt for this command appears to occasionally
    # require an extra "\n" to show the prompt, which is why the "\n" is
    # added to the end of the value passed in.
    # However the extra "\n" also results in an extra prompt - hence
    # resulting in an off-by-one prompt issue. Which is why we need to
    # resync the connection.
    # See burt 626354.
    my $apiset = $opts{apiset};
    my %additional_args;
    if (defined($opts{file_data_policy})) {
        my $file_data_policy = $opts{file_data_policy};
        $additional_args{'connectrec-match_table'}
            = [
            "Enter the contents of the file data policy for .* when done" =>
                "$file_data_policy\n",
            ];
    }
    try {
        $pkg->_base_backend_cmode_cli(
            %opts,
            api             => 'vserver_data_policy_import',
            additional_args => \%additional_args
        );
    }
    finally {

        # Regardless of whether the command passed or failed, resync the
        # connection
        $apiset->get_connection()->resync();
    };

    $Log->exit() if $may_exit;
} ## end sub _do_import_cmode_cli

sub _do_import_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_zapi(@args,
        api    => 'vserver_data_policy_import',
        copy   => [qw(data-policy-expression )],
        map    => {domain => "domain-name", file_data_policy => "data-policy-expression"},
    );
    $Log->exit() if $may_exit;
}

=head2 export

    NACL::C::VserverDataPolicy->export(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to export a data policy to stdout.

Uses a CMode CLI APISet or CMode ZAPI APISet.
Invokes 'vserver-data-policy-export' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'domain' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

 CM ZAPI:
	<vserver-data-policy-export>
	    <domain-name>$value</domain-name>
	</vserver-data-policy-export>

=item C<< 'vserver' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'data-policy-expression' => $string >>

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver data-policy export" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub export {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( domain vserver

                    data-policy-expression
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'vserver data-policy export',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_export_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_export_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },

        ],
        frontend => 'NACL::C::VserverDataPolicy::export',
    );

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub export

sub _export_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $response;

    $pkg->_base_backend_cmode_cli(
        @opts,
        api                 => 'vserver_data_policy_export',
        apiset_response_obj => \$response
    );
    my $data_policy_str = $response->get_processed_output();
    $Log->exit() if $may_exit;
    return $data_policy_str;
} ## end sub _export_cmode_cli

sub _export_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $response;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api                 => 'vserver_data_policy_export',
        map                 => {domain => "domain-name"},
        apiset_response_obj => \$response
    );
    my $data_policy_str = $response->{'complete_parsed_output'}[0]{'results'}[0]
        ->{'data-policy-expression'};
    $Log->exit() if $may_exit;
    return $data_policy_str;
}

=head2 validate

    NACL::C::VserverDataPolicy->validate(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to validate a data policy without import.

Uses a CMode CLI APISet or CMode ZAPI APISet.
Invokes 'vserver-data-policy-validation-get-error' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< 'expression' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

 CM ZAPI:
	<vserver-data-policy-validation-get-error>
        <expression>$value</expression>
    </vserver-data-policy-validation-get-error>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver data-policy validate" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub validate {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vserver expression
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'vserver data-policy validate',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_validate_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_validate_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },

        ],
        frontend => 'NACL::C::VserverDataPolicy::validate',
    );

    $Log->exit() if $may_exit;
} ## end sub validate

sub _validate_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'vserver_data_policy_validate',);
    $Log->exit() if $may_exit;
} ## end sub _validate_cmode_cli

sub _validate_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(@opts,
        api    => 'vserver_data_policy_validation_get_error',
        copy   => [qw( expression )],
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

1;
