# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverFcpWwpnAlias Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverFcpWwpnAlias

=head1 DESCRIPTION

C<NACL::C::VserverFcpWwpnAlias> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverFcpWwpnAlias.  A related class is
L<NACL::CS::VserverFcpWwpnAlias|lib-NACL-CS-VserverFcpWwpnAlias-pm>, which represents
the past state of an ONTAP VserverFcpWwpnAlias.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=head2 alias

=cut

package NACL::C::VserverFcpWwpnAlias;

use strict;
use warnings;
use NACL::CS::VserverFcpWwpnAlias;
use NACL::ComponentUtils qw (Dumper _optional_scalars);
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $VserverFcpWwpnAlias = NACL::C::VserverFcpWwpnAlias->new(command_interface=>$ci,
                                   vserver =>$vserver,
                                   alias =>$alias,
                                   );

Return a new VserverFcpWwpnAlias component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'alias',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $VserverFcpWwpnAlias  = NACL::C::VserverFcpWwpnAlias->find(command_interface => $ci, ...);
    my @VserverFcpWwpnAlias  = NACL::C::VserverFcpWwpnAlias->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $VserverFcpWwpnAlias_state = $NACL::C::VserverFcpWwpnAlias->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverFcpWwpnAlias would return
a NACL::CS::VserverFcpWwpnAlias object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 remove

   NACL::C::VserverFcpWwpnAlias->remove(
                                command_interface => $ci,
                                vserver => $vserver,
                                alias => $alias,
                                  ...);

(Class method)
This method is used to removes an alias for a World Wide Port Name of an initiator..

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "fcp-wwpnalias-remove" API for CMode ZAPI.

Invokes "fcp wwpn alias remove" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: vserver

=item C<< "alias" => $alias >>

(Required)

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: alias

7M CLI : -a <value> (Programmatic name: "alias")

=item C<< "wwpn" => $string >>

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: wwpn

7M CLI : wwpn port (Programmatic name: "wwpn")

=item C<< "force" => $string >>

Applicable for 7Mode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver fcp wwpn-alias remove" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub remove {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( wwpn alias
                    )
            ),
        },
        cli_cmd => 'vserver fcp wwpn-alias remove',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_remove_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_remove_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => "_remove_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
        frontend => 'NACL::C::VserverFcpWwpnAlias::remove',
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub remove

sub _remove_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api => 'vserver_fcp_wwpn_alias_remove',);
    $Log->exit() if $may_exit;
} ## end sub _remove_cmode_cli

sub _remove_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        copy => [qw( wwpn alias )],
        api  => 'fcp_wwpnalias_remove',
    );
    $Log->exit() if $may_exit;
} ## end sub _remove_cmode_zapi

sub _remove_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my %fcp_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(force wwpn alias)],
        target => \%fcp_args,
    );

    $apiset->fcp_wwpn_alias_remove(%fcp_args);
    $Log->exit() if $may_exit;
} ## end sub _remove_7mode_cli

=head2 modify

   NACL::C::VserverFcpWwpnAlias->modify(
                                command_interface => $ci,
                                vserver => $vserver,
                                alias => $alias,
                                  ...);

(Class method)

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required)

Applicable for CMode CLI.

=item C<< "alias" => $alias >>

(Required)

Applicable for CMode CLI.

=item C<< "wwpn" => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver fcp wwpn-alias modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( wwpn
                    )
            ),
        },
        api => 'vserver_fcp_wwpn_alias_modify',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverFcpWwpnAlias::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $vserver = $opts{vserver};

    $pkg->_base_backend_cmode_cli(
        %opts,
        api => 'vserver_fcp_wwpn_alias_modify',
        vserver_operated_on => \$vserver,
    );

    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

=head2 set

   NACL::C::VserverFcpWwpnAlias->set(
                                command_interface => $ci,
                                vserver => $vserver,
                                alias => $alias,
                                  ...);

(Class method)
This method is used to set an alias for a World Wide Port Name of an initiator that might login to the target..

Uses a CMode CLI APISet or CMode ZAPI APISet or 7Mode CLI APISet.

Invokes "fcp-wwpnalias-set" API for CMode ZAPI.

Invokes "fcp wwpn alias set" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: vserver

=item C<< "alias" => $alias >>

(Required)

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: alias

7M CLI : set alias path (Programmatic name: "set-alias")

=item C<< "wwpn" => $string >>

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: wwpn

7M CLI : set wwpn port (Programmatic name: "set-wwpn")

=item C<< "force" => $string >>

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: force

7M CLI : -f (Programmatic name: "force")

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver fcp wwpn-alias set" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub set {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( wwpn force
                    )
            ),
        },
        cli_cmd => 'vserver fcp wwpn-alias set',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_set_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_set_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => "_set_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
        frontend => 'NACL::C::VserverFcpWwpnAlias::set',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub set

sub _set_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api => 'vserver_fcp_wwpn_alias_set',);
    $Log->exit() if $may_exit;
} ## end sub _set_cmode_cli

sub _set_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        copy => [qw( wwpn alias force )],
        api  => 'fcp_wwpnalias_set',
    );
    $Log->exit() if $may_exit;
} ## end sub _set_cmode_zapi

sub _set_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my %fcp_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(force)],
        map    => {
            'alias' => 'set-alias',
            'wwpn'  => 'set-wwpn'
        },
        target => \%fcp_args,
    );
    %opts = %fcp_args;
    $apiset->fcp_wwpn_alias_set(%opts);
    $Log->exit() if $may_exit;
} ## end sub _set_7mode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver' => {type => SCALAR | UNDEF, optional => 1},
        'alias'   => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
