# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverFpolicy Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverFpolicy

=head1 DESCRIPTION

C<NACL::C::VserverFpolicy> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverFpolicy.  A related class is
L<NACL::CS::VserverFpolicy|lib-NACL-CS-VserverFpolicy-pm>, which represents
the past state of an ONTAP VserverFpolicy.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=head2 policy_name

=cut

package NACL::C::VserverFpolicy;

use strict;
use warnings;

use NACL::CS::VserverFpolicy;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate
    qw(validate validate_with SCALAR SCALARREF ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::Exceptions::VserverFpolicy::AlreadyDisabled;
use NACL::C::Exceptions::VserverFpolicy::AlreadyEnabled;
use NATE::BaseException qw(:try);

=head1 METHODS

=head2 new

  my $VserverFpolicy = NACL::C::VserverFpolicy->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
                                   'policy-name' => $policy_name,
                                  );

Return a new VserverFpolicy component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'policy_name',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $VserverFpolicy_obj  = NACL::C::VserverFpolicy->find(command_interface => $ci, ...);
    my @VserverFpolicy_objs = NACL::C::VserverFpolicy->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my VserverFpolicy_state = $VserverFpolicy_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverFpolicy would return
a NACL::CS::VserverFpolicy object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 disable

    NACL::C::VserverFpolicy->disable(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'policy-name' => $policy_name,
                                ...);

                                 or

    $VserverFpolicy_obj->disable(...);

(Class or instance method)
This method is used to disable a Policy.

Uses a CMode CLI/ZAPI APISet.

Invokes "fpolicy-disable-policy" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< 'policy-name' => $policy_name >>

(Required for class method, Not Applicable for instance method)


Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: policy-name

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver fpolicy disable" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::VserverFpolicy::AlreadyDisabled>

This type of exception gets thrown if an attempt is made to disable
the fpolicy which is already disabled

=back

=back

=cut

sub disable {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        api    => 'vserver_fpolicy_disable',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_disable_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_disable_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::VserverFpolicy::disable',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VserverFpolicy::AlreadyDisabled
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
} ## end sub disable

sub _disable_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_fpolicy_disable',);
    $Log->exit() if $may_exit;
} ## end sub _disable_cmode_cli

sub _disable_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkj_or_obj, @args) = @_;
    my %opts = $pkj_or_obj->_validate_backend_opts(@args);

    my $command_interface     = $opts{command_interface};
    my $apiset                = $opts{apiset};
    my $vserver               = $opts{vserver};
    my $allowed_cmode_options = [qw( policy-name )];

    my %cmode_args;
    $pkj_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkj_or_obj->_handle_zapi_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $vserver,
        command_interface => $command_interface
    );

    $apiset->fpolicy_disable_policy(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _disable_cmode_zapi

=head2 enable

    my $fpolicy_obj = NACL::C::VserverFpolicy->enable(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'policy-name' => $policy_name,
                                ...);


(Class or instance method)
This method is used to enable a Policy and returns an object modeling it.
                                 or

    $VserverFpolicy_obj->enable(...);

(Class or instance method)
This method is used to enable a Policy.

Uses a CMode CLI/ZAPI APISet.

Invokes "fpolicy-enable-policy" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< 'policy-name' => $policy_name >>

(Required for class method, Not Applicable for instance method)


Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: policy-name

=item C<< 'sequence-number' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: sequence-number

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver fpolicy enable" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::VserverFpolicy::AlreadyEnabled>

This type of exception gets thrown if an attempt is made to enable
the fpolicy which is already enabled

=back

=back

=cut

sub enable {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( sequence-number
                    )
            ),
        },
        cli_cmd => 'vserver fpolicy enable',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_enable_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_enable_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::VserverFpolicy::enable',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VserverFpolicy::AlreadyEnabled
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $vserver,
        policy_name       => $opts{'policy-name'},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub enable

sub _enable_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'vserver_fpolicy_enable',);
    $Log->exit() if $may_exit;
} ## end sub _enable_cmode_cli

sub _enable_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $allowed_cmode_options = [qw( sequence-number policy-name )];

    $pkg->_base_backend_cmode_zapi(@args,
        api    => 'fpolicy_enable_policy',
        copy   => $allowed_cmode_options,
    );
    $Log->exit() if $may_exit;
} ## end sub _enable_cmode_zapi

=head2 engine_connect

    NACL::C::VserverFpolicy->engine_connect(
                                command_interface => $ci,
	                            'vserver' => $vserver,
	                            'policy-name' => $policy_name,
                                ...);
	                             
								 or

    $VserverFpolicy_obj->engine_connect(...);

(Class or instance method)
This method is used to establish a connection to FPolicy server.
Uses CMode CLI/ZAPI .
Invokes  fpolicy-server-connect for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver

Applicable for CMode CLI

=item C<< 'policy-name' => $policy_name >>

(Required for class method, Not Applicable for instance method)
Policy

Applicable for CMode CLI/ZAPI

=item C<< 'server' => $string >>

Server

Applicable for CMode CLI/ZAPI


Maps to:

CMode ZAPI : fpolicy-server

=item C<< 'node' => $string >>

Node possible value(s) are, <nodename>,local

Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "vserver fpolicy engine connect" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub engine_connect {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( server node
                    )
            ),
        },
        api => 'vserver_fpolicy_engine_connect',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_engine_connect_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_engine_connect_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverFpolicy::engine_connect',
    );

    $Log->exit() if $may_exit;
}

sub _engine_connect_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_fpolicy_engine_connect',);
    $Log->exit() if $may_exit;
}

sub _engine_connect_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(policy-name node server)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'fpolicy-server-connect',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

=head2 engine_disconnect

    NACL::C::VserverFpolicy->engine_disconnect(
                                command_interface => $ci,
	                            'vserver' => $vserver,
	                            'policy-name' => $policy_name,
                                ...);
	                             
								 or

    $VserverFpolicy_obj->engine_disconnect(...);

(Class or instance method)
This method is used to terminate connection to FPolicy server.
Uses CMode CLI/ZAPI .
Invokes  fpolicy-server-disconnect for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver

Applicable for CMode CLI

=item C<< 'policy-name' => $policy_name >>

(Required for class method, Not Applicable for instance method)
Policy

Applicable for CMode CLI/ZAPI

=item C<< 'server' => $string >>

Server

Applicable for CMode CLI/ZAPI


=item C<< 'node' => $string >>

Node possible value(s) are, <nodename>,local

Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "vserver fpolicy engine disconnect" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub engine_disconnect {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( server node
                    )
            ),
        },
        api => 'vserver_fpolicy_engine_disconnect',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_engine_disconnect_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_engine_disconnect_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverFpolicy::engine_disconnect',
    );

    $Log->exit() if $may_exit;
}

sub _engine_disconnect_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_fpolicy_engine_disconnect',);
    $Log->exit() if $may_exit;
}

sub _engine_disconnect_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(policy-name node server)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'fpolicy-server-disconnect',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

=head2 show_enabled

    my $show_enabled_obj = NACL::C::VserverFpolicy->show_enabled(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_enabled_objs = NACL::C::VserverFpolicy->show_enabled(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_enabled_obj = $VserverFpolicy_obj->show_enabled(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-VserverFpolicyShowEnabled-pm/ATTRIBUTES>
of NACL::CS::VserverFpolicyShowEnabled as input. 
This method returns objects of the type L<NACL::CS::VserverFpolicyShowEnabled|lib-NACL-CS-VserverFpolicyShowEnabled-pm>.

=cut

sub show_enabled {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::VserverFpolicyShowEnabled;
    my @state_objs = NACL::CS::VserverFpolicyShowEnabled->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 show_engine

    my $show_engine_obj = NACL::C::VserverFpolicy->show_engine(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_engine_objs = NACL::C::VserverFpolicy->show_engine(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_engine_obj = $VserverFpolicy_obj->show_engine(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-VserverFpolicyShowEngine-pm/ATTRIBUTES>
of NACL::CS::VserverFpolicyShowEngine as input. 
This method returns objects of the type L<NACL::CS::VserverFpolicyShowEngine|lib-NACL-CS-VserverFpolicyShowEngine-pm>.

=cut

sub show_engine {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::VserverFpolicyShowEngine;
    my @state_objs = NACL::CS::VserverFpolicyShowEngine->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 show_passthrough_read_connection

    my $show_passthrough_read_connection_obj = NACL::C::VserverFpolicy->show_passthrough_read_connection(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_passthrough_read_connection_objs = NACL::C::VserverFpolicy->show_passthrough_read_connection(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_passthrough_read_connection_obj = $VserverFpolicy_obj->show_passthrough_read_connection(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-VserverFpolicyShowPassthroughReadConnection-pm/ATTRIBUTES>
of NACL::CS::VserverFpolicyShowPassthroughReadConnection as input. 
This method returns objects of the type L<NACL::CS::VserverFpolicyShowPassthroughReadConnection|lib-NACL-CS-VserverFpolicyShowPassthroughReadConnection-pm>.

=cut

sub show_passthrough_read_connection {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::VserverFpolicyShowPassthroughReadConnection;
    my @state_objs
        = NACL::CS::VserverFpolicyShowPassthroughReadConnection->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver'     => {type => SCALAR | UNDEF, 'optional' => '1'},
        'policy-name' => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
