# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverHttpDigestUser Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverHttpDigestUser

=head1 DESCRIPTION

C<NACL::C::VserverHttpDigestUser> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverHttpDigestUser.  A related class is
L<NACL::CS::VserverHttpDigestUser|lib-NACL-CS-VserverHttpDigestUser-pm>, which represents
the past state of an ONTAP VserverHttpDigestUser.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=head2 user_name

=cut

package NACL::C::VserverHttpDigestUser;

use strict;
use warnings;

use NACL::CS::VserverHttpDigestUser;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $VserverHttpDigestUser = NACL::C::VserverHttpDigestUser->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
                                   'user-name' => $user_name,
								  );

Return a new VserverHttpDigestUser component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'user_name',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $VserverHttpDigestUser_obj  = NACL::C::VserverHttpDigestUser->find(command_interface => $ci, ...);
	my @VserverHttpDigestUser_objs = NACL::C::VserverHttpDigestUser->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my VserverHttpDigestUser_state = $VserverHttpDigestUser_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverHttpDigestUser would return 
a NACL::CS::VserverHttpDigestUser object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 create

    my $VserverHttpDigestUser_obj	= NACL::C::VserverHttpDigestUser->create(
                                command_interface => $ci,
	                            'vserver' => $vserver,
	                            'user-name' => $user_name,
                                ...);
   
(Class method)
This method is used to create an HTTP digest user.
Uses CMode CLI/ZAPI .
Invokes  http-digest-user-create  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'vserver' => $vserver >>

(Required)
Vserver
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'user-name' => $user_name >>

(Required)
User Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'user-password' => $string >>

The password for the HTTP digest user
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "vserver http digest user create" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( user-password
                    )
            ),
        },
        cli_cmd => 'vserver http digest-user create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_create_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::VserverHttpDigestUser::create',
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $vserver,
        user_name         => $opts{'user-name'},
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $password     = delete $opts{'user-password'};
    my $prompt_answers = [
       'Enter the password:'   => $password,
       'Confirm the password:' => $password,
    ];
    my %additional_args;
    $additional_args{'connectrec-match_table'} = $prompt_answers;
    $pkg->_base_backend_cmode_cli(
        %opts,
        api => 'vserver_http_digest_user_create',
        additional_args => \%additional_args
    );
    $Log->exit() if $may_exit;
}

sub _create_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(user-password user-name)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'http-digest-user-create',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [qw(vserver user-name)],
        _vs_context   => 1
    );
    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::VserverHttpDigestUser->delete(
                                command_interface => $ci,
	                            'vserver' => $vserver,
	                            'user-name' => $user_name,
                                ...);
	                             
								 or

    $VserverHttpDigestUser_obj->delete(...);

(Class or instance method)
This method is used to delete an HTTP digest user.
Uses CMode CLI/ZAPI .
Invokes  http-digest-user-delete  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'user-name' => $user_name >>

(Required for class method, Not Applicable for instance method)
User Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "vserver http digest user delete" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'vserver http digest-user delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_delete_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::VserverHttpDigestUser::delete',
    );

    $Log->exit() if $may_exit;
}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_http_digest_user_delete',);
    $Log->exit() if $may_exit;
}

sub _delete_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(user-name)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'http-digest-user-delete',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

=head2 set_password

    NACL::C::VserverHttpDigestUser->set_password(
                                command_interface => $ci,
	                            'vserver' => $vserver,
	                            'user-name' => $user_name,
                                ...);
	                             
								 or

    $VserverHttpDigestUser_obj->set_password(...);

(Class or instance method)
This method is used to set a password for an HTTP digest user.
Uses CMode CLI/ZAPI .
Invokes  http-digest-user-set-password  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'user-name' => $user_name >>

(Required for class method, Not Applicable for instance method)
User Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'user-password' => $string >>

The Password for an HTTP digest user
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "vserver http digest user set password" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub set_password {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( user-password
                    )
            ),
        },
        cli_cmd => 'vserver http digest-user set-password',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_set_password_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_set_password_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::VserverHttpDigestUser::set_password',
    );

    $Log->exit() if $may_exit;
}

sub _set_password_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $password     = delete $opts{'user-password'};
    my $prompt_answers = [
       'Enter the password:'   => $password,
       'Confirm the password:' => $password,
    ];
    my %additional_args;
    $additional_args{'connectrec-match_table'} = $prompt_answers;
    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api => 'vserver_http_digest_user_set_password',
        additional_args => \%additional_args,
    );
    $Log->exit() if $may_exit;
}

sub _set_password_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(user-password user-name)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'http-digest-user-set-password',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver'   => {type => SCALAR},
        'user-name' => {type => SCALAR},
    );
}

1;
