# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverIscsiInterface Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverIscsiInterface

=head1 DESCRIPTION

C<NACL::C::VserverIscsiInterface> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverIscsiInterface.  A related class is
L<NACL::CS::VserverIscsiInterface|lib-NACL-CS-VserverIscsiInterface-pm>, which represents
the past state of an ONTAP VserverIscsiInterface.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=head2 lif

=cut

package NACL::C::VserverIscsiInterface;

use strict;
use warnings;

use NACL::CS::VserverIscsiInterface;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with UNDEF SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Exceptions::Argument qw(:try);
use NACL::C::_Mixins::VserverIscsi qw(:all);

=head1 METHODS

=head2 new

  my $VserverIscsiInterface = NACL::C::VserverIscsiInterface->new(command_interface=>$ci,
                                   vserver =>$vserver,
                                   lif =>$lif,
                                );

Return a new VserverIscsiInterface component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'lif',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $VserverIscsiInterface  = NACL::C::VserverIscsiInterface->find(command_interface => $ci, ...);
    my @VserverIscsiInterfaces = NACL::C::VserverIscsiInterface->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $VserverIscsiInterface_state = $NACL::C::VserverIscsiInterface->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverIscsiInterface would return
a NACL::CS::VserverIscsiInterface object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 enable

   NACL::C::VserverIscsiInterface->enable(
                                command_interface => $ci,
                                vserver => $vserver,
                                lif => [ $lif ],
                                  ...);

(Class method)
This method is used to enable the specified interfaces for iSCSI service.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "iscsi-interface-enable" API for CMode ZAPI.

Invokes "iscsi interface enable" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required)

=item C<< "lif" => [ $lif1, $lif2, ... ] >>
Applicable for CMode CLI.

=item C<< "lif" => $lif >>

(Required)
Applicable for CMode CLI/ZAPI

Maps to:
	CM ZAPI : interface-name

Applicable for CMode CLI.

=item C<< "all" => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : -a (Programmatic name: "all")

=item C<< "force" => $string >>

Applicable for CMode CLI.

=item C<< "lifs" => [ $lif1, $lif2, ... ] >>

Applicable for CMode CLI.

=item C<< 'interface' => $string >>

Applicable for 7Mode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi interface enable" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub enable {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "lifs" => {type => ARRAYREF, optional => 1},
            "lif"  => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( all interface vserver
                    force
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'vserver iscsi interface enable',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_enable_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_enable_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
            {   method    => "_enable_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::VserverIscsiInterface::enable',
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub enable

sub _enable_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,
        api     => 'vserver_iscsi_interface_enable',
    );
    $Log->exit() if $may_exit;
} ## end sub _enable_cmode_cli

sub _enable_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my %cmode_args;
    $pkg->_handle_zapi_vserver_context(
         api_opts          => \%cmode_args,
         vserver           => $opts{vserver},
         command_interface => $command_interface,
    );

    $opts{lif} = $pkg->_handle_lif_all_options(%opts);

    my $exception;
    foreach my $array_element (qw(lifs lif)) {
        if (defined $opts{$array_element}) {
            foreach my $interface (@{$opts{$array_element}}) {
                try{
                    $apiset->iscsi_interface_enable(%cmode_args,
                    'interface-name' => $interface);
                } otherwise {
                    #Burt952331 Don't fail immediately, but execute all ZAPIs first
                    $exception = $_[0];
                };
            }
        }
    }
    if ($exception){
        $Log->exit() if $may_exit;
        $exception->throw();
    }

    $Log->exit() if $may_exit;
}

sub _enable_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset                = delete $opts{apiset};
    my $allowed_7mode_options = [qw( all interface)];
    my %mode7_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%mode7_args,
    );
    %opts = %mode7_args;

    if ($mode7_args{'interface'}) {
        delete $opts{'all'};
    }
    $apiset->iscsi_interface_enable(%opts);
    $Log->exit() if $may_exit;
}

=head2 disable

   NACL::C::VserverIscsiInterface->disable(
                                command_interface => $ci,
                                vserver => $vserver,
                                lif => [ $lif ],
                                  ...);

(Class method)
This method is used to disable the specified interfaces for iSCSI service.

Uses a CMode CLI APISet.

Invokes "iscsi interface disable" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required)

=item C<< "lif" => [ $lif1, $lif2, ... ] >>
Applicable for CMode CLI.

=item C<< "lif" => $lif >>

(Required)

Applicable for CMode CLI.

=item C<< "all" => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : -a (Programmatic name: "all")

=item C<< "force" => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : -f (Programmatic name: "force")

=item C<< "interface" => $string >>

Applicable for 7Mode CLI.

=item C<< "lifs" => [ $lif1, $lif2, ... ] >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi interface disable" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub disable {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "lifs" => {type => ARRAYREF, optional => 1},
            "lif"  => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( all interface force vserver
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'vserver iscsi interface disable',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_disable_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_disable_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
            {   method    => "_disable_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
        frontend => 'NACL::C::VserverIscsiInterface::disable',
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub disable

sub _disable_cmode_cli {
    $Log->enter() if $may_enter;
    my($pkg, %opts) = @_;
    my $vserver = $opts{vserver};

    $pkg->_base_backend_cmode_cli(
         %opts,
         api => 'vserver_iscsi_interface_disable',
         vserver_operated_on => \$vserver,
    );

    $Log->exit() if $may_exit;
} ## end sub _disable_cmode_cli

sub _disable_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my %cmode_args;

    $pkg->_handle_zapi_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $opts{vserver},
        command_interface => $command_interface,
    );

    $opts{lif} = $pkg->_handle_lif_all_options(%opts);

    my $exception;
    foreach my $array_element (qw(lifs lif)) {
        if (defined $opts{$array_element}) {
            foreach my $interface (@{$opts{$array_element}}) {
                try{
                    $apiset->iscsi_interface_disable(%cmode_args,
                        'interface-name' => $interface);
                } otherwise {
                    #Burt952331 Don't fail immediately, but execute all ZAPIs first
                    $exception = $_[0];
                };
            }
        }
    }
    if ($exception){
        $Log->exit() if $may_exit;
        $exception->throw();
    }
    $Log->exit() if $may_exit;
}

sub _disable_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset                = delete $opts{apiset};
    my $allowed_7mode_options = [
        qw( all interface force
            )
    ];
    my %mode7_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%mode7_args,
    );
    %opts = %mode7_args;
    if ($mode7_args{'interface'}) {
        delete $opts{'all'};
    }
    $apiset->iscsi_interface_disable(%opts);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver' => {type => SCALAR | UNDEF, optional => 1},
        'lif'     => {type => SCALAR,         optional => 1},
    );
}

1;
