# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverIscsiIsns Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverIscsiIsns

=head1 DESCRIPTION

C<NACL::C::VserverIscsiIsns> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverIscsiIsns.  A related class is
L<NACL::CS::VserverIscsiIsns|lib-NACL-CS-VserverIscsiIsns-pm>, which represents
the past state of an ONTAP VserverIscsiIsns.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=cut

package NACL::C::VserverIscsiIsns;

use strict;
use warnings;

use NACL::CS::VserverIscsiIsns;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::Exceptions::VserverIscsiIsns::AlreadyExists;
use NACL::C::Exceptions::VserverIscsiIsns::DoesNotExist;

=head1 METHODS

=head2 new

  my $VserverIscsiIsns = NACL::C::VserverIscsiIsns->new(command_interface=>$ci,
                                   vserver =>$vserver,
                                  );

Return a new VserverIscsiIsns component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'vserver',];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 start

    my $VserverIscsiIsns_object    = NACL::C::VserverIscsiIsns->start(
                                command_interface => $ci,
                                 vserver => $vserver,
                                  ...);

(Class method)
This method is used to start the iSNS service.

Uses a CMode CLI, 7Mode CLI APISet.

Invokes "iscsi isns start" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required)

Applicable for CMode CLI.

=item C<< "status" => $string >>

Applicable for CMode CLI.

=item C<< "hostname" => $string >>

Applicable for CMode CLI.

=item C<< "address" => $string >>

Applicable for CMode CLI.

=item C<< "enabled" => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( status hostname address enabled
                    )
            ),
        },
        cli_cmd => 'vserver iscsi isns start',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_start_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_start_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
    );

    my $obj = $pkg->new(
       command_interface => $opts{command_interface},
       vserver           => $opts{vserver},
    );

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub start

sub _start_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $allowed_cmode_options = [
        qw( vserver status hostname address enabled
            extended_query )
    ];

    $pkg->_base_backend_cmode_cli(%opts,
        api    => 'vserver_iscsi_isns_start',
        copy   => $allowed_cmode_options,
    );

    $Log->exit() if $may_exit;
} ## end sub _start_cmode_cli

sub _start_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    $apiset->iscsi_isns_start();
    $Log->exit() if $may_exit;
    return $pkg->new(command_interface => $command_interface,);
} ## end sub _start_7mode_cli

=head2 config

   NACL::C::VserverIscsiIsns->config(
                                command_interface => $ci,
                                 vserver => $vserver,
                                  ...);

                                 or

    $VserverIscsiIsns_object->config(...);

(Class or instance method)
This method is used to configure the iSNS service with the hostname or IP address of the iSNS server.

Uses a CMode CLI, 7Mode CLI APISet.

Invokes "iscsi isns config" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "status" => $string >>

Applicable for CMode CLI.

=item C<< "hostname" => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : host name (Programmatic name: "host")

=item C<< "address" => $string >>

Applicable for CMode CLI.

=item C<< "enabled" => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi isns config" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub config {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( status hostname address enabled
                    )
            ),
        },
        cli_cmd => 'vserver iscsi isns config',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $Log->exit() if $may_exit;

    return $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_config_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_config_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
        frontend => 'NACL::C::VserverIscsiIsns::config',
    );
} ## end sub config

sub _config_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_iscsi_isns_config',);
    $Log->exit() if $may_exit;
} ## end sub _config_cmode_cli

sub _config_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %mode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%mode_args,
        map    => {'hostname' => 'host',},
    );
    %opts = %mode_args;

    $apiset->iscsi_isns_config(%opts);
    $Log->exit() if $may_exit;
} ## end sub _config_7mode_cli

=head2 stop

   NACL::C::VserverIscsiIsns->stop(
                                command_interface => $ci,
                                 vserver => $vserver,
                                  ...);

                                 or

    $VserverIscsiIsns_object->stop(...);

(Class or instance method)
This method is used to stop the iSNS service.

Uses a CMode CLI, 7Mode CLI APISet.

Invokes "iscsi isns stop" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "status" => $string >>

Applicable for CMode CLI.

=item C<< "hostname" => $string >>

Applicable for CMode CLI.

=item C<< "address" => $string >>

Applicable for CMode CLI.

=item C<< "enabled" => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi isns stop" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub stop {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( status hostname address enabled
                    )
            ),
        },
        cli_cmd => 'vserver iscsi isns stop',
    );
    $Log->exit() if $may_exit;

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_stop_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_stop_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
        frontend => 'NACL::C::VserverIscsiIsns::stop',
    );

} ## end sub stop

sub _stop_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_iscsi_isns_stop',);
    $Log->exit() if $may_exit;
} ## end sub _stop_cmode_cli

sub _stop_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    $apiset->iscsi_isns_stop();
    $Log->exit() if $may_exit;
} ## end sub _stop_7mode_cli

=head2 update

   NACL::C::VserverIscsiIsns->update(
                                command_interface => $ci,
                                 vserver => $vserver,
                                  ...);

                                 or

    $VserverIscsiIsns_object->update(...);

(Class or instance method)
This method is used to force an update of the registration information with the iSNS server..

Uses a CMode CLI APISet, 7Mode CLI APISet.

Invokes "iscsi isns update" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "status" => $string >>

Applicable for CMode CLI.

=item C<< "hostname" => $string >>

Applicable for CMode CLI.

=item C<< "address" => $string >>

Applicable for CMode CLI.

=item C<< "enabled" => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi isns update" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub update {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( status hostname address enabled
                    )
            ),
        },
        cli_cmd => 'vserver iscsi isns update',
    );
    $Log->exit() if $may_exit;

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_update_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_update_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },

        ],
        frontend => 'NACL::C::VserverIscsiIsns::update',
    );

} ## end sub update

sub _update_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_iscsi_isns_update',);
    $Log->exit() if $may_exit;
} ## end sub _update_cmode_cli

sub _update_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    $apiset->iscsi_isns_update();
    $Log->exit() if $may_exit;
} ## end sub _update_7mode_cli

=head2 query

    NACL::C::VserverIscsiIsns->query(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                ...);

                                 or

    $VserverIscsiIsns_obj->query(...);

(Class or instance method)
This method is used to force update of registered iSNS information.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi isns query" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub query {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'vserver iscsi isns query',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_query_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverIscsiIsns::query',
    );

    $Log->exit() if $may_exit;
} ## end sub query

sub _query_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_iscsi_isns_query',);
    $Log->exit() if $may_exit;
} ## end sub _query_cmode_cli

=head2 modify

    NACL::C::VserverIscsiIsns->modify(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                ...);

                                 or

    $VserverIscsiIsns_obj->modify(...);

(Class or instance method)
This method is used to modify a Vserver's iSNS service.

Uses a CMode CLI/ZAPI APISet.

Invokes "iscsi-isns-config,iscsi-isns-start,iscsi-isns-stop" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< 'status-admin' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

Invokes iscsi-isns-start API if status-admin option passed in is "up".

Invokes iscsi-isns-stop API if status-admin option passed in is "down".

=item C<< 'address' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: isns-ip-addr

=item C<< 'force' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: force

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi isns modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( status-admin address force
                    )
            ),
        },
        cli_cmd => 'vserver iscsi isns modify',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverIscsiIsns::modify',
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_iscsi_isns_modify',);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $vserver           = delete $opts{vserver};

    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw( force )],
        map    => {'address' => 'isns-ip-addr'},
        target => \%cmode_args,
    );

    $pkg_or_obj->_handle_zapi_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $vserver,
        command_interface => $command_interface,
    );

    if (defined $opts{'address'}) {
        $apiset->iscsi_isns_config(%cmode_args);
    }

    if (defined $opts{'status-admin'}) {
        delete $cmode_args{'force'};
        if ($opts{'status-admin'} eq "up") {
            $apiset->iscsi_isns_start(%cmode_args);
        } elsif ($opts{'status-admin'} eq "down") {
            $apiset->iscsi_isns_stop(%cmode_args);
        } else {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw(
                "Invalid value  passed for modify method\n");
        }
    }
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

=head2 delete

    NACL::C::VserverIscsiIsns->delete(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                ...);

                                 or

    $VserverIscsiIsns_obj->delete(...);

(Class or instance method)
This method is used to remove the configured iSNS server IP address.

Uses a CMode CLI/ZAPI APISet.

Invokes "iscsi-isns-destroy" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi isns delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'vserver iscsi isns delete',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverIscsiIsns::delete',
        nacl_exceptions => [ qw(DoesNotExist) ],
    );

    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_iscsi_isns_delete',);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@opts,
        api => 'iscsi_isns_destroy',);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 create

    my $VserverIscsiIsns_obj    = NACL::C::VserverIscsiIsns->create(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                ...);

(Class method)
This method is used to configure iSNS server IP address.

Uses a CMode CLI/ZAPI APISet.

Invokes "iscsi-isns-create" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< 'status-admin' => $string >>

Applicable for CMode CLI.

=item C<< 'address' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: isns-ip-addr

=item C<< 'force' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: force

=item C<< 'start' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: start

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi isns create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( status-admin address force start
                    )
            ),
        },
        cli_cmd => 'vserver iscsi isns create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverIscsiIsns::create',
        nacl_exceptions => [ qw(AlreadyExists) ],
    );
    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $vserver,
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'vserver_iscsi_isns_create',);
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(@opts,
        api   => 'iscsi_isns_create',
        copy  => [qw ( force start )],
        map   => {'address' => 'isns-ip-addr',},
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

=head2 disable

    NACL::C::VserverIscsiIsns->disable(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to disable isns capbility.

Uses a CMode CLI/ZAPI APISet.

Invokes "iscsi-disable" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi isns disable" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub disable {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        api                 => 'vserver_iscsi_isns_disable',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_disable_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_disable_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverIscsiIsns::disable',
    );

    $Log->exit() if $may_exit;
} ## end sub disable

sub _disable_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'vserver_iscsi_isns_disable',
    );
    $Log->exit() if $may_exit;
} ## end sub _disable_cmode_cli

sub _disable_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    $apiset->iscsi_disable();
    $Log->exit() if $may_exit;
} ## end sub _disable_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('vserver' => {type => SCALAR | UNDEF, optional => 1},);
}

1;
