# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverIscsiTpgroup Component Module
## @author tanil@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverIscsiTpgroup

=head1 DESCRIPTION

C<NACL::C::VserverIscsiTpgroup> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverIscsiTpgroup.  A related class is
L<NACL::CS::VserverIscsiTpgroup|lib-NACL-CS-VserverIscsiTpgroup-pm>, which represents
the past state of an ONTAP VserverIscsiTpgroup.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 tpgroup

=head2 vserver

=cut

package NACL::C::VserverIscsiTpgroup;

use strict;
use warnings;

use NACL::CS::VserverIscsiTpgroup;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $VserverIscsiTpgroup = NACL::C::VserverIscsiTpgroup->new(command_interface=>$ci,
                                   tpgroup =>$tpgroup,
                                   vserver =>$vserver,
                                  );

Return a new VserverIscsiTpgroup component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'tpgroup',
    scalar => 'vserver',
    scalar => 'tpgroup_tag'
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $VserverIscsiTpgroup  = NACL::C::VserverIscsiTpgroup->find(command_interface => $ci, ...);
    my @VserverIscsiTpgroups = NACL::C::VserverIscsiTpgroup->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $VserverIscsiTpgroup_state = $NACL::C::VserverIscsiTpgroup->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverIscsiTpgroup would return
a NACL::CS::VserverIscsiTpgroup object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 modify

   NACL::C::VserverIscsiTpgroup->modify(
                                command_interface => $ci,
                                 tpgroup => $tpgroup,
                                 vserver => $vserver,
                                  ...);

                                 or

    $VserverIscsiTpgroup_object->modify(...);

(Class or instance method)
This method is used to modify a user-defined target portal group.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "tpgroup" => $tpgroup >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI

=item C<< "name" => $string >>

  Applicable only for CMode CLI

=item C<< "userDefined" => $string >>

  Applicable only for CMode CLI

=item C<< "force" => $string >>

  Applicable only for CMode CLI

=item C<< "lif" => [ $lif1, $lif2, ... ] >>

  Applicable only for CMode CLI

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi tpgroup modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "lif" => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( name userDefined force
                    )
            ),
        },
        api => 'vserver_iscsi_tpgroup_modify',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverIscsiTpgroup::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli{
    $Log->enter() if $may_enter;
    my($pkg_or_obj, %opts) = @_;
    my $vserver = $opts{vserver};

    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api => 'vserver_iscsi_tpgroup_modify',
        vserver_operated_on => \$vserver,
    );

    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

=head2 create

    my $VserverIscsiTpgroup_object    = NACL::C::VserverIscsiTpgroup->create(
                                command_interface => $ci,
                                 tpgroup => $tpgroup,
                                 vserver => $vserver,
                                  ...);

(Class method)
This method is used to create a user-defined target portal group.

Uses a CMode CLI APISet or CMode ZAPI APISet or 7Mode CLI APISet.
  Invokes 'iscsi-tpgroup-create' API for CMode ZAPI.
  Invokes 'iscsi tpgroup create' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "tpgroup" => $tpgroup >>

(Required)
  Applicable for CMode CLI/ZAPI,7Mode CLI
  Maps to:
    CM ZAPI : tpgroup-name
    7M CLI  : tpgroup name (Programmatic option: 'tpgroup-name')

=item C<< "vserver" => $vserver >>

(Required)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : ZAPI will be invoked in the context of the vserver specified to
              this option.

=item C<< "name" => $string >>

  Applicable only for CMode CLI

=item C<< "tpgtag" => $string >>

  Applicable only for CMode CLI

=item C<< "tpgroup-tag" => $string >>

  Applicable for CMode CLI/ZAPI, 7Mode CLI
  Maps to :
    CM ZAPI : tpgroup-tag
    7M CLI  : -t <value> (Programmatic name: tpgroup-tag)

=item C<< "userDefined" => $string >>

  Applicable only for CMode CLI

=item C<< "force" => $string >>

  Applicable only for CMode CLI

=item C<< "lif" => [ $lif1, $lif2, ... ] >>

Applicable for CMode CLI, 7Mode CLI
Maps to :
    7M CLI : interface name (Programmatic option: 'interface')

=item C<< "extended_query" => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi tpgroup create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        only_static_method => 1,
        params             => \@args,
        additional_spec    => {
            "lif" => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( name tpgtag userDefined force tpgroup-tag
                    )
            ),
        },
        cli_cmd => 'vserver iscsi tpgroup create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_create_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        frontend => 'NACL::C::VserverIscsiTpgroup::create',
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        tpgroup           => $opts{'tpgroup'},
        vserver           => $opts{vserver},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,
        api     => 'vserver_iscsi_tpgroup_create',
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $allowed_cmode_options = [qw(tpgroup-tag)];
    $pkg->_base_backend_cmode_zapi(@args,
        api    => 'iscsi_tpgroup_create',
        copy   => $allowed_cmode_options,
        map    => {"tpgroup" => "tpgroup-name"},
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

sub _create_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};

    my $apiset               = delete $opts{apiset};
    my $allowed_mode_options = [qw(tpgroup-tag)];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_mode_options,
        target => \%cmode_args,
        map    => {
            'tpgroup' => 'tpgroup-name',
            'lif'     => 'interface',

        },
    );

    foreach my $array_element (qw(interface)) {
        if (defined $opts{$array_element}) {
            $opts{$array_element} = join ',', @{$opts{$array_element}};
        }
    }

    $apiset->iscsi_tpgroup_create(%cmode_args);
    $Log->exit() if $may_exit;
    return $pkg->new(
        command_interface => $command_interface,
        tpgroup           => $opts{'tpgroup'},
        vserver           => $opts{'vserver'},
    );
} ## end sub _create_7mode_cli

=head2 remove

   NACL::C::VserverIscsiTpgroup->remove(
                                command_interface => $ci,
                                 tpgroup => $tpgroup,
                                 vserver => $vserver,
                                  ...);

                                 or

    $VserverIscsiTpgroup_object->remove(...);

(Class or instance method)
This method is used to remove LIFs to a user-defined target portal group.

Uses a CMode CLI APISet or CMode ZAPI APISet or 7Mode CLI APISet.
  Invokes 'iscsi-tpgroup-interface-delete' API for CMode ZAPI.
  Invokes 'iscsi tpgroup remove' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "tpgroup" => $tpgroup >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7Mode CLI: tpgroup name (Programmatic option: 'tpgroup-name')

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< "name" => $string >>

  Applicable only for CMode CLI

=item C<< "force" => $string >>

  Applicable only for CMode CLI

=item C<< "lif" => [ $lif1, $lif2, ... ] >>

  Applicable for CMode CLI, 7Mode CLI
  Maps to :
    7M CLI : interface name (Programmatic option: 'interface')

=item C<< "tpgroup-tag" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : tpgroup-tag

=item C<< "extended_query" => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi tpgroup remove" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub remove {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "lif" => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( name force tpgroup-tag
                    )
            ),
        },
        cli_cmd => 'vserver iscsi tpgroup remove',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_remove_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_remove_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_remove_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        frontend => 'NACL::C::VserverIscsiTpgroup::remove',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub remove

sub _remove_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,
        api     => 'vserver_iscsi_tpgroup_remove',
    );
    $Log->exit() if $may_exit;
} ## end sub _remove_cmode_cli

sub _remove_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $vserver           = delete $opts{vserver};
    my $tpgroup           = delete $opts{tpgroup};

    my $allowed_cmode_options = [qw( tpgroup-tag )];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    %opts = %cmode_args;
    $pkg_or_obj->_handle_zapi_vserver_context(
        api_opts          => \%opts,
        vserver           => $vserver,
        command_interface => $command_interface
    );

    my $tpgroup_tag = $pkg_or_obj->_get_tpgroup_tag(
        vserver           => $vserver,
        command_interface => $command_interface,
        tpgroup           => $tpgroup
    );

    $opts{'tpgroup-tag'} = $tpgroup_tag;

    foreach my $array_element (@{$opts{lif}}) {
        $apiset->iscsi_tpgroup_interface_delete(%opts,
            'interface-name' => $array_element);
    }
    $Log->exit() if $may_exit;
} ## end sub _remove_cmode_zapi

sub _remove_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %mode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%mode_args,
        map    => {
            'tpgroup' => 'tpgroup-name',
            'lif'     => 'interface',
        },
    );
    %opts = %mode_args;

    foreach my $array_element (qw(interface)) {
        if (defined $opts{$array_element}) {
            $opts{$array_element} = join ',', @{$opts{$array_element}};
        }
    }

    $apiset->iscsi_tpgroup_remove(%opts);
    $Log->exit() if $may_exit;
} ## end sub _remove_7mode_cli

=head2 destroy

   NACL::C::VserverIscsiTpgroup->destroy(
                                command_interface => $ci,
                                 tpgroup => $tpgroup,
                                 vserver => $vserver,
                                  ...);

                                 or

    $VserverIscsiTpgroup_object->destroy(...);

(Class or instance method)
This method is used to destroy a user-defined target portal group.

Uses a CMode CLI APISet or 7Mode CLI APISet.
  Invokes 'iscsi tpgroup destroy' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "tpgroup" => $tpgroup >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI, 7Mode CLI
  Maps to :
    7Mode CLI : tpgroup name (Programmatic option: 'tpgroup-name')

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI

=item C<< "name" => $string >>

  Applicable only for CMode CLI

=item C<< "force" => $string >>

  Applicable only for CMode CLI

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi tpgroup destroy" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub destroy {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( name force
                    )
            ),
        },
        cli_cmd => 'vserver iscsi tpgroup destroy',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_destroy_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_destroy_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        frontend => 'NACL::C::VserverIscsiTpgroup::destroy',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub destroy

sub _destroy_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'vserver_iscsi_tpgroup_destroy',
    );
    $Log->exit() if $may_exit;
} ## end sub _destroy_cmode_cli

sub _destroy_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %mode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%mode_args,
        map    => {'tpgroup' => 'tpgroup-name'},
    );
    %opts = %mode_args;

    $apiset->iscsi_tpgroup_destroy(%opts);
    $Log->exit() if $may_exit;
} ## end sub _destroy_7mode_cli

=head2 delete

   NACL::C::VserverIscsiTpgroup->delete(
                                command_interface => $ci,
                                     tpgroup => $tpgroup,
                                     vserver => $vserver,
                                  ...);

                                                                 or

    $VserverIscsiTpgroup_object->delete(...);

(Class or instance method)
This method is used to delete a user-defined target portal group.

Uses a CMode CLI APISet or CMode ZAPI APISet or 7Mode CLI APISet.
  Invokes 'iscsi-tpgroup-destroy' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "tpgroup" => $tpgroup >>

(Required for class method, Not Applicable for instance method)
  Applicable only for CMode CLI

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< "force" => $string >>

  Applicable only for CMode CLI

=item C<< "tpgroup-tag" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : tpgroup-tag

=item C<< "extended_query" => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi tpgroup delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( force tpgroup-tag
                    )
            ),
        },
        cli_cmd => 'vserver iscsi tpgroup delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_destroy_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                check     => '_delete_zapi_check',
            },
        ],
        frontend => 'NACL::C::VserverIscsiTpgroup::delete',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'vserver_iscsi_tpgroup_delete',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $vserver           = delete $opts{vserver};
    my $tpgroup           = delete $opts{tpgroup};

    my $allowed_cmode_options = [qw( tpgroup-tag )];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    %opts = %cmode_args;
    $pkg_or_obj->_handle_zapi_vserver_context(
        api_opts          => \%opts,
        vserver           => $vserver,
        command_interface => $command_interface
    );

    my $tpgroup_tag = $pkg_or_obj->_get_tpgroup_tag(
        vserver           => $vserver,
        command_interface => $command_interface,
        tpgroup           => $tpgroup
    );

    $opts{'tpgroup-tag'} = $tpgroup_tag;

    $apiset->iscsi_tpgroup_destroy(%opts);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

sub _delete_zapi_check {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my @fields_to_check = ('tpgroup');
    $pkg_or_obj->_fallback_for_relational_regex(%opts, fields_to_check => \@fields_to_check);

    $Log->exit() if $may_exit;
} ## end sub _delete_zapi_check    

=head2 add

   NACL::C::VserverIscsiTpgroup->add(
                                command_interface => $ci,
                                 tpgroup => $tpgroup,
                                 vserver => $vserver,
                                  ...);

                                 or

    $VserverIscsiTpgroup_object->add(...);

(Class or instance method)
This method is used to add LIFs to a user-defined target portal group.

Uses a CMode CLI APISet or CMode ZAPI APISet or 7Mode CLI APISet.
  Invokes 'iscsi-tpgroup-interface-add' API for CMode ZAPI.
  Invokes 'iscsi tpgroup add' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "tpgroup" => $tpgroup >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7Mode CLI: tpgroup name (Programmatic option: 'tpgroup-name')

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
    Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< "name" => $string >>

  Applicable only for CMode CLI

=item C<< "force" => $string >>

  Applicable only for CMode CLI

=item C<< "lif" => [ $lif1, $lif2, ... ] >>

  Applicable for CMode CLI, 7Mode CLI
  Maps to:
    7Mode CLI: interface name (Programmatic option: 'interface')

=item C<< "tpgroup-tag" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : tpgroup-tag

=item C<< "extended_query" => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi tpgroup add" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "lif" => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( name force tpgroup-tag
                    )
            ),
        },
        cli_cmd => 'vserver iscsi tpgroup add',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_add_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_add_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_add_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        frontend => 'NACL::C::VserverIscsiTpgroup::add',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub add

sub _add_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'vserver_iscsi_tpgroup_add',
    );
    $Log->exit() if $may_exit;
} ## end sub _add_cmode_cli

sub _add_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my $vserver           = delete $opts{vserver};
    my $tpgroup           = delete $opts{tpgroup};

    my $allowed_cmode_options = [qw( tpgroup-tag )];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    %opts = %cmode_args;
    $pkg_or_obj->_handle_zapi_vserver_context(
        api_opts          => \%opts,
        vserver           => $vserver,
        command_interface => $command_interface
    );

    my $tpgroup_tag = $pkg_or_obj->_get_tpgroup_tag(
        vserver           => $vserver,
        command_interface => $command_interface,
        tpgroup           => $tpgroup
    );

    $opts{'tpgroup-tag'} = $tpgroup_tag;

    foreach my $array_element (@{$opts{lif}}) {
        $apiset->iscsi_tpgroup_interface_add(%opts,
            'interface-name' => $array_element,);
    }
    $Log->exit() if $may_exit;
} ## end sub _add_cmode_zapi

sub _add_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %mode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%mode_args,
        map    => {
            'tpgroup' => 'tpgroup-name',
            'lif'     => 'interface',
        },
    );
    %opts = %mode_args;

    foreach my $array_element (qw(interface)) {
        if (defined $opts{$array_element}) {
            $opts{$array_element} = join ',', @{$opts{$array_element}};
        }
    }

    $apiset->iscsi_tpgroup_add(%opts);
    $Log->exit() if $may_exit;
} ## end sub _add_7mode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'tpgroup' => {type => SCALAR},
        'vserver' => {type => SCALAR | UNDEF, optional => 1},
    );
} ## end sub _primary_keys_validate_spec

sub _get_tpgroup_tag {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate(
        @_,
        {   command_interface => {isa  => 'NACL::C::CommandInterface::ONTAP'},
            vserver           => {type => SCALAR | UNDEF},
            tpgroup           => {type => SCALAR}
        }
    );
    if ($pkg_or_obj->tpgroup_tag()) {
        $Log->exit() if $may_exit;
        return $pkg_or_obj->tpgroup_tag();
    } else {
        my $ci = delete $opts{command_interface};
        my $cs = NACL::CS::VserverIscsiTpgroup->fetch(
            command_interface => $ci,
            filter            => \%opts,
            requested_fields  => [qw(tpgroup-tag)],
            apiset_should     => {interface => 'ZAPI'}
        );
        my $tag = $cs->tpgroup_tag();
        $Log->debug("tpgroup-tag value is $tag");
        if (ref $pkg_or_obj) {
            $pkg_or_obj->tpgroup_tag($tag);
        }
        $Log->exit() if $may_exit;
        return $tag;
    } ## end else [ if ( $pkg_or_obj->tpgroup_tag...
} ## end sub _get_tpgroup_tag

1;
