# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverLocks Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverLocks

=head1 DESCRIPTION

C<NACL::C::VserverLocks> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverLocks.  A related class is
L<NACL::CS::VserverLocks|lib-NACL-CS-VserverLocks-pm>, which represents
the past state of an ONTAP VserverLocks.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 volume

=head2 vserver

=head2 path

Object Path. If this field is present, then lockid will not be present.

=head2 lockid

The Lock UUID. If this field is present, then none of volume, path or lif will
be present

=head2 lif

Logical Interface. If this field is present, then lockid will not be present.

=head2 type

The type of lock.

=cut

package NACL::C::VserverLocks;

use strict;
use warnings;

use NACL::CS::VserverLocks;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NACL::C::Exceptions::VserverLocks::DoesNotExist qw(:try);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use Params::Validate qw(:all);

=head1 METHODS

=head2 new

  my $VserverLocks = NACL::C::VserverLocks->new(
                                   command_interface => $ci,
                                   'volume' => $volume,
                                   'vserver' => $vserver,
                                   'lif' => $lif,
                                  );
                   or
  my $VserverLocks = NACL::C::VserverLocks->new(command_interface => $ci,
                                                vserver           => $vserver,
                                                lockid            => $lockid);

Return a new VserverLocks component object with the given attributes.
(either (volume + path + lif) or (lockid) can be provided but not all of them
at the same time)

=cut

use Class::MethodMaker [
    scalar => 'volume',
    scalar => 'vserver',
    scalar => 'path',
    scalar => 'lockid',
    scalar => 'lif',
    scalar => 'type',
];

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $VserverLocks_obj  = NACL::C::VserverLocks->find(command_interface => $ci, ...);
    my @VserverLocks_objs = NACL::C::VserverLocks->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my VserverLocks_state = $VserverLocks_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverLocks would return
a NACL::CS::VserverLocks object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;
    $self->SUPER::init(%opts);
    $self->_vserver_context_init(%opts);
    if (   exists $opts{volume}
        && exists $opts{path}
        && exists $opts{lif}
        && exists $opts{lockid})
    {
        NATE::Exceptions::Argument->throw('Either (volume + path + lif) or '
                . 'lockid can be provided, but we cannot provide them all '
                . 'together');
    } elsif (!exists $opts{volume}
        && !exists $opts{path}
        && !exists $opts{lif}
        && !exists $opts{lockid})
    {
        NATE::Exceptions::Argument->throw('Either (volume + path + lif) or '
                . 'lockid should be provided, but none were');
    } elsif (exists $opts{lockid}
        && (exists $opts{volume} || exists $opts{path} || exists $opts{lif}))
    {
        NATE::Exceptions::Argument->throw(
                  "We cannot provide any of 'volume', "
                . "'path' or 'lif' when 'lockid' is provided");
    } elsif (!exists $opts{lockid}
        && !(exists $opts{volume} && exists $opts{path} && exists $opts{lif}))
    {
        NATE::Exceptions::Argument->throw(
            "All of 'volume' and 'path' and 'lif' should be provided");
    } elsif (exists $opts{vserver} && exists $opts{lockid}) {
        NATE::Exceptions::Argument->throw(
            'Either vserver or lockid can be provided, but we cannot provide both'
        );
    }
    $Log->exit() if $may_exit;
}

sub find {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my $pkg = $pkg_or_obj->get_package_name();
    # If invoked with an object, extract the primary keys and send that
    # as the filter. Also extract the command interface.
    $pkg_or_obj->_extract_from_obj_for_find(arr_ref => \@args);
    my %opts = validate @args, $pkg->_find_validate_spec();
    my @requested_fields = (
        $pkg->get_primary_keys(
            command_interface => $opts{command_interface}
        )
    );
    
    my @state_objs
        = NACL::CS::VserverLocks->fetch( %opts, requested_fields => \@requested_fields );
    foreach my $state_obj (@state_objs) {
        if (exists $state_obj->{volume}
            && exists $state_obj->{path}
            && exists $state_obj->{lif}
            && exists $state_obj->{lockid}) {
            delete $state_obj->{lockid};
        }
    } 

   my @components;
    foreach my $state_obj (@state_objs) {
        my %new_hash;
        foreach my $requested_field (@requested_fields) {
            my $value = $state_obj->$requested_field();
            $new_hash{$requested_field} = $value if ( defined $value );
        }
        push @components,
            $pkg->new( %new_hash,
            command_interface => $opts{command_interface}, );
    }
    
    return wantarray ? @components : $components[0];
} ## end sub find

=head2 break

    NACL::C::VserverLocks->break(
                                command_interface => $ci,
                                'volume' => $volume,
                                'vserver' => $vserver,
                                ...);

                                 or

    $VserverLocks_obj->break(...);

(Note that either of (volume + path + lif) or (lockid) must be provided. All
parameters should not be provided at the same time.)

(Class or instance method)
This method is used to break file locks based on a set of criteria.
Uses CMode CLI/ZAPI .
Invokes  lock-break-iter for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)
Volume

Applicable for CMode CLI/ZAPI

CMode ZAPI :
 <lock-info volume="$value" />


=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver

Applicable for CMode CLI/ZAPI

CMode ZAPI :
 <lock-info vserver="$value" />


=item C<< 'path' => $path >>

(Required for class method, Not Applicable for instance method)
Object Path

Applicable for CMode CLI/ZAPI

CMode ZAPI :
 <lock-info path="$value" />

=item C<< 'lockid' => $lockid >>

(Required for class method, Not Applicable for instance method)
Lock UUID

Applicable for CMode CLI/ZAPI

CMode ZAPI :
 <lock-info lockid="$value" />
(Not Applicable for instance method, should not be
provided if (volume + path + lif) are provided)

=item C<< 'lif' => $lif >>

(Required for class method, Not Applicable for instance method)
Logical Interface

Applicable for CMode CLI/ZAPI

CMode ZAPI :
 <lock-info lif="$value" />


=item C<< '*' >>

This method takes all other options supported by "vserver locks break" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub break {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        cli_cmd         => 'vserver locks break',
        additional_spec => {_optional_scalars(qw(vserver)),},
    );
    try {
        my $vserver;
    unless ( defined $opts{lockid}){
        $opts{vserver_operated_on} = \$vserver;
    }
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_break_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_break_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::VserverLocks::break',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception     = shift;
        my $doesnot_exist = 'NACL::C::Exceptions::VserverLocks::DoesNotExist';
        NACL::C::Exceptions::VserverLocks::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
}

sub _break_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts, api => 'vserver_locks_break',
    );
    $Log->exit() if $may_exit;
}

sub _break_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(lockid volume vserver path lif)];
    $pkg_or_obj->_iter_helper(
        @opts,
        api  => 'lock-break-iter',
        copy => $copy,
        does_not_exist_exception =>
            'NACL::C::Exceptions::VserverLocks::DoesNotExist',
        'show-cmd' => 'vserver locks show',
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
       'volume'  => {type => SCALAR, optional => 1},
      'vserver' => {type => SCALAR, optional => 1},
       'path'    => {type => SCALAR, optional => 1},
        'lockid'  => {type => SCALAR, optional => 1},
       'lif'     => {type => SCALAR, optional => 1},
       'type'    => {type => SCALAR, optional => 1},
    );
}

1;
