# $Id: //depot/prod/test/nacldev/lib/NACL/C/VserverPeerTransition.pm#15 $

#
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary VserverPeerTransition Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status public
## @pod here

=head1 NAME

NACL::C::VserverPeerTransition

=head1 DESCRIPTION

C<NACL::C::VserverPeerTransition> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverPeerTransition.  A related class is
L<NACL::CS::VserverPeerTransition|lib-NACL-CS-VserverPeerTransition-pm>, which represents
the past state of an ONTAP VserverPeerTransition.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 src_filer_name

=head2 local_vserver

=cut

package NACL::C::VserverPeerTransition;

use strict;
use warnings;

use NACL::CS::VserverPeerTransition;
use NACL::ComponentUtils qw (_optional_scalars Dumper
    get_hash_of_real_field_names
);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF OBJECT);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::C::_Mixins::VserverPeerTransition qw(:all);
use NACL::C::Exceptions::VserverPeerTransition::DoesNotExist;
use NATE::Exceptions::Argument qw(:try);

=head1 METHODS

=head2 new

  my $VserverPeerTransition = NACL::C::VserverPeerTransition->new(
                                   command_interface => $ci,
                                   'src_filer_name' => $src_filer_name,
                                   'local_vserver' => $local_vserver,
                                  );

Return a new VserverPeerTransition component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'src_filer_name',
    scalar => 'local_vserver',

    # supported for backwards compatibility
    scalar => [
        {   '-read_cb' => sub {
                return (defined $_[1]) ? $_[1] : $_[0]->src_filer_name;
                }
        },
        'srcFilerName'
    ],
    scalar => [
        {   '-read_cb' => sub {
                return (defined $_[1]) ? $_[1] : $_[0]->local_vserver;
                }
        },
        'localVserver'
    ],
];

=head2 find

    my $VserverPeerTransition_obj  = NACL::C::VserverPeerTransition->find(command_interface => $ci, ...);
    my @VserverPeerTransition_objs = NACL::C::VserverPeerTransition->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'vserver-peer-transition-get-iter' api for CMode ZAPI.

=head2 state

    my VserverPeerTransition_state = $VserverPeerTransition_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverPeerTransition would return
a NACL::CS::VserverPeerTransition object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 create

    my $VserverPeerTransition_obj    = NACL::C::VserverPeerTransition->create(
                                command_interface => $ci,
                                'src-filer-name' => $src_filer_name,
                                'local-vserver' => $local_vserver,
                                ...);

(Class method)
This method is used to create a new Transition peering relationship.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'vserver-peer-transition-create' api for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'src-filer-name' => $src_filer_name >>

(Required)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : src-filer-name

=item C<< 'local-vserver' => $local_vserver >>

(Required)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : local-vserver

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'multi-path-address' => $string >>

Alternate IP Address for Source Filer
  Applicable for CMode CLI

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver peer transition create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( alternate-ip multi-path-address
                    )
            )
        },
        cli_cmd => 'vserver peer transition create',
    );
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverPeerTransition::create',
        nacl_exceptions => [qw(AlreadyExists)],
    );
    my $target = $pkg->_hash_copy(
        source => \%opts,
        copy   => [$pkg->get_primary_keys()]
    );

    $pkg->_process_options(options => $target);
    $Log->exit() if $may_exit;
    return $pkg->new(%$target, command_interface => $opts{command_interface});

} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,
        api => 'vserver_peer_transition_create',
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_zapi(%opts,
        api  => 'vserver_peer_transition_create',
        map  => $pkg->_zapi_option_map(
            command_interface => $opts{command_interface}
        ),
    );
    $Log->exit;
} ## end sub _create_cmode_zapi

=head2 delete

    NACL::C::VserverPeerTransition->delete(
                                command_interface => $ci,
                                'src-filer-name' => $src_filer_name,
                                'local-vserver' => $local_vserver,
                                ...);

                                 or

    $VserverPeerTransition_obj->delete(...);

(Class or instance method)
This method is used to delete a Transition peering relationship.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'vserver-peer-transition-destroy-iter' api for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'src-filer-name' => $src_filer_name >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : src-filer-name

=item C<< 'local-vserver' => $local_vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : local-vserver

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver peer transition delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::VserverPeerTransition::DoesNotExist>

This type of exception gets thrown if an attempt is made to delete a
non-existing vserver peer transition 

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        cli_cmd    => 'vserver peer transition delete',
    );
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverPeerTransition::delete',
        nacl_exceptions => [qw(DoesNotExist)],
    );
    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
     $Log->enter() if $may_enter;
     my ($pkg_or_obj, @args) = @_;

     $pkg_or_obj->_base_backend_cmode_cli(
         @args,
         api => 'vserver_peer_transition_delete',
     );

     $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_validate_backend_opts(@args);

    $pkg_or_obj->_iter_helper(
        @args,
        api      => 'vserver-peer-transition-destroy-iter',
        show_cmd => 'vserver peer transition show',
        map      => $pkg_or_obj->_zapi_option_map(
            command_interface => $opts{command_interface}
        ),
        does_not_exist_error => "entry doesn't exist",
        does_not_exist_exception =>
            'NACL::C::Exceptions::VserverPeerTransition::DoesNotExist'
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 modify

    NACL::C::VserverPeerTransition->modify(
                                command_interface => $ci,
                                'src-filer-name' => $src_filer_name,
                                'local-vserver' => $local_vserver,
                                ...);

                                 or

    $VserverPeerTransition_obj->modify(...);

(Class or instance method)
This method is used to modify a Transition peering relationship.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'vserver-peer-transition-modify' api for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'src-filer-name' => $src_filer_name >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : src-filer-name

=item C<< 'local-vserver' => $local_vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : local-vserver

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'multi-path-address' => $string >>

  Applicable for CMode CLI

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver peer transition modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( alternate-ip multi-path-address
                    )
            )
        },
        cli_cmd => 'vserver peer transition modify',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverPeerTransition::modify',
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api  => 'vserver_peer_transition_modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
        api    => 'vserver_peer_transition_modify',
        map    => $pkg_or_obj->_zapi_option_map(
            command_interface => $opts{command_interface}
        ),
        copy   => ['alternate-ip'],
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate(
        @args,
        {   command_interface => {
                type     => OBJECT,
                isa      => 'NACL::C::CommandInterface::ONTAP',
                optional => 1
            }
        }
    );
    my $command_interface = $opts{command_interface};

    my %default_spec = (
        'srcFilerName'   => {type => SCALAR,          optional => 1},
        'localVserver'   => {type => SCALAR | OBJECT, optional => 1},
        'src-filer-name' => {type => SCALAR,          optional => 1},
        'local-vserver'  => {type => SCALAR | OBJECT, optional => 1},
    );

    if (defined $command_interface) {
        if ($command_interface->has_uichange(
                uichange => "vserver-peer-transition"
            )
            )
        {
            $Log->exit() if $may_exit;
            return (
                'src-filer-name' => {type => SCALAR},
                'local-vserver'  => {type => SCALAR | OBJECT},
            );
        } else {
            $Log->exit() if $may_exit;
            return (
                'srcFilerName' => {type => SCALAR},
                'localVserver' => {type => SCALAR | OBJECT},
            );
        }
    } else {
        $Log->exit() if $may_exit;
        return %default_spec;
    }
} ## end sub _primary_keys_validate_spec

sub init {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;
    $self->SUPER::init(%opts);
    my %hash = get_hash_of_real_field_names(attributes => [keys %opts]);
    my %new_opts = ();
    foreach my $given_option (keys %hash) {
        $new_opts{$hash{$given_option}} = $opts{$given_option};
    }
    $self->_check_for_uichange_keys(%new_opts);
    $Log->exit() if $may_exit;
} ## end sub init

sub _common_validate_with {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    %opts = $pkg_or_obj->SUPER::_common_validate_with(%opts);
    $pkg_or_obj->_check_for_uichange_keys(%opts);
    $Log->exit() if $may_exit;
    return %opts;
}

# With this uichange "vserver-peer-transition"
# 'srcFilerName' is renamed to 'src-filer-name'
# 'localVserver' is renamed to 'local-vserver'
# Users can only pass either the old set or new set.
sub _check_for_uichange_keys {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my @dup = ();

    # Both should not be specified
    @dup = $pkg_or_obj->_validate_input_for_dup_keys(%opts);
    if (@dup) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "You can pass only either of the following arguments:\n"
                . join(', ', @dup));
    }

    # Atleast one of them should be specified
    if (!exists $opts{'srcFilerName'} && !exists $opts{'src-filer-name'}) {
        push(@dup, 'src-filer-name');
    }
    if (!exists $opts{'localVserver'} && !exists $opts{'local-vserver'}) {
        push(@dup, 'local-vserver');
    }
    if (@dup) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "Following arguments are mandatory:\n" . join(', ', @dup));
    }
    $Log->exit() if $may_exit;
}

# Ugh, so many uichanges in this feature!
# This overriden call_on_apiset handles converting the option
# 'alternate-ip' to 'multi-path-address' for both CLI and ZAPI.
sub call_on_apiset {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_handle_alternate_ip_change(
        command_interface => $opts{command_interface},
        opts              => \%opts
    );
    $Log->exit() if $may_exit;
    return $pkg_or_obj->SUPER::call_on_apiset(%opts);
}

1;
