# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverSecurityFileDirectoryPolicy Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, pkj@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverSecurityFileDirectoryPolicy

=head1 DESCRIPTION

C<NACL::C::VserverSecurityFileDirectoryPolicy> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverSecurityFileDirectoryPolicy.  A related class is
L<NACL::CS::VserverSecurityFileDirectoryPolicy|lib-NACL-CS-VserverSecurityFileDirectoryPolicy-pm>, which represents
the past state of an ONTAP VserverSecurityFileDirectoryPolicy.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=head2 policy_name

=cut

package NACL::C::VserverSecurityFileDirectoryPolicy;

use strict;
use warnings;

use NACL::CS::VserverSecurityFileDirectoryPolicy;
use NACL::C::Exceptions::VserverSecurityFileDirectoryPolicy::AlreadyExists ();
use NACL::C::Exceptions::VserverSecurityFileDirectoryPolicy::DoesNotExist  ();
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $VserverSecurityFileDirectoryPolicy = NACL::C::VserverSecurityFileDirectoryPolicy->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
                                   'policy-name' => $policy_name,
                                  );

Return a new VserverSecurityFileDirectoryPolicy component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'policy_name',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $VserverSecurityFileDirectoryPolicy_obj  = NACL::C::VserverSecurityFileDirectoryPolicy->find(command_interface => $ci, ...);
    my @VserverSecurityFileDirectoryPolicy_objs = NACL::C::VserverSecurityFileDirectoryPolicy->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my VserverSecurityFileDirectoryPolicy_state = $VserverSecurityFileDirectoryPolicy_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverSecurityFileDirectoryPolicy would return
a NACL::CS::VserverSecurityFileDirectoryPolicy object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 create

    my $VserverSecurityFileDirectoryPolicy_obj    = NACL::C::VserverSecurityFileDirectoryPolicy->create(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'policy-name' => $policy_name,
                                ...);

(Class method)
This method is used to create a file security policy.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'file-directory-security-policy-create' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< 'policy-name' => $policy_name >>

(Required)
Policy Name
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : policy-name

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverSecurityFileDirectoryPolicy::AlreadyExists>

This type of exception gets thrown if an attempt is made to create a
Policy that already exists.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver security file-directory policy create" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params   => \@args,
        cli_cmd  => 'vserver security file-directory policy create',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_create_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_create_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::VserverSecurityFileDirectoryPolicy::create'
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VserverSecurityFileDirectoryPolicy::AlreadyExists
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $vserver,
        policy_name       => $opts{'policy-name'},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'vserver_security_file_directory_policy_create',);
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api  => 'file-directory-security-policy-create',
        copy => [qw(policy-name)],
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

=head2 delete

    NACL::C::VserverSecurityFileDirectoryPolicy->delete(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'policy-name' => $policy_name,
                                ...);

                                 or

    $VserverSecurityFileDirectoryPolicy_obj->delete(...);

(Class or instance method)
This method is used to remove a file security policy.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'file-directory-security-policy-delete' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< 'policy-name' => $policy_name >>

(Required for class method, Not Applicable for instance method)
Policy Name
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : policy-name

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverSecurityFileDirectoryPolicy::DoesNotExist>

This type of exception gets thrown if an attempt is made to delete a
Policy which does not exists.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver security file-directory policy delete" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params   => \@args,
        cli_cmd  => 'vserver security file-directory policy delete',
    );
    try {

        my $vserver;
        $opts{vserver_operated_on} = \$vserver;
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_delete_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::VserverSecurityFileDirectoryPolicy::delete'
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VserverSecurityFileDirectoryPolicy::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_security_file_directory_policy_delete',);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api  => 'file-directory-security-policy-delete',
        copy => [qw(policy-name)], 
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver'     => {type => SCALAR},
        'policy-name' => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
