# Copyright (c) 2001-2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverSecurityTraceFilter Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverSecurityTraceFilter

=head1 DESCRIPTION

C<NACL::C::VserverSecurityTraceFilter> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverSecurityTraceFilter.  A related class is
L<NACL::CS::VserverSecurityTraceFilter|lib-NACL-CS-VserverSecurityTraceFilter-pm>, which represents
the past state of an ONTAP VserverSecurityTraceFilter.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 index

=head2 vserver

=cut

package NACL::C::VserverSecurityTraceFilter;

use strict;
use warnings;

use NACL::CS::VserverSecurityTraceFilter;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $VserverSecurityTraceFilter = NACL::C::VserverSecurityTraceFilter->new(
                                   command_interface => $ci,
                                   'index' => $index,
                                   'vserver' => $vserver,
                                  );

Return a new VserverSecurityTraceFilter component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'index',
    scalar => 'vserver',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $VserverSecurityTraceFilter_obj  = NACL::C::VserverSecurityTraceFilter->find(command_interface => $ci, ...);
    my @VserverSecurityTraceFilter_objs = NACL::C::VserverSecurityTraceFilter->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my VserverSecurityTraceFilter_state = $VserverSecurityTraceFilter_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverSecurityTraceFilter would return
a NACL::CS::VserverSecurityTraceFilter object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 delete

    NACL::C::VserverSecurityTraceFilter->delete(
                                command_interface => $ci,
                                'index' => $index,
                                'vserver' => $vserver,
                                ...);
                                 or

    $VserverSecurityTraceFilter_obj->delete(...);

(Class or instance method)
This method is used to delete a sectrace entry..

Uses a CMode CLI/ZAPI APISet.
  Invokes 'security-trace-filter-delete' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'index' => $index >>

(Required for class method, Not Applicable for instance method)
Filter Index
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : index

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver security trace filter delete" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params   => \@args,
        cli_cmd  => 'vserver security trace filter delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverSecurityTraceFilter::delete',
        nacl_exceptions => [qw(DoesNotExist)]
    );

    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_security_trace_filter_delete',);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(index)],
        api  => 'security_trace_filter_delete',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 create

    my $VserverSecurityTraceFilter_obj    = NACL::C::VserverSecurityTraceFilter->create(
                                command_interface => $ci,
                                'index' => $index,
                                'vserver' => $vserver,
                                ...);

(Class method)
This method is used to create a sectrace entry..

Uses a CMode CLI/ZAPI APISet.
  Invokes 'security-trace-filter-create' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'index' => $index >>

(Required)
Filter Index
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : index

=item C<< 'vserver' => $vserver >>

(Required)
Vserver
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< 'time-enabled' => $string >>

Minutes Filter is Enabled
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : time-enabled

=item C<< 'modified-time' => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : modified-time

=item C<< 'unix-name' => $string >>

UNIX User Name
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : unix-name

=item C<< 'win-path' => $string >>

Cifs Path
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : win-path

=item C<< 'client-ip' => $string >>

Client IP Address to match
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : client-ip

=item C<< 'trace-allow' => $string >>

Trace Allow Events, possible value(s) are yes,no
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : trace-allow

=item C<< 'windows-name' => $string >>

Windows User Name
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : windows-name

=item C<< 'enabled' => $string >>

Filter Enabled, possible value(s) are enabled,disabled
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : enabled

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< 'path' => $string >>

Path
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: path

=item C<< '*' >>

This method takes all other options supported by
"vserver security trace filter create" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_arrayrefs(qw(protocols)),
            _optional_scalars(
                qw( time-enabled modified-time unix-name win-path client-ip trace-allow
                    windows-name enabled

                    path
                    )
            ),
        },
        cli_cmd => 'vserver security trace filter create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverSecurityTraceFilter::create',
        nacl_exceptions => [qw(AlreadyExists)]
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        index             => $opts{'index'},
        vserver           => $vserver,
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_security_trace_filter_create',);
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [
        qw( time-enabled modified-time unix-name win-path client-ip trace-allow
            index windows-name enabled path protocols)
    ];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'security_trace_filter_create',
        copy => $copy,
	zapi_field_translations => { yes_no_boolean => [qw(trace-allow)],}
    );
    $Log->exit() if $may_exit;

} ## end sub _create_cmode_zapi

=head2 modify

    NACL::C::VserverSecurityTraceFilter->modify(
                                command_interface => $ci,
                                'index' => $index,
                                'vserver' => $vserver,
                                ...);

                                 or

    $VserverSecurityTraceFilter_obj->modify(...);

(Class or instance method)
This method is used to modify a sectrace entry..

Uses a CMode CLI/ZAPI APISet.
  Invokes 'security-trace-filter-modify' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'index' => $index >>

(Required for class method, Not Applicable for instance method)
Filter Index
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : index

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< 'time-enabled' => $string >>

Minutes Filter is Enabled
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : time-enabled

=item C<< 'modified-time' => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : modified-time

=item C<< 'unix-name' => $string >>

UNIX User Name
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : unix-name

=item C<< 'win-path' => $string >>

Cifs Path
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : win-path

=item C<< 'client-ip' => $string >>

Client IP Address to match
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : client-ip

=item C<< 'trace-allow' => $string >>

Trace Allow Events, possible value(s) are yes,no
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : trace-allow

=item C<< 'windows-name' => $string >>

Windows User Name
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : windows-name

=item C<< 'enabled' => $string >>

Filter Enabled, possible value(s) are enabled,disabled
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : enabled

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< 'path' => $string >>

Path
Applicable for CMode CLI/ZAPI
Maps to :
CM ZAPI : path

=item C<< '*' >>

This method takes all other options supported by
"vserver security trace filter modify" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_arrayrefs(qw(protocols)),
            _optional_scalars(
                qw( time-enabled modified-time unix-name win-path client-ip trace-allow
                    windows-name enabled

                    path
                    )
            ),
        },
        cli_cmd => 'vserver security trace filter modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverSecurityTraceFilter::modify',
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_security_trace_filter_modify',);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [
        qw( time-enabled modified-time unix-name win-path client-ip trace-allow
            index windows-name enabled protocols)
    ];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => $copy,
        api  => 'security_trace_filter_modify',
	zapi_field_translations => { yes_no_boolean => [qw(trace-allow)],}
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'index'   => {type => SCALAR},
        'vserver' => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

sub _optional_arrayrefs {
    my (@names) = @_;
    return (map { $_ => {type => ARRAYREF, optional => 1} } @names);
}
1;
