#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverServicesDnsHosts Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverServicesDnsHosts

=head1 DESCRIPTION

C<NACL::C::VserverServicesDnsHosts> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverServicesDnsHosts.  A related class is
L<NACL::CS::VserverServicesDnsHosts|lib-NACL-CS-VserverServicesDnsHosts-pm>, which represents
the past state of an ONTAP VserverServicesDnsHosts.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=head2 address

=cut

package NACL::C::VserverServicesDnsHosts;

use strict;
use warnings;

use NACL::CS::VserverServicesDnsHosts;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $VserverServicesDnsHosts = NACL::C::VserverServicesDnsHosts->new(command_interface=>$ci,
                                   vserver =>$vserver,
                                   address =>$address,
                                  );

Return a new VserverServicesDnsHosts component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'address',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $VserverServicesDnsHosts  = NACL::C::VserverServicesDnsHosts->find(command_interface => $ci, ...);
    my @VserverServicesDnsHostss = NACL::C::VserverServicesDnsHosts->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $VserverServicesDnsHosts_state = $NACL::C::VserverServicesDnsHosts->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverServicesDnsHosts would return
a NACL::CS::VserverServicesDnsHosts object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 modify

   NACL::C::VserverServicesDnsHosts->modify(
                                command_interface => $ci,
                                 vserver => $vserver,
                                 address => $address,
                                  ...);

                                 or

    $VserverServicesDnsHosts_object->modify(...);

(Class or instance method)
This method is used to modify hostname or aliases.

Uses a CMode CLI or CMode ZAPI APISet.
  Invokes 'net-hosts-modify' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< "address" => $address >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : host-ip-address

=item C<< "hostname" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : hostname

=item C<< "aliases" => [ $aliase1, $aliase2, ... ] >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : aliases

=item C<< "extended_query" => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver services dns hosts modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "aliases" => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( hostname
                    )
            ),
        },
        cli_cmd => 'vserver services dns hosts modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverServicesDnsHosts::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'vserver_services_dns_hosts_modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my $allowed_cmode_options = [qw( aliases hostname )];

    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api    => 'net_hosts_modify',
        copy   => $allowed_cmode_options,
        map    => {"address" => "host-ip-address"},
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

=head2 delete

   NACL::C::VserverServicesDnsHosts->delete(
                                command_interface => $ci,
                                 vserver => $vserver,
                                 address => $address,
                                  ...);

                                 or

    $VserverServicesDnsHosts_object->delete(...);

(Class or instance method)
This method is used to remove a host table entry.

Uses a CMode CLI or CMode ZAPI APISet.
  Invokes 'net-hosts-destroy' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< "address" => $address >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : host-ip-address

=item C<< extended_query => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver services dns hosts delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'vserver services dns hosts delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                check     => '_delete_zapi_check',
            },
        ],
        frontend => 'NACL::C::VserverServicesDnsHosts::delete',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'vserver_services_dns_hosts_delete',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api => 'net_hosts_destroy',
        map => {"address" => "host-ip-address"},
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

sub _delete_zapi_check {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my @fields_to_check = ('vserver','address');
    $pkg_or_obj->_fallback_for_relational_regex(%opts, fields_to_check => \@fields_to_check);

    $Log->exit() if $may_exit;
} ## end sub _delete_zapi_check    

=head2 create

    my $VserverServicesDnsHosts_object    = NACL::C::VserverServicesDnsHosts->create(
                                command_interface => $ci,
                                 vserver => $vserver,
                                 address => $address,
                                  ...);

(Class method)
This method is used to create a new host table entry.

Uses a CMode CLI or CMode ZAPI APISet.
  Invokes 'net-hosts-create' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< "address" => $address >>
(Required)
  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : host-ip-address

=item C<< "hostname" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : hostname

=item C<< "aliases" => [ $aliase1, $aliase2, ... ] >>

  Applicable for CMode CLI/ZAPI
  Maps to :
    CM ZAPI : aliases

=item C<< "extended_query" => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver services dns hosts create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "aliases" => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( hostname
                    )
            ),
        },
        cli_cmd => 'vserver services dns hosts create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverServicesDnsHosts::create',
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $opts{vserver},
        address           => $opts{'address'},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,
        api     => 'vserver_services_dns_hosts_create',
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $allowed_cmode_options = [qw( aliases hostname )];

    $pkg->_base_backend_cmode_zapi(@args,
        api  => 'net_hosts_create',
        copy => $allowed_cmode_options,
        map  => {"address" => "host-ip-address"},
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver' => {type => SCALAR},
        'address' => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
