# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverServicesKerberosRealm Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverServicesKerberosRealm

=head1 DESCRIPTION

"vserver services kerberos-realm" commands have got aliased to
"vserver nfs kerberos realm" commands. So usage of this component is deprecated.Use
L<NACL::C::VserverNfsKerberosRealm|lib-NACL-C-VserverNfsKerberosRealm-pm> instead.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 configname

=head2 vserver

=head2 realm

=cut

package NACL::C::VserverServicesKerberosRealm;

use strict;
use warnings;

use NACL::CS::VserverServicesKerberosRealm;
use NACL::ComponentUtils qw (_optional_scalars);
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF OBJECT);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $VserverServicesKerberosRealm = NACL::C::VserverServicesKerberosRealm->new(command_interface=>$ci,
                                   configname =>$configname,
                                  );

Return a new VserverServicesKerberosRealm component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'configname',
    scalar => 'vserver',
    scalar => 'realm',
];

=head2 find

Usage of this method is deprecated. It is recommended to use
   NACL::C::VserverNfsKerberosRealm->find(...); instead.

=head2 state

Usage of this method is deprecated. It is recommended to use
   NACL::C::VserverNfsKerberosRealm->state(...); instead.

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;
    $self->SUPER::init(%opts);
    if ($opts{command_interface}
        ->is_feature_supported(feature => 'vserver_kerberos_realm'))
    {
        $self->_vserver_context_init(%opts);
    }
    $Log->exit() if $may_exit;
}

=head2 modify

Usage of this method is deprecated. It is recommended to use
   NACL::C::VserverNfsKerberosRealm->modify(...); instead.

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts ) = @_;
    my $ret = $pkg_or_obj->_redirect_call(_method => 'modify', %opts);
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

=head2 create

Usage of this method is deprecated. It is recommended to use
   NACL::C::VserverNfsKerberosRealm->create(...); instead.

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, %opts ) = @_;
    my $obj = $pkg->_redirect_call(_method => 'create', %opts);
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

=head2 delete

Usage of this method is deprecated. It is recommended to use
   NACL::C::VserverNfsKerberosRealm->delete(...); instead.

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts ) = @_;
    my $ret = $pkg_or_obj->_redirect_call(_method => 'delete', %opts);
    $Log->exit() if $may_exit;
    return $ret;
}

sub _redirect_call {
    $Log->enter() if $may_enter;

    my ($pkg_or_obj, %opts) = @_;
    my $command = delete $opts{_method};
    $Log->warn("The command 'vserver services kerberos-realm $command' got aliased to "
                . "'vserver nfs kerberos realm $command'. Please use NACL::C::VserverNfsKerberosRealm->$command. ");
    if ( ref $pkg_or_obj ) {
        my @keys = $pkg_or_obj->_get_keys_names();
        push @keys, 'command_interface';
        foreach my $key ( @keys ) {
            my $key_isset = $key."_isset";
            if ( $pkg_or_obj->$key_isset() ) {
                $opts{$key} = $pkg_or_obj->$key();
            }
        }
    }
    require NACL::C::VserverNfsKerberosRealm;
    $Log->exit() if $may_exit;
    return NACL::C::VserverNfsKerberosRealm->$command(%opts);
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
my ($pkg_or_obj, @args) = @_;
    my %opts = validate(
        @args,
        {   command_interface => {
                type     => OBJECT,
                isa      => 'NACL::C::CommandInterface',
                optional => 1
            }
        }
    );
    my $command_interface = $opts{command_interface};
    my %default_spec      = (
        vserver    => {type => SCALAR, optional => 1},
        realm      => {type => SCALAR, optional => 1},
        configname => {type => SCALAR, optional => 1},
    );
    if (defined $command_interface) {
        my $vserverized = $command_interface->is_feature_supported(
            feature => 'vserver_kerberos_realm');
        if ($vserverized) {
            return (
                vserver => {type => SCALAR},
                realm   => {type => SCALAR},
            );
        } else {
            return (configname => {type => SCALAR},);
        }
    } else {
        return %default_spec;
    }
}

1;
