# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverVscan Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverVscan

=head1 DESCRIPTION

C<NACL::C::VserverVscan> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverVscan.  A related class is
L<NACL::CS::VserverVscan|lib-NACL-CS-VserverVscan-pm>, which represents
the past state of an ONTAP VserverVscan.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=cut

package NACL::C::VserverVscan;

use strict;
use warnings;

use NACL::CS::VserverVscan;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $VserverVscan = NACL::C::VserverVscan->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
                       );

Return a new VserverVscan component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'vserver',];

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $VserverVscan_obj  = NACL::C::VserverVscan->find(command_interface => $ci, ...);
    my @VserverVscan_objs = NACL::C::VserverVscan->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my VserverVscan_state = $VserverVscan_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverVscan would return
a NACL::CS::VserverVscan object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 enable

    NACL::C::VserverVscan->enable(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);

								 or

    $VserverVscan_obj->enable(...);

(Class or instance method)
This method is used to enable Vscan on a Vserver.
Uses CMode CLI/ZAPI .
Invokes  vscan-status-modify for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "vserver vscan enable" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub enable {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params   => \@args,
        cli_cmd  => 'vserver vscan enable',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_enable_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_enable_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        nacl_exceptions => [qw(AlreadyEnabled)],
        frontend => 'NACL::C::VserverVscan::enable',
    );

    $Log->exit() if $may_exit;
}

sub _enable_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_vscan_enable',);
    $Log->exit() if $may_exit;
}

sub _enable_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@opts,
        additional_args => {'is-vscan-enabled' => 'true'},
        api => 'vscan-status-modify',);
    $Log->exit() if $may_exit;
}

=head2 disable

    NACL::C::VserverVscan->disable(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);

								 or

    $VserverVscan_obj->disable(...);

(Class or instance method)
This method is used to disable Vscan on a Vserver.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "vserver vscan disable" command in ONTAP
Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub disable {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params   => \@args,
        cli_cmd  => 'vserver vscan disable',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_disable_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        nacl_exceptions => [qw(AlreadyDisabled)],
        frontend => 'NACL::C::VserverVscan::disable',
    );

    $Log->exit() if $may_exit;
}

sub _disable_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_vscan_disable',);
    $Log->exit() if $may_exit;
}

=head2 reset

    NACL::C::VserverVscan->reset(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);

								 or

    $VserverVscan_obj->reset(...);

(Class or instance method)
This method is used to discard cached scan information.
Uses CMode CLI/ZAPI .
Invokes  vscan-reset for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "vserver vscan reset" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub reset {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params   => \@args,
        cli_cmd  => 'vserver vscan reset',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_reset_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_reset_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverVscan::reset',
    );

    $Log->exit() if $may_exit;
}

sub _reset_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_vscan_reset',);
    $Log->exit() if $may_exit;
}

sub _reset_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@opts, api => 'vscan-reset',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('vserver' => {type => SCALAR},);
}

1;
