# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverVscanOnAccessPolicy Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverVscanOnAccessPolicy

=head1 DESCRIPTION

C<NACL::C::VserverVscanOnAccessPolicy> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverVscanOnAccessPolicy.  A related class is
L<NACL::CS::VserverVscanOnAccessPolicy|lib-NACL-CS-VserverVscanOnAccessPolicy-pm>, which represents
the past state of an ONTAP VserverVscanOnAccessPolicy.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=head2 policy_name

=cut

package NACL::C::VserverVscanOnAccessPolicy;

use strict;
use warnings;

use NACL::CS::VserverVscanOnAccessPolicy;
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use NACL::C::UnitNormalization qw (convert_to_bytes_int);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use NACL::C::Exceptions::VserverVscanOnAccessPolicy::AlreadyDisabled;
use NACL::C::Exceptions::VserverVscanOnAccessPolicy::AlreadyEnabled;
use NACL::C::Exceptions::VserverVscanOnAccessPolicy::AlreadyExists;
use NACL::C::Exceptions::VserverVscanOnAccessPolicy::DoesNotExist;


=head1 METHODS

=head2 new

  my $VserverVscanOnAccessPolicy = NACL::C::VserverVscanOnAccessPolicy->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
                                   'policy-name' => $policy_name,
								  );

Return a new VserverVscanOnAccessPolicy component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'policy_name',
];

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $VserverVscanOnAccessPolicy_obj  = NACL::C::VserverVscanOnAccessPolicy->find(command_interface => $ci, ...);
	my @VserverVscanOnAccessPolicy_objs = NACL::C::VserverVscanOnAccessPolicy->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my VserverVscanOnAccessPolicy_state = $VserverVscanOnAccessPolicy_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverVscanOnAccessPolicy would return 
a NACL::CS::VserverVscanOnAccessPolicy object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 enable

    NACL::C::VserverVscanOnAccessPolicy->enable(
                                command_interface => $ci,
	                            'vserver' => $vserver,
	                            'policy-name' => $policy_name,
                                ...);
	                             
								 or

    $VserverVscanOnAccessPolicy_obj->enable(...);

(Class or instance method)
This method is used to enable an On-Access policy.
Uses CMode CLI/ZAPI .
Invokes  vscan-on-access-policy-status-modify for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver

Applicable for CMode CLI

=item C<< 'policy-name' => $policy_name >>

(Required for class method, Not Applicable for instance method)
Policy

Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "vserver vscan on access policy enable" command in ONTAP

Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub enable {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        cli_cmd => 'vserver vscan on-access-policy enable',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_enable_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_enable_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverVscanOnAccessPolicy::enable',
        nacl_exceptions => [qw(AlreadyEnabled)],
    );

    $Log->exit() if $may_exit;
}

sub _enable_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_vscan_on_access_policy_enable',);
    $Log->exit() if $may_exit;
}

sub _enable_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(policy-name)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        additional_args => {'policy-status' => 'true'},
        api  => 'vscan-on-access-policy-status-modify',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

=head2 disable

    NACL::C::VserverVscanOnAccessPolicy->disable(
                                command_interface => $ci,
	                            'vserver' => $vserver,
	                            'policy-name' => $policy_name,
                                ...);
	                             
								 or

    $VserverVscanOnAccessPolicy_obj->disable(...);

(Class or instance method)
This method is used to disable an On-Access policy.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver

Applicable for CMode CLI

=item C<< 'policy-name' => $policy_name >>

(Required for class method, Not Applicable for instance method)
Policy

Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "vserver vscan on access policy disable" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub disable {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        cli_cmd => 'vserver vscan on-access-policy disable',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_disable_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverVscanOnAccessPolicy::disable',
        nacl_exceptions => [qw(AlreadyDisabled)],
    );

    $Log->exit() if $may_exit;
}

sub _disable_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_vscan_on_access_policy_disable',);
    $Log->exit() if $may_exit;
}

=head2 create

    my $VserverVscanOnAccessPolicy_obj	= NACL::C::VserverVscanOnAccessPolicy->create(
                                command_interface => $ci,
	                            'vserver' => $vserver,
	                            'policy-name' => $policy_name,
                                ...);
   
(Class method)
This method is used to create an On-Access policy.
Uses CMode CLI/ZAPI .
Invokes  vscan-on-access-policy-create for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required)
Vserver

Applicable for CMode CLI/ZAPI

=item C<< 'policy-name' => $policy_name >>

(Required)
Policy

Applicable for CMode CLI/ZAPI

=item C<< 'protocol' => $string >>

File-Access Protocol

Applicable for CMode CLI/ZAPI

=item C<< 'max-file-size' => $string >>

Max File Size Allowed for Scanning.
For CMode-ZAPI, will be converted into bytes if specified with suffix like GB, etc.

Applicable for CMode CLI/ZAPI

=item C<< 'paths-to-exclude' => [ $paths-to-exclude1, $paths-to-exclude2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)


Applicable for CMode CLI/ZAPI

=item C<< 'filters' => [ $filter1, $filter2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)


Applicable for CMode CLI/ZAPI

=item C<< 'file-ext-to-exclude' => [ $file-ext-to-exclude1, $file-ext-to-exclude2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)


Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "vserver vscan on access policy create" command in ONTAP

Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'paths-to-exclude'    => {type => ARRAYREF, optional => 1},
            'filters'             => {type => ARRAYREF, optional => 1},
            'file-ext-to-exclude' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( protocol max-file-size
                    )
            ),
        },
        cli_cmd => 'vserver vscan on-access-policy create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverVscanOnAccessPolicy::create',
        nacl_exceptions => [ qw(AlreadyExists) ],
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $vserver,
        policy_name       => $opts{'policy-name'},
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'vserver_vscan_on_access_policy_create',);
    $Log->exit() if $may_exit;
}

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    # convert to bytes for ZAPI call
    if (defined $opts{'max-file-size'}) {
        my $value_in_bytes = convert_to_bytes_int($opts{'max-file-size'});
        $opts{'max-file-size'} = $value_in_bytes;
    }

    my $copy = [
        qw(paths-to-exclude protocol policy-name filters file-ext-to-exclude
            max-file-size)
    ];
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api  => 'vscan-on-access-policy-create',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::VserverVscanOnAccessPolicy->delete(
                                command_interface => $ci,
	                            'vserver' => $vserver,
	                            'policy-name' => $policy_name,
                                ...);
	                             
								 or

    $VserverVscanOnAccessPolicy_obj->delete(...);

(Class or instance method)
This method is used to delete an On-Access policy.
Uses CMode CLI/ZAPI .
Invokes  vscan-on-access-policy-delete for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver

Applicable for CMode CLI/ZAPI

=item C<< 'policy-name' => $policy_name >>

(Required for class method, Not Applicable for instance method)
Policy

Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "vserver vscan on access policy delete" command in ONTAP

Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        cli_cmd => 'vserver vscan on-access-policy delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverVscanOnAccessPolicy::delete',
        nacl_exceptions => [qw(DoesNotExist)],
    );

    $Log->exit() if $may_exit;
}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_vscan_on_access_policy_delete',);
    $Log->exit() if $may_exit;
}

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(policy-name)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'vscan-on-access-policy-delete',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::VserverVscanOnAccessPolicy->modify(
                                command_interface => $ci,
	                            'vserver' => $vserver,
	                            'policy-name' => $policy_name,
                                ...);
	                             
								 or

    $VserverVscanOnAccessPolicy_obj->modify(...);

(Class or instance method)
This method is used to modify an On-Access policy.
Uses CMode CLI/ZAPI .
Invokes  vscan-on-access-policy-modify for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver

Applicable for CMode CLI/ZAPI

=item C<< 'policy-name' => $policy_name >>

(Required for class method, Not Applicable for instance method)
Policy

Applicable for CMode CLI/ZAPI

=item C<< 'max-file-size' => $string >>

Max File Size Allowed for Scanning.
For CMode-ZAPI, will be converted into bytes if specified with suffix like GB, etc.

Applicable for CMode CLI/ZAPI

=item C<< 'paths-to-exclude' => [ $paths-to-exclude1, $paths-to-exclude2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)


Applicable for CMode CLI/ZAPI

=item C<< 'filters' => [ $filter1, $filter2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)


Applicable for CMode CLI/ZAPI

=item C<< 'file-ext-to-exclude' => [ $file-ext-to-exclude1, $file-ext-to-exclude2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)


Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "vserver vscan on access policy modify" command in ONTAP

Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'paths-to-exclude'    => {type => ARRAYREF, optional => 1},
            'filters'             => {type => ARRAYREF, optional => 1},
            'file-ext-to-exclude' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( max-file-size
                    )
            ),
        },
        cli_cmd => 'vserver vscan on-access-policy modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverVscanOnAccessPolicy::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_vscan_on_access_policy_modify',);
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    # convert to bytes for ZAPI call
    if (defined $opts{'max-file-size'}) {
        my $value_in_bytes = convert_to_bytes_int($opts{'max-file-size'});
        $opts{'max-file-size'} = $value_in_bytes;
    }

    my $copy = [
        qw(paths-to-exclude policy-name filters file-ext-to-exclude
            max-file-size)
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api  => 'vscan-on-access-policy-modify',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver'     => {type => SCALAR},
        'policy-name' => {type => SCALAR},
    );
}

1;
