# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Wreck Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Wreck

=head1 DESCRIPTION

C<NACL::C::Wreck> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to an ONTAP Wreck.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=cut

package NACL::C::Wreck;

use strict;
use warnings;

use Params::Validate qw(validate_with SCALAR ARRAYREF BOOLEAN SCALARREF);
use NACL::ComponentUtils qw (_optional_scalars );
use base 'NACL::C::Component::ONTAP';

=head1 METHODS

=head2 wreck

  NACL::C::Wreck->wreck(command_interface=>$ci,
                                              aggregate=>$aggr_name,...);

(Class method) Request run of a new wreck element. Return apiset raw_output
on completion.

Uses a 7Mode-CLI or Nodescope CLI APISet.
  Invokes 'wreck' command for 7Mode/Nodescope CLI.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< volume=>$volume >>

Applicable only for 7Mode CLI.

=item C<< aggregate=>$aggregate >>

Applicable only for 7Mode CLI.

=item C<< "inject-file"=>$file >>

Applicable only for 7Mode CLI.

=item C<< "inject-command"=>$string >>

Applicable only for 7Mode CLI.

=item C<< "pseudo-bad"=>$boolean >>

Applicable only for 7Mode CLI.

=item C<< unverified=>$boolean >>

Applicable only for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut


sub wreck {
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( volume aggregate inject-file inject-command
                    pseudo-bad unverified )
            ),
        },
    );
    return $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_wreck_7mode_cli",
                interface => "CLI",
                set       => "7Mode|Nodescope",
            },
        ],
    );
} ## end sub create

sub _wreck_7mode_cli {
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $response = $apiset->wreck(%opts , 'privilege-level' => "test");
    return $response->get_raw_output();

} ## end sub _create_7mode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ();
}

1;
