#
# Copyright (c) 2001-2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Common methods for the NetworkPort Component and ComponentState
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Mixins::NetworkPort;

use strict;
use warnings;

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(_common_zapi_map _common_zapi_copy _map_role_to_ipspace);
    our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );
}

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _common_zapi_map {
    my %map = (
        'link'                => 'link-status',
        'autonegotiate-admin' => 'is-administrative-auto-negotiate',
        'duplex-admin'        => 'administrative-duplex',
        'speed-admin'         => 'administrative-speed',
        'type'                => 'port-type',
        'flowcontrol-admin'   => 'administrative-flowcontrol',
        'ifgrp-distr-func'    => 'ifgrp-distribution-function',
        'up-admin'            => 'is-administrative-up',
        'autonegotiate-oper'  => 'is-operational-auto-negotiate',
        'mac'                 => 'mac-address',
        'duplex-oper'         => 'operational-duplex',
        'flowcontrol-oper'    => 'operational-flowcontrol',
        'speed-oper'          => 'operational-speed',
        'vlan-tag'            => 'vlan-id',
    );
    return wantarray ? %map : \%map;
}

sub _common_zapi_copy {
    my @copy = (qw(port node ifgrp-mode role mtu ipspace autorevert-delay
        ifgrp-node ifgrp-port remote-device-id vlan-node vlan-port broadcast-domain));
    return wantarray ? @copy : \@copy;
}

sub _map_role_to_ipspace {
    $Log->enter() if $may_enter;

    my ($pkg, $opts) = @_;
    my $ipspace;
    #map only mgmt,intercluster,cluster-mgmt and cluster. pass through others.
    if ($opts->{role} =~ /mgmt$|intercluster$/i) {
        $Log->comment("role{".$opts->{role}.
                "} is mapped to ipspace{Default}");
        $opts->{ipspace} = 'Default';
        delete $opts->{role};
    } elsif ($opts->{role} =~ /cluster/i) {
        $Log->comment("role{".$opts->{role}.
                "} is mapped to ipspace{Cluster}");
        $opts->{ipspace} = 'Cluster';
        delete $opts->{role};
    } else {
        $Log->warn(
        "Role '". $opts->{role}."' cannot be mapped to ipspace, ".
        " 'ipspace' should be provided in filter");
    }

    $Log->exit() if $may_exit;
}
1;
