#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Common methods for the SecurityLogin Component and ComponentState
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Mixins::SecurityLogin;

use strict;
use warnings;
use NACL::Exceptions::InvalidChoice qw(:try);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(_common_zapi_map _common_zapi_copy _check_for_zapi);
    our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );
}
use NATE::Exceptions::Argument qw(:try);

sub _common_zapi_map {
    my %map = (
          "username"   => "user-name",
          "user-or-group-name" => "user-name",
          "authmethod" => "authentication-method",
          "role"       => "role-name",
          "acctlocked" => "is-locked",
    );
    return wantarray ? %map : \%map;

}

sub _common_zapi_copy {
    my @copy = (qw(vserver application comment));
    return wantarray ? @copy : \@copy;
}

sub _check_for_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    if (defined ($opts{uri})) {
        my $msg = 'Could not use ZAPI, since the CMode'
           . 'ZAPI does not support "uri" option';
        $Log->comment($msg);
        $Log->exit() if $may_exit;
        NACL::Exceptions::InvalidChoice->throw($msg);
    }
    $Log->exit() if $may_exit;
} ## end sub _check_for_zapi

1;
