#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Mix-in for common functionality for Vlan C
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Mixins::Vlan;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(_get_vlan_interface _get_vlan_interface_linux 
                        _get_vlan_interface_solaris); 
    our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );
}

sub _get_vlan_interface {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $vlan_interface = $pkg_or_obj->call_on_apiset(
        @opts,
        choices => [
            {   method => '_get_vlan_interface_linux',
                set    => 'Linux',
            },
            {   method => '_get_vlan_interface_solaris',
                set    => 'Solaris',
            },
        ]
    );
    $Log->exit() if $may_exit;
    return $vlan_interface;
} ## end sub _get_vlan_interface

sub _get_vlan_interface_linux {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $vlan_interface = $opts{interface} . '.' . $opts{'vlan-id'};
    $Log->exit() if $may_exit;
    return $vlan_interface;
} ## end sub _get_vlan_interface_linux


sub _get_vlan_interface_solaris {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $interface = $opts{interface};
    if ($interface =~ /(\w+)(\d+)/i) {

        # Device name will be "hme" or "pcn"
        my $device_name = $1;

        # Device PPA will be the "0,1,2" of the "hme0,pcn1.."
        my $device_ppa     = $2;
        my $PPA            = ($opts{'vlan-id'} * 1000) + $device_ppa;
        my $vlan_interface = "$device_name" . "$PPA";
        $Log->exit() if $may_exit;
        return $vlan_interface;
    } else {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "interface is not correct: $opts{interface}");
    }
} ## end sub _get_vlan_interface_solaris

1;
