#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Common methods for the Vserver Component and ComponentState
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Mixins::Vserver;

use strict;
use warnings;

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(_common_zapi_map _common_zapi_copy);
    our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );
}

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

sub _common_zapi_map {
    my %map = (
        'vserver'                   => 'vserver-name',
        'type'                      => 'vserver-type',
        'rootvolume'                => 'root-volume',
        'aggregate'                 => 'root-volume-aggregate',
        'rootvolume-security-style' => 'root-volume-security-style',
        'nisdomain'                 => 'nis-domain',
        'ldap-client'               => 'ldap-domain',
        'admin-state'               => 'state',
        'is-repository'             => 'is-repository-vserver',
        subtype                     => 'vserver-subtype',
        'allowed-protocols' => [
            make_zapi_array('allowed-protocols'),
            make_zapi_skip('protocol')
        ],
        'disallowed-protocols' => [
            make_zapi_array('disallowed-protocols'),
            make_zapi_skip('protocol')
        ],
        'ns-switch' => [
            make_zapi_array('name-server-switch'),
            make_zapi_skip('nsswitch'),
        ],
        'nm-switch' => [
            make_zapi_array('name-mapping-switch'),
            make_zapi_skip('nmswitch'),
        ],
        'aggr-list' =>
            [ make_zapi_array('aggr-list'), make_zapi_skip('aggr-name') ],
        'config-lock' => 'is-config-locked-for-changes',
    );
    return wantarray ? %map : \%map;
}

sub _common_zapi_copy {
    my @copy = (
        qw(antivirus-on-access-policy caching-policy comment ipspace max-volumes
            language operational-state operational-state-stopped-reason
            quota-policy qos-policy-group snapshot-policy uuid
            volume-delete-retention-hours)
    );
    return wantarray ? @copy : \@copy;
}

1;
