# $Id: //depot/prod/test/nacldev/lib/NACL/C/_Mixins/VserverPeerTransition.pm#1 $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Common methods for the VserverPeerTransition Component and ComponentState
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Mixins::VserverPeerTransition;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use Params::Validate qw(validate_with OBJECT HASHREF);

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(_validate_input_for_dup_keys _zapi_option_map _handle_alternate_ip_change);
    our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );
}
use NACL::ComponentUtils qw(_hash_copy);

# With this uichange "vserver-peer-transition"
# 'srcFilerName' is renamed to 'src-filer-name'
# 'localVserver' is renamed to 'local-vserver'
# Users can only pass either the old set or new set.
sub _validate_input_for_dup_keys {
    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;
    my @dup = ();

    # Both should not be specified
    if ( exists $opts{'srcFilerName'} && exists $opts{'src-filer-name'} ) {
        push( @dup, 'src-filer-name <--> srcFilerName' );
    }
    if ( exists $opts{'localVserver'} && exists $opts{'local-vserver'} ) {
        push( @dup, 'local-vserver <--> localVserver' );
    }

    # Not throwing the exception here. let the caller have more customized
    # error message.
    $Log->exit() if $may_exit;
    return @dup;
}

sub _zapi_option_map {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => {
            command_interface => {
                type => OBJECT,
                isa  => 'NACL::C::CommandInterface::ONTAP'
            },
        }
    );
    my %map;

    if ( $opts{command_interface}
        ->has_uichange( uichange => "vserver-peer-transition" ) )
    {
        %map = (
            'localVserver'   => 'local-vserver',
            'srcFilerName'   => 'src-filer-name',
            'local-vserver'  => 'local-vserver',
            'src-filer-name' => 'src-filer-name',
        );
    } else {
        %map = (
            'localVserver'   => 'localvserver',
            'srcFilerName'   => 'srcfilername',
            'local-vserver'  => 'localvserver',
            'src-filer-name' => 'srcfilername',
        );
    }
    # In older SN builds, 'alternate-ip' option was not there. So if it was
    # passed by the caller, then it would have been ignored by the ZAPI
    # backend. In the newer builds, the equivalent option 'multi-path-address'
    # is supported for both CLI and ZAPI. So if one is passing this newer
    # option, then they are testing on the newer builds, so we have to copy
    # this option as well.
    $map{'multi-path-address'} = 'multi-path-address';
    $Log->exit() if $may_exit;
    return \%map;
}

# This method handles converting the option
# 'alternate-ip' to 'multi-path-address' for both CLI and ZAPI.
sub _handle_alternate_ip_change {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, %args ) = @_;
    validate_with(
        params => [%args],
        spec   => {
            command_interface => $pkg_or_obj->_ontap_ci_validate_spec,
            opts => {
                type => HASHREF
            },
        }
    );
    my $opts = $args{opts};
    # Provides only forward compatibility.
    if (defined $opts->{'alternate-ip'} && !(defined $opts->{'multi-path-address'})) {
        my $new_build = $args{command_interface}->is_feature_supported(
            feature => 'vserver_peer_transition_alternate_ip_option'
        );
        if ($new_build) {
            $opts->{'multi-path-address'} = delete $opts->{'alternate-ip'};
        }
    }
    $Log->exit() if $may_exit;
}


1;
