# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Application ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::Application

=head1 DESCRIPTION

C<NACL::CS::Application> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP Application.  A related class is
L<NACL::C::Application|lib-NACL-C-Application-pm>, which represents access
to an ONTAP Application.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the Application
element are the attributes of the Application ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< inflight >>

In-flight References
Filled in for CMode CLI/ZAPI iter

=item C<< vserver >>

Vserver
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-info vserver="$value" />


=item C<< application >>

Application Name
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-info application="$value" />


=item C<< application_type >>

Application Type
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-info application-type="$value" />


=item C<< application_version >>

Application Version
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-info application-version="$value" />


=item C<< application_uuid >>

Application UUID
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-info application-uuid="$value" />


=item C<< volume_count >>

Volume Count
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-info volume-count="$value" />


=item C<< vserver_uuid >>

Vserver UUID
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-info vserver-uuid="$value" />


=item C<< inflight_str >>

Filled in for CMode CLI/ZAPI iter
=back

=cut

package NACL::CS::Application;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'inflight',
    scalar => 'vserver',
    scalar => 'application',
    scalar => 'application_type',
    scalar => 'application_version',
    scalar => 'application_uuid',
    scalar => 'volume_count',
    scalar => 'vserver_uuid',
    scalar => 'inflight_str',
];
my $Log       = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $Application_state  = NACL::CS::Application->fetch(command_interface => $ci, ...);
	my @Application_states = NACL::CS::Application->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes application-get-iter API for CMode ZAPI iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'application show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter',
            },

        ],
        exception_text => 'No matching Application(s) found',
        frontend       => 'NACL::CS::Application::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@args, api => 'application_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(vserver-uuid volume-count application-uuid vserver application
                application-version application-type
                )
        ],
        api => "application-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
