# $Id$

#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ApplicationDetail ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::ApplicationDetail

=head1 DESCRIPTION

C<NACL::CS::ApplicationDetail> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP ApplicationDetail.  A related class is
L<NACL::C::ApplicationDetail|lib::NACL-C-ApplicationDetail-pm>, which represents access
to an ONTAP ApplicationDetail.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the ApplicationDetail
element are the attributes of the ApplicationDetail ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< element_name >>

Name of object
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info>
  <application-components name="application-component-info">
    <element-detail-info name="application-element-detail-info" element-name="$value" />
  </application-components>
</application-detail-info>


=item C<< application_component_sslc >>

Storage Service Level
Filled in for CMode CLI/ZAPI iter

=item C<< application_component_name >>

Application Component Name
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info>
  <application-components name="application-component-info" component-name="$value" />
</application-detail-info>


=item C<< application_template_type >>

Application Template Type
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info application-template-type="$value" />

=item C<< application_template_version >>

Application Template Version
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info application-template-version="$value" />

=item C<< application_template_protocol >>

Application Template Protocol
Filled in for CMode CLI


=item C<< nas_header_str >>

Filled in for CMode CLI/ZAPI iter

=item C<< igroup >>

Igroup Name
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info>
  <application-components name="application-component-info">
    <element-detail-info name="application-element-detail-info" igroup="$value" />
  </application-components>
</application-detail-info>


=item C<< lun >>

LUN Name
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info>
  <application-components name="application-component-info">
    <element-detail-info name="application-element-detail-info" lun="$value" />
  </application-components>
</application-detail-info>

=item C<< lun-id >>

LUN ID
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info>
  <application-components name="application-component-info">
    <element-detail-info name="application-element-detail-info" lun-id="$value" />
  </application-components>
</application-detail-info>

=item C<< element_type >>

Type of object possible value(s) are, lun,lun-clone,volume,igroup
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info>
  <application-components name="application-component-info" element-type="$value" />
</application-detail-info>


=item C<< app_comp_header_str >>

Filled in for CMode CLI/ZAPI iter

=item C<< nfs_access_tos >>

NFS clients
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $nfs_access_tos = $obj->nfs_access_tos();
 
# $nfs_access_tos contains a reference to the array of values
 
my @nfs_access_tos = $obj->nfs_access_tos();
 
# @nfs_access_tos contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { nfs_access_tos = [ value1, value2...] }

Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info>
  <application-components name="application-component-info">
    <element-detail-info name="application-element-detail-info">
      <nfs-permissions name="nfs-permission-info" nfs-access-to="@values" />
    </element-detail-info>
  </application-components>
</application-detail-info>


=item C<< san_protocol_header_str >>

Filled in for CMode CLI/ZAPI iter

=item C<< aux_lun_header >>

Auxiliary LUN header flag
Filled in for CMode CLI/ZAPI iter

=item C<< nas_protocol_header >>

Protocol info header
Filled in for CMode CLI/ZAPI iter

=item C<< nas_protocol_header_str >>

Filled in for CMode CLI/ZAPI iter

=item C<< nfs_permissions_str >>

Filled in for CMode CLI/ZAPI iter

=item C<< cifs_permissions >>

CIFS User Permissions
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $cifs_permissions = $obj->cifs_permissions();
 
# $cifs_permissions contains a reference to the array of values
 
my @cifs_permissions = $obj->cifs_permissions();
 
# @cifs_permissions contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { cifs_permissions = [ value1, value2...] }

Filled in for CMode CLI

=item C<< serial >>

LUN Serial
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info>
  <application-components name="application-component-info">
    <element-detail-info name="application-element-detail-info" serial="$value" />
  </application-components>
</application-detail-info>


=item C<< generic_san_header >>

Generic SAN header flag
Filled in for CMode CLI/ZAPI iter

=item C<< nas_element_name >>

Volume Name
Filled in for CMode CLI

=item C<< nas_header >>

NAS header flag
Filled in for CMode CLI/ZAPI iter

=item C<< cifs_permissions_str >>

Filled in for CMode CLI/ZAPI iter

=item C<< cifs_access_ips >>

CIFS Lifs
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $cifs_access_ips = $obj->cifs_access_ips();
 
# $cifs_access_ips contains a reference to the array of values
 
my @cifs_access_ips = $obj->cifs_access_ips();
 
# @cifs_access_ips contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { cifs_access_ips = [ value1, value2...] }

Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info>
  <application-components name="application-component-info">
    <element-detail-info name="application-element-detail-info">
      <cifs-access-ips ip-address="@values" />
    </element-detail-info>
  </application-components>
</application-detail-info>


=item C<< nas_element_size >>

Volume Size
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info>
  <application-components name="application-component-info">
    <element-detail-info name="application-element-detail-info" nas-element-size="$value" />
  </application-components>
</application-detail-info>


=item C<< app_comp_header >>

Application component Header
Filled in for CMode CLI/ZAPI iter

=item C<< igroup_protocol >>

Igroup Name
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info>
  <application-components name="application-component-info">
    <element-detail-info name="application-element-detail-info" igroup="$value" />
  </application-components>
</application-detail-info>


=item C<< cifs_server_fqdn >>

CIFS Server Name
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info>
  <application-components name="application-component-info">
    <element-detail-info name="application-element-detail-info" cifs-server-fqdn="$value" />
  </application-components>
</application-detail-info>


=item C<< nfs_permissions >>

NFS Permissions
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $nfs_permissions = $obj->nfs_permissions();
 
# $nfs_permissions contains a reference to the array of values
 
my @nfs_permissions = $obj->nfs_permissions();
 
# @nfs_permissions contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { nfs_permissions = [ value1, value2...] }

Filled in for CMode CLI

=item C<< application_uuid >>

Application UUID
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info application-uuid="$value" />


=item C<< aux_lun_header_str >>

Filled in for CMode CLI/ZAPI iter


=item C<< vserver >>

Vserver
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info vserver="$value" />


=item C<< application_component_sslc_str >>

Filled in for CMode CLI/ZAPI iter

=item C<< application >>

Application Name
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info application="$value" />


=item C<< nfs_export_policy_name >>

Export policy Name
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info>
  <application-components name="application-component-info">
    <element-detail-info name="application-element-detail-info" nfs-export-policy-name="$value" />
  </application-components>
</application-detail-info>


=item C<< nas_element_name_str >>

Filled in for CMode CLI/ZAPI iter

=item C<< generic_san_header_str >>

Filled in for CMode CLI/ZAPI iter

=item C<< initiator >>

Initiators
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $initiator = $obj->initiator();
 
# $initiator contains a reference to the array of values
 
my @initiator = $obj->initiator();
 
# @initiator contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { initiator = [ value1, value2...] }

Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info>
  <application-components name="application-component-info">
    <element-detail-info name="application-element-detail-info">
      <initiator string="@values" />
    </element-detail-info>
  </application-components>
</application-detail-info>


=item C<< application_component_uuid >>

Application Component UUID
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info>
  <application-components name="application-component-info" component-uuid="$value" />
</application-detail-info>


=item C<< nfs_access_levels >>

Permission granted to clients
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $nfs_access_levels = $obj->nfs_access_levels();
 
# $nfs_access_levels contains a reference to the array of values
 
my @nfs_access_levels = $obj->nfs_access_levels();
 
# @nfs_access_levels contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { nfs_access_levels = [ value1, value2...] }

Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info>
  <application-components name="application-component-info">
    <element-detail-info name="application-element-detail-info">
      <nfs-permissions name="nfs-permission-info" nfs-access-level="@values" />
    </element-detail-info>
  </application-components>
</application-detail-info>


=item C<< cifs_share_name >>

CIFS Share Name
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <application-detail-info>
  <application-components name="application-component-info">
    <element-detail-info name="application-element-detail-info" cifs-share-name="$value" />
  </application-components>
</application-detail-info>


=item C<< san_protocol_header >>

Protocol info header
Filled in for CMode CLI/ZAPI iter
=back

=cut

package NACL::CS::ApplicationDetail;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'element_name',
    scalar => 'application_component_sslc',
    scalar => 'application_component_name',
    scalar => 'application_template_type',
    scalar => 'nas_header_str',
    scalar => 'igroup',
    scalar => 'lun',
    scalar => 'element_type',
    scalar => 'app_comp_header_str',
    array  => 'nfs_access_tos',
    scalar => 'san_protocol_header_str',
    scalar => 'aux_lun_header',
    scalar => 'nas_protocol_header',
    scalar => 'nas_protocol_header_str',
    scalar => 'nfs_permissions_str',
    array  => 'cifs_permissions',
    scalar => 'serial',
    scalar => 'generic_san_header',
    scalar => 'nas_element_name',
    scalar => 'nas_header',
    scalar => 'cifs_permissions_str',
    array  => 'cifs_access_ips',
    scalar => 'nas_element_size',
    scalar => 'app_comp_header',
    scalar => 'igroup_protocol',
    scalar => 'cifs_server_fqdn',
    array  => 'nfs_permissions',
    array  => 'cifs_access_levels',
    scalar => 'lun_path',
    scalar => 'size',
    array  => 'nfs_access_ips',
    scalar => 'application_uuid',
    scalar => 'aux_lun_header_str',
    scalar => 'nas_element_path',
    array  => 'cifs_access_tos',
    scalar => 'vserver',
    scalar => 'application_component_sslc_str',
    scalar => 'application',
    scalar => 'nfs_export_policy_name',
    scalar => 'nas_element_name_str',
    scalar => 'generic_san_header_str',
    array  => 'initiator',
    scalar => 'application_component_uuid',
    array  => 'nfs_access_levels',
    scalar => 'cifs_share_name',
    scalar => 'san_protocol_header',
    scalar => 'lun_id',
    scalar => 'application_template_version',
    scalar => 'application_template_protocol',
];
my $Log       = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _extra_filter_fields {
    $Log->enter() if $may_enter;
    $Log->exit()  if $may_exit;
    return [qw( protocol config )];
}

=head1 METHODS

=head2 fetch

    my $ApplicationDetail_state  = NACL::CS::ApplicationDetail->fetch(command_interface => $ci, ...);
	my @ApplicationDetail_states = NACL::CS::ApplicationDetail->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes application-details-get-iter API for CMode ZAPI iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'application detail show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter',
                check     => '_check_cmode_zapi',
            },

        ],
        exception_text => 'No matching ApplicationDetail(s) found',
        frontend       => 'NACL::CS::ApplicationDetail::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'application_detail_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _check_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $filter = $opts{filter};
    my @requested = $opts{requested_fields};
    if ( defined $filter->{'nas-element-name'} || 
        defined $filter->{'cifs-permissions'} || defined $filter->{'nfs-permissions'} || 
            defined $filter->{'application-template-protocol'}) {
            my $msg = 'The following fields provided '
                 . 'in the call to filter NACL::CS::ApplicationDetail->fetch() '
                 . 'are not returned by the ZAPI, so the ZAPI back-end is being '
                 . "skipped";
            $Log->comment($msg);
            $Log->exit() if $may_exit;
            NACL::Exceptions::InvalidChoice->throw($msg);
    }
    if ('nas-element-name'~~ @requested ||
        'cifs-permissions' ~~ @requested || 'nfs-permissions' ~~ @requested ||
           'application-template-protocol'  ~~ @requested ){

            my $msg = 'The following fields provided '
                 . 'in the call to requested_fields NACL::CS::ApplicationDetail->fetch() '
                 . 'are not returned by the ZAPI, so the ZAPI back-end is being '
                 . "skipped";
            $Log->comment($msg);
            $Log->exit() if $may_exit;
            NACL::Exceptions::InvalidChoice->throw($msg);
    }
    $Log->exit() if $may_exit;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $requested_fields = $opts{requested_fields};
    my $filter           = $opts{filter};
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
	api => "application-details-get-iter",
	inapplicable_for_help_xml => [qw(initiator cifs-access-ips nfs-access-ips)],
	suppress_type_warning => 1,
        copy => [
            qw(application-template-type application-uuid vserver application application-template-version 
                )
        ],
        map => {
            'element-name' => [
                make_zapi_array('application-components'),
                make_zapi_skip('application-component-info'),
                make_zapi_array('element-detail-info'),
                make_zapi_skip('application-element-detail-info'),
                make_zapi_array('element-name')
            ],
            'application-component-name' => [
                make_zapi_array('application-components'),
                make_zapi_skip('application-component-info'),
                make_zapi_array('component-name'),
            ],
            'lun' => [
               make_zapi_array('application-components'),
               make_zapi_skip('application-component-info'),
               make_zapi_array('element-detail-info'),
               make_zapi_skip('application-element-detail-info'),
               make_zapi_array('lun'),
            ],
            'lun-id' => [
               make_zapi_array('application-components'),
               make_zapi_skip('application-component-info'),
               make_zapi_array('element-detail-info'),
               make_zapi_skip('application-element-detail-info'),
               make_zapi_array('lun-id'),
            ],
            'igroup' => [
                make_zapi_array('application-components'),
                make_zapi_skip('application-component-info'),
                make_zapi_array('element-detail-info'),
                make_zapi_skip('application-element-detail-info'),
                make_zapi_array('igroup')
            ],
            'element-type' => [
                make_zapi_array('application-components'),
                make_zapi_skip('application-component-info'),
                make_zapi_array('element-type'),
            ],
            'serial' => [
                make_zapi_array('application-components'),
                make_zapi_skip('application-component-info'),
                make_zapi_array('element-detail-info'),
                make_zapi_skip('application-element-detail-info'),
                make_zapi_array('serial')
            ],

            'nas-element-size' => [
                make_zapi_array('application-components'),
                make_zapi_skip('application-component-info'),
                make_zapi_array('element-detail-info'),
                make_zapi_skip('application-element-detail-info'),
                make_zapi_array('nas-element-size'),
            ],
            'igroup-protocol' => [
                make_zapi_array('application-components'),
                make_zapi_skip('application-component-info'),
                make_zapi_array('element-detail-info'),
                make_zapi_skip('application-element-detail-info'),
                make_zapi_array('igroup')
            ],
            'cifs-server-fqdn' => [
                make_zapi_array('application-components'),
                make_zapi_skip('application-component-info'),
                make_zapi_array('element-detail-info'),
                make_zapi_skip('application-element-detail-info'),
                make_zapi_array('cifs-server-fqdn')
            ],
            'lun-path' => [
                make_zapi_array('application-components'),
                make_zapi_skip('application-component-info'),
                make_zapi_array('element-detail-info'),
                make_zapi_skip('application-element-detail-info'),
                make_zapi_array('lun-path')
            ],
            'size' => [
                make_zapi_array('application-components'),
                make_zapi_skip('application-component-info'),
                make_zapi_array('element-detail-info'),
                make_zapi_skip('application-element-detail-info'),
                make_zapi_array('size')
            ],
            'nas-element-path' => [
                make_zapi_array('application-components'),
                make_zapi_skip('application-component-info'),
                make_zapi_array('element-detail-info'),
                make_zapi_skip('application-element-detail-info'),
                make_zapi_array('nas-element-path')
            ],
            'nfs-export-policy-name' => [
                make_zapi_array('application-components'),
                make_zapi_skip('application-component-info'),
                make_zapi_array('element-detail-info'),
                make_zapi_skip('application-element-detail-info'),
                make_zapi_array('nfs-export-policy-name')
            ],
            'application-component-uuid' => [
                make_zapi_array('application-components'),
                make_zapi_skip('application-component-info'),
                make_zapi_array('component-uuid'),
            ],
            'application-component-sslc' => [
                make_zapi_array('application-components'),
                make_zapi_skip('application-component-info'),
                make_zapi_array('component-sslc'),
            ],
            'cifs-share-name' => [
                make_zapi_array('application-components'),
                make_zapi_skip('application-component-info'),
                make_zapi_array('element-detail-info'),
                make_zapi_skip('application-element-detail-info'),
                make_zapi_array('cifs-share-name')
            ],
            'cifs-access-tos' => [
                make_zapi_array('application-components'),
                make_zapi_skip('application-component-info'),
                make_zapi_array('element-detail-info'),
                make_zapi_skip('application-element-detail-info'),
                make_zapi_array('cifs-permissions'),
                make_zapi_array('cifs-access-to'),
            ],
            'nfs-access-tos' => [
               make_zapi_array('application-components'),
               make_zapi_skip('application-component-info'),
               make_zapi_array('element-detail-info'),
               make_zapi_skip('application-element-detail-info'),
               make_zapi_array('nfs-permissions'),
               make_zapi_array('nfs-access-to'),
           ],
           'cifs-access-levels' => [
               make_zapi_array('application-components'),
               make_zapi_skip('application-component-info'),
               make_zapi_array('element-detail-info'),
               make_zapi_skip('application-element-detail-info'),
               make_zapi_array('cifs-permissions'),
               make_zapi_array('cifs-access-level'),
           ],
           'nfs-access-levels' => [
               make_zapi_array('application-components'),
               make_zapi_skip('application-component-info'),
               make_zapi_array('element-detail-info'),
               make_zapi_skip('application-element-detail-info'),
               make_zapi_array('nfs-permissions'),
               make_zapi_array('nfs-access-level'),
           ],

	},
        # Handle input mapping for initiator
	fix_api_opts => sub {
            my %fix_api_args = @_;
            my $api_opts = $fix_api_args{api_opts};
            my %req_field_hash = map {$_ => 1} @$requested_fields;

            if (exists $req_field_hash{'initiator'}) {
                $api_opts->{'desired-attributes'}{'application-components'}
                    [0]{'element-detail-info'}[0]->{initiator} = [''];
            }
            if (exists $req_field_hash{'cifs-access-ips'}) {
                $api_opts->{'desired-attributes'}{'application-components'}
                    [0]{'element-detail-info'}[0]->{'cifs-access-ips'} = [''];
            }
            if (exists $req_field_hash{'nfs-access-ips'}) {
                $api_opts->{'desired-attributes'}{'application-components'}
                    [0]{'element-detail-info'}[0]->{'nfs-access-ips'} = [''];
            }

            if (exists $filter->{initiator}) {
                $api_opts->{'query'}{'application-components'}
                    [0]{'element-detail-info'}[0]->{initiator} = $filter->{initiator};
            }
	    if (exists $filter->{'cifs-access-ips'}) {
                $api_opts->{'query'}{'application-components'}
                    [0]{'element-detail-info'}[0]->{'cifs-access-ips'} = $filter->{'cifs-access-ips'};
            }
	    if (exists $filter->{'nfs-access-ips'}) {
                $api_opts->{'query'}{'application-components'}
                    [0]{'element-detail-info'}[0]->{'nfs-access-ips'} = $filter->{'nfs-access-ips'};
            }
        },
	# Handle output mapping
        fix_response => sub {
            my %opts = @_;
            my $zapi = $opts{zapi};
            my $cli  = $opts{cli};
	    my @all_initiator;
	    my $application_comp = $zapi->{'application-components'}[0]->{'application-component-info'};
	    foreach my $application (@$application_comp){
                my $element_info = $application->{'element-detail-info'}[0]->{'application-element-detail-info'};
		foreach my $element (@$element_info ) {
		    if (exists  $element->{initiator} ) {
			my $initiator =  $element->{initiator}[0]->{string};
			$cli->{initiator} = $initiator;
		    }
		    if (exists  $element->{'cifs-access-ips'}) {
			my $cifs_access =  $element->{'cifs-access-ips'}[0]->{'ip-address'};
			$cli->{'cifs-access-ips'} = $cifs_access;
		    }
                    if (exists  $element->{'nfs-access-ips'}) {
                        my $nfs_access =  $element->{'nfs-access-ips'}[0]->{'ip-address'};
                        $cli->{'nfs-access-ips'} = $nfs_access;
                    }
               }
            }
	},
	fix_records => sub {
            my %opts = @_;
            my $records = $opts{records};
            my @new_records;
            foreach my $record (@$records) {
		my $application_component = delete($record->{'application-components'});
                my %record_deep_copy = %$record;
		foreach my $application_comp (@$application_component) {
		    my $application_comp_info = ($application_comp->{'application-component-info'});
		    foreach my $application_comp(@$application_comp_info) {
		        my $element_info =  ($application_comp->{'element-detail-info'});
			my $component_name =  $application_comp->{'component-name'};
			foreach my $ele_info (@$element_info) {
			    my $application_element_info = ($ele_info->{'application-element-detail-info'});
			    foreach my $val (@$application_element_info) {
			        my %new_record = %record_deep_copy;
				my %app_comp = %$application_comp;
				my %element_in = %$ele_info;
				$new_record{'application-components'}[0]{'application-component-info'} = [\%app_comp] ;
				$new_record{'application-components'}[0]{'application-component-info'}[0]{'element-detail-info'} =  [\%element_in];
			        $new_record{'application-components'}[0]{'application-component-info'}[0]{'element-detail-info'}[0]{'application-element-detail-info'} = [$val];
			        push @new_records, \%new_record;
			    }
			}
		    }
		}
            }
            return \@new_records;
        },
    );
    foreach my $obj(@state_objs) {
        my @array =  keys %{$obj};
        @array = grep { $_  !~ /initiator/ } @array;
        @array = grep { $_  !~ /cifs_access_ips|cifs_access_levels|cifs_access_tos/ } @array;
        @array = grep { $_  !~ /nfs_access_ips|nfs_access_tos|nfs_access_levels/ } @array;
        @array = grep { $_  !~ /application_template_type/ } @array;
        @array = grep { $_  !~ /application_uuid/ } @array;
        foreach my $field(@array) {
            if(  ref($obj->{$field}) eq 'ARRAY'){
                if(defined($obj->{$field}->[0])){
                    my $tmp = $obj->{$field};
                    $obj->{$field} = join(',',@{$tmp});
                } else {
                    $obj->{$field} = "-";
                }
           }
        }
    }
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
