# $Id$

#
# Copyright (c) 2001-2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ApplicationVolume ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::ApplicationVolume

=head1 DESCRIPTION

C<NACL::CS::ApplicationVolume> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP ApplicationVolume.  A related class is
L<NACL::C::ApplicationVolume|lib-NACL-C-ApplicationVolume-pm>, which represents access
to an ONTAP ApplicationVolume.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the ApplicationVolume
element are the attributes of the ApplicationVolume ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< volume >>

Volume Name
Filled in for CMode CLI/ZAPI iter


=item C<< application_uuid_str >>

Filled in for CMode CLI/ZAPI iter

=item C<< vserver >>

Vserver
Filled in for CMode CLI/ZAPI iter


=item C<< application >>

Application Name
Filled in for CMode CLI/ZAPI iter

=item C<< application_str >>

Filled in for CMode CLI/ZAPI iter

=item C<< volume_msid_str >>

Filled in for CMode CLI/ZAPI iter

=item C<< size >>

Volume Size
Filled in for CMode CLI/ZAPI iter


=item C<< state >>


Volume State possible value(s) are, online,restricted,offline,force-online,force-offline,mixed
Filled in for CMode CLI/ZAPI iter


=item C<< application_uuid >>

Application UUID
Filled in for CMode CLI/ZAPI iter

=item C<< vserver_uuid >>

Vserver UUID
Filled in for CMode CLI/ZAPI iter


=item C<< volume_msid >>

Volume Master Data Set ID
Filled in for CMode CLI/ZAPI iter
=back

=cut

package NACL::CS::ApplicationVolume;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'volume',
    scalar => 'application_uuid_str',
    scalar => 'vserver',
    scalar => 'application',
    scalar => 'application_str',
    scalar => 'volume_msid_str',
    scalar => 'size',
    scalar => 'state',
    scalar => 'application_uuid',
    scalar => 'vserver_uuid',
    scalar => 'volume_msid',
];
my $Log       = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $ApplicationVolume_state  = NACL::CS::ApplicationVolume->fetch(command_interface => $ci, ...);
	my @ApplicationVolume_states = NACL::CS::ApplicationVolume->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes application-volume-get-iter API for CMode ZAPI iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'application volume show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter',
            },

        ],
        exception_text => 'No matching ApplicationVolume(s) found',
        frontend       => 'NACL::CS::ApplicationVolume::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _update_state_objs_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->SUPER::_update_state_objs_cmode_zapi(@args,
        zapi_field_translations => {hyphenate_value => [qw(state)],},);
    $Log->exit() if $may_exit;
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'application_volume_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $copy = [ qw(volume vserver state size vserver-uuid
            application application-uuid volume-msid )];
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => $copy,
        api => "application-volume-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
