# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary CapacityBalanceVolumeRebalance ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::CapacityBalanceVolumeRebalance

=head1 DESCRIPTION

C<NACL::CS::CapacityBalanceVolumeRebalance> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP CapacityBalanceVolumeRebalance.  A related class is
L<NACL::C::CapacityBalanceVolumeRebalance|lib-NACL-C-CapacityBalanceVolumeRebalance-pm>, which represents access
to an ONTAP CapacityBalanceVolumeRebalance.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the CapacityBalanceVolumeRebalance
element are the attributes of the CapacityBalanceVolumeRebalance ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< target >>

Target Amount

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<volume-capacity-balance-info target-bytes="$value" />



=item C<< volume >>

Volume Name

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<volume-capacity-balance-info volume-name="$value" />



=item C<< storage_service >>

Storage Service

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<volume-capacity-balance-info storage-service="$value" />



=item C<< vserver >>

Vserver

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<volume-capacity-balance-info vserver-name="$value" />



=item C<< progress >>

Progress

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<volume-capacity-balance-info progress="$value" />



=item C<< transferred >>

Amount Transferred

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<volume-capacity-balance-info transferred-bytes="$value" />



=item C<< transferred_percent >>

Percentage Transferred

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<volume-capacity-balance-info transferred-percent="$value" />



=item C<< state >>

State

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<volume-capacity-balance-info state="$value" />



=back

=cut

package NACL::CS::CapacityBalanceVolumeRebalance;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'target',
    scalar => 'volume',
    scalar => 'storage_service',
    scalar => 'vserver',
    scalar => 'progress',
    scalar => 'transferred',
    scalar => 'transferred_percent',
    scalar => 'state',
];

=head1 METHODS

=head2 fetch

    my $CapacityBalanceVolumeRebalance_state  = NACL::CS::CapacityBalanceVolumeRebalance->fetch(command_interface => $ci, ...);
	my @CapacityBalanceVolumeRebalance_states = NACL::CS::CapacityBalanceVolumeRebalance->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes volume-capacity-balance-get-iter API for CMode ZAPI.


=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        exception_text =>
            'No matching capacity-balance volume rebalance(s) found',
        frontend => 'NACL::CS::CapacityBalanceVolumeRebalance::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'capacity_balance_volume_rebalance_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(storage-service progress transferred-percent state
                )
        ],
        map => {
            'target'      => 'target-bytes',
            'volume'      => 'volume-name',
            'transferred' => 'transferred-bytes',
            'vserver'     => 'vserver-name',
        },
        api => "volume-capacity-balance-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
