# $Id: //depot/prod/test/nacldev/lib/NACL/CS/Client/Ping.pm#1 $

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Ping ComponentState Module
## @author sshaik@netapp.com,dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::Client::Ping

=head1 DESCRIPTION

This module does not represent the state of any element, but is an object
representation of the output of ping

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the Ping 
element are the attributes of the Ping ComponentState.

=over

<< bytes >>

(Applicable for Linux & Windows)
The size of the data segment of the ping packet &
the size of the entire outbound IP packet withing braces.

=item C<< packets_transmitted >>

(Applicable for Linux & Windows)
Total number of packets transmitted.

=item C<< loss_percent >>

(Applicable for Linux & Windows)
The packet loss in percentage

=item C<< ip >>

(Applicable for Linux & Windows)
The ipaddress even if the destination is hostname.

=item C<< received >>

(Applicable for Linux & Windows)
Total number of packets received

=item C<< stats >>

(Applicable for Linux & Windows)
The statistics of each packet sent includes ttl, icmp_seq, time etc.

=item C<< host >>

(Applicable for Linux & Solaris)
The ipaddress or fully qualified name of the destination if the destination is hostname

=item C<< avg >>

(Applicable for Linux & Windows)
The average round trip time for echo request & echo reply packets in ms.

=item C<< min >>

(Applicable for Linux & Windows)
The minimum round trip time for echo request & echo reply packets in ms.

=item C<< max >>

(Applicable for Linux & Windows)
The maximum round trip time for echo request & echo reply packets in ms.

=item C<< loss >>

(Applicable for Windows)
The number of packet lost.

=item C<< state >>

(Applicable for Solaris)
The ping state is alive for sucessful ping.

=item C<< time >>

(Applicable for Linux)
Total rrt taken in ms.

=item C<< mdev >>

(Applicable for Linux)
The mean deviation round trip time for echo request & echo reply packets in ms.

=cut

package NACL::CS::Client::Ping;

use strict;
use warnings;
use base qw(NACL::CS::ComponentState::Client);

use Class::MethodMaker [
    scalar => 'destination',
    ## Linux & Windows
    scalar => 'bytes',
    scalar => 'packets_transmitted',
    scalar => 'loss_percent',
    scalar => 'ip',
    scalar => 'received',
    array  => 'stats',
    scalar => 'avg',
    scalar => 'min',
    scalar => 'max',

    ## Linux & Solaris
    scalar => 'host',

    ## Linux
    scalar => 'time',
    scalar => 'mdev',

    ## Windows
    scalar => 'loss',

    ## Solaris
    scalar => 'state',
];

1;
