#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ClusterIdentity ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::ClusterIdentity

=head1 DESCRIPTION

C<NACL::CS::ClusterIdentity> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP ClusterIdentity.  A related class is
L<NACL::C::ClusterIdentity|lib-NACL-C-ClusterIdentity-pm>, which represents access
to an ONTAP ClusterIdentity.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the ClusterIdentity
element are the attributes of the ClusterIdentity ComponentState.

=over

=item C<< "location" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

Input:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

output:

 <cluster-identity-info>
    <cluster-location>$value</cluster-location>
 </cluster-identity-info>

=item C<< "serialnumber" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

Input:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

output:

 <cluster-identity-info>
    <cluster-serial-number>$value</cluster-serial-number>
 </cluster-identity-info>

=item C<< "name" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

Input:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

output:

 <cluster-identity-info>
    <cluster-name>$value</cluster-name>
 </cluster-identity-info>

=item C<< "contact" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

Input:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

output:

 <cluster-identity-info>
    <cluster-contact>$value</cluster-contact>
 </cluster-identity-info>

=item C<< "uuid" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

Input:
 
For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

output:

 <cluster-identity-info>
    <cluster-uuid>$value</cluster-uuid>
 </cluster-identity-info>

=item C<< "uuid_str" >>

Filled in for CMode CLI.

=item C<< "rdb_uuid" >>

Filled in for CMode CLI.

=back

=cut

package NACL::CS::ClusterIdentity;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use NACL::APISet::Exceptions::ResponseException qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'location',
    scalar => 'serialnumber',
    scalar => 'name',
    scalar => 'contact',
    scalar => 'uuid',
    scalar => 'uuid_str',
    scalar => 'rdb_uuid',
];

=head1 METHODS

=head2 fetch

    my $ClusterIdentity_state  = NACL::CS::ClusterIdentity->fetch(command_interface => $ci, ...);
	my @ClusterIdentity_states = NACL::CS::ClusterIdentity->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "cluster-identity-get" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching cluster identity(s) found',
        is_singleton   => 1,
        show_cmd       => 'cluster identity show',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs;
    try {
        @state_objs = $pkg->SUPER::_fetch_cmode_cli(
            @args,
            api          => 'cluster_identity_show',
            is_singleton => 1
        );
    }
    catch NACL::APISet::Exceptions::CommandNotFoundException with {
        my $exception = $_[0];
        $Log->exit() if $may_exit;
        NACL::C::Exceptions::ClusterIdentity::PreClusterMode
            ->convert_and_throw(exception => $exception);
    };
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();
    my $apiset = $opts{apiset};
    my $requested_fields = $opts{requested_fields};
    my $filter           = $opts{filter};
    my $caught_exception;
    my $ci_response;
    my %ci_list_info_args;

    # find all the cluster identity present.
    try {
        $ci_response = $apiset->cluster_identity_get(%ci_list_info_args);
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        if ($exception->text() =~ /ZAPI is not enabled in pre-cluster mode/) {
            $Log->exit() if $may_exit;
            NACL::C::Exceptions::ClusterIdentity::PreClusterMode
                ->convert_and_throw(exception => $exception);
        } else {

     # A caught exception indicates that the cluster identity being looked for
     # does not exist. We catch the exception and return immediately. The
     # 'fetch' frontend decides whether to throw a NoElementsFound
     # exception based on the value of 'allow_empty'
            $caught_exception = 1;
        }
    };
    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }

    my $ci_output = $ci_response->get_parsed_output();
    $Log->debug(Dumper($ci_output));
    my @state_objs;
    foreach
        my $row (@{$ci_output->[0]{'attributes'}[0]{'cluster-identity-info'}})
    {
        my $state_fields = $pkg->_zapi_hash_copy(
            source                  => $row,
            source_has_extra_arrays => 1,
            map                     => [
                "cluster-contact"       => "contact",
                "cluster-location"      => "location",
                "cluster-name"          => "name",
                "cluster-uuid"          => "uuid",
                "cluster-serial-number" => "serialnumber",
            ],
        );
        my $obj = $pkg->new(command_interface => $opts{command_interface});
        $obj->_set_fields(row => $state_fields);
        push @state_objs, $obj;
    } ## end foreach my $row ( @{ $ci_output...

    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
