# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::ClusterImagePackage->show_local() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::ClusterImagePackageShowLocal

=head1 DESCRIPTION

C<NACL::CS::ClusterImagePackageShowLocal> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::ClusterImagePackage->show_local() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::ClusterImagePackage->show_local() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< download_ver >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <cluster-image-package-local-info>
    <package-version>$value</package-version>
 </cluster-image-package-local-info>

=item C<< component_name >>

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $component_name = $obj->component_name();
# $component_name contains a reference to the array of values
my @component_name = $obj->component_name();
# @component_name contains the array of values

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <cluster-image-package-local-info>
    <package-details>
        <package-details-info>
            <component-name>@values</component-name>
        </package-details-info>
    <package-details>
 </cluster-image-package-local-info>

=item C<< component_version >>

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $component_version = $obj->component_version();
# $component_version contains a reference to the array of values
my @component_version = $obj->component_version();
# @component_version contains the array of values

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <cluster-image-package-local-info>
    <package-details>
        <package-details-info>
            <component-version>@values</component-version>
        </package-details-info>
    <package-details>
 </cluster-image-package-local-info>

=back

=cut

package NACL::CS::ClusterImagePackageShowLocal;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'download_ver',
    array  => 'component_name',
    array  => 'component_version',
];

=head1 METHODS

=head2 fetch

    my $ClusterImagePackageShowLocal_state  = NACL::CS::ClusterImagePackageShowLocal->fetch(command_interface => $ci, ...);
    my @ClusterImagePackageShowLocal_states = NACL::CS::ClusterImagePackageShowLocal->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "cluster-image-package-local-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching cluster image package(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(
        @_,
        api => 'cluster_image_package_show_local',

    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api => 'cluster_image_package_local_get_iter',
        map => {
            'download_ver'      => 'package-version',
            "component-version" => [
                make_zapi_array('package-details'),
                make_zapi_skip('package-details-info'),
                'component-version',
            ],
            "component-name" => [
                make_zapi_array('package-details'),
                make_zapi_skip('package-details-info'),
                'component-name',
            ],
        },
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
