# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ClusterKernelService ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::ClusterKernelService

=head1 DESCRIPTION

C<NACL::CS::ClusterKernelService> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP ClusterKernelService.  A related class is
L<NACL::C::ClusterKernelService|lib-NACL-C-ClusterKernelService-pm>, which represents access
to an ONTAP ClusterKernelService.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the ClusterKernelService
element are the attributes of the ClusterKernelService ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< rc_retries >>

Filled in for CMode CLI.

=item C<< version >>

Filled in for CMode CLI.

=item C<< rc_timeout_oper >>

Filled in for CMode CLI.

=item C<< failover >>

Filled in for CMode CLI.

=item C<< nodelist >>

Filled in for CMode CLI.
(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $nodelist = $obj->nodelist();
# $nodelist contains a reference to the array of values
my @nodelist = $obj->nodelist();
# @nodelist contains the array of values

Filled in for CMode CLI.

=item C<< node >>

Filled in for CMode CLI.

=item C<< state >>

Filled in for CMode CLI.

=item C<< rc_timeout >>

Filled in for CMode CLI.

=item C<< fsm >>

Filled in for CMode CLI.

=item C<< quorum >>

Filled in for CMode CLI.
(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $quorum = $obj->quorum();
# $quorum contains a reference to the array of values
my @quorum = $obj->quorum();
# @quorum contains the array of values

Filled in for CMode CLI.

=item C<< rc_retries_oper >>

Filled in for CMode CLI.

=item C<< dump >>

Core On Takeover possible value(s) are, enabled,disabled

Filled in for CMode CLI.


=item C<< status_oper >>


Operational Status possible value(s) are, unknown,operational,not-operational

Filled in for CMode CLI.

=item C<< master_node >>

Node possible value(s) are, <nodename>,local

Filled in for CMode CLI.

=item C<< status_quorum >>

Quorum Status possible value(s) are, out of quorum,in quorum

Filled in for CMode CLI.

=item C<< cluster_node >>

Cluster Node

Filled in for CMode CLI.

=item C<< status_avail >>

Availability Status possible value(s) are, true,false

Filled in for CMode CLI.


=back

=cut

package NACL::CS::ClusterKernelService;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'rc_retries',
    scalar => 'version',
    scalar => 'rc_timeout_oper',
    scalar => 'failover',
    array  => 'nodelist',
    scalar => 'node',
    scalar => 'state',
    scalar => 'rc_timeout',
    scalar => 'fsm',
    array  => 'quorum',
    scalar => 'rc_retries_oper',
    scalar => 'dump',
    scalar => 'status_oper',
    scalar => 'master_node',
    scalar => 'status_quorum',
    scalar => 'cluster_node',
    scalar => 'status_avail',
];

=head1 METHODS

=head2 fetch

    my $ClusterKernelService_state  = NACL::CS::ClusterKernelService->fetch(command_interface => $ci, ...);
	my @ClusterKernelService_states = NACL::CS::ClusterKernelService->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI APISet.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'cluster kernel-service show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching cluster kernel-service(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'cluster_kernel_service_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

1;
