#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ClusterPeerAddressActive ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::ClusterPeerAddressActive

=head1 DESCRIPTION

C<NACL::CS::ClusterPeerAddressActive> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP ClusterPeerAddressActive.  A related class is
L<NACL::C::ClusterPeerAddressActive|lib-NACL-C-ClusterPeerAddressActive-pm>, which represents access
to an ONTAP ClusterPeerAddressActive.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the ClusterPeerAddressActive
element are the attributes of the ClusterPeerAddressActive ComponentState.

=over

=item C<< "cluster" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

Input:

For "filter":

 <cluster-peer-active-address>
    <cluster-name>$value</cluster-name>
 </cluster-peer-active-address>

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

output:

 <cluster-peer-active-address>
    <cluster-name>$value</cluster-name>
 </cluster-peer-active-address>

=item C<< "addrs" >>

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $addrs = $obj->addrs();

# $addrs contains a reference to the array of values

my @addrs = $obj->addrs();

# @addrs contains the array of values

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

Input:

For "filter": cluster-name

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

output:

 <cluster-peer-active-address>
    <peer-addresses>
        <remote-inet-address>@values</remote-inet-address>
    <peer-addresses>
 </cluster-peer-active-address>

=back

=cut

package NACL::CS::ClusterPeerAddressActive;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one Dumper);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'cluster',
    array  => 'addrs',
];

=head1 METHODS

=head2 fetch

    my $ClusterPeerAddressActive_state   = NACL::CS::ClusterPeerAddressActive->fetch(command_interface => $ci, ...);
	my @ClusterPeerAddressActive_states = NACL::CS::ClusterPeerAddressActive->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "cluster-peer-active-addresses-get" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                check     => '_zapi_check'
            },
        ],
        exception_text => 'No matching cluster peer address active(s) found',
        show_cmd       => 'cluster peer address active show',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'cluster_peer_address_active_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs   = $pkg->_fetch_cmode_zapi_non_iter(
        @args,
        api => 'cluster-peer-active-addresses-get',
        map => {
            cluster => 'cluster-name',
            addrs   => [
                make_zapi_array('peer-addresses'),
                make_zapi_skip('remote-inet-address'),
            ],
        },
        primary_keys => ['cluster'],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _zapi_check {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(@opts, _primary_keys => ['cluster']);
    $Log->exit() if $may_exit;
}

1;
