# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ClusterPeerOffer ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::ClusterPeerOffer

=head1 DESCRIPTION

C<NACL::CS::ClusterPeerOffer> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP ClusterPeerOffer.  A related class is
L<NACL::C::ClusterPeerOffer|lib-NACL-C-ClusterPeerOffer-pm>, which represents access
to an ONTAP ClusterPeerOffer.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the ClusterPeerOffer
element are the attributes of the ClusterPeerOffer ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< offer_expiration >>

Authentication Offer Expiration Time
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <cluster-peer-authn-offer offer-expiration="$value" />


=item C<< salt >>

Intro Salt
Filled in for CMode CLI/ZAPI iter, non-iter.

=item C<< cluster >>

Peer Cluster Name
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <cluster-peer-authn-offer peer-cluster-name="$value" />

Mapping for CMode ZAPI non-iter: peer-cluster-name


=item C<< cluster_uuid >>

Cluster UUID
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <cluster-peer-authn-offer peer-cluster-uuid="$value" />

Mapping for CMode ZAPI non-iter: peer-cluster-uuid


=item C<< auth_status_operational >>

Authentication Status Operational
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <cluster-peer-authn-offer auth-status-operational="$value" />


=item C<< intro_key >>

Intro Key
Filled in for CMode CLI/ZAPI iter, non-iter.

=item C<< offer_creation >>

Authentication Offer Creation Time
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <cluster-peer-authn-offer offer-creation="$value" />


=item C<< auth_status_admin >>

Authentication Status Administrative
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <cluster-peer-authn-offer auth-status-admin="$value" />

=back

=cut

package NACL::CS::ClusterPeerOffer;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'offer_expiration',
    scalar => 'salt',
    scalar => 'cluster',
    scalar => 'cluster_uuid',
    scalar => 'auth_status_operational',
    scalar => 'intro_key',
    scalar => 'offer_creation',
    scalar => 'auth_status_admin',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [qw(cluster)],
        _vs_context   => 0
    );
    $Log->exit() if $may_exit;
}

=head1 METHODS

=head2 fetch

    my $ClusterPeerOffer_state  = NACL::CS::ClusterPeerOffer->fetch(command_interface => $ci, ...);
	my @ClusterPeerOffer_states = NACL::CS::ClusterPeerOffer->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes cluster-peer-authn-offer-get-iter API for CMode ZAPI iter.

Invokes cluster-peer-authn-offer-get API for CMode ZAPI non-iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'cluster peer offer show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_check_non_iter'
            },
        ],
        exception_text => 'No matching cluster peer offer(s) found',
        frontend       => 'NACL::CS::ClusterPeerOffer::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'cluster_peer_offer_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(offer-expiration auth-status-operational offer-creation auth-status-admin
                )
        ],
        map => {
            'cluster'      => 'peer-cluster-name',
            'cluster-uuid' => 'peer-cluster-uuid',
        },
        api => "cluster-peer-authn-offer-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        copy => [
            qw(offer-expiration auth-status-operational offer-creation auth-status-admin
                )
        ],
        map => {
            'cluster'      => 'peer-cluster-name',
            'cluster-uuid' => 'peer-cluster-uuid',
        },
        api          => "cluster-peer-authn-offer-get",
        primary_keys => [qw( cluster )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
