# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::DebugCpsServer->show_summary() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::DebugCpsServerShowSummary

=head1 DESCRIPTION

C<NACL::CS::DebugCpsServerShowSummary> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::DebugCpsServer->show_summary() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::DebugCpsServer->show_summary() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< cps_storage_areas >>


CPS Storage Areas possible value(s) are, crs-scratch,crs-sender-queues,crs-receiver-queues
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $cps_storage_areas = $obj->cps_storage_areas();
 
# $cps_storage_areas contains a reference to the array of values
 
my @cps_storage_areas = $obj->cps_storage_areas();
 
# @cps_storage_areas contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { cps_storage_areas = [ value1, value2...] }

Filled in for CMode CLI

=item C<< used_active_volume >>

Used Size on Current Active Volume

Filled in for CMode CLI

=item C<< number_successful_storage_area_repairs >>

Number of successful Storage Area Repairs

Filled in for CMode CLI

=item C<< host_node_active >>


Host Node for Current Active Volume possible value(s) are, <nodename>,local

Filled in for CMode CLI

=item C<< current_standby_volume >>

Current Standby Volume

Filled in for CMode CLI

=item C<< number_failed_storage_area_repairs >>

Number of failed Storage Area Repairs

Filled in for CMode CLI

=item C<< group >>

Storage Area Group

Filled in for CMode CLI

=item C<< available_active_volume >>

Available Size on Current Active Volume

Filled in for CMode CLI

=item C<< path_to_store >>

Current Path to Persistent Storage

Filled in for CMode CLI

=item C<< number_current_abandoned_volumes >>

Number of current abandoned volumes

Filled in for CMode CLI

=item C<< used_standby_volume >>

Used Size on Current Standby Volume

Filled in for CMode CLI

=item C<< aggr_size_available_standby_volume >>

Parent Aggr Avail Current Standby Volume

Filled in for CMode CLI

=item C<< percent_used_standby >>

Used Percentage on Current Standby Volume

Filled in for CMode CLI

=item C<< number_volume_relocations >>

Number of Volume Relocations

Filled in for CMode CLI

=item C<< aggr_size_available_active_volume >>

Parent Aggr Avail Current Active Volume

Filled in for CMode CLI

=item C<< current_active_volume >>

Current Active Volume

Filled in for CMode CLI

=item C<< percent_used_active >>

Used Percentage on Current Active Volume

Filled in for CMode CLI

=item C<< host_node_standby >>


Host Node for Current Standby Volume possible value(s) are, <nodename>,local

Filled in for CMode CLI

=item C<< available_standby_volume >>

Available Size on Current Standby Volume

Filled in for CMode CLI

=item C<< number_io_errors_active >>

Number of I/O Errors Current Active Volume

Filled in for CMode CLI

=item C<< number_storage_area_repairs >>

Number of Storage Area Repairs on Current Active Volume

Filled in for CMode CLI.


=item C<< number_active_clients >>

Number of Active Clients

Filled in for CMode CLI.


=back

=cut

package NACL::CS::DebugCpsServerShowSummary;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    array  => 'cps_storage_areas',
    scalar => 'used_active_volume',
    scalar => 'number_successful_storage_area_repairs',
    scalar => 'host_node_active',
    scalar => 'current_standby_volume',
    scalar => 'number_failed_storage_area_repairs',
    scalar => 'group',
    scalar => 'available_active_volume',
    scalar => 'path_to_store',
    scalar => 'number_current_abandoned_volumes',
    scalar => 'used_standby_volume',
    scalar => 'aggr_size_available_standby_volume',
    scalar => 'percent_used_standby',
    scalar => 'number_volume_relocations',
    scalar => 'aggr_size_available_active_volume',
    scalar => 'current_active_volume',
    scalar => 'percent_used_active',
    scalar => 'host_node_standby',
    scalar => 'available_standby_volume',
    scalar => 'number_io_errors_active',
    scalar => 'number_storage_area_repairs',
    scalar => 'number_active_clients',
];

=head1 METHODS

=head2 fetch

    my $DebugCpsServerShowSummary_state  = NACL::CS::DebugCpsServerShowSummary->fetch(command_interface => $ci, ...);
	my @DebugCpsServerShowSummary_states = NACL::CS::DebugCpsServerShowSummary->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
	show_cmd => 'debug cps server show-summary',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching debug cps server(s) found',
        frontend       => 'NACL::CS::DebugCpsServerShowSummary::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'debug_cps_server_show_summary',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
