# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::DebugCrsDomain->show_objects() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::DebugCrsDomainShowObjects

=head1 DESCRIPTION

C<NACL::CS::DebugCrsDomainShowObjects> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::DebugCrsDomain->show_objects() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::DebugCrsDomain->show_objects() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< list_of_objects >>

List of Objects
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $list_of_objects = $obj->list_of_objects();
 
# $list_of_objects contains a reference to the array of values
 
my @list_of_objects = $obj->list_of_objects();
 
# @list_of_objects contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { list_of_objects = [ value1, value2...] }

Filled in for CMode CLI

=item C<< domain >>

Replication Domain possible value(s) are, cluster,test,vserver

Filled in for CMode CLI

=back

=cut

package NACL::CS::DebugCrsDomainShowObjects;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    array  => 'list_of_objects',
    scalar => 'domain',
];

=head1 METHODS

=head2 fetch

    my $DebugCrsDomainShowObjects_state  = NACL::CS::DebugCrsDomainShowObjects->fetch(command_interface => $ci, ...);
	my @DebugCrsDomainShowObjects_states = NACL::CS::DebugCrsDomainShowObjects->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
	show_cmd => 'debug crs domain show-objects',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching debug crs domain(s) found',
        frontend       => 'NACL::CS::DebugCrsDomainShowObjects::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'debug_crs_domain_show_objects',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
