# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary DebugDmVserverStream ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::DebugDmVserverStream

=head1 DESCRIPTION

C<NACL::CS::DebugDmVserverStream> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP DebugDmVserverStream.  A related class is
L<NACL::C::DebugDmVserverStream|lib-NACL-C-DebugDmVserverStream-pm>, which represents access
to an ONTAP DebugDmVserverStream.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the DebugDmVserverStream
element are the attributes of the DebugDmVserverStream ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< app_instance_uuid >>

Application Instance UUID

Filled in for CMode CLI

=item C<< queue_storage_capacity >>

Queue Storage Capacity

Filled in for CMode CLI

=item C<< stream_uuid >>

Stream UUID

Filled in for CMode CLI

=item C<< league_uuid >>

League UUID

Filled in for CMode CLI

=item C<< service_notification >>

Service Notify Options

Filled in for CMode CLI

=item C<< vserver >>

Vserver

Filled in for CMode CLI

=item C<< remote_cluster_uuid >>

Remote Cluster UUID

Filled in for CMode CLI

=item C<< replication_type >>

Replication Type possible value(s) are, async,sync

Filled in for CMode CLI

=item C<< path_to_persistent_storage >>

Path to Persistent Storage

Filled in for CMode CLI

=item C<< app_callback >>

Application Call Back Table Name

Filled in for CMode CLI

=item C<< data >>

Data

Filled in for CMode CLI

=item C<< replication_request >>


Replication Request possible value(s) are, abort , apply-baseline-till-baseline-marker , apply-baseline-till-sync-marker , apply-manual-baseline , apply-manual-rebaseline , apply-manual-update , apply-till-seq-id , apply-till-sync-marker , apply-update-till-seq-id , apply-update-till-sync-marker , generate-baseline , generate-local-baseline , generate-flip-rebaseline , generate-rebaseline , insert-marker , peek-and-apply , process-backlog

Filled in for CMode CLI

=item C<< direction >>

Stream Direction possible value(s) are, sender,receiver

Filled in for CMode CLI

=item C<< message_notification >>

Message Notify Options

Filled in for CMode CLI

=item C<< retry_count >>

Retry Count

Filled in for CMode CLI

=item C<< domain_instance_uuid >>

Domain Instance UUID

Filled in for CMode CLI

=item C<< app_name >>

Application Name possible value(s) are, async-dr,sync-dr

Filled in for CMode CLI

=item C<< in_drain_phase >>

In Drain Phase possible value(s) are, true,false

Filled in for CMode CLI.


=item C<< notification >>

Notification Options
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $notification = $obj->notification();
 
# $notification contains a reference to the array of values
 
my @notification = $obj->notification();
 
# @notification contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { notification = [ value1, value2...] }

Filled in for CMode CLI.


=item C<< in_error >>

Is Stream in Error? possible value(s) are, true,false

Filled in for CMode CLI.


=item C<< is_reverse >>

Stream for FlipRebaseline possible value(s) are, true,false

Filled in for CMode CLI.


=back

=cut

package NACL::CS::DebugDmVserverStream;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'app_instance_uuid',
    scalar => 'queue_storage_capacity',
    scalar => 'stream_uuid',
    scalar => 'league_uuid',
    scalar => 'service_notification',
    scalar => 'vserver',
    scalar => 'remote_cluster_uuid',
    scalar => 'replication_type',
    scalar => 'path_to_persistent_storage',
    scalar => 'app_callback',
    scalar => 'data',
    scalar => 'replication_request',
    scalar => 'direction',
    scalar => 'message_notification',
    scalar => 'retry_count',
    scalar => 'domain_instance_uuid',
    scalar => 'app_name',
    scalar => 'in_drain_phase',
    array  => 'notification',
    scalar => 'in_error',
    scalar => 'is_reverse',
];

=head1 METHODS

=head2 fetch

    my $DebugDmVserverStream_state  = NACL::CS::DebugDmVserverStream->fetch(command_interface => $ci, ...);
	my @DebugDmVserverStream_states = NACL::CS::DebugDmVserverStream->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
	show_cmd => 'debug dm vserver stream show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching debug dm vserver stream(s) found',
        frontend       => 'NACL::CS::DebugDmVserverStream::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'debug_dm_vserver_stream_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
