# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary DebugNetworkRecovery ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::DebugNetworkRecovery

=head1 DESCRIPTION

C<NACL::CS::DebugNetworkRecovery> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP DebugNetworkRecovery.  A related class is
L<NACL::C::DebugNetworkRecovery|lib-NACL-C-DebugNetworkRecovery-pm>, which represents access
to an ONTAP DebugNetworkRecovery.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the DebugNetworkRecovery
element are the attributes of the DebugNetworkRecovery ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< timestamp >>

Timestamp

Filled in for CMode CLI

=item C<< in_flight >>

In-Flight possible value(s) are, true,false

Filled in for CMode CLI

=item C<< action >>

Recovery Action

Filled in for CMode CLI

=item C<< object_name >>

Object Name

Filled in for CMode CLI

=item C<< object_id >>

Object Identifier

Filled in for CMode CLI

=item C<< instance_id >>

Instance Counter

Filled in for CMode CLI

=item C<< process >>

Process Name

Filled in for CMode CLI.


=back

=cut

package NACL::CS::DebugNetworkRecovery;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'timestamp',
    scalar => 'in_flight',
    scalar => 'action',
    scalar => 'object_name',
    scalar => 'object_id',
    scalar => 'instance_id',
    scalar => 'process',
];

=head1 METHODS

=head2 fetch

    my $DebugNetworkRecovery_state  = NACL::CS::DebugNetworkRecovery->fetch(command_interface => $ci, ...);
	my @DebugNetworkRecovery_states = NACL::CS::DebugNetworkRecovery->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
	show_cmd => 'debug network recovery show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching debug network recovery(s) found',
        frontend       => 'NACL::CS::DebugNetworkRecovery::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'debug_network_recovery_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
