# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary DebugProcessMemory ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::DebugProcessMemory

=head1 DESCRIPTION

C<NACL::CS::DebugProcessMemory> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP DebugProcessMemory.  A related class is
L<NACL::C::DebugProcessMemory|lib-NACL-C-DebugProcessMemory-pm>, which represents access
to an ONTAP DebugProcessMemory.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the DebugProcessMemory
element are the attributes of the DebugProcessMemory ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< virtual_size >>

Virtual Size

Filled in for CMode CLI

=item C<< heap_used >>

Heap

Filled in for CMode CLI

=item C<< max_virtual_size >>

Max virtual size

Filled in for CMode CLI

=item C<< rss >>

The RSS size

Filled in for CMode CLI

=item C<< node >>

Node possible value(s) are, <nodename>,local

Filled in for CMode CLI

=item C<< process >>

Process Name

Filled in for CMode CLI

=item C<< open_fds >>

Open Files

Filled in for CMode CLI.


=back

=cut

package NACL::CS::DebugProcessMemory;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'virtual_size',
    scalar => 'heap_used',
    scalar => 'max_virtual_size',
    scalar => 'rss',
    scalar => 'node',
    scalar => 'process',
    scalar => 'open_fds',
];

=head1 METHODS

=head2 fetch

    my $DebugProcessMemory_state  = NACL::CS::DebugProcessMemory->fetch(command_interface => $ci, ...);
	my @DebugProcessMemory_states = NACL::CS::DebugProcessMemory->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
	show_cmd => 'debug process-memory show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching debug process-memory(s) found',
        frontend       => 'NACL::CS::DebugProcessMemory::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'debug_process_memory_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
