#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary JobPrivate ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::JobPrivateShowCompleted

=head1 DESCRIPTION

C<NACL::CS::JobPrivate> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP JobPrivate.  A related class is
L<NACL::C::JobPrivate|lib-NACL-C-JobPrivate-pm>, which represents access
to an ONTAP JobPrivate.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the JobPrivate
element are the attributes of the JobPrivate ComponentState.

=over

=item C<< "priority" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <job-priority>$value</job-priority>
 </job-info>

=item C<< "restarted" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <is-restarted>$value</is-restarted>
 </job-info>

=item C<< "node" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <job-node>$value</job-node>
 </job-info>

=item C<< "uuid" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <job-uuid>$value</job-uuid>
 </job-info>

=item C<< "state" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <job-state>$value</job-state>
 </job-info>

=item C<< "starttime" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <job-start-time>$value</job-start-time>
 </job-info>

=item C<< "id" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <job-id>$value</job-id>
 </job-info>

=item C<< "category" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <job-category>$value</job-category>
 </job-info>

=item C<< "endtime" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <job-end-time>$value</job-end-time>
 </job-info>

=item C<< "queuetime" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <job-queue-time>$value</job-queue-time>
 </job-info>

=item C<< "code" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <job-status-code>$value</job-status-code>
 </job-info>

=item C<< "dropdeadtime" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <job-dropdead-time>$value</job-dropdead-time>
 </job-info>

=item C<< "vserver" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <job-vserver>$value</job-vserver>
 </job-info>

=item C<< "progress" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <job-progress>$value</job-progress>
 </job-info>

=item C<< "name" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <job-name>$value</job-name>
 </job-info>

=item C<< "description" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <job-description>$value</job-description>
 </job-info>

=item C<< "completion" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <job-completion>$value</job-completion>
 </job-info>

=item C<< "schedule" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <job-schedule>$value</job-schedule>
 </job-info>

=item C<< "username" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <job-username>$value</job-username>
 </job-info>

=item C<< "jobtype" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-info>
     <job-type>$value</job-type>
 </job-info>

=back

=cut

package NACL::CS::JobPrivateShowCompleted;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one Dumper);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'priority',
    scalar => 'restarted',
    scalar => 'node',
    scalar => 'uuid',
    scalar => 'state',
    scalar => 'starttime',
    scalar => 'id',
    scalar => 'category',
    scalar => 'endtime',
    scalar => 'queuetime',
    scalar => 'code',
    scalar => 'dropdeadtime',
    scalar => 'vserver',
    scalar => 'progress',
    scalar => 'name',
    scalar => 'description',
    scalar => 'completion',
    scalar => 'schedule',
    scalar => 'username',
    scalar => 'jobtype',
];

=head1 METHODS

=head2 fetch

    my $JobPrivate_state   = NACL::CS::JobPrivate->fetch(command_interface => $ci, ...);
	my @JobPrivate_states = NACL::CS::JobPrivate->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "job-private-completed-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
	show_cmd       => 'job private show-completed',
        exception_text => 'No matching job private show completed(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'job_private_show_completed');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg    = shift;
    my %opts   = @_;
    my @states = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        api => "job_private_completed_get_iter",
        map => {
            "node"         => ['job-node'],
            "id"           => ['job-id'],
            "name"         => ['job-name'],
            "description"  => ['job-description'],
            "priority"     => ['job-priority'],
            "schedule"     => ['job-schedule'],
            "queuetime"    => ['job-queue-time'],
            "starttime"    => ['job-start-time'],
            "endtime"      => ['job-end-time'],
            "dropdeadtime" => ['job-dropdead-time'],
            "restarted"    => ['is-restarted'],
            "state"        => ['job-state'],
            "code"         => ['job-status-code'],
            "completion"   => ['job-completion'],
            "jobtype"      => ['job-type'],
            "category"     => ['job-category'],
            "uuid"         => ['job-uuid'],
            "progress"     => ['job-progress'],
            "vserver"      => ['job-vserver'],
            "username"     => ['job-username'],
        },
    );
    $Log->exit() if $may_exit;
    return @states;
} ## end sub _fetch_cmode_zapi

1;
