# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary League ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::League

=head1 DESCRIPTION

C<NACL::CS::League> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP League.  A related class is
L<NACL::C::League|lib-NACL-C-League-pm>, which represents access
to an ONTAP League.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the League
element are the attributes of the League ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< master_uuid >>

Master Cluster UUID

Filled in for CMode CLI

=item C<< league_uuid >>

League UUID

Filled in for CMode CLI

=item C<< league_job_uuid >>

League Job UUID

Filled in for CMode CLI

=item C<< job_status >>

Job Status

Filled in for CMode CLI

=item C<< league_ipspace >>

League IPspace

Filled in for CMode CLI

=item C<< master_name >>

Master Cluster

Filled in for CMode CLI

=item C<< admin_contact >>

League Administrator's Contact Info

Filled in for CMode CLI

=item C<< comment >>

League Comment

Filled in for CMode CLI

=item C<< roster_version >>

Current Roster Version

Filled in for CMode CLI

=item C<< passphrase >>

Identification Passphrase

Filled in for CMode CLI

=item C<< league_name >>

League Name

Filled in for CMode CLI

=item C<< self_league_status >>


Status of Self in League possible value(s) are, master,member,application_pending,join_in_progress,rejected,deleted,abandoned

Filled in for CMode CLI

=item C<< updates_enabled >>

Accepting Updates? possible value(s) are, true,false

Filled in for CMode CLI

=back

=cut

package NACL::CS::League;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'master_uuid',
    scalar => 'league_uuid',
    scalar => 'league_job_uuid',
    scalar => 'job_status',
    scalar => 'league_ipspace',
    scalar => 'master_name',
    scalar => 'admin_contact',
    scalar => 'comment',
    scalar => 'roster_version',
    scalar => 'passphrase',
    scalar => 'league_name',
    scalar => 'self_league_status',
    scalar => 'updates_enabled',
];

=head1 METHODS

=head2 fetch

    my $League_state  = NACL::CS::League->fetch(command_interface => $ci, ...);
	my @League_states = NACL::CS::League->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
	show_cmd => 'league show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching league(s) found',
        frontend       => 'NACL::CS::League::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@args, api => 'league_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
