# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary LeagueApplication ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::LeagueApplication

=head1 DESCRIPTION

C<NACL::CS::LeagueApplication> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP LeagueApplication.  A related class is
L<NACL::C::LeagueApplication|lib-NACL-C-LeagueApplication-pm>, which represents access
to an ONTAP LeagueApplication.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the LeagueApplication
element are the attributes of the LeagueApplication ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< salt >>

Applicant Key Salt

Filled in for CMode CLI

=item C<< remote_admin_contact >>

Remote Administrator's Contact Info

Filled in for CMode CLI

=item C<< remote_cluster_ips >>

List of Remote IP Addresses
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $remote_cluster_ips = $obj->remote_cluster_ips();
 
# $remote_cluster_ips contains a reference to the array of values
 
my @remote_cluster_ips = $obj->remote_cluster_ips();
 
# @remote_cluster_ips contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { remote_cluster_ips = [ value1, value2...] }

Filled in for CMode CLI

=item C<< league_uuid >>

League UUID

Filled in for CMode CLI

=item C<< incoming_address >>

Application Source Network Address

Filled in for CMode CLI

=item C<< need_keyphrase >>

Whether Application Requires a Key possible value(s) are, true,false

Filled in for CMode CLI

=item C<< application_number >>

Application Serial Number

Filled in for CMode CLI

=item C<< remote_cluster >>

Name of Remote Cluster

Filled in for CMode CLI

=item C<< passphrase >>

Passphrase for Identification

Filled in for CMode CLI

=item C<< league_name >>

League Name

Filled in for CMode CLI

=item C<< incoming_timestamp >>

When Application was Received

Filled in for CMode CLI

=back

=cut

package NACL::CS::LeagueApplication;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'salt',
    scalar => 'remote_admin_contact',
    array  => 'remote_cluster_ips',
    scalar => 'league_uuid',
    scalar => 'incoming_address',
    scalar => 'need_keyphrase',
    scalar => 'application_number',
    scalar => 'remote_cluster',
    scalar => 'passphrase',
    scalar => 'league_name',
    scalar => 'incoming_timestamp',
];

=head1 METHODS

=head2 fetch

    my $LeagueApplication_state  = NACL::CS::LeagueApplication->fetch(command_interface => $ci, ...);
	my @LeagueApplication_states = NACL::CS::LeagueApplication->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
	show_cmd => 'league application show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching league application(s) found',
        frontend       => 'NACL::CS::LeagueApplication::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'league_application_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
