# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::LunMapping->show_initiator() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::LunMappingShowInitiator

=head1 DESCRIPTION

C<NACL::CS::LunMappingShowInitiator> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::LunMapping->show_initiator() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::LunMapping->show_initiator() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< vserver >>

Vserver Name

Filled in for CMode CLI/ZAPI

=item C<< reporting_nodes >>

Reporting Nodes
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $reporting_nodes = $obj->reporting_nodes();

# $reporting_nodes contains a reference to the array of values

my @reporting_nodes = $obj->reporting_nodes();

# @reporting_nodes contains the array of values

If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { reporting_nodes = [ value1, value2...] }


Filled in for CMode CLI/ZAPI

=item C<< path >>

LUN Path

Filled in for CMode CLI/ZAPI

=item C<< node >>

LUN Node

Filled in for CMode CLI/ZAPI

=item C<< initiator >>

Initiator Name

Filled in for CMode CLI/ZAPI

=item C<< initiator_group >>

Initiator Name

Filled in for CMode CLI/ZAPI

=item C<< initiator_group_uuid >>

Initiator Name

Filled in for CMode CLI/ZAPI

=item C<< igroup >>

Igroup Name

Filled in for CMode CLI

=item C<< lun_uuid >>

LUN UUID

Filled in for CMode CLI/ZAPI

=item C<< vdisk_id >>

Vdisk ID

Filled in for CMode CLI

=item C<< vserver_uuid >>

VServer UUID

Filled in for CMode CLI

=item C<< igroup_uuid >>

Igroup UUID

Filled in for CMode CLI

=item C<< lun_id >>

Logical Unit Number ranges from 0 to 4095

Filled in for CMode CLI/ZAPI

=back

=cut

package NACL::CS::LunMappingShowInitiator;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'vserver',
    array  => 'reporting_nodes',
    scalar => 'path',
    scalar => 'node',
    scalar => 'initiator',
    scalar => 'initiator_group',
    scalar => 'initiator_group_uuid',
    scalar => 'igroup',
    scalar => 'lun_uuid',
    scalar => 'vdisk_id',
    scalar => 'vserver_uuid',
    scalar => 'igroup_uuid',
    scalar => 'lun_id',
];

=head1 METHODS

=head2 fetch

    my $LunMappingShowInitiator_state  = NACL::CS::LunMappingShowInitiator->fetch(command_interface => $ci, ...);
    my @LunMappingShowInitiator_states = NACL::CS::LunMappingShowInitiator->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.


=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'lun mapping show-initiator',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        exception_text => 'No matching lun mapping(s) found',
        frontend       => 'NACL::CS::LunMappingShowInitiator::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'lun_mapping_show_initiator',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();
    my @state_objs;
    my $vserver           = $opts{filter}->{vserver};
    my $initiator         = $opts{filter}->{initiator};
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my %cmode_args;
    $pkg->_handle_zapi_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $vserver,
        command_interface => $command_interface
    );

    $cmode_args{'initiator'} = $initiator;
    my $response = $apiset->lun_initiator_list_map_info(%cmode_args);
    my $output   = $response->get_parsed_output();
    my $array    = $output->[0]->{'lun-maps'}->[0]->{'lun-map-info'};

    foreach my $row (@{$array}) {
        my $targ = {};
        $pkg->_hash_copy(
            source => $row,
            copy   => [
                qw (lun-uuid vserver initiator-group node path
                    initiator-group-uuid reporting-nodes)
            ],
            target => $targ,
        );
        my $obj = $pkg->new(command_interface => $command_interface,);
        $obj->_set_fields(row => $targ);
        push @state_objs, $obj;
    }
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
