# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary LunRelocation ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::LunRelocation

=head1 DESCRIPTION

NACL::CS::LunRelocation has been deprecated, NACL::CS::LunMove should be used
instead.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the LunRelocation
element are the attributes of the LunRelocation ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< scanner_total >>

LUN Relocation Total

Filled in for CMode CLI

=item C<< job_status >>

LUN Relocation Status

Filled in for CMode CLI

=item C<< lun_index >>

LUN Index

Filled in for CMode CLI

=item C<< last_failure_reason >>

Last Failure Reason

Filled in for CMode CLI

=item C<< destination_path >>

Destination Path

Filled in for CMode CLI

=item C<< job_uuid >>

LUN Relocation Job UUID

Filled in for CMode CLI

=item C<< is_snapshot_fenced >>

Is Snapshot Fenced possible value(s) are, true,false

Filled in for CMode CLI

=item C<< cutover_time >>

Cutover Time (secs)

Filled in for CMode CLI/ZAPI

=item C<< max_throughput >>

Maximum Transfer Rate (per sec)

Filled in for CMode CLI

=item C<< progress_percent >>

LUN Relocation Progress (%)

Filled in for CMode CLI

=item C<< vserver >>

Vserver Name

Filled in for CMode CLI/ZAPI

=item C<< elapsed_time >>

Elapsed Time

Filled in for CMode CLI/ZAPI

=item C<< is_destination_ready >>

Is Destination Ready possible value(s) are, true,false

Filled in for CMode CLI/ZAPI

=item C<< is_promoted_late >>

Is Destination Promoted Late possible value(s) are, true,false

Filled in for CMode CLI/ZAPI

=item C<< scanner_progress >>

LUN Relocation Progress

Filled in for CMode CLI/ZAPI

=item C<< source_path >>

Source Path

Filled in for CMode CLI/ZAPI

=item C<< source_vserver >>

Source Vserver

Filled in for CMode CLI/ZAPI

=item C<< source_snapshot_instance_uuid >>

Source Snapshot Instance UUID

Filled in for CMode CLI.

=item C<< source_snapshot >>

Source Snapshot Name

Filled in for CMode CLI.

=item C<< source_snapshot_id >>

Source Snapshot ID

Filled in for CMode CLI.


=back

=cut

package NACL::CS::LunRelocation;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::C::_Mixins::LunCopyRelocation qw(:all);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'scanner_total',
    scalar => 'job_status',
    scalar => 'lun_index',
    scalar => 'last_failure_reason',
    scalar => 'destination_path',
    scalar => 'job_uuid',
    scalar => 'is_snapshot_fenced',
    scalar => 'cutover_time',
    scalar => 'max_throughput',
    scalar => 'progress_percent',
    scalar => 'vserver',
    scalar => 'elapsed_time',
    scalar => 'is_destination_ready',
    scalar => 'is_promoted_late',
    scalar => 'scanner_progress',
    scalar => 'source_path',
    scalar => 'source_vserver',
    scalar => 'source_snapshot_instance_uuid',
    scalar => 'source_snapshot',
    scalar => 'source_snapshot_id',
];

=head1 METHODS

=head2 fetch

    my $LunRelocation_state  = NACL::CS::LunRelocation->fetch(command_interface => $ci, ...);
	my @LunRelocation_states = NACL::CS::LunRelocation->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $command_interface = $opts{'command_interface'};
    if ($command_interface->has_uichange(
            uichange => 'lun-move-renamed-to-lun-move-in-volume')) {
        my @ret = $pkg->_redirect_call(%opts);
        $Log->exit() if $may_exit;
        return wantarray ? @ret : $ret[0];;
    }

    my @state_objs = $pkg->SUPER::fetch(
        %opts,
        show_cmd => 'lun relocation show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        exception_text => 'No matching lun relocation(s) found',
        frontend       => 'NACL::CS::LunRelocation::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@args, api => 'lun_relocation_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $common_copy = $pkg->_common_fetch_zapi_copy();
    push @$common_copy, ('is-promoted-late','vserver');
    my $map = {'source-vserver'    => 'vserver'};    
    my @states = $pkg->SUPER::_fetch_cmode_zapi(
        @opts,
        copy => $common_copy,
        map  => $map,
        api  => 'lun-relocation-get-iter',
    );
    $Log->exit() if $may_exit;
    return @states;
}

sub _redirect_call {
    $Log->enter() if $may_enter;

    my ($pkg_or_obj, %opts) = @_;
    $Log->warn("The command 'lun relocation show' was renamed to "
                    . "'lun move show'. Please change the call to use the new "
                    . "name. Support for this command will be removed from the "
                    . "library soon.");
    require NACL::CS::LunMove;
    $Log->exit() if $may_exit;
    return NACL::CS::LunMove->fetch(%opts);
}

1;
