#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MemoryUsageCluster ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::MemoryUsageCluster

=head1 DESCRIPTION

C<NACL::CS::MemoryUsageCluster> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP MemoryUsageCluster.  A related class is
L<NACL::C::MemoryUsageCluster|lib-NACL-C-MemoryUsageCluster-pm>, which represents access
to an ONTAP MemoryUsageCluster.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the MemoryUsageCluster
element are the attributes of the MemoryUsageCluster ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< numAlloc >>

Number of Memory Allocations

Maps to:

 CMode ZAPI:

<cost-center-statistics-info>
  <num-allocations>$value</num-allocations>
</cost-center-statistics-info>

Filled in for CMode CLI/ZAPI

=item C<< ccName >>

The memory usage category

Maps to:

 CMode ZAPI:

<cost-center-statistics-info>
  <cost-center-name>$value</cost-center-name>
</cost-center-statistics-info>

Filled in for CMode CLI/ZAPI

=item C<< numFree >>

Number of Memory Deallocations

Maps to:

 CMode ZAPI:

<cost-center-statistics-info>
  <num-free>$value</num-free>
</cost-center-statistics-info>

Filled in for CMode CLI/ZAPI

=item C<< sizeAlloc >>

Amount of Memory Allocated (bytes)

Maps to:

 CMode ZAPI:

<cost-center-statistics-info>
  <amount-allocated>$value</amount-allocated>
</cost-center-statistics-info>

Filled in for CMode CLI/ZAPI

=item C<< node >>

Node possible value(s) are, <nodename>,local

Filled in for CMode CLI

=item C<< process >>

The name of the process

Filled in for CMode CLI

=item C<< sizeFree >>

Amount of Memory Freed (Kb)

Maps to:

 CMode ZAPI:

<cost-center-statistics-info>
  <amount-freed>value</amount-freed>
</cost-center-statistics-info>

Filled in for CMode CLI/ZAPI

=item C<< inUse >>

Amount of Memory In Use (Kb)

Maps to:

 CMode ZAPI:

<cost-center-statistics-info>
  <amount-in-use>value</amount-in-use>
</cost-center-statistics-info>


Filled in for CMode CLI/ZAPI

=back

=cut

package NACL::CS::MemoryUsageCluster;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'numAlloc',
    scalar => 'ccName',
    scalar => 'numFree',
    scalar => 'sizeAlloc',
    scalar => 'node',
    scalar => 'process',
    scalar => 'sizeFree',
    scalar => 'inUse',
];

=head1 METHODS

=head2 fetch

    my $MemoryUsageCluster_state  = NACL::CS::MemoryUsageCluster->fetch(command_interface => $ci, ...);
	my @MemoryUsageCluster_states = NACL::CS::MemoryUsageCluster->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI
Invokes "cost-center-statistics-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
	show_cmd       => 'memory usage cluster show',
        exception_text => 'No matching memory usage cluster(s) found',
        frontend       => 'NACL::CS::MemoryUsageCluster::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'memory_usage_cluster_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        api  => 'cost-center-statistics-get-iter',
        copy => [qw( node process )],
        map  => {
            'numAlloc'  => 'num-allocations',
            'ccName'    => 'cost-center-name',
            'numFree'   => 'num-free',
            'sizeAlloc' => 'amount-allocated',
            'sizeFree'  => 'amount-freed',
            'inUse'     => 'amount-in-use'
        },
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
