#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MetroclusterCheckAggr ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::MetroclusterCheckAggr

=head1 DESCRIPTION

C<NACL::CS::MetroclusterCheckAggr> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP MetroclusterCheckAggr.  A related class is
L<NACL::C::MetroclusterCheckAggr|lib-NACL-C-MetroclusterCheckAggr-pm>, which represents access
to an ONTAP MetroclusterCheckAggr.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the MetroclusterCheckAggr
element are the attributes of the MetroclusterCheckAggr ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< cluster >>

Name of Cluster

Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI: cluster-name

=item C<< check >>

Description

Filled in for CMode CLI/ZAPI

=item C<< additional_info >>

Additional information/Recovery steps
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $additional_info = $obj->additional_info();
 
# $additional_info contains a reference to the array of values
 
my @additional_info = $obj->additional_info();
 
# @additional_info contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { additional_info = [ value1, value2...] }

Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI:

<additional-info>
<string>$value</string>
</additional-info>

=item C<< node >>

Name of the node

Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI: node-name

=item C<< aggr >>

Name of the aggregate

Filled in for CMode CLI

=item C<< result >>

Result of the check

Filled in for CMode CLI/ZAPI

=item C<< aggregate >>

Name of the aggregate

Filled in for CMode CLI/ZAPI

=back

=cut

package NACL::CS::MetroclusterCheckAggr;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::CS::ComponentState::ZapiSkip  qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'cluster',
    scalar => 'check',
    array  => 'additional_info',
    scalar => 'node',
    scalar => 'aggr',
    scalar => 'result',
    scalar => 'aggregate',
];

=head1 METHODS

=head2 fetch

    my $MetroclusterCheckAggr_state  = NACL::CS::MetroclusterCheckAggr->fetch(command_interface => $ci, ...);
	my @MetroclusterCheckAggr_states = NACL::CS::MetroclusterCheckAggr->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes "metrocluster-check-aggregate-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        exception_text => 'No matching metrocluster check aggr(s) found',
        frontend       => 'NACL::CS::MetroclusterCheckAggr::fetch',
        show_cmd       => 'metrocluster check aggregate show',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'metrocluster_check_aggr_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
   $Log->enter() if $may_enter;
   my ($pkg, %opts) = @_;

   my @states = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        api  => "metrocluster-check-aggregate-get-iter",
        copy => [qw(aggregate check result)],
        map  => {
                 "additional-info" => [
                       make_zapi_array('additional-info'),
                       make_zapi_skip('string'),
                       ],
                 "cluster"         =>  "cluster-name",
                 "node"            =>  "node-name",
                },
   );

   $Log->exit() if $may_exit;
   return @states;

} ## end sub _fetch_cmode_zapi

1;
