# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::MetroclusterCheckConfigReplication->show_aggregate_eligibility() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::MetroclusterCheckConfigReplicationShowAggregateEligibility

=head1 DESCRIPTION

C<NACL::CS::MetroclusterCheckConfigReplicationShowAggregateEligibility> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::MetroclusterCheckConfigReplication->show_aggregate_eligibility() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::MetroclusterCheckConfigReplication->show_aggregate_eligibility() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< group >>

Storage Area Group

Filled in for CMode CLI.

=item C<< comment >>

Comment for Eligibility Status

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<metrocluster-check-aggregate-eligibility-info comment="$value" />



=item C<< hosted_configuration_replication_volumes >>

Currently Hosted Configuration Replication Volumes
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $hosted_configuration_replication_volumes = $obj->hosted_configuration_replication_volumes();
 
# $hosted_configuration_replication_volumes contains a reference to the array of values
 
my @hosted_configuration_replication_volumes = $obj->hosted_configuration_replication_volumes();
 
# @hosted_configuration_replication_volumes contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { hosted_configuration_replication_volumes = [ value1, value2...] }


Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<metrocluster-check-aggregate-eligibility-info>
  <hosted-configuration-replication-volumes volume-name="@values" />
</metrocluster-check-aggregate-eligibility-info>



=item C<< is_eligible_to_host_additional_volumes >>


Eligibility to Host Another Configuration Replication Volume possible value(s) are, true,false

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<metrocluster-check-aggregate-eligibility-info is-eligible-to-host-additional-volumes="$value" />



=item C<< aggregate >>

Aggregate Name

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<metrocluster-check-aggregate-eligibility-info aggregate="$value" />



=back

=cut

package NACL::CS::MetroclusterCheckConfigReplicationShowAggregateEligibility;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'group',
    scalar => 'comment',
    array  => 'hosted_configuration_replication_volumes',
    scalar => 'is_eligible_to_host_additional_volumes',
    scalar => 'aggregate',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $MetroclusterCheckConfigReplicationShowAggregateEligibility_state  = NACL::CS::MetroclusterCheckConfigReplicationShowAggregateEligibility->fetch(command_interface => $ci, ...);
	my @MetroclusterCheckConfigReplicationShowAggregateEligibility_states = NACL::CS::MetroclusterCheckConfigReplicationShowAggregateEligibility->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes metrocluster-check-aggregate-eligibility-get-iter API for CMode ZAPI.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'metrocluster check config-replication show-aggregate-eligibility',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        exception_text =>
            'No matching metrocluster check config-replication(s) found',
        frontend =>
            'NACL::CS::MetroclusterCheckConfigReplicationShowAggregateEligibility::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api =>
            'metrocluster_check_config_replication_show_aggregate_eligibility',
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(is-eligible-to-host-additional-volumes aggregate comment
                )
        ],
        map => {
            'hosted-configuration-replication-volumes' => [
                make_zapi_array('hosted-configuration-replication-volumes'),
                make_zapi_skip('volume-name'),
            ],
        },
        api => "metrocluster-check-aggregate-eligibility-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
