#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MetroclusterInterconnect ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::MetroclusterInterconnect

=head1 DESCRIPTION

C<NACL::CS::MetroclusterInterconnect> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP MetroclusterInterconnect.  A related class is
L<NACL::C::MetroclusterInterconnect|lib-NACL-C-MetroclusterInterconnect-pm>, which represents access
to an ONTAP MetroclusterInterconnect.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the MetroclusterInterconnect
element are the attributes of the MetroclusterInterconnect ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< dr_systemid >>

DR System ID

Filled in for CMode CLI

=item C<< ha_partnername >>

HA Partner Name

Filled in for CMode CLI

=item C<< aux_systemid >>

Auxiliary System ID

Filled in for CMode CLI

=item C<< node_systemid >>

Local System ID

Filled in for CMode CLI

=item C<< status >>

Status

Filled in for CMode CLI/ZAPI

Maps to:

CMode ZAPI:

<metrocluster-interconnect>
 <link-status>$value</link-status>
</metrocluster-interconnect>

=item C<< partner_type >>

Partner Type

Filled in for CMode CLI/ZAPI

=item C<< partner_sysid >>

Partner System ID

Filled in for CMode CLI

=item C<< partner_name >>

Node name

Filled in for CMode CLI/ZAPI

=item C<< node >>

Node name possible value(s) are, <nodename>,local

Filled in for CMode CLI/ZAPI

Maps to:

CMode ZAPI:

<metrocluster-interconnect>
 <node-name>$value</node-name>
</metrocluster-interconnect>

=item C<< dr_partnername >>

DR Partner Name

Filled in for CMode CLI

=item C<< mirror_status >>

Mirror Status

Filled in for CMode CLI/ZAPI

Maps to:

CMode ZAPI:

<metrocluster-interconnect>
 <mirroring-status>$value</mirroring-status>
</metrocluster-interconnect>

=item C<< ha_systemid >>

HA System ID

Filled in for CMode CLI

=item C<< aux_partnername >>

Aux DR Partner Name

Filled in for CMode CLI/ZAPI

=item C<< adapter >>

Adapter

Filled in for CMode CLI/ZAPI

Maps to:

CMode ZAPI:

<metrocluster-interconnect>
 <interconnect-adapter>$value</interconnect-adapter>
</metrocluster-interconnect>

=item C<< type >>

Type

Filled in for CMode CLI/ZAPI

Maps to:

CMode ZAPI:

<metrocluster-interconnect>
 <interconnect-type>$value</interconnect-type>
</metrocluster-interconnect>

=item C<< next_partner >>

Next Partner

Filled in for CMode CLI.


=item C<< adapter_index >>

Adapter Index

Filled in for CMode CLI.

=item C<< index >>

Partner SysId Index

Filled in for CMode CLI.


=back

=cut

package NACL::CS::MetroclusterInterconnect;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::CS::MetroclusterInterconnectMirror;

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'dr_systemid',
    scalar => 'ha_partnername',
    scalar => 'aux_systemid',
    scalar => 'node_systemid',
    scalar => 'status',
    scalar => 'partner_type',
    scalar => 'partner_sysid',
    scalar => 'partner_name',
    scalar => 'node',
    scalar => 'dr_partnername',
    scalar => 'mirror_status',
    scalar => 'ha_systemid',
    scalar => 'aux_partnername',
    scalar => 'adapter',
    scalar => 'type',
    scalar => 'next_partner',
    scalar => 'adapter_index',
    scalar => 'index',
];

=head1 METHODS

=head2 fetch

    my $MetroclusterInterconnect_state  = NACL::CS::MetroclusterInterconnect->fetch(command_interface => $ci, ...);
    my @MetroclusterInterconnect_states = NACL::CS::MetroclusterInterconnect->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI

Invokes "metrocluster-interconnect-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args, $pkg->_fetch_validate_spec();

    if ($opts{command_interface}->has_uichange(uichange => "mcc-interconnect-commands-review",
                                               throw_exception => 0)) {

        $Log->warn("NACL::CS::MetroclusterInterconnect is replaced by"
                   ." NACL::CS::MetroclusterInterconnectMirror.\n"
                   ."Note also that the 'mirror-status' option was renamed to 'mirror-oper-status'\n"
                   ." Please update the script"
        );

       my @req_fields = @{$opts{'requested_fields'}};

       foreach (@req_fields) {
           s/^mirror-status$/mirror-oper-status/;
       }

       my %filter = %{$opts{'filter'}};
       if (defined $filter{'mirror-status'}) {
            $filter{'mirror-oper-status'} =
                    delete $filter{'mirror-status'};
       }

       $opts{'requested_fields'} = \@req_fields;
       $opts{'filter'}           = \%filter;

       $Log->exit() if $may_exit;
       return NACL::CS::MetroclusterInterconnectMirror->fetch(%opts);
    }

    my @state_objs = $pkg->SUPER::fetch(
        %opts,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching metrocluster interconnect(s) found',
        frontend       => 'NACL::CS::MetroclusterInterconnect::fetch',
        show_cmd       => 'metrocluster interconnect show',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'metrocluster_interconnect_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        api  => 'metrocluster-interconnect-get-iter',
        copy => [qw (partner-type partner-name)],
        map  => {
            node            => 'node-name',
            adapter         => 'interconnect-adapter',
            type            => 'interconnect-type',
            status          => 'link-status',
            'mirror-status' => 'mirroring-status',
        }
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
