# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MetroclusterOperation ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::MetroclusterOperation

=head1 DESCRIPTION

C<NACL::CS::MetroclusterOperation> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP MetroclusterOperation.  A related class is
L<NACL::C::MetroclusterOperation|lib-NACL-C-MetroclusterOperation-pm>, which represents access
to an ONTAP MetroclusterOperation.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the MetroclusterOperation
element are the attributes of the MetroclusterOperation ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< additional >>

Filled in for CMode CLI.

=item C<< dr_state >>

Filled in for CMode CLI.

=item C<< cluster_name >>

Cluster where the command was issued

Filled in for CMode CLI.

=item C<< start_time >>

Start time

Filled in for CMode CLI.

=item C<< node_name >>

Node where the command was run possible value(s) are, <nodename>,local

Filled in for CMode CLI.

=item C<< end_time >>

End time

Filled in for CMode CLI.

=item C<< operation >>

Name of the operation

Filled in for CMode CLI.

=item C<< error_list >>

Error list for the operation
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $error_list = $obj->error_list();
 
# $error_list contains a reference to the array of values
 
my @error_list = $obj->error_list();
 
# @error_list contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { error_list = [ value1, value2...] }

Filled in for CMode CLI.

=item C<< state >>

State of the operation

Filled in for CMode CLI.


=back

=cut

package NACL::CS::MetroclusterOperation;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'additional',
    scalar => 'dr_state',
    scalar => 'cluster_name',
    scalar => 'start_time',
    scalar => 'node_name',
    scalar => 'end_time',
    scalar => 'operation',
    array  => 'error_list',
    scalar => 'state',
];

=head1 METHODS

=head2 fetch

    my $MetroclusterOperation_state  = NACL::CS::MetroclusterOperation->fetch(command_interface => $ci, ...);
(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. This being a "singleton" table (i.e. one which can only have one
row) is callable only in scalar context.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI APISet.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'metrocluster operation show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching metrocluster operation(s) found',
        is_singleton   => 1
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(
        @_,
        api          => 'metrocluster_operation_show',
        is_singleton => 1,

    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

1;
