# $Id $

#
# Copyright (c) 2001-2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkInterfaceFailover ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::NetworkInterfaceFailover

=head1 DESCRIPTION

C<NACL::CS::NetworkInterfaceFailover> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP NetworkInterfaceFailover.  A related class is
L<NACL::C::NetworkInterfaceFailover|lib-NACL-C-NetworkInterfaceFailover-pm>, which represents access
to an ONTAP NetworkInterfaceFailover.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the NetworkInterfaceFailover
element are the attributes of the NetworkInterfaceFailover ComponentState.

=over

=item C<< "lif" >>

Filled in for CMode CLI.

=item C<< "failover-group-usage" >>

Filled in for CMode CLI.

=item C<< "failover-targets" >>

Filled in for CMode CLI.

=item C<< "vserver" >>

Filled in for CMode CLI.

=item C<< "home-node" >>

Filled in for CMode CLI.

=item C<< "home-port" >>

Filled in for CMode CLI.

=item C<< "failover-group" >>

Filled in for CMode CLI.

=item C<< "logical-interface" >>

Filled in for CMode CLI.

=item C<< "node" >>

Filled in for CMode CLI.

=item C<< "port" >>

Filled in for CMode CLI.

=back

=cut

package NACL::CS::NetworkInterfaceFailover;

use strict;
use warnings;
use Params::Validate qw(validate_with validate :types);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::Exceptions::NoElementsFound qw(:try);
use base 'NACL::CS::ComponentState::ONTAP';
use base 'NACL::C::Component';

use Class::MethodMaker [
    scalar => 'lif',
    scalar => 'failover_group_usage',
    array  => 'failover_targets',
    scalar => 'vserver',
    scalar => 'home_node',
    scalar => 'home_port', 
    scalar => 'failover_group',
    scalar => 'logical_interface',
    scalar => 'node',
    scalar => 'port',
];

=head1 METHODS

=head2 fetch

    my $NetworkInterfaceFailover_state   = NACL::CS::NetworkInterfaceFailover->fetch(command_interface => $ci, ...);
    my @NetworkInterfaceFailover_states  = NACL::CS::NetworkInterfaceFailover->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

For CMode CLI, 'lif' have to be provided in the filter. 
options are mandatory for the underlying CMode CLI call)

Supports CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut


sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
	show_cmd => 'network interface show',
        exception_text => 'No matching found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

# Burt845991, 'network interface failover show' command is
# being removed in FS.0 in favor of 'network interface show -failover' command.
# As part of this implementation, NetworkInterfaceFailover CS should now invoke
# 'network interface show -failover' command instead of 'network interface failover
# show'.

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'network_interface_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli


sub _compute_cmode_cli_options {
    $Log->enter() if $may_enter;
    my ($pkg,%opts) = @_;

    # 'network interface show -failover' doesn't have a '-fields' option. Hence,
    # requested_fields must be passed as an empty array ref.
    $opts{'requested_fields'} = [];
    my $cli_options = $pkg->SUPER::_compute_cmode_cli_options(%opts);
    $cli_options->{failover} = '';
    $Log->exit() if $may_exit;
    return $cli_options;
}

1;
