# $Id:  //depot/prod/test/nacldev/lib/NACL/CS/NetworkInterfacePlacement.pm#2 $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkInterfacePlacement ComponentState Module (auto-generated by CGT)
## @author pbala@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::NetworkInterfacePlacement

=head1 DESCRIPTION

C<NACL::CS::NetworkInterfacePlacement> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP NetworkInterfacePlacement.  A related class is
L<NACL::C::NetworkInterfacePlacement|lib-NACL-C-NetworkInterfacePlacement-pm>, which represents access
to an ONTAP NetworkInterfacePlacement.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the NetworkInterfacePlacement
element are the attributes of the NetworkInterfacePlacement ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< in_progress >>

Filled in for CMode CLI.

=item C<< vserver >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <net-placement-cache>
     <vserver>$value</vserver>
 </net-placement-cache>

=item C<< netmask_length >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <net-placement-cache>
     <netmask-length>$value</netmask-length>
 </net-placement-cache>

=item C<< address >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <net-placement-cache>
     <address>$value</address>
 </net-placement-cache>

=item C<< port >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <net-placement-cache>
     <port>$value</port>
 </net-placement-cache>

=item C<< found >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <net-placement-cache>
     <found>$value</found>
 </net-placement-cache>

=item C<< node >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <net-placement-cache>
     <node>$value</node>
 </net-placement-cache>

=back

=cut

package NACL::CS::NetworkInterfacePlacement;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'in_progress',
    scalar => 'vserver',
    scalar => 'netmask_length',
    scalar => 'address',
    scalar => 'port',
    scalar => 'found',
    scalar => 'node',
];

=head1 METHODS

=head2 fetch

    my $NetworkInterfacePlacement_state  = NACL::CS::NetworkInterfacePlacement->fetch(command_interface => $ci, ...);
	my @NetworkInterfacePlacement_states = NACL::CS::NetworkInterfacePlacement->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "net-placement-cache-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching network interface placement(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'network_interface_placement_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => 'net_placement_cache_get_iter',
        copy => [qw ( vserver netmask-length address port found node )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
