# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::NetworkInterface->show_routing_group() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::NetworkInterfaceShowRoutingGroup

=head1 DESCRIPTION

C<NACL::CS::NetworkInterfaceShowRoutingGroup> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::NetworkInterface->show_routing_group() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::NetworkInterface->show_routing_group() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< vserver >>

Filled in for CMode CLI/ZAPI.

=item C<< vif >>

(B<DEPRECATED>: use C<lif> instead)

Filled in for CMode CLI/ZAPI.

=item C<< server >>

(B<DEPRECATED>: use C<vserver> instead)

Filled in for CMode CLI/ZAPI.

=item C<< lif >>

Filled in for CMode CLI/ZAPI.

=item C<< routing_group >>

Filled in for CMode CLI/ZAPI.
	
=back

=cut

package NACL::CS::NetworkInterfaceShowRoutingGroup;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::CS::NetworkInterface;
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'vif',
    scalar => 'server',
    scalar => 'lif',
    scalar => 'routing_group',
];

=head1 METHODS

=head2 fetch

    my $NetworkInterfaceShowRoutingGroup_state  = NACL::CS::NetworkInterfaceShowRoutingGroup->fetch(command_interface => $ci, ...);
	my @NetworkInterfaceShowRoutingGroup_states = NACL::CS::NetworkInterfaceShowRoutingGroup->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI or CMode ZAPI APISet.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching network interface(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'network_interface_show_routing_group',);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    
    # Delete these from %opts since they are not applicable for calls to fetch()
    my $apiset       = delete $opts{apiset};
    my $apply_filter = delete $opts{_apply_filter};
    
    my @fields = $pkg->get_all_real_field_names();
    
    # All details are available in NACL::CS::NetworkInterface, so reuse
    my @lifs = NACL::CS::NetworkInterface->fetch(
        %opts,

        # Even if no requested_fields are provided, the only ones we need to
        # request from NetworkInterface->fetch() are the ones applicable to
        # this CS package.
        requested_fields => \@fields,
        
        # Prevent the call to fetch from failing.
        allow_empty => 1,
        
        # Ensure it goes through ZAPI
        apiset_should => {interface => 'ZAPI'},
    );
    
    # Extract details from NACL::CS::NetworkInterface objects and store in
    # NACL::CS::NetworkInterfaceShowRoutingGroup objects.
    my @state_objs;
    foreach my $lif (@lifs) {
        my $obj = $pkg->new(command_interface => $opts{command_interface});
        foreach my $field (@fields) {
            my $field_isset = "${field}_isset";
            $obj->$field($lif->$field()) if ($lif->$field_isset());
        }
        push @state_objs, $obj;
    }

    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
