# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkIpspace ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::NetworkIpspace

=head1 DESCRIPTION

C<NACL::CS::NetworkIpspace> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP NetworkIpspace.  A related class is
L<NACL::C::NetworkIpspace|lib-NACL-C-NetworkIpspace-pm>, which represents access
to an ONTAP NetworkIpspace.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the NetworkIpspace
element are the attributes of the NetworkIpspace ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< vservers >>

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $vservers = $obj->vservers();
# $vservers contains a reference to the array of values
my @vservers = $obj->vservers();
# @vservers contains the array of values

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <net-ipspaces-info>
     <vservers>
         <vserver-name>@values</string>
     </vservers>
</net-ipspaces-info>

=item C<< ports >>

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $ports = $obj->ports();
# $ports contains a reference to the array of values
my @ports = $obj->ports();
# @ports contains the array of values

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <net-ipspaces-info>
     <ports>
         <net-qualified-port-name>@values</string>
     </ports>
</net-ipspaces-info>

=item C<< is_private >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <net-ipspaces-info>
     <is_private>$value</is_private>
 </net-ipspaces-info>

=item C<< ipspace >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <net-ipspaces-info>
     <ipspace>$value</ipspace>
 </net-ipspaces-info>

=item C<< is_hidden >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <net-ipspaces-info>
     <is-hidden>$value</is-hidden>
 </net-ipspaces-info>

=item C<< id >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <net-ipspaces-info>
     <id>$value</id>
 </net-ipspaces-info>

=item C<< uuid >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <net-ipspaces-info>
     <uuid>$value</uuid>
 </net-ipspaces-info>

=item C<< broadcast_domains >>

Broadcast Domains
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $broadcast_domains = $obj->broadcast_domains();
 
# $broadcast_domains contains a reference to the array of values
 
my @broadcast_domains = $obj->broadcast_domains();
 
# @broadcast_domains contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { broadcast_domains = [ value1, value2...] }

Filled in for CMode CLI/ZAPI.


=back

=cut

package NACL::CS::NetworkIpspace;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    array  => 'vservers',
    array  => 'ports',
    scalar => 'is_private',
    scalar => 'ipspace',
    scalar => 'is_hidden',
    scalar => 'id',
    scalar => 'uuid',
    array  => 'broadcast_domains',
];

=head1 METHODS

=head2 fetch

    my $NetworkIpspace_state  = NACL::CS::NetworkIpspace->fetch(command_interface => $ci, ...);
    my @NetworkIpspace_states = NACL::CS::NetworkIpspace->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "net-ipspaces-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        exception_text => 'No matching network ipspace(s) found',
        show_cmd       => 'network ipspace show',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@_, api => 'network_ipspace_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        api  => 'net_ipspaces_get_iter',
        copy => [
            qw(ipspace is-hidden is-private uuid id)
        ],
        map => {
            vservers => [
                make_zapi_array('vservers'),
                make_zapi_skip('vserver-name'),
            ],
            ports    => [
                make_zapi_array('ports'),
                make_zapi_skip('net-qualified-port-name'),
            ],
            'broadcast-domains' => [
                make_zapi_array('broadcast-domains'),
                make_zapi_skip('broadcast-domain-name'),
            ],
        },
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

=head2 get_ports_without_nodename
     my @ports = $obj->get_ports_without_nodename();

Determines the ports from the fetch object and returns the ports
without nodename.

=cut

sub get_ports_without_nodename {
    $Log->enter() if $may_enter;
    my $self = $_[0];
    my @ports = $self->ports();
    my @ports_without_nodename = map { $_ =~ s/^.+://g ; $_ ;} @ports;
    $Log->exit() if $may_exit;
    return @ports_without_nodename;
}

1;
