# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkNdpNeighbor ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::NetworkNdpNeighbor

=head1 DESCRIPTION

C<NACL::CS::NetworkNdpNeighbor> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP NetworkNdpNeighbor.  A related class is
L<NACL::C::NetworkNdpNeighbor|lib-NACL-C-NetworkNdpNeighbor-pm>, which represents access
to an ONTAP NetworkNdpNeighbor.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the NetworkNdpNeighbor
element are the attributes of the NetworkNdpNeighbor ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< neighbor >>

Neighbor Address

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<ndp-neighbor-info neighbor-address="$value" />



=item C<< mac_address >>

MAC Address

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<ndp-neighbor-info mac-address="$value" />



=item C<< vserver >>

Vserver Name

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<ndp-neighbor-info vserver="$value" />



=back

=cut

package NACL::CS::NetworkNdpNeighbor;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'neighbor',
    scalar => 'mac_address',
    scalar => 'vserver',
];

=head1 METHODS

=head2 fetch

    my $NetworkNdpNeighbor_state  = NACL::CS::NetworkNdpNeighbor->fetch(command_interface => $ci, ...);
	my @NetworkNdpNeighbor_states = NACL::CS::NetworkNdpNeighbor->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes net-ndp-neighbor-get-iter API for CMode ZAPI.


=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        show_cmd => 'network ndp neighbor show',
	exception_text => 'No matching network ndp neighbor(s) found',
        frontend       => 'NACL::CS::NetworkNdpNeighbor::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'network_ndp_neighbor_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(mac-address vserver
                )
        ],
        map => {'neighbor' => 'neighbor-address',},
        api => "net-ndp-neighbor-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
