# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkRouteActiveEntry ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::NetworkRouteActiveEntry

=head1 DESCRIPTION

C<NACL::CS::NetworkRouteActiveEntry> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP NetworkRouteActiveEntry.  A related class is
L<NACL::C::NetworkRouteActiveEntry|lib-NACL-C-NetworkRouteActiveEntry-pm>, which represents access
to an ONTAP NetworkRouteActiveEntry.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the NetworkRouteActiveEntry
element are the attributes of the NetworkRouteActiveEntry ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< subnet_group >>

Subnet Group possible value(s) are, <subnet>,default

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<net-active-routes-info subnet-group="$value" />



=item C<< flags >>

Flags

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<net-active-routes-info flags="$value" />



=item C<< reference_count >>

Reference Count

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<net-active-routes-info reference-count="$value" />



=item C<< vserver >>

Vserver Name

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<net-active-routes-info vserver="$value" />



=item C<< destination >>

Destination

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<net-active-routes-info destination="$value" />



=item C<< gateway >>

Gateway

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<net-active-routes-info gateway="$value" />



=item C<< metric >>

Metric

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<net-active-routes-info metric="$value" />



=item C<< lookup_count >>

Lookup Count

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<net-active-routes-info lookup-count="$value" />



=item C<< node >>

Node possible value(s) are, <nodename>,local

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<net-active-routes-info node-name="$value" />



=item C<< path_mtu >>

Path MTU

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<net-active-routes-info path-mtu="$value" />



=item C<< interface >>

Interface Name

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<net-active-routes-info interface="$value" />



=item C<< route_interface_address >>

Route Interface Address

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<net-active-routes-info route-interface-address="$value" />



=item C<< address_type >>

Address Family

Filled in for CMode CLI.


=back

=cut

package NACL::CS::NetworkRouteActiveEntry;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'subnet_group',
    scalar => 'flags',
    scalar => 'reference_count',
    scalar => 'vserver',
    scalar => 'destination',
    scalar => 'gateway',
    scalar => 'metric',
    scalar => 'lookup_count',
    scalar => 'node',
    scalar => 'path_mtu',
    scalar => 'interface',
    scalar => 'route_interface_address',
    scalar => 'address_type',
];

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $NetworkRouteActiveEntry_state  = NACL::CS::NetworkRouteActiveEntry->fetch(command_interface => $ci, ...);
	my @NetworkRouteActiveEntry_states = NACL::CS::NetworkRouteActiveEntry->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes net-active-routes-get-iter API for CMode ZAPI.


=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        show_cmd => 'network route active-entry show',
	exception_text => 'No matching network route active-entry(s) found',
        frontend       => 'NACL::CS::NetworkRouteActiveEntry::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'network_route_active_entry_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(reference-count flags subnet-group gateway destination vserver lookup-count
                metric path-mtu route-interface-address interface
                )
        ],
        map => {'node' => 'node-name',},
        api => "net-active-routes-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
