# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkRouteCdb ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::NetworkRouteCdb

=head1 DESCRIPTION

C<NACL::CS::NetworkRouteCdb> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP NetworkRouteCdb.  A related class is
L<NACL::C::NetworkRouteCdb|lib-NACL-C-NetworkRouteCdb-pm>, which represents access
to an ONTAP NetworkRouteCdb.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the NetworkRouteCdb
element are the attributes of the NetworkRouteCdb ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< vsid >>

Filled in for CMode CLI.

=item C<< destination >>

Filled in for CMode CLI.

=item C<< vserver >>

Filled in for CMode CLI.

=item C<< gateway >>

Filled in for CMode CLI.

=item C<< node >>

Filled in for CMode CLI.

=item C<< metric >>

Filled in for CMode CLI.

=item C<< address_family >>

Address Family possible value(s) are, ipv4,ipv6,ipv6z

Filled in for CMode CLI.

=item C<< ipspaceid >>

IPspace ID

Filled in for CMode CLI.


=back

=cut

package NACL::CS::NetworkRouteCdb;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'vsid',
    scalar => 'destination',
    scalar => 'vserver',
    scalar => 'gateway',
    scalar => 'metric',
    scalar => 'node',
    scalar => 'address_family',
    scalar => 'ipspaceid',
];

=head1 METHODS

=head2 fetch

    my $NetworkRouteCdb_state  = NACL::CS::NetworkRouteCdb->fetch(command_interface => $ci, ...);
	my @NetworkRouteCdb_states = NACL::CS::NetworkRouteCdb->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI APISet.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'network route cdb show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching network route cdb(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@_, api => 'network_route_cdb_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
