# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::NetworkRoute->show_lifs() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::NetworkRouteShowLifs

=head1 DESCRIPTION

C<NACL::CS::NetworkRouteShowLifs> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::NetworkRoute->show_lifs() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::NetworkRoute->show_lifs() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over


=item C<< vserver >>

Vserver Name
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <net-vs-routes-lif-info vserver="$value" />


=item C<< destination >>

Destination/Mask
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <net-vs-routes-lif-info destination="$value" />


=item C<< gateway >>

Gateway
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <net-vs-routes-lif-info gateway="$value" />


=item C<< address_family >>

Address Family possible value(s) are, ipv4,ipv6,ipv6z
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <net-vs-routes-lif-info address-family="$value" />


=item C<< lifs >>

Logical Interfaces
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $lifs = $obj->lifs();
 
# $lifs contains a reference to the array of values
 
my @lifs = $obj->lifs();
 
# @lifs contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { lifs = [ value1, value2...] }

Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <net-vs-routes-lif-info>
  <lif-names lif-name="@values" />
</net-vs-routes-lif-info>

Mapping for CMode ZAPI non-iter: <lif-names lif-name="@values" />

=back

=cut

package NACL::CS::NetworkRouteShowLifs;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'destination',
    scalar => 'gateway',
    scalar => 'address_family',
    array  => 'lifs',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [qw(vserver destination gateway)],
        _vs_context   => 1
    );
    $Log->exit() if $may_exit;
}

=head1 METHODS

=head2 fetch

    my $NetworkRouteShowLifs_state  = NACL::CS::NetworkRouteShowLifs->fetch(command_interface => $ci, ...);
	my @NetworkRouteShowLifs_states = NACL::CS::NetworkRouteShowLifs->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes net-routes-lifs-get-iter API for CMode ZAPI iter.

Invokes net-routes-lifs-get API for CMode ZAPI non-iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_check_non_iter'
            },
        ],

        show_cmd => 'network route show-lifs',
	exception_text => 'No matching network route(s) found',
        frontend       => 'NACL::CS::NetworkRouteShowLifs::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'network_route_show_lifs',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(gateway destination vserver address-family
                )
        ],
        map => {
            'lifs' =>
                [make_zapi_array('lif-names'), make_zapi_skip('lif-name'),],
        },
        api => "net-routes-lifs-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        copy => [
            qw(gateway destination vserver address-family
                )
        ],
        map => {
            'lifs' =>
                [make_zapi_array('lif-names'), make_zapi_skip('lif-name'),],
        },
        api                 => "net-routes-lifs-get",
        vserver_operated_on => \my $vserver,
        primary_keys        => [qw( gateway destination )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
