# $Id: //depot/prod/test/nacldev/lib/NACL/CS/NetworkRoutingGroups.pm#20 $
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkRoutingGroups ComponentState Module
## @author Sumit.Mittal@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::NetworkRoutingGroups

=head1 DESCRIPTION

C<NACL::CS::NetworkRoutingGroups> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>.  It represents the
state  of a Routing Group.  A related class is
L<NACL::C::NetworkRoutingGroups|lib-NACL-C-NetworkRoutingGroups-pm>, which represents
access to ONTAP Routing Groups.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the
element are the attributes of the  ComponentState.

=over

=item C<< vserver >>

Filled in for CMode CLI.

=item C<< server >>

Filled in for CMode CLI.

=item C<< routing_group >>

Filled in for CMode CLI.

=item C<< subnet >>

Filled in for CMode CLI.

=item C<< routing_group >>

Filled in for CMode CLI.

=item C<< role >>

Filled in for CMode CLI.

=item C<< metric >>

Filled in for CMode CLI.

=item C<< address_family >>

Filled in for CMode CLI.

=back

=cut

package NACL::CS::NetworkRoutingGroups;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [

    # These fields are from the C Mode "network routing-groups show"
    scalar => 'vserver',
    scalar => 'server',
    scalar => 'routing_group',
    scalar => 'subnet',
    scalar => 'role',
    scalar => 'metric',
    scalar => 'address_family',
];

sub isa {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, $kind) = @_;
    my $isa = $pkg_or_obj->_build_isa(
        kind  => $kind,
        alias => 'NACL::CS::RoutingGroups'
    );
    $Log->exit() if $may_exit;
    return $isa;
}

=head1 METHODS

=head2 fetch

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

Supports CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'network routing-groups show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching network routing groups(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'network_routing_groups_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

1;
