# Copyright (c) 2015 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
#
## @summary Persistent Fence Component Module
## @author rishis@netapp.com,ng-wafl-replica-qa@netapp.com
## @status shared

=head1 NAME

NACL::CS::PersistentFenceGet

=head1 DESCRIPTION

C<NACL::CS::PersistentFenceGet> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>.
It represents the state of volume level persistent fence. A related class is
L<NACL::C::PersistentFenceGet|lib-NACL-C-PersistentFence-pm>, which represents
access to nodescope 'persistent_fence' set of commands.

=head1 ATTRIBUTES

=over

=item C<< owner >>

Owner of persistent fence.

Filled in for 7Mode Nodescope.

=item C<< caller_id >>

Caller id of persistent fence

Filled in for 7Mode Nodescope.

=item C<< timemout >>

Timeout for particular caller of persistent fence.

Filled in for 7Mode Nodescope.

=cut

package NACL::CS::PersistentFenceGet;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base 'NACL::CS::ComponentState::ONTAP';
use Params::Validate qw(validate);
use Data::Dumper;
use NATE::Log qw(log_global);

use Class::MethodMaker [
    scalar => 'owner',
    scalar => 'caller_id',
    scalar => 'volume'
];

=head1 METHODS

=head2 fetch

    my $state = NACL::CS::Persistent_Fence->fetch(command_interface => $ci,);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Supports 7Mode CLI

=over

=cut

__PACKAGE__->_fetch_7mode_cli_method_builder(
    apiset_method                      => 'persistent_fence_get',
    filter_keys_to_be_passed_to_apiset => [qw/volume/],
);

1;
