# $Id$

#
# Copyright (c) 2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Reallocate ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, kathar.hidayath@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::Reallocate

=head1 DESCRIPTION

C<NACL::CS::Reallocate> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP Reallocate.  A related class is
L<NACL::C::Reallocate|lib-NACL-C-Reallocate-pm>, which represents access
to an ONTAP Reallocate.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the Reallocate
element are the attributes of the Reallocate ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< once >>

Filled in for CMode CLI.

=item C<< no_check >>

Filled in for CMode CLI.

=item C<< threshold >>

Filled in for CMode CLI.

=item C<< vserver >>

Filled in for CMode CLI.

=item C<< progress >>

Filled in for CMode CLI.

=item C<< path >>

Filled in for CMode CLI.

=item C<< description >>

Filled in for CMode CLI.

=item C<< interval >>

Filled in for CMode CLI.

=item C<< state >>

Filled in for CMode CLI.

=item C<< logpath >>

Filled in for CMode CLI.

=item C<< force >>

Filled in for CMode CLI.

=item C<< schedule >>

Filled in for CMode CLI.

=item C<< space_optimized >>

Filled in for CMode CLI.

=item C<< global_status >>

Filled in for CMode CLI.

=item C<< cron >>

Filled in for CMode CLI.

=item C<< measure >>

Filled in for CMode CLI.

=item C<< id >>

Filled in for CMode CLI.

=back

=cut

package NACL::CS::Reallocate;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'once',
    scalar => 'no_check',
    scalar => 'threshold',
    scalar => 'vserver',
    scalar => 'progress',
    scalar => 'path',
    scalar => 'description',
    scalar => 'interval',
    scalar => 'state',
    scalar => 'logpath',
    scalar => 'force',
    scalar => 'schedule',
    scalar => 'space_optimized',
    scalar => 'global_status',
    scalar => 'cron',
    scalar => 'measure',
    scalar => 'id',
];

=head1 METHODS

=head2 fetch

    my $Reallocate_state  = NACL::CS::Reallocate->fetch(command_interface => $ci, ...);
    my @Reallocate_states = NACL::CS::Reallocate->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode or 7Mode CLI APISet.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'reallocate show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
        exception_text => 'No matching reallocate(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@_, api => 'reallocate_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();
    my @state_objs;
    my $apiset = delete $opts{apiset};
    my %cmd_opts;
    my %copy_filter           = %{$opts{filter}};
    my @copy_requested_fields = @{$opts{requested_fields}};
    my $deleted_filter        = $pkg->_remove_relational_regex_filters(
        filter           => \%copy_filter,
        requested_fields => \@copy_requested_fields
    );
    my $filter           = \%copy_filter;
    my $requested_fields = \@copy_requested_fields;
    $cmd_opts{'path'} = $filter->{'path'};
    my $response = $apiset->reallocate_status(%cmd_opts);
    my $output   = $response->get_parsed_output();

    foreach my $row (@$output) {
        my $obj  = $pkg->new();
        my $temp = undef;
        while (my ($key, $value) = each %$row) {
            if ($key =~ /^(schedule|state)$/) {
                $obj->$key($value);
            } elsif ($key =~ /^path$/) {
                $temp .= "$value,";
            } elsif ($key =~ /^(threshold|interval|optimization)$/) {
                $temp .= "$key=$value,";
            }
        }
        chop($temp);
        $obj->description($temp);
        $obj->command_interface($opts{'command_interface'});
        push @state_objs, $obj;
    }
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
