#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Repository ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::Repository

=head1 DESCRIPTION

C<NACL::CS::Repository> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP Repository.  A related class is
L<NACL::C::Repository|lib-NACL-C-Repository-pm>, which represents access
to an ONTAP Repository.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the Repository
element are the attributes of the Repository ComponentState.

=over

=item C<< percent_used >>

Filled in for CMode CLI.

=item C<< status >>

Filled in for CMode CLI.

=item C<< junction_path >>

Filled in for CMode CLI.

=item C<< perform_objcons_check >>

Filled in for CMode CLI.

=item C<< available_size >>

Filled in for CMode CLI.

=item C<< size >>

Filled in for CMode CLI.

=item C<< namespace_aggregate >>

Filled in for CMode CLI.

=item C<< sis_space_saved_percent >>

Filled in for CMode CLI.

=item C<< group >>

Filled in for CMode CLI.

=item C<< policy >>

Filled in for CMode CLI.

=item C<< user >>

Filled in for CMode CLI.

=item C<< dedup_space_saved >>

Filled in for CMode CLI.

=item C<< default_object_volume_group >>

Filled in for CMode CLI.

=item C<< sys_aggr_list >>

(Array)

Filled in for CMode CLI.

=item C<< compression_space_saved_percent >>

Filled in for CMode CLI.

=item C<< volume >>

Filled in for CMode CLI.

=item C<< vserver >>

Filled in for CMode CLI.

=item C<< total >>

Filled in for CMode CLI.

=item C<< compression_space_saved >>

Filled in for CMode CLI.

=item C<< sis_space_saved >>

Filled in for CMode CLI.

=item C<< snapshot_policy >>

Filled in for CMode CLI.

=item C<< unix_permissions >>

Filled in for CMode CLI.

=item C<< type >>

Filled in for CMode CLI.

=item C<< foreground >>

Filled in for CMode CLI.

=item C<< nvfail >>

Filled in for CMode CLI.

=item C<< dedup_space_saved_percent >>

Filled in for CMode CLI.

=back

=cut

package NACL::CS::Repository;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'percent_used',
    scalar => 'status',
    scalar => 'junction_path',
    scalar => 'perform_objcons_check',
    scalar => 'available_size',
    scalar => 'size',
    scalar => 'namespace_aggregate',
    scalar => 'sis_space_saved_percent',
    scalar => 'group',
    scalar => 'policy',
    scalar => 'user',
    scalar => 'dedup_space_saved',
    scalar => 'default_object_volume_group',
    array  => 'sys_aggr_list',
    scalar => 'compression_space_saved_percent',
    scalar => 'volume',
    scalar => 'vserver',
    scalar => 'total',
    scalar => 'compression_space_saved',
    scalar => 'sis_space_saved',
    scalar => 'snapshot_policy',
    scalar => 'unix_permissions',
    scalar => 'type',
    scalar => 'foreground',
    scalar => 'nvfail',
    scalar => 'dedup_space_saved_percent',
];

=head1 METHODS

=head2 fetch

    my $Repository_state   = NACL::CS::Repository->fetch(command_interface => $ci, ...);
	my @Repository_states = NACL::CS::Repository->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut



sub fetch {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching repository(s) found',
        show_cmd       => 'repository show',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli( @_, api => 'repository_show' );
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
