#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SecurityAudit ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status public
## @pod here

=head1 NAME

NACL::CS::SecurityAudit

=head1 DESCRIPTION

C<NACL::CS::SecurityAudit> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SecurityAudit.  A related class is
L<NACL::C::SecurityAudit|lib-NACL-C-SecurityAudit-pm>, which represents access
to an ONTAP SecurityAudit.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the SecurityAudit
element are the attributes of the SecurityAudit ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item C<< "httpset" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <audit-info>
    <http-set>$value</http-set>
 </audit-info>

=item C<< "httpget" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <audit-info>
    <http-get>$value</http-get>
 </audit-info>

=item C<< "isFromCluster" >>

Filled in for CMode CLI.

=item C<< "snmpset" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <audit-info>
    <snmp-set>$value</snmp-set>
 </audit-info>

=item C<< "cliset" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <audit-info>
    <cli-set>$value</cli-set>
 </audit-info>

=item C<< "snmpget" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <audit-info>
    <snmp-get>$value</snmp-get>
 </audit-info>

=item C<< "cliget" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <audit-info>
    <cli-get>$value</cli-get>
 </audit-info>

=item C<< "oldsnmpget" >>

Filled in for CMode CLI.

=item C<< ontapiset >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <audit-info>
    <ontapi-set>$value</ontapi-set>
 </audit-info>

=item C<< ontapiget >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <audit-info>
    <ontapi-get>$value</ontapi-get>
 </audit-info>

=back

=cut

package NACL::CS::SecurityAudit;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::_Mixins::SecurityAudit qw(:all);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'httpset',
    scalar => 'httpget',
    scalar => 'isFromCluster',
    scalar => 'snmpset',
    scalar => 'cliset',
    scalar => 'snmpget',
    scalar => 'cliget',
    scalar => 'oldsnmpget',
    scalar => 'ontapiset',
    scalar => 'ontapiget',
];

=head1 METHODS

=head2 fetch

    my $SecurityAudit_state   = NACL::CS::SecurityAudit->fetch(command_interface => $ci, ...);
	my @SecurityAudit_states = NACL::CS::SecurityAudit->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI APIset.

Invokes "audit-get" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching security audit(s) found',
        show_cmd       => 'security audit show',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(
        @_,
        api          => 'security_audit_show',
        is_singleton => 1
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();
        my $cli_to_zapi_map = $pkg->_cli_to_zapi_map();
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        %opts,
        api          => 'audit-get',
        map          => {%$cli_to_zapi_map},
        primary_keys => [qw()], #No primary keys
    );
    $Log->exit() if $may_exit;
    return @state_objs;

} ## end sub _fetch_cmode_zapi
sub _update_state_objs_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->SUPER::_update_state_objs_cmode_zapi(@args,
        zapi_field_translations => {on_off_boolean => [qw( snmpset httpget httpset snmpget cliset cliget ontapiset ontapiget)],},);
    $Log->exit() if $may_exit;
}
1;
