#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SecurityCertificateFile ComponentState Module (auto-generated by CGT)
## @author Akshay Mohit(makshay@netapp.com), dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SecurityCertificateFile

=head1 DESCRIPTION

C<NACL::CS::SecurityCertificateFile> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SecurityCertificateFile.  A related class is
L<NACL::C::SecurityCertificateFile|lib-NACL-C-SecurityCertificateFile-pm>, which represents access
to an ONTAP SecurityCertificateFile.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the SecurityCertificateFile
element are the attributes of the SecurityCertificateFile ComponentState.

=over

=item C<< "protocol" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <certificate-file-info >
    <protocol>$value</protocol>
 </certificate-file-info >

=item C<< "common_name" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <certificate-file-info >
    <common_name>$value</common_name>
 </certificate-file-info >

=item C<< "vserver" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <certificate-file-info >
    <vserver>$value</vserver>
 </certificate-file-info >

=item C<< "path" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <certificate-file-info >
    <path>$value</path>
 </certificate-file-info >

=item C<< "type" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <certificate-file-info >
    <type>$value</type>
 </certificate-file-info >

=item C<< "service" >>

Filled in for CMode CLI.

=item C<< "node" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <certificate-file-info >
    <node-name>$value</node-name>
 </certificate-file-info >

=item C<< serial >>
Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <certificate-file-info >
    <serial-number>$value</serial-number>
 </certificate-file-info >

=item C<< file >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <certificate-file-info >
    <file>$value</file>
 </certificate-file-info >

=item C<< ca >>

Filled in for CMode CLI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <certificate-file-info >
    <certificate-authority>$value</certificate-authority>
 </certificate-file-info >

=item C<< exists >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <certificate-file-info >
    <exists>$value</exists>
 </certificate-file-info >

=back

=cut

package NACL::CS::SecurityCertificateFile;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one Dumper);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'protocol',
    scalar => 'common_name',
    scalar => 'vserver',
    scalar => 'path',
    scalar => 'type',
    scalar => 'service',
    scalar => 'node',
    scalar => 'serial',
    scalar => 'file',
    scalar => 'ca',
    scalar => 'exists',
];

=head1 METHODS

=head2 fetch

    my $SecurityCertificateFile_state   = NACL::CS::SecurityCertificateFile->fetch(command_interface => $ci, ...);
    my @SecurityCertificateFile_states = NACL::CS::SecurityCertificateFile->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI/ZAPI APIset.

Invokes "security-certificate-file-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'security certificate file show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching security certificate file(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'security_certificate_file_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => 'security_certificate_file_get_iter',
        copy => [qw ( protocol path type file exists common-name vserver )],
        map  => {
            'node'   => 'node-name',
            'serial' => 'serial-number',
            'ca'     => 'certificate-authority',
        },
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
