# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SecurityLoginRoleConfig ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SecurityLoginRoleConfig

=head1 DESCRIPTION

C<NACL::CS::SecurityLoginRoleConfig> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SecurityLoginRoleConfig.  A related class is
L<NACL::C::SecurityLoginRoleConfig|lib-NACL-C-SecurityLoginRoleConfig-pm>, which represents access
to an ONTAP SecurityLoginRoleConfig.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the SecurityLoginRoleConfig
element are the attributes of the SecurityLoginRoleConfig ComponentState.

=over

=item C<< "profile" >>

=item C<< "passwd_alphanum" >>

=item C<< "passwd_alphanum_zapi" >>

=item C<< "disallowed_reuse" >>

=item C<< "locktime" >>

=item C<< "passwd_minsize" >>

=item C<< "passwd_expwarntime" >>

=item C<< "account_exptime" >>

=item C<< "username_alphanum" >>

=item C<< "username_minsize" >>

=item C<< "vserver" >>

=item C<< "passwd_exptime" >>

=item C<< "maxattempts" >>

=item C<< "require_update" >>

=item C<< "isFromCluster" >>

=item C<< "username_alphanum_zapi" >>

=item C<< "change_delay" >>

=item C<< "role" >>

=item C<< passwd_expiry_time >>

Filled in for CMode CLI.

=item C<< username_minlength >>

Filled in for CMode CLI.

=item C<< passwd_min_special_chars >>

Filled in for CMode CLI.

=item C<< max_failed_login_attempts >>

Filled in for CMode CLI.

=item C<< passwd_minlength >>

Filled in for CMode CLI.

=item C<< lockout_duration >>

Filled in for CMode CLI.

=item C<< require_initial_passwd_update >>

Filled in for CMode CLI.

=item C<< delay_login_attempts >>

Filled in for CMode CLI.


=item C<< require_initial_passwd_update_zapi >>

Filled in for CMode CLI.


=back

=cut

package NACL::CS::SecurityLoginRoleConfig;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'profile',
    scalar => 'passwd_alphanum',
    scalar => 'passwd_alphanum_zapi',
    scalar => 'disallowed_reuse',
    scalar => 'locktime',
    scalar => 'passwd_minsize',
    scalar => 'passwd_expwarntime',
    scalar => 'account_exptime',
    scalar => 'username_alphanum',
    scalar => 'username_minsize',
    scalar => 'vserver',
    scalar => 'passwd_exptime',
    scalar => 'maxattempts',
    scalar => 'require_update',
    scalar => 'isFromCluster',
    scalar => 'username_alphanum_zapi',
    scalar => 'change_delay',
    scalar => 'role',
    scalar => 'passwd_expiry_time',
    scalar => 'username_minlength',
    scalar => 'passwd_min_special_chars',
    scalar => 'max_failed_login_attempts',
    scalar => 'passwd_minlength',
    scalar => 'lockout_duration',
    scalar => 'require_initial_passwd_update',
    scalar => 'delay_login_attempts',
    scalar => 'require_initial_passwd_update_zapi',
];

=head1 METHODS

=head2 fetch

    my $SecurityLoginRoleConfig_state   = NACL::CS::SecurityLoginRoleConfig->fetch(command_interface => $ci, ...);
	my @SecurityLoginRoleConfig_states = NACL::CS::SecurityLoginRoleConfig->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'security login role config show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching security login role config(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'security_login_role_config_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;
    my $map;
    my $command_interface = $opts{command_interface};

    $map = {
        "role"              => ['role-name'],
        "username-alphanum" => ['require-username-alpha-numeric'],
        "passwd-alphanum"   => ['require-password-alpha-numeric'],
        "disallowed-reuse"  => ['last-passwords-disallowed-count'],
        "change-delay"      => ['change-password-duration-in-days'],
        'require-initial-passwd-update' =>
            ['require-initial-password-update'],
        'passwd-min-special-chars'  => ['min-passwd-specialchar'],
        'passwd-expiry-time'        => ['password-expiration-duration'],
        'passwd-minsize'            => ['min-password-size'],
        'username-minsize'          => ['min-username-size'],
    };

    my @states = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        api  => "security_login_role_config_get_iter",
        copy => [qw(vserver max-failed-login-attempts lockout-duration)],
        map  => $map
    );
    $Log->exit() if $may_exit;
    return @states;
} ## end sub _fetch_cmode_zapi

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    delete $opts{filter};
    my @state_objs;

    $apiset->options('security.passwd.rules.enable' => 'on');
    my $response = $apiset->options('security.passwd');
    my $output   = $response->get_parsed_output();
    my $row      = $output->[0];

    if (   ($row->{'security.passwd.rules.minimum.alphabetic'} > 1)
        && ($row->{'security.passwd.rules.minimum.digit'} > 0))
    {
        $row->{'security.passwd.rules.minimum.alphabetic'} = "ENABLED";
        $row->{'security.passwd.rules.minimum.digit'}      = "ENABLED";
    } ## end if ( ( $row->{'security.passwd.rules.minimum.alphabetic'...
    my $final_attributes = $pkg->_hash_copy(
        source => $row,
        map    => {
            'security.passwd.rules.minimum'            => 'passwd-minsize',
            'security.passwd.rules.history'            => 'disallowed-reuse',
            'security.passwd.rules.minimum.alphabetic' => 'passwd-alphanum',
            'security.passwd.rules.minimum.digit'      => 'passwd-alphanum',
        },
    );
    my $obj = $pkg->new(command_interface => $command_interface);
    $obj->_set_fields(row => $final_attributes);

    push @state_objs, $obj;
    return @state_objs;

    $Log->exit() if $may_exit;

} ## end sub _fetch_7mode_cli
1;
