# $Id$ //depot/prod/test/nacldev/lib/NACL/CS/SecurityLoginRoleShowUserCapability.pm#3

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::SecurityLoginRole->show_user_capability() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SecurityLoginRoleShowUserCapability

=head1 DESCRIPTION

C<NACL::CS::SecurityLoginRoleShowUserCapability> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::SecurityLoginRole->show_user_capability() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::SecurityLoginRole->show_user_capability() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< operation_api_name >>

*Operation's API Name

Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <capability-info>
    <operation-list>
        <operation-info>
            <api-name>@value</api-name>
        </operation-info>
    </operation-list>
 </capability-info>

=item C<< object_name >>

*Object Name
Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <capability-info>
    <object-name>@value</object-name>
 </capability-info>

=item C<< operation_command_directory_path >>

*Operation Command Directory Path
Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <capability-info>
    <operation-list>
        <operation-info>
            <command-path>@value</command-path>
        </operation-info>
    </operation-list>
 </capability-info>

=item C<< operation_name >>

*Operation Name
Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <capability-info>
    <operation-list>
        <operation-info>
            <name>@value</name>
        </operation-info>
    </operation-list>
 </capability-info>

=back

=cut

package NACL::CS::SecurityLoginRoleShowUserCapability;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    array  => 'operation_api_name',
    scalar => 'object_name',
    array  => 'operation_command_directory_path',
    array  => 'operation_name',
];

=head1 METHODS

=head2 fetch

    my $SecurityLoginRoleShowUserCapability_state  = NACL::CS::SecurityLoginRoleShowUserCapability->fetch(command_interface => $ci, ...);
	my @SecurityLoginRoleShowUserCapability_states = NACL::CS::SecurityLoginRoleShowUserCapability->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI APISet.

Invokes 'system-user-capability-get-iter' API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'security login role show-user-capability',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching security login role(s) found',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'security_login_role_show_user_capability',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [qw( object-name )],
        map  => {
            'operation-api-name' => [
                make_zapi_array('operation-list'),
                make_zapi_skip('operation-info'),
                'api-name'
            ],
            'operation-command-directory-path' => [
                make_zapi_array('operation-list'),
                make_zapi_skip('operation-info'),
                'command-path'
            ],
            'operation-name' => [
                make_zapi_array('operation-list'),
                make_zapi_skip('operation-info'),
                'name'
            ],
        },
        api => 'system_user_capability_get_iter'
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}    #end of sub _fetch_cmode_zapi

1;
