# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SecuritySsh ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SecuritySsh

=head1 DESCRIPTION

C<NACL::CS::SecuritySsh> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SecuritySsh.  A related class is
L<NACL::C::SecuritySsh|lib-NACL-C-SecuritySsh-pm>, which represents access
to an ONTAP SecuritySsh.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the SecuritySsh
element are the attributes of the SecuritySsh ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< ciphers >>

Ciphers
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $ciphers = $obj->ciphers();
 
# $ciphers contains a reference to the array of values
 
my @ciphers = $obj->ciphers();
 
# @ciphers contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { ciphers = [ value1, value2...] }

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<security-ssh-get-iter>
    <ciphers>
        <ciphers><@values></ciphers>
    </ciphers>
</security-ssh-get-iter>

=item C<< vserver >>

Vserver

Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

<security-ssh-get-iter>
    <vserver-name><$value></vserver-name>
</security-ssh-get-iter>

=item C<< key_exchange_algorithms >>

Key Exchange Algorithms
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $key_exchange_algorithms = $obj->key_exchange_algorithms();
 
# $key_exchange_algorithms contains a reference to the array of values
 
my @key_exchange_algorithms = $obj->key_exchange_algorithms();
 
# @key_exchange_algorithms contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { key-exchange-algorithms = [ value1, value2...] }

Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

<security-ssh-get-iter>
    <key-exchange-algorithms>
        <kex-algorithms><@values></kex-algorithms>
    </key-exchange-algorithms>
</security-ssh-get-iter>

=back

=cut

package NACL::CS::SecuritySsh;

use strict;
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    array  => 'ciphers',
    scalar => 'vserver',
    array  => 'key_exchange_algorithms',
];

=head1 METHODS

=head2 fetch

    my $SecuritySsh_state  = NACL::CS::SecuritySsh->fetch(command_interface => $ci, ...);
	my @SecuritySsh_states = NACL::CS::SecuritySsh->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes "security-ssh-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        show_cmd       => 'security ssh show',
        exception_text => 'No matching security ssh(s) found',
        frontend       => 'NACL::CS::SecuritySsh::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@args, api => 'security_ssh_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        map => {
            'ciphers' =>
                [make_zapi_array('ciphers'), make_zapi_skip('ciphers')],
            'vserver'                 => 'vserver-name',
            'key-exchange-algorithms' => [
                make_zapi_array('key-exchange-algorithms'),
                make_zapi_skip('kex-algorithms'),
            ]
        },
        api => 'security-ssh-get-iter',
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
