#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Spm ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::Spm

=head1 DESCRIPTION

C<NACL::CS::Spm> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP Spm.  A related class is
L<NACL::C::Spm|lib-NACL-C-Spm-pm>, which represents access
to an ONTAP Spm.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the Spm
element are the attributes of the Spm ComponentState.

=over

=item C<< "status" >>
(Array)

scalar or list context.
my $status = $obj->status();
# $status contains a reference to the array of values
my @status = $obj->status();
# @status  contains the array of values

Filled in for CMode CLI.

=item C<< "node" >>

Filled in for CMode CLI.

=item C<< "state" >>

Filled in for CMode CLI.

=item C<< "page_reclaims" >>

Filled in for CMode CLI.

=item C<< "involuntary_context_swaps" >>

Filled in for CMode CLI.

=item C<< "pid" >>

Filled in for CMode CLI.

=item C<< "num_swaps" >>

Filled in for CMode CLI.

=item C<< "cmd_line" >>

Filled in for CMode CLI.

=item C<< "num_fs_input" >>

Filled in for CMode CLI.

=item C<< "restarts" >>

Filled in for CMode CLI.

=item C<< "max_restarts" >>

Filled in for CMode CLI.

=item C<< "page_faults" >>

Filled in for CMode CLI.

=item C<< "name" >>

Filled in for CMode CLI.

=item C<< "num_fs_output" >>

Filled in for CMode CLI.

=item C<< "current_rss" >>

Filled in for CMode CLI.

=item C<< "handle" >>

Filled in for CMode CLI.

=item C<< "voluntary_context_swaps" >>

Filled in for CMode CLI.

=back

=cut

package NACL::CS::Spm;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    array  => 'status',
    scalar => 'node',
    scalar => 'state',
    scalar => 'page_reclaims',
    scalar => 'involuntary_context_swaps',
    scalar => 'pid',
    scalar => 'num_swaps',
    scalar => 'cmd_line',
    scalar => 'num_fs_input',
    array  => 'restarts',
    scalar => 'max_restarts',
    scalar => 'page_faults',
    scalar => 'name',
    scalar => 'num_fs_output',
    scalar => 'current_rss',
    scalar => 'handle',
    scalar => 'voluntary_context_swaps',
];

=head1 METHODS

=head2 fetch

    my $Spm_state   = NACL::CS::Spm->fetch(command_interface => $ci, ...);
	my @Spm_states = NACL::CS::Spm->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'spm show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching spm(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_, api => 'spm_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

1;
