# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Statistics2 ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::Statistics2

=head1 DESCRIPTION

C<NACL::CS::Statistics2> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP Statistics2.  A related class is
L<NACL::C::Statistics2|lib-NACL-C-Statistics2-pm>, which represents access
to an ONTAP Statistics2.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the Statistics2
element are the attributes of the Statistics2 ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< aggr_display >>

Filled in for CMode CLI.

=item C<< object >>

Filled in for CMode CLI.

=item C<< base_value >>

Filled in for CMode CLI.

=item C<< denominator >>

Filled in for CMode CLI.

=item C<< base_units >>

Filled in for CMode CLI.

=item C<< scope_display_desc >>

Filled in for CMode CLI.

=item C<< scope_display >>

Filled in for CMode CLI.

=item C<< object_name >>

Filled in for CMode CLI.

=item C<< numerators >>

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $numerators = $obj->numerators();
# $numerators contains a reference to the array of values
my @numerators = $obj->numerators();
# @numerators contains the array of values

Filled in for CMode CLI.

=item C<< node >>

Filled in for CMode CLI.

=item C<< countername >>

Filled in for CMode CLI.

=item C<< instance_name >>

Filled in for CMode CLI.

=item C<< base_delta >>

Filled in for CMode CLI.

=item C<< base_rate >>

Filled in for CMode CLI.

=item C<< labels >>

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $labels = $obj->labels();
# $labels contains a reference to the array of values
my @labels = $obj->labels();
# @labels contains the array of values

Filled in for CMode CLI.

=item C<< valueorlabel >>

Filled in for CMode CLI.

=item C<< vserver >>

Filled in for CMode CLI.

=item C<< value >>

Filled in for CMode CLI.

=item C<< numerator >>

Filled in for CMode CLI.

=item C<< values >>

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $values = $obj->values();
# $values contains a reference to the array of values
my @values = $obj->values();
# @values contains the array of values

Filled in for CMode CLI.

=item C<< description >>

Filled in for CMode CLI.

=item C<< instance >>

Filled in for CMode CLI.

=item C<< prop >>

Filled in for CMode CLI.

=item C<< delta >>

Filled in for CMode CLI.

=item C<< objectname >>

Filled in for CMode CLI.

=item C<< denominators >>

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $denominators = $obj->denominators();
# $denominators contains a reference to the array of values
my @denominators = $obj->denominators();
# @denominators contains the array of values

Filled in for CMode CLI.

=item C<< counter >>

Filled in for CMode CLI.

=item C<< raw >>

Filled in for CMode CLI.


=item C<< sort_by_counter >>

Filled in for CMode CLI.

=item C<< filter >>

Filled in for CMode CLI.

=item C<< sample_id >>

Filled in for CMode CLI.

=item C<< sort_order >>

Filled in for CMode CLI.


=item C<< instance_uuid >>

Filled in for CMode CLI.


=item C<< display_value >>

Filled in for CMode CLI.


=back

=cut

package NACL::CS::Statistics2;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'aggr_display',
    scalar => 'object',
    scalar => 'base_value',
    scalar => 'denominator',
    scalar => 'base_units',
    scalar => 'scope_display_desc',
    scalar => 'scope_display',
    scalar => 'object_name',
    array  => 'numerators',
    scalar => 'node',
    scalar => 'countername',
    scalar => 'instance_name',
    scalar => 'base_delta',
    scalar => 'base_rate',
    array  => 'labels',
    scalar => 'valueorlabel',
    scalar => 'vserver',
    scalar => 'value',
    scalar => 'numerator',
    array  => 'values',
    scalar => 'description',
    scalar => 'instance',
    scalar => 'prop',
    scalar => 'delta',
    scalar => 'objectname',
    array  => 'denominators',
    scalar => 'counter',
    scalar => 'raw',
    scalar => 'sort_by_counter',
    scalar => 'filter',
    scalar => 'sample_id',
    scalar => 'sort_order',
    scalar => 'instance_uuid',
    scalar => 'display_value',
];

=head1 METHODS

=head2 fetch

    my $Statistics2_state  = NACL::CS::Statistics2->fetch(command_interface => $ci, ...);
	my @Statistics2_states = NACL::CS::Statistics2->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'statistics2 show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching statistics2(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli( @_, api => 'statistics2_show', );
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
