# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StatisticsCatalogCounter ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::StatisticsCatalogCounter

=head1 DESCRIPTION

C<NACL::CS::StatisticsCatalogCounter> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP StatisticsCatalogCounter.  A related class is
L<NACL::C::StatisticsCatalogCounter|lib-NACL-C-StatisticsCatalogCounter-pm>, which represents access
to an ONTAP StatisticsCatalogCounter.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the StatisticsCatalogCounter
element are the attributes of the StatisticsCatalogCounter ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< object >>

Object

Filled in for CMode CLI

=item C<< size >>

*Counter Size

Filled in for CMode CLI

=item C<< desc >>

Filled in for CMode CLI

=item C<< base_counter >>

*Denominator For Percent/Average

Filled in for CMode CLI

=item C<< aggregation_style >>

Filled in for CMode CLI

=item C<< is_key >>

Is Key Counter?, possible value(s) are true,false

Filled in for CMode CLI

=item C<< name >>

Filled in for CMode CLI

=item C<< instance >>

Filled in for CMode CLI

=item C<< description >>

Description

Filled in for CMode CLI

=item C<< properties >>

*Counter Properties

Filled in for CMode CLI

=item C<< translation_input_counter >>

*Translation Input Counter

Filled in for CMode CLI

=item C<< unit >>

*Counter Unit

Filled in for CMode CLI

=item C<< objectname >>

Filled in for CMode CLI

=item C<< privilege >>

Privilegel Level

Filled in for CMode CLI

=item C<< label >>

Labels for Array Counters

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $label = $obj->label();
# $label contains a reference to the array of values
my @label = $obj->label();
# @label contains the array of values
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref
# filter => { label = [ value1, value2...] }

Filled in for CMode CLI

=item C<< type >>

*Counter Type

Filled in for CMode CLI

=item C<< counter >>

Counter

Filled in for CMode CLI

=item C<< vserver >>

Filled in for CMode CLI.

=item C<< privilege_override >>

Filled in for CMode CLI.

=item C<< is_key_counter >>

Is Key Counter possible value(s) are, true,false

Filled in for CMode CLI.


=item C<< replaced_by >>

Replaced By Counter If Deprecated

Filled in for CMode CLI.

=item C<< is_deprecated >>

Is Counter Deprecated possible value(s) are, true,false

Filled in for CMode CLI.


=back

=cut

package NACL::CS::StatisticsCatalogCounter;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'object',
    scalar => 'size',
    scalar => 'desc',
    scalar => 'base_counter',
    scalar => 'aggregation_style',
    scalar => 'is_key',
    scalar => 'name',
    scalar => 'instance',
    scalar => 'description',
    scalar => 'properties',
    scalar => 'translation_input_counter',
    scalar => 'unit',
    scalar => 'objectname',
    scalar => 'privilege',
    array  => 'label',
    scalar => 'type',
    scalar => 'counter',
    scalar => 'vserver',
    scalar => 'privilege_override',
    scalar => 'is_key_counter',
    scalar => 'replaced_by',
    scalar => 'is_deprecated',
];

=head1 METHODS

=head2 fetch

    my $StatisticsCatalogCounter_state  = NACL::CS::StatisticsCatalogCounter->fetch(command_interface => $ci, ...);
	my @StatisticsCatalogCounter_states = NACL::CS::StatisticsCatalogCounter->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'statistics catalog counter show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching statistics catalog counter(s) found',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'statistics_catalog_counter_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
