#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StatisticsCatalogInstance ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::StatisticsCatalogInstance

=head1 DESCRIPTION

C<NACL::CS::StatisticsCatalogInstance> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP StatisticsCatalogInstance.  A related class is
L<NACL::C::StatisticsCatalogInstance|lib-NACL-C-StatisticsCatalogInstance-pm>, which represents access
to an ONTAP StatisticsCatalogInstance.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the StatisticsCatalogInstance
element are the attributes of the StatisticsCatalogInstance ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< required_input_filter >>

Filled in for CMode CLI

=item C<< object >>

Object

Filled in for CMode CLI

=item C<< filter >>

Filter Data

Filled in for CMode CLI

=item C<< outputName >>

Filled in for CMode CLI

=item C<< node >>

Node Name

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $node = $obj->node();
# $node contains a reference to the array of values
my @node = $obj->node();
# @node contains the array of values
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref
# filter => { node = [ value1, value2...] }

Filled in for CMode CLI

=item C<< instance >>

Instance Name

Filled in for CMode CLI

=item C<< objectname >>

Filled in for CMode CLI

=item C<< filter_data >>

Filled in for CMode CLI

=item C<< required_filter >>

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $required_filter = $obj->required_filter();
# $required_filter contains a reference to the array of values
my @required_filter = $obj->required_filter();
# @required_filter contains the array of values

Filled in for CMode CLI

=item C<< outputUUID >>

Filled in for CMode CLI.

=item C<< instanceUUID >>

Instance UUID

Filled in for CMode CLI.


=back

=cut

package NACL::CS::StatisticsCatalogInstance;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'required_input_filter',
    scalar => 'object',
    scalar => 'filter',
    scalar => 'outputName',
    array  => 'node',
    scalar => 'instance',
    scalar => 'objectname',
    scalar => 'filter_data',
    array  => 'required_filter',
    scalar => 'outputUUID',
    scalar => 'instanceUUID',
];

=head1 METHODS

=head2 fetch

    my $StatisticsCatalogInstance_state  = NACL::CS::StatisticsCatalogInstance->fetch(command_interface => $ci, ...);
	my @StatisticsCatalogInstance_states = NACL::CS::StatisticsCatalogInstance->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'statistics catalog instance show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching statistics catalog instance(s) found',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'statistics_catalog_instance_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
