# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StorageAggregateRelocation ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::StorageAggregateRelocation

=head1 DESCRIPTION

C<NACL::CS::StorageAggregateRelocation> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP StorageAggregateRelocation.  A related class is
L<NACL::C::StorageAggregateRelocation|lib-NACL-C-StorageAggregateRelocation-pm>, which represents access
to an ONTAP StorageAggregateRelocation.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the StorageAggregateRelocation
element are the attributes of the StorageAggregateRelocation ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< aggr_index >>

Filled in for CMode CLI.

=item C<< relocation_status >>

Filled in for CMode CLI/ZAPI.

Maps to:
    CMode ZAPI:
        <aggr-relocation-status>
            <relocation-status>$value</relocation-status>
        <aggr-relocation-status>

=item C<< destination >>

Filled in for CMode CLI/ZAPI.

Maps to:
    CMode ZAPI:
        <aggr-relocation-status>
            <destination>$value</destination>
        <aggr-relocation-status>

=item C<< aggregate >>

Filled in for CMode CLI/ZAPI.

Maps to:
    CMode ZAPI:
        <aggr-relocation-status>
            <aggregate>$value</aggregate>
        <aggr-relocation-status>

=item C<< node >>

Filled in for CMode CLI/ZAPI.

Maps to:
    CMode ZAPI:
        <aggr-relocation-status>
            <source-node>$value</source-node>
        <aggr-relocation-status>

=item C<< sfo_am_status >>

SFO Aggregate Move Status

Filled in for CMode CLI.


=back

=cut

package NACL::CS::StorageAggregateRelocation;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::ComponentUtils qw(Dumper);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'aggr_index',
    scalar => 'relocation_status',
    scalar => 'destination',
    scalar => 'aggregate',
    scalar => 'node',
    scalar => 'sfo_am_status',
];

=head1 METHODS

=head2 fetch

    my $StorageAggregateRelocation_state  = NACL::CS::StorageAggregateRelocation->fetch(command_interface => $ci, ...);
	my @StorageAggregateRelocation_states = NACL::CS::StorageAggregateRelocation->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'aggr-relocation-status' API for CMode ZAPI

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts       = @args;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }    ## end

        ],
	show_cmd       => 'storage aggregate relocation show',
        exception_text => 'No matching storage aggregate relocation(s) found'
    );

    my @cs;
    foreach my $state_obj (@state_objs) {
        my $aggr = $state_obj->aggregate() || '';
        push @cs, $state_obj if ($aggr ne '-');
    }

    if (!@cs && !$opts{allow_empty}) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::NoElementsFound->throw(
            "No matching storage aggregate relocation(s) found");
    }
    $Log->exit() if $may_exit;
    return wantarray ? @cs : $cs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'storage_aggregate_relocation_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $allowed_cmode_options = [
        qw( destination aggregate relocation-status
            )
    ];
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => $allowed_cmode_options,
        map  => {node => 'source-node'},
        api  => 'aggr_relocation_status',
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

sub _compute_cmode_cli_options {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $is_aggr_present = 0;
    my $opts            = $pkg->SUPER::_compute_cmode_cli_options(@args);

    if (defined $opts->{fields}) {
        foreach my $field (split ',', $opts->{fields}) {
            $is_aggr_present = 1 if ($field =~ m[^aggregate$]i);
        }
        if (!$is_aggr_present) {
            $opts->{fields} .= ',aggregate';
        }
    }
    $Log->exit() if $may_exit;
    return $opts;
} ## end sub _compute_cmode_cli_options

1;
