#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StorageDiskOption ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::StorageDiskOption

=head1 DESCRIPTION

C<NACL::CS::StorageDiskOption> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP StorageDiskOption.  A related class is
L<NACL::C::StorageDiskOption|lib-NACL-C-StorageDiskOption-pm>, which represents access
to an ONTAP StorageDiskOption.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the StorageDiskOption
element are the attributes of the StorageDiskOption ComponentState.

=over

=item C<< "bkg_firmware_update" >>

=item C<< "autocopy" >>

=item C<< "autoassign" >>

=item C<< "node" >>

=item C<< autoassign_shelf >>

Filled in for CMode CLI.


=item C<< autoassign_policy >>

Auto Assignment Policy possible value(s) are, default,bay,shelf,stack

Filled in for CMode CLI.


=back

=cut

package NACL::CS::StorageDiskOption;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'bkg_firmware_update',
    scalar => 'autocopy',
    scalar => 'autoassign',
    scalar => 'node',
    scalar => 'disk_auto_assign',
    scalar => 'autoassign_shelf',
    scalar => 'autoassign_policy',
];

=head1 METHODS

=head2 fetch

    my $StorageDiskOption_state   = NACL::CS::StorageDiskOption->fetch(command_interface => $ci, ...);
	my @StorageDiskOption_states = NACL::CS::StorageDiskOption->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'storage disk option show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        exception_text => 'No matching storage disk option(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'storage_disk_option_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my @state_objs;
    my $pkg    = shift;
    my %opts   = validate @_, $pkg->_fetch_backend_validate_spec();
    my $apiset = delete $opts{apiset};

    my (%temp_attributes, %final_attributes);

    my $response = $apiset->options('disk.auto_assign');
    my $output   = $response->get_parsed_output();
    $temp_attributes{'disk_auto_assign'} = $output->[0]->{'disk.auto_assign'};
    $pkg->_hash_copy(
        source => \%temp_attributes,
        map    => {'disk_auto_assign' => "autoassign",},
        target => \%final_attributes
    );

    my $obj = $pkg->new(command_interface => $opts{command_interface});
    $obj->_set_fields(row => \%final_attributes);
    push @state_objs, $obj;
    return @state_objs;
} ## end sub _fetch_7mode_cli
1;
