# $Id: //depot/prod/test/nacldev/lib/NACL/CS/StorageFailoverHwassistStats.pm#9 $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StorageFailoverHwassistStats ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::StorageFailoverHwassistStats

=head1 DESCRIPTION

C<NACL::CS::StorageFailoverHwassistStats> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP StorageFailoverHwassistStats.  A related class is
L<NACL::C::StorageFailoverHwassistStats|lib-NACL-C-StorageFailoverHwassistStats-pm>, which represents access
to an ONTAP StorageFailoverHwassistStats.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the StorageFailoverHwassistStats
element are the attributes of the StorageFailoverHwassistStats ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< times_throttled >>

Filled in for CMode CLI.

=item C<< test >>

Filled in for CMode CLI.

=item C<< power_loss >>

Filled in for CMode CLI.

=item C<< keep_alive >>

Filled in for CMode CLI.

=item C<< loss_of_heartbeat >>

Filled in for CMode CLI.

=item C<< power_cycle_via_rlm >>

Filled in for CMode CLI.

=item C<< reset_via_sp >>

Filled in for CMode CLI.

=item C<< unknown_alerts >>

Filled in for CMode CLI.

=item C<< reset_via_rlm >>

Filled in for CMode CLI.

=item C<< node >>

Filled in for CMode CLI/ZAPI.

=item C<< bad_nvram_id >>

Filled in for CMode CLI.

=item C<< power_cycle_via_sp >>

Filled in for CMode CLI.

=item C<< power_off_via_sp >>

Filled in for CMode CLI.

=item C<< power_off_via_rlm >>

Filled in for CMode CLI.

=item C<< ss_mismatch >>

Filled in for CMode CLI.

=item C<< watchdog_reset >>

Filled in for CMode CLI.

=item C<< abnormal_reboot >>

Filled in for CMode CLI.

=item C<< alert_event >>

Filled in for CMode CLI.

=item C<< post_error >>

Filled in for CMode CLI.

=back

=cut

package NACL::CS::StorageFailoverHwassistStats;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'times_throttled',
    scalar => 'test',
    scalar => 'power_loss',
    scalar => 'keep_alive',
    scalar => 'loss_of_heartbeat',
    scalar => 'power_cycle_via_rlm',
    scalar => 'reset_via_sp',
    scalar => 'unknown_alerts',
    scalar => 'reset_via_rlm',
    scalar => 'node',
    scalar => 'bad_nvram_id',
    scalar => 'power_cycle_via_sp',
    scalar => 'power_off_via_sp',
    scalar => 'power_off_via_rlm',
    scalar => 'ss_mismatch',
    scalar => 'watchdog_reset',
    scalar => 'abnormal_reboot',
    scalar => 'post_error',
    scalar => 'alert_type',
    scalar => 'alert_event',
    scalar => 'count',
    scalar => 'takeover',
    scalar => 'last_received',
];

=head1 METHODS

=head2 fetch

    my $StorageFailoverHwassistStats_state  = NACL::CS::StorageFailoverHwassistStats->fetch(command_interface => $ci, ...);
	my @StorageFailoverHwassistStats_states = NACL::CS::StorageFailoverHwassistStats->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI/ZAPI APISet.
This method executes cf-hwassist-stats for CMode zapi

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },

        ],
        exception_text =>
            'No matching storage failover hwassist stats(s) found',
        show_cmd       =>
            'storage failover hwassist stats show',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();

    my $apiset            = $opts{apiset};
    my $command_interface = $opts{command_interface};
    my $filter            = $opts{filter};

    my @state_objs;

    my $response = $apiset->storage_failover_hwassist_stats_show(
        node => $filter->{node});
    my $output = $response->get_parsed_output();

    my $state_fields = {};
    my ($alert_rows, $times_throttled, $node_name);

    foreach my $node (@$output) {
        foreach (keys %$node) {
            $node_name  = $_;
            $alert_rows = $node->{$node_name};
            if (ref($alert_rows) =~ /array/i) {
                foreach my $row (@$alert_rows) {
                    if (defined(
                            $row->{
                                'alerts_throttled'}
                        )
                        )
                    {
                        $times_throttled = $row->{
                            'alerts_throttled'};
                        last;
                    }
                }
                foreach my $alert_row (@$alert_rows) {
                    next
                        if (
                        defined(
                            $alert_row->{
                                'alerts_throttled'}
                        )
                        );
                    my $obj
                        = $pkg->new(command_interface => $command_interface);
                    $pkg->_hash_copy(
                        source => $alert_row,
                        copy   => [
                            qw(count takeover alert_type alert_event last_received )
                        ],
                        target => $state_fields,
                    );
                    $state_fields->{'node'}            = $node_name;
                    $state_fields->{'times-throttled'} = $times_throttled;
                    $obj->_set_fields(row => $state_fields);
                    push @state_objs, $obj;
                }
            } elsif ($node_name =~ /node/) {
                $state_fields->{'node'} = $node->{$node_name};
                my $obj = $pkg->new(command_interface => $command_interface);
                $obj->_set_fields(row => $state_fields);
                push @state_objs, $obj;
            }
        }
    }

    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg    = shift;
    my %opts   = validate @_, $pkg->_fetch_backend_validate_spec();
    my $apiset = $opts{apiset};
    my $filter = $opts{filter};
    my $hwassist_response;
    my @state_objs;

    my %hw_args;
    $hw_args{'node'} = $filter->{'node'};
    $hwassist_response = $apiset->cf_hwassist_stats(%hw_args);
    my $hwassist_output = $hwassist_response->get_parsed_output();

    foreach my $row (@{$hwassist_output}) {
        foreach my $value (values %{$row}) {
            my $state_fields;
            if ($value
                =~ /^\s*(\w+(\s+)?(\w+)?)\s+(\w+|---)\s+(\d+)\s+(\w+|---)\s+(.*)?/
                )
            {
                my $obj = $pkg->new(
                    command_interface => $opts{command_interface});
                $state_fields->{'alert-type'}    = $1;
                $state_fields->{'alert-event'}   = $4;
                $state_fields->{'count'}         = $5;
                $state_fields->{'takeover'}      = $6;
                $state_fields->{'last-received'} = $7;
                $state_fields->{'node'}          = $hw_args{'node'};
                $obj->_set_fields(row => $state_fields);
                push @state_objs, $obj;
            }
        }
    }
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;

    my $pkg               = shift;
    my %opts              = validate @_, $pkg->_fetch_backend_validate_spec();
    my $apiset            = delete $opts{apiset};
    my $command_interface = delete $opts{command_interface};
    my @state_objs;
    my $response = $apiset->cf_hw_assist_stats();
    my $output   = $response->get_parsed_output();
    my $final_attributes;

    foreach my $row (@$output) {
        foreach my $key (keys(%$row)) {
            if ($key eq "partner") {
                next;
            }
            my $state_settings = $row->{$key};
            foreach my $row1 (@$state_settings) {
                $final_attributes = $pkg->_hash_copy(
                    source => $row1,
                    copy   => [
                        qw (alert_type alert_event count takeover last_received)
                    ],
                );
                my $obj = $pkg->new();
                $obj->_set_fields(row => $final_attributes);
                push @state_objs, $obj;

            } ## end foreach my $row1 (@$state_settings)
        } ## end foreach my $key ( keys(%$row...

    } ## end foreach my $row (@$output)

    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_7mode_cli
1;
