# $Id$ //depot/prod/test/nacldev/lib/NACL/CS/StorageFailoverInterconnectStatus.pm#3
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::StorageFailoverInterconnect->status() (auto-generated by CGT)
## @author Akshay Mohit(makshay@netapp.com), dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::StorageFailoverInterconnectStatus

=head1 DESCRIPTION

C<NACL::CS::StorageFailoverInterconnectStatus> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::StorageFailoverInterconnect->status() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::StorageFailoverInterconnect->status() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< active_link >>

Filled in for CMode-CLI.

=item C<< link >>

Filled in for 7Mode-CLI.

=item C<< link_0 >>

Filled in for 7Mode-CLI.

=item C<< link_1 >>

Filled in for 7Mode-CLI.

=item C<< node >>

Filled in for CMode-CLI.

=item C<< state >>

Maps to :
    7M CLI : cfo_rv_connection_state

=back

=cut

package NACL::CS::StorageFailoverInterconnectStatus;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'active_link',
    scalar => 'node',
    scalar => 'state',

    # specific to 7Mode
    scalar => 'link',
    scalar => 'link_0',
    scalar => 'link_1',
];

=head1 METHODS

=head2 fetch

    my $StorageFailoverInterconnectStatus_state  = NACL::CS::StorageFailoverInterconnectStatus->fetch(command_interface => $ci, ...);
	my @StorageFailoverInterconnectStatus_states = NACL::CS::StorageFailoverInterconnectStatus->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode-CLI or 7Mode-CLI APISet.
Invokes 'ic_status' api for 7Mode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'storage failover interconnect status',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode'
            }
        ],
        exception_text => 'No matching storage failover interconnect(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli( @_,
        api => 'storage_failover_interconnect_status', );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();
    my $apiset = delete $opts{apiset};
    my @state_objs;
    my $link;
    my $response = $apiset->ic_status();
    my $output   = $response->get_parsed_output();
    foreach my $row (@$output) {
        my $obj = $pkg->new( command_interface => $opts{command_interface} );
        my $row_modified = $pkg->_hash_copy(
            source => $row,
            copy   => [qw( cfo_rv_nic_used link link_0 link_1)],
            map    => { 'cfo_rv_connection_state' => 'state', },
        );
        $obj->_set_fields( row => $row_modified );
        if ( defined( $obj->state() ) ) {
            if ( $obj->state() =~ /CONNECTED/i ) {
                $obj->state('Established');
            }
        }
        push @state_objs, $obj;
    }
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_7mode_cli
1;
