# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::StoragePool->show_aggregate() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::StoragePoolShowAggregate

=head1 DESCRIPTION

C<NACL::CS::StoragePoolShowAggregate> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::StoragePool->show_aggregate() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::StoragePool->show_aggregate() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over


=item C<< allocation_unit_size >>

Allocation Unit Size
Filled in for CMode CLI/ZAPI iter

=item C<< allocated_unit_count >>

Number of AU's assigned to this aggregate
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-aggregate-info allocated-unit-count="$value" />


=item C<< capacity >>

Capacity
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-aggregate-info used-capacity="$value" />


=item C<< original_owner >>

Original Owner Name
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-aggregate-info original-owner="$value" />


=item C<< node >>

Node possible value(s) are, <nodename>,local
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-aggregate-info node="$value" />


=item C<< pool_total_size >>

Total Size of the Storage Pool
Filled in for CMode CLI/ZAPI iter

=item C<< aggregate >>

Aggregate
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-aggregate-info aggregate="$value" />


=item C<< storage_pool_uuid >>

Uuid of Storage Pool
Filled in for CMode CLI/ZAPI iter

=item C<< storage_pool >>

Name of Storage Pool
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-aggregate-info storage-pool="$value" />

=back

=cut

package NACL::CS::StoragePoolShowAggregate;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'allocation_unit_size',
    scalar => 'allocated_unit_count',
    scalar => 'capacity',
    scalar => 'original_owner',
    scalar => 'node',
    scalar => 'pool_total_size',
    scalar => 'aggregate',
    scalar => 'storage_pool_uuid',
    scalar => 'storage_pool',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $StoragePoolShowAggregate_state  = NACL::CS::StoragePoolShowAggregate->fetch(command_interface => $ci, ...);
	my @StoragePoolShowAggregate_states = NACL::CS::StoragePoolShowAggregate->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes storage-pool-aggregate-get-iter API for CMode ZAPI iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },

        ],
        show_cmd => 'storage pool show-aggregate',
	exception_text => 'No matching storage pool(s) found',
        frontend       => 'NACL::CS::StoragePoolShowAggregate::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'storage_pool_show_aggregate',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(allocated-unit-count node original-owner aggregate storage-pool
                )
        ],
        map => {'capacity' => 'used-capacity',},
        api => "storage-pool-aggregate-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
