# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::StoragePool->show_disks() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::StoragePoolShowDisks

=head1 DESCRIPTION

C<NACL::CS::StoragePoolShowDisks> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::StoragePool->show_disks() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::StoragePool->show_disks() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over


=item C<< disk >>

Name of the disk
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-disk-info disk="$value" />


=item C<< disk_type >>


Disk Type possible value(s) are, ATA , BSAS , EATA , FCAL , FSAS , LUN , MSATA , SAS  , SATA , SCSI , SSD , XATA , XSAS , VMDISK
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-disk-info disk-type="$value" />


=item C<< is_disk_offline >>

Is Disk Offline? possible value(s) are, true,false
Filled in for CMode CLI/ZAPI iter

=item C<< is_sdc_active >>

Is SDC Active? possible value(s) are, true,false
Filled in for CMode CLI/ZAPI iter

=item C<< total_size >>

Total Size
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-disk-info disk-total-size="$value" />


=item C<< disk_ui_status >>

Disk UI Status
Filled in for CMode CLI/ZAPI iter

=item C<< is_disk_sick >>

Is Disk Sick? possible value(s) are, true,false
Filled in for CMode CLI/ZAPI iter

=item C<< sdc_target >>

SDC Target
Filled in for CMode CLI/ZAPI iter

=item C<< disk_uid >>

Disk Unique ID
Filled in for CMode CLI/ZAPI iter

=item C<< storage_pool_uuid >>

Uuid of Storage Pool
Filled in for CMode CLI/ZAPI iter

=item C<< usable_size >>

Disk Usable Size
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-disk-info disk-usable-size="$value" />


=item C<< storage_pool >>

Name of Storage Pool
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-disk-info storage-pool="$value" />

=back

=cut

package NACL::CS::StoragePoolShowDisks;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'disk',
    scalar => 'disk_type',
    scalar => 'is_disk_offline',
    scalar => 'is_sdc_active',
    scalar => 'total_size',
    scalar => 'disk_ui_status',
    scalar => 'is_disk_sick',
    scalar => 'sdc_target',
    scalar => 'disk_uid',
    scalar => 'storage_pool_uuid',
    scalar => 'usable_size',
    scalar => 'storage_pool',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $StoragePoolShowDisks_state  = NACL::CS::StoragePoolShowDisks->fetch(command_interface => $ci, ...);
	my @StoragePoolShowDisks_states = NACL::CS::StoragePoolShowDisks->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes storage-pool-disk-get-iter API for CMode ZAPI iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },

        ],
        show_cmd => 'storage pool show-disks',
	exception_text => 'No matching storage pool(s) found',
        frontend       => 'NACL::CS::StoragePoolShowDisks::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'storage_pool_show_disks',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(disk-type disk storage-pool
               )
        ],
        map => {
            'total-size'  => 'disk-total-size',
            'usable-size' => 'disk-usable-size',
        },
        api => "storage-pool-disk-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
