# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StorageShelfAcp ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::StorageShelfAcp

=head1 DESCRIPTION

C<NACL::CS::StorageShelfAcp> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP StorageShelfAcp.  A related class is
L<NACL::C::StorageShelfAcp|lib-NACL-C-StorageShelfAcp-pm>, which represents access
to an ONTAP StorageShelfAcp.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the StorageShelfAcp
element are the attributes of the StorageShelfAcp ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< error_severity >>


Error Severity possible value(s) are, unknown,notice,warning,error,critical
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< error_type >>


Error Type possible value(s) are, No-Error,Connection-Issue,Connection-Activity,Module-Error,Shelf-Error
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< is_enabled >>

Is Enabled? possible value(s) are, true,false
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< channel >>

Channel possible value(s) are, unknown,out-of-band,in-band
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< node >>

Node possible value(s) are, <nodename>,local
Filled in for CMode CLI/ZAPI, non-iter.
Mapping for CMode ZAPI non-iter: node-name


=item C<< port >>

Port
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< error_id >>

Error ID
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< connection_status >>


Connection Status possible value(s) are, no-connectivity,partial-connectivity,full-connectivity,additional-connectivity,unknown-connectivity,not-available,connection-disabled
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< error_text >>

Error Text
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< corrective_action >>

Corrective Action
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< address >>

IP Address
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< netmask >>

Netmask
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< subnet >>

Subnet
Filled in for CMode CLI/ZAPI, non-iter.
=back

=cut

package NACL::CS::StorageShelfAcp;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'error_severity',
    scalar => 'error_type',
    scalar => 'is_enabled',
    scalar => 'channel',
    scalar => 'node',
    scalar => 'port',
    scalar => 'error_id',
    scalar => 'connection_status',
    scalar => 'error_text',
    scalar => 'corrective_action',
    scalar => 'address',
    scalar => 'netmask',
    scalar => 'subnet',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _extra_filter_fields {
    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;
    return [qw( errors )];
}

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(@opts, _primary_keys => [qw(node)],);
    $Log->exit() if $may_exit;
}

=head1 METHODS

=head2 fetch

    my $StorageShelfAcp_state  = NACL::CS::StorageShelfAcp->fetch(command_interface => $ci, ...);
	my @StorageShelfAcp_states = NACL::CS::StorageShelfAcp->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes storage-shelf-acp-get API for CMode ZAPI non-iter.


=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'storage shelf acp show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        exception_text => 'No matching StorageShelfAcp(s) found',
        frontend       => 'NACL::CS::StorageShelfAcp::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _update_state_objs_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->SUPER::_update_state_objs_cmode_zapi(@args,
        zapi_field_translations => {hyphenate_value => [qw(error-severity)],},
    );
    $Log->exit() if $may_exit;
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'storage_shelf_acp_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        copy => [
            qw(error-severity error-type is-enabled port error-id connection-status
                error-text corrective-action address netmask subnet
                )
        ],
        map          => {'node' => 'node-name',},
        api          => "storage-shelf-acp-get",
        primary_keys => [qw( node )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
