# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::StorageShelfAcpFirmware->show_update_status() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::StorageShelfAcpFirmwareShowUpdateStatus

=head1 DESCRIPTION

C<NACL::CS::StorageShelfAcpFirmwareShowUpdateStatus> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::StorageShelfAcpFirmware->show_update_status() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::StorageShelfAcpFirmware->show_update_status() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over


=item C<< update_status >>

Disk Shelf Firmware Update Status possible value(s) are, running,idle
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <storage-shelf-acp-firmware-update-info update-status="$value" />


=item C<< node >>

Node
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <storage-shelf-acp-firmware-update-info node="$value" />


=item C<< in_progress_count >>

Number of Shelves with Earlier Revisions Being Updated
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <storage-shelf-acp-firmware-update-info in-progress-count="$value" />

=back

=cut

package NACL::CS::StorageShelfAcpFirmwareShowUpdateStatus;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'update_status',
    scalar => 'node',
    scalar => 'in_progress_count',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(@opts, _primary_keys => [qw(node)],);
    $Log->exit() if $may_exit;
}

=head1 METHODS

=head2 fetch

    my $StorageShelfAcpFirmwareShowUpdateStatus_state  = NACL::CS::StorageShelfAcpFirmwareShowUpdateStatus->fetch(command_interface => $ci, ...);
	my @StorageShelfAcpFirmwareShowUpdateStatus_states = NACL::CS::StorageShelfAcpFirmwareShowUpdateStatus->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes storage-shelf-acp-firmware-update-info-get-iter API for CMode ZAPI iter.

Invokes storage-shelf-acp-firmware-update-info-get API for CMode ZAPI non-iter.


=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'storage shelf acp firmware show-update-status',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_check_non_iter'
            },
        ],
        exception_text =>
            'No matching StorageShelfAcpFirmwareShowUpdateStatus(s) found',
        frontend =>
            'NACL::CS::StorageShelfAcpFirmwareShowUpdateStatus::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _update_state_objs_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->SUPER::_update_state_objs_cmode_zapi(@args,
        zapi_field_translations => {hyphenate_value => [qw(update-status)],},
    );
    $Log->exit() if $may_exit;
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'storage_shelf_acp_firmware_show_update_status',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(node update-status in-progress-count
                )
        ],
        api => "storage-shelf-acp-firmware-update-info-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        copy => [
            qw(node update-status in-progress-count
                )
        ],
        api          => "storage-shelf-acp-firmware-update-info-get",
        primary_keys => [qw( node )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
