# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::StorageTape->show_errors() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::StorageTapeShowErrors

=head1 DESCRIPTION

C<NACL::CS::StorageTapeShowErrors> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::StorageTape->show_errors() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::StorageTape->show_errors() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< device_name >>

Device Name for Tape Operations

Filled in for CMode CLI

=item C<< serial_number >>

Serial Number

Filled in for CMode CLI

=item C<< wwpn >>

World Wide Port Name

Filled in for CMode CLI

=item C<< node >>

Node possible value(s) are, <nodename>,local

Filled in for CMode CLI

=item C<< description >>

Description

Filled in for CMode CLI

=item C<< device_if_type >>

Device If Type possible value(s) are, unknown,fibre-channel,SAS,pSCSI

Filled in for CMode CLI

=item C<< device_id >>

Device ID

Filled in for CMode CLI

=item C<< initiator >>

Initiator Port

Filled in for CMode CLI

=item C<< alias_name >>

Alias Name

Filled in for CMode CLI

=item C<< wwnn >>

World Wide Node Name

Filled in for CMode CLI

=item C<< error >>

Tape Drive Error Description

Filled in for CMode CLI

=item C<< device_type >>

Device Type

Filled in for CMode CLI

=back

=cut

package NACL::CS::StorageTapeShowErrors;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'device_name',
    scalar => 'serial_number',
    scalar => 'wwpn',
    scalar => 'node',
    scalar => 'description',
    scalar => 'device_if_type',
    scalar => 'device_id',
    scalar => 'initiator',
    scalar => 'alias_name',
    scalar => 'wwnn',
    scalar => 'error',
    scalar => 'device_type',
];

=head1 METHODS

=head2 fetch

    my $StorageTapeShowErrors_state  = NACL::CS::StorageTapeShowErrors->fetch(command_interface => $ci, ...);
	my @StorageTapeShowErrors_states = NACL::CS::StorageTapeShowErrors->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
	show_cmd => 'storage tape show-errors',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching storage tape(s) found',
        frontend       => 'NACL::CS::StorageTapeShowErrors::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'storage_tape_show_errors',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
