# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary CifsSymlinks ComponentState Module
## @author Vikas.P@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SysManager::CifsSymlinks

=head1 DESCRIPTION

C<NACL::CS::SysManager::CifsSymlinks> is a derived class of
L<NACL::CS::ComponentState::SysManager|lib-NACL-CS-ComponentState-SysManager-pm>.  It represents the  
state of an Cifs Symlinks present on the System Manger GUI Screen.  A related class is
L<NACL::C::SysManager::CifsSymlinks|lib-NACL-C-SysManager-CifsSymlinks-pm>, which represents access
to an cifs setup

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the cifs setup 
element are the attributes of the cifs ComponentState.

=over

=item C<< vserver >>

Filled in for CMode.

=item C<< path >>

Filled in for CMode.

=item C<< symlink_unixpath >>

Filled in for CMode.

=item C<< symlink_cifsserver >>

Filled in for CMode.

=item C<< symlink_cifsshare >>

Filled in for CMode.

=item C<< symlink_cifspath >>

Filled in for CMode.

=item C<< symlink_locality >>

Filled in for CMode.

=back

=cut

package NACL::CS::SysManager::CifsSymlinks;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use Params::Validate qw(validate);

use base 'NACL::CS::ComponentState::SysManager';

use NACL::C::SysManager::CifsSymlinks;
use NACL::Exceptions::NoElementsFound qw(:try);

# Import Exception class
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException;

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'path',
    scalar => 'symlink_unixpath',
    scalar => 'symlink_cifsserver',
    scalar => 'symlink_cifsshare',
    scalar => 'symlink_cifspath',
    scalar => 'symlink_locality',
];

=head1 METHODS 

=head2 fetch

    my $cifs_state  = 
		NACL::CS::SysManager::CifsSymlinks->fetch(command_interface=>$ci,...);
    my @cifs_states = 
		NACL::CS::SysManager::CifsSymlinks->fetch(command_interface=>$ci,...);

see L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm>.

Uses a CMode GUI or a 7Mode GUI APISet.

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method => "_fetch_cmode_gui",
                set    => "SysManager_CMode",
            },
        ],
        exception_text => 'CIFS not licensed/Configured',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();

    my $parsed_output_symlink;
    my $response;
    my $caught_exception;

    my $apiset            = $opts{apiset};
    my $requested_fields  = $opts{requested_fields};
    my $filter            = $opts{filter};
    my $command_interface = $opts{command_interface};

    my $cifs_args = $pkg->_hash_copy(
        source => $filter,
        map    => {
            'path'    => '-unixpath',
            "vserver" => "-vservername",
        },
    );

    $pkg->_check_for_required_filter_fields(
        filter          => $filter,
        required_fields => [qw(path vserver)],
    );

    try {

        # caling the gui api to get cifs Symlink status
        $response              = $apiset->get_cifs_symlinks_info(%$cifs_args);
        $parsed_output_symlink = $response->get_parsed_output();
    } ## end try
    catch NACL::APISet::Exceptions::CommandFailedException with {

     # A caught exception indicates that the CifsSymlinks symlink being looked
     # for does not exist. We catch the exception and return immediately.
     # The 'fetch' frontend decides whether to throw a NoElementsFound
     # exception based on the value of 'allow_empty'
        $Log->debug( 'CommandFailedException caught. Requested Symlink does '
                . 'not exist' );
        $caught_exception = 1;
    };
    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }

    my %state_base_field_settings;
    $state_base_field_settings{'vserver'} = $cifs_args->{'-vservername'};
    $state_base_field_settings{'path'}    = $cifs_args->{'-unixpath'};

    my %final_attributes
        = ( %state_base_field_settings, %$parsed_output_symlink );

    my $obj = $pkg->new( command_interface => $command_interface );
    $obj->_set_fields( row => \%final_attributes );
    $Log->exit() if $may_exit;

    return $obj;
} ## end sub _fetch_cmode_gui

1;
