# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary ExportPolicyRule ComponentState Module
## @author vikas.p@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SysManager::ExportPolicyRule

=head1 DESCRIPTION

C<NACL::CS::SysManager::ExportPolicyRule> is a derived class of
L<NACL::CS::ComponentState::SysManager|lib-NACL-CS-ComponentState-SysManager-pm>.  It represents the  
state of an ONTAP ExportPolicyRule.  A related class is
L<NACL::C::SysManager::ExportPolicyRule|lib-NACL-C-SysManager-ExportPolicyRule-pm>, 
which represents access to an ONTAP ExportPolicyRule.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the ExportPolicyRule 
element are the attributes of the ExportPolicyRule ComponentState.

=over

=item C<< policy >>

The name of the Export Policy in which the rule exists.

=item C<< ruleindex >>

The Index of the ExportPolicyRule element whose state is being representend.

=item C<< vserver >>

=item C<< client_specification >>

=item C<< access_protocol >>

=item C<< qtree_secstyle >>

=item C<< read_only_rule >>

=item C<< read_write_rule >>

=item C<< superuser >>

=back

=cut

package NACL::CS::SysManager::ExportPolicyRule;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use Params::Validate qw(validate);

use base 'NACL::CS::ComponentState::SysManager';

use NACL::C::SysManager::ExportPolicyRule;
use NACL::Exceptions::NoElementsFound qw(:try);

# Import Exception class
use NACL::APISet::Exceptions::ResponseException qw(:try);

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'policy',
    scalar => 'ruleindex',
    scalar => 'client_specification',
    scalar => 'access_protocol',
    scalar => 'read_only_rule',
    scalar => 'read_write_rule',
    scalar => 'superuser',
];

=head1 METHODS 

=head2 fetch

    my $export_policy_rule_state  = 
		NACL::CS::SysManager::ExportPolicyRule->fetch(command_interface => $ci,...);
    my @export_policy_rule_states = 
		NACL::CS::SysManager::ExportPolicyRule->fetch(command_interface => $ci,...);

see L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm>.

Uses a CMode GUI APISet.
(Works only when policy, ruleindex and vserver parameters are passed under filter option).

=cut

####################### GUI Fetch ###############################
sub fetch {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method => "_fetch_cmode_gui",
                set    => "SysManager_CMode",
            },
        ],
        exception_text => 'No matching export policy rule found',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

####################### GUI CMode Fetch ###############################
sub _fetch_cmode_gui {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();

    my $apiset            = $opts{apiset};
    my $requested_fields  = $opts{requested_fields};
    my $filter            = $opts{filter};
    my $command_interface = $opts{command_interface};

    my $export_policy_rule_args = $pkg->_hash_copy(
        source => $filter,
        map    => {
            "policy"    => "-policyName",
            "ruleindex" => "-ruleName",
            "vserver"   => "-vservername",
        }
    );

    my ( $response, $caught_exception, $parsed_output );
    try {

        # caling the gui api to get Export policy rule status
        $response
            = $apiset->get_export_policy_rule_info(%$export_policy_rule_args);
        $parsed_output = $response->get_parsed_output();
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {

    # A caught exception indicates that the ExportPolicy Rule being looked for
    # does not exist. We catch the exception and return immediately.
    # The 'fetch' frontend decides whether to throw a NoElementsFound
    # exception based on the value of 'allow_empty'
        $Log->debug(
            'ResponseException caught. Requested Export policy rule does '
                . 'not exist' );
        $caught_exception = 1;
    };
    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }

    my %state_base_field_settings;

    $state_base_field_settings{'policy'}
        = $export_policy_rule_args->{'-policyName'};
    $state_base_field_settings{'ruleindex'}
        = $export_policy_rule_args->{'-ruleName'};
    $state_base_field_settings{'vserver'}
        = $export_policy_rule_args->{'-vservername'};

    my %final_attributes = ( %state_base_field_settings, %$parsed_output );

    my $obj = $pkg->new( command_interface => $command_interface );
    $obj->_set_fields( row => \%final_attributes );
    $Log->exit() if $may_exit;

    return $obj;
} ## end sub _fetch_cmode_gui

1;
