#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemConfigurationBackup ComponentState Module
## @author dl-nacl-dev@netapp.com raghaveb@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemConfigurationBackup

=head1 DESCRIPTION

C<NACL::CS::SystemConfigurationBackup> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemConfigurationBackup.  A related class is
L<NACL::C::SystemConfigurationBackup|lib-NACL-C-SystemConfigurationBackup-pm>, which represents access
to an ONTAP SystemConfigurationBackup.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the SystemConfigurationBackup
element are the attributes of the SystemConfigurationBackup ComponentState.

=over

=item C<< "version" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <config-backup-info-type>
    <version>$value</version>
 <config-backup-info-type>

=item C<< "time" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <config-backup-info-type>
    <backup-creation-time>$value</backup-creation-time>
 <config-backup-info-type>

=item C<< "is_auto" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <config-backup-info-type>
    <is-auto>$value</is-auto>
 <config-backup-info-type>

=item C<< "local" >>

Filled in for CMode CLI.

=item C<< "node" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <config-backup-info-type>
    <node>$value</node>
 <config-backup-info-type>

=item C<< "size" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <config-backup-info-type>
    <backup-size>$value</backup-size>
 <config-backup-info-type>

=item C<< "schedule" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <config-backup-info-type>
    <schedule>$value</schedule>
 <config-backup-info-type>

=item C<< "cluster_name" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <config-backup-info-type>
    <cluster-name>$value</cluster-name>
 <config-backup-info-type>

=item C<< "cluster_uuid" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <config-backup-info-type>
    <cluster-uuid>$value</cluster-uuid>
 <config-backup-info-type>

=item C<< "backup_type" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <config-backup-info-type>
    <backup-type>$value</backup-type>
 <config-backup-info-type>

=item C<< "nodes_in_backup" >>

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
 my $nodes_in_backup = $obj->nodes_in_backup();
 # $nodes_in_backup contains a reference to the array of values
 my @nodes_in_backup = $obj->nodes_in_backup();
 # @nodes_in_backup contains the array of values

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <config-backup-info-type>
    <nodes-in-backup>
        <string>$value</string>
    <nodes-in-backup>
 <config-backup-info-type>


=item C<< "backup" >>
 
Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <config-backup-info-type>
    <backup-name>$value</backup-name>
 <config-backup-info-type>

=back

=cut

package NACL::CS::SystemConfigurationBackup;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'version',
    scalar => 'time',
    scalar => 'is_auto',
    scalar => 'local',
    scalar => 'node',
    scalar => 'size',
    scalar => 'schedule',
    scalar => 'cluster_name',
    scalar => 'cluster_uuid',
    scalar => 'backup_type',
    array  => 'nodes_in_backup',
    scalar => 'backup',
    array  => 'support_address',
];

=head1 METHODS

=head2 fetch

    my $SystemConfigurationBackup_state   = NACL::CS::SystemConfigurationBackup->fetch(command_interface => $ci, ...);
	my @SystemConfigurationBackup_states = NACL::CS::SystemConfigurationBackup->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "config-backup-info-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        show_cmd       => 'system configuration backup show',
        exception_text => 'No matching system configuration backup(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'system_configuration_backup_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    return $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => "config-backup-info-get-iter",
        copy => [
            qw (   backup-type cluster-name cluster-uuid
                is-auto node schedule version nodes-in-backup )
        ],
        map => {
            "nodes-in-backup" => [
                make_zapi_array('nodes-in-backup'), make_zapi_skip('string')
            ],
            'time'   => 'backup-creation-time',
            'backup' => 'backup-name',
            'size'   => 'backup-size',
        },
    );
    $Log->exit() if $may_exit;
} ## end sub _fetch_cmode_zapi

1;
