# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemHealthClusterSwitch ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemHealthClusterSwitch

=head1 DESCRIPTION

C<NACL::CS::SystemHealthClusterSwitch> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemHealthClusterSwitch.  A related class is
L<NACL::C::SystemHealthClusterSwitch|lib-NACL-C-SystemHealthClusterSwitch-pm>, which represents access
to an ONTAP SystemHealthClusterSwitch.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the SystemHealthClusterSwitch
element are the attributes of the SystemHealthClusterSwitch ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< model >>

Model Number, possible value(s) are NX5010,NX5020,CAT2960

Filled in for CMode CLI

=item C<< device >>

Remote SNMP Agent

Filled in for CMode CLI

=item C<< discovered >>

Is Discovered, possible value(s) are true,false

Filled in for CMode CLI

=item C<< community >>

Community String

Filled in for CMode CLI

=item C<< type >>

Switch Network, possible value(s) are cluster,management

Filled in for CMode CLI

=item C<< snmpversion >>

SNMP Version, possible value(s) are SNMPv1,SNMPv2c

Filled in for CMode CLI

=item C<< ipaddress >>

IP Address

Filled in for CMode CLI

=item C<< swversion >>

Software Version

Filled in for CMode CLI

=item C<< sw_version >>

Software Version

Filled in for CMode CLI.

=item C<< snmp_version >>

SNMP Version possible value(s) are, SNMPv1,SNMPv2c

Filled in for CMode CLI.

=item C<< address >>

IP Address

Filled in for CMode CLI.


=item C<< is_discovered >>

Is Discovered possible value(s) are, true,false

Filled in for CMode CLI.

=item C<< reason >>

Reason For Not Monitoring

Filled in for CMode CLI.

=item C<< monitoring_status >>

Is Monitored ? possible value(s) are, yes,no

Filled in for CMode CLI.


=item C<< version_source >>

Source Of Switch Version

Filled in for CMode CLI.


=item C<< node >>

Node possible value(s) are, <nodename>,local

Filled in for CMode CLI.


=item C<< is_monitoring_enabled_operational >>

Is Monitored ? possible value(s) are, true,false

Filled in for CMode CLI.


=back

=cut

package NACL::CS::SystemHealthClusterSwitch;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'model',
    scalar => 'device',
    scalar => 'discovered',
    scalar => 'community',
    scalar => 'type',
    scalar => 'snmpversion',
    scalar => 'ipaddress',
    scalar => 'swversion',
    scalar => 'sw_version',
    scalar => 'snmp_version',
    scalar => 'address',
    scalar => 'is_discovered',
    scalar => 'reason',
    scalar => 'monitoring_status',
    scalar => 'version_source',
    scalar => 'node',
    scalar => 'is_monitoring_enabled_operational',
];

=head1 METHODS

=head2 fetch

    my $SystemHealthClusterSwitch_state  = NACL::CS::SystemHealthClusterSwitch->fetch(command_interface => $ci, ...);
	my @SystemHealthClusterSwitch_states = NACL::CS::SystemHealthClusterSwitch->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'system health cluster-switch show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching system health cluster-switch(s) found',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'system_health_cluster_switch_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
