# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemHealthController ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemHealthController

=head1 DESCRIPTION

C<NACL::CS::SystemHealthController> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemHealthController.  A related class is
L<NACL::C::SystemHealthController|lib-NACL-C-SystemHealthController-pm>, which represents access
to an ONTAP SystemHealthController.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the SystemHealthController
element are the attributes of the SystemHealthController ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< status >>

*Status of the controller, possible value(s) are
Ok,Ok_with_suppressed,degraded,unreachable

Filled in for CMode CLI

=item C<< serial_number >>

*System Serial Number

Filled in for CMode CLI

=item C<< model >>

*System Model

Filled in for CMode CLI

=item C<< node >>

*Node

Filled in for CMode CLI

=item C<< revision >>

*System revision

Filled in for CMode CLI

=item C<< controller_type >>

*Controller Type

Filled in for CMode CLI

=item C<< system_id >>

*System ID

Filled in for CMode CLI

=item C<< part_number >>

*System Part Number

Filled in for CMode CLI

=item C<< chassis_id >>

*Chassis ID

Filled in for CMode CLI

=back

=cut

package NACL::CS::SystemHealthController;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'status',
    scalar => 'serial_number',
    scalar => 'model',
    scalar => 'node',
    scalar => 'revision',
    scalar => 'controller_type',
    scalar => 'system_id',
    scalar => 'part_number',
    scalar => 'chassis_id',
];

=head1 METHODS

=head2 fetch

    my $SystemHealthController_state  = NACL::CS::SystemHealthController->fetch(command_interface => $ci, ...);
	my @SystemHealthController_states = NACL::CS::SystemHealthController->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'system health controller show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching system health controller(s) found',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'system_health_controller_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
