#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemHealthStatus ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemHealthStatus

=head1 DESCRIPTION

C<NACL::CS::SystemHealthStatus> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemHealthStatus.  A related class is
L<NACL::C::SystemHealthStatus|lib-NACL-C-SystemHealthStatus-pm>, which represents access
to an ONTAP SystemHealthStatus.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the SystemHealthStatus
element are the attributes of the SystemHealthStatus ComponentState.

=over

=item C<< "status" >>

=back

=cut

package NACL::CS::SystemHealthStatus;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [scalar => 'status',];

=head1 METHODS

=head2 fetch

    my $SystemHealthStatus_state   = NACL::CS::SystemHealthStatus->fetch(command_interface => $ci, ...);
	my @SystemHealthStatus_states = NACL::CS::SystemHealthStatus->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => "_fetch_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        exception_text => 'No matching system health status(s) found',
        is_singleton   => 1,
        show_cmd       => 'system health status show',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(
        @_,
        api          => 'system_health_status_show',
        is_singleton => 1
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        %opts,
        api          => 'diagnosis-status-get',
        copy         => [qw(status)],
        primary_keys => [qw()]
    );
    $Log->exit() if $may_exit;

    return @state_objs;
} ## end sub _fetch_cmode_zapi
1;
