#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemHealthSubscriptions ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemHealthSubscriptions

=head1 DESCRIPTION

C<NACL::CS::SystemHealthSubscriptions> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemHealthSubscriptions.  A related class is
L<NACL::C::SystemHealthSubscriptions|lib-NACL-C-SystemHealthSubscriptions-pm>, which represents access
to an ONTAP SystemHealthSubscriptions.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the SystemHealthSubscriptions
element are the attributes of the SystemHealthSubscriptions ComponentState.

=over

=item C<< "class_name" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <diagnosis-subscriptions-info>
    <class-name>$value</class-name>
 </diagnosis-subscriptions-info>

=item C<< "monitor" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <diagnosis-subscriptions-info>
    <monitor>$value</monitor>
 </diagnosis-subscriptions-info>

=item C<< "node" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <diagnosis-subscriptions-info>
    <node>$value</node>
 </diagnosis-subscriptions-info>

=item C<< "fail_thresh" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <diagnosis-subscriptions-info>
    <fail-thresh>$value</fail-thresh>
 </diagnosis-subscriptions-info>

=item C<< "instance_name" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <diagnosis-subscriptions-info>
    <instance-name>$value</instance-name>
 </diagnosis-subscriptions-info>

=item C<< "max_notify_period" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <diagnosis-subscriptions-info>
    <max-notify-period>$value</max-notify-period>
 </diagnosis-subscriptions-info>

=item C<< "creation_time" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <diagnosis-subscriptions-info>
    <creation-time>$value</creation-time>
 </diagnosis-subscriptions-info>

=item C<< "subscription_id" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <diagnosis-subscriptions-info>
    <notify-fptr>$value</notify-fptr>
 </diagnosis-subscriptions-info>

=item C<< "notify_fptr" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <diagnosis-subscriptions-info>
    <>$value</>
 </diagnosis-subscriptions-info>

=item C<< "psc_option" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <diagnosis-subscriptions-info>
    <psc-option>$value</psc-option>
 </diagnosis-subscriptions-info>

=item C<< "notify_table" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <diagnosis-subscriptions-info>
    <notify-dest-hm>$value</notify-dest-hm>
 </diagnosis-subscriptions-info>

=item C<< "notify_dest_hm" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <diagnosis-subscriptions-info>
    <>$value</>
 </diagnosis-subscriptions-info>

=item C<< "event_type" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <diagnosis-subscriptions-info>
    <event-type>$value</event-type>
 </diagnosis-subscriptions-info>

=item C<< "notify_type" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <diagnosis-subscriptions-info>
    <notify-type>$value</notify-type>
 </diagnosis-subscriptions-info>

=item C<< "notify_dest_node" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <diagnosis-subscriptions-info>
    <notify-dest-node>$value</notify-dest-node>
 </diagnosis-subscriptions-info>

=item C<< "time_gap_notify" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <diagnosis-subscriptions-info>
    <time-gap-notify>$value</time-gap-notify>
 </diagnosis-subscriptions-info>

=item C<< process >>

Filled in for CMode CLI.

=back

=cut

package NACL::CS::SystemHealthSubscriptions;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'class_name',
    scalar => 'monitor',
    scalar => 'node',
    scalar => 'fail_thresh',
    scalar => 'instance_name',
    scalar => 'max_notify_period',
    scalar => 'creation_time',
    scalar => 'subscription_id',
    scalar => 'notify_fptr',
    scalar => 'psc_option',
    scalar => 'notify_table',
    scalar => 'notify_dest_hm',
    scalar => 'event_type',
    scalar => 'notify_type',
    scalar => 'notify_dest_node',
    scalar => 'time_gap_notify',
    scalar => 'process',
];

=head1 METHODS

=head2 fetch

    my $SystemHealthSubscriptions_state   = NACL::CS::SystemHealthSubscriptions->fetch(command_interface => $ci, ...);
	my @SystemHealthSubscriptions_states = NACL::CS::SystemHealthSubscriptions->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "diagnosis-subscriptions-get-iter" API for CMode ZAPI

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching system health subscriptions(s) found',
        show_cmd       => 'system health subscriptions show',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(
        @_,
        api => 'system_health_subscriptions_show',

    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;
    my $copy = [
        qw(class-name creation-time event-type
            fail-thresh instance-name max-notify-period
            monitor node notify-dest-hm notify-dest-node
            notify-fptr notify-table notify-type psc-option
            subscription-id time-gap-notify)
    ];

    $Log->exit() if $may_exit;
    return $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        api  => "diagnosis-subscriptions-get-iter",
        copy => $copy,
    );

} ## end sub _fetch_cmode_zapi

1;
