# $Id$ //depot/prod/test/nacldev/lib/NACL/CS/SystemManagedFeature.pm#3

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemManagedFeature ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemManagedFeature

=head1 DESCRIPTION

C<NACL::CS::SystemManagedFeature> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemManagedFeature.  A related class is
L<NACL::C::SystemManagedFeature|lib-NACL-C-SystemManagedFeature-pm>, which represents access
to an ONTAP SystemManagedFeature.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the SystemManagedFeature
element are the attributes of the SystemManagedFeature ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< feature_display >>

Filled in for CMode CLI

=item C<< status >>

Managed Feature Status, possible value(s) are on,running,off,not-available

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <managed-feature-status-info>
    <status>$value</status>
 </managed-feature-status-info>

=item C<< notes >>

Managed Feature Notes

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <managed-feature-status-info>
    <notes>$value</notes>
 </managed-feature-status-info>

=item C<< feature_name >>

Managed Feature Name

Filled in for CMode CLI/ZAP.

Maps to:

CM ZAPI:

 <managed-feature-status-info>
    <feature-name>$value</feature-name>
 </managed-feature-status-info>

=back

=cut

package NACL::CS::SystemManagedFeature;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'feature_display',
    scalar => 'status',
    scalar => 'notes',
    scalar => 'feature_name',
];

=head1 METHODS

=head2 fetch

    my $SystemManagedFeature_state  = NACL::CS::SystemManagedFeature->fetch(command_interface => $ci, ...);
	my @SystemManagedFeature_states = NACL::CS::SystemManagedFeature->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI APISet.

Invokes 'feature-status-list-info' API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching system managed-feature(s) found',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'system_managed_feature_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();
    my @state_objs;
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my $filter            = $opts{filter};
    my $response          = $apiset->feature_status_list_info();
    my $output            = $response->get_parsed_output();

    foreach my $list_output (
        @{  $output->[0]->{'managed-feature-status-list'}->[0]
                ->{'managed-feature-status-info'}
        }
        )
    {
        my $target = $pkg->_hash_copy(
            source => $list_output,
            copy   => [qw( feature-name notes status )],
        );
        my $obj = $pkg->new(command_interface => $opts{command_interface});
        $obj->_set_fields(row => $target);
        push @state_objs, $obj;
    } ## end foreach
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
