# $Id$ //depot/prod/test/nacldev/lib/NACL/CS/SystemNodeCoredumpStatus.pm#3 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::SystemNodeCoredump->status() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, gupendra@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemNodeCoredumpStatus

=head1 DESCRIPTION

C<NACL::CS::SystemNodeCoredumpStatus> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::SystemNodeCoredump->status() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::SystemNodeCoredump->status() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< blocks_needed >>

Filled in for CMode CLI.

=item C<< spraycore_supported >>

Filled in for CMode CLI.

=item C<< spraycore_disks >>

Filled in for CMode CLI.

=item C<< node >>

Filled in for CMode CLI.

=item C<< state >>

Filled in for CMode CLI.

=item C<< numdisks >>

Filled in for CMode CLI.

=item C<< corename >>

Filled in for CMode CLI.

=item C<< total_blocks >>

Filled in for CMode CLI.

=item C<< partial_disks >>

Filled in for CMode CLI.

=item C<< unsaved_cores >>

Filled in for CMode CLI.

=item C<< space_available >>

Filled in for CMode CLI.

=item C<< disks_needed >>

Filled in for CMode CLI.

=item C<< spares_available >>

Filled in for CMode CLI.

=item C<< disks_used >>

Filled in for CMode CLI.

=item C<< blocks_saved >>

Filled in for CMode CLI.

=item C<< partial_cores >>

Filled in for CMode CLI.

=item C<< spraycore_blocks >>

Filled in for CMode CLI.

=item C<< type >>

Filled in for CMode CLI.

=item C<< space_needed >>

Filled in for CMode CLI.

=item C<< min_free >>

Filled in for CMode CLI.

=back

=cut

package NACL::CS::SystemNodeCoredumpStatus;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'blocks_needed',
    scalar => 'spraycore_supported',
    scalar => 'spraycore_disks',
    scalar => 'node',
    scalar => 'state',
    scalar => 'numdisks',
    scalar => 'corename',
    scalar => 'total_blocks',
    scalar => 'partial_disks',
    scalar => 'unsaved_cores',
    scalar => 'space_available',
    scalar => 'disks_needed',
    scalar => 'spares_available',
    scalar => 'disks_used',
    scalar => 'blocks_saved',
    scalar => 'partial_cores',
    scalar => 'spraycore_blocks',
    scalar => 'type',
    scalar => 'space_needed',
    scalar => 'min_free',
];

=head1 METHODS

=head2 fetch

    my $SystemNodeCoredumpStatus_state  = NACL::CS::SystemNodeCoredumpStatus->fetch(command_interface => $ci, ...);
    my @SystemNodeCoredumpStatus_states = NACL::CS::SystemNodeCoredumpStatus->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode  and 7Mode CLI APISet.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut


sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'system node coredump status',
	choices => [
            {
                method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {
                method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode'
            }
        ],
        exception_text => 'No matching system node coredump(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs =
      $pkg->SUPER::_fetch_cmode_cli( @_, api => 'system_node_coredump_status',
      );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_7mode_cli {
    my $pkg = shift;
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();
    my %api_opts;
    my @state_objs;

    my $apiset            = delete $opts{apiset};
    my $command_interface = delete $opts{command_interface};

    my %copy_filter           = %{ $opts{filter} };
    my @copy_requested_fields = @{ $opts{requested_fields} };
    $pkg->_remove_relational_regex_filters(
        filter           => \%copy_filter,
        requested_fields => \@copy_requested_fields
    );

    my $req_field_filter;
    $req_field_filter->{'requested_fields'} = \@copy_requested_fields;
    $req_field_filter->{'filter'}           = \%copy_filter;

    if ($pkg->_want_any_field_of(
            %{$req_field_filter},
            fields_filled_by_api => [qw(node total_blocks state corename blocks_saved)] ) )
    {
        my $savecore_progress_resp   = $apiset->savecore( 'display-progress' => "1" );
        my $output = $savecore_progress_resp->get_parsed_output();
        foreach my $list_output ( @{$output} ) {
            my $target = $pkg->_hash_copy(
                source => $list_output,
                copy   => [qw(corename coreid node total_blocks state )],
                map    => {
                    'saved_blocks' => "blocks_saved",
                    'core_name'    => "corename",
                },
            );
            my $obj = $pkg->new( command_interface => $command_interface, );
            $obj->_set_fields( row => $target );
            push @state_objs, $obj;
        } ## end foreach my $list_output (@$output)
    }
    if ($pkg->_want_any_field_of(
            %{$req_field_filter},
            fields_filled_by_api => [qw(type spraycore-blocks spraycore-disks)] ) )
    {
        my $savecore_info_resp   = $apiset->savecore( 'display-info' => "1" );
        my $output               = $savecore_info_resp->get_parsed_output();
        foreach my $list_output ( @{$output} ) {
            my $target = $pkg->_hash_copy(
                source => $list_output,
                copy   => [qw(type)],
                map    => {
                    spray_dumpcore_blocks => 'spraycore_blocks',
                    spray_dumpcore_disks  => 'spraycore_disks' ,
                }
            );
            my $obj = $pkg->new( command_interface => $command_interface, );
            $obj->_set_fields( row => $target );
            push @state_objs, $obj;
        } ## end foreach my $list_output (@$output)
    }
    return @state_objs;
} ## end sub _fetch_7mode_cli

1;
