# Copyright (c) 2015 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
#
## @summary Node Hammer Status CS Module
## @author rishis@netapp.com,ng-svat@netapp.com,ng-wafl-replica-qa@netapp.com
## @status shared

=head1 NAME

NACL::CS::SystemNodeHammerStatus

=head1 DESCRIPTION

C<NACL::CS::SystemNodeHammerStatus> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>.
It represents the state of hammer(s) running on a Node.

=head1 ATTRIBUTES

=over

=item C<< size_written >>

Size of data written on file

Filled in for 7Mode Nodescope.

=item C<< iterations_done >>

Iterations of data write completed

Filled in for 7Mode Nodescope.

=item C<< file_name >>

File name for hammer

Filled in for 7Mode Nodescope.

=item C<< status >>

Status for hammer

Filled in for 7Mode Nodescope.

=item C<< total_iterations>>

Total iterations to be run by hammer

Filled in for 7Mode Nodescope.

=item C<< write_size >>

Write size for each iteration

Filled in for 7Mode Nodescope.

=item C<< total_size >>

Total size to be written

Filled in for 7Mode Nodescope.

=item C<< hammer_id >>

Id of hammer instance

Filled in for 7Mode Nodescope.

=cut

package NACL::CS::SystemNodeHammerStatus;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base 'NACL::CS::ComponentState::ONTAP';
use Params::Validate qw(validate);
use Data::Dumper;
use NATE::Log qw(log_global);

use Class::MethodMaker [
    scalar => 'size_written',
    scalar => 'iterations_done',
    scalar => 'file_name',
    scalar => 'status',
    scalar => 'total_iterations',
    scalar => 'write_size',
    scalar => 'total_size',
    scalar => 'hammer_id',
];

=head1 METHODS

=head2 fetch

    my $state = NACL::CS::SystemNodeHammerStatus->fetch(command_interface => $ci,);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Supports 7Mode CLI

=over

=cut

__PACKAGE__->_fetch_7mode_cli_method_builder(apiset_method => 'hammer_status',
);

1;
