# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemOntapiLimits ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemOntapiLimits

=head1 DESCRIPTION

C<NACL::CS::SystemOntapiLimits> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemOntapiLimits.  A related class is
L<NACL::C::SystemOntapiLimits|lib-NACL-C-SystemOntapiLimits-pm>, which represents access
to an ONTAP SystemOntapiLimits.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the SystemOntapiLimits
element are the attributes of the SystemOntapiLimits ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< add_sleeper_to_list >>

Filled in for CMode CLI/ZAPI.

=item C<< pretend_promoted >>

Filled in for CMode CLI.

=item C<< monitor_mode >>

Filled in for CMode CLI/ZAPI.

=item C<< max_reply_size >>

Filled in for CMode CLI/ZAPI.

=item C<< max_run_time >>

Filled in for CMode CLI/ZAPI.

=item C<< simulate_promoted >>

Filled in for CMode CLI/ZAPI.

=item C<< add_sleeper_to_exceptions >>

Filled in for CMode CLI/ZAPI.


=back

=cut

package NACL::CS::SystemOntapiLimits;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'add_sleeper_to_list',
    scalar => 'pretend_promoted',
    scalar => 'monitor_mode',
    scalar => 'max_reply_size',
    scalar => 'max_run_time',
    scalar => 'simulate_promoted',
    scalar => 'add_sleeper_to_exceptions',
];

=head1 METHODS

=head2 fetch

    my $SystemOntapiLimits_state  = NACL::CS::SystemOntapiLimits->fetch(command_interface => $ci, ...);
(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. This being a "singleton" table (i.e. one which can only have one
row) is callable only in scalar context.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI/ZAPI APISet.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching system ontapi limits(s) found',
        is_singleton   => 1
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(
        @_,
        api          => 'system_ontapi_limits_show',
        is_singleton => 1,
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg    = shift;
    my %opts   = validate @_, $pkg->_fetch_backend_validate_spec();
    my $apiset = $opts{apiset};
    my @state_objs;
    my $system_ontap_limits = $apiset->system_ontapi_limits_get();
    my $cs_parsed_output    = $system_ontap_limits->get_parsed_output();
    foreach my $row (@{$cs_parsed_output}) {
        my $state_fields = $pkg->_zapi_hash_copy(
            source => $row,
            copy   => [
                qw( simulate-promoted  monitor-mode  max-run-time
                    max-reply-size add-sleeper-to-exceptions)
            ],
        );
        my $obj = $pkg->new(command_interface => $opts{command_interface},);
        $obj->_set_fields(row => $state_fields);
        push @state_objs, $obj;
    }
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi
1;
