# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemServiceProcessor ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemServiceProcessor

=head1 DESCRIPTION

C<NACL::CS::SystemServiceProcessor> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemServiceProcessor.  A related class is
L<NACL::C::SystemServiceProcessor|lib-NACL-C-SystemServiceProcessor-pm>, which represents access
to an ONTAP SystemServiceProcessor.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the SystemServiceProcessor
element are the attributes of the SystemServiceProcessor ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< ip_configured >>

Is Network Configured possible value(s) are, true,false
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-info is-ip-configured="$value" />

Mapping for CMode ZAPI non-iter: is-ip-configured


=item C<< new_fw_available >>

Is New Firmware Available possible value(s) are, true,false
Filled in for CMode CLI/ZAPI iter, non-iter.

=item C<< status >>


Status possible value(s) are, online,offline,sp-daemon-offline,node-offline,degraded,rebooting,unknown
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-info status="$value" />


=item C<< fw_version >>

Firmware Version
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-info firmware-version="$value" />

Mapping for CMode ZAPI non-iter: firmware-version


=item C<< ipmi_ver >>

IPMI Version
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-info ipmi-version="$value" />

Mapping for CMode ZAPI non-iter: ipmi-version


=item C<< serial_num >>

Serial Number
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-info serial-num="$value" />


=item C<< node >>

Node possible value(s) are, <nodename>,local
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-info node="$value" />


=item C<< autoupdate_enabled >>

Is Firmware Autoupdate Enabled possible value(s) are, true,false
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-info is-autoupdate-enabled="$value" />

Mapping for CMode ZAPI non-iter: is-autoupdate-enabled


=item C<< new_fw_version >>

New Firmware Version
Filled in for CMode CLI/ZAPI iter, non-iter.

=item C<< is_configured >>

Is the Service Processor Configured possible value(s) are, true,false
Filled in for CMode CLI/ZAPI iter, non-iter.

=item C<< part_num >>

Part Number
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-info part-num="$value" />


=item C<< address >>

Public IP Address
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $address = $obj->address();
 
# $address contains a reference to the array of values
 
my @address = $obj->address();
 
# @address contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { address = [ value1, value2...] }

Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-info>
  <ip-address ip-address="@values" />
</service-processor-info>

Mapping for CMode ZAPI non-iter: <ip-address ip-address="@values" />


=item C<< type >>

Type of Device possible value(s) are, RLM,SP,NONE
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-info type="$value" />


=item C<< mac >>

MAC Address
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-info mac-address="$value" />

Mapping for CMode ZAPI non-iter: mac-address


=item C<< dev_rev >>

Device Revision
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-info device-revision="$value" />

Mapping for CMode ZAPI non-iter: device-revision

=back

=cut

package NACL::CS::SystemServiceProcessor;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'ip_configured',
    scalar => 'new_fw_available',
    scalar => 'status',
    scalar => 'fw_version',
    scalar => 'ipmi_ver',
    scalar => 'serial_num',
    scalar => 'node',
    scalar => 'autoupdate_enabled',
    scalar => 'new_fw_version',
    scalar => 'is_configured',
    scalar => 'part_num',
    array  => 'address',
    scalar => 'type',
    scalar => 'mac',
    scalar => 'dev_rev',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [qw(node)],
        _vs_context   => 0
    );
    $Log->exit() if $may_exit;
}

=head1 METHODS

=head2 fetch

    my $SystemServiceProcessor_state  = NACL::CS::SystemServiceProcessor->fetch(command_interface => $ci, ...);
	my @SystemServiceProcessor_states = NACL::CS::SystemServiceProcessor->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes service-processor-get-iter API for CMode ZAPI iter.

Invokes service-processor-get API for CMode ZAPI non-iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'system service-processor show',
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_check_non_iter'
            },
        ],
        exception_text => 'No matching system service-processor(s) found',
        frontend       => 'NACL::CS::SystemServiceProcessor::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'system_service_processor_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(status serial-num node part-num type
                )
        ],
        map => {
            'ip-configured'      => 'is-ip-configured',
            'autoupdate-enabled' => 'is-autoupdate-enabled',
            'address'            => [
                make_zapi_array('ip-address'), make_zapi_skip('ip-address'),
            ],
            'mac'        => 'mac-address',
            'ipmi-ver'   => 'ipmi-version',
            'fw-version' => 'firmware-version',
            'dev-rev'    => 'device-revision',
        },
        api => "service-processor-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        copy => [
            qw(status serial-num node part-num type
                )
        ],
        map => {
            'ip-configured'      => 'is-ip-configured',
            'autoupdate-enabled' => 'is-autoupdate-enabled',
            'address'            => [
                make_zapi_array('ip-address'), make_zapi_skip('ip-address'),
            ],
            'mac'        => 'mac-address',
            'ipmi-ver'   => 'ipmi-version',
            'fw-version' => 'firmware-version',
            'dev-rev'    => 'device-revision',
        },
        api          => "service-processor-get",
        primary_keys => [qw( node )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _update_state_objs_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->SUPER::_update_state_objs_cmode_zapi(
        %opts,
        zapi_field_translations => {
            uc              => [qw(type)],
            hyphenate_value => [qw(status)],
        },
    );
    $Log->exit() if $may_exit;

}


1;
