# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemServiceProcessorImageUpdateProgress ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemServiceProcessorImageUpdateProgress

=head1 DESCRIPTION

C<NACL::CS::SystemServiceProcessorImageUpdateProgress> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemServiceProcessorImageUpdateProgress.  A related class is
L<NACL::C::SystemServiceProcessorImageUpdateProgress|lib-NACL-C-SystemServiceProcessorImageUpdateProgress-pm>, which represents access
to an ONTAP SystemServiceProcessorImageUpdateProgress.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the SystemServiceProcessorImageUpdateProgress
element are the attributes of the SystemServiceProcessorImageUpdateProgress ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< start_time >>

Latest SP Firmware Update Start Timestamp
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< end_time >>

Latest SP Firmware Update End Timestamp
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< in_progress >>

Is Update in Progress possible value(s) are, yes,no
Filled in for CMode CLI/ZAPI, non-iter.
Mapping for CMode ZAPI non-iter: is-in-progress


=item C<< percent_done >>

Latest SP Firmware Update Percentage Done ranges from 0 to 100
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< node >>

Node possible value(s) are, <nodename>,local
Filled in for CMode CLI/ZAPI, non-iter.
=back

=cut

package NACL::CS::SystemServiceProcessorImageUpdateProgress;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'start_time',
    scalar => 'end_time',
    scalar => 'in_progress',
    scalar => 'percent_done',
    scalar => 'node',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $SystemServiceProcessorImageUpdateProgress_state  = NACL::CS::SystemServiceProcessorImageUpdateProgress->fetch(command_interface => $ci, ...);
	my @SystemServiceProcessorImageUpdateProgress_states = NACL::CS::SystemServiceProcessorImageUpdateProgress->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes service-processor-image-update-progress-get API for CMode ZAPI non-iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'system service-processor image update-progress show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        exception_text =>
            'No matching system service-processor image update-progress(s) found',
        frontend =>
            'NACL::CS::SystemServiceProcessorImageUpdateProgress::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'system_service_processor_image_update_progress_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        copy => [
            qw(percent-done node start-time end-time
                )
        ],
        map => {'in-progress' => 'is-in-progress',},
        api          => "service-processor-image-update-progress-get",
        primary_keys => [qw( node )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _update_state_objs_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->SUPER::_update_state_objs_cmode_zapi(
        @args,
        zapi_field_translations => {
            yes_no_boolean        => [qw(in-progress)],
            timestamp_to_mmddyyyy => [qw(start-time end-time)],
        },
    );
    $Log->exit() if $may_exit;
}

1;
