# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemServiceProcessorNetwork ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemServiceProcessorNetwork

=head1 DESCRIPTION

C<NACL::CS::SystemServiceProcessorNetwork> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemServiceProcessorNetwork.  A related class is
L<NACL::C::SystemServiceProcessorNetwork|lib-NACL-C-SystemServiceProcessorNetwork-pm>, which represents access
to an ONTAP SystemServiceProcessorNetwork.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the SystemServiceProcessorNetwork
element are the attributes of the SystemServiceProcessorNetwork ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< link_status >>

Link Status possible value(s) are, up,down,disabled,unknown
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-network-info link-status="$value" />


=item C<< status >>


Status possible value(s) are, online,offline,sp-daemon-offline,node-offline,degraded,rebooting,unknown
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-network-info status="$value" />


=item C<< router_ip >>

Router Assigned IP Address
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-network-info router-ip-address="$value" />

Mapping for CMode ZAPI non-iter: router-ip-address


=item C<< gateway >>

Gateway IP Address
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-network-info gateway-ip-address="$value" />

Mapping for CMode ZAPI non-iter: gateway-ip-address


=item C<< prefix_length >>

Prefix Length of Subnet Mask ranges from 0 to 64
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-network-info prefix-length="$value" />


=item C<< time_last_updated >>

Time Last Updated
Filled in for CMode CLI/ZAPI iter, non-iter.

=item C<< address_family >>

Address Family possible value(s) are, IPv4,IPv6
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-network-info address-type="$value" />

Mapping for CMode ZAPI non-iter: address-type


=item C<< node >>

Node possible value(s) are, <nodename>,local
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-network-info node="$value" />


=item C<< link_local_ip >>

Link Local IP Address
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-network-info link-local-ip-address="$value" />

Mapping for CMode ZAPI non-iter: link-local-ip-address


=item C<< ip_address >>

IP Address
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-network-info ip-address="$value" />


=item C<< netmask >>

Netmask
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-network-info netmask="$value" />


=item C<< mac >>

MAC Address
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-network-info mac-address="$value" />

Mapping for CMode ZAPI non-iter: mac-address


=item C<< setup_failure_reason >>


SP Network Setup Failure Reason possible value(s) are, success,subnet-out-of-address,invalid-subnet,other-error
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-network-info setup-failure-reason="$value" />


=item C<< setup_status >>


SP Network Setup Status possible value(s) are, not-setup,succeeded,in-progress,failed
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-network-info setup-status="$value" />


=item C<< enable >>

Interface Enabled possible value(s) are, true,false
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-network-info is-enabled="$value" />

Mapping for CMode ZAPI non-iter: is-enabled


=item C<< gateway_mac >>

MAC Address of the Gateway
Filled in for CMode CLI/ZAPI iter, non-iter.

=item C<< is_ipv6_ra_enabled >>

Enable IPv6 Router Assigned Address possible value(s) are, true,false
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-network-info is-ipv6-ra-enabled="$value" />


=item C<< subnet_name >>

Subnet Name
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-network-info subnet-name="$value" />


=item C<< arp_interval >>

Gratuitous ARP Interval ranges from 0 to 60
Filled in for CMode CLI/ZAPI iter, non-iter.

=item C<< type >>

Type of Device possible value(s) are, RLM,SP,NONE
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-network-info device-type="$value" />

Mapping for CMode ZAPI non-iter: device-type


=item C<< dhcp >>

DHCP Status possible value(s) are, v4,none
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <service-processor-network-info dhcp="$value" />


=item C<< role >>


Role of the Ethernet Port possible value(s) are, public,private,ACP,public-IPv6
Filled in for CMode CLI/ZAPI iter, non-iter.
=back

=cut

package NACL::CS::SystemServiceProcessorNetwork;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'link_status',
    scalar => 'status',
    scalar => 'router_ip',
    scalar => 'gateway',
    scalar => 'prefix_length',
    scalar => 'time_last_updated',
    scalar => 'address_family',
    scalar => 'address_type',
    scalar => 'node',
    scalar => 'link_local_ip',
    scalar => 'ip_address',
    scalar => 'netmask',
    scalar => 'mac',
    scalar => 'setup_failure_reason',
    scalar => 'setup_status',
    scalar => 'enable',
    scalar => 'gateway_mac',
    scalar => 'is_ipv6_ra_enabled',
    scalar => 'subnet_name',
    scalar => 'arp_interval',
    scalar => 'type',
    scalar => 'dhcp',
    scalar => 'role',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [qw(address-family node)],
        _vs_context   => 0
    );
    $Log->exit() if $may_exit;
}

=head1 METHODS

=head2 fetch

    my $SystemServiceProcessorNetwork_state  = NACL::CS::SystemServiceProcessorNetwork->fetch(command_interface => $ci, ...);
	my @SystemServiceProcessorNetwork_states = NACL::CS::SystemServiceProcessorNetwork->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes service-processor-network-get-iter API for CMode ZAPI iter.

Invokes service-processor-network-get API for CMode ZAPI non-iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'system service-processor network show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_check_non_iter'
            },
        ],
        exception_text =>
            'No matching system service-processor network(s) found',
        frontend => 'NACL::CS::SystemServiceProcessorNetwork::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'system_service_processor_network_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(link-status status prefix-length node netmask ip-address
                setup-failure-reason setup-status is-ipv6-ra-enabled subnet-name dhcp
                address-type)
        ],
        map => {
            'gateway'        => 'gateway-ip-address',
            'router-ip'      => 'router-ip-address',
            'address-family' => 'address-type',
            'link-local-ip'  => 'link-local-ip-address',
            'mac'            => 'mac-address',
            'enable'         => 'is-enabled',
            'type'           => 'device-type',
        },
        api => "service-processor-network-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        copy => [
            qw(link-status status prefix-length node netmask ip-address
                setup-failure-reason setup-status is-ipv6-ra-enabled subnet-name dhcp
                address-type)
        ],
        map => {
            'gateway'        => 'gateway-ip-address',
            'router-ip'      => 'router-ip-address',
            'address-family' => 'address-type',
            'link-local-ip'  => 'link-local-ip-address',
            'mac'            => 'mac-address',
            'enable'         => 'is-enabled',
            'type'           => 'device-type',
        },
        api          => "service-processor-network-get",
        primary_keys => [qw( address-family node )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
