#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeAutosize ComponentState Module
## @author dl-nacl-dev@netapp.com , anupamap@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VolumeAutosize

=head1 DESCRIPTION

This module does not represent the state of any element, but is an object
representation of the output of autosize

=head1 ATTRIBUTES

The individual pieces of data that are part of the VolumeAutosize
element are the attributes of the VolumeAutosize ComponentState.

=over

=item C<< volume >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Output:

 <volume-autosize-get>
    <volume>$value</volume>
 </volume-autosize-get>

7M CLI : volume (Programmatic name: "volume")

=item C<< state >>

Filled in for CMode CLI, 7Mode CLI.

7M CLI : state (Programmatic name: "state")

=item C<< maximum_size >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Output:

 <volume-autosize-get>
    <maximum-size>$value</maximum-size>
 </volume-autosize-get>

7M CLI : -m <value> (Programmatic name: "maximum-size")

=item C<< increment_size >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Output:

 <volume-autosize-get>
    <increment-size>$value</increment-size>
 </volume-autosize-get>

7M CLI : -i <value> (Programmatic name: "increment-size")

=item C<< grow_threshold_percent  >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Output:

 <volume-autosize-get>
    <grow-threshold-percent>$value</grow-threshold-percent>
 </volume-autosize-get>

7M CLI : -grow-threshold-percent <value> (Programmatic name: "grow-threshold-percent")

=item C<< minimum_size  >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Output:

 <volume-autosize-get>
    <minimum-size>$value</minimum-size>
 </volume-autosize-get>

7M CLI : -minimum-size <value> (Programmatic name: "minimum-size")

=item C<< shrink_threshold_percent >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

Output:

 <volume-autosize-get>
    <shrink-threshold-percent>$value</shrink-threshold-percent>
 </volume-autosize-get>

7M CLI : -shrink-threshold-percent <value> (Programmatic name: "shrink-threshold-percent")

=item C<< mode >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI:

Output:

 <volume-autosize-get>
    <mode>$value</mode>
 </volume-autosize-get>

7M CLI : autosize-mode (Programmatic name: "autosize-mode")

=item C<< vserver >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

output:

 <volume-autosize-get>
    <vserver>$value</vserver>
 </volume-autosize-get>

=back

=cut

package NACL::CS::VolumeAutosize;

use strict;
use warnings;
use base qw(NACL::CS::ComponentState::ONTAP);
use Class::MethodMaker [
    scalar => 'volume',
    scalar => 'vserver',
    scalar => 'state',
    scalar => 'maximum_size',
    scalar => 'increment_size',
    scalar => 'grow_threshold_percent',
    scalar => 'minimum_size',
    scalar => 'shrink_threshold_percent',
    scalar => 'mode',
];

1;
