
#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::VolumeFileCloneSplit->status() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VolumeFileCloneSplitStatus

=head1 DESCRIPTION

C<NACL::CS::VolumeFileCloneSplitStatus> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::VolumeFileCloneSplit->status() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::VolumeFileCloneSplit->status() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< unsplit_size >>

Unsplit Clone Size

Filled in for CMode CLI

=item C<< volume >>

Volume Name

Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI:

<clone-split-status>
  <volume-name>$value</volume-name>
  </clone-split-status>

=item C<< vserver >>

Vserver Name

Filled in for CMode CLI/ZAPI

=item C<< pending_clone_splits >>

Number of Pending Clone Splits

Filled in for CMode CLI

=back

=cut

package NACL::CS::VolumeFileCloneSplitStatus;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'unsplit_size',
    scalar => 'volume',
    scalar => 'vserver',
    scalar => 'pending_clone_splits',
];

=head1 METHODS

=head2 fetch

    my $VolumeFileCloneSplitStatus_state  = NACL::CS::VolumeFileCloneSplitStatus->fetch(command_interface => $ci, ...);
	my @VolumeFileCloneSplitStatus_states = NACL::CS::VolumeFileCloneSplitStatus->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes API 'clone-split-status' for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching volume file clone split(s) found',
        frontend       => 'NACL::CS::VolumeFileCloneSplitStatus::fetch',
        show_cmd       => 'volume file clone split status',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'volume_file_clone_split_status',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();
    my @state_objs;
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my $vserver           = $opts{filter}->{vserver};
    my %filter = %{$opts{filter}};
    my %vol;
    $vol{'volume-name'}       =  $opts{filter}->{volume};

    #Need to execute in vserver context
    my %cmode_args;
    $pkg->_handle_zapi_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $vserver,
        command_interface => $command_interface
    );
    my $response = $apiset->clone_split_status(%cmode_args, %vol);
    my $output   = $response->get_parsed_output();
    my $row = $output->[0]->{'clone-split-info'};
    my $target = {};

    foreach my $status (@$row) {
        $pkg->_hash_copy(
            source => $status,
            copy => [qw( unsplit-size )],
            map => {"volume" => "volume-name"},
            target => $target,
        );
        my $obj = $pkg->new( command_interface => $command_interface, );
        $target->{'volume'} = $opts{filter}->{volume};
        $obj->_set_fields(row => $target);
        push @state_objs, $obj;
    }
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
