# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeFileFingerprint ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VolumeFileFingerprint

=head1 DESCRIPTION

C<NACL::CS::VolumeFileFingerprint> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VolumeFileFingerprint.  A related class is
L<NACL::C::VolumeFileFingerprint|lib-NACL-C-VolumeFileFingerprint-pm>, which represents access
to an ONTAP VolumeFileFingerprint.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VolumeFileFingerprint
element are the attributes of the VolumeFileFingerprint ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< volume >>

Volume Name
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <fingerprint-status-info volume="$value" />


=item C<< progress_percentage >>

Progress Percentage
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <fingerprint-status-info progress-percentage="$value" />


=item C<< vserver >>

Vserver
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <fingerprint-status-info vserver="$value" />


=item C<< file >>

File Path
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <fingerprint-status-info file="$value" />


=item C<< operation_status >>


Operation Status possible value(s) are, Unknown,In-Progress,Failed,Aborting,Completed
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <fingerprint-status-info operation-status="$value" />


=item C<< session_id >>

Session ID of fingerprint operation
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <fingerprint-status-info session-id="$value" />

=back

=cut

package NACL::CS::VolumeFileFingerprint;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'volume',
    scalar => 'progress_percentage',
    scalar => 'vserver',
    scalar => 'file',
    scalar => 'operation_status',
    scalar => 'session_id',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $VolumeFileFingerprint_state  = NACL::CS::VolumeFileFingerprint->fetch(command_interface => $ci, ...);
	my @VolumeFileFingerprint_states = NACL::CS::VolumeFileFingerprint->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes file-fingerprint-get-iter API for CMode ZAPI iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'volume file fingerprint show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },

        ],
        exception_text => 'No matching VolumeFileFingerprint(s) found',
        frontend       => 'NACL::CS::VolumeFileFingerprint::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'volume_file_fingerprint_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(progress-percentage volume vserver operation-status file session-id
                )
        ],
        api => "file-fingerprint-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _update_state_objs_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->SUPER::_update_state_objs_cmode_zapi(
        @opts,
        hash_map_translations => {
            'op_status' => {
                fields_to_translate => ['operation-status'],
                hash_map => {
                    unknown     => 'Unknown',
                    in_progress => 'In-Progress',
                    failed      => 'Failed',
                    aborting    => 'Aborting',
                    completed   => 'Completed'
                },
            },
        }
    );
    $Log->exit() if $may_exit;
}

1;
