#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::VolumeFile->show_inode() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VolumeFileShowInode

=head1 DESCRIPTION

C<NACL::CS::VolumeFileShowInode> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::VolumeFile->show_inode() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::VolumeFile->show_inode() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< file_name >>

File Name

Filled in for CMode CLI/ZAPI

=item C<< file_path >>

File Path

Filled in for CMode CLI/ZAPI

=item C<< inode_number >>

Inode Number

Filled in for CMode CLI/ZAPI

=item C<< volume >>

Volume Name

Filled in for CMode CLI/ZAPI

=item C<< snapshot_name >>

Snapshot Name

Filled in for CMode CLI/ZAPI

=item C<< vserver >>

Vserver Name

Filled in for CMode CLI

=item C<< parent_inode_number >>

Parent Inode Number

Filled in for CMode CLI/ZAPI

=item C<< snapshot_id >>

Physical Snapshot ID

Filled in for CMode CLI/ZAPI

=item C<< parent_dir_cookie >>

Parent Directory Cookie

Maps to:

 CMode ZAPI:
  <file-inode-info-get-iter>
   <parent-dir-index>$value</parent-dir-index>
  </file-inode-info-get-iter>

Filled in for CMode CLI/ZAPI

=item C<< dsid >>

Volume DSID

Filled in for CMode CLI.

=item C<< index >>

Index

Filled in for CMode CLI.


=back

=cut

package NACL::CS::VolumeFileShowInode;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'file_name',
    scalar => 'file_path',
    scalar => 'inode_number',
    scalar => 'volume',
    scalar => 'snapshot_name',
    scalar => 'vserver',
    scalar => 'parent_inode_number',
    scalar => 'snapshot_id',
    scalar => 'parent_dir_cookie',
    scalar => 'dsid',
    scalar => 'index',
];

=head1 METHODS

=head2 fetch

    my $VolumeFileShowInode_state  = NACL::CS::VolumeFileShowInode->fetch(command_interface => $ci, ...);
	my @VolumeFileShowInode_states = NACL::CS::VolumeFileShowInode->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI

Invokes "file-inode-info-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
	show_cmd       => 'volume file show-inode',
        exception_text => 'No matching volume file(s) found',
        frontend       => 'NACL::CS::VolumeFileShowInode::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'volume_file_show_inode',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();

    #Need to execute in vserver context
    my %pass_through_opts;
    my $vserver = $pkg->_handle_zapi_vserver_context(
        api_opts          => \%pass_through_opts,
        vserver           => $opts{filter}->{vserver},
        command_interface => $opts{command_interface},
        frontend_method   => 'fetch',
    );

    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        api               => 'file-inode-info-get-iter',
        pass_through_opts => \%pass_through_opts,
        copy              => [
            qw (file-path file-name inode-number
                parent-inode-number snapshot-id snapshot-name volume )
        ],
        map => {'parent-dir-cookie' => 'parent-dir-index',}
    );
    $Log->exit() if $may_exit;

    return @state_objs;

}

1;
