#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
#
## @summary VolumeFileWaflGetChildAndNextFbn Component Module
## @author onkarb@netapp.com,asmitak@netapp.com,kvishal@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VolumeFileWaflGetChildAndNextFbn

=head1 DESCRIPTION

C<NACL::CS::VolumeFileWaflGetChildAndNextFbn> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>.
It represents the child id, next fbn and is_eob(end of buffer status) for a given fbn and level.

=head1 ATTRIBUTES

=over

=item C<< child_id >>

This is VVBN for WAFL and EID for MetaWAFL

Filled in for 7Mode Nodescope.

=item C<< next_fbn >>

This is the fbn at the next index in the indirect buffer

Filled in for 7Mode Nodescope

=item C<< is_eob >>

This is true when we have reached the end of the indirect buffer
and false otherwise.

=cut

package NACL::CS::VolumeFileWaflGetChildAndNextFbn;

use strict;
use warnings;

use Tharn qw($Log);
use Params::Validate qw(validate);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'path',
    scalar => 'fbn',
    scalar => 'level',
    scalar => 'child_id',
    scalar => 'next_fbn',
    scalar => 'is_eob',
];

=head1 METHODS

=head2 fetch

    my $state = NACL::CS::VolumeFileWaflGetChildAndNextFbn->fetch(
        command_interface => $ci,
        filter => { 'path' => file_path,
                    'fbn'      => fbn,
                    'level'    => level,
        },
    );

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Supports 7Mode CLI

=over

=cut

__PACKAGE__->_fetch_7mode_cli_method_builder(
    apiset_method => 'wafl_get_child_and_next_fbn',
    filter_keys_to_be_passed_to_apiset => ['path', 'fbn', 'level'],
);
1;

