# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeQtreePolicy ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VolumeQtreePolicy

=head1 DESCRIPTION

C<NACL::CS::VolumeQtreePolicy> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VolumeQtreePolicy.  A related class is
L<NACL::C::VolumeQtreePolicy|lib-NACL-C-VolumeQtreePolicy-pm>, which represents access
to an ONTAP VolumeQtreePolicy.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VolumeQtreePolicy
element are the attributes of the VolumeQtreePolicy ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< policyname >>

Policy Name

Filled in for CMode CLI

=item C<< volume >>

Volume Name

Filled in for CMode CLI

=item C<< vserver >>

Vserver Name

Filled in for CMode CLI

=item C<< policyowner >>

Policy Owner possible value(s) are, true,false

Filled in for CMode CLI

=item C<< qtree_id >>

Qtree Id ranges from 0 to 4994

Filled in for CMode CLI

=item C<< qtree_fh >>

Qtree File Handle

Filled in for CMode CLI

=item C<< policyid >>

Policy Id

Filled in for CMode CLI

=item C<< qtree >>

Qtree Name

Filled in for CMode CLI

=item C<< policy >>

Policy Name

Filled in for CMode CLI.

=item C<< policy_id >>

Policy ID

Filled in for CMode CLI.


=back

=cut

package NACL::CS::VolumeQtreePolicy;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'policyname',
    scalar => 'volume',
    scalar => 'vserver',
    scalar => 'policyowner',
    scalar => 'qtree_id',
    scalar => 'qtree_fh',
    scalar => 'policyid',
    scalar => 'qtree',
    scalar => 'policy',
    scalar => 'policy_id',
];

=head1 METHODS

=head2 fetch

    my $VolumeQtreePolicy_state  = NACL::CS::VolumeQtreePolicy->fetch(command_interface => $ci, ...);
	my @VolumeQtreePolicy_states = NACL::CS::VolumeQtreePolicy->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
	show_cmd => 'volume qtree policy show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching volume qtree policy(s) found',
        frontend       => 'NACL::CS::VolumeQtreePolicy::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'volume_qtree_policy_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
