# $Id$
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::VolumeQtree->statistics() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VolumeQtreeStatistics

=head1 DESCRIPTION

C<NACL::CS::VolumeQtreeStatistics> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::VolumeQtree->statistics() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::VolumeQtree->statistics() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< nfs_ops >>

Filled in for CMode CLI.

=item C<< volume >>

Filled in for CMode CLI.

=item C<< cifs_ops >>

Filled in for CMode CLI.

=item C<< no_reset_internal_ops >>

Filled in for CMode CLI.

=item C<< vserver >>

Filled in for CMode CLI.

=item C<< qtree_path >>

Filled in for CMode CLI.

=item C<< no_reset_cifs_ops >>

Filled in for CMode CLI.

=item C<< qtree >>

Filled in for CMode CLI.

=item C<< internal_ops >>

Filled in for CMode CLI.

=item C<< no_reset_nfs_ops >>

Filled in for CMode CLI.

=back

=cut

package NACL::CS::VolumeQtreeStatistics;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'nfs_ops',
    scalar => 'volume',
    scalar => 'cifs_ops',
    scalar => 'no_reset_internal_ops',
    scalar => 'vserver',
    scalar => 'qtree_path',
    scalar => 'no_reset_cifs_ops',
    scalar => 'qtree',
    scalar => 'internal_ops',
    scalar => 'no_reset_nfs_ops',
];

=head1 METHODS

=head2 fetch

    my $VolumeQtreeStatistics_state  = NACL::CS::VolumeQtreeStatistics->fetch(command_interface => $ci, ...);
	my @VolumeQtreeStatistics_states = NACL::CS::VolumeQtreeStatistics->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI and 7Mode CLI APISet.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'volume qtree statistics',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode'
            },
        ],
        exception_text => 'No matching volume qtree(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@_, api => 'volume_qtree_statistics',
        );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();

    my $apiset = $opts{apiset};
    my $response;

    try {
        $response = $apiset->qtree_stats("id" => '1');
    } ## end try
    catch NACL::APISet::Exceptions::InvalidParamValueException with {

        # A caught exception indicates that the qtree being looked for
        # does not exist. We catch the exception and return immediately. The
        # 'fetch' frontend decides whether to throw a NoElementsFound
        # exception based on the value of 'allow_empty'
        $Log->exit() if $may_exit;
        return;
    };
    my $output = $response->get_parsed_output();
    my @state_objs;
    foreach my $row (@$output) {
        my $final_attributes = $pkg->_hash_copy(
            source => $row,
            copy   => [qw( volume cifs_ops nfs_ops internal_ops)],
            map    => {'tree' => 'qtree',},
        );
        my $obj = $pkg->new(command_interface => $opts{command_interface});
        $obj->_set_fields(row => $final_attributes);
        push @state_objs, $obj;
    } ## end foreach my $row (@$output)
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_7mode_cli

1;
