
#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::VolumeQuota->status() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VolumeQuotaStatus

=head1 DESCRIPTION

C<NACL::CS::VolumeQuotaStatus> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::VolumeQuota->status() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::VolumeQuota->status() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< volume >>

Volume Name possible value(s) are, volume name

Filled in for CMode CLI.

=item C<< vserver >>

Vserver Name possible value(s) are, vserver name

Filled in for CMode CLI.

=item C<< sub_status_7g >>

Sub Quota Status for 7G possible value(s) are, text

Filled in for CMode CLI.

=item C<< state >>

Quota State possible value(s) are, quota_state

Filled in for CMode CLI.

=item C<< last_error >>

=item C<< scan_status >>

=item C<< quota_status >>

=item C<< sub_status >>

=back

=cut

package NACL::CS::VolumeQuotaStatus;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::CS::_Mixins::VolumeQuota qw(:all);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'volume',
    scalar => 'vserver',
    scalar => 'sub_status_7g',
    scalar => 'state',
    scalar => 'last_error',
    scalar => 'scan_status',
    array  => 'quota_status',
    scalar => 'sub_status',
];

=head1 METHODS

=head2 fetch

    my $VolumeQuotaStatus_state  = NACL::CS::VolumeQuotaStatus->fetch(command_interface => $ci, ...);
	my @VolumeQuotaStatus_states = NACL::CS::VolumeQuotaStatus->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {
               method     => '_fetch_cmode_zapi',
               interface  => 'ZAPI',
               set        => 'CMode'
            },
        ],
	show_cmd       => 'volume quota status',
        exception_text => 'No matching volume quota(s) found',
        frontend       => 'NACL::CS::VolumeQuotaStatus::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli( @args, api => 'volume_quota_status',
        );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ( $pkg, %opts ) = @_;
    my $map = $pkg->_common_zapi_map();
    my $copy = $pkg->_common_zapi_copy();
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        api  => "quota-status-iter",
        map  => $map,
        copy => $copy,
    );
    $Log->exit() if $may_exit;
    return @state_objs;

} ## end sub _fetch_cmode_zapi

1;
