# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeReallocation ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VolumeReallocation

=head1 DESCRIPTION

C<NACL::CS::VolumeReallocation> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VolumeReallocation.  A related class is
L<NACL::C::VolumeReallocation|lib-NACL-C-VolumeReallocation-pm>, which represents access
to an ONTAP VolumeReallocation.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VolumeReallocation
element are the attributes of the VolumeReallocation ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< once >>

Once
Filled in for CMode CLI

=item C<< threshold >>

Threshold ranges from 3 to 10
Filled in for CMode CLI

=item C<< vserver >>

Vserver
Filled in for CMode CLI

=item C<< progress >>

Execution Progress
Filled in for CMode CLI

=item C<< path >>

Path
Filled in for CMode CLI

=item C<< description >>

Job Description
Filled in for CMode CLI

=item C<< state >>


Job State possible value(s) are, Initial,Queued,Running,Waiting,Pausing,Paused,Quitting,Success,Failure,Reschedule,Error,Quit,Dead,Unknown,Restart,Dormant
Filled in for CMode CLI

=item C<< schedule >>

Schedule Name
Filled in for CMode CLI

=item C<< global_status >>

Global State of Scans
Filled in for CMode CLI

=item C<< id >>

Job ID
Filled in for CMode CLI
=back

=cut

package NACL::CS::VolumeReallocation;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'once',
    scalar => 'threshold',
    scalar => 'vserver',
    scalar => 'progress',
    scalar => 'path',
    scalar => 'description',
    scalar => 'state',
    scalar => 'schedule',
    scalar => 'global_status',
    scalar => 'id',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _extra_filter_fields {
    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;
    return [qw( v )];
}

=head1 METHODS

=head2 fetch

    my $VolumeReallocation_state  = NACL::CS::VolumeReallocation->fetch(command_interface => $ci, ...);
	my @VolumeReallocation_states = NACL::CS::VolumeReallocation->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'volume reallocation show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        exception_text => 'No matching VolumeReallocation(s) found',
        frontend       => 'NACL::CS::VolumeReallocation::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'volume_reallocation_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
