#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverActiveDirectory ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverActiveDirectory

=head1 DESCRIPTION

C<NACL::CS::VserverActiveDirectory> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverActiveDirectory.  A related class is
L<NACL::C::VserverActiveDirectory|lib-NACL-C-VserverActiveDirectory-pm>, which represents access
to an ONTAP VserverActiveDirectory.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverActiveDirectory
element are the attributes of the VserverActiveDirectory ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< account_name >>

Active Directory NetBIOS Name

Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

<active-directory-account-config>
    <account-name><$value></account-name>
</active-directory-account-config>

=item C<< ou >>

Organizational Unit

Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

<active-directory-account-config>
    <organizational-unit><$value></organizational-unit>
</active-directory-account-config>

=item C<< domain_workgroup >>

NetBIOS Domain/Workgroup Name

Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

<active-directory-account-config>
    <domain-workgroup><$value></domain-workgroup>
</active-directory-account-config>

=item C<< vserver >>

Vserver

Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

<active-directory-account-config>
    <vserver><$value></vserver>
</active-directory-account-config>

=item C<< hostname >>

Hostname

Filled in for CMode CLI

=item C<< kerberos_keyblock_row >>

Kerberos Keyblock DB Row
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $kerberos_keyblock_row = $obj->kerberos_keyblock_row();
 
# $kerberos_keyblock_row contains a reference to the array of values
 
my @kerberos_keyblock_row = $obj->kerberos_keyblock_row();
 
# @kerberos_keyblock_row contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { kerberos_keyblock_row = [ value1, value2...] }

Filled in for CMode CLI

=item C<< domain >>

Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

<active-directory-account-config>
    <domain><$value></domain>
</active-directory-account-config>

Filled in for CMode CLI

=item C<< cifs_security_row >>

Cifs Server Security Row

Filled in for CMode CLI

=item C<< machine_account_db_v2_row >>

Machine Account DB Row

Filled in for CMode CLI

=item C<< auth_style >>

Authentication Style possible value(s) are, domain,workgroup

Filled in for CMode CLI.


=back

=cut

package NACL::CS::VserverActiveDirectory;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'account_name',
    scalar => 'ou',
    scalar => 'domain_workgroup',
    scalar => 'vserver',
    scalar => 'hostname',
    array  => 'kerberos_keyblock_row',
    scalar => 'domain',
    scalar => 'cifs_security_row',
    scalar => 'machine_account_db_v2_row',
    scalar => 'auth_style',
];

=head1 METHODS

=head2 fetch

    my $VserverActiveDirectory_state  = NACL::CS::VserverActiveDirectory->fetch(command_interface => $ci, ...);
	my @VserverActiveDirectory_states = NACL::CS::VserverActiveDirectory->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes "active-directory-account-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
	show_cmd       => 'vserver active-directory show',
        exception_text => 'No matching vserver active-directory(s) found',
        frontend       => 'NACL::CS::VserverActiveDirectory::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_active_directory_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        copy => [qw( domain vserver account-name domain-workgroup)],
        map  => {'ou' => 'organizational-unit',},
        api  => 'active-directory-account-get-iter',
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
