#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifsShareAccessControl ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com , benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverCifsShareAccessControl

=head1 DESCRIPTION

C<NACL::CS::VserverCifsShareAccessControl> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverCifsShareAccessControl.  A related class is
L<NACL::C::VserverCifsShareAccessControl|lib-NACL-C-VserverCifsShareAccessControl-pm>, which represents access
to an ONTAP VserverCifsShareAccessControl.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the VserverCifsShareAccessControl
element are the attributes of the VserverCifsShareAccessControl ComponentState.

=over

=item C<< "share" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-share-access-control>
    <share>$values</share>
 </cifs-share-access-control>

=item C<< "user_or_group" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-share-access-control>
    <user-or-group>$values</user-or-group>
 </cifs-share-access-control>

=item C<< "vserver" >>

Filled in for CMode CLI/ZAPI.

=item C<< "permission" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-share-access-control>
    <permission>$values</permission>
 </cifs-share-access-control>

=item C<< "winsid" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-share-access-control>
    <winsid>$values</winsid>
 </cifs-share-access-control>

=item C<< user_group_type >>

User/Group Type possible value(s) are, windows,unix-user,unix-group

Filled in for CMode CLI.

=item C<< unixId >>

User/Group ID

Filled in for CMode CLI.


=back

=cut

package NACL::CS::VserverCifsShareAccessControl;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'share',
    scalar => 'user_or_group',
    scalar => 'vserver',
    scalar => 'permission',
    scalar => 'winsid',
    scalar => 'user_group_type',
    scalar => 'unixId',
];

=head1 METHODS

=head2 fetch

    my $VserverCifsShareAccessControl_state   = NACL::CS::VserverCifsShareAccessControl->fetch(command_interface => $ci, ...);
	my @VserverCifsShareAccessControl_states = NACL::CS::VserverCifsShareAccessControl->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "cifs-share-access-control-get-iter" API for CMode ZAPI

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'vserver cifs share access-control show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text =>
            'No matching vserver cifs share access control(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'vserver_cifs_share_access_control_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => 'cifs_share_access_control_get_iter',
        copy => [qw(permission share user-or-group winsid vserver)],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

sub _update_state_objs_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->SUPER::_update_state_objs_cmode_zapi(
        %opts,
        zapi_field_translations => {
            hyphenate_value => [qw(user-group-type)],
            ucfirst         => [qw(permission)],
        },
    );

    # There is one value for 'permission' which is 'Full_Control' which does not follow the normal
    # convention of ucfirst. We need to update the first character to uc after
    # the underscore too.
    foreach my $cs (@{$opts{state_objs}}) {
        if ( $cs->permission() eq "Full_control") {
            $cs->permission("Full_Control");
        }
    }

    $Log->exit() if $may_exit;
}

1;
