# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifsShareProperties ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverCifsShareProperties

=head1 DESCRIPTION

C<NACL::CS::VserverCifsShareProperties> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverCifsShareProperties.  A related class is
L<NACL::C::VserverCifsShareProperties|lib-NACL-C-VserverCifsShareProperties-pm>, which represents access
to an ONTAP VserverCifsShareProperties.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverCifsShareProperties
element are the attributes of the VserverCifsShareProperties ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< share_name >>

Share
Filled in for CMode CLI

=item C<< vserver >>

Vserver
Filled in for CMode CLI

=item C<< share_properties >>

Share Properties
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $share_properties = $obj->share_properties();
# $share_properties contains a reference to the array of values
my @share_properties = $obj->share_properties();
# @share_properties contains the array of values
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref
# filter => { share_properties = [ value1, value2...] }
Filled in for CMode CLI

=back

=cut

package NACL::CS::VserverCifsShareProperties;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'share_name',
    scalar => 'vserver',
    array  => 'share_properties',
];

=head1 METHODS

=head2 fetch

    my $VserverCifsShareProperties_state  = NACL::CS::VserverCifsShareProperties->fetch(command_interface => $ci, ...);
	my @VserverCifsShareProperties_states = NACL::CS::VserverCifsShareProperties->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI APISet.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
	show_cmd => 'vserver cifs share properties show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text =>
            'No matching vserver cifs share properties(s) found',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_cifs_share_properties_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
