#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifsSymlink ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com,benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverCifsSymlink

=head1 DESCRIPTION

C<NACL::CS::VserverCifsSymlink> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverCifsSymlink.  A related class is
L<NACL::C::VserverCifsSymlink|lib-NACL-C-VserverCifsSymlink-pm>, which represents access
to an ONTAP VserverCifsSymlink.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the VserverCifsSymlink
element are the attributes of the VserverCifsSymlink ComponentState.

=over

=item C<< "home_directory" >>

Filled in for CMode CLI.

=item C<< "share_name" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-symlink>
    <share-name>$values</share-name>
 </cifs-symlink>

=item C<< "vserver" >>

Filled in for CMode CLI/ZAPI.

=item C<< "cifs_server" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-symlink>
    <cifs-server>$values</cifs-server>
 </cifs-symlink>
 
=item C<< "unix_path" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-symlink>
    <unix-path>$values</unix-path>
 </cifs-symlink>
 
=item C<< "locality" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-symlink>
    <locality>$values</locality>
 </cifs-symlink>

=item C<< "cifs_path" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-symlink>
    <cifs-path>$values</cifs-path>
 </cifs-symlink>

=back

=cut

package NACL::CS::VserverCifsSymlink;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'home_directory',
    scalar => 'share_name',
    scalar => 'vserver',
    scalar => 'cifs_server',
    scalar => 'unix_path',
    scalar => 'locality',
    scalar => 'cifs_path',
];

=head1 METHODS

=head2 fetch

    my $VserverCifsSymlink_state   = NACL::CS::VserverCifsSymlink->fetch(command_interface => $ci, ...);
	my @VserverCifsSymlink_states = NACL::CS::VserverCifsSymlink->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI APISet.

Invokes "cifs-symlink-get-iter" API for CMode ZAPI

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
         show_cmd      => 'vserver cifs symlink show',
        exception_text => 'No matching vserver cifs symlink(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'vserver_cifs_symlink_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        api  => 'cifs_symlink_get_iter',
        copy => [
            qw ( cifs-path cifs-server locality share-name unix-path vserver)
        ],
    );

    if ($opts{command_interface}->isa('NACL::C::Vserver')) {
        my $vserver = $opts{command_interface}->vserver();
        foreach my $state_obj (@state_objs) {
            $state_obj->vserver($vserver);
        }
    }

    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
