# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifsUsersAndGroupsPrivilege ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, raghaveb@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverCifsUsersAndGroupsPrivilege

=head1 DESCRIPTION

C<NACL::CS::VserverCifsUsersAndGroupsPrivilege> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverCifsUsersAndGroupsPrivilege.  A related class is
L<NACL::C::VserverCifsUsersAndGroupsPrivilege|lib-NACL-C-VserverCifsUsersAndGroupsPrivilege-pm>, which represents access
to an ONTAP VserverCifsUsersAndGroupsPrivilege.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverCifsUsersAndGroupsPrivilege
element are the attributes of the VserverCifsUsersAndGroupsPrivilege ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< privilege_bits >>

Filled in for CMode CLI.

=item C<< privileges >>

Filled in for CMode CLI/ZAPI.
(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $privileges = $obj->privileges();
# $privileges contains a reference to the array of values
my @privileges = $obj->privileges();
# @privileges contains the array of values

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-privilege>
    <privileges>
        <cifs-privilege-entries>@array</cifs-privilege-entries>
     </privileges>
 </cifs-privilege>

=item C<< entry_type >>

Filled in for CMode CLI.

=item C<< vserver >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-privilege>
    <vserver>$values</vserver>
 </cifs-privilege>

=item C<< domain_name >>

Filled in for CMode CLI.

=item C<< have_privilege_bits >>

Filled in for CMode CLI.

=item C<< entry_rid >>

Filled in for CMode CLI.

=item C<< user_or_group_name >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-privilege>
    <user-or-group-name>$array</user-or-group-name>
 </cifs-privilege>

=item C<< localOrAdObjectName >>

Local or AD object Name

Filled in for CMode CLI.

=item C<< localOrAdObjectType >>

Local or AD object Type

Filled in for CMode CLI.

=item C<< localOrAdObjectAsid >>

Local or AD object Asid

Filled in for CMode CLI.


=back

=cut

package NACL::CS::VserverCifsUsersAndGroupsPrivilege;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use Class::MethodMaker [
    scalar => 'privilege_bits',
    array  => 'privileges',
    scalar => 'entry_type',
    scalar => 'vserver',
    scalar => 'domain_name',
    scalar => 'have_privilege_bits',
    scalar => 'entry_rid',
    scalar => 'user_or_group_name',
    scalar => 'localOrAdObjectName',
    scalar => 'localOrAdObjectType',
    scalar => 'localOrAdObjectAsid',
];

=head1 METHODS

=head2 fetch

    my $VserverCifsUsersAndGroupsPrivilege_state  = NACL::CS::VserverCifsUsersAndGroupsPrivilege->fetch(command_interface => $ci, ...);
    my @VserverCifsUsersAndGroupsPrivilege_states = NACL::CS::VserverCifsUsersAndGroupsPrivilege->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "cifs-privilege-get-iter" API for CMode ZAPI

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        show_cmd       => 'vserver cifs users-and-groups privilege show',
        exception_text =>
            'No matching vserver cifs users-and-groups privilege(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'vserver_cifs_users_and_groups_privilege_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg    = shift;
    my %opts   = validate @_, $pkg->_fetch_backend_validate_spec();
    my $apiset = $opts{apiset};

    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => 'cifs_privilege_get_iter',
        copy => [qw( user-or-group-name vserver)],
        map  => {
            'privileges' => [
                make_zapi_array('privileges'),
                make_zapi_skip('cifs-privilege-entries')
            ],
        }
    );

    $Log->exit() if $may_exit;
    return @state_objs;

} ## end sub _fetch_cmode_zapi

1;
