# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverFcpInterface ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com ,raghaveb@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverFcpInterface

=head1 DESCRIPTION

C<NACL::CS::VserverFcpInterface> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverFcpInterface.  A related class is
L<NACL::C::VserverFcpInterface|lib-NACL-C-VserverFcpInterface-pm>, which represents access
to an ONTAP VserverFcpInterface.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverFcpInterface
element are the attributes of the VserverFcpInterface ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< curr_node >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fcp-interface-info>
    <current-node>$value</current-node>
 </fcp-interface-info>

=item C<< port_address >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fcp-interface-info>
    <port-address>$value</port-address>
 </fcp-interface-info>

=item C<< status_admin >>

Filled in for CMode CLI.

=item C<< vserver >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fcp-interface-info>
    <vserver>$value</vserver>
 </fcp-interface-info>

=item C<< wwpn >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fcp-interface-info>
    <port-name>$value</port-name>
 </fcp-interface-info>

=item C<< status_extended >>

Filled in for CMode CLI.

=item C<< status_oper >>

Filled in for CMode CLI.

=item C<< curr_port >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fcp-interface-info>
    <current-port>$value</current-port>
 </fcp-interface-info>

=item C<< relative_port_id >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fcp-interface-info>
    <relative-port-id>$value</relative-port-id>
 </fcp-interface-info>

=item C<< wwnn >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fcp-interface-info>
    <node-name>$value</node-name>
 </fcp-interface-info>

=item C<< is_home >>

Filled in for CMode CLI.

=item C<< lif >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fcp-interface-info>
    <interface-name>$value</interface-name>
 </fcp-interface-info>

=item C<< vserver_uuid >>

Filled in for CMode CLI.

=item C<< zapi_port_address >>

Filled in for CMode CLI.

=item C<< curr_node_str >>

Filled in for CMode CLI.

=back

=cut

package NACL::CS::VserverFcpInterface;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'curr_node',
    scalar => 'port_address',
    scalar => 'status_admin',
    scalar => 'vserver',
    scalar => 'wwpn',
    scalar => 'status_extended',
    scalar => 'status_oper',
    scalar => 'curr_port',
    scalar => 'relative_port_id',
    scalar => 'wwnn',
    scalar => 'is_home',
    scalar => 'lif',
    scalar => 'vserver_uuid',
    scalar => 'zapi_port_address',
    scalar => 'curr_node_str',
];

=head1 METHODS

=head2 fetch

    my $VserverFcpInterface_state  = NACL::CS::VserverFcpInterface->fetch(command_interface => $ci, ...);
	my @VserverFcpInterface_states = NACL::CS::VserverFcpInterface->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI APISet.

Invokes "fcp-interface-get-iter" API for CMode ZAPI

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        'show_cmd'=>'vserver fcp interface show',
        choices   => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        exception_text => 'No matching vserver fcp interface(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'vserver_fcp_interface_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => 'fcp_interface_get_iter',
        copy => [qw ( port-address relative-port-id vserver)],
        map  => {
            'curr-node' => 'current-node',
            'curr-port' => 'current-port',
            'lif'       => 'interface-name',
            'wwpn'      => 'port-name',
            'wwnn'      => 'node-name',
        },
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
