# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverFcpWwpnAlias ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverFcpWwpnAlias

=head1 DESCRIPTION

C<NACL::CS::VserverFcpWwpnAlias> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverFcpWwpnAlias.  A related class is
L<NACL::C::VserverFcpWwpnAlias|lib-NACL-C-VserverFcpWwpnAlias-pm>, which represents access
to an ONTAP VserverFcpWwpnAlias.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the VserverFcpWwpnAlias
element are the attributes of the VserverFcpWwpnAlias ComponentState.

=over

=item C<< "vserver" >>

=item C<< "wwpn" >>

=item C<< "alias" >>

=item C<< vserver_uuid >>

Vserver UUID

Filled in for CMode CLI.


=back

=cut

package NACL::CS::VserverFcpWwpnAlias;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'wwpn',
    scalar => 'alias',
    scalar => 'vserver_uuid',
];

=head1 METHODS

=head2 fetch

    my $VserverFcpWwpnAlias_state   = NACL::CS::VserverFcpWwpnAlias->fetch(command_interface => $ci, ...);
    my @VserverFcpWwpnAlias_states = NACL::CS::VserverFcpWwpnAlias->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        show_cmd       => 'vserver fcp wwpn-alias show',
        exception_text => 'No matching vserver fcp portname(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'vserver_fcp_wwpn_alias_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;
    my $copy = [qw(vserver)];
    my $map  = {
        "wwpn"  => ['aliases-wwpn'],
        "alias" => ['aliases-alias'],
    };

    $Log->exit() if $may_exit;
    return $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        api  => "fcp-wwpnalias-get-iter",
        copy => $copy,
        map  => $map,
    );

} ## end sub _fetch_cmode_zapi

sub _fetch_7mode_cli {
    my $pkg = shift;
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();
    my @state_objs;
    my $apiset            = delete $opts{apiset};
    my $command_interface = delete $opts{command_interface};
    my %cmd_opts;
    my %copy_filter           = %{$opts{filter}};
    my @copy_requested_fields = @{$opts{requested_fields}};
    my $deleted_filter        = $pkg->_remove_relational_regex_filters(
        filter           => \%copy_filter,
        requested_fields => \@copy_requested_fields
    );
    my $filter           = \%copy_filter;
    my $requested_fields = \@copy_requested_fields;

    if ($filter->{alias}) {
        $cmd_opts{'alias'} = $filter->{'alias'};
    }
    my $response = $apiset->fcp_wwpn_alias_show(%cmd_opts);
    my $output   = $response->get_parsed_output();
    foreach my $list_output (@$output) {
        my $target = $pkg->_hash_copy(
            source => $list_output,
            copy   => [qw(alias wwpn)],
        );
        my $obj = $pkg->new(command_interface => $command_interface);
        $obj->_set_fields(row => $target);
        push @state_objs, $obj;
    } ## end foreach my $list_output (@$output)
    return @state_objs;
} ## end sub _fetch_7mode_cli
1;
