# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverFpolicyPolicyEvent ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com,benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverFpolicyPolicyEvent

=head1 DESCRIPTION

C<NACL::CS::VserverFpolicyPolicyEvent> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverFpolicyPolicyEvent.  A related class is
L<NACL::C::VserverFpolicyPolicyEvent|lib-NACL-C-VserverFpolicyPolicyEvent-pm>, which represents access
to an ONTAP VserverFpolicyPolicyEvent.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverFpolicyPolicyEvent
element are the attributes of the VserverFpolicyPolicyEvent ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< event_name >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fpolicy-event-options-config
    <event-name>$value</event-name>
 </fpolicy-event-options-config

=item C<< protocols >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fpolicy-event-options-config
    <protocol>$value</protocol>
 </fpolicy-event-options-config

=item C<< vserver >>

Filled in for CMode CLI.

=item C<< volume_operations >>

lled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fpolicy-event-options-config
    <volume-operations>
       <fpolicy-volops>@values</fpolicy-volops>
    </volume-operations>
 </fpolicy-event-options-config

=item C<< file_operations >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fpolicy-event-options-config
    <file-operations>
       <fpolicy-operation>@values</fpolicy-operation>
    </file-operations>
 </fpolicy-event-options-config

=item C<< filters >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fpolicy-event-options-config
    <filter-string>
       <fpolicy-filter>@values</fpolicy-filter>
    </filter-string>
 </fpolicy-event-options-config

=item C<< protocol >>

Filled in for CMode CLI.


=item C<< volume_operation >>

Volume Operation, possible value(s) are true,false
Filled in for CMode CLI.


=back

=cut

package NACL::CS::VserverFpolicyPolicyEvent;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use Class::MethodMaker [
    scalar => 'event_name',
    array  => 'protocols',
    scalar => 'vserver',
    array  => 'volume_operations',
    array  => 'file_operations',
    array  => 'filters',
    scalar => 'protocol',
    scalar => 'volume_operation',
];

=head1 METHODS

=head2 fetch

    my $VserverFpolicyPolicyEvent_state  = NACL::CS::VserverFpolicyPolicyEvent->fetch(command_interface => $ci, ...);
    my @VserverFpolicyPolicyEvent_states = NACL::CS::VserverFpolicyPolicyEvent->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI APISet.

Invokes "fpolicy-policy-event-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'vserver fpolicy policy event show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching vserver fpolicy policy event(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'vserver_fpolicy_policy_event_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => 'fpolicy_policy_event_get_iter',
        copy => [qw ( event-name )],
        map  => {
            'volume-operations' => [
                make_zapi_array('volume-operations'),
                make_zapi_skip('fpolicy-volops')
            ],
            'file-operations' => [
                make_zapi_array('file-operations'),
                make_zapi_skip('fpolicy-operation')
            ],
            'filters' => [
                make_zapi_array('filter-string'),
                make_zapi_skip('fpolicy-filter')
            ],
            'protocols'  => ['protocol'],
            'event_name' => ['event-name']
        },
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
