# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::VserverFpolicy->show_engine() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverFpolicyShowEngine

=head1 DESCRIPTION

C<NACL::CS::VserverFpolicyShowEngine> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::VserverFpolicy->show_engine() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::VserverFpolicy->show_engine() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< vserver >>

Vserver

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<fpolicy-server-status-info vserver="$value" />



=item C<< disconnect_reason >>

Reason for FPolicy Server Disconnection

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<fpolicy-server-status-info disconnect-reason="$value" />



=item C<< policy_name >>

Policy

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<fpolicy-server-status-info policy-name="$value" />



=item C<< node >>

Node possible value(s) are, <nodename>,local

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<fpolicy-server-status-info node="$value" />



=item C<< server_type >>

Server Type

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<fpolicy-server-status-info server-type="$value" />



=item C<< session_id >>

Session ID

Filled in for CMode CLI.

=item C<< disconnect_reason_id >>

ID for FPolicy Server Disconnection

Filled in for CMode CLI.

=item C<< server_status >>

Server Status

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<fpolicy-server-status-info server-status="$value" />



=item C<< server >>

Server

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<fpolicy-server-status-info fpolicy-server="$value" />



=item C<< disconnected_since >>

Time FPolicy Server was Disconnected

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<fpolicy-server-status-info disconnected-since="$value" />



=item C<< connected_since >>

Time FPolicy Server was Connected

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<fpolicy-server-status-info connected-since="$value" />



=back

=cut

package NACL::CS::VserverFpolicyShowEngine;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'disconnect_reason',
    scalar => 'policy_name',
    scalar => 'node',
    scalar => 'server_type',
    scalar => 'session_id',
    scalar => 'disconnect_reason_id',
    scalar => 'server_status',
    scalar => 'server',
    scalar => 'disconnected_since',
    scalar => 'connected_since',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $VserverFpolicyShowEngine_state  = NACL::CS::VserverFpolicyShowEngine->fetch(command_interface => $ci, ...);
	my @VserverFpolicyShowEngine_states = NACL::CS::VserverFpolicyShowEngine->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes fpolicy-server-status-get-iter API for CMode ZAPI.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'vserver fpolicy show-engine',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        exception_text => 'No matching vserver fpolicy(s) found',
        frontend       => 'NACL::CS::VserverFpolicyShowEngine::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_fpolicy_show_engine',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(disconnect-reason vserver policy-name server-type node server-status
                disconnected-since connected-since
                )
        ],
        map => {'server' => 'fpolicy-server',},
        api => "fpolicy-server-status-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
