#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverIscsiInterfaceAccesslist ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, pkj@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverIscsiInterfaceAccesslist

=head1 DESCRIPTION

C<NACL::CS::VserverIscsiInterfaceAccesslist> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverIscsiInterfaceAccesslist.  A related class is
L<NACL::C::VserverIscsiInterfaceAccesslist|lib-NACL-C-VserverIscsiInterfaceAccesslist-pm>, which represents access
to an ONTAP VserverIscsiInterfaceAccesslist.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the VserverIscsiInterfaceAccesslist
element are the attributes of the VserverIscsiInterfaceAccesslist ComponentState.

=over

=item C<< "initiator" >>

=item C<< "vserver" >>

=item C<< "lif" >>
This maps to iscsi-interface-accesslist-entry-info for CMode zapi.
(Array)

=item C<< "all" >>

=item C<< initiator_name >>

Filled in for CMode CLI/ZAPI.


=item C<< vserver_uuid >>

Vserver UUID

Filled in for CMode CLI.


=back

=cut

package NACL::CS::VserverIscsiInterfaceAccesslist;

use strict;
use warnings;
use NACL::ComponentUtils qw (_dump_one Dumper);
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'initiator',
    scalar => 'vserver',
    array  => 'lif',
    scalar => 'all',
    scalar => 'initiator_name',
    scalar => 'vserver_uuid',
];

=head1 METHODS

=head2 fetch

    my $VserverIscsiInterfaceAccesslist_state   = NACL::CS::VserverIscsiInterfaceAccesslist->fetch(command_interface => $ci, ...);
	my @VserverIscsiInterfaceAccesslist_states = NACL::CS::VserverIscsiInterfaceAccesslist->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI/ZAPI and 7Mode CLI APISet.
This methos executes iscsi-interface-accesslist-get-iter for CMode zapi.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'vserver iscsi interface accesslist show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        exception_text =>
            'No matching vserver iscsi interface accesslist(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'vserver_iscsi_interface_accesslist_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => 'iscsi_interface_accesslist_get_iter',
        copy => [qw ( vserver initiator)],
        map  => {
            'lif' =>
                ['iscsi-interface-accesslist-entry-info', 'interface-name'],
        },
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg         = shift;
    my %opts        = validate @_, $pkg->_fetch_backend_validate_spec();
    my $apiset      = $opts{apiset};
    my %copy_filter = %{$opts{filter}};
    my @copy_requested_fields = @{$opts{requested_fields}};
    my $deleted_filter        = $pkg->_remove_relational_regex_filters(
        filter           => \%copy_filter,
        requested_fields => \@copy_requested_fields
    );
    my $filter           = \%copy_filter;
    my $requested_fields = \@copy_requested_fields;
    my $req_field_filter;
    $req_field_filter->{'requested_fields'} = $requested_fields;
    $req_field_filter->{'filter'}           = \%copy_filter;
    my %cmd_opts;
    my @state_objs;

    if ($filter->{'initiator-name'}) {
        $cmd_opts{'initiator-name'} = $filter->{'initiator-name'};
    } else {
        $cmd_opts{'all'} = 1;
    }

    my $response = $apiset->iscsi_interface_accesslist_show(%cmd_opts);
    my $output   = $response->get_parsed_output();

    foreach my $row (@$output) {
        my $obj;
        $obj = $pkg->new(command_interface => $opts{command_interface});
        my $members      = $row->{access_list};
        my $row_modified = $pkg->_hash_copy(
            source => $row,
            map    => {'initiator_name' => 'initiator-name',},
        );
        $$row_modified{'lif'} = $members;
        $obj->_set_fields(row => $row_modified);
        push @state_objs, $obj;
    } ## end foreach my $row (@$output)
    $Log->exit() if $may_exit;
    $Log->debug("value of the parsed output is :" . Dumper(\@state_objs));

    return @state_objs;
} ## end sub _fetch_7mode_cli

1;
