# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverIscsiPortal ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverIscsiPortal

=head1 DESCRIPTION

C<NACL::CS::VserverIscsiPortal> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverIscsiPortal.  A related class is
L<NACL::C::VserverIscsiPortal|lib-NACL-C-VserverIscsiPortal-pm>, which represents access
to an ONTAP VserverIscsiPortal.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the VserverIscsiPortal
element are the attributes of the VserverIscsiPortal ComponentState.

=over

=item C<< "tpgname" >>

=item C<< "curr_node" >>

=item C<< "status_admin" >>

=item C<< "vserver" >>

=item C<< "ip_port" >>

=item C<< "status_oper" >>

=item C<< "curr_port" >>

=item C<< "is_home" >>

=item C<< "tpgroup" >>

=item C<< "numeric_id" >>

=item C<< "tpgtag" >>

=item C<< "address" >>

=item C<< "lif" >>

=item C<< "enabled" >>

=item C<< relative_port_id >>

Filled in for CMode CLI.

=item C<< tpgroup_tag >>

Filled in for CMode CLI.


=item C<< vserver_uuid >>

Vserver UUID
Filled in for CMode CLI.


=back

=cut

package NACL::CS::VserverIscsiPortal;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one Dumper);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::Exceptions::NoElementsFound qw(:try);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'tpgname',
    scalar => 'curr_node',
    scalar => 'status_admin',
    scalar => 'vserver',
    scalar => 'ip_port',
    scalar => 'status_oper',
    scalar => 'curr_port',
    scalar => 'is_home',
    scalar => 'tpgroup',
    scalar => 'numeric_id',
    scalar => 'tpgtag',
    scalar => 'address',
    scalar => 'lif',
    scalar => 'enabled',
    scalar => 'relative_port_id',
    scalar => 'tpgroup_tag',
    scalar => 'vserver_uuid',
];

=head1 METHODS

=head2 fetch

    my $VserverIscsiPortal_state   = NACL::CS::VserverIscsiPortal->fetch(command_interface => $ci, ...);
    my @VserverIscsiPortal_states = NACL::CS::VserverIscsiPortal->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.
supports only CMODE CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'vserver iscsi portal show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        exception_text => 'No matching vserver iscsi portal(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'vserver_iscsi_portal_show');
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();
    my @state_objs;
    my $apiset                = delete $opts{apiset};
    my %copy_filter           = %{$opts{filter}};
    my @copy_requested_fields = @{$opts{requested_fields}};
    my $deleted_filter        = $pkg->_remove_relational_regex_filters(
        filter           => \%copy_filter,
        requested_fields => \@copy_requested_fields
    );
    my $filter           = \%copy_filter;
    my $requested_fields = \@copy_requested_fields;
    my $response         = $apiset->iscsi_portal_show();
    my $output           = $response->get_parsed_output();
    my $obj;
    my $row_modified;

    foreach my $row (@$output) {
        $obj          = $pkg->new();
        $row_modified = $pkg->_hash_copy(
            source => $row,
            map    => {
                "ip_address" => "address",
                "tcp_port"   => "ip_port",
                "interface"  => "curr_port"
            },
            copy => [qw ( tpgroup )],
        );
        $row_modified->{'command_interface'} = $opts{command_interface};
        $obj->_set_fields(row => $row_modified);
        push @state_objs, $obj;
    } ## end foreach my $row (@$output)
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
