# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverNfsKerberosInterface ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverNfsKerberosInterface

=head1 DESCRIPTION

C<NACL::CS::VserverNfsKerberosInterface> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverNfsKerberosInterface.  A related class is
L<NACL::C::VserverNfsKerberosInterface|lib-NACL-C-VserverNfsKerberosInterface-pm>, which represents access
to an ONTAP VserverNfsKerberosInterface.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverNfsKerberosInterface
element are the attributes of the VserverNfsKerberosInterface ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< kerberos >>

Kerberos Enabled possible value(s) are, enabled,disabled
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <kerberos-config-info is-kerberos-enabled="$value" />

Mapping for CMode ZAPI non-iter: is-kerberos-enabled


=item C<< vserver >>

Vserver
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <kerberos-config-info vserver="$value" />


=item C<< permitted_enc_types >>

Permitted Encryption Types
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $permitted_enc_types = $obj->permitted_enc_types();

# $permitted_enc_types contains a reference to the array of values

my @permitted_enc_types = $obj->permitted_enc_types();

# @permitted_enc_types contains the array of values

If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { permitted_enc_types = [ value1, value2...] }

Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <kerberos-config-info>
  <permitted-enc-types string="@values" />
</kerberos-config-info>

Mapping for CMode ZAPI non-iter: <permitted-enc-types string="@values" />


=item C<< kerberos_keyblock_row >>

Kerberos Keyblock rdb row
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $kerberos_keyblock_row = $obj->kerberos_keyblock_row();

# $kerberos_keyblock_row contains a reference to the array of values

my @kerberos_keyblock_row = $obj->kerberos_keyblock_row();

# @kerberos_keyblock_row contains the array of values

If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { kerberos_keyblock_row = [ value1, value2...] }

Filled in for CMode CLI/ZAPI iter, non-iter.

=item C<< spn >>

Service Principal Name
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <kerberos-config-info service-principal-name="$value" />

Mapping for CMode ZAPI non-iter: service-principal-name


=item C<< numeric_id >>

Numeric Id
Filled in for CMode CLI/ZAPI iter, non-iter.

=item C<< address >>

IP Address
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <kerberos-config-info ip-address="$value" />

Mapping for CMode ZAPI non-iter: ip-address


=item C<< lif >>

Logical Interface
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <kerberos-config-info interface-name="$value" />

Mapping for CMode ZAPI non-iter: interface-name

=back

=cut

package NACL::CS::VserverNfsKerberosInterface;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'kerberos',
    scalar => 'vserver',
    array  => 'permitted_enc_types',
    array  => 'kerberos_keyblock_row',
    scalar => 'spn',
    scalar => 'numeric_id',
    scalar => 'address',
    scalar => 'lif',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [qw(vserver lif)],
        _vs_context   => 1
    );
    $Log->exit() if $may_exit;
}

=head1 METHODS

=head2 fetch

    my $VserverNfsKerberosInterface_state  = NACL::CS::VserverNfsKerberosInterface->fetch(command_interface => $ci, ...);
    my @VserverNfsKerberosInterface_states = NACL::CS::VserverNfsKerberosInterface->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes kerberos-config-get-iter API for CMode ZAPI iter.

Invokes kerberos-config-get API for CMode ZAPI non-iter.


=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_check_non_iter'
            },
        ],
        exception_text => 'No matching VserverNfsKerberosInterface(s) found',
        frontend       => 'NACL::CS::VserverNfsKerberosInterface::fetch',
        show_cmd       => 'vserver nfs kerberos interface show',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _update_state_objs_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->SUPER::_update_state_objs_cmode_zapi(@args,
        zapi_field_translations => {enable_disable => [qw(kerberos)],},);
    $Log->exit() if $may_exit;
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_nfs_kerberos_interface_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(vserver
                )
        ],
        map => {
            'kerberos'            => 'is-kerberos-enabled',
            'permitted-enc-types' => [
                make_zapi_array('permitted-enc-types'),
                make_zapi_skip('string'),
            ],
            'spn'     => 'service-principal-name',
            'address' => 'ip-address',
            'lif'     => 'interface-name',
        },
        api => "kerberos-config-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        copy => [
            qw(vserver
                )
        ],
        map => {
            'kerberos'            => 'is-kerberos-enabled',
            'permitted-enc-types' => [
                make_zapi_array('permitted-enc-types'),
                make_zapi_skip('string'),
            ],
            'spn'     => 'service-principal-name',
            'address' => 'ip-address',
            'lif'     => 'interface-name',
        },
        api                 => "kerberos-config-get",
        vserver_operated_on => \my $vserver,
        primary_keys        => [qw( lif )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
