# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverNfsKerberosRealm ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverNfsKerberosRealm

=head1 DESCRIPTION

C<NACL::CS::VserverNfsKerberosRealm> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverNfsKerberosRealm.  A related class is
L<NACL::C::VserverNfsKerberosRealm|lib-NACL-C-VserverNfsKerberosRealm-pm>, which represents access
to an ONTAP VserverNfsKerberosRealm.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverNfsKerberosRealm
element are the attributes of the VserverNfsKerberosRealm ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< adminserver_port >>

Admin Server Port ranges from 1 to 65535

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<kerberos-realm admin-server-port="$value" />



=item C<< vserver >>

Vserver

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<kerberos-realm vserver-name="$value" />



=item C<< kdc_ip >>

KDC IP Address

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<kerberos-realm kdc-ip="$value" />



=item C<< passwordserver_port >>

Password Server Port ranges from 1 to 65535

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<kerberos-realm password-server-port="$value" />



=item C<< permitted_enc_types >>

Permitted Encryption Types
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $permitted_enc_types = $obj->permitted_enc_types();
 
# $permitted_enc_types contains a reference to the array of values
 
my @permitted_enc_types = $obj->permitted_enc_types();
 
# @permitted_enc_types contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { permitted_enc_types = [ value1, value2...] }


Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<kerberos-realm>
  <permitted-enc-types string="@values" />
</kerberos-realm>



=item C<< kdc_port >>

KDC Port ranges from 1 to 65535

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<kerberos-realm kdc-port="$value" />



=item C<< adserver_name >>

Active Directory Server Name

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<kerberos-realm ad-server-name="$value" />



=item C<< adserver_ip >>

Active Directory Server IP Address

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<kerberos-realm ad-server-ip="$value" />



=item C<< comment >>

Comment

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<kerberos-realm comment="$value" />



=item C<< passwordserver_ip >>

Password Server IP Address

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<kerberos-realm password-server-ip="$value" />



=item C<< realm >>

Kerberos Realm

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<kerberos-realm realm="$value" />



=item C<< kdc_vendor >>

KDC Vendor

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<kerberos-realm kdc-vendor="$value" />



=item C<< clock_skew >>

Clock Skew

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<kerberos-realm clock-skew="$value" />



=item C<< adminserver_ip >>

Admin Server IP Address

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<kerberos-realm admin-server-ip="$value" />



=item C<< configname >>

Configuration Name

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<kerberos-realm config-name="$value" />



=back

=cut

package NACL::CS::VserverNfsKerberosRealm;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'adminserver_port',
    scalar => 'vserver',
    scalar => 'kdc_ip',
    scalar => 'passwordserver_port',
    array  => 'permitted_enc_types',
    scalar => 'kdc_port',
    scalar => 'adserver_name',
    scalar => 'adserver_ip',
    scalar => 'comment',
    scalar => 'passwordserver_ip',
    scalar => 'realm',
    scalar => 'kdc_vendor',
    scalar => 'clock_skew',
    scalar => 'adminserver_ip',
    scalar => 'configname',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $VserverNfsKerberosRealm_state  = NACL::CS::VserverNfsKerberosRealm->fetch(command_interface => $ci, ...);
	my @VserverNfsKerberosRealm_states = NACL::CS::VserverNfsKerberosRealm->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes kerberos-realm-get-iter API for CMode ZAPI.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'vserver nfs kerberos realm show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        exception_text => 'No matching vserver nfs kerberos realm(s) found',
        frontend       => 'NACL::CS::VserverNfsKerberosRealm::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_nfs_kerberos_realm_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(kdc-ip kdc-port comment realm clock-skew kdc-vendor
                )
        ],
        map => {
            'vserver'             => 'vserver-name',
            'adminserver-port'    => 'admin-server-port',
            'passwordserver-port' => 'password-server-port',
            'permitted-enc-types' => [
                make_zapi_array('permitted-enc-types'),
                make_zapi_skip('string'),
            ],
            'adserver-name'     => 'ad-server-name',
            'adserver-ip'       => 'ad-server-ip',
            'passwordserver-ip' => 'password-server-ip',
            'configname'        => 'config-name',
            'adminserver-ip'    => 'admin-server-ip',
        },
        api => "kerberos-realm-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
