# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverPeerTransition ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status public
## @pod here

=head1 NAME

NACL::CS::VserverPeerTransition

=head1 DESCRIPTION

C<NACL::CS::VserverPeerTransition> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverPeerTransition.  A related class is
L<NACL::C::VserverPeerTransition|lib-NACL-C-VserverPeerTransition-pm>, which represents access
to an ONTAP VserverPeerTransition.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverPeerTransition
element are the attributes of the VserverPeerTransition ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< localVsUuid >>

Filled in for CMode CLI.

=item C<< srcFilerUuid >>

Filled in for CMode CLI.

=item C<< src_filer_name >>

Source Filer Name

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <vserver-peer-transition-info>
     <src-filer-name>$value</src-filer-name>
 </vserver-peer-transition-info>

=item C<< local_vserver >>

Local Vserver Name

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <vserver-peer-transition-info>
     <local-vserver>$value</local-vserver>
 </vserver-peer-transition-info>

=item C<< src_filer_uuid >>

*Source Filer UUID

Filled in for CMode CLI.

=item C<< local_vs_uuid >>

Local Vserver UUID

Filled in for CMode CLI.

=item C<< multi-path-address >>

Alternate IP Address for Source Filer

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <vserver-peer-transition-info>
     <multi-path-address>$value</multi-path-address>
 </vserver-peer-transition-info>

=back

=cut

package NACL::CS::VserverPeerTransition;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::C::_Mixins::VserverPeerTransition qw(:all);

use NACL::Exceptions::NoElementsFound qw(:try);
use NATE::Exceptions::Argument;

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'localVsUuid',
    scalar => 'srcFilerUuid',
    scalar => 'src_filer_name',
    scalar => 'src_filer_uuid',
    scalar => 'local_vserver',
    scalar => 'local_vs_uuid',
    scalar => 'multi_path_address',
    scalar => [
        {   '-read_cb' => sub {
                return (defined $_[1]) ? $_[1] : $_[0]->multi_path_address;
                }
        },
        'alternate_ip'
    ],
    scalar => [
        {   '-read_cb' => sub {
                return (defined $_[1]) ? $_[1] : $_[0]->src_filer_name;
                }
        },
        'srcFilerName'
    ],
    scalar => [
        {   '-read_cb' => sub {
                return (defined $_[1]) ? $_[1] : $_[0]->local_vserver;
                }
        },
        'localVserver'
    ],
];

=head1 METHODS

=head2 fetch

    my $VserverPeerTransition_state  = NACL::CS::VserverPeerTransition->fetch(command_interface => $ci, ...);
    my @VserverPeerTransition_states = NACL::CS::VserverPeerTransition->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "vserver-peer-transition-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args, $pkg->_fetch_validate_spec();

    $pkg->_validate_input(%opts);

    # Handle converting the option
    # 'alternate-ip' to 'multi-path-address' for both CLI and ZAPI.
    my $filter_copy = {%{$opts{filter}}};
    my %requested_fields_copy = map { $_ => 1 } @{$opts{requested_fields}};

    # Update requested_fields
    $pkg->_handle_alternate_ip_change(
        command_interface => $opts{command_interface},
        opts              => \%requested_fields_copy
    );

    # Update filter
    $pkg->_handle_alternate_ip_change(
        command_interface => $opts{command_interface},
        opts              => $filter_copy
    );

    my @state_objs = $pkg->SUPER::fetch(
        %opts,
        show_cmd => 'vserver peer transition show',
        requested_fields => [keys %requested_fields_copy],
        filter           => $filter_copy,
        choices          => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        exception_text => 'No matching vserver peer transition(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_peer_transition_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        %args,
        api => 'vserver-peer-transition-get-iter',
        map => $pkg->_zapi_option_map(
            command_interface => $args{command_interface}
        ),
        copy => ['alternate-ip'],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

sub _validate_input {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my @dup        = ();
    my $error_text = "";
    if (exists $opts{filter}) {
        @dup = $pkg->_validate_input_for_dup_keys(%{$opts{filter}});
        if (@dup) {
            $error_text
                = "You can pass only either of the following "
                . "arguments in the filter:\n"
                . join(', ', @dup);
        }
    }
    if (exists $opts{requested_fields}) {
        my %req_fields;
        foreach (@{$opts{requested_fields}}) {
            $req_fields{$_} = 1;
        }
        @dup = $pkg->_validate_input_for_dup_keys(%req_fields);
        if (@dup) {
            $error_text
                = "You can pass only either of the following "
                . "arguments in the requested_fields:\n"
                . join(', ', @dup);
        }
    }
    if ($error_text) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw($error_text);
    }
    $Log->exit() if $may_exit;
}

1;
