# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverSecurityFileDirectoryNtfs ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, pkj@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverSecurityFileDirectoryNtfs

=head1 DESCRIPTION

C<NACL::CS::VserverSecurityFileDirectoryNtfs> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverSecurityFileDirectoryNtfs.  A related class is
L<NACL::C::VserverSecurityFileDirectoryNtfs|lib-NACL-C-VserverSecurityFileDirectoryNtfs-pm>, which represents access
to an ONTAP VserverSecurityFileDirectoryNtfs.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverSecurityFileDirectoryNtfs
element are the attributes of the VserverSecurityFileDirectoryNtfs ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< owner >>

Owner of the Security Descriptor

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <file-directory-security-ntfs>
     <owner>$value</owner>
 </file-directory-security-ntfs>

=item C<< group >>

*Primary Group of the Owner

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <file-directory-security-ntfs>
     <group>$value</group>
 </file-directory-security-ntfs>
 
=item C<< vserver >>

Vserver

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <file-directory-security-ntfs>
     <vserver>$value</vserver>
 </file-directory-security-ntfs>

=item C<< control_flags_raw >>

*Raw Control Flags

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <file-directory-security-ntfs>
     <control-flags-raw>$value</control-flags-raw>
 </file-directory-security-ntfs>

=item C<< ntfs_sd >>

Security Descriptor Name

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <file-directory-security-ntfs>
     <ntfs-sd>$value</ntfs-sd>
 </file-directory-security-ntfs>

=item C<< ntfs_sd_string >>

Security Descriptor String

Filled in for CMode CLI.

=item C<< admin_account >>

BUILTIN Administrators

Filled in for CMode CLI.

=item C<< owner_account >>

CREATOR OWNER

Filled in for CMode CLI.

=item C<< users_account >>

BUILTIN Users

Filled in for CMode CLI.

=item C<< system_account >>

NT AUTHORITY SYSTEM

Filled in for CMode CLI.


=back

=cut

package NACL::CS::VserverSecurityFileDirectoryNtfs;

use strict;
use warnings;
use Data::Dumper;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'owner',
    scalar => 'group',
    scalar => 'vserver',
    scalar => 'control_flags_raw',
    scalar => 'ntfs_sd',
    scalar => 'ntfs_sd_string',
    scalar => 'admin_account',
    scalar => 'owner_account',
    scalar => 'users_account',
    scalar => 'system_account',
];

=head1 METHODS

=head2 fetch

    my $VserverSecurityFileDirectoryNtfs_state  = NACL::CS::VserverSecurityFileDirectoryNtfs->fetch(command_interface => $ci, ...);
    my @VserverSecurityFileDirectoryNtfs_states = NACL::CS::VserverSecurityFileDirectoryNtfs->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "file-directory-security-ntfs-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'vserver security file-directory ntfs show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        exception_text =>
            'No matching vserver security file-directory ntfs(s) found',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'vserver_security_file_directory_ntfs_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => 'file_directory_security_ntfs_get_iter',
        copy => [qw(control-flags-raw group ntfs-sd vserver owner)],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
