# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverSecurityFileDirectoryNtfsSacl ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, raghaveb@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverSecurityFileDirectoryNtfsSacl

=head1 DESCRIPTION

C<NACL::CS::VserverSecurityFileDirectoryNtfsSacl> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverSecurityFileDirectoryNtfsSacl.  A related class is
L<NACL::C::VserverSecurityFileDirectoryNtfsSacl|lib-NACL-C-VserverSecurityFileDirectoryNtfsSacl-pm>, which represents access
to an ONTAP VserverSecurityFileDirectoryNtfsSacl.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverSecurityFileDirectoryNtfsSacl
element are the attributes of the VserverSecurityFileDirectoryNtfsSacl ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< rights >>

Access Rights

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <file-directory-security-ntfs-sacl>
     <readable-access-rights>$value</readable-access-rights>
 </file-directory-security-ntfs-sacl>

=item C<< account >>

Account Name or SID

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <file-directory-security-ntfs-sacl>
     <account>$value</account>
 </file-directory-security-ntfs-sacl>

=item C<< vserver >>

Vserver

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <file-directory-security-ntfs-sacl>
     <vserver>$value</vserver>
 </file-directory-security-ntfs-sacl>

=item C<< ntfs_sd >>

Security Descriptor Name

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

  <file-directory-security-ntfs-sacl>
     <ntfs-sd>$value</ntfs-sd>
 </file-directory-security-ntfs-sacl>

=item C<< advanced_rights >>

Advanced Access Rights

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $advanced_rights = $obj->advanced_rights();
# $advanced_rights contains a reference to the array of values
my @advanced_rights = $obj->advanced_rights();
# @advanced_rights contains the array of values
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref
# filter => { advanced_rights = [ value1, value2...] }

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

  <file-directory-security-ntfs-sacl>
     <advanced-rights>
         <advanced-access-rights>@values</advanced-access-rights>
     </advanced-rights>
 </file-directory-security-ntfs-sacl>

=item C<< access_type >>

Access type for Specified Access Rights

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

  <file-directory-security-ntfs-sacl>
     <audit-access-type>$value</audit-access-type>
 </file-directory-security-ntfs-sacl>

=item C<< rights_raw >>

*Raw Access Rights

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

  <file-directory-security-ntfs-sacl>
     <rights-raw>$value</rights-raw>
 </file-directory-security-ntfs-sacl>

=item C<< apply_to >>

Apply To

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $apply_to = $obj->apply_to();
# $apply_to contains a reference to the array of values
my @apply_to = $obj->apply_to();
# @apply_to contains the array of values
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref
# filter => { apply_to = [ value1, value2...] }

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

  <file-directory-security-ntfs-sacl>
     <apply-to>
         <inheritance-level>@values</inheritance-level>
     </apply-to>
 </file-directory-security-ntfs-sacl>

=item C<< readable_access_rights >>

Access Rights

Filled in for CMode CLI.

=item C<< ntfs_sd_string >>

Security Descriptor String

Filled in for CMode CLI.


=item C<< admin_account >>

BUILTIN Administrators

Filled in for CMode CLI.

=item C<< owner_account >>

CREATOR OWNER

Filled in for CMode CLI.

=item C<< users_account >>

BUILTIN Users

Filled in for CMode CLI.

=item C<< system_account >>

NT AUTHORITY SYSTEM

Filled in for CMode CLI.


=back

=cut

package NACL::CS::VserverSecurityFileDirectoryNtfsSacl;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'rights',
    scalar => 'account',
    scalar => 'vserver',
    scalar => 'ntfs_sd',
    array  => 'advanced_rights',
    scalar => 'access_type',
    scalar => 'rights_raw',
    array  => 'apply_to',
    scalar => 'readable_access_rights',
    scalar => 'ntfs_sd_string',
    scalar => 'admin_account',
    scalar => 'owner_account',
    scalar => 'users_account',
    scalar => 'system_account',
];

=head1 METHODS

=head2 fetch

    my $VserverSecurityFileDirectoryNtfsSacl_state  = NACL::CS::VserverSecurityFileDirectoryNtfsSacl->fetch(command_interface => $ci, ...);
    my @VserverSecurityFileDirectoryNtfsSacl_states = NACL::CS::VserverSecurityFileDirectoryNtfsSacl->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "file-directory-security-ntfs-sacl-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'vserver security file-directory ntfs sacl show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        exception_text =>
            'No matching vserver security file-directory ntfs sacl(s) found',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'vserver_security_file_directory_ntfs_sacl_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => 'file_directory_security_ntfs_sacl_get_iter',
        copy => [qw( account ntfs-sd rights-raw vserver )],
        map  => {
            'access-type' => 'audit-access-type',
            'rights'      => 'readable-access-rights',
            'apply-to'    => [
                make_zapi_array('apply-to'),
                make_zapi_skip('inheritance-level')
            ],
            'advanced-rights' => [
                make_zapi_array('advanced-rights'),
                make_zapi_skip('advanced-access-rights')
            ],
        },
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _update_state_objs_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->SUPER::_update_state_objs_cmode_zapi(
        @args,
        zapi_field_translations => {
            hyphenate_value => [qw(advanced-rights rights apply-to)],
        },
    );
    $Log->exit() if $may_exit;
}

1;
