# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverSecurityFileDirectoryPolicy ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, pkj@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverSecurityFileDirectoryPolicy

=head1 DESCRIPTION

C<NACL::CS::VserverSecurityFileDirectoryPolicy> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverSecurityFileDirectoryPolicy.  A related class is
L<NACL::C::VserverSecurityFileDirectoryPolicy|lib-NACL-C-VserverSecurityFileDirectoryPolicy-pm>, which represents access
to an ONTAP VserverSecurityFileDirectoryPolicy.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverSecurityFileDirectoryPolicy
element are the attributes of the VserverSecurityFileDirectoryPolicy ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< vserver >>

Vserver

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <file-directory-security-policy>
     <vserver> $value </vserver>
 </file-directory-security-policy>

=item C<< policy_name >>

Policy Name

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <file-directory-security-policy>
     <policy-name> $value </policy-name>
 </file-directory-security-policy>

=item C<< policy_name_string >>

Policy Name String

Filled in for CMode CLI.


=back

=cut

package NACL::CS::VserverSecurityFileDirectoryPolicy;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'policy_name',
    scalar => 'policy_name_string',
];

=head1 METHODS

=head2 fetch

    my $VserverSecurityFileDirectoryPolicy_state  = NACL::CS::VserverSecurityFileDirectoryPolicy->fetch(command_interface => $ci, ...);
    my @VserverSecurityFileDirectoryPolicy_states = NACL::CS::VserverSecurityFileDirectoryPolicy->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "file-directory-security-policy-get-iter" API for CMode ZAPI. 

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        show_cmd       => 'vserver security file-directory policy show',
        exception_text =>
            'No matching vserver security file-directory policy(s) found',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'vserver_security_file_directory_policy_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => 'file_directory_security_policy_get_iter',
        copy => [qw(policy-name vserver)],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
