# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverSecurityTraceFilter ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverSecurityTraceFilter

=head1 DESCRIPTION

C<NACL::CS::VserverSecurityTraceFilter> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverSecurityTraceFilter.  A related class is
L<NACL::C::VserverSecurityTraceFilter|lib-NACL-C-VserverSecurityTraceFilter-pm>, which represents access
to an ONTAP VserverSecurityTraceFilter.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverSecurityTraceFilter
element are the attributes of the VserverSecurityTraceFilter ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< vserver >>

Vserver

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <security-trace-filter-attributes>
     <vserver>$value</vserver>
 </security-trace-filter-attributes>

=item C<< time_enabled >>

Minutes Filter is Enabled

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <security-trace-filter-attributes>
     <time-enabled>$value</time-enabled>
 </security-trace-filter-attributes>

=item C<< modified_time >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <security-trace-filter-attributes>
     <modified-time>$value</modified-time>
 </security-trace-filter-attributes>

=item C<< unix_name >>

UNIX User Name

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <security-trace-filter-attributes>
     <unix-name>$value</unix-name>
 </security-trace-filter-attributes>

=item C<< win_path >>

Cifs Path

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <security-trace-filter-attributes>
     <win-path>$value</win-path>
 </security-trace-filter-attributes>

=item C<< client_ip >>

Client IP Address to match

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <security-trace-filter-attributes>
     <client-ip>$value</client-ip>
 </security-trace-filter-attributes>

=item C<< trace_allow >>

Trace Allow Events, possible value(s) are yes,no

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <security-trace-filter-attributes>
     <trace-allow>$value</trace-allow>
 </security-trace-filter-attributes>

=item C<< index >>

Filter Index

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <security-trace-filter-attributes>
     <index>$value</index>
 </security-trace-filter-attributes>

=item C<< windows_name >>

Windows User Name

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <security-trace-filter-attributes>
     <windows-name>$value</windows-name>
 </security-trace-filter-attributes>

=item C<< enabled >>

Filter Enabled, possible value(s) are enabled,disabled

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <security-trace-filter-attributes>
     <enabled>$value</enabled>
 </security-trace-filter-attributes>

=item C<< path >>

Path

Filled in for CMode CLI.

=back

=cut

package NACL::CS::VserverSecurityTraceFilter;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'time_enabled',
    scalar => 'modified_time',
    scalar => 'unix_name',
    scalar => 'win_path',
    scalar => 'client_ip',
    scalar => 'trace_allow',
    scalar => 'index',
    scalar => 'windows_name',
    scalar => 'enabled',
    scalar => 'path',
    array  => 'protocols',
];

=head1 METHODS

=head2 fetch

    my $VserverSecurityTraceFilter_state  = NACL::CS::VserverSecurityTraceFilter->fetch(command_interface => $ci, ...);
    my @VserverSecurityTraceFilter_states = NACL::CS::VserverSecurityTraceFilter->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "security-trace-filter-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        show_cmd       => 'vserver security trace filter show',
        exception_text =>
            'No matching vserver security trace filter(s) found',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'vserver_security_trace_filter_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts)      = @_;
    if( exists $opts{filter}->{'trace-allow'} ) {
	my %filter = %{$opts{filter}};
	if ( $filter{'trace-allow'} eq 'yes') {
	    $filter{'trace-allow'} = 'true';
	} elsif ($filter{'trace-allow'} eq 'no') {
	    $filter{'trace-allow'} = 'false';
	}
	$opts{filter} = \%filter;
    }
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        api  => 'security_trace_filter_get_iter',
        copy => [
            qw( client-ip enabled index modified-time time-enabled trace-allow unix-name vserver win-path windows-name protocols)
        ],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi


sub _update_state_objs_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->SUPER::_update_state_objs_cmode_zapi(
        @args,
        zapi_field_translations => {
            yes_no_boolean        => [qw(trace-allow)]
        },
    );
    $Log->exit() if $may_exit;
}
1;
