#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverServicesDnsHosts ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverServicesDnsHosts

=head1 DESCRIPTION

C<NACL::CS::VserverServicesDnsHosts> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverServicesDnsHosts.  A related class is
L<NACL::C::VserverServicesDnsHosts|lib-NACL-C-VserverServicesDnsHosts-pm>, which represents access
to an ONTAP VserverServicesDnsHosts.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the VserverServicesDnsHosts
element are the attributes of the VserverServicesDnsHosts ComponentState.

=over

=item C<< "vserver" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <hosts-info>
     <vserver-name>$value</vserver-name>
 </hosts-info>

=item C<< "aliases" >>

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $aliases = $obj->aliases();
# $aliases contains a reference to the array of values
my @aliases = $obj->aliases();
# @aliases contains the array of values

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:
 <hosts-info>
     <aliases>
         <string>@values</string>
     </aliases>
 </hosts-info>

=item C<< "hostname" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:
 <hosts-info>
     <hostname>$value</hostname>
 </hosts-info>

=item C<< "address" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:
 <hosts-info>
     <host-ip-address>$value</host-ip-address>
 </hosts-info>

=back

=cut

package NACL::CS::VserverServicesDnsHosts;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'vserver',
    array  => 'aliases',
    scalar => 'hostname',
    scalar => 'address',
];

=head1 METHODS

=head2 fetch

    my $VserverServicesDnsHosts_state   = NACL::CS::VserverServicesDnsHosts->fetch(command_interface => $ci, ...);
	my @VserverServicesDnsHosts_states = NACL::CS::VserverServicesDnsHosts->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "net-hosts-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'vserver services dns hosts show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching vserver services dns hosts(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'vserver_services_dns_hosts_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => 'net_hosts_get_iter',
        copy => [qw(hostname aliases)],
        map  => {
            "vserver" => ['vserver-name'],
            "address" => ['host-ip-address'],
        },
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi
1;
