# $Id$
# This is a prototype.	It needs documentation.

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverServicesLdap ComponentState Module
## @author Rupamjyoti.Baruah@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverServicesLdap

=head1 DESCRIPTION

C<NACL::CS::VserverServicesLdap> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverServicesLdap.  A related class is
L<NACL::C::VserverServicesLdap|lib-NACL-C-VserverServicesLdap-pm>, which represents access
to an ONTAP VserverServicesLdap.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the VserverServicesLdap
element are the attributes of the VserverServicesLdap ComponentState.

=over

=item C<< "vserver" >>

Filled in for CMode CLI/ZAPI.

Maps to:

For "requested_fields", "filter" and Output mapping:

 <ldap-config>
     <vserver>$value</vserver>
 </ldap-config>

=item C<< "client_config" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <ldap-config>
     <client-config>$value</client-config>
 </ldap-config>

=item C<< "client_enabled" >>

Filled in for CMode CLI/ZAPI.

Maps to:

For "requested_fields", "filter" and Output mapping:

 <ldap-config>
     <client-enabled>$value</client-enabled>
 </ldap-config>

=item C<< client_config_uuid >>

LDAP Client config uuid

Filled in for CMode CLI.


=back

=cut

package NACL::CS::VserverServicesLdap;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::Exceptions::NoElementsFound qw(:try);
use Data::Dumper;
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [

    # These fields are from the C Mode "vserver services ldap show"
    scalar => 'vserver',
    scalar => 'client_config',
    scalar => 'client_enabled',
    scalar => 'client_config_uuid',
];

=head1 METHODS

=head2 fetch

    my $VserverServicesLdap_state   = NACL::CS::VserverServicesLdap->fetch(command_interface => $ci, ...);
    my @VserverServicesLdap_states = NACL::CS::VserverServicesLdap->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "ldap-config-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'vserver services ldap show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },

        ],
        exception_text => 'No matching vserver services ldap(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    my $pkg = shift;
    return $pkg->SUPER::_fetch_cmode_cli(@_,
        api => "vserver_services_ldap_show");
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    my @states = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        api  => "ldap-config-get-iter",
        copy => [qw(vserver client-config client-enabled)],
    );

    $Log->exit() if $may_exit;
    return @states;

} ## end sub _fetch_cmode_zapi

1;
