# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::VserverServicesNameServiceDns->show_statistics() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverServicesNameServiceDnsShowStatistics

=head1 DESCRIPTION

C<NACL::CS::VserverServicesNameServiceDnsShowStatistics> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::VserverServicesNameServiceDns->show_statistics() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::VserverServicesNameServiceDns->show_statistics() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over


=item C<< ip_index >>

Index of Name Server
Filled in for CMode CLI/ZAPI iter, non-iter.

=item C<< avg_rtt >>

Average Round Trip Time in Milliseconds During Data Access
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <nameservice-dns-statistics avg-rtt="$value" />


=item C<< errors >>

Number of Errors Returned During Data Access
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <nameservice-dns-statistics errors="$value" />


=item C<< vserver >>

Vserver
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <nameservice-dns-statistics vserver="$value" />


=item C<< queries >>

Number of Queries Made During Data Access
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <nameservice-dns-statistics queries="$value" />


=item C<< node >>

Node possible value(s) are, <nodename>,local
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <nameservice-dns-statistics node="$value" />


=item C<< name_server >>

Name Server
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <nameservice-dns-statistics name-server="$value" />

=back

=cut

package NACL::CS::VserverServicesNameServiceDnsShowStatistics;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'ip_index',
    scalar => 'avg_rtt',
    scalar => 'errors',
    scalar => 'vserver',
    scalar => 'queries',
    scalar => 'node',
    scalar => 'name_server',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [qw(ip-index vserver node)],
        _vs_context   => 0
    );
    $Log->exit() if $may_exit;
}

=head1 METHODS

=head2 fetch

    my $VserverServicesNameServiceDnsShowStatistics_state  = NACL::CS::VserverServicesNameServiceDnsShowStatistics->fetch(command_interface => $ci, ...);
	my @VserverServicesNameServiceDnsShowStatistics_states = NACL::CS::VserverServicesNameServiceDnsShowStatistics->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes name-service-dns-statistics-get-iter API for CMode ZAPI iter.

Invokes nameservice-dns-statistics-get API for CMode ZAPI non-iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_check_non_iter'
            },
        ],
	show_cmd => 'vserver services name-service dns show-statistics',
        exception_text =>
            'No matching vserver services name-service dns(s) found',
        frontend =>
            'NACL::CS::VserverServicesNameServiceDnsShowStatistics::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_services_name_service_dns_show_statistics',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(errors avg-rtt vserver queries name-server node
                )
        ],
        api => "name-service-dns-statistics-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        copy => [
            qw(errors avg-rtt vserver queries name-server node
                )
        ],
        api          => "nameservice-dns-statistics-get",
        primary_keys => [qw( ip-index vserver node )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
