# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::VserverServicesNameServiceNisDomain->show_statistics() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverServicesNameServiceNisDomainShowStatistics

=head1 DESCRIPTION

C<NACL::CS::VserverServicesNameServiceNisDomainShowStatistics> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::VserverServicesNameServiceNisDomain->show_statistics() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::VserverServicesNameServiceNisDomain->show_statistics() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over


=item C<< num_rtx >>

Number of Network Retransmissions in YP Lookups During Data Access
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <nameservice-nis-statistics num-rtx="$value" />


=item C<< vserver >>

Vserver
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <nameservice-nis-statistics vserver="$value" />


=item C<< node >>

Node possible value(s) are, <nodename>,local
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <nameservice-nis-statistics node="$value" />


=item C<< total_rtt >>

Total Time Spent in YP Lookups During Data Access
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <nameservice-nis-statistics total-rtt="$value" />


=item C<< ip_index >>

Index of NIS Server
Filled in for CMode CLI/ZAPI iter, non-iter.

=item C<< avg_rtt >>

Average Time Spent in YP Lookups During Data Access
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <nameservice-nis-statistics avg-rtt="$value" />


=item C<< lookups >>

Number of YP Lookups During Data Access
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <nameservice-nis-statistics lookups="$value" />


=item C<< server >>

NIS Server
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <nameservice-nis-statistics server="$value" />


=item C<< min_rtt >>

Minimum Time Spent in a YP Lookup During Data Access
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <nameservice-nis-statistics min-rtt="$value" />


=item C<< max_rtt >>

Maximum Time Spent in a YP Lookup During Data Access
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <nameservice-nis-statistics max-rtt="$value" />

=back

=cut

package NACL::CS::VserverServicesNameServiceNisDomainShowStatistics;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'num_rtx',
    scalar => 'vserver',
    scalar => 'node',
    scalar => 'total_rtt',
    scalar => 'ip_index',
    scalar => 'avg_rtt',
    scalar => 'lookups',
    scalar => 'server',
    scalar => 'min_rtt',
    scalar => 'max_rtt',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [qw(node vserver ip-index)],
        _vs_context   => 0
    );
    $Log->exit() if $may_exit;
}

=head1 METHODS

=head2 fetch

    my $VserverServicesNameServiceNisDomainShowStatistics_state  = NACL::CS::VserverServicesNameServiceNisDomainShowStatistics->fetch(command_interface => $ci, ...);
	my @VserverServicesNameServiceNisDomainShowStatistics_states = NACL::CS::VserverServicesNameServiceNisDomainShowStatistics->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes name-service-nis-statistics-get-iter API for CMode ZAPI iter.

Invokes nameservice-nis-statistics-get API for CMode ZAPI non-iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_check_non_iter'
            },
        ],
	show_cmd => 'vserver services name-service nis-domain show-statistics',
	exception_text =>
            'No matching vserver services name-service nis-domain(s) found',
        frontend =>
            'NACL::CS::VserverServicesNameServiceNisDomainShowStatistics::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_services_name_service_nis_domain_show_statistics',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(num-rtx vserver node total-rtt avg-rtt lookups server min-rtt max-rtt
                )
        ],
        api => "name-service-nis-statistics-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        copy => [
            qw(num-rtx vserver node total-rtt avg-rtt lookups server min-rtt max-rtt
                )
        ],
        api          => "nameservice-nis-statistics-get",
        primary_keys => [qw( node vserver ip-index )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
