# $Id: $

# This is a prototype.	It needs documentation.

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverServicesUnixGroup ComponentState Module
## @author Dheeraj.K@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverServicesUnixGroup

=head1 DESCRIPTION

C<NACL::CS::VserverServicesUnixGroup> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverServicesUnixGroup.  A related class is
L<NACL::C::VserverServicesUnixGroup|lib-NACL-C-VserverServicesUnixGroup-pm>, which represents access
to an ONTAP VserverServicesUnixGroup.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the VserverServicesUnixGroup
element are the attributes of the VserverServicesUnixGroup ComponentState.

=over

=item C<< vserver >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <unix-group-info>
     <vserver>$value</vserver>
 </unix-group-info>

=item C<< name >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <unix-group-info>
     <group-name>$value</group-name>
 </unix-group-info>

=item C<< id >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <unix-group-info>
     <group-id>$value</group-id>
 </unix-group-info>

=item C<< users >>

Filled in for CMode CLI.

=back

=cut

package NACL::CS::VserverServicesUnixGroup;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);

use NACL::Exceptions::NoElementsFound qw(:try);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [

    # These fields are from the C Mode "vserver services unix group show"
    scalar => 'vserver',
    scalar => 'name',
    scalar => 'id',
    array  => 'users',
];

sub isa {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, $kind) = @_;
    my $isa = $pkg_or_obj->_build_isa(
        kind  => $kind,
        alias => 'NACL::CS::UnixGroup'
    );
    $Log->exit() if $may_exit;
    return $isa;
}

=head1 METHODS

=head2 fetch

    my $VserverServicesUnixGroup_state   = NACL::CS::VserverServicesUnixGroup->fetch(command_interface => $ci, ...);
    my @VserverServicesUnixGroup_states = NACL::CS::VserverServicesUnixGroup->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "name-mapping-unix-group-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'vserver services unix-group show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },

            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching unix group(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    my $pkg = shift;
    return $pkg->SUPER::_fetch_cmode_cli(@_,
        api => "vserver_services_unix_group_show");
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        api  => "name_mapping_unix_group_get_iter",
        copy => [qw(vserver)],
        map  => {
            "id"   => ['group-id'],
            "name" => ['group-name'],
            "users"=> [
                        'users',
                        make_zapi_skip('unix-user-name'),
                        make_zapi_skip('user-name')
                      ],
        },
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
