# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverVscanConnectionStatus ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverVscanConnectionStatus

=head1 DESCRIPTION

C<NACL::CS::VserverVscanConnectionStatus> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverVscanConnectionStatus.  A related class is
L<NACL::C::VserverVscanConnectionStatus|lib-NACL-C-VserverVscanConnectionStatus-pm>, which represents access
to an ONTAP VserverVscanConnectionStatus.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverVscanConnectionStatus
element are the attributes of the VserverVscanConnectionStatus ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< vserver >>

Vserver

Filled in for CMode CLI

=item C<< connection_count >>

Number of Connected Vscan Servers Serving the Vserver

Filled in for CMode CLI

=item C<< servers >>

List of Connected Vscan Servers
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $servers = $obj->servers();
 
# $servers contains a reference to the array of values
 
my @servers = $obj->servers();
 
# @servers contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { servers = [ value1, value2...] }


Filled in for CMode CLI

=item C<< node >>

Node possible value(s) are, <nodename>,local

Filled in for CMode CLI

=back

=cut

package NACL::CS::VserverVscanConnectionStatus;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'connection_count',
    array  => 'servers',
    scalar => 'node',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $VserverVscanConnectionStatus_state  = NACL::CS::VserverVscanConnectionStatus->fetch(command_interface => $ci, ...);
	my @VserverVscanConnectionStatus_states = NACL::CS::VserverVscanConnectionStatus->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
	show_cmd => 'vserver vscan connection-status show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
        ],
        exception_text =>
            'No matching vserver vscan connection-status(s) found',
        frontend => 'NACL::CS::VserverVscanConnectionStatus::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_vscan_connection_status_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
