# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::VserverVscanConnectionStatus->show_all() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverVscanConnectionStatusShowAll

=head1 DESCRIPTION

C<NACL::CS::VserverVscanConnectionStatusShowAll> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::VserverVscanConnectionStatus->show_all() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::VserverVscanConnectionStatus->show_all() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< privileged_user >>

Privileged User Used for Connection

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-connection-status-all-info privileged-user="$value" />



=item C<< vserver >>

Vserver

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-connection-status-all-info vserver="$value" />



=item C<< version >>

Vscanner Version

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-connection-status-all-info version="$value" />



=item C<< disconnect_reason >>

Server Disconnect Reason

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-connection-status-all-info disconnect-reason="$value" />



=item C<< node >>

Node possible value(s) are, <nodename>,local

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-connection-status-all-info node="$value" />



=item C<< server_type >>

Server Type

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-connection-status-all-info server-type="$value" />



=item C<< server_status >>

Server Status

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-connection-status-all-info server-status="$value" />



=item C<< server >>

Server

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-connection-status-all-info server="$value" />



=item C<< disconnected_since >>

Time When Vscanner Was Disconnected

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-connection-status-all-info disconnected-since="$value" />



=item C<< connected_since >>

Time When Vscanner Was Connected

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-connection-status-all-info connected-since="$value" />



=item C<< vendor >>

Vscanner Vendor

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-connection-status-all-info vendor="$value" />



=back

=cut

package NACL::CS::VserverVscanConnectionStatusShowAll;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'privileged_user',
    scalar => 'vserver',
    scalar => 'version',
    scalar => 'disconnect_reason',
    scalar => 'node',
    scalar => 'server_type',
    scalar => 'server_status',
    scalar => 'server',
    scalar => 'disconnected_since',
    scalar => 'connected_since',
    scalar => 'vendor',
];

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $VserverVscanConnectionStatusShowAll_state  = NACL::CS::VserverVscanConnectionStatusShowAll->fetch(command_interface => $ci, ...);
	my @VserverVscanConnectionStatusShowAll_states = NACL::CS::VserverVscanConnectionStatusShowAll->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes vscan-connection-status-all-get-iter API for CMode ZAPI.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'vserver vscan connection-status show-all',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        exception_text =>
            'No matching vserver vscan connection-status(s) found',
        frontend => 'NACL::CS::VserverVscanConnectionStatusShowAll::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_vscan_connection_status_show_all',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(privileged-user disconnect-reason version vserver server-type node
                server-status server disconnected-since vendor connected-since
                )
        ],
        api => "vscan-connection-status-all-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
