# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::VserverVscanConnectionStatus->show_extended_stats() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverVscanConnectionStatusShowExtendedStats

=head1 DESCRIPTION

C<NACL::CS::VserverVscanConnectionStatusShowExtendedStats> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::VserverVscanConnectionStatus->show_extended_stats() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::VserverVscanConnectionStatus->show_extended_stats() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< extended_stats >>

Extended-statistics from Vscanner
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $extended_stats = $obj->extended_stats();
 
# $extended_stats contains a reference to the array of values
 
my @extended_stats = $obj->extended_stats();
 
# @extended_stats contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { extended_stats = [ value1, value2...] }

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-connection-extended-stats-info>
  <extended-stats string="@values" />
</vscan-connection-extended-stats-info>

=item C<< vserver >>

Vserver

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-connection-extended-stats-info vserver="$value" />

=item C<< version >>

Vscanner Version

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-connection-extended-stats-info version="$value" />

=item C<< server_status >>

Server Status

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-connection-extended-stats-info server-status="$value" />

=item C<< server >>

Server

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-connection-extended-stats-info server="$value" />

=item C<< node >>

Node possible value(s) are, <nodename>,local

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-connection-extended-stats-info node="$value" />

=item C<< server_type >>

Server Type

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-connection-extended-stats-info server-type="$value" />

=item C<< vendor >>

Vscanner Vendor

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-connection-extended-stats-info vendor="$value" />

=back

=cut

package NACL::CS::VserverVscanConnectionStatusShowExtendedStats;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    array  => 'extended_stats',
    scalar => 'vserver',
    scalar => 'version',
    scalar => 'server_status',
    scalar => 'server',
    scalar => 'node',
    scalar => 'server_type',
    scalar => 'vendor',
];

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $VserverVscanConnectionStatusShowExtendedStats_state  = NACL::CS::VserverVscanConnectionStatusShowExtendedStats->fetch(command_interface => $ci, ...);
	my @VserverVscanConnectionStatusShowExtendedStats_states = NACL::CS::VserverVscanConnectionStatusShowExtendedStats->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes vscan-connection-extended-stats-get-iter API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        show_cmd => 'vserver vscan connection-status show-extended-stats',
	exception_text =>
            'No matching vserver vscan connection-status(s) found',
        frontend =>
            'NACL::CS::VserverVscanConnectionStatusShowExtendedStats::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_vscan_connection_status_show_extended_stats',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(version vserver server-type node server-status server vendor
                )
        ],
        map => {
            'extended-stats' => [
                make_zapi_array('extended-stats'), make_zapi_skip('string'),
                'extended-stats'
            ],
        },
        api => "vscan-connection-extended-stats-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
