# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverVscanOnAccessPolicy ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverVscanOnAccessPolicy

=head1 DESCRIPTION

C<NACL::CS::VserverVscanOnAccessPolicy> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverVscanOnAccessPolicy.  A related class is
L<NACL::C::VserverVscanOnAccessPolicy|lib-NACL-C-VserverVscanOnAccessPolicy-pm>, which represents access
to an ONTAP VserverVscanOnAccessPolicy.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverVscanOnAccessPolicy
element are the attributes of the VserverVscanOnAccessPolicy ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< protocol >>

File-Access Protocol

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-on-access-policy-info protocol="$value" />



=item C<< owner >>

Policy Config Owner

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-on-access-policy-info admin-owner="$value" />



=item C<< paths_to_exclude >>

File-Paths Not to Scan
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $paths_to_exclude = $obj->paths_to_exclude();

# $paths_to_exclude contains a reference to the array of values

my @paths_to_exclude = $obj->paths_to_exclude();

# @paths_to_exclude contains the array of values

If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { paths_to_exclude = [ value1, value2...] }


Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-on-access-policy-info></vscan-on-access-policy-info>



=item C<< vserver >>

Vserver

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-on-access-policy-info vserver="$value" />



=item C<< policy_status >>

Policy Status possible value(s) are, on,off

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-on-access-policy-info is-policy-enabled="$value" />



=item C<< filters >>


Filters possible value(s) are, scan-mandatory,scan-ro-volume,scan-execute-access
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $filters = $obj->filters();

# $filters contains a reference to the array of values

my @filters = $obj->filters();

# @filters contains the array of values

If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { filters = [ value1, value2...] }


Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-on-access-policy-info></vscan-on-access-policy-info>



=item C<< policy_name >>

Policy

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-on-access-policy-info policy-name="$value" />



=item C<< file_ext_to_exclude >>

File-Extensions Not to Scan
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $file_ext_to_exclude = $obj->file_ext_to_exclude();

# $file_ext_to_exclude contains a reference to the array of values

my @file_ext_to_exclude = $obj->file_ext_to_exclude();

# @file_ext_to_exclude contains the array of values

If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { file_ext_to_exclude = [ value1, value2...] }


Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-on-access-policy-info></vscan-on-access-policy-info>



=item C<< max_file_size >>

Max File Size Allowed for Scanning

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-on-access-policy-info max-file-size="$value" />



=back

=cut

package NACL::CS::VserverVscanOnAccessPolicy;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'protocol',
    scalar => 'owner',
    array  => 'paths_to_exclude',
    scalar => 'vserver',
    scalar => 'policy_status',
    array  => 'filters',
    scalar => 'policy_name',
    array  => 'file_ext_to_exclude',
    scalar => 'max_file_size',
];

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $VserverVscanOnAccessPolicy_state  = NACL::CS::VserverVscanOnAccessPolicy->fetch(command_interface => $ci, ...);
    my @VserverVscanOnAccessPolicy_states = NACL::CS::VserverVscanOnAccessPolicy->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes vscan-on-access-policy-get-iter API for CMode ZAPI.


=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
	show_cmd       => 'vserver vscan on-access-policy show',
        exception_text =>
            'No matching vserver vscan on-access-policy(s) found',
        frontend => 'NACL::CS::VserverVscanOnAccessPolicy::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_vscan_on_access_policy_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(protocol vserver policy-name max-file-size
                )
        ],
        map => {
            'paths-to-exclude' => [
                make_zapi_array('paths-to-exclude'),
                make_zapi_skip('file-path')
            ],
            'owner'         => 'admin-owner',
            'policy-status' => 'is-policy-enabled',
            'filters'       => [
                make_zapi_array('filters'),
                make_zapi_skip('vscan-on-access-policy-filter'),
            ],
            'file-ext-to-exclude' => [
                make_zapi_array('file-ext-to-exclude'),
                make_zapi_skip('file-extension'),
            ],
        },
        api => "vscan-on-access-policy-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
