# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::VserverVscanScannerPool->show_active() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverVscanScannerPoolShowActive

=head1 DESCRIPTION

C<NACL::CS::VserverVscanScannerPoolShowActive> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::VserverVscanScannerPool->show_active() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::VserverVscanScannerPool->show_active() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< session_setup_timeout >>

Session Setup Timeout

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-active-scanner-pool-info session-setup-timeout="$value" />



=item C<< vserver >>

Vserver

Filled in for CMode CLI.

=item C<< request_timeout >>

Request Service Timeout

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-active-scanner-pool-info request-timeout="$value" />



=item C<< privileged_users >>

Merged List of Privileged Users
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $privileged_users = $obj->privileged_users();
 
# $privileged_users contains a reference to the array of values
 
my @privileged_users = $obj->privileged_users();
 
# @privileged_users contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { privileged_users = [ value1, value2...] }


Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-active-scanner-pool-info>
  <privileged-users privileged-user="@values" />
</vscan-active-scanner-pool-info>



=item C<< scanner_pools >>

List of Enabled Scanner Pools
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $scanner_pools = $obj->scanner_pools();
 
# $scanner_pools contains a reference to the array of values
 
my @scanner_pools = $obj->scanner_pools();
 
# @scanner_pools contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { scanner_pools = [ value1, value2...] }


Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-active-scanner-pool-info>
  <scanner-pools scanner-pool="@values" />
</vscan-active-scanner-pool-info>



=item C<< max_session_setup_retries >>

Max Number of Consecutive Session Setup Attempts

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-active-scanner-pool-info max-session-setup-retries="$value" />



=item C<< scan_queue_timeout >>

Scan Queue Timeout

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-active-scanner-pool-info scan-queue-timeout="$value" />



=item C<< servers >>

Merged List of IPs of Allowed Vscan Servers
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $servers = $obj->servers();
 
# $servers contains a reference to the array of values
 
my @servers = $obj->servers();
 
# @servers contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { servers = [ value1, value2...] }


Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-active-scanner-pool-info>
  <servers ip-address="@values" />
</vscan-active-scanner-pool-info>



=item C<< session_teardown_timeout >>

Session Teardown Timeout

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-active-scanner-pool-info session-teardown-timeout="$value" />



=back

=cut

package NACL::CS::VserverVscanScannerPoolShowActive;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'session_setup_timeout',
    scalar => 'vserver',
    scalar => 'request_timeout',
    array  => 'privileged_users',
    array  => 'scanner_pools',
    scalar => 'max_session_setup_retries',
    scalar => 'scan_queue_timeout',
    array  => 'servers',
    scalar => 'session_teardown_timeout',
];

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $VserverVscanScannerPoolShowActive_state  = NACL::CS::VserverVscanScannerPoolShowActive->fetch(command_interface => $ci, ...);
	my @VserverVscanScannerPoolShowActive_states = NACL::CS::VserverVscanScannerPoolShowActive->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes vscan-active-scanner-pool-get-iter API for CMode ZAPI.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'vserver vscan scanner-pool show-active',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        exception_text => 'No matching vserver vscan scanner-pool(s) found',
        frontend => 'NACL::CS::VserverVscanScannerPoolShowActive::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_vscan_scanner_pool_show_active',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(session-setup-timeout request-timeout max-session-setup-retries
                scan-queue-timeout session-teardown-timeout
                )
        ],
        map => {
            'privileged-users' => [
                make_zapi_array('privileged-users'),
                make_zapi_skip('privileged-user'),
                'privileged-users'
            ],
            'scanner-pools' => [
                make_zapi_array('scanner-pools'),
                make_zapi_skip('scanner-pool'),
                'scanner-pools'
            ],
            'servers' => [
                make_zapi_array('servers'), make_zapi_skip('ip-address'),
                'servers'
            ],
        },
        api => "vscan-active-scanner-pool-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
