#
# $Id$
#
# Copyright (c) 2009-2011 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NACL::Exceptions::LoadCPUFailed that will be thrown when the
##          CPU load tuning has hit some failure in the maintenance
##          phase
##
## @author isshwar@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=pod

=head1 Package Name

NACL::Exceptions::LoadCPUFailedInMaintenancePhase

=head1 Author/Maintainer

dl-nacl-dev

=head1 Description

An exception thrown from NACL::MTask::Load when the tuning of CPU load fails
in maintenance phase

=head1 See Also

NACL::Exceptions::LoadCPUFailed, NATE::BaseException

=head1 NACL::Exceptions::LoadCPUFailedInMaintenancePhase

=head2 Expected use

This exception should only be thrown from NACL::MTask::Load

=head2 Base Class

NACL::Exceptions::LoadCPUFailed

=head2 ATTRIBUTES

=over

=item actual_exception

The original exception raised in the tuning worker process. This is an
optional attribute and it is populated, only if an exception is thrown
from the underlying code. In general, it's very common to expect for
the underlying libraries to report any failures through exceptions.

Any other forms of failure (such as the ones reported with 'die($str)')
will be received by the Load MTask as error string, in which case,
this attribute will be undefined.

=back

=cut

package NACL::Exceptions::LoadCPUFailedInMaintenancePhase;
use base qw(NACL::Exceptions::LoadCPUFailed);
use Class::MethodMaker [
    scalar => [
        { '-type' => 'NATE::BaseException' },
          'actual_exception'
    ],
];


1;
