#
# Copyright (c) 2001-2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary Global NACL variables
## @author  dl-nacl-dev@netapp.com
## @status  Shared
## @pod     here

package NACL::Global;

=head1 NAME

NACL::Global

=head1 DESCRIPTION

C<NACL::Global> is a collection of global convenience variables for use in NACL
and applications without the overhead of loading any additional NACL libraries.
The use of these variables with NACL components will still require the loading
of those NACL components' libraries.

=cut

#
# Compiler Directives
#
use strict;
use warnings;

#
# Release name-to-version variables
#
our $ONTAP_Release_Longboard_0_short = '9.0';
our $ONTAP_Release_Longboard_0       = '9.0.0';
our $ONTAP_Release_Longboard_1       = '9.1.0';
our $ONTAP_Release_Harpoon_0_short   = '9.2';
our $ONTAP_Release_Harpoon_0         = '9.2.0';
our $ONTAP_Release_FatTire_0_short   = '9.3';
our $ONTAP_Release_FatTire_0         = '9.3.0';


# This flag is needed to deal with mixed stack runs, i.e runs that load
# with many different entries in NATE_LIB, pointing to different versions of
# the same code. What is possible is for NACL/OnDemandLoading.pm to be found
# late in NATE_LIB since its a new file, and for the NACL files having the
# changes required for it to run to be found early. By having this flag,
# NACL::OnDemandLoading can detect whether the code it depends on is indeed
# going to be loaded since this file, a preexisting one, is likely to be loaded
# from the same NATE_LIB as the rest of NACL.
#
# Short version: NATE_LIBs are evil and gotta go
#
our $is_nacl_on_demand_loading_possible = 1;


=head2 CURRENTLY AVAILABLE VARIABLES

    $NACL::Global::ONTAP_Release_Longboard_0_short : '9.0'
    $NACL::Global::ONTAP_Release_Longboard_0       : '9.0.0';
    $NACL::Global::ONTAP_Release_Longboard_1       : '9.1.0';
    $NACL::Global::ONTAP_Release_Harpoon_0_short   : '9.2';
    $NACL::Global::ONTAP_Release_Harpoon_0         : '9.2.0';
    $NACL::Global::ONTAP_Release_FatTire_0_short   : '9.3';
    $NACL::Global::ONTAP_Release_FatTire           : '9.3.0';

=cut

####
1; #
####
