#
#   Copyright (c) 2016 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#    without prior written consent from NetApp, Inc.
#

package NACL::Init::Config::ONTAP::MultiClusterVsim;

use strict;
use warnings;

use NATE::ParamSet qw(param_global);;
use NATE::BaseException;
use Carp qw(confess);

use Moo;
extends qw( NACL::Init::Config::ONTAP::SingleClusterVsim );

sub generate_resources {
    my ($self) = shift;
    my $resources_param;

    # Get resource values.
    my $nodes = param_global()->get('NODES');
    # CIT_cr_suite passes in NODES and DST_NODES
    my $dst_nodes = param_global()->get('DST_NODES');

    if(!$nodes && !$dst_nodes) {
        NATE::BaseException->throw("Missing Mandatory NODES or DST_NODES");
    }
    # if both $nodes and $dst_nodes are defined
    elsif($nodes && $dst_nodes) {
        # join these two nodes to setup the cluster
        $resources_param = join(',',$nodes, $dst_nodes);
    }
    # if not defined $nodes
    elsif(!$nodes && $dst_nodes) {
        $resources_param = $dst_nodes;
    }
    # if not defined $dst_nodes
    else {
        $resources_param = $nodes;
    }
    my @resources = split(',', $resources_param);
    $self->resources(\@resources);
}

sub generate_setup_tasks {

    my ($self, %args) = @_;

    my $resources = $self->resources;

    # Inherit single cluster vsim setup tasks
    $self->SUPER::generate_setup_tasks(%args);

    # ClusterSetupNoSimplicityWizard
    # get the task object of ClusterSetupNoSimplicityWizard base class task
    # and modify the task parameters
    my ($cluster_setup_task_obj) = ($self->get_task (
        task_name   => "NACL::Init::Tasks::ONTAP::ClusterSetupNoSimplicityWizard",
        alias       => "cluster_setup",
    ))[0];

    if(defined $cluster_setup_task_obj && $cluster_setup_task_obj ne ''){
        $cluster_setup_task_obj->{alias} = "cluster_setup";
        $cluster_setup_task_obj->{run_scheme} = "parallel_group_of_size_n";
        $cluster_setup_task_obj->{task_args} = { RUNSCHEME_GROUP_SIZE => 2 };
    } else {
        confess "No matching task";
    }

    # BroadcastDomain
    # get the task object of BroadcastDomain base class task
    # and modify the task parameters
    my ($broadcast_domain_task_obj) = ($self->get_task (
        task_name   => "NACL::Init::Tasks::ONTAP::BroadcastDomain",
        alias       => "broadcast_domain",
    ))[0];

    if(defined $broadcast_domain_task_obj && $broadcast_domain_task_obj ne ''){
        $broadcast_domain_task_obj->{alias} = "broadcast_domain";
        $broadcast_domain_task_obj->{run_scheme} = "parallel_group_of_size_n";
        $broadcast_domain_task_obj->{task_args} = { RUNSCHEME_GROUP_SIZE => 2 };
    } else {
        confess "No matching task";
    }

}

1;

