#
#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#    without prior written consent from NetApp, Inc.
#

package NACL::Init::Config::ONTAP::SingleClusterHardware;

use strict;
use warnings;

use NATE::ParamSet qw(param_global);;
use NATE::BaseException;

use Moo;
extends qw( NATE::Init::Config );

sub generate_resources {
    my ($self) = shift;

    # Get the value of the Resources.
    my $resources_param = param_global()->get('NODES') // NATE::BaseException->throw("Missing Mandatory NODES");
    my @resources = split(',', $resources_param);
    $self->resources(\@resources);
}

sub generate_setup_tasks {
    my ($self, %args) = @_;

    my $resources = $self->resources;

    $self->add_task(
        task_name => "NACL::Init::Tasks::ONTAP::ShutdownCold",
        alias => "shutdown_cold",
    );

    # Setenv on vsims in parallel
    $self->add_task (
        task_name => "NACL::Init::Tasks::ONTAP::Setenv",
        alias     => "ontap_setenv",
        task_args =>  {
            CONFIG_SETENV => {
                "AUTOBOOT" => "false",

                "bootarg.init.trace_on_panic" => "1",
                "panic-on-rc-shutdown-timeout?" => "true",

                "ntap.bootmenu.allowall" => "true",
                "bootarg.bootmenu.allowall"    => "true",

                "ntap.init.clearvarfsnvram"    => "true",
                "bootarg.init.clearvarfsnvram" => "true",

                "ntap.mgwd.maxlogfiles"        => "15",
                "bootarg.mgwd.maxlogfiles"        => "15",

                "ntap.init.usebootp" => "false",
                "bootarg.init.usebootp" => "false",

                "ntap.rlm.gdb" => 1,
                "bootarg.rlm.gdb" => 1,

                "ntap.init.console_muted"      => "true",
                "bootarg.init.console_muted"   => "true",

                "bootarg.dblade.aggr_default_64bit" => "false",
                "bootarg.dblade.aggr_default_32bit" => "false",

                "ntap.activatebsdport" => "false",
                "bootarg.activatebsdport" => "false",

                "bootarg.init.auto_cluster_lif.disable" => "true",

                "bootarg.init.dhcp.disable"    => "true",
            },
            UNSETENV => [
                "AUTOBOOT",
            ],
        },
    );

    # Netboot ONTAP
    $self->add_task(
        task_name => "NACL::Init::Tasks::ONTAP::NetbootONTAP",
        alias => "netboot_ontap",
    );

    # Install ONTAP
    $self->add_task(
        task_name => "NACL::Init::Tasks::ONTAP::InstallONTAP",
        alias => "install_ontap",
    );

    # Assign Disks
    $self->add_task(
        task_name => "NACL::Init::Tasks::ONTAP::AssignDisks",
        alias     => "assign_disks",
    );

    # Wipe Config
    $self->add_task(
        task_name => "NACL::Init::Tasks::ONTAP::WipeConfig",
        alias     => "wipe_config",
    );

    # Init
    $self->add_task(
        task_name => "NACL::Init::Tasks::ONTAP::Init",
        alias     => "disk_init",
    );

    # Network Config
    $self->add_task(
        task_name => "NACL::Init::Tasks::ONTAP::NetworkConfig",
        alias     => "network_config",
    );

    # ClusterSetupNoSimplicityWizard
    $self->add_task(
        task_name => "NACL::Init::Tasks::ONTAP::ClusterSetupNoSimplicityWizard",
        alias     => "cluster_setup",
    );

    # BroadcastDomain
    $self->add_task(
        task_name => "NACL::Init::Tasks::ONTAP::BroadcastDomain",
        alias     => "broadcast_domain",
    );

    # ConfigureStorageFailover
    $self->add_task(
        task_name => "NACL::Init::Tasks::ONTAP::ConfigureStorageFailover",
        alias     => "configure_sfo",
    );

    # Configure Flash Optimized
    $self->add_task(
        task_name => "NACL::Init::Tasks::ONTAP::ConfigureFlashOptimized",
        alias     => "configure_flash_optimized",
    );

    return;
}

sub generate_cleanup_tasks {
    my ($self, %args) = @_;

    # noop for this config

    return;
}

1;

