#
#   Copyright (c) 2016 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#    without prior written consent from NetApp, Inc.
#

package NACL::Init::Config::ONTAP::SingleClusterVsim;

use strict;
use warnings;

use NATE::ParamSet qw(param_global);;
use NATE::BaseException;

use Moo;
extends qw( NATE::Init::Config );

sub generate_resources {
    my ($self) = shift;

    # Get the value of the Resources.
    my $resources_param = param_global()->get('NODES') // NATE::BaseException->throw("Missing Mandatory NODES");
    my @resources = split(',', $resources_param);
    $self->resources(\@resources);
}

sub generate_setup_tasks {
    my ($self, %args) = @_;

    my $resources = $self->resources;

    # Make Vsims
    $self->add_task(
        task_name => "NACL::Init::Tasks::Vsim::Make",
        alias     => "make",
    );

    # HA create Vsims
    $self->add_task(
        task_name => "NACL::Init::Tasks::Vsim::HACreate",
        alias     => "ha_create",
    );

    # Bind Vsims
    $self->add_task(
        task_name  => "NACL::Init::Tasks::Vsim::Bind",
        alias      => "bind",
    );

    # Setenv on vsims in parallel
    $self->add_task(
        task_name => "NACL::Init::Tasks::Vsim::Setenv",
        alias     => "setenv",
    );

    # Start Vsims
    $self->add_task (
        task_name => "NACL::Init::Tasks::Vsim::Start",
        alias     => "start",
    );

    # Setenv on vsims in parallel
    $self->add_task (
        task_name => "NACL::Init::Tasks::ONTAP::Setenv",
        alias     => "ontap_setenv",
        task_args =>  {
            CONFIG_SETENV => {
                "ntap.bootmenu.allowall"       => "true",
                "bootarg.bootmenu.allowall"    => "true",
                "ntap.init.clearvarfsnvram"    => "true",
                "bootarg.init.clearvarfsnvram" => "true",
                "ntap.init.console_muted"      => "true",
                "bootarg.init.console_muted"   => "true",
                "bootarg.init.dhcp.disable"    => "true",
                "bootarg.sim"                  => "true",
                "console"                      => "comconsole"
            },
            UNSETENV => ["AUTOBOOT"],
        },
    );

    # Assign Disks
    $self->add_task(
        task_name => "NACL::Init::Tasks::ONTAP::AssignDisks",
        alias     => "assign_disks",
    );

    # Init
    $self->add_task(
        task_name => "NACL::Init::Tasks::ONTAP::Init",
        alias     => "disk_init",
    );

    # Network Config
    $self->add_task(
        task_name => "NACL::Init::Tasks::ONTAP::NetworkConfig",
        alias     => "network_config",
    );

    # ClusterSetupNoSimplicityWizard
    $self->add_task(
        task_name => "NACL::Init::Tasks::ONTAP::ClusterSetupNoSimplicityWizard",
        alias     => "cluster_setup",
    );

    # BroadcastDomain
    $self->add_task(
        task_name => "NACL::Init::Tasks::ONTAP::BroadcastDomain",
        alias     => "broadcast_domain",
    );

}

sub generate_cleanup_tasks {
    my ($self, %args) = @_;

    my $resources = $self->resources;

    # Rm the Vsims
    $self->add_task(
        task_name => "NACL::Init::Tasks::Vsim::Rm",
        alias     => "remove",
    );

}

1;

