#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#    without prior written consent from NetApp, Inc.

###
# Documentation:
# https://confluence.eng.netapp.com/x/r13S
###

package NACL::Init::Config::ONTAPSelect::Deploy::Appliance::Upgrade;

use strict;
use warnings;
use List::Util qw(first);

use NATE::ParamSet qw(param_global);

use NACL::Init::Utils qw(process_init_args);

use Moo;
extends qw(NATE::Init::Config);

sub global_args
{
    my $self = shift;

    my @definitions = (
        {
            'name'        => 'BUILDROOT',
            'description' => 'Path to the ONTAP buildroot containing the Upgrade tgz file',
            'required'    => 1,
        }
    );

    return process_init_args(\@definitions);
}

sub generate_resources {
    my $self = shift;

    # Get the value of the Resources.
    my $resources_param = param_global->get('DEPLOY_APPLIANCE') // NATE::BaseException->throw('Missing Mandatory DEPLOY_APPLIANCE');
    my @resources = split(',', $resources_param);
    $self->resources(\@resources);
}

sub generate_setup_tasks
{
    my $self = shift;

    my $global_args = $self->global_args;

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Deploy::Appliance::Upgrade',
        'alias'     => 'upgrade',
        'task_args'  => $global_args,
    );
}

sub generate_cleanup_tasks
{
    my $self = shift;
}

1;
