#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#    without prior written consent from NetApp, Inc.

###
# Documentation:
# https://confluence.eng.netapp.com/x/r13S
###

package NACL::Init::Config::ONTAPSelect::Deploy::Host::KVM;

# core
use strict;
use warnings;

# cpan
use List::MoreUtils qw(uniq);
use Moo;
extends qw(NATE::Init::Config);

# nate
use NATE::ParamSet qw(param_global);

# nacl
use NACL::Init::Utils qw(process_init_args);

sub global_args
{
    my $self = shift;

    my @definitions = (
        {
            'name'        => 'DEPLOY_APPLIANCE',
            'description' => 'ONTAP Select Deploy appliance name - Host file must exist',
            'required'    => 1,
        },
        {
            'name'        => 'ONLY_JSON',
            'description' => 'Only generate the host.json file',
            'default'     => 0,
        }
    );

    return process_init_args(\@definitions);
}

sub generate_resources {
    my $self = shift;

    # Get the value of the Resources.
    my $resources_param = param_global->get('HOSTS') // NATE::BaseException->throw('Missing Mandatory HOSTS');
    my @resources = split(m/,|:/, $resources_param);
    $self->resources(\@resources);
}

sub generate_setup_tasks
{
    my $self = shift;

    my $global_args = $self->global_args;

    $self->add_task(
        'task_name'  => 'NACL::Init::Tasks::ONTAPSelect::Server::Power',
        'alias'      => 'power_on_host',
        'task_args'  => {'HOST_OS' => 'linux', 'POWER_ACTION' => 'on'},
    );

    # Does not work in parallel
    $self->add_task(
        'task_name'  => 'NACL::Init::Tasks::ONTAPSelect::Deploy::Host::KVM::GenerateJSON',
        'alias'      => 'generate_json',
        'task_args'  => $global_args,
        'run_scheme' => 'all_together',
    );

    if ($global_args->{'ONLY_JSON'})
    {
        return 1;
    }

    # Does not work in parallel
    $self->add_task(
        'task_name'  => 'NACL::Init::Tasks::ONTAPSelect::Deploy::Host::Add',
        'alias'      => 'host_add',
        'task_args'  => $global_args,
        'run_scheme' => 'all_together',
    );
}

sub generate_cleanup_tasks
{
    my $self = shift;

    my $global_args = $self->global_args;
}

1;
