#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#    without prior written consent from NetApp, Inc.

###
# Documentation:
# https://wikid.netapp.com/w/AERO/ONTAP_Select_Projects/init.thpl#NACL::Init::Config::ONTAPSelect::ESX::Host
###

package NACL::Init::Config::ONTAPSelect::ESX::Host;

use strict;
use warnings;

use NATE::ParamSet qw(param_global);

use NACL::Init::Utils qw(process_init_args);

use Moo;
extends qw(NATE::Init::Config);

sub global_args
{
    my $self = shift;

    my @definitions = (
        {
            'name'        => 'HOST_OS',
            'description' => 'OS for the host (ESXi55 or ESXi60)',
            'required'    => 1,
        },
        {
            'name'        => 'ESX_VSWITCH_NAME',
            'description' => 'Name of the ONTAPSelect vSwitch to create on the host',
            'default'     => 'vSwitch0',
        },
    );

    return process_init_args(\@definitions);
}

sub generate_resources {
    my $self = shift;

    # Get the value of the Resources.
    my $resources_param = param_global->get('HOSTS') // NATE::BaseException->throw('Missing Mandatory HOSTS');
    my @resources = split(m/,|:/, $resources_param);
    $self->resources(\@resources);
}

sub generate_setup_tasks
{
    my $self = shift;

    my $global_args = $self->global_args;

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::ESX::Host::Add',
        'alias'     => 'add',
        'task_args'  => $global_args,
    );
    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::ESX::Host::FirewallEnableRulesets',
        'alias'     => 'firewall_enable_rulesets',
        'task_args'  => $global_args,
    );
    # $self->add_task(
    #     'task_name' => 'NACL::Init::Tasks::ONTAPSelect::ESX::Host::NetworkCreate',
    #     'alias'     => 'network_create',
    #     'task_args'  => $global_args,
    # );
    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::ESX::Host::LicenseAssign',
        'alias'     => 'license_assign',
        'task_args'  => $global_args,
    );
    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::ESX::Host::DatastoreCreate',
        'alias'     => 'datastore_create',
        'task_args'  => $global_args,
    );
}

sub generate_cleanup_tasks
{
    my $self = shift;

    my $global_args = $self->global_args;

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::ESX::Node::PowerOff',
        'alias'     => 'power_off',
        'task_args'  => $global_args,
    );
    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::ESX::Node::Delete',
        'alias'     => 'delete',
        'task_args'  => $global_args,
    );
    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::ESX::Host::DatastoresDestroy',
        'alias'     => 'datastore_destroy',
        'task_args'  => $global_args,
    );
    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::ESX::Host::NetworkRemove',
        'alias'     => 'network_remove',
        'task_args'  => $global_args,
    );
    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::ESX::Host::Destroy',
        'alias'     => 'destroy',
        'task_args'  => $global_args,
    );
}

1;
