#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#    without prior written consent from NetApp, Inc.

###
# Documentation:
# https://wikid.netapp.com/w/AERO/ONTAP_Select_Projects/init.thpl#NACL::Init::Config::ONTAPSelect::ESX::Node
###

package NACL::Init::Config::ONTAPSelect::ESX::Node;

use strict;
use warnings;

use NATE::ParamSet qw(param_global);

use NACL::Init::Utils qw(process_init_args);

use Moo;
extends qw(NATE::Init::Config);

sub global_args
{
    my $self = shift;

    my @definitions = (
        {
            'name'        => 'ESXADMIN_PATH',
            'description' => 'Path to esxadmin_cli',
            'default'     => '/x/eng/rlse/DOT/devN/offtap/lib/deploy/src/esxadmin/src/esxadmin_cli',
        },
        {
            'name'        => 'MEDIATOR_IP',
            'description' => 'IP address of an ONTAPSelect mediator',
        },
        {
            'name'        => 'CLUSTER_NAME',
            'description' => 'Name of the ONTAPSelect cluster',
            'default'     => eval(
                 "my \$suffix = qx(cat /dev/urandom | tr -cd 'A-F0-9' | head -c 8);
                 chomp(\$suffix); # mutatable
                 return \"C\$suffix\";"
            ),
        },
        {
            'name'        => 'VCENTER',
            'description' => 'vCenter the node will reside on',
        },
    );

    my $set = process_init_args(\@definitions);

    # Must always be this for this config
    $set->{'NODE_TYPE'} = 'esx';

    return $set;
}

sub generate_resources
{
    my ($self) = shift;

    # Get the value of the Resources.
    my $resources_param = param_global->get('NODES') // NATE::BaseException->throw('Missing Mandatory NODES');
    my @resources = split(',', $resources_param);
    $self->resources(\@resources);
}

sub generate_setup_tasks
{
    my $self = shift;

    my $global_args = $self->global_args;

    $self->add_task(
        'task_name'  => 'NACL::Init::Tasks::ONTAPSelect::Node::MediatorSetup',
        'alias'      => 'mediator_setup',
        'run_scheme' => 'all_together',
        'task_args'  => $global_args,
    );
    $self->add_task(
        'task_name'  => 'NACL::Init::Tasks::ONTAPSelect::Node::GenerateJSON',
        'alias'      => 'generate_json',
        'run_scheme' => 'all_together',
        'task_args'  => $global_args,
    );
    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::ESX::Node::Create',
        'alias'     => 'create',
        'task_args'  => $global_args,
    );
    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::ESX::Node::PowerOn',
        'alias'     => 'power_on',
        'task_args'  => $global_args,
    );
    $self->add_task(
        'task_name'  => 'NACL::Init::Tasks::ONTAPSelect::Node::ClusterValidate',
        'alias'      => 'cluster_validate',
        'run_scheme' => 'all_together',
        'task_args'  => $global_args,
    );
    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Node::DiskAssign',
        'alias'     => 'disk_assign',
        'task_args'  => $global_args,
    );
    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Node::LicenseInstall',
        'alias'     => 'license_install',
        'task_args'  => $global_args,
    );
}

sub generate_cleanup_tasks
{
    my $self = shift;

    my $global_args = $self->global_args;

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::ESX::Node::PowerOff',
        'alias'     => 'power_off',
        'task_args'  => $global_args,
    );
    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::ESX::Node::Delete',
        'alias'     => 'delete',
        'task_args'  => $global_args,
    );
}

1;
