#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#    without prior written consent from NetApp, Inc.

###
# Documentation:
###

package NACL::Init::Config::ONTAPSelect::KVM::Host::Datastore;

use strict;
use warnings;

use Hostrec;
use NATE::ParamSet qw(param_global);

use NACL::Init::Utils qw(process_init_args);

use Moo;
extends qw(NATE::Init::Config);

use constant MAX_RETRY_COUNT => 3;

sub generate_resources {
    my $self = shift;

    # Get the value of the Resources.
    my $resources_param = param_global->get('HOSTS') // NATE::BaseException->throw('Missing Mandatory HOSTS');
    my @resources = split(m/,|:/, $resources_param);
    $self->resources(\@resources);
}

sub generate_setup_tasks
{
    my $self = shift;

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::KVM::Host::DomainsDestroy',
        'alias'     => 'domain_destroy',
    );

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::KVM::Host::DomainsUndefine',
        'alias'     => 'domain_undefine',
    );

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::KVM::Host::DatastoresDestroy',
        'alias'     => 'datastore_destroy',
    );

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::KVM::Host::DatastoreCreate',
        'alias'     => 'datastore_create',
    );
}

sub generate_cleanup_tasks
{
    my $self = shift;

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::KVM::Host::DomainsDestroy',
        'alias'     => 'domain_destroy',
    );

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::KVM::Host::DomainsUndefine',
        'alias'     => 'domain_undefine',
    );

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::KVM::Host::DatastoresDestroy',
        'alias'     => 'datastore_destroy',
    );
}

1;
