#   Copyright (c) 2016 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAP::BroadcastDomain;

use strict;
use warnings;
use NATE::Log qw(log_global);
use Data::Dumper;
use NACL::Init::Tasks::VsimUtils;
use Carp qw(confess);
use NACL::C::Node;
use NACL::C::NetworkPortBroadcastDomain;
use NACL::CS::NetworkPortBroadcastDomain;

use Moo;
extends qw(NATE::Init::Task);

has +run_scheme => (
    is => 'ro',
    default => "all_together"
);

sub execute {
    my ($self, %args) = @_;

    my $node_ref = $args{resource};
    my @node_names = @{$node_ref};

    my ($task, %task_args);
    $task = $self->task_args;
    %task_args = %$task;

    my @nodes = ();
    foreach my $node (@node_names) {
        push(@nodes, NACL::C::Node->new(node => $node));
    }

    # fetch all BDs
    my @bds = NACL::CS::NetworkPortBroadcastDomain->fetch(command_interface => $nodes[0]);

    # get the mgmt ports to split
    my %mgmt_ports = ();
    my $new_bd_name;
    foreach my $node (@nodes) {
        my $hostrec = $node->hostrec();
        my @nets    = $hostrec->net_sort(include=>{nettype=>'management'});
        foreach my $net (@nets) {
            next unless ($net->{broadcast_domain});
            $mgmt_ports{$node->node().':'.$net->{interface}}++;
            $new_bd_name = "mgmt_bd_".$net->{mtu};
        }
    }

    # split the management ports away from the Default broadcast domain
    # if we have one that needs to be created.
    if ($new_bd_name) {
        NACL::C::NetworkPortBroadcastDomain->split(
            command_interface => $nodes[0],
            'ipspace' => "Default",
            'broadcast-domain' => "Default",
            'new-broadcast-domain' => $new_bd_name,
            'ports' => [keys %mgmt_ports],
        );
    }
}

1;
