#
#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

# @unittest test/lib/NACL/UnitTest/Init/Tasks/ONTAP/configure_security_certificates.thpl

package NACL::Init::Tasks::ONTAP::ConfigureSecurityCertificates;

use Moo;

extends 'NATE::Init::Task';

BEGIN {undef *with}

use NATE::BaseException qw(:try);
use NATE::Log qw(log_global);
use NACL::C::Node;
use NACL::CS::Vserver;
use NACL::C::SecurityCertificate;
use NATE::Inc qw(get_file);

has +run_scheme => (
    is => 'ro',
    default => "all_together"
);

has cert_files => (
    is => 'rw',
    default => sub {
        ["certificates/common/netapp_corp_issuing_ca_1.crt",
         "certificates/common/netapp_corp_root_ca.crt",
         "certificates/eng.netapp.com/netapp_corp_issuing_ca_1.crt",
         "certificates/eng.netapp.com/sgws_data_smoke.crt",
         "certificates/eng.netapp.com/data_smoke.crt",
         "certificates/rtp.openeng.netapp.com/netapp_corp_issuing_ca_1.crt"]
    }
);

sub execute {
    my ($self, %args) = @_;

    my $resources = $self->resources();

    if (!scalar(@{$resources})) {
        NATE::BaseException->throw("No resources were provided in 'resources'");
    }

    # we only need to do this on one of the nodes since it applies to the
    # entire cluster (what about multi cluster cases?)
    my $node_obj = NACL::C::Node->new(name => $resources->[0]);

    my $cert_files = $self->cert_files();

    my $cert_contents = {};
    foreach my $cert_file (@{$cert_files}) {
        $cert_contents->{$cert_file} = get_file($cert_file);
    }

    my ($admin_vs) = NACL::CS::Vserver->fetch(command_interface => $node_obj,
                                              filter => {type => "admin"});

    my @sorted_cert_keys = sort (keys %{$cert_contents});

    foreach my $cert_file (@sorted_cert_keys) {
        log_global()->log("Installing Certificate '$cert_file'");
        my $certificate = $cert_contents->{$cert_file};
        NACL::C::SecurityCertificate->install(command_interface => $node_obj,
                                              type => "server-ca",
                                              certificate => $certificate,
                                              vserver => $admin_vs->vserver());
    }
}

1;
