#   Copyright (c) 2016 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAP::FakeInit;

use strict;
use warnings;
use NATE::Log qw(log_global);
use Data::Dumper;
use NACL::Init::Tasks::VsimUtils;
use Carp qw(confess);
use NACL::Transit;
use Connectrec;
use NACL::C::Node;
use Tie::IxHash;
use NACL::APISet::Exceptions::ResponseException;


use Moo;
extends qw(NATE::Init::Task);

# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

use NATE::BaseException qw(:try);

sub execute {

    my ($self, %args) = @_;

    my @nodes = $args{resource};

    my ($task, %task_args);
    $task = $self->task_args;
    %task_args = %$task;

    foreach my $node (@nodes) {
        log_global()->comment("Waiting for loader prompt");
        my $t = NACL::Transit->new(name => $node);
        $t->change_state(to => "FIRMWARE");

        my ($host, $conn, $api_set_firmware_obj);
        try {
            $host = Hostrec->new( id => $node );
            $conn = Connectrec->new( host => $host, id => "console" );
            $api_set_firmware_obj = NACL::APISet->new(
                connection => $conn,
                interface  => 'CLI',
                category   => 'Node',
                set        => 'Firmware',
            );
        } catch NATE::BaseException with {
            my $ex = shift;
            $ex->throw();
        };

        # boot at the console
        log_global()->comment("boot");
        try {
            my @match_table = (
                "Press Ctrl-C"  => "\cc",
                "Selection"     => "24\/6",
                ".*without zeroing disks.*" => "yes",
                "Type yes to confirm and continue {yes}:" => "exit",
            );
            # TODO: seeing if calling "boot" is the problem
            my $resObj = $api_set_firmware_obj->execute_raw_command(
                            command                 => "boot",
                            'connectrec-match_table'=> \@match_table,
                            'connectrec-timeout'    => "900",
                            'connectrec-prompts'    => ['login:'],
            );
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            log_global()->comment("Error is: " . $exception_object->text());
            my $resObj = $exception_object->get_response_object();
            log_global()->comment("Raw output was:\n" . $resObj->get_raw_output());
        };
    }
}

1;
