#
#   Copyright (c) 2016 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::Init::Tasks::ONTAP::Init;

use strict;
use warnings;

use Carp qw(confess);
use NACL::Transit;
use Hostrec;

use Moo;
extends qw(NATE::Init::Task);

use constant defaults => {
    ONTAP_DISK_INIT_TYPE => "24\/6",
    ONTAP_DISK_INIT_TIMEOUT => 900,
};

# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

sub execute {
    my ($self, %args) = @_;

    my $node = $args{resource};

    my $task_args = $self->task_args;

    my $init_option  = $task_args->{ONTAP_DISK_INIT_TYPE} //
        confess "Missing required disk init type option 'ONTAP_DISK_INIT_TYPE'";
    my $init_timeout = $task_args->{ONTAP_DISK_INIT_TIMEOUT};

    # transition to the BOOT MENU
    my $host = Hostrec->new( id => $node );
    my $t = NACL::Transit->new(name => $host);
    $t->change_state(to => "BOOT_MENU");
    my $conn = $t->get_connection();

    # initialize the disks and end up at the login prompt
    $conn->execute(command => "$init_option",
                   match_table => [
                       "without zeroing disks" => "yes",
                       "Zero the disks if needed" => "yes",
                       "are you sure" => "yes",
                       "Install a new file system" => "yes",
                       "Type yes to confirm and continue {yes}:" => "exit",
                   ],
                   timeout => $init_timeout,
                   prompts => ['login:']);

    return;
}

1;
