#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAP::ShutdownCold;

use strict;
use warnings;

use Hostrec;
use Connectrec;
use NATE::ParamSet qw(param_global);
use NACL::APISet;
use NACL::Transit;

use Moo;
extends qw(NATE::Init::Task);

# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

sub execute {
    my ($self, %opts) = @_;

    my $node =  $opts{resource};

    # this will allow us to boot other users off the service processors
    param_global()->set(key => "NATE_FORCE_SP_LOGIN",
                        value => "y");

    my $h = Hostrec->new(id => $node);

    # connect to the service processor and perform a power cycle
    my $sp_apiset = NACL::APISet->new(
        hostobj => $h,
        category => "Node",
        interface => "CLI",
        set => "SP",
        connid => "sp_login",
        conntype => "libssh-persist",
    );

    # power cycle the node
    my $sp_conn = $sp_apiset->get_connection();
    $sp_conn->execute(
        command => "system power cycle",
        'match_table' => [
            'Continue\?' => "y"
        ],
    );

    # transit to the firmware (not using the service processor)
    my $t = NACL::Transit->new(name => $h);
    $t->change_state(to => "FIRMWARE");
}

1;
