#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::Deploy::Cluster::Add;

# core
use strict;
use warnings;
use Carp         qw(croak);
use Data::Dumper qw(Dumper);
use File::Spec::Functions qw(catfile);

# nate
use Hostrec;
use NATE::Events qw(eventloop);
use NATE::Log    qw(log_global);

# cpan
use Moo;
extends qw(NATE::Init::Task);
use JSON::XS qw(decode_json);

# nate
use NATE::ParamSet qw(param_global);

# nacl
use NACL::Init::Utils::JSON qw(decode_json_from_file);
use NACL::Init::Utils::ONTAPSelect::Deploy qw(get_deploy_apiclient);

sub execute
{
    my ($self, %args) = @_;

    my $log       = log_global;
    my %task_args = %{$self->task_args};

    my $client = get_deploy_apiclient($task_args{'DEPLOY_APPLIANCE'});

    my $struct = decode_json_from_file(catfile(param_global->get('TOP_LOGDIR') . '/cluster.json'));
    if (! $struct)
    {
        croak('Unable to find a JSON file to use!  It should have been either '
            . 'the set by GenerateJSON task or by passing a JSON_FILE parameter.');
    }

    $client->POST('path' => 'clusters', 'data' => $struct);

    $log->comment("Polling for the state of the $struct->{name} cluster until it is online");

    my $online = 0;
    while(! $online)
    {
        $log->comment('Waiting 30 more seconds for a state check...');
        eventloop('seconds' => 30);

        my $cluster = eval {
            $client->GET('path' => 'clusters/' . $struct->{'name'});
        };
        if (my $err = $@)
        {
            $log->error("Error encountered while getting information for the $struct->{name} cluster: $err");
        }
        if (!$cluster)
        {
            $log->error("Unable to find the $struct->{name} cluster yet.");
        }
        else
        {
            $log->comment("$struct->{name} cluster state is: $cluster->{state}.");
            if ($cluster->{'state'} eq 'online')
            {
                $online = 1;
            }
            elsif ($cluster->{'state'} eq 'create_failed')
            {
                croak("Failed to create the cluster: " . Dumper($cluster));
            }
        }
    }
}

1;
