#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::Deploy::Cluster::ConfigureConsole;

use strict;
use warnings;
use Carp qw(croak);

use Hostrec;
use NATE::Log qw(log_global);

use Moo;
extends qw(NATE::Init::Task);

use NACL::Init::Utils::ONTAPSelect::Deploy qw(get_create_json
                                              get_deploy_apiclient);

sub execute
{
    my ($self, %args) = @_;

    my $log       = log_global;
    my @nodes     = @{$args{'resource'}};
    my %task_args = %{$self->task_args};

    my $client = get_deploy_apiclient($task_args{'DEPLOY_APPLIANCE'});

    my $struct = get_create_json();
    if (! $struct && !$task_args{'CLUSTER_NAME'})
    {
        croak('Unable to find a JSON file to use!  It should have been either '
            . 'the set by GenerateJSON task or by passing a JSON_FILE parameter. '
            . 'You would not need one if you passed CLUSTER_NAME for this task.');
    }

    my $cluster_name = $struct->{'name'} ? $struct->{'name'} : $task_args{'CLUSTER_NAME'};

    for my $node (@nodes)
    {
        my $hostrec_node = Hostrec->new('id' => $node);
        my $port = $hostrec_node->{'CONSOLE_PORT'};
        eval {
            $client->PUT(
                'path' => "clusters/$cluster_name/nodes/$node/serial/console",
                'data' => {
                    'enable' => \1, # allows json encode as true
                    'port'   => $port,
                },
            );
        };
        if ($@ && $@ =~ m/port is identical/)
        {
            $log->comment("$node serial port is already configured for $port port...nothing to do on this one");
            next;
        }
        elsif ($@)
        {
            croak($@);
        }
        else
        {
            $log->comment("Successfully configured $node serial port to listen on port $port");
        }
    }
}

1;
