#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::Deploy::Cluster::InstallLicense;

use strict;
use warnings;
use MIME::Base64 qw(encode_base64);

use Hostrec;
use NATE::Log qw(log_global);

use Moo;
extends qw(NATE::Init::Task);

use NACL::Init::Utils::ONTAPSelect::Deploy qw(get_deploy_apiclient);

use constant 'defaults' => {
    'BUILDROOT'    => '/x/eng/bbrtp/daemon/DOT/R9.2x/cit-ok',
};

sub execute
{
    my ($self, %args) = @_;

    my $log       = log_global;
    my @nodes     = @{$args{'resource'}};
    my %task_args = %{$self->task_args};

    my $client = get_deploy_apiclient($task_args{'DEPLOY_APPLIANCE'});

	# IMPORTANT:
    # /usr/software python2.7.5 is the only python 2 for rhel6 and rhel 7 with appropriate modules
    # WTB - Containers
    # python2.7.5 /x/eng/bbrtp/daemon/DOT/R9.2x/cit-ok/offtap/lib/license_generator/license_generator.py capacity 320000022 50 --premium

    my $serial = 320000022; # beginning of the itnernal serial numbers
    for my $node (@nodes)
    {
    	my $hostrec = Hostrec->new('id' => $node);
	    my $license = qx(/usr/software/bin/python2.7.5 $task_args{'BUILDROOT'}/offtap/lib/license_generator/license_generator.py capacity $serial 50);
	    $client->POST(
	        'path' => "licenses",
	        'data' => {
	        	'ontap_user'     => $hostrec->default_username,
	        	'ontap_password' => $hostrec->default_password,
	        	'license_file'   => encode_base64($license), # has to be encoded
	        },
	    );
	    # will throw exception when error, assume success otherwise
	    $log->comment("Successfully installed license: $license");

	    $serial++;
	}
}

1;
