#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::Deploy::Host::Add;

# core
use strict;
use warnings;
use Carp qw(croak);
use Data::Dumper qw(Dumper);
use File::Spec::Functions qw(catfile);

# nate
use Hostrec;
use NATE::Events   qw(eventloop);
use NATE::Log      qw(log_global);

# cpan
use Moo;
extends qw(NATE::Init::Task);

# nate
use NATE::ParamSet qw(param_global);

# nacl
use NACL::Init::Utils::JSON qw(decode_json_from_file);
use NACL::Init::Utils::ONTAPSelect::Deploy qw(get_deploy_apiclient);

sub execute
{
    my ($self, %args) = @_;

    my @hosts = @{$self->resources};
    my $log = log_global;
    my %task_args = %{$self->task_args};

    my $client = get_deploy_apiclient($task_args{'DEPLOY_APPLIANCE'});

    # Enable Dark Content for Host Discovery
    $log->comment('Enabling Dark Content: Host_Discovery');
    my $dc_response = $client->PATCH(
        'path' => 'system',
        'data' => {
            'flags'=> [{'name' => 'Host_Discovery', 'value' => 'True'}],
        },
    );
    $log->comment('Response: ' . $dc_response);

    # Lets assume that all hosts are on the same VC and domain
    my $hostrec_host = Hostrec->new('id' => $hosts[0]);
    my $domain       = $hostrec_host->{'DOMAIN'};

    my @fq_hosts;
    for my $host (@hosts)
    {
        push(@fq_hosts, $host . '.' . $domain);
    }

    my $data = decode_json_from_file(catfile(param_global->get('TOP_LOGDIR'), 'host.json'));

    $log->comment('Sending to PUT hosts api: ' . Dumper($data));

    # Add Hosts
    while(1)
    {
        my $response = eval {
            $client->PUT(
                'path' => 'hosts',
                'data' => $data,
            );
        };
        if (my $err = $@)
        {
            if ($err =~ m/concurrent operation limit/m)
            {
                $log->comment('Deploys concurrent operation limit hit, waiting 10 seconds...');
                eventloop('seconds' => 10);
                next;
            }
            else
            {
                croak($err);
            }
        }
        else
        {
            $log->comment('Response: ' . $response);
            last;
        }
    };

    # Wait for them to be authenticated
    while (1)
    {
        my $authenticated = 1;
        # eval mostly to ensure the 10 second eventloop
        eval {
            my $added_hosts = $client->GET('path' => 'hosts');
            my %host_data = map {$_->{'host'} => $_} @{$added_hosts->{'hosts'}};
            $log->comment('x' x 78);
            for my $host (@fq_hosts)
            {
                $log->comment('Checking for host: ' . $host);

                if (! $host_data{$host})
                {
                    $log->comment("$host not found yet");
                    $authenticated = 0;
                }
                elsif ($host_data{$host}->{'status'} ne 'authenticated')
                {
                    $log->comment("$host status is not yet authenticated, it is $host_data{$host}->{'status'} instead.");
                    $authenticated = 0;
                }
                else
                {
                    $log->comment("$host is authenticated");
                }
            }
            $log->comment('x' x 78);
        };
        if (my $err = $@)
        {
            $log->error($err);
        }

        last if $authenticated;

        eventloop('seconds' => 10);
    }
}

1;
