#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::Deploy::Host::ESX::GenerateJSON;

# core
use strict;
use warnings;
use Carp qw(croak);
use Cwd qw(abs_path getcwd);
use File::Copy;
use File::Spec::Functions qw(catfile);
use IO::File;

# cpan
use Moo;
extends qw(NATE::Init::Task);
use JSON::XS qw(encode_json);

# nate
use NATE::Log qw(log_global);
use NATE::ParamSet qw(param_global);

# nacl
use NACL::Init::Utils::STB qw(in_a_stb copy_file_to_stb);
use NACL::Init::Utils::VMware qw(rot13);

sub execute
{
    my ($self, %args) = @_;

    my $log = log_global;
    my @hosts = @{$args{'resource'}};

    # Lets assume that all hosts are on the same VC and domain
    my $hostrec_host = Hostrec->new('id' => $hosts[0]);
    my $vcenter      = $hostrec_host->{'VCENTER'};
    my $domain       = $hostrec_host->{'DOMAIN'};

    my $hostrec_vc = Hostrec->new('id' => $vcenter);

    my @fq_hosts;
    for my $host (@hosts)
    {
        push(@fq_hosts, $host . '.' . $domain);
    }

    my $username = $hostrec_vc->{'SDOTADMIN_PASSWORD'} ? 'sdotadmin' : $hostrec_vc->default_username;
    my $password = $hostrec_vc->{'SDOTADMIN_PASSWORD'} // rot13('string' => $hostrec_vc->default_password);

    my %data = (
        'host_list' => \@fq_hosts,
        'username'  => $username,
        'password'  => $password,
        'vcenter'   => $hostrec_vc->default_ip,
    );

    my $filepath = catfile(param_global->get('TOP_LOGDIR'), 'host.json');
    $filepath = abs_path($filepath);
    my $json = encode_json(\%data);
    my $fh = IO::File->new($filepath, 'w');
    my $failure = $!;
    if ($fh)
    {
        print({$fh} $json);
        $fh->close;
        $log->log($json);
        $log->log("Wrote file to: $filepath");

        # Move json file to testbed directory if within an STB testbed
        if (in_a_stb())
        {
            copy_file_to_stb('source' => $filepath, 'dest' => 'hosts.json');
        }
    }
    else
    {
        if ($failure)
        {
            croak("Failed to write json file with error: $failure");
        }
        else
        {
            croak("Failed to write json file for an unknown reason.");
        }
    }
}

1;
