#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::ESX::Host::DatastoreDestroy;

use strict;
use warnings;
use Carp qw(croak);

# vmware
use VMware::VICommon;
use VMware::VIMRuntime;
use VMware::VIM25Stub;
use VMware::VILib;
use VMware::VIExt;

use Moo;
extends qw(NATE::Init::Task);
# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

use Hostrec;
use NATE::BaseException qw(:try);
use NATE::Log           qw(log_global);

use constant 'defaults' => {
    'VCENTER'     => '',
    'HOST_DOMAIN' => '',
};

sub execute
{
    my ($self, %args) = @_;

    my $node      = $args{'resource'};
    my $log       = log_global;
    my %task_args = %{$self->task_args};

    my $host = $node;
    $host =~ s/[a-z]$//;
    my $hostrec = Hostrec->new('id' => $host); # for data within host file
    $host .= '.' . $task_args{'HOST_DOMAIN'};

    my $hostrec_vc = Hostrec->new('id' => $task_args{'VCENTER'});
    $log->info("Logging in to $task_args{VCENTER} vcenter...");
    my $vc = Vim::login(
        'service_url' => 'https://' . $task_args{'VCENTER'} . '/sdk/vimService',
        'user_name'   => $hostrec_vc->default_username,
        'password'    => $hostrec_vc->default_password,
    );
    $log->info("Logged in to $task_args{VCENTER}.");

    my $hostSystem = $vc->find_entity_view(
        'view_type' => 'HostSystem',
        'filter'    => {'name' => $host},
    );

    my $datastoreSystem = $vc->get_view('mo_ref' => $hostSystem->configManager->datastoreSystem);

    if (!$hostSystem->datastore)
    {
        $log->info("No datastores to cleanup, exiting.");
        return 1;
    }

    for my $dataStore (@{$hostSystem->datastore})
    {
        $datastoreSystem->RemoveDatastore('datastore' => $dataStore);
    }
}

1;
