#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::ESX::Host::NetworkRemove;

use strict;
use warnings;
use Carp qw(croak);

# vmware
use VMware::VICommon;
use VMware::VIMRuntime;
use VMware::VIM25Stub;
use VMware::VILib;
use VMware::VIExt;

use Moo;
extends qw(NATE::Init::Task);
# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

use Hostrec;
use NATE::BaseException qw(:try);
use NATE::Log           qw(log_global);

use constant 'defaults' => {
    'VCENTER'          => '',
    'ESX_VSWITCH_NAME' => 'vSwitchSdot',
    'HOST_DOMAIN'      => '',
};

sub execute
{
    my ($self, %args) = @_;

    my $host = $args{'resource'};
    my $log = log_global;
    my %task_args = %{$self->task_args};

    my $hostrec = Hostrec->new('id' => $host); # for data within host file
    $host .= '.' . $hostrec->{'HOST_DOMAIN'};

    $log->info("Logging in to $hostrec->{VCENTER} vcenter...");
    my $hostrec_vc = Hostrec->new('id' => $hostrec->{'VCENTER'});
    my $vc = Vim::login(
        'service_url' => 'https://' . $hostrec->{'VCENTER'} . '/sdk/vimService',
        'user_name'   => $hostrec_vc->default_username,
        'password'    => $hostrec_vc->default_password,
    );
    $log->info("Logged in to $hostrec->{VCENTER}.");

    my $hostSystem = $vc->find_entity_view(
        'view_type' => 'HostSystem',
        'filter'    => {'name' => $host},
    );

    my $networkSystem = $vc->get_view('mo_ref' => $hostSystem->configManager->networkSystem);

    eval {
        $networkSystem->RemoveVirtualSwitch(
            'vswitchName' => $hostrec->{'ESX_VSWITCH_NAME'},
        );
    };
    if (my $err = $@)
    {
        if ($err =~ m/NotFoundFault/)
        {
            $log->info("No vSwitch named $hostrec->{ESX_VSWITCH_NAME} found, exiting.");
            return 1;
        }
        else
        {
            croak($err);
        }
    }
}

1;