#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::ESX::Node::ConfigureConsole;

use strict;
use warnings;
use Carp qw(croak);

# vmware
use VMware::VICommon;
use VMware::VIMRuntime;
use VMware::VIM25Stub;
use VMware::VILib;
use VMware::VIExt;

use NATE::Events qw(eventloop);
use NATE::Log qw(log_global);

use Moo;
extends qw(NATE::Init::Task);

use NACL::Init::Utils::VMware qw(rot13);

use constant 'defaults' => {
    'VCENTER' => '',
};

sub execute
{
    my ($self, %args) = @_;

    my $node = $args{'resource'};

    my $log = log_global;

    my %task_args = %{$self->task_args};

    my $host = $node;
    $host =~ s/[a-z]$//;

    my $hostrec_node = Hostrec->new('id' => $node);
    my $hostrec_host = Hostrec->new('id' => $hostrec_node->simhost);
    my $vcname = $hostrec_host->{'VCENTER'};
    my $hostrec_vc   = Hostrec->new('id' => $vcname);

    $log->info("Logging in to $vcname vcenter...");

    # my $hostrec_vc   = Hostrec->new('id' => $task_args{'VCENTER'});
    my $vc = Vim::login(
        'service_url' => 'https://' . $vcname . '/sdk/vimService',
        'user_name'   => $hostrec_vc->default_username,
        'password'    => rot13('string' => $hostrec_vc->default_password),
    );
    $log->info("Logged in to $vcname.");

    my $vm = $vc->find_entity_view(
        'view_type' => 'VirtualMachine',
        'filter'    => {'name' => $node},
    );
	if (! defined($vm))
	{
		croak('Unable to find VM ' . $node . ' on vCenter ' . $vcname);
	}

	my $devices = $vm->config->hardware->device;

	# have to do this twice for some reason, first fails
	for my $i (0 .. 1)
	{
		eval {
			for my $device (@$devices) {
				next if ($device->deviceInfo->label ne 'Serial port 1');

				my $backing = VirtualSerialPortURIBackingInfo->new(
					'serviceURI' => 'telnet://:' . $hostrec_node->{'CONSOLE_PORT'},
					'direction'  => 'server',
					'proxyURI'   => '',
				);

				my $changeDevice = VirtualSerialPort->new(
					'backing'       => $backing,
					'deviceInfo'    => $device->deviceInfo,
					'controllerKey' => $device->controllerKey,
					'key'           => $device->key,
					'unitNumber'    => $device->unitNumber,
					'yieldOnPoll'   => $device->yieldOnPoll,
				);

				my $spec = VirtualMachineConfigSpec->new(
					'changeVersion' => $vm->config->changeVersion,
					'deviceChange'  => [
					   VirtualDeviceConfigSpec->new(
							'operation' => VirtualDeviceConfigSpecOperation->new('edit'),
							'device'    => $changeDevice,
						),
					],
				);
				$vm->ReconfigVM('spec' => $spec);
				$log->info("Virtual machine '" . $vm->name . "' reconfigured successfully.");
			}
		};
		if (my $err = $@)
		{
			if ($err =~ m/connection control operation failed/)
			{
				if ($i == 0)
				{
					$log->info('Found expected error around connection control operation failed, trying again.');
				}
				else
				{
					croak('Found ' . $err . ' again, and this was the second try.  Giving up.');
				}
			}
			else
			{
				croak("Unexpected error: $err");
			}
		}
		else
		{
			last;
		}
	}
}

1;
