#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::ESX::Node::Create;

use strict;
use warnings;
use Carp qw(croak);
use Cwd  qw(abs_path);

use NATE::Log      qw(log_global);
use NATE::ParamSet qw(param_global);

use Moo;
extends qw(NATE::Init::Task);

use NACL::Init::Utils::VMware qw(wait_for_task);

use constant 'defaults' => {
    'ESXADMIN_PATH' => '/x/eng/rlse/DOT/devN/offtap/lib/deploy/src/esxadmin/src/esxadmin_cli',
    'VCENTER'       => '',
};

sub execute
{
    my ($self, %args) = @_;

    my $node      = $args{'resource'};
    my $log       = log_global;
    my $param     = param_global;
    my %task_args = %{$self->task_args};
    my $host      = $node;

    $host =~ s/[a-z]$//;

    my $file = File::Spec->catfile($param->get('TOP_LOGDIR'), "$node.json");
    $file = abs_path($file);

    my $hostrec_vc = Hostrec->new('id' => $task_args{'VCENTER'});

    my @command = (
        $task_args{'ESXADMIN_PATH'},
        '-s', $task_args{'VCENTER'},
        '-h', $host,
        '-u', $hostrec_vc->default_username,
        '-p', $hostrec_vc->default_password,
        'vm',
        'create',
        "--json=$file",
    );

    $log->info("Running: " . join(' ', @command));
    my $child = NATE::Child::ExecLocal->new(
        'command'        => \@command,
        'on_termination' => [sub {
            my $child_obj = shift;
            my $exit_status = NATE::Child::decode_status_raw($child_obj->{exit_status});

            if ($exit_status)
            {
                my @output = <CMD_OUT>;
                my $str = join('', @output);
                croak("\nExit Status: $exit_status\nOutput:\n$str");
            }
        }],
    );
    $child->wait;
}

1;
