#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::ESX::Node::Delete;

use strict;
use warnings;
use Carp qw(croak);

# vmware
use VMware::VICommon;
use VMware::VIMRuntime;
use VMware::VIM25Stub;
use VMware::VILib;
use VMware::VIExt;

use NATE::Events qw(eventloop);
use NATE::Log qw(log_global);

use NACL::Init::Utils::VMware qw(wait_for_task rot13);

use Moo;
extends qw(NATE::Init::Task);

sub execute
{
    my ($self, %args) = @_;

    my $node = $args{'resource'};

    my $log = log_global;

    my $host = $node;
    $host =~ s/[a-z]$//;

    my $hostrec = Hostrec->new('id' => $host);

    $log->info("Logging in to $hostrec->{'VCENTER'} vcenter...");
    my $hostrec_vc = Hostrec->new('id' => $hostrec->{'VCENTER'});
    my $vc = Vim::login(
        'service_url' => 'https://' . $hostrec->{'VCENTER'} . '/sdk/vimService',
        'user_name'   => $hostrec_vc->default_username,
        'password'    => rot13('string' => $hostrec_vc->default_password),
    );
    $log->info("Logged in to $hostrec->{'VCENTER'}.");

    my $vm = eval {
        $vc->find_entity_view(
            'view_type' => 'VirtualMachine',
            'filter'    => {'name' => $node},
        );
    };
    if (!$vm)
    {
        $log->info("No node found to delete, returning.");
        return 1;
    }

    wait_for_task(
        'message' => "Succcessfully removed $node.",
        'task'    => $vm->Destroy_Task,
    );
}

1;
