#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::ESX::Node::PowerOff;

use strict;
use warnings;
use Carp qw(croak);

# vmware
use VMware::VICommon;
use VMware::VIMRuntime;
use VMware::VIM25Stub;
use VMware::VILib;
use VMware::VIExt;

use NATE::Events qw(eventloop);
use NATE::Log qw(log_global);

use Moo;
extends qw(NATE::Init::Task);

use NACL::Init::Utils::VMware qw(rot13);

sub execute
{
    my ($self, %args) = @_;

    my $node = $args{'resource'};

    my $log = log_global;

    my $host = $node;
    $host =~ s/[a-z]$//;

    my $hostrec = Hostrec->new('id' => $host);

    $log->info("Logging in to $hostrec->{'VCENTER'} vcenter...");
    my $hostrec_vc = Hostrec->new('id' => $hostrec->{'VCENTER'});
    my $vc = Vim::login(
        'service_url' => 'https://' . $hostrec->{'VCENTER'} . '/sdk/vimService',
        'user_name'   => $hostrec_vc->default_username,
        'password'    => rot13('string' => $hostrec_vc->default_password),
    );
    $log->info("Logged in to $hostrec->{'VCENTER'}.");

    my $vm = $vc->find_entity_view(
        'view_type' => 'VirtualMachine',
        'filter'    => {'name' => $node},
    );

    if (! defined($vm))
    {
        $log->info("Unable to find $node VM on vCenter $hostrec->{'VCENTER'}, nothing to power off. Exiting.");
        return 1;
    }

    if ($vm->runtime->powerState->val eq 'poweredOff')
    {
        $log->info("$node is already powered off, exiting.");
        return 1;
    }

    $vm->PowerOffVM;

    $log->info("Waiting for VM to be powered off...");
    while ($vm->runtime->powerState->val ne 'poweredOff')
    {
        eventloop('seconds' => 5);
        $log->info("Still waiting for VM to be powered off...");
        $vm->update_view_data; # get any state changes, etc.
    }
    $log->info("$node is powered off.");
}

1;
