#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::ESX::Node::PowerOn;

use strict;
use warnings;
use Carp qw(croak);

# vmware
use VMware::VICommon;
use VMware::VIMRuntime;
use VMware::VIM25Stub;
use VMware::VILib;
use VMware::VIExt;

use NATE::Events qw(eventloop);
use NATE::Log qw(log_global);

use Moo;
extends qw(NATE::Init::Task);

use constant 'defaults' => {
    'VCENTER' => '',
};

sub execute
{
    my ($self, %args) = @_;

    my $node = $args{'resource'};

    my $log = log_global;

    my %task_args = %{$self->task_args};

    my $host = $node;
    $host =~ s/[a-z]$//;

    $log->info("Logging in to $task_args{VCENTER} vcenter...");
    my $hostrec_vc = Hostrec->new('id' => $task_args{'VCENTER'});
    my $vc = Vim::login(
        'service_url' => 'https://' . $task_args{'VCENTER'} . '/sdk/vimService',
        'user_name'   => $hostrec_vc->default_username,
        'password'    => $hostrec_vc->default_password,
    );
    $log->info("Logged in to $task_args{VCENTER}.");

    my $vm = $vc->find_entity_view(
        'view_type' => 'VirtualMachine',
        'filter'    => {'name' => $node},
    );

    if (! defined($vm))
    {
        croak("Unable to find $node VM on vCenter $task_args{VCENTER}.");
    }

    $vm->PowerOnVM;

    $log->info("Waiting for VM to be powered on...");
    while ($vm->runtime->powerState->val ne 'poweredOn')
    {
        eventloop('seconds' => 5);
        $log->info("Still waiting for VM to be powered on...");
        $vm->update_view_data; # get any state changes, etc.
    }
    $log->info("$node is powered on.");
}

1;
