#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::KVM::Host::DatastoreCreate;

use strict;
use warnings;
use Carp qw(croak);

use Moo;
extends qw(NATE::Init::Task);
# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

use Hostrec;
use NATE::BaseException qw(:try);
use NATE::Events        qw(eventloop);
use NATE::Log           qw(log_global);

sub execute
{
    my ($self, %args) = @_;

    my $host = $args{'resource'};
    my $log = log_global;

    my $host_short = $host;

    my $hostrec = Hostrec->new('id' => $host);

    my $conn = Connectrec->new(
        'id'       => $host,
        'host'     => $host,
        'template' => 'kvm_ssh',
    );

    my $device_path;
    if ($hostrec->{'POOL_DISK_TYPE'} eq 'LUN')
    {
        # Just to show full output of multipath -ll before parsing
        $conn->execute(
            'command'      => "multipath -ll",
            'check_status' => 1,
        );

        # Domain exists at this point, undefine it.
        my $device_id = $conn->execute(
            'command'      => "multipath -ll | grep -B1 size=[0-9]*T | head -1 | sed 's/ .*//'",
            'check_status' => 1,
        );
        chomp($device_id);
        $log->info("Device ID: $device_id");

        $device_path = "/dev/mapper/$device_id";
    }
    else
    {
        $device_path = $hostrec->{'POOL_DISK_DEVICE'};
    }

    $conn->execute(
        'check_status' => 1,
        'command'      => [
            'virsh', 'pool-define-as', 'ontap-select-storage-pool', 'logical',
            '--source-dev', $device_path,
            '--target', '/dev/ontap-select-storage-pool',
        ],
    );

    $log->info("Taking a 20 second break...");
    eventloop('seconds' => 20);

    # Cleanup old LVMs on device_path
    my $lsblk_output = $conn->execute(
        'check_status' => 1,
        'command'      => ['lsblk', '--raw', $device_path,],
    );
	my @lvms;
    for my $line (split(/\n/, $lsblk_output))
    {
        if ($line =~ m/lvm/)
        {
            my @cols = split(/\s+/, $line);
            push(@lvms, $cols[0]);
        }
    }

    for my $lvm (@lvms)
    {
        $conn->execute(
            'check_status' => 1,
            'command'      => ['dmsetup', 'remove', $lvm],
        );
    }

    $log->info("Taking a 20 second break again...");
    eventloop('seconds' => 20);

    if ($hostrec->{'POOL_DISK_TYPE'} eq 'LUN')
    {
        $conn->execute(
            'command'      => ['virsh', 'pool-build', 'ontap-select-storage-pool'],
            'check_status' => 1,
        );
    }
    else
    {
        $conn->execute(
            'command'      => ['virsh', 'pool-build', 'ontap-select-storage-pool', '--overwrite'],
            'check_status' => 1,
        );
    }

    $conn->execute(
        'command'      => ['virsh', 'pool-start', 'ontap-select-storage-pool'],
        'check_status' => 1,
    );

    $conn->execute(
        'command'      => ['virsh', 'pool-autostart', 'ontap-select-storage-pool'],
        'check_status' => 1,
    );

}

1;
