#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::KVM::Host::DatastoresDestroy;

use strict;
use warnings;
use Carp qw(croak);

use Moo;
extends qw(NATE::Init::Task);
# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

use Hostrec;
use NATE::BaseException qw(:try);
use NATE::Log           qw(log_global);

sub execute
{
    my ($self, %args) = @_;

    my $resource = $args{'resource'};
    my $log  = log_global;

    my $hostrec = Hostrec->new('id' => $resource);

    # It may be a node resource
    my $host = $hostrec->simhost // $resource;

    my $conn = Connectrec->new(
        'id'       => $host,
        'host'     => $host,
        'template' => 'kvm_ssh',
    );

    my $output = $conn->execute(
        'command'      => ['virsh', 'pool-list', '--all'],
        'check_status' => 1,
    );

    my @pools;
    for my $line (split(/\n/, $output))
    {
        next if ($line =~ m/^ Name/);
        if ($line =~ m/^\s+([A-Za-z0-9\-_]+)/)
        {
            my $match = $1;
            chomp($match);
            push(@pools, $1);
        }
    }

    for my $pool (@pools)
    {
        next if (!$pool); # Just in case
        $conn->execute(
            'command' => ['virsh', 'pool-destroy', $pool],
        );

        $conn->execute(
            'command' => ['virsh', 'pool-undefine', $pool],
        );

        $conn->execute(
            'command' => ['vgremove', '-f', $pool],
        );

        $conn->execute(
            'command' => ['rm', '-rf', "/dev/$pool"],
        );
    }
}

1;
