#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::KVM::Node::Create;

use strict;
use warnings;
use Carp qw(croak);
use Cwd  qw(abs_path);

use NATE::Log      qw(log_global);
use NATE::ParamSet qw(param_global);

use NACL::Init::Utils::ONTAPSelect::KVMadmin qw(get_host_sub_shell);

use Moo;
extends qw(NATE::Init::Task);

use constant 'defaults' => {
    'KVMADMIN_PATH' => '/x/eng/rlse/ontapdeploy/devN/offtap/lib/deploy/src/kvmadmin/kvmadmin_cli.py',
};

sub execute
{
    my ($self, %args) = @_;

    my $node = $args{'resource'};

    my $log   = log_global;
    my $param = param_global;

    my %task_args = %{$self->task_args};

    my $host = $node;
    $host =~ s/[a-z]$//;

    # Used for credentials down below
    my $conn = Connectrec->new(
        'id'       => $host,
        'host'     => $host,
        'template' => 'kvm_ssh',
        'connect'  => 0,
    );

    my $file = File::Spec->catfile($param->get('TOP_LOGDIR'), "$node.json");
    $file = abs_path($file);

    my $exp = get_host_sub_shell(
        'script'   => $task_args{'KVMADMIN_PATH'},
        'host'     => $host,
        'sub'      => 'vm',
        'password' => $conn->password,
    );

    $exp->send("create json=$file\n");
    $exp->expect(30, 'DataONTAPv-kvm.raw.tar');
    $exp->expect(1800, 'VM created');
    $exp->expect(30, "KVMadmin host $host vm >");
}

1;
