#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::KVM::Node::DomainUndefine;

use strict;
use warnings;
use Carp qw(croak);

use Params::Validate qw(validate :types);

use Connectrec;
use NATE::Log qw(log_global);

use Moo;
extends qw(NATE::Init::Task);
# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}
use NATE::BaseException qw(:try);

sub execute
{
    my ($self, %args) = @_;

    my $host = $args{'resource'};

    my $log   = log_global;

    my %task_args = %{$self->task_args};

    my $conn = Connectrec->new(
        'id'       => $host,
        'host'     => $host,
        'template' => 'kvm_ssh',
    );

    my $output = $conn->execute(
        'command'    => ['virsh', 'list', '--state-shutoff'],
        'must_match' => '\s*-\s+'.$node.'\s+shut off',
    );
    my @stopped_domains;
    for my $line (split(/\n/, $output))
    {
        if ($line =~ m/^\s+[0-9]+\s+(\w+)/)
        {
            push(@stopped_domains, $1);
        }
    }

    # Domain exists at this point, undefine it.
    for my $domain (@stopped_domains)
    $log->info("Found shut down domain '$domain', undefining it.");
    $conn->execute(
        'command'      => ['virsh', 'undefine', $node],
        'must_match'   => "Domain $domain has been undefined",
        'check_status' => 1,
    );
}

1;
