#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::KVM::Node::PoolDestroy;

use strict;
use warnings;
use Carp qw(croak);

use Params::Validate qw(validate :types);

use Connectrec;
use NATE::Log qw(log_global);

use Moo;
extends qw(NATE::Init::Task);
# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}
use NATE::BaseException qw(:try);

use constant 'defaults' => {
    'POOL_NAME' => 'ontap-select-storage-pool',
};

sub execute
{
    my ($self, %args) = @_;

    my $node = $args{'resource'};

    my $log   = log_global;

    my %task_args = %{$self->task_args};

    my $host = $node;
    $host =~ s/[a-z]$//;

    my $conn = Connectrec->new(
        'id'       => $host,
        'host'     => $host,
        'template' => 'kvm_ssh',
    );

    try {
        $conn->execute(
            'command'    => ['virsh', 'pool-list'],
            'must_match' => '\s*'.$task_args{'POOL_NAME'}.'\s+active',
        );
    } catch NATE::BaseException with {
        my $e = shift;
        if ($e->text =~ m/Command output did not match/)
        {
            $log->info('Did not find an active pool named ' . $task_args{'POOL_NAME'} . ', nothing to do.');
            exit(0);
        }
        else
        {
            croak($e->text);
        }
    };

    # Domain exists at this point, undefine it.
    $log->info("Found an active pool named '".$task_args{'POOL_NAME'}."', destroying it.");
    $conn->execute(
        'command'      => ['virsh', 'pool-destroy', $task_args{'POOL_NAME'}],
        'must_match'   => "Pool $task_args{POOL_NAME} destroyed",
        'check_status' => 1,
    );
}

1;
