#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::KVM::Node::PowerOn;

use strict;
use warnings;
use Carp qw(croak);

use NATE::Log    qw(log_global);
use NATE::Events qw(eventloop);

use NACL::Init::Utils::ONTAPSelect::KVMadmin qw(get_host_sub_shell);

use Moo;
extends qw(NATE::Init::Task);

use constant 'defaults' => {
    'KVMADMIN_PATH' => '/x/eng/rlse/ontapdeploy/devN/offtap/lib/deploy/src/kvmadmin/kvmadmin_cli.py',
};

sub execute
{
    my ($self, %args) = @_;

    my $node = $args{'resource'};

    my $log   = log_global;

    my %task_args = %{$self->task_args};

    my $host = $node;
    $host =~ s/[a-z]$//;

    # Used for credentials down below
    my $conn = Connectrec->new(
        'id'       => $host,
        'host'     => $host,
        'template' => 'kvm_ssh',
        'connect'  => 0,
    );

    my $exp = get_host_sub_shell(
        'script'   => $task_args{'KVMADMIN_PATH'},
        'host'     => $host,
        'sub'      => 'vm',
        'password' => $conn->password,
    );

    $exp->send("start $node\n");
    $exp->expect(30, "KVMadmin host $host vm >");

    # Poll until it is stopped
    while (1)
    {
        $exp->send("show $node\n");
        $exp->expect(30, '-re', '^.*"state":.*$');
        if ($exp->exp_match =~ m/poweredon/)
        {
            $log->info("$node is powered on");
            last;
        }
        eventloop('seconds' => 5);
    }
}

1;
