#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::Node::LicenseInstall;

use strict;
use warnings;
use Carp                  qw(croak);
use Data::Dumper          qw(Dumper);
use File::Spec::Functions qw(catfile);
use IO::File;
use List::Util            qw(first);

use NATE::Child::ExecLocal;
use NATE::Events qw(eventloop);
use NATE::Inc    qw(find_file);
use NATE::Log    qw(log_global);

use NACL::Init::Utils::ONTAPSelect::KVMadmin qw(get_host_sub_shell);

use Moo;
extends qw(NATE::Init::Task);

use constant 'defaults' => {
    'LICENSE_PATH'        => '/x/eng/ctl/share/sdot_licenses',
    'LICENSE_FILE_PREFIX' => 'NLF-',
    'LICENSE_FILE_SUFFIX' => '.txt.json',
    'LICENSE_START'       => '32000000',
    'NODES'               => '',
    'ADMIN_PASSWORD'      => 'netapp1!',
};

sub execute
{
    my ($self, %args) = @_;

    my $node = $args{'resource'};

    my $log = log_global;

    my %task_args = %{$self->task_args};

    # Get index of this node for $n
    my @nodes = split(/,/, $task_args{'NODES'});
    my $idx = first {$nodes[$_] eq $node} 0 .. $#nodes;

    $log->info("Found node as idx $idx");

    my $license = $task_args{'LICENSE_START'} . $idx + 32;

    my $file = catfile($task_args{'LICENSE_PATH'}, $task_args{'LICENSE_FILE_PREFIX'} . $license . $task_args{'LICENSE_FILE_SUFFIX'});
    my $fh = IO::File->new($file, 'r');
    croak("Unable to open: $file") if (!$fh);
    my $api = <$fh>;
    $fh->close;

    # has wrapping single quotes, remove em
    $api =~ s/^'//;
    $api =~ s/'$//;

    $log->info($api);

    my $child = NATE::Child::ExecLocal->new(
        'on_termination' => [\&_child_done],
        'command'        => [
            '/usr/software/test/bin/ontapi',
            $node,
            'admin',
            $task_args{'ADMIN_PASSWORD'},
            '-xs',
            '-i',
            $api,
        ],
    );
    $child->wait;
}

sub _child_done
{
    my $child_obj = shift;

    my $exit_status = NATE::Child::decode_status_raw($child_obj->{'exit_status'});
    log_global->comment("Decoded exit_status_raw: $exit_status");

    if ($exit_status != 0)
    {
        croak("Failed running with exit_status: $exit_status");
    }
}

1;
