#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::Node::MediatorSetup;

use strict;
use warnings;
use Carp qw(croak);

use HTTP::Request::Common;
use JSON::XS;
use LWP::UserAgent;

use NATE::Events             qw(eventloop);
use NATE::Log                qw(log_global);
use NATE::ServiceAPI::KeyVal qw(keyval_global);

use Moo;
extends qw(NATE::Init::Task);

# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}
use NATE::BaseException qw(:try);

use constant 'defaults' => {
    'MEDIATOR_IP'  => undef,
    'CLUSTER_NAME' => undef,
};

sub execute
{
    my($self, %args) = @_;

    my $log = log_global;

    my @nodes = @{$args{'resource'}};

    my %task_args = %{$self->task_args};

    if (scalar(@nodes) != 2)
    {
        $log->info('You only need to use an external mediator for a 2 Node sDOT cluster -- Nothing to do here.');
        return 1;
    }

    croak("Must provide a MEDIATOR_IP!")  if (!$task_args{'MEDIATOR_IP'});
    croak("Must provide a CLUSTER_NAME!") if (!$task_args{'CLUSTER_NAME'});

    $ENV{'PERL_LWP_SSL_VERIFY_HOSTNAME'} = 0; # ignore certificate check

    my $base_uri = 'https://' . $task_args{'MEDIATOR_IP'} . ':3000';

    my $create_tgt_req = HTTP::Request::Common::PUT(
        sprintf('%s/targets/%s', $base_uri, $task_args{'CLUSTER_NAME'}),
    );
    $create_tgt_req->authorization_basic('', 'netapp');

    my $response = LWP::UserAgent->new->request($create_tgt_req);

    if ($response->is_success)
    {
        my $json = eval {
            JSON::XS::decode_json($response->content);
        };
        if (my $err = $@)
        {
            croak('Response content from mediator was not json.  Content was ' . $response->content);
        }

        if (! $json->{'iqn'})
        {
            croak('Got HTTP 200 back, but did not contain an iqn in the content.  Content was: ' . $response->content);
        }

        keyval_global->hset('hash_name' => 'sdot', 'key' => 'ISCSI_XMEDIATOR_TGT', 'val' => $json->{'iqn'});

        $log->info('Successfully created mailbox disk on mediator: ' . $json->{'iqn'});

        return 1;
    }
    else
    {
        croak('Failed to create mailbox disk on mediator.  HTTP code was ' . $response->code . ' and content was ' . $response->content);
    }

}

1;
