#   Copyright (c) 2016 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::Vsim::Bind;

use strict;
use warnings;
use NATE::Log qw(log_global);
use NACL::Init::Tasks::VsimUtils qw(get_vsim_details_from_node_name prepare_client_apiset);
use Carp qw(confess);
use NATE::Util::Sockets qw(get_fqdn);
use NATE::BaseException qw(try);
use NACL::C::Node;

use Moo;
extends qw(NATE::Init::Task);

use constant defaults => {
    VSIM_SCRIPT      => "/usr/software/test/bin/vsim",
    VSPC_SCRIPT      => "/usr/software/test/bin/vSPC.py",
    BIND_SUDO_USER   => 1,
    VSIM_BIND_IDBG   => undef,
    VSIM_BIND_DOMAIN => undef,
};

sub execute {

    my ($self, %args) = @_;

    my @nodes = $args{resource};

    my ($task, %task_args);
    $task = $self->task_args;
    %task_args = %$task;

    my $vsim_tool   = $task->{VSIM_SCRIPT};
    my $vspc_script = $task->{VSPC_SCRIPT};
    my $user        = $task->{BIND_SUDO_USER};
    my $idbg        = $task->{VSIM_BIND_IDBG};
    my $domain      = $task->{VSIM_BIND_DOMAIN};
    my $blade;

    log_global()->comment("Bind smokes vsims");

    foreach my $node (@nodes) {
        my $node_obj = NACL::C::Node->new( node => $node );
        my $node_name = $node;
        my ($last_char) = ($node_name =~ /(.)$/);
        my $pod = get_fqdn($node_obj->hostrec()->{POD})->[0]->{fqdn};
        my $vspc_server = $node_obj->hostrec()->{VSPC_SERVER};
        my $vspc_port = $node_obj->hostrec()->{VSPC_PORT};

        # identify the vsim user to operate on for this
        my $vsim_details = get_vsim_details_from_node_name($node);
        my $vsim = $vsim_details->{user};
        my $vsim_bind_user_command = "";
        my $sudo_user = "";

        if(defined($idbg)) {
            $idbg = "-idbg $idbg";
        } else {
            $idbg = "";
        }
        if(defined($domain)) {
            $blade = "$vsim_details->{blade}.$domain";
        } else {
            $blade = "";
        }
        if($user == 1) {
            # smoke configuration
            $sudo_user = "sudo -i -u ".$vsim_details->{user};
        } else {
            # CTL5 configuration
            $vsim_bind_user_command = "-u ".$vsim;
        }
        if(!defined($pod)) {
            $pod = "";
        }

        log_global()->comment("Preparing client APISet");

        my $c_api = prepare_client_apiset();

        my $normalized_node_name = $node;
        $normalized_node_name =~ s/-/_/g;

        my $status = 0;

        my $command = "$sudo_user $vsim_tool bind $vsim_bind_user_command -esx -smoke $idbg ".
                      "-vspc $vspc_server ".
                      "-vspc_port $vspc_port ".
                      "-simdir /u/".$vsim."/vsims ".
                      "$vsim_details->{id} $blade $pod $last_char";
        my ($output) = $c_api->execute_raw_command(command => $command,
                                                   get_status => \$status);

        if ($status ne 0) {
            NATE::BaseException->throw("Encountered a bad status while binding the VSim $node_name");
        }
    }
}

1;
