#   Copyright (c) 2016 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::Vsim::HACreate;

use strict;
use warnings;
use NATE::Log qw(log_global);
use Data::Dumper;
use NACL::Init::Tasks::VsimUtils qw(get_vsim_details_from_node_name prepare_client_apiset);
use NACL::C::Node;
use Carp qw(confess);
use NATE::BaseException qw(try);
use Moo;
extends qw(NATE::Init::Task);

use constant defaults => {
    VSIM_SCRIPT       => "/usr/software/test/bin/vsim",
    NICS            => "5",
    VSPC_SCRIPT     => "/usr/software/test/bin/vSPC.py",
    ICMAC_PREFIX    => "00:50:56",
};

has +run_scheme => (
    is => 'ro',
    default => "parallel_group_of_size_n"
);

sub execute {

    my ($self, %args) = @_;

    my $node = $args{resource};
    my @nodes = @{$node};

    my ($task, %task_args);
    $task = $self->task_args;
    %task_args = %$task;

    my $Vsim_tool       = $task->{"VSIM_SCRIPT"};
    my $Vspc_script     = $task->{"VSPC_SCRIPT"};
    my $nics            = $task->{"NICS"};
    my $icmac_prefix    = $task->{"ICMAC_PREFIX"};

    log_global()->comment("HA CREATE");
    # loop through each node and create a node object
    my $node_obj;
    my @node_objs;
    foreach my $node (@nodes) {
        push(@node_objs, NACL::C::Node->new(node => $node));
    }

    # generate dynamic ic mac addresses
    my $icmac = "";
    if (scalar(@node_objs) > 1) {
        my @icmacs = ();
        log_global()->comment("Generating MAC addresses from prefix $icmac_prefix");
        foreach my $n (@node_objs) {
            my $mgmt_ip = $n->hostrec()->{MGMT_IP};
            if ($mgmt_ip =~ /^([0-9]+?)\.([0-9]+?)\.([0-9]+?)\.([0-9]+?)$/) {
                my $mac = $icmac_prefix . ":" . sprintf("%02x:%02x:%02x", $2, $3, $4);
                push(@icmacs, $mac);
            } else {
                NATE::BaseException("Dynamic IC MAC address calculation failed for VSIM HA, mgmt ip $mgmt_ip in nate host file is not proper");
            }
        }
        $icmac = "-icmac " . join(',', @icmacs);
    }

    my $icport = ["e0e", "e0f", "e0g", "e0h"]->[$nics - 5];

    my $vsim_var = "";
    my $vsim_details = get_vsim_details_from_node_name($nodes[0]);
    my $vsim_user = $vsim_details->{user};
    my $ha_name = $vsim_details->{blade};
    my $vsim_dir = "/u/".$vsim_user."/vsims/";
    my $sudo_user = "/usr/software/bin/sudo -i -u ".$vsim_details->{user};
    foreach my $node (@nodes) {
        # identify the vsim user to operate on for this
        my $vsim_details = get_vsim_details_from_node_name($node);
        $ha_name = $ha_name.$vsim_details->{instance};
        $vsim_var = $vsim_var." ".$vsim_details->{id};
    }

    my $cmd = $ha_name." ".$vsim_var;
    log_global()->comment("Preparing client APISet");
    my $c_api = prepare_client_apiset();

    my $status = 0;
    $c_api->execute_raw_command(command => "$sudo_user $Vsim_tool hacreate -smoke -icport $icport -noicupdate $icmac -simdir $vsim_dir $cmd",
                                    get_status => \$status);
    if ($status ne 0) {
        NATE::BaseException->throw("Encountered a bad status while making vsim $node");
    }
}

1;
