#   Copyright (c) 2016 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Topology::ONTAP::VolumeGranularEncryption;

use strict;
use warnings;
use NATE::ParamSet qw(param_global);
use NATE::BaseException;
use Moo;
extends qw( NATE::Init::Config );

sub generate_resources {
    my ($self) = shift;
    my $resources_param;

    # Get resource values.
    my $nodes = param_global()->get('NODES');
    # CIT_cr_suite passes in NODES and DST_NODES
    my $dst_nodes = param_global()->get('DST_NODES');

    if(!$nodes && !$dst_nodes) {
        NATE::BaseException->throw("Missing NODES and DST_NODES");
    }
    # if both $nodes and $dst_nodes are defined
    elsif($nodes && $dst_nodes) {
        # join these two nodes to setup the cluster
        $resources_param = join(',',$nodes, $dst_nodes);
    }
    # if not defined $nodes
    elsif(!$nodes && $dst_nodes) {
        $resources_param = $dst_nodes;
    }
    # if not defined $dst_nodes
    else {
        $resources_param = $nodes;
    }
    my @resources = split(',', $resources_param);
    $self->resources(\@resources);
}

sub generate_setup_tasks {

    my($self, %args) = @_;

    my $resources = $self->resources;

    # add the topology task - volume granular encryption setup
    my $task_obj = $self->add_task (
                       task_name => "NACL::Init::Tasks::ONTAP::VolumeGranularEncryption",
                       alias     => "vge_setup",
                       resources => $resources,
                   );
}

sub generate_cleanup_tasks {

    my ($self, %args) = @_;

}

1;
