#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#    without prior written consent from NetApp, Inc.

package NACL::Init::Utils;

use strict;
use warnings;
use List::Util qw(first);

use NATE::ParamSet qw(param_global);

BEGIN {
    require Exporter;
    our @ISA = qw(Exporter);
    our @EXPORT_OK = qw(
        process_init_args
    );
}

# Temporary subroutine to use until better parameter/argument support in NATE::Init::Config/Task
sub process_init_args
{
    # NATE::ParamSet definition
    # http://web.eng.netapp.com/engineering/nate/releases/current/html/NATE/ParamSet.html#parameters
    my $definitions = shift;

    param_global->parameters('definitions' => $definitions);
    param_global->process;

    my @params = map {$_->{'name'}} @{$definitions};

    # Only pull out the parameters defined within the ParamSet definition
    my @paramset = param_global->walk(
        'filter' => sub {
            my ($element) = @_;
            return first {$_ eq $element->longname} @params;
        },
    );

    # Turn in to k/v hash
    my %set = map {$_->{'key'} => $_->{'value'}} @paramset;

    return \%set;
}

1;
