#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#    without prior written consent from NetApp, Inc.

package NACL::Init::Utils::ONTAPSelect::KVMadmin;

use strict;
use warnings;

use Expect;
use Params::Validate qw(validate :types);

use NATE::Log qw(log_global);
use NATE::ParamSet qw(param_global);

BEGIN {
    require Exporter;
    our @ISA = qw(Exporter);
    our @EXPORT_OK = qw(
        get_host_sub_shell
        node_exists
    );
}

sub get_host_sub_shell
{
    my %opts = validate(@_, {
        'script'   => {'type' => SCALAR},
        'host'     => {'type' => SCALAR},
        'sub'      => {'type' => SCALAR},
        'password' => {'type' => SCALAR},
    });

    log_global->log("Executing: bash -c 'export HOME=".param_global->get('TOP_LOGDIR')." && $opts{'script'}'");

    my $exp = Expect->spawn("bash -c 'export HOME=".param_global->get('TOP_LOGDIR')." && $opts{'script'}'");
    $exp->expect(30, ('KVMadmin >'));

    $exp->send("host $opts{host}\n");

    $exp->expect(30,
        [ qr/Are you sure you want to continue connecting/ => sub {
                my $exp = shift;
                $exp->send("yes\n");
                exp_continue;
            }
        ],
        [ qr/password:/ => sub {
                my $exp = shift;
                $exp->send($opts{'password'} . "\n");
                exp_continue;
            }
        ],
        "KVMadmin host $opts{host} >",
    );

    $exp->send($opts{'sub'} . "\n");
    $exp->expect(30, "KVMadmin host $opts{host} $opts{sub} >");

    return $exp;
}

sub node_exists
{
    my %opts = validate(@_, {
        'exp'  => {'type' => OBJECT, 'isa' => 'Expect'},
        'node' => {'type' => SCALAR},
    });

    $opts{'exp'}->send("show $opts{node}\n");
    my $found = $opts{'exp'}->expect(10, '"name":', "VM \"$opts{node}\" not found");

    if ($found == 1)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

1;
