#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#    without prior written consent from NetApp, Inc.

package NACL::Init::Utils::STB;

# core
use strict;
use warnings;
use Carp qw(croak);
use Cwd qw(abs_path getcwd);
use File::Basename qw(dirname);
use File::Copy qw(copy);
use File::Path qw(make_path);

# cpan
use Params::Validate qw(validate :types);

# nate
use NATE::ParamSet qw(param_global);

BEGIN {
    require Exporter;
    our @ISA = qw(Exporter);
    our @EXPORT_OK = qw(
        in_a_stb
        copy_file_to_stb
    );
}

sub in_a_stb
{
    my $param = param_global;
    if ($param->get('SITE') && $param->get('LAB') && $param->get('TESTBED'))
    {
        my $cwd = getcwd;
        my $top_cwd = $cwd;
        $top_cwd =~ s/^.*\///;
        # Make sure this is being run from the testbed itself
        if ($top_cwd eq $param->get('TESTBED'))
        {
            # in a shared testbed
            return 1;
        }
    }

    # not in a shared testbed
    return 0;
}

sub copy_file_to_stb
{
    my %opts = validate(@_, {
        'source' => {'type' => SCALAR},
        'dest'   => {'type' => SCALAR},
    });

    croak("You are not in a Shared Testbed!") if (! in_a_stb());

    if ($opts{'dest'} =~ m/^\//)
    {
        croak("You cannot copy outside of the shared testbed!");
    }

    # cwd will be base of the STB
    my $cwd = getcwd;

    # if directories passed, make sure they exist
    my $dest = "$cwd/$opts{'dest'}";
    if (my $dir_path = dirname($dest))
    {
        make_path($dir_path);
    }

    copy($opts{'source'}, $dest) or croak("Unable to copy file: $!");

    return 1;
}

1;
