#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::MTask::CLC::ComponentLifeCycle;

use Moose;
use Moose::Util::TypeConstraints;

use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

################# Attribute Section ########################
#
# number of components to go through a in a cycle
#
has 'number_of_lifecycles' => (
    is => 'ro',
    isa => 'Int',
    default => 1,
);

#
# the component
#
has 'component' => (
    is => 'ro',
    isa => 'Str',
    required => 1,
    default => '',
);

subtype 'method_attributes' => (
    as 'HashRef[Str]'
);

subtype 'methods' => (
    as 'HashRef[method_attributes]',
);

#
# component methods
#
has 'component_methods' => (
    is => 'ro',
    isa => 'Str | HashRef[methods]',
    required => 1,
    default => 'all',
);

#
# pause between methods In seconds
#
has 'pause_between_methods' => (
    is => 'ro',
    isa => 'Int',
    default => 30,
);

#
# do not cleanup on failure
#
has 'retain_config' => (
    is => 'ro',
    isa => 'Bool',
    default => 0,
);

#
# retries_on_failure of a method
#
has 'retries_on_failure' => (
    is => 'ro',
    isa => 'Int',
    default => 3,
);

#
# time_between_retries in seconds
#
has 'time_between_retries' => (
    is => 'ro',
    isa => 'Int',
    default => 60,
);

subtype 'Execution_Style' => (
    as 'Str',
    where ($_ eq 'serial' || $_ eq 'parallel'),
    message { "" };
);
#
# execution - serial or parallel
#
has 'execution_style' => (
    is => 'ro',
    isa 'Execution_Style',
    default => 'parallel',
);
#
# Protocol on how to report as it goes through the cycle
#
enum 'Reporting_Protocols', [qw(pass error warn config)];

#
# reporting protocol => continue on failure, error or warn
#
has 'reporting_protocol' => (
    is => 'ro',
    isa => 'Reporting_Protocols',
    default => 'pass',
);

####################### Method section #################################

sub BUILD {
    $Log->enter() if $may_enter;
    my $self = $_[0];

    $Log->exit() if $may_exit;
}

sub run {
    $Log->enter() if $may_enter;
    my $self = $_[0];

    $Log->exit() if $may_exit;
}

sub stop {
    $Log->enter() if $may_enter;
    my $self = $_[0];

    $Log->exit() if $may_exit;
}

sub wait_for_completion {
    $Log->enter() if $may_enter;
    my $self = $_[0];

    $Log->exit() if $may_exit;
}

sub status {
    $Log->enter() if $may_enter;
    my $self = $_[0];

    $Log->exit() if $may_exit;
}

##################### Private Methods #################################
sub _get_exec_mode
{
    my %exec_mode;
    $exec_mode{'serial'} = 0;
    $exec_mode{'parallel'} = 1;

    return $exec_mode{$_[0]};
}
1;
