# $Id: //depot/prod/test/UM6.0_integ/lib/NACL/MTask/DFM/StorageService.pm# $
#
## Copyright (c) 2012 NetApp, Inc., All Rights Reserved
##   Any use, modification, or distribution is prohibited
##   without prior written consent from NetApp, Inc.
##

#
## @summary StorageService Task Module
## @author vinothg, dl-mpo-nacl-dev@netapp.com
## @change
##
## @status shared
## @pod here

=head1 NAME

NACL::MTask::DFM::StorageService

=head1 DESCRIPTION

This task is to be used for
- Creating a StorageService with two node mirror topology.
- Creating a StorageService with N nodes fanout mirror topology.
- Deleting a StorageService

Creating a StorageService with two node mirror topology involves
-Find the Source and destination aggregate's in the nodes w.r.t to the user input.
-Create a Vserver in the src and dst aggr with default vserver properties.
-Creating a Volume in the src  vservers with the default volume properties with the count passed by the user.
-Creating the resource groups by adding aggr/vserver details.
-Creating the service workflow info
-Construct the topology info using connections and node details.
-Add Cluster into the Bard machine
-Creating a StorageService with the above inputs.
-Subscribe the volumes as member to the storage services.
-Conform the subscription to initialize the relation.
-Validating the storage service relation as per the artifacts passed.

Creating a StorageService with N nodes fanout mirror topology involves
-This can be done for N nodes similar to 2 nodes.
-In addition creating intercluster peering between source and destination cluster

Deleting an StorageService involves
-Destroy the storage service which in turn unsubscribe the members
-Delete and validate the volume and  vserver based on the flag value passed as the input.
-Delete the group based on the flag value passed as input
-Delete the intercluster peering if created based on the flag passed as input

=head1 ATTRIBUTES

=head2 src_cluster

cluster to be added to the dfm

=head2 dst_cluster

cluster to be added to the dfm

=head2 src_aggr_obj

object return by the Aggregate->fetch() task

=head2 dst_aggr_objs

Array of object return by the  Aggregate->fetch() task

=head2 src_vserver_obj

objects return by the Vserver->create() task

=head2 dst_vserver_obj

objects return by the Vserver->create() task

=head2 src_volume_objs

object return by the Volume->create() task

=head2 dst_volume_objs

Array of objects return by the Volume->create() task

=head2 resource_group

resource group used to subscribe the storage serive

=head2 service_workflow

The storage service releationship created based on the service workflow type

=head2 topology_info

Topology hashref used for creating storage service

=back

=cut

=head1 METHODS


=head2 create_storage_service_and_subscribe_volume

        $storageservice_obj = NACL::MTask::DFM::StorageService->create_storage_service_and_subscribe_volume(
                        command_interface  => $ci,
                        src_cluster_interface => $src_cluster,
			dst_cluster_interface => $dst_cluster,
			inter_cluster_flag => '1',
			volume_count       =>'2',
			dest_node_count    =>'2',);
(Class method)

=item Options

=over

=item C<< command_interface => $ci >>

(Required, isa NACL::C::DFM::DFMClient)
The command interface through which the monitors for object will be refreshed.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=item C<< src_cluster_interface => $ci_cluster >>
(Required, isa NACL::C::Cluster)
The cluster interface through which the Vserver and Volume will be created in src cluster.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=item C<< dst_cluster_interface => $ci_cluster >>
(Required, isa NACL::C::Cluster)
The dst cluster interface through which the Vserver and Volume will be subscribed.
See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

= item C<< clusterpeer >>
(optional, Hashref)
A hash containg the input for creating the intercluster peering , It takes the input
given for clusterpeer create function of L<NACL::MTask::ClusterPeer|lib-NACL-MTask-ClusterPeer-pm>.
 $peer_obj = NACL::MTask::ClusterPeer->create(
            home_cluster_interface    => $src_cluster_nodes[0],
            remote_cluster_interface  => $dst_cluster_nodes[0],
            "nacltask_if_exists_peer" => 'reuse',
            nacltask_verify           => 1,
            _was_created              => \$peer_was_established,
        );


= item C<< aggregate_properties >>
(Optional, Hashref)
A Hash containing the input to get the list of aggregate ,It takes the input
given for find function of L<NACL::STask::Aggregate|lib-NACL-STask-Aggregate-pm>.

Usage:
     ....
       my @aggregate = NACL::STask::Aggregate->find(
        command_interface => $src_cluster_interface,
        filter            => { root => 'false', }
    );

= item C<< vserver_properties >>

(Optional, Hashref)
A hash containing the inputs for Vserver creation. It takes all the inputs given for
the create function of L<NACL::STask::Vserver|lib-NACL-STask-Vserver-pm>.
By default, the name of the Vserver would be "storageservice_vs".

Usage:
     ...
      my $src_vserver = NACL::STask::Vserver->create(
        command_interface  => $src_cluster_interface,
        aggregate          => $aggregate,
        vserver            => $vservername,
        nacltask_if_exists => 'purge',
        %vserver_properties
    );
...
Refer L<NACL::STask::Vserver|lib-NACL-STask-Vserver-pm> for default values.

= item C<< volume_properties >>

(Optional, Hashref)
A hashrefcontianing the inputs for Volume creation. It takes all the inputs given for
the create function of L<NACL::STask::Volume|lib-NACL-STask-Volume-pm>.

Usage:
     ...
      $vol_obj = NACL::STask::Volume->create(
            command_interface  => $src_cluster_interface,
            nacltask_if_exists => 'purge',
            vserver            => $vservername,
            aggregate          => $aggregate,
            volume             => $volumename,
            %volume_properties
        );
...
Refer L<NACL::STask::Volume|lib-NACL-STask-Volume-pm> for default values.

= item C<< resource_group >>
(Optional,Hasref)
A Hash reference containing the input for create resource group

Usage:
    ...
       $group_obj = NACL::C::DFM::Group->create(
        command_interface => $command_interface,
        'group-name'      => $groupname,
    );


= item C<< service-workflow >>
(Optional)

 my $service_resource_key = NACL::C::DFM::Resource->lookup(
        command_interface => $command_interface,
        'type'            => "service_workflow",
        "qualified-name"  => "Mirror Destination",
    );


=item C<< topology_info >>


(Required, Hashref)
A hash containg the input for storage service creation, It has the information about the
topology of a storage service which composite entity representing the nodes and
connections of a protection and provisioning strategy

Usage:
     ...
        %topology_info = (
        'connections' => \@topology_connections_info,
        'nodes'       => \@topology_nodes_info
    );

=back

=over

=cut

package NACL::MTask::DFM::StorageService;

use strict;
use warnings;
use base qw(NACL::MTask::MTask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use Params::Validate
  qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR SCALARREF);
use NATE::Exceptions::Argument qw(:try);
use NACL::ComponentUtils qw( _optional_scalars get_package_name );

use NACL::STask::Vserver;
use NACL::STask::Volume;
use NACL::STask::Aggregate;
use NACL::CS::DFM::Aggregate;
use NACL::STask::DFM::Datasource;
use NACL::STask::DFM::StorageService;
use NACL::C::DFM::Group;
use NACL::MTask::ClusterPeer;
use NACL::STask::Cluster;
use NACL::CS::DFM::Vserver;
use NACL::CS::DFM::Volume;
use NACL::C::DFM::Resource;
use Bard::lib::StorageServiceWorkflows;
use NACL::GeneralUtils qw(random_name_generator);

use Class::MethodMaker [
    new => [ '-hash', 'new' ],
    scalar => [ { -type => 'NACL::C::DFM::DFMClient' }, 'command_interface', ],
    scalar => ['topology_info'],
    scalar => [ { -type => 'NACL::CS::DFM::Aggregate' }, 'src_aggregate' ],
    scalar => [ { -type => 'NACL::CS::DFM::Aggregate' }, 'dst_aggregate' ],
    scalar => [ { -type => 'NACL::CS::DFM::Vserver' }, 'src_vserver' ],
    scalar => [ { -type => 'NACL::CS::DFM::Vserver' }, 'dst_vserver' ],
    scalar => ['src_volumes'],
    scalar => ['dst_volumes'],
    scalar => [ { -type => 'NACL::C::DFM::Group' }, 'group_obj' ],
    scalar => [ { -type => 'NACL::MTask::ClusterPeer' }, 'peer_obj' ],
    scalar => [ { -type => 'NACL::STask::DFM::Datasource' }, 'src_cluster' ],
    scalar => [ { -type => 'NACL::STask::DFM::Datasource' }, 'dst_cluster' ],
    scalar => [ { -type => 'NACL::C::Cluster' }, 'dest_cluster_interface' ],
    scalar => [ { -type => 'NACL::C::Cluster' }, 'src_cluster_interface' ],
    scalar =>
      [ { -type => 'NACL::C::DFM::StorageService' }, 'storage_service_obj', ],
];

use constant PREFIX           => 'SS_MTASK';
use constant SRC_NODE_PREFIX  => PREFIX . '_SRC';
use constant DEST_NODE_PREFIX => PREFIX . '_DEST';

my (@datasource_added_flag,$command_interface);

sub create_n_node_mirror_fanout_topology {

    $Log->enter() if $may_enter;

    my $pkg = shift;
    my $storage_service_type;

    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            dest_node_count => { type => SCALAR, optional => 1, default => 2 },
            volume_count    => { type => SCALAR, optional => 1, default => 2 },
            _was_created               => { type => SCALARREF, optional => 1 },
            vserver_properties         => { type => HASHREF,   optional => 1 },
            volume_properties          => { type => HASHREF,   optional => 1 },
            storage_service_client_tag => {
                type     => SCALAR,
                optional => 1,
                default  => PREFIX . "CLIENTTAG"
            },
            storage_service_contact_list => {
                type     => ARRAYREF,
                optional => 1,
                default  => ['admin@netapp.com']
            },
            storage_service_description => {
                type     => SCALAR,
                optional => 1,
                default  => 'creation of storage service MTask'
            },
            storage_service_owner =>
              { type => SCALAR, optional => 1, default => 'admin' },
            storage_service_name => {
                type     => SCALAR,
                optional => 1,
                default  => "ServiceMtask" . random_name_generator
            },
            subscription_context => {
                type     => SCALAR,
                optional => 1,
            },

        },
        ignore_primary_keys => 1,
        allow_extra         => 1,
    );
    $command_interface            = $opts{command_interface};
    my $src_cluster_interface        = $opts{src_cluster_interface};
    my $dest_cluster_interface       = $opts{dest_cluster_interface};
    my $volume_count                 = $opts{volume_count};
    my $dest_node_count              = $opts{dest_node_count};
    my $storage_service_name         = $opts{storage_service_name};
    my $storage_service_owner        = $opts{storage_service_owner};
    my $storage_service_description  = $opts{storage_service_description};
    my $storage_service_contact_list = $opts{storage_service_contact_list};
    my $storage_service_client_tag   = $opts{storage_service_client_tag};
    my $vserver_properties           = delete $opts{vserver_properties};
    my $volume_properties            = delete $opts{volume_properties};
    my $per_vol_properties           = delete $opts{per_vol_properties};
    my $subscription_context         = delete $opts{subscription_context};
    my %vserver_properties;
    my %volume_properties;
    my %per_vol_properties = %{$per_vol_properties}
      if ( defined $per_vol_properties );
    my %volume_context = %{$subscription_context}
      if ( defined $subscription_context );
    %vserver_properties = %{$vserver_properties}
      if ( defined $vserver_properties );
    %volume_properties = %{$volume_properties}
      if ( defined $volume_properties );
    $dest_cluster_interface ||= $src_cluster_interface
      if ( !defined $dest_cluster_interface );
    my $self                       = $pkg->new();
    my %storage_service_properties = (
        'storage-service-client-tag'   => $storage_service_client_tag,
        'storage-service-contact-list' => $storage_service_contact_list,
        'storage-service-description'  => $storage_service_description,
        'storage-service-owner'        => $storage_service_owner,
    );
    my (
        @dfm_vol_names,         $volumename,
        @topology_nodes_info,   @topology_connections_info,
        @volume_objs,           $dst_aggr_resource_key,
        @clusters,              $vserver_resource_key,
        $datasource_added_flag, @datasource_objs,
        $src_vservername,       $dst_vservername,
        $dst_aggregate,         @src_cluster_nodes,
        @dst_cluster_nodes,     $peer_obj,
        $peer_was_established,  $src_cluster_name,
        $dst_cluster_name,      $src_vserver_obj,
        $dst_vserver_obj,       $vservername,
        $dst_aggr_obj,          @vol_resources,
        $intercluster_flag,     @contexts,
    );

## Peering the clusters
    if ( $src_cluster_interface != $dest_cluster_interface ) {
        $intercluster_flag = 1;
        push( @clusters, $src_cluster_interface, $dest_cluster_interface );
        $src_cluster_name  = $src_cluster_interface->name();
        $dst_cluster_name  = $dest_cluster_interface->name();
        @src_cluster_nodes = $src_cluster_interface->node_objs();
        @dst_cluster_nodes = $dest_cluster_interface->node_objs();

        $peer_obj = NACL::MTask::ClusterPeer->create(
            home_cluster_interface    => $src_cluster_nodes[0],
            remote_cluster_interface  => $dst_cluster_nodes[0],
            "nacltask_if_exists_peer" => 'reuse',
            nacltask_verify           => 1,
            _was_created              => \$peer_was_established,
        );

        if ($peer_was_established) {
            $Log->log(
"Cluster peer established between $src_cluster_name  and $dst_cluster_name"
            );
        } else {
            $Log->log(
"Cluster peer already established between $src_cluster_name and $dst_cluster_name"
            );
        }
    } else {
        push( @clusters, $src_cluster_interface );
        $dest_cluster_interface = $src_cluster_interface;
    }

## Finding aggregates in source cluster
    my @aggregate =
      NACL::STask::Aggregate->find_online_not_root(
        command_interface => $src_cluster_interface, );

## Creating Vserver in source cluster
    $src_vservername = "src_vserver" . random_name_generator;
    my $src_vserver = NACL::STask::Vserver->create(
        command_interface  => $src_cluster_interface,
        aggregate          => $aggregate[0],
        vserver            => $src_vservername,
        nacltask_if_exists => 'reuse',
        %vserver_properties
    );

## creating Volume in source cluster
    for ( my $i = 1 ; $i <= $volume_count ; $i++ ) {
        my %volume_properties = %{ $per_vol_properties{$i} }
          if ( defined $per_vol_properties{$i} );

        $volumename = "src_vol" . random_name_generator;
        my $vol_name = NACL::STask::Volume->create(
            command_interface  => $src_cluster_interface,
            nacltask_if_exists => 'purge',
            vserver            => $src_vservername,
            aggregate          => $aggregate[0],
            volume             => $volumename,
            %volume_properties
        );
        my $volname = "$src_vservername:/" . $volumename;
        push( @dfm_vol_names, $volname );
    }
## Finding aggregate in destination cluster
    if ($intercluster_flag) {
        $dst_aggregate =
          NACL::STask::Aggregate->find_online_not_root(
            command_interface => $dest_cluster_interface, );
    } else {
        $dst_aggregate = $aggregate[1];
    }

## Creating Vserver in destination aggregates

    $dst_vservername = "dst_vservername" . random_name_generator;
    my $dst_vserver = NACL::STask::Vserver->create(
        command_interface  => $dest_cluster_interface,
        aggregate          => $dst_aggregate,
        vserver            => $dst_vservername,
        nacltask_if_exists => 'purge',
        %vserver_properties
    );

## Adding Datasource to the DFM
    my @datasource_added_flag;
    foreach my $cluster (@clusters) {
        my $cluster_obj = NACL::STask::DFM::Datasource->add(
            command_interface       => $command_interface,
            hostobj                 => $cluster->hostrec(),
            nacltask_if_exists      => 'reuse',
            wait_till_object_exists => "cluster",
            _was_added              => \$datasource_added_flag,
        );
        push( @datasource_objs,       $cluster_obj );
        push( @datasource_added_flag, $datasource_added_flag );
    }

## getting the src and dst aggr and vserver obj

    $src_vserver_obj = _fetch_vserver_dfm_obj($src_vservername);
    $dst_vserver_obj = _fetch_vserver_dfm_obj($dst_vservername);
    my $src_aggr_obj = _fetch_aggr_dfm_obj( $aggregate[0] );
    $dst_aggr_obj = _fetch_aggr_dfm_obj($dst_aggregate);
    $vserver_resource_key  = $dst_vserver_obj->vserver_resource_key();
    $dst_aggr_resource_key = $dst_aggr_obj->aggregate_resource_key();

## finding the resource key for volumes
    foreach my $vol_name (@dfm_vol_names) {
        my $vol_obj = NACL::CS::DFM::Volume->fetch(
            command_interface => $command_interface,
            filter            => { "volume-name" => $vol_name, }
        );
        my $vol_resource_key = $vol_obj->volume_resource_key();
        push( @vol_resources, $vol_resource_key );
        push( @volume_objs,   $vol_obj );
    }
## finding service workflow resource key

    my $service_resource_key = NACL::C::DFM::Resource->lookup(
        command_interface => $command_interface,
        'type'            => "service_workflow",
        "qualified-name" =>
          'STORAGE_SERVICE_WORKFLOW_VAULT_DESTINATION_WITHOUT_DEDUPE',
    );
##  Creating the resource group

    my $groupname = "group_name" . random_name_generator;
    my $group_obj = NACL::C::DFM::Group->create(
        command_interface => $command_interface,
        'group-name'      => $groupname,
    );
    my $group_resource_key = $group_obj->resource_key();

## Adding member to the group
    $group_obj->add_member( 'member-resource-key' => $dst_aggr_resource_key );
    $group_obj->add_member( 'member-resource-key' => $vserver_resource_key );

## adding member to the topology
    push( @topology_nodes_info, { 'node-name' => SRC_NODE_PREFIX, } );
    for ( my $i = 0, my $j = 1 ; $i < $dest_node_count ; $i++, $j++ ) {
        ## topology info creation
        my $dest_node = "DEST_NODE_PREFIX" . "$j";
        push(
            @topology_nodes_info,
            {
                'node-name'                     => $dest_node,
                'resource-groups'               => [$group_resource_key],
                'service-workflow-resource-key' => $service_resource_key,
            }
        );
        push(
            @topology_connections_info,
            {
                'connection-type'       => 'mirror',
                'destination-node-name' => $dest_node,
                'source-node-name'      => SRC_NODE_PREFIX,
                'is-enabled'            => 'true',
            }
        );
    }

    my %topology_info = (
        'connections' => \@topology_connections_info,
        'nodes'       => \@topology_nodes_info
    );

## Create Storageservice
    my $storageservice_obj = NACL::STask::DFM::StorageService->create(
        command_interface               => $command_interface,
        'storage-service-name'          => $storage_service_name,
        'storage-service-topology-info' => \%topology_info,
        nacltask_if_exists              => 'purge',
        %storage_service_properties
    );
    my $storage_service_resource_key =
      $storageservice_obj->storage_service_resource_key();

##subscription of volume
    @contexts = keys(%volume_context);
    if ( scalar(@contexts) ) {
        for ( my $i = 0 ; $i < @contexts ; $i++ ) {
            my @vol_reskeys;
            my @vol_list = @{ $volume_context{ $contexts[$i] } };

            foreach my $list (@vol_list) {
                $list = $list - 1;
                push( @vol_reskeys, $vol_resources[$list] );
            }
            my $vol_ref = \@vol_reskeys;
            $storageservice_obj->subscribe(
                'subscription-context'        => "$contexts[$i]",
                'subscription-members'        => $vol_ref,
                nacltask_if_object_subscribed => 'pass',
            );
        }
    } else {
        my $vol_ref = \@vol_resources;
        $subscription_context = "Storage service subscription";
        $storageservice_obj->subscribe(
            'subscription-context'        => $subscription_context,
            'subscription-members'        => $vol_ref,
            nacltask_if_object_subscribed => 'pass',
        );
    }
## Cconforming the storage service
    #    $storageservice_obj->conform();

## return the storage service object
    my $ref                  = \@volume_objs;
    my %storage_service_info = (
        'storage_service_obj'    => $storageservice_obj,
        'src_cluster'            => $datasource_objs[0],
        'dst_cluster'            => $datasource_objs[1],
        'src_aggregate'          => $src_aggr_obj,
        'dst_aggregate'          => $dst_aggr_obj,
        'src_vserver'            => $src_vserver_obj,
        'dst_vserver'            => $dst_vserver_obj,
        'src_volumes'            => \@volume_objs,
        'topology_info'          => \%topology_info,
        'peer_obj'               => $peer_obj,
        'group_obj'              => $group_obj,
        'src_cluster_interface'  => $src_cluster_interface,
        'dest_cluster_interface' => $dest_cluster_interface,
    );
    return $pkg->new(
        'command_interface' => $command_interface,
        %storage_service_info,
    );

## private subroutines

    sub _fetch_aggr_dfm_obj {
        my $aggregate = shift;
        my $node_name = $aggregate->get_local_node_name();
        my $aggr_name = $node_name . ":" . $aggregate->aggregate();

        return NACL::CS::DFM::Aggregate->fetch(
            command_interface => $command_interface,
            filter            => { "aggregate-name" => $aggr_name, }
        );

    }

    sub _fetch_vserver_dfm_obj {

        return NACL::CS::DFM::Vserver->fetch(
            command_interface => $command_interface,
            filter            => { "vserver-name" => shift, }
        );

    }
}

=head2 purge

(Instance method)

    NACL::MTask::DFM::StorageService->purge(
        command_interface => $ci,
        );
(Class method)

=item Options

=over

=item C<< nacltask_if_purged => die | pass >>

(Optional, defaults to "die")
Specifies what to do if the StorageService we're trying to delete no longer
exists. A value of C<die> (the default) will result in an exception
being thrown. A value of C<pass> will mean that the exception that would
normally occur when we're trying to destroy a StorageService which does not exist
is suppressed.

=item C<< if_vserver_purged=> die | pass >>

(Optional, defaults to "pass")

Specifies what to do if the vserver we're trying to remove is already removed
 A value of C<die> will result in an exception
being thrown. A value of C<pass> (default value) will mean that the exception that would
normally occur when we're trying to remove a volume which does not exist
is suppressed.

=item C<< if_volume_purged=> die | pass >>

(Optional, defaults to "pass")

Specifies what to do if the volume we're trying to remove is already removed
 A value of C<die> will result in an exception
being thrown. A value of C<pass> (default value) will mean that the exception that would
normally occur when we're trying to remove a volume which does not exist
is suppressed.


=item C<< group_obj >>

(Optional)

Delete the group created while creating the storage service.

=item C<< nacltask_delete_lifs => 0|1 >>

(Optional, defaults to "1")

Specifies what to do if the cluster peering we're trying to remove is already removed
 A value of C<0> will result in an exception
being thrown. A value of C<1> (default value) will mean that the exception that would
normally occur when we're trying to remove a volume which does not exist
is suppressed.


=back

=over


=cut;

sub purge {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = $self->_common_validate_with(
        params              => \@_,
        ignore_primary_keys => 1,
        allow_extra         => 1,
    );

    my $src_cluster_interface  = $self->src_cluster_interface();
    my $dest_cluster_interface = $self->dest_cluster_interface();
    my $storage_service_obj    = $self->storage_service_obj();
    my $src_vserver_obj        = $self->src_vserver();
    my $dst_vserver_obj        = $self->dst_vserver();
    my $src_cluster            = $self->src_cluster();
    my $dst_cluster            = $self->dst_cluster();
    my $clusterPeer_obj        = $self->peer_obj();
    my $group_obj              = $self->group_obj();
    my $nacltask_if_purged     = "pass";

    ##Destroy StorageService
    if ( defined $storage_service_obj ) {
        try {
            $storage_service_obj->purge(
                nacltask_if_purged => $nacltask_if_purged, );
        }
        catch NATE::BaseException with {
            my $ex = shift;
            $Log->debug(
                "Error in storage service cleanup: " . $ex->stacktrace );
        };

    } else {
        $Log->debug(" the storage service obj not defined ");
    }
    ##Destroy Vserver
    if ( defined $src_vserver_obj ) {
        my $vserver_object =
          vserver_obj( $src_vserver_obj, $src_cluster_interface );
        try {
            $vserver_object->purge( nacltask_if_purged => "pass" );
        }
        catch NATE::BaseException with {
            my $ex = shift;
            $Log->debug( "Error in src vserver cleanup: " . $ex->stacktrace );
        };

    }
    if ( defined $dst_vserver_obj && $dest_cluster_interface ) {
        my $vserver_object =
          vserver_obj( $dst_vserver_obj, $dest_cluster_interface );
        try {
            $vserver_object->purge( nacltask_if_purged => "pass" );
        }
        catch NATE::BaseException with {
            my $ex = shift;
            $Log->debug( "Error in dst vserver cleanup: " . $ex->stacktrace );
        };

    } elsif ( defined $dst_vserver_obj && $src_cluster_interface ) {
        my $vserver_object =
          vserver_obj( $dst_vserver_obj, $src_cluster_interface );
        try {
            $vserver_object->purge( nacltask_if_purged => "pass" );
        }
        catch NATE::BaseException with {
            my $ex = shift;
            $Log->debug( "Error in dst vserver cleanup: " . $ex->stacktrace );
        };

    }

    sub vserver_obj() {
        my $vserver_obj    = shift;
        my $interface      = shift;
        my $vserver_name   = $vserver_obj->vserver_name();
        my $vserver_object = NACL::STask::Vserver->new(
            command_interface => $interface,
            'vserver'         => $vserver_name,
        );

        return $vserver_object;

    }
    if ( defined $clusterPeer_obj ) {
        try {
            $clusterPeer_obj->purge( nacltask_delete_lifs => 1 );
        }
        catch NATE::BaseException with {
            my $ex = shift;
            $Log->debug( "Error in Cluster peer relationship cleanup: "
                  . $ex->stacktrace );
        };
    }

    if ( defined $group_obj ) {
        try {
            $group_obj->destroy();
        }
        catch NATE::BaseException with {
            my $ex = shift;
            $Log->debug( "Error in group cleanup: " . $ex->stacktrace );
        };

    }
    ## deleting datasource from dfm if added
    if ( defined $src_cluster && defined $datasource_added_flag[0] ) {
        $src_cluster->purge();
    }
    if ( defined $dst_cluster && defined $datasource_added_flag[1] ) {
        $dst_cluster->purge();
    }
}
1;

