# Copyright (c) 2009-2014 NetApp Inc.
# All rights reserved.
#
## @summary Exception with gather_diagnostics method defined on it.
## Any exception thrown by NACL::MTask::HA library will be
## caught and NACL::MTask::Exceptions::HAGatherDiagException exception will
## be thrown by it so that gather_diagnostics method can be called by TCD.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=pod

=head1 Package Name

NACL::MTask::Exceptions::HAGatherDiagException

=head1 Author/Maintainer

dl-nacl-dev

=head1 Description

Exception with gather_diagnostics method defined on it.
Any exception thrown by NACL::MTask::HA library will be
caught and NACL::MTask::Exceptions::HAGatherDiagException exception will
be thrown by it so that gather_diagnostics method can be called by TCD.

=head1 See Also

Package inherits from the BaseException module in NATE.

=head1 NACL::MTask::Exceptions::HAGatherDiagException

=head2 Base Class

NATE::BaseException

=cut

package NACL::MTask::Exceptions::HAGatherDiagException;
use NACL::APISet::_Mixins::Exceptions::Common qw(convert);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base NATE::BaseException;
use NATE::BaseException qw(:try);
use Class::MethodMaker [
    hash => 'sfo_args'
];

sub make_base_of {
    my ($self, $ex) = @_;
    our @ISA = (ref($ex));
}

sub _exec_diagnostic_find_command {
    my ($self, %args) = @_;
    my $package = $args{package};
    my %find_args;
    $find_args{command_interface} = $args{command_interface} if defined $args{command_interface};
    $find_args{filter} = $args{filter} if defined $args{filter};

    try {
        eval "require $package";
        $package->find(
            %find_args,
        );
    } catch NATE::BaseException with {

        # Print out the exception, then don't do anything else
        my $exception_obj = shift;
        $Log->comment( $exception_obj->stringify() );

    };

}

sub _exec_diagnostic_fetch_command {
    my ($self, %args) = @_;
    my $package = $args{package};
    my %fetch_args;
    $fetch_args{command_interface} = $args{command_interface} if defined $args{command_interface};
    $fetch_args{filter} = $args{filter} if defined $args{filter};
    $fetch_args{requested_fields} = $args{requested_fields} if defined $args{requested_fields};

    try {
        eval "require $package";
        $package->fetch(
            %fetch_args,
            allow_empty => 1
        );
    } catch NATE::BaseException with {

        # Print out the exception, then don't do anything else
        my $exception_obj = shift;
	if($exception_obj->text() !~ /Please run metrocluster interconnect show/i){
           $Log->comment( $exception_obj->stringify() );
	}

    };

}

sub gather_diagnostics {
    my $self = shift;
     use Data::Dumper;
    my %sfo_args = $self->sfo_args();
    my $command_interface = $sfo_args{command_interface};
    # Perform diagnostic commands
    $Log->comment('Gathering diagnostic info for exception ' . ref($self));
    my @diagnostic_fetch_commands = qw(StorageFailoverShowGiveback StorageFailover
    StorageAggregate Cluster ClusterDate StorageFailoverInterconnectShowLink
    StorageFailoverShowTakeover StorageAggregateRelocation);
    foreach my $command ( @diagnostic_fetch_commands ) {
        $self->_exec_diagnostic_fetch_command(
            package => "NACL::CS::$command",
            command_interface => $command_interface,
        );
    }
    $Log->comment('Finished gathering diagnostic info for exception ' . ref($self));
}
1;
