# $Id$
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NACL::MTask::Group::SFOPair::CMode Module
## @author radish, dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::MTask::Group::SFOPair::CMode;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::MTask::Group::SFOPair;

use base qw (NACL::MTask::Group::SFOPair);

use Class::MethodMaker [
    scalar => [
        {   '-static'   => 1,
            '-default'  => "CMode",
            '-store_cb' => sub {
                die
                    "Cannot overwrite mode in NACL::MTask::Group::SFOPair::CMode";
            },
        },
        'mode'
    ],
];

=head1 NAME

NACL::MTask::Group::SFOPair::CMode

=head1 DESCRIPTION

"NACL::MTask::Group::SFOPair::CMode" Groups Node objects that are in the
same SFO pair.  

=head1 ATTRIBUTES

=head2 pair

The SFOPAIR string.

=head2 mode

The mode of the SFOPair.  For this class "CMode".

=head2 node_set

The collection of node objects in the sfo pair.

=head1 METHODS

=cut

=head2 find 

(Class method only)

Given a list of NACL::C::SystemNode (or any class derived from SystemNode like
NACL::C::Node or NACL::STask::Node) objects,
will return a list of NACL::MTask::Group::SFOPair::CMode objects, each 
containing a list of nodes for that SFO pair.

ex:

    my @nodes = NACL::STask::Node->find();
    my @sgroups = NACL::MTask::Group::SFOPair::7Mode->find( nodes => \@nodes );
    my @cgroups = NACL::MTask::Group::SFOPair::CMode->find( nodes => \@nodes );

=over

=item Options

=over

=item C<< nodes => \@nodes >>

(Required, isa NACL::C::SystemNode)

The list of NACL Node objects from which to construct
SFOPairs.  All nodes for the SFOPairs must be present
(i.e. you cannot only provide the first node of an SFO
pair).  This class requires the following Hostrec variable
be defined for both nodes of a pair (SFOPAIR) and that that
variable must contain the same string for both nodes.

=back

=back

=head2 pair

(Instance method only)

    my $pair = $SFOPair->pair();

Returns the SFOPAIR string stored in the Hostrec.

=head2 mode

(Instance method only)

    my $mode = $SFOPair->mode();

Returns the mode of this SFOPair.  For this class it is "CMode".

=head2 nodes

(Instance method only - ReadOnly)

    my $nodes = $SFOPair->nodes();

Returns the names of the nodes in this SFO pair.
The nodes() method is just an alternate view of the
nodes_set() method.

=head2 node_set

(Instance method only)

    my $ns = $SFOPair->node_set();

Returns a NACL::MTask::Set::Component set of the nodes in this SFO pair.
Component Sets allow many basic set operations like contains, intersection,
union, difference, etc...

Returns a NACL::MTask::Set::Component set of the nodes in this sfo pair.
Component Sets allow many basic set operations like contains, intersection,
union, difference, etc...

The node objects returned will be of the same type as the node
objects passed into NACL::MTask::Group::SFOPair::CMode->find().

In other words if you did:

    my @nodes = NACL::STask::Node->find();
    my @groups = NACL::MTask::Group::SFOPair::CMode->find( nodes => \@nodes );
    my $ns = $groups[0]->node_set();
    my @group_nodes = $ns->get_all_members();

The nodes in $ns will be of type NACL::STask::Node, but if the @nodes
were returned by NACL::C::SystemNode->find() then the returned nodes would be of
type NACL::C:SystemNode.

=cut

1;
