#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::MTask::LunIterators::UnitTestLun;

use Moose;
use Tharn;
use NACL::STask::Lun;

has 'path' => (
    is => 'ro',
    isa => 'Str',
);

has 'volume' => (
    is => 'ro',
    isa => 'Object',
);

has '_type' => (
    is => 'rw',
    isa => 'Str',
    required => 0,
);

has '_luns' => (
    is => 'rw',
    isa => 'ArrayRef',
    default => sub { return []; },
);

# Make a fake directory tree of luns for testing purposes
my @luns = (
    { path => '/', _type => 'dir', _luns => [
        { path => '/dir_empty_1', _type => 'dir', },
        { path => '/dir_one_lun', _type => 'dir', _luns => [
            { path => '/dir_one_lun/lun', _type => 'lun', },
        ]},
        { path => '/dir_b', _type => 'dir', _luns => [
            { path => '/dir_b/lun_match', _type => 'lun', },
            { path => '/dir_b/lun_other', _type => 'lun', },
            { path => '/dir_b/dir_bc', _type => 'dir', _luns => [
                { path => '/dir_b/dir_bc/canary_lun', _type => 'lun', },
            ]},
        ]},
        { path => '/a_lun', _type => 'lun', },
        { path => '/symlink', _type => 'symlink', },
    ]},
);

sub BUILD {
    my $self = shift;

    # Need to special case the creation of 'pa
    if ($self->path() eq '/' && !$self->_type()) {
        $self->_type($luns[0]->{_type});
        $self->_luns($luns[0]->{_luns});
    }
}

sub raw_name {
    my $self = $_[0];
    return $self->path();
}

sub standard_type {
    my $self = $_[0];
    return $self->_type();
}

sub is_directory {
    my $self = $_[0];
    if ($self->_type() eq 'dir') {
        return 1;
    }
    return 0;
}

sub enumerate_directory {
    my $self = $_[0];
    if (scalar(@{$self->_luns()})) {
        my $type = ref $self;
        $Log->comment("Making objects for ".
            join ',', map { $_->{path} } @{$self->_luns()}
        );
        return map {
            $type->new(
                %{$_},
                volume => $self->volume(),
            )
        } @{$self->_luns()};
    }
    return;
}

sub make_lun {
    my $self = $_[0];
    return NACL::STask::Lun->new(
        command_interface => $self->volume()->command_interface(),
        vserver => $self->volume()->vserver(),
        path => '/vol/'.$self->volume()->volume().$self->path(),
    );
}

1;
=head1 NAME

This is private and only used for unit tests on the LunIterator.

Do not use!

=cut
