#
# Copyright (c) 2011-2021 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MCC Mirrored Aggregate Stub MTask Module
## @author dl-mcc-dpg-qa@netapp.com
## @status shared
## @pod here

package NACL::MTask::MCC::Mirrored_Aggregate;

use strict;
use warnings;
use Carp;

use base qw (NACL::MTask::MTask);

use NACL::C::Aggregate ();

use NACL::Exceptions::NoElementsFound qw(:try);
use NATE::BaseException qw(:try);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use Params::Validate
    qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR);
use Readonly;
use Data::Dumper;


Readonly my $TO_BE_DETERMINED => undef;


=head1 NAME

NACL::MTask::MCC::Mirrored_Aggregate

=head1 DESCRIPTION

This task is to be used for C-Mode MetroCluster testing.

    # Start test
    use NACL::MTask::MCC::MetroCluster;
    use NACL::MTask::MCC::Node;
    use NACL::MTask::MCC::Mirrored_Aggregate;
    ...
    my $mcc_node = $mcc->get_singleton_DR_group()->
                     get_HA_group_pair()->{'A'}->get_group_mcc_node()->[$i];
    my @aggrs = @{ $mcc_node->get_primary_mirrored_aggregates() };
    ...
    # Perform test here
    ...

    # End test

=cut

=head1 Public METHODS

There is no public constructor for this class.  A
NACL::MTask::MCC::Mirrored_Aggregate object is obtained via calls such as

=over

    my $mcc_node = $mcc->get_singleton_DR_group()->
                     get_HA_group_pair()->{'A'}->get_group_mcc_node()->[$i];
    my @aggrs = @{ $mcc_node->get_primary_mirrored_aggregates() };

=back

=cut

# automatically generate a (Non-public) constructor (includes a call to 'init')
# and (Non-public) member accessor methods
use Class::MethodMaker [
    new    => [ '-init', 'new' ],
    scalar => [
                { -type => 'NACL::C::Aggregate' },
                '_aggregate_component',
              ],
                        ];

# validate the parameters passed in and what else?
sub init {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
#TBD: several parameters will be passed in here to uniquely identify an aggregate, which will al be used to find the particular NACL::C::Aggregate component currently associated with them.
                  },
                            );

#    $self->_aggregate_component(...);
    $Log->exit() if $may_exit;
}

=head2 get_Aggregate_component()

    my $disk_comp = $disk->get_Aggregate_component();

Returns the component object (NACL::C::Aggregate) associated with this
mirrored aggregate.

=cut

sub get_Aggregate_component {
    $Log->enter() if $may_enter;
    my $self = shift;

    $Log->exit() if $may_exit;
    return $self->_aggregate_component();
}

=head2 get_plex_0_disks()

    my @disks = @{ $mirrored_aggr->get_plex_0_disks() };

Returns array ref of NACL::MTask::MCC::Disk objects associated with the local
plex of the mirrored aggregate.

=cut

sub get_plex_0_disks {
    $Log->enter() if $may_enter;
    my $self = shift;
die 'Not yet implemented.';

    $Log->exit() if $may_exit;
    return $TO_BE_DETERMINED;
}

=head2 get_plex_1_disks()

    my @disks = @{ $mirrored_aggr->get_plex_1_disks() };

Returns array ref of NACL::MTask::MCC::Disk objects associated with the remote
plex of the mirrored aggregate.

=cut

sub get_plex_1_disks {
    $Log->enter() if $may_enter;
    my $self = shift;
die 'Not yet implemented.';

    $Log->exit() if $may_exit;
    return $TO_BE_DETERMINED;
}

1;
