# $Id: $
#
## Copyright (c) 2014 NetApp, Inc., All Rights Reserved
##   Any use, modification, or distribution is prohibited
##   without prior written consent from NetApp, Inc.
##

#
## @summary WAFL Ontap Setup MTask Constants Module
## The exported constants are the ones that are exposed to end users
## All constants exported by MTask Setup libraries are defined here.
##
## @author dl-awg-warp-svl
## @change
##
## @status shared
## @pod here

package NACL::MTask::Setup::Constants;

use strict;
use warnings;

use Exporter qw( import );

use Readonly;

Readonly our $Replica => '_Replica';
Readonly our $Flexgroup => '_Flexgroup';

my @SETUP_TYPES = qw(
    $Replica
    $Flexgroup
);

our @EXPORT_OK = (@SETUP_TYPES,);
our %EXPORT_TAGS = (
    setup_types    => [@SETUP_TYPES],
);

__END__

=pod

=head1 NAME

NACL::MTask::Setup::Constants

=head1 DESCRIPTION

This library exports Constants used by the Setup MTask and exposed to end users
The constants are grouped based on functionality and exported as tags.

=head1 Setup Types

These are the setup types passed for type input when NACL::MTask::Setup is
instantiated. This defines the type of setup subclass to be loaded and used

    use NACL::MTask::Setup::Constants qw(:setup_types);
    use NACL::MTask::Setup::Constants qw($Replica);

Following are the supported setup types

=head2 $Replica

This results in a replica setup to be created
See L<NACL::MTask::Setup::_Replica|lib-NACL-MTask-Setup-_Replica-pm/new>
for all the methods available for replica setup

=head2 $Flexgroup

This results in a flexgroup setup to be created
See L<NACL::MTask::Setup::_Flexgroup|lib-NACL-MTask-Setup-_Flexgroup-pm/new>
for all the methods available for flexgroup setup

=cut
