#t  $Id: $
#
## Copyright (c) 2014 NetApp, Inc., All Rights Reserved
##   Any use, modification, or distribution is prohibited
##   without prior written consent from NetApp, Inc.
##

#
## @summary WAFL Ontap Setup MTask GlobalParams Module
## All globalparams needed for Mtask Setup libraries are defined here.
## They are classifed based on objects and exported as tags
##
## @author dl-awg-warp-svl
## @change
##
## @status shared
## @pod here

package NACL::MTask::Setup::GlobalParams;

use strict;
use warnings;

use Exporter qw( import );

use Readonly;

#################Global params##########################
Readonly our $GLOBAL_PREFIX             => 'NACL.MTASK.SETUP';

Readonly our %GLOBAL_PARAM_FOR => (
    DISABLE_PRE_SETUP   => "$GLOBAL_PREFIX.DISABLE_PRE_SETUP",
    DISABLE_POST_SETUP  => "$GLOBAL_PREFIX.DISABLE_POST_SETUP",
    DISABLE_PRE_CLEANUP => "$GLOBAL_PREFIX.DISABLE_PRE_CLEANUP",
    PRE_SETUP_TASKS => "$GLOBAL_PREFIX.PRE_SETUP_TASKS",
    POST_SETUP_TASKS => "$GLOBAL_PREFIX.POST_SETUP_TASKS",
    PRE_CLEANUP_TASKS => "$GLOBAL_PREFIX.PRE_CLEANUP_TASKS",
);

Readonly our %PRE_SETUP_TASKS_OBJECT_TYPE_FOR => (
    CHECK_CLUSTER_HEALTH => 'node',
);

Readonly our %POST_SETUP_TASKS_OBJECT_TYPE_FOR => (
    CACHE_BLOG_COUNTERS => 'node',
    HYBRID_AGGR_COUNTERS => 'aggregate',
    VOLUME_CLONE => 'volume',
    VOLUME_MOVE => 'volume',
);

Readonly our %PRE_CLEANUP_TASKS_OBJECT_TYPE_FOR => (
    COMPARE_BLOG_COUNTERS => 'node',
);

my @GLOBAL_PARAMS = qw(
    %GLOBAL_PARAM_FOR
    %PRE_SETUP_TASKS_OBJECT_TYPE_FOR
    %POST_SETUP_TASKS_OBJECT_TYPE_FOR
    %PRE_CLEANUP_TASKS_OBJECT_TYPE_FOR
);

#################Node params############################

#################Aggregate params############################
Readonly our %AGGREGATE_PARAM_FOR => (
    NAMES               => "$GLOBAL_PREFIX.AGGREGATE.NAMES",
    IS_HYBRID           => "$GLOBAL_PREFIX.AGGREGATE.IS_HYBRID",
    DISABLE_SPACE_CHECK => "$GLOBAL_PREFIX.AGGREGATE.DISABLE_SPACE_CHECK",
);

#################Vserver params###########################
Readonly our %VSERVER_PARAM_FOR => (NAMES => "$GLOBAL_PREFIX.VSERVER.NAMES",);

#################Volume params############################
Readonly our %VOLUME_PARAM_FOR => (
    NAMES => "$GLOBAL_PREFIX.VOLUME.NAMES",
    SIZE  => "$GLOBAL_PREFIX.VOLUME.SIZE",
    COUNT => "$GLOBAL_PREFIX.VOLUME.COUNT",
);

our @EXPORT_OK = (
    qw(
        %AGGREGATE_PARAM_FOR
        %VSERVER_PARAM_FOR
        %VOLUME_PARAM_FOR
    ),
    @GLOBAL_PARAMS
);

our %EXPORT_TAGS = (
    global    => [@GLOBAL_PARAMS],
    Node      => [],
    Aggregate => [qw(%AGGREGATE_PARAM_FOR)],
    Vserver   => [qw(%VSERVER_PARAM_FOR)],
    Volume    => [qw(%VOLUME_PARAM_FOR)],
);

__END__

=pod

=head1 NAME

NACL::MTask::Setup::GlobalParams

=head1 DESCRIPTION

This library exports GlobalParams used by the Setup MTask. It has params that
are generally used for object setup, pre setup, post setup and pre cleanup.
This params have higher precendence than script passed params.

=head1 Global

NACL.MTASK.SETUP.DISABLE_PRE_SETUP - This param disables pre_setup

NACL.MTASK.SETUP.DISABLE_POST_SETUP - This param disables post_setup

NACL.MTASK.SETUP.DISABLE_PRE_CLEANUP - This param disables pre_cleanup

=head1 Object

=head2 Aggregate

NACL.MTASK.SETUP.AGGREGATE.NAMES - This param defines the csv list of aggregates to be used for
                  setup

NACL.MTASK.SETUP.AGGREGATE.IS_HYBRID - This param results in hybrid aggregate being found or created

NACL.MTASK.SETUP.AGGREGATE.DISABLE_SPACE_CHECK - This param results in space check being skipped for aggregate. If this is not set, an aggregate is selected that can accommodate all the volume sizes

=head2 Vserver

NACL.MTASK.SETUP.VSERVER.NAMES - This param defines the csv list of vservers to be used for
                setup

=head2 Volume

NACL.MTASK.SETUP.VOLUME.NAMES - This param defines the csv list of volumes to be used for
               setup

NACL.MTASK.SETUP.VOLUME.SIZE - This param defines the size of volume to be used

NACL.MTASK.SETUP.VOLUME.COUNT - This param defines the number of volumes to be found or created

=head1 Pre setup tasks

Pass the below tasks as csv to NACL.MTASK.SETUP.PRE_SETUP_TASKS

=head2 Node

CLUSTER_HEALTH_CHECK - This param results in chluster health check being performed
                       before setup creation begins

=head1 Post setup tasks

Pass the below tasks as csv to NACL.MTASK.SETUP.POST_SETUP_TASKS

=head2 Node

CACHE_BLOG_COUNTERS - This param leads to caching of blog counters

=head2 Aggregate

CACHE_HYBRID_AGGR_COUNTERS - This param results in hybrid aggr counters being logged

=head2 Volume

VOLUME_CLONE - This results in volume clone operation being done on all volumes found or created by setup

VOLUME_MOVE - This results in volume move operation being done on all volumes found or created by setup

=head1 Pre cleanup tasks

Pass the below tasks as csv to NACL.MTASK.SETUP.PRE_CLEANUP_TASKS

=head2 Node

COMPARE_BLOG_COUNTERS - This param results in comparison of cached blog counters
                        with counter values in pre cleanup.

=cut
