# $Id: $
#
## Copyright (c) 2014 NetApp, Inc., All Rights Reserved
##   Any use, modification, or distribution is prohibited
##   without prior written consent from NetApp, Inc.
##

#
## @summary WAFL Ontap Setup MTask Module
## @author dl-awg-warp-svl
## @change
##
## @status shared
## @pod here

package NACL::MTask::Setup::_Replica;

use strict;
use warnings;
use parent qw(NACL::MTask::Setup);
use NATE::Log qw(log_global);
use Params::Validate qw(validate_with :types);

use NACL::MTask::Setup::Object::Snapmirror;

use Class::MethodMaker [
    new    => 'new',
    scalar => [
        {-type => 'NACL::MTask::Setup::Object::Snapmirror'},
        '_mirror_mtask_obj'
    ],
    scalar => [
        {-type => 'NACL::MTask::Setup::Object::Aggregate'},
        '_source_aggregate_mtask_obj'
    ],
    scalar => [
        {-type => 'NACL::MTask::Setup::Object::Aggregate'},
        '_destination_aggregate_mtask_obj'
    ],
    scalar => [
        {-type => 'NACL::MTask::Setup::Object::Volume'},
        '_source_volume_mtask_obj'
    ],
    scalar => [
        {-type => 'NACL::MTask::Setup::Object::Volume'},
        '_destination_volume_mtask_obj'
    ],

    scalar => [{-type => 'NACL::STask::Node'},      'source_node'],
    scalar => [{-type => 'NACL::STask::Aggregate'}, 'source_aggregate'],
    scalar => [{-type => 'NACL::STask::Vserver'},   'source_vserver'],
    array  => [{-type => 'NACL::STask::Volume'},    'source_volumes'],

    scalar => [{-type => 'NACL::STask::Node'},      'destination_node'],
    scalar => [{-type => 'NACL::STask::Aggregate'}, 'destination_aggregate'],
    scalar => [{-type => 'NACL::STask::Vserver'},   'destination_vserver'],
    array  => [{-type => 'NACL::STask::Volume'},    'destination_volumes'],

    array => [{-type => 'NACL::STask::Snapmirror'}, 'snapmirrors'],
    scalar => ['snapmirror_options'],
];

my $Log = log_global;

sub create {
    my $self = shift;

    $self->_verify_invocation(style => 'instance_only');

    my %opts = $self->_common_validate_with(
        params          => \@_,
        additional_spec => {
            source_aggregate_options      => {type => HASHREF, default => {}},
            destination_aggregate_options => {type => HASHREF, default => {}},
            source_volume_options         => {type => HASHREF, default => {}},
            destination_volume_options    => {type => HASHREF, default => {}},
            snapmirror_options            => {type => HASHREF, default => {}},
            skip_snapmirror_creation      => {type => BOOLEAN, default => 0},
        },
        allow_extra          => 1,
        ignore_primary_keys  => 1,
        no_command_interface => 1,
    );

    my $src_aggr_options   = delete $opts{source_aggregate_options};
    my $dest_aggr_options  = delete $opts{destination_aggregate_options};
    my $src_vol_options    = delete $opts{source_volume_options};
    my $dest_vol_options   = delete $opts{destination_volume_options};
    $self->snapmirror_options(delete $opts{snapmirror_options});
    my $skip_snapmirror_creation = delete $opts{skip_snapmirror_creation};

    $opts{volume_options}    = $src_vol_options  if ($src_vol_options);
    $opts{aggregate_options} = $src_aggr_options if ($src_aggr_options);
    $self->SUPER::create(%opts);

    my $dst_node = $self->destination_node;

    $self->_set_opts_find_or_create_dest_aggr_set_attributes(
        $dest_aggr_options, $dst_node);

    $self->_set_opts_find_or_create_dest_vols_set_attributes(
        $dest_vol_options, $dst_node);

    $self->_node_mtask_obj->add_license_for_features('SnapVault');

    $self->create_snapmirrors if(! $skip_snapmirror_creation);
} ## end sub create

sub _set_node_attributes {
    my $self = shift;
    my ($node_mtask_object) = @_;

    $self->_node_mtask_obj($node_mtask_object);
    $self->nodes( $node_mtask_object->nodes );
    $self->source_node($node_mtask_object->nodes->[0]);
    my $index = ($node_mtask_object->is_1node_setup) ? 0 : 1;
    $self->destination_node($node_mtask_object->nodes->[$index]);
}

sub _set_vserver_attributes {
    my $self = shift;
    my ($vserver_mtask_obj) = @_;
    $self->_vserver_mtask_obj($vserver_mtask_obj);
    $self->vservers( $vserver_mtask_obj->vservers );
    $self->source_vserver($vserver_mtask_obj->vservers->[0]);
    $self->destination_vserver($vserver_mtask_obj->vservers->[0]);
}

sub _set_aggregate_attributes {
    my $self = shift;
    my ($aggr_mtask_obj) = @_;
    $self->_aggregate_mtask_obj($aggr_mtask_obj);
    $self->aggregates( $aggr_mtask_obj->aggregates );
    $self->source_aggregate($aggr_mtask_obj->aggregates->[0]);
}

sub _set_volume_attributes {
      my $self = shift;
      my ($vol_mtask_obj) = @_;
      $self->_volume_mtask_obj($vol_mtask_obj);
      $self->source_volumes($vol_mtask_obj->volumes); 
}

sub _set_opts_find_or_create_dest_aggr_set_attributes {
    my $self           = shift;
    my $aggr_mtask_obj = $self->_set_opts_find_or_create_dest_aggr(@_);
    $self->_set_dest_aggregate_attributes($aggr_mtask_obj);
}

sub _set_opts_find_or_create_dest_aggr {
    my $self = shift;
    my ($dest_aggr_options, $dst_node) = @_;

    $self->_set_node_filters_on_aggregate($dest_aggr_options, $dst_node);
    return  $self->_find_or_create_aggregates($dest_aggr_options, $dst_node);
} ## end sub _set_opts_find_or_create_dest_aggr

sub _set_dest_aggregate_attributes {
    my $self = shift;
    my ($aggr_mtask_obj) = @_;
    $self->_destination_aggregate_mtask_obj($aggr_mtask_obj);
    $self->destination_aggregate($aggr_mtask_obj->aggregates->[0]);
}

sub _set_opts_find_or_create_dest_vols_set_attributes {
    my $self          = shift;
    my $vol_mtask_obj = $self->_set_opts_find_or_create_dest_vols(@_);
    $self->_set_dest_volume_attributes($vol_mtask_obj);
}

sub _set_opts_find_or_create_dest_vols {
    my $self = shift;
    my ($dest_vol_options, $dst_node) = @_;
    $self->_set_volume_create_options(
        $dest_vol_options,
        {   vserver   => $self->destination_vserver->vserver,
            aggregate => $self->destination_aggregate->aggregate,
            type      => 'DP',
        }
    );
    return $self->_find_or_create_volumes($dest_vol_options, $dst_node);
} ## end sub _set_opts_find_or_create_dest_vols

sub _set_dest_volume_attributes {
      my $self = shift;
      my ($vol_mtask_obj) = @_;
      $self->_destination_volume_mtask_obj($vol_mtask_obj);
      $self->destination_volumes($vol_mtask_obj->volumes); 
}

sub create_snapmirrors {
    my $self = shift;
    my (%snapmirror_options) = @_;
    %snapmirror_options = %{$self->snapmirror_options}
        if(! %snapmirror_options);

    $self->_mirror_mtask_obj(
        NACL::MTask::Setup::Object::Snapmirror->new(
            node                => $self->destination_node,
            cleanup_manager => $self->cleanup_manager,
            source_volumes      => [$self->source_volumes],
            destination_volumes => [$self->destination_volumes],
            %snapmirror_options,
        )
    );
    $self->snapmirrors($self->_mirror_mtask_obj->snapmirrors);
} ## end sub create_snapmirrors


1;

__END__

=pod

=head1 NAME

NACL::MTask::Setup::_Replica

=head1 DESCRIPTION

This is library does a replica setup. It sets up aggregates, vservers,
source volumes, desintation volumes and relationships

=head1 ATTRIBUTES

=head2 source_node

STask Node object used as source node

=head2 source_aggregate

STask aggregate object used as source aggregate

=head2 source_vserver

STask vserver object used as source vserver

=head2 source_volumes

List of STask volume objects used as source volumes

=head2 destination_node

STask Node object used as destination node

=head2 destination_aggregate

STask aggregate object used as destination aggregate

=head2 destination_vserver

STask vserver object used as destination vserver

=head2 destination_volumes

List of STask volume objects used as destination volumes

=head2 snapmirrors

List of snapmirror STask objects created 

=head1 METHODS

=head2 create

    my $setup = NACL::MTask::Setup->new( type => $Replica );

    $setup->create(
        source_volume_options      => {count => 1},
        destination_volume_options => {count => 3},
    );

    my $setup = NACL::MTask::Setup->new(type => $Replica);

    $setup->create(
        node_options => {
            template => '2nodes',
            is_debug => 1,
        },
        skip_snapmirror_creation => 1,
    );

    $setup->create_snapmirrors(create_options => {type => 'XDP'});

    my $setup = NACL::MTask::Setup->new(type => $Replica);

    $setup->create(
        node_options => {
            template => '2nodes',
            is_debug => 1,
        },
        snapmirror_options => {
            create_options => {type => 'XDP'}
        }
        run_pre_setup => 0,
        skip_snapmirror_creation => 1,
    );

    $setup->create_snapmirrors();
(Instance method)

This method does the following

=over

=item * Finds source and destination nodes

=item * Find or create source and destination Aggregates

=item * Find or create source and destination Vservers

=item * Find or Create Source Volumes

=item * Find or Create Destination Volumes

=item * Adds snapvault license on both the nodes

=item * Create relationships optionally

=back

=over

=item Options

=over

=item C<< node_options => $node_options >>

(Optional, Hashref)
Parameters taken by node setup MTask.

See L<NACL::MTask::Setup::Object::Node::new()|lib-NACL-MTask-Setup-Object-Node-pm/new>

=item C<< source_aggregate_options => $aggregate_options >>

(Optional, Hashref)
Parameters taken by aggregate setup MTask.

See L<NACL::MTask::Setup::Object::Aggregate::new()|lib-NACL-MTask-Setup-Object-Aggregate-pm/new>

=item C<< destination_aggregate_options => $aggregate_options >>

(Optional, Hashref)
Parameters taken by aggregate setup MTask.

See L<NACL::MTask::Setup::Object::Aggregate::new()|lib-NACL-MTask-Setup-Object-Aggregate-pm/new>

=item C<< vserver_options => $vserver_options >>

(Optional, Hashref)
Parameters taken by vserver setup MTask.

See L<NACL::MTask::Setup::Object::Vserver::new()|lib-NACL-MTask-Setup-Object-Vserver-pm/new>

=item C<< source_volume_options => $volume_options >>

(Optional, Hashref)
Parameters taken by volume setup MTask.

See L<NACL::MTask::Setup::Object::Volume::new()|lib-NACL-MTask-Setup-Object-Volume-pm/new>

=item C<< destination_volume_options => $volume_options >>

(Optional, Hashref)
Parameters taken by volume setup MTask.

See L<NACL::MTask::Setup::Object::Volume::new()|lib-NACL-MTask-Setup-Object-Volume-pm/new>
Count passed as part of this will decide the number of mirrors.

=item C<< snapmirror_options => $snapmirror_options >>

(Optional, Hashref)
Parameters taken by snapmirror setup MTask.

See L<NACL::MTask::Setup::Object::Snapmirror::new()|lib-NACL-MTask-Setup-Object-Snapmirror-pm/new>
Count passed as part of this will decide the number of mirrors.

=item C<< skip_snapmirror_creation => 0|1 >>

(Optional)
This is indicates whether or not relationships have to be created. If not, then
it can be created at a later time using create_snapmirrors method.

=back

=back

=head2 post_setup

$setup->post_setup;

(Instance method)

Post setup operations are controlled by global parameters. The
post setup operations for objects can be found here L<NACL::MTask::Setup::GlobalParams#Post setup|lib-NACL-MTask-Setup-GlobalParams-pm/Post setup tasks>
Operations are performed on Nodes, Aggregates, Vservers, Volumes, Snapmirrors

=head1 Pre setup operations

Pre setup operations are controlled by global parameters. The
pre setup operations can be found here L<NACL::MTask::Setup::GlobalParams#Pre setup|lib-NACL-MTask-Setup-GlobalParams-pm/Pre setup tasks>
Operations are performed on Nodes, Aggregates, Vservers, Volumes, Snapmirrors

=head1 Pre cleanup operations

Pre cleanup operations are controlled by global parameters. The
pre cleanup operations can be found here
L<NACL::MTask::Setup::GlobalParams#Pre cleanup|lib-NACL-MTask-Setup-GlobalParams-pm/Pre cleanup tasks>
Operations are performed on Nodes, Aggregates, Vservers, Volumes, Snapmirrors

=cut
