#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#

## @summary Helper Module for Wafliron Task
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::MTask::Wafliron::WaflironUtils

=head1 DESCRIPTION

C<NACL::MTask::Wafliron::WaflironUtils> Provides an helper class to the
NACL::MTask::Wafliron. This basically extends the NACL::MTask::Wafliron Class
with only helper functions consolidated in this class. None of the methods in
this class are exposed to the users.

=cut

package NACL::MTask::Wafliron::WaflironUtils;

use strict;
use warnings;
use constant IRON_STATUS_CHECK_TIMEOUT => 600;

use vars qw (@ISA %EXPORT_TAGS @EXPORT_OK);
use Tharn qw(host );
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::APISet;
require Exporter;
use Params::Validate qw(validate_with SCALAR OBJECT);

use NATE::Exceptions::Argument                           ();
use NACL::APISet::Exceptions::InvalidOptionException     ();
use NACL::APISet::Exceptions::UnsupportedOptionException ();
use NACL::MTask::Exceptions::IronStartFailure;
use NACL::Exceptions::UnexpectedOutputException;
use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use constant INOFILE_PROMPT =>
    qr/Which\s+inofile\s+should\s+be\s+used\s+for\s+(\w+)\?/i;
use constant VOL_PROMPT1 =>
    qr/Which\s+flexvols\s+in\s+aggregate\s+\w+\s+should\s+perform\s+prev_cp/i;
use constant VOL_PROMPT2 =>
    qr/List\s+of\s+volumes\s+on\s+which\s+Wafliron\s+in\s+Previous-Checkpoint\s+mode\s+can\s+run\s+is:/i;
use constant VOL_INOFILE_PROMPT1 =>
    qr/Which\s+inofile\s+should\s+be\s+used\s+for\s+(?:\w+:)*(\w+)\?/i;
use constant VOL_INOFILE_PROMPT2 =>
    qr/Which\s+inofile\s+should\s+be\s+used\s+for\s+(\w+)\@vserver:/i;
use constant VOL_INOFILE_PROMPT3 =>
    qr/Which\s+inofile\s+should\s+be\s+used\s+for\s+volume\s+(\w+)\@vserver:/i;
use constant VOL_INOFILE_PROMPT4 =>
    qr/Which\s+inofile\s+should\s+be\s+used\s+for\s+volume\s+(\w+)\?/i;
use constant VOLINFO_VBN_PROMPT => qr/prev_volinfo_vbn\s*:\s*(\d+)/i;
use constant INVALID_SEL_PROMPT =>
    qr/invalid\s+inofile\s+selection|bad\s+volinfo_vbn/i;

@ISA       = qw(Exporter);
@EXPORT_OK = qw(
    _mode
    _create_apiset_objs
    _get_default_scan_speed
    _set_flags
    _start_wafliron
    _prev_cp_semi_interactive
    _prev_cp_interactive
    _prev_cp_non_interactive
    _get_wafliron_status
    _set_scan_speed
    _get_wafliron_args
    _aggr_name_or_obj_validate_spec
    _retry_iron_start
    _check_corruption_msgs
    _handle_node_panic
    _get_matched_logs

);

%EXPORT_TAGS = (
    HELPER_METHODS => [
        qw( _create_apiset_objs
            _get_default_scan_speed
            _set_flags
            _start_wafliron
            _prev_cp_semi_interactive
            _prev_cp_interactive
            _prev_cp_non_interactive
            _get_wafliron_status
            _set_scan_speed
            _get_wafliron_args
            _mode
            _aggr_name_or_obj_validate_spec
            _retry_iron_start
            _check_corruption_msgs
            _handle_node_panic
            _get_matched_logs
            )
    ]
);

=begin COMMENTED_OUT

    $self->_create_apiset_objs();

(For developers of Wafliron task)
Creates Apiset object (by calling _create_apiset_objs()) and stores it in
the object as the "filer_api" value. (Stored in $self->filer_api()). This
apiset object is used to call wafliron start command for normal, optional
commit and prev-cp.

=end COMMENTED_OUT

=cut

sub _create_apiset_objs {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->filer_api(
        $self->{command_interface}->apiset(
            category  => 'Node',
            interface => 'CLI',
            set       => $self->_mode()
        )
    );
    $Log->exit() if $may_exit;
} ## end sub _create_apiset_objs

=begin COMMENTED_OUT

    $self->_get_default_scan_speed();

(For developers of Wafliron task)
This method returns the default wafl scan speed. First it stores the original
wafl scan speed and sets the wafl scan speed to 0, which reset the scan speed
to default dynamic scan speed. Before returning the default wafl scan speed,
this method sets the wafl scan speed back to original speed. This method
needs to be updated once QoS framework is available.

=end COMMENTED_OUT

=cut

sub _get_default_scan_speed {
    $Log->enter() if $may_enter;
    my $self = shift;

    # Store the original wafl scan speed of the filer
    my $apiset_obj = $self->command_interface()->get_7m_or_nodescope_apiset();
    my $response_obj = $apiset_obj->wafl_scan_speed();
    my $original_scanspeed
        = $response_obj->get_parsed_output()->[0]->{wafl_scan_speed};

    # Set the wafl scan speed to zero and get the
    # scan speed which is default scan speed
    $apiset_obj->wafl_scan_speed( speed => 0 );
    $response_obj = $apiset_obj->wafl_scan_speed();
    my $default_scanspeed
        = $response_obj->get_parsed_output()->[0]->{wafl_scan_speed};

    # Set the original scan speed back
    $apiset_obj->wafl_scan_speed( speed => $original_scanspeed );

    $Log->exit() if $may_exit;
    return $default_scanspeed;
} ## end sub _get_default_scan_speed

=begin COMMENTED_OUT

    $self->_set_flags(%wafl_flags);

(For developers of Wafliron task)
This method sets the different flags as specified in start() method call.
Before setting any flag, it verifies whether the specified flag is
available in the current release or not. If not it logs a warning message.

=end COMMENTED_OUT

=cut

sub _set_flags {
    $Log->enter() if $may_enter;
    my ( $self, %opts ) = @_;
    my $flags      = delete $opts{wafl_flags};
    my $apiset_obj = $self->command_interface()->get_7m_or_nodescope_apiset();
    my %available_flags
        = %{ $apiset_obj->printflag( 'connectrec-log_level' => 'trace' )
            ->get_parsed_output()->[0] };
    foreach my $flag ( keys %$flags ) {
        if ( !exists $available_flags{$flag} ) {
            $Log->warn("Can't set the $flag, as this flag is unavailable");
            next;
        }
        $apiset_obj->setflag(
            flag  => $flag,
            value => $flags->{$flag},
        );
    } ## end foreach my $flag ( keys %$flags)
    $Log->exit() if $may_exit;
} ## end sub _set_flags

=begin COMMENTED_OUT

    $self->_start_wafliron(%opts);

(For developers of Wafliron task)
Starts normal wafliron (without prev-cp). This method uses apiset
($self->filer_api()) to execute the wafliron start command.

=end COMMENTED_OUT

=cut

sub _start_wafliron {
    $Log->enter() if $may_enter;
    my ( $self, %opts ) = @_;
    my $timeout    = $opts{'method-timeout'};
    my $iron_retry = delete $opts{nacltask_iron_start_retry};

    # phrase the command based on context
    my $cmd
        = $self->_mode() =~ /7Mode/i
        ? "aggr_wafliron_start"
        : "storage_aggregate_wafliron_start";

    # get the wafliron args
    my $args = $self->_get_wafliron_args(%opts);
    $self->_retry_iron_start(
        command                   => $cmd,
        'connectrec-timeout'      => $timeout,
        nacltask_iron_start_retry => $iron_retry,
        %{$args}
    );
    $Log->exit() if $may_exit;
} ## end sub _start_wafliron

=begin COMMENTED_OUT

    $self->_prev_cp_semi_interactive(%opts);

(For developers of Wafliron task)
Starts wafliron in prev-cp semi interactive mode. This method validates
aggregate_response, volume_response and incomplete_volume_response as
these are mandatory parameters for prev-cp semi interactive and throws
NATE::Exceptions::Argument exception if not specified.

=end COMMENTED_OUT

=cut

sub _prev_cp_semi_interactive {
    my ( $self, %opts ) = @_;

    # Get the relevent apiset obj based on context
    my $apiset_obj = $self->filer_api();

    # Validate the parameters.
    # For prev_cp in semi-interactive mode, aggregate_response,
    # volume_response and incomplete_volume_response must be provided
    if (!(     defined $opts{nacltask_prev_cp}->{aggregate_response}
            || defined $opts{nacltask_prev_cp}->{volume_response}
            || defined $opts{nacltask_prev_cp}->{incomplete_volume_response}
        )
        )
    {
        NATE::Exceptions::Argument->throw(
                  "For wafliron to start in prev_cp semi-interactive mode, "
                . "parameters: aggregate_response, volume_response"
                . " and incomplete_volume_response must be provided" );
    } ## end if ( !( defined $opts{...

    # Get the nacltask_prev_cp hash into %Input
    my %Input = %{ $opts{nacltask_prev_cp} };

    # Build the wafliron params
    my $args    = $self->_get_wafliron_args(%opts);
    my $timeout = $opts{'method-timeout'};

    # List of vvols for interative mode
    my $list_of_vols = join( ',', keys %{ $Input{volume_response} } );
    my $answer_vol_prompt = sub {
        my $response
            = defined $list_of_vols
            ? $list_of_vols . ',' . $Input{volume_with_snap_id_list}
            : $Input{volume_with_snap_id_list};
        return $response;
    };

    my $validate_and_answer = sub {

        # If prompted for aggregate inofile, answer the aggr inofile
        # If there is response provided, return the same
        # If there is no response provided, 'r' is used as default
        if ( defined $Input{aggregate_response}->{$1} ) {
            return $Input{aggregate_response}->{$1};
        } elsif ( defined $Input{volume_response}->{$1} ) {
            return $Input{volume_response}->{$1};
        } else {
            return 'r';
        }
    };
    my $answer_invalid_selection_prompt = sub {
        return 'r';
    };

    my $match_table = [
        eval INOFILE_PROMPT      => $validate_and_answer,
        eval VOL_PROMPT1         => $answer_vol_prompt,
        eval VOL_PROMPT2         => $answer_vol_prompt,
        eval VOL_INOFILE_PROMPT1 => $validate_and_answer,
        eval VOL_INOFILE_PROMPT2 => $validate_and_answer,
        eval INVALID_SEL_PROMPT  => $answer_invalid_selection_prompt
    ];

    # Phrase the command based on mode
    my $cmd
        = $self->_mode() =~ /7Mode/i
        ? "aggr_wafliron_start"
        : "storage_aggregate_wafliron_start";

    $apiset_obj->$cmd(
        %{$args},
        'connectrec-match_table' => $match_table,
        'connectrec-timeout'     => $timeout,
    );
} ## end sub _prev_cp_semi_interactive

=begin COMMENTED_OUT

    $self->_prev_cp_interactive(%opts);

(For developers of Wafliron task)
Starts wafliron in prev-cp interactive mode. It validates aggregate_response,
volume_response and as these are mandatory parameters for prev-cp interactive
mode and throws NATE::Exceptions::Argument exception if not specified. This
method also checks volume vbn and throws NACL::APISet::Exceptions::InvalidOptionException
if invalid volinfo vbn option is selected.

=end COMMENTED_OUT

=cut

sub _prev_cp_interactive {
    my ( $self, %opts ) = @_;

    my $vol_list_answered = 0;
    my $timeout           = $opts{'method-timeout'};

    # Get the relevent apiset obj based on context
    my $apiset_obj = $self->filer_api();

    # Validate the parameters.
    # For prev_cp in interactive mode, both aggregate_response
    # and volume_response must be provided.
    if (   !defined $opts{nacltask_prev_cp}->{aggregate_response}
        || !defined $opts{nacltask_prev_cp}->{volume_response} )
    {
        NATE::Exceptions::Argument->throw(
                  "For wafliron to start in prev_cp interactive mode, "
                . "parameters: aggregate_response and volume_response"
                . " must be provided for RollingRock and above" );
    } ## end if ( !defined $opts{nacltask_prev_cp...

    # Build the wafliron params
    my $args                            = $self->_get_wafliron_args(%opts);
    my $previous_volinfo_vbn_dependency = 0;

    # Get the prev_cp hash into %Input
    my %Input = %{ $opts{nacltask_prev_cp} };

    # List of vvols for interative mode
    my $list_of_vols = join( ',', keys %{ $Input{volume_response} } );
    $list_of_vols =~ s/,\s+/,/g;

    my $answer_vol_prompt = sub {
        my $response
            = defined $Input{volume_with_snap_id_list}
            ? $list_of_vols . ',' . $Input{volume_with_snap_id_list}
            : $list_of_vols;
        return $response;
    };

    my $validate_and_answer = sub {

        # If the previous prompt was answered '#', this prompt
        # is invalid. Hence log an error and exit
        if ($previous_volinfo_vbn_dependency) {
            NACL::APISet::Exceptions::InvalidOptionException->throw(
                "Invalid vvol matching done!");
        }

        # if prompted for aggregate inofile, answer the aggr inofile
        if ( defined $Input{aggregate_response}->{$1} ) {
            return $Input{aggregate_response}->{$1};
        }

        # When #0 is the vvol option, it should be invalidated and
        # better option to use is 'r' instead.
        # When '#' is the vvol option send '#' as answer and handle
        # previous_volinfo_vbn in the previous_volinfo_vbn prompt
        # with #previous_volinfo_vbn
        if ( $Input{volume_response}->{$1} eq "#0" ) {
            $Log->debug("Invalid prev_vol_vbn no; using r instead");
            return 'r';
        } elsif ( $Input{volume_response}->{$1} eq "#" ) {
            $previous_volinfo_vbn_dependency = 1;
            return '#';
        } else {
            return $Input{volume_response}->{$1};
        }
    };

    my $answer_previous_volinfo_vbn = sub {
        if ($previous_volinfo_vbn_dependency) {
            $previous_volinfo_vbn_dependency = 0;
            return "#" . $1;
        } else {
            NACL::APISet::Exceptions::UnsupportedOptionException->throw(
                "Unsupported prompt found");
        }
    };
    my $answer_invalid_selection_prompt = sub { return 'r' };

    my $match_table = [
        eval INOFILE_PROMPT      => $validate_and_answer,
        eval VOL_PROMPT1         => $answer_vol_prompt,
        eval VOL_PROMPT2         => $answer_vol_prompt,
        eval VOL_INOFILE_PROMPT1 => $validate_and_answer,
        eval VOL_INOFILE_PROMPT2 => $validate_and_answer,
        eval VOL_INOFILE_PROMPT3 => $validate_and_answer,
        eval VOL_INOFILE_PROMPT4 => $validate_and_answer,
        eval VOLINFO_VBN_PROMPT  => $answer_previous_volinfo_vbn,
        eval INVALID_SEL_PROMPT  => $answer_invalid_selection_prompt
    ];

    # Phrase the command based on mode
    my $cmd
        = $self->_mode() =~ /7Mode/i
        ? "aggr_wafliron_start"
        : "storage_aggregate_wafliron_start";

    $apiset_obj->$cmd(
        %{$args},
        'connectrec-match_table' => $match_table,
        'connectrec-timeout'     => $timeout,
    );
} ## end sub _prev_cp_interactive

=begin COMMENTED_OUT

    $self->_prev_cp_non_interactive(%opts);

(For developers of Wafliron task)
Starts wafliron in prev-cp non-interactive mode. This method validates only
aggregate_response as this is mandatory parameter for prev-cp non interactive
mode and throws NATE::Exceptions::Argument exception if not specified.

=end COMMENTED_OUT

=cut

sub _prev_cp_non_interactive {
    my ( $self, %opts ) = @_;
    my $timeout    = $opts{'method-timeout'};
    my $apiset_obj = $self->filer_api();
    my $args       = $self->_get_wafliron_args(%opts);

    if ( !defined $opts{nacltask_prev_cp}->{aggregate_response} ) {
        NATE::Exceptions::Argument->throw(
                  "For wafliron to start in prev_cp non-interactive mode, "
                . "parameters: aggregate_response must be provided for "
                . "RollingRock and above" );
    } ## end if ( !defined $opts{nacltask_prev_cp...

    $args->{'connectrec-match_table'} = [
        eval INOFILE_PROMPT => sub {
            return $opts{nacltask_prev_cp}->{aggregate_response}->{$1};
        },
    ];

    # Phrase the command based on mode
    my $cmd
        = $self->_mode() =~ /7Mode/i
        ? "aggr_wafliron_start"
        : "storage_aggregate_wafliron_start";

    $apiset_obj->$cmd( %{$args}, 'connectrec-timeout' => $timeout, );
} ## end sub _prev_cp_non_interactive

=begin COMMENTED_OUT

    $self->_get_wafliron_status();

(For developers of Wafliron task)
This method returns boolean value depending on the wafliron active
/not_running state.

Return value : 1, For wafliron active state (starting, scanning,
                  checking_lost_blocks, checking_lost_inodes, finishing,
                  aborting, Checking Inodes).

Return value : 0, For not_running, waiting_for_commit_or_reject and
                  waiting_for_commit state.

=end COMMENTED_OUT

=cut

sub _get_wafliron_status {
    $Log->enter() if $may_enter;
    my $self = shift;
    my $apiset = $self->filer_api();

    # Check wafliron state
    my $wafliron_state;
    try {
        $wafliron_state = $self->wafliron_component()
                 ->state('method-timeout' => 60)->iron_status();
    }
    catch NATE::BaseException with {
        my $exception_obj = shift;
        $self->_handle_node_panic();
        $exception_obj->throw();
    };

    if (   ( $wafliron_state eq "starting" )
        || ( $wafliron_state eq "scanning" )
        || ( $wafliron_state eq "checking_lost_blocks" )
        || ( $wafliron_state eq "checking_lost_inodes" )
        || ( $wafliron_state eq "Checking for lost blocks" )
        || ( $wafliron_state eq "Checking for lost inodes" )
        || ( $wafliron_state eq "Checking link counts" )
        || ( $wafliron_state eq "finishing" )
        || ( $wafliron_state eq "Mounting" )
        || ( $wafliron_state eq "aborting" )
        || ( $wafliron_state eq "Cleaning up" )
        || ( $wafliron_state eq "Checking Inodes" )
        || ( $wafliron_state eq "Aborting" ) )
    {
        $Log->exit() if $may_exit;
        return 1;
    } elsif (   ($wafliron_state eq "not_running")
             || ($wafliron_state eq "waiting_for_commit_or_reject")
             || ($wafliron_state eq "waiting_for_commit") ) {
        $Log->exit() if $may_exit;
        return 0;
    } else {
        $Log->exit() if $may_exit;
        NACL::Exceptions::UnexpectedOutputException->throw( 
            'text' => "unexpected wafliron state : $wafliron_state");
    }

} ## end sub _get_wafliron_status

=begin COMMENTED_OUT

    $self->_get_wafliron_args(%opts);

(For developers of Wafliron task)
Depending on the mode and specified options, it returns the wafliron
parameters. This method also constructs the volume response string
(for cmd line option) for prev-cp non interactive and incomplete volume
response for prev-cp semi interactive mode.

=end COMMENTED_OUT

=cut

sub _get_wafliron_args {
    $Log->enter() if $may_enter;
    my ( $self, %opts ) = @_;
    my %params;
    my $prev_cp_attribute;

    $params{aggregate}     = $self->aggregate()->aggregate();
    $params{backup}        = $opts{backup};
    $params{poll_interval} = delete $opts{poll_interval};

    if ( $self->_mode() eq '7Mode' ) {
        $params{force} = $opts{nacltask_force};
        if ( defined $opts{nacltask_inject_errors} ) {
            $params{'inject-errors'} = $opts{nacltask_inject_errors};
        }
        if ( defined $opts{nacltask_volume_to_iron_first} ) {
            $params{'volume-to-iron-first'}
                = $opts{nacltask_volume_to_iron_first};
        }
        if ( defined $opts{nacltask_optional_commit} ) {
            if (   $opts{nacltask_optional_commit} eq 'prev_cp'
                || $opts{nacltask_optional_commit} eq 'true' )
            {
                $params{'optional-commit'} = 1;
            } else {
                $params{'optional-commit'} = 0;
            }
        } ## end if ( defined $opts{nacltask_optional_commit...

        if ( defined $opts{use_backup} ) {
            if ( $opts{use_backup} eq 'true' ) {
                $params{'use-backup'} = 1;
            } else {
                $params{'use-backup'} = 0;
            }
        }

        $prev_cp_attribute = 'prev_cp';
    } else {

        if ( defined $opts{use_backup} ) {
            $params{'use-backup'} = $opts{use_backup};
        }
        if ( defined $opts{nacltask_include_mirrors} ) {
            $params{'include-mirrors'}
                = $opts{nacltask_include_mirrors} ? 'true' : 'false';
        }
        if ( defined $opts{nacltask_optional_commit} ) {
            if ( $opts{nacltask_optional_commit} eq 'prev_cp' ) {
                $params{'optional-commit'} = 'true';
            } else {
                $params{'optional-commit'} = $opts{nacltask_optional_commit};
            }
        } ## end if ( defined $opts{nacltask_optional_commit...

        $prev_cp_attribute = 'previous-cp';

        if ( defined $opts{'nacltask_nohalt'} ) {
            $params{'nohalt'} = $opts{'nacltask_nohalt'};
        }
    } ## end else [ if ( $self->_mode() eq...

    if ( ref $opts{nacltask_prev_cp} eq "HASH" ) {
        if ( $opts{nacltask_prev_cp}->{prev_cp_mode} eq 'non_interactive' ) {
            my %vol_resp_hash
                = %{ $opts{nacltask_prev_cp}->{volume_response} };
            $params{$prev_cp_attribute} = join( ',',
                map { $_ . ':' . $vol_resp_hash{$_}; }
                    keys %vol_resp_hash );
        } elsif ( $opts{nacltask_prev_cp}->{prev_cp_mode} eq 'interactive' ) {
            $params{$prev_cp_attribute} = "";
        } elsif (
            $opts{nacltask_prev_cp}->{prev_cp_mode} eq 'semi_interactive' )
        {
            my $response
                = $opts{nacltask_prev_cp}->{incomplete_volume_response};
            $response =~ s/\s+//;
            $params{$prev_cp_attribute} = $response;
        } else {
            if ( $opts{nacltask_prev_cp}->{prev_cp_mode}
                !~ m/^interactive$|^non_interactive$|^semi_interactive$/ )
            {
                $Log->exit() if $may_exit;
                NATE::Exceptions::Argument->throw(
                          "Invalid arguement value for prev_cp_mode: "
                        . "$opts{nacltask_prev_cp}->{prev_cp_mode}" );
            } ## end if ( $opts{nacltask_prev_cp...
        } ## end else [ if ( $opts{nacltask_prev_cp...
    } ## end if ( ref $opts{nacltask_prev_cp...
    $Log->exit() if $may_exit;
    return \%params;
} ## end sub _get_wafliron_args

=begin COMMENTED_OUT

    $self->_set_scan_speed($speed);

(For developers of Wafliron task)
This method sets and verifies wafl scan speed.  This method needs to be
updated once QoS framework is available.

=end COMMENTED_OUT

=cut

sub _set_scan_speed {
    $Log->enter() if $may_enter;
    my ( $self, $speed ) = @_;

    my $mode       = $self->_mode();
    my $apiset_obj = $self->command_interface()->get_7m_or_nodescope_apiset();

    $apiset_obj->wafl_scan_speed( 'speed' => $speed );

    my $parsed_output = $apiset_obj->wafl_scan_speed()->get_parsed_output();
    my $value         = $parsed_output->[0]->{'wafl_scan_speed'};

    if ( $speed < 0 ) {
        if ( $value != 0 ) {
            NACL::Exceptions::VerifyFailure->throw(
                "Failed to set the wafl scan speed to negative value");
        }
    } elsif ( $speed == 0 ) {
        if ( $value <= 0 ) {
            NACL::Exceptions::VerifyFailure->throw(
                "Failed to set default wafl scan speed");
        }
    } else {
        if ( $value != $speed ) {
            NACL::Exceptions::VerifyFailure->throw(
                "Could not set the wafl scan speed to $speed");
        }
    }
    $Log->exit() if $may_exit;
} ## end sub _set_scan_speed

=begin COMMENTED_OUT

    $self->_mode();

(For developers of Wafliron task)
Returns ONTAP mode (CMode/7Mode).

=end COMMENTED_OUT

=cut

sub _mode {
    my $self = shift;
    return $self->{command_interface}->mode();
}

=begin COMMENTED_OUT

    $self->_retry_iron_start(%opts)

(For developers of Wafliron task)
It starts wafliron and confirms if it has started by verifying the state
or event messages. This method provides the retry mechanism while initializing
wafliron depending on state or event messages.

=end COMMENTED_OUT

=cut

sub _retry_iron_start {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            poll_interval             => { type => SCALAR, default => 60 },
            command                   => { type => SCALAR },
            nacltask_iron_start_retry => { type => SCALAR }
        },
        allow_extra => 1
    );
    my $cmd           = delete $opts{command};
    my $poll_interval = delete $opts{poll_interval};
    my $retries       = delete $opts{nacltask_iron_start_retry};
    my $apiset        = $self->filer_api();
    my $is_cmode      = $self->command_interface->is_cmode();
    my $log_detector  = $self->event_log_object();
    my $start_verified = 0;
    my $aggregate    = $self->aggregate()->aggregate();
    my $endtime;

    my $wait = sub {
        while ( time() < $endtime ) {
            my $wafl_cs;
            try {
                $wafl_cs = $self->wafliron_component->state(
                                       'method-timeout' => 60);
            }
            catch NATE::BaseException with {
                my $exception_obj = shift;
                $self->_handle_node_panic();
                $exception_obj->throw();
            };
            my $wafliron_state = $wafl_cs->iron_status;
            my $iron_last_start_error_info = $wafl_cs->iron_last_start_error_info;
            my @states         = (
                'scanning',                 'checking_lost_blocks',
                'starting',                 'checking_lost_inodes',
                'checking for lost blocks', 'checking for lost inodes',
                'checking link counts',     'finishing',
                'cleaning up',              'checking inodes',
                'aborting',                 'mounting',
                'unmounting',               'waiting_for_commit',
                'waiting_for_commit_or_reject',
            );
            my $states_regex = '(' . ( join "\$\|\^", @states ) . ')';
            $states_regex = qr/$states_regex/i;
            if ( $wafliron_state =~ $states_regex ) {
                $start_verified = 1;
            } elsif ( $wafliron_state eq "not_running" ) {
                if ($iron_last_start_error_info =~ /Operation not permitted on read-only/i) {
                    $Log->debug("start failed because of : $iron_last_start_error_info");
                    $Log->exit() if $may_exit;
                    return 0;
                } else {
                    my @events = $log_detector->stop(check => 0);
                    my $end_date   = $log_detector->event_end_time();
                    my @messages_obj = $self->_get_matched_logs(log_detector => $log_detector,
                                                                check_for_all_presence =>['wafl.iron.start',
                                                                'wafl.aggregate.busy',],
                                                                event_end_time => $end_date,
                                                                return_string => 0,);
					foreach my $obj (@messages_obj) {
                       if ($obj->messagename() =~ m[wafl\.iron\.start] &&
                           $obj->event() =~ /$aggregate/ ) {
                           $start_verified = 1;
                           last;
                       } elsif ($obj->messagename() =~ m[wafl\.aggregate\.busy] &&
                                $obj->event() =~ /$aggregate/ ) {
                           Tharn::snooze ( $endtime - time() );
                           last;
                       }
                   }
                }
            }

            if ($start_verified) {
		$Log->exit() if $may_exit;
                return 0;
            }

            Tharn::snooze 5;
        }
        $Log->exit() if $may_exit;
        return 1;
    };

    my $error_msg;
    my $start_wi = sub {

        my ( $raw_output, $busy );
        try {
            $raw_output = $apiset->$cmd(%opts)->get_raw_output();
        }
        catch NACL::APISet::Exceptions::CommandFailedException with {
            my $ex = shift;
            if ( $ex->text() !~ /in use by other operation/i ) {
                # see what is holding vol ref
                my $node_scope_api =
                    $self->command_interface()->get_7m_or_nodescope_apiset();
                $node_scope_api->wafl_vref_status();
                $Log->exit() if $may_exit;
                $ex->throw();
            }
            $busy = 1;
        };

        if ( $is_cmode ) {
            $Log->comment('Check agrgegate busy msg in _retry_iron_start');
            Tharn::snooze 5;
            my $wafl_cs;
            try {
                $wafl_cs = $self->wafliron_component->state(
                                          'method-timeout' => 60);
            }
            catch NATE::BaseException with {
                my $ex_obj = shift;
                $self->_handle_node_panic();
                $Log->exit() if $may_exit;
                $ex_obj->throw();
            };

            my $wafliron_state = $wafl_cs->iron_status;
            my $error_info     = $wafl_cs->iron_last_start_error_info;
            if ( $wafliron_state eq 'not_running'
                        && $error_info =~
                        m[(Operation not permitted|Unable to online the aggregate)]mi ) {
                $Log->exit() if $may_exit;
                NACL::MTask::Exceptions::IronStartFailure->throw($error_info);
            }
            if ( $wafliron_state eq 'not_running'
                        && $error_info =~
                        m[(in use by other operation)]mi ) {
                $busy = 1;
                $error_msg = $error_info;
            }
        }

        if ( !$busy || $raw_output =~ /Initiated wafliron\-start on/mi ) {
            $Log->exit() if $may_exit;
            return 1;
        } else {
            $Log->exit() if $may_exit;
            return 0;
        }
    };

    while ($retries) {

        $endtime = time() + 60 + ( IRON_STATUS_CHECK_TIMEOUT / $retries );

        # Start wafliron
        my $wi_started = $start_wi->();

        # Check wafliron status
        my $retry_required = $wait->() if ($wi_started);
        last if ( !$retry_required );

        Tharn::snooze $poll_interval;
        $retries--;
    }

    my $error = "wafliron did not start within the timeout period:"
        .  IRON_STATUS_CHECK_TIMEOUT;
    $error = $error . '. ' . $error_msg if ( defined $error_msg );

    if ( !$start_verified ) {
        $Log->exit() if $may_exit;
        NACL::MTask::Exceptions::IronStartFailure->throw($error);
    }

    $Log->exit() if $may_exit;
}

=begin COMMENTED_OUT

    my %opts = validate_with(
        params => \@_,
        spec   => {
            ...
            %{ $self->_aggr_name_or_obj_validate_spec() }
        }
    );

(For developers of Wafliron task)
Returns validate spec for C<aggregate>. Supports name of the
aggregate on which we want to start the wafliron or aggregate
component or task object.

=end COMMENTED_OUT

=cut

sub _aggr_name_or_obj_validate_spec {
    my $self = shift;
    my $callback_msg
        = "Specify either aggregate name or"
        . " object. The object provided must be of type"
        . " 'NACL::STask::Aggregate' or 'NACL::C::Aggregate'";

    return {
        aggregate => {
            type      => SCALAR | OBJECT,
            callbacks => {
                $callback_msg => sub {
                    my $aggr_obj_or_name = $_[0];
                    if ( ref($aggr_obj_or_name) ) {
                        if ( $aggr_obj_or_name->isa("NACL::C::Aggregate") ) {
                            return 1;
                        }
                        return 0;
                    } ## end if ( ref($aggr_obj_or_name...
                    return 1;
                    }
            }
        }
    };
} ## end sub _aggr_name_or_obj_validate_spec

sub _check_corruption_msgs {
    $Log->enter() if $may_enter;
    my ( $self, @args ) = @_;
    my $log_detector  = $self->event_log_object();
    my $corruption_found = 0;

	my @no_corruption_msg = $self->_get_matched_logs(log_detector => $log_detector,
												check_for_all_presence => ['wafl.iron.oc.noCorruption',],
												'nacltask_filter' => { 'severity' => "CRITICAL|ALERT|WARNING|ERROR" },
												return_string => 0);

    if (scalar @no_corruption_msg) {
        $corruption_found = 0;
    } else {

        my @msg_log = $self->_get_matched_logs(log_detector => $log_detector,
                                           check_for_all_presence => [
                                            'callhome.wafliron.chg.pending', 'wafl.check.info',
                                            'wafl.iron.unrecoverable', 'wafl.iron.incompleteVol',
                                            'wafl.iron.badfsid','wafl.iron.oc.committedChangeLog',
                                            'wafl.iron.oc.abort.clog_full','wafl.iron.oc.abort.bad_blk',
                                            'wafl.iron.oc.errorCommitLog','wafl.iron.oc.root.lowMemory','wafl.iron.invalid.vvol',],
                                            'nacltask_filter' => { 'severity' => "CRITICAL|ALERT|WARNING|ERROR" },
                                            return_string => 0);
        if (scalar @msg_log) {
            $corruption_found = 1;
        }
	}

    $self->corruption_found($corruption_found);
    $Log->exit() if $may_exit;
} ## end sub _check_corruption_msgs

sub _handle_node_panic {
    $Log->enter() if $may_enter;
    my ( $self, @args ) = @_;

    my $apiset = $self->filer_api();
    my $is_aggr_relocated = 0;

    if ( $apiset->api_is_session_alive() ) { $Log->exit(); return; }

    my $transit_obj = NACL::Transit->new(
              name => $self->command_interface->name() );
    try {
        # Transit to maintenance mode
        $transit_obj->change_state( 'to' => 'MAINT' );
        my $maint_mode_api = $self->command_interface->apiset(
                     interface => 'CLI', set => 'Maintenance' );

        # Offline aggr
        try {
            $maint_mode_api->aggr_offline(
                  'aggregate'          => $self->aggregate()->aggregate(),
                  'connectrec-timeout' => '300' );
        }
        catch NACL::APISet::Exceptions::InvalidParamValueException with {
            my $exception = shift;
            $is_aggr_relocated++
                    if $exception->text() =~ m[No aggregate named]i;
        }
        catch NACL::APISet::Exceptions::UnexpectedOutputException with {
            # Ignore and go to CLI prompt
            my $exception = shift;
            $Log->comment("UnexpectedOutputException - ". $exception->text());
        }
        catch NACL::APISet::Exceptions::CommandFailedException with {
            my $exception = shift;
            if ( $exception->text() !~ /already offline/is ) {
                $Log->exit() if $may_exit;
                $exception->throw();
            }
        };

        # Go to CLI
        $transit_obj->change_state( to => 'CLI',
                           iftakenover => 'error',
                           timeout     => 1500 );
    }
    catch NACL::Transit::Exceptions::TransitException with {
        my $e = shift;
        $Log->comment("TransitException - ". $e->text());
        if ( $e->text =~ m[partner has taken over]i ) {
            $self->command_interface->refresh_command_interface();
            my $partner_name = $self->partner();
            my $partner_obj  = NACL::C::SystemNode->new( name => $partner_name );
            try {
                NACL::STask::StorageFailover->giveback(
                    command_interface     => $partner_obj,
                    node                  => $partner_obj,
                    partner               => $self->command_interface,
                    'require-partner-waiting'  => "true",
                    'override-vetoes'          => "true",
                    'wait-for-asup-generation' => "0",
                    'failure-reason' => 'giveback_failed_from_iron_mtask',
                    'method-timeout' => '1500',
                );
            }
            catch NACL::Exceptions::HighAvailabilityError with {
                my $ex = shift;
                Tharn::snooze 60;
            };
        }
    };

    # Re-connect filer session
    $Log->comment("Refreshing api interface...");
    $self->command_interface->apiset->api_reconnect_session();

    # Try to run IOC
    $Log->comment("Running IOC and leave aggr with out commiting the changes...");
    my $ioc_obj = NACL::MTask::Wafliron->new(
                command_interface => $self->command_interface,
                aggregate         => $self->aggregate()->aggregate() );

    $ioc_obj->start(
            nacltask_optional_commit => 'true',
            nacltask_wait            => 1,
            nacltask_flags_to_set    => {
                wafl_iron_panic_on_corruption   => 0,
                wafl_inconsistent_panic_testing => 1 },
            nacltask_wafl_scan_speed => 3000 );
    $ioc_obj->review();

    $Log->comment("Lost filer connection while iron/ioc run. Please check for panic...");
    NATE::BaseException->throw(
           "Iron aborted as we lost connection to filer. Please check filer...");

    $Log->exit() if $may_exit;
} ## end sub _handle_node_panic

sub _get_matched_logs {
    $Log->enter() if $may_enter;
    my ( $self, %opts ) = @_;
    my $log_detector = delete $opts{log_detector};
    my $return_string = delete $opts{return_string};
    my @messages;
    my $msg_log;
    try {
        @messages = $log_detector->check(%opts);
    } catch NACL::Exceptions::EventCheckFailure with {
        my $e = shift;
        if ( $e->text() !~ /No matching events found for the following messages|There are no entries matching your query|No events to check/ ) {
             $Log->exit() if $may_exit;
             $e->throw();
        }
        @messages = @{ $e->matched_events()} if(exists $e->{matched_events}) ;
    };
    if ($return_string) {
        if (scalar (@messages)) {
            my @events = map {$_->event()} @messages;
            $msg_log =  join (", ", @events);
            $Log->exit() if $may_exit;
            return $msg_log;
        }
    } else {
        $Log->exit() if $may_exit;
        return @messages;
    }
}

1;
