# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.

## @summary Setup Deep Equality Matcher
## @author  Adam.Ciapponi@netapp.com
## @pod     here

=head1 NAME

NACL::Matchers::DeeplyEqualMatcher

=head1 DESCRIPTION

This module implements a matcher that determines whether it matches by calling eq_deeply

=head1 METHODS

=cut

package NACL::Matchers::DeeplyEqualMatcher;

# Compiler directives.
use strict;
use warnings;
use CR::Setup::Constants;
use Data::Dumper;
use Params::Validate qw(:all);
use Test::Deep::NoTest qw(eq_deeply);

use base qw(NACL::Matcher);

=head2 matches

Builds up a string to evaluate comparisons/expressions and return a boolean value.
This only evaluates the value stored in the object, not equality of objects.

=cut

sub matches {
    validate_pos(@{[@_[1..$#_]]}, NACL::Matcher::matcher_value_validate_spec());

    $_[0]->_set_state_from_matcher_value( matcher_value => $_[1], which_value => 'actual' );

    if ( eq_deeply( $_[0]->{'actual'} , $_[0]->{'expected'} ) ) {
        $TRUE;
    }
    else {
        $FALSE;
    }
}
1;
