#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::Plugin::Driver::Resource::Util;
use strict;
use warnings;

BEGIN {
    require Exporter;
    our @ISA = qw(Exporter);
    our @EXPORT_OK = qw(redirect_logger reset_loggers);
};

use File::Spec::Functions qw(catdir catfile);
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT);

use NATE::ParamSet qw(param_global);
use NATE::Log qw(log_global);

###################################################################
# Re-direct resource discovery NACL log output to a separate
# log file to avoid it getting print in the subtest which uses
# this plug-in(NATE Driver Manager).
###################################################################
sub redirect_logger {
    my %opts = validate(@_, {
        resource_obj   => {type => OBJECT, isa => 'NATE::Driver::Plugin::Resource'},
    });
    my $resource_obj = $opts{resource_obj};
    my $var = param_global();
    my $top_logdir = $var->get('TOP_LOGDIR');
    my $existing_runid = $var->get('RUNID');
    my $logfile_name = '.'.lc($resource_obj->RESOURCE_TYPE_PARAMETER_NAME)."_rcs.log";
    my $new_log_dir = catdir($top_logdir, $existing_runid);
    my $new_log_file = catfile($new_log_dir, $logfile_name);
    my $log = log_global();
    $resource_obj->{_log} = $log;
    my %redirect_opts = (
        path => $new_log_file,
        print_log_redirection => 0,
        log_runid => 0
    );
    $resource_obj->{_log}->redirect(%redirect_opts);
    $resource_obj->{logger_redirected} = 1;
}

sub reset_loggers {
    my %opts = validate(@_, {
        resource_obj   => {type => OBJECT, isa => 'NATE::Driver::Plugin::Resource'},
    });
    my $resource_obj = $opts{resource_obj};
    $resource_obj->{_log}->reset_path() if($resource_obj->{logger_redirected});
    $resource_obj->{logger_redirected} = 0;
}

1;
