use utf8;
package NACL::ResourceList::Schema::Result::Element;

=head1 NAME

NACL::ResourceList::Schema::Result::Element

=cut

use strict;
use warnings;

use base 'DBIx::Class::Core';

=head1 TABLE: C<Elements>

=cut

__PACKAGE__->table("Elements");

=head1 ACCESSORS

=head2 id

  data_type: 'integer'
  is_auto_increment: 1
  is_nullable: 0

=head2 group_id

  data_type: 'integer'
  is_foreign_key: 1
  is_nullable: 1

=head2 host_id

  data_type: 'integer'
  is_foreign_key: 1
  is_nullable: 1


=cut

__PACKAGE__->add_columns(
  "id",
  { data_type => "integer", is_auto_increment => 1, is_nullable => 0 },
  "package_id",
  { data_type => "integer", is_foreign_key => 1, is_nullable => 1 },
  "group_id",
  { data_type => "integer", is_foreign_key => 1, is_nullable => 1 },
  "host_id",
  { data_type => "integer", is_foreign_key => 1, is_nullable => 1 },
);

=head1 PRIMARY KEY

=over 4

=item * L</id>

=back

=cut

__PACKAGE__->set_primary_key("id");

=head1 RELATIONS

=head2 command_histories

Type: has_many

Related object: L<NACL::ResourceList::Schema::Result::CommandHistory>

=cut

__PACKAGE__->has_many(
  "command_histories",
  "NACL::ResourceList::Schema::Result::CommandHistory",
  { "foreign.element_id" => "self.id" },
  { cascade_copy => 0, cascade_delete => 1 },
);

=head2 group

Type: belongs_to

Related object: L<NACL::ResourceList::Schema::Result::Group>

=cut

__PACKAGE__->belongs_to(
  "group",
  "NACL::ResourceList::Schema::Result::Group",
  { id => "group_id" },
  {
    is_deferrable => 0,
    join_type     => "LEFT",
    on_delete     => "RESTRICT",
    on_update     => "CASCADE",
  },
);

=head2 host

Type: belongs_to

Related object: L<NACL::ResourceList::Schema::Result::Host>

=cut

__PACKAGE__->belongs_to(
  "host",
  "NACL::ResourceList::Schema::Result::Host",
  { id => "host_id" },
  {
    is_deferrable => 0,
    join_type     => "LEFT",
    on_delete     => "RESTRICT",
    on_update     => "CASCADE",
  },
);

=head2 package

Type: belongs_to

Related object: L<NACL::ResourceList::Schema::Result::Package>

=cut

__PACKAGE__->belongs_to(
  "package",
  "NACL::ResourceList::Schema::Result::Package",
  { id => "package_id" },
  {
    is_deferrable => 0,
    join_type     => "LEFT",
    on_delete     => "RESTRICT",
    on_update     => "CASCADE",
  },
);

=head2 primary_keys

Type: has_many

Related object: L<NACL::ResourceList::Schema::Result::PrimaryKey>

=cut

__PACKAGE__->has_many(
  "primary_keys",
  "NACL::ResourceList::Schema::Result::PrimaryKey",
  { "foreign.element_id" => "self.id" },
  { cascade_copy => 0, cascade_delete => 1 },
);

1;
