# $Id: $
#
# Copyright (c) 2001-2012 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NATE::BaseExceptions that will be thrown whenever
##          job wait for state fails with unexpected status
##	    for one or more job
##
## @author poornima@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::STask::Exceptions::DFM::JobFailure

=head1 DESCRIPTION

This exception is thrown when the job wait for state fails to complete
with the expected status for one or more jobs

It holds an array of failures as one of its attribute .

Each array member is a typeref which contains various fields defined in the
NACL::CS::DFM::JobWaitForStateFailure module

=head1 ATTRIBUTES

=head2 failures 

=over

This attribute is an array . It holds information about the various fields for 
each task returned by the NACL::CS::DFM::JobWaitForStateFailure module

=back

=head1 EXAMPLE


    Example of throwing the exception

    NACL::STask::Exceptions::DFM::JobFailure->throw(
                 ## Reference to an array which hold the information
                 ## as arrayref with various JobWaitForStateFailure related info.
                 failures => \@failure_objs  );

    Example of catching the exception

   use NACL::STask::Exceptions::DFM::JobWaitForStateFailure qw(:try);

   try { NACL::STask::DFM::Job->wait_for_state(  command_interface => $ci,
                                        'job-list' => $job_list,
					....);

   } catch NACL::Exceptions::DFM::JobFailure with {
      my $exception = shift;

        ## Getting the list of unexpected failures
        my $failure_obj = $exception->{'failures'};

        or

        my @failure_obj = $exception->{'failures'};

   };

=cut

package NACL::STask::Exceptions::DFM::JobFailure;
use base qw(NATE::BaseException);

use Class::MethodMaker [
    array => [ { -type => 'NACL::STask::Exceptions::DFM::JobWaitForStateFailure' }, 'failures' ],
    ];

1;
