#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::STask::Exceptions::FijiClearFailure

=head1 DESCRIPTION
This exception is used in NACL::MTask::Fiji task to report
errors in clearing the fiji

=head1 ATTRIBUTES

=head2 fiji_failure

This attribute is a hash-reference, which will hold all the details of 
matching/unmatching/partial_matched objest.

=head1 EXAMPLE

   use NACL::STask::Exceptions::FijiClearFailure qw(:try);
   use NACL::MTask::Fiji;

   try {
       NACL::MTask::Fiji->clear();
   } catch NACL::STask::Exceptions::FijiClearFailure with {
        my $exception = shift;
        $Log->comment("Fiji Clear failed " 
           . $exception->text());
        my %status = $exception->fiji_failure();
        foreach my $node (keys %status) {
            $Log->comment("Fiji clear fails on $node");
        }
   };

=cut

package NACL::STask::Exceptions::FijiClearFailure;
use base qw(NATE::BaseException);
use Params::Validate qw(validate_with SCALAR  );

use Class::MethodMaker [ hash => 'fiji_failure' ];

sub new {
    my ( $pkg, @args ) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => { text => { type => SCALAR } },
        allow_extra => 1
    );
    my $text = delete $opts{text};

    # BaseException requires text to be an un-named argument
    my $obj = $pkg->SUPER::new( $text, %opts );
    return $obj;
} ## end sub new

1;
