#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when a Lun move/copy is failed
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::STask::Exceptions::LunMoveCopyFailed

=head1 DESCRIPTION

This exception is thrown when lun move/copy gets failed

=head1 EXAMPLE

   use NACL::STask::Exceptions::LunMoveCopyFailed qw(:try);

   try {
       NACL::STask::LunMove->start(
           command_interface             => $Node,
           vserver                       => $Vserver,
           'source-paths'                => [$source_path],
           'destination-paths'           => [$dest_path],
           nacltask_wait_for_destination => 1,
       );
   } catch NACL::Exceptions::LunMoveCopyFailed with {
        # LUN move/copy operation failed
   };

=cut

package NACL::STask::Exceptions::LunMoveCopyFailed;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
);
sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Destination LUN.*already exists*/i,]
    );
    $Log->exit();
    return $detected;
}

1;
