#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NATE::BaseException that will be thrown when verification of one
##          or more fields does not match field values present in the dblade or smdb tables.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::STask::Exceptions::MVRcheckFailure

=head1 DESCRIPTION

This exception is thrown when verification of of one or more fields does not match
field values present in the dblade or smdb tables.
In addition to the attributes that exception objects generally contain,
this exception also contains C<unmatched_fields> attribute.
This is an array-reference, which will hold all the fields which
did not match with the dblade & smdb entries.

=head1 ATTRIBUTES

=head2 unmatched_fields

=over

This attribute is an array-reference, which will hold all the fields which did not
 match with the dblade & smdb entries.

=head3 EXAMPLE

    use NACL::STask::Exceptions::MVRcheckFailure qw(:try);
    use NACL::STask::Aggregate;

    try {
       NACL::STask::Aggregate->verify_mirror_votes(
           command_interface => $command_interface,
           aggregate         => $Aggregate,
           mvr_info          => {
                                  nplexes    => 2,
                                  rdb_dirty  => 0,
                                }
       );
    }
    catch NACL::Exceptions::MVRcheckFailure with {
        my $exception = shift;
        $Log->comment( "MVR check match failed" . $exception->text() );
        my $unmatched_fields_arrayref = $exception->unmatched_fields();
    );

=back

=cut

package NACL::STask::Exceptions::MVRcheckFailure;
use base qw( NATE::BaseException );

use Class::MethodMaker [ scalar => 'unmatched_fields' ];

1;
