#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NATE::BaseException that will be thrown when verification of one
##          or more field values does not match the provided value.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::STask::Exceptions::SpaceComparisonFailure

=head1 DESCRIPTION

This exception is thrown when verification of field values does not pass, i.e
the field value for one or more fields does not match what was provided.

=head1 ATTRIBUTES

=head2 mismatched_value

=over

This is a hashref which contains the "attribute" and its value,
on which mismatch has occurred.

=back

=back

=head1 EXAMPLE

   use NACL::STask::Exceptions::SpaceComparisonFailure qw(:try);

   try {
       NACL::STask::Aggregate->space_comparison(
          command_interface => $command_interface,
          aggregate         => $Aggregate,
          %other_options
       );
   } catch NACL::STask::Exceptions::SpaceComparisonFailure with {
        my $exception = shift;
        $Log->comment("Unexpected fields and their values: "
           . $exception->text());
        my $mismatched_value = $exception->mismatched_value();
   };

=cut

package NACL::STask::Exceptions::SpaceComparisonFailure;
use base qw(NATE::BaseException);

use Class::MethodMaker [
    scalar => 'mismatched_value',
];

1;
