=head1 NAME

NACL::STask::VolumeEfficiencyPolicy

=head1 DESCRIPTION

C<NACL::STask::VolumeEfficiencyPolicy> provides a number of well-defined but
potentially complex or multi-step methods related to Volume Efficiency Policy
in ONTAP.

It builds on top of, and is a derived class of C<NACL::C::VolumeEfficiencyPolicy>,
and so it also provides methods that are more in the scope of
individual volume-related commands.  See C<NACL::C::VolumeEfficiencyPolicy> for
details.  This also means that a C<NACL::STask::VolumeEfficiencyPolicy> object may
generally be used in place of a component object.


=head1 METHODS

=head2 create

This accepts the same options and provides the same facilities as
L<NACL::C::VolumeEfficiencyPolicy::create|lib-NACL-C-VolumeEfficiencyPolicy-pm/create>.
Additionally, the STask provides the following facilities: C<Cleanup>,
C<nacltask_verify>, C<MCC checks>. The description of these features are
present at:
L<Common Create options|lib-NACL-STask-STask-pm/Common_Create_Options>.

=head2 purge

This accepts the same options and provides the same facilities as
L<NACL::C::VolumeEfficiencyPolicy::delete|lib-NACL-C-VolumeEfficiencyPolicy-pm/delete>.
Additionally, the STask provides the following facilities: C<nacltask_verify>,
C<MCC checks>. The description of these features are
present at:
L<Common Purge options|lib-NACL-STask-STask-pm/Common_Purge_Options>.

=cut

package NACL::STask::VolumeEfficiencyPolicy;

use base qw(NACL::C::VolumeEfficiencyPolicy NACL::STask::STask);

use strict;
use warnings;

__PACKAGE__->create_method_builder(mcc_check => 1);
__PACKAGE__->purge_method_builder(mcc_check => 1);

1;
