=head1 NAME

NACL::STask::VserverExportPolicyRule

=head1 DESCRIPTION

C<NACL::STask::VserverExportPolicyRule> provides a number of well-defined but
potentially complex or multi-step methods related to Vserver Export Policy Rule
in ONTAP.

It builds on top of, and is a derived class of C<NACL::C::VserverExportPolicyRule>,
and so it also provides methods that are more in the scope of
individual vserver-related commands.  See C<NACL::C::VserverExportPolicyRule> for
details.  This also means that a C<NACL::STask::VserverExportPolicyRule> object may
generally be used in place of a component object.


=head1 METHODS

=head2 create

This accepts the same options and provides the same facilities as
L<NACL::C::VserverExportPolicyRule::create|lib-NACL-C-VserverExportPolicyRule-pm/create>.
Additionally, the STask provides the following facilities: C<Cleanup>,
C<nacltask_verify>,C<MCC checks>.  The description of these features are
present at:
L<Common Create options|lib-NACL-STask-STask-pm/Common_Create_Options>.

=head2 purge

This accepts the same options and provides the same facilities as
L<NACL::C::VserverExportPolicyRule::delete|lib-NACL-C-VserverExportPolicyRule-pm/delete>.
Additionally, the STask provides the following facilities: C<nacltask_verify>,C<MCC checks>.
The description of these features are present at:
L<Common Purge options|lib-NACL-STask-STask-pm/Common_Purge_Options>.

=cut

package NACL::STask::VserverExportPolicyRule;

use base qw(NACL::C::VserverExportPolicyRule NACL::STask::STask);

use strict;
use warnings;
use Params::Validate qw( SCALAR );

__PACKAGE__->create_method_builder(
    mcc_check => 1,
    additional_spec => {
        ruleindex => { type => 'SCALAR', optional => 1 },
    }
);
__PACKAGE__->purge_method_builder(mcc_check => 1,);

1;
