# $Id: //depot/prod/test/nacldev/lib/NACL/STask/_Mixins/VolumeClone.pm#1 $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Mix-in for the VolumeClone STasks
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::STask::_Mixins::VolumeClone

=head1 DESCRIPTION

This mix-in contains the implementation for wait_for_creation method.

=cut

package NACL::STask::_Mixins::VolumeClone;

use strict;
use warnings;

BEGIN {
    use Exporter qw(import);
    use NACL::STask::_AggregateVolume qw(_job_validate_spec _extract_job
        POLL_DELTA AGGREGATE_TIMEOUT _valid_intermediate_states);
    our @EXPORT_OK = ( 'wait_for_creation', qw(_job_validate_spec _extract_job
        POLL_DELTA AGGREGATE_TIMEOUT _valid_intermediate_states) );
    our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );
}

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR SCALARREF BOOLEAN CODEREF);
use NATE::BaseException qw(:try);
use NACL::STask::VolumeClone;
use NACL::ComponentUtils qw(Dumper);

=head2 wait_for_creation

Wait for the volume or volume clone creation to complete.

See L<NACL::STask::Volume|lib-NACL-STask-Volume-pm> for a description of wait_for_creation
for volume creation.

See L<NACL::STask::VolumeClone|lib-NACL-STask-VolumeClone-pm> for a description of
wait_for_creation for volume clone creation.

=cut

sub wait_for_creation {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec   => {
            state => { type => SCALAR, default => 'online' },
            $pkg_or_obj->_job_validate_spec(),
            polling_interval => { type => SCALAR, default => POLL_DELTA },
        },
    );

    %opts       = $pkg_or_obj->_extract_job(%opts);
    my $state = delete $opts{state};
    my $till_value;

    # Map a state value of "force-online" and "force-offline"
    # to a terminal state of "online" and "offline" respectively
    if ($state eq 'force-online') {
        $till_value = 'online';
    } elsif ($state eq 'force-offline') {
        $till_value = 'offline';
    } else {
        $till_value = $state;
    }

    $opts{'method-timeout'} ||= AGGREGATE_TIMEOUT;

    my $pkg        = $pkg_or_obj->get_package_name();
    $pkg->wait_for_completion(
        %opts,
        attribute_to_check        => 'state',
        till_value                => $state,
        valid_intermediate_states => $pkg->_valid_intermediate_states(),
    );
    $Log->exit() if $may_exit;
} ## end sub wait_for_creation

1;
