#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::Service::ASUPTrigger;

use strict;
use warnings;

use parent qw(NATE::Service);

sub init {
    my ($self, %opts) = @_;
    $self->SUPER::init(%opts);

    my @public_apis = qw/
                                before_normal_test_start
        normal_test_started     normal_test_ended
        all_tests_ended         testcase_started
        testcase_ended          asup_put_url
        asup_post_url           asup_trigger_count
        add_api_tag             has_running_asups_for_api_tag
        testcase_name           enable_testcase_asup_tracking
        asup_put_url_v4         asup_post_url_v4
        asup_put_url_v6         asup_periodic_interval_max
        asup_post_url_v6        get_asup_activity_report
        get_errors              post_job_started
        nodes                   report_comment_to_syslog
        remove_api_tag          asup_periodic_interval_min
        has_running_test        asup_trigger_delay
        enable_asup_triggers    has_completed_asups_for_api_tag
    /;
    # These are for unit testing only
    my @testing_apis = qw/
        disable_asup_triggers   running_test
        completed_tests         running_asups
        running_testcases       completed_testcase_backlog
        completed_testcases     completed_asups
        clear_test_state        log_nate_error
        get_syslog_info         asup_type
        has_running_asups       has_completed_asups
        completed_asups         asup_validation_interval
        clear_asup_state        asup_trigger_interval_min
        asup_host_v4_ip         has_completed_testcase_backlog
        _add_running_asup       asup_validation_languish_timeout
        clear_errors            trust_hostrec
        reset                   enable_asup_connectivity_check
        asup_retry_count        asup_retry_interval
        conserve_asup_urls      wait_for_asups_collected
        node_timeout            last_validation_min_duration
        languishing_asups       last_validation_min_test_runtime
        has_languishing_asups   last_validation_max_duration
        has_timer_running       _remove_asup_periodic_timer
        remove_all_timers       state_dump
        has_node_objs           get_completed_asups_for_test
    /;

    my $structure = {};
    for my $api ( @public_apis, @testing_apis  ){
        $structure->{$api} = {
            worker_type => 'NACL::Service::ASUPTrigger::Worker',
            api_settings=> {
                client_sync     => 1,
                positional_args => 1,
            }
        };
    }
    $self->api_register_bulk(
        namespace => "NACL::Service::ASUPTrigger",
        structure => $structure
    );
}

use constant worker_pool_maximums => {
    'NACL::Service::ASUPTrigger::Worker' => 1,
};

1;
