#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @pod here
## @author dl-nacl-dev@netapp.com
## @summary An example formatter module for Xgen

package NACL::Service::Xgen::Examples::ExampleFormatter;

use strict;
use warnings;

###############################################################
### All formatter modules must derived classes of the base  ###
### formatter module.                                       ###
###############################################################
use parent qw(NACL::Service::Xgen::Formatter);

###############################################################
### This method to that will be executed against            ###
### each resource in the configuration.                     ###
###############################################################
sub apply_format_to_resource {
    my ($self, $resource_obj) = @_;

    # perform manipulation of the resource object

    # For example, lets say we need to upper case all attributes of the resource
    foreach my $key (keys %{$resource_obj}) {
        $resource_obj->{uc($key)} = delete $resource_obj->{$key};
    }
}

###############################################################
### This method that will be executed against               ###
### the entire resource list in the configuration.          ###
###############################################################
sub apply_format_to_resource_list {
    my ($self, $resource_hash_ref) = @_;

    # after "apply_format_to_resource" has been processed on all
    # resources, this method can be used to perform some post processing
    # of the entire resource hash.

    return $resource_hash_ref;
}

1;

__END__

=head1 NAME

NACL::Service::Xgen::Examples::ExampleFormatter

=head1 DESCRIPTION

This is a simple example implementation of an Xgen Formatter module.

=head1 IMPLEMENTATION

To create a formatter module, the very minimum that is needed is to make it a derived class of the base class formatter:
L<C<NACL::Service::Xgen::Formatter>|lib-NACL-Service-Xgen-Formatter-pm>. You should then implement either or both of the
abstract methods C<apply_format_to_resource> and C<apply_format_to_resource_list> that are defined by the
formatter base class.

=head2 Code

=over

 package NACL::Service::Xgen::Examples::ExampleFormatter;

 use strict;
 use warnings;

 ###############################################################
 ### All formatter modules must derived classes of the base  ###
 ### formatter module.                                       ###
 ###############################################################
 use parent qw(NACL::Service::Xgen::Formatter);

 ###############################################################
 ### This method to that will be executed against            ###
 ### each resource in the configuration.                     ###
 ###############################################################
 sub apply_format_to_resource {
     my ($self, $resource_obj) = @_;

     # perform manipulation of the resource object

     # For example, lets say we need to upper case all attributes of the resource
     foreach my $key (keys %{$resource_obj}) {
         $resource_obj->{uc($key)} = delete $resource_obj->{$key};
     }
 }

 ###############################################################
 ### This method that will be executed against               ###
 ### the entire resource list in the configuration.          ###
 ###############################################################
 sub apply_format_to_resource_list {
     my ($self, $resource_hash_ref) = @_;

     # after "apply_format_to_resource" has been processed on all
     # resources, this method can be used to perform some post processing
     # of the entire resource hash.

     return $resource_hash_ref;
 }

=back

=head1 SEE ALSO

See the base class for all APIs L<C<NACL::Service::Xgen::Formatter>|lib-NACL-Service-Xgen-Formatter-pm> that are available.

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut
