#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @pod here
## @author dl-nacl-dev@netapp.com
## @summary A complex example resource module for Xgen

package NACL::Service::Xgen::Examples::ExampleResourceComplex;

use strict;
use warnings;

# This is a derived class of the resource module
use parent qw(NACL::Service::Xgen::Resource);

use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NACL::Service::Xgen::Exceptions::InvalidAttribute;

#################################################################
### If this resource has a NACL ComponentState library that   ###
### it can be paired with, define it here. Pairing a resource ###
### with a NACL ComponentState library allows this resource   ###
### to be validated using the ComponentState Library's        ###
### constructor parameter validation.                         ###
#################################################################
# use constant nacl_cs_class => "NACL::CS::MyComponentStateLibrary";

##################################################################################
### If additional constructor validation needs to be performed against the     ###
### private attributes of this resource, the _validate_private_attrs() method  ###
### can be overridden. By default no private attribute validation is performed ###
### unless this method is overridden.                                           ###
##################################################################################
sub _validate_private_attrs {
    my ($self, $args) = @_;

    eval {
        $args = validate_with(
            params => [%{$args}],
            spec => {
                required_param1 => {type => SCALAR},
                required_param2 => {type => ARRAYREF},
                optional_param1 => {type => SCALAR, optional => 1},
            });
    };
    if (my $err = $@) {
        PTEFIT::Xgen::Exceptions::InvalidAttribute->throw($err);
    }

    return $args;
}

1;

__END__

=head1 NAME

NACL::Service::Xgen::Examples::ExampleResourceComplex

=head1 DESCRIPTION

This is a complex example implementation of an Xgen Resource module. See the base class for all APIs
L<C<NACL::Service::Xgen::Resource>|lib-NACL-Service-Xgen-Resource-pm> that are available.

=head1 IMPLEMENTATION

The private attributes of a resource can be validated.

=head2 Code

=over

 package NACL::Service::Xgen::Examples::ExampleResourceComplex;

 use strict;
 use warnings;

 # This is a derived class of the resource module
 use parent qw(NACL::Service::Xgen::Resource);

 use Params::Validate qw(validate_with SCALAR ARRAYREF);
 use NACL::Service::Xgen::Exceptions::InvalidAttribute;

 #################################################################
 ### If this resource has a NACL ComponentState library that   ###
 ### it can be paired with, define it here. Pairing a resource ###
 ### with a NACL ComponentState library allows this resource   ###
 ### to be validated using the ComponentState Library's        ###
 ### constructor parameter validation.                         ###
 #################################################################
 # use constant nacl_cs_class => "NACL::CS::MyComponentStateLibrary";

 ##################################################################################
 ### If additional constructor validation needs to be performed against the     ###
 ### private attributes of this resource, the _validate_private_attrs() method  ###
 ### can be overridden. By default no private attribute validation is performed ###
 ### unless this method is overridden.                                           ###
 ##################################################################################
 sub _validate_private_attrs {
     my ($self, $args) = @_;
     $Log->enter();

     eval {
         $args = validate_with(
             params => [%{$args}],
             spec => {
                 required_param1 => {type => SCALAR},
                 required_param2 => {type => ARRAYREF},
                 optional_param1 => {type => SCALAR, default => 0},
             });
     };
     if (my $err = $@) {
         PTEFIT::Xgen::Exceptions::InvalidAttribute->throw($err);
     }

     $Log->exit();
     return $args;
 }

=back

=head1 SEE ALSO

For a more simpler example, see
L<C<NACL::Service::Xgen::Examples::ExampleResource>|lib-NACL-Service-Xgen-Examples-ExampleResource-pm>

To see this resource being used in a configuration module, see
L<C<NACL::Service::Xgen::Examples::ExampleConfig>|lib-NACL-Service-Xgen-Examples-ExampleConfig-pm>

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut
