#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MergeCheckin Tools Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here


=head1 NAME

NACL::Tools::cdef_merge::MergeCheckin

=head1 DESCRIPTION

NACL::Tools::cdef_merge::MergeCheckin is a class which hold the routines
that will take care of all the check-in process, when the path
of directory which contains files that are to be checked-in.

=head1 METHODS

=head2 check_in

This method will take all the cdefs files present in the naclpath location.
and add or edit the file appropriately , creates a change number with the burt number
passed , and finally reopens all the files to that change nuber and checks in
returns the result of "nc_submit" irrespective of the submission result.

=cut

package NACL::Tools::cdef_merge::MergeCheckin;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub check_in {

    my ( $self, %args ) = @_;
    my $dir         = "$args{'nacl-cdefs-path'}->{$args{buildtype}}";
    my $burt_number = $args{burt_id};
    my $reviewer    = $args{reviewer};

    opendir( DIR, $dir )
        or do {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "couldnt open directory $dir ,Reason: $! \n");
        };
    my @cdef_files = grep( /^\S+\.(cdefs|xml)$/, readdir(DIR) )
        ;    #get the cdef files into an array
    closedir(DIR);

    #get the present directory location asn save it
    $present = `/bin/pwd`;
    chomp $present;

# change the current working direectory to $dir for convenience of check-in process
    chdir $dir;

    do {

# append all the file names that are to be checked-in this will be input to p4 commands
        $files = join( " ", @cdef_files );
        $files_reopen = $files
            ; # store all the file names just in case if merge process fail we can revert changes.
        } if (
        scalar(@cdef_files)
        or do {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "there are no cdef files please check directory $dir");
        }
        );

    # Get the current client and user as it is used in creating change file .
    open( FH1, "/usr/software/rats/bin/p4 info |" );
    @p4_info = <FH1>;
    close FH1;
    for my $rec (@p4_info) {
        if ( substr( $rec, 0, 12 ) eq "Client name:" ) {
            ( $info, $client ) = split( /: /, $rec );
        }
        if ( substr( $rec, 0, 10 ) eq "User name:" ) {
            ( $info, $user ) = split( /: /, $rec );
        }
    }

    chomp( $user, $client );

    open FH2,
        "> ./change.template";    #write a file with the present change data
    print FH2
        "Change: new\n\nClient: $client\n\nUser: $user\n\nStatus:
		new\n\nDescription:\n\tcheck-in of the cdef files\n\n\tburt$burt_number partial ;;\n\tReviewer: $reviewer ;;\n\tTesting: DONE <Unit Tested > ;;\n\t#######################\n\t##IMPORTANT NOTE##\n\t##DO NOT MODIFY BELOW VARIABLES IF YOU ARE NOT SURE##\n\tCONTINUE_ON_WARNING:0\n\tP4BURTCK:0\n\tP4REVCK:0\n\t########################\n";
    close FH2;
    open( FH3, "/usr/software/rats/bin/p4 change -i < ./change.template |" )
        ; # use p4 change command with -i option which takes data interactively
          #save the output of 'p4 change'.
    $change_number = <FH3>;
    close FH3;
    system("rm ./change.template")
        ;    #remove temporary file which contains change filedata
             # Get the change number from the output.
    $change_number =~ s/Change\s+(\d+)\s+created./$1/ig;
    $Log->trace(
        "the change number genrated is CHANGE_NUMBER: $change_number \n");

    #  execute p4 add on all the files and get the output
    $p4_add = `/usr/software/rats/bin/p4 add $files`;
    if ( $p4_add
        =~ /(It doesn't look like you're working in an existing client workspace)/
        ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "you're not working in an existing client workspace\n");
    }
    @p4_add_output = split( /\n/, $p4_add );
    my @p4_only_add_output = ();
    foreach $p4_add_result (@p4_add_output) {

        if ( $p4_add_result =~ /(.*)#\w+\s+-\s+opened for add/ ) {
            $filename = $1;
            $filename =~ s/.*\///;
            push @p4_only_add_output, $filename;
            $files =~ s/\s+$filename//;
        }
    }

    $Log->trace(" the files to be edited are $files \n");

    # run 'p4 edit' on the files which are to be edited if any.
    if ( $files !~ /^\s+$/ ) {
        @p4_edit_output = `/usr/software/rats/bin/p4 edit $files`;
    }

    # Revert the files which have no difference from the previous revision,
    # or if there is a difference of only whitespace or blank line
    my @string =  `/usr/software/rats/bin/p4 diff -db $files_reopen`;
    my ($revert, @revert_files);
    for(my $i=0; $i<= scalar (@string); $i++) {
        if ($string[$i] =~ /^==== .*\/(.*\.cdefs) ====$/) {
            $revert = $1;
            if ($string[$i+1] =~ /^==== .*\/(.*\.cdefs) ====$/) {
                push(@revert_files, $revert);
                $files_reopen =~ s/$revert //;
                next;
           }
           my $flag = 0;
           while($string[$i+1] !~ /^==== .*\/(.*\.cdefs) ====$/) {
               $i = $i + 1;
               if ($string[$i] !~ /^(<\s*|>\s*|---\s*|[\d,]+\w[\d,]+\s*)$/) {
                   $flag = 1;
                   last;
               }
           }
           if (!$flag) {
               push(@revert_files, $revert);
               $files_reopen =~ s/$revert //;
           }
       }
    }

    if(scalar(@revert_files)) {
        my $files_revert = join( " ", @revert_files );
        $Log->trace(" the files to be reverted are $files_revert \n");
        @string_revert =  `/usr/software/rats/bin/p4 revert $files_revert`;
    }

    #reopen all the files with the present change number
    chomp($change_number);
    open( FH4, "/usr/software/rats/bin/p4 reopen -c $change_number $files_reopen|" );
    @change_var = <FH4>;
    close FH4;

    
    # submit the change number using p4 submit
    $submit_result = `/usr/software/rats/bin/p4 submit -c $change_number `;
    if ( $? ) {
        `/usr/software/rats/bin/p4 unlock -c $change_number`;
        `/usr/software/rats/bin/p4 revert $files_reopen`;    # if submit fails revert all the changes
        unlink(@p4_only_add_output);
        chdir $present;
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("$submit_result");
    }

    # change the directory to intial working directory
    chdir $present;
    return $submit_result;

} ## end sub check_in
1;

