#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Smokestest Tools Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here


=head1 NAME

NACL::Tools::cdef_merge::Smokestest - runs smokes presub, cit-nacl (both debug
and non-debug variants)

=head1 REQUIRES

Tharn, File::Copy::Recursive,NATE::BaseException.

=head1 DESCRIPTION

NACL::Tools::cdef_merge::Smokestest is a class which runs smoke tests
that validates the results of cdef_merging. Based on the results
obtained the call for the other process like checkin will be decided.

=head1 METHODS

=head2 smoke_test

It will validate the results of the merge process based on the results obtained
and returns "1".

=cut

package NACL::Tools::cdef_merge::Smokestest;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::BaseException qw(:try);
use Connectrec;

sub smoke_test {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    my $buildroot = $args{workspace};
    # Strip  out the /test portion to get the buildroot
    $buildroot =~ s/test//g;
    my $smoke_client = $args{smoke_client};
    my $c = Connectrec->new(host=>$smoke_client, template => "rsh");
    
    # Run the Smokes presub and wait for it to finish so that we can capture the results
    $c->execute_start( command =>
                "/u/smoke/p4/smoke/web/v2/include/submit_job/submit.cgi -hardware_type "
                . "vsim_64_bit -remotesite RTP -buildroot $buildroot -variant "
                . "x86_64.sim -select_action Test_the_test -username ptcbuild -watchjob");

    my $c1 = Connectrec->new(host=>$smoke_client, template => "rsh");
    # Run cit-nacl with non-debug variant       
     $c1->execute_start( command =>
            "/u/smoke/p4/smoke/web/v2/include/submit_job/submit.cgi -hardware_type "
            . "vsim_64_bit -remotesite RTP -buildroot $buildroot -variant "
            . "x86_64.sim -suite cit-nacl -select_action CIT -username ptcbuild -watchjob");

    my $c2 = Connectrec->new(host=>$smoke_client, template => "rsh");
    # Run cit-nacl with debug variant
    $c2->execute_start( command =>
             "/u/smoke/p4/smoke/web/v2/include/submit_job/submit.cgi -hardware_type "
             . "vsim_64_bit -remotesite RTP -buildroot $buildroot -variant "
             . "x86_64.debug.sim -suite cit-nacl -select_action CIT -username ptcbuild -watchjob");

     # Capture all the results from the smoke runs
     my $result_presub = $c->execute_expect(timeout => 10800);
     my $result_cit_nacl_non_debug = $c1->execute_expect(timeout => 10800);
     my $result_cit_nacl_debug = $c2->execute_expect(timeout => 10800);

     my $smoke_report;
     my $failed;
     foreach my $test ($result_presub, $result_cit_nacl_non_debug, $result_cit_nacl_debug) {  
         if ($test !~ /ALL TESTS HAVE PASSED/) {
             $Log->comment("Smokes have failed. Please check logs");   
             $smoke_report .= "Smokes have failed. Please check logs -\n $test\n";
             $failed = 1;        
         } 
     }

     if ($failed) {
           $Log->exit() if $may_exit;
           NATE::BaseException->throw("$smoke_report\n");
     };     

}

1;

