# A shared library for http_proxy.pl and http_server.pl

package http_util;

use strict;
use warnings;

use Exporter qw(import);
our @EXPORT_OK = qw(my_url best_local_ip_for_peer);

use IO::Socket qw(INADDR_ANY);
use Socket qw(inet_ntoa);

# Like HTTP::Daemon->url, but prefer IP address to name (avoid gethostbyaddr)

sub my_url {
    my $httpd = shift;
    my $url   = "http://";
    my $addr  = $httpd->sockaddr;
    if ( !$addr || $addr eq INADDR_ANY ) {
        require Sys::Hostname;
        $url .= lc Sys::Hostname::hostname();
    }
    else {
        $url .= inet_ntoa($addr);
    }
    my $port = $httpd->sockport;
    $url .= ":$port" if $port != 80;
    $url .= "/";
    $url;
} ## end sub my_url

# ----------------------------------------------------------------------
# best_local_ip_for_peer
#
# For a given remote address, return the proper local IP address to
# send to that peer if it wishes to contact us.

# Just exactly what the right address is depends on which interface
# we're using to reach the peer.  Instead of using nonstandardized
# interfaces to try to read through routing tables, let's use
# standardized interfaces to connect a UDP socket (which doesn't
# actually send any packets) and then ask what local address was
# chosen by the system for that socket.

sub best_local_ip_for_peer {
    my ($addr) = @_;
    my $sock = IO::Socket::INET->new(
        Proto    => "udp",
        PeerAddr => $addr,
        PeerPort => 7,       # don't care
    ) or die "Could not create/connect socket to \"$addr\": $!";
    return inet_ntoa( $sock->sockaddr() );
} ## end sub best_local_ip_for_peer

1;
