#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::UnitTest::AppBuilder::EnGenX::EnGenX_Xgen_Demo::Demo3;

use strict;
use warnings;

use parent qw( NACL::UnitTest::AppBuilder::EnGenX::EnGenX_Xgen_Demo::Demo2 );

use NACL::AppBuilder::Xgen::Resource::Vserver;

use NACL::Service::Xgen::Util qw(convert_percent_layout_to_count_layout
                                 get_next_layout_element);

# This config is based on Demo2 and will build the same numbers as Demo2.
# In this config we update the generate_vserver_resources to generate vservers in
# a differet way based on the below percent layout mapping.
# 50% of vservers will have ntfs, 50% will have mixed
use constant VSERVER_ROOT_VOLUME_SECURITY_LAYOUT => {
    ntfs  => 50,
    mixed => 50
};

# Generates vserver resources based on the TOTAL_VSERVER_COUNT constant
# and the VSERVER_ROOT_VOLUME_SECURITY_LAYOUT constant
# If there is more than one vserver then vserver will alternate aggrs
# based on the aggrs found in ONTAP.pm
sub generate_vserver_resources {
    my ($self, %args) = @_;

    my $aggr_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Aggregate");
    my @aggrs = @{$aggr_resources};
    my @vservers;

    # This will get the layout in terms of numbers instead of percents based on the total object count passed in.
    my $layout = convert_percent_layout_to_count_layout(layout => $self->VSERVER_ROOT_VOLUME_SECURITY_LAYOUT,
                                                        total_objects => $self->TOTAL_VSERVER_COUNT);

    for (my $i = 0; $i < $self->TOTAL_VSERVER_COUNT; $i++) {
        my $aggr = shift(@aggrs);
        push(@aggrs, $aggr);

        # get the next security element to use from the layout
        my $sec = get_next_layout_element(iterator => $i,      # This is not required but allows the get_next_layout_element to
                                                               # rotate through the layout instead of exhausting the first, then
                                                               # the second, etc.
                                          layout => $layout);  # layout is the numbers layout we got earlier.

        my $vs_res = NACL::AppBuilder::Xgen::Resource::Vserver->new(vserver =>"Vserver$i",
                                                                    aggregate => $aggr->get_attribute("aggregate"),
                                                                    'rootvolume-security-style' => $sec);

        # connect the vs resource to the aggr's cluster parent resource
        $vs_res->add_parent_resources(resources => [$aggr->get_ancestors(type => "NACL::AppBuilder::Xgen::Resource::Cluster")]);

        push(@vservers, $vs_res);
    }
    $self->add_resources(resources => \@vservers);
}

1;

__END__

=head1 NAME

PTEFIT::Xgen::Config::Example

=head1 AUTHOR/MAINTAINER

=over

=item PTEFIT AUTOMATION (ng-ptefit-automation-core@netapp.com)

=back

   Last revised $Date: $ by $Author: $

=cut
