#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::UnitTest::AppBuilder::Xgen::Config::Aggregate;

use strict;
use warnings;

use parent qw( NACL::AppBuilder::Xgen::Config::ONTAP );

use NACL::AppBuilder::Xgen::Resource::Aggregate;
# These are the constant values that determine what happens in the config.
use constant TOTAL_AGGR_COUNT => 1;
use Tharn;

sub build {
    my ($self, %args) = @_;

    $self->SUPER::build(%args);
    # Call to generate aggregate resources.
    $self->generate_aggregate_resources(%args);	

}

sub generate_aggregate_resources {
    my ($self, %args) = @_;


    my $nacl_node_objs = $self->{nacl_node_objs};

    my $aggr_info = NACL::AppBuilder::Xgen::Resource::Aggregate->generate_aggr_info(nacl_node_objs => $nacl_node_objs);

    # create Aggr objects based on the content of $aggr_info
    my @aggr_resources = ();

    # loop over all nodes in the cluster
    foreach my $node_name (keys(%{$aggr_info})) {
        my $aggr_name_hash = {};
        my $node_resource = $self->get_node_resource($node_name);

        # create Aggreate Resource objects
        for (my $i = 0; $i < TOTAL_AGGR_COUNT; $i++) {
            # base aggregate name
            my $aggr_name = "Aggr_" . $node_name . "_" . $i;
            $aggr_name =~ s/-/_/g;
            my $aggr_resource = new NACL::AppBuilder::Xgen::Resource::Aggregate(aggregate   => $aggr_name,
                                                                      diskcount => 5,
                                                                      nodes => [$node_name],);

            $aggr_resource->add_parent_resources(resources => [$node_resource]);
            push (@aggr_resources, $aggr_resource);
        }
    }

    $self->add_resources(resources => \@aggr_resources);

}

1;

