#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::UnitTest::AppBuilder::Xgen::Config::File;

use strict;
use warnings;

use parent qw( NACL::UnitTest::AppBuilder::Xgen::Config::Volume );

use NACL::AppBuilder::Xgen::Resource::File;

# These are the constant values that determine what happens in the config.
use constant TOTAL_FILE_COUNT => 1;
use constant FILE_SIZE => '20m';

# Override the volume size since we need a bigger volume to contain the file.
use constant VOLUME_SIZE => '100m';

# This demo is a very simple example that creates vservers and volumes based on constant values.
# This demo makes 1 vserver and 2 volumes.  It uses aggregates that are found from it's parent's (ONTAP.pm)
# build function.  This demo assumes that there are data aggregates that already exist.
sub build {
    my ($self, %args) = @_;

    $self->SUPER::build(%args);

    # Call to generate file resources.
    $self->generate_file_resources(%args);

}

# Generates file resources based on the TOTAL_FILE_COUNT constant.
# If there is more than one file then file will alternate volumes
# based on the volumes created in NACL::UnitTest::AppBuilder::Xgen::Config::Volume
sub generate_file_resources {
    my ($self, %args) = @_;

    my $volume_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Volume");
    my @volumes = @{$volume_resources};
    my @files;

    for (my $i = 0; $i < $self->TOTAL_FILE_COUNT; $i++) {
        my $volume = shift(@volumes);
        push(@volumes, $volume);

        my $path = '/vol/' . $volume->get_attribute('volume') . "/file$i";
        my $file_res = NACL::AppBuilder::Xgen::Resource::File->new('vserver'   => $volume->get_attribute("vserver"),
                                                                   'path'      => $path,
                                                                   'file-size' => $self->FILE_SIZE);

        # connect the vs resource to the volume's cluster parent resource
        $file_res->add_parent_resources(resources => [$volume]);

        push(@files, $file_res);
    }
    $self->add_resources(resources => \@files);
}

1;

