#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::UnitTest::AppBuilder::Xgen::Config::Iscsi;

use strict;
use warnings;

use parent qw( NACL::AppBuilder::Xgen::Config::ONTAP );

use NACL::AppBuilder::Xgen::Resource::Vserver;
use NACL::AppBuilder::Xgen::Resource::Iscsi;

# These are the constant values that determine what happens in the config.
use constant TOTAL_VSERVER_COUNT => 1;

# This demo is a very simple example that creates vservers and volumes based on constant values.
# This demo makes 1 vserver and 2 volumes.  It uses aggregates that are found from it's parent's (ONTAP.pm)
# build function.  This demo assumes that there are data aggregates that already exist.
sub build {
    my ($self, %args) = @_;

    $self->SUPER::build(%args);

    # Call to generate vserver resources.
    $self->generate_vserver_resources(%args);

    # Call to generate iscsi resourses
    $self->generate_iscsi_resources(%args);
}

# Generates vserver resources based on the TOTAL_VSERVER_COUNT constant.
# If there is more than one vserver then vserver will alternate aggrs
# based on the aggrs found in ONTAP.pm
sub generate_vserver_resources {
    my ($self, %args) = @_;

    my $aggr_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Aggregate");
    my @aggrs = @{$aggr_resources};
    my @vservers;

    for (my $i = 0; $i < $self->TOTAL_VSERVER_COUNT; $i++) {
        my $aggr = shift(@aggrs);
        push(@aggrs, $aggr);

        my $vs_res = NACL::AppBuilder::Xgen::Resource::Vserver->new(vserver =>"Vserver$i",
                                                                    aggregate => $aggr->get_attribute("aggregate"));

        # connect the vs resource to the aggr's cluster parent resource
        $vs_res->add_parent_resources(resources => [$aggr->get_ancestors(type => "NACL::AppBuilder::Xgen::Resource::Cluster")]);

        push(@vservers, $vs_res);
    }
    $self->add_resources(resources => \@vservers);
}

sub generate_iscsi_resources {
    my ($self, %args) = @_;

    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
    my @vservers = @{$vserver_resources};
    my @iscsi;

    foreach my $vs (@vservers) {
        my $iscsi_res = NACL::AppBuilder::Xgen::Resource::Iscsi->new(vserver   => $vs->get_attribute("vserver"),
                                                                 'status-admin' => "up",
                                                                 'target-alias' => $vs->get_attribute("vserver"));

        $iscsi_res->add_parent_resources(resources => [$vs]);

        push(@iscsi,$iscsi_res);
    }
    $self->add_resources(resources => \@iscsi);
}

1;

