#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::UnitTest::AppBuilder::Xgen::Config::Lun;

use strict;
use warnings;

use parent qw( NACL::UnitTest::AppBuilder::Xgen::Config::Volume );

use NACL::AppBuilder::Xgen::Resource::Vserver;
use NACL::AppBuilder::Xgen::Resource::Volume;
use NACL::AppBuilder::Xgen::Resource::Lun;

# These are the constant values that determine what happens in the config.
use constant VOLUME_SIZE => "200m";

use constant LUN_SIZE => "32m";
use constant TOTAL_LUN_COUNT => "1";
use constant LUN_OS_TYPE => "windows_2008";

sub build {
    my ($self, %args) = @_;

    $self->SUPER::build(%args);

    # Call to generate lun resources
    $self->generate_lun_resources(%args);
}

sub generate_lun_resources {
    my ($self, %args) = @_;

    my $vol_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Volume");
    my @vol = @{$vol_resources};
    my @luns;

    for (my $i = 0; $i < $self->TOTAL_LUN_COUNT; $i++) {
        my $vol = shift(@vol);
        push(@vol, $vol);

        my $lun_res = NACL::AppBuilder::Xgen::Resource::Lun->new(vserver    => $vol->get_attribute("vserver"),
                                                                 volume     => $vol->get_attribute("volume"),
                                                                 lun        => "Lun_netapp$i",
                                                                 size       => $self->LUN_SIZE,
                                                                 ostype     => $self->LUN_OS_TYPE,);
        $lun_res->add_parent_resources(resources => [$vol]);
        push(@luns, $lun_res);
    }
    $self->add_resources(resources => \@luns);
}

1;

