#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::UnitTest::AppBuilder::Xgen::Config::LunBind;

use strict;
use warnings;

use parent qw( NACL::UnitTest::AppBuilder::Xgen::Config::Lun );

use NACL::AppBuilder::Xgen::Resource::LunBind;

use constant TOTAL_LUN_COUNT => "2";


sub build {
    my ($self, %args) = @_;

    $self->SUPER::build(%args);

    # Call to generate lun resources.
    $self->generate_lun_bind_resources(%args);
}

# Generates 1 lun_bind resource.
sub generate_lun_bind_resources {
    my ($self, %args) = @_;

    my $lun_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Lun");
    my @luns = @{$lun_resources};

    # Can only do this if we have both a vvol and protocol-endpoint.
    return unless (scalar @luns >= 2);

    #Use the first 2 luns in the list.
    my $vvol = $luns[0];
    my $pe   = $luns[1];
    my $vvol_path = "/vol/" . $vvol->get_attribute('volume') . "/" . $vvol->get_attribute('lun');
    my $pe_path = "/vol/" . $pe->get_attribute('volume') . "/" . $pe->get_attribute('lun');
    $vvol->set_private_attributes('class' => 'vvol');
    $pe->set_private_attributes('class' => 'protocol-endpoint');
    my $lun_bind_res = NACL::AppBuilder::Xgen::Resource::LunBind->new('vserver'                 => $vvol->get_attribute("vserver"),
                                                                      'protocol-endpoint-path'  => $pe_path,
                                                                      'vvol-path'               => $vvol_path);

    # connect the lun_binds resource to the lun parent resources
    $lun_bind_res->add_parent_resources(resources => [$vvol,$pe]);

    $self->add_resources(resources => [$lun_bind_res]);
}

1;

