#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::UnitTest::AppBuilder::Xgen::Config::Retry_Error_Strings;

use strict;
use warnings;

use parent qw( PTEFIT::Xgen::Config::ONTAP );

use NACL::AppBuilder::Xgen::Resource::Vserver;
use NACL::AppBuilder::Xgen::Resource::Volume;

use constant TOTAL_VSERVER_COUNT => 1;
use constant TOTAL_VOLUME_COUNT => 3;


sub build {
    my ($self, %args) = @_;

    $self->SUPER::build(%args);

    my $aggr_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Aggregate");
    my @aggrs = @{$aggr_resources};
    my @vservers;
    my @volumes;

    for (my $i = 0; $i < $self->TOTAL_VSERVER_COUNT; $i++) {
        my $aggr = shift(@aggrs);
        push(@aggrs, $aggr);

        my $vs_res = NACL::AppBuilder::Xgen::Resource::Vserver->new(vserver =>"Vserver$i",
                                                                    aggregate => $aggr->get_attribute("aggregate"));

        # connect the vs resource to the aggr's cluster parent resource
        $vs_res->add_parent_resources(resources => [$aggr->get_ancestors(type => "NACL::AppBuilder::Xgen::Resource::Cluster")]);

        push(@vservers, $vs_res);
    }
    $self->add_resources(resources => \@vservers);

    for (my $i = 0; $i < $self->TOTAL_VOLUME_COUNT; $i++) {
        my $aggr = shift(@aggrs);
        push(@aggrs, $aggr);

        my $vserver = shift(@vservers);
        push(@vservers, $vserver);

        my %extra_args = ();
        $extra_args{xgen_metadata} = {retry_strings => ['Duplicate volume name Vol1']} if ($i == 1);
        my $volume_res = NACL::AppBuilder::Xgen::Resource::Volume->new(vserver   => $vserver->get_attribute("vserver"),
                                                                       aggregate => $aggr->get_attribute("aggregate"),
                                                                       volume    => "Vol$i",
                                                                       size      => '20m',
                                                                       %extra_args);

        $volume_res->set_metadata_attributes(retry_strings => ['Duplicate volume name Vol0']) if($i == 0);

        # connect the vs resource to the aggr's cluster parent resource
        $volume_res->add_parent_resources(resources => [$vserver,$aggr]);

        push(@volumes, $volume_res);
    }
    $self->add_resources(resources => \@volumes);
}

1;

__END__

=head1 NAME

PTEFIT::Xgen::Config::Example

=head1 AUTHOR/MAINTAINER

=over

=item PTEFIT AUTOMATION (ng-ptefit-automation-core@netapp.com)

=back

   Last revised $Date: $ by $Author: $

=cut
