#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::UnitTest::AppBuilder::Xgen::Config::VserverPeer;

use strict;
use warnings;

use parent qw( NACL::UnitTest::AppBuilder::Xgen::Config::Vserver );

use NACL::AppBuilder::Xgen::Resource::Vserver;
use NACL::AppBuilder::Xgen::Resource::VserverPeer;

# These are the constant values that determine what happens in the config.
use constant TOTAL_VSERVER_COUNT => 2;

sub build {
    my ($self, %args) = @_;

    $self->SUPER::build(%args);

    $self->generate_vserverpeer_resources(%args);

}


sub generate_vserverpeer_resources {
    my ($self, %args) = @_;

    my $cluster_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Cluster");
    my @clusters = @{$cluster_resources};
    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
    my @vservers =  @{$vserver_resources};
    my @vserverpeer;

    my $cluster = shift(@clusters);
    push(@clusters, $cluster);

    my $vs_res = NACL::AppBuilder::Xgen::Resource::VserverPeer->new(vserver => $vservers[0]->get_attribute("vserver"),
                                                                    'peer-vserver' => $vservers[1]->get_attribute("vserver"),
                                                                    );

    # connect the vs resource to the aggr's cluster parent resource
    $vs_res->add_parent_resources(resources => [$cluster]);

    push(@vserverpeer, $vs_res);
    $self->add_resources(resources => \@vserverpeer);
}

1;

