#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ChooseAPISet Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::UnitTest::C::ChooseAPISet

=head1 DESCRIPTION

C<NACL::UnitTest::C::ChooseAPISet> is a derived class of
L<NACL::C::Component|lib-NACL-C-Component-pm>. As the name suggests, it is a
dummy piece of code and does not model any ontap feature. It is used to check if
proper APISet is chosen based on the interface/mode/param values.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=cut

package NACL::UnitTest::C::ChooseAPISet;

use strict;
use warnings;

use Params::Validate qw(validate_with SCALAR ARRAYREF BOOLEAN SCALARREF);
use base 'NACL::C::Component::ONTAP';

=head1 METHODS

=head2 dummy_method

  my $Dummy = NACL::UnitTest::C::ChooseAPISet->dummy_method(command_interface=>$ci);

Return the apiset chosen.

=cut

sub dummy_method {
    my $pkg  = shift;
    my %opts = @_;
    return $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_dummy_to_return_apiset",
            },
        ],
    );
}

sub dummy_choices_method {
    my $pkg  = shift;
    my %opts = @_;
    return $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_dummy_to_return_apiset",
	        interface => 'CLI|ZAPI'
            },
        ],
    );
}

# Back end method that just returns the apiset chosen.

my $routine = sub {
    my $pkg = shift;
    my %opts = @_;

    my $apiset = $opts{apiset};

    my $parent_method = (caller(1))[3];

    $parent_method =~ s/.*::(\S+)/$1/;

    $apiset->{method} = $parent_method;

    return $apiset;

};

sub _dummy_to_return_apiset {
    $routine->(@_);
}

sub _dummy_to_return_cli {
    $routine->(@_);
}

sub _dummy_to_return_zapi {
    $routine->(@_);
}

sub _dummy_to_return_zapi_non_iter {
    $routine->(@_);
}


sub dummy_zapi_type_method {
    my $pkg  = shift;
    my %opts = @_;
    return $pkg->call_on_apiset(
        %opts,
            choices => [
                {   method    => "_dummy_to_return_cli",
                    interface => "CLI",
                    set       => "CMode",
                    zapi_type => 'none',
                },
                {   method    => "_dummy_to_return_zapi",
                    interface => "ZAPI",
                    zapi_type => "iter",
                    set       => "CMode",
                },
                {   method    => "_dummy_to_return_zapi_non_iter",
                    interface => "ZAPI",
                    zapi_type => "non-iter",
                    set       => "CMode",
                },
            ],
    );
}

# Tests for the non-iter check method. We set vserver-context and primary keys
# as pk1 and pk2
sub dummy_non_iter_check {
    my ($pkg, %opts) = @_;
    return $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_backend_zapi',
                interface => 'ZAPI',
                zapi_type => 'iter',
            },
            {   method    => '_backend_zapi_non_iter',
                interface => 'ZAPI',
                zapi_type => 'non-iter',
                check     => '_check_method',
            },
        ],
    );
}

sub _backend_zapi { 'iter' }
sub _backend_zapi_non_iter { 'non-iter' }

sub _check_method {
    my ($pkg, %opts) = @_;
    $pkg->_base_check_non_iter(
        %opts,
        _primary_keys => [qw(pk1 pk2)],
        _vs_context   => 1
    );
}

1;
