# Copyright (c) 2009-2014 NetApp Inc.
# All rights reserved.
#
## @summary Library to unittest GatherDiagnostics functionality
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=pod

=head1 Package Name

NACL::UnitTest::GatherDiagnostics

=head1 Author/Maintainer

dl-nacl-dev

=head1 Description

Library to unittest GatherDiagnostics functionality

=head1 NACL::Exceptions::GatherDiagnostics::UnitTestMethod1

This exception is thrown by method1_executed

=head1 NACL::Exceptions::GatherDiagnostics::UnitTestMethod2

This exception is thrown by method2_executed

=head2 Base Class

NATE::BaseException

=cut

package NACL::UnitTest::GatherDiagnostics;

use Params::Validate qw(validate_with SCALAR HASHREF);

use NACL::Exceptions::GatherDiagnostics::UnitTest;
use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use Storable qw(freeze thaw);

sub method1_called {
    my ($self, %args) = @_;
    $self->_execute_with_gather_diag(
        method => 'method1_executed',
        exception => 'NACL::Exceptions::GatherDiagnostics::UnitTest',
        method_args => \%args,
    );
}

sub method1_executed {
    my ($self) = @_;
    NACL::Exceptions::NoElementsFound->throw(
        "NACL::UnitTest::GatherDiagnostics::method1_executed fails to execute"
    );
}

sub method2_called {
    my ($self, %args) = @_;
    $self->_execute_with_gather_diag(
        method => 'method2_executed',
        exception => 'NACL::Exceptions::GatherDiagnostics::UnitTest',
        method_args => \%args,
    );
}

sub method2_executed {
    my ($self) = @_;
    NACL::Exceptions::Timeout->throw(
        "NACL::UnitTest::GatherDiagnostics::method2_executed fails to execute"
    );
}

sub _execute_with_gather_diag {
    my ($self, @opts) = @_;

    my %opts = validate_with(
        params => \@opts,
        spec   => {
            method => { type => SCALAR },
            exception => { type => SCALAR, },
            method_args => { type => HASHREF, optional => 1},
        },
    );

    my $method = $opts{method};
    my $exception = $opts{exception};
    my %args;
    if (defined $opts{method_args}) {
        %args = $opts{method_args};
    }

    try {
        $self->$method(%args);
    } otherwise {
        my $ex = shift();
        my $ex_obj = thaw(freeze($ex));;
        
        # Reblessing $exception to be of type $ex_obj
        $exception->convert(exception => $ex_obj);
        
        $ex_obj->make_base_of($ex);
        $ex_obj->throw(text => $ex->text);
    };
}

1;
