#!/usr/software/bin/perl5.14.0

use strict;
use warnings;

use Test::Most import => ['!subtest'];

use NACL::UnitTest::MTask::MCC::MockObject;
use NACL::UnitTest::MTask::MCC::TestUtils ':all';

MAIN: {
    test_use_mcc();
    test_steady_state();
    test_uso();
    test_nso();
    test_nso_mirrored();
    test_nso_resync();
    test_nso_throws();
    test_heal_aggregates();
    test_heal_aggregates_resync();
    test_heal_roots();
    test_heal_roots_dies();
    test_heal_roots_fake_init();
    test_heal_aggrs_reconstructing();
    test_normal_reconstructing();
    done_testing();
}

sub test_steady_state {

    my $mcc = make_mock_mcc();

    $mcc->set_always(
        'get_disaster_status',
        {   DR_state        => 'normal',
            'survivor_site' => 'A',
            'disaster_site' => 'B',
            'live_sites'    => ['A', 'B']
        }
    );

    my $mock_aggr_mod = mock_many_aggregates(
        mcc => $mcc,
        aggr_states => [
        ({
            state => 'online',
            raidstatus => 'raid_dp, mirrored, normal',
            'is_home' => 'true'
        }) x 4
    ]
    );


    $mcc->set_always(
        'get_disaster_status',
        {   DR_state        => 'normal',
            'live_sites'    => ['A', 'B']
        }
    );

    my $rc;

    lives_ok {
        $rc = $mcc->verify_aggregate_status('die_on_failure' => 1);
    };

    return;


} ## end sub test_aggr_verify_many_steady_state

sub test_uso {

    my $mcc = make_mock_mcc();

    $mcc->set_always(
        'get_disaster_status',
        {   DR_state        => 'switchover_completed',
            'survivor_site' => 'A',
            'disaster_site' => 'B',
            'live_sites'    => ['A', 'B']
        }
    );

    my $mock_aggr_mod = mock_many_aggregates(
        mcc => $mcc,
        aggr_states => [
            ({
                state => 'online',
                raidstatus => 'raid_dp, mirrored, normal',
                'is_home' => 'true'
            }) x 4,
            ({
                'state' => 'offline',
                'raidstatus' => 'raid_dp, mirror degraded',
                'is_home' => 'false'
            })
        ]
    );

    my $rc;

    lives_ok {
        $rc = $mcc->verify_aggregate_status('die_on_failure' => 1);
    };

    return;


}

sub test_nso {

    my $mcc = make_mock_mcc();

    $mcc->set_always(
        'get_disaster_status',
        {   DR_state        => 'switchover_completed',
            'survivor_site' => 'A',
            'disaster_site' => 'B',
            'live_sites'    => ['A', 'B']
        }
    );

    my $mock_aggr_mod = mock_many_aggregates(
        mcc => $mcc,
        aggr_states => [
            ({
                state => 'online',
                raidstatus => 'raid_dp, mirrored, normal',
                'is_home' => 'true'
            }) x 4,
            ({
                'state' => 'offline',
                'raidstatus' => 'raid_dp, mirror degraded',
                'is_home' => 'false',
            }),
            ({
                'state' => 'online',
                'raidstatus' => 'raid_dp, mirrored, normal',
                'is_home' => 'false',
            }) x 4,

        ]
    );

    my $rc;

    lives_ok {
        $rc = $mcc->verify_aggregate_status('die_on_failure' => 1);
    };

    return;


}

sub test_nso_mirrored {

    my $mcc = make_mock_mcc();

    $mcc->set_always(
        'get_disaster_status',
        {   DR_state        => 'switchover_completed',
            'survivor_site' => 'A',
            'disaster_site' => 'B',
            'live_sites'    => ['A', 'B']
        }
    );

    my $mock_aggr_mod = mock_many_aggregates(
        mcc => $mcc,
        aggr_states => [
            ({
                'state' => 'online',
                'raidstatus' => 'raid_dp, mirror degraded',
                'is_home' => 'true',
            }),
            ({
                'state' => 'offline',
                'raidstatus' => 'raid_dp, mirror degraded',
                'is_home' => 'false',
            }),
            ({
                'state' => 'online',
                'raidstatus' => 'raid_dp, mirrored, normal',
                'is_home' => 'false',
            }) x 4,

        ]
    );

    my $rc;

    lives_ok {
        $rc = $mcc->verify_aggregate_status('die_on_failure' => 1);
    };

    return;


}

sub test_nso_resync {

    my $mcc = make_mock_mcc();

    $mcc->set_always(
        'get_disaster_status',
        {   DR_state        => 'switchover_completed',
            'survivor_site' => 'A',
            'disaster_site' => 'B',
            'live_sites'    => ['A', 'B']
        }
    );

    my $mock_aggr_mod = mock_many_aggregates(
        mcc => $mcc,
        aggr_states => [
            ({
                'state' => 'online',
                'raidstatus' => 'raid_dp, resyncing',
                'is_home' => 'true',
            }),
            ({
                'state' => 'offline',
                'raidstatus' => 'raid_dp, mirror degraded',
                'is_home' => 'false',
            }),
            ({
                'state' => 'online',
                'raidstatus' => 'raid_dp, mirrored, normal',
                'is_home' => 'false',
            }) x 4,

        ]
    );

    my $rc;

    lives_ok {
        $rc = $mcc->verify_aggregate_status('die_on_failure' => 1);
    };

    return;

}


sub test_nso_throws {

    my $mcc = make_mock_mcc();

    $mcc->set_always(
        'get_disaster_status',
        {   DR_state        => 'switchover_completed',
            'survivor_site' => 'A',
            'disaster_site' => 'B',
            'live_sites'    => ['A', 'B']
        }
    );

    my $mock_aggr_mod = mock_many_aggregates(
        mcc => $mcc,
        aggr_states => [
            ({
                state => 'online',
                raidstatus => 'raid_dp, mirrored, abnormal',
                'is_home' => 'true'
            }) x 4,
            ({
                'state' => 'offline',
                'raidstatus' => 'raid_dp, mirror degradeded',
                'is_home' => 'false',
            }),
            ({
                'state' => 'online',
                'raidstatus' => 'raid_dp, mirrored, abnormal',
                'is_home' => 'false',
            }) x 4,

        ]
    );

    my $rc;

    dies_ok {
        $rc = $mcc->verify_aggregate_status('die_on_failure' => 1);
    } 'Should die on abnormal';

    return;


}

sub test_heal_aggregates {

    my $mcc = make_mock_mcc();

    $mcc->set_always(
        'get_disaster_status',
        {   DR_state        => 'heal_aggrs_completed',
            'survivor_site' => 'A',
            'disaster_site' => 'B',
            'live_sites'    => ['A', 'B']
        }
    );

    my $mock_aggr_mod = mock_many_aggregates(
        mcc => $mcc,
        aggr_states => [
        ({
            state => 'online',
            raidstatus => 'raid_dp, mirrored, normal',
            'is_home' => 'true'
        }) x 4,
        ({
            state => 'online',
            raidstatus => 'raid_dp, resyncing',
            'is_home' => 'false'
        }),
    ]
    );


    my $rc;

    lives_ok {
        $rc = $mcc->verify_aggregate_status('die_on_failure' => 1);
    };

    return;


}

sub test_heal_aggregates_resync {

    my $mcc = make_mock_mcc();

    $mcc->set_always(
        'get_disaster_status',
        {   DR_state        => 'heal_aggrs_completed',
            'survivor_site' => 'A',
            'disaster_site' => 'B',
            'live_sites'    => ['A', 'B']
        }
    );

    my $mock_aggr_mod = mock_many_aggregates(
        mcc => $mcc,
        aggr_states => [
        ({
            state => 'online',
            raidstatus => 'raid_dp, mirrored, resyncing',
            'is_home' => 'true'
        }) x 4,
        ({
            state => 'online',
            raidstatus => 'raid_dp, resyncing',
            'is_home' => 'false'
        }),
    ]
    );


    my $rc;

    lives_ok {
        $rc = $mcc->verify_aggregate_status('die_on_failure' => 1);
    };

    return;


}

sub test_heal_roots {
    my $mcc = make_mock_mcc();

    $mcc->set_always(
        'get_disaster_status',
        {   DR_state        => 'heal_roots_completed',
            'survivor_site' => 'A',
            'disaster_site' => 'B',
            'live_sites'    => ['A', 'B']
        }
    );

    my $mock_aggr_mod = mock_many_aggregates(
        mcc => $mcc,
        aggr_states => [
        ({
            state => 'online',
            raidstatus => 'raid_dp, mirrored, normal',
            'is_home' => 'true',
            'empty' => 0,
        }) x 4,
        ({
            state => 'online',
            raidstatus => 'raid_dp, mirrored, normal',
            'is_home' => 'false',
            'empty' => 0
        }),
    ]);

    my $rc;

    lives_ok {
        $rc = $mcc->verify_aggregate_status('die_on_failure' => 1);
    };

    ok(defined $mcc->{'_validation_log'}{'aggregate'}{'successful'}, 'logged success');
}

sub test_heal_roots_fake_init {
    my $mcc = make_mock_mcc();

    $mcc->set_always(
        'get_disaster_status',
        {   DR_state        => 'heal_roots_completed',
            'survivor_site' => 'A',
            'disaster_site' => 'B',
            'live_sites'    => ['A', 'B']
        }
    );

    my $mock_aggr_mod = mock_many_aggregates(
        mcc => $mcc,
        aggr_states => [
        ({
            state => 'online',
            raidstatus => q{raid_dp, mirrored, parity uninit'd},
            'is_home' => 'true',
            'empty' => 0,
        }) x 4,
        ({
            state => 'online',
            raidstatus => 'raid_dp, mirrored, normal',
            'is_home' => 'false',
            'empty' => 0
        }),
    ]);

    my $rc;

    lives_ok {
        $rc = $mcc->verify_aggregate_status('die_on_failure' => 1);
    };

    # Should have skipped validation
    ok(! defined $mcc->{'_validation_log'}{'aggregate'}, 'no success logged for fake_init' );


}


sub test_heal_roots_dies {
    my $mcc = make_mock_mcc();

    $mcc->set_always(
        'get_disaster_status',
        {   DR_state        => 'heal_roots_completed',
            'survivor_site' => 'A',
            'disaster_site' => 'B',
            'live_sites'    => ['A', 'B']
        }
    );

    my $mock_aggr_mod = mock_many_aggregates(
        mcc => $mcc,
        aggr_states => [
        ({
            state => 'online',
            raidstatus => 'raid_dp, mirrored, normal',
            'is_home' => 'true',
        }) x 4,
        ({
            state => 'online',
            raidstatus => 'raid_dp, degraded, normal',
            'is_home' => 'false',
        }),
    ]);

    my $rc;

    dies_ok {
        $rc = $mcc->verify_aggregate_status('die_on_failure' => 1);
    };

}

sub test_heal_aggrs_reconstructing {
    my $mcc = make_mock_mcc();

    set_mcc_state_heal_aggregates($mcc);

    my $mock_aggr_mod = mock_many_aggregates(
        mcc => $mcc,
        aggr_states => [
        ({
            state => 'online',
            raidstatus => 'raid_dp, mirrored, normal',
            'is_home' => 'true',
            'empty' => 0,
        }) x 4,
        ({
            state => 'online',
            raidstatus => 'raid_dp, degraded, mirrored',
            'is_home' => 'true',
            raidgroups => [
                q{/sti6220mcc_229_aggr1/plex0/rg0 (block)},
                q{/sti6220mcc_229_aggr1/plex1/rg0 (block, recomputing parity:0%)}
            ]
        }),
        ({
            state => 'online',
            raidstatus => 'raid_dp, reconstruct, mirrored',
            'is_home' => 'true',
            'empty' => 0,
        }) x 4
    ]);

    my $rc;

    lives_ok {
        $rc = $mcc->verify_aggregate_status('die_on_failure' => 1);
    };

    ok(defined $mcc->{'_validation_log'}{'aggregate'}{'successful'}, 'logged success for reconstructing');


}


sub test_normal_reconstructing {
    my $mcc = make_mock_mcc();

    set_mcc_state_normal($mcc);

    my $mock_aggr_mod = mock_many_aggregates(
        mcc => $mcc,
        aggr_states => [
        ({
            state => 'online',
            raidstatus => 'raid_dp, reconstruct, mirrored',
            'is_home' => 'true',
            raidgroups => [
                '"/aggr0_sti6220mcc_208_0/plex0/rg0 (block)"',
                '"/aggr0_sti6220mcc_208_0/plex2/rg0 (block, reconstructing:40%)"'
            ]
        }) x 4
    ]);

    my $rc;

    lives_ok {
        $rc = $mcc->verify_aggregate_status('die_on_failure' => 1);
    } 'post root ok';

    ok(defined $mcc->{'_validation_log'}{'aggregate'}{'successful'}, 'logged success for reconstructing');


}
