#!/usr/software/bin/perl5.14.0

use strict;
use warnings;

use Test::Most import => ['!subtest'];

use NACL::UnitTest::MTask::MCC::MockObject;
use NACL::UnitTest::MTask::MCC::TestUtils ':all';
use Carp;
use Params::Validate
    qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR UNDEF);

use NACL::MTask::MCC::Cluster;

MAIN: {
    test_use_mcc();
    test_cluster_validate();
    done_testing();
}

sub test_cluster_validate {

    # Create mock MCC
    my $mcc = make_mock_mcc();

    # Serialize calls to 'subtest'.
    my ($mcc_mod, $mock_subtest) = mock_subtest();

    # Make mock cluster
    my $cluster = NACL::MTask::MCC::Cluster->new(
        metrocluster => $mcc,
        site         => 'A'
    );

    set_mcc_state_normal($mcc);

    $mcc->set_always('L_determine_live_sites', ['A', 'B']);

    my $result = Test::MockObject->new();
    my $cs_mcc = Test::MockModule->new('NACL::CS::MetroclusterOperation');
    $cs_mcc->mock(
        'fetch',
        sub {
            return $result;
        }
    );

    $result->set_always('operation', 'check');
    $result->set_always('state',     'successful');
    $result->set_always('error_list', [
              q{"Use the "metrocluster check show" command }
            . q{to view the results of this operation."} ]);

    lives_ok(
        sub {
            $cluster->validate_operation();

        },
        "Did not die with a successful metrocluster check command"
    );

} ## end sub test_cluster_validate

