#!/usr/software/bin/perl5.14.0

use strict;
use warnings;

use Test::Most import => ['!subtest'];

use NACL::UnitTest::MTask::MCC::MockObject;
use NACL::UnitTest::MTask::MCC::TestUtils ':all';
use NACL::MTask::MCC::Validator::ClusterShow;

use Carp;
use Params::Validate
    qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR UNDEF);

MAIN: {
    test_use_mcc();
    test_vtor_basic();
    test_vtor_dies();
    done_testing();
}

sub test_vtor_basic {

    # Create mock MCC
    my $mcc = make_mock_mcc();

    my ($cs_cluster, $cs_mod) = mock_cs_cluster();

    check_lives_ok( metrocluster => $mcc, name => 'normal ok' );

    set_mcc_state_switchover($mcc);

    check_lives_ok( metrocluster => $mcc, name => 'switchover ok' );

    set_mcc_state_heal_aggregates($mcc);

    check_lives_ok( metrocluster => $mcc, name => 'heal aggr ok' );

    set_mcc_state_heal_roots($mcc);

    check_lives_ok( metrocluster => $mcc, name => 'heal roots ok' );

    # Cleanup
    set_mcc_state_normal($mcc);

} ## end sub test_vtor_normal

sub check_lives_ok {

    my %in = validate_with(
        params => \@_,
        spec   => {
            'metrocluster' => {
                'type' => OBJECT,
            },
            'name' => {
                'type' => SCALAR,
            }
        }
    );

    lives_ok {
        my $vtor = NACL::MTask::MCC::Validator::ClusterShow->new(
            metrocluster => $in{'metrocluster'}
        );

        $vtor->run();
    } $in{'name'}


}

sub test_vtor_dies {


    # Create mock MCC
    my $mcc = make_mock_mcc();

    my ($cs_cluster, $cs_mod) = mock_cs_cluster();

    $cs_cluster->set_always('health', 'false');

    throws_ok {
        my $vtor = NACL::MTask::MCC::Validator::ClusterShow->new(
            metrocluster => $mcc
        );

        $vtor->run();
    } 'NACL::MTask::MCC::Exceptions::Validation';

}
