#!/usr/software/bin/perl5.14.0
# This Perl version should match the Perl used by NATE

# This is a UnitTest for Options when used outside of NATE

use common::sense;
use Data::Dumper;

use FindBin;
use lib "$FindBin::Bin/../..";
use Options;

my $Options = new Options;

$Options->parameters(definitions=>
    [
        {
            name         => "THIRTY_SEVEN_SECONDS",
            spec         => "time",
            description  => "Time string",
            default      => "37s",
        },
        {
            name         => "THIRTY_SEVEN",
            spec         => "time",
            description  => "Time string",
            default      => "37",
        },
        {
            name         => "EIGHTY_FOUR_MINUTES",
            spec         => "time",
            description  => "Time string",
            default      => "84m",
        },
        {
            name         => "EIGHTY_FOUR_MINUTES_LONG",
            spec         => "time",
            description  => "Time string",
            default      => "84 mins",
        },
        {
            name         => "TEN_MINUTES_THREE_SECONDS",
            spec         => "time",
            description  => "Time string",
            default      => "10m3s",
        },
        {
            name         => "TEN_MINUTES_THREE_SECONDS_LONG",
            spec         => "time",
            description  => "Time string",
            default      => "10 minutes 3 seconds",
        },
        {
            name         => "TWO_HOURS",
            spec         => "time",
            description  => "Time string",
            default      => "2h",
        },
        {
            name         => "TWO_HOURS_FIVE_MINUTES",
            spec         => "time",
            description  => "Time string",
            default      => "2h5m",
        },
        {
            name         => "TWO_HOURS_FIVE_MINUTES_HMS",
            spec         => "time",
            description  => "Time string",
            default      => "2:05:00",
        },
        {
            name         => "TWO_HOURS_FIVE_SECONDS",
            spec         => "time",
            description  => "Time string",
            default      => "2h5s",
        },
        {
            name         => "TWO_HOURS_FIVE_MINUTES_TEN_SECONDS",
            spec         => "time",
            description  => "Time string",
            default      => "2h5m10s",
        },
        {
            name         => "TWO_HOURS_FIVE_MINUTES_TEN_SECONDS_LONG",
            spec         => "time",
            description  => "Time string",
            default      => "2 hours 5 mins 10s",
        },
    ]
);

$Options->process();

my $opt_href = $Options->get_hash();

    my %verify = (
                     TWO_HOURS_FIVE_MINUTES_TEN_SECONDS => (2*60*60 + 5*60 + 10),
                     TWO_HOURS_FIVE_MINUTES_TEN_SECONDS_LONG  => (2*60*60 + 5*60 + 10),
                     TWO_HOURS_FIVE_MINUTES             => (2*60*60 + 5*60),
                     TWO_HOURS_FIVE_MINUTES_HMS         => (2*60*60 + 5*60),
                     TWO_HOURS                          => (2*60*60),
                     TWO_HOURS_FIVE_SECONDS             => (2*60*60 + 5),
                     THIRTY_SEVEN                       => 37,
                     THIRTY_SEVEN_SECONDS               => 37,
                     EIGHTY_FOUR_MINUTES                => (84*60),
                     EIGHTY_FOUR_MINUTES_LONG           => (84*60),
                     TEN_MINUTES_THREE_SECONDS          => (10*60) + 3,
                     TEN_MINUTES_THREE_SECONDS_LONG     => (10*60) + 3,
                 );

print "\n";

    foreach my $key (sort keys %{$opt_href}) {
        if ( $key !~ /NUFFO/ ) {
            if ( exists( $verify{$key} ) ) {
                if ( $opt_href->{$key} eq $verify{$key} ) {
                    print("PARAM ".sprintf("%35s", $key)." equals ".sprintf("%5s", $verify{$key})." as expected\n");
                } else {
                    die("PARAM ".sprintf("%35s", $key)." should equal ".sprintf("%5s", $verify{$key}). " but is ".$opt_href->{$key} . "\n");
                }
            } else {
                warn("PARAM $key has no validation data defined in \%verify hash\n");
            }
        }
    }

print "\n";
