# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary General Utility methods
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::Util::General;

=head1 NAME

NACL::Util::General

=head1 DESCRIPTION

C<NACL::Util::General> is a collection of utility methods which are
specific to only  NACL libraries and is not meant to be used outside of
NACL code.

=cut

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate :types);

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw( _throw_internal_error );
}

=head2 _throw_internal_error

    $pkg->_throw_internal_error( exception => $exception_obj );

This method is used to throw the internal error by NACL libraries.

=over

=item Options

=over

=item C<< exception => $exception_obj >>

(Mandatory, OBJECT)
Exception object to be used to throw the error.

=back

=back

=cut

sub _throw_internal_error {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate( @args, { exception => { type => OBJECT, isa => 'NATE::BaseException' } } );
    my $exception = $opts{'exception'};

    my $error = "Internal error: \n";
    $error .= $exception->text() . "\n";
    $error
        .= "Email dl-nacl-dev or create BURT(type=NACL,subtype=nacl_core)\n";
    $exception->set_text($error);
    $Log->exit() if $may_exit;
    $exception->throw($error);
}

1;
