# $Id$
#---------------------------------------------------
# Copyright (c) 2016 Network Appliance, Inc.
# All rights reserved.
#---------------------------------------------------
## @summary
##      Wrapper Utility for Selenium webDriver functions.
##
## @description
##      This modules contains all helper functions for SMTT libraries
##      to interact with selenium web driver
##
## @synopsis
##      use NACL::APISet::Host::GUI::SMTT::V3_1::SMTTUtils;
##
##          _login(%args);
##          _wait_for_element_not_present(%args);
##          _wait_for_element_present(%args);
##          _wait_for_loading(%args);
##          _is_element_present(%args);
##          _is_element_enabled(%args);
##          _find_element(%args);
##          _find_elements(%args);
##          _type(%args);
##          _click(%args);
##          _check_for_error(%args);
##          _get_all_window_names(%args);
##          _select_window(%args);
##          _is_checked(%args);
##          _get_value(%args);
##          _get_text(%args);
##          _clear_window(%args);
##          _click_button_having_specific_text(%args);
##          _click_button_having_specific_text_no_error_check(%args);
##          _smtt_navigate(%args);
##          _close_tab(%args);
##          _throw_exception(%args);
##          _take_screenshot(%args);
##          _repair_session(%args);
##          _get_host_from_ip(%args);
##          _session_exist(%args);
##          _get_all_sessions(%args);
##          _add_volumepairs_for_session(%args);
##          _add_vserver(%args);
##          _select_aggregate(%args);
##          _do_volume_mapping(%args);
##          _change_session_name_to_given_value(%args);
##          _add_volumes_given_vol_ref_array(%args);
##          _is_button_enabled(%args);
##          _wait_on_attribute(%args);
##          _add_credentials(%args);
##          _remove_credentials(%args);
##          _add_schedule_days(%args)
##          _is_key_x_present_for_session_y(%args);
##          _get_value_for_key_x_key_y_in_session_z(%args);
##          _put_value_for_key_x_key_y_in_session_z(%args);
##          _get_volume_sm_details(%args);
##          _get_data_copy_schedule_details(%args);
##          _create_helper(%args);
##          _create_primary(%args);
##          _get_volume_selection_details(%args);
##          _get_volume_selection_details_helper(%args);
##          _create_screenshot(%args);
##          _perform_operation(%args);
##          _select_vsm(%args);
##          _cft_create_helper(%args);
##          _cft_prerquisite_handle(%args);
##          is_element_present(%args);
##          is_element_enabled(%args);
##          is_checked(%args);
##          get_value(%args);
##          get_text(%args);
##          get_all_sessions(%args);
##          is_button_enabled(%args);
##          findElement(%args);
##          findElements(%args);
##          get_all_window_names(\%args);
##          session_exist(\%args);
##          set_page(\%args);
##          get_session_details(\%args);
##          _plan_configuration(\%args);
##          _edit_configuration(\%args);
##          _edit_data_IP_tab(\%args);
##          get_edit_tab_details(\%args);
##          _edit_SVM_Map(%args);
##          _edit_SVM_tab_edit_Cluster_Config(%args);
##          _edit_Network_Configuration(%args);
##          _edit_Source_Volumes(%args);
##          _edit_Plan_config(%args)
##          _edit_Config_Schedule(%args)
##          _edit_Map_Volumes(%args);
##
## @status Review
##
## @author santosh6@netapp.com, rohit4@netapp.com, dl-nacl-dev@netapp.com
##
## @change     2015-06-24 santosh6 : Added CFT supporting subroutines
## @change     2015-07-13 santosh6 : parsed_output logic for all CFT related commands
## @change     2016-02-28 dinoc    : Added changes for V3.0 parser
## @change     2016-03-07 ashan    : Added changes in _cft_create_helper for CFT
## @change     2016-03-10 bipink   : Added changes in _create_helper for CBT
## @change     2016-03-21 ashan    : Added changes for plan configuration (_plan_configuration)
## @change     2016-06-02 ashan    : Added _edit_configuration
## @change     2016-07-18 dinoc    : Added changes for V3.1 parser

package NACL::APISet::Host::GUI::SMTT::V3_1::SMTTUtils;

#
## Compiler directives.
#
use strict;
use warnings;

## Module import.
use Selenium::Remote::WebElement;
use Selenium::Remote::WDKeys;

use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::APISet::Host::GUI::SMTT::V3_1::GlobalConst;
use NACL::APISet::Host::GUI::SMTT::GConst;
use NATE::BaseException qw(:try);

use base 'Selenium::Remote::Driver';

################################################################################
# P U B L I C   M E T H O D S
################################################################################

###############################################################################
## @name ::_login
##
## @summary
##     Logs in to SMTT
## @description
##     This method can be used to log into the SMTT
## @synopsis
##     _login(%args);

sub _login {

    my ( $self ) = @_;

    $Log->enter() if $may_enter;

    $self->_type( $LOGIN_USERNAME ,$self->{username});
    $self->_type( $LOGIN_PASSWORD ,$self->{password});

    $self->_click(
        id                      => $LOGIN_BUTTON,
        error_checking_needed   => 0,
    ) ;

    #my @elems=$self->find_elements("//*[\@type='button']");
    #$elems[1]->click();

    print "finished login\n";
    $Log->exit() if $may_exit;

    return 1;
} ## end sub _login

###############################################################################
## @name ::_wait_for_element_not_present
##
## @summary
##     This method is used to wait till the element disappears or timeout occurs
## @synopsis
##     _wait_for_element_not_present(%args);

sub _wait_for_element_not_present {

    my ( $self, $debugID, $timeout ) = @_;

    $Log->enter() if $may_enter;

    my $retVal = 1;
    $self->_wait_for_loading();

    my $sleep_count = $timeout ? $timeout : ($GENERIC_WAIT / 1000);
    for ( my $i = 0 ; $i < $sleep_count ; $i++ ) {
        eval {
            $self->_find_element($debugID);
            $Log->comment("Element $debugID still exists");
            Tharn::snooze(1);
            $retVal = 0;
        };
        if ($@) {
            $retVal = 1;
            last;
        }
    }

    $Log->exit() if $may_exit;

    return $retVal;
} ## end sub _wait_for_element_not_present

###############################################################################
## @name ::_wait_for_element_present
##
## @summary
##     This method is used to wait till the element appears or timeout occurs
## @synopsis
##     _wait_for_element_present(%args);

sub _wait_for_element_present {

    my ( $self, $debugID, $timeout ) = @_;

    $Log->enter() if $may_enter;

    my $retVal = 0;
    $self->_wait_for_loading();

    my $sleep_count = $timeout ? $timeout : ($GENERIC_WAIT / 1000);
    for ( my $i = 0 ; $i < $sleep_count ; $i++ ) {
        eval {
            $self->_find_element($debugID);
            $retVal = 1;
        };
        if ($@) {
            $Log->comment( "Waiting for $debugID : " . $@ );
            Tharn::snooze(1);
        } else {
            last;
        }
    }

    $Log->exit() if $may_exit;

    return $retVal;
} ## end sub _wait_for_element_present

###############################################################################
## @name ::_wait_for_loading
##
## @summary
##     This method is used to wait till the page loads
## @synopsis
##     _wait_for_loading(%args);

sub _wait_for_loading {

    my ( $self ) = @_;

    $Log->enter() if $may_enter;

    my $sleep_count = $self->{timeout} || $GENERIC_WAIT / 1000;
    Tharn::snooze(5);

    $self->set_implicit_wait_timeout(1000);

    #$GENERIC_RETRIEVING_WAIT
    #while ($self->_is_element_present($GENERIC_PLEASE_WAIT) ) {
        while ($self->_is_element_present($GENERIC_RETRIEVING_WAIT) ) {
        $Log->comment("Waiting for Page to load..");
        Tharn::snooze(2);
        $sleep_count = $sleep_count - 1;
        if ($sleep_count == 0) {
            $self->_throw_exception("Page load timeout in _wait_for_loading()");
        }
    }

    $self->set_implicit_wait_timeout(5000);

    $Log->exit() if $may_exit;

    return 1;
} ## end sub _wait_for_loading

###############################################################################
## @name ::_is_element_present
##
## @summary
##     This method is used check if the element exists on the page
## @synopsis
##     _is_element_present(%args);

sub _is_element_present {

    my ( $self, $debugID ) = @_;

    $Log->enter() if $may_enter;

    eval { $self->_find_element($debugID); };
    if ($@) {
        $Log->exit() if $may_exit;
        return 0;
    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub _is_element_present

###############################################################################
## @name ::is_element_present
##
## @summary
##     This method is used check if the element exists on the page
## @synopsis
##     is_element_present(%args);

sub is_element_present {

    my ( $self, $debugID ) = @_;

    $Log->enter() if $may_enter;

    my $output->{parsed_output}->{response} = $self->_is_element_present($debugID);

    $Log->exit() if $may_exit;

    return $output;
} ## end sub is_element_present

###############################################################################
## @name ::_is_element_enabled
##
## @summary
##     This method is used check if the element is enabled
## @synopsis
##     _is_element_enabled(%args);

sub _is_element_enabled {

    my ( $self, $debugID ) = @_;

    $Log->enter() if $may_enter;

    my $res;
    eval {
        my $elem = $self->_find_element($debugID);
        $res = $elem->is_enabled();
    };

    if ($@) {
        $Log->exit() if $may_exit;
        return 0 ;
    }
    unless ($res) {
        $Log->exit() if $may_exit;
        return 0 ;
    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub _is_element_enabled

###############################################################################
## @name ::is_element_enabled
##
## @summary
##     This method is used check if the element is enabled
## @synopsis
##     is_element_enabled(%args);

sub is_element_enabled {

    my ( $self, $debugID ) = @_;

    $Log->enter() if $may_enter;

    my $output->{parsed_output}->{response} = $self->_is_element_enabled($debugID);

    $Log->exit() if $may_exit;

    return $output;
} ## end sub is_element_enabled

###############################################################################
## @name ::_find_element
##
## @summary
##     This method returns the element if it is found on the page
## @synopsis
##     _find_element(%args);

sub _find_element {

    my ( $self, $debugID ) = @_;

    $Log->enter() if $may_enter;

    my $elem_obj;
    if ( $debugID =~ /^\/\// ) {
        eval { $elem_obj = $self->find_element($debugID); };
        if ($@) {
            $self->_throw_exception("$@");
        }
    } else {
        eval { $elem_obj = $self->find_element( $debugID, 'id' ); };
        if ($@) {
            $self->_throw_exception("$@");
        }
    }

    $Log->exit() if $may_exit;

    return $elem_obj;
} ## end sub _find_element

###############################################################################
## @name ::findElement
##
## @summary
##     This method returns the element if it is found on the page
## @synopsis
##     findElement(%args);

sub findElement {

    my ( $self, $debugID ) = @_;

    $Log->enter() if $may_enter;

    my $output->{parsed_output}->{response} = $self->_find_element($debugID);

    $Log->exit() if $may_exit;

    return $output;
} ## end sub findElement

###############################################################################
## @name ::_find_elements
##
## @summary
##     This method returns the elements if they are found on the page
## @synopsis
##     _find_elements(%args);

sub _find_elements {

    my ( $self, $debugID ) = @_;

    $Log->enter() if $may_enter;

    my @elem_obj ;
    if ( $debugID =~ /^\/\// ) {
        eval {
            @elem_obj = $self->find_elements($debugID);
        };
    } else {
        eval {
            @elem_obj = $self->find_elements( $debugID, 'id' );;
        };
    }

    $Log->exit() if $may_exit;

    return @elem_obj;
} ## end sub _find_elements

###############################################################################
## @name ::findElements
##
## @summary
##     This method returns the elements if they are found on the page
## @synopsis
##     findElements(%args);

sub findElements {

    my ( $self, $debugID ) = @_;

    $Log->enter() if $may_enter;

    my @elemt = $self->_find_elements($debugID);
    my $output->{parsed_output}->{response} = \@elemt ;

    $Log->exit() if $may_exit;

    return $output;
} ## end sub findElements

###############################################################################
## @name ::_type
##
## @summary
##     This method is used for typing the text in the input box specified
## @synopsis
##     _type(%args);

sub _type {

    my ( $self, $debugID, $typeText ) = @_;

    $Log->enter() if $may_enter;

    my $elem = $self->_find_element($debugID);
    $elem->clear();
    $elem->send_keys($typeText);

    $Log->exit() if $may_exit;

    return 1;
} ## end sub _type

###############################################################################
## @name ::_click
##
## @summary
##     This method is used to click the element specified
## @synopsis
##     _click(%args);

sub _click {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;
    my $aref;
    if ( ref($self) =~ /WebElement/ ) {
        $self->click();
        $self->_wait_for_loading();
        $aref = $self->_check_for_error(arguments => $args{arguments}) if ( $args{error_checking_needed}
        );
        $Log->exit() if $may_exit;
        return $aref;
    }

    $Log->comment("Click Element : $args{id}");
    my $elem = $self->_find_element($args{id});
    $elem->click();
    $self->_wait_for_loading();
    $aref = $self->_check_for_error( arguments => $args{arguments}) if ( $args{error_checking_needed}
    );

    $Log->comment("Click on Element $args{id} successfull");

    $Log->exit() if $may_exit;

    return $aref;
} ## end sub _click

###############################################################################
## @name ::_check_for_error
##
## @summary
##     This method is used to check for errors on the page after each click
## @synopsis
##     _check_for_error(%args);

sub _check_for_error{

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;

    # DS to store the parsed output
    my $aref=[];

    $self->set_implicit_wait_timeout(4000);

    while( $self->_is_element_present($GENERIC_POPUP) ){

        my $sleep_count = $GENERIC_WAIT / 1000;
        Tharn::snooze($sleep_count);

        my $popup_title;

        # Putting an eval, maybe popup was there but not present now
        eval{
            $popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
        };
        print Dumper($popup_title);
        # Since popup was present earlier but not present now so skipping checking
        unless ($@) {

            if ( $popup_title =~ /Error/ ) {

                # Dying here with the error message
                my $popup_text=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_MESSAGE);
                $self->_create_screenshot("$popup_text");
                $self->_repair_session();
                $self->_throw_exception(
                    "Popup Title : $popup_title , Popup error message : $popup_text");

            }elsif ( $popup_title =~ /Volume loading/ ){

                # Wait till all volumes are being loaded
                my $wait = $self->{timeout} || $GENERIC_VOLUMEPAIR_LOAD;
                unless( $self->_wait_for_element_not_present( $GENERIC_POPUP,$wait ) ){
                    $self->_create_screenshot("Volume_not_found");
                    $self->_repair_session();
                    $self->_throw_exception(
                        "Volumes did not get loaded even after $wait s of wait");
                }

            }elsif ( $popup_title =~ /Warning/i || $popup_title =~ /Information/i
                || $popup_title =~ /Incomplete Subproject/i
                || $popup_title =~  /Confirmation/i || $popup_title =~  /Transition Pause/i
                || $popup_title =~  /Transition Abort/i){
                my $popup_txt;
                # Handle all Generic pop-ups here
                # We do not want to fail if any click fails
                eval{
                    $popup_txt=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_MESSAGE);
                    $self->_click(
                        id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                        error_checking_needed   => 0,
                    ) ;
                };
                if($popup_txt =~ /cutover operation/i){
                    eval{
                        $popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
                    };
                    unless($@) {
                        if ( $popup_title =~ /Transition Complete/i) {
                            if ($self->_is_element_present($CFT_CHECKBOX_CONFIRM)) {
                                $self->_click(
                                    id                      => $CFT_CHECKBOX_CONFIRM,
                                    error_checking_needed   => 0,
                                ) ;
                            }
                            # Handle all Generic pop-ups here
                            # We do not want to fail if any click fails
                            eval{
                                $self->_click(
                                    id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                                    error_checking_needed   => 0,
                                ) ;
                            };
                        }
                    }
                } elsif ($popup_txt =~/remove this 7\-Mode/i){
                    eval{
                        $popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
                    };
                    if ( $popup_title =~ /Error/i ) {

                        # Dying here with the error message
                        my $popup_text=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_MESSAGE);
                        $self->_create_screenshot("$popup_text");
                        $self->_repair_session();
                        $self->_throw_exception(
                            "Popup Title : $popup_title , Popup Error message : $popup_text"
                        );
                    } 
                }
            }elsif ( $popup_title =~ /audit logs destination path/i || $popup_title =~ /Add\/Remove Volumes List/i){

                # Handle Audit logging here
                eval {
                    $self->_click_button_having_specific_text("Continue");
                };
                eval{
                    $self->_click(
                        id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                        error_checking_needed   => 0,
                    ) ;
                };
            }elsif ( $popup_title =~ /New Transition Project Name/i ) {

                # Returning back from here and handling this in the caller
                $Log->exit() if $may_exit;
                return 1;

            }elsif ( $popup_title =~ /Create Data Copy Schedule/i ) {

                # Returning back from here and handling this in the caller
                $Log->exit() if $may_exit;
                return 1;

            }elsif ($popup_title =~ /Operation Progress/i || $popup_title =~ /Precheck Status/i ) {

                my $timer = $self->{timeout} || $GENERIC_PRECHECKS_WAIT;
                my $new_popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
                while ( ($new_popup_title !~ /Precheck Status/) && ($timer > 0) ) {

                    # Waiting here for prechecks to get completed
                    $timer--;
                    my $summary_text = $self->_get_text($PRECHECK_SUMMARY_TEXT);
                    my $sleep_count = $GENERIC_WAIT / 1000;
                    Tharn::snooze($sleep_count);
                    $new_popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
                    if ( ($new_popup_title =~ /Operation Progress/i) && ($summary_text =~ /finished successfully/) ){
                        last;
                    }
                    if ( ($new_popup_title =~ /Operation Progress/i) && ($summary_text =~ /has failed/) ){
                        last;
                    }

                }

                if($timer == 0){
                    eval {
                        $self->_click(
                            id                      => $CLOSE_BUTTON,
                            error_checking_needed   => 0,
                        ) ;
                    };

                    $self->_take_screenshot();
                    $self->_repair_session();
                    $self->_throw_exception(
                        "Operation did not get completed even after 1800 secs");
                }

                eval{
                    $self->_click(
                        id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                        error_checking_needed   => 0,
                    ) if ($new_popup_title =~ /Precheck Status/);
                };

                my $summary_text = $self->_get_text($PRECHECK_SUMMARY_TEXT);

                # Getting job results data in a DS
                my $href={};
                $href->{Informational}=[];
                $href->{Error}=[];
                $href->{Warning}=[];
                eval {
                    if($self->_is_element_present($VIEW_RESULTS)){
                        $self->_click(
                            id                      => $VIEW_RESULTS,
                            error_checking_needed   => 0,
                        ) ;
                    }elsif($self->_is_element_present($VIEW_ROLLBACK_STEP_RESULTS)){
                        $self->_click(
                            id                      => $VIEW_ROLLBACK_STEP_RESULTS,
                            error_checking_needed   => 0,
                        ) ;
                    }
                };
                if (! $@ ){
                    my @keys = ("code","type","message","corrective_action","affected_objects");

                    my $row = 1;
                    while (1){
                        my ( $type, %chunk_hash );
                        my $x=2;
                        my $y = $JOB_RESULTS;
                        $y =~ s/(.*)ROW_NUMBER(.*)COLUMN_NUMBER(.*)/$1$row$2$x$3/;
                        eval{
                            $type = $self->_get_text($y) ;
                        };
                        if ($@) {
                            last;
                        }
                        if ($self->_is_element_present($GENERATE_REVERT_MANUAL)) {
                            my $x=2;
                            my $y = $JOB_RESULTS;
                            $y =~ s/(.*)ROW_NUMBER(.*)COLUMN_NUMBER(.*)/$1$row$2$x$3/;
                            my $message;
                            $message = $self->_get_text($y) ;
                            my @final = split(" : ",$message);
                            $chunk_hash{"code"} = $final[0];
                            $chunk_hash{"message"} = $final[1];

                            my $msg_type = $MSG_TYPE;
                            $msg_type =~ s/(.*)XXYYZZ(.*)/$1$row$2/;

                            if ($self->_is_element_present($msg_type)) {
                                my $value = $self->_get_text($msg_type);
                                $chunk_hash{"type"} = $self->_get_text($msg_type);
                                push @{$href->{$value}} , {%chunk_hash};
                            }else{
                                my $msg_step = $MSG_STEP;
                                $msg_step =~ s/(.*)XXYYZZ(.*)/$1$row$2/;
                                my $value = $row - 2;;
                                $chunk_hash{"step"} = "Step ".$value ;
                                my $affected_obj = $AFFECTED_OBJ;
                                $affected_obj =~ s/(.*)XXYYZZ(.*)/$1$row$2/;
                                if ($self->_is_element_present($affected_obj)) {
                                    $self->_click(
                                        id                      => $affected_obj,
                                        error_checking_needed   => 0,
                                    ) ;
                                    if ($self->_is_element_present($CLUSTERSHELL)) {
                                        my $cmode_ip = $args{"info_hash"}{"c_hostname"};
                                        my $var = 1;
                                        while ( 1 ) {
                                            my $msg = $REVERT_COMMAND;
                                            $msg =~ s/(.*)XXYYZZ(.*)/$1$var$2/;
                                            if ($self->_is_element_present($msg)) {
                                                $msg = $self->_get_text($msg);
                                                push(@{$chunk_hash{$cmode_ip}},$msg);
                                                $var++;
                                            }else{
                                                last;
                                            }
                                        }
                                    }elsif($self->_is_element_present($S_MODE_COMMAND)){
                                        if($self->_is_element_present($SEVEN_MODE_CONTROLLER)){
                                            my $var = 1;
                                            while ( 1 ) {
                                                my $msg = $SEVEN_MODE_COMMAND;
                                                $msg =~ s/(.*)XXYYZZ(.*)/$1$var$2/;
                                                if ($self->_is_element_present($msg)) {
                                                    $msg = $self->_get_text($msg);
                                                    my $smode_ip = $SEVEN_MODE_IP;
                                                    $smode_ip =~ s/(.*)XXYYZZ(.*)/$1$var$2/;
                                                    $smode_ip = $self->_get_text($smode_ip);
                                                    if ($smode_ip =~ /(.*)[(](.*)[)]/){
                                                        $smode_ip = $2;
                                                    }
                                                    push(@{$chunk_hash{$smode_ip}},$msg);
                                                    $var++;
                                                }else{
                                                    last;
                                                }
                                            }
                                        }else{
                                            my $smode_ip = $args{"info_hash"}{"s_hostname"};
                                            my $var = 1;
                                            while ( 1 ) {
                                                my $msg = $REVERT_COMMAND;
                                                $msg =~ s/(.*)XXYYZZ(.*)/$1$var$2/;
                                                if ($self->_is_element_present($msg)) {
                                                    $msg = $self->_get_text($msg);
                                                    push(@{$chunk_hash{$smode_ip}},$msg);
                                                    $var++;
                                                }else{
                                                    last;
                                                }
                                            }
                                        }
                                    }
                                    $self->_click(
                                            id                      => $CFT_REVERT_STEPS_CLOSE,
                                            error_checking_needed   => 0,
                                        ) ;
                                }
                                push @{$href->{"Step"}} , {%chunk_hash};
                            }
                            $row++;
                        }else{
                            foreach my $col (1..@keys){
                                my $job_results = $JOB_RESULTS;
                                $job_results =~ s/(.*)ROW_NUMBER(.*)COLUMN_NUMBER(.*)/$1$row$2$col$3/;
                                if ($col == 5){
                                    if($self->_is_element_present($job_results)){
                                        $chunk_hash{"affected_objects"} = $self->_get_text($job_results);
                                    }
                                } else {
                                    $chunk_hash{$keys[$col-1]} = $self->_get_text($job_results);
                                }
                            }

                            push @{$href->{$type}} , {%chunk_hash};

                            $row++;
                        }
                    }
                    push @$aref , $href;

                }

                eval{
                    $self->_click(
                        id                      => $CLOSE_BUTTON,
                        error_checking_needed   => 0,
                    ) ;

                    unless ($summary_text =~ /finished successfully/){
                        $self->_create_screenshot($summary_text);
                        $self->_repair_session();
                        $self->_throw_exception("Operation completed with error : ".$summary_text);
                    }

                };

            } else {
                # Closing the pop-up if none of the above conditions meet
                # We dont want to fail in case if the pop-up can't be closed
                eval {
                    $self->_click(
                        id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                        error_checking_needed   => 0,
                    ) ;
                };
            }
        }

    }

    $self->set_implicit_wait_timeout(1000);
    if($self->_is_element_present($GENERIC_SVM_AUDIT_PATH)){
        eval {
            $self->_click_button_having_specific_text("Skip");
        };
    }

    $self->set_implicit_wait_timeout(5000);

    $Log->exit() if $may_exit;

    return $aref;
} ## end sub _check_for_error

sub _precutover_cutover_checks {
    my ($self,%args) = @_;

    if($self->_is_element_present($GENERIC_PRECUTOVER)){
        # Add precutover arguments here
        eval {
             $self->_click(
                id                      => $ADD_INFO_DURING_PRECUTOVER,
                error_checking_needed   => 0,
            ) ;
        };

        if ( defined $args{"arguments"}{"available-transfers-percentage"} ) {
            # Adding available-transfers-percentage
            $self->_click(
                id                      => $GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO) ); 
            $self->_type(
                $GENERIC_AUTO_CONTROL_STREAM_SELECTOR ,
                $args{"arguments"}{"available-transfers-percentage"}
            );
        }

        if ( defined $args{"arguments"}{"max-cap-concurrent-transfers"} ) {
            # Adding stream count
            $self->_click(
                id                      => $GENERIC_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_STREAM_SELECTOR_RADIO) ); 
            $self->_type(
                $GENERIC_STREAM_SELECTOR ,
                $args{"arguments"}{"max-cap-concurrent-transfers"}
            );
        }

        # Adding throttle
        if ( defined $args{"arguments"}{"snapmirror-throttle"} ){
            $self->_click(
                id                      => $GENERIC_THROTTLE_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) ;
            $self->_type(
                $GENERIC_THROTTLE_SELECTOR ,
                $args{"arguments"}{"snapmirror-throttle"}
            );
        }
        $self->_click_button_having_specific_text("Continue");

    }

    if($self->_is_element_present($GENERIC_CUTOVER) || $self->_is_element_present($GENERIC_UPDATE)){

        # Add precutover arguments here
        eval {
             $self->_click(
                id                      => $ADD_INFO_DURING_PRECUTOVER,
                error_checking_needed   => 0,
            ) ;
        };

        if ( defined $args{"arguments"}{"available-transfers-percentage"} ) {
            # Adding available-transfers-percentage
            $self->_click(
                id                      => $GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO) ); 
            $self->_type(
                $GENERIC_AUTO_CONTROL_STREAM_SELECTOR ,
                $args{"arguments"}{"available-transfers-percentage"}
            );
        }

        if ( defined $args{"arguments"}{"max-cap-concurrent-transfers"} ) {
            # Adding stream count
            $self->_click(
                id                      => $GENERIC_STREAM_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) if(! $self->_is_checked($GENERIC_STREAM_SELECTOR_RADIO) ); 
            $self->_type( $GENERIC_STREAM_SELECTOR ,
                $args{"arguments"}{"max-cap-concurrent-transfers"}
            );
        }

        # Adding throttle
        if ( defined $args{"arguments"}{"session-snapmirror-throttle"} ){
            $self->_click(
                id                      => $GENERIC_THROTTLE_SELECTOR_RADIO,
                error_checking_needed   => 0,
            ) ;
            $self->_type( $GENERIC_THROTTLE_SELECTOR ,
                $args{"arguments"}{"session-snapmirror-throttle"}
            );
        }

        $self->_click_button_having_specific_text("Continue");

    }
    return 1;
}
###############################################################################
## @name ::_get_all_window_names
##
## @summary
##     This method is used to get the names of all windows which are opened
## @synopsis
##     _get_all_window_names(%args);

sub _get_all_window_names {

    my ( $self ) = @_;

    $Log->enter() if $may_enter;

    my $handles = $self->get_window_handles;

    $Log->exit() if $may_exit;

    return @$handles;
} ## end sub _get_all_window_names

###############################################################################
## @name ::get_all_window_names
##
## @summary
##     This method is used to get the names of all windows which are opened
## @synopsis
##     get_all_window_names(%args);

sub get_all_window_names {

    my ( $self ) = @_;

    $Log->enter() if $may_enter;

    my @w_h = $self->_get_all_window_names();
    my $output->{parsed_output}->{response} = \@w_h ;

    $Log->exit() if $may_exit;

    return $output;
} ## end sub get_all_window_names

###############################################################################
## @name ::_select_window
##
## @summary
##     This method is used to switch to the window specified
## @synopsis
##     _select_window(%args);

sub _select_window {

    my ( $self, $selWin ) = @_;

    $Log->enter() if $may_enter;

    $self->switch_to_window($selWin);

    $Log->exit() if $may_exit;

    return 1;
} ## end sub _select_window

###############################################################################
## @name ::_is_checked
##
## @summary
##     This method returns true if the element is selected/checked
## @synopsis
##     _is_checked(%args);

sub _is_checked {

    my ( $self, $debugID ) = @_;

    $Log->enter() if $may_enter;

    my $value;
    if ( ref($self) =~ /WebElement/ ) {
        $value = $self->is_selected();
        $Log->exit() if $may_exit;
        return $value;
    }

    $value = $self->_find_element($debugID)->is_selected();

    $Log->exit() if $may_exit;

    return $value;
} ## end sub _is_checked

###############################################################################
## @name ::is_checked
##
## @summary
##     This method returns true if the element is selected/checked
## @synopsis
##     is_checked(%args);

sub is_checked {

    my ( $self, $debugID ) = @_;

    $Log->enter() if $may_enter;

    my $output->{parsed_output}->{response} = $self->_is_checked($debugID);

    $Log->exit() if $may_exit;

    return $output;

} ## end sub is_checked

###############################################################################
## @name ::_get_value
##
## @summary
##     This method returns the value of the element specified
## @synopsis
##     _get_value(%args);

sub _get_value {

    my ( $self, $debugID ) = @_;

    $Log->enter() if $may_enter;

    my $output_value;
    if ( ref($self) =~ /WebElement/ ) {
        $output_value = $self->get_value();
        $Log->exit() if $may_exit;
        return $output_value;
    }

    my $elem = $self->_find_element($debugID);
    $output_value = $elem->get_value();

    $Log->exit() if $may_exit;

    return $output_value;
} ## end sub _get_value

###############################################################################
## @name ::get_value
##
## @summary
##     This method returns the value of the element specified
## @synopsis
##     get_value(%args);

sub get_value {

    my ( $self, $debugID ) = @_;

    $Log->enter() if $may_enter;

    my $output->{parsed_output}->{response} = $self->_get_value($debugID);

    $Log->exit() if $may_exit;

    return $output;
} ## end sub get_value

###############################################################################
## @name ::_get_text
##
## @summary
##     This method returns the text of the element specified
## @synopsis
##     _get_text(%args);

sub _get_text {
# _find_element is not eval'ed so it can raise exception without capturing snapshot
    my ( $self, $debugID ) = @_;

    $Log->enter() if $may_enter;

    my $output_text;
    if ( ref($self) =~ /WebElement/ ) {
        $output_text = $self->get_text();
        $Log->exit() if $may_exit;
        return $output_text;
    }

    my $elem = $self->_find_element($debugID);
    $output_text = $elem->get_text();

    $Log->exit() if $may_exit;

    return $output_text;
} ## end sub _get_text

###############################################################################
## @name ::get_text
##
## @summary
##     This method returns the text of the element specified
## @synopsis
##     get_text(%args);

sub get_text {
# _find_element is not eval'ed so it can raise exception without capturing snapshot
    my ( $self, $debugID ) = @_;

    $Log->enter() if $may_enter;

    my $output->{parsed_output}->{response} = $self->_get_text($debugID);
    $Log->exit() if $may_exit;

    return $output;

} ## end sub get_text

###############################################################################
## @name ::_clear_window
##
## @summary
##     This method is used to close the pop-ups
## @synopsis
##     _clear_window(%args);

sub _clear_window {

    my ( $self ) = @_;

    $Log->enter() if $may_enter;

    if($self->_is_element_present($GENERIC_POPUP)){
        $self->_click(
            id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
            error_checking_needed   => 0,
        ) ;
    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub _clear_window

###############################################################################
## @name ::_click_button_having_specific_text
##
## @summary
##     This method is used to click the button having the specified text
## @synopsis
##     _click_button_having_specific_text(%args);

sub _click_button_having_specific_text{

    my ( $self, $text ) = @_;

    $Log->enter() if $may_enter;

    my @elems=$self->find_elements("//*[\@type='button']");
    foreach (@elems){
        # $_ is a webelement object so cant use private methods
        my $txt = $_->get_text();
        if(($_->get_text()) =~ /$text/i){
            $_->click();
            last;
        }
    }
    sleep(8);
    $Log->exit() if $may_exit;

    return 1;
} ## end sub _click_button_having_specific_text

###############################################################################
## @name ::_click_button_having_specific_text_no_error_check
##
## @summary
##     This method is used to click the button having the specified text
## @synopsis
##     _click_button_having_specific_text_no_error_check(%args);

sub _click_button_having_specific_text_no_error_check{

    my ( $self, $text ) = @_;

    $Log->enter() if $may_enter;

    my @elems=$self->find_elements("//*[\@type='button']");
    foreach (@elems){
        # $_ is a webelement object so cant use private methods
        if(($_->get_text()) =~ /$text/i){
            $_->click();
            last;
        }
    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub _click_button_having_specific_text_no_error_check


###############################################################################
## @name ::_smtt_navigate
##
## @summary
##     This method is used to navigate in the 7MTT tool based on navigation steps
## @synopsis
##     _smtt_navigate(%args);

sub _smtt_navigate{

    my ( $self, $navigation_steps ) = @_;

    $Log->enter() if $may_enter;

    while ( $navigation_steps =~ /([\w\s]+):?/g ){
        if ( $1 eq "Edit Configuration" ){
            $self->_click(
                id                      => $EDIT_CONFIGURATION,
                error_checking_needed   => 0,
            ) ;
        }elsif ( $1 eq "Save Project and Continue" ){
            $self->_click(
                id                      => $SAVE_PROJECT_AND_CONTINUE,
                error_checking_needed   => 1,
            ) ;
        }elsif ( $1 eq "Next" ){
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;
        }elsif ( $1 eq "Skip" ){
            $self->_click(
                id                      => $SKIP,
                error_checking_needed   => 1,
            ) ;
        }elsif ( $1 eq "StorageSyatems" ){
            $self->_click(
                id                      => $STORAGE_SYSTEMS,
                error_checking_needed   => 0,
            ) ;
        }elsif ( $1 eq "Get Started" ){
            $self->_click(
                id                      => $GET_STARTED,
                error_checking_needed   => 0,
            ) ;
        }else{
            $self->_click_button_having_specific_text($1);
        }

    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub _smtt_navigate

###############################################################################
## @name ::_close_tab
##
## @summary
##     This method is used to close the unwanted tabs
## @synopsis
##     _close_tab(%args);

sub _close_tab{

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;

    if (defined $args{project}){

        my $tab=$GENERIC_PROJECT_TAB;
        my $close=$GENERIC_PROJECT_TAB_CLOSE;
        my $project = $args{project};#."_project";
        $tab =~ s/(.*)XXYYZZ(.*)/$1$project$2/;
        $close =~ s/(.*)XXYYZZ(.*)/$1$project$2/;
        if($self->_is_element_present($tab)){
            $self->_click(
                id                      => $close,
                error_checking_needed   => 1,
            ) ;
        }

    }elsif (defined $args{string}){

        my ( $tab, $close );
        if($args{string} =~ /^Dashboard$/){
            $tab=$GENERIC_SESSION_TAB;
            $close=$GENERIC_SESSION_TAB_CLOSE;
        }else{
            $tab=$GENERIC_TAB;
            $close=$GENERIC_TAB_CLOSE;
        }
        my $tab_name = $args{string};
        $tab =~ s/(.*)XXYYZZ(.*)/$1$tab_name$2/;
        $close =~ s/(.*)XXYYZZ(.*)/$1$tab_name$2/;
        if($self->_is_element_present($tab)){
            $self->_click(
                id                      => $close,
                error_checking_needed   => 1,
            ) ;
        }

    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub _close_tab

###############################################################################
## @name ::_throw_exception
##
## @summary
##     This method is used to throw exception after capturing screenshot
## @synopsis
##     _throw_exception(%args);

sub _throw_exception{

    my ( $self, $exception_text ) = @_;

    $Log->enter() if $may_enter;

    $Log->exit() if $may_exit;

    NATE::BaseException->throw($exception_text);

} ## end sub _throw_exception

###############################################################################
## @name ::_take_screenshot
##
## @summary
##     This method is used to capture screenshot
## @synopsis
##     _take_screenshot(%args);

sub _take_screenshot{

    my ( $self ) = @_;

    $Log->enter() if $may_enter;

    $Log->comment("Taking a base-64 encoded screenshot.. Use freeformatter to decode");
    my $img = $self->screenshot();
    $Log->comment($img);

    $Log->exit() if $may_exit;

} ## end sub _take_screenshot

###############################################################################
## @name ::_repair_session
##
## @summary
##     This method is used to repair session object
## @synopsis
##     _repair_session(%args);

sub _repair_session{

    my ( $self ) = @_;

    $Log->enter() if $may_enter;

    $self->refresh();
    my $sleep_count = $GENERIC_WAIT / 1000;
    Tharn::snooze($sleep_count);
    $self->_login();

    $Log->exit() if $may_exit;

    return 1;
} ## end sub _repair_session

###############################################################################
## @name ::_get_host_from_ip
##
## @summary
##     This method returns hostname from the given IP address
## @synopsis
##     _get_host_from_ip(%args);

sub _get_host_from_ip{

    my ( $self, $ip, $mode ) = @_;

    $Log->enter() if $may_enter;

    # Navigating to storage tab
     $self->_click(
        id                      => $STORAGE_SYSTEMS,
        error_checking_needed   => 0,
    ) ;

    my $selector;
    if ($mode =~ /c/){
        $selector=$GENERIC_CMODE_HOSTNAME;
    }elsif ($mode =~ /s/){
        $selector=$GENERIC_SMODE_HOSTNAME;
    }
    $selector =~ s/(.*)XXYYZZ(.*)/$1$ip$2/;

    my $host;
    if( $self->_is_element_present($selector) ){
        #my $elem = $self->_find_element($selector);
        $host = $self->_get_text($selector);
        $self->_close_tab(
            string => "Storage",
        );
    }else{
        $self->_create_screenshot("$ip not_found");
        $self->_repair_session();
        $self->_throw_exception("Did not get the hostname from the IP in the storage "
            ."systems.. Prabably the credentials are not added .. "
            ."Unable to find the element with ".$selector);
    }

    $Log->exit() if $may_exit;

    return $host;
} ## end sub _get_host_from_ip

###############################################################################
## @name ::_session_exist
##
## @summary
##     This method returns true if session exists
## @synopsis
##     _session_exist(%args);

sub _session_exist{

    my ( $self, $session_name ) = @_;

    $Log->enter() if $may_enter;

    my %projects_sub_projects_details = $self->_get_all_sessions();

    my ($key, $value );

    while ( ($key, $value) = each %projects_sub_projects_details ) {
        foreach ( @{$value} ) {
            if ($_ eq $session_name){
                $Log->exit() if $may_exit;
                return "gwt-debug-projecttreepanel__gwt-debug-projecttreepanel_".$session_name.".$key";
            }
        }
    }

    $Log->exit() if $may_exit;

    return 0;
} ## end sub _session_exist

###############################################################################
## @name ::session_exist
##
## @summary
##     This method returns true if session exists
## @synopsis
##     session_exist(%args);

sub session_exist{

    my ( $self, $session_name ) = @_;

    $Log->enter() if $may_enter;

    my $output->{parsed_output}->{response} = $self->_session_exist($session_name);

    $Log->exit() if $may_exit;

    return $output;
} ## end sub session_exist

###############################################################################
## @name ::_get_project_id
##
## @summary
##     This method returns the project ID from the session ID
## @synopsis
##     _get_project_id(%args);

sub _get_project_id{

    my ( $self, $session_id ) = @_;

    $Log->enter() if $may_enter;

    $session_id =~ s/(gwt-debug-projecttreepanel__gwt-debug-projecttreepanel_)(.*)\.(.*)/$1$3/gi;

    return $session_id;

    $Log->exit() if $may_exit;

    return 0;
} ## end sub _get_project_id

###############################################################################
## @name ::_get_all_sessions
##
## @summary
##     This method returns all the session names
## @synopsis
##     _get_all_sessions(%args);

sub _get_all_sessions{

    my ( $self ) = @_;

    $Log->enter() if $may_enter;

    my %projects_sub_projects_details;

    my @sess_objs = $self->_find_elements( $PROJECT_SELECTOR ) ;
    foreach (@sess_objs){
        my $id = $_->get_attribute('id');
        if ($id =~ /\./){
            # It's a sub-project
            if ($id =~ /gwt-debug-projecttreepanel__gwt-debug-projecttreepanel_(.*)\.(.*)/){
                push @{$projects_sub_projects_details{$2}},$1;
            }
        }
    }

    $Log->exit() if $may_exit;

    return %projects_sub_projects_details;
} ## end sub _get_all_sessions

###############################################################################
## @name ::get_all_sessions
##
## @summary
##     This method returns all the session names
## @synopsis
##     get_all_sessions(%args);

sub get_all_sessions{

    my ( $self ) = @_;

    $Log->enter() if $may_enter;

    my %hash = $self->_get_all_sessions();
    my $output->{parsed_output}->{response} = \%hash ;

    $Log->exit() if $may_exit;

    return $output;
} ## end sub get_all_sessions

###############################################################################
## @name ::_add_volumepairs_for_session
##
## @summary
##     This method is used to add volumepairs to the session
## @synopsis
##     _add_volumepairs_for_session(%args);

sub _add_volumepairs_for_session{

    my ( $self, $session_name, $skip_navigation,$smode,%Smode_credentials) = @_;
    my $node;
    $Log->enter() if $may_enter;
    if( ! $skip_navigation){
        # Navigating to the correct vfiler to add volumes
        my $text;
        my $vfiler = $self->_get_value_for_key_x_key_y_in_session_z(
            "transition_create_args","vfiler",$session_name
        );
        $node = $self->_get_value_for_key_x_key_y_in_session_z(
            "transition_volumepair_add_args","s_hostname",$session_name
        );

        if ($vfiler eq "vfiler0"){
            $text = $node."(".$smode.")";
        }else{
            $text = $vfiler.".".$node;
        }
        my $selector=$GENERIC_VFILER_SELECTOR;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$text$2/;

        if( $self->_is_element_present($selector) ){
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 1,
            ) ;
        }else{
            $self->_create_screenshot("Volumepair_add_error");
            $self->_repair_session();
            $self->_throw_exception("Probably the vfiler requested for does not "
                ."exist.. Unable to find the element with ".$selector);
        }
    }

    # Adding volumes now
    my $session_type = $self->_get_value_for_key_x_key_y_in_session_z(
        "transition_create_args","session-type",$session_name
    );
    my $vol_ar = $self->_get_value_for_key_x_key_y_in_session_z(
        "transition_volumepair_add_args","volume-name-7-mode",$session_name
    );
    if($session_type eq "standalone"){
        $self->_add_volumes_given_vol_ref_array($vol_ar);
    }elsif ($session_type eq "secondary"){
        $self->_add_volumes_given_vol_ref_array($vol_ar,"staggered",%Smode_credentials);
    }else{
        # Create arguments not present. Adding volumes standalone style
        $self->_add_volumes_given_vol_ref_array($vol_ar);
    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub _add_volumepairs_for_session

###############################################################################
## @name ::_add_vserver
##
## @summary
##     This method is used to add vserver to the session
## @synopsis
##     _add_vserver(%args);

sub _add_vserver{

    my ( $self, $session_name, $host_cmode ) = @_;

    $Log->enter() if $may_enter;

    my $selector = $VOLPAIR_CMODE_SYSTEM_SELECT;
    $selector =~ s/XXYYZZ/$host_cmode/;
    if( $self->_is_element_present($selector) ){
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 1,
        ) ;
    }
    
    my $vserver = $self->_get_value_for_key_x_key_y_in_session_z(
        "transition_create_args","svm-name",$session_name
    );
    $selector = $GENERIC_VSERVER_SELECTOR;
    $selector =~ s/(.*)XXYYZZ(.*)/$1$vserver$2/;

    if( $self->_is_element_present($selector) ) {
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 1,
        ) if(! $self->_is_checked($selector) );
    }else{
        $self->_create_screenshot("Vserver_add_error");
        $self->_repair_session();
        $self->_throw_exception("Probably the vserver requested for does not exist"
            .".. Unable to find the element with ".$selector);
    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub _add_vserver

###############################################################################
## @name ::_select_aggregate
##
## @summary
##     This method is used for aggregate selection
## @synopsis
##     _select_aggregate(%args);

sub _select_aggregate{

    my ( $self, $session_name ) = @_;

    $Log->enter() if $may_enter;

    # Sleeping here for sometime
    my $sleep_count = $GENERIC_WAIT / 1000;
    Tharn::snooze($sleep_count);
    Tharn::snooze($sleep_count);
    my $aggr_ar = $self->_get_value_for_key_x_key_y_in_session_z(
        "transition_volumepair_add_args","aggregate-name",$session_name
    );
    my $vol_7_ar = $self->_get_value_for_key_x_key_y_in_session_z(
        "transition_volumepair_add_args","volume-name-7-mode",$session_name
    );
    foreach (0..@{$aggr_ar}-1){

        my $aggregate = $aggr_ar->[$_];
        my $vol_7 = $vol_7_ar->[$_];

        my $aggr_sel = $GENERIC_AGGR;
        $aggr_sel =~ s/(.*)XXYYZZ(.*)/$1$vol_7$2/;
        my $elem = $self->_find_element( $aggr_sel ) ;
        $elem->click();
        # Sleeping to allow drop-down some time to appear
        Tharn::snooze($sleep_count);
        my $selector=$GENERIC_AGGR_SELECTOR;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$aggregate$2/;

        if( $self->_is_element_present($selector) ){
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 1,
            ) ;
        }else{
            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception("Probably the aggregate requested for does "
                ."not exist.. Unable to find the element with ".$selector);
        }

    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub _select_aggregate

###############################################################################
## @name ::_do_volume_mapping
##
## @summary
##     This method is used for volume mapping
## @synopsis
##     _do_volume_mapping(%args);

sub _do_volume_mapping{

    my ( $self, $session_name ) = @_;

    $Log->enter() if $may_enter;

    my $vol_7_ar = $self->_get_value_for_key_x_key_y_in_session_z(
        "transition_volumepair_add_args","volume-name-7-mode",$session_name);
    my $vol_c_ar = $self->_get_value_for_key_x_key_y_in_session_z(
        "transition_volumepair_add_args","volume-name-c-mode",$session_name);

    foreach (0..@{$vol_c_ar}-1){

        my $vol_7 = $vol_7_ar->[$_];
        my $vol_c = $vol_c_ar->[$_];

        # Changing C-Mode volumes names to requested values
        my $cmode_vol_sel = $GENERIC_CMODE_VOLUME_SELECTOR;
        $cmode_vol_sel =~ s/(.*)XXYYZZ(.*)/$1$vol_7$2/;
        $self->_type( $cmode_vol_sel ,$vol_c);

    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub _do_volume_mapping

###############################################################################
## @name ::_change_session_name_to_given_value
##
## @summary
##     This method is used for changing session names
## @synopsis
##     _change_session_name_to_given_value(%args);

sub _change_session_name_to_given_value{

    my ( $self, $session_name , $old_name ) = @_;

    $Log->enter() if $may_enter;
    my $name_modify_list = $PROJECT_MODIFY;
    $name_modify_list =~ s/XXYYZZ/$old_name/;
    if($self->_is_element_present($name_modify_list)){
        $self->_click(
            id                      => $name_modify_list,
            error_checking_needed   => 0,
        ) ;
        $self->_click(
            id                      => $PROJECT_RENAME,
            error_checking_needed   => 0,
        ) ;
        $self->_type( $PROJECT_NAME_INPUT, $session_name ) if( $session_name );
        $self->_click(
            id                      => $PROJECT_SAVE,
            error_checking_needed   => 0,
        ) ;
        my $sleep_count = $GENERIC_WAIT / 1000;
        Tharn::snooze($sleep_count);
        $self->_click(
            id                      => $EDIT_OK_BUTTON,
            error_checking_needed   => 1,
        ) ;
    }
    $Log->exit() if $may_exit;
    return 1;
} ## end sub _change_session_name_to_given_value

###############################################################################
## @name ::_add_volumes_given_vol_ref_array
##
## @summary
##     This is a helper method for adding volumepairs
## @synopsis
##     _add_volumes_given_vol_ref_array(%args);

sub _add_volumes_given_vol_ref_array{

    my ( $self, $vol_ar, $ses_type,%args) = @_;
    $Log->enter() if $may_enter;
    # Add support for qtree transition also
    foreach (0..@{$vol_ar}-1){
        my $vol = $vol_ar->[$_];
        my $checkbox=$GENERIC_VOLUME_STANDALONE_SELECTOR;
        $checkbox =~ s/(.*)XXYYZZ(.*)/$1$vol$2/;
        if($self->_is_element_enabled($checkbox) ){
            if( $self->_is_checked($checkbox) ){
                $self->_create_screenshot($vol."_Added_error");
                $self->_repair_session();
                $self->_throw_exception("Volume $vol is already been added to the session");
            }
            if ($ses_type eq "staggered"){
                $checkbox=$GENERIC_VOLUME_STAGGERED_SELECTOR;
                $checkbox =~ s/(.*)XXYYZZ(.*)/$1$vol$2/;
            }
            $self->_click(
                id                      => $checkbox,
                error_checking_needed   => 0,
            ) ;

            if($self->_is_element_present($VFILERHOSTNAME)) {
                $self->_add_credentials(%args);
            }
            eval{
                my $popup_txt=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_MESSAGE);
                $self->_click(
                    id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                    error_checking_needed   => 0,
                ) ;
            };
        }else{
            $self->_create_screenshot("Vol_Not_Found");
            $self->_repair_session();
            $self->_throw_exception("Probably the volume requested for does not exist"
                ." or is offline/involved in some other subproject.. "
                ."Unable to find the element with ".$checkbox);
        }
    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub _add_volumes_given_vol_ref_array

###############################################################################
## @name ::_is_button_enabled
##
## @summary
##     This method returns true if button is enabled
## @synopsis
##     _is_button_enabled(%args);

sub _is_button_enabled {

    my ( $self, $debugID ) = @_;

    $Log->enter() if $may_enter;

    my $retStatus = 1;

    if (
        $self->_is_element_present(
            "//table[\@id='$debugID']/tbody/tr[2]/td[2]/em/button[contains(\@aria-disabled,'true')]")
      ) {
        $retStatus = 0;
    }

    ( $retStatus == 1 ) ? $Log->comment("Button: $debugID is Enabled")
        : $Log->comment("Button: $debugID is Disabled");

    $Log->exit() if $may_exit;

    return $retStatus;
} ## end sub _is_button_enabled

###############################################################################
## @name ::_is_button_enabled_cbt
##
## @summary
##     This method returns true if button is enabled
## @synopsis
##     _is_button_enabled_cbt(%args);

sub _is_button_enabled_cbt {

    my ( $self, $debugID ) = @_;

    $Log->enter() if $may_enter;

    my $retStatus = 1;

    if (
        $self->_is_element_present(
            "//table/following::div[\@id='$debugID' and contains(\@class,'x-item-disabled')]")
      ) {
        $retStatus = 0;
    } elsif(
        $self->_is_element_present(
            $debugID." and contains(\@disabled,'')]")
    ) {
        $retStatus = 0;
    }

    ( $retStatus == 1 ) ? $Log->comment("Button: $debugID is Enabled")
        : $Log->comment("Button: $debugID is Disabled");

    $Log->exit() if $may_exit;

    return $retStatus;
} ## end sub _is_button_enabled

###############################################################################
## @name ::is_button_enabled
##
## @summary
##     This method returns true if button is enabled
## @synopsis
##     is_button_enabled(%args);

sub is_button_enabled {

    my ( $self, $debugID ) = @_;

    $Log->enter() if $may_enter;

    my $output->{parsed_output}->{response} = $self->_is_button_enabled($debugID);

    $Log->exit() if $may_exit;

    return $output;
} ## end sub is_button_enabled

###############################################################################
## @name ::_wait_on_attribute
##
## @summary
##     This method is used to wait on a given attribute
## @synopsis
##     _wait_on_attribute(%args);

sub _wait_on_attribute {

    my ( $self , %args ) = @_;

    $Log->enter() if $may_enter;

    my $attribute_to_check = $args{attribute_to_check};
    my $method_timeout     = $args{"method-timeout"};

    $self->_click(
        id                      => $AUTO_REFRESH,
        error_checking_needed   => 0,
    ) if( $self->_is_checked($AUTO_REFRESH) ); ;

    my $bool=0;
    foreach (1..$method_timeout) {
        $self->_click(
            id                      => $REFRESH,
            error_checking_needed   => 0,
        ) ;
        if ( $self->_is_element_present( $attribute_to_check ) ) {
            $bool = 1;
            last;
        }
    }

    unless ( $bool ){
        #$self->_take_screenshot();
        $self->_create_screenshot("$attribute_to_check not_found");
        $self->_repair_session();
        $self->_throw_exception("Even after waiting for $method_timeout secs "
            .", still the desired attribute $attribute_to_check did not appear");
    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub wait_on_attribute

###############################################################################
## @name ::_add_schedule_days
##
## @summary
##     This method is used to add days to schedule
## @synopsis
##     _add_schedule_days(%args);

sub _add_schedule_days {

    my ( $self, $days_range ) = @_;

    if ($days_range ne "0-6" ){

        # Adding days-range
        $self->_click(
            id                      => $GENERIC_SCHEDULE_DAYS_SELECT_RADIO,
            error_checking_needed   => 0,
        ) ;

        my $days = {
            0 => "sunday",
            1 => "monday",
            2 => "tuesday",
            3 => "wednesday",
            4 => "thursday",
            5 => "friday",
            6 => "saturday",
        };
        my @arr;
        foreach (split(",",$days_range)){
            if($_ =~ /-/){
                my @temp = split("-",$_);
                if( $temp[0] > $temp[1] ){
                    push @arr,$temp[0];
                    while( $temp[0] ne $temp[1] ){
                        if (++$temp[0] eq 7){
                            $temp[0]=0;
                            push @arr,$temp[0];
                        }else{
                            push @arr,$temp[0];
                        }
                    }
                }else{
                    my $i=0;
                    push @arr,$temp[0];
                    while( $temp[0] ne $temp[1] ){
                        push @arr,++$temp[0];
                        $i++;
                    }
                }
            }else{
                push @arr,$_;
            }
        }

        my %unique = ();
        foreach my $item (@arr) {
            $unique{$item} ++ ;
        }
        my @myuniquearray = keys %unique;
        foreach (@myuniquearray){
            my $day = $days->{$_};
            my $selector =  $GENERIC_SCHEDULE_DAYS_SELECT;
            $selector =~ s/(.*)XXYYZZ(.*)/$1$day$2/;
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) ;
        }

    }

}

###############################################################################
## @name ::_add_credentials
##
## @summary
##     This method is used to add credentials
## @synopsis
##     _add_credentials(%args);

sub _add_credentials {

    my ( $self , %args ) = @_;

    $Log->enter() if $may_enter;

    $Log->comment("Adding credentials");
    if($self->_is_element_present($HOSTNAME)) {
        $self->_type( $HOSTNAME ,$args{"host-name"});
    } elsif($self->_is_element_present($VFILERHOSTNAME)){
        $self->_type( $VFILERHOSTNAME ,$args{"host-name"});
    }
    if($self->_is_element_present($HOST_USERNAME)) {
        $self->_type( $HOST_USERNAME ,$args{"username"});
    } elsif ($self->_is_element_present($LOGIN_USERNAME)){
        $self->_type( $LOGIN_USERNAME ,$args{"username"});
    }
    if($self->_is_element_present($HOST_PASSWORD)) {
        $self->_type( $HOST_PASSWORD ,$args{"password"});
    } elsif ($self->_is_element_present($LOGIN_PASSWORD)){
        $self->_type( $LOGIN_PASSWORD ,$args{"password"});
    }
    if($self->_is_element_present($HOST_ADD)) {
        $self->_click(
            id                      => $HOST_ADD,
            error_checking_needed   => 1,
        ) ;
    } elsif ($self->_is_element_present($VFILERHOST_SAVE)){
        $self->_click(
            id                      => $VFILERHOST_SAVE,
            error_checking_needed   => 1,
        ) ;
    }
    $Log->exit() if $may_exit;

    return 1;
} ## end sub _add_credentials


################################################################################
## @name ::_remove_credentials
## @summary
##     This method is used to remove storage system credentials
## @synopsis
##     _remove_credentials(%args);
################################################################################
sub _remove_credentials {
    my ( $self , %args) = @_;
    my $flag=0;
    $Log->enter() if $may_enter;
    $Log->comment("Removing credentials");
    my $hostname = $args{"host-name"};
    my $delete_cred = $DELETE_CRED;
    $delete_cred =~ s/XXYYZZ/$hostname/;
    my $var = 0;
    my ($sicon,$cicon);
    while($var < 20){
        $sicon = $DELETE_SICON;
        $sicon =~ s/XXYYZZ/$var/;
        my $delete_sbutton = $delete_cred.$sicon;
        if ($self->_is_element_present($delete_sbutton)) {
            $self->_click(
                id                      => $delete_sbutton,
                error_checking_needed   => 1,
            ) ;
            $flag=1;
            last;
        }
       $var++;
    }
    if(!$flag){
       $var=0;
       while($var < 20){
            $cicon = $DELETE_CICON;
            $cicon =~ s/XXYYZZ/$var/;
            my $delete_cbutton = $delete_cred.$cicon;
            if ($self->_is_element_present($delete_cbutton)) {
                $self->_click(
                     id                      => $delete_cbutton,
                     error_checking_needed   => 1,
                ) ;
                $flag = 1;
                last;
            }
           $var++;
       }
   }
   if(!$flag){
          $self->_create_screenshot($hostname."_credentials_not_found_to_remove");
          $self->_repair_session();
          $self->_throw_exception("$hostname is not present in storage systems");
   }
} #end of sub _remove_credentials


###############################################################################
## @name ::_is_key_x_present_for_session_y
##
## @summary
##     This method is internal helper method
## @synopsis
##     _is_key_x_present_for_session_y(%args);

sub _is_key_x_present_for_session_y{

    my ( $self , $x , $y ) = @_;

    $Log->enter() if $may_enter;

    foreach (@{$self->{$y}}){
        if ( defined $_->{$x}){
            $Log->exit() if $may_exit;
            return 1;
        }
    }

    $Log->exit() if $may_exit;

    return 0;
} ## end sub _is_key_x_present_for_session_y

###############################################################################
## @name ::_get_value_for_key_x_key_y_in_session_z
##
## @summary
##     This method is internal helper method
## @synopsis
##     _get_value_for_key_x_key_y_in_session_z(%args);

sub _get_value_for_key_x_key_y_in_session_z{

    my ( $self, $x , $y, $z) = @_;

    $Log->enter() if $may_enter;

    foreach (@{$self->{$z}}){
        if(defined $_->{$x} ){
            $Log->exit() if $may_exit;
            return $_->{$x}->{$y};
        }
    }

    $Log->exit() if $may_exit;

    return undef;

} ## end sub _get_value_for_key_x_key_y_in_session_z

###############################################################################
## @name ::_put_value_for_key_x_key_y_in_session_z
##
## @summary
##     This method is internal helper method
## @synopsis
##     _put_value_for_key_x_key_y_in_session_z(%args);

sub _put_value_for_key_x_key_y_in_session_z{

    my ( $self, $x , $y, $z, $val) = @_;

    $Log->enter() if $may_enter;

    foreach (@{$self->{$z}}){
        if(defined $_->{$x} ){
            $_->{$x}->{$y}=$val;
            last;
        }
    }

    $Log->exit() if $may_exit;

    return 1;

} ## end sub _put_value_for_key_x_key_y_in_session_z

###############################################################################
## @name ::_get_volume_sm_details
##
## @summary
##     This method returns volume snapmirror status for a given session
## @synopsis
##     _get_volume_sm_details(%args);

sub _get_volume_sm_details{

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;

    my $id = undef ;
    $self->_repair_session();
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output = {parsed_output => { "response" => undef } };

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        # Click on volume tab
        $self->_click(
            id                      => $VOLUME_TAB,
            error_checking_needed   => 1,
        ) ;

        # Populate this array if you want to fetch any new entries
        my @keys = ("Origin_Volume","Target_Volume","Current_Operation","Last_Operation",
            "Mirror_State","SM_Status","Data_Pending"
        );

        my $row = 1;
        while (1){
            eval {
                my $href = undef;
                foreach my $col (1..@keys){
                    my $volume_table_selector = $VOLUME_TABLE_SELECTOR;
                    $volume_table_selector =~ s/(.*)ROW_NUMBER(.*)COLUMN_NUMBER(.*)/$1$row$2$col$3/;
                    $href->{$keys[$col-1]} = $self->_get_text($volume_table_selector);
                }
                push (@$aref,$href);
            };
            if ($@) {
                $self->_close_tab(
                    string => "Dashboardtab",
                );
                $output->{parsed_output}->{response} = $aref ;
                $Log->exit() if $may_exit;
                return $output;
            }
            $row++;
        }

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $self->_close_tab(
        string => "Dashboardtab",
    );

    $output->{parsed_output}->{response} = $aref ;

    $Log->exit() if $may_exit;

    return $output;

} ## end sub _get_volume_sm_details

###############################################################################
## @name ::_get_data_copy_schedule_details
##
## @summary
##     This method returns data copy schedule_details for a given session
## @synopsis
##     _get_data_copy_schedule_details(%args);

sub _get_data_copy_schedule_details{

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;

    my $id = undef ;
    $self->_repair_session();
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($args{"session-name"});
    }

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output = {parsed_output => { "response" => undef } };

    if($id){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        # Click on schedule tab
        $self->_click(
            id                      => $SCHEDULE_TAB,
            error_checking_needed   => 1,
        ) ;

        # Populate this array if you want to fetch any new entries
        my @keys = ("Status","Name","Recurring_Days","Start_Time",
            "Duration","End_Time","Update_Frequency","Available_Transfers_Percentage",
            "Currently_Used","Throttle"
        );

        my $row = 1;
        while (1){
            eval {
                my $href = undef;
                foreach my $col (1..@keys){
                    my $schedule_selector = $SCHEDULE_SELECTOR;
                    $schedule_selector =~ s/(.*)ROW_NUMBER(.*)COLUMN_NUMBER(.*)/$1$row$2$col$3/;
                    $href->{$keys[$col-1]} = $self->_get_text($schedule_selector);
                }
                push (@$aref,$href);
            };
            if ($@) {
                $self->_close_tab(
                    string => "Dashboardtab",
                );
                $output->{parsed_output}->{response} = $aref ;
                $Log->exit() if $may_exit;
                return $output;
            }
            $row++;
        }

    }else{
        $self->_take_screenshot();
        $self->_repair_session();
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist");
    }

    $self->_close_tab(
        string => "Dashboardtab",
    );

    $output->{parsed_output}->{response} = $aref ;

    $Log->exit() if $may_exit;

    return $output;

} ## end sub _get_data_copy_schedule_details

###############################################################################
## @name ::_create_helper
##
## @summary
##     This method is a helper method for create subroutine
## @synopsis
##     _create_helper(%args);

sub _create_helper {

    my ($self, $args) = @_;
    my %args=%{$args};
    $Log->enter() if $may_enter;
    my $S_mode_credentials      = delete $args{S_mode_credentials};
    my $C_mode_credentials      = delete $args{C_mode_credentials};
    my $volumepair_add_args     = delete $args{volumepair_add_args};
    my $schedule_add_args       = delete $args{schedule_add_args};
    my $plan_configuration      = delete $args{plan_configuration};
    my $create_new_grp          = delete $args{create_new_grp};
    _navigate_plan_design($self, "CBT");
    my $id = undef ;
    my $host_cmode;
    push @{$self->{$args{"session-name"}}} , { transition_create_args => \%args} ;
   if ( $S_mode_credentials || $C_mode_credentials || $volumepair_add_args ) {

        # Adding credentials
        $self->_add_credentials( %{$S_mode_credentials} ) if ( $S_mode_credentials ) ;
        $self->_add_credentials( %{$C_mode_credentials} ) if ( $C_mode_credentials ) ;
             if ( $volumepair_add_args ) {

   	           # Storing volume pair add arguments in the object
		    foreach (@{$self->{$args{"session-name"}}}){
		        undef $_ if (defined $_->{transition_volumepair_add_args});
		    }
		    @{$self->{$args{"session-name"}}} = grep defined, @{$self->{$args{"session-name"}}};
		    push @{$self->{$args{"session-name"}}} , { transition_volumepair_add_args => $volumepair_add_args} ;
        
        if ( $S_mode_credentials || $C_mode_credentials ) {
            # Lets go to Home Tab -> Storage systems and fetch the hostnames
            my $selector=$GENERIC_TAB;
            $selector =~ s/(.*)XXYYZZ(.*)/$1Hometab$2/;
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) ;

            # Closing the Untitled project
            $selector=$GENERIC_TAB_CLOSE;
            $selector =~ s/(.*)XXYYZZ(.*)/$1\(CBT\)$2/;
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) ;

        }
           my $ips = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","7-mode-host-name",$args{"session-name"});
            my $host_smode = $self->_cbt_get_host_from_ip($ips,"s");
            my $host;
            if($host_smode =~ /(.*)\($ips\)/){
                $host = $1;
            }
            $host =~ s/(.*)\s+/$1/;
            $self->_put_value_for_key_x_key_y_in_session_z(
                "transition_volumepair_add_args","s_hostname",$args{"session-name"},$host);
            my $ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","c-mode-host-name",$args{"session-name"});
            $host_cmode = $self->_cbt_get_host_from_ip($ip,"c");
            if($host_cmode =~ /(.*)\($ip\)/){
                $host = $1;
            }
            $host =~ s/(.*)\s+/$1/;
            $host_cmode =~ s/\s+(.*)/$1/;
            $self->_put_value_for_key_x_key_y_in_session_z(
                "transition_volumepair_add_args","c_hostname",$args{"session-name"},$host);
            _navigate_plan_design($self, "CBT");
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;
            $self->_add_volumepairs_for_session($args{"session-name"},0,$ips,%{$S_mode_credentials});
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 0,
            ) ;
            $self->_create_screenshot($args{"session-name"}."_Flex_Snaplock_Volumes_add_Warning_Message");
            my $popup_title;
            eval{
                $popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
            };
            if($popup_title =~ /Warning/i) { 
                $self->_click(
                    id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                    error_checking_needed   => 0,
                ) ;
            }
            eval{
                $popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
                $self->_click(
                    id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                    error_checking_needed   => 0,
                ) ;
                if ($self->_is_element_present($CFT_YES_BUTTON)) {
                    $self->_click(
                        id                      => $CFT_YES_BUTTON,
                        error_checking_needed   => 0,
                    ) ;
                }
            };
            $self->_create_screenshot("Project_add");
            $Log->comment("Adding Project name");
            $self->_type( $PROJECT_NAME_INPUT ,$args{"session-name"});
            if ($args{"group-name"}) {
                   my $create_grp =$args{create_new_grp};
                   my $group = $args{"group-name"};
                   $self->_click(
                       id                      => $SELECT_GROUP,
                       error_checking_needed   => 0,
                   ) ;
                   my $existing_group = $EXISTING_GROUP;
                   $existing_group =~ s/(.*)XXYYZZ(.*)/$1$group$2/;
                   if ($self->_is_element_present($existing_group) && !($create_new_grp)) {
                       $self->_click(
                           id                      => $existing_group,
                           error_checking_needed   => 0,
                       ) ;
                   }else{
                       $self->_click(
                           id                      => $CREATE_NEW_GROUP,
                           error_checking_needed   => 0,
                       ) ;
                       $self->_type( $GROUP_NAME ,$group);
                       $self->_create_screenshot("New_group_add");
                   }
               }else{
                   my $group = "Default";
                   $self->_click(
                       id                      => $SELECT_GROUP,
                       error_checking_needed   => 0,
                   ) ;
                   my $existing_group = $EXISTING_GROUP;
                   $existing_group =~ s/(.*)XXYYZZ(.*)/$1$group$2/;
                   if ($self->_is_element_present($existing_group)) {
                       $self->_click(
                           id                      => $existing_group,
                           error_checking_needed   => 0,
                       ) ;
                   }
               }
               $self->_create_screenshot($args{"session-name"}."_Project_created");
               $self->_click(
                   id                      => $SAVE_PROJECT,
                   error_checking_needed   => 0,
               );
               if($self->_is_element_present($GROUPNAME_VALID)){
				$self->_repair_session();
                $self->_throw_exception(
                    "The maximum number of characters allowed in a project group name is 40");
			}
			if($self->_is_element_present($GROUPNAME_VALID_SPLCHAR)){
				$self->_repair_session();
                $self->_throw_exception(
                    "special characters are not allowed");
			}
            if($self->_is_element_present($GROUP_EXIST)){
				$self->_repair_session();
                $self->_throw_exception(
                    " A Project group with this group name already exist");
			}
            if($self->_is_element_present($PROJECT_EXIST)){
                $self->_repair_session();
                $self->_throw_exception(
                    " A project with same name already exists");
			}
            my $dc_ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","data-copy-ipaddress",$args{"session-name"});
            $self->_type( $DATA_COPY_IP, $dc_ip ) if( $dc_ip );
            my $m_ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "transition_create_args","multipathing-ipaddress",$args{"session-name"});
            $self->_type( $MULTIPATH_IP, $m_ip ) if( $m_ip );
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;
            $self->_add_vserver($args{"session-name"},$host_cmode);
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;
        }
        if ( $volumepair_add_args && $schedule_add_args ) {

            foreach ( @{$schedule_add_args} ) {

                $self->_click(
                    id                      => $CREATE_SCHEDULE,
                    error_checking_needed   => 0,
                ) ;

                $self->_type( $GENERIC_SCHEDULE_NAME ,$_->{"schedule-name"});

                $self->_add_schedule_days($_->{"days-range"});

                # Adding start time
                my @s_t = split(":",$_->{"start-time"});
                my $selector=$GENERIC_LISTITEM_SELECTOR;
                $self->_click(
                    id                      => $GENERIC_SCHEDULE_START_TIME_HOUR_INPUT,
                    error_checking_needed   => 0,
                ) ;
                $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[0]$2/;
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                ) ;
                $self->_click(
                    id                      => $GENERIC_SCHEDULE_START_TIME_MIN_INPUT,
                    error_checking_needed   => 0,
                ) ;
                $selector=$GENERIC_LISTITEM_SELECTOR;
                $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[1]$2/;
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                ) ;

                # Adding duration
                my @duration = split(":",$_->{"duration"});
                $self->_type( $GENERIC_SCHEDULE_DURATION_HOUR_INPUT ,$duration[0] );
                $self->_click(
                    id                      => $GENERIC_SCHEDULE_DURATION_MIN_INPUT,
                    error_checking_needed   => 0,
                ) ;
                $selector=$GENERIC_LISTITEM_SELECTOR;
                $selector =~ s/(.*)XXYYZZ(.*)/$1$duration[1]$2/;
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                ) ;

                if ( defined $_->{"available-transfers-percentage"} ) {
                    # Adding available-transfers-percentage
                    $self->_click(
                        id                      => $GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO,
                        error_checking_needed   => 0,
                    ) if(! $self->_is_checked($GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO) );
                    $self->_type(
                        $GENERIC_AUTO_CONTROL_STREAM_SELECTOR ,$_->{"available-transfers-percentage"} );
                }

                if ( defined $_->{"max-cap-concurrent-transfers"} ) {
                    # Adding stream count
                    $self->_click(
                        id                      => $GENERIC_STREAM_SELECTOR_RADIO,
                        error_checking_needed   => 0,
                    ) if(! $self->_is_checked($GENERIC_STREAM_SELECTOR_RADIO) );
                    $self->_type( $GENERIC_STREAM_SELECTOR ,$_->{"max-cap-concurrent-transfers"} );
                }

                # Adding throttle
                if (defined $_->{"session-snapmirror-throttle"}){
                    $self->_click(
                        id                      => $GENERIC_THROTTLE_SELECTOR_RADIO,
                        error_checking_needed   => 1,
                    ) ;
                    $self->_type( $GENERIC_THROTTLE_SELECTOR ,$_->{"session-snapmirror-throttle"} );
                }

                # Adding update frequency
                if (defined $_->{"update-frequency"}){
                    my @u_f = split(":",$_->{"update-frequency"});
                    $self->_type( $GENERIC_SCHEDULE_UPDATE_FREQUENCY_HOUR_INPUT ,$u_f[0]);
                    $self->_click(
                        id                      => $GENERIC_SCHEDULE_UPDATE_FREQUENCY_MIN_INPUT,
                        error_checking_needed   => 0,
                    ) ;
                    $selector=$GENERIC_AGGR_SELECTOR;
                    $selector =~ s/(.*)XXYYZZ(.*)/$1$u_f[1]$2/;
                    $self->_click(
                        id                      => $selector,
                        error_checking_needed   => 0,
                    ) ;
                }

                $self->_click(
                    id                      => $GENERIC_SCHEDULE_CREATE_BUTTON,
                    error_checking_needed   => 1,
                ) ;

                if($self->_is_element_present($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE)){
                    my $text = $self->_get_text($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE);
                    $self->_take_screenshot();
                    $self->_repair_session();
                    $self->_throw_exception("Error while adding schedule : ".$text);
                }

            }

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 0,
            ) ;
        #Plan Configuration
        _plan_configuration($self, $plan_configuration);

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;
            my $proj_name = $args{"session-name"};
            $self->_create_screenshot($proj_name."_created");
            $self->_close_tab(
                string => "Migration",
            );

        } else {

            # Since schedule add args not present , lets close the tab
            $self->_close_tab(
                session => $args{"session-name"},
            );

        }
    }

    $Log->exit() if $may_exit;
} ## end sub _create_helper

###############################################################################
## @name ::_create_primary
##
## @summary
##     This method is a helper method for create primary subroutine
## @synopsis
##     _create_helper(%args);

sub _create_primary {

    my ( $self, $args, $prim_sess, $sec_sess, $sec_req , $group ) = @_;
    my %args=%{$args};

    $Log->enter() if $may_enter;
    my $volumepair_add_args     = delete $args{volumepair_add_args};
    my $schedule_add_args       = delete $args{schedule_add_args};
    my $c_hostname              = $args{volumepair_add_args}{c_hostname};
    my $c_ip                    = $args{"c-mode-host-name"};
    my $plan_configuration      = delete $args{plan_configuration};
    my $host_cmode              = $c_hostname."(".$c_ip.")";
    my $id = undef ;
    my $old_session_name;
    if( $self->_is_button_enabled($MIGRATION_DASHBOARD_LINK) ) {
        $self->_click(
            id                      => $MIGRATION_DASHBOARD_LINK,
            error_checking_needed   => 1,
        ) ;
        $self->_project_selector($group);
        my $locator = "gwt-debug-".$args{"session-name"};
        if ($self->_is_element_present($locator)) {
            $id = $locator;
        }
        $old_session_name = $args{"session-name"};
        $args{"session-name"} = $prim_sess;
        push @{$self->{$args{"session-name"}}} , { transition_create_args => \%args} ;
    }

    if($id){
        # Session exists
        # Changing session name to desired value
        $self->_change_session_name_to_given_value($args{"session-name"},$old_session_name);
        $self->_change_session_name_to_given_value($sec_req,$sec_sess);
        $self->_create_screenshot("Projects_Renamed");
        my $project_edit_config = $PROJECT_EDIT_CONFIG;
        $project_edit_config =~ s/XXYYZZ/$args{"session-name"}/;
        if($self->_is_element_present($project_edit_config)){
            $self->_click(
                id                      => $project_edit_config,
                error_checking_needed   => 0,
            ) ;
            $self->_click(
                id                      => $EDIT_DATA_IP,
                error_checking_needed   => 1,
            ) ;
        }
        my $dc_ip = $self->_get_value_for_key_x_key_y_in_session_z(
            "transition_create_args","data-copy-ipaddress",$args{"session-name"});
        $self->_type( $DATA_COPY_IP, $dc_ip ) if( $dc_ip );

        my $m_ip = $self->_get_value_for_key_x_key_y_in_session_z(
            "transition_create_args","multipathing-ipaddress",$args{"session-name"});
        $self->_type( $MULTIPATH_IP, $m_ip ) if( $m_ip );

        $self->_click(
            id                      => $NEXT,
            error_checking_needed   => 0,
        ) ;

        if ( $args{"seven-mode-only"} ) {
            $self->_click(
                id                      => $SMODE_PRECHECK,
                error_checking_needed   => 1,
            ) ;
            $self->_click(
                id                      => $SMODE_PRECHECK_NEXT_BUTTON,
                error_checking_needed   => 0,
            ) ;
        }

        # Vserver selection page
        
        $self->_add_vserver($args{"session-name"},$host_cmode);
        $self->_click(
            id                      => $NEXT,
            error_checking_needed   => 1,
        ) ;

        # Aggr selection page
        $self->_select_aggregate($args{"session-name"})
            if defined ($volumepair_add_args->{"aggregate-name"});

        # Volume mapping
        $self->_do_volume_mapping($args{"session-name"})
            if defined ($volumepair_add_args->{"volume-name-c-mode"});

        $self->_click(
            id                      => $NEXT,
            error_checking_needed   => 1,
        ) ;

        if ( $volumepair_add_args && $schedule_add_args ) {

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            foreach ( @{$schedule_add_args} ) {

                $self->_click(
                    id                      => $CREATE_SCHEDULE,
                    error_checking_needed   => 0,
                ) ;

                $self->_type( $GENERIC_SCHEDULE_NAME ,$_->{"schedule-name"});

                $self->_add_schedule_days($_->{"days-range"});

                # Adding start time
                my @s_t = split(":",$_->{"start-time"});
                my $selector=$GENERIC_AGGR_SELECTOR;
                $self->_click(
                    id                      => $GENERIC_SCHEDULE_START_TIME_HOUR_INPUT,
                    error_checking_needed   => 0,
                ) ;
                $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[0]$2/;
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                ) ;
                $self->_click(
                    id                      => $GENERIC_SCHEDULE_START_TIME_MIN_INPUT,
                    error_checking_needed   => 0,
                ) ;
                $selector=$GENERIC_AGGR_SELECTOR;
                $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[1]$2/;
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                ) ;

                # Adding duration
                my @duration = split(":",$_->{"duration"});
                $self->_type( $GENERIC_SCHEDULE_DURATION_HOUR_INPUT ,$duration[0] );
                $self->_click(
                    id                      => $GENERIC_SCHEDULE_DURATION_MIN_INPUT,
                    error_checking_needed   => 0,
                ) ;
                $selector=$GENERIC_AGGR_SELECTOR;
                $selector =~ s/(.*)XXYYZZ(.*)/$1$duration[1]$2/;
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                ) ;

                if ( defined $_->{"available-transfers-percentage"} ) {
                    # Adding available-transfers-percentage
                    $self->_click(
                        id                      => $GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO,
                        error_checking_needed   => 0,
                    ) if(! $self->_is_checked($GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO) );
                    $self->_type(
                        $GENERIC_AUTO_CONTROL_STREAM_SELECTOR ,$_->{"available-transfers-percentage"} );
                }

                if ( defined $_->{"max-cap-concurrent-transfers"} ) {
                    # Adding stream count
                    $self->_click(
                        id                      => $GENERIC_STREAM_SELECTOR_RADIO,
                        error_checking_needed   => 0,
                    ) if(! $self->_is_checked($GENERIC_STREAM_SELECTOR_RADIO) );
                    $self->_type( $GENERIC_STREAM_SELECTOR ,$_->{"max-cap-concurrent-transfers"} );
                }

                # Adding throttle
                if (defined $_->{"session-snapmirror-throttle"}){
                    $self->_click(
                        id                      => $GENERIC_THROTTLE_SELECTOR_RADIO,
                        error_checking_needed   => 1,
                    ) ;
                    $self->_type( $GENERIC_THROTTLE_SELECTOR ,$_->{"session-snapmirror-throttle"} );
                }

                # Adding update frequency
                if (defined $_->{"update-frequency"}){
                    my @u_f = split(":",$_->{"update-frequency"});
                    $self->_type( $GENERIC_SCHEDULE_UPDATE_FREQUENCY_HOUR_INPUT ,$u_f[0]);
                    $self->_click(
                        id                      => $GENERIC_SCHEDULE_UPDATE_FREQUENCY_MIN_INPUT,
                        error_checking_needed   => 0,
                    ) ;
                    $selector=$GENERIC_AGGR_SELECTOR;
                    $selector =~ s/(.*)XXYYZZ(.*)/$1$u_f[1]$2/;
                    $self->_click(
                        id                      => $selector,
                        error_checking_needed   => 0,
                    ) ;
                }

                $self->_click(
                    id                      => $GENERIC_SCHEDULE_CREATE_BUTTON,
                    error_checking_needed   => 1,
                ) ;

                if($self->_is_element_present($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE)){
                    my $text = $self->_get_text($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE);
                    $self->_take_screenshot();
                    $self->_repair_session();
                    $self->_throw_exception("Error while adding schedule : ".$text);
                }

            }

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 0,
            ) ;

            #Plan Configuration
            _plan_configuration($self, $plan_configuration);

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;
            $self->_create_screenshot($prim_sess."_created");
            $self->_close_tab(
                string => "Migration",
            );
        } else {

            # Since schedule add args not present , lets close the tab
            $self->_close_tab(
                string => "(CBT)",
            );

        }

    }else{
        $self->_throw_exception("Session ".$args{"session-name"}." does not exist ");
    }

    $Log->exit() if $may_exit;
} ## end sub _create_primary

###############################################################################
## @name ::_get_volume_selection_details
##
## @summary
##     This method returns volume selection details in an array ref from Volume selection page
## @synopsis
##     _get_volume_selection_details(%args);

sub _get_volume_selection_details{

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;

    my $sess_name = $args{"session-name"};
    my $ip = $args{"7-mode-host-name"};

    # Array of hash references which will be returned to the caller
    my $aref=[];
    my $output = {parsed_output => { "response" => undef } };

    if ( $sess_name ) {

        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        my $id = $self->_session_exist($sess_name);
        if ($id){
            $self->_click(
                id                      => $id,
                error_checking_needed   => 1,
            ) ;
            $self->_click(
                id                      => $EDIT_CONFIGURATION,
                error_checking_needed   => 1,
            ) ;
            $aref = $self->_get_volume_selection_details_helper($sess_name);

        }else {
            $self->_close_tab(
                string => "Dashboardtab",
            ) ;
            $self->_throw_exception("Session $sess_name does not exist ");
        }

    } else {
        my $host = $self->_get_host_from_ip($ip,"s");
        # Lets Get Started
        $self->_click(
            id                      => $GET_STARTED,
            error_checking_needed   => 0,
        ) ;

        $self->_click(
            id                      => $NEXT,
            error_checking_needed   => 1,
        ) ;

        # Create new project here
        $Log->debug("Creating new project");
        $self->_type( $PROJECT_NAME_INPUT ,$args{temp_session_name} );
        $self->_click(
            id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
            error_checking_needed   => 0,
        ) ;

        # Wait till volume pairs are loaded
        $self->_check_for_error();
        $aref = $self->_get_volume_selection_details_helper();
    }

    $output->{parsed_output}->{response} = $aref ;

    $Log->exit() if $may_exit;

    return $output;

} ## end sub _get_volume_selection_details

###############################################################################
## @name ::_get_volume_selection_details_helper
##
## @summary
##     Helper method for _get_volume_selection_details
## @synopsis
##     _get_volume_selection_details_helper(%args);

sub _get_volume_selection_details_helper{

    my ( $self, $sess_name ) = @_;

    my $aref=[];

    # Populate this array if you want to fetch any new entries
    my @keys = ("Name","Used_Size(Gb)","Total_Size(Gb)","Snapmirror_Relationship",
        "Qtree_Snapmirror_Relationship","Snapmirror_Source_Or_Destination",
        "Transition_As_Standalone","Transition_With_Snapmirror_Relationship",
    );

    my $row = 1;
    while (1){
        eval {
            my $href = undef;
            foreach my $col (1..@keys){

                my $volume_info_table_selector = $VOLUME_INFO_TABLE_SELECTOR;
                $volume_info_table_selector =~ s/(.*)ROW_NUMBER(.*)COLUMN_NUMBER(.*)/$1$row$2$col$3/;

                my ( $stand, $stag, $checkbox, $text );
                if ( $keys[$col-1] eq "Transition_As_Standalone" ) {
                    $stand=1;
                    $href->{Transition_As_Standalone}={};
                    $checkbox=$GENERIC_VOLUME_STANDALONE_SELECTOR;
                    $text="Transition_As_Standalone";
                }
                if ( $keys[$col-1] eq "Transition_With_Snapmirror_Relationship" ) {
                    $stag=1;
                    $href->{Transition_With_Snapmirror_Relationship}={};
                    $checkbox=$GENERIC_VOLUME_STAGGERED_SELECTOR;
                    $text="Transition_With_Snapmirror_Relationship";
                }
                if ( $stand || $stag ) {

                    my $vol=$href->{Name};
                    $checkbox =~ s/(.*)XXYYZZ(.*)/$1$vol$2/;
                    if($self->_is_element_enabled($checkbox) ){
                        $href->{$text}->{is_enabled} = "true";
                        if($self->_is_checked($checkbox) ){
                            $href->{$text}->{is_checked} = "true";
                        }else {
                            $href->{$text}->{is_checked} = "false";
                        }
                    }else {
                        $href->{$text}->{is_enabled} = "false";
                        $href->{$text}->{is_checked} = "false";
                    }
                    my $elem = $self->_find_element($volume_info_table_selector."/div");
                    my $id = $elem->get_attribute('id');
                    $href->{$text}->{tool_tip} = $id;

                }else {
                    $href->{$keys[$col-1]} = $self->_get_text($volume_info_table_selector);
                }
            }
            push (@$aref,$href);
        };
        if ($@) {
            if ($sess_name){
                $self->_close_tab(
                    string => $sess_name."tab",
                );
                $self->_close_tab(
                    string => "Dashboardtab",
                ) ;
            } else {
                # Closing the Untitled project
                my $selector=$GENERIC_TAB_CLOSE;
                $selector =~ s/(.*)XXYYZZ(.*)/$1Untitled$2/;
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                ) ;
            }
            return $aref;
        }
        $row++;
    }

} ## end sub _get_volume_selection_details_helper

###############################################################################
## @name ::set_page
##
## @summary
##     Takes the user to the page requested for
## @synopsis
##     set_page(%args);

sub set_page{

    my ( $self, %args ) = @_;

    my $sess_name = $args{"session-name"};
    my $navigate_to = $args{"navigate_to"};

    $Log->enter() if $may_enter;

    my $id = undef;
    my $dashboard_enabled = undef;
    if( $self->_is_button_enabled($DASHBOARD) ) {
        $dashboard_enabled = 1;
        $self->_click(
            id                      => $DASHBOARD,
            error_checking_needed   => 1,
        ) ;
        $id = $self->_session_exist($sess_name);
    }

    if( $id ){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;

        # Any new navigation steps should be added to this map
        my $map = {
            VOLUME_SELECTION_PAGE       => $NAVIGATE_VOLUME_SELECTION_PAGE,
            IP_CONFIGURATION            => $NAVIGATE_IP_CONFIGURATION,
            PRECHECK                    => $NAVIGATE_PRECHECK,
            SVM_MAPPING                 => $NAVIGATE_SVM_MAPPING,
            VOLUME_MAPPING              => $NAVIGATE_VOLUME_MAPPING,
            INTERFACE_MAPPING           => $NAVIGATE_INTERFACE_MAPPING,
            DATA_COPY_SCHEDULE          => $NAVIGATE_DATA_COPY_SCHEDULE,
            FINAL_PRECHECK              => $NAVIGATE_FINAL_PRECHECK,
            SUBPROJECT_SUMMARY          => $NAVIGATE_SUBPROJECT_SUMMARY,
        };

        my $navigation_steps = $map->{$navigate_to};

        while ( $navigation_steps =~ /([\w\s-]+):?/g ){
            $self->_click(
                id                      => $1,
                error_checking_needed   => 1,
            ) ;
        }
    } elsif ( $navigate_to eq "VOLUME_SELECTION_PAGE" ){
        $self->_close_tab(
            string => "Dashboardtab",
        );
        $self->_smtt_navigate($NAVIGATE_TRANSITION_CREATE);
        # Create new project here
        my $ses_name="t".int (rand(time));
        $self->_type( $PROJECT_NAME_INPUT ,$ses_name );
        $self->_click(
            id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
            error_checking_needed   => 0,
        ) ;
    } else {
        $self->_close_tab(
            string => "Dashboardtab",
        );
    }

    $Log->exit() if $may_exit;

    return 1;

} ## end sub set_page

###############################################################################
## @name ::get_session_details
##
## @summary
##     Returns the session details to the user
## @synopsis
##     get_session_details(%args);

sub get_session_details{

    my ( $self, %args ) = @_;
    my $project = $args{"project-name"};
    my $group   = $args{"group"};
    my $operation = $args{"operation"};
    my $trans_tab = $args{'trans_mode'};
    my ($output,$color_pixel);
    $Log->enter() if $may_enter;
    my $id = undef;
    my $flag = 0;
    my $hash_ref={};
    if( $self->_is_button_enabled($MIGRATION_DASHBOARD_LINK) ) {
    sleep 5;
        $self->_click(
            id                      => $MIGRATION_DASHBOARD_LINK,
            error_checking_needed   => 0,
        ) ;
        $self->_project_selector($group,$trans_tab);
        my $locator = "gwt-debug-".$project;
        if ($self->_is_element_present($locator)) {
            $id = $locator;
        }
    }
    if( $id ){
        # Session exists

        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;
        if($operation =~ /rollback/){
            if ($self->_is_element_present($REVERT_DASHBOARD)) {
                $self->_click(
                    id                      => $REVERT_DASHBOARD,
                    error_checking_needed   => 0,
                ) ;
            }
        }
        my $proj_name  = $args{arguments}->{'session-name'};
        $self->_create_screenshot($proj_name.$operation."_machine_status");
        my %project_stat = (
            precheck=>  [   "precheck-status:prechecks not run yet",
                            "precheck-status:prechecks completed",
                            "precheck-status:prechecks failed",
                            "gwt-debug-PlanningState:Prechecks not run",
                            "gwt-debug-PlanningState:Precheck Failed",
                            "gwt-debug-PlanningState:Precheck Successful",
                            "gwt-debug-PlanningState:Prechecks not run yet",
                            "gwt-debug-PlanningState",
                            "precheck-status:transition aborted",
            ],
            start   =>  [   "baseline-status:baseline data copy not started",
                            "baseline-status:baseline data copy is scheduled",
                            "baseline-status:baseline data copy paused",
                            "baseline-status:baseline data copy completed",
                            "baseline-status:baseline data copy failed",
                            "baseline-status:transition aborted",
            ],
            incremental=>[ "incremental-status:incremental data copy is scheduled",
                           "incremental-status:incremental data copy paused",
                           "incremental-status:incremental data copy failed",
                           "incremental-status:transition aborted",
                         ],
            precutover=> [  "config-status:apply configuration not started",
                            "config-status:apply configuration in progress",
                            "config-status:copy resync in progress",
                            "config-status:apply configuration completed",
                            "config-status:apply configuration failed",
                            "config-status:transition aborted",
            ],
            cutover=>    [  "cutover-status:cutover not started",
                            "cutover-status:cutover in progress",
                            "cutover-status:cutover failed",
                            "cutover-status:transition completed",
                            "cutover-status:transition aborted",
            ],
            apply_svm_config=> [ "gwt-debug-SVMProvisionState",
                                 "gwt-debug-SVMProvisionState:SVM Configuration Failed", 
                                 "gwt-debug-SVMProvisionState:SVM Configuration Successful",
                                 "gwt-debug-SVMProvisionState:SVM Configuration not run",
            ],
            export=> [  "gwt-debug-ExportState",
                        "gwt-debug-ExportState:Successful",
                        "gwt-debug-ExportState:Failed",
                        "gwt-debug-ExportState:not run",
            ],
            cabling=> [ "gwt-debug-CablingState",
                        "gwt-debug-CablingState:Cabling Verify Successful",
                        "gwt-debug-CablingState:Cabling Verify Failed",
                        "gwt-debug-CablingState:Cabling Verify not run",
            ],
            import=> [  "gwt-debug-ImportState",
                        "gwt-debug-ImportState:Successful",
                        "gwt-debug-ImportState:Failed",
                        "gwt-debug-ImportState:not run",
            ],
            commit=> [ "gwt-debug-CommitState",
                       "gwt-debug-CommitState:Successful",
                       "gwt-debug-CommitState:Failed", 
            ],
            rollback_precheck=> [ "gwt-debug-RollbackState:Rollback:Rollback Initiated",
                                  "gwt-debug-RollbackState:Rollback Steps Generated",
            ],
            rollback_verify_7mode=>[ "gwt-debug-RollbackState",
                                     "gwt-debug-RollbackState:Rollback Completed",
            ],
        );

        my %color_code = (
            orange => "rgb(255, 191, 0)",
            red    => "rgb(250, 88, 88)",
            green  => "rgb(4, 180, 49)",
            white  => "rgb(255, 255, 255)",
        );

        foreach my $key (keys %project_stat){
            if($key eq $operation) {
                foreach my $id (@{$project_stat{$key}}){
                    if ( $self->_is_element_present($id) ){
                        my $elem  = $self->_find_element( "//div[\@id='".$id."']/div/div[1]") ;
                        my $style = $elem->get_attribute('style');
                        my @arr = split (";",$style);
                        foreach my $p (@arr){
                            if($p =~ /background:(.*)(rgb.*)/i) {
                                $color_pixel = $2;
                                last;
                            }
                        }
                        foreach my $i (keys %color_code){
                            if($color_code{$i} eq $color_pixel) {
                                $hash_ref->{'icon_color'} = $i;
                                last;
                            } 
                        }
                        $id =~ /([\w\s-]+):(.*)/g;
                        $hash_ref->{"status"}="$2";
                        $flag = 1;
                        last;
                    }
                }
                last;
            } else {
                next;
            }
        }
        if($flag == 0) {
            $hash_ref->{"status"} = "undefined";
            $hash_ref->{'icon_color'} = "undefined";
        }
        $self->_close_tab(
            string => "Migration",
        );
    } else {
        $Log->comment("project does not exist in the tool");
        $self->_close_tab(
            string => "Migration",
        );
    }

    $output->{parsed_output} = $hash_ref ;

    $Log->exit() if $may_exit;

    return $output;

} ## end sub get_session_details


###############################################################################
## @name ::get_edit_tab_details
##
## @summary
##     Returns the session details to the user
## @synopsis
##     get_edit_tab_details(%args);

sub get_edit_tab_details{

    my ( $self, %args ) = @_;
    my $project = $args{"project-name"};
    my $group = (defined $args{"group-name"}) ? $args{"group-name"}: "Default";
    my $operation = $args{"goto_tab"};
    my $trans_tab = $args{'trans_mode'};
    my $output;
    $Log->enter() if $may_enter;

    my $id = undef;
    my $flag = 0;
    my $hash_ref={};

    if( $self->_is_button_enabled($MIGRATION_DASHBOARD_LINK) ) {
        $self->_click(
            id                      => $MIGRATION_DASHBOARD_LINK,
            error_checking_needed   => 1,
        ) ;
        $self->_project_selector($group,$trans_tab);
        my $locator = "gwt-debug-".$project;
        if ($self->_is_element_present($locator)) {
            $id = $locator;
        }
    } else {
        $Log->comment("Migration Tab is not present");
        $self->_close_tab(
            string => "Migration",
        );
        return $hash_ref;
    }

    my %tab_code = (
        Select_Source_Volumes  => $SWITCH_SOURCE_VOLUMES_TAB,
        Select_Data_Copy_IP    => $SWITCH_DATA_IP_TAB,
        Select_SVM             => $SWITCH_SVM_TAB,
        Map_Volumes            => $SWITCH_MAP_VOLUMES_TAB,
        Network_Configuration  => $SWITCH_NETWORK_TAB,
        Configure_Schedule     => $SWITCH_CONFIGURE_SCHEDULE_TAB,
        Plan_Configuration     => $SWITCH_PLAN_CONFIG_TAB,
        Select_CFT_Source      => $SWITCH_CFT_SOURCE_TAB,
        Select_CFT_Cluster     => $SWITCH_CFT_CLUSTER_TAB,
        CFT_SVM_Map            => $SWITCH_MAP_SVMS_TAB,
        CFT_Network_Config     => $SWITCH_CFT_NETWORK_TAB,
    );

    if( $id ){
        $self->_click(
            id                      => $id,
            error_checking_needed   => 1,
        ) ;
        if(($self->_is_element_present($PROJECT_INSIDE_EDIT))
            || ($self->_is_element_present($PROJECT_CFT_EDIT))){
            if($operation =~ /Select Data Copy IP/i){    #CBT DATA IP
                $self->_edit_data_IP_tab(%args);
            } elsif($operation =~/Select SVM/i) {        #CBT SVM
                $self->_edit_SVM_tab(%args);
            } elsif($operation =~/Plan Configuration/i){ #CBT/CFT Plan Config
                $self->_edit_Plan_config(%args);
            } elsif($operation =~/Configure Schedule/i){ #CBT schedule edit
                $self->_edit_Config_Schedule(%args);
            } elsif($operation =~/Map Volumes/i) {  #CBT Map cluster volumes
                $self->_edit_Map_Volumes(%args);
            } elsif($operation =~ /Source Volumes/i) { #CBT source volumes
                $self->_edit_Source_Volumes(%args);
            } elsif($operation =~ /Network Configuration/i){ #CBT/CFT Network LIF edit
                $self->_edit_Network_Configuration(%args);
            } elsif($operation =~ /Cluster Edit/i){  #CFT Cluster edit
                $self->_edit_Cluster_Config(%args);
            } elsif($operation =~ /CFT SVM Mapping/){ #CFT SVM Mapping Edit
                $self->_edit_SVM_Map(%args);
            }
            foreach my $i (keys %tab_code){
                my $tab_id = $tab_code{$i};
                if($self->_is_element_present($tab_id)) {
                    my $elem  = $self->_find_element($tab_id);
                    my $class = $elem->get_attribute('class');
                    if ($class =~ /next/i) {
                        $hash_ref->{$i} = "tab_disabled";
                    } else {
                        $hash_ref->{$i} = "tab_enabled";
                    }
                }
            }
            if($operation !~ /Plan Configuration/i && $operation !~ /Source Volumes/i){
                $self->_click(
                    id                      => $NEXT,
                    error_checking_needed   => 1,
                ) ;
            }elsif($operation =~ /Source Volumes/) {
                eval {
                    $self->_click(
                        id                      => $NEXT,
                        error_checking_needed   => 0,
                    ) ;
                };
                eval{
                    $self->_click(
                        id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                        error_checking_needed   => 0,
                    ) ;
                }; ## if snaplock volumes edit exists
                eval{
                    $self->_click(
                        id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                        error_checking_needed   => 0,
                    ) ;
                }; ##continue after snaplock volumes warning
            }

            if($self->_is_element_present($SWITCH_SOURCE_VOLUMES_TAB)) {
                $self->_click(
                    id                      => $SWITCH_SOURCE_VOLUMES_TAB,
                    error_checking_needed   => 1,
                ) ;
            } else {
                $self->_click(
                    id                      => $SWITCH_CFT_SOURCE_TAB,
                    error_checking_needed   => 1,
                ) ;
            }
            my $i=0;
            while($i<50){
                if($self->_is_element_present($MIGRATION_TAB)) {
                    $self->_close_tab(
                        string => "Migration",
                    );
                    last;
                } elsif($self->_is_button_enabled($NEXT)) {
                    $self->_click(
                        id                      => $NEXT,
                        error_checking_needed   => 0,
                    ) ;
                    $self->_create_screenshot("next");
                    sleep(2);
                    eval{
                        $self->_click(
                            id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                            error_checking_needed   => 0,
                        ) ;
                    };
                } else {
                    $self->_create_screenshot("couldnot_find_migration_tab_OrNext");
                    $self->_repair_session();
                    $self->_throw_exception("couldnot find next page control");
                }
                $i++;
            }
            if($i == 50){
                $self->_create_screenshot("couldnot_find_migration_tab_OrNext");
                $self->_repair_session();
                $self->_throw_exception("couldnot find Migration Tab or next button");
            }
        }
    } else {
        $Log->comment("project does not exist in the tool");
        $self->_close_tab(
            string => "Migration",
        );
    }
    $output->{parsed_output} = $hash_ref ;

    $Log->exit() if $may_exit;

    return $output;

} ## end sub get_session_details


#################_edit_Source_Volumes###########################################
sub _edit_Source_Volumes {
    my ( $self, %args ) = @_;
    my $project = $args{'project-name'};
    my $Smode_credentials = $args{S_mode_credentials};
    my $session_type = $args{'session-type'};
    my $add_volume   = $args{'volumepair_add_args'}->{'volume-name-add'};
    my $remove_volume = $args{'volumepair_add_args'}->{'volume-name-remove'};
    my $project_edit_config = $PROJECT_INSIDE_EDIT;
    $self->_click(
        id                      => $project_edit_config,
        error_checking_needed   => 0,
    );
    $self->_click(
        id                      => $EDIT_SOURCE_VOLUMES,
        error_checking_needed   => 1,
    ) ;
    $self->_close_tab(
        string => "Migration",
    );
    if(defined $add_volume) {
        if($session_type eq "standalone"){
            $self->_add_remove_volumes_from_ref_array($add_volume, "add",$project);
        }elsif ($session_type eq "secondary"){
            $self->_add_remove_volumes_from_ref_array($add_volume, "add",$project,"staggered",%{$Smode_credentials});
        }else{
            # Create arguments not present. Adding volumes standalone style
            $self->_add_remove_volumes_from_ref_array($add_volume, "add",$project);
        }
    }
    if(defined $remove_volume) {
        if($session_type eq "standalone"){
            $self->_add_remove_volumes_from_ref_array($remove_volume, "remove",$project);
        }elsif ($session_type eq "secondary"){
            $self->_add_remove_volumes_from_ref_array($remove_volume, "remove",$project,"staggered",%{$Smode_credentials});
        }else{
            # Create arguments not present. Adding volumes standalone style
            $self->_add_remove_volumes_from_ref_array($remove_volume, "remove",$project);
        }
    }
}

###############################################################################
## @name ::_add_volumes_given_vol_ref_array
##
## @summary
##     This is a helper method for adding volumepairs
## @synopsis
##     _add_volumes_given_vol_ref_array(%args);

sub _add_remove_volumes_from_ref_array{

    my ( $self, $vol_ar, $operation, $project, $ses_type, %args) = @_;
    $Log->enter() if $may_enter;
    # Add support for qtree transition also
    foreach (0..@{$vol_ar}-1){
        my $vol = $vol_ar->[$_];
        my $checkbox=$GENERIC_VOLUME_STANDALONE_SELECTOR;
        $checkbox =~ s/(.*)XXYYZZ(.*)/$1$vol$2/;
        if($self->_is_element_enabled($checkbox) ){
            if(($operation =~/add/)&&($self->_is_checked($checkbox))) {
                $self->_create_screenshot($vol."_Already_Added_error");
                $self->_repair_session();
                $self->_throw_exception("Volume $vol is already been added to the session");
            }
            if ($ses_type eq "staggered"){
                $checkbox=$GENERIC_VOLUME_STAGGERED_SELECTOR;
                $checkbox =~ s/(.*)XXYYZZ(.*)/$1$vol$2/;
            }
            $self->_click(
                id                      => $checkbox,
                error_checking_needed   => 0,
            ) ;

            if($self->_is_element_present($VFILERHOSTNAME)) {
                $self->_add_credentials(%args);
            }
            eval{
                my $popup_txt=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_MESSAGE);
                $self->_click(
                    id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                    error_checking_needed   => 0,
                ) ;
            };
        }else{
            $self->_create_screenshot("Vol_Not_Found");
            $self->_repair_session();
            $self->_throw_exception("Probably the volume requested for does not exist"
                ." or is offline/involved in some other subproject.. "
                ."Unable to find the element with ".$checkbox);
        }
    }
    $self->_create_screenshot($project.$operation."_Vol_edit");
    $Log->exit() if $may_exit;

    return 1;
} ## end sub _add_remove_volumes_from_ref_array


#################_edit_Map_Volumes##############################################
sub _edit_Map_Volumes{
    my ( $self, %args ) = @_;
    my $project = $args{'project-name'};
    my $project_edit_config = $PROJECT_INSIDE_EDIT;
    my $volume_map_args = $args{'cluster_volume_map_args'};
    my $volume_mount  = (defined $args{"mount_policy"}) ? $args{'mount_policy'} : "Preserve 7-Mode Mount paths";
    $self->_click(
        id                      => $project_edit_config,
        error_checking_needed   => 0,
    ) ;
    $self->_click(
        id                      => $EDIT_TARGET_VOLUME,
        error_checking_needed   => 1,
    ) ;
    $self->_close_tab(
        string => "Migration",
    );
    $self->_click(
        id                    => $EDIT_VOLUME_MOUNT,
        error_checking_needed => 0,
    );
    my $policyselector =  $GENERIC_LISTITEM_SELECTOR;
    $policyselector =~ s/(.*)XXYYZZ(.*)/$1$volume_mount$2/;
    if($self->_is_element_present($policyselector)){
        $self->_click(
            id                      => $policyselector,
            error_checking_needed   => 0,
        );
    } else {
        $self->_create_screenshot($project."_Target_Mount_Policy_Not_Found");
        $self->_repair_session();
        $self->_throw_exception($volume_mount." is not present in the Target mount policy list");
    }
    if(defined $volume_map_args) {
        foreach my $aggr (keys %$volume_map_args){
            print Dumper($aggr);
            foreach my $vol_arr (@$volume_map_args{$aggr}){
                foreach my $vol (@$vol_arr){
                    my $aggr_mount = $EDIT_AGGR_MOUNT;
                    $aggr_mount =~ s/(.*)XXYYZZ(.*)/$1$vol$2/;
                    $self->_click(
                        id                    => $aggr_mount,
                        error_checking_needed => 0,
                    );
                    my $aggrselector =  $GENERIC_LISTITEM_SELECTOR;
                    $aggrselector =~ s/(.*)XXYYZZ(.*)/$1$aggr$2/;
                    if($self->_is_element_present($aggrselector)){
                        $self->_click(
                            id                      => $aggrselector,
                            error_checking_needed   => 0,
                        );
                    } else {
                        $self->_create_screenshot($project."_Target_Aggr_Not_Found");
                        $self->_repair_session();
                        $self->_throw_exception($aggr." is not present in the Target aggregate list");
                    }
                }
            }
        }
    }
    $self->_create_screenshot($project."_cluster_volumes_edit_mapping");
}

#################_edit_Network_Configuration##############################################
sub _edit_Network_Configuration{
    my ( $self, %args ) = @_;
    my $project = $args{'project-name'};
    my $sleep_count = $GENERIC_WAIT / 1000;
    if($self->_is_element_present($PROJECT_INSIDE_EDIT)){
        $self->_click(
            id                      => $PROJECT_INSIDE_EDIT,
            error_checking_needed   => 0,
        ) ;
    }else {
        $self->_click(
            id                      => $PROJECT_CFT_EDIT,
            error_checking_needed   => 0,
        ) ;
    }
    $self->_click(
        id                      => $NETWORKING_TAB,
        error_checking_needed   => 1,
    ) ;
    $self->_close_tab(
        string => "Migration",
    );
    if(defined $args{'networking_add'}) {
        my $source_ip = $args{'networking_add'}->{"ip-address"}; 
        my $gateway   = $args{'networking_add'}->{"default-gateway"};
        my $netmask   = $args{'networking_add'}->{"netmask"};
        my $homenode  = $args{'networking_add'}->{"home-node"};
        my $homeport  = $args{'networking_add'}->{"home-port"};
        my $svm       = $args{'networking_add'}->{"svm-name"};
        $self->_click(
            id                      => $GENERIC_ADDLIF_BUTTON,
            error_checking_needed   => 0,
        ) ;
        sleep $sleep_count;
        $self->_type($GENERIC_TARGET_IP_INPUT,$source_ip);
        sleep 2;
        $self->_type($GENERIC_TARGET_GATEWAY_INPUT,$gateway);
        sleep 2;
        $self->_type($GENERIC_TARGET_NETMASK_INPUT,$netmask);
        sleep 2;
        if (defined $svm){
            $self->_click(
                id                      => $GENERIC_SVM_INPUT,
                error_checking_needed   => 0,
            );
             # Sleeping to allow drop-down some time to appear
            sleep $sleep_count;
            my $vserver = $svm;
            my $portselector =  $GENERIC_LISTITEM_SELECTOR;
            $portselector =~ s/(.*)XXYYZZ(.*)/$1$vserver$2/;
            if($self->_is_element_present($portselector)){
                $self->_click(
                    id                      => $portselector,
                    error_checking_needed   => 0,
                );
            } else {
                $self->_create_screenshot($project."_Target_SVM_Not_Found");
                $self->_repair_session();
                $self->_throw_exception($svm." is not present in the Target svm list");
            }
        }
        if (defined $homenode){
            $self->_click(
                id                      => $GENERIC_TARGET_NODE_INPUT,
                error_checking_needed   => 0,
            );
            # Sleeping to allow drop-down some time to appear
            sleep $sleep_count;
            my $node = $homenode;
            my $nodeselector =  $GENERIC_LISTITEM_SELECTOR;
            $nodeselector =~ s/(.*)XXYYZZ(.*)/$1$node$2/;
            if($self->_is_element_present($nodeselector)){
                $self->_click(
                    id                      => $nodeselector,
                    error_checking_needed   => 0,
                );
            } else {
                $self->_create_screenshot($project."_Target_Node_Not_Found");
                $self->_repair_session();
                $self->_throw_exception($homenode." is not present in the Target node list");
            }
        }

        if (defined $homeport){
            $self->_click(
                id                      => $GENERIC_TARGET_PORT_INPUT,
                error_checking_needed   => 0,
            );
             # Sleeping to allow drop-down some time to appear
            sleep $sleep_count;
            my $port = $homeport;
            my $portselector =  $GENERIC_LISTITEM_SELECTOR;
            $portselector =~ s/(.*)XXYYZZ(.*)/$1$port$2/;
            if($self->_is_element_present($portselector)){
                $self->_click(
                    id                      => $portselector,
                    error_checking_needed   => 0,
                );
            } else {
                $self->_create_screenshot($project."_Target_Port_Not_Found");
                $self->_repair_session();
                $self->_throw_exception($homeport." is not present in the Target port list");
            }
        }

        $self->_click(
            id                      => $GENERIC_SAVELIF_BUTTON,
            error_checking_needed   => 1,
        );
        $self->_wait_for_element_not_present($GENERIC_SPINNING_ICON);
        $self->_create_screenshot("Lif_Add_".$project);
    }

    if(defined $args{'networking_edit'}) {
        my $source_ip = $args{'networking_edit'}->{"ip-address"};
        my $gateway   = $args{'networking_edit'}->{"default-gateway"};
        my $netmask   = $args{'networking_edit'}->{"netmask"};
        my $homenode  = $args{'networking_edit'}->{"home-node"};
        my $homeport  = $args{'networking_edit'}->{"home-port"};
        my $ip_select = $GENERIC_LIF_IP;
        $ip_select =~ s/XXYYZZ/$source_ip/;
        my $project_edit = $ip_select.$GENERIC_LIF_EDIT;
        $self->_click(
            id                      => $project_edit,
            error_checking_needed   => 0,
        ) ;
        sleep $sleep_count;
        if(defined $gateway){$self->_type($GENERIC_TARGET_GATEWAY_INPUT,$gateway);}
        sleep 2;
        if(defined $netmask){$self->_type($GENERIC_TARGET_NETMASK_INPUT,$netmask);}
        if (defined $homenode){
            $self->_click(
                id                      => $GENERIC_TARGET_NODE_INPUT,
                error_checking_needed   => 0,
            );
            # Sleeping to allow drop-down some time to appear
            sleep $sleep_count;
            my $node = $homenode;
            my $nodeselector =  $GENERIC_LISTITEM_SELECTOR;
            $nodeselector =~ s/(.*)XXYYZZ(.*)/$1$node$2/;
            if($self->_is_element_present($nodeselector)){
                $self->_click(
                    id                      => $nodeselector,
                    error_checking_needed   => 0,
                );
            } else {
                $self->_create_screenshot($project."_Target_Edit_Node_Not_Found");
                $self->_repair_session();
                $self->_throw_exception($homenode." is not present in the Target Node list");
            }
        }

        if (defined $homeport){
            $self->_click(
                id                      => $GENERIC_TARGET_PORT_INPUT,
                error_checking_needed   => 0,
            );
             # Sleeping to allow drop-down some time to appear
            sleep $sleep_count;
            my $port = $homeport;
            my $portselector =  $GENERIC_LISTITEM_SELECTOR;
            $portselector =~ s/(.*)XXYYZZ(.*)/$1$port$2/;
            if($self->_is_element_present($portselector)){
                $self->_click(
                    id                      => $portselector,
                    error_checking_needed   => 0,
                );
            } else {
                $self->_create_screenshot($project."_Target_Edit_Port_Not_Found");
                $self->_repair_session();
                $self->_throw_exception($homeport." is not present in the Target Port list");
            }
        }
        $self->_click(
            id                      => $GENERIC_SAVELIF_BUTTON,
            error_checking_needed   => 1,
        );
        $self->_wait_for_element_not_present($GENERIC_SPINNING_ICON);
        $self->_create_screenshot("Lif_Edit_".$project);
    }

    if(defined $args{'networking_delete'}) {
        my $delete_ip = $args{'networking_delete'}->{"ip-address"};
        my $ip_select = $GENERIC_LIF_IP;
        $ip_select =~ s/XXYYZZ/$delete_ip/;
        my $project_edit = $ip_select.$GENERIC_LIF_REMOVE;
        $self->_click(
            id                      => $project_edit,
            error_checking_needed   => 1,
        ) ;

        $self->_wait_for_element_not_present($GENERIC_SPINNING_ICON);
        $self->_create_screenshot("Lif_Delete_".$project);
    }
    if(defined $args{'add_smode_trans_lif'}) {
        my $ip = $args{'add_smode_trans_lif'}->{'ip-address'};
        my $gateway   = $args{'add_smode_trans_lif'}->{"default-gateway"};
        my $netmask   = $args{'add_smode_trans_lif'}->{"netmask"};
        my $homenode  = $args{'add_smode_trans_lif'}->{"home-node"};
        my $homeport  = $args{'add_smode_trans_lif'}->{"home-port"};
        $self->_click(
            id                      => $GENERIC_SMODE_LIF,
            error_checking_needed   => 0,
        );
        my $generic_lif = $TRANS_LIF_IP;
        $generic_lif =~ s/XXYYZZ/$ip/;
        my $gateway_select = $generic_lif.$TRANS_LIF_GATEWAY;
        my $netmask_select = $generic_lif.$TRANS_LIF_NETMASK;
        my $node_select    = $generic_lif.$TRANS_LIF_NODE;
        my $port_select    = $generic_lif.$TRANS_LIF_PORT;
        if(defined $gateway){$self->_type($gateway_select,$gateway);}
        if(defined $netmask){$self->_type($netmask_select,$netmask);}
        if(defined $homenode){
            $self->_click(
                id                      => $node_select,
                error_checking_needed   => 0,
            );
            my $homeselector =  $GENERIC_LISTITEM_SELECTOR;
            $homeselector =~ s/(.*)XXYYZZ(.*)/$1$homenode$2/;
            if($self->_is_element_present($homeselector)){
                $self->_click(
                        id                      => $homeselector,
                        error_checking_needed   => 0,
                );
            } else {
                $self->_create_screenshot($project."_Target_Trans_Lif_Node_Not_Found");
                $self->_repair_session();
                $self->_throw_exception($homenode." is not present in the Target Node list");
            }
        }

        if(defined $homeport){
            $self->_click(
                id                      => $port_select,
                error_checking_needed   => 0,
            );
            my $portselector =  $GENERIC_LISTITEM_SELECTOR;
            $portselector =~ s/(.*)XXYYZZ(.*)/$1$homeport$2/;
            if($self->_is_element_present($portselector)){
                $self->_click(
                        id                      => $portselector,
                        error_checking_needed   => 0,
                );
            } else {
                $self->_create_screenshot($project."_Target_Trans_Lif_Port_Not_Found");
                $self->_repair_session();
                $self->_throw_exception($homeport." is not present in the Target Port list");
            }
        }
        my $smode_trans_lif_select = $GENERIC_SMODE_LIF_SELECT;
        $smode_trans_lif_select =~ s/XXYYZZ/$ip/;
        if($self->_is_element_present($smode_trans_lif_select)){
            $self->_click(
                id                      => $smode_trans_lif_select,
                error_checking_needed   => 0,
            ) ;
        }
        $self->_create_screenshot("Smode_transLif_".$project);
        sleep(2);
        $self->_click(
            id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
            error_checking_needed   => 1,
        ) ;
    }
}

#################_edit_Config_Schedule##############################################
sub _edit_Config_Schedule{
    my ( $self, %args ) = @_;
    my $project = $args{'project-name'};
    my $schedule_remove = $args{"remove_schedule"};
    my $schedule_add    = $args{'schedule_add_args'};
    my $schedule_edit   = $args{'schedule_edit_args'};
    my $project_edit_config =$PROJECT_INSIDE_EDIT;
    $self->_click(
        id                      => $project_edit_config,
        error_checking_needed   => 0,
    ) ;
    $self->_click(
        id                      => $EDIT_CONFIG_SCHEDULE,
        error_checking_needed   => 1,
    ) ;
    $self->_close_tab(
        string => "Migration",
    );
    if(defined $schedule_remove){
        my $remove = $SCHEDULE_REMOVE;
        $remove =~ s/XXYYZZ/$schedule_remove/;
        $self->_click(
            id                      => $remove,
            error_checking_needed   => 1,
        ) ;
        $self->_create_screenshot("Schedule_remove_".$project);
    }

    if ($schedule_add ) {

        foreach ( @{$schedule_add} ) {

            $self->_click(
                id                      => $CREATE_SCHEDULE,
                error_checking_needed   => 0,
            ) ;

            $self->_type( $GENERIC_SCHEDULE_NAME ,$_->{"schedule-name"});

            $self->_add_schedule_days($_->{"days-range"});

            # Adding start time
            my @s_t = split(":",$_->{"start-time"});
            my $selector=$GENERIC_LISTITEM_SELECTOR;
            $self->_click(
                id                      => $GENERIC_SCHEDULE_START_TIME_HOUR_INPUT,
                error_checking_needed   => 0,
            ) ;
            $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[0]$2/;
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) ;
            $self->_click(
                id                      => $GENERIC_SCHEDULE_START_TIME_MIN_INPUT,
                error_checking_needed   => 0,
            ) ;
            $selector=$GENERIC_LISTITEM_SELECTOR;
            $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[1]$2/;
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) ;

            # Adding duration
            my @duration = split(":",$_->{"duration"});
            $self->_type( $GENERIC_SCHEDULE_DURATION_HOUR_INPUT ,$duration[0] );
            $self->_click(
                id                      => $GENERIC_SCHEDULE_DURATION_MIN_INPUT,
                error_checking_needed   => 0,
            ) ;
            $selector=$GENERIC_LISTITEM_SELECTOR;
            $selector =~ s/(.*)XXYYZZ(.*)/$1$duration[1]$2/;
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) ;

            if ( defined $_->{"available-transfers-percentage"} ) {
                # Adding available-transfers-percentage
                $self->_click(
                    id                      => $GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO,
                    error_checking_needed   => 0,
                ) if(! $self->_is_checked($GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO) );
                $self->_type(
                    $GENERIC_AUTO_CONTROL_STREAM_SELECTOR ,$_->{"available-transfers-percentage"} );
            }

            if ( defined $_->{"max-cap-concurrent-transfers"} ) {
                # Adding stream count
                $self->_click(
                    id                      => $GENERIC_STREAM_SELECTOR_RADIO,
                    error_checking_needed   => 0,
                ) if(! $self->_is_checked($GENERIC_STREAM_SELECTOR_RADIO) );
                $self->_type( $GENERIC_STREAM_SELECTOR ,$_->{"max-cap-concurrent-transfers"} );
            }

            # Adding throttle
            if (defined $_->{"session-snapmirror-throttle"}){
                $self->_click(
                    id                      => $GENERIC_THROTTLE_SELECTOR_RADIO,
                    error_checking_needed   => 1,
                ) ;
                $self->_type( $GENERIC_THROTTLE_SELECTOR ,$_->{"session-snapmirror-throttle"} );
            }

            # Adding update frequency
            if (defined $_->{"update-frequency"}){
                my @u_f = split(":",$_->{"update-frequency"});
                $self->_type( $GENERIC_SCHEDULE_UPDATE_FREQUENCY_HOUR_INPUT ,$u_f[0]);
                $self->_click(
                    id                      => $GENERIC_SCHEDULE_UPDATE_FREQUENCY_MIN_INPUT,
                    error_checking_needed   => 0,
                ) ;
                $selector=$GENERIC_AGGR_SELECTOR;
                $selector =~ s/(.*)XXYYZZ(.*)/$1$u_f[1]$2/;
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                ) ;
            }

            $self->_click(
                id                      => $GENERIC_SCHEDULE_CREATE_BUTTON,
                error_checking_needed   => 1,
            ) ;
            if($self->_is_element_present($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE)){
                my $text = $self->_get_text($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE);
                $self->_take_screenshot();
                $self->_repair_session();
                $self->_throw_exception("Error while adding schedule : ".$text);
            }

        }
        $self->_create_screenshot("Schedule_create_".$project);
    }
    if ($schedule_edit) {

        foreach ( @{$schedule_edit} ) {

            my $schedule_name = $_->{"schedule-name"};
            my $edit_schedule = $EDIT_SCHEDULE;
            $edit_schedule =~ s/XXYYZZ/$schedule_name/;
            $self->_click(
                id                      => $edit_schedule,
                error_checking_needed   => 0,
            ) ;

            $self->_add_schedule_days($_->{"days-range"});

            # Adding start time
            my @s_t = split(":",$_->{"start-time"});
            my $selector=$GENERIC_LISTITEM_SELECTOR;
            $self->_click(
                id                      => $GENERIC_SCHEDULE_START_TIME_HOUR_INPUT,
                error_checking_needed   => 0,
            ) ;
            $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[0]$2/;
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) ;
            $self->_click(
                id                      => $GENERIC_SCHEDULE_START_TIME_MIN_INPUT,
                error_checking_needed   => 0,
            ) ;
            $selector=$GENERIC_LISTITEM_SELECTOR;
            $selector =~ s/(.*)XXYYZZ(.*)/$1$s_t[1]$2/;
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) ;

            # Adding duration
            my @duration = split(":",$_->{"duration"});
            $self->_type( $GENERIC_SCHEDULE_DURATION_HOUR_INPUT ,$duration[0] );
            $self->_click(
                id                      => $GENERIC_SCHEDULE_DURATION_MIN_INPUT,
                error_checking_needed   => 0,
            ) ;
            $selector=$GENERIC_LISTITEM_SELECTOR;
            $selector =~ s/(.*)XXYYZZ(.*)/$1$duration[1]$2/;
            $self->_click(
                id                      => $selector,
                error_checking_needed   => 0,
            ) ;

            if ( defined $_->{"available-transfers-percentage"} ) {
                # Adding available-transfers-percentage
                $self->_click(
                    id                      => $GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO,
                    error_checking_needed   => 0,
                ) if(! $self->_is_checked($GENERIC_AUTO_CONTROL_STREAM_SELECTOR_RADIO) );
                $self->_type(
                    $GENERIC_AUTO_CONTROL_STREAM_SELECTOR ,$_->{"available-transfers-percentage"} );
            }

            if ( defined $_->{"max-cap-concurrent-transfers"} ) {
                # Adding stream count
                $self->_click(
                    id                      => $GENERIC_STREAM_SELECTOR_RADIO,
                    error_checking_needed   => 0,
                ) if(! $self->_is_checked($GENERIC_STREAM_SELECTOR_RADIO) );
                $self->_type( $GENERIC_STREAM_SELECTOR ,$_->{"max-cap-concurrent-transfers"} );
            }

            # Adding throttle
            if (defined $_->{"session-snapmirror-throttle"}){
                $self->_click(
                    id                      => $GENERIC_THROTTLE_SELECTOR_RADIO,
                    error_checking_needed   => 1,
                ) ;
                $self->_type( $GENERIC_THROTTLE_SELECTOR ,$_->{"session-snapmirror-throttle"} );
            }

            # Adding update frequency
            if (defined $_->{"update-frequency"}){
                my @u_f = split(":",$_->{"update-frequency"});
                $self->_type( $GENERIC_SCHEDULE_UPDATE_FREQUENCY_HOUR_INPUT ,$u_f[0]);
                $self->_click(
                    id                      => $GENERIC_SCHEDULE_UPDATE_FREQUENCY_MIN_INPUT,
                    error_checking_needed   => 0,
                ) ;
                $selector=$GENERIC_AGGR_SELECTOR;
                $selector =~ s/(.*)XXYYZZ(.*)/$1$u_f[1]$2/;
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                ) ;
            }

            $self->_click(
                id                      => $GENERIC_SCHEDULE_CREATE_BUTTON,
                error_checking_needed   => 1,
            ) ;

            if($self->_is_element_present($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE)){
                my $text = $self->_get_text($GENERIC_SCHEDULE_CREATE_ERROR_MESSAGE);
                $self->_take_screenshot();
                $self->_repair_session();
                $self->_throw_exception("Error while adding schedule : ".$text);
            }

        }
        $self->_create_screenshot("Schedule_Edit_".$project);
    }
}

#######################_edit_SVM_Map############################################
sub _edit_SVM_Map{
    my ( $self, %args ) = @_;
    my $project = $args{'project-name'};
    my $cft_svm_mapping_add_pri = $args{'cft_svm_mapping_add_pri'};
    my $cft_svm_mapping_add_sec = $args{'cft_svm_mapping_add_sec'};
    $self->_click(
        id                      => $PROJECT_CFT_EDIT,
        error_checking_needed   => 0,
    ) ;
    $self->_click(
        id                      => $EDIT_SVM_MAPPING,
        error_checking_needed   => 0,
    ) ;
    $self->_close_tab(
        string => "Migration",
    );
    if (defined $cft_svm_mapping_add_pri) {
        my $ip = $cft_svm_mapping_add_pri->{'7-mode-host-name'};
        my $vfiler = $cft_svm_mapping_add_pri->{'vfiler-name'};
        my $vsm = $cft_svm_mapping_add_pri->{'svm-name'};
        $self->_select_vsm($ip,$vfiler,$vsm);
        $self->_create_screenshot("svmMap_primary_edit_".$project);
    }

    if (defined $cft_svm_mapping_add_sec) {
        my $ip = $cft_svm_mapping_add_sec->{'7-mode-host-name'};
        my $vfiler = $cft_svm_mapping_add_sec->{'vfiler-name'};
        my $vsm = $cft_svm_mapping_add_sec->{'svm-name'};
        $self->_select_vsm($ip,$vfiler,$vsm);
        $self->_create_screenshot("svmMap_sec_edit_".$project);
    }
    $self->_create_screenshot($project."_SVM_mapping_Edit");
}

#################_edit_data_IP_tab##############################################
sub _edit_data_IP_tab{
    my ( $self, %args ) = @_;
    my $project = $args{'project-name'};
    my $project_edit_config = $PROJECT_INSIDE_EDIT;
    my $dc_ip = $args{"data-copy-ipaddress"};
    my $m_ip = $args{"multipathing-ipaddress"};
    $self->_click(
        id                      => $project_edit_config,
        error_checking_needed   => 0,
    ) ;
    $self->_click(
        id                      => $EDIT_DATA_IP,
        error_checking_needed   => 1,
    ) ;
    $self->_close_tab(
        string => "Migration",
    );
    $self->_type( $DATA_COPY_IP, $dc_ip ) if( $dc_ip );
    $self->_type( $MULTIPATH_IP, $m_ip ) if( $m_ip );
    $self->_create_screenshot("Data_multipath_edit_".$project);
}

#################_edit_Cluster_Config###########################################
sub _edit_Cluster_Config{
    my ( $self, %args ) = @_;
    my $S_mode_credentials      = $args{S_mode_credentials};
    my $S_mode_credentials_HA   = $args{S_mode_credentials_HA};
    my $C_mode_credentials      = $args{C_mode_credentials};
    my $cft_clusterinfo_add     = $args{cft_clusterinfo_add};
    my $cft_nodemapping_set     = $args{cft_nodemapping_set};
    my $project = $args{'project-name'};
    if($self->_is_element_present($PROJECT_INSIDE_EDIT)){
        $self->_click(
            id                      => $PROJECT_INSIDE_EDIT,
            error_checking_needed   => 0,
        ) ;
    }else {
        $self->_click(
            id                      => $PROJECT_CFT_EDIT,
            error_checking_needed   => 0,
        ) ;
    }
    $self->_click(
        id                      => $GENERIC_CLUSTER_EDIT,
        error_checking_needed   => 1,
    ) ;
    $self->_close_tab(
        string => "Migration",
    );
    my $ip = $args{$cft_clusterinfo_add}->{"c-mode-host-name"};
    my $host_cmode = $self->_get_host_from_ip($ip,"c");
    my $host;
    if($host_cmode =~ /(.*)\($ip\)/){
        $host = $1;
    }
    $host =~ s/(.*)\s+/$1/;
    my $pri_clus_node = $PRIMARY_CLUS_NODE;
    my $pri_cmode = $cft_nodemapping_set->{'cluster-node1-name'};
    $pri_clus_node =~ s/(.*)XXYYZZ(.*)/$1$pri_cmode$2/;
    my $sec_clus_node = $SECONDARY_CLUS_NODE;
    my $sec_cmode = $cft_nodemapping_set->{'cluster-node2-name'};
    $sec_clus_node =~ s/(.*)XXYYZZ(.*)/$1$sec_cmode$2/;
    my $cft_cluster = $CFT_CLUSTER;
    $cft_cluster =~ s/(.*)XXYYZZ(.*)/$1$host$2/;
    if($self->is_element_present($cft_cluster)){
        $self->_click(
            id                      => $cft_cluster,
            error_checking_needed   => 0,
        ) ;
        my $var = 0;
        #Iterating 12 times because a cluster can have 12 pairs of nodes
        while ($var < 12) {
            my $c_ha_selector = "//div[contains(text(),'$pri_cmode/$sec_cmode')]/parent::div/div/div/div";
                if ($self->_is_element_present($c_ha_selector)) {
                    $self->_click(
                        id                      => $c_ha_selector,
                        error_checking_needed   => 0,
                    ) ;
                    last;
                }
            $var++;
        }
        if ( $var == 12 ){
            $self->_create_screenshot("HApair_not_present");
            $self->_close_tab(
                string => "\(CFT\)",
            );
            $self->_throw_exception("The Cluster $pri_cmode/$sec_cmode with is not present");
        }
        if ($cft_nodemapping_set) {
            if (($self->_is_element_present($pri_clus_node)) && ($self->_is_element_present($sec_clus_node))) {
                print"the node mapping is already done\n";
            }else{
                $self->_click(
                    id                      => $SWAP_BUTTON,
                    error_checking_needed   => 0,
                ) ;
            }
        }
    }else{
        $self->_create_screenshot("Node_not_present");
        $self->_close_tab(
            string => "\(CFT\)",
        );
        $self->_throw_exception("The Cluster  is not present");
    }
    $self->_create_screenshot($project."ClusterHA_Edit");
}

#################_edit_Plan_config##############################################
sub _edit_Plan_config{
    my ( $self, %args ) = @_;
    my $project = $args{'project-name'};
    my $plan_configuration = $args{'plan_configuration'};
    if($self->_is_element_present($PROJECT_INSIDE_EDIT)){
        $self->_click(
            id                      => $PROJECT_INSIDE_EDIT,
            error_checking_needed   => 0,
        ) ;
    }else {
        $self->_click(
            id                      => $PROJECT_CFT_EDIT,
            error_checking_needed   => 0,
        ) ;
    }
    if($self->_is_element_present($EDIT_PLAN_CONFIG)){
        $self->_click(
            id                      => $EDIT_PLAN_CONFIG,
            error_checking_needed   => 1,
        ) ;
    } else {
        $self->_click(
            id                      => $CFT_EDIT_PLAN_CONFIG,
            error_checking_needed   => 1,
        ) ;
    }
    $self->_close_tab(
        string => "Migration",
    );
    _plan_configuration($self, $plan_configuration);
    $self->_create_screenshot($project."PlanConfig_Edit");
}

###################_edit_SVM_tab################################################
sub _edit_SVM_tab {
    my ($self, %args) = @_;
    my ($host_cmode,$host);
    my $ip      = $args{'c-mode-host-name'};
    my $vserver = $args{'svm-name'};
    my $project = $args{'project-name'};
    my $project_edit_config = $PROJECT_INSIDE_EDIT;
    $Log->enter() if $may_enter;
    $host_cmode = $self->_cbt_get_host_from_ip($ip,"c");
    if($host_cmode =~ /(.*)\($ip\)/){
        $host = $1;
    }
    $host =~ s/(.*)\s+/$1/;
    $host_cmode = $host."(".$ip.")";
    $self->_click(
        id                      => $project_edit_config,
        error_checking_needed   => 0,
    ) ;
    $self->_click(
        id                      => $EDIT_SVM,
        error_checking_needed   => 1,
    ) ;
    $self->_close_tab(
        string => "Migration",
    );
    my $selector = $VOLPAIR_CMODE_SYSTEM_SELECT;
    $selector =~ s/XXYYZZ/$host_cmode/;
    if( $self->_is_element_present($selector) ){
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 1,
        ) ;
    }
    $selector = $GENERIC_VSERVER_SELECTOR;
    $selector =~ s/(.*)XXYYZZ(.*)/$1$vserver$2/;

    if( $self->_is_element_present($selector) ) {
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 1,
        ) if(! $self->_is_checked($selector) );
    }else{
        $self->_create_screenshot("Vserver_edit_error");
        $self->_repair_session();
        $self->_throw_exception("Probably the vserver requested for does not exist"
            .".. Unable to find the element with ".$selector);
    }
    $self->_create_screenshot($project."Vserver_edit");
    $Log->exit() if $may_exit;
}

###############################################################################
## @name ::_navigate_plan_design
##
## @summary
##    Navigate to the Plan & Design page
## @synopsis
##    _navigate_plan_design();

sub _navigate_plan_design {
    my ($self,$type ) = @_;
    sleep 20;
    if($type=~ "CFT" ){
        $self->_click(
            id                        => $PLAN_DESIGN_CFT_RADIO_BUTTON,
            error_checking_needed    => 0,
        );
    }elsif($type =~ "CBT" ){
        $self->_click(
            id                        => $PLAN_DESIGN_CBT_RADIO_BUTTON,
            error_checking_needed    => 0,
        );
    }

    $self->_click(
        id                        => $START_PLAN,
        error_checking_needed    => 0,
    )
}

###############################################################################
## @name ::_edit_configuration
##
## @summary
## Edit configuration
## @synopsis
## _edit_configuration();

sub _edit_configuration {

    my ($self, $args) = @_;
    
	print ("Entered _edit_configuration subroutine\n");
    
	$self->_click(
		id                      => $MIGRATION_DASHBOARD_LINK,
		error_checking_needed   => 0,
	) ;
    
	$self->_project_selector($args->{"group-name"});
	
	my $project = $args->{"session-name"};
	my $locator = "gwt-debug-".$project;
	
	if ($self->_is_element_present($locator)) {
		$self->_click(
			id                      => $locator,
			error_checking_needed   => 0,
		) ;
	}

	$self->_click(
		id                      => $EDIT_CONFIGURATION,
		error_checking_needed   => 0,
	) ;
	
	$self->_click(
		id                      => $EDIT_PLAN_CONFIG,
		error_checking_needed   => 0,
	) ;
    
	my $plan_config = $args->{"plan_configuration"};
	_plan_configuration($self, $plan_config);
    
	return 1;
    
} #END sub _edit_configuration

###############################################################################
## @name ::_plan_configuration
## @summary
##    Select the configurations to be transitioned
## @synopsis
##    _plan_configuration();

sub _plan_configuration {
    my ($self, $plan_configuration) = @_;
    use Data::Dumper;
    if ($plan_configuration->{"ignore-all"}) {
        $self->_click(
           id                        => $DESELECT_ALL_CONF,
            error_checking_needed    => 1,
        );
    }elsif ($plan_configuration->{"select-all"}) {
        $self->_click(
            id                      => $SELECT_ALL_CONF,
            error_checking_needed   => 1,
        );
    }else {
        if ($plan_configuration->{"ignore-all-cifs-configurations-transition"}) {
                $self->_click(
                        id                      => $SVM_CONF_CIFS,
                        error_checking_needed   => 1,
                );
                $self->_click(
                        id                      => $VOLUME_CONF_CIFS,
                        error_checking_needed   => 1,
                );
        }else{
                if ($plan_configuration->{"ignore-local-users-groups-transition"}) {
                    $self->_click(
                        id                      => $CIFS_LOCAL_USERS_AND_GROUPS,
                        error_checking_needed   => 1,
                    );
                }
                if ($plan_configuration->{"ignore-cifs-home-directory-paths-transition"}) {
                    $self->_click(
                        id                      => $HOME_DIR_PATHS,
                        error_checking_needed   => 1,
                    );
                }
                if ($plan_configuration->{"ignore-cifs-widelinks-transition"}) {
                    $self->_click(
                        id                      => $WIDELINKS,
                        error_checking_needed   => 1,
                    );
                }
                if ($plan_configuration->{"ignore-cifs-shares-and-acls-transition"}) {
                    $self->_click(
                        id                      => $CIFS_SHARE_ACLS,
                        error_checking_needed   => 1,
                    );
                }
                if ($plan_configuration->{"ignore-cifs-options-transition"}) {
                    $self->_click(
                        id                      => $CIFS_OPTIONS,
                        error_checking_needed   => 1,
                    );
                }
                if ($plan_configuration->{"ignore-cifs-name-mapping-transition"}) {
                    $self->_click(
                        id                      => $CIFS_USER_MAPPING_CONF,
                        error_checking_needed   => 1,
                    );
                }
                if ($plan_configuration->{"ignore-cifs-audit-transition"}) {
                    $self->_click(
                        id                      => $AUDIT_CONFIGURATION,
                        error_checking_needed   => 1,
                    );
                }
                if ($plan_configuration->{"ignore-cifs-preferred-domain-controllers-list-transition"}) {
                    $self->_click(
                        id                      => $PREFERRED_DOMAIN_CONTROLLERS_LIST,
                        error_checking_needed   => 1,
                    );
                }
            }
        if ($plan_configuration->{"ignore-all-san-configurations-transition"}) {
            $self->_click(
                id                      => $VOLUME_CONF_SAN,
                error_checking_needed   => 1,
            );
        }else{
            if ($plan_configuration->{"ignore-igroup-transition"}) {
                $self->_click(
                        id                      => $VOLUME_CONF_SAN,
                        error_checking_needed   => 1,
                );
            }
            if ($plan_configuration->{"ignore-lunmapping-transition"}) {
                $self->_click(
                        id                      => $IGROUPS_LUN_MAPPING,
                        error_checking_needed   => 1,
                );
            }
        }

        if ($plan_configuration->{"ignore-all-nameservices-configurations-transition"}) {
                $self->_click(
                        id                      => $NAME_SERVICES,
                        error_checking_needed   => 1,
                );
        }else{
            if ($plan_configuration->{"ignore-netgroups-transition"}) {
                $self->_click(
                        id                      => $NETGROUPS,
                        error_checking_needed   => 1,
                );
            }
            if ($plan_configuration->{"ignore-unix-users-groups-transition"}) {
                $self->_click(
                        id                      => $UNIX_USERS_AND_GROUPS,
                        error_checking_needed   => 1,
                );
            }
            if ($plan_configuration->{"ignore-nis-transition"}) {
                $self->_click(
                        id                      => $NIS_CONFIGURATION,
                        error_checking_needed   => 1,
                );
            }
            if ($plan_configuration->{"ignore-dns-transition"}) {
                $self->_click(
                        id                      => $DNS_CONFIGURATION,
                        error_checking_needed   => 1,
                );
            }
            if ($plan_configuration->{"ignore-ldap-transition"}) {
                $self->_click(
                        id                      => $LDAP_CONFIGURATION,
                        error_checking_needed   => 1,
                );
            }
            if ($plan_configuration->{"ignore-nsswitch-transition"}) {
                $self->_click(
                        id                      => $NAME_SERVICE_SWITCH_CONF,
                        error_checking_needed   => 1,
                );
            }
            if ($plan_configuration->{"ignore-nmswitch-transition"}) {
                $self->_click(
                        id                      => $NAME_MAPPING_SWITCH_CONF,
                        error_checking_needed   => 1,
                );
            }
            if ($plan_configuration->{"ignore-etc-hosts-transition"}) {
                $self->_click(
                        id                      => $HOST_CONFIGURATION,
                        error_checking_needed   => 1,
                );
            }
        }

    if ($plan_configuration->{"ignore-all-nfs-configurations-transition"}) {
                $self->_click(
                        id                      => $SVM_CONF_NFS,
                        error_checking_needed   => 1,
                );
                $self->_click(
                        id                      => $VOLUME_CONF_NFS,
                        error_checking_needed   => 1,
                );
    }else{
            if ($plan_configuration->{"ignore-nfs-exports-transition"}) {
                $self->_click(
                        id                      => $NFS_EXPORTS,
                        error_checking_needed   => 1,
                );
            }
            if ($plan_configuration->{"ignore-nfs-options-transition"}) {
                $self->_click(
                        id                      => $NFS_OPTIONS,
                        error_checking_needed   => 1,
                );
            }
        }
       if ($plan_configuration->{"ignore-all-snapshot-schedule-transition"}) {
                $self->_click(
                        id                      => $SNAPSHOT_SCHEDULES_CONF,
                        error_checking_needed   => 1,
                );
        }else {
            if ($plan_configuration->{"ignore-snapshot-schedule-transition"}) {
                $self->_click(
                        id                      => $SNAPSHOT_SCHEDULES,
                        error_checking_needed   => 1,
                );
            }
            if ($plan_configuration->{"consolidate-similar-7mode-snapshot-policies"}) {
                $self->_click(
                        id                      => $CONSOLIDATE_7MODE_SNAP_POLICIES,
                        error_checking_needed   => 1,
                );
            }
            if ($plan_configuration->{"reuse-matching-svm-snapshot-policies"}) {
                $self->_click(
                        id                      => $REUSE_SNAPSHOT_POLICIES,
                        error_checking_needed   => 1,
                );
            }

        }
}
    $self->_click(
    id => $SAVE_CONF_CHANGES,
   error_checking_needed   => 1,
    ) ;

}

###############################################################################
## @name ::_cft_create_helper
##
## @summary
##     Returns the project object
## @synopsis
##     _cft_create_helper (%args);

sub _cft_create_helper {

    my ($self, $args) = @_;
    my %args=%{$args};

    $Log->enter() if $may_enter;
    my $S_mode_credentials      = delete $args{S_mode_credentials};
    my $S_mode_credentials_HA   = delete $args{S_mode_credentials_HA};
    my $C_mode_credentials      = delete $args{C_mode_credentials};
    my $cft_clusterinfo_add     = delete $args{cft_clusterinfo_add};
    my $cft_svm_mapping_add_pri = delete $args{cft_svm_mapping_add_pri};
    my $cft_svm_mapping_add_sec = delete $args{cft_svm_mapping_add_sec};
    my $cft_nodemapping_set     = delete $args{cft_nodemapping_set};
    my $plan_configuration      = delete $args{plan_configuration};
    
    _navigate_plan_design($self, "CFT");

    push @{$self->{$args{"project-name"}}} , { transition_cft_create_args => \%args} ;
    use Data::Dumper;
    $Log->comment( sub { "Input contains " . Dumper( \$self ) } );

    my ($popup_title,$host_cmode,$host3);
    #Handling CFT prerequisite page
    my $ret = _cft_prerquisite_handle($self);
    if(!$ret){
        $self->_create_screenshot("CFT_PRErequisite_page_Error");
        $self->_repair_session();
        $self->_throw_exception("Error in handling CFT Prerequisite page");
    }
    #Adding credentials
    $self->_add_credentials( %{$S_mode_credentials} ) if ( $S_mode_credentials ) ;
    $self->_add_credentials( %{$C_mode_credentials} ) if ( $C_mode_credentials ) ;
    $self->_add_credentials( %{$S_mode_credentials_HA} ) if ( $S_mode_credentials_HA ) ;

    $self->_create_screenshot("Storage_system");
    if ( $S_mode_credentials || $C_mode_credentials ) {
        # Lets go to Home Tab -> Storage systems and fetch the hostnames
         my $selector=$GENERIC_TAB;
        $selector =~ s/(.*)XXYYZZ(.*)/$1Hometab$2/;
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;

        # Closing the Untitled project
        $selector=$GENERIC_TAB_CLOSE;
        $selector =~ s/(.*)XXYYZZ(.*)/$1\(CFT\)$2/;
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;

    }
    #Get hostname from ip and store it in APISet object in session hash
    my $ip = $self->_get_value_for_key_x_key_y_in_session_z(
        "transition_cft_create_args","7-mode1-host-name",$args{"project-name"}
    );
    my $host_smode1 = $self->_get_host_from_ip($ip,"s");
    my $host1;
    if($host_smode1 =~ /(.*)\($ip\)/){
        $host1 = $1;
    }
    $host1 =~ s/(.*)\s+/$1/;
    $self->_put_value_for_key_x_key_y_in_session_z(
        "transition_cft_create_args",$ip,$args{"project-name"},$host1);
    $ip = $self->_get_value_for_key_x_key_y_in_session_z(
        "transition_cft_create_args","7-mode2-host-name",$args{"project-name"});
    my $host_smode2 = $self->_get_host_from_ip($ip,"s");
    my $host2;
    if($host_smode2 =~ /(.*)\($ip\)/){
        $host2 = $1;
    }
    $host2 =~ s/(.*)\s+/$1/;
    $self->_put_value_for_key_x_key_y_in_session_z(
        "transition_cft_create_args",$ip,$args{"project-name"},$host2);
    if ($cft_clusterinfo_add) {
        push @{$self->{$args{"project-name"}}} , { cft_clusterinfo_add_args => $cft_clusterinfo_add} ;
        use Data::Dumper;
        $Log->comment( sub { "Input contains after addition " . Dumper( \$self ) } );
        $ip = $self->_get_value_for_key_x_key_y_in_session_z(
            "cft_clusterinfo_add_args","c-mode-host-name",$args{"project-name"});
        $host_cmode = $self->_get_host_from_ip($ip,"c");
        if($host_cmode =~ /(.*)\($ip\)/){
            $host3 = $1;
        }
        $host3 =~ s/(.*)\s+/$1/;
        $self->_put_value_for_key_x_key_y_in_session_z(
            "transition_cft_create_args",$ip,$args{"project-name"},$host3
        );
    }
    if ($cft_svm_mapping_add_pri || $cft_svm_mapping_add_sec ) {
        push @{$self->{$args{"project-name"}}} , { cft_svm_mapping_add_pri => $cft_svm_mapping_add_pri} ;
        push @{$self->{$args{"project-name"}}} , { cft_svm_mapping_add_sec => $cft_svm_mapping_add_sec} ;
        $Log->comment( sub { "Input contains after cft_svm_mapping_add_pri " . Dumper( \$self ) } );
    }
    _navigate_plan_design($self, "CFT");
    #Handling CFT prerequisite page
    _cft_prerquisite_handle($self);
    $self->_click(
        id                      => $NEXT,
        error_checking_needed   => 0,
    ) ;
    my $var = 0;

    # @HA-pair selection
    while (1) {
        my $selector=$HAPAIRGRID_GROUPID;
        $selector =~ s/(.*)XXYYZZ(.*)/$1$var$2/;
        if ($self->_is_element_present($selector)) {
            if ($self->_is_element_present("//*[\@id='gwt-debug-haPairgrid-gp-groupid-$var-hd']/div[contains(text(),'$host1')]")) {
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                ) ;
                last;
            }
        }
        $var++;
    }

    $self->_click(
        id                      => $NEXT,
        error_checking_needed   => 0,
    ) ;

    #Selecting existing or creating new group
    $Log->comment("Adding Project name");
    $self->_type( $CFT_PROJECT_NAME ,$args{"project-name"});
    if ($args{"group-name"}) {
        my $group = $args{"group-name"};
        $self->_click(
            id                      => $SELECT_GROUP,
            error_checking_needed   => 0,
        ) ;
        my $existing_group = $EXISTING_GROUP;
        $existing_group =~ s/(.*)XXYYZZ(.*)/$1$group$2/;
        if ($self->_is_element_present($existing_group)) {
            $self->_click(
                id                      => $existing_group,
                error_checking_needed   => 0,
            ) ;
        }else{
            $self->_click(
                id                      => $CREATE_NEW_GROUP,
                error_checking_needed   => 0,
            ) ;
            $self->_type( $GROUP_NAME ,$group);
            $self->_create_screenshot("New_group_add");
        }
    }else{
        my $group = "Default";
        $self->_click(
            id                      => $SELECT_GROUP,
            error_checking_needed   => 0,
        ) ;
        my $existing_group = $EXISTING_GROUP;
        $existing_group =~ s/(.*)XXYYZZ(.*)/$1$group$2/;
        if ($self->_is_element_present($existing_group)) {
            $self->_click(
                id                      => $existing_group,
                error_checking_needed   => 0,
            ) ;
        }
    }
    $self->_create_screenshot($args{"project-name"}."_Project_created");
    $self->_click(
        id                      => $SAVE_PROJECT,
        error_checking_needed   => 0,
    ) ;

    #@cluster info add page
    if ($cft_clusterinfo_add || $cft_svm_mapping_add_pri || $cft_svm_mapping_add_pri) {
        push @{$self->{$args{"project-name"}}} , { cft_nodemapping_set_args => $cft_nodemapping_set} ;
        my $pri_clus_node = $PRIMARY_CLUS_NODE;
        my $pri_cmode = $self->_get_value_for_key_x_key_y_in_session_z(
            "cft_nodemapping_set_args","cluster-node1-name",$args{"project-name"});
        $pri_clus_node =~ s/(.*)XXYYZZ(.*)/$1$pri_cmode$2/;
        my $sec_clus_node = $SECONDARY_CLUS_NODE;
        my $sec_cmode = $self->_get_value_for_key_x_key_y_in_session_z(
            "cft_nodemapping_set_args","cluster-node2-name",$args{"project-name"});
        $sec_clus_node =~ s/(.*)XXYYZZ(.*)/$1$sec_cmode$2/;
        my $cft_cluster = $CFT_CLUSTER;
        $cft_cluster =~ s/(.*)XXYYZZ(.*)/$1$host3$2/;
        if($self->is_element_present($cft_cluster)){
            $self->_click(
                id                      => $cft_cluster,
                error_checking_needed   => 0,
            ) ;
            my $var = 0;
            #Iterating 12 times because a cluster can have 12 pairs of nodes
            while ($var < 12) {
                my $c_ha_selector = "//div[contains(text(),'$pri_cmode/$sec_cmode')]/parent::div/div/div/div";
                if ($self->_is_element_present($c_ha_selector)) {
                    $self->_click(
                        id                      => $c_ha_selector,
                        error_checking_needed   => 0,
                    ) ;
                    last;
                }
                $var++;
            }
            if ( $var == 12 ){
                $self->_create_screenshot("HApair_not_present");
                $self->_close_tab(
                    project => $args{"project-name"},
                );
                $self->_close_tab(
                    string => "Migration",
                );
                $self->_throw_exception("The Cluster $pri_cmode/$sec_cmode with is not present");
            }
            if ($cft_nodemapping_set) {
                if (($self->_is_element_present($pri_clus_node)) && ($self->_is_element_present($sec_clus_node))) {
                    print"the node mapping is already done\n";
                }else{
                    $self->_click(
                        id                      => $SWAP_BUTTON,
                        error_checking_needed   => 0,
                    ) ;
                }
            }
            $self->_create_screenshot($args{"project-name"}."_Target_Cluster_added");
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 0,
            ) ;
        }else{
            $self->_create_screenshot("Node_not_present");
            $self->_close_tab(
                project => $args{"project-name"},
            );
            $self->_close_tab(
                string => "Migration",
            );
            $self->_throw_exception("The Cluster  is not present");
        }
        #@svmmapping/node mapping page
        if ( $cft_svm_mapping_add_pri && $cft_svm_mapping_add_pri ) {
            $ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "cft_svm_mapping_add_pri","7-mode-host-name",$args{"project-name"});
            my $vfiler = $self->_get_value_for_key_x_key_y_in_session_z(
                "cft_svm_mapping_add_pri","vfiler-name",$args{"project-name"});
            my $vsm = $self->_get_value_for_key_x_key_y_in_session_z(
                "cft_svm_mapping_add_pri","svm-name",$args{"project-name"});

            $self->_select_vsm($ip,$vfiler,$vsm);

            $ip = $self->_get_value_for_key_x_key_y_in_session_z(
                "cft_svm_mapping_add_sec","7-mode-host-name",$args{"project-name"});
            $vfiler = $self->_get_value_for_key_x_key_y_in_session_z(
                "cft_svm_mapping_add_sec","vfiler-name",$args{"project-name"});
            $vsm = $self->_get_value_for_key_x_key_y_in_session_z(
                "cft_svm_mapping_add_sec","svm-name",$args{"project-name"});
            $self->_select_vsm($ip,$vfiler,$vsm);
            $self->_create_screenshot($args{"project-name"}."_SVM_mapping");
            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;

            $self->_click(
                id                      => $NEXT,
                error_checking_needed   => 1,
            ) ;
        }
        #Plan Configuration
        if(defined $plan_configuration) {
            foreach my $Vserver (keys($plan_configuration)){
                    my $var = 0;
                    print "Vserver : $Vserver \n";
                    # SVM selection
                    for($var=0; $var<128;$var++){
                        my $selector = $SVM_PLAN_CONFIG;
                        $selector =~ s/XX/$var/;
                        $selector =~ s/YY/$Vserver/;
                        print "Selector : $selector";
                        if ($self->_is_element_present($selector)) {
                            $self->_click(
                                id                      => $selector,
                                error_checking_needed   => 0,
                                ) ;
                            last;
                        }
                    }
                    _plan_configuration($self, $plan_configuration->{$Vserver});
            }
        }

        $self->_click(
            id                      => $NEXT,
            error_checking_needed   => 0,
        ) ;

        $self->_close_tab(
            string => "Migration",
        );
    }else{
        my $selector=$GENERIC_TAB_CLOSE;
        $selector =~ s/(.*)XXYYZZ(.*)/$1Untitled$2/;
        $self->_click(
            id                      => $selector,
            error_checking_needed   => 0,
        ) ;
    }
    $Log->exit() if $may_exit;
   return 0;
}

################################################################################
## @name ::_cft_prerquisite_handle
##
## @summary
##     Does the handling prerequisite page during CFT project creation
## @synopsis
##     _cft_prerquisite_handle();
sub _cft_prerquisite_handle {
    my $self = shift;
    my $popup_title;
    eval{
        $popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
    };
    if( $popup_title =~ /Prerequisite for Copy-Free Transition/i ){
        if ($self->_is_element_present($CFT_ASSESSMENT)) {
            $self->_click(
                id                      => $CFT_ASSESSMENT,
                error_checking_needed   => 0,
            ) ;
        }
        if ($self->_is_element_present($CFT_HOST_REMEDY)) {
            $self->_click(
                id                      => $CFT_HOST_REMEDY,
                error_checking_needed   => 0,
            ) ;
        }
        if ($self->_is_element_present($CFT_FC_SAN_NA)) {
            $self->_click(
                id                      => $CFT_FC_SAN_NA,
                error_checking_needed   => 0,
            ) ;
        }
        eval{
            $self->_click(
                id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                error_checking_needed   => 0,
            ) ;
        };
    } else {
        return 0;
    }
    return 1;
}
################################################################################
## @name ::_select_vsm
##
## @summary
##     Does the vfiler - VSM mapping
## @synopsis
##     _select_vsm(%args);

sub _select_vsm{

    my ( $self, $ip, $vfiler, $vsm ) = @_;

    $Log->enter() if $may_enter;

    # Sleeping here for sometime
    my $sleep_count = $GENERIC_WAIT / 1000;
    Tharn::snooze($sleep_count);
    Tharn::snooze($sleep_count);
    my $var = 0;
    while ( $var < 2 ) {
        my $vfiler_grid = $VFILER_GRID;
        $vfiler_grid =~ s/(.*)XXYYZZ(.*)/$1$var$2/;
        my $vfiler_grid_selector = $vfiler_grid."//div[contains(text(),'$ip') ]";
        if ($self->_is_element_present($vfiler_grid_selector)) {
            my $vfiler_row = 1;
            while ($vfiler_row < 63) {
                my $vfiler_selector = $VFILER_SELECTOR;
                $vfiler_selector =~ s/(.*)XXYYZZ(.*)/$1$vfiler_row$2/;
                $vfiler_selector = $vfiler_grid.$vfiler_selector."//div[contains(text(),'$vfiler') ]";
                if ($self->_is_element_present($vfiler_selector)){
                    my $vsm_drop_down = $VSM_DROP_DOWN;
                    $vsm_drop_down =~ s/(.*)XXYYZZ(.*)/$1$vfiler_row$2/;
                    $vsm_drop_down = $vfiler_grid.$vsm_drop_down;
                    $self->_click(
                        id                      => $vsm_drop_down,
                        error_checking_needed   => 0,
                    ) ;
                    my $vsm_selector = $GENERIC_VSM_SELECTOR;
                    $vsm_selector =~ s/(.*)XXYYZZ(.*)/$1$vsm$2/;
                    $self->_click(
                        id                      => $vsm_selector,
                        error_checking_needed   => 1,
                    ) ;
                    my $vsm_save = $VSM_MAPPING_SAVE;
                    $vsm_save =~ s/(.*)XXYYZZ(.*)/$1$vfiler_row$2/;
                    $vsm_save = $vfiler_grid.$vsm_save;
                    $self->_click(
                        id                      => $vsm_save,
                        error_checking_needed   => 1,
                    ) ;
                    last;
                }
                $vfiler_row++;
            }
            if ($vfiler_row == 63) {
                $self->_create_screenshot("Vfiler_not_present");
                $self->_close_tab(
                    string => "Migration",
                );
                $self->_throw_exception("The vfiler $vfiler doesnot exists in $ip node");
            }
            last;
        }
        $var++;
    }
    if ($var == 2) {
        $self->_create_screenshot("Node_not_present");
        $self->_close_tab(
            string => "Migration",
        );
        $self->_throw_exception("The Node $ip is not present");
    }
    $Log->exit() if $may_exit;

    return 1;
}

###############################################################################
## @name ::_create_screenshot
##
## @summary
##     Creates snapshot of the page
## @synopsis
##     _create_screenshot(%args);

sub _create_screenshot {
    my ( $self, $filename ) = @_;
    $Log->enter() if $may_enter;

    my $logdir = Tharn::param('LOGDIR');
    print"taking screenshot in $logdir\n";
    require MIME::Base64;
    my $file = $logdir."/".$filename.".png";
    print"taking screenshot file name $file\n";
    open(FH,'>',$file);
    binmode FH;
    my $png_base64 = $self->screenshot();
    print FH MIME::Base64::decode_base64($png_base64);
    close FH;

    $Log->exit() if $may_exit;
    return 1;

}

###############################################################################
## @name ::_perform_operation
##
## @summary
##     Returns the parsed ouput of the respective successful operation
## @synopsis
##     _perform_operation(%args);

sub _perform_operation{

    my ($self,$project,$selector,$group,$operation,$parsed_output,$run_background,$state,$arg) = @_;
    $Log->enter() if $may_enter;

    $self->_click(
        id                      => $MIGRATION_DASHBOARD_LINK,
        error_checking_needed   => 0,
    ) ;
    print "The run background value is $run_background";
    my $var=1;
    my $aref=[];
    my ($output,$undef_state_message);
    my $trans_tab = $arg->{'trans_mode'};
    $arg->{'operation'} = $operation;
    $self->_project_selector($group,$trans_tab);
    my $locator = "gwt-debug-".$project;
    if ($self->_is_element_present($locator)) {
        $self->_click(
            id                      => $locator,
            error_checking_needed   => 0,
        ) ;
        if ($selector eq $REVERT_PRECHECK) {
            if ($self->_is_element_present($REVERT_DASHBOARD)) {
                $self->_click(
                    id                      => $REVERT_DASHBOARD,
                    error_checking_needed   => 0,
                ) ;
            }
        }

        my %ip;
        if ($selector eq $GENERATE_MANUAL_STEPS) {
            if ($self->_is_element_present($PROJECT_DETAILS)) {
                $self->_click(
                    id                      => $PROJECT_DETAILS,
                    error_checking_needed   => 0,
                ) ;
            }
            my $host_c =$self->_get_text($DASHBOARD_CLUSTER_IP);
            print "the host cluster ip is $host_c\n";
            if ($host_c =~ /(.*)\s+[(](.*)[)]/) {
                $host_c = $2;
            }
            my $host_s =$self->_get_text($DASHBOARD_S_IP);
            print "the host cluster ip is $host_s\n";
            if ($host_s =~ /(.*)\s+[(](.*)[)]/) {
                $host_s = $2;
            }

            %ip = (
                "c_hostname"         => $host_c,
                "s_hostname"         => $host_s,
            );
            push @{$self->{$project}} , { transition_cft_generate_manual_step => \%ip} ;
            $Log->comment( sub { "Input contains " . Dumper( \$self ) } );
            my $s = $self->_get_value_for_key_x_key_y_in_session_z(
            "transition_cft_generate_manual_step","s_hostname",$project
                );
            my $c = $self->_get_value_for_key_x_key_y_in_session_z(
            "transition_cft_generate_manual_step","c_hostname",$project
            );
        }
        sleep(8);
        if( $self->_is_button_enabled($selector) ) {
            if(defined $arg->{mode}){
                if( $arg->{mode} eq "rw_test" ){
                    if($self->_is_button_enabled_cbt($TEST_MODE_ID)){
                        $self->_click(
                            id                      => $TEST_MODE,
                            error_checking_needed   => 0,
                        ) ;
                    } else {
                        $self->_create_screenshot($project."_Test_mode_checkbox_Error");
                        $self->_repair_session();
                        $self->_throw_exception("Test mode checkbox is not enabled");
                    }
                }
            }
            if ($parsed_output) {
                $aref = $self->_cft_click(
                    id                      => $selector,
                    error_checking_needed   => 1,
                    info_hash               => \%ip,
                    arguments               => $arg,
                ) ;
            }else{
                $self->_click(
                    id                      => $selector,
                    error_checking_needed   => 0,
                    arguments               => $arg,
                ) ;
                my $sleep_count = $GENERIC_WAIT / 1000;
                Tharn::snooze($sleep_count);
                if ($self->_is_element_present($GENERIC_PRECUTOVER)) { 
                        $self->_precutover_cutover_checks(arguments => $arg,);
                } else {
                    my $popup_title;
                    eval{
                        $popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
                    };

                    if($popup_title =~ /Confirmation/i) {
                        if ($self->_is_element_present($CFT_CHECKBOX_MESSAGE)) {
                            $self->_click(
                                id                      => $CFT_CHECKBOX_CONFIRM,
                                error_checking_needed   => 0,
                            ) ;
                        }
                        if ($self->_is_element_present($CFT_YES_BUTTON)) {
                            $self->_click(
                                id                      => $CFT_YES_BUTTON,
                                error_checking_needed   => 0,
                            ) ;
                        }
                        eval{
                            $popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
                        };
                    }
                    if ($popup_title =~ /Error/i) {
                        my $popup_text=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_MESSAGE);
                        $self->_create_screenshot("$popup_text");
                        $self->_repair_session();
                        $self->_throw_exception(
                            "Popup Title : $popup_title , Popup error message : $popup_text");
                    }elsif( ($popup_title !~ /Operation Status/i ) && ($popup_title =~ /Information/i || $popup_title =~ /Warning/i || $popup_title =~ /Transition Pause/i || $popup_title =~ /Transition Abort/i )){
                        my $popup_txt;
                        eval{
                            $popup_txt=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_MESSAGE);
                        };
                        # Handle all Generic pop-ups here
                        # We dont want to fail if any click fails
                        if ($self->_is_element_present($CFT_CHECKBOX_CONFIRM)) {
                            $self->_click(
                                id                      => $CFT_CHECKBOX_CONFIRM,
                                error_checking_needed   => 0,
                            ) ;
                        }
                        if ($self->_is_element_present($CFT_YES_BUTTON)) {
                            $self->_click(
                                id                      => $CFT_YES_BUTTON,
                                error_checking_needed   => 0,
                            ) ;
                        }
                        eval{
                            $self->_click(
                                id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                                error_checking_needed   => 0,
                            ) ;
                        };
                        if($popup_txt =~ /cutover/i){
                            $self->_precutover_cutover_checks(arguments => $arg,);
                        }
                    }
                }
                # Putting an eval, maybe popup was there but not present now
                my ($popup_title,$summary_text);
                eval{
                    $popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
                };
                # Since popup was present earlier but not present now so skipping checking
                unless ($@) {
                    if ($popup_title =~ /Refreshes/i || $popup_title =~ /Precheck Status/i
                        || $popup_title =~/Operation Status: Successful/i
                        || $popup_title =~/Operation Status: Failed/i) {
                        if ( $run_background == 1 ) {
                            eval {
                                $self->_click(
                                    id                      => $CFT_JOB_RESULT_CLOSE,
                                    error_checking_needed   => 0,
                                ) ;
                            };
                            
                            eval{
                                $popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
                            };
                            
                            
                            if ($popup_title =~ /Confirmation/i){
                                eval{
                                    $self->_click(
                                        id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                                        error_checking_needed   => 0,
                                    ) ;
                                };
                            }
                            
                        }else{
                            my $timer = $self->{timeout} || $GENERIC_PRECHECKS_WAIT;
                            my $new_popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
                            while ( ($new_popup_title =~ /Refreshes/) && ($timer > 0) ) {
                                # Waiting here for prechecks to get completed
                                print "Waiting in message\n";
                                $timer--;
                                my $sleep_count = $GENERIC_WAIT / 1000;
                                Tharn::snooze($sleep_count);
                                $new_popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
                                $summary_text = $self->_get_text($PRECHECK_SUMMARY_TEXT);
                                if ( ($new_popup_title =~ /Operation Status: Successful/i) || ($summary_text =~ /finished successfully/) ){

                                    if(!defined $state) { $undef_state_message = $new_popup_title };
                                        last;
                                }
                                if ( ($new_popup_title =~ /Operation Status: Failed/i) || ($summary_text =~ /has failed/) ){
                                    if(!defined $state) { $undef_state_message = $new_popup_title };
                                    last;
                                }
                            }

                            if($timer == 0){
                                eval {
                                    $self->_click(
                                        id                      => $CLOSE_BUTTON,
                                        error_checking_needed   => 0,
                                    ) ;
                                };

                                $self->_take_screenshot();
                                $self->_repair_session();
                                $self->_throw_exception(
                                    "Operation did not get completed even after 1800 secs");
                            }
                            $self->_create_screenshot($project."_".$operation."_job_result");
                            eval{
                                $self->_click(
                                    id                      => $CFT_JOB_RESULT_CLOSE,
                                    error_checking_needed   => 0,
                                ) ;
                            };
                            $self->_wait_for_loading();
                        }
                    }
                }
            }
            
            if (!defined $run_background) {
                if (defined $state) {
                    if(defined $operation) {
                        my $sleep_count = 20;
                        while(!($self->_is_element_present($state)) && $sleep_count > 0 ){
                            $self->_project_selector($group,$trans_tab);#refresh colour of round object for getting proper state of operation.
                            if ($self->_is_element_present($locator)) {
                                $self->_click(
                                    id                      => $locator,
                                    error_checking_needed   => 0,
                                ) ;
                            }
                            $sleep_count--;
                        }
                    }
                    if ($self->_is_element_present($state)) {
                        sleep(2);
                        $self->_create_screenshot($project."_".$operation."_success");
                    }else{
                        $self->_create_screenshot($project."_".$operation."_failed");
                        $self->_repair_session();
                        $self->_throw_exception($project."_"."$operation failed with an error");
                    }
                }
            }
            if(defined $undef_state_message) {
                if($undef_state_message =~ /Failed/i) {
                    $self->_repair_session();
                    $self->_throw_exception($project."_"."$operation failed with an error");
                }
            }
            if(($operation =~ /cutover/i) || ($operation =~ /commit/i) || 
               ($operation =~ /verify(.*)mode/i)){
                if ($self->_is_element_present($PROJECT_HISTORY_TAB)) {
                    $self->_click(
                        id                      => $PROJECT_HISTORY_TAB,
                        error_checking_needed   => 0,
                    ) ;
                    $self->_create_screenshot($project."_"."Transition_time");
                }
            }
            $self->_close_tab(
                string => "Migration",
            );
        }else{
            $self->_create_screenshot($operation."_disabled");
            $self->_close_tab(
                string => "Migration",
            );
            $self->_throw_exception("$operation button is disabled");
        }
    }else{
        $self->_create_screenshot("Project_does_not_exits");
        $self->_close_tab(
            string => "Migration",
        );
        $self->_throw_exception("The $project project is not present");
    }

    if (defined $aref->[0]) {
        $output->{parsed_output} = $aref->[0] ;
        $Log->exit() if $may_exit;
        return $output;
    }else {
        $Log->exit() if $may_exit;
        return undef;
    }

}

###############################################################################
## @name ::_project_selector
##
## @summary
##     selects projects under the given group
## @synopsis
##     _project_selector(%args);
sub _project_selector {
    my ($self,$group,$trans_tab) = @_;
    my $var = 0;
    while ($var ne '100') {
        my $group_path = "//div/div[$var]/div/span/b[contains(text(),'$group')]";
        if ($self->_is_element_present($group_path)) {
            $self->_click(
                id                      => $group_path,
                error_checking_needed   => 0,
            ) ;
            last;
        }
        $var++;
    }
    if(defined $trans_tab){
        if ($self->_is_element_present($trans_tab)) {
            $self->_click(
                id                      => $trans_tab,
                error_checking_needed   => 0,
            ) ;
        }
    }
    return 1;
}

###############################################################################
## @name ::_cbt_get_host_from_ip
##
## @summary
##     This method returns hostname from the given IP address
## @synopsis
##     _cbt_get_host_from_ip(%args);

sub _cbt_get_host_from_ip{

    my ( $self, $ip, $mode ) = @_;

    $Log->enter() if $may_enter;

    # Navigating to storage tab
     $self->_click(
        id                      => $STORAGE_SYSTEMS,
        error_checking_needed   => 0,
    ) ;

    my $selector;
    if ($mode =~ /c/){
        $selector=$GENERIC_CMODE_HOSTNAME;
    }elsif ($mode =~ /s/){
        $selector=$GENERIC_SMODE_HOSTNAME;
    }

    $selector =~ s/(.*)XXYYZZ(.*)/$1$ip$2/;

    my $host;
    if( $self->_is_element_present($selector) ){
        my $elem = $self->_find_element($selector);
        $host = $elem->get_text();
        $self->_close_tab(
            string => "Storage",
        );
    }else{
        $self->_create_screenshot("$ip not_found");
        $self->_repair_session();
        $self->_throw_exception("Did not get the hostname from the IP in the storage "
            ."systems.. Prabably the credentials are not added .. "
            ."Unable to find the element with ".$selector);
    }

    $Log->exit() if $may_exit;

    return $host;
} ## end sub _cbt_get_host_from_ip

###############################################################################
## @name ::_cft_click
##
## @summary
##     This method is used to click the element specified
## @synopsis
##     _cft_click(%args);

sub _cft_click {

    my ( $self, %args ) = @_;

    $Log->enter() if $may_enter;

    my $aref;
    if ( ref($self) =~ /WebElement/ ) {
        $self->click();
        $self->_wait_for_loading();
        $aref = $self->_cft_check_for_error( args => $args{info_hash},
            arguments => $args{arguments} ) if ( $args{error_checking_needed}
        );
        $Log->exit() if $may_exit;
        return $aref;
    }

    $Log->comment("Click Element : $args{id}");
    my $elem = $self->_find_element($args{id});
    $elem->click();
    $self->_wait_for_loading();
    $aref = $self->_cft_check_for_error( info_hash => $args{info_hash},
        arguments => $args{arguments} ) if ( $args{error_checking_needed}
    );

    $Log->comment("Click on Element $args{id} successfull");

    $Log->exit() if $may_exit;

    return $aref;
} ## end sub _cft_click

###############################################################################
## @name ::_cft_check_for_error
##
## @summary
##     This method is used to check the error and parse the precheck messages
## @synopsis
##     _cft_check_for_error(%args);

sub _cft_check_for_error{

    my ( $self, %args ) = @_;
    $Log->enter() if $may_enter;

    # DS to store the parsed output
    my $aref=[];

    $self->set_implicit_wait_timeout(4000);

    my $sleep_count = $GENERIC_WAIT / 1000;
    Tharn::snooze($sleep_count);

    my $popup_title;
    if ($self->_is_element_present($GENERIC_PRECUTOVER)) { 
        $self->_precutover_cutover_checks(arguments => $args{'arguments'},);
    } else {
        eval{
            $popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
        };
        if($popup_title =~ /Confirmation/i) {
            if ($self->_is_element_present($CFT_CHECKBOX_MESSAGE)) {
                $self->_click(
                    id                      => $CFT_CHECKBOX_CONFIRM,
                    error_checking_needed   => 0,
                ) ;
            }
            if ($self->_is_element_present($CFT_YES_BUTTON)) {
                $self->_click(
                    id                      => $CFT_YES_BUTTON,
                    error_checking_needed   => 0,
                ) ;
            } else {
                eval{
                    $self->_click(
                        id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                        error_checking_needed   => 0,
                    ) ;
                };
            }
            sleep(3);
            eval{
                $popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);

            };
        }

        if ($popup_title =~ /Error/i) {
            my $popup_text=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_MESSAGE);
            $self->_create_screenshot("$popup_text");
            $self->_repair_session();
            $self->_throw_exception(
            "Popup Title : $popup_title , Popup error message : $popup_text");
        }elsif ( ($popup_title !~ /Operation Status:/i ) && ($popup_title =~ /Information/i || $popup_title =~ /Confirmation/i || $popup_title =~ /Warning/i || $popup_title =~ /Transition Pause/i || $popup_title =~ /Transition Abort/i )) {
            my $popup_txt;
            eval{
                $popup_txt=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_MESSAGE);

            };
        # Handle all Generic pop-ups here
        # We dont want to fail if any click fails
            if ($self->_is_element_present($CFT_CHECKBOX_CONFIRM)) {
                $self->_click(
                    id                      => $CFT_CHECKBOX_CONFIRM,
                    error_checking_needed   => 0,
                ) ;
            }
            eval{
                if($popup_txt =~ /cutover/i){
                    if ($self->_is_element_present($CFT_YES_BUTTON)) {
                        $self->_click(
                            id                      => $CFT_YES_BUTTON,
                            error_checking_needed   => 0,
                        ) ;
                    }
                    $self->_click(
                        id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                        error_checking_needed   => 0,
                    ) ;
                    $self->_precutover_cutover_checks(arguments => $args{'arguments'},);
                } else {
                    $self->_click(
                        id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                        error_checking_needed   => 0,
                    ) ;
                }
            };
        }
    }
    # Putting an eval, maybe popup was there but not present now
    eval{
        $popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
    };
    if ($popup_title =~ /Operation Progress/i || $popup_title =~ /Precheck Status/i || $popup_title =~ /Running/i
        ||  $popup_title =~/Operation Status: Successful/i  ||  $popup_title =~/Operation Status: Failed/i ) {

        my $timer = $self->{timeout} || $GENERIC_PRECHECKS_WAIT;
        my $new_popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
        my $trans_oper = $args{arguments}->{'operation'};
        while ( ($new_popup_title =~ /Running/) && ($timer > 0) ) {
            print "The $trans_oper operation is still in progress\n";
                    # Waiting here for prechecks to get completed
            $timer--;
            my $sleep_count = $GENERIC_WAIT / 1000;
            Tharn::snooze($sleep_count);
            $new_popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
            my $summary_text = $self->_get_text($PRECHECK_SUMMARY_TEXT);
            if ( ($new_popup_title =~ /Operation Status: Successful/i) && ($summary_text =~ /finished successfully/) ){

                last;
            }
            if ( ($new_popup_title =~ /Operation Status: Failed/i) && ($summary_text =~ /has failed/) ){
                last;
            }
        }

        if($timer == 0){
            eval {
                $self->_click(
                    id                      => $CLOSE_BUTTON,
                    error_checking_needed   => 0,
                ) ;
            };

            $self->_take_screenshot();
            $self->_repair_session();
            $self->_throw_exception(
                "Operation did not get completed even after 1800 secs");
        }
        my $summary_text = $self->_get_text($PRECHECK_SUMMARY_TEXT);
        my $proj_name  = $args{arguments}->{'session-name'};
        if(!defined $proj_name) {$proj_name = $args{arguments}->{'project-name'};}
        my $range = 100;
        my $random_number = int(rand($range));
        $self->_create_screenshot($proj_name."_job_results_".$random_number.$trans_oper);
        print "the operation is completed\n";
        # Getting job results data in a DS
        my $href={};
        $href->{Informational}=[];
        $href->{Error}=[];
        $href->{Warning}=[];

        $new_popup_title=$self->_get_text($GENERIC_POPUP.$GENERIC_POPUP_TITLE);
        if ( ($new_popup_title =~ /Operation: Generate Rollback Manual Steps/i) && ($summary_text =~ /finished successfully/) ){
            my $stream_row_count=0;
            while (1) {
                my ( $type, %chunk_hash );
                my $stream_row = $STREAM_ROW;
                $stream_row =~ s/(.*)XXYYZZ(.*)/$1$stream_row_count$2/;
                my $row_element = $ROW_ELEMENT;
                $row_element =~ s/(.*)XXYYZZ(.*)/$1$stream_row_count$2/;
                print "The stream row is $stream_row\n";
                my $msg_count = $MSG_COUNT;
                $msg_count =~ s/(.*)XXYYZZ(.*)/$1$stream_row_count$2/;
                if ($self->_is_element_present($msg_count)) {
                    $msg_count = $self->_get_text($msg_count);
                    print "The message count is $msg_count\n";
                }
                my $value=1;
                if ($self->_is_element_present($stream_row)) {
                    if ($msg_count !~ m/Errors: 0 Warnings: 0 Info: 0/) {
                        my $row = 2;
                        while (1) {
                            my %chunk_hash;
                            print "Reading the values\n";
                            my $msg = $stream_row."//div[2]/div[".$row."]";
                            print "the row value is $msg\n";
                            if ($self->_is_element_present($msg)) {
                                print "this is to check the $msg\n";
                                foreach my $col (1..3){
                                    my $job_results = $stream_row.$PRECHECK_MSG;
                                    $job_results =~ s/(.*)ROW_NUMBER(.*)COLUMN_NUMBER(.*)/$1$row$2$col$3/;
                                    if ($col == 1) {
                                        if ($self->_is_element_present($job_results)) {
                                            print "I'm in message\n";
                                            my $text = $self->_get_text($job_results);
                                            $chunk_hash{"type"} = $text;
                                        }else{
                                            my $step = $stream_row.$REVERT_ROW;
                                            $step =~ s/(.*)ROW_NUMBER(.*)COLUMN_NUMBER(.*)/$1$row$2$col$3/;
                                            print "I'm in step\n";
                                            $chunk_hash{"step"} = "Step ".$value ;
                                            $value++;
                                        }
                                    }elsif($col == 2){
                                        my $text = $self->_get_text($job_results);
                                        my @arr = split(" : ",$text);
                                        if ($arr[0]=~ /(\d\d\d\d+)/) {
                                            $chunk_hash{"code"} = $arr[0];
                                            $chunk_hash{"message"} = $arr[1];
                                        }
                                    }else{
                                        my $affected_obj = $stream_row.$REVERT_ROW;
                                        $affected_obj =~ s/(.*)ROW_NUMBER(.*)COLUMN_NUMBER(.*)/$1$row$2$col$3/;
                                        $affected_obj = $affected_obj."/a[\@id='gwt-debug-affectedobjects']";
                                        if($self->_is_element_present($affected_obj)){
                                            $self->_click(
                                                id   => $affected_obj,
                                                error_checking_needed   => 0,
                                            ) ;
                                            if ($self->_is_element_present($CLUSTERSHELL)) {
                                                my $cmode_ip = $args{"info_hash"}{"c_hostname"};
                                                my $var = 1;
                                                while ( 1 ) {
                                                    my $msg = $REVERT_COMMAND;
                                                    $msg =~ s/(.*)XXYYZZ(.*)/$1$var$2/;
                                                    if ($self->_is_element_present($msg)) {
                                                        $msg = $self->_get_text($msg);
                                                        push(@{$chunk_hash{$cmode_ip}},$msg);
                                                        $var++;
                                                    }else{
                                                        last;
                                                    }
                                                }
                                            }elsif($self->_is_element_present($S_MODE_COMMAND)){
                                                if($self->_is_element_present($SEVEN_MODE_CONTROLLER)){
                                                    my $var = 1;
                                                    while ( 1 ) {
                                                        my $msg = $SEVEN_MODE_COMMAND;
                                                        $msg =~ s/(.*)XXYYZZ(.*)/$1$var$2/;
                                                        if ($self->_is_element_present($msg)) {
                                                            $msg = $self->_get_text($msg);
                                                            my $smode_ip = $SEVEN_MODE_IP;
                                                            $smode_ip =~ s/(.*)XXYYZZ(.*)/$1$var$2/;
                                                            $smode_ip = $self->_get_text($smode_ip);
                                                            if ($smode_ip =~ /(.*)[(](.*)[)]/){
                                                                $smode_ip = $2;
                                                            }
                                                            push(@{$chunk_hash{$smode_ip}},$msg);
                                                            $var++;
                                                        }else{
                                                            last;
                                                        }
                                                    }
                                                }else{
                                                    my $smode_ip = $args{"info_hash"}{"s_hostname"};
                                                    my $var = 1;
                                                    while ( 1 ) {
                                                        my $msg = $REVERT_COMMAND;
                                                        $msg =~ s/(.*)XXYYZZ(.*)/$1$var$2/;
                                                        if ($self->_is_element_present($msg)) {
                                                            $msg = $self->_get_text($msg);
                                                            push(@{$chunk_hash{$smode_ip}},$msg);
                                                            $var++;
                                                        }else{
                                                            last;
                                                        }
                                                    }
                                                }
                                            }
                                            $self->_click(
                                                id                      => $CFT_REVERT_STEPS_CLOSE,
                                                error_checking_needed   => 0,
                                            ) ;
                                        }
                                    }
                                }
                            }else{
                                last;
                            }
                            $row++;
                            use Data::Dumper;
                            $Log->comment( sub { "The hash is  " . Dumper( \%chunk_hash ) } );

                            my @keys = keys %chunk_hash;
                            my @found;
                            if ( @found = grep { $_ eq "step" } @keys ){
                                push @{$href->{"Step"}} , {%chunk_hash};
                            }else{
                                push @{$href->{$chunk_hash{"type"}}} , {%chunk_hash};
                            }
                        }
                    }
                }else{
                    last;
                }
                $stream_row_count++;
                push @$aref , $href;
            }
        }

        elsif ( ($summary_text =~ /finished successfully/) || ($summary_text =~ /has failed/) ){
            my @keys = ("code","type","message","corrective_action","affected_objects");
            my $stream_row_count=0;
            while (1) {
                my ( $type, %chunk_hash );
                my $stream_row = $STREAM_ROW;
                $stream_row =~ s/(.*)XXYYZZ(.*)/$1$stream_row_count$2/;
                my $row_element = $ROW_ELEMENT;
                $row_element =~ s/(.*)XXYYZZ(.*)/$1$stream_row_count$2/;
                print "The stream row is $stream_row\n";
                my $msg_count = $MSG_COUNT;
                $msg_count =~ s/(.*)XXYYZZ(.*)/$1$stream_row_count$2/;
                if ($self->_is_element_present($msg_count)) {
                    $msg_count = $self->_get_text($msg_count);
                    print "The message count is $msg_count\n";
                }
                if ( $self->_is_element_present($stream_row)) {
                    if ($msg_count !~ m/Errors: 0 Warnings: 0 Info: 0/) {
                        if ($msg_count !~ m/Errors: 0/) {
                            $row_element = $ERROR_ROW_ELEMENT;
                            $row_element =~ s/(.*)XXYYZZ(.*)/$1$stream_row_count$2/;
                            $self->_click(
                                id                      => $row_element,
                                error_checking_needed   => 0,
                            ) ;
                        }else{
                            $self->_click(
                                id                      => $row_element,
                                error_checking_needed   => 0,
                            ) ;
                        }
                        my $row = 2;
                        while (1) {
                            my $msg = $stream_row."//div[2]/div[".$row."]";
                            print "the row value is $msg\n";
                            my $value=0;
                            if ($self->_is_element_present($msg)) {
                                foreach my $col (1..3){
                                    my $job_results = $stream_row.$PRECHECK_MSG;
                                    $job_results =~ s/(.*)ROW_NUMBER(.*)COLUMN_NUMBER(.*)/$1$row$2$col$3/;
                                   my $text = $self->_get_text($job_results);
                                    my @arr = split("\n",$text);
                                    if ($arr[1]=~ /\((\d\d\d\d+)\)/) {
                                        $chunk_hash{$keys[$value]} = $1;
                                        $value++;
                                        $chunk_hash{$keys[$value]} = $arr[0];
                                        $type = $arr[0];
                                    }else{
                                        $chunk_hash{$keys[$value]} = $text;
                                    }
                                    $value++;
                                }
                                push @{$href->{$type}} , {%chunk_hash};
                            }else{
                                last;
                            }
                            $row++;
                        }
                        $self->_click(
                            id                      => $row_element,
                            error_checking_needed   => 0,
                        ) ;
                    }
                }else{
                    last;
                }
                $stream_row_count++;
            }
            push @$aref , $href;

        }

        eval{
            $self->_click(
                id                      => $CFT_JOB_RESULT_CLOSE,
                error_checking_needed   => 0,
            ) ;
        };
    } else {
                # Closing the pop-up if none of the above conditions meet
                # We dont want to fail in case if the pop-up can't be closed
        eval {
            $self->_click(
                id                      => $GENERIC_POPUP.$GENERIC_POPUP_CLOSE,
                error_checking_needed   => 0,
            ) ;
        };
    }

    $self->set_implicit_wait_timeout(1000);

    $self->set_implicit_wait_timeout(5000);

    $Log->exit() if $may_exit;
    return $aref;
} ## end sub _cft_check_for_error

1;
