# $Id$ //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/iomem.pm#8
#
#Copyright (c) 2011-2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse iomem command
##
## @description
##    This is a library to parse the raw output of the iomem command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##    For Example, the parser function for "iomem nvds get" will be iomem_nvds_get
##
## @synopsis
##      use iomem;
##      iomem_address_show(%args);
##      iomem_address_set(%args);
##      iomem_address_get(%args);
##      iomem_buffer_show(%args);
##      iomem_buffer_read(%args);
##      iomem_buffer_fill(%args);
##      iomem_buffer_set(%args);
##      iomem_buffer_copy(%args);
##      iomem_buffer_compare(%args);
##      iomem_system_current_get(%args);
##      iomem_system_current_set(%args);
##      iomem_system_channel(%args);
##      iomem_nvds_get(%args);
##      iomem_nvds_clear(%args);
##      iomem_nvds_increment(%args);
##      iomem_led_show(%args);
##      iomem_led_get(%args);
##      iomem_nor_read(%args);
##      iomem_nor_erase(%args);
##      iomem_nor_program(%args);
##      iomem_bbm_get_bb(%args);
##      iomem_bbm_card_bb(%args);
##      iomem_bbm_check(%args);
##      iomem_bbm_set(%args);
##      iomem_bbm_empty(%args);
##      iomem_bbm_revert(%args);
##      iomem_power_set(%args);
##      iomem_power_show(%args);
##      iomem_training_show(%args);
##      iomem_training_force(%args);
##      iomem_training_auto(%args);
##      iomem_stats_show(%args);
##      iomem_stats_zero(%args);
##      iomem_register_show(%args)
##      iomem_register_set(%args)
##      iomem_register_get(%args)
##      iomem_config_show(%args);
##      iomem_config_enable(%args);
##      iomem_config_disable(%args);
##      iomem_config_state(%args);
##      iomem_vmargin_set(%args);
##      iomem_vmargin_show(%args);
##      iomem_temp_show(%args);
##      iomem_fru_show(%args);
##      iomem_fru_set(%args);
##      iomem_fpga_show(%args);
##      iomem_fpga_reload(%args);
##      iomem_fpga_update(%args);
##      iomem_fw_show(%args);
##      iomem_fw_reload(%args);
##      iomem_fw_update(%args);
##      iomem_fw_jam(%args);
##      iomem_ecc_show(%args);
##      iomem_ecc_detect(%args);
##      iomem_ecc_correct(%args);
##      iomem_ecc_inject(%args);
##      iomem_dma_verify(%args);
##      iomem_pio_fill(%args);
##      iomem_bbm_mode(%args);
##      iomem_admin_timer(%args);
##      iomem_pio_read(%args);
##      iomem_pio_write(%args);
##      iomem_pio_erase(%args);
##      iomem_zodiac_read(%args);
##      iomem_zodiac_write(%args);
##      iomem_zodiac_erase(%args);
##      iomem_bbm_bbs(%args);
##      iomem_address_stripe_map(%args);
##
## @author  dl-nacl-dev@netapp.com,yadag@netapp.com,rroshan@netapp.com,
##          udaykk@netapp.com,skotagir@netapp.com,kathar@netapp.com
##
## @status public
#
################################################################################
package NACL::APISet::Node::CLI::7Mode::default::iomem;
###################
# Everytime
###################
use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
# Include exceptions which will be thrown

use NATE::BaseException qw(:try);
use NACL::APISet::Exceptions::UnexpectedOutputException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use Data::Dumper;
use Tharn;
## @name ::iomem_address_show
##
## @summary Parses the raw output from the "iomem address show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_address_show with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_address_show(%args);
## INPUTS TO METHOD CALL 'iomem_address_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem address show; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'number_of_addresses' => '2',
##&nbsp;            'number_of_channels' => '96'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_address_show {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $value );
    my $i = 0;
    foreach $value (@lines) {
        if ( $value =~ /^(.*):\s+(.*)$/ ) {
            $Res[$i]->{ to_key_name( item => $1 ) } = $2;
        }
    }
    return \@Res;
} ## end sub iomem_address_show
## @name ::iomem_address_set
##
## @summary Parses the raw output from the "iomem address set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_address_set with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_address_set(%args);
## INPUTS TO METHOD CALL 'iomem_address_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-P' </td> <td> 'increment' </td> <td> '1' </td> </tr>
## <tr> <td> '-F' </td> <td> 'first-erase-unit' </td> <td> '1' </td> </tr>
## <tr> <td> '-I' </td> <td> 'index' </td> <td> '1' </td> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'patt-or-value' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'field' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-cards' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> <td> '0' </td> </tr>
## <tr> <td> '-d' </td> <td> 'channel' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'system' </td> <td> '0' </td> </tr>
## <tr> <td> '-W' </td> <td> 'write-protection' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem address set -c 2 -f iface -v 1; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'address_field' => 'iface',
##&nbsp;            'status' => 'set_successfully'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_address_set {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $value );
    my $i = 0;
    foreach $value (@lines) {
        if ( $value =~ /^(Address\sfield)\s"(.*)"\s(set\ssuccessfully)$/ ) {
            $Res[$i]->{ to_key_name( item => $1 ) } = $2;
            $Res[$i]->{"status"} = to_key_name( item => $3 );
        }
    }
    return \@Res;
} ## end sub iomem_address_set
## @name ::iomem_address_get
##
## @summary Parses the raw output from the "iomem address get" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_address_get with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_address_get(%args);
## INPUTS TO METHOD CALL 'iomem_address_get' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-cards' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem address get; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'address_type' => 'initialized_unit'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_address_get {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $value );
    my $i = 0;
    foreach $value (@lines) {
        if ( $value =~ /^(.*):\s+(.*)$/ ) {
            $Res[$i]->{ to_key_name( item => $1 ) } = $2;
        }
    }
    return \@Res;
} ## end sub iomem_address_get
## @name ::iomem_buffer_show
##
## @summary Parses the raw output from the "iomem buffer show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_buffer_show with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_buffer_show(%args);
## INPUTS TO METHOD CALL 'iomem_buffer_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem buffer show -A; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot 6' => 'flash_cache',
##&nbsp;            'slot_6_info' => [
##&nbsp;                               {
##&nbsp;                                 'buffer_size' => '4096',
##&nbsp;                                 'number_of_buffers' => '8'
##&nbsp;                               }
##&nbsp;                             ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'slot 7' => 'flash_cache',
##&nbsp;            'slot_7_info' => [
##&nbsp;                               {
##&nbsp;                                 'buffer_size' => '4096',
##&nbsp;                                 'number_of_buffers' => '8'
##&nbsp;                               }
##&nbsp;                             ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_buffer_show {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $option = $command;
    $option =~ s/iomem buffer show?//;
    if ( $option =~ /-A/ ) {

        #Now we can parse the output
        my @lines = get_lines(
            "raw_output"      => $output,
            "response_object" => $resp
        );
        my ( @Res, $value, $slot, $slot_val );
        my $i = -1;
        foreach $value (@lines) {
            if ( $value =~ /^Slot\s+(\d+):\s+(.*)$/ ) {
                $i++;
                $slot     = $1;
                $slot_val = $2;
                $Res[$i]->{ to_key_name( item => "slot $slot" ) } = $slot_val;
                next;
            } elsif ( $value =~ /^(.*):\s+(.*)$/ ) {
                $Res[$i]->{ "slot_" . $slot . "_info" }->[0]
                    ->{ to_key_name( item => $1 ) } = $2;
            }
        }
        return \@Res;
    } elsif ( $option =~ /-c/ ) {
        my @lines = get_lines(
            "raw_output"      => $output,
            "response_object" => $resp
        );
        my ( @Res, $value );
        my $i = 0;
        my $j = 0;
        foreach $value (@lines) {
            if ( $value =~ /^(.*):\s+(.*)$/ ) {
                $Res[$i]->{ to_key_name( item => $1 ) } = $2;
            }
        }
        return \@Res;
    }
} ## end sub iomem_buffer_show
## @name ::iomem_buffer_read
##
## @summary Parses the raw output from the "iomem buffer read" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_buffer_read with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_buffer_read(%args);
## INPUTS TO METHOD CALL 'iomem_buffer_read' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-j' </td> <td> 'bytes-or-offset' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'size-or-offset' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'buffer' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem buffer read -c 2 -b 0 -n 64; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'info' => [
##&nbsp;                         {
##&nbsp;                           'data' => ' 01 00 00 00 6f 6e 66 69 00 50 87 cf 03 ff ff ff ',
##&nbsp;                           'address' => '00000000',
##&nbsp;                           'miscellaneous' => ' ....onfi.P......'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'data' => ' 38 50 87 cf 03 ff ff ff 00 00 00 00 00 00 00 00 ',
##&nbsp;                           'address' => '00000010',
##&nbsp;                           'miscellaneous' => ' 8P..............'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'data' => ' 00 40 e8 42 04 ff ff ff f8 91 cb 0f 01 ff ff ff ',
##&nbsp;                           'address' => '00000020',
##&nbsp;                           'miscellaneous' => ' .@.B............'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'data' => ' 00 60 87 cf 03 ff ff ff 00 00 00 00 00 00 00 00 ',
##&nbsp;                           'address' => '00000030',
##&nbsp;                           'miscellaneous' => ' .`..............'
##&nbsp;                         }
##&nbsp;                       ]
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
sub iomem_buffer_read {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $option = $command;
    $option =~ s/iomem buffer read?//;
    if ( $option =~ /-A/ ) {

        #Now we can parse the output
        my @lines = get_lines(
            "raw_output"      => $output,
            "response_object" => $resp
        );
        my ( @Res, $value, $slot, $slot_val );
        my $i = -1;
        my $j = 0;
        foreach $value (@lines) {
            if ( $value =~ /^Slot\s+(\d+):\s+(.*)$/ ) {
                $i++;
                $j        = -1;
                $slot     = $1;
                $slot_val = $2;
                next;
            } elsif ( $value =~ /^(\S+)\s+\|(.*)\|(.*)$/ ) {
                $j++;
                $Res[$i]->{"info"}->[$j]->{ to_key_name( item => "address" ) }
                    = $1;
                $Res[$i]->{"info"}->[$j]->{ to_key_name( item => "data" ) }
                    = $2;
                $Res[$i]->{"info"}->[$j]
                    ->{ to_key_name( item => "miscellaneous" ) } = $3;
                $Res[$i]->{ to_key_name( item => "slot $slot" ) } = $slot_val;
            }
        }
        return \@Res;
    } elsif ( ( $option =~ /-c/ )
        || ( $option =~ /-n/ )
        || ( $option =~ /-b/ ) ) {
        my @lines = get_lines(
            "raw_output"      => $output,
            "response_object" => $resp
        );
        my ( @Res, $value );
        my $i = 0;
        my $j = 0;
        foreach $value (@lines) {
            if ( $value =~ /^(\S+)\s+\|(.*)\|(.*)$/ ) {
                $Res[$i]->{"info"}->[$j]->{ to_key_name( item => "address" ) }
                    = $1;
                $Res[$i]->{"info"}->[$j]->{ to_key_name( item => "data" ) }
                    = $2;
                $Res[$i]->{"info"}->[$j]
                    ->{ to_key_name( item => "miscellaneous" ) } = $3;
                $j++;
            }
        }
        return \@Res;
    }
} ## end sub iomem_buffer_read
## @name ::iomem_buffer_fill
##
## @summary Parses the raw output from the "iomem buffer fill" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_buffer_fill with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_buffer_fill(%args);
## INPUTS TO METHOD CALL 'iomem_buffer_fill' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-n' </td> <td> 'size-or-offset' </td> </tr>
## <tr> <td> '-v' </td> <td> 'patt-or-value' </td> </tr>
## <tr> <td> '-b' </td> <td> 'buffer' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem buffer fill -c 2 -b 7 -v 0xA5A5A5A55A5A5A5A; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'filled_with_pattern' => '0xa5a5a5a55a5a5a5a',
##&nbsp;            'buffer' => '7'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; iomem buffer fill -A -b 0 -v 00; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot 6' => 'flash_cache',
##&nbsp;            'slot_6_info' => [
##&nbsp;                               {
##&nbsp;                                 'filled_with_pattern' => '0x0',
##&nbsp;                                 'buffer' => '0'
##&nbsp;                               }
##&nbsp;                             ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'slot 7' => 'flash_cache',
##&nbsp;            'slot_7_info' => [
##&nbsp;                               {
##&nbsp;                                 'filled_with_pattern' => '0x0',
##&nbsp;                                 'buffer' => '0'
##&nbsp;                               }
##&nbsp;                             ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_buffer_fill {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $option = $command;
    $option =~ s/iomem buffer fill?//;
    if ( $option =~ /-A/ ) {

        #Now we can parse the output
        my @lines = get_lines(
            "raw_output"      => $output,
            "response_object" => $resp
        );
        my ( @Res, $value, $slot, $slot_val );
        my $i = -1;
        foreach $value (@lines) {
            if ( $value =~ /^Slot\s+(\d+):\s+(.*)$/ ) {
                $i++;
                $slot     = $1;
                $slot_val = $2;
                $Res[$i]->{ to_key_name( item => "slot $slot" ) } = $slot_val;
                next;
            } elsif (
                $value =~ /^(Buffer)\s(.*)\s(filled\swith\spattern)\s(.*).$/ )
            {
                $Res[$i]->{ "slot_" . $slot . "_info" }->[0]
                    ->{ to_key_name( item => $1 ) } = $2;
                $Res[$i]->{ "slot_" . $slot . "_info" }->[0]
                    ->{ to_key_name( item => $3 ) } = $4;
            }
        }
        return \@Res;
    } elsif ( ( $option =~ /-c/ ) ) {
        my @lines = get_lines(
            "raw_output"      => $output,
            "response_object" => $resp
        );
        my ( @Res, $value );
        my $i = 0;
        foreach $value (@lines) {
            if ($value =~ /^(Buffer)\s(.*)\s(filled\swith\spattern)\s(.*).$/ )
            {
                $Res[$i]->{ to_key_name( item => $1 ) } = $2;
                $Res[$i]->{ to_key_name( item => $3 ) } = $4;
            }
        }
        return \@Res;
    }
} ## end sub iomem_buffer_fill
## @name ::iomem_buffer_set
##
## @summary Parses the raw output from the "iomem buffer set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_buffer_set with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_buffer_set(%args);
## INPUTS TO METHOD CALL 'iomem_buffer_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-n' </td> <td> 'size-or-offset' </td> </tr>
## <tr> <td> '-v' </td> <td> 'patt-or-value' </td> </tr>
## <tr> <td> '-b' </td> <td> 'buffer' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem buffer set -c 2 -b 2 -n 64 -v 0X5; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = priv set -q diag; iomem buffer set -A; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_6' => 'flash_cache'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'slot_7' => 'flash_cache'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_buffer_set {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $value );
    my $i = 0;
    foreach $value (@lines) {
        if ( $value =~ /^(.*):\s+(.*)$/ ) {
            $Res[$i]->{ to_key_name( item => $1 ) } = $2;
            $i++;
        }
    }
    return \@Res;
} ## end sub iomem_buffer_set
## @name ::iomem_buffer_copy
##
## @summary Parses the raw output from the "iomem buffer copy" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_buffer_copy with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_buffer_copy(%args);
## INPUTS TO METHOD CALL 'iomem_buffer_copy' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'buffer' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'destination-buffer' </td> <td> '0' </td> </tr>
## <tr> <td> '-o' </td> <td> 'other' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem buffer copy -c 2 -b 7 -o 2; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'buffer 7' => 'copied_to_buffer 2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_buffer_copy {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $option = $command;
    $option =~ s/iomem buffer copy?//;
    if ( $option =~ /-A/ ) {

        #Now we can parse the output
        my @lines = get_lines(
            "raw_output"      => $output,
            "response_object" => $resp
        );
        my ( @Res, $value, $slot, $slot_val );
        my $i = -1;
        foreach $value (@lines) {
            if ( $value =~ /^Slot\s+(\d+):\s+(.*)$/ ) {
                $i++;
                $slot     = $1;
                $slot_val = $2;
                $Res[$i]->{ to_key_name( item => "slot $slot" ) } = $slot_val;
                next;
            } elsif ( $value =~ /^Buffer\s(.*)\scopied\sto\sbuffer\s(.*).$/ )
            {
                $Res[$i]->{ "slot_" . $slot . "_info" }->[0]
                    ->{ "buffer_" . to_key_name( item => $1 ) }
                    = "copied_to_buffer " . $2;
            }
        }
        return \@Res;
    } elsif ( ( $option =~ /-c/ ) ) {
        my @lines = get_lines(
            "raw_output"      => $output,
            "response_object" => $resp
        );
        my ( @Res, $value );
        my $i = 0;
        foreach $value (@lines) {
            if ( $value =~ /^Buffer\s(.*)\scopied\sto\sbuffer\s(.*).$/ ) {
                $Res[$i]->{ "buffer_" . to_key_name( item => $1 ) }
                    = "copied_to_buffer " . $2;
            }
        }
        return \@Res;
    }
} ## end sub iomem_buffer_copy
## @name ::iomem_buffer_compare
##
## @summary Parses the raw output from the "iomem buffer compare" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_buffer_compare with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_buffer_compare(%args);
## INPUTS TO METHOD CALL 'iomem_buffer_compare' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-o' </td> <td> 'other' </td> </tr>
## <tr> <td> '-b' </td> <td> 'buffer' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## <tr> <td> '-v' </td> <td> 'patt-or-value' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem buffer compare -c 2 -b 7 -o 2; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'buffers' => 'identical'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_buffer_compare {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $option = $command;
    $option =~ s/iomem buffer compare?//;
    if ( $option =~ /-A/ ) {

        #Now we can parse the output
        my @lines = get_lines(
            "raw_output"      => $output,
            "response_object" => $resp
        );
        my ( @Res, $value, $slot, $slot_val );
        my $i = -1;
        foreach $value (@lines) {
            if ( $value =~ /^Slot\s+(\d+):\s+(.*)$/ ) {
                $i++;
                $slot     = $1;
                $slot_val = $2;
                $Res[$i]->{ to_key_name( item => "slot $slot" ) } = $slot_val;
                next;
            } elsif ( $value =~ /^(Buffers)\sare\s(identical).$/ ) {
                $Res[$i]->{ "slot_" . $slot . "_info" }->[0]
                    ->{ to_key_name( item => $1 ) } = $2;
            }
        }
        return \@Res;
    } elsif ( ( $option =~ /-c/ ) ) {
        my @lines = get_lines(
            "raw_output"      => $output,
            "response_object" => $resp
        );
        my ( @Res, $value );
        my $i = 0;
        foreach $value (@lines) {
            if ( $value =~ /^(Buffers)\sare\s(identical).$/ ) {
                $Res[$i]->{ to_key_name( item => $1 ) } = $2;
            }
        }
        return \@Res;
    }
} ## end sub iomem_buffer_compare
## @name ::iomem_system_current_get
##
## @summary Parses the raw output from the "iomem system current get" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_system_current_get with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_system_current_get(%args);
## INPUTS TO METHOD CALL 'iomem_system_current_get' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem system current get; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'using_system' => 'flash'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_system_current_get {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $value );
    my $i = 0;
    foreach $value (@lines) {
        if ( $value =~ /^(.*):\s+(.*)$/ ) {
            $Res[$i]->{ to_key_name( item => $1 ) } = $2;
        }
    }
    return \@Res;
} ## end sub iomem_system_current_get
## @name ::iomem_system_current_set
##
## @summary Parses the raw output from the "iomem system current set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_system_current_set with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_system_current_set(%args);
## INPUTS TO METHOD CALL 'iomem_system_current_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-s' </td> <td> 'state' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem system current set -s flash; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub iomem_system_current_set {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub iomem_system_current_set
## @name ::iomem_system_channel
##
## @summary Parses the raw output from the "iomem system channel" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_system_channel with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_system_channel(%args);
## INPUTS TO METHOD CALL 'iomem_system_channel' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem system channel; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '0'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '1'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '2'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '3'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '4'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '5'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '6'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '7'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '8'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '9'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '10'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '11'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '12'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '13'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '14'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '15'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '16'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '17'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '18'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '19'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '20'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '21'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '22'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '23'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '24'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '25'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '26'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '27'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '28'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '29'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '30'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '31'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '32'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '33'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '34'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '35'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '36'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '37'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '38'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '39'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '40'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '41'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '42'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '43'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '44'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '45'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '46'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '47'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '48'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '49'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '50'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '51'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '52'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '53'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '54'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '55'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '56'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '57'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '58'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '59'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '60'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '61'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '62'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '63'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '64'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '65'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '66'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '67'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '68'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '69'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '70'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '71'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '72'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '73'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '74'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '75'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '76'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '77'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '78'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '79'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '80'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '81'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '82'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '83'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '84'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '85'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '86'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '87'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '88'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '89'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '90'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '91'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '92'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '93'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '94'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'status' => ' online',
##&nbsp;            'channel' => '95'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_system_channel {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $value );
    my $i = 0;
    foreach $value (@lines) {
        if ( $value =~ /^(Channel)\s+(\d+):(.*)$/ ) {
            $Res[$i]->{ to_key_name( item => $1 ) } = $2;
            $Res[$i]->{"status"} = $3;
        }
        $i++;
    }
    return \@Res;
} ## end sub iomem_system_channel
## @name ::iomem_nvds_get
##
## @summary Parses the raw output from the "iomem nvds get" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_nvds_get with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_nvds_get(%args);
## INPUTS TO METHOD CALL 'iomem_nvds_get' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem nvds get -A; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Slot 2: Flash Cache
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_2' => 'Flash Cache'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; iomem nvds get -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub iomem_nvds_get {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key );

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _parse_for_errors(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $i = -1;
    my @arr = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@arr) {
        if ( $line =~ /\w+\s(\d+)\:\s+(.*)/ ) {
            $i++;
            $result_array[$i]->{ to_key_name( item => "slot $1" ) } = "$2";
            $key = to_key_name( item => "slot_details" );
        } elsif ( $line =~ /^ount\:\s+(\d+)/ ) {
            $result_array[$i]->{$key}->[0]
                ->{ to_key_name( item => "reboot_count" ) } = $1;
        } elsif ( $line =~ /(Total.*)\:\s+(\d+)/ ) {
            $result_array[$i]->{$key}->[0]->{ to_key_name( item => "$1" ) }
                = $2;
        } elsif ( $line =~ /^\w+\s+\w+\:\s+(\d+)/ ) {
            $result_array[$i]->{$key}->[0]
                ->{ to_key_name( item => "reboot_count" ) } = $1;
        }
    }
    return \@result_array;
} ## end sub iomem_nvds_get
## @name ::iomem_nvds_clear
##
## @summary Parses the raw output from the "iomem nvds clear" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_nvds_clear with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_nvds_clear(%args);
## INPUTS TO METHOD CALL 'iomem_nvds_clear' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem nvds clear -A; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Slot 2: Flash Cache
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_2' => 'Flash Cache'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; iomem nvds clear -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub iomem_nvds_clear {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key );

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _parse_for_errors(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $i = -1;
    my @arr = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@arr) {
        if ( $line =~ /\w+\s(\d+)\:\s+(.*)/ ) {
            $i++;
            $result_array[$i]->{ to_key_name( item => "slot $1" ) } = "$2";
            $key = to_key_name( item => "slot_details" );
        } elsif ( $line =~ /^ount\:\s+(\d+)/ ) {
            $result_array[$i]->{$key}->[0]
                ->{ to_key_name( item => "reboot_count" ) } = $1;
        } elsif ( $line =~ /(Total.*)\:\s+(\d+)/ ) {
            $result_array[$i]->{$key}->[0]->{ to_key_name( item => "$1" ) }
                = $2;
        } elsif ( $line =~ /^\w+\s+\w+\:\s+(\d+)/ ) {
            $result_array[$i]->{$key}->[0]
                ->{ to_key_name( item => "reboot_count" ) } = $1;
        }
    }
    return \@result_array;
} ## end sub iomem_nvds_clear
## @name ::iomem_nvds_increment
##
## @summary Parses the raw output from the "iomem nvds increment" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_nvds_increment with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_nvds_increment(%args);
## INPUTS TO METHOD CALL 'iomem_nvds_increment' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem nvds increment -A; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Slot 2: Flash Cache
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_2' => 'Flash Cache'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; iomem nvds increment -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub iomem_nvds_increment {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key );

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _parse_for_errors(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $i = -1;
    my @arr = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@arr) {
        if ( $line =~ /\w+\s(\d+)\:\s+(.*)/ ) {
            $i++;
            $result_array[$i]->{ to_key_name( item => "slot $1" ) } = "$2";
            $key = to_key_name( item => "slot_details" );
        } elsif ( $line =~ /^ount\:\s+(\d+)/ ) {
            $result_array[$i]->{$key}->[0]
                ->{ to_key_name( item => "reboot_count" ) } = $1;
        } elsif ( $line =~ /(Total.*)\:\s+(\d+)/ ) {
            $result_array[$i]->{$key}->[0]->{ to_key_name( item => "$1" ) }
                = $2;
        } elsif ( $line =~ /^\w+\s+\w+\:\s+(\d+)/ ) {
            $result_array[$i]->{$key}->[0]
                ->{ to_key_name( item => "reboot_count" ) } = $1;
        }
    }
    return \@result_array;
} ## end sub iomem_nvds_increment
## @name ::iomem_led_show
##
## @summary Parses the raw output from the "iomem led show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_led_show with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_led_show(%args);
## INPUTS TO METHOD CALL 'iomem_led_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem led show -A; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Slot 2: Flash Cache
##&nbsp;	     fault: on
##&nbsp;	  activity: on
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_2' => 'Flash Cache',
##&nbsp;            'led_info' => [
##&nbsp;                            {
##&nbsp;                              'activity' => 'on',
##&nbsp;                              'fault' => 'on'
##&nbsp;                            }
##&nbsp;                          ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; iomem led show -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;	     fault: on
##&nbsp;	  activity: on
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'led_info' => [
##&nbsp;                            {
##&nbsp;                              'activity' => 'on',
##&nbsp;                              'fault' => 'on'
##&nbsp;                            }
##&nbsp;                          ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_led_show {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key );

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _parse_for_errors(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $i = -1;
    my @arr = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@arr) {
        if ( $line =~ /\w+\s(\d+)\:\s+(.*)/ ) {
            $i++;
            $result_array[$i]->{ to_key_name( item => "slot $1" ) } = "$2";
            $key = to_key_name( item => "led_info" );
        } elsif ( $line =~ /(fault|activity):\s(\w+)/ ) {
            if ( $i == -1 ) {
                $i = 0;
                $key = to_key_name( item => "led_info" );
            }
            $result_array[$i]->{$key}->[0]->{ to_key_name( item => "$1" ) }
                = $2;
        }
    }
    return \@result_array;
} ## end sub iomem_led_show
## @name ::iomem_led_set
##
## @summary Parses the raw output from the "iomem led set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_led_set with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_led_set(%args);
## INPUTS TO METHOD CALL 'iomem_led_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-r' </td> <td> 'led' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## <tr> <td> '-s' </td> <td> 'state' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem led set -A -s on; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Slot 2: Flash Cache
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_2' => 'Flash Cache'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; iomem led set -c 2 -s on -r fault; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub iomem_led_set {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key );

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _parse_for_errors(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $i = -1;
    my @arr = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@arr) {
        if ( $line =~ /\w+\s(\d+)\:\s+(.*)/ ) {
            $i++;
            $result_array[$i]->{ to_key_name( item => "slot $1" ) } = "$2";
            $key = to_key_name( item => "led_info" );
        } elsif ( $line =~ /(fault|activity):\s(\w+)/ ) {
            $result_array[$i]->{$key}->[0]->{ to_key_name( item => "$1" ) }
                = $2;
        }
    }
    return \@result_array;
} ## end sub iomem_led_set

## @name ::iomem_nor_read
##
## @summary Parses the raw output from the "iomem nor read" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_nor_read with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_nor_read(%args);
## INPUTS TO METHOD CALL 'iomem_nor_read' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem nor read -A; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Slot 2: Flash Cache
##&nbsp;00000000 | 55 55 ff ff ff ff ff ff ff ff ff ff ff ff ff ff | UU..............
##&nbsp;00000010 | ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff | ................
##&nbsp;00000020 | ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff | ................
##&nbsp;00000030 | ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff | ................
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'info' => [
##&nbsp;                        {
##&nbsp;                          'data' => ' 55 55 ff ff ff ff ff ff ff ff ff ff ff ff ff ff ',
##&nbsp;                          'address' => '00000000',
##&nbsp;                          'miscellaneous' => ' UU..............'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'data' => ' ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ',
##&nbsp;                          'address' => '00000010',
##&nbsp;                          'miscellaneous' => ' ................'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'data' => ' ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ',
##&nbsp;                          'address' => '00000020',
##&nbsp;                          'miscellaneous' => ' ................'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'data' => ' ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ',
##&nbsp;                          'address' => '00000030',
##&nbsp;                          'miscellaneous' => ' ................'
##&nbsp;                        }
##&nbsp;                      ],
##&nbsp;            'slot_2' => 'flash_cache'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; iomem nor read -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;00000000 | 55 55 ff ff ff ff ff ff ff ff ff ff ff ff ff ff | UU..............
##&nbsp;00000010 | ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff | ................
##&nbsp;00000020 | ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff | ................
##&nbsp;00000030 | ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff | ................
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'info' => [
##&nbsp;                        {
##&nbsp;                          'data' => ' 55 55 ff ff ff ff ff ff ff ff ff ff ff ff ff ff ',
##&nbsp;                          'address' => '00000000',
##&nbsp;                          'miscellaneous' => ' UU..............'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'data' => ' ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ',
##&nbsp;                          'address' => '00000010',
##&nbsp;                          'miscellaneous' => ' ................'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'data' => ' ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ',
##&nbsp;                          'address' => '00000020',
##&nbsp;                          'miscellaneous' => ' ................'
##&nbsp;                        },
##&nbsp;                        {
##&nbsp;                          'data' => ' ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ',
##&nbsp;                          'address' => '00000030',
##&nbsp;                          'miscellaneous' => ' ................'
##&nbsp;                        }
##&nbsp;                      ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_nor_read {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key );

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _parse_for_errors(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $i = -1;
    my $j;
    my @arr = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@arr) {
        if ( $line =~ /\w+\s(\d+)\:\s+(.*)/ ) {
            $i++;
            $j = 0;
            $result_array[$i]->{ to_key_name( item => "slot $1" ) } = "$2";
            $key = to_key_name( item => "info" );
        } elsif ( $line =~ /(\S+)\s+\|(.*)\|(.*)/ ) {
            if ( $i == -1 ) {
                $i = 0;
                $key = to_key_name( item => "info" );
            }
            $result_array[$i]->{$key}->[$j]
                ->{ to_key_name( item => "address" ) } = $1;
            $result_array[$i]->{$key}->[$j]->{ to_key_name( item => "data" ) }
                = $2;
            $result_array[$i]->{$key}->[$j]
                ->{ to_key_name( item => "miscellaneous" ) } = $3;
            $j++;
        }
    }
    return \@result_array;
} ## end sub iomem_nor_read
## @name ::iomem_nor_erase
##
## @summary Parses the raw output from the "iomem nor erase" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_nor_erase with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_nor_erase(%args);
## INPUTS TO METHOD CALL 'iomem_nor_erase' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem nor erase -A; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Slot 2: Flash Cache
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_2' => 'Flash Cache'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; iomem nor erase -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub iomem_nor_erase {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key );

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _parse_for_errors(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $i = -1;
    my @arr = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@arr) {
        if ( $line =~ /\w+\s(\d+)\:\s+(.*)/ ) {
            $i++;
            $result_array[$i]->{ to_key_name( item => "slot $1" ) } = "$2";
        }
    }
    return \@result_array;
} ## end sub iomem_nor_erase
## @name ::iomem_nor_program
##
## @summary Parses the raw output from the "iomem nor program" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_nor_program with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_nor_program(%args);
## INPUTS TO METHOD CALL 'iomem_nor_program' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-v' </td> <td> 'patt-or-value' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem nor program -A; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Slot 2: Flash Cache
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_2' => 'Flash Cache'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; iomem nor program -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub iomem_nor_program {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key );

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _parse_for_errors(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $i = -1;
    my @arr = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@arr) {
        if ( $line =~ /\w+\s(\d+)\:\s+(.*)/ ) {
            $i++;
            $result_array[$i]->{ to_key_name( item => "slot $1" ) } = "$2";
        }
    }
    return \@result_array;
} ## end sub iomem_nor_program
## @name ::iomem_nor_write_protect
##
## @summary Parses the raw output from the "iomem nor write-protect" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_nor_write_protect with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_nor_write_protect(%args);
## INPUTS TO METHOD CALL 'iomem_nor_write_protect' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-W' </td> <td> 'write-protection' </td> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-o' </td> <td> 'other-or-cid' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem nor write-protect -A; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Slot 2: Flash Cache
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_2' => 'Flash Cache'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; iomem nor write-protect -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub iomem_nor_write_protect {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key );

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _parse_for_errors(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $i = -1;
    my @arr = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@arr) {
        if ( $line =~ /\w+\s(\d+)\:\s+(.*)/ ) {
            $i++;
            $result_array[$i]->{ to_key_name( item => "slot $1" ) } = "$2";
        }
    }
    return \@result_array;
} ## end sub iomem_nor_write_protect
## @name ::iomem_bbm_get_bb
##
## @summary Parses the raw output from the "iomem bbm get-bb" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_bbm_get_bb with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_bbm_get_bb(%args);
## INPUTS TO METHOD CALL 'iomem_bbm_get_bb' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-n' </td> <td> 'no-of-bytes' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem bbm get-bb -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Slot: 2  Initialization Complete  Interface: 0  Bank: 0  Lane: 0
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bad_block_entries' => [
##&nbsp;                                     {
##&nbsp;                                       'bank' => '0',
##&nbsp;                                       'interface' => '0',
##&nbsp;                                       'lane' => '0'
##&nbsp;                                     }
##&nbsp;                                   ],
##&nbsp;            'slot_2' => 'Initialization Complete '
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_bbm_get_bb {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key );

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _parse_for_errors(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $i   = -1;
    my $j   = 0;
    my @arr = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@arr) {
        if ( $line
            =~ /(\w+):\s+(\d+)\s+(Ini.*)\s+(Interface)\:\s+(\d+)\s+(\w+)\:\s+(\d+)\s+(\w+)\:\s+(\d+)/
            ) {
            $i++;
            $result_array[$i]->{ to_key_name( item => "$1_$2" ) } = "$3";
            $key = to_key_name( item => "bad_block_entries" );
            $result_array[$i]->{$key}->[$j]->{ to_key_name( item => "$4" ) }
                = $5;
            $result_array[$i]->{$key}->[$j]->{ to_key_name( item => "$6" ) }
                = $7;
            $result_array[$i]->{$key}->[$j]->{ to_key_name( item => "$8" ) }
                = $9;
        } elsif ( $line
            =~ /(\w+):\s+(\d+)\s+(Dis.*|Fac.*)\s+(Interface)\:\s+(\d+)\s+(\w+)\:\s+(\d+)\s+(\w+)\:\s+(\d+)\s+(\w+)\:\s+(\d+)/
            ) {
            $i++;
            $result_array[$i]->{ to_key_name( item => "$1_$2" ) } = "$3";
            $key = to_key_name( item => "bad_block_entries" );
            $result_array[$i]->{$key}->[$j]->{ to_key_name( item => "$4" ) }
                = $5;
            $result_array[$i]->{$key}->[$j]->{ to_key_name( item => "$6" ) }
                = $7;
            $result_array[$i]->{$key}->[$j]->{ to_key_name( item => "$8" ) }
                = $9;
            $result_array[$i]->{$key}->[$j]->{ to_key_name( item => "$10" ) }
                = $11;
        }
    }
    return \@result_array;
} ## end sub iomem_bbm_get_bb
## @name ::iomem_bbm_card_bb
##
## @summary Parses the raw output from the "iomem bbm card-bb" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_bbm_card_bb with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_bbm_card_bb(%args);
## INPUTS TO METHOD CALL 'iomem_bbm_card_bb' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> <td>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem bbm card-bb -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;0 bad blocks
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bad_blocks' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_bbm_card_bb {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key );

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _parse_for_errors(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $i   = -1;
    my $j   = 0;
    my @arr = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@arr) {
        if ( $line =~ /(\d+)\s+(.*)/ ) {
            $i++;
            $result_array[$i]->{ to_key_name( item => "$2" ) } = "$1";
        }
    }
    return \@result_array;
} ## end sub iomem_bbm_card_bb
## @name ::iomem_bbm_check
##
## @summary Parses the raw output from the "iomem bbm check" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_bbm_check with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_bbm_check(%args);
## INPUTS TO METHOD CALL 'iomem_bbm_check' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem bbm check -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Block is good.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'block' => 'good'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_bbm_check {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key );

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _parse_for_errors(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $i   = -1;
    my $j   = 0;
    my @arr = get_lines( raw_output => $output, response_object => $resp );
    foreach my $line (@arr) {
        if ( $line =~ /(\w+)\s+\w+\s+(\w+)/ ) {
            $i++;
            $result_array[$i]->{ to_key_name( item => "$1" ) } = "$2";
        }
    }
    return \@result_array;
} ## end sub iomem_bbm_check
## @name ::iomem_bbm_set
##
## @summary Parses the raw output from the "iomem bbm set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_bbm_set with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_bbm_set(%args);
## INPUTS TO METHOD CALL 'iomem_bbm_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## <tr> <td> '-E' </td> <td> 'core-fail' </td> <td> '1' </td> </tr>
## <tr> <td> '-D' </td> <td> 'fail-inherent-traits' </td> <td> '1' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem bbm set -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub iomem_bbm_set {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key );

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _parse_for_errors(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return \@result_array;
} ## end sub iomem_bbm_set
## @name ::iomem_bbm_empty
##
## @summary Parses the raw output from the "iomem bbm empty" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_bbm_empty with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_bbm_check(%args);
## INPUTS TO METHOD CALL 'iomem_bbm_empty' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem bbm empty -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub iomem_bbm_empty {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key );

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _parse_for_errors(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return \@result_array;
} ## end sub iomem_bbm_empty
## @name ::iomem_bbm_revert
##
## @summary Parses the raw output from the "iomem bbm revert" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_bbm_revert with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_bbm_revert(%args);
## INPUTS TO METHOD CALL 'iomem_bbm_revert' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem bbm revert -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub iomem_bbm_revert {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_actual_command();
    my $error_level = $resp->get_execution_status();
    my ( $error, $key );

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _parse_for_errors(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return \@result_array;
} ## end sub iomem_bbm_revert
## @name ::iomem_power_set
##
## @summary Parses the raw output from the "iomem power set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_power_set with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_power_set(%args);
## INPUTS TO METHOD CALL 'iomem_power_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## <tr> <td> '-s' </td> <td> 'state' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = iomem power set -c 6 -s low
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;        Power state set to low.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'power_state' => 'low'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_power_set {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $value );
    foreach $value (@lines) {
        if ( $value =~ /Slot\s+(\d+):\s+(.*)/ ) {
            $info{"slot_number"} = $1;
            $info{"card_name"}   = $2;
        } elsif ( $value =~ /(\s+)?Power\s+state\s+set\s+to\s+(\w+).*/ ) {
            $info{"power_state"} = $2;
            push( @Res, \%info );
        } elsif ( $value =~ /(No\s+cards\s+installed.*)/i ) {
            $info{"status"} = $1;
            push( @Res, \%info );
        }
    }
    return \@Res;
} ## end sub iomem_power_set

## @name ::iomem_power_show
##
## @summary Parses the raw output from the "iomem power show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_power_show with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_power_show(%args);
## INPUTS TO METHOD CALL 'iomem_power_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = iomem power show -c 6
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;        Power state: High
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'power_state' => 'Low'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_power_show {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        print "error is inside  $error\n";
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $value );
    foreach $value (@lines) {
        if ( $value =~ /Slot\s+(\d+):\s+(.*)/ ) {
            $info{"slot_number"} = $1;
            $info{"card_name"}   = $2;
        } elsif ( $value =~ /Power\s+state:\s+(.*)/ ) {
            $info{"power_state"} = $1;
            push( @Res, \%info );
        } elsif ( $value =~ /(No\s+cards\s+installed.*)/i ) {
            $info{"status"} = $1;
            push( @Res, \%info );
        }
    }
    return \@Res;
} ## end sub iomem_power_show
## @name ::iomem_training_show
##
## @summary Parses the raw output from the "iomem training show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_training_show with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_training_show(%args);
## INPUTS TO METHOD CALL 'iomem_training_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = iomem training show -c 6
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Training overridden: NO
##&nbsp;Memory controller 0: READY
##&nbsp;     Bank  5 idelay: 53
##&nbsp;     Bank  9 idelay: 54
##&nbsp;Memory controller 1: READY
##&nbsp;     Bank 11 idelay: 53
##&nbsp;     Bank  7 idelay: 53
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'memory_controllers' => [
##&nbsp;                                      {
##&nbsp;                                        'memory_controller' => '0',
##&nbsp;                                        'bank_9_idelay' => '54',
##&nbsp;                                        'status' => 'READY',
##&nbsp;                                        'bank_5_idelay' => '54'
##&nbsp;                                      },
##&nbsp;                                      {
##&nbsp;                                        'bank_7_idelay' => '53',
##&nbsp;                                        'memory_controller' => '1',
##&nbsp;                                        'status' => 'READY',
##&nbsp;                                        'bank_11_idelay' => '54'
##&nbsp;                                      }
##&nbsp;                                    ],
##&nbsp;            'traning_overridden' => 'NO'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_training_show {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    my $result = _iomem_training(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );

    return $result;
} ## end sub iomem_training_show
## @name ::iomem_training_force
##
## @summary Parses the raw output from the "iomem training force" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_training_force with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_training_force(%args);
## INPUTS TO METHOD CALL 'iomem_training_force' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-y' </td> <td> 'idelay-mem-contr-1-bank-11' </td> <td> '0' </td> </tr>
## <tr> <td> '-z' </td> <td> 'idelay-mem-contr-1-bank-7' </td> <td> '0' </td> </tr>
## <tr> <td> '-w' </td> <td> 'idelay-mem-contr-0-bank-5' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '-x' </td> <td> 'idelay-mem-contr-0-bank-9' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = iomem training force -c 6
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Training overridden: YES
##&nbsp;Memory controller 0: READY
##&nbsp;     Bank  5 idelay: 54
##&nbsp;     Bank  9 idelay: 54
##&nbsp;Memory controller 1: READY
##&nbsp;     Bank 11 idelay: 54
##&nbsp;     Bank  7 idelay: 53
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'memory_controllers' => [
##&nbsp;                                      {
##&nbsp;                                        'memory_controller' => '0',
##&nbsp;                                        'bank_9_idelay' => '54',
##&nbsp;                                        'status' => 'READY',
##&nbsp;                                        'bank_5_idelay' => '54'
##&nbsp;                                      },
##&nbsp;                                      {
##&nbsp;                                        'bank_7_idelay' => '53',
##&nbsp;                                        'memory_controller' => '1',
##&nbsp;                                        'status' => 'READY',
##&nbsp;                                        'bank_11_idelay' => '54'
##&nbsp;                                      }
##&nbsp;                                    ],
##&nbsp;            'traning_overridden' => 'YES'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_training_force {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    my $result = _iomem_training(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );

    return $result;
} ## end sub iomem_training_force
## @name ::iomem_training_auto
##
## @summary Parses the raw output from the "iomem training auto" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_training_auto with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_training_auto(%args);
## INPUTS TO METHOD CALL 'iomem_training_auto' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = iomem training auto -c 6
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Training overridden: NO
##&nbsp;Memory controller 0: READY
##&nbsp;     Bank  5 idelay: 53
##&nbsp;     Bank  9 idelay: 53
##&nbsp;Memory controller 1: READY
##&nbsp;     Bank 11 idelay: 54
##&nbsp;     Bank  7 idelay: 53
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'memory_controllers' => [
##&nbsp;                                      {
##&nbsp;                                        'memory_controller' => '0',
##&nbsp;                                        'bank_9_idelay' => '53',
##&nbsp;                                        'status' => 'READY',
##&nbsp;                                        'bank_5_idelay' => '53'
##&nbsp;                                      },
##&nbsp;                                      {
##&nbsp;                                        'bank_7_idelay' => '53',
##&nbsp;                                        'memory_controller' => '1',
##&nbsp;                                        'status' => 'READY',
##&nbsp;                                        'bank_11_idelay' => '54'
##&nbsp;                                      }
##&nbsp;                                    ],
##&nbsp;            'traning_overridden' => 'NO'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_training_auto {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    my $result = _iomem_training(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );

    return $result;
} ## end sub iomem_training_auto

sub _iomem_training {
    my %args         = @_;
    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my %info         = ();

    #Now we can parse the output
    my @lines = get_lines(
        "raw_output"      => $raw_output,
        "response_object" => $response
    );
    my @result_array;
    my $i = -1;
    my $j = 0;
    my $k = 0;
    my $key;
    if ( $command_line =~ /-c/ ) {

        foreach my $line (@lines) {
            if ( $line =~ /Training\s+overridden:\s+(.*)/i ) {
                $i++;
                $result_array[$i]
                    ->{ to_key_name( item => "traning_overridden" ) } = $1;
                $key = to_key_name( item => "memory_controllers" );
            } elsif ( $line =~ /Memory\s+controller\s+(\d+):\s+(.*)/ ) {
                $result_array[$i]->{$key}->[$j]
                    ->{ to_key_name( item => "memory_controller" ) } = $1;
                $result_array[$i]->{$key}->[$j]
                    ->{ to_key_name( item => "status" ) } = $2;
            } elsif ( $line =~ /(\s+)?Bank\s+(\d+\s+idelay):\s+(\d+)/ ) {
                $k++;
                my @key1 = split( / /, $2 );
                my $key1 = join( "_", @key1 );
                $key1 =~ tr/[A-Z]/[a-z]/;
                $result_array[$i]->{$key}->[$j]
                    ->{ to_key_name( item => "bank" . "_" . $key1 ) } = $3;
                if ( $k == 2 ) {
                    $j++;
                }
            } elsif ( $line =~ /(No\s+cards\s+installed.*)/i ) {
                $info{"status"} = $1;
                push( @result_array, \%info );
            }
        }
    } elsif ( $command_line =~ /-A/ ) {
        foreach my $line (@lines) {
            if ( $line =~ /Slot\s+(\d+):\s+(.*)/i ) {
                $i++;
                $result_array[$i]->{ to_key_name( item => "slot_number" ) }
                    = $1;
                $result_array[$i]->{ to_key_name( item => "card_name" ) }
                    = $2;
            } elsif ( $line =~ /Training\s+overridden:\s+(.*)/i ) {
                $result_array[$i]
                    ->{ to_key_name( item => "traning_overridden" ) } = $1;
                $key = to_key_name( item => "memory_controllers" );
            } elsif ( $line =~ /Memory\s+controller\s+(\d+):\s+(.*)/ ) {
                $result_array[$i]->{$key}->[$j]
                    ->{ to_key_name( item => "memory_controller" ) } = $1;
                $result_array[$i]->{$key}->[$j]
                    ->{ to_key_name( item => "status" ) } = $2;
            } elsif ( $line =~ /(\s+)?Bank\s+(\d+\s+idelay):\s+(\d+)/ ) {
                $k++;
                my @key1 = split( / /, $2 );
                my $key1 = join( "_", @key1 );
                $key1 =~ tr/[A-Z]/[a-z]/;
                $result_array[$i]->{$key}->[$j]
                    ->{ to_key_name( item => "bank" . "_" . $key1 ) } = $3;
                if ( $k == 2 ) {
                    $j++;
                }
            } elsif ( $line =~ /(No\s+cards\s+installed.*)/i ) {
                $info{"status"} = $1;
                push( @result_array, \%info );
            }
        }
    }
    return \@result_array;
} ## end sub _iomem_training

## @name ::iomem_stats_zero
##
## @summary Parses the raw output from the "iomem stats zero" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_stats_zero with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_stats_zero(%args);
## INPUTS TO METHOD CALL 'iomem_stats_zero' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem stats zero -A; result -l'
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Slot 6: Flash Cache
##&nbsp;        Statistics zeroed.
##&nbsp;
##&nbsp;Slot 7: Flash Cache
##&nbsp;        Statistics zeroed.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'slot_number' => '6',
##&nbsp;             'status' => 'Statistics zeroed',
##&nbsp;             'name' => 'Flash Cache'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'slot_number' => '7',
##&nbsp;             'status' => 'Statistics zeroed',
##&nbsp;             'name' => 'Flash Cache'
##&nbsp;           }
##&nbsp;        ];
##&nbsp;

sub iomem_stats_zero {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $line, $i );
    if ( $command =~ /-A/ ) {
        $i = -1;
    } else {
        $i = 0;
    }
    foreach $line (@lines) {
        if ( $line =~ /Slot\s+(\d+):\s+(.*)/ ) {
            $i++;
            $Res[$i]->{slot_number} = $1;
            $Res[$i]->{name}        = $2;
        } elsif ( $line =~ /(Statistics\s+zeroed)/ ) {
            $Res[$i]->{status} = $1;
        }
    }
    return \@Res;
} ## end sub iomem_stats_zero

## @name ::iomem_stats_show
##
## @summary Parses the raw output from the "iomem stats show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_stats_show with any of its options as described in the
##    corresponding cdef file 'iomem_register.cdefs'
## @synopsis
##      iomem_stats_show(%args);
## INPUTS TO METHOD CALL 'iomem_stats_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem stats show -A; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Slot 2: Flash Cache
##&nbsp;                           DMA Engine 0 Statistics                  DMA Engine 1 Statistics
##&nbsp;                      Total       Appendix      Data_CRC       Total        Appendix      Data_CRC
##&nbsp;                  ------------  ------------  ------------  ------------  ------------  ------------
##&nbsp;        Received:           33             -             -             0             -             -
##&nbsp;     Descriptors:           33            32             0             0             0             0
##&nbsp;       DMA Reads:           32            32             0             0             0             0
##&nbsp;      DMA Writes:            0             0             0             0             0             0
##&nbsp; Local DMA Reads:            0             0             0             0             0             0
##&nbsp;Local DMA Writes:            0             0             0             0             0             0
##&nbsp;DMA Engine Halts:            1             -             -             0             -             -
##&nbsp;     CECC Errors:            0             -             -             0             -             -
##&nbsp;     UECC Errors:            0             -             -             0             -             -
##&nbsp; Desc CRC Errors:            0             -             -             0             -             -
##&nbsp; Data CRC Errors:            0             -             -             0             -             -
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_number' => '2',
##&nbsp;            'statistics' => [
##&nbsp;                              {
##&nbsp;                                'data_crc' => [
##&nbsp;                                                {
##&nbsp;                                                  'local_dma_reads' => '0',
##&nbsp;                                                  'dma_reads' => '0',
##&nbsp;                                                  'uecc_errors' => '-',
##&nbsp;                                                  'local_dma_writes' => '0',
##&nbsp;                                                  'dma_engine_halts' => '-',
##&nbsp;                                                  'cecc_errors' => '-',
##&nbsp;                                                  'dma_writes' => '0',
##&nbsp;                                                  'desc_crc_errors' => '-',
##&nbsp;                                                  'data_crc_errors' => '-',
##&nbsp;                                                  'descriptors' => '0',
##&nbsp;                                                  'received' => '-'
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;                                'appendix' => [
##&nbsp;                                                {
##&nbsp;                                                  'local_dma_reads' => '0',
##&nbsp;                                                  'dma_reads' => '32',
##&nbsp;                                                  'uecc_errors' => '-',
##&nbsp;                                                  'local_dma_writes' => '0',
##&nbsp;                                                  'dma_engine_halts' => '-',
##&nbsp;                                                  'cecc_errors' => '-',
##&nbsp;                                                  'dma_writes' => '0',
##&nbsp;                                                  'desc_crc_errors' => '-',
##&nbsp;                                                  'data_crc_errors' => '-',
##&nbsp;                                                  'descriptors' => '32',
##&nbsp;                                                  'received' => '-'
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;                                'engine_no' => 0,
##&nbsp;                                'total' => [
##&nbsp;                                             {
##&nbsp;                                               'local_dma_reads' => '0',
##&nbsp;                                               'dma_reads' => '32',
##&nbsp;                                               'uecc_errors' => '0',
##&nbsp;                                               'local_dma_writes' => '0',
##&nbsp;                                               'dma_engine_halts' => '1',
##&nbsp;                                               'cecc_errors' => '0',
##&nbsp;                                               'dma_writes' => '0',
##&nbsp;                                               'desc_crc_errors' => '0',
##&nbsp;                                               'data_crc_errors' => '0',
##&nbsp;                                               'descriptors' => '33',
##&nbsp;                                               'received' => '33'
##&nbsp;                                             }
##&nbsp;                                           ]
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'data_crc' => [
##&nbsp;                                                {
##&nbsp;                                                  'local_dma_reads' => '0',
##&nbsp;                                                  'dma_reads' => '0',
##&nbsp;                                                  'uecc_errors' => '-',
##&nbsp;                                                  'local_dma_writes' => '0',
##&nbsp;                                                  'dma_engine_halts' => '-',
##&nbsp;                                                  'cecc_errors' => '-',
##&nbsp;                                                  'dma_writes' => '0',
##&nbsp;                                                  'desc_crc_errors' => '-',
##&nbsp;                                                  'data_crc_errors' => '-',
##&nbsp;                                                  'descriptors' => '0',
##&nbsp;                                                  'received' => '-'
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;                                'appendix' => [
##&nbsp;                                                {
##&nbsp;                                                  'local_dma_reads' => '0',
##&nbsp;                                                  'dma_reads' => '0',
##&nbsp;                                                  'uecc_errors' => '-',
##&nbsp;                                                  'local_dma_writes' => '0',
##&nbsp;                                                  'dma_engine_halts' => '-',
##&nbsp;                                                  'cecc_errors' => '-',
##&nbsp;                                                  'dma_writes' => '0',
##&nbsp;                                                  'desc_crc_errors' => '-',
##&nbsp;                                                  'data_crc_errors' => '-',
##&nbsp;                                                  'descriptors' => '0',
##&nbsp;                                                  'received' => '-'
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;                                'engine_no' => 1,
##&nbsp;                                'total' => [
##&nbsp;                                             {
##&nbsp;                                               'local_dma_reads' => '0',
##&nbsp;                                               'dma_reads' => '0',
##&nbsp;                                               'uecc_errors' => '0',
##&nbsp;                                               'local_dma_writes' => '0',
##&nbsp;                                               'dma_engine_halts' => '0',
##&nbsp;                                               'cecc_errors' => '0',
##&nbsp;                                               'dma_writes' => '0',
##&nbsp;                                               'desc_crc_errors' => '0',
##&nbsp;                                               'data_crc_errors' => '0',
##&nbsp;                                               'descriptors' => '0',
##&nbsp;                                               'received' => '0'
##&nbsp;                                             }
##&nbsp;                                           ]
##&nbsp;                              }
##&nbsp;                            ],
##&nbsp;            'name' => 'Flash Cache'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub iomem_stats_show {
    my %args = @_;
    my $error_message;
    my $resp      = $args{response_object};
    my $output    = $resp->get_processed_output();
    my $can_throw = 1;
    my $command   = $resp->get_actual_command();
    my %info      = ();
    my $error     = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );

    if ($error) {
        $error     = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $can_throw = 0;
    }

    #Now we can parse the output
    my @lines = get_lines(
        "raw_output"      => $output,
        can_throw         => $can_throw,
        "response_object" => $resp
    );
    my ( @Res, $line, $i );
    if ( $command =~ /-A/ ) {
        $i = -1;
    } else {
        $i = 0;
    }
    foreach $line (@lines) {
        if ( $line =~ /Slot\s+(\d+):\s+(.*)/ ) {
            $i++;
            $Res[$i]->{slot_number} = $1;
            $Res[$i]->{name}        = $2;
        } elsif ( $line =~ /^DMA\s+Engine\s+(\d+)\s+Statistics/ ) {
            $Res[$i]->{statistics}->[0]->{engine_no} = 0;
            $Res[$i]->{statistics}->[1]->{engine_no} = 1;
        } elsif ( $line =~ /(Total\s+Appendix\s+Data_CRC)|(-----)/ ) {
            next;
        } elsif (
            $line =~ /(Received|Descriptors|Reads|Writes|Halts|Errors)/ ) {
            $line =~ /([A-za-z\s]+):(.*)/;
            my $key    = $1;
            my $value  = $2;
            my @values = split( /\s+/, $value );
            shift(@values);
            $Res[$i]->{statistics}->[0]->{total}->[0]
                ->{ to_key_name( item => $key ) } = $values[0];
            $Res[$i]->{statistics}->[0]->{appendix}->[0]
                ->{ to_key_name( item => $key ) } = $values[1];
            $Res[$i]->{statistics}->[0]->{data_crc}->[0]
                ->{ to_key_name( item => $key ) } = $values[2];
            $Res[$i]->{statistics}->[1]->{total}->[0]
                ->{ to_key_name( item => $key ) } = $values[3];
            $Res[$i]->{statistics}->[1]->{appendix}->[0]
                ->{ to_key_name( item => $key ) } = $values[4];
            $Res[$i]->{statistics}->[1]->{data_crc}->[0]
                ->{ to_key_name( item => $key ) } = $values[5];
        }
    }
    if ($error) {
        $resp->set_parsed_output( parsed_output => \@Res );
        $error->throw( text => $error_message, response_object => $resp );
    }

    return \@Res;
} ## end sub iomem_stats_show

## @name ::iomem_config_show
##
## @summary Parses the raw output from the "iomem config show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_config_show with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_config_show(%args);
## INPUTS TO METHOD CALL 'iomem_config_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem config show -A; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;Response get_parsed_output:
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'serial_number' => '55',
##&nbsp;            'memory_size' => '520 GiB',
##&nbsp;            'state' => 'Enabled',
##&nbsp;            'model_name' => 'X1971A-R5',
##&nbsp;            'slot_number' => '6',
##&nbsp;            'link_width' => '8',
##&nbsp;            'fpga_image' => 'Primary',
##&nbsp;            'fpga_release' => '2.2',
##&nbsp;            'name' => 'flash cache',
##&nbsp;            'pci_vendor_id' => '1275',
##&nbsp;            'pci_bus' => '60',
##&nbsp;            'pci_device' => '0',
##&nbsp;            'part_number' => '111-00708',
##&nbsp;            'revision' => '10',
##&nbsp;            'fpga_build' => '201012201350',
##&nbsp;            'pci_device_id' => '774c'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'serial_number' => '55',
##&nbsp;            'memory_size' => '520 GiB',
##&nbsp;            'state' => 'Enabled',
##&nbsp;            'model_name' => 'X1938A-R5',
##&nbsp;            'slot_number' => '7',
##&nbsp;            'link_width' => '8',
##&nbsp;            'fpga_image' => 'Primary',
##&nbsp;            'fpga_release' => '1.10',
##&nbsp;            'name' => 'flash cache',
##&nbsp;            'pci_vendor_id' => '1275',
##&nbsp;            'pci_bus' => '142',
##&nbsp;            'pci_device' => '0',
##&nbsp;            'part_number' => '111-00525',
##&nbsp;            'revision' => '21',
##&nbsp;            'fpga_build' => '201012200653',
##&nbsp;            'pci_device_id' => '774c'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_config_show {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my @result_array;
    my ( $val, $key1, $key2, $value1, $value2, $name, $i, $j, $flag );
    my $option = $command;
    $i = -1;
    $j = 0;
    $flag = 0;
    foreach $val (@lines) {

        if ( $val
            =~ /DIMM\s+Size\s+Part\s+Number\s+Serial\s+Number\s+Status$/ ) {
            $name = "dimm_info";
            $i    = 0;
        } elsif ( $val =~ /^Slot\s+(\d+):\s+(\w+(\s+\w+)*)/ ) {
            $i++;
            $result_array[$i]->{"slot_number"} = $1;
            $result_array[$i]->{"name"}        = $2;
            $flag = 1;
        } elsif ( $val
            =~ /(\w+(\s+\w+)?(\s+\w+)?):\s+(\w+((-|.)\w+)?)\s+(\w+(\s+\w+)?(\s+\w+)?):\s+(\w+((-|.)\w+)?)/
            ) {
            $i = 0 if($flag == 0);
            my @key1 = split( / /, $1 );
            $key1 = join( "_", @key1 );
            $key1 =~ tr/[A-Z]/[a-z]/;
            my @key2 = split( / /, $7 );
            $key2 = join( "_", @key2 );
            $key2 =~ tr/[A-Z]/[a-z]/;
            $result_array[$i]->{"$key1"} = $4;
            $result_array[$i]->{"$key2"} = $10;
        } elsif ( $val
            =~ /(\d+):\s+(\d+\s+\w+)\s+(\w+)\s+(\d+-\d+-\d+-\w+)\s+(\w+)/ ) {
            $result_array[$i]->{"$name"}->[$j]->{"dimm"}          = $1;
            $result_array[$i]->{"$name"}->[$j]->{"size"}          = $2;
            $result_array[$i]->{"$name"}->[$j]->{"part_number"}   = $3;
            $result_array[$i]->{"$name"}->[$j]->{"serial_number"} = $4;
            $result_array[$i]->{"$name"}->[$j]->{"status"}        = $5;
            $j++;
        } elsif ( $val =~ /(.*):\s+(.*)/ ) {
            $result_array[$i]->{to_key_name(item => $1)} = $2;
        }
    }
    return \@result_array;
} ## end sub iomem_config_show

## @name ::iomem_config_enable
##
## @summary Parses the raw output from the "iomem config enable" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_config_enable with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_config_enable(%args);
## INPUTS TO METHOD CALL 'iomem_config_enable' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '-P' </td> <td> 'persistent' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem config enable -A; result -l
##&nbsp;
##&nbsp;-------- Parsed Output 1 ------
##&nbsp;Response get_parsed_output:
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_number' => '6',
##&nbsp;            'name' => 'flash cache',
##&nbsp;            'card_state' => 'enabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'slot_number' => '7',
##&nbsp;            'name' => 'flash cache',
##&nbsp;            'card_state' => 'enabled'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; iomem config enable -c 6; result -l
##&nbsp;
##&nbsp;-------- Parsed Output 2 ------
##&nbsp;Response get_parsed_output:
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'card_state' => 'enabled'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_config_enable {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my @result_array;
    my ( $val, $key1, $key2, $value1, $value2, $i );
    $i = -1;
    my $option = $command;
    foreach $val (@lines) {
        if ( $option =~ /-A/ ) {
            if ( $val =~ /^Slot\s+(\d+):\s+(\w+(\s+\w+)*)/ ) {
                $i++;
                $result_array[$i]->{"slot_number"} = $1;
                $result_array[$i]->{"name"}        = $2;
            } elsif ( $val =~ /Card\s+(\w+)/ ) {
                $result_array[$i]->{"card_state"} = $1;
            }
        }
        if ( $option =~ /-c/ ) {
            if ( $val =~ /Card\s+(\w+)/ ) {
                $result_array[0]->{"card_state"} = $1;
            }
        }
    }
    return \@result_array;
} ## end sub iomem_config_enable

## @name ::iomem_config_disable
##
## @summary Parses the raw output from the "iomem config disable" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_config_disable with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_config_disable(%args);
## INPUTS TO METHOD CALL 'iomem_config_disable' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '-P' </td> <td> 'persistent' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem config disable -A; result -l
##&nbsp;
##&nbsp;-------- Parsed Output 1 ------
##&nbsp;Response get_parsed_output:
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_number' => '6',
##&nbsp;            'name' => 'flash cache',
##&nbsp;            'card_state' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'slot_number' => '7',
##&nbsp;            'name' => 'flash cache',
##&nbsp;            'card_state' => 'disabled'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; iomem config disable -c 6; result -l
##&nbsp;
##&nbsp;-------- Parsed Output 2 ------
##&nbsp;Response get_parsed_output:
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'card_state' => 'disabled'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_config_disable {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my @result_array;
    my ( $val, $key1, $key2, $value1, $value2, $i );
    $i = -1;
    my $option = $command;
    foreach $val (@lines) {
        if ( $option =~ /-A/ ) {
            if ( $val =~ /^Slot\s+(\d+):\s+(\w+(\s+\w+)*)/ ) {
                $i++;
                $result_array[$i]->{"slot_number"} = $1;
                $result_array[$i]->{"name"}        = $2;
            } elsif ( $val =~ /Card\s+(\w+)/ ) {
                $result_array[$i]->{"card_state"} = $1;
            }
        }
        if ( $option =~ /-c/ ) {
            if ( $val =~ /Card\s+(\w+)/ ) {
                $result_array[0]->{"card_state"} = $1;
            }
        }
    }
    return \@result_array;
} ## end sub iomem_config_disable

## @name ::iomem_config_state
##
## @summary Parses the raw output from the "iomem config state" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_config_state with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_config_state(%args);
## INPUTS TO METHOD CALL 'iomem_config_state' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '-P' </td> <td> 'persistent' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'state' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem config state -A -s disable; result -l
##&nbsp;
##&nbsp;-------- Parsed Output 1 ------
##&nbsp;Response get_parsed_output:
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_number' => '6',
##&nbsp;            'name' => 'flash cache',
##&nbsp;            'card_state' => 'disabled'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'slot_number' => '7',
##&nbsp;            'name' => 'flash cache',
##&nbsp;            'card_state' => 'disabled'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; iomem config state -c 6 -s uninit; result -l
##&nbsp;
##&nbsp;-------- Parsed Output 2 ------
##&nbsp;Response get_parsed_output:
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'card_state' => 'uninitialized'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_config_state {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my @result_array;
    my ( $val, $key1, $key2, $value1, $value2, $i );
    $i = -1;
    my $option = $command;
    foreach $val (@lines) {
        if ( $option =~ /-A/ ) {
            if ( $val =~ /^Slot\s+(\d+):\s+(\w+(\s+\w+)*)/ ) {
                $i++;
                $result_array[$i]->{"slot_number"} = $1;
                $result_array[$i]->{"name"}        = $2;
            } elsif ( $val =~ /^State:\s+(\w+)/ ) {
                $result_array[$i]->{"state"} = $1;
            }
        }
        if ( $option =~ /-c/ ) {
            if ( $val =~ /^State:\s+(\w+)/ ) {
                $result_array[0]->{"card_state"} = $1;
            }
        }
    }
    return \@result_array;
} ## end sub iomem_config_state

## @name ::iomem_register_show
##
## @summary Parses the raw output from the "iomem register show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_register_show with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_register_show(%args);
## INPUTS TO METHOD CALL 'iomem_register_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem register show -A; result -l'
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp; Slot 2: Flash Cache\r\n
##&nbsp;
##&nbsp; Memory (virgo MMR group) 0x0000000000000000-0x000000000000081f\r\n
##&nbsp;
##&nbsp; 0x00000000 0x010a0653 release_version(minute(0x53),hour(0x6),minor(0xa),major(0x1))\r\n
##&nbsp; 0x00000008 0x23100080 device_control(activity_led(0x0),fault_led(0x0),dma,volt_margin_1_0v(0x0),volt_margin_1_8v(0x0),volt_margin_3_3v(0x0),max_pci_read_req(0x10),int_coalesce_timer(0x18),completion_msi)\r\n
##&nbsp; 0x00000818 0xffffffffffffffff chip_kill_3(bank_0(0xf),bank_1(0xf),bank_2(0xf),bank_3(0xf),bank_4(0xf),bank_5(0xf),bank_6(0xf),bank_7(0xf),bank_8(0xf),bank_9(0xf),bank_10(0xf),bank_11(0xf),bank_12(0xf),bank_13(0xf),bank_14(0xf),bank_15(0xf))\r\n
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; Response get_parsed_output:
##&nbsp; $VAR1 = [
##&nbsp;           {
##&nbsp;             'info' => [
##&nbsp;                         {
##&nbsp;                           'register' => 'release_version',
##&nbsp;                           'end-address' => '0x010a0653',
##&nbsp;                           'masks' => [
##&nbsp;                                        'minute(0x53)',
##&nbsp;                                        'hour(0x6)',
##&nbsp;                                        'minor(0xa)',
##&nbsp;                                        'major(0x1)'
##&nbsp;                                      ],
##&nbsp;                           'start-address' => '0x00000000'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'register' => 'device_control',
##&nbsp;                           'end-address' => '0x23100080',
##&nbsp;                           'masks' => [
##&nbsp;                                        'activity_led(0x0)',
##&nbsp;                                        'fault_led(0x0)',
##&nbsp;                                        'dma',
##&nbsp;                                        'volt_margin_1_0v(0x0)',
##&nbsp;                                        'volt_margin_1_8v(0x0)',
##&nbsp;                                        'volt_margin_3_3v(0x0)',
##&nbsp;                                        'max_pci_read_req(0x10)',
##&nbsp;                                        'int_coalesce_timer(0x18)',
##&nbsp;                                        'completion_msi'
##&nbsp;                                      ],
##&nbsp;                           'start-address' => '0x00000008'
##&nbsp;                         },
##&nbsp;                         {
##&nbsp;                           'register' => 'chip_kill_3',
##&nbsp;                           'end-address' => '0xffffffffffffffff',
##&nbsp;                           'masks' => [
##&nbsp;                                        'bank_0(0xf)',
##&nbsp;                                        'bank_1(0xf)',
##&nbsp;                                        'bank_2(0xf)',
##&nbsp;                                        'bank_3(0xf)',
##&nbsp;                                        'bank_4(0xf)',
##&nbsp;                                        'bank_5(0xf)',
##&nbsp;                                        'bank_6(0xf)',
##&nbsp;                                        'bank_7(0xf)',
##&nbsp;                                        'bank_8(0xf)',
##&nbsp;                                        'bank_9(0xf)',
##&nbsp;                                        'bank_10(0xf)',
##&nbsp;                                        'bank_11(0xf)',
##&nbsp;                                        'bank_12(0xf)',
##&nbsp;                                        'bank_13(0xf)',
##&nbsp;                                        'bank_14(0xf)',
##&nbsp;                                        'bank_15(0xf)'
##&nbsp;                                      ],
##&nbsp;                           'start-address' => '0x00000818'
##&nbsp;                         }
##&nbsp;                       ],
##&nbsp;             'slot_number' => '2',
##&nbsp;             'memory' => [
##&nbsp;                           {
##&nbsp;                             'group' => 'virgo MMR group',
##&nbsp;                             'range' => '0x0000000000000000-0x000000000000081f'
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;             'name' => 'Flash Cache'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;

sub iomem_register_show {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $line, $i, $j );
    $j = -1;
    if ( $command =~ /-A/ ) {
        $i = -1;
    } else {
        $i = 0;
    }
    foreach $line (@lines) {
        if ( $line =~ /Slot\s+(\d+):\s+(.*)/ ) {
            $i++;
            $Res[$i]->{"slot_number"} = $1;
            $Res[$i]->{"name"}        = $2;
        } elsif ( $line =~ /^Memory\s+\((.*)\)\s+(.*)/ ) {
            $Res[$i]->{"memory"}->[0]->{"group"} = $1;
            $Res[$i]->{"memory"}->[0]->{"range"} = $2;
        } elsif (
            $line =~ /([0-9xa-zA-Z]+)\s+([0-9xa-zA-Z]+)\s+(\w+)\((.*)\)/ ) {
            $j++;
            my $mask  = $4;
            my $k     = -1;
            my @masks = split( /,/, $mask );
            $Res[$i]->{"info"}->[$j]->{"start_address"} = $1;
            $Res[$i]->{"info"}->[$j]->{"end_address"}   = $2;
            $Res[$i]->{"info"}->[$j]->{"register"}      = $3;
            foreach my $masks (@masks) {
                $k++;
                $Res[$i]->{"info"}->[$j]->{"masks"}->[$k] = $masks;
            }
        }
    }
    return \@Res;
} ## end sub iomem_register_show

## @name ::iomem_register_set
##
## @summary Parses the raw output from the "iomem register set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_register_set with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_register_set(%args);
## INPUTS TO METHOD CALL 'iomem_register_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-r' </td> <td> 'register' </td> </tr>
## <tr> <td> '-n' </td> <td> 'no-of-bytes' </td> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-v' </td> <td> 'value' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## <tr> <td> '-m' </td> <td> 'mask' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = 'priv set -q diag; iomem register set -A -r device_control -m int_coalesce_timer -v 5; result -l'
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Slot 5: Flash Cache\r\n
##&nbsp;
##&nbsp;Slot 6: Flash Cache\r\n
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; Response get_parsed_output:
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'slot_number' => '5',
##&nbsp;             'name' => 'Flash Cache'
##&nbsp;           },
##&nbsp;           {
##&nbsp;             'slot_number' => '6',
##&nbsp;             'name' => 'Flash Cache'
##&nbsp;           }
##&nbsp;        ];
##&nbsp;

sub iomem_register_set {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $line, $i );
    if ( $command =~ /-A/ ) {
        $i = -1;
    }
    foreach $line (@lines) {
        if ( $line =~ /Slot\s+(\d+):\s+(.*)/ ) {
            $i++;
            $Res[$i]->{"slot_number"} = $1;
            $Res[$i]->{"name"}        = $2;
        }
    }
    return \@Res;
} ## end sub iomem_register_set

## @name ::iomem_register_get
##
## @summary Parses the raw output from the "iomem register get" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_register_get with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_register_get(%args);
## INPUTS TO METHOD CALL 'iomem_register_get' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-r' </td> <td> 'register' </td> </tr>
## <tr> <td> '-n' </td> <td> 'no-of-bytes' </td> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## <tr> <td> '-m' </td> <td> 'mask' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = 'priv set -q diag; iomem register get -A -r device_control -m int_coalesce_timer; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Slot 2: Flash Cache\r\n
##&nbsp;
##&nbsp;Memory (virgo MMR group) 0x0000000000000000-0x000000000000081f\r\n
##&nbsp;
##&nbsp;0x00000008 0x20b00080 device_control(activity_led(0x0),fault_led(0x0),dma,volt_margin_1_0v(0x0),volt_margin_1_8v(0x0),volt_margin_3_3v(0x0),max_pci_read_req(0x10),int_coalesce_timer(0x5),completion_msi)\r\n
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp; Response get_parsed_output:
##&nbsp; $VAR1 = [
##&nbsp;             {
##&nbsp;               'info' => [
##&nbsp;                           {
##&nbsp;                             'register' => 'device_control',
##&nbsp;                             'end-address' => '0x20b00080',
##&nbsp;                             'masks' => [
##&nbsp;                                          'activity_led(0x0)',
##&nbsp;                                          'fault_led(0x0)',
##&nbsp;                                          'dma',
##&nbsp;                                          'volt_margin_1_0v(0x0)',
##&nbsp;                                          'volt_margin_1_8v(0x0)',
##&nbsp;                                          'volt_margin_3_3v(0x0)',
##&nbsp;                                          'max_pci_read_req(0x10)',
##&nbsp;                                          'int_coalesce_timer(0x5)',
##&nbsp;                                          'completion_msi'
##&nbsp;                                        ],
##&nbsp;                             'start-address' => '0x00000008'
##&nbsp;                           }
##&nbsp;                         ],
##&nbsp;               'slot_number' => '2',
##&nbsp;               'memory' => [
##&nbsp;                             {
##&nbsp;                               'group' => 'virgo MMR group',
##&nbsp;                               'range' => '0x0000000000000000-0x000000000000081f'
##&nbsp;                             }
##&nbsp;                           ],
##&nbsp;               'name' => 'Flash Cache'
##&nbsp;             }
##&nbsp;         ];
##&nbsp;

sub iomem_register_get {
    my %args = @_;

    my $result_array = iomem_register_show(%args);
    return $result_array;
} ## end sub iomem_register_get

## @name ::iomem_vmargin_set
##
## @summary Parses the raw output from the "iomem vmargin set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_vmargin_set with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_vmargin_set(%args);
## INPUTS TO METHOD CALL 'iomem_vmargin_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-s' </td> <td> 'state-or-serial' </td> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## <tr> <td> '-r' </td> <td> 'rail-or-rev' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem vmargin set -A -s high; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Slot 6: Performance Acceleration Module I
##&nbsp;        Margining for 1.2V: High
##&nbsp;        Margining for 1.5V: High
##&nbsp;        Margining for 1.8V: High
##&nbsp;        Margining for 2.5V: High
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_6' => [
##&nbsp;                          {
##&nbsp;                            'rail_info' => [
##&nbsp;                                             {
##&nbsp;                                               'rail' => '1.2V',
##&nbsp;                                               'state' => 'High'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'rail' => '1.5V',
##&nbsp;                                               'state' => 'High'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'rail' => '1.8V',
##&nbsp;                                               'state' => 'High'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'rail' => '2.5V',
##&nbsp;                                               'state' => 'High'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;                            'card_name' => 'Performance Acceleration Module I'
##&nbsp;                          }
##&nbsp;                        ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_vmargin_set {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $key );
    my $j = 0;
    foreach my $value (@lines) {
        if ( $value =~ /^(Slot\s+\d+):\s+(.*)$/ ) {
            $key = $1;
            $j   = 0;
            $Res[0]->{ to_key_name( item => $key ) }->[0]->{'card_name'} = $2;
        } elsif ( $value =~ /Margining\s+\w+\s+(\S+):\s+(\w+)/ ) {
            if ($key) {
                $Res[0]->{ to_key_name( item => $key ) }->[0]->{'rail_info'}
                    ->[$j]->{'rail'} = $1;
                $Res[0]->{ to_key_name( item => $key ) }->[0]->{'rail_info'}
                    ->[$j]->{'state'} = $2;
            } else {
                $Res[0]->{'rail_info'}->[$j]->{'rail'}  = $1;
                $Res[0]->{'rail_info'}->[$j]->{'state'} = $2;
            }
            $j++;
        }
    }
    return \@Res;
} ## end sub iomem_vmargin_set

## @name ::iomem_vmargin_show
##
## @summary Parses the raw output from the "iomem vmargin show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_vmargin_show with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_vmargin_show(%args);
## INPUTS TO METHOD CALL 'iomem_vmargin_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem vmargin show -A; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; system node run -node local -command "priv set -q diag; iomem vmargin show -A; result -l"
##&nbsp;
##&nbsp;Slot 5: Flash Cache
##&nbsp;    Margining for 1.0V: Low
##&nbsp;    Margining for 1.8V: Low
##&nbsp;    Margining for 3.3V: Low
##&nbsp;
##&nbsp;Slot 6: Flash Cache
##&nbsp;    Margining for 1.0V: Low
##&nbsp;    Margining for 1.8V: Low
##&nbsp;    Margining for 3.3V: Low
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_5' => [
##&nbsp;                          {
##&nbsp;                            'rail_info' => [
##&nbsp;                                             {
##&nbsp;                                               'rail' => '1.0V',
##&nbsp;                                               'state' => 'Low'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'rail' => '1.8V',
##&nbsp;                                               'state' => 'Low'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'rail' => '3.3V',
##&nbsp;                                               'state' => 'Low'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;                            'card_name' => 'Flash Cache'
##&nbsp;                          }
##&nbsp;                        ],
##&nbsp;            'slot_6' => [
##&nbsp;                          {
##&nbsp;                            'rail_info' => [
##&nbsp;                                             {
##&nbsp;                                               'rail' => '1.0V',
##&nbsp;                                               'state' => 'Low'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'rail' => '1.8V',
##&nbsp;                                               'state' => 'Low'
##&nbsp;                                             },
##&nbsp;                                             {
##&nbsp;                                               'rail' => '3.3V',
##&nbsp;                                               'state' => 'Low'
##&nbsp;                                             }
##&nbsp;                                           ],
##&nbsp;                            'card_name' => 'Flash Cache'
##&nbsp;                          }
##&nbsp;                        ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_vmargin_show {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $key );
    my $j = 0;
    foreach my $value (@lines) {
        if ( $value =~ /^(Slot\s+\d+):\s+(.*)$/ ) {
            $key = $1;
            $j   = 0;
            $Res[0]->{ to_key_name( item => $key ) }->[0]->{'card_name'} = $2;
        } elsif ( $value =~ /Margining\s+\w+\s+(\S+):\s+(\w+)/ ) {
            if ($key) {
                $Res[0]->{ to_key_name( item => $key ) }->[0]->{'rail_info'}
                    ->[$j]->{'rail'} = $1;
                $Res[0]->{ to_key_name( item => $key ) }->[0]->{'rail_info'}
                    ->[$j]->{'state'} = $2;
            } else {
                $Res[0]->{'rail_info'}->[$j]->{'rail'}  = $1;
                $Res[0]->{'rail_info'}->[$j]->{'state'} = $2;
            }
            $j++;
        }
    }
    return \@Res;
} ## end sub iomem_vmargin_show

## @name ::iomem_temp_show
##
## @summary Parses the raw output from the "iomem temp show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_temp_show with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_temp_show(%args);
## INPUTS TO METHOD CALL 'iomem_temp_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem temp show -c 6; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;    Sensor temperature: 29 deg C
##&nbsp;      FPGA temperature: 34 deg C
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'fpga_temperature' => '34 deg C',
##&nbsp;            'sensor_temperature' => '29 deg C'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_temp_show {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $key );
    foreach my $value (@lines) {
        if ( $value =~ /^(Slot\s+\d+):\s+(.*)$/ ) {
            $key = $1;
            $Res[0]->{ to_key_name( item => $key ) }->[0]->{'card_name'} = $2;
        } elsif ( $value =~ /^(\w+\s+\w+):\s+(\d+\D+)/ ) {
            if ($key) {
                $Res[0]->{ to_key_name( item => $key ) }->[0]
                    ->{ to_key_name( item => $1 ) } = $2;
            } else {
                $Res[0]->{ to_key_name( item => $1 ) } = $2;
            }
        }
    }
    return \@Res;
} ## end sub iomem_temp_show

## @name ::iomem_fru_show
##
## @summary Parses the raw output from the "iomem fru show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_fru_show with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_fru_show(%args);
## INPUTS TO METHOD CALL 'iomem_fru_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-R' </td> <td> 're-read' </td> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem fru show -c 6; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; system node run -node local -command "priv set -q diag; iomem fru show -c 6; result -l"
##&nbsp;                   Status: Cached
##&nbsp;        Manufacterer Name: "NetApp"
##&nbsp;               Model Name: "X1971A-R5"
##&nbsp;              Part Number: "111-00525"
##&nbsp;            Serial Number: "5000330819"
##&nbsp;                 Revision: "21"
##&nbsp;                 Flash ID: ec:d5:51:a6:68
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'flash_id' => 'ec:d5:51:a6:68',
##&nbsp;            'serial_number' => '5000330819',
##&nbsp;            'revision' => '21',
##&nbsp;            'part_number' => '111-00525',
##&nbsp;            'manufacterer_name' => 'NetApp',
##&nbsp;            'status' => 'Cached',
##&nbsp;            'model_name' => 'X1971A-R5'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_fru_show {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $key );
    my $j = 0;
    foreach my $value (@lines) {
        if ( $value =~ /^(Slot\s+\d+):\s+(.*)$/ ) {
            $key = $1;
            $Res[0]->{ to_key_name( item => $key ) }->[0]->{'card_name'} = $2;
        } elsif ( $value =~ /^(\w+(\s+\w+)?):\s+["]?(\S+?)["]?$/ ) {
            if ($key) {
                $Res[0]->{ to_key_name( item => $key ) }->[0]
                    ->{ to_key_name( item => $1 ) } = $3;
            } else {
                $Res[0]->{ to_key_name( item => $1 ) } = $3;
            }
        }
    }
    return \@Res;
} ## end sub iomem_fru_show

## @name ::iomem_fru_set
##
## @summary Parses the raw output from the "iomem fru set" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_fru_set with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_fru_set(%args);
## INPUTS TO METHOD CALL 'iomem_fru_set' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-f' </td> <td> 'fru-data' </td> <td> '0' </td> </tr>
## <tr> <td> '-f' </td> <td> 'field' </td> <td> '0' </td> </tr>
## <tr> <td> '-s' </td> <td> 'state-or-serial' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'mode-or-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'version' </td> <td> '0' </td> </tr>
## <tr> <td> '-p' </td> <td> 'part' </td> <td> '0' </td> </tr>
## <tr> <td> '-m' </td> <td> 'model' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '-r' </td> <td> 'rail-or-rev' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem fru set -c 6 -m X1971A-R5 -s 55; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;        FRU information programmed successfully
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'FRU information programmed successfully'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_fru_set {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $key );
    foreach my $value (@lines) {
        if ( $value =~ /^(Slot\s+\d+):\s+(.*)$/ ) {
            $key = $1;
            $Res[0]->{ to_key_name( item => $key ) }->[0]->{'card_name'} = $2;
        } elsif ( $value =~ /^(FRU\s+.*)/ ) {
            if ($key) {
                $Res[0]->{ to_key_name( item => $key ) }->[0]->{'status'}
                    = $1;
            } else {
                $Res[0]->{'status'} = $1;
            }
        }
    }
    return \@Res;
} ## end sub iomem_fru_set

## @name ::iomem_dimm_show
##
## @summary Parses the raw output from the "iomem dimm show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_dimm_show with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_dimm_show(%args);
## INPUTS TO METHOD CALL 'iomem_dimm_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## <tr> <td> '-d' </td> <td> 'dimm-num' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem dimm show -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;-------------------------- DIMM 1 --------------------------
##&nbsp;
##&nbsp;        State: Enabled     Part Number: SG572124AG8RWDBME
##&nbsp;     Capacity: 4GB       Serial Number: 94-01-0723-0001CB53
##&nbsp;        Banks: 8           CECC Errors: 0
##&nbsp;        Ranks: 2           UECC Errors: 0
##&nbsp;
##&nbsp;-------------------------- DIMM 2 --------------------------
##&nbsp;
##&nbsp;        State: Enabled     Part Number: SG572124AG8RWDBME
##&nbsp;     Capacity: 4GB       Serial Number: 94-01-0723-0001CB54
##&nbsp;        Banks: 8           CECC Errors: 0
##&nbsp;        Ranks: 2           UECC Errors: 0
##&nbsp;
##&nbsp;-------------------------- DIMM 3 --------------------------
##&nbsp;
##&nbsp;        State: Enabled     Part Number: SG572124AG8RWDBME
##&nbsp;     Capacity: 4GB       Serial Number: 94-01-0723-0001CB4D
##&nbsp;        Banks: 8           CECC Errors: 0
##&nbsp;        Ranks: 2           UECC Errors: 0
##&nbsp;
##&nbsp;-------------------------- DIMM 4 --------------------------
##&nbsp;
##&nbsp;        State: Enabled     Part Number: SG572124AG8RWDBME
##&nbsp;     Capacity: 4GB       Serial Number: 94-01-0723-0001CB52
##&nbsp;        Banks: 8           CECC Errors: 0
##&nbsp;        Ranks: 2           UECC Errors: 0
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'dimm_info' => [
##&nbsp;                             {
##&nbsp;                               'ranks' => '2',
##&nbsp;                               'serial_number' => '94-01-0723-0001CB53',
##&nbsp;                               'uecc_errors' => '0',
##&nbsp;                               'cecc_errors' => '0',
##&nbsp;                               'capacity' => '4GB',
##&nbsp;                               'state' => 'Enabled',
##&nbsp;                               'part_number' => 'SG572124AG8RWDBME',
##&nbsp;                               'banks' => '8',
##&nbsp;                               'dimm_number' => '1'
##&nbsp;                             },
##&nbsp;                             {
##&nbsp;                               'ranks' => '2',
##&nbsp;                               'serial_number' => '94-01-0723-0001CB54',
##&nbsp;                               'uecc_errors' => '0',
##&nbsp;                               'cecc_errors' => '0',
##&nbsp;                               'capacity' => '4GB',
##&nbsp;                               'state' => 'Enabled',
##&nbsp;                               'part_number' => 'SG572124AG8RWDBME',
##&nbsp;                               'banks' => '8',
##&nbsp;                               'dimm_number' => '2'
##&nbsp;                             },
##&nbsp;                             {
##&nbsp;                               'ranks' => '2',
##&nbsp;                               'serial_number' => '94-01-0723-0001CB4D',
##&nbsp;                               'uecc_errors' => '0',
##&nbsp;                               'cecc_errors' => '0',
##&nbsp;                               'capacity' => '4GB',
##&nbsp;                               'state' => 'Enabled',
##&nbsp;                               'part_number' => 'SG572124AG8RWDBME',
##&nbsp;                               'banks' => '8',
##&nbsp;                               'dimm_number' => '3'
##&nbsp;                             },
##&nbsp;                             {
##&nbsp;                               'ranks' => '2',
##&nbsp;                               'serial_number' => '94-01-0723-0001CB52',
##&nbsp;                               'uecc_errors' => '0',
##&nbsp;                               'cecc_errors' => '0',
##&nbsp;                               'capacity' => '4GB',
##&nbsp;                               'state' => 'Enabled',
##&nbsp;                               'part_number' => 'SG572124AG8RWDBME',
##&nbsp;                               'banks' => '8',
##&nbsp;                               'dimm_number' => '4'
##&nbsp;                             }
##&nbsp;                           ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_dimm_show {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $key );
    my $j = -1;
    foreach my $value (@lines) {
        if ( $value =~ /^(Slot\s+\d+):\s+(.*)$/ ) {
            $key = $1;
            $j   = -1;
            $Res[0]->{ to_key_name( item => $key ) }->[0]->{'card_name'} = $2;
        } elsif ( $value =~ /^[-]+\s+\w+\s+(\d+).*/ ) {
            $j++;
            if ($key) {
                $Res[0]->{ to_key_name( item => $key ) }->[0]->{'dimm_info'}
                    ->[$j]->{'dimm_number'} = $1;
            } else {
                $Res[0]->{'dimm_info'}->[$j]->{'dimm_number'} = $1;
            }
        } elsif ( $value =~ /^(\w+):\s+(\w+)\s+(\D+):\s+(.*)/ ) {
            if ($key) {
                $Res[0]->{ to_key_name( item => $key ) }->[0]->{'dimm_info'}
                    ->[$j]->{ to_key_name( item => $1 ) } = $2;
                $Res[0]->{ to_key_name( item => $key ) }->[0]->{'dimm_info'}
                    ->[$j]->{ to_key_name( item => $3 ) } = $4;
            } else {
                $Res[0]->{'dimm_info'}->[$j]->{ to_key_name( item => $1 ) }

                    = $2;
                $Res[0]->{'dimm_info'}->[$j]->{ to_key_name( item => $3 ) }
                    = $4;
            }
        }
    }
    return \@Res;
} ## end sub iomem_dimm_show

## @name ::iomem_dimm_spd_show
##
## @summary Parses the raw output from the "iomem dimm spd show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_dimm_spd_show with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_dimm_spd_show(%args);
## INPUTS TO METHOD CALL 'iomem_dimm_spd_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## <tr> <td> '-d' </td> <td> 'dimm-num' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem dimm spd show -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;------------------------- DIMM 1 -------------------------
##&nbsp;
##&nbsp;00000000 | 80 08 08 0e 0b 61 48 00 05 50 60 06 82 04 04 00 | .....aH..P`.....
##&nbsp;00000010 | 0c 08 18 03 10 09 03 50 60 00 00 3c 1e 3c 28 02 | .......P`..<.<(.
##&nbsp;00000020 | 35 47 15 27 3c 28 1e 00 06 37 7f 80 23 2d 0f 00 | 5G.'<(...7..#-..
##&nbsp;00000030 | 00 00 00 00 00 00 00 00 00 00 00 00 00 00 12 39 | ...............9
##&nbsp;00000040 | 7f 94 00 00 00 00 00 00 01 53 47 35 37 32 31 32 | .........SG57212
##&nbsp;00000050 | 34 41 47 38 52 57 44 42 4d 45 20 01 00 07 23 00 | 4AG8RWDBME ...#.
##&nbsp;00000060 | 01 cb 53 53 4d 41 52 54 4d 6f 64 75 6c 61 72 54 | ..SSMARTModularT
##&nbsp;00000070 | 65 63 68 6e 6f 6c 6f 67 69 65 73 20 20 20 00 00 | echnologies   ..
##&nbsp;
##&nbsp;------------------------- DIMM 2 -------------------------
##&nbsp;
##&nbsp;00000000 | 80 08 08 0e 0b 61 48 00 05 50 60 06 82 04 04 00 | .....aH..P`.....
##&nbsp;00000010 | 0c 08 18 03 10 09 03 50 60 00 00 3c 1e 3c 28 02 | .......P`..<.<(.
##&nbsp;00000020 | 35 47 15 27 3c 28 1e 00 06 37 7f 80 23 2d 0f 00 | 5G.'<(...7..#-..
##&nbsp;00000030 | 00 00 00 00 00 00 00 00 00 00 00 00 00 00 12 39 | ...............9
##&nbsp;00000040 | 7f 94 00 00 00 00 00 00 01 53 47 35 37 32 31 32 | .........SG57212
##&nbsp;00000050 | 34 41 47 38 52 57 44 42 4d 45 20 01 00 07 23 00 | 4AG8RWDBME ...#.
##&nbsp;00000060 | 01 cb 54 53 4d 41 52 54 4d 6f 64 75 6c 61 72 54 | ..TSMARTModularT
##&nbsp;00000070 | 65 63 68 6e 6f 6c 6f 67 69 65 73 20 20 20 00 00 | echnologies   ..
##&nbsp;
##&nbsp;------------------------- DIMM 3 -------------------------
##&nbsp;
##&nbsp;00000000 | 80 08 08 0e 0b 61 48 00 05 50 60 06 82 04 04 00 | .....aH..P`.....
##&nbsp;00000010 | 0c 08 18 03 10 09 03 50 60 00 00 3c 1e 3c 28 02 | .......P`..<.<(.
##&nbsp;00000020 | 35 47 15 27 3c 28 1e 00 06 37 7f 80 23 2d 0f 00 | 5G.'<(...7..#-..
##&nbsp;00000030 | 00 00 00 00 00 00 00 00 00 00 00 00 00 00 12 39 | ...............9
##&nbsp;00000040 | 7f 94 00 00 00 00 00 00 01 53 47 35 37 32 31 32 | .........SG57212
##&nbsp;00000050 | 34 41 47 38 52 57 44 42 4d 45 20 01 00 07 23 00 | 4AG8RWDBME ...#.
##&nbsp;00000060 | 01 cb 4d 53 4d 41 52 54 4d 6f 64 75 6c 61 72 54 | ..MSMARTModularT
##&nbsp;00000070 | 65 63 68 6e 6f 6c 6f 67 69 65 73 20 20 20 00 00 | echnologies   ..
##&nbsp;
##&nbsp;------------------------- DIMM 4 -------------------------
##&nbsp;
##&nbsp;00000000 | 80 08 08 0e 0b 61 48 00 05 50 60 06 82 04 04 00 | .....aH..P`.....
##&nbsp;00000010 | 0c 08 18 03 10 09 03 50 60 00 00 3c 1e 3c 28 02 | .......P`..<.<(.
##&nbsp;00000020 | 35 47 15 27 3c 28 1e 00 06 37 7f 80 23 2d 0f 00 | 5G.'<(...7..#-..
##&nbsp;00000030 | 00 00 00 00 00 00 00 00 00 00 00 00 00 00 12 39 | ...............9
##&nbsp;00000040 | 7f 94 00 00 00 00 00 00 01 53 47 35 37 32 31 32 | .........SG57212
##&nbsp;00000050 | 34 41 47 38 52 57 44 42 4d 45 20 01 00 07 23 00 | 4AG8RWDBME ...#.
##&nbsp;00000060 | 01 cb 52 53 4d 41 52 54 4d 6f 64 75 6c 61 72 54 | ..RSMARTModularT
##&nbsp;00000070 | 65 63 68 6e 6f 6c 6f 67 69 65 73 20 20 20 00 00 | echnologies   ..
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'dimm_4' => [
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' .....aH..P`.....',
##&nbsp;                            '64bit_info' => '80 08 08 0e 0b 61 48 00 05 50 60 06 82 04 04 00',
##&nbsp;                            'address' => '00000000'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' .......P`..<.<(.',
##&nbsp;                            '64bit_info' => '0c 08 18 03 10 09 03 50 60 00 00 3c 1e 3c 28 02',
##&nbsp;                            'address' => '00000010'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' 5G.\'<(...7..#-..',
##&nbsp;                            '64bit_info' => '35 47 15 27 3c 28 1e 00 06 37 7f 80 23 2d 0f 00',
##&nbsp;                            'address' => '00000020'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' ...............9',
##&nbsp;                            '64bit_info' => '00 00 00 00 00 00 00 00 00 00 00 00 00 00 12 39',
##&nbsp;                            'address' => '00000030'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' .........SG57212',
##&nbsp;                            '64bit_info' => '7f 94 00 00 00 00 00 00 01 53 47 35 37 32 31 32',
##&nbsp;                            'address' => '00000040'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' 4AG8RWDBME ...#.',
##&nbsp;                            '64bit_info' => '34 41 47 38 52 57 44 42 4d 45 20 01 00 07 23 00',
##&nbsp;                            'address' => '00000050'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' ..RSMARTModularT',
##&nbsp;                            '64bit_info' => '01 cb 52 53 4d 41 52 54 4d 6f 64 75 6c 61 72 54',
##&nbsp;                            'address' => '00000060'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' echnologies   ..',
##&nbsp;                            '64bit_info' => '65 63 68 6e 6f 6c 6f 67 69 65 73 20 20 20 00 00',
##&nbsp;                            'address' => '00000070'
##&nbsp;                          }
##&nbsp;                        ],
##&nbsp;            'dimm_2' => [
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' .....aH..P`.....',
##&nbsp;                            '64bit_info' => '80 08 08 0e 0b 61 48 00 05 50 60 06 82 04 04 00',
##&nbsp;                            'address' => '00000000'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' .......P`..<.<(.',
##&nbsp;                            '64bit_info' => '0c 08 18 03 10 09 03 50 60 00 00 3c 1e 3c 28 02',
##&nbsp;                            'address' => '00000010'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' 5G.\'<(...7..#-..',
##&nbsp;                            '64bit_info' => '35 47 15 27 3c 28 1e 00 06 37 7f 80 23 2d 0f 00',
##&nbsp;                            'address' => '00000020'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' ...............9',
##&nbsp;                            '64bit_info' => '00 00 00 00 00 00 00 00 00 00 00 00 00 00 12 39',
##&nbsp;                            'address' => '00000030'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' .........SG57212',
##&nbsp;                            '64bit_info' => '7f 94 00 00 00 00 00 00 01 53 47 35 37 32 31 32',
##&nbsp;                            'address' => '00000040'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' 4AG8RWDBME ...#.',
##&nbsp;                            '64bit_info' => '34 41 47 38 52 57 44 42 4d 45 20 01 00 07 23 00',
##&nbsp;                            'address' => '00000050'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' ..TSMARTModularT',
##&nbsp;                            '64bit_info' => '01 cb 54 53 4d 41 52 54 4d 6f 64 75 6c 61 72 54',
##&nbsp;                            'address' => '00000060'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' echnologies   ..',
##&nbsp;                            '64bit_info' => '65 63 68 6e 6f 6c 6f 67 69 65 73 20 20 20 00 00',
##&nbsp;                            'address' => '00000070'
##&nbsp;                          }
##&nbsp;                        ],
##&nbsp;            'dimm_3' => [
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' .....aH..P`.....',
##&nbsp;                            '64bit_info' => '80 08 08 0e 0b 61 48 00 05 50 60 06 82 04 04 00',
##&nbsp;                            'address' => '00000000'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' .......P`..<.<(.',
##&nbsp;                            '64bit_info' => '0c 08 18 03 10 09 03 50 60 00 00 3c 1e 3c 28 02',
##&nbsp;                            'address' => '00000010'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' 5G.\'<(...7..#-..',
##&nbsp;                            '64bit_info' => '35 47 15 27 3c 28 1e 00 06 37 7f 80 23 2d 0f 00',
##&nbsp;                            'address' => '00000020'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' ...............9',
##&nbsp;                            '64bit_info' => '00 00 00 00 00 00 00 00 00 00 00 00 00 00 12 39',
##&nbsp;                            'address' => '00000030'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' .........SG57212',
##&nbsp;                            '64bit_info' => '7f 94 00 00 00 00 00 00 01 53 47 35 37 32 31 32',
##&nbsp;                            'address' => '00000040'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' 4AG8RWDBME ...#.',
##&nbsp;                            '64bit_info' => '34 41 47 38 52 57 44 42 4d 45 20 01 00 07 23 00',
##&nbsp;                            'address' => '00000050'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' ..MSMARTModularT',
##&nbsp;                            '64bit_info' => '01 cb 4d 53 4d 41 52 54 4d 6f 64 75 6c 61 72 54',
##&nbsp;                            'address' => '00000060'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' echnologies   ..',
##&nbsp;                            '64bit_info' => '65 63 68 6e 6f 6c 6f 67 69 65 73 20 20 20 00 00',
##&nbsp;                            'address' => '00000070'
##&nbsp;                          }
##&nbsp;                        ],
##&nbsp;            'dimm_1' => [
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' .....aH..P`.....',
##&nbsp;                            '64bit_info' => '80 08 08 0e 0b 61 48 00 05 50 60 06 82 04 04 00',
##&nbsp;                            'address' => '00000000'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' .......P`..<.<(.',
##&nbsp;                            '64bit_info' => '0c 08 18 03 10 09 03 50 60 00 00 3c 1e 3c 28 02',
##&nbsp;                            'address' => '00000010'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' 5G.\'<(...7..#-..',
##&nbsp;                            '64bit_info' => '35 47 15 27 3c 28 1e 00 06 37 7f 80 23 2d 0f 00',
##&nbsp;                            'address' => '00000020'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' ...............9',
##&nbsp;                            '64bit_info' => '00 00 00 00 00 00 00 00 00 00 00 00 00 00 12 39',
##&nbsp;                            'address' => '00000030'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' .........SG57212',
##&nbsp;                            '64bit_info' => '7f 94 00 00 00 00 00 00 01 53 47 35 37 32 31 32',
##&nbsp;                            'address' => '00000040'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' 4AG8RWDBME ...#.',
##&nbsp;                            '64bit_info' => '34 41 47 38 52 57 44 42 4d 45 20 01 00 07 23 00',
##&nbsp;                            'address' => '00000050'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' ..SSMARTModularT',
##&nbsp;                            '64bit_info' => '01 cb 53 53 4d 41 52 54 4d 6f 64 75 6c 61 72 54',
##&nbsp;                            'address' => '00000060'
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'miscleneous' => ' echnologies   ..',
##&nbsp;                            '64bit_info' => '65 63 68 6e 6f 6c 6f 67 69 65 73 20 20 20 00 00',
##&nbsp;                            'address' => '00000070'
##&nbsp;                          }
##&nbsp;                        ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_dimm_spd_show {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $key, $key1 );
    my $j = -1;
    my $l = -1;
    foreach my $value (@lines) {
        if ( $value =~ /^(Slot\s+\d+):\s+(.*)$/ ) {
            $key = $1;
            $j   = -1;
            $Res[0]->{ to_key_name( item => $key ) }->[0]->{'card_name'} = $2;
        } elsif ( $value =~ /^[-]+\s+(\w+\s+\d+).*/ ) {
            $l    = -1;
            $key1 = $1;
            $j++;
        } elsif ( $value =~ /(\d+)\s+\|\s+(.*)\s+\|(.*)/ ) {
            if ($key) {
                $l++;

                $Res[0]->{ to_key_name( item => $key ) }->[0]
                    ->{ to_key_name( item => $key1 ) }->[$l]->{'address'}
                    = $1;
                $Res[0]->{ to_key_name( item => $key ) }->[0]
                    ->{ to_key_name( item => $key1 ) }->[$l]->{'64bit_info'}
                    = $2;
                $Res[0]->{ to_key_name( item => $key ) }->[0]
                    ->{ to_key_name( item => $key1 ) }->[$l]->{'miscleneous'}
                    = $3;
            } else {
                $l++;
                $Res[0]->{ to_key_name( item => $key1 ) }->[$l]->{'address'}
                    = $1;
                $Res[0]->{ to_key_name( item => $key1 ) }->[$l]
                    ->{'64bit_info'} = $2;
                $Res[0]->{ to_key_name( item => $key1 ) }->[$l]
                    ->{'miscleneous'} = $3;
            }
        }
    }
    return \@Res;
} ## end sub iomem_dimm_spd_show

## @name ::iomem_dimm_log_show
##
## @summary Parses the raw output from the "iomem dimm log show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_dimm_log_show with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_dimm_log_show(%args);
## INPUTS TO METHOD CALL 'iomem_dimm_log_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-V' </td> <td> 'verbrose' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## <tr> <td> '-d' </td> <td> 'dimm-num' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem dimm log show -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------------------------- DIMM 1 -------------------------
##&nbsp;
##&nbsp;    Log Checksum: 0xa9cf1057    UECC Errors: 0
##&nbsp;        Log Size: 128           CECC Errors: 0
##&nbsp;  Format Version: 1           System Serial: 0087438
##&nbsp;           Flags: 0x0             Card Slot: 2
##&nbsp;     DIMM Number: 1              Card Flags: 0x00
##&nbsp;  Log Generation: 0           PCI Device ID: 0x774a
##&nbsp;First Error Time: 0              FPGA Build: 0x200706131558
##&nbsp; Last Error Time: 0             Card Serial: EP NET and more
##&nbsp;
##&nbsp;No logged uncorrectable errors.
##&nbsp;
##&nbsp;No logged correctable errors.
##&nbsp;
##&nbsp;-------------------------- DIMM 2 -------------------------
##&nbsp;
##&nbsp;    Log Checksum: 0x47a9134e    UECC Errors: 0
##&nbsp;        Log Size: 128           CECC Errors: 0
##&nbsp;  Format Version: 1           System Serial: 0087438
##&nbsp;           Flags: 0x0             Card Slot: 2
##&nbsp;     DIMM Number: 2              Card Flags: 0x00
##&nbsp;  Log Generation: 0           PCI Device ID: 0x774a
##&nbsp;First Error Time: 0              FPGA Build: 0x200706131558
##&nbsp; Last Error Time: 0             Card Serial: EP NET and more
##&nbsp;
##&nbsp;No logged uncorrectable errors.
##&nbsp;
##&nbsp;No logged correctable errors.
##&nbsp;
##&nbsp;-------------------------- DIMM 3 -------------------------
##&nbsp;
##&nbsp;    Log Checksum: 0xab5bef86    UECC Errors: 0
##&nbsp;        Log Size: 128           CECC Errors: 0
##&nbsp;  Format Version: 1           System Serial: 0087438
##&nbsp;           Flags: 0x0             Card Slot: 2
##&nbsp;     DIMM Number: 3              Card Flags: 0x00
##&nbsp;  Log Generation: 0           PCI Device ID: 0x774a
##&nbsp;First Error Time: 0              FPGA Build: 0x200706131558
##&nbsp; Last Error Time: 0             Card Serial: EP NET and more
##&nbsp;
##&nbsp;No logged uncorrectable errors.
##&nbsp;
##&nbsp;No logged correctable errors.
##&nbsp;
##&nbsp;-------------------------- DIMM 4 -------------------------
##&nbsp;
##&nbsp;    Log Checksum: 0x4014133d    UECC Errors: 0
##&nbsp;        Log Size: 128           CECC Errors: 0
##&nbsp;  Format Version: 1           System Serial: 0087438
##&nbsp;           Flags: 0x0             Card Slot: 2
##&nbsp;     DIMM Number: 4              Card Flags: 0x00
##&nbsp;  Log Generation: 0           PCI Device ID: 0x774a
##&nbsp;First Error Time: 0              FPGA Build: 0x200706131558
##&nbsp; Last Error Time: 0             Card Serial: EP NET and more
##&nbsp;
##&nbsp;No logged uncorrectable errors.
##&nbsp;
##&nbsp;No logged correctable errors.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'log_info' => [
##&nbsp;                            {
##&nbsp;                              'format_version' => '1',
##&nbsp;                              'log_checksum' => '0xa9cf1057',
##&nbsp;                              'error_info' => [
##&nbsp;                                                {
##&nbsp;                                                  'uecc_error' => [
##&nbsp;                                                                    {
##&nbsp;                                                                      'status' => 'No logged uncorrectable errors'
##&nbsp;                                                                    }
##&nbsp;                                                                  ]
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'cecc_error' => [
##&nbsp;                                                                    {
##&nbsp;                                                                      'status' => 'No logged correctable errors'
##&nbsp;                                                                    }
##&nbsp;                                                                  ]
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;                              'first_error_time' => '0',
##&nbsp;                              'last_error_time' => '0',
##&nbsp;                              'uecc_errors' => '0',
##&nbsp;                              'card_slot' => '2',
##&nbsp;                              'flags' => '0x0',
##&nbsp;                              'cecc_errors' => '0',
##&nbsp;                              'system_serial' => '0087438',
##&nbsp;                              'log_generation' => '0',
##&nbsp;                              'card_serial' => 'EP NET and more',
##&nbsp;                              'log_size' => '128',
##&nbsp;                              'card_flags' => '0x00',
##&nbsp;                              'fpga_build' => '0x200706131558',
##&nbsp;                              'dimm_number' => '1',
##&nbsp;                              'pci_device_id' => '0x774a'
##&nbsp;                            },
##&nbsp;                            {
##&nbsp;                              'format_version' => '1',
##&nbsp;                              'log_checksum' => '0x47a9134e',
##&nbsp;                              'error_info' => [
##&nbsp;                                                {
##&nbsp;                                                  'uecc_error' => [
##&nbsp;                                                                    {
##&nbsp;                                                                      'status' => 'No logged uncorrectable errors'
##&nbsp;                                                                    }
##&nbsp;                                                                  ]
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'cecc_error' => [
##&nbsp;                                                                    {
##&nbsp;                                                                      'status' => 'No logged correctable errors'
##&nbsp;                                                                    }
##&nbsp;                                                                  ]
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;                              'first_error_time' => '0',
##&nbsp;                              'last_error_time' => '0',
##&nbsp;                              'uecc_errors' => '0',
##&nbsp;                              'card_slot' => '2',
##&nbsp;                              'flags' => '0x0',
##&nbsp;                              'cecc_errors' => '0',
##&nbsp;                              'system_serial' => '0087438',
##&nbsp;                              'log_generation' => '0',
##&nbsp;                              'card_serial' => 'EP NET and more',
##&nbsp;                              'log_size' => '128',
##&nbsp;                              'card_flags' => '0x00',
##&nbsp;                              'fpga_build' => '0x200706131558',
##&nbsp;                              'dimm_number' => '2',
##&nbsp;                              'pci_device_id' => '0x774a'
##&nbsp;                            },
##&nbsp;                            {
##&nbsp;                              'format_version' => '1',
##&nbsp;                              'log_checksum' => '0xab5bef86',
##&nbsp;                              'error_info' => [
##&nbsp;                                                {
##&nbsp;                                                  'uecc_error' => [
##&nbsp;                                                                    {
##&nbsp;                                                                      'status' => 'No logged uncorrectable errors'
##&nbsp;                                                                    }
##&nbsp;                                                                  ]
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'cecc_error' => [
##&nbsp;                                                                    {
##&nbsp;                                                                      'status' => 'No logged correctable errors'
##&nbsp;                                                                    }
##&nbsp;                                                                  ]
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;                              'first_error_time' => '0',
##&nbsp;                              'last_error_time' => '0',
##&nbsp;                              'uecc_errors' => '0',
##&nbsp;                              'card_slot' => '2',
##&nbsp;                              'flags' => '0x0',
##&nbsp;                              'cecc_errors' => '0',
##&nbsp;                              'system_serial' => '0087438',
##&nbsp;                              'log_generation' => '0',
##&nbsp;                              'card_serial' => 'EP NET and more',
##&nbsp;                              'log_size' => '128',
##&nbsp;                              'card_flags' => '0x00',
##&nbsp;                              'fpga_build' => '0x200706131558',
##&nbsp;                              'dimm_number' => '3',
##&nbsp;                              'pci_device_id' => '0x774a'
##&nbsp;                            },
##&nbsp;                            {
##&nbsp;                              'format_version' => '1',
##&nbsp;                              'log_checksum' => '0x4014133d',
##&nbsp;                              'error_info' => [
##&nbsp;                                                {
##&nbsp;                                                  'uecc_error' => [
##&nbsp;                                                                    {
##&nbsp;                                                                      'status' => 'No logged uncorrectable errors'
##&nbsp;                                                                    }
##&nbsp;                                                                  ]
##&nbsp;                                                },
##&nbsp;                                                {
##&nbsp;                                                  'cecc_error' => [
##&nbsp;                                                                    {
##&nbsp;                                                                      'status' => 'No logged correctable errors'
##&nbsp;                                                                    }
##&nbsp;                                                                  ]
##&nbsp;                                                }
##&nbsp;                                              ],
##&nbsp;                              'first_error_time' => '0',
##&nbsp;                              'last_error_time' => '0',
##&nbsp;                              'uecc_errors' => '0',
##&nbsp;                              'card_slot' => '2',
##&nbsp;                              'flags' => '0x0',
##&nbsp;                              'cecc_errors' => '0',
##&nbsp;                              'system_serial' => '0087438',
##&nbsp;                              'log_generation' => '0',
##&nbsp;                              'card_serial' => 'EP NET and more',
##&nbsp;                              'log_size' => '128',
##&nbsp;                              'card_flags' => '0x00',
##&nbsp;                              'fpga_build' => '0x200706131558',
##&nbsp;                              'dimm_number' => '4',
##&nbsp;                              'pci_device_id' => '0x774a'
##&nbsp;                            }
##&nbsp;                          ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_dimm_log_show {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $key, @temp );
    my $j = -1;
    my $k = 0;
    my $l = -1;
    foreach my $value (@lines) {
        if ( $command =~ /-V/ ) {
            if ( $value =~ /^(Slot\s+\d+):\s+(.*)$/ ) {
                $j   = -1;
                $key = $1;
                $Res[0]->{ to_key_name( item => $key ) }->[0]->{'card_name'}
                    = $2;
            } elsif ( $value =~ /^[-]+\s+\w+\s+(\d+).*/ ) {
                $k = 0;
                $l = -1;
                $j++;
            } elsif ( $value =~ /^(\D+):\s+(\w+)\s+(\D+):\s+(.*)/ ) {
                if ($key) {
                    $Res[0]->{ to_key_name( item => $key ) }->[0]
                        ->{'log_info'}->[$j]->{ to_key_name( item => $1 ) }
                        = $2;
                    $Res[0]->{ to_key_name( item => $key ) }->[0]
                        ->{'log_info'}->[$j]->{ to_key_name( item => $3 ) }
                        = $4;
                } else {
                    $Res[0]->{'log_info'}->[$j]->{ to_key_name( item => $1 ) }
                        = $2;
                    $Res[0]->{'log_info'}->[$j]->{ to_key_name( item => $3 ) }
                        = $4;
                }
            } elsif ( $value
                =~ /^(\D+\d+):\s+(\w+)=(\d+)\s+(\w+)=(\d+)\s+(\w+)=(\d+)\s+(\w+)=(\d+)/
                ) {
                if ( $1 =~ /UECC Error.*/ ) {
                    if ($key) {
                        $Res[0]->{ to_key_name( item => $key ) }->[0]
                            ->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $1 ) } = $2;
                        $Res[0]->{ to_key_name( item => $key ) }->[0]
                            ->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $3 ) } = $4;
                        $Res[0]->{ to_key_name( item => $key ) }->[0]
                            ->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $5 ) } = $6;
                        $Res[0]->{ to_key_name( item => $key ) }->[0]
                            ->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $7 ) } = $8;
                    } else {
                        $Res[0]->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $1 ) } = $2;
                        $Res[0]->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $3 ) } = $4;
                        $Res[0]->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $5 ) } = $6;
                        $Res[0]->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $7 ) } = $8;
                    }
                } elsif ( $1 =~ /CECC Error.*/ ) {
                    if ($key) {
                        $Res[0]->{ to_key_name( item => $key ) }->[0]
                            ->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'cecc_error'}->[$k]
                            ->{ to_key_name( item => $1 ) } = $2;
                        $Res[0]->{ to_key_name( item => $key ) }->[0]
                            ->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'cecc_error'}->[$k]
                            ->{ to_key_name( item => $3 ) } = $4;
                        $Res[0]->{ to_key_name( item => $key ) }->[0]
                            ->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'cecc_error'}->[$k]
                            ->{ to_key_name( item => $5 ) } = $6;
                        $Res[0]->{ to_key_name( item => $key ) }->[0]
                            ->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'cecc_error'}->[$k]
                            ->{ to_key_name( item => $7 ) } = $8;
                    } else {
                        $Res[0]->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $1 ) } = $2;
                        $Res[0]->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $3 ) } = $4;
                        $Res[0]->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $5 ) } = $6;
                        $Res[0]->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $7 ) } = $8;
                    }
                }
                $k++;
            } elsif ( $value =~ /(No logged uncorrectable errors)/ ) {
                if ($key) {
                    $Res[0]->{ to_key_name( item => $key ) }->[0]
                        ->{'log_info'}->[$j]->{'error_info'}->[0]
                        ->{'uecc_error'}->[0]->{'status'} = $1;
                } else {
                    $Res[0]->{'log_info'}->[$j]->{'error_info'}->[0]
                        ->{'uecc_error'}->[0]->{'status'} = $1;
                }
            } elsif ( $value =~ /(No logged correctable errors)/ ) {
                if ($key) {
                    $Res[0]->{ to_key_name( item => $key ) }->[0]
                        ->{'log_info'}->[$j]->{'error_info'}->[$k]
                        ->{'cecc_error'}->[0]->{'status'} = $1;
                } else {
                    $Res[0]->{'log_info'}->[$j]->{'error_info'}->[$k]
                        ->{'cecc_error'}->[0]->{'status'} = $1;
                }

            } elsif ( $value =~ /(\d+)\s+\|\s+(.*)\s+\|(.*)/ ) {
                if ($key) {
                    $l++;
                    $Res[0]->{ to_key_name( item => $key ) }->[0]
                        ->{'log_info'}->[$j]->{'verbrose_info'}->[$l]
                        ->{'address'} = $1;
                    $Res[0]->{ to_key_name( item => $key ) }->[0]
                        ->{'log_info'}->[$j]->{'verbrose_info'}->[$l]
                        ->{'64bit_info'} = $2;
                    $Res[0]->{ to_key_name( item => $key ) }->[0]
                        ->{'log_info'}->[$j]->{'verbrose_info'}->[$l]
                        ->{'miscleneous'} = $3;
                } else {
                    $l++;
                    $Res[0]->{'log_info'}->[$j]->{'verbrose_info'}->[$l]
                        ->{'address'} = $1;
                    $Res[0]->{'log_info'}->[$j]->{'verbrose_info'}->[$l]
                        ->{'64bit_info'} = $2;
                    $Res[0]->{'log_info'}->[$j]->{'verbrose_info'}->[$l]
                        ->{'miscleneous'} = $3;
                }
            }
        } else {
            if ( $value =~ /^(Slot\s+\d+):\s+(.*)$/ ) {

                $key = $1;
                $Res[0]->{ to_key_name( item => $key ) }->[0]->{'card_name'}
                    = $2;
            } elsif ( $value =~ /^[-]+\s+\w+\s+(\d+).*/ ) {
                $k = 0;
                $j++;
            } elsif ( $value =~ /^(\D+):\s+(\w+)\s+(\D+):\s+(.*)/ ) {
                if ($key) {
                    $Res[0]->{ to_key_name( item => $key ) }->[0]
                        ->{'log_info'}->[$j]->{ to_key_name( item => $1 ) }
                        = $2;
                    $Res[0]->{ to_key_name( item => $key ) }->[0]
                        ->{'log_info'}->[$j]->{ to_key_name( item => $3 ) }
                        = $4;
                } else {
                    $Res[0]->{'log_info'}->[$j]->{ to_key_name( item => $1 ) }
                        = $2;
                    $Res[0]->{'log_info'}->[$j]->{ to_key_name( item => $3 ) }
                        = $4;
                }
            } elsif ( $value
                =~ /^(\D+\d+):\s+(\w+)=(\d+)\s+(\w+)=(\d+)\s+(\w+)=(\d+)\s+(\w+)=(\d+)/
                ) {
                if ( $1 =~ /UECC Error.*/ ) {
                    if ($key) {
                        $Res[0]->{ to_key_name( item => $key ) }->[0]
                            ->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $1 ) } = $2;
                        $Res[0]->{ to_key_name( item => $key ) }->[0]
                            ->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $3 ) } = $4;
                        $Res[0]->{ to_key_name( item => $key ) }->[0]
                            ->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $5 ) } = $6;
                        $Res[0]->{ to_key_name( item => $key ) }->[0]
                            ->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $7 ) } = $8;
                    } else {
                        $Res[0]->{'log_info'}->[$j]->{'error_info'}->[0]

                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $1 ) } = $2;
                        $Res[0]->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $3 ) } = $4;
                        $Res[0]->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $5 ) } = $6;
                        $Res[0]->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $7 ) } = $8;
                    }
                } elsif ( $1 =~ /CECC Error.*/ ) {
                    if ($key) {
                        $Res[0]->{ to_key_name( item => $key ) }->[0]
                            ->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'cecc_error'}->[$k]
                            ->{ to_key_name( item => $1 ) } = $2;
                        $Res[0]->{ to_key_name( item => $key ) }->[0]
                            ->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'cecc_error'}->[$k]
                            ->{ to_key_name( item => $3 ) } = $4;
                        $Res[0]->{ to_key_name( item => $key ) }->[0]
                            ->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'cecc_error'}->[$k]
                            ->{ to_key_name( item => $5 ) } = $6;
                        $Res[0]->{ to_key_name( item => $key ) }->[0]
                            ->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'cecc_error'}->[$k]
                            ->{ to_key_name( item => $7 ) } = $8;
                    } else {
                        $Res[0]->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $1 ) } = $2;
                        $Res[0]->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $3 ) } = $4;
                        $Res[0]->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $5 ) } = $6;
                        $Res[0]->{'log_info'}->[$j]->{'error_info'}->[0]
                            ->{'uecc_error'}->[$k]
                            ->{ to_key_name( item => $7 ) } = $8;
                    }
                }
                $k++;
            } elsif ( $value =~ /(No logged uncorrectable errors)/ ) {
                if ($key) {
                    $Res[0]->{ to_key_name( item => $key ) }->[0]
                        ->{'log_info'}->[$j]->{'error_info'}->[0]
                        ->{'uecc_error'}->[0]->{'status'} = $1;
                } else {
                    $Res[0]->{'log_info'}->[$j]->{'error_info'}->[0]
                        ->{'uecc_error'}->[0]->{'status'} = $1;
                }
                $k++;
            } elsif ( $value =~ /(No logged correctable errors)/ ) {
                if ($key) {
                    $Res[0]->{ to_key_name( item => $key ) }->[0]
                        ->{'log_info'}->[$j]->{'error_info'}->[$k]
                        ->{'cecc_error'}->[0]->{'status'} = $1;
                } else {
                    $Res[0]->{'log_info'}->[$j]->{'error_info'}->[$k]
                        ->{'cecc_error'}->[0]->{'status'} = $1;
                }
            }
        }
    }
    return \@Res;
} ## end sub iomem_dimm_log_show
## @name ::iomem_dimm_log_sync
##
## @summary Parses the raw output from the "iomem dimm log sync" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_dimm_log_sync with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_dimm_log_sync(%args);
## INPUTS TO METHOD CALL 'iomem_dimm_log_sync' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem dimm log sync -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Synced DIMM logs successfully.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'Synced DIMM logs successfully.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_dimm_log_sync {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );

    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $key );
    foreach my $value (@lines) {
        if ( $value =~ /^(Slot\s+\d+):\s+(.*)$/ ) {
            $key = $1;
            $Res[0]->{ to_key_name( item => $key ) }->[0]->{'card_name'} = $2;
        } elsif ( $value =~ /^(Synced.*)/ ) {
            if ($key) {
                $Res[0]->{ to_key_name( item => $key ) }->[0]->{'status'}
                    = $1;
            } else {
                $Res[0]->{'status'} = $1;
            }
        }
    }
    return \@Res;
} ## end sub iomem_dimm_log_sync

## @name ::iomem_dimm_log_reset
##
## @summary Parses the raw output from the "iomem dimm log reset" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_dimm_log_reset with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_dimm_log_reset(%args);
## INPUTS TO METHOD CALL 'iomem_dimm_log_reset' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## <tr> <td> '-d' </td> <td> 'dimm-num' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem dimm log reset -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Reset DIMM 1 error log successfully.
##&nbsp;Reset DIMM 2 error log successfully.
##&nbsp;Reset DIMM 3 error log successfully.
##&nbsp;Reset DIMM 4 error log successfully.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'log_info' => [
##&nbsp;                            {
##&nbsp;                              'status' => 'error log successfully.',
##&nbsp;                              'dimm_number' => '1'
##&nbsp;                            },
##&nbsp;                            {
##&nbsp;                              'status' => 'error log successfully.',
##&nbsp;                              'dimm_number' => '2'
##&nbsp;                            },
##&nbsp;                            {
##&nbsp;                              'status' => 'error log successfully.',
##&nbsp;                              'dimm_number' => '3'
##&nbsp;                            },
##&nbsp;                            {
##&nbsp;                              'status' => 'error log successfully.',
##&nbsp;                              'dimm_number' => '4'
##&nbsp;                            }
##&nbsp;                          ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_dimm_log_reset {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $key );

    my $i = -1;
    my $j = -1;
    foreach my $value (@lines) {
        if ( $value =~ /^(Slot\s+\d+):\s+(.*)$/ ) {
            $key = $1;
            $i   = -1;
            $j   = -1;
            $Res[0]->{ to_key_name( item => $key ) }->[0]->{'card_name'} = $2;
        } elsif ( $value =~ /^Reset\s+\w+\s+(\d+)\s+(.*)/ ) {
            $j++;
            $i++ if ( $i == -1 );
            if ($key) {
                $Res[0]->{ to_key_name( item => $key ) }->[$i]->{'log_info'}
                    ->[$j]->{'status'} = $2;
                $Res[0]->{ to_key_name( item => $key ) }->[$i]->{'log_info'}
                    ->[$j]->{'dimm_number'} = $1;
            } else {
                $Res[$i]->{'log_info'}->[$j]->{'status'}      = $2;
                $Res[$i]->{'log_info'}->[$j]->{'dimm_number'} = $1;
            }
        }
    }
    return \@Res;
} ## end sub iomem_dimm_log_reset

## @name ::iomem_dma_read
##
## @summary Parses the raw output from the "iomem dma read" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_dma_read with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_dma_read(%args);
## INPUTS TO METHOD CALL 'iomem_dma_read' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-n' </td> <td> 'size-or-offset' </td> </tr>
## <tr> <td> '-b' </td> <td> 'buffer' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> </tr>
## <tr> <td> '-d' </td> <td> 'channel' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'crc-data' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem dma read -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;DMA read complete.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'DMA read complete.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q diag; iomem dma read -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;DMA read complete.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'DMA read complete.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_dma_read {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );

    my ( @Res, $key );
    foreach my $value (@lines) {
        if ( $value =~ /^(Slot\s+\d+):\s+(.*)$/ ) {
            $key = $1;
            $Res[0]->{ to_key_name( item => $key ) }->[0]->{'card_name'} = $2;
        } elsif ( $value =~ /^(DMA.*)/ ) {
            if ($key) {
                $Res[0]->{ to_key_name( item => $key ) }->[0]->{'status'}
                    = $1;
            } else {
                $Res[0]->{'status'} = $1;
            }
        }
    }
    return \@Res;
} ## end sub iomem_dma_read

## @name ::iomem_dma_write
##
## @summary Parses the raw output from the "iomem dma write" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_dma_write with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_dma_write(%args);
## INPUTS TO METHOD CALL 'iomem_dma_write' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-n' </td> <td> 'size-or-offset' </td> </tr>
## <tr> <td> '-b' </td> <td> 'buffer' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> </tr>
## <tr> <td> '-d' </td> <td> 'channel' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'crc-data' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem dma write -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;DMA write complete.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'DMA write complete.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_dma_write {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $key );
    foreach my $value (@lines) {
        if ( $value =~ /^(Slot\s+\d+):\s+(.*)$/ ) {
            $key = $1;
            $Res[0]->{ to_key_name( item => $key ) }->[0]->{'card_name'} = $2;
        } elsif ( $value =~ /^(DMA.*)/ ) {
            if ($key) {

                $Res[0]->{ to_key_name( item => $key ) }->[0]->{'status'}
                    = $1;
            } else {
                $Res[0]->{'status'} = $1;
            }
        }
    }
    return \@Res;
} ## end sub iomem_dma_write

## @name ::iomem_dma_fill
##
## @summary Parses the raw output from the "iomem dma fill" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_dma_fill with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_dma_fill(%args);
## INPUTS TO METHOD CALL 'iomem_dma_fill' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-n' </td> <td> 'size-or-offset' </td> </tr>
## <tr> <td> '-v' </td> <td> 'patt-or-value' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> </tr>
## <tr> <td> '-d' </td> <td> 'channel' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem dma fill -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;DMA fill complete.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'DMA fill complete.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_dma_fill {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $key );
    foreach my $value (@lines) {
        if ( $value =~ /^(Slot\s+\d+):\s+(.*)$/ ) {
            $key = $1;
            $Res[0]->{ to_key_name( item => $key ) }->[0]->{'card_name'} = $2;
        } elsif ( $value =~ /^(DMA.*)/ ) {
            if ($key) {
                $Res[0]->{ to_key_name( item => $key ) }->[0]->{'status'}
                    = $1;
            } else {
                $Res[0]->{'status'} = $1;
            }
        }
    }
    return \@Res;
} ## end sub iomem_dma_fill

## @name ::iomem_dma_lread
##
## @summary Parses the raw output from the "iomem dma lread" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_dma_lread with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_dma_lread(%args);
## INPUTS TO METHOD CALL 'iomem_dma_lread' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-n' </td> <td> 'size-or-offset' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> </tr>
## <tr> <td> '-d' </td> <td> 'channel' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem dma lread -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Local DMA read complete.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'Local DMA read complete.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_dma_lread {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $key );
    foreach my $value (@lines) {
        if ( $value =~ /^(Slot\s+\d+):\s+(.*)$/ ) {
            $key = $1;
            $Res[0]->{ to_key_name( item => $key ) }->[0]->{'card_name'} = $2;
        } elsif ( $value =~ /^(Local.*)/ ) {
            if ($key) {
                $Res[0]->{ to_key_name( item => $key ) }->[0]->{'status'}
                    = $1;
            } else {
                $Res[0]->{'status'} = $1;
            }
        }
    }
    return \@Res;
} ## end sub iomem_dma_lread

## @name ::iomem_dma_pattern
##
## @summary Parses the raw output from the "iomem dma pattern" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_dma_pattern with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_dma_pattern(%args);
## INPUTS TO METHOD CALL 'iomem_dma_pattern' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## <tr> <td> '-s' </td> <td> 'seed' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem dma pattern -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Patterning 0x000000000 -> 0x040000000
##&nbsp;Patterning 0x040000000 -> 0x080000000
##&nbsp;Patterning 0x080000000 -> 0x0c0000000
##&nbsp;Patterning 0x0c0000000 -> 0x100000000
##&nbsp;Patterning 0x100000000 -> 0x140000000
##&nbsp;Patterning 0x140000000 -> 0x180000000
##&nbsp;Patterning 0x180000000 -> 0x1c0000000
##&nbsp;Patterning 0x1c0000000 -> 0x200000000
##&nbsp;Patterning 0x200000000 -> 0x240000000
##&nbsp;Patterning 0x240000000 -> 0x280000000
##&nbsp;Patterning 0x280000000 -> 0x2c0000000
##&nbsp;Patterning 0x2c0000000 -> 0x300000000
##&nbsp;Patterning 0x300000000 -> 0x340000000
##&nbsp;Patterning 0x340000000 -> 0x380000000
##&nbsp;Patterning 0x380000000 -> 0x3c0000000
##&nbsp;Patterning 0x3c0000000 -> 0x400000000
##&nbsp;Patterning complete.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'patterning' => [
##&nbsp;                            '0x000000000 -> 0x040000000',
##&nbsp;                            '0x040000000 -> 0x080000000',
##&nbsp;                            '0x080000000 -> 0x0c0000000',
##&nbsp;                            '0x0c0000000 -> 0x100000000',
##&nbsp;                            '0x100000000 -> 0x140000000',
##&nbsp;                            '0x140000000 -> 0x180000000',
##&nbsp;                            '0x180000000 -> 0x1c0000000',
##&nbsp;                            '0x1c0000000 -> 0x200000000',
##&nbsp;                            '0x200000000 -> 0x240000000',
##&nbsp;                            '0x240000000 -> 0x280000000',
##&nbsp;                            '0x280000000 -> 0x2c0000000',
##&nbsp;                            '0x2c0000000 -> 0x300000000',
##&nbsp;                            '0x300000000 -> 0x340000000',
##&nbsp;                            '0x340000000 -> 0x380000000',
##&nbsp;                            '0x380000000 -> 0x3c0000000',
##&nbsp;                            '0x3c0000000 -> 0x400000000'
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub iomem_dma_pattern {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @temp, @Res, $key, $key1 );
    my $i = -1;
    foreach my $value (@lines) {
        if ( $value =~ /^(Slot\s+\d+):\s+(.*)$/ ) {
            $key = $1;
            $i   = -1;
            $Res[0]->{ to_key_name( item => $key ) }->[0]->{'card_name'} = $2;
        } elsif ( $value =~ /^(\w+.*complete).*/ ) {
            next;
        } elsif ( $value =~ /^(\w+)\s+(.*)/ ) {
            $key1 = $1;
            $i++;
            $temp[$i] = $2;
        }
    }
    if ($key) {
        $Res[0]->{ to_key_name( item => $key ) }->[0]
            ->{ to_key_name( item => $key1 ) } = \@temp;
    } else {
        $Res[0]->{ to_key_name( item => $key1 ) } = \@temp;
    }
    return \@Res;
} ## end sub iomem_dma_pattern

## @name ::iomem_dma_zero
##
## @summary Parses the raw output from the "iomem dma zero" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_dma_zero with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_dma_zero(%args);
## INPUTS TO METHOD CALL 'iomem_dma_zero' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-v' </td> <td> 'patt-or-value' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem dma zero -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Process 0x000000000 -> 0x040000000
##&nbsp;Process 0x040000000 -> 0x080000000
##&nbsp;Process 0x080000000 -> 0x0c0000000
##&nbsp;Process 0x0c0000000 -> 0x100000000
##&nbsp;Process 0x100000000 -> 0x140000000
##&nbsp;Process 0x140000000 -> 0x180000000
##&nbsp;Process 0x180000000 -> 0x1c0000000
##&nbsp;Process 0x1c0000000 -> 0x200000000
##&nbsp;Process 0x200000000 -> 0x240000000
##&nbsp;Process 0x240000000 -> 0x280000000
##&nbsp;Process 0x280000000 -> 0x2c0000000
##&nbsp;Process 0x2c0000000 -> 0x300000000
##&nbsp;Process 0x300000000 -> 0x340000000
##&nbsp;Process 0x340000000 -> 0x380000000
##&nbsp;Process 0x380000000 -> 0x3c0000000
##&nbsp;Process 0x3c0000000 -> 0x400000000
##&nbsp;Zeroing complete.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'process' => [
##&nbsp;                           '0x000000000 -> 0x040000000',
##&nbsp;                           '0x040000000 -> 0x080000000',
##&nbsp;                           '0x080000000 -> 0x0c0000000',
##&nbsp;                           '0x0c0000000 -> 0x100000000',
##&nbsp;                           '0x100000000 -> 0x140000000',
##&nbsp;                           '0x140000000 -> 0x180000000',
##&nbsp;                           '0x180000000 -> 0x1c0000000',
##&nbsp;                           '0x1c0000000 -> 0x200000000',
##&nbsp;                           '0x200000000 -> 0x240000000',
##&nbsp;                           '0x240000000 -> 0x280000000',
##&nbsp;                           '0x280000000 -> 0x2c0000000',
##&nbsp;                           '0x2c0000000 -> 0x300000000',
##&nbsp;                           '0x300000000 -> 0x340000000',
##&nbsp;                           '0x340000000 -> 0x380000000',
##&nbsp;                           '0x380000000 -> 0x3c0000000',
##&nbsp;                           '0x3c0000000 -> 0x400000000'
##&nbsp;                         ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_dma_zero {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my %info    = ();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    #Now we can parse the output
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my ( @Res, $key, $key1, @temp );
    my $i = -1;
    foreach my $value (@lines) {
        if ( $value =~ /^(Slot\s+\d+):\s+(.*)$/ ) {
            $key = $1;
            $i   = -1;
            $Res[0]->{ to_key_name( item => $key ) }->[0]->{'card_name'} = $2;
        } elsif ( $value =~ /^(Process)\s+(.*)[.]?$/ ) {
            $key1 = $1;
            $i++;
            $temp[$i] = $2;
        }
    }
    if ($key) {
        $Res[0]->{ to_key_name( item => $key ) }->[0]
            ->{ to_key_name( item => $key1 ) } = \@temp;
    } else {
        $Res[0]->{ to_key_name( item => $key1 ) } = \@temp;
    }
    return \@Res;
} ## end sub iomem_dma_zero

## @name ::iomem_fw_show
##
## @summary Parses the raw_output from the "iomem fw show" command
##
## @description
##    This method can be used for parsing the raw_output obtained after calling
##    the method from iomem_fw_show with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_fw_show(%args);
## INPUTS TO METHOD CALL 'iomem_fw_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-slots' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw_output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed raw_output.
## @examples

##&nbsp;Command = priv set -q diag; iomem fw show -A; result -l
##&nbsp;
##&nbsp;-------- Parsed raw_output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot'  => '4',
##&nbsp;            'fpga_release' => '1.0',
##&nbsp;            'name' => 'Performance Acceleration Module I',
##&nbsp;            'fpga_build' => '200706131558',
##&nbsp;            'loaded_image' => 'Primary',
##&nbsp;            'fpga_type' => 'XC4VFX60'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_fw_show {
    my %args = @_;

    my @result_array = NACL::APISet::Node::CLI::7Mode::default::iomem::iomem_fpga_show(%args);

    return \@result_array;
}

## @name ::iomem_fw_reload
##
## @summary Parses the raw_output from the "iomem fw reload" command
##
## @description
##    This method can be used for parsing the raw_output obtained after calling
##    the method from iomem_fw_reload with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis

##      iomem_fw_reload(%args);
## INPUTS TO METHOD CALL 'iomem_fw_reload' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-D' </td> <td> 'disable-reload' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-slots' </td> </tr>
## <tr> <td> '-b' </td> <td> 'image' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw_output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed raw_output.
## @examples

##&nbsp;Command = priv set -q diag; iomem fw reload -A; result -l
##&nbsp;
##&nbsp;-------- Parsed raw_output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;             'slot' => '4',
##&nbsp;             'info' => 'FPGA will reload primary image on next reset.',
##&nbsp;             'name' => 'Performance Acceleration Module I'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_fw_reload {
    my %args = @_;

    my @result_array = NACL::APISet::Node::CLI::7Mode::default::iomem::iomem_fpga_reload(%args);

    return \@result_array;
}

## @name ::iomem_fw_update
##
## @summary Parses the raw_output from the "iomem fw update" command
##
## @description
##    This method can be used for parsing the raw_output obtained after calling
##    the method from iomem_fw_update with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_fw_update(%args);
## INPUTS TO METHOD CALL 'iomem_fw_update' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'build-id' </td> </tr>
## <tr> <td> '-t' </td> <td> 'target' </td> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-slots' </td> </tr>
## <tr> <td> '-f' </td> <td> 'image-path' </td> </tr>
## <tr> <td> '-F' </td> <td> 'force' </td> </tr>
## <tr> <td> '-w' </td> <td> 'fw-type' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw_output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed raw_output.
## @examples

##&nbsp;Command = priv set -q diag; iomem fw update -c 4 -f /vol/vol0/etc/firmware/zdi_fw.zpk -v 0x201012201350 -F; result -l
##&nbsp;
##&nbsp;-------- Parsed raw_output ------
##&nbsp;  $VAR1 = [
##&nbsp;           {
##&nbsp;             'info' => 'FPGA image uploaded successfully.',
##&nbsp;             'target_device' => 'PAMII_306-01649',
##&nbsp;             'version' => '2.2',
##&nbsp;             'build' => '0x201012201350',
##&nbsp;             'description' => 'Flash Cache Firmware',
##&nbsp;             'size' => '5380288',
##&nbsp;             'checksum' => '0x80a6f50e',
##&nbsp;             'file_name' => 'PAMII_306-01649_10_12_20.prom',
##&nbsp;             'flash_cache_firmware_upgrade' => [
##&nbsp;                                                 'slot4,phase1of4,0%complete',
##&nbsp;                                                 'slot4,phase1of4,14%complete',
##&nbsp;                                                 'slot4,phase1of4,28%complete',
##&nbsp;                                                 'slot4,phase1of4,71%complete',
##&nbsp;                                                 'slot4,phase1of4,85%complete',
##&nbsp;                                                 'slot4,phase2of4,0%complete',
##&nbsp;                                                 'slot4,phase2of4,5%complete',
##&nbsp;                                                 'slot4,phase2of4,7%complete',
##&nbsp;                                                 'slot4,phase2of4,7%complete',
##&nbsp;                                                 'slot4,phase2of4,8%complete',
##&nbsp;                                                 'slot4,phase2of4,70%complete',
##&nbsp;                                                 'slot4,phase2of4,91%complete',
##&nbsp;                                                 'slot4,phase3of4,35%complete',
##&nbsp;                                                 'slot4,phase3of4,50%complete',
##&nbsp;                                                 'slot4,phase3of4,64%complete',
##&nbsp;                                                 'slot4,phase3of4,78%complete',
##&nbsp;                                                 'slot4,phase3of4,92%complete',
##&nbsp;                                                 'slot4,phase4of4,3%complete',
##&nbsp;                                                 'slot4,phase4of4,80%complete',
##&nbsp;                                                 'slot4,phase4of4,100%complete'
##&nbsp;                                               ]
##&nbsp;           }
##&nbsp;         ];
sub iomem_fw_update {
    my %args = @_;

    my @result_array = NACL::APISet::Node::CLI::7Mode::default::iomem::iomem_fpga_update(%args);

    return \@result_array;
}

## @name ::iomem_fw_jam
##
## @summary Parses the raw_output from the "iomem fw jam" command
##
## @description
##    This method can be used for parsing the raw_output obtained after calling
##    the method from iomem_fw_jam with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_fw_jam(%args);
## INPUTS TO METHOD CALL 'iomem_fw_jam' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'build-id' </td> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-slots' </td> </tr>
## <tr> <td> '-f' </td> <td> 'image-path' </td> </tr>
## <tr> <td> '-j' </td> <td> 'action' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw_output, command line constructed and execution status)
## @return
## Returns an empty array reference.
sub iomem_fw_jam {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error      = _parse_for_errors(
        "raw_output"        => $raw_output,
        "command"           => 'iomem fw jam',
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    
    return [];
}

## @name ::iomem_fpga_show
##
## @summary Parses the raw_output from the "iomem fpga show" command
##
## @description
##    This method can be used for parsing the raw_output obtained after calling
##    the method from iomem_fpga_show with any of its options as described in the
##    corresponding cdef file 'iomem1.cdefs'
## @synopsis
##      iomem_fpga_show(%args);
## INPUTS TO METHOD CALL 'iomem_fpga_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-slots' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw_output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed raw_output.
## @examples

##&nbsp;Command = priv set -q diag; iomem fpga show -A; result -l
##&nbsp;
##&nbsp;-------- Parsed raw_output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot'  => '4',
##&nbsp;            'fpga_release' => '1.0',
##&nbsp;            'name' => 'Performance Acceleration Module I',
##&nbsp;            'fpga_build' => '200706131558',
##&nbsp;            'loaded_image' => 'Primary',
##&nbsp;            'fpga_type' => 'XC4VFX60'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_fpga_show {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error      = _parse_for_errors(
        "raw_output"        => $raw_output,
        "command"           => 'iomem fpga show',
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my ( @result_array, $key, $i );
    my @lines = get_lines(
        raw_output      => $raw_output,
        can_throw       => 0,
        response_object => $resp
    );
    $i = -1;
    if ( $command =~ /-A/ ) {
        foreach my $line (@lines) {
            if ( $line =~ /^Slot\s+(\d+):\s+(.*)/ ) {
                $i++;
                $result_array[$i]->{"slot"} = $1;
                $result_array[$i]->{"type"} = $2;
            } elsif ( $line =~ /([\w\s]+)\:\s+(.*)/ ) {
                $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
            }
        }
    } elsif ( $command =~ /-c/ ) {
        foreach my $line (@lines) {
            if ( $line =~ /([\w\s]+)\:\s+(.*)/ ) {
                $result_array[0]->{ to_key_name( item => $1 ) } = $2;
            }
        }
    }
    return \@result_array;
} ## end sub iomem_fpga_show

## @name ::iomem_fpga_reload
##
## @summary Parses the raw_output from the "iomem fpga reload" command
##
## @description
##    This method can be used for parsing the raw_output obtained after calling
##    the method from iomem_fpga_reload with any of its options as described in the
##    corresponding cdef file 'iomem1.cdefs'
## @synopsis

##      iomem_fpga_reload(%args);
## INPUTS TO METHOD CALL 'iomem_fpga_reload' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-D' </td> <td> 'disable-reload' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-slots' </td> </tr>
## <tr> <td> '-b' </td> <td> 'image' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw_output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed raw_output.
## @examples

##&nbsp;Command = priv set -q diag; iomem fpga reload -A; result -l
##&nbsp;
##&nbsp;-------- Parsed raw_output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;             'slot' => '4',
##&nbsp;             'info' => 'FPGA will reload primary image on next reset.',
##&nbsp;             'name' => 'Performance Acceleration Module I'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_fpga_reload {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error      = _parse_for_errors(
        "raw_output"        => $raw_output,
        "command"           => 'iomem fpga reload',
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my ( @result_array, $key, $i );
    my @lines = get_lines(
        raw_output => $raw_output,
        can_throw  => 0,

        response_object => $resp
    );
    $i = -1;
    if ( $command =~ /-A/ ) {
        foreach my $line (@lines) {
            if ( $line =~ /^Slot\s+(\d+):\s+(.*)/ ) {
                $i++;
                $result_array[$i]->{"slot"} = $1;
                $result_array[$i]->{"type"} = $2;
            } elsif ( $line =~ /(FPGA.*)/ ) {
                $result_array[$i]->{"info"} = $1;
            }
        }
    } elsif ( $command =~ /-c/ ) {
        foreach my $line (@lines) {
            if ( $line =~ /(FPGA.*)/ ) {
                $result_array[0]->{"info"} = $1;
            }
        }
    }
    return \@result_array;
} ## end sub iomem_fpga_reload

## @name ::iomem_fpga_update
##
## @summary Parses the raw_output from the "iomem fpga update" command
##
## @description
##    This method can be used for parsing the raw_output obtained after calling
##    the method from iomem_fpga_update with any of its options as described in the
##    corresponding cdef file 'iomem1.cdefs'
## @synopsis
##      iomem_fpga_update(%args);
## INPUTS TO METHOD CALL 'iomem_fpga_update' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-v' </td> <td> 'build-id' </td> </tr>
## <tr> <td> '-t' </td> <td> 'target' </td> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-slots' </td> </tr>
## <tr> <td> '-f' </td> <td> 'image-path' </td> </tr>
## <tr> <td> '-F' </td> <td> 'force' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw_output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed raw_output.
## @examples

##&nbsp;Command = priv set -q diag; iomem fpga update -c 4 -f /vol/vol0/etc/firmware/zdi_fw.zpk -v 0x201012201350 -F; result -l
##&nbsp;
##&nbsp;-------- Parsed raw_output ------
##&nbsp;  $VAR1 = [
##&nbsp;           {
##&nbsp;             'info' => 'FPGA image uploaded successfully.',
##&nbsp;             'target_device' => 'PAMII_306-01649',
##&nbsp;             'version' => '2.2',
##&nbsp;             'build' => '0x201012201350',
##&nbsp;             'description' => 'Flash Cache Firmware',
##&nbsp;             'size' => '5380288',
##&nbsp;             'checksum' => '0x80a6f50e',
##&nbsp;             'file_name' => 'PAMII_306-01649_10_12_20.prom',
##&nbsp;             'flash_cache_firmware_upgrade' => [
##&nbsp;                                                 'slot4,phase1of4,0%complete',
##&nbsp;                                                 'slot4,phase1of4,14%complete',
##&nbsp;                                                 'slot4,phase1of4,28%complete',
##&nbsp;                                                 'slot4,phase1of4,71%complete',
##&nbsp;                                                 'slot4,phase1of4,85%complete',
##&nbsp;                                                 'slot4,phase2of4,0%complete',
##&nbsp;                                                 'slot4,phase2of4,5%complete',
##&nbsp;                                                 'slot4,phase2of4,7%complete',
##&nbsp;                                                 'slot4,phase2of4,7%complete',
##&nbsp;                                                 'slot4,phase2of4,8%complete',
##&nbsp;                                                 'slot4,phase2of4,70%complete',
##&nbsp;                                                 'slot4,phase2of4,91%complete',
##&nbsp;                                                 'slot4,phase3of4,35%complete',
##&nbsp;                                                 'slot4,phase3of4,50%complete',
##&nbsp;                                                 'slot4,phase3of4,64%complete',
##&nbsp;                                                 'slot4,phase3of4,78%complete',
##&nbsp;                                                 'slot4,phase3of4,92%complete',
##&nbsp;                                                 'slot4,phase4of4,3%complete',
##&nbsp;                                                 'slot4,phase4of4,80%complete',
##&nbsp;                                                 'slot4,phase4of4,100%complete'
##&nbsp;                                               ]
##&nbsp;           }
##&nbsp;         ];

sub iomem_fpga_update {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error      = _parse_for_errors(
        "raw_output"        => $raw_output,
        "command"           => 'iomem fpga update',
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @result_array = ();
    my $i            = -1;
    my $j            = 0;
    my ( $val, @arr, $key );
    my @lines = get_lines(
        raw_output      => $raw_output,
        can_throw       => 0,
        response_object => $resp
    );
    my $opt = $command;
    if ( $command =~ /-A/ ) {

        foreach my $line (@lines) {
            if ( $line =~ /Slot (\d+):([\w\s]+)/ ) {
                $i++;
                $result_array[$i]->{"slot"} = $2;
                $result_array[$i]->{"type"} = to_key_name( item => $2 );
            } elsif ( $line =~ /([\w\s]+):([\w\s\-\.]+)$/ ) {
                $key = $1;
                $val = $2;
                $val =~ s/^\s//g;
                $result_array[$i]->{ to_key_name( item => $key ) } = $val;
            } elsif ( $line =~ /(FPGA image.*)/ ) {
                $result_array[$i]->{"info"} = $1;
            } elsif ( $line =~ /Flash Cache firmware upgrade:(.*)/ ) {
                $val = $1;
                $val =~ s/Flash Cache firmware upgrade://g;
                @arr = split( /complete/, $val );
                foreach my $value (@arr) {
                    $value =~ s/\s+//g;
                    $result_array[0]->{"flash_cache_firmware_upgrade"}
                        ->[ $j++ ] = $value . "complete";
                }
            }
        }
    } elsif ( $command =~ /-c/ ) {
        foreach my $line (@lines) {
            if ( $line =~ /([\w\s]+):([\w\s\-\.]+)$/ ) {
                $key = $1;
                $val = $2;
                $val =~ s/^\s//g;
                $result_array[0]->{ to_key_name( item => $key ) } = $val;
            } elsif ( $line =~ /(FPGA image.*)/ ) {
                $result_array[0]->{"info"} = $1;
            } elsif ( $line =~ /Flash Cache firmware upgrade:(.*)/ ) {
                $val = $1;
                $val =~ s/Flash Cache firmware upgrade://g;
                @arr = split( /complete/, $val );
                foreach my $value (@arr) {
                    $value =~ s/\s+//g;
                    $result_array[0]->{"flash_cache_firmware_upgrade"}
                        ->[ $j++ ] = $value . "complete";
                }
            }
        }
    }
    return \@result_array;
} ## end sub iomem_fpga_update

## @name ::iomem_ecc_show
##
## @summary Parses the raw_output from the "iomem ecc show" command
##
## @description
##    This method can be used for parsing the raw_output obtained after calling
##    the method from iomem_ecc_show with any of its options as described in the
##    corresponding cdef file 'iomem1.cdefs'
## @synopsis
##      iomem_ecc_show(%args);
## INPUTS TO METHOD CALL 'iomem_ecc_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-slots' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw_output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed raw_output.
## @examples

##&nbsp;Command = priv set -q diag; iomem ecc show -A; result -l
##&nbsp;
##&nbsp;-------- Parsed raw_output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot' => '4',
##&nbsp;            'ecc_error_correction' => ' Enabled',
##&nbsp;            'info' => 'Performance Acceleration Module I',
##&nbsp;            'ecc_error_detection' => ' Enabled'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_ecc_show {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error      = _parse_for_errors(
        "raw_output"        => $raw_output,
        "command"           => 'iomem ecc show',
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my ( @result_array, $key, $i );
    my @lines = get_lines(
        raw_output      => $raw_output,
        can_throw       => 0,
        response_object => $resp
    );
    my $opt = $command;
    $i = -1;
    if ( $command =~ /-A/ ) {
        foreach my $line (@lines) {
            if ( $line =~ /^Slot\s+(\d+):\s+(.*)/ ) {
                $i++;
                $result_array[$i]->{"slot"} = $1;
                $result_array[$i]->{"info"} = $2;
            } elsif ( $line =~ /(ECC.*):(.*)/ ) {
                $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
            }
        }
    } elsif ( $command =~ /-c/ ) {
        foreach my $line (@lines) {
            if ( $line =~ /(ECC.*):(.*)/ ) {
                $result_array[0]->{ to_key_name( item => $1 ) } = $2;
            }
        }
    }
    return \@result_array;
} ## end sub iomem_ecc_show

## @name ::iomem_ecc_detect
##
## @summary Parses the raw output from the "iomem ecc detect" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_ecc_detect with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_ecc_detect(%args);
## INPUTS TO METHOD CALL 'iomem_ecc_detect' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-slots' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'ecc-state' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem ecc detect enable -A; result -l
##&nbsp;
##&nbsp;-------- Parsed raw_output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot' => '4',
##&nbsp;            'info' => 'Performance Acceleration Module I',
##&nbsp;            'ecc_error_detection' => ' enabled.'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_ecc_detect {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error      = _parse_for_errors(
        "raw_output"        => $raw_output,
        "command"           => 'iomem ecc detect',
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my ( @result_array, $key, $i );
    my @lines = get_lines(
        raw_output      => $raw_output,
        can_throw       => 0,
        response_object => $resp
    );
    my $opt = $command;
    $i = -1;
    if ( $command =~ /-A/ ) {
        foreach my $line (@lines) {
            if ( $line =~ /^Slot\s+(\d+):\s+(.*)/ ) {
                $i++;
                $result_array[$i]->{"slot"} = $1;
                $result_array[$i]->{"info"} = $2;
            } elsif ( $line =~ /(ECC error detection)(.*)/ ) {
                $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
            }
        }
    } elsif ( $command =~ /-c/ ) {
        foreach my $line (@lines) {
            if ( $line =~ /(ECC error detection)(.*)/ ) {
                $result_array[0]->{ to_key_name( item => $1 ) } = $2;
            }
        }
    }
    return \@result_array;
} ## end sub iomem_ecc_detect

## @name ::iomem_ecc_correct
##
## @summary Parses the raw output from the "iomem ecc correct" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_ecc_correct with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_ecc_correct(%args);
## INPUTS TO METHOD CALL 'iomem_ecc_correct' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-slots' </td> <td> '1' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'ecc-state' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem ecc correct enable -A; result -l
##&nbsp;
##&nbsp;-------- Parsed raw_output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot' => '4',
##&nbsp;            'ecc_error_correction' => ' enabled.',
##&nbsp;            'info' => 'Performance Acceleration Module I'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_ecc_correct {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error      = _parse_for_errors(
        "raw_output"        => $raw_output,
        "command"           => 'iomem ecc correct',
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my ( @result_array, $key, $i );
    my @lines = get_lines(
        raw_output      => $raw_output,
        can_throw       => 0,
        response_object => $resp
    );
    my $opt = $command;
    $i = -1;
    if ( $command =~ /-A/ ) {
        foreach my $line (@lines) {
            if ( $line =~ /^Slot\s+(\d+):\s+(.*)/ ) {
                $i++;
                $result_array[$i]->{"slot"} = $1;
                $result_array[$i]->{"type"} = $2;
            } elsif ( $line =~ /(ECC error correction)(.*)/ ) {
                $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
            }
        }
    } elsif ( $command =~ /-c/ ) {
        foreach my $line (@lines) {
            if ( $line =~ /(ECC error correction)(.*)/ ) {
                $result_array[0]->{ to_key_name( item => $1 ) } = $2;
            }
        }
    }
    return \@result_array;
} ## end sub iomem_ecc_correct

## @name ::iomem_ecc_inject
##
## @summary Parses the raw output from the "iomem ecc inject" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_ecc_inject with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_ecc_inject(%args);
## INPUTS TO METHOD CALL 'iomem_ecc_inject' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'value' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-slots' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; iomem ecc inject -c 6 -a 0x20 -v 0xA5A5A5; result -l
##&nbsp;
##&nbsp;-------- Parsed raw_output ------
##&nbsp;$VAR1 = [
##&nbsp;           {
##&nbsp;             'info' => 'ECC error injected successfully.'
##&nbsp;           }
##&nbsp;         ];
##&nbsp;
sub iomem_ecc_inject {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error      = _parse_for_errors(
        "raw_output"        => $raw_output,
        "command"           => 'iomem ecc inject',
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my ( @result_array, $key, $i );
    my @lines = get_lines(
        raw_output      => $raw_output,
        can_throw       => 0,
        response_object => $resp
    );
    my $opt = $command;
    $i = -1;
    if ( $command =~ /-A/ ) {
        foreach my $line (@lines) {
            if ( $line =~ /^Slot\s+(\d+):\s+(.*)/ ) {
                $i++;
                $result_array[$i]->{"slot"} = $1;
                $result_array[$i]->{"type"} = $2;
            } elsif ( $line =~ /(ECC.*)/ ) {
                $result_array[$i]->{"info"} = $1;
            }
        }
    } elsif ( $command =~ /-c/ ) {
        foreach my $line (@lines) {
            if ( $line =~ /(ECC.*)/ ) {
                $result_array[0]->{"info"} = $1;
            }
        }
    }
    return \@result_array;
} ## end sub iomem_ecc_inject

## @name ::iomem_stress_start
##
## @summary Parses the raw output from the "iomem stress start" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_stress_start with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_stress_start(%args);
## INPUTS TO METHOD CALL 'iomem_stress_start' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-V' </td> <td> 'verbose' </td> </tr>
## <tr> <td> '-S' </td> <td> 'io-size' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-slots' </td> </tr>
## <tr> <td> '-X' </td> <td> 'appendix-dma' </td> </tr>
## <tr> <td> '-N' </td> <td> 'nor-flash' </td> </tr>
## <tr> <td> '-n' </td> <td> 'iterations' </td> </tr>
## <tr> <td> '-j' </td> <td> 'bytes-or-offset' </td> </tr>
## <tr> <td> '-D' </td> <td> 'data-crc' </td> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-I' </td> <td> 'i2c-test' </td> </tr>
## <tr> <td> '-L' </td> <td> 'local-dma' </td> </tr>
## <tr> <td> '-R' </td> <td> 'read' </td> </tr>
## <tr> <td> '-E' </td> <td> 'erase' </td> </tr>
## <tr> <td> '-W' </td> <td> 'write' </td> </tr>
## <tr> <td> '-C' </td> <td> 'randomize-dma' </td> </tr>
## <tr> <td> '-p' </td> <td> 'pages' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem stress start -c 5; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'stress_test_is_already_running_on' => '5'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_stress_start {
    my %args = @_;
    my $error_message;
    my $can_throw  = 1;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my %info       = ();
    my @raw_output;
    my $error = _parse_for_errors(
        "raw_output"    => $raw_output,
        "command"       => $command,
        "error_message" => \$error_message
    );

    if ($error) {
        $error     = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $can_throw = 0;
    }

    #Now we can parse the raw_output
    my @lines = get_lines(
        "raw_output"      => $raw_output,
        "response_object" => $resp
    );
    my ( @Res, $value );

    my $i = 0;
    foreach $value (@lines) {
        if ( $value =~ /(DMA\s+stress test\s+)(\w+)/ ) {
            $Res[$i]->{"dma_stress_test"} = $2;
            $i++;
        } elsif ( $value
            =~ /Slot\s+(\d+)\:\s+(Performance Acceleration\s+\w+\s+\w+.*) / )
        {
            $Res[$i]->{"slot"} = $1;
            $Res[$i]->{"info"} = $2;
        } elsif ( $value =~ /^Slot\s+(\d+):\s+(.*)/ ) {
            $Res[$i]->{"slot"} = $1;
            $Res[$i]->{"type"} = $2;
        } elsif ( $value
            =~ /iomem:\s+(Stress test is already running on)\s+\w+\s+\w+\s+\w+\s+(\d+)/
            ) {
            $Res[$i]->{"stress_test_is_already_running_on"} = $2;
            $i++;
        }
    }
    if ($error) {
        $resp->set_parsed_output( parsed_output => \@Res );
        $error->throw( text => $error_message, response_object => $resp );
    }
    return \@Res;
} ## end sub iomem_stress_start

## @name ::iomem_stress_show
##
## @summary Parses the raw output from the "iomem stress show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_stress_show with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_stress_show(%args);
## INPUTS TO METHOD CALL 'iomem_stress_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-V' </td> <td> 'verbose' </td> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-slots' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem stress show -c 5; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'flags' => [
##&nbsp;                         'Erase/Write/Read',
##&nbsp;                         ' NO Read verify',
##&nbsp;                         ' NO NOR',
##&nbsp;                         ' NO I2C'
##&nbsp;                       ],
##&nbsp;            'totals' => {
##&nbsp;                          'bytes' => '1348253271040 (1 TiB)',
##&nbsp;                          'Erase' => {
##&nbsp;                                       'ok' => '158272',
##&nbsp;                                       'failed' => '12'
##&nbsp;                                     },
##&nbsp;                          'duration' => '1363 seconds',
##&nbsp;                          'Read ' => {
##&nbsp;                                       'ok' => '10127594',
##&nbsp;                                       ' verify failed' => '0',
##&nbsp;                                       'failed' => '0'
##&nbsp;                                     },
##&nbsp;                          'active_stripes' => '114920 (0)',
##&nbsp;                          'iteration' => '2:91',
##&nbsp;                          'Write' => {
##&nbsp;                                       'ok' => '10128615',
##&nbsp;                                       'failed' => '1'
##&nbsp;                                     },
##&nbsp;                          'performance' => '989180683 B/s (943 MiB/s)'
##&nbsp;                        },
##&nbsp;            'op' => 'NAND'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_stress_show {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $can_throw  = 1;
    my %info       = ();
    my $error      = _parse_for_errors(
        "raw_output"    => $raw_output,
        "command"       => $command,
        "error_message" => \$error_message
    );

    if ($error) {
        $error     = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $can_throw = 0;
    }

    #Now we can parse the raw_output
    my @lines = get_lines(
        "raw_output"      => $raw_output,
        "response_object" => $resp
    );
    my ( @Res, $value, $head1, $head2, $head3, $head4 );
    my $flag = 0;
    my $i    = -1, my @ar;
    my $k    = 0;
    my $j    = 0;
    foreach $value (@lines) {

        if ( $value =~ /===\s+(\w+).*/ ) {
            $Res[$i]->{"op"} = $1;
        } elsif ( $value =~ /^Slot\s+(\d+):\s+(.*)$/ ) {
            $i++ ;
            $Res[$i]->{"slot"} = $1;
            $Res[$i]->{"type"} = $2;
        } elsif ( $value
            =~ /Slot\s+(\d+)\:\s+(Performance Acceleration\s+\w+\s+\w+.*) / )
        {
            $Res[$i]->{"slot"} = $1;
            $Res[$i]->{"info"} = $2;
        } elsif ( $value =~ /^\*\s+(Flags)\s+\*$/ ) {
            $flag = 1;
            $head1 = to_key_name( item => $1 );
        } elsif ( $value =~ /^\*\s+(Totals)\s+\*$/ ) {
            $flag = 2;
            $head2 = to_key_name( item => $1 );
        } elsif ( $value =~ /\s*(DMA_Channel_\d+)\s+(DMA_Channel_\d+)/ ) {
            $flag  = 3;
            $head3 = to_key_name( item => $1 );
            $head4 = to_key_name( item => $2 );
        } elsif ( $value
            =~ /(\w+\(\w+\))\:\s+(\w+)\:(\d+)\s+(\w+)\:(\d+\(\d+\))\s+(\w+)\:(\w+)\s+(\w+)\:(\d+)\s+(\w+)\:(\d+)\s+(\w+)\:(\d+)\s+(\w+)\:(\d+)\s+(\w+)\:(\d+)\s+(\w+)\:(\d+)\s+(\w+)\:(\d+)/
            ) {
            $Res[$i]->{"status"}->[$k]->{"no"} = $1;
            $Res[$i]->{"status"}->[$k]->{ to_key_name( item => $2 ) }  = $3;
            $Res[$i]->{"status"}->[$k]->{ to_key_name( item => $4 ) }  = $5;
            $Res[$i]->{"status"}->[$k]->{ to_key_name( item => $6 ) }  = $7;
            $Res[$i]->{"status"}->[$k]->{ to_key_name( item => $8 ) }  = $9;
            $Res[$i]->{"status"}->[$k]->{ to_key_name( item => $10 ) } = $11;
            $Res[$i]->{"status"}->[$k]->{ to_key_name( item => $12 ) } = $13;
            $Res[$i]->{"status"}->[$k]->{ to_key_name( item => $14 ) } = $15;
            $Res[$i]->{"status"}->[$k]->{ to_key_name( item => $16 ) } = $17;
            $Res[$i]->{"status"}->[$k]->{ to_key_name( item => $18 ) } = $19;
            $Res[$i]->{"status"}->[$k]->{ to_key_name( item => $20 ) } = $21;
            $k++;
        }

        if ( $flag == 1 ) {
            if ( $value =~ /(Erase\/Write\/Read)\,(.*verify)\,(.*)\,(.*)/ ) {
                $j = 0 ;
                @ar = split( /,/, $value );
                for ( my $n = 0; $n < @ar; $n++ ) {
                    $Res[$i]->{$head1}->[$j] = $ar[$n];
                    $j++;
                }
            }
        } elsif ( $flag == 2 ) {
            if ( $value =~ /^\s*(\w+)\s+\-\s+(\d+\:\d+)$/ ) {
                $Res[$i]->{$head2}->{ to_key_name( item => $1 ) } = $2;
                $j++;
            } elsif ( $value =~ /^\s*(.*)\:\s+(.*)$/ ) {
                $Res[$i]->{$head2}->{ to_key_name( item => $1 ) } = $2;
                $j++;
            } elsif ( $value
                =~ /\s*(.*)\s+\-\s+(\w+)\:(\d+)\,\s+(\w+)\:(\d+)\,(.*)\:(\d+)/
                ) {
                $Res[$i]->{$head2}->{$1}->{$2} = $3;
                $Res[$i]->{$head2}->{$1}->{$4} = $5;
                $Res[$i]->{$head2}->{$1}->{$6} = $7;
                $j++;
            } elsif (
                $value =~ /\s*(.*)\s+\-\s+(\w+)\:(\d+)\,\s+(\w+)\:(\d+)/ ) {
                $Res[$i]->{$head2}->{$1}->{$2} = $3;
                $Res[$i]->{$head2}->{$1}->{$4} = $5;
                $j++;
            }
        } elsif ( $flag == 3 ) {
            if ( $value =~ /\s*(.*)\:\s+(\d+)\s+(\d+)/ ) {
                $Res[$i]->{$head3}->[$j]->{ to_key_name( item => $1 ) } = $2;
                $Res[$i]->{$head4}->[$j]->{ to_key_name( item => $1 ) } = $3;
            }
        }
        if ($error) {
            $resp->set_parsed_output( parsed_output => \@Res );
            $error->throw( text => $error_message, response_object => $resp );
        }
    }
    return \@Res;
} ## end sub iomem_stress_show

## @name ::iomem_stress_stop
##
## @summary Parses the raw output from the "iomem stress stop" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_stress_stop with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_stress_stop(%args);
## INPUTS TO METHOD CALL 'iomem_stress_stop' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-slots' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem stress stop -c 5; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'test' => 'stopped'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_stress_stop {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my %info       = ();
    my $can_throw  = 1;
    my $error      = _parse_for_errors(
        "raw_output"    => $raw_output,
        "command"       => $command,
        "error_message" => \$error_message
    );

    if ($error) {
        $error     = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $can_throw = 0;
    }

    #Now we can parse the raw_output
    my @lines = get_lines(
        "raw_output"      => $raw_output,
        "response_object" => $resp
    );
    my ( @Res, $value );
    my $i = 0;
    foreach $value (@lines) {
        if ( $value =~ /^(\w+)\s+\w+\s+\w+\s+(\w+).*$/ ) {
            $Res[$i]->{ to_key_name( item => $1 ) } = $2;
            $i++;
        } elsif ( $value =~ /^Slot\s+(\d+):\s+(.*)/ ) {
            $Res[$i]->{"slot"} = $1;
            $Res[$i]->{"type"} = $2;
        } elsif ( $value
            =~ /Slot\s+(\d+)\:\s+(Performance Acceleration\s+\w+\s+\w+.*) / )
        {
            $Res[$i]->{"slot"} = $1;
            $Res[$i]->{"info"} = $2;
        }
    }
    if ($error) {
        $resp->set_parsed_output( parsed_output => \@Res );
        $error->throw( text => $error_message, response_object => $resp );
    }
    return \@Res;
} ## end sub iomem_stress_stop

## @name ::iomem_dma_verify
##
## @summary Parses the raw output from the "iomem dma verify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_dma_verify with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_dma_verify(%args);
## INPUTS TO METHOD CALL 'iomem_dma_verify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '-s' </td> <td> 'seed' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub iomem_dma_verify {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub iomem_dma_verify

## @name ::iomem_pio_fill
##
## @summary Parses the raw output from the "iomem pio fill" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_pio_fill with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_pio_fill(%args);
## INPUTS TO METHOD CALL 'iomem_pio_fill' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-E' </td> <td> 'write-asynchronous' </td> <td> '1' </td> </tr>
## <tr> <td> '-n' </td> <td> 'no-of-bytes' </td> <td> '0' </td> </tr>
## <tr> <td> '-N' </td> <td> 'block-number' </td> <td> '1' </td> </tr>
## <tr> <td> '-j' </td> <td> 'bytes-or-offset' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'buffer' </td> <td> '0' </td> </tr>
## <tr> <td> '-D' </td> <td> 'no-dma' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub iomem_pio_fill {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub iomem_pio_fill

## @name ::iomem_bbm_mode
##
## @summary Parses the raw output from the "iomem bbm mode" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_bbm_mode with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_bbm_mode(%args);
## INPUTS TO METHOD CALL 'iomem_bbm_mode' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'mode-or-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub iomem_bbm_mode {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub iomem_bbm_mode

## @name ::iomem_admin_timer
##
## @summary Parses the raw output from the "iomem admin timer" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_admin_timer with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_admin_timer(%args);
## INPUTS TO METHOD CALL 'iomem_admin_timer' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-j' </td> <td> 'bytes-or-offset' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'period' </td> <td> '0' </td> </tr>
## <tr> <td> '-v' </td> <td> 'timer' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub iomem_admin_timer {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub iomem_admin_timer

## @name ::iomem_pio_read
##
## @summary Parses the raw output from the "iomem pio read" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_pio_read with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_pio_read(%args);
## INPUTS TO METHOD CALL 'iomem_pio_read' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-n' </td> <td> 'no-of-bytes' </td> <td> '0' </td> </tr>
## <tr> <td> '-j' </td> <td> 'bytes-or-offset' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'card-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-cards' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'buffer' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> <td> '0' </td> </tr>
## <tr> <td> '-D' </td> <td> 'no-dma' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub iomem_pio_read {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub iomem_pio_read

## @name ::iomem_pio_write
##
## @summary Parses the raw output from the "iomem pio write" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_pio_write with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_pio_write(%args);
## INPUTS TO METHOD CALL 'iomem_pio_write' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-n' </td> <td> 'no-of-bytes' </td> <td> '0' </td> </tr>
## <tr> <td> '-j' </td> <td> 'bytes-or-offset' </td> <td> '0' </td> </tr>
## <tr> <td> '-c' </td> <td> 'card-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-cards' </td> <td> '1' </td> </tr>
## <tr> <td> '-b' </td> <td> 'buffer' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> <td> '0' </td> </tr>
## <tr> <td> '-D' </td> <td> 'no-dma' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub iomem_pio_write {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub iomem_pio_write

## @name ::iomem_pio_erase
##
## @summary Parses the raw output from the "iomem pio erase" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_pio_erase with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_pio_erase(%args);
## INPUTS TO METHOD CALL 'iomem_pio_erase' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'card-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-cards' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub iomem_pio_erase {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub iomem_pio_erase

## @name ::iomem_zodiac_read
##
## @summary Parses the raw output from the "iomem zodiac read" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_zodiac_read with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_zodiac_read(%args);
## INPUTS TO METHOD CALL 'iomem_zodiac_read' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'card-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'size-or-offset' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-cards' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'chain-length' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'buffer' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> <td> '0' </td> </tr>
## <tr> <td> '-X' </td> <td> 'appendix-read' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub iomem_zodiac_read {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub iomem_zodiac_read

## @name ::iomem_zodiac_write
##
## @summary Parses the raw output from the "iomem zodiac write" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_zodiac_write with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_zodiac_write(%args);
## INPUTS TO METHOD CALL 'iomem_zodiac_write' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'card-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-n' </td> <td> 'size-or-offset' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-cards' </td> <td> '1' </td> </tr>
## <tr> <td> '-d' </td> <td> 'chain-length' </td> <td> '0' </td> </tr>
## <tr> <td> '-b' </td> <td> 'buffer' </td> <td> '0' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> <td> '0' </td> </tr>
## <tr> <td> '-X' </td> <td> 'appendix-read' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub iomem_zodiac_write {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub iomem_zodiac_write

## @name ::iomem_zodiac_erase
##
## @summary Parses the raw output from the "iomem zodiac erase" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_zodiac_erase with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_zodiac_erase(%args);
## INPUTS TO METHOD CALL 'iomem_zodiac_erase' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'card-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-cards' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub iomem_zodiac_erase {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub iomem_zodiac_erase

## @name ::iomem_bbm_bbs
##
## @summary Parses the raw output from the "iomem bbm bbs" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_bbm_bbs with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_bbm_bbs(%args);
## INPUTS TO METHOD CALL 'iomem_bbm_bbs' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'card-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-cards' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub iomem_bbm_bbs {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub iomem_bbm_bbs

## @name ::iomem_address_stripe_map
##
## @summary Parses the raw output from the "iomem address stripe-map" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_address_stripe_map with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_address_stripe_map(%args);
## INPUTS TO METHOD CALL 'iomem_address_stripe_map' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-cards' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address-buffer' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; iomem address stripe-map -c 2; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Logical | Start | Current block mapping
##&nbsp;stripe  | block | [] - remapped block
##&nbsp;---------------------------------------
##&nbsp;
##&nbsp;Spares:
##&nbsp;      0 |       |   4096  :   4097
##&nbsp;      1 |       |   4098  :   4099
##&nbsp;      2 |       |   4100  :   4101
##&nbsp;      3 |       |   4102  :   4103
##&nbsp;      4 |       |   4104  :   4105
##&nbsp;      5 |       |   4106  :   4107
##&nbsp;      6 |       |   4108  :   4109
##&nbsp;
##&nbsp;0 out of 2048 stripes failed
##&nbsp;Started with 60 initial spares. Stack can grow upto 296 entries.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'spares' => [
##&nbsp;                          {
##&nbsp;                            'start_block' => 'undef',
##&nbsp;                            'current_block_mapping_remappedblock' => '   4096  :   4097',
##&nbsp;                            'logical_stripe' => '0 '
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'start_block' => 'undef',
##&nbsp;                            'current_block_mapping_remappedblock' => '   4098  :   4099',
##&nbsp;                            'logical_stripe' => '1 '
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'start_block' => 'undef',
##&nbsp;                            'current_block_mapping_remappedblock' => '   4100  :   4101',
##&nbsp;                            'logical_stripe' => '2 '
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'start_block' => 'undef',
##&nbsp;                            'current_block_mapping_remappedblock' => '   4102  :   4103',
##&nbsp;                            'logical_stripe' => '3 '
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'start_block' => 'undef',
##&nbsp;                            'current_block_mapping_remappedblock' => '   4104  :   4105',
##&nbsp;                            'logical_stripe' => '4 '
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'start_block' => 'undef',
##&nbsp;                            'current_block_mapping_remappedblock' => '   4106  :   4107',
##&nbsp;                            'logical_stripe' => '5 '
##&nbsp;                          },
##&nbsp;                          {
##&nbsp;                            'start_block' => 'undef',
##&nbsp;                            'current_block_mapping_remappedblock' => '   4108  :   4109',
##&nbsp;                            'logical_stripe' => '6 '
##&nbsp;                          }
##&nbsp;                        ]
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'progress' => 'Started with 60 initial spares. Stack can grow upto 296 entries.',
##&nbsp;            'result' => '0 out of 2048 stripes failed'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub iomem_address_stripe_map {

my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
      }

    my @lines = get_lines( raw_output => $output );
    my @headings =("Logical stripe","Start block","Current block mapping remappedblock");
    my (@result_array,$result_array,$newkey,$key,$line,$spare_started);
    while($line !~ /------.+/) {
        $line  = shift @lines;
     }
    $line = shift @lines ;
    my $p = -1 ;
    my $key_val= '';
    while(@lines){
        if($line =~ /Spares/i){
            $key_val = "spares";
            $p = -1 ;
            $line  = shift @lines ;
            next ;
        }else {
            $key_val = "block" if($key_val !~ /spares/i);
        }
        if ($line =~ /[|]+/ ) {
            $p++;
            my @values = split( /[|]/, $line );
            for ( my  $i = 0; $i < scalar(@headings); $i++) {
                $key = to_key_name(item => $headings[$i]);
                if ( $values[$i] =~ /(\d+)|fail/) {
                    $result_array[0]->{$key_val}[$p]->{ to_key_name(item => $headings[$i]) } = $values[$i];
                }else{
                    $result_array[0]->{$key_val}[$p]->{ to_key_name(item => $headings[$i]) } = 'undef';
                }
            }
        }else {
            my  $last_line = shift(@lines);
            $result_array[1]->{'result'} =$line;
            $result_array[1]->{'progress'} = $last_line ;
        }
    $line  = shift @lines ;
    }
return \@result_array ;
}## end sub iomem_address_stripe_map

## @name ::iomem_bbm_core_bb
##
## @summary Parses the raw output from the "iomem bbm core-bb" command
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from iomem_bbm_core_bb with any of its options as described in the
##    corresponding cdef file 'iomem.cdefs'
## @synopsis
##      iomem_bbm_core_bb(%args);
## INPUTS TO METHOD CALL 'iomem_address_stripe_map' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-c' </td> <td> 'card' </td> <td> '0' </td> </tr>
## <tr> <td> '-A' </td> <td> 'all-cards' </td> <td> '1' </td> </tr>
## <tr> <td> '-a' </td> <td> 'address-buffer' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; iomem bbm core-bb -c 1 ; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;5 bad blocks
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bad_blocks' => '5'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub iomem_bbm_core_bb {
    my %args = @_;
    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _parse_for_errors(
        "raw_output"    => $output,
        "command"       => $command,
        "error_message" => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @result_array;
    if ( $output =~ /^(\d+)\s+(.*)/ ) {
        $result_array[0]->{ to_key_name( item => $2 ) } = $1;
    }
    return \@result_array;

} ## end sub iomem_bbm_core_bb

sub _parse_for_errors {
    my %args          = @_;
    my $output        = $args{raw_output};
    my $error_message = $args{error_message};
    my $error;
    if ( $output =~ /usage.+/si ) {
        $error          = "InvalidParam";
        $$error_message = "Invalid options are used in the command";
    } elsif ( $output =~ /Unrecognized field.  Valid -f values:/si ) {
        $error          = "InvalidParamValue";
        $$error_message = $output;
    } elsif ( $output =~ /iomem: Unknown option.*/si ) {
        $error          = "InvalidParam";
        $$error_message = $output;
    } elsif ( $output =~ /iomem: One of either.*/si ) {
        $error          = "InvalidParam";
        $$error_message = $output;
    } elsif ( $output =~ /iomem: Invalid.*/si ) {
        $error          = "InvalidParamValue";
        $$error_message = $output;
    } elsif ( $output =~ /iomem: Options.*are mutually exclusive./si ) {
        $error          = "InvalidParam";
        $$error_message = $output;
    } elsif ( $output =~ /iomem: Error on card.*/si ) {
        $error          = "InvalidParam";
        $$error_message = $output;
    } elsif ( $output =~ /iomem: Missing argument.*/si ) {
        $error          = "InvalidParamValue";
        $$error_message = $output;
    } elsif ( $output =~ /iomem:\s+Missing\s+argument\s+for\s+option.*/si ) {
        $error          = "MissingRequiredParam";
        $$error_message = $output;
    } elsif ( $output =~ /No cards installed.*/si ) {
        $error          = "CommandFailed";
        $$error_message = $output;
    }
    return $error;
} ## end sub _parse_for_errors
1;
