#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Node/CLI/7Mode/default/system.pm#10 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse 7Mode system command
##
## @description
##    This is a library to parse the raw output of 7Mode system  command.
##    The function name will be the same as command name with underscores being
##    used instead of spaces.
##
## @synopsis
##    use system ;
##
##    system(%args);
##
## @author dl-nacl-dev@netapp.com, veerr@netapp.com
## @author dl-nacl-dev@netapp.com, veerap@netapp.com
##
## @status public
#
package NACL::APISet::Node::CLI::7Mode::default::system;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);

## @name ::system_health_node_connectivity_adapter_show
##
## @summary Parses the raw output from the "system health node-connectivity adapter show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_health_node_connectivity_adapter_show with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_health_node_connectivity_adapter_show(%args);
## INPUTS TO METHOD CALL 'system_health_node_connectivity_adapter_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-iom-port' </td> <td> 'iom-port' </td> </tr>
## <tr> <td> '-port-name' </td> <td> 'port-name' </td> </tr>
## <tr> <td> '-ioma-list' </td> <td> 'ioma-list' </td> </tr>
## <tr> <td> '-status' </td> <td> 'status' </td> </tr>
## <tr> <td> '-is-enabled' </td> <td> 'is-enabled' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> </tr>
## <tr> <td> '-adapter-name' </td> <td> 'adaptername' </td> </tr>
## <tr> <td> '-iomb-list' </td> <td> 'iomb-list' </td> </tr>
## <tr> <td> '-circle-port-list' </td> <td> 'circle-port-list' </td> </tr>
## <tr> <td> '-num-shelf' </td> <td> 'numofshelf' </td> </tr>
## <tr> <td> '-in-use' </td> <td> 'in-use' </td> </tr>
## <tr> <td> '-square-port-list' </td> <td> 'square-port-list' </td> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> </tr>
## <tr> <td> '-shelf-list' </td> <td> 'shelf-list' </td> </tr>
## <tr> <td> '-num-ioma' </td> <td> 'num-ioma' </td> </tr>
## <tr> <td> '-is-dual-attached' </td> <td> 'is-dual-attached' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> </tr>
## <tr> <td> '-wwn' </td> <td> 'wwn' </td> </tr>
## <tr> <td> '-num-square-port' </td> <td> 'num-square-port' </td> </tr>
## <tr> <td> '-num-circle-port' </td> <td> 'num-circle-port' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> </tr>
## <tr> <td> '-num-iomb' </td> <td> 'num-iomb' </td> </tr>
## <tr> <td> '-slot-num' </td> <td> 'slot-num' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; system health node-connectivity adapter show -instance; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;                                      Node: csiqa-2040-16b
##&nbsp;                              Adapter Name: 0c
##&nbsp;                              Monitor Name: node-connect
##&nbsp;                                       WWN: 5:00a098:000251e:c0
##&nbsp;                                   In Use?: true
##&nbsp;                               Is Enabled?: true
##&nbsp;                     Adapter Dual Attached: false
##&nbsp;                                    Status: OK
##&nbsp;                               Slot Number: 0
##&nbsp;                                 Port Name: c
##&nbsp;               IOM Port to be Connected to: square
##&nbsp;                         Number of Shelves: 1
##&nbsp;                           List of Shelves: 50:0c:0f:f0:0d:38:b4:3c
##&nbsp;       Number of Shelves Connected to IOMA: 1
##&nbsp;         List of Shelves Connected to IOMA: 50:0c:0f:f0:0d:38:b4:3c
##&nbsp;       Number of Shelves Connected to IOMB: 0
##&nbsp;         List of Shelves Connected to IOMB: -
##&nbsp;Number of Shelves Connected to Circle port: 0
##&nbsp;  List of Shelves Connected to Circle port: -
##&nbsp;Number of Shelves Connected to Square port: 1
##&nbsp;  List of Shelves Connected to Square port: 50:0c:0f:f0:0d:38:b4:3c
##&nbsp;
##&nbsp;                                      Node: csiqa-2040-16b
##&nbsp;                              Adapter Name: 0d
##&nbsp;                              Monitor Name: node-connect
##&nbsp;                                       WWN: 5:00a098:000251e:c4
##&nbsp;                                   In Use?: false
##&nbsp;                               Is Enabled?: false
##&nbsp;                     Adapter Dual Attached: true
##&nbsp;                                    Status: OK
##&nbsp;                               Slot Number: 0
##&nbsp;                                 Port Name: d
##&nbsp;               IOM Port to be Connected to: circle
##&nbsp;                         Number of Shelves: 0
##&nbsp;                           List of Shelves: -
##&nbsp;       Number of Shelves Connected to IOMA: 0
##&nbsp;         List of Shelves Connected to IOMA: -
##&nbsp;       Number of Shelves Connected to IOMB: 0
##&nbsp;         List of Shelves Connected to IOMB: -
##&nbsp;Number of Shelves Connected to Circle port: 0
##&nbsp;  List of Shelves Connected to Circle port: -
##&nbsp;Number of Shelves Connected to Square port: 0
##&nbsp;  List of Shelves Connected to Square port: -
##&nbsp;2 entries were displayed.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'number_of_shelves_connected_to_circle_port' => '0',
##&nbsp;            'list_of_shelves_connected_to_iomb' => '-',
##&nbsp;            'number_of_shelves_connected_to_ioma' => '1',
##&nbsp;            'status' => 'OK',
##&nbsp;            'number_of_shelves_connected_to_iomb' => '0',
##&nbsp;            'node' => 'csiqa-2040-16b',
##&nbsp;            'list_of_shelves_connected_to_square_port' => '50:0c:0f:f0:0d:38:b4:3c',
##&nbsp;            'number_of_shelves_connected_to_square_port' => '1',
##&nbsp;            'list_of_shelves_connected_to_circle_port' => '-',
##&nbsp;            'iom_port_to_be_connected_to' => 'square',
##&nbsp;            'slot_number' => '0',
##&nbsp;            'monitor_name' => 'node-connect',
##&nbsp;            'port_name' => 'c',
##&nbsp;            'number_of_shelves' => '1',
##&nbsp;            'list_of_shelves_connected_to_ioma' => '50:0c:0f:f0:0d:38:b4:3c',
##&nbsp;            'wwn' => '5:00a098:000251e:c0',
##&nbsp;            'list_of_shelves' => '50:0c:0f:f0:0d:38:b4:3c',
##&nbsp;            'adapter_name' => '0c',
##&nbsp;            'adapter_dual_attached' => 'false'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'number_of_shelves_connected_to_circle_port' => '0',
##&nbsp;            'list_of_shelves_connected_to_iomb' => '-',
##&nbsp;            'number_of_shelves_connected_to_ioma' => '0',
##&nbsp;            'status' => 'OK',
##&nbsp;            'number_of_shelves_connected_to_iomb' => '0',
##&nbsp;            'node' => 'csiqa-2040-16b',
##&nbsp;            'list_of_shelves_connected_to_square_port' => '-',
##&nbsp;            'number_of_shelves_connected_to_square_port' => '0',
##&nbsp;            'list_of_shelves_connected_to_circle_port' => '-',
##&nbsp;            'iom_port_to_be_connected_to' => 'circle',
##&nbsp;            'slot_number' => '0',
##&nbsp;            'monitor_name' => 'node-connect',
##&nbsp;            'port_name' => 'd',
##&nbsp;            'number_of_shelves' => '0',
##&nbsp;            'list_of_shelves_connected_to_ioma' => '-',
##&nbsp;            'wwn' => '5:00a098:000251e:c4',
##&nbsp;            'list_of_shelves' => '-',
##&nbsp;            'adapter_name' => '0d',
##&nbsp;            'adapter_dual_attached' => 'true'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'total entries displayed' => '2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub system_health_node_connectivity_adapter_show {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'system health node-connectivity adapter show',
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my ( @result_array, @keyarr, @valarr );
    my $i = -1;
    if ( $raw_output =~ /There are no entries matching your query/ ) {
        $result_array[0]->{status} = "no matching entries found";
    } else {
        my @lines = get_lines(
            raw_output      => $raw_output,
            can_throw       => 0,
            response_object => $resp
        );
        if ( $command =~ /-fields/ ) {
            foreach my $line (@lines) {
                if ( $line =~ /^node.*/ ) {
                    @keyarr = split( /\s+/, $line );
                } elsif ( $line =~ /---------.*/ ) {
                    next;
                } elsif ( $line =~ /(\d+) entries were displayed.$/ ) {
                    $i++;
                    $result_array[$i]->{"total entries displayed"} = $1;
                } elsif ( $line =~ /(\S+\s){0,9}/ ) {
                    @valarr = split( /\s+/, $line );
                    $i++;
                    for ( my $m = 0; $m <= $#keyarr; $m++ ) {
                        $valarr[$m] =~ s/\s+//g;
                        $result_array[$i]->{ $keyarr[$m] } = $valarr[$m];
                    }
                }
            } ## end foreach my $line (@lines)
        } ## end if ($command =~ /-fields/)
        elsif ( $command =~ /show$/ ) {
            foreach my $line (@lines) {
                if ( $line
                    =~ /([\w\-]+)\s+([\w\.]+)\s+(\w+)\s+(\w+)\s+(\d+)\s+(\w+)/
                    ) {
                    $i++;
                    $result_array[$i]->{"node"}        = $1;
                    $result_array[$i]->{"adaptername"} = $2;
                    $result_array[$i]->{"inuse"}       = $3;
                    $result_array[$i]->{"enabled"}     = $4;
                    $result_array[$i]->{"numshelves"}  = $5;
                    $result_array[$i]->{"status"}      = $6;
                } elsif ( $line =~ /(\d+) entries were displayed/ ) {
                    $i++;
                    $result_array[$i]->{"total_entries_displayed"} = $1;
                }
            } ## end foreach my $line (@lines)
        } else {
            while (my $line = shift(@lines)){
                if ( $line =~ /Node:.*/ ) {
                    $i++;
                    $result_array[$i]->{'node'}=$1;
                } elsif ( $line=~/In Use\?:\s+(\w+)/) {
                    $result_array[$i]->{'in_use'}=$1;
                } elsif ($line=~/Is Enabled\?:\s+(\w+)/) {
                    $result_array[$i]->{'is_enabled'}=$1;
                } elsif ($line =~ /(List of Shelves.*?):\s+(.*)/){ 
                    my $shelves = $2;
                    my $key = $1;
                    $line = shift @lines;
                    while ($line !~ /Number of Shelves.*?/ and ( scalar(@lines) > 0 )){
                        if ($line=~ /Node:\s+.*/ ){
                            last;
                        }
                        $shelves = $shelves. " " .$line;
                        $line = shift @lines;
                    }
                    $result_array[$i]->{to_key_name(item=>$key)} = $shelves;
                    unshift @lines, $line;
                } elsif ($line=~/([\w\s]+):\s+([\w\-\:]+)/ ) {
                    $result_array[$i]->{to_key_name( item => $1 ) } = $2;
                } elsif ( $line =~ /(\d+) entries were displayed/ ) {
                    $i++;
                    $result_array[$i]->{"total entries displayed"} = $1;
                }
            } ## end of outer while loop
		}
	}
    return \@result_array;
} ## end sub system_health_node_connectivity_adapter_show

## @name ::system_health_node_connectivity_disk_show
##
## @summary Parses the raw output from the "system health node-connectivity disk show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_health_node_connectivity_disk_show with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_health_node_connectivity_disk_show(%args);
## INPUTS TO METHOD CALL 'system_health_node_connectivity_disk_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> </tr>
## <tr> <td> '-status' </td> <td> 'status' </td> </tr>
## <tr> <td> '-disk-name' </td> <td> 'diskname' </td> </tr>
## <tr> <td> '-serial-no' </td> <td> 'serialno' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> </tr>
## <tr> <td> '-uuid' </td> <td> 'uuid' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> </tr>
## <tr> <td> '-bay-no' </td> <td> 'bay-no' </td> </tr>
## <tr> <td> '-shelf-id' </td> <td> 'shelf-id' </td> </tr>
## <tr> <td> '-sec-name' </td> <td> 'secname' </td> </tr>
## <tr> <td> '-num-paths' </td> <td> 'num-paths' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; system health node-connectivity disk show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;                                                            Num
##&nbsp;Node            Disk Name                             Bay   Paths  Status
##&nbsp;--------------- ------------------------------------- ----  -----  ----------
##&nbsp;csiqa-2040-16b  0c.00.0                                  0    1    OK
##&nbsp;csiqa-2040-16b  0c.00.7                                  7    1    OK
##&nbsp;csiqa-2040-16b  0c.00.5                                  5    1    OK
##&nbsp;csiqa-2040-16b  0c.00.4                                  4    1    OK
##&nbsp;csiqa-2040-16b  0c.00.9                                  9    1    OK
##&nbsp;csiqa-2040-16b  0c.00.11                                11    1    OK
##&nbsp;csiqa-2040-16b  0c.00.1                                  1    1    OK
##&nbsp;csiqa-2040-16b  0c.00.2                                  2    1    OK
##&nbsp;csiqa-2040-16b  0c.00.6                                  6    1    OK
##&nbsp;csiqa-2040-16b  0c.00.3                                  3    1    OK
##&nbsp;csiqa-2040-16b  0c.00.8                                  8    1    OK
##&nbsp;csiqa-2040-16b  0c.00.10                                10    1    OK
##&nbsp;12 entries were displayed.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'bay' => '0',
##&nbsp;            'status' => 'OK',
##&nbsp;            'numpaths' => '1',
##&nbsp;            'diskname' => '0c.00.0',
##&nbsp;            'node' => 'csiqa-2040-16b'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '7',
##&nbsp;            'status' => 'OK',
##&nbsp;            'numpaths' => '1',
##&nbsp;            'diskname' => '0c.00.7',
##&nbsp;            'node' => 'csiqa-2040-16b'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '5',
##&nbsp;            'status' => 'OK',
##&nbsp;            'numpaths' => '1',
##&nbsp;            'diskname' => '0c.00.5',
##&nbsp;            'node' => 'csiqa-2040-16b'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '4',
##&nbsp;            'status' => 'OK',
##&nbsp;            'numpaths' => '1',
##&nbsp;            'diskname' => '0c.00.4',
##&nbsp;            'node' => 'csiqa-2040-16b'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '9',
##&nbsp;            'status' => 'OK',
##&nbsp;            'numpaths' => '1',
##&nbsp;            'diskname' => '0c.00.9',
##&nbsp;            'node' => 'csiqa-2040-16b'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '11',
##&nbsp;            'status' => 'OK',
##&nbsp;            'numpaths' => '1',
##&nbsp;            'diskname' => '0c.00.11',
##&nbsp;            'node' => 'csiqa-2040-16b'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '1',
##&nbsp;            'status' => 'OK',
##&nbsp;            'numpaths' => '1',
##&nbsp;            'diskname' => '0c.00.1',
##&nbsp;            'node' => 'csiqa-2040-16b'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '2',
##&nbsp;            'status' => 'OK',
##&nbsp;            'numpaths' => '1',
##&nbsp;            'diskname' => '0c.00.2',
##&nbsp;            'node' => 'csiqa-2040-16b'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '6',
##&nbsp;            'status' => 'OK',
##&nbsp;            'numpaths' => '1',
##&nbsp;            'diskname' => '0c.00.6',
##&nbsp;            'node' => 'csiqa-2040-16b'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '3',
##&nbsp;            'status' => 'OK',
##&nbsp;            'numpaths' => '1',
##&nbsp;            'diskname' => '0c.00.3',
##&nbsp;            'node' => 'csiqa-2040-16b'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '8',
##&nbsp;            'status' => 'OK',
##&nbsp;            'numpaths' => '1',
##&nbsp;            'diskname' => '0c.00.8',
##&nbsp;            'node' => 'csiqa-2040-16b'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'bay' => '10',
##&nbsp;            'status' => 'OK',
##&nbsp;            'numpaths' => '1',
##&nbsp;            'diskname' => '0c.00.10',
##&nbsp;            'node' => 'csiqa-2040-16b'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'total_entries_displayed' => '12'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub system_health_node_connectivity_disk_show {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'system health node-connectivity disk show',
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my ( @result_array, @keyarr, @valarr );
    my $i = -1;
    if ( $raw_output =~ /There are no entries matching your query/ ) {
        $result_array[0]->{"status"} = "no matching entries found";
    } else {
        my @lines = get_lines(
            raw_output      => $raw_output,
            can_throw       => 0,
            response_object => $resp
        );
        if ( $command =~ /-instance$/ ) {
            foreach my $line (@lines) {
                if ( $line =~ /Node:.*/ ) {
                    $i++;
                }
                if ( $line =~ /([\w\s]+):\s+([\w\-\:]+)/ ) {
                    $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
                } elsif ( $line =~ /(\d+) entries were displayed/ ) {
                    $i++;
                    $result_array[$i]->{"total entries displayed"} = $1;
                }
            } ## end foreach my $line (@lines)
        }  elsif ( $command =~ /-fields/ ) {
            foreach my $line (@lines) {
                if ( $line =~ /^node.*/ ) {
                    @keyarr = split( /\s+/, $line );
                } elsif ( $line =~ /---------.*/ ) {
                    next;
                } elsif ( $line =~ /(\d+) entries were displayed.$/ ) {
                    $i++;
                    $result_array[$i]->{"total entries displayed"} = $1;
                } elsif ( $line =~ /(\S+\s){0,9}/ ) {
                    @valarr = split( /\s+/, $line );
                    $i++;
                    for ( my $m = 0; $m <= $#keyarr; $m++ ) {
                        $valarr[$m] =~ s/\s+//g;
                        $result_array[$i]->{ $keyarr[$m] } = $valarr[$m];
                    }
                }
            } ## end foreach my $line (@lines)
        } ## end elsif ($command =~ /-fields/)
        elsif ( $command =~ /show/ ) {
            foreach my $line (@lines) {
                if ($line =~ /([\w\-]+)\s+([\w\.]+)\s+(\d+)\s+(\d+)\s+(\w+)/ )
                {
                    $i++;
                    $result_array[$i]->{"node"}     = $1;
                    $result_array[$i]->{"diskname"} = $2;
                    $result_array[$i]->{"bay"}      = $3;
                    $result_array[$i]->{"numpaths"} = $4;
                    $result_array[$i]->{"status"}   = $5;
                } elsif ( $line =~ /(\d+) entries were displayed/ ) {
                    $i++;
                    $result_array[$i]->{"total_entries_displayed"} = $1;
                }
            } ## end foreach my $line (@lines)
        }
    } ## end else [ if ($raw_output =~ /There are no entries matching your query/)
    return \@result_array;
} ## end sub system_health_node_connectivity_disk_show

## @name ::system_health_node_connectivity_shelf_show
##
## @summary Parses the raw output from the "system health node-connectivity shelf show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_health_node_connectivity_shelf_show with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_health_node_connectivity_shelf_show(%args);
## INPUTS TO METHOD CALL 'system_health_node_connectivity_shelf_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-iomb-adapter' </td> <td> 'iomb-adapter' </td> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> </tr>
## <tr> <td> '-uuid' </td> <td> 'uuid' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> </tr>
## <tr> <td> '-iomb-adapter-slot' </td> <td> 'iomb-adapter-slot' </td> </tr>
## <tr> <td> '-id' </td> <td> 'shelfid' </td> </tr>
## <tr> <td> '-shelf-name' </td> <td> 'shelf-name' </td> </tr>
## <tr> <td> '-num-disks' </td> <td> 'num-disks' </td> </tr>
## <tr> <td> '-ioma-adapter-slot' </td> <td> 'ioma-adapter-slot' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> </tr>
## <tr> <td> '-ioma-adapter' </td> <td> 'ioma-adapter' </td> </tr>
## <tr> <td> '-iomb-adapter-port' </td> <td> 'iomb-adapter-port' </td> </tr>
## <tr> <td> '-ioma-adapter-port' </td> <td> 'ioma-adapter-port' </td> </tr>
## <tr> <td> '-num-paths' </td> <td> 'num-paths' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; system health node-connectivity shelf show -instance; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;                                    Node: csiqa-2040-16b
##&nbsp;                              Shelf UUID: 50:0c:0f:f0:0d:38:b4:3c
##&nbsp;                            Monitor Name: node-connect
##&nbsp;                              Shelf Name: 0c.shelf0
##&nbsp;                         Number of Paths: 1
##&nbsp;                         Number of Disks: 12
##&nbsp;               Adapter Connected to IOMA:
##&nbsp;               Adapter Connected to IOMB: 0c
##&nbsp;Slot number of Adapter Connected to IOMA: 0
##&nbsp;Slot number of Adapter Connected to IOMB: 0
##&nbsp;  Port name of Adapter Connected to IOMA:
##&nbsp;  Port name of Adapter Connected to IOMB: c
##&nbsp;                                  Status: OK
##&nbsp;                                Shelf ID: 0
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'slot_number_of_adapter_connected_to_iomb' => '0',
##&nbsp;            'number_of_paths' => '1',
##&nbsp;            'status' => 'OK',
##&nbsp;            'shelf_id' => '0',
##&nbsp;            'shelf_uuid' => '50:0c:0f:f0:0d:38:b4:3c',
##&nbsp;            'node' => 'csiqa-2040-16b',
##&nbsp;            'port_name_of_adapter_connected_to_iomb' => 'c',
##&nbsp;            'number_of_disks' => '12',
##&nbsp;            'monitor_name' => 'node-connect',
##&nbsp;            'shelf_name' => '0c',
##&nbsp;            'adapter_connected_to_iomb' => '0c',
##&nbsp;            'slot_number_of_adapter_connected_to_ioma' => '0'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub system_health_node_connectivity_shelf_show {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'system health node-connectivity shelf show',
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my ( @result_array, @keyarr, @valarr );
    my $i = -1;
    if ( $raw_output =~ /There are no entries matching your query/ ) {
        $result_array[0]->{"status"} = "no matching entries found";
    } else {
        my @lines = get_lines(
            raw_output      => $raw_output,
            can_throw       => 0,
            response_object => $resp
        );
        if ( $command =~ /-instance$/ ) {
            foreach my $line (@lines) {
                if ( $line =~ /Node:.*/ ) {
                    $i++;
                }
                if ( $line =~ /([\w\s]+):\s+([\w\-\:]+)/ ) {
                    $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
                } elsif ( $line =~ /(\d+) entries were displayed/ ) {
                    $i++;
                    $result_array[$i]->{"total entries displayed"} = $1;
                }
            } ## end foreach my $line (@lines)
        } elsif ( $command =~ /show$/ ) {
            foreach my $line (@lines) {
                if ($line =~ /([\w\-]+)\s+([\w\.]+)\s+(\d+)\s+(\d+)\s+(\w+)/ )
                {
                    $i++;
                    $result_array[$i]->{"node"}     = $1;
                    $result_array[$i]->{"diskname"} = $2;
                    $result_array[$i]->{"bay"}      = $3;
                    $result_array[$i]->{"numpaths"} = $4;
                    $result_array[$i]->{"status"}   = $5;
                } elsif ( $line =~ /(\d+) entries were displayed/ ) {
                    $i++;
                    $result_array[$i]->{"total_entries_displayed"} = $1;
                }
            } ## end foreach my $line (@lines)
        } elsif ( $command =~ /-fields/ ) {
            foreach my $line (@lines) {
                if ( $line =~ /^node.*/ ) {
                    @keyarr = split( /\s+/, $line );
                } elsif ( $line =~ /---------.*/ ) {
                    next;
                } elsif ( $line =~ /(\d+) entries were displayed.$/ ) {
                    $i++;
                    $result_array[$i]->{"total entries displayed"} = $1;
                } elsif ( $line =~ /(\S+\s){0,9}/ ) {
                    @valarr = split( /\s+/, $line );
                    $i++;
                    for ( my $m = 0; $m <= $#keyarr; $m++ ) {
                        $valarr[$m] =~ s/\s+//g;
                        $result_array[$i]->{ $keyarr[$m] } = $valarr[$m];
                    }
                }
            } ## end foreach my $line (@lines)
        } ## end elsif ($command =~ /-fields/)
    } ## end else [ if ($raw_output =~ /There are no entries matching your query/)
    return \@result_array;
} ## end sub system_health_node_connectivity_shelf_show

## @name ::system_health_alert_definition_show
##
## @summary Parses the raw output from the "system health alert definition show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_health_alert_definition_show with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_health_alert_definition_show(%args);
## INPUTS TO METHOD CALL 'system_health_alert_definition_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-alert-id' </td> <td> 'alert-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> <td> '0' </td> </tr>
## <tr> <td> '-perceived-severity' </td> <td> 'perceived-severity' </td> <td> '0' </td> </tr>
## <tr> <td> '-additional-information' </td> <td> 'additional-information' </td> <td> '0' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> <td> '0' </td> </tr>
## <tr> <td> '-subsystem' </td> <td> 'subsystem' </td> <td> '0' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> <td> '1' </td> </tr>
## <tr> <td> '-possible-effect' </td> <td> 'possible-effect' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-type' </td> <td> 'alert-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause-description' </td> <td> 'probable-cause-description' </td> <td> '0' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> <td> '0' </td> </tr>
## <tr> <td> '-corrective-actions' </td> <td> 'corrective-actions' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause' </td> <td> 'probable-cause' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; system health alert definition show -instance; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;                      Node: skotagir-vsim-1
##&nbsp;                   Monitor: node-connect
##&nbsp;            Class of Alert: HaReadyCifsNdo_Alert
##&nbsp;         Severity of Alert: Major
##&nbsp;            Probable Cause: Configuration_error
##&nbsp;Probable Cause Description: One or more files hosted by a volume in an aggregate on node $(cifs_ndo_node_resource.node) have been opened through a continuously available CIFS share with the promise of persistence in the event of a failure. However, the HA relationship with the partner is either not configured or not healthy.
##&nbsp;           Possible Effect: In the event that $(cifs_ndo_node_resource.node) fails, its aggregates will become unavailable and open file state will be lost, which will prevent CIFS clients from recovering persistent open files and may result in a disruption to the associated client applications.
##&nbsp;        Corrective Actions: This alert will persist while a node is in takeover state. If that is the case, issue a giveback to return the aggregates to the partner once it is ready to accept them. Otherwise, verify that there is a properly configured and enabled HA relationship between $(cifs_ndo_node_resource.node) and its partner. Alternatively volumes can be moved to an aggregate on a node that is part of an operable HA pair.
##&nbsp;            Subsystem Name: CIFS-NDO
##&nbsp;  Additional Relevant Data: -
##&nbsp;     Additional Alert Tags: -
##&nbsp;
##&nbsp;                      Node: skotagir-vsim-1
##&nbsp;                   Monitor: node-connect
##&nbsp;            Class of Alert: StorageFailoverImpossible_Alert
##&nbsp;         Severity of Alert: Major
##&nbsp;            Probable Cause: Loss_of_redundancy
##&nbsp;Probable Cause Description: Storage failover is not possible on the node "$(ha_node_pair_info.node-name)" due to some failure condition.
##&nbsp;           Possible Effect: There will be storage downtime if the node "$(ha_node_pair_info.node-name)" fails.
##&nbsp;        Corrective Actions: Take the required actions as determined by the "storage failover show" command. Refer to the "Data ONTAP Cluster-Mode High Availability Configuration Guide" for additional information.
##&nbsp;            Subsystem Name: HA-health
##&nbsp;  Additional Relevant Data: -
##&nbsp;     Additional Alert Tags: -
##&nbsp;
##&nbsp;Warning: Unable to list entries for nphm on node skotagir-vsim-1. RPC: Unable
##&nbsp;         to send
##&nbsp;5 entries were displayed.
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'corrective_actions' => 'This alert will persist while a node is in takeover state. If that is the case, issue a giveback to return the aggregates to the partner once it is ready to accept them. Otherwise, verify that there is a properly configured and enabled HA relationship between $(cifs_ndo_node_resource.node) and its partner. Alternatively volumes can be moved to an aggregate on a node that is part of an operable HA pair.',
##&nbsp;            'monitor' => 'node-connect',
##&nbsp;            'node' => 'skotagir-vsim-1',
##&nbsp;            'probable_cause_description' => 'One or more files hosted by a volume in an aggregate on node $(cifs_ndo_node_resource.node) have been opened through a continuously available CIFS share with the promise of persistence in the event of a failure. However, the HA relationship with the partner is either not configured or not healthy.',
##&nbsp;            'subsystem_name' => 'CIFS-NDO',
##&nbsp;            'additional_alert_tags' => '-',
##&nbsp;            'severity_of_alert' => 'Major',
##&nbsp;            'possible_effect' => 'In the event that $(cifs_ndo_node_resource.node) fails, its aggregates will become unavailable and open file state will be lost, which will prevent CIFS clients from recovering persistent open files and may result in a disruption to the associated client applications.',
##&nbsp;            'additional_relevant_data' => '-',
##&nbsp;            'class_of_alert' => 'HaReadyCifsNdo_Alert',
##&nbsp;            'probable_cause' => 'Configuration_error'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'corrective_actions' => 'Take the required actions as determined by the "storage failover show" command. Refer to the "Data ONTAP Cluster-Mode High Availability Configuration Guide" for additional information.',
##&nbsp;            'monitor' => 'node-connect',
##&nbsp;            'node' => 'skotagir-vsim-1',
##&nbsp;            'probable_cause_description' => 'Storage failover is not possible on the node "$(ha_node_pair_info.node-name)" due to some failure condition.',
##&nbsp;            'subsystem_name' => 'HA-health',
##&nbsp;            'additional_alert_tags' => '-',
##&nbsp;            'severity_of_alert' => 'Major',
##&nbsp;            'possible_effect' => 'There will be storage downtime if the node "$(ha_node_pair_info.node-name)" fails.',
##&nbsp;            'warning' => 'Unable to list entries for nphm on node skotagir-vsim-1. RPC: Unable',
##&nbsp;            'additional_relevant_data' => '-',
##&nbsp;            'class_of_alert' => 'StorageFailoverImpossible_Alert',
##&nbsp;            'probable_cause' => 'Loss_of_redundancy'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'total_entries_displayed' => '5'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; system health alert definition show -instance; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;                      Node: skotagir-vsim-1
##&nbsp;                   Monitor: node-connect
##&nbsp;            Class of Alert: HaReadyCifsNdo_Alert
##&nbsp;         Severity of Alert: Major
##&nbsp;            Probable Cause: Configuration_error
##&nbsp;Probable Cause Description: One or more files hosted by a volume in an aggregate on node $(cifs_ndo_node_resource.node) have been opened through a continuously available CIFS share with the promise of persistence in the event of a failure. However, the HA relationship with the partner is either not configured or not healthy.
##&nbsp;           Possible Effect: In the event that $(cifs_ndo_node_resource.node) fails, its aggregates will become unavailable and open file state will be lost, which will prevent CIFS clients from recovering persistent open files and may result in a disruption to the associated client applications.
##&nbsp;        Corrective Actions: This alert will persist while a node is in takeover state. If that is the case, issue a giveback to return the aggregates to the partner once it is ready to accept them. Otherwise, verify that there is a properly configured and enabled HA relationship between $(cifs_ndo_node_resource.node) and its partner. Alternatively volumes can be moved to an aggregate on a node that is part of an operable HA pair.
##&nbsp;            Subsystem Name: CIFS-NDO
##&nbsp;  Additional Relevant Data: -
##&nbsp;     Additional Alert Tags: -
##&nbsp;
##&nbsp;                      Node: skotagir-vsim-1
##&nbsp;                   Monitor: node-connect
##&nbsp;            Class of Alert: StorageFailoverImpossible_Alert
##&nbsp;         Severity of Alert: Major
##&nbsp;            Probable Cause: Loss_of_redundancy
##&nbsp;Probable Cause Description: Storage failover is not possible on the node "$(ha_node_pair_info.node-name)" due to some failure condition.
##&nbsp;           Possible Effect: There will be storage downtime if the node "$(ha_node_pair_info.node-name)" fails.
##&nbsp;        Corrective Actions: Take the required actions as determined by the "storage failover show" command. Refer to the "Data ONTAP Cluster-Mode High Availability Configuration Guide" for additional information.
##&nbsp;            Subsystem Name: HA-health
##&nbsp;  Additional Relevant Data: -
##&nbsp;     Additional Alert Tags: -
##&nbsp;
##&nbsp;Warning: Unable to list entries for nphm on node skotagir-vsim-1. RPC: Unable
##&nbsp;         to send
##&nbsp;5 entries were displayed.
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'corrective_actions' => 'This alert will persist while a node is in takeover state. If that is the case, issue a giveback to return the aggregates to the partner once it is ready to accept them. Otherwise, verify that there is a properly configured and enabled HA relationship between $(cifs_ndo_node_resource.node) and its partner. Alternatively volumes can be moved to an aggregate on a node that is part of an operable HA pair.',
##&nbsp;            'monitor' => 'node-connect',
##&nbsp;            'node' => 'skotagir-vsim-1',
##&nbsp;            'probable_cause_description' => 'One or more files hosted by a volume in an aggregate on node $(cifs_ndo_node_resource.node) have been opened through a continuously available CIFS share with the promise of persistence in the event of a failure. However, the HA relationship with the partner is either not configured or not healthy.',
##&nbsp;            'subsystem_name' => 'CIFS-NDO',
##&nbsp;            'additional_alert_tags' => '-',
##&nbsp;            'severity_of_alert' => 'Major',
##&nbsp;            'possible_effect' => 'In the event that $(cifs_ndo_node_resource.node) fails, its aggregates will become unavailable and open file state will be lost, which will prevent CIFS clients from recovering persistent open files and may result in a disruption to the associated client applications.',
##&nbsp;            'additional_relevant_data' => '-',
##&nbsp;            'class_of_alert' => 'HaReadyCifsNdo_Alert',
##&nbsp;            'probable_cause' => 'Configuration_error'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'corrective_actions' => 'Take the required actions as determined by the "storage failover show" command. Refer to the "Data ONTAP Cluster-Mode High Availability Configuration Guide" for additional information.',
##&nbsp;            'monitor' => 'node-connect',
##&nbsp;            'node' => 'skotagir-vsim-1',
##&nbsp;            'probable_cause_description' => 'Storage failover is not possible on the node "$(ha_node_pair_info.node-name)" due to some failure condition.',
##&nbsp;            'subsystem_name' => 'HA-health',
##&nbsp;            'additional_alert_tags' => '-',
##&nbsp;            'severity_of_alert' => 'Major',
##&nbsp;            'possible_effect' => 'There will be storage downtime if the node "$(ha_node_pair_info.node-name)" fails.',
##&nbsp;            'warning' => 'Unable to list entries for nphm on node skotagir-vsim-1. RPC: Unable',
##&nbsp;            'additional_relevant_data' => '-',
##&nbsp;            'class_of_alert' => 'StorageFailoverImpossible_Alert',
##&nbsp;            'probable_cause' => 'Loss_of_redundancy'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'total_entries_displayed' => '5'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub system_health_alert_definition_show {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "command"           => 'system health alert definition show',
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my ( @result_array, @keyarr, @valarr );
    my $i = -1;
    my $j;
    my ( $key, $key1, $val );
    if ( $raw_output =~ /There are no entries matching your query/ ) {
        $result_array[0]->{"status"} = "no matching entries found";
    } else {
        my @lines = get_lines(
            raw_output      => $raw_output,
            can_throw       => 0,
            response_object => $resp
        );
        if ( $command =~ /-instance$/ ) {
            while( my $line = shift(@lines)) {
                if ( $line =~ /Node:.*/ ) {
                    $i++;
                }
                if ( $line =~ /(Corrective Actions):\s*(.*)/ ) {
                    $key = $1;
                    my $corrective_action = $2;
                    $line = shift(@lines);
                    while (  $line !~ /Subsystem\s+Name:/ ) {
                        chomp($line);
                        $corrective_action = $corrective_action . " " . $line;
                        $line = shift(@lines);
                    }
                    $result_array[$i]->{to_key_name( 'item' => $key )} = $corrective_action;
                    unshift(@lines , $line);
                } elsif ( $line =~ /([\w\s]+):\s+(.*)/ ) {
                    $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
                } elsif ( $line =~ /(\d+) entries were displayed/ ) {
                    $i++;
                    $result_array[$i]->{"total_entries_displayed"} = $1;
                }
            } ## end foreach my $line (@lines)
        } elsif ( $command =~ /show$/ ) {
            for ( my $row = 0; $row < scalar @lines; $row++ ) {
                if ( $lines[$row] =~ /(\d+) entries were displayed/ ) {
                    $i++;
                    $result_array[$i]->{"total_entries_displayed"} = $1;
                } elsif ( $lines[$row] =~ /Node.*/ ) {
                    next;
                } elsif ( $lines[$row] =~ /-----.*/ ) {
                    next;
                } elsif ( $lines[$row]
                    =~ /^([\w\-\_]+)\s+([\w\-\_]+)\s+([\w\-\_]+)\s+([\w\-\_]+)$/
                    ) {
                    $i++;
                    $result_array[$i]->{"node"}      = $1;
                    $result_array[$i]->{"monitor"}   = $2;
                    $result_array[$i]->{"subsystem"} = $3;
                    my $val = $4;
                    while ( $lines[ ++$row ]
                        !~ /([\w\s]+):\s+([\w\-\:\_\$\s\(\)\.]+)/ ) {
                        my $str = trim_string( string => $lines[$row] );
                        $val = $val . $str;
                    }
                    $row--;
                    $result_array[$i]->{"alertid"} = $val;
                } elsif ( $lines[$row] =~ /(\s+)?Alert$/ ) {
                    next;
                } elsif ( $lines[$row] =~ /(Corrective Actions):\s*(.*)/ ) {
                    my $key = $1;
                    my $val = $2;
                    $row++;
                    while ( $lines[$row] !~ /Additional\s+Info:/ ) {
                        $val = $val . " " . $lines[$row];
                        $row++;
                    }
                    $row--;
                    $result_array[$i]->{to_key_name( 'item' => $key)} = $val;

                } elsif ( $lines[$row] =~ /Probable Cause Description:(.*)/ )
                {
                    my $val = $1;
                    while ( $lines[ ++$row ] !~ /Possible Effect:/ ) {
                        $val = $val . " " . $lines[$row];
                    }
                    $row--;
                    $result_array[$i]->{"probable_cause_description"} = $val;
                } elsif ( $lines[$row] =~ /Possible Effect:(.*)/ ) {
                    my $val = $1;
                    $row++;
                    while ( $lines[$row] !~ /Corrective Actions:/ ) {
                        $val = $val . " " . $lines[$row];
                        $row++;
                    }
                    $row--;
                    $result_array[$i]->{"possible_effect"} = $val;
                } elsif (
                    $lines[$row] =~ /([\w\s]+):\s+([\w\-\:\_\$\s\(\)\.]+)/ ) {
                    $key1 = $1;
                    $result_array[$i]->{ to_key_name( item => $key1 ) } = $2;
                }
            } ## end for (my $row = 0 ; $row...
        } elsif ( $command =~ /-fields/ ) {
            foreach my $line (@lines) {
                if ( $line =~ /^node.*/ ) {
                    @keyarr = split( /\s+/, $line );
                } elsif ( $line =~ /---------.*/ ) {
                    next;
                } elsif ( $line =~ /(\d+) entries were displayed.$/ ) {
                    $i++;
                    $result_array[$i]->{"total_entries_displayed"} = $1;
                } elsif ( $line =~ /(\S+\s){0,9}/ ) {
                    @valarr = split( /\s+/, $line );
                    $i++;
                    for ( my $m = 0; $m <= $#keyarr; $m++ ) {
                        $valarr[$m] =~ s/\s+//g;
                        $result_array[$i]->{ $keyarr[$m] } = $valarr[$m];
                    }
                }
            } ## end foreach my $line (@lines)
        } ## end elsif ($command =~ /-fields/)
    } ## end else [ if ($raw_output =~ /There are no entries matching your query/)
    return \@result_array;
} ## end sub system_health_alert_definition_show
## @name ::system_health_autosupport_trigger_history_show
##
## @summary Parses the raw output from the "system health autosupport trigger history show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_health_autosupport_trigger_history_show with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_health_autosupport_trigger_history_show(%args);
## INPUTS TO METHOD CALL 'system_health_autosupport_trigger_history_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-alert-id' </td> <td> 'alert-id' </td> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> </tr>
## <tr> <td> '-perceived-severity' </td> <td> 'perceived-severity' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> </tr>
## <tr> <td> '-subsystem' </td> <td> 'subsystem' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> </tr>
## <tr> <td> '-autosupport-triggered' </td> <td> 'autosupport-triggered' </td> </tr>
## <tr> <td> '-indication-time' </td> <td> 'indication-time' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> </tr>
## <tr> <td> '-corrective-actions' </td> <td> 'corrective-actions' </td> </tr>
## <tr> <td> '-probable-cause' </td> <td> 'probable-cause' </td> </tr>
## <tr> <td> '-alerting-resource' </td> <td> 'alerting-resource' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; system health autosupport trigger history show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;There are no entries matching your query.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'no matching entries found'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub system_health_autosupport_trigger_history_show {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error      = _check_error(
        "raw_output" => $raw_output,
        "command"    => 'system health autosupport trigger history show',
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my ( @result_array, @keyarr, @valarr );
    my $i = -1;
    if ( $raw_output =~ /There are no entries matching your query/ ) {
        $result_array[0]->{"status"} = "no matching entries found";
    } else {
        my @lines = get_lines(
            raw_output      => $raw_output,
            can_throw       => 0,
            response_object => $resp
        );
        if ( $command =~ /-instance$/ ) {
            foreach my $line (@lines) {
                if ( $line =~ /Node:.*/ ) {
                    $i++;
                }
                if ( $line =~ /([\w\s]+):\s+([\w\-\:]+)/ ) {
                    $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
                } elsif ( $line =~ /(\d+) entries were displayed/ ) {
                    $i++;
                    $result_array[$i]->{"total_entries_displayed"} = $1;
                }
            } ## end foreach my $line (@lines)
        } elsif ( $command =~ /show$/ ) {
            foreach my $line (@lines) {
                if ($line =~ /([\w\-]+)\s+([\w\.]+)\s+(\d+)\s+(\d+)\s+(\w+)/ )
                {
                    $i++;
                    $result_array[$i]->{"node"}     = $1;
                    $result_array[$i]->{"diskname"} = $2;
                    $result_array[$i]->{"bay"}      = $3;
                    $result_array[$i]->{"numpaths"} = $4;
                    $result_array[$i]->{"status"}   = $5;
                } elsif ( $line =~ /(\d+) entries were displayed/ ) {
                    $i++;
                    $result_array[$i]->{"total_entries_displayed"} = $1;
                }
            } ## end foreach my $line (@lines)
        } elsif ( $command =~ /-fields/ ) {
            foreach my $line (@lines) {
                if ( $line =~ /^node.*/ ) {
                    @keyarr = split( /\s+/, $line );
                } elsif ( $line =~ /---------.*/ ) {
                    next;
                } elsif ( $line =~ /(\d+) entries were displayed.$/ ) {
                    $i++;
                    $result_array[$i]->{"total_entries_displayed"} = $1;
                } elsif ( $line =~ /(\S+\s){0,9}/ ) {
                    @valarr = split( /\s+/, $line );
                    $i++;
                    for ( my $m = 0; $m <= $#keyarr; $m++ ) {
                        $valarr[$m] =~ s/\s+//g;
                        $result_array[$i]->{ $keyarr[$m] } = $valarr[$m];
                    }
                }
            } ## end foreach my $line (@lines)
        } ## end elsif ($command =~ /-fields/)
    } ## end else [ if ($raw_output =~ /There are no entries matching your query/)
    return \@result_array;
} ## end sub system_health_autosupport_trigger_history_show

## @name ::system_health_config_show
##
## @summary Parses the raw output from the "system health config show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_health_config_show with any of its options as described in the
##    corresponding cdef file 'system_health.cdefs'
## @synopsis
##      system_health_config_show(%args);
## INPUTS TO METHOD CALL 'system_health_config_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-resources' </td> <td> 'resources' </td> </tr>
## <tr> <td> '-aggregator' </td> <td> 'aggregator' </td> </tr>
## <tr> <td> '-pol-version' </td> <td> 'pol-version' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> </tr>
## <tr> <td> '-health' </td> <td> 'health' </td> </tr>
## <tr> <td> '-subsystem' </td> <td> 'subsystem' </td> </tr>
## <tr> <td> '-node' </td> <td> 'node' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> </tr>
## <tr> <td> '-context' </td> <td> 'context' </td> </tr>
## <tr> <td> '-mon-version' </td> <td> 'mon-version' </td> </tr>
## <tr> <td> '-sub-pol-versions' </td> <td> 'sub-pol-versions' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; system health config show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Node          Monitor                Subsystem         Health
##&nbsp;------------- ---------------------- ----------------- ------------------
##&nbsp;csiqa-2040-16a node-connect          SAS-connect       OK
##&nbsp;csiqa-2040-16a system                SAS-connect       OK
##&nbsp;2 entries were displayed.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'monitor' => 'node-connect',
##&nbsp;            'health' => 'OK',
##&nbsp;            'subsystem' => 'SAS-connect',
##&nbsp;            'node' => 'csiqa-2040-16a'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'monitor' => 'system',
##&nbsp;            'health' => 'OK',
##&nbsp;            'subsystem' => 'SAS-connect',
##&nbsp;            'node' => 'csiqa-2040-16a'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'entries_were_displayed' => '2'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub system_health_config_show {
    my %args = @_;
    my @result_array1;
    my ( $i, $j, $key1, $key2, $value, $value2 );
    my ( $error_message, @result_array );
    my %hash2;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $option = $command;
    $option =~ s/system\s+health\s+config\s+show\s?//;
    if ( $option =~ /(-instance|-monitor)/ ) {
        my @lines = get_lines(
            raw_output      => $output,
            response_object => $resp,
            can_throw       => 1
        );
        my $i = -1;
        my $j = 0;
        foreach my $line (@lines) {
            if ( $line =~ /^(Node):\s+(.*)$/ ) {
                $i++;
                $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
            } elsif ( $line =~ /^([\w\s]+):\s?([\w\.\,\s\-]+|)$/ ) {
                $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
            } elsif ( $line =~ /^(\d+)\s+(entries\s+were\s+displayed).$/ ) {
                $result_array[ $i + 1 ]->{ to_key_name( item => $2 ) } = $1;
            }
        } ## end foreach my $line (@lines)
        return \@result_array;
    } elsif (
        ( $option eq "" )
        || ( $option
            =~ /(-subsystem|-mon-version|-context|-aggregator|-resources)/ )
        ) {
        my @lines = get_lines(
            raw_output      => $output,
            response_object => $resp,
            can_throw       => 1
        );
        my $i = -1;
        my @Head_ar;
        foreach my $line (@lines) {
            if ( $line =~ /Node\s+Monitor\s+Subsystem\s+Health/ ) {
                @Head_ar = split( /\s+/, $line );
            } elsif ( $line =~ /^-+.*/ ) {
                next;
            } elsif ( $line =~ /^([\w\-]+)\s+([\w\-]+)\s+([\w\-]+)\s+(OK)$/ )
            {
                $i++;
                $result_array[$i]->{ to_key_name( item => $Head_ar[0] ) }
                    = $1;
                $result_array[$i]->{ to_key_name( item => $Head_ar[1] ) }
                    = $2;
                $result_array[$i]->{ to_key_name( item => $Head_ar[2] ) }
                    = $3;
                $result_array[$i]->{ to_key_name( item => $Head_ar[3] ) }
                    = $4;
            } elsif ( $line =~ /^(\d+)\s+(entries\s+were\s+displayed).$/ ) {
                $result_array[ $i + 1 ]->{ to_key_name( item => $2 ) } = $1;
            }
        } ## end foreach my $line (@lines)
        return \@result_array;
    } elsif ( $option =~ /-fields/ ) {
        my @lines = get_lines(
            raw_output      => $output,
            response_object => $resp
        );
        my ( @head_arr, @value_arr, $k, $count );
        my $i = 0;
        my $m = 0;
        foreach my $line (@lines) {
            if ( $line =~ /^node\s+monitor.*/ ) {
                @head_arr = split( /\s+/, $line );
                next;
            } elsif ( $line =~ /^-+.*/ ) {
                next;
            } elsif ( $line
                =~ /^([\w\-]+)\s+(node-connect|system-connect|system).*/ ) {
                @value_arr = split( //, $line );
                $count = 0;
                foreach my $k (@value_arr) {
                    if ( $k =~ /\"/ ) {
                        $count = $count + 1;
                    }
                    if ( $count % 2 == 1 ) {
                        $k =~ s/\s+/####/;
                    }
                }
                my $var1 = join( "", @value_arr );
                @value_arr = split( /\s+/, $var1 );
                for ( my $m = 0; $m <= $#head_arr; $m++ ) {
                    $value_arr[$m] =~ s/####/ /g;
                    $value_arr[$m] =~ s/\"//g;
                    $result_array[$i]->{ $head_arr[$m] } = $value_arr[$m];
                }
                $i++;
            } ## end elsif ($line =~ ...
        } ## end foreach my $line (@lines)
        return \@result_array;
    } ## end elsif ($option =~ /-fields/)
}    ## system_health_config_show
## @name ::system_health_status_show
##
## @summary Parses the raw output from the "system health status show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_health_status_show with any of its options as described in the
##    corresponding cdef file 'system_health.cdefs'
## @synopsis
##      system_health_status_show(%args);
## INPUTS TO METHOD CALL 'system_health_status_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; system health status show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Status
##&nbsp;---------------
##&nbsp;OK
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'OK'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub system_health_status_show {
    my %args = @_;
    my @result_array1;
    
    my ( $error_message, @result_array );
    my %hash2;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines(
        raw_output      => $output,
        response_object => $resp,
        can_throw       => 1
    );
    my $key1;
    my $i = 0;
    foreach my $line (@lines) {
        if ( $line =~ /^(Status)$/ ) {
            $key1 = $1;
        } elsif ( $line =~ /^-+.*/ ) {
            next;
        } else {
            $result_array[$i]->{ to_key_name( item => $key1 ) } = $line;
            last;
        }
    }
    return \@result_array;
} ## end sub system_health_status_show
## @name ::system_health_policy_definition_modify
##
## @summary Parses the raw output from the "system health policy definition modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_health_policy_definition_modify with any of its options as described in the
##    corresponding cdef file 'system_health.cdefs'
## @synopsis
##      system_health_policy_definition_modify(%args);
## INPUTS TO METHOD CALL 'system_health_policy_definition_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-enable' </td> <td> 'enable' </td> </tr>
## <tr> <td> '-policy-id' </td> <td> 'policy-id' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> </tr>
## <tr> <td> '-node' </td> <td> 'node' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; system health policy definition modify -node csiqa-2040-16a -monitor node-connect  -policy-id new_polisy -enable true; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub system_health_policy_definition_modify {
    my %args = @_;
    my ( $error_message, @result_array );
    my %hash2;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub system_health_policy_definition_modify
## @name ::system_health_policy_definition_show
##
## @summary Parses the raw output from the "system health policy definition show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_health_policy_definition_show with any of its options as described in the
##    corresponding cdef file 'system_health.cdefs'
## @synopsis
##      system_health_policy_definition_show(%args);
## INPUTS TO METHOD CALL 'system_health_policy_definition_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> '-alert-id' </td> <td> 'alert-id' </td> </tr>
## <tr> <td> '-enable' </td> <td> 'enable' </td> </tr>
## <tr> <td> '-rule-expression' </td> <td> 'rule-expression' </td> </tr>
## <tr> <td> '-responsible-resource-info' </td> <td> 'responsible-resource-info' </td> </tr>
## <tr> <td> '-alert-count' </td> <td> 'alert-count' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> </tr>
## <tr> <td> '-node' </td> <td> 'node' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> </tr>
## <tr> <td> '-where' </td> <td> 'where' </td> </tr>
## <tr> <td> '-prev-alert-creation-time' </td> <td> 'prev-alert-creation-time' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> </tr>
## <tr> <td> '-policy-id' </td> <td> 'policy-id' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; system health policy definition show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;Node          Monitor                Policy
##&nbsp;------------- ---------------------- ----------------------
##&nbsp;csiqa-2040-16a node-connect          ControllerToShelfIomA_Policy
##&nbsp;Policy Rule Expression: nschm_shelf_info.num-paths == 2 &&
##&nbsp;                        nschm_shelf_info.iomb-adapter == NULL
##&nbsp;                 Where: -
##&nbsp;                Enable: true
##&nbsp;              Alert ID: ControllerToShelfIomA_Alert
##&nbsp;      Number of Alerts:            -
##&nbsp;   Previous Alert Time: -
##&nbsp;  Responsible Resource: nschm_shelf_info.name
##&nbsp;
##&nbsp;csiqa-2040-16a node-connect          ControllerToShelfIomB_Policy
##&nbsp;Policy Rule Expression: nschm_shelf_info.num-paths == 2 &&
##&nbsp;                        nschm_shelf_info.ioma-adapter == NULL
##&nbsp;                 Where: -
##&nbsp;                Enable: true
##&nbsp;              Alert ID: ControllerToShelfIomB_Alert
##&nbsp;      Number of Alerts:            -
##&nbsp;   Previous Alert Time: -
##&nbsp;  Responsible Resource: nschm_shelf_info.name
##&nbsp;
##&nbsp;csiqa-2040-16a node-connect          HbaPortsAB_Policy
##&nbsp;Policy Rule Expression: nschm_adapter_count.adapter-count == 1 &&
##&nbsp;                        nschm_shelf_info.num-paths == 2 &&
##&nbsp;                        nschm_shelf_info.ioma-adapter-port == "a" &&
##&nbsp;                        nschm_shelf_info.iomb-adapter-port == "b" &&
##&nbsp;                        nschm_adapter_count.port-count != 2
##&nbsp;                 Where: 1
##&nbsp;                Enable: true
##&nbsp;              Alert ID: HbaPortsAB_Alert
##&nbsp;      Number of Alerts:            -
##&nbsp;   Previous Alert Time: -
##&nbsp;  Responsible Resource: nschm_shelf_info.name
##&nbsp;
##&nbsp;csiqa-2040-16a node-connect          HbaPortsBA_Policy
##&nbsp;Policy Rule Expression: nschm_adapter_count.adapter-count == 1 &&
##&nbsp;                        nschm_shelf_info.num-paths == 2 &&
##&nbsp;                        nschm_shelf_info.ioma-adapter-port == "b" &&
##&nbsp;                        nschm_shelf_info.iomb-adapter-port == "a" &&
##&nbsp;                        nschm_adapter_count.port-count != 2
##&nbsp;                 Where: 1
##&nbsp;                Enable: true
##&nbsp;              Alert ID: HbaPortsBA_Alert
##&nbsp;      Number of Alerts:            -
##&nbsp;   Previous Alert Time: -
##&nbsp;  Responsible Resource: nschm_shelf_info.name
##&nbsp;
##&nbsp;csiqa-2040-16a node-connect          HbaPortsCD_Policy
##&nbsp;Policy Rule Expression: nschm_adapter_count.adapter-count == 1 &&
##&nbsp;                        nschm_shelf_info.num-paths == 2 &&
##&nbsp;                        nschm_shelf_info.ioma-adapter-port == "c" &&
##&nbsp;                        nschm_shelf_info.iomb-adapter-port == "d"
##&nbsp;                 Where: 1
##&nbsp;                Enable: true
##&nbsp;              Alert ID: HbaPortsCD_Alert
##&nbsp;      Number of Alerts:            -
##&nbsp;   Previous Alert Time: -
##&nbsp;  Responsible Resource: nschm_shelf_info.name
##&nbsp;
##&nbsp;csiqa-2040-16a node-connect          HbaPortsDC_Policy
##&nbsp;Policy Rule Expression: nschm_adapter_count.adapter-count == 1 &&
##&nbsp;                        nschm_shelf_info.num-paths == 2 &&
##&nbsp;                        nschm_shelf_info.ioma-adapter-port == "d" &&
##&nbsp;                        nschm_shelf_info.iomb-adapter-port == "c"
##&nbsp;                 Where: 1
##&nbsp;                Enable: true
##&nbsp;              Alert ID: HbaPortsDC_Alert
##&nbsp;      Number of Alerts:            -
##&nbsp;   Previous Alert Time: -
##&nbsp;  Responsible Resource: nschm_shelf_info.name
##&nbsp;
##&nbsp;csiqa-2040-16a node-connect          InterShelfIomCable_Policy
##&nbsp;Policy Rule Expression: nschm_adapter_info.num-ioma != 0 &&
##&nbsp;                        nschm_adapter_info.num-iomb != 0
##&nbsp;                 Where: -
##&nbsp;                Enable: true
##&nbsp;              Alert ID: InterShelfIomCable_Alert
##&nbsp;      Number of Alerts:            -
##&nbsp;   Previous Alert Time: -
##&nbsp;  Responsible Resource: nschm_adapter_info.name
##&nbsp;
##&nbsp;csiqa-2040-16a node-connect          InterShelfIomPortCable_Policy
##&nbsp;Policy Rule Expression: nschm_adapter_info.num-shelf > 0 &&
##&nbsp;                        nschm_adapter_info.num-square-port != 0 &&
##&nbsp;                        nschm_adapter_info.num-circle-port != 0
##&nbsp;                 Where: -
##&nbsp;                Enable: true
##&nbsp;              Alert ID: InterShelfIomPortCable_Alert
##&nbsp;      Number of Alerts:            -
##&nbsp;   Previous Alert Time: -
##&nbsp;  Responsible Resource: nschm_adapter_info.name
##&nbsp;
##&nbsp;8 entries were displayed.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'enable' => 'true',
##&nbsp;            'responsible_resource' => 'nschm_shelf_info.name',
##&nbsp;            'monitor' => 'node-connect',
##&nbsp;            'node' => 'csiqa-2040-16a',
##&nbsp;            'number_of_alerts' => '           -',
##&nbsp;            'where' => '-',
##&nbsp;            'previous_alert_time' => '-',
##&nbsp;            'policy_rule_expression' => 'nschm_shelf_info.num-paths == 2 &&',
##&nbsp;            'policy' => 'ControllerToShelfIomA_Policy',
##&nbsp;            'alert_id' => 'ControllerToShelfIomA_Alert'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'enable' => 'true',
##&nbsp;            'responsible_resource' => 'nschm_shelf_info.name',
##&nbsp;            'monitor' => 'node-connect',
##&nbsp;            'node' => 'csiqa-2040-16a',
##&nbsp;            'number_of_alerts' => '           -',
##&nbsp;            'where' => '-',
##&nbsp;            'previous_alert_time' => '-',
##&nbsp;            'policy_rule_expression' => 'nschm_shelf_info.num-paths == 2 &&',
##&nbsp;            'policy' => 'ControllerToShelfIomB_Policy',
##&nbsp;            'alert_id' => 'ControllerToShelfIomB_Alert'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'enable' => 'true',
##&nbsp;            'responsible_resource' => 'nschm_shelf_info.name',
##&nbsp;            'monitor' => 'node-connect',
##&nbsp;            'node' => 'csiqa-2040-16a',
##&nbsp;            'number_of_alerts' => '           -',
##&nbsp;            'where' => '1',
##&nbsp;            'previous_alert_time' => '-',
##&nbsp;            'policy_rule_expression' => 'nschm_adapter_count.adapter-count == 1 &&',
##&nbsp;            'policy' => 'HbaPortsAB_Policy',
##&nbsp;            'alert_id' => 'HbaPortsAB_Alert'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'enable' => 'true',
##&nbsp;            'responsible_resource' => 'nschm_shelf_info.name',
##&nbsp;            'monitor' => 'node-connect',
##&nbsp;            'node' => 'csiqa-2040-16a',
##&nbsp;            'number_of_alerts' => '           -',
##&nbsp;            'where' => '1',
##&nbsp;            'previous_alert_time' => '-',
##&nbsp;            'policy_rule_expression' => 'nschm_adapter_count.adapter-count == 1 &&',
##&nbsp;            'policy' => 'HbaPortsBA_Policy',
##&nbsp;            'alert_id' => 'HbaPortsBA_Alert'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'enable' => 'true',
##&nbsp;            'responsible_resource' => 'nschm_shelf_info.name',
##&nbsp;            'monitor' => 'node-connect',
##&nbsp;            'node' => 'csiqa-2040-16a',
##&nbsp;            'number_of_alerts' => '           -',
##&nbsp;            'where' => '1',
##&nbsp;            'previous_alert_time' => '-',
##&nbsp;            'policy_rule_expression' => 'nschm_adapter_count.adapter-count == 1 &&',
##&nbsp;            'policy' => 'HbaPortsCD_Policy',
##&nbsp;            'alert_id' => 'HbaPortsCD_Alert'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'enable' => 'true',
##&nbsp;            'responsible_resource' => 'nschm_shelf_info.name',
##&nbsp;            'monitor' => 'node-connect',
##&nbsp;            'node' => 'csiqa-2040-16a',
##&nbsp;            'number_of_alerts' => '           -',
##&nbsp;            'where' => '1',
##&nbsp;            'previous_alert_time' => '-',
##&nbsp;            'policy_rule_expression' => 'nschm_adapter_count.adapter-count == 1 &&',
##&nbsp;            'policy' => 'HbaPortsDC_Policy',
##&nbsp;            'alert_id' => 'HbaPortsDC_Alert'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'enable' => 'true',
##&nbsp;            'responsible_resource' => 'nschm_adapter_info.name',
##&nbsp;            'monitor' => 'node-connect',
##&nbsp;            'node' => 'csiqa-2040-16a',
##&nbsp;            'number_of_alerts' => '           -',
##&nbsp;            'where' => '-',
##&nbsp;            'previous_alert_time' => '-',
##&nbsp;            'policy_rule_expression' => 'nschm_adapter_info.num-ioma != 0 &&',
##&nbsp;            'policy' => 'InterShelfIomCable_Policy',
##&nbsp;            'alert_id' => 'InterShelfIomCable_Alert'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'enable' => 'true',
##&nbsp;            'responsible_resource' => 'nschm_adapter_info.name',
##&nbsp;            'monitor' => 'node-connect',
##&nbsp;            'node' => 'csiqa-2040-16a',
##&nbsp;            'number_of_alerts' => '           -',
##&nbsp;            'where' => '-',
##&nbsp;            'previous_alert_time' => '-',
##&nbsp;            'policy_rule_expression' => 'nschm_adapter_info.num-shelf > 0 &&',
##&nbsp;            'policy' => 'InterShelfIomPortCable_Policy',
##&nbsp;            'alert_id' => 'InterShelfIomPortCable_Alert'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'entries_were_displayed' => '8'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub system_health_policy_definition_show {
    my %args = @_;
    my @result_array1;
    my ( $i, $j, $key1, $key2, $value, $value2 );
    my ( $error_message, @result_array );
    my %hash2;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error   = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $option = $command;
    $option =~ s/system\shealth\spolicy\sdefinition\sshow\s?//;
    if ( $option =~ /(-instance)/ ) {
        my @lines = get_lines(
            raw_output      => $output,
            response_object => $resp,
            can_throw       => 1
        );
        my $i = -1;
        my $j = 0;
        foreach my $line (@lines) {
            if ( $line =~ /^(Node):\s+(.*)$/ ) {
                $i++;
                $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
            } elsif ( $line =~ /^([\w\s]+):\s?([\w\.\,\s\-\=\!\&\>\"]+|)$/ ) {
                $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
            } elsif ( $line =~ /^(\d+)\s+(entries\s+were\s+displayed).$/ ) {
                $result_array[ $i + 1 ]->{ to_key_name( item => $2 ) } = $1;
            }
        } ## end foreach my $line (@lines)
        return \@result_array;
    } elsif ( $option =~ /-fields/ ) {
        my @lines = get_lines(
            raw_output      => $output,
            response_object => $resp
        );
        my ( @head_arr, @value_arr, $k, $count );
        my $i = 0;
        my $m = 0;
        foreach my $line (@lines) {
            if ( $line =~ /^node\s+monitor\s+policy-id.*/ ) {
                @head_arr = split( /\s+/, $line );
                next;
            } elsif ( $line =~ /^-+.*/ ) {
                next;
            } elsif ( $line
                =~ /^([\w\-]+)\s+(node-connect|system-connect|system|controller|chassis|cluster-switch).*/ ) {
                @value_arr = split( //, $line );
                $count = 0;
                foreach my $k (@value_arr) {
                    if ( $k =~ /\"|\{|\}/ ) {
                        $count = $count + 1;
                    }
                    if ( $count % 2 == 1 ) {
                        $k =~ s/\s+/####/;
                    }
                }
                my $var1 = join( "", @value_arr );
                @value_arr = split( /\s+/, $var1 );
                for ( my $m = 0; $m <= $#head_arr; $m++ ) {
                    $value_arr[$m] =~ s/####/ /g;
                    $value_arr[$m] =~ s/\"|\{|\}//g;
                    $result_array[$i]->{ $head_arr[$m] } = $value_arr[$m];
                }
            } elsif ( $line =~ /^(\d+)\s+(entries\s+were\s+displayed).$/ ) {
                $result_array[ $i]->{ to_key_name( item => $2 ) } = $1;
            }
            $i++;
        } ## end foreach my $line (@lines)
        return \@result_array;
    } ## end elsif ($option =~ /-fields/)
    elsif (
        ( $option eq "" )
        || ( $option
            =~ /(-node|-monitor|-policy-id|-rule-expression|-enable|-alert-id|-responsible-resource-info|-alert-count|-prev-alert-creation-time)/
        )
        ) {
        my @lines = get_lines(
            raw_output      => $output,
            response_object => $resp,
            can_throw       => 1
        );
        my $i = -1;
        my @Head_ar;
        foreach my $line (@lines) {
            if ( $line =~ /^Node\s+Monitor\s+Policy$/ ) {
                @Head_ar = split( /\s+/, $line );
            } elsif ( $line =~ /^-+.*/ ) {
                next;
            } elsif ( $line =~ /^([\w\-]+)\s+([\w\-]+)\s+([\w]+)$/ ) {
                $i++;
                $result_array[$i]->{ to_key_name( item => $Head_ar[0] ) }
                    = $1;
                $result_array[$i]->{ to_key_name( item => $Head_ar[1] ) }
                    = $2;
                $result_array[$i]->{ to_key_name( item => $Head_ar[2] ) }
                    = $3;
            } elsif ( $line =~ /^([\w\s]+):\s?([\w\.\,\s\-\=\!\&\>\"]+|)$/ ) {
                $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
            } elsif ( $line =~ /^(\d+)\s+(entries\s+were\s+displayed).$/ ) {
                $result_array[ $i + 1 ]->{ to_key_name( item => $2 ) } = $1;
            }
        } ## end foreach my $line (@lines)
        return \@result_array;
    }
} ## end sub system_health_policy_definition_show

sub _check_error {
    my %args          = @_;
    my $raw_output    = $args{"raw_output"};
    my $command       = $args{"command"};
    my $error_message = $args{"ptr_error_message"};
    my $error         = undef;
    if ( $raw_output =~ /Error: invalid argument/ ) {
        $error         = "InvalidParam";
        $$error_message = $raw_output;
    } elsif ( $raw_output =~ /Error: Missing value for -fields/ ) {
        $error         = "InvalidParamValue";
        $$error_message = $raw_output;
    } elsif ( $raw_output =~ /(Error:.* is an invalid value for field.*)/ ) {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif (
        $raw_output =~ /(Error: You need to provide some values to apply.)/ )
    {
        $error          = "InvalidParamValue";
        $$error_message = $1;
    } elsif ( $raw_output =~ /(Error:.* is not a recognized command)/ ) {
        $error          = "CommandFailed";
        $$error_message = $1;
    } elsif ( $raw_output =~ /Error:\s+command\s+failed:.*/ ) {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    } elsif ( $raw_output =~ /[uU]sage.*/ ) {
        $error          = "CommandFailed";
        $$error_message = $raw_output;
    }
    return $error;
} ## end sub _check_error

## @name ::system_node_service_processor_autosupport
##
## @summary Parses the raw output from the "system node service-processor autosupport " command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_node_service_processor_autosupport with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_node_service_processor_autosupport(%args);
## INPUTS TO METHOD CALL 'system_node_service_processor_autosupport' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-resources' </td> <td> 'resources' </td> <td> '0' </td> </tr>
## <tr> <td> '-port-name' </td> <td> 'port-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-id' </td> <td> 'alert-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-link-status' </td> <td> 'link-status' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-enabled' </td> <td> 'is-enabled' </td> <td> '0' </td> </tr>
## <tr> <td> '-autoupdate-enabled' </td> <td> 'autoupdate-enabled' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-list' </td> <td> 'iomb-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-device-id' </td> <td> 'device-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-prev-alert-creation-time' </td> <td> 'prev-alert-creation-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause-description' </td> <td> 'probable-cause-description' </td> <td> '0' </td> </tr>
## <tr> <td> '-indication-time' </td> <td> 'indication-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-configured' </td> <td> 'is-configured' </td> <td> '0' </td> </tr>
## <tr> <td> '-clear-logs' </td> <td> 'clear-logs' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-shelf' </td> <td> 'numofshelf' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause' </td> <td> 'probable-cause' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter-port' </td> <td> 'ioma-adapter-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter-port' </td> <td> 'iomb-adapter-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-mac' </td> <td> 'mac' </td> <td> '0' </td> </tr>
## <tr> <td> '-square-port-list' </td> <td> 'square-port-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter' </td> <td> 'iomb-adapter' </td> <td> '0' </td> </tr>
## <tr> <td> '-ipmi-ver' </td> <td> 'ipmi-ver' </td> <td> '0' </td> </tr>
## <tr> <td> '-name' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-dual-attached' </td> <td> 'is-dual-attached' </td> <td> '0' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> <td> '1' </td> </tr>
## <tr> <td> '-num-square-port' </td> <td> 'num-square-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-autosupport-triggered' </td> <td> 'autosupport-triggered' </td> <td> '0' </td> </tr>
## <tr> <td> '-mon-version' </td> <td> 'mon-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-sec-name' </td> <td> 'secname' </td> <td> '0' </td> </tr>
## <tr> <td> '-slot-num' </td> <td> 'slot-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-follow' </td> <td> 'follow' </td> <td> '0' </td> </tr>
## <tr> <td> '-address-type' </td> <td> 'address-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-adapter-name' </td> <td> 'adaptername' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-count' </td> <td> 'alert-count' </td> <td> '0' </td> </tr>
## <tr> <td> '-serial-num' </td> <td> 'serial-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-update-type' </td> <td> 'update-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-possible-effect' </td> <td> 'possible-effect' </td> <td> '0' </td> </tr>
## <tr> <td> '-corrective-actions' </td> <td> 'corrective-actions' </td> <td> '0' </td> </tr>
## <tr> <td> '-netmask' </td> <td> 'netmask' </td> <td> '0' </td> </tr>
## <tr> <td> '-sub-pol-versions' </td> <td> 'sub-pol-versions' </td> <td> '0' </td> </tr>
## <tr> <td> '-rule-expression' </td> <td> 'rule-expression' </td> <td> '0' </td> </tr>
## <tr> <td> '-enable' </td> <td> 'enable' </td> <td> '0' </td> </tr>
## <tr> <td> '-responsible-resource-info' </td> <td> 'responsible-resource-info' </td> <td> '0' </td> </tr>
## <tr> <td> '-version' </td> <td> 'version' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-ioma' </td> <td> 'num-ioma' </td> <td> '0' </td> </tr>
## <tr> <td> '-serial-no' </td> <td> 'serialno' </td> <td> '0' </td> </tr>
## <tr> <td> '-additional-information' </td> <td> 'additional-information' </td> <td> '0' </td> </tr>
## <tr> <td> '-subsystem' </td> <td> 'subsystem' </td> <td> '0' </td> </tr>
## <tr> <td> '-health' </td> <td> 'health' </td> <td> '0' </td> </tr>
## <tr> <td> '-where' </td> <td> 'where' </td> <td> '0' </td> </tr>
## <tr> <td> '-autoupdate' </td> <td> 'autoupdate' </td> <td> '0' </td> </tr>
## <tr> <td> '-image' </td> <td> 'image' </td> <td> '0' </td> </tr>
## <tr> <td> '-package' </td> <td> 'package' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-circle-port' </td> <td> 'num-circle-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter-slot' </td> <td> 'ioma-adapter-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-alerting-resource' </td> <td> 'alerting-resource' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-id' </td> <td> 'shelf-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-dhcp' </td> <td> 'dhcp' </td> <td> '0' </td> </tr>
## <tr> <td> 'test' </td> <td> 'test' </td> <td> '1' </td> </tr>
## <tr> <td> '-new-fw-available' </td> <td> 'new-fw-available' </td> <td> '0' </td> </tr>
## <tr> <td> '-disk-name' </td> <td> 'diskname' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-list' </td> <td> 'ioma-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-prefix-length' </td> <td> 'prefix-length' </td> <td> '0' </td> </tr>
## <tr> <td> '-perceived-severity' </td> <td> 'perceived-severity' </td> <td> '0' </td> </tr>
## <tr> <td> '-uuid' </td> <td> 'uuid' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-name' </td> <td> 'shelf-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-new-fw-version' </td> <td> 'new-fw-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-baseline' </td> <td> 'baseline' </td> <td> '0' </td> </tr>
## <tr> <td> '-circle-port-list' </td> <td> 'circle-port-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-bay-no' </td> <td> 'bay-no' </td> <td> '0' </td> </tr>
## <tr> <td> '-address' </td> <td> 'address' </td> <td> '0' </td> </tr>
## <tr> <td> '-part-num' </td> <td> 'part-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-in-use' </td> <td> 'in-use' </td> <td> '0' </td> </tr>
## <tr> <td> '-aggregator' </td> <td> 'aggregator' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter-slot' </td> <td> 'iomb-adapter-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-wwn' </td> <td> 'wwn' </td> <td> '0' </td> </tr>
## <tr> <td> '-id' </td> <td> 'shelfid' </td> <td> '0' </td> </tr>
## <tr> <td> '-revision' </td> <td> 'revision' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter' </td> <td> 'ioma-adapter' </td> <td> '0' </td> </tr>
## <tr> <td> '-policy-id' </td> <td> 'policy-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-type' </td> <td> 'type' </td> <td> '0' </td> </tr>
## <tr> <td> '-product-id' </td> <td> 'product-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-man-id' </td> <td> 'man-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-paths' </td> <td> 'num-paths' </td> <td> '0' </td> </tr>
## <tr> <td> '-dev-rev' </td> <td> 'dev-rev' </td> <td> '0' </td> </tr>
## <tr> <td> '-iom-port' </td> <td> 'iom-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-status' </td> <td> 'status' </td> <td> '0' </td> </tr>
## <tr> <td> '-router-ip' </td> <td> 'router-ip' </td> <td> '0' </td> </tr>
## <tr> <td> '-gateway' </td> <td> 'gateway' </td> <td> '0' </td> </tr>
## <tr> <td> '-pol-version' </td> <td> 'pol-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> <td> '0' </td> </tr>
## <tr> <td> '-link-local-ip' </td> <td> 'link-local-ip' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-disks' </td> <td> 'num-disks' </td> <td> '0' </td> </tr>
## <tr> <td> '-context' </td> <td> 'context' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-list' </td> <td> 'shelf-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-fw-version' </td> <td> 'fw-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-type' </td> <td> 'alert-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-iomb' </td> <td> 'num-iomb' </td> <td> '0' </td> </tr>
## <tr> <td> 'invoke' </td> <td> 'invoke' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; system node service-processor autosupport  invoke; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub system_node_service_processor_autosupport {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub system_node_service_processor_autosupport

## @name ::system_node_service_processor_snmp
##
## @summary Parses the raw output from the "system node service-processor snmp" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_node_service_processor_snmp with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_node_service_processor_snmp(%args);
## INPUTS TO METHOD CALL 'system_node_service_processor_snmp' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-resources' </td> <td> 'resources' </td> <td> '0' </td> </tr>
## <tr> <td> '-port-name' </td> <td> 'port-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-id' </td> <td> 'alert-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-link-status' </td> <td> 'link-status' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-enabled' </td> <td> 'is-enabled' </td> <td> '0' </td> </tr>
## <tr> <td> '-autoupdate-enabled' </td> <td> 'autoupdate-enabled' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-list' </td> <td> 'iomb-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-device-id' </td> <td> 'device-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-prev-alert-creation-time' </td> <td> 'prev-alert-creation-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause-description' </td> <td> 'probable-cause-description' </td> <td> '0' </td> </tr>
## <tr> <td> '-indication-time' </td> <td> 'indication-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-configured' </td> <td> 'is-configured' </td> <td> '0' </td> </tr>
## <tr> <td> '-clear-logs' </td> <td> 'clear-logs' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-shelf' </td> <td> 'numofshelf' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause' </td> <td> 'probable-cause' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter-port' </td> <td> 'ioma-adapter-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter-port' </td> <td> 'iomb-adapter-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-mac' </td> <td> 'mac' </td> <td> '0' </td> </tr>
## <tr> <td> '-square-port-list' </td> <td> 'square-port-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter' </td> <td> 'iomb-adapter' </td> <td> '0' </td> </tr>
## <tr> <td> '-ipmi-ver' </td> <td> 'ipmi-ver' </td> <td> '0' </td> </tr>
## <tr> <td> '-name' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-dual-attached' </td> <td> 'is-dual-attached' </td> <td> '0' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> <td> '1' </td> </tr>
## <tr> <td> '-num-square-port' </td> <td> 'num-square-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-autosupport-triggered' </td> <td> 'autosupport-triggered' </td> <td> '0' </td> </tr>
## <tr> <td> '-mon-version' </td> <td> 'mon-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-sec-name' </td> <td> 'secname' </td> <td> '0' </td> </tr>
## <tr> <td> '-slot-num' </td> <td> 'slot-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-follow' </td> <td> 'follow' </td> <td> '0' </td> </tr>
## <tr> <td> '-address-type' </td> <td> 'address-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-adapter-name' </td> <td> 'adaptername' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-count' </td> <td> 'alert-count' </td> <td> '0' </td> </tr>
## <tr> <td> '-serial-num' </td> <td> 'serial-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-update-type' </td> <td> 'update-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-possible-effect' </td> <td> 'possible-effect' </td> <td> '0' </td> </tr>
## <tr> <td> '-corrective-actions' </td> <td> 'corrective-actions' </td> <td> '0' </td> </tr>
## <tr> <td> '-netmask' </td> <td> 'netmask' </td> <td> '0' </td> </tr>
## <tr> <td> '-sub-pol-versions' </td> <td> 'sub-pol-versions' </td> <td> '0' </td> </tr>
## <tr> <td> '-rule-expression' </td> <td> 'rule-expression' </td> <td> '0' </td> </tr>
## <tr> <td> '-enable' </td> <td> 'enable' </td> <td> '0' </td> </tr>
## <tr> <td> '-responsible-resource-info' </td> <td> 'responsible-resource-info' </td> <td> '0' </td> </tr>
## <tr> <td> '-version' </td> <td> 'version' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-ioma' </td> <td> 'num-ioma' </td> <td> '0' </td> </tr>
## <tr> <td> '-serial-no' </td> <td> 'serialno' </td> <td> '0' </td> </tr>
## <tr> <td> '-additional-information' </td> <td> 'additional-information' </td> <td> '0' </td> </tr>
## <tr> <td> '-subsystem' </td> <td> 'subsystem' </td> <td> '0' </td> </tr>
## <tr> <td> '-health' </td> <td> 'health' </td> <td> '0' </td> </tr>
## <tr> <td> '-where' </td> <td> 'where' </td> <td> '0' </td> </tr>
## <tr> <td> '-autoupdate' </td> <td> 'autoupdate' </td> <td> '0' </td> </tr>
## <tr> <td> '-image' </td> <td> 'image' </td> <td> '0' </td> </tr>
## <tr> <td> '-package' </td> <td> 'package' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-circle-port' </td> <td> 'num-circle-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter-slot' </td> <td> 'ioma-adapter-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-alerting-resource' </td> <td> 'alerting-resource' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-id' </td> <td> 'shelf-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-dhcp' </td> <td> 'dhcp' </td> <td> '0' </td> </tr>
## <tr> <td> 'test' </td> <td> 'test' </td> <td> '1' </td> </tr>
## <tr> <td> '-new-fw-available' </td> <td> 'new-fw-available' </td> <td> '0' </td> </tr>
## <tr> <td> '-disk-name' </td> <td> 'diskname' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-list' </td> <td> 'ioma-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-prefix-length' </td> <td> 'prefix-length' </td> <td> '0' </td> </tr>
## <tr> <td> '-perceived-severity' </td> <td> 'perceived-severity' </td> <td> '0' </td> </tr>
## <tr> <td> '-uuid' </td> <td> 'uuid' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-name' </td> <td> 'shelf-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-new-fw-version' </td> <td> 'new-fw-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-baseline' </td> <td> 'baseline' </td> <td> '0' </td> </tr>
## <tr> <td> '-circle-port-list' </td> <td> 'circle-port-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-bay-no' </td> <td> 'bay-no' </td> <td> '0' </td> </tr>
## <tr> <td> '-address' </td> <td> 'address' </td> <td> '0' </td> </tr>
## <tr> <td> '-part-num' </td> <td> 'part-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-in-use' </td> <td> 'in-use' </td> <td> '0' </td> </tr>
## <tr> <td> '-aggregator' </td> <td> 'aggregator' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter-slot' </td> <td> 'iomb-adapter-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-wwn' </td> <td> 'wwn' </td> <td> '0' </td> </tr>
## <tr> <td> '-id' </td> <td> 'shelfid' </td> <td> '0' </td> </tr>
## <tr> <td> '-revision' </td> <td> 'revision' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter' </td> <td> 'ioma-adapter' </td> <td> '0' </td> </tr>
## <tr> <td> '-policy-id' </td> <td> 'policy-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-type' </td> <td> 'type' </td> <td> '0' </td> </tr>
## <tr> <td> '-product-id' </td> <td> 'product-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-man-id' </td> <td> 'man-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-paths' </td> <td> 'num-paths' </td> <td> '0' </td> </tr>
## <tr> <td> '-dev-rev' </td> <td> 'dev-rev' </td> <td> '0' </td> </tr>
## <tr> <td> '-iom-port' </td> <td> 'iom-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-status' </td> <td> 'status' </td> <td> '0' </td> </tr>
## <tr> <td> '-router-ip' </td> <td> 'router-ip' </td> <td> '0' </td> </tr>
## <tr> <td> '-gateway' </td> <td> 'gateway' </td> <td> '0' </td> </tr>
## <tr> <td> '-pol-version' </td> <td> 'pol-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> <td> '0' </td> </tr>
## <tr> <td> '-link-local-ip' </td> <td> 'link-local-ip' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-disks' </td> <td> 'num-disks' </td> <td> '0' </td> </tr>
## <tr> <td> '-context' </td> <td> 'context' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-list' </td> <td> 'shelf-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-fw-version' </td> <td> 'fw-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-type' </td> <td> 'alert-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-iomb' </td> <td> 'num-iomb' </td> <td> '0' </td> </tr>
## <tr> <td> 'invoke' </td> <td> 'invoke' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; system node service-processor snmp test; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub system_node_service_processor_snmp {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub system_node_service_processor_snmp

## @name ::system_node_service_processor_reboot_sp
##
## @summary Parses the raw output from the "system node service-processor reboot-sp" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_node_service_processor_reboot_sp with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_node_service_processor_reboot_sp(%args);
## INPUTS TO METHOD CALL 'system_node_service_processor_reboot_sp' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q diag; system node service-processor reboot-sp; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub system_node_service_processor_reboot_sp {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub system_node_service_processor_reboot_sp

## @name ::system_node_service_processor_show
##
## @summary Parses the raw output from the "system node service-processor show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_node_service_processor_show with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_node_service_processor_show(%args);
## INPUTS TO METHOD CALL 'system_node_service_processor_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-resources' </td> <td> 'resources' </td> <td> '0' </td> </tr>
## <tr> <td> '-port-name' </td> <td> 'port-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-id' </td> <td> 'alert-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-link-status' </td> <td> 'link-status' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-enabled' </td> <td> 'is-enabled' </td> <td> '0' </td> </tr>
## <tr> <td> '-autoupdate-enabled' </td> <td> 'autoupdate-enabled' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-list' </td> <td> 'iomb-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-device-id' </td> <td> 'device-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-prev-alert-creation-time' </td> <td> 'prev-alert-creation-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause-description' </td> <td> 'probable-cause-description' </td> <td> '0' </td> </tr>
## <tr> <td> '-indication-time' </td> <td> 'indication-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-configured' </td> <td> 'is-configured' </td> <td> '0' </td> </tr>
## <tr> <td> '-clear-logs' </td> <td> 'clear-logs' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-shelf' </td> <td> 'numofshelf' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause' </td> <td> 'probable-cause' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter-port' </td> <td> 'ioma-adapter-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter-port' </td> <td> 'iomb-adapter-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-mac' </td> <td> 'mac' </td> <td> '0' </td> </tr>
## <tr> <td> '-square-port-list' </td> <td> 'square-port-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter' </td> <td> 'iomb-adapter' </td> <td> '0' </td> </tr>
## <tr> <td> '-ipmi-ver' </td> <td> 'ipmi-ver' </td> <td> '0' </td> </tr>
## <tr> <td> '-name' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-dual-attached' </td> <td> 'is-dual-attached' </td> <td> '0' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> <td> '1' </td> </tr>
## <tr> <td> '-num-square-port' </td> <td> 'num-square-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-autosupport-triggered' </td> <td> 'autosupport-triggered' </td> <td> '0' </td> </tr>
## <tr> <td> '-mon-version' </td> <td> 'mon-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-sec-name' </td> <td> 'secname' </td> <td> '0' </td> </tr>
## <tr> <td> '-slot-num' </td> <td> 'slot-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-follow' </td> <td> 'follow' </td> <td> '0' </td> </tr>
## <tr> <td> '-address-type' </td> <td> 'address-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-adapter-name' </td> <td> 'adaptername' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-count' </td> <td> 'alert-count' </td> <td> '0' </td> </tr>
## <tr> <td> '-serial-num' </td> <td> 'serial-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-update-type' </td> <td> 'update-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-possible-effect' </td> <td> 'possible-effect' </td> <td> '0' </td> </tr>
## <tr> <td> '-corrective-actions' </td> <td> 'corrective-actions' </td> <td> '0' </td> </tr>
## <tr> <td> '-netmask' </td> <td> 'netmask' </td> <td> '0' </td> </tr>
## <tr> <td> '-sub-pol-versions' </td> <td> 'sub-pol-versions' </td> <td> '0' </td> </tr>
## <tr> <td> '-rule-expression' </td> <td> 'rule-expression' </td> <td> '0' </td> </tr>
## <tr> <td> '-enable' </td> <td> 'enable' </td> <td> '0' </td> </tr>
## <tr> <td> '-responsible-resource-info' </td> <td> 'responsible-resource-info' </td> <td> '0' </td> </tr>
## <tr> <td> '-version' </td> <td> 'version' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-ioma' </td> <td> 'num-ioma' </td> <td> '0' </td> </tr>
## <tr> <td> '-serial-no' </td> <td> 'serialno' </td> <td> '0' </td> </tr>
## <tr> <td> '-additional-information' </td> <td> 'additional-information' </td> <td> '0' </td> </tr>
## <tr> <td> '-subsystem' </td> <td> 'subsystem' </td> <td> '0' </td> </tr>
## <tr> <td> '-health' </td> <td> 'health' </td> <td> '0' </td> </tr>
## <tr> <td> '-where' </td> <td> 'where' </td> <td> '0' </td> </tr>
## <tr> <td> '-autoupdate' </td> <td> 'autoupdate' </td> <td> '0' </td> </tr>
## <tr> <td> '-image' </td> <td> 'image' </td> <td> '0' </td> </tr>
## <tr> <td> '-package' </td> <td> 'package' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-circle-port' </td> <td> 'num-circle-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter-slot' </td> <td> 'ioma-adapter-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-alerting-resource' </td> <td> 'alerting-resource' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-id' </td> <td> 'shelf-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-dhcp' </td> <td> 'dhcp' </td> <td> '0' </td> </tr>
## <tr> <td> 'test' </td> <td> 'test' </td> <td> '1' </td> </tr>
## <tr> <td> '-new-fw-available' </td> <td> 'new-fw-available' </td> <td> '0' </td> </tr>
## <tr> <td> '-disk-name' </td> <td> 'diskname' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-list' </td> <td> 'ioma-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-prefix-length' </td> <td> 'prefix-length' </td> <td> '0' </td> </tr>
## <tr> <td> '-perceived-severity' </td> <td> 'perceived-severity' </td> <td> '0' </td> </tr>
## <tr> <td> '-uuid' </td> <td> 'uuid' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-name' </td> <td> 'shelf-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-new-fw-version' </td> <td> 'new-fw-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-baseline' </td> <td> 'baseline' </td> <td> '0' </td> </tr>
## <tr> <td> '-circle-port-list' </td> <td> 'circle-port-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-bay-no' </td> <td> 'bay-no' </td> <td> '0' </td> </tr>
## <tr> <td> '-address' </td> <td> 'address' </td> <td> '0' </td> </tr>
## <tr> <td> '-part-num' </td> <td> 'part-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-in-use' </td> <td> 'in-use' </td> <td> '0' </td> </tr>
## <tr> <td> '-aggregator' </td> <td> 'aggregator' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter-slot' </td> <td> 'iomb-adapter-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-wwn' </td> <td> 'wwn' </td> <td> '0' </td> </tr>
## <tr> <td> '-id' </td> <td> 'shelfid' </td> <td> '0' </td> </tr>
## <tr> <td> '-revision' </td> <td> 'revision' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter' </td> <td> 'ioma-adapter' </td> <td> '0' </td> </tr>
## <tr> <td> '-policy-id' </td> <td> 'policy-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-type' </td> <td> 'type' </td> <td> '0' </td> </tr>
## <tr> <td> '-product-id' </td> <td> 'product-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-man-id' </td> <td> 'man-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-paths' </td> <td> 'num-paths' </td> <td> '0' </td> </tr>
## <tr> <td> '-dev-rev' </td> <td> 'dev-rev' </td> <td> '0' </td> </tr>
## <tr> <td> '-iom-port' </td> <td> 'iom-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-status' </td> <td> 'status' </td> <td> '0' </td> </tr>
## <tr> <td> '-router-ip' </td> <td> 'router-ip' </td> <td> '0' </td> </tr>
## <tr> <td> '-gateway' </td> <td> 'gateway' </td> <td> '0' </td> </tr>
## <tr> <td> '-pol-version' </td> <td> 'pol-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> <td> '0' </td> </tr>
## <tr> <td> '-link-local-ip' </td> <td> 'link-local-ip' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-disks' </td> <td> 'num-disks' </td> <td> '0' </td> </tr>
## <tr> <td> '-context' </td> <td> 'context' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-list' </td> <td> 'shelf-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-fw-version' </td> <td> 'fw-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-type' </td> <td> 'alert-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-iomb' </td> <td> 'num-iomb' </td> <td> '0' </td> </tr>
## <tr> <td> 'invoke' </td> <td> 'invoke' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;
##&nbsp;Command = priv set -q test; system node service-processor show -instance; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;
##&nbsp;                               Node: rtp-plat-2240-4-02b
##&nbsp;      Type of the Service Processor: SP
##&nbsp;                             Status: online
##&nbsp;Is the Service Processor Configured: false
##&nbsp;                  Public IP Address: 10.60.172.29
##&nbsp;                        MAC Address: 00:a0:98:18:38:12
##&nbsp;                   Firmware Version: 2.2X2
##&nbsp;                        Part Number: Not Applicable
##&nbsp;                           Revision: Not Applicable
##&nbsp;                      Serial Number: Not Applicable
##&nbsp;                          Device ID: Not Applicable
##&nbsp;                         Product ID: Not Applicable
##&nbsp;                    Manufacturer ID: Not Applicable
##&nbsp;                       IPMI Version: 2.0
##&nbsp;                    Device Revision: Not Applicable
##&nbsp;  Is SP Firmware Autoupdate Enabled: false
##&nbsp;       Is New SP Firmware Available: false
##&nbsp;               New Firmware Version: 0.0.0
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'serial_number' => 'Not Applicable',
##&nbsp;            'type_of_the_service_processor' => 'SP',
##&nbsp;            'status' => 'online',
##&nbsp;            'device_revision' => 'Not Applicable',
##&nbsp;            'new_firmware_version' => '0.0.0',
##&nbsp;            'is_sp_firmware_autoupdate_enabled' => 'false',
##&nbsp;            'ipmi_version' => '2.0',
##&nbsp;            'mac_address' => '00:a0:98:18:38:12',
##&nbsp;            'product_id' => 'Not Applicable',
##&nbsp;            'manufacturer_id' => 'Not Applicable',
##&nbsp;            'is_new_sp_firmware_available' => 'false',
##&nbsp;            'firmware_version' => '2.2X2',
##&nbsp;            'public_ip_address' => '10.60.172.29',
##&nbsp;            'is_the_service_processor_configured' => 'false',
##&nbsp;            'device_id' => 'Not Applicable',
##&nbsp;            'revision' => 'Not Applicable',
##&nbsp;            'part_number' => 'Not Applicable'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; system node service-processor show -fields node,status,is-configured,address,mac,fw-version,part-num,revision,serial-num,device-id,product-id,man-id,ipmi-ver,dev-rev,autoupdate-enabled,new-fw-available,new-fw-version; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;node                status is-configured address      mac               fw-version part-num         revision         serial-num       device-id        product-id       man-id           ipmi-ver dev-rev          autoupdate-enabled new-fw-available new-fw-version
##&nbsp;------------------- ------ ------------- ------------ ----------------- ---------- ---------------- ---------------- ---------------- ---------------- ---------------- ---------------- -------- ---------------- ------------------ ---------------- --------------
##&nbsp;rtp-plat-2240-4-02b online false         10.60.172.29 00:a0:98:18:38:12 2.2X2      "Not Applicable" "Not Applicable" "Not Applicable" "Not Applicable" "Not Applicable" "Not Applicable" 2.0      "Not Applicable" false              false            0.0.0
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'online',
##&nbsp;            'ipmi_ver' => '2.0',
##&nbsp;            'serial_num' => 'Not Applicable',
##&nbsp;            'part_num' => 'Not Applicable',
##&nbsp;            'node' => 'rtp-plat-2240-4-02b',
##&nbsp;            'is_configured' => 'false',
##&nbsp;            'fw_version' => '2.2X2',
##&nbsp;            'product_id' => 'Not Applicable',
##&nbsp;            'address' => '10.60.172.29',
##&nbsp;            'mac' => '00:a0:98:18:38:12',
##&nbsp;            'new_fw_version' => '0.0.0',
##&nbsp;            'man_id' => 'Not Applicable',
##&nbsp;            'device_id' => 'Not Applicable',
##&nbsp;            'new_fw_available' => 'false',
##&nbsp;            'revision' => 'Not Applicable',
##&nbsp;            'autoupdate_enabled' => 'false',
##&nbsp;            'dev_rev' => 'Not Applicable'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; system node service-processor show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;                               Is           Firmware
##&nbsp;Node          Type Status      Configured   Version   IP Address
##&nbsp;------------- ---- ----------- ------------ --------- -------------------------
##&nbsp;rtp-plat-2240-4-02b SP online  true         2.2X6     10.60.172.29
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'is_configured' => 'true',
##&nbsp;            'status' => 'online',
##&nbsp;            'firmware_version' => '2.2X6',
##&nbsp;            'ip_address' => '10.60.172.29',
##&nbsp;            'type' => 'SP',
##&nbsp;            'node' => 'rtp-plat-2240-4-02b'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub system_node_service_processor_show {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( raw_output => $raw_output );
    my ( @head_arr, @value_arr, @result_array, $k, $count );
    my $i = 0;
    my $m = 0;
    foreach my $line (@lines) {
        if ( $line =~ /^[Nn]ode.*/ ) {
            if ( $line =~ /^Node\s+.*/ ) {
                @head_arr = (
                    "node",             "type",
                    "status",           "is_configured",
                    "firmware_version", "ip_address"
                );
            } elsif ( $line =~ /(.*):\s+(.*)/ ) {
                $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
            } else {
                @head_arr = split( /\s+/, $line );
                for ( my $j = 0; $j < @head_arr; $j++ ) {
                    $head_arr[$j] =~ s/\-/_/g;
                }
            }
            next;
        } elsif ( $line =~ /^-.+/ ) {
            next;
        } elsif ( $line =~ /(.*):\s+(.*)/ ) {
            $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
        } elsif ( $line =~ /^\w+.*/ ) {
            @value_arr = split( //, $line );
            $count = 0;
            foreach my $k (@value_arr) {
                if ( $k =~ /\"/ ) {
                    $count = $count + 1;
                }
                if ( $count % 2 == 1 ) {
                    $k =~ s/\s+/####/;
                }
            }
            my $var1 = join( "", @value_arr );
            @value_arr = split( /\s+/, $var1 );
            for ( my $m = 0; $m <= $#head_arr; $m++ ) {
                $value_arr[$m] =~ s/####/ /g;
                $value_arr[$m] =~ s/\"//g;
                $result_array[$i]->{ to_key_name( item => $head_arr[$m] ) }
                    = $value_arr[$m];
            }
        } ## end elsif ($line =~ /^\w+.*/)
    } ## end foreach my $line (@lines)
    return \@result_array;
} ## end sub system_node_service_processor_show

## @name ::system_node_service_processor_network_show
##
## @summary Parses the raw output from the "system node service-processor network show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_node_service_processor_network_show with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_node_service_processor_network_show(%args);
## INPUTS TO METHOD CALL 'system_node_service_processor_network_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-link-status' </td> <td> 'link-status' </td> <td> '0' </td> </tr>
## <tr> <td> '-address-type' </td> <td> 'address-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> <td> '0' </td> </tr>
## <tr> <td> '-gateway' </td> <td> 'gateway' </td> <td> '0' </td> </tr>
## <tr> <td> '-router-ip' </td> <td> 'router-ip' </td> <td> '0' </td> </tr>
## <tr> <td> '-status' </td> <td> 'status' </td> <td> '0' </td> </tr>
## <tr> <td> '-prefix-length' </td> <td> 'prefix-length' </td> <td> '0' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> <td> '1' </td> </tr>
## <tr> <td> '-link-local-ip' </td> <td> 'link-local-ip' </td> <td> '0' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> <td> '0' </td> </tr>
## <tr> <td> '-netmask' </td> <td> 'netmask' </td> <td> '0' </td> </tr>
## <tr> <td> '-ip-address' </td> <td> 'ip-address' </td> <td> '0' </td> </tr>
## <tr> <td> '-type' </td> <td> 'type' </td> <td> '0' </td> </tr>
## <tr> <td> '-mac' </td> <td> 'mac' </td> <td> '0' </td> </tr>
## <tr> <td> '-dhcp' </td> <td> 'dhcp' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; system node service-processor network show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;
##&nbsp;                             Address
##&nbsp;Node          Status         Type      Link state  IP Address
##&nbsp;------------- -------------- --------- ----------- ------------------------
##&nbsp;f2220-142-106 online         IPv4      disabled    -
##&nbsp;                    DHCP: -
##&nbsp;             MAC Address: 00:a0:98:18:d0:54
##&nbsp;Network mask (IPv4 only): -
##&nbsp;         Network Gateway: -
##&nbsp;Prefix length(IPv6 only): -
##&nbsp;f2220-142-106 online         IPv6      disabled    -
##&nbsp;                    DHCP: none
##&nbsp;             MAC Address: 00:a0:98:18:d0:54
##&nbsp;Network mask (IPv4 only): -
##&nbsp;         Network Gateway: -
##&nbsp;Prefix length(IPv6 only): -
##&nbsp;2 entries were displayed.
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'network_mask_ipv4_only' => '-',
##&nbsp;            'status' => 'online',
##&nbsp;            'node' => 'f2220-142-106',
##&nbsp;            'prefix_lengthipv6_only' => '-',
##&nbsp;            'mac_address' => '00:a0:98:18:d0:54',
##&nbsp;            'network_gateway' => '-',
##&nbsp;            'link_state' => 'disabled',
##&nbsp;            'ip_address' => '-',
##&nbsp;            'dhcp' => '-',
##&nbsp;            'address_type' => 'IPv4'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'network_mask_ipv4_only' => '-',
##&nbsp;            'status' => 'online',
##&nbsp;            'node' => 'f2220-142-106',
##&nbsp;            'prefix_lengthipv6_only' => '-',
##&nbsp;            'mac_address' => '00:a0:98:18:d0:54',
##&nbsp;            'network_gateway' => '-',
##&nbsp;            'link_state' => 'disabled',
##&nbsp;            'ip_address' => '-',
##&nbsp;            'dhcp' => 'none',
##&nbsp;            'address_type' => 'IPv6'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub system_node_service_processor_network_show {
    my %args = @_;
    my $error_message;
    my (@result_array);
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $option = $command;

    $option =~ s/system node service-processor network show//;

    if ( $option =~ /-fields/ ) {
        my $k          = 0;
        my @lines      = get_lines( raw_output => $raw_output );
        my $first_line = shift(@lines);
        my @headings   = split( /\s+/, $first_line );
        foreach my $line (@lines) {
            my $i = 0;
            if (   ( $line !~ /(------|entries were displayed)/ )
                && ( $line ne "" ) ) {
                my @values = split( /\s+/, $line );
                foreach my $key (@headings) {
                    $key =~ s/-/_/g;
                    $result_array[$k]->{ to_key_name( item => $key ) }
                        = $values[ $i++ ];
                }

                $k++;
            }
        }
    } else {
        my @lines = get_lines( raw_output => $raw_output );
        my $x = 0;
        my $flag = 0;
        foreach my $line (@lines) {
            if ( $line =~ /(\S+)+\s+(\S+)\s+(IPv4|IPv6)\s+(\S+)\s+(\S+)/ ) {
                $x++ if ($flag);
                $result_array[$x]->{'node'}         = $1;
                $result_array[$x]->{'status'}       = $2;
                $result_array[$x]->{'address_type'} = $3;
                $result_array[$x]->{'link_state'}   = $4;
                $result_array[$x]->{'ip_address'}   = $5;
                $flag = 1;
            } elsif ( $line =~ /(.*):\s+(.*)/ ) {
                $result_array[$x]->{ to_key_name( item => $1 ) } = $2;
                $x++ if ( $option =~ /-address-type/ );
            }
        }

    }
    return \@result_array;

} ## end sub system_node_service_processor_network_show

## @name ::system_node_service_processor_network_modify
##
## @summary Parses the raw output from the "system node service-processor network modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_node_service_processor_network_modify with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_node_service_processor_network_modify(%args);
## INPUTS TO METHOD CALL 'system_node_service_processor_network_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-resources' </td> <td> 'resources' </td> <td> '0' </td> </tr>
## <tr> <td> '-port-name' </td> <td> 'port-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-id' </td> <td> 'alert-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-link-status' </td> <td> 'link-status' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-enabled' </td> <td> 'is-enabled' </td> <td> '0' </td> </tr>
## <tr> <td> '-autoupdate-enabled' </td> <td> 'autoupdate-enabled' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-list' </td> <td> 'iomb-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-device-id' </td> <td> 'device-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-prev-alert-creation-time' </td> <td> 'prev-alert-creation-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause-description' </td> <td> 'probable-cause-description' </td> <td> '0' </td> </tr>
## <tr> <td> '-indication-time' </td> <td> 'indication-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-configured' </td> <td> 'is-configured' </td> <td> '0' </td> </tr>
## <tr> <td> '-clear-logs' </td> <td> 'clear-logs' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-shelf' </td> <td> 'numofshelf' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause' </td> <td> 'probable-cause' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter-port' </td> <td> 'ioma-adapter-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter-port' </td> <td> 'iomb-adapter-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-mac' </td> <td> 'mac' </td> <td> '0' </td> </tr>
## <tr> <td> '-square-port-list' </td> <td> 'square-port-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter' </td> <td> 'iomb-adapter' </td> <td> '0' </td> </tr>
## <tr> <td> '-ipmi-ver' </td> <td> 'ipmi-ver' </td> <td> '0' </td> </tr>
## <tr> <td> '-name' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-dual-attached' </td> <td> 'is-dual-attached' </td> <td> '0' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> <td> '1' </td> </tr>
## <tr> <td> '-num-square-port' </td> <td> 'num-square-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-autosupport-triggered' </td> <td> 'autosupport-triggered' </td> <td> '0' </td> </tr>
## <tr> <td> '-mon-version' </td> <td> 'mon-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-sec-name' </td> <td> 'secname' </td> <td> '0' </td> </tr>
## <tr> <td> '-slot-num' </td> <td> 'slot-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-follow' </td> <td> 'follow' </td> <td> '0' </td> </tr>
## <tr> <td> '-address-type' </td> <td> 'address-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-adapter-name' </td> <td> 'adaptername' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-count' </td> <td> 'alert-count' </td> <td> '0' </td> </tr>
## <tr> <td> '-serial-num' </td> <td> 'serial-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-update-type' </td> <td> 'update-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-possible-effect' </td> <td> 'possible-effect' </td> <td> '0' </td> </tr>
## <tr> <td> '-corrective-actions' </td> <td> 'corrective-actions' </td> <td> '0' </td> </tr>
## <tr> <td> '-netmask' </td> <td> 'netmask' </td> <td> '0' </td> </tr>
## <tr> <td> '-sub-pol-versions' </td> <td> 'sub-pol-versions' </td> <td> '0' </td> </tr>
## <tr> <td> '-rule-expression' </td> <td> 'rule-expression' </td> <td> '0' </td> </tr>
## <tr> <td> '-enable' </td> <td> 'enable' </td> <td> '0' </td> </tr>
## <tr> <td> '-responsible-resource-info' </td> <td> 'responsible-resource-info' </td> <td> '0' </td> </tr>
## <tr> <td> '-version' </td> <td> 'version' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-ioma' </td> <td> 'num-ioma' </td> <td> '0' </td> </tr>
## <tr> <td> '-serial-no' </td> <td> 'serialno' </td> <td> '0' </td> </tr>
## <tr> <td> '-additional-information' </td> <td> 'additional-information' </td> <td> '0' </td> </tr>
## <tr> <td> '-subsystem' </td> <td> 'subsystem' </td> <td> '0' </td> </tr>
## <tr> <td> '-health' </td> <td> 'health' </td> <td> '0' </td> </tr>
## <tr> <td> '-where' </td> <td> 'where' </td> <td> '0' </td> </tr>
## <tr> <td> '-autoupdate' </td> <td> 'autoupdate' </td> <td> '0' </td> </tr>
## <tr> <td> '-image' </td> <td> 'image' </td> <td> '0' </td> </tr>
## <tr> <td> '-package' </td> <td> 'package' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-circle-port' </td> <td> 'num-circle-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter-slot' </td> <td> 'ioma-adapter-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-alerting-resource' </td> <td> 'alerting-resource' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-id' </td> <td> 'shelf-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-dhcp' </td> <td> 'dhcp' </td> <td> '0' </td> </tr>
## <tr> <td> 'test' </td> <td> 'test' </td> <td> '1' </td> </tr>
## <tr> <td> '-new-fw-available' </td> <td> 'new-fw-available' </td> <td> '0' </td> </tr>
## <tr> <td> '-disk-name' </td> <td> 'diskname' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-list' </td> <td> 'ioma-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-prefix-length' </td> <td> 'prefix-length' </td> <td> '0' </td> </tr>
## <tr> <td> '-perceived-severity' </td> <td> 'perceived-severity' </td> <td> '0' </td> </tr>
## <tr> <td> '-uuid' </td> <td> 'uuid' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-name' </td> <td> 'shelf-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-new-fw-version' </td> <td> 'new-fw-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-baseline' </td> <td> 'baseline' </td> <td> '0' </td> </tr>
## <tr> <td> '-circle-port-list' </td> <td> 'circle-port-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-bay-no' </td> <td> 'bay-no' </td> <td> '0' </td> </tr>
## <tr> <td> '-address' </td> <td> 'address' </td> <td> '0' </td> </tr>
## <tr> <td> '-part-num' </td> <td> 'part-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-in-use' </td> <td> 'in-use' </td> <td> '0' </td> </tr>
## <tr> <td> '-aggregator' </td> <td> 'aggregator' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter-slot' </td> <td> 'iomb-adapter-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-wwn' </td> <td> 'wwn' </td> <td> '0' </td> </tr>
## <tr> <td> '-id' </td> <td> 'shelfid' </td> <td> '0' </td> </tr>
## <tr> <td> '-revision' </td> <td> 'revision' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter' </td> <td> 'ioma-adapter' </td> <td> '0' </td> </tr>
## <tr> <td> '-policy-id' </td> <td> 'policy-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-type' </td> <td> 'type' </td> <td> '0' </td> </tr>
## <tr> <td> '-product-id' </td> <td> 'product-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-man-id' </td> <td> 'man-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-paths' </td> <td> 'num-paths' </td> <td> '0' </td> </tr>
## <tr> <td> '-dev-rev' </td> <td> 'dev-rev' </td> <td> '0' </td> </tr>
## <tr> <td> '-iom-port' </td> <td> 'iom-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-status' </td> <td> 'status' </td> <td> '0' </td> </tr>
## <tr> <td> '-router-ip' </td> <td> 'router-ip' </td> <td> '0' </td> </tr>
## <tr> <td> '-gateway' </td> <td> 'gateway' </td> <td> '0' </td> </tr>
## <tr> <td> '-pol-version' </td> <td> 'pol-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> <td> '0' </td> </tr>
## <tr> <td> '-link-local-ip' </td> <td> 'link-local-ip' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-disks' </td> <td> 'num-disks' </td> <td> '0' </td> </tr>
## <tr> <td> '-context' </td> <td> 'context' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-list' </td> <td> 'shelf-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-fw-version' </td> <td> 'fw-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-type' </td> <td> 'alert-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-iomb' </td> <td> 'num-iomb' </td> <td> '0' </td> </tr>
## <tr> <td> 'invoke' </td> <td> 'invoke' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub system_node_service_processor_network_modify {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub system_node_service_processor_network_modify

## @name ::system_node_service_processor_image_show
##
## @summary Parses the raw output from the "system node service-processor image show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_node_service_processor_image_show with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_node_service_processor_image_show(%args);
## INPUTS TO METHOD CALL 'system_node_service_processor_image_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> <td> '0' </td> </tr>
## <tr> <td> '-status' </td> <td> 'status' </td> <td> '0' </td> </tr>
## <tr> <td> '-last-installed-version' </td> <td> 'last-installed-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> <td> '1' </td> </tr>
## <tr> <td> '-autoupdate' </td> <td> 'autoupdate' </td> <td> '0' </td> </tr>
## <tr> <td> '-image' </td> <td> 'image' </td> <td> '0' </td> </tr>
## <tr> <td> '-last-update-status' </td> <td> 'last-update-status' </td> <td> '0' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> <td> '0' </td> </tr>
## <tr> <td> '-type' </td> <td> 'type' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-current' </td> <td> 'is-current' </td> <td> '0' </td> </tr>
## <tr> <td> '-running-version' </td> <td> 'running-version' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; system node service-processor image show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;                           Node: f2220-142-106
##&nbsp;                           Type: SP
##&nbsp;Last Installed Firmware Version: 2.2X6
##&nbsp;                          Image: primary
##&nbsp;                   Image Status: Not-Updating
##&nbsp;               Is Image Current: true
##&nbsp;       Running Firmware Version: 2.2X6
##&nbsp;            Firmware Autoupdate: false
##&nbsp;             Last Update Status: Passed
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'last_update_status' => 'Passed',
##&nbsp;            'image_status' => 'Not-Updating',
##&nbsp;            'node' => 'f2220-142-106',
##&nbsp;            'image' => 'primary',
##&nbsp;            'last_installed_firmware_version' => '2.2X6',
##&nbsp;            'firmware_autoupdate' => 'false',
##&nbsp;            'running_firmware_version' => '2.2X6',
##&nbsp;            'type' => 'SP',
##&nbsp;            'is_image_current' => 'true'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub system_node_service_processor_image_show {
    my %args = @_;
    my $error_message;
    my (@result_array);
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $command    = $resp->get_actual_command();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $option = $command;

    $option =~ s/system node service-processor image show//;

    if ( $option =~ /-fields/ ) {
        my $k          = 0;
        my @lines      = get_lines( raw_output => $raw_output );
        my $first_line = shift(@lines);
        my @headings   = split( /\s+/, $first_line );
        foreach my $line (@lines) {
            my $i = 0;
            if (   ( $line !~ /(------|entries were displayed)/ )
                && ( $line ne "" ) ) {
                my @values = split( /\s+/, $line );
                foreach my $key (@headings) {
                    $result_array[$k]->{$key} = $values[ $i++ ];
                }

                $k++;
            }
        }
    } else {
        my $k = -1;
        my @lines = get_lines( raw_output => $raw_output );
        foreach my $line (@lines) {
            $k++ if ( $line =~ /Node:/ );
            if ( $line =~ /(.*):\s+(.*)/ ) {
                $result_array[$k]->{ to_key_name( item => $1 ) } = $2;
            }
        }
    }
    return \@result_array;

} ## end sub system_node_service_processor_image_show

## @name ::system_node_service_processor_image_show_update_progress
##
## @summary Parses the raw output from the "system node service-processor image show-update-progress" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_node_service_processor_image_show_update_progress with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_node_service_processor_image_show_update_progress(%args);
## INPUTS TO METHOD CALL 'system_node_service_processor_image_show_update_progress' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-resources' </td> <td> 'resources' </td> <td> '0' </td> </tr>
## <tr> <td> '-port-name' </td> <td> 'port-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-id' </td> <td> 'alert-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-link-status' </td> <td> 'link-status' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-enabled' </td> <td> 'is-enabled' </td> <td> '0' </td> </tr>
## <tr> <td> '-autoupdate-enabled' </td> <td> 'autoupdate-enabled' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-list' </td> <td> 'iomb-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-device-id' </td> <td> 'device-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-prev-alert-creation-time' </td> <td> 'prev-alert-creation-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause-description' </td> <td> 'probable-cause-description' </td> <td> '0' </td> </tr>
## <tr> <td> '-indication-time' </td> <td> 'indication-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-configured' </td> <td> 'is-configured' </td> <td> '0' </td> </tr>
## <tr> <td> '-clear-logs' </td> <td> 'clear-logs' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-shelf' </td> <td> 'numofshelf' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause' </td> <td> 'probable-cause' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter-port' </td> <td> 'ioma-adapter-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter-port' </td> <td> 'iomb-adapter-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-mac' </td> <td> 'mac' </td> <td> '0' </td> </tr>
## <tr> <td> '-square-port-list' </td> <td> 'square-port-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter' </td> <td> 'iomb-adapter' </td> <td> '0' </td> </tr>
## <tr> <td> '-ipmi-ver' </td> <td> 'ipmi-ver' </td> <td> '0' </td> </tr>
## <tr> <td> '-name' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-dual-attached' </td> <td> 'is-dual-attached' </td> <td> '0' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> <td> '1' </td> </tr>
## <tr> <td> '-num-square-port' </td> <td> 'num-square-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-autosupport-triggered' </td> <td> 'autosupport-triggered' </td> <td> '0' </td> </tr>
## <tr> <td> '-mon-version' </td> <td> 'mon-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-sec-name' </td> <td> 'secname' </td> <td> '0' </td> </tr>
## <tr> <td> '-slot-num' </td> <td> 'slot-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-follow' </td> <td> 'follow' </td> <td> '0' </td> </tr>
## <tr> <td> '-address-type' </td> <td> 'address-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-adapter-name' </td> <td> 'adaptername' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-count' </td> <td> 'alert-count' </td> <td> '0' </td> </tr>
## <tr> <td> '-serial-num' </td> <td> 'serial-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-update-type' </td> <td> 'update-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-possible-effect' </td> <td> 'possible-effect' </td> <td> '0' </td> </tr>
## <tr> <td> '-corrective-actions' </td> <td> 'corrective-actions' </td> <td> '0' </td> </tr>
## <tr> <td> '-netmask' </td> <td> 'netmask' </td> <td> '0' </td> </tr>
## <tr> <td> '-sub-pol-versions' </td> <td> 'sub-pol-versions' </td> <td> '0' </td> </tr>
## <tr> <td> '-rule-expression' </td> <td> 'rule-expression' </td> <td> '0' </td> </tr>
## <tr> <td> '-enable' </td> <td> 'enable' </td> <td> '0' </td> </tr>
## <tr> <td> '-responsible-resource-info' </td> <td> 'responsible-resource-info' </td> <td> '0' </td> </tr>
## <tr> <td> '-version' </td> <td> 'version' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-ioma' </td> <td> 'num-ioma' </td> <td> '0' </td> </tr>
## <tr> <td> '-serial-no' </td> <td> 'serialno' </td> <td> '0' </td> </tr>
## <tr> <td> '-additional-information' </td> <td> 'additional-information' </td> <td> '0' </td> </tr>
## <tr> <td> '-subsystem' </td> <td> 'subsystem' </td> <td> '0' </td> </tr>
## <tr> <td> '-health' </td> <td> 'health' </td> <td> '0' </td> </tr>
## <tr> <td> '-where' </td> <td> 'where' </td> <td> '0' </td> </tr>
## <tr> <td> '-autoupdate' </td> <td> 'autoupdate' </td> <td> '0' </td> </tr>
## <tr> <td> '-image' </td> <td> 'image' </td> <td> '0' </td> </tr>
## <tr> <td> '-package' </td> <td> 'package' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-circle-port' </td> <td> 'num-circle-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter-slot' </td> <td> 'ioma-adapter-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-alerting-resource' </td> <td> 'alerting-resource' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-id' </td> <td> 'shelf-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-dhcp' </td> <td> 'dhcp' </td> <td> '0' </td> </tr>
## <tr> <td> 'test' </td> <td> 'test' </td> <td> '1' </td> </tr>
## <tr> <td> '-new-fw-available' </td> <td> 'new-fw-available' </td> <td> '0' </td> </tr>
## <tr> <td> '-disk-name' </td> <td> 'diskname' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-list' </td> <td> 'ioma-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-prefix-length' </td> <td> 'prefix-length' </td> <td> '0' </td> </tr>
## <tr> <td> '-perceived-severity' </td> <td> 'perceived-severity' </td> <td> '0' </td> </tr>
## <tr> <td> '-uuid' </td> <td> 'uuid' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-name' </td> <td> 'shelf-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-new-fw-version' </td> <td> 'new-fw-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-baseline' </td> <td> 'baseline' </td> <td> '0' </td> </tr>
## <tr> <td> '-circle-port-list' </td> <td> 'circle-port-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-bay-no' </td> <td> 'bay-no' </td> <td> '0' </td> </tr>
## <tr> <td> '-address' </td> <td> 'address' </td> <td> '0' </td> </tr>
## <tr> <td> '-part-num' </td> <td> 'part-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-in-use' </td> <td> 'in-use' </td> <td> '0' </td> </tr>
## <tr> <td> '-aggregator' </td> <td> 'aggregator' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter-slot' </td> <td> 'iomb-adapter-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-wwn' </td> <td> 'wwn' </td> <td> '0' </td> </tr>
## <tr> <td> '-id' </td> <td> 'shelfid' </td> <td> '0' </td> </tr>
## <tr> <td> '-revision' </td> <td> 'revision' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter' </td> <td> 'ioma-adapter' </td> <td> '0' </td> </tr>
## <tr> <td> '-policy-id' </td> <td> 'policy-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-type' </td> <td> 'type' </td> <td> '0' </td> </tr>
## <tr> <td> '-product-id' </td> <td> 'product-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-man-id' </td> <td> 'man-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-paths' </td> <td> 'num-paths' </td> <td> '0' </td> </tr>
## <tr> <td> '-dev-rev' </td> <td> 'dev-rev' </td> <td> '0' </td> </tr>
## <tr> <td> '-iom-port' </td> <td> 'iom-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-status' </td> <td> 'status' </td> <td> '0' </td> </tr>
## <tr> <td> '-router-ip' </td> <td> 'router-ip' </td> <td> '0' </td> </tr>
## <tr> <td> '-gateway' </td> <td> 'gateway' </td> <td> '0' </td> </tr>
## <tr> <td> '-pol-version' </td> <td> 'pol-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> <td> '0' </td> </tr>
## <tr> <td> '-link-local-ip' </td> <td> 'link-local-ip' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-disks' </td> <td> 'num-disks' </td> <td> '0' </td> </tr>
## <tr> <td> '-context' </td> <td> 'context' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-list' </td> <td> 'shelf-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-fw-version' </td> <td> 'fw-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-type' </td> <td> 'alert-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-iomb' </td> <td> 'num-iomb' </td> <td> '0' </td> </tr>
## <tr> <td> 'invoke' </td> <td> 'invoke' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q diag; system node service-processor image show-update-progress -node rtp-plat-6280-01 -follow false; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub system_node_service_processor_image_show_update_progress {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub system_node_service_processor_image_show_update_progress

## @name ::system_node_service_processor_image_update
##
## @summary Parses the raw output from the "system node service-processor image update" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_node_service_processor_image_update with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_node_service_processor_image_update(%args);
## INPUTS TO METHOD CALL 'system_node_service_processor_image_update' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-resources' </td> <td> 'resources' </td> <td> '0' </td> </tr>
## <tr> <td> '-port-name' </td> <td> 'port-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-id' </td> <td> 'alert-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-link-status' </td> <td> 'link-status' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-enabled' </td> <td> 'is-enabled' </td> <td> '0' </td> </tr>
## <tr> <td> '-autoupdate-enabled' </td> <td> 'autoupdate-enabled' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-list' </td> <td> 'iomb-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-device-id' </td> <td> 'device-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-prev-alert-creation-time' </td> <td> 'prev-alert-creation-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause-description' </td> <td> 'probable-cause-description' </td> <td> '0' </td> </tr>
## <tr> <td> '-indication-time' </td> <td> 'indication-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-configured' </td> <td> 'is-configured' </td> <td> '0' </td> </tr>
## <tr> <td> '-clear-logs' </td> <td> 'clear-logs' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-shelf' </td> <td> 'numofshelf' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause' </td> <td> 'probable-cause' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter-port' </td> <td> 'ioma-adapter-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter-port' </td> <td> 'iomb-adapter-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-mac' </td> <td> 'mac' </td> <td> '0' </td> </tr>
## <tr> <td> '-square-port-list' </td> <td> 'square-port-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter' </td> <td> 'iomb-adapter' </td> <td> '0' </td> </tr>
## <tr> <td> '-ipmi-ver' </td> <td> 'ipmi-ver' </td> <td> '0' </td> </tr>
## <tr> <td> '-name' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-dual-attached' </td> <td> 'is-dual-attached' </td> <td> '0' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> <td> '1' </td> </tr>
## <tr> <td> '-num-square-port' </td> <td> 'num-square-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-autosupport-triggered' </td> <td> 'autosupport-triggered' </td> <td> '0' </td> </tr>
## <tr> <td> '-mon-version' </td> <td> 'mon-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-sec-name' </td> <td> 'secname' </td> <td> '0' </td> </tr>
## <tr> <td> '-slot-num' </td> <td> 'slot-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-follow' </td> <td> 'follow' </td> <td> '0' </td> </tr>
## <tr> <td> '-address-type' </td> <td> 'address-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-adapter-name' </td> <td> 'adaptername' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-count' </td> <td> 'alert-count' </td> <td> '0' </td> </tr>
## <tr> <td> '-serial-num' </td> <td> 'serial-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-update-type' </td> <td> 'update-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-possible-effect' </td> <td> 'possible-effect' </td> <td> '0' </td> </tr>
## <tr> <td> '-corrective-actions' </td> <td> 'corrective-actions' </td> <td> '0' </td> </tr>
## <tr> <td> '-netmask' </td> <td> 'netmask' </td> <td> '0' </td> </tr>
## <tr> <td> '-sub-pol-versions' </td> <td> 'sub-pol-versions' </td> <td> '0' </td> </tr>
## <tr> <td> '-rule-expression' </td> <td> 'rule-expression' </td> <td> '0' </td> </tr>
## <tr> <td> '-enable' </td> <td> 'enable' </td> <td> '0' </td> </tr>
## <tr> <td> '-responsible-resource-info' </td> <td> 'responsible-resource-info' </td> <td> '0' </td> </tr>
## <tr> <td> '-version' </td> <td> 'version' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-ioma' </td> <td> 'num-ioma' </td> <td> '0' </td> </tr>
## <tr> <td> '-serial-no' </td> <td> 'serialno' </td> <td> '0' </td> </tr>
## <tr> <td> '-additional-information' </td> <td> 'additional-information' </td> <td> '0' </td> </tr>
## <tr> <td> '-subsystem' </td> <td> 'subsystem' </td> <td> '0' </td> </tr>
## <tr> <td> '-health' </td> <td> 'health' </td> <td> '0' </td> </tr>
## <tr> <td> '-where' </td> <td> 'where' </td> <td> '0' </td> </tr>
## <tr> <td> '-autoupdate' </td> <td> 'autoupdate' </td> <td> '0' </td> </tr>
## <tr> <td> '-image' </td> <td> 'image' </td> <td> '0' </td> </tr>
## <tr> <td> '-package' </td> <td> 'package' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-circle-port' </td> <td> 'num-circle-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter-slot' </td> <td> 'ioma-adapter-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-alerting-resource' </td> <td> 'alerting-resource' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-id' </td> <td> 'shelf-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-dhcp' </td> <td> 'dhcp' </td> <td> '0' </td> </tr>
## <tr> <td> 'test' </td> <td> 'test' </td> <td> '1' </td> </tr>
## <tr> <td> '-new-fw-available' </td> <td> 'new-fw-available' </td> <td> '0' </td> </tr>
## <tr> <td> '-disk-name' </td> <td> 'diskname' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-list' </td> <td> 'ioma-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-prefix-length' </td> <td> 'prefix-length' </td> <td> '0' </td> </tr>
## <tr> <td> '-perceived-severity' </td> <td> 'perceived-severity' </td> <td> '0' </td> </tr>
## <tr> <td> '-uuid' </td> <td> 'uuid' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-name' </td> <td> 'shelf-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-new-fw-version' </td> <td> 'new-fw-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-baseline' </td> <td> 'baseline' </td> <td> '0' </td> </tr>
## <tr> <td> '-circle-port-list' </td> <td> 'circle-port-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-bay-no' </td> <td> 'bay-no' </td> <td> '0' </td> </tr>
## <tr> <td> '-address' </td> <td> 'address' </td> <td> '0' </td> </tr>
## <tr> <td> '-part-num' </td> <td> 'part-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-in-use' </td> <td> 'in-use' </td> <td> '0' </td> </tr>
## <tr> <td> '-aggregator' </td> <td> 'aggregator' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter-slot' </td> <td> 'iomb-adapter-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-wwn' </td> <td> 'wwn' </td> <td> '0' </td> </tr>
## <tr> <td> '-id' </td> <td> 'shelfid' </td> <td> '0' </td> </tr>
## <tr> <td> '-revision' </td> <td> 'revision' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter' </td> <td> 'ioma-adapter' </td> <td> '0' </td> </tr>
## <tr> <td> '-policy-id' </td> <td> 'policy-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-type' </td> <td> 'type' </td> <td> '0' </td> </tr>
## <tr> <td> '-product-id' </td> <td> 'product-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-man-id' </td> <td> 'man-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-paths' </td> <td> 'num-paths' </td> <td> '0' </td> </tr>
## <tr> <td> '-dev-rev' </td> <td> 'dev-rev' </td> <td> '0' </td> </tr>
## <tr> <td> '-iom-port' </td> <td> 'iom-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-status' </td> <td> 'status' </td> <td> '0' </td> </tr>
## <tr> <td> '-router-ip' </td> <td> 'router-ip' </td> <td> '0' </td> </tr>
## <tr> <td> '-gateway' </td> <td> 'gateway' </td> <td> '0' </td> </tr>
## <tr> <td> '-pol-version' </td> <td> 'pol-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> <td> '0' </td> </tr>
## <tr> <td> '-link-local-ip' </td> <td> 'link-local-ip' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-disks' </td> <td> 'num-disks' </td> <td> '0' </td> </tr>
## <tr> <td> '-context' </td> <td> 'context' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-list' </td> <td> 'shelf-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-fw-version' </td> <td> 'fw-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-type' </td> <td> 'alert-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-iomb' </td> <td> 'num-iomb' </td> <td> '0' </td> </tr>
## <tr> <td> 'invoke' </td> <td> 'invoke' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub system_node_service_processor_image_update {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub system_node_service_processor_image_update

## @name ::system_node_service_processor_image_modify
##
## @summary Parses the raw output from the "system node service-processor image modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_node_service_processor_image_modify with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_node_service_processor_image_modify(%args);
## INPUTS TO METHOD CALL 'system_node_service_processor_image_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-resources' </td> <td> 'resources' </td> <td> '0' </td> </tr>
## <tr> <td> '-port-name' </td> <td> 'port-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-id' </td> <td> 'alert-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-link-status' </td> <td> 'link-status' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-enabled' </td> <td> 'is-enabled' </td> <td> '0' </td> </tr>
## <tr> <td> '-autoupdate-enabled' </td> <td> 'autoupdate-enabled' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-list' </td> <td> 'iomb-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-device-id' </td> <td> 'device-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-prev-alert-creation-time' </td> <td> 'prev-alert-creation-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause-description' </td> <td> 'probable-cause-description' </td> <td> '0' </td> </tr>
## <tr> <td> '-indication-time' </td> <td> 'indication-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-configured' </td> <td> 'is-configured' </td> <td> '0' </td> </tr>
## <tr> <td> '-clear-logs' </td> <td> 'clear-logs' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-shelf' </td> <td> 'numofshelf' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause' </td> <td> 'probable-cause' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter-port' </td> <td> 'ioma-adapter-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter-port' </td> <td> 'iomb-adapter-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-mac' </td> <td> 'mac' </td> <td> '0' </td> </tr>
## <tr> <td> '-square-port-list' </td> <td> 'square-port-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter' </td> <td> 'iomb-adapter' </td> <td> '0' </td> </tr>
## <tr> <td> '-ipmi-ver' </td> <td> 'ipmi-ver' </td> <td> '0' </td> </tr>
## <tr> <td> '-name' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-dual-attached' </td> <td> 'is-dual-attached' </td> <td> '0' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> <td> '1' </td> </tr>
## <tr> <td> '-num-square-port' </td> <td> 'num-square-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-autosupport-triggered' </td> <td> 'autosupport-triggered' </td> <td> '0' </td> </tr>
## <tr> <td> '-mon-version' </td> <td> 'mon-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-sec-name' </td> <td> 'secname' </td> <td> '0' </td> </tr>
## <tr> <td> '-slot-num' </td> <td> 'slot-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-follow' </td> <td> 'follow' </td> <td> '0' </td> </tr>
## <tr> <td> '-address-type' </td> <td> 'address-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-adapter-name' </td> <td> 'adaptername' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-count' </td> <td> 'alert-count' </td> <td> '0' </td> </tr>
## <tr> <td> '-serial-num' </td> <td> 'serial-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-update-type' </td> <td> 'update-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-possible-effect' </td> <td> 'possible-effect' </td> <td> '0' </td> </tr>
## <tr> <td> '-corrective-actions' </td> <td> 'corrective-actions' </td> <td> '0' </td> </tr>
## <tr> <td> '-netmask' </td> <td> 'netmask' </td> <td> '0' </td> </tr>
## <tr> <td> '-sub-pol-versions' </td> <td> 'sub-pol-versions' </td> <td> '0' </td> </tr>
## <tr> <td> '-rule-expression' </td> <td> 'rule-expression' </td> <td> '0' </td> </tr>
## <tr> <td> '-enable' </td> <td> 'enable' </td> <td> '0' </td> </tr>
## <tr> <td> '-responsible-resource-info' </td> <td> 'responsible-resource-info' </td> <td> '0' </td> </tr>
## <tr> <td> '-version' </td> <td> 'version' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-ioma' </td> <td> 'num-ioma' </td> <td> '0' </td> </tr>
## <tr> <td> '-serial-no' </td> <td> 'serialno' </td> <td> '0' </td> </tr>
## <tr> <td> '-additional-information' </td> <td> 'additional-information' </td> <td> '0' </td> </tr>
## <tr> <td> '-subsystem' </td> <td> 'subsystem' </td> <td> '0' </td> </tr>
## <tr> <td> '-health' </td> <td> 'health' </td> <td> '0' </td> </tr>
## <tr> <td> '-where' </td> <td> 'where' </td> <td> '0' </td> </tr>
## <tr> <td> '-autoupdate' </td> <td> 'autoupdate' </td> <td> '0' </td> </tr>
## <tr> <td> '-image' </td> <td> 'image' </td> <td> '0' </td> </tr>
## <tr> <td> '-package' </td> <td> 'package' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-circle-port' </td> <td> 'num-circle-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter-slot' </td> <td> 'ioma-adapter-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-alerting-resource' </td> <td> 'alerting-resource' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-id' </td> <td> 'shelf-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-dhcp' </td> <td> 'dhcp' </td> <td> '0' </td> </tr>
## <tr> <td> 'test' </td> <td> 'test' </td> <td> '1' </td> </tr>
## <tr> <td> '-new-fw-available' </td> <td> 'new-fw-available' </td> <td> '0' </td> </tr>
## <tr> <td> '-disk-name' </td> <td> 'diskname' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-list' </td> <td> 'ioma-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-prefix-length' </td> <td> 'prefix-length' </td> <td> '0' </td> </tr>
## <tr> <td> '-perceived-severity' </td> <td> 'perceived-severity' </td> <td> '0' </td> </tr>
## <tr> <td> '-uuid' </td> <td> 'uuid' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-name' </td> <td> 'shelf-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-new-fw-version' </td> <td> 'new-fw-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-baseline' </td> <td> 'baseline' </td> <td> '0' </td> </tr>
## <tr> <td> '-circle-port-list' </td> <td> 'circle-port-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-bay-no' </td> <td> 'bay-no' </td> <td> '0' </td> </tr>
## <tr> <td> '-address' </td> <td> 'address' </td> <td> '0' </td> </tr>
## <tr> <td> '-part-num' </td> <td> 'part-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-in-use' </td> <td> 'in-use' </td> <td> '0' </td> </tr>
## <tr> <td> '-aggregator' </td> <td> 'aggregator' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter-slot' </td> <td> 'iomb-adapter-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-wwn' </td> <td> 'wwn' </td> <td> '0' </td> </tr>
## <tr> <td> '-id' </td> <td> 'shelfid' </td> <td> '0' </td> </tr>
## <tr> <td> '-revision' </td> <td> 'revision' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter' </td> <td> 'ioma-adapter' </td> <td> '0' </td> </tr>
## <tr> <td> '-policy-id' </td> <td> 'policy-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-type' </td> <td> 'type' </td> <td> '0' </td> </tr>
## <tr> <td> '-product-id' </td> <td> 'product-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-man-id' </td> <td> 'man-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-paths' </td> <td> 'num-paths' </td> <td> '0' </td> </tr>
## <tr> <td> '-dev-rev' </td> <td> 'dev-rev' </td> <td> '0' </td> </tr>
## <tr> <td> '-iom-port' </td> <td> 'iom-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-status' </td> <td> 'status' </td> <td> '0' </td> </tr>
## <tr> <td> '-router-ip' </td> <td> 'router-ip' </td> <td> '0' </td> </tr>
## <tr> <td> '-gateway' </td> <td> 'gateway' </td> <td> '0' </td> </tr>
## <tr> <td> '-pol-version' </td> <td> 'pol-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> <td> '0' </td> </tr>
## <tr> <td> '-link-local-ip' </td> <td> 'link-local-ip' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-disks' </td> <td> 'num-disks' </td> <td> '0' </td> </tr>
## <tr> <td> '-context' </td> <td> 'context' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-list' </td> <td> 'shelf-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-fw-version' </td> <td> 'fw-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-type' </td> <td> 'alert-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-iomb' </td> <td> 'num-iomb' </td> <td> '0' </td> </tr>
## <tr> <td> 'invoke' </td> <td> 'invoke' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub system_node_service_processor_image_modify {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub system_node_service_processor_image_modify

## @name ::debug_service_processor_log_trigger_upload
##
## @summary Parses the raw output from the "debug service-processor log trigger-upload" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from debug_service_processor_log_trigger_upload with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      debug_service_processor_log_trigger_upload(%args);
## INPUTS TO METHOD CALL 'debug_service_processor_log_trigger_upload' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-resources' </td> <td> 'resources' </td> <td> '0' </td> </tr>
## <tr> <td> '-port-name' </td> <td> 'port-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-id' </td> <td> 'alert-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-link-status' </td> <td> 'link-status' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-enabled' </td> <td> 'is-enabled' </td> <td> '0' </td> </tr>
## <tr> <td> '-autoupdate-enabled' </td> <td> 'autoupdate-enabled' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-list' </td> <td> 'iomb-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-device-id' </td> <td> 'device-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-prev-alert-creation-time' </td> <td> 'prev-alert-creation-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause-description' </td> <td> 'probable-cause-description' </td> <td> '0' </td> </tr>
## <tr> <td> '-indication-time' </td> <td> 'indication-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-configured' </td> <td> 'is-configured' </td> <td> '0' </td> </tr>
## <tr> <td> '-clear-logs' </td> <td> 'clear-logs' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-shelf' </td> <td> 'numofshelf' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause' </td> <td> 'probable-cause' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter-port' </td> <td> 'ioma-adapter-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter-port' </td> <td> 'iomb-adapter-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-mac' </td> <td> 'mac' </td> <td> '0' </td> </tr>
## <tr> <td> '-square-port-list' </td> <td> 'square-port-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter' </td> <td> 'iomb-adapter' </td> <td> '0' </td> </tr>
## <tr> <td> '-ipmi-ver' </td> <td> 'ipmi-ver' </td> <td> '0' </td> </tr>
## <tr> <td> '-name' </td> <td> 'name' </td> <td> '0' </td> </tr>
## <tr> <td> '-is-dual-attached' </td> <td> 'is-dual-attached' </td> <td> '0' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> <td> '1' </td> </tr>
## <tr> <td> '-num-square-port' </td> <td> 'num-square-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-autosupport-triggered' </td> <td> 'autosupport-triggered' </td> <td> '0' </td> </tr>
## <tr> <td> '-mon-version' </td> <td> 'mon-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-sec-name' </td> <td> 'secname' </td> <td> '0' </td> </tr>
## <tr> <td> '-slot-num' </td> <td> 'slot-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-follow' </td> <td> 'follow' </td> <td> '0' </td> </tr>
## <tr> <td> '-address-type' </td> <td> 'address-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-adapter-name' </td> <td> 'adaptername' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-count' </td> <td> 'alert-count' </td> <td> '0' </td> </tr>
## <tr> <td> '-serial-num' </td> <td> 'serial-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-update-type' </td> <td> 'update-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-possible-effect' </td> <td> 'possible-effect' </td> <td> '0' </td> </tr>
## <tr> <td> '-corrective-actions' </td> <td> 'corrective-actions' </td> <td> '0' </td> </tr>
## <tr> <td> '-netmask' </td> <td> 'netmask' </td> <td> '0' </td> </tr>
## <tr> <td> '-sub-pol-versions' </td> <td> 'sub-pol-versions' </td> <td> '0' </td> </tr>
## <tr> <td> '-rule-expression' </td> <td> 'rule-expression' </td> <td> '0' </td> </tr>
## <tr> <td> '-enable' </td> <td> 'enable' </td> <td> '0' </td> </tr>
## <tr> <td> '-responsible-resource-info' </td> <td> 'responsible-resource-info' </td> <td> '0' </td> </tr>
## <tr> <td> '-version' </td> <td> 'version' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-ioma' </td> <td> 'num-ioma' </td> <td> '0' </td> </tr>
## <tr> <td> '-serial-no' </td> <td> 'serialno' </td> <td> '0' </td> </tr>
## <tr> <td> '-additional-information' </td> <td> 'additional-information' </td> <td> '0' </td> </tr>
## <tr> <td> '-subsystem' </td> <td> 'subsystem' </td> <td> '0' </td> </tr>
## <tr> <td> '-health' </td> <td> 'health' </td> <td> '0' </td> </tr>
## <tr> <td> '-where' </td> <td> 'where' </td> <td> '0' </td> </tr>
## <tr> <td> '-autoupdate' </td> <td> 'autoupdate' </td> <td> '0' </td> </tr>
## <tr> <td> '-image' </td> <td> 'image' </td> <td> '0' </td> </tr>
## <tr> <td> '-package' </td> <td> 'package' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-circle-port' </td> <td> 'num-circle-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter-slot' </td> <td> 'ioma-adapter-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-alerting-resource' </td> <td> 'alerting-resource' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-id' </td> <td> 'shelf-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-dhcp' </td> <td> 'dhcp' </td> <td> '0' </td> </tr>
## <tr> <td> 'test' </td> <td> 'test' </td> <td> '1' </td> </tr>
## <tr> <td> '-new-fw-available' </td> <td> 'new-fw-available' </td> <td> '0' </td> </tr>
## <tr> <td> '-disk-name' </td> <td> 'diskname' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-list' </td> <td> 'ioma-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-prefix-length' </td> <td> 'prefix-length' </td> <td> '0' </td> </tr>
## <tr> <td> '-perceived-severity' </td> <td> 'perceived-severity' </td> <td> '0' </td> </tr>
## <tr> <td> '-uuid' </td> <td> 'uuid' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-name' </td> <td> 'shelf-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-new-fw-version' </td> <td> 'new-fw-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-baseline' </td> <td> 'baseline' </td> <td> '0' </td> </tr>
## <tr> <td> '-circle-port-list' </td> <td> 'circle-port-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-bay-no' </td> <td> 'bay-no' </td> <td> '0' </td> </tr>
## <tr> <td> '-address' </td> <td> 'address' </td> <td> '0' </td> </tr>
## <tr> <td> '-part-num' </td> <td> 'part-num' </td> <td> '0' </td> </tr>
## <tr> <td> '-in-use' </td> <td> 'in-use' </td> <td> '0' </td> </tr>
## <tr> <td> '-aggregator' </td> <td> 'aggregator' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> <td> '0' </td> </tr>
## <tr> <td> '-iomb-adapter-slot' </td> <td> 'iomb-adapter-slot' </td> <td> '0' </td> </tr>
## <tr> <td> '-wwn' </td> <td> 'wwn' </td> <td> '0' </td> </tr>
## <tr> <td> '-id' </td> <td> 'shelfid' </td> <td> '0' </td> </tr>
## <tr> <td> '-revision' </td> <td> 'revision' </td> <td> '0' </td> </tr>
## <tr> <td> '-ioma-adapter' </td> <td> 'ioma-adapter' </td> <td> '0' </td> </tr>
## <tr> <td> '-policy-id' </td> <td> 'policy-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-type' </td> <td> 'type' </td> <td> '0' </td> </tr>
## <tr> <td> '-product-id' </td> <td> 'product-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-man-id' </td> <td> 'man-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-paths' </td> <td> 'num-paths' </td> <td> '0' </td> </tr>
## <tr> <td> '-dev-rev' </td> <td> 'dev-rev' </td> <td> '0' </td> </tr>
## <tr> <td> '-iom-port' </td> <td> 'iom-port' </td> <td> '0' </td> </tr>
## <tr> <td> '-status' </td> <td> 'status' </td> <td> '0' </td> </tr>
## <tr> <td> '-router-ip' </td> <td> 'router-ip' </td> <td> '0' </td> </tr>
## <tr> <td> '-gateway' </td> <td> 'gateway' </td> <td> '0' </td> </tr>
## <tr> <td> '-pol-version' </td> <td> 'pol-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> <td> '0' </td> </tr>
## <tr> <td> '-link-local-ip' </td> <td> 'link-local-ip' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-disks' </td> <td> 'num-disks' </td> <td> '0' </td> </tr>
## <tr> <td> '-context' </td> <td> 'context' </td> <td> '0' </td> </tr>
## <tr> <td> '-shelf-list' </td> <td> 'shelf-list' </td> <td> '0' </td> </tr>
## <tr> <td> '-fw-version' </td> <td> 'fw-version' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-type' </td> <td> 'alert-type' </td> <td> '0' </td> </tr>
## <tr> <td> '-num-iomb' </td> <td> 'num-iomb' </td> <td> '0' </td> </tr>
## <tr> <td> 'invoke' </td> <td> 'invoke' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = priv set -q test; debug service-processor log trigger-upload -node sivakum1-vsim-4 -type statistics; result -l
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub debug_service_processor_log_trigger_upload {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub debug_service_processor_log_trigger_upload
## @name ::system_node_service_processor_image_update_progress_show
##
## @summary Parses the raw output from the "system node service-processor image update-progress show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_node_service_processor_image_update_progress_show with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_node_service_processor_image_update_progress_show(%args);
## INPUTS TO METHOD CALL 'system_node_service_processor_image_update_progress_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'start-time,' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'instance,' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'fields,' </td> <td> '0' </td> </tr>
## <tr> <td> '-in-progress' </td> <td> 'in-progress' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'percent-done,' </td> <td> '0' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'end-time,' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; system node service-processor image update-progress show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;
##&nbsp;                                     Node: rtp-plat-2240-4-02b
##&nbsp;Latest SP Firmware Update Start Timestamp: Mon May  7 2012 11:25:14
##&nbsp;Latest SP Firmware Update Percentage Done: 100
##&nbsp;  Latest SP Firmware Update End Timestamp: Mon May  7 2012 11:51:40
##&nbsp;                    Is Update in Progress: no
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'latest_sp_firmware_update_percentage_done' => '100',
##&nbsp;            'latest_sp_firmware_update_start_timestamp' => 'Mon May  7 2012 11:25:14',
##&nbsp;            'is_update_in_progress' => 'no',
##&nbsp;            'node' => 'rtp-plat-2240-4-02b',
##&nbsp;            'latest_sp_firmware_update_end_timestamp' => 'Mon May  7 2012 11:51:40'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; system node service-processor image update-progress show -field percent-done,in-progress,start-time,end-time,node; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;node                start-time                 percent-done end-time                   in-progress
##&nbsp;------------------- -------------------------- ------------ -------------------------- -----------
##&nbsp;rtp-plat-2240-4-02b "Mon May  7 2012 11:25:14" 100          "Mon May  7 2012 11:51:40" no
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'in_progress' => 'no',
##&nbsp;            'end_time' => 'Mon May  7 2012 11:51:40',
##&nbsp;            'start_time' => 'Mon May  7 2012 11:25:14',
##&nbsp;            'node' => 'rtp-plat-2240-4-02b',
##&nbsp;            'percent_done' => '100'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
##&nbsp;Command = priv set -q test; system node service-processor image update-progress show -instance; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;
##&nbsp;                                     Node: rtp-plat-2240-4-02b
##&nbsp;Latest SP Firmware Update Start Timestamp: Mon May  7 2012 11:25:14
##&nbsp;Latest SP Firmware Update Percentage Done: 100
##&nbsp;  Latest SP Firmware Update End Timestamp: Mon May  7 2012 11:51:40
##&nbsp;                    Is Update in Progress: no
##&nbsp;
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'latest_sp_firmware_update_percentage_done' => '100',
##&nbsp;            'latest_sp_firmware_update_start_timestamp' => 'Mon May  7 2012 11:25:14',
##&nbsp;            'is_update_in_progress' => 'no',
##&nbsp;            'node' => 'rtp-plat-2240-4-02b',
##&nbsp;            'latest_sp_firmware_update_end_timestamp' => 'Mon May  7 2012 11:51:40'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub system_node_service_processor_image_update_progress_show {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( raw_output => $raw_output );
    my ( @head_arr, @value_arr, @result_array, $k, $count );
    my $i = 0;
    my $m = 0;
    foreach my $line (@lines) {
        if ( $line =~ /^[n]ode.*/ ) {
            @head_arr = split( /\s+/, $line );
            for ( my $j = 0; $j < @head_arr; $j++ ) {
                $head_arr[$j] =~ s/\-/_/g;
            }
            next;
        } elsif ( $line =~ /^-.+/ ) {
            next;
        } elsif ( $line =~ /(.*):\s+(.*)/ ) {
            $result_array[$i]->{ to_key_name( item => $1 ) } = $2;
        } elsif ( $line =~ /^\w+.*/ ) {
            @value_arr = split( //, $line );
            $count = 0;
            foreach my $k (@value_arr) {
                if ( $k =~ /\"/ ) {
                    $count = $count + 1;
                }
                if ( $count % 2 == 1 ) {
                    $k =~ s/\s+/####/;
                }
            }
            my $var1 = join( "", @value_arr );
            @value_arr = split( /\s+/, $var1 );
            for ( my $m = 0; $m <= $#head_arr; $m++ ) {
                $value_arr[$m] =~ s/####/ /g;
                $value_arr[$m] =~ s/\"//g;
                $result_array[$i]->{ to_key_name( item => $head_arr[$m] ) }
                    = $value_arr[$m];
            }
        } ## end elsif ($line =~ /^\w+.*/)
    } ## end foreach my $line (@lines)
    return \@result_array;
} ## end sub system_node_service_processor_image_update_progress_show

## @name ::system_health_alert_show
##
## @summary Parses the raw output from the "system health alert show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_health_alert_show with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_health_alert_show(%args);
## INPUTS TO METHOD CALL 'system_health_alert_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-alert-id' </td> <td> 'alert-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-additional-info' </td> <td> 'additional-info' </td> <td> '0' </td> </tr>
## <tr> <td> '-perceived-severity' </td> <td> 'perceived-severity' </td> <td> '0' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> <td> '0' </td> </tr>
## <tr> <td> '-acknowledge' </td> <td> 'acknowledge' </td> <td> '0' </td> </tr>
## <tr> <td> '-possible-effect' </td> <td> 'possible-effect' </td> <td> '0' </td> </tr>
## <tr> <td> '-suppressor' </td> <td> 'suppressor' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause-description' </td> <td> 'probable-cause-description' </td> <td> '0' </td> </tr>
## <tr> <td> '-indication-time' </td> <td> 'indication-time' </td> <td> '0' </td> </tr>
## <tr> <td> '-policy' </td> <td> 'policy' </td> <td> '0' </td> </tr>
## <tr> <td> '-corrective-actions' </td> <td> 'corrective-actions' </td> <td> '0' </td> </tr>
## <tr> <td> '-probable-cause' </td> <td> 'probable-cause' </td> <td> '0' </td> </tr>
## <tr> <td> '-suppress' </td> <td> 'suppress' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> <td> '0' </td> </tr>
## <tr> <td> '-alerting-resource-name' </td> <td> 'alerting-resource-name' </td> <td> '0' </td> </tr>
## <tr> <td> '-instance' </td> <td> 'instance' </td> <td> '1' </td> </tr>
## <tr> <td> '-subsystem' </td> <td> 'subsystem' </td> <td> '0' </td> </tr>
## <tr> <td> '-tags' </td> <td> 'tags' </td> <td> '0' </td> </tr>
## <tr> <td> '-acknowledger' </td> <td> 'acknowledger' </td> <td> '0' </td> </tr>
## <tr> <td> '-fields' </td> <td> 'fields' </td> <td> '0' </td> </tr>
## <tr> <td> '-alerting-resource' </td> <td> 'alerting-resource' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = priv set -q test; system health alert show; result -l
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;               Node: plat-8040-3b
##&nbsp;           Resource: Shelf ID 1
##&nbsp;           Severity: Major
##&nbsp;     Probable Cause: Disk shelf 1 is connected to controller plat-8040-3b
##&nbsp;                     through IOM A only.
##&nbsp;    Possible Effect: Access to disk shelf 1 will be lost with a single
##&nbsp;                     hardware failure of IOM A, HBA 0d, or any intervening
##&nbsp;                     IOM or SAS cable.
##&nbsp; Corrective Actions: 1. Halt controller plat-8040-3b and all controllers attached to disk shelf 1.
##&nbsp;                     2. Connect disk shelf 1 IOM A and IOM B to controller plat-8040-3b following the rules in the Universal SAS and ACP Cabling Guide.
##&nbsp;                     3. Reboot the halted controllers.
##&nbsp;                     4. Contact support personnel if the alert persists.
##&nbsp;
##&nbsp;               Node: plat-8040-3b
##&nbsp;           Resource: Shelf ID 0
##&nbsp;           Severity: Major
##&nbsp;     Probable Cause: Disk shelf 0 is connected to controller plat-8040-3b
##&nbsp;                     through IOM A only.
##&nbsp;    Possible Effect: Access to disk shelf 0 will be lost with a single
##&nbsp;                     hardware failure of IOM A, HBA 0d, or any intervening
##&nbsp;                     IOM or SAS cable.
##&nbsp; Corrective Actions: 1. Halt controller plat-8040-3b and all controllers attached to disk shelf 0.
##&nbsp;                     2. Connect disk shelf 0 IOM A and IOM B to controller plat-8040-3b following the rules in the Universal SAS and ACP Cabling Guide.
##&nbsp;                     3. Reboot the halted controllers.
##&nbsp;                     4. Contact support personnel if the alert persists.
##&nbsp;
##&nbsp;               Node: plat-8040-3b
##&nbsp;           Resource: Adapter 0a
##&nbsp;           Severity: Minor
##&nbsp;     Probable Cause: All IOMs connected to HBA 0a are not of the same type
##&nbsp;                     (IOM A or IOM B).
##&nbsp;    Possible Effect: Access to disk shelves may be lost in a single
##&nbsp;                     hardware failure scenario (e.g. cable, HBA, or IOM
##&nbsp;                     failure).
##&nbsp; Corrective Actions: 1. Halt controller plat-8040-3b and any controller that is attached to the same disk shelves as plat-8040-3b port 0a.
##&nbsp;                     2. Connect adapter 0a to IOMs of the same type (IOM A or IOM B) throughout the shelf stack following the rules in the Universal SAS anc ACP Cabling Guide.
##&nbsp;                     3. Reboot the halted controllers.
##&nbsp;                     4. Contact support personnel if the alert persists.
##&nbsp;
##&nbsp;               Node: plat-8040-3b
##&nbsp;           Resource: Adapter 0d
##&nbsp;           Severity: Minor
##&nbsp;     Probable Cause: All IOMs connected to HBA 0d are not of the same type
##&nbsp;                     (IOM A or IOM B).
##&nbsp;    Possible Effect: Access to disk shelves may be lost in a single
##&nbsp;                     hardware failure scenario (e.g. cable, HBA, or IOM
##&nbsp;                     failure).
##&nbsp; Corrective Actions: 1. Halt controller plat-8040-3b and any controller that is attached to the same disk shelves as plat-8040-3b port 0d.
##&nbsp;                     2. Connect adapter 0d to IOMs of the same type (IOM A or IOM B) throughout the shelf stack following the rules in the Universal SAS anc ACP Cabling Guide.
##&nbsp;                     3. Reboot the halted controllers.
##&nbsp;                     4. Contact support personnel if the alert persists.
##&nbsp;
##&nbsp;4 entries were displayed.
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'corrective_actions' => [
##&nbsp;                                      '1. Halt controller plat-8040-3b and all controllers attached to disk shelf 1.',
##&nbsp;                                      '2. Connect disk shelf 1 IOM A and IOM B to controller plat-8040-3b following the rules in the Universal SAS and ACP Cabling Guide.',
##&nbsp;                                      '3. Reboot the halted controllers.',
##&nbsp;                                      '4. Contact support personnel if the alert persists.'
##&nbsp;                                    ],
##&nbsp;            'resource' => 'Shelf ID 1',
##&nbsp;            'possible_effect' => 'Access to disk shelf 1 will be lost with a single hardware failure of IOM A, HBA 0d, or any intervening IOM or SAS cable.',
##&nbsp;            'severity' => 'Major',
##&nbsp;            'node' => 'plat-8040-3b',
##&nbsp;            'probable_cause' => 'Disk shelf 1 is connected to controller plat-8040-3b through IOM A only.'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'corrective_actions' => [
##&nbsp;                                      '1. Halt controller plat-8040-3b and all controllers attached to disk shelf 0.',
##&nbsp;                                      '2. Connect disk shelf 0 IOM A and IOM B to controller plat-8040-3b following the rules in the Universal SAS and ACP Cabling Guide.',
##&nbsp;                                      '3. Reboot the halted controllers.',
##&nbsp;                                      '4. Contact support personnel if the alert persists.'
##&nbsp;                                    ],
##&nbsp;            'resource' => 'Shelf ID 0',
##&nbsp;            'possible_effect' => 'Access to disk shelf 0 will be lost with a single hardware failure of IOM A, HBA 0d, or any intervening IOM or SAS cable.',
##&nbsp;            'severity' => 'Major',
##&nbsp;            'node' => 'plat-8040-3b',
##&nbsp;            'probable_cause' => 'Disk shelf 0 is connected to controller plat-8040-3b through IOM A only.'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'corrective_actions' => [
##&nbsp;                                      '1. Halt controller plat-8040-3b and any controller that is attached to the same disk shelves as plat-8040-3b port 0a.',
##&nbsp;                                      '2. Connect adapter 0a to IOMs of the same type (IOM A or IOM B) throughout the shelf stack following the rules in the Universal SAS anc ACP Cabling Guide.',
##&nbsp;                                      '3. Reboot the halted controllers.',
##&nbsp;                                      '4. Contact support personnel if the alert persists.'
##&nbsp;                                    ],
##&nbsp;            'resource' => 'Adapter 0a',
##&nbsp;            'possible_effect' => 'Access to disk shelves may be lost in a single hardware failure scenario (e.g. cable, HBA, or IOM failure).',
##&nbsp;            'severity' => 'Minor',
##&nbsp;            'node' => 'plat-8040-3b',
##&nbsp;            'probable_cause' => 'All IOMs connected to HBA 0a are not of the same type (IOM A or IOM B).'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'corrective_actions' => [
##&nbsp;                                      '1. Halt controller plat-8040-3b and any controller that is attached to the same disk shelves as plat-8040-3b port 0d.',
##&nbsp;                                      '2. Connect adapter 0d to IOMs of the same type (IOM A or IOM B) throughout the shelf stack following the rules in the Universal SAS anc ACP Cabling Guide.',
##&nbsp;                                      '3. Reboot the halted controllers.',
##&nbsp;                                      '4. Contact support personnel if the alert persists.'
##&nbsp;                                    ],
##&nbsp;            'resource' => 'Adapter 0d',
##&nbsp;            'possible_effect' => 'Access to disk shelves may be lost in a single hardware failure scenario (e.g. cable, HBA, or IOM failure).',
##&nbsp;            'severity' => 'Minor',
##&nbsp;            'node' => 'plat-8040-3b',
##&nbsp;            'probable_cause' => 'All IOMs connected to HBA 0d are not of the same type (IOM A or IOM B).'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'total_entries_displayed' => '12'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub system_health_alert_show {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my @lines = get_lines( raw_output => $raw_output );
    my @result_array;
    my $count=-1;
	my $line;

    while ($line = shift @lines) {
        if ($line=~/(Node):\s+(.*)/){
            $count++;
            $result_array[$count]->{to_key_name(item => $1)} = $2;
        } elsif ($line=~/Probable Cause|Possible Effect|Corrective Actions/) {
            my (@correctiveline,%hash);
            my ($key, $value) = split(':\s+', $line);
            push (@correctiveline, $value);
            $line = shift(@lines);
            # Parse the corrective and causing actions
            while($line!~/^\w+.*:\s+\w+.*/ && scalar (@lines) > 0){
                if ($value=~/^\d+/|$line=~/^\d+\./) {
                    push (@correctiveline, $line);
                } else {
                        $correctiveline[0]= join(' ', $correctiveline[0],$line );
                }
                $line = shift(@lines);
            }
            # Since the line is already have next key (Possible Effect) store it back to @lines
            unshift @lines, $line;
            if($key=~/Corrective Actions/) {
                $result_array[$count]->{to_key_name(item => $key)}= \@correctiveline;
			} else{
                $result_array[$count]->{to_key_name(item => $key)}= $correctiveline[0];
            }
        }
        elsif($line=~/Additional Information/){
            my (@result,%hash);
            #Till the line matches "Alerting Resource Name" create hash structre for Additional Information
            while($line!~/Alerting Resource Name/ && scalar (@lines) > 0){
                $line=~s/Additional Information:\s+//;
                if($line=~/^(\w+.*):\s+(.*)$/){
				    my ($key,$value) = ($1, $2);

                    if($value=~/\,/){
                        my @separatefields=split('\,\s+', $value);
                        $hash{to_key_name(item => $key)} = \@separatefields;
                        last;
                    }
                    $hash{to_key_name(item => $key)} = $value;
                } elsif($line=~/^(\w+.*):$/){
                    $hash{to_key_name(item => $1)} = '';
                }
                $line = shift @lines;
            }
            unshift @lines, $line;
            push (@{$result_array[$count]->{additional_information}}, \%hash);
        }
        elsif ($line=~/^(\w+.*):\s+(.*)/) {
            $result_array[$count]->{to_key_name(item => $1)} = $2;
        } elsif ($line=~/(\d+)\s+(entries were displayed)/){
            $result_array[$count+1]->{to_key_name(item => $2)} = $1;
        }
    }
    return \@result_array;
}

## @name ::system_health_alert_modify
##
## @summary Parses the raw output from the "system health alert modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_health_alert_modify with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_health_alert_modify(%args);
## INPUTS TO METHOD CALL 'system_health_alert_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-suppress' </td> <td> 'suppress' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-id' </td> <td> 'alert-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> <td> '0' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> <td> '0' </td> </tr>
## <tr> <td> '-acknowledger' </td> <td> 'acknowledger' </td> <td> '0' </td> </tr>
## <tr> <td> '-acknowledge' </td> <td> 'acknowledge' </td> <td> '0' </td> </tr>
## <tr> <td> '-suppressor' </td> <td> 'suppressor' </td> <td> '0' </td> </tr>
## <tr> <td> '-alerting-resource' </td> <td> 'alerting-resource' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub system_health_alert_modify {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::system_health_alert_delete
##
## @summary Parses the raw output from the "system health alert delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_health_alert_delete with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_health_alert_delete(%args);
## INPUTS TO METHOD CALL 'system_health_alert_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-alert-id' </td> <td> 'alert-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> <td> '0' </td> </tr>
## <tr> <td> '-alerting-resource' </td> <td> 'alerting-resource' </td> <td> '0' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub system_health_alert_delete {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::system_health_autosupport_trigger_history_create
##
## @summary Parses the raw output from the "system health autosupport trigger history create" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_health_autosupport_trigger_history_create with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_health_autosupport_trigger_history_create(%args);
## INPUTS TO METHOD CALL 'system_health_autosupport_trigger_history_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-autosupport-triggered' </td> <td> 'autosupport-triggered' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-id' </td> <td> 'alert-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> <td> '0' </td> </tr>
## <tr> <td> '-alerting-resource' </td> <td> 'alerting-resource' </td> <td> '0' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub system_health_autosupport_trigger_history_create {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::system_health_autosupport_trigger_history_modify
##
## @summary Parses the raw output from the "system health autosupport trigger history modify" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_health_autosupport_trigger_history_modify with any of its options as described in the
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_health_autosupport_trigger_history_modify(%args);
## INPUTS TO METHOD CALL 'system_health_autosupport_trigger_history_modify' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-autosupport-triggered' </td> <td> 'autosupport-triggered' </td> <td> '0' </td> </tr>
## <tr> <td> '-alert-id' </td> <td> 'alert-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> <td> '0' </td> </tr>
## <tr> <td> '-alerting-resource' </td> <td> 'alerting-resource' </td> <td> '0' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub system_health_autosupport_trigger_history_modify {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}

## @name ::system_health_autosupport_trigger_history_delete
##
## @summary Parses the raw output from the "system health autosupport trigger history delete" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from system_health_autosupport_trigger_history_delete with any of its options as described in the 
##    corresponding cdef file 'system.cdefs'
## @synopsis
##      system_health_autosupport_trigger_history_delete(%args);
## INPUTS TO METHOD CALL 'system_health_autosupport_trigger_history_delete' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-alert-id' </td> <td> 'alert-id' </td> <td> '0' </td> </tr>
## <tr> <td> '-node' </td> <td> 'nodename' </td> <td> '0' </td> </tr>
## <tr> <td> '-alerting-resource' </td> <td> 'alerting-resource' </td> <td> '0' </td> </tr>
## <tr> <td> '-monitor' </td> <td> 'monitor' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub system_health_autosupport_trigger_history_delete {
    my %args = @_;
    my $error_message;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_processed_output();
    my $error      = _check_error(
        "raw_output"        => $raw_output,
        "ptr_error_message" => \$error_message,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
}
1;
