# Copyright (c) 2013-2015 NetApp Inc.
# All rights reserved.
#
## @summary Provide a common set of libraries to work on controllers
## @author dl-san-automation
## @status private
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller is used to Config, Setup, Cleanup and
Deconfig a controller.

=cut

package NACL::AppBuilder::EnGenX::Controller;

# Compiler directives.
use strict;

# NATE and Standard Module import
use Tharn;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use NATE::Util::DataTypes qw(TRUE FALSE);
use Tie::IxHash;
use Time::HiRes;

# Exceptions Imported
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use Params::Validate
    qw(OBJECT SCALAR SCALARREF ARRAYREF HASHREF UNDEF GLOBREF);


# NACL libraries in use
use NACL::C::Node;
use NACL::STask::Cluster;
use NACL::CS::NetworkInterface;

# Load resource specific modules
use NACL::AppBuilder::EnGenX::Controller::Cluster;
use NACL::AppBuilder::EnGenX::Controller::Aggregate;
use NACL::AppBuilder::EnGenX::Controller::Aggregate::ObjectStore;
use NACL::AppBuilder::EnGenX::Controller::Vserver;
use NACL::AppBuilder::EnGenX::Controller::Lif;
use NACL::AppBuilder::EnGenX::Controller::NetworkRoute;
use NACL::AppBuilder::EnGenX::Controller::JobScheduleCron;
use NACL::AppBuilder::EnGenX::Controller::Igroup;
use NACL::AppBuilder::EnGenX::Controller::Initiator;
use NACL::AppBuilder::EnGenX::Controller::Portset;
use NACL::AppBuilder::EnGenX::Controller::Volume;
use NACL::AppBuilder::EnGenX::Controller::Volume::Clone;
use NACL::AppBuilder::EnGenX::Controller::ConstituentVolume;
use NACL::AppBuilder::EnGenX::Controller::Qtree;
use NACL::AppBuilder::EnGenX::Controller::File;
use NACL::AppBuilder::EnGenX::Controller::Lun;
use NACL::AppBuilder::EnGenX::Controller::LunMap;
use NACL::AppBuilder::EnGenX::Controller::LunBind;
use NACL::AppBuilder::EnGenX::Controller::LunAttribute;
use NACL::AppBuilder::EnGenX::Controller::Volume_Efficiency;
use NACL::AppBuilder::EnGenX::Controller::Iscsi;
use NACL::AppBuilder::EnGenX::Controller::IscsiSecurity;
use NACL::AppBuilder::EnGenX::Controller::Isns;
use NACL::AppBuilder::EnGenX::Controller::FCP;
use NACL::AppBuilder::EnGenX::Controller::VolumeEfficiencyPolicy;
use NACL::AppBuilder::EnGenX::Controller::Vlan;
use NACL::AppBuilder::EnGenX::Controller::CIFS::VserverCifsDomainPreferredDc;
use NACL::AppBuilder::EnGenX::Controller::CIFS::DomainPasswordSchedule;
use NACL::AppBuilder::EnGenX::Controller::Ifgrp;
use NACL::AppBuilder::EnGenX::Controller::NIS;
use NACL::AppBuilder::EnGenX::Controller::DNS;
use NACL::AppBuilder::EnGenX::Controller::Dynamic_DNS;
use NACL::AppBuilder::EnGenX::Controller::NFS;
use NACL::AppBuilder::EnGenX::Controller::CIFS;
use NACL::AppBuilder::EnGenX::Controller::CIFS::Branchcache;
use NACL::AppBuilder::EnGenX::Controller::CIFS::Local_User;
use NACL::AppBuilder::EnGenX::Controller::CIFS::Privilege;
use NACL::AppBuilder::EnGenX::Controller::CIFS::Local_Group;
use NACL::AppBuilder::EnGenX::Controller::CIFS::LocalGroup::Membership;
use NACL::AppBuilder::EnGenX::Controller::CIFS::Security;
use NACL::AppBuilder::EnGenX::Controller::CIFS::Symlink;
use NACL::AppBuilder::EnGenX::Controller::CIFSShare;
use NACL::AppBuilder::EnGenX::Controller::CIFS::HomeDirSearchPath;
use NACL::AppBuilder::EnGenX::Controller::CIFS::CIFS_Options;
use NACL::AppBuilder::EnGenX::Controller::CIFS::GroupPolicy;
use NACL::AppBuilder::EnGenX::Controller::SecurityLogin;
use NACL::AppBuilder::EnGenX::Controller::ExportPolicy;
use NACL::AppBuilder::EnGenX::Controller::ExportPolicyRule;
use NACL::AppBuilder::EnGenX::Controller::QosPolicyGroup;
use NACL::AppBuilder::EnGenX::Controller::Audit;
use NACL::AppBuilder::EnGenX::Controller::ClusterPeer;
use NACL::AppBuilder::EnGenX::Controller::Name_Mapping;
use NACL::AppBuilder::EnGenX::Controller::VserverNfsKerberosRealm;
use NACL::AppBuilder::EnGenX::Controller::VserverNfsKerberosInterface;
use NACL::AppBuilder::EnGenX::Controller::FailoverGroup;
use NACL::AppBuilder::EnGenX::Controller::Volume::Mount;
use NACL::AppBuilder::EnGenX::Controller::IPspace;
use NACL::AppBuilder::EnGenX::Controller::IPspace::BroadcastDomain::Port;
use NACL::AppBuilder::EnGenX::Controller::IPspace::BroadcastDomain;
use NACL::AppBuilder::EnGenX::Controller::QuotaPolicy;
use NACL::AppBuilder::EnGenX::Controller::QuotaRule;
use NACL::AppBuilder::EnGenX::Controller::VolumeQuota;
use NACL::AppBuilder::EnGenX::Controller::Aggregate::WaflIron;
use NACL::AppBuilder::EnGenX::Controller::Vserver::Security_Trace_Filter;
use NACL::AppBuilder::EnGenX::Controller::Fpolicy::Fpolicy;
use NACL::AppBuilder::EnGenX::Controller::Fpolicy::FpolicyEvent;
use NACL::AppBuilder::EnGenX::Controller::Fpolicy::ExternalEngine;
use NACL::AppBuilder::EnGenX::Controller::Fpolicy::FpolicyScope;
use NACL::AppBuilder::EnGenX::Controller::Fpolicy::FpolicyEnable;
use NACL::AppBuilder::EnGenX::Controller::Fpolicy::ExternalEngineConnection;
use NACL::AppBuilder::EnGenX::Controller::Fsecurity::FileDirectoryPolicy;
use NACL::AppBuilder::EnGenX::Controller::Fsecurity::FileDirectoryNTFS;
use NACL::AppBuilder::EnGenX::Controller::Fsecurity::PolicyTask;
use NACL::AppBuilder::EnGenX::Controller::Fsecurity::Sacl;
use NACL::AppBuilder::EnGenX::Controller::Fsecurity::Dacl;
use NACL::AppBuilder::EnGenX::Controller::Fsecurity::FileDirectoryApply;
use NACL::AppBuilder::EnGenX::Controller::SnapshotAutodelete;
use NACL::AppBuilder::EnGenX::Controller::Vscan::ScannerPool;
use NACL::AppBuilder::EnGenX::Controller::Vscan::OnAccessPolicy;
use NACL::AppBuilder::EnGenX::Controller::Vscan::OnDemandTask;
use NACL::AppBuilder::EnGenX::Controller::Vscan;
use NACL::AppBuilder::EnGenX::Controller::CIFS::ShareAccessControl;
use NACL::AppBuilder::EnGenX::Controller::VserverPeer;
use NACL::AppBuilder::EnGenX::Controller::Snapmirror;
use NACL::AppBuilder::EnGenX::Controller::SnapmirrorInitialize;
use NACL::AppBuilder::EnGenX::Controller::SnapmirrorPolicy;
use NACL::AppBuilder::EnGenX::Controller::SnapshotPolicy;
use NACL::AppBuilder::EnGenX::Controller::SnapmirrorPolicyRule;

use parent qw(NACL::AppBuilder::EnGenX::Util);

###############################################################################
#                               Public Methods
###############################################################################

#==============================================================================
# Method:     _new
# Objective:  Create new resource object.
# Details:    Steps: 1)   TBD
# Inputs:     TBD
# Outputs:    TBD
# Exceptions: Argument
#             BaseException
#==============================================================================
sub _new {
    my ($proto, %args) = @_;
    $Log->enter() if $may_enter;

    my $self = {};

    my $class = ref($proto) || $proto;
    bless($self, $class);

    &ParamsMgr::process_args(%args);
    my $parent = &ParamsMgr::get_arg(
        -arg      => "parent",
        -required => 1,
        -default  => ""
    );
    if (&ParamsMgr::error()) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "Missing required argument parent: " . &ParamsMgr::get_error());
    }

    # Save parent
    $self->{parent} = $parent;

    $Log->exit() if $may_exit;
    return $self;
}    # end sub new()

=head1 METHODS

=head2 setup

Launch subtest for each controller to perform setup

  $self->{sm_controller_object}->setup(
                 run_id_prefix => $self->{run_id_prefix},
                 resource_list => \%{$self->{resource_list}});

=over

=item Options

TBD

=over

=item C<< TBD >>

TBD

=back

=item Return

TBD

=item Exceptions

BaseException

=cut

sub setup {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    # Locals
    # Create a dispatch with the supported types.
    # To preserve the order of the types (keys) need to tie it.
    my %pre_snapmirror_dispatch;
    tie( %pre_snapmirror_dispatch, 'Tie::IxHash');
    my %snapmirror_dispatch;
    my %vserver_peer_dispatch;
    tie( %vserver_peer_dispatch, 'Tie::IxHash');
    tie( %snapmirror_dispatch, 'Tie::IxHash');
    my %post_snapmirror_dispatch;
    tie( %post_snapmirror_dispatch, 'Tie::IxHash');
    my %snapmirror_initialize_dispatch;
    tie( %snapmirror_initialize_dispatch, 'Tie::IxHash');
    # Note: The order of the types (keys) in the hash will be the same at the
    #       time of the execution, so, in this case the first type to be
    #       executed will be Ifgrp, then Initiator and so on. It is important
    #       to preserve the order.

    %pre_snapmirror_dispatch=(
        'Cluster'                       => sub {NACL::AppBuilder::EnGenX::Controller::Cluster->create(@_)},
        'Job_Schedule_Cron'             => sub {NACL::AppBuilder::EnGenX::Controller::JobScheduleCron->create(@_)},
        'IPspace'                       => sub {NACL::AppBuilder::EnGenX::Controller::IPspace->create(@_)},
        'Cluster_Peer'                  => sub {NACL::AppBuilder::EnGenX::Controller::ClusterPeer->create(@_)},
        'Object_Store'                  => sub {NACL::AppBuilder::EnGenX::Controller::Aggregate::ObjectStore->create(@_)},
        'Aggregate'                     => sub {NACL::AppBuilder::EnGenX::Controller::Aggregate->create(@_)},
        'Vserver'                       => sub {NACL::AppBuilder::EnGenX::Controller::Vserver->create(@_)},
        'Snapmirror_Policy'             => sub {NACL::AppBuilder::EnGenX::Controller::SnapmirrorPolicy->create(@_)},
        'Snapmirror_Policy_Rule'        => sub {NACL::AppBuilder::EnGenX::Controller::SnapmirrorPolicyRule->create(@_)},
        'Snapshot_Policy'               => sub {NACL::AppBuilder::EnGenX::Controller::SnapshotPolicy->create(@_)},
        'Failover_Group'                => sub {NACL::AppBuilder::EnGenX::Controller::FailoverGroup->create(@_)},
        'Qos_Policy_Group'              => sub {NACL::AppBuilder::EnGenX::Controller::QosPolicyGroup->create(@_)},
        'Ifgrp'                         => sub {NACL::AppBuilder::EnGenX::Controller::Ifgrp->create(@_)},
        'Vlan'                          => sub {NACL::AppBuilder::EnGenX::Controller::Vlan->create(@_)},
        'Broadcast_Domain'              => sub {NACL::AppBuilder::EnGenX::Controller::IPspace::BroadcastDomain->create(@_)},
        'BD_Port'                       => sub {NACL::AppBuilder::EnGenX::Controller::IPspace::BroadcastDomain::Port->create(@_)},
        'Lif'                           => sub {NACL::AppBuilder::EnGenX::Controller::Lif->create(@_)},
        'Network_Route'                 => sub {NACL::AppBuilder::EnGenX::Controller::NetworkRoute->create(@_)},
        'NIS_Service'                   => sub {NACL::AppBuilder::EnGenX::Controller::NIS->create(@_)},
        'DNS_Service'                   => sub {NACL::AppBuilder::EnGenX::Controller::DNS->create(@_)},
        'FCP'                           => sub {NACL::AppBuilder::EnGenX::Controller::FCP->create(@_)},
        'iSCSI'                         => sub {NACL::AppBuilder::EnGenX::Controller::Iscsi->create(@_)},
        'iSCSI_Security'                => sub {NACL::AppBuilder::EnGenX::Controller::IscsiSecurity->create(@_)},
        'iSNS'                          => sub {NACL::AppBuilder::EnGenX::Controller::Isns->create(@_)},
        'NFS'                           => sub {NACL::AppBuilder::EnGenX::Controller::NFS->create(@_)},
        'CIFS'                          => sub {NACL::AppBuilder::EnGenX::Controller::CIFS->create(@_)},
        'CIFS_Local_User'               => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::Local_User->create(@_)},
        'CIFS_Local_Group'              => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::Local_Group->create(@_)},
        'CIFS_Privilege'                => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::Privilege->create(@_)},
        'Local_Group_Membership'        => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::LocalGroup::Membership->create(@_)},
        'CIFS_Group_Policy'             => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::GroupPolicy->create(@_)},
        'CIFS_Domain_Password_Schedule' => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::DomainPasswordSchedule->create(@_)},
        'Security_Login'                => sub {NACL::AppBuilder::EnGenX::Controller::SecurityLogin->create(@_)},
        'Export_Policy'                 => sub {NACL::AppBuilder::EnGenX::Controller::ExportPolicy->create(@_)},
        'Export_Policy_Rule'            => sub {NACL::AppBuilder::EnGenX::Controller::ExportPolicyRule->create(@_)},
        'DNS_Dynamic_Update'            => sub {NACL::AppBuilder::EnGenX::Controller::Dynamic_DNS->create(@_)},
        'Name_Mapping'                  => sub {NACL::AppBuilder::EnGenX::Controller::Name_Mapping->create(@_)},
        'Nfs_Kerberos_Realm'            => sub {NACL::AppBuilder::EnGenX::Controller::VserverNfsKerberosRealm->create(@_)},
        'Nfs_Kerberos_Interface'        => sub {NACL::AppBuilder::EnGenX::Controller::VserverNfsKerberosInterface->create(@_)},
        'Igroup'                        => sub {NACL::AppBuilder::EnGenX::Controller::Igroup->create(@_)},
        'Initiator'                     => sub {NACL::AppBuilder::EnGenX::Controller::Initiator->create(@_)},
        'Portset'                       => sub {NACL::AppBuilder::EnGenX::Controller::Portset->create(@_)},
        'Volume_Efficiency_Policy'      => sub {NACL::AppBuilder::EnGenX::Controller::VolumeEfficiencyPolicy->create(@_)},
        'Volume'                        => sub {NACL::AppBuilder::EnGenX::Controller::Volume->create(@_)},
        'Volume_Efficiency'             => sub {NACL::AppBuilder::EnGenX::Controller::Volume_Efficiency->create(@_)},
        'Volume_Clone'                  => sub {NACL::AppBuilder::EnGenX::Controller::Volume::Clone->create(@_)},
        'Snapshot_Autodelete_options'   => sub {NACL::AppBuilder::EnGenX::Controller::SnapshotAutodelete->create(@_)},
        'Qtree'                         => sub {NACL::AppBuilder::EnGenX::Controller::Qtree->create(@_)},
        'Volume_Mount'                  => sub {NACL::AppBuilder::EnGenX::Controller::Volume::Mount->create(@_)},
        'Constituent_Volume'            => sub {NACL::AppBuilder::EnGenX::Controller::ConstituentVolume->create(@_)},
        'Quota_Policy'                  => sub {NACL::AppBuilder::EnGenX::Controller::QuotaPolicy->create(@_)},
        'Quota_Rule'                    => sub {NACL::AppBuilder::EnGenX::Controller::QuotaRule->create(@_)},
        'Volume_Quota'                  => sub {NACL::AppBuilder::EnGenX::Controller::VolumeQuota->create(@_)},
        'File'                          => sub {NACL::AppBuilder::EnGenX::Controller::File->create(@_)},
        'Lun'                           => sub {NACL::AppBuilder::EnGenX::Controller::Lun->create(@_)},
        'Lun_Bind'                      => sub {NACL::AppBuilder::EnGenX::Controller::LunBind->create(@_)},
        'Lun_Map'                       => sub {NACL::AppBuilder::EnGenX::Controller::LunMap->create(@_)},
        'Lun_Attribute'                 => sub {NACL::AppBuilder::EnGenX::Controller::LunAttribute->create(@_)},
        'CIFS_Share'                    => sub {NACL::AppBuilder::EnGenX::Controller::CIFSShare->create(@_)},
        'Share_Access_Control'          => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::ShareAccessControl->create(@_)},
        'CIFS_Symlink'                  => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::Symlink->create(@_)},
        'CIFS_Branchcache'              => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::Branchcache->create(@_)},
        'CIFS_Security'                 => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::Security->create(@_)},
        'CIFS_vserver_options'          => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::CIFS_Options->create(@_)},
        'CIFS_Domain_preferred_dc'      => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::VserverCifsDomainPreferredDc->create(@_)},
        'HomeDir_Search_Path'           => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::HomeDirSearchPath->create(@_)},
        'Audit'                         => sub {NACL::AppBuilder::EnGenX::Controller::Audit->create(@_)},
        'Security_Trace_Filter'         => sub {NACL::AppBuilder::EnGenX::Controller::Vserver::Security_Trace_Filter->create(@_)},
        'External_Engine'               => sub {NACL::AppBuilder::EnGenX::Controller::Fpolicy::ExternalEngine->create(@_)},
        'Fpolicy_Event'                 => sub {NACL::AppBuilder::EnGenX::Controller::Fpolicy::FpolicyEvent->create(@_)},
        'Fpolicy'                       => sub {NACL::AppBuilder::EnGenX::Controller::Fpolicy::Fpolicy->create(@_)},
        'Fpolicy_Scope'                 => sub {NACL::AppBuilder::EnGenX::Controller::Fpolicy::FpolicyScope->create(@_)},
        'Fpolicy_Enable'                => sub {NACL::AppBuilder::EnGenX::Controller::Fpolicy::FpolicyEnable->create(@_)},
        'External_Engine_Connection'    => sub {NACL::AppBuilder::EnGenX::Controller::Fpolicy::ExternalEngineConnection->create(@_)},
        'File_Directory_Policy'         => sub {NACL::AppBuilder::EnGenX::Controller::Fsecurity::FileDirectoryPolicy->create(@_)},
        'File_Directory_Ntfs'           => sub {NACL::AppBuilder::EnGenX::Controller::Fsecurity::FileDirectoryNTFS->create(@_)},
        'Policy_Task'                   => sub {NACL::AppBuilder::EnGenX::Controller::Fsecurity::PolicyTask->create(@_)},
        'Ntfs_Sacl_Setting'             => sub {NACL::AppBuilder::EnGenX::Controller::Fsecurity::Sacl->create(@_)},
        'Ntfs_Dacl_Setting'             => sub {NACL::AppBuilder::EnGenX::Controller::Fsecurity::Dacl->create(@_)},
        'File_Directory_Policy_Apply'   => sub {NACL::AppBuilder::EnGenX::Controller::Fsecurity::FileDirectoryApply->create(@_)},
        'Scanner_Pool'                  => sub {NACL::AppBuilder::EnGenX::Controller::Vscan::ScannerPool->create(@_)},
        'On_Access_Policy'              => sub {NACL::AppBuilder::EnGenX::Controller::Vscan::OnAccessPolicy->create(@_)},
        'Vscan'                         => sub {NACL::AppBuilder::EnGenX::Controller::Vscan->create(@_)},
        'On_Demand_Task'                => sub {NACL::AppBuilder::EnGenX::Controller::Vscan::OnDemandTask->create(@_)},
    );

    %vserver_peer_dispatch=(
         'Vserver_Peer'                  => sub {NACL::AppBuilder::EnGenX::Controller::VserverPeer->create(@_)},
    );

    %snapmirror_dispatch=(
        'Snapmirror'                    => sub {NACL::AppBuilder::EnGenX::Controller::Snapmirror->create(@_)},
        'Lif'                           => sub {NACL::AppBuilder::EnGenX::Controller::Lif->create(@_)},
        'DNS_Service'                   => sub {NACL::AppBuilder::EnGenX::Controller::DNS->create(@_)},
        'CIFS'                          => sub {NACL::AppBuilder::EnGenX::Controller::CIFS->create(@_)},
    );

    %snapmirror_initialize_dispatch=(
        'Snapmirror_Initialize'         => sub {NACL::AppBuilder::EnGenX::Controller::SnapmirrorInitialize->create(@_)},
    );

    $self->_controller_execution_helper(
                    dispatch_table=> \%pre_snapmirror_dispatch,
                    action => "setup",
                    %args
        );

    $self->_controller_execution_helper(
                    dispatch_table=> \%vserver_peer_dispatch,
                    action => "setup_snapmirror",
                    %args
        );

    $self->_controller_execution_helper(
                    dispatch_table=> \%snapmirror_dispatch,
                    action => "setup_snapmirror",
                    %args
        );

    # This is needed because of how cascade snapmirror initialization
    # requires an order for initialization that spans multiple clusters.
    # Example SM1 is C1->C2; SM2 is C2-C3, SM1 must be initialized before SM2.
    $self->_setup_snapmirror_initialize_execution_helper(
                    dispatch_table=> \%snapmirror_initialize_dispatch,
                    %args
        );

    $self->_controller_execution_helper(
                    dispatch_table=> \%pre_snapmirror_dispatch,
                    action => "setup_after_snapmirror",
                    %args
        );
    $Log->comment("Cluster Setup completed");
    $Log->exit() if $may_exit;
    return TRUE;

} ## end sub setup

=head2 cleanup

Launch subtest for each controller to perform cleanup

  $self->{sm_controller_object}
        ->cleanup(run_id_prefix => $self->{run_id_prefix},
                 resource_list => \%{$self->{resource_list}});

=over

=item Options

TBD

=over

=item C<< TBD >>

TBD

=back

=item Return

TBD

=item Exceptions

BaseException

=cut

sub cleanup {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;
    # Locals
    # Create a dispatch table with the supported types.
    # To preserve the order of the types (keys) need to tie it.
    my %pre_lun_snapmirror_dispatch;
    tie( %pre_lun_snapmirror_dispatch, 'Tie::IxHash');
    my %lun_snapmirror_dispatch;
    tie( %lun_snapmirror_dispatch, 'Tie::IxHash');
    my %pre_volume_snapmirror_dispatch;
    tie( %pre_volume_snapmirror_dispatch, 'Tie::IxHash');
    my %volume_snapmirror_dispatch;
    tie( %volume_snapmirror_dispatch, 'Tie::IxHash');
    my %pre_vserver_snapmirror_dispatch;
    tie( %pre_vserver_snapmirror_dispatch, 'Tie::IxHash');
    my %vserver_snapmirror_dispatch;
    tie( %vserver_snapmirror_dispatch, 'Tie::IxHash');
    my %vserver_peer_dispatch;
    tie( %vserver_peer_dispatch, 'Tie::IxHash');
    my %post_vserver_snapmirror_dispatch;
    tie( %post_vserver_snapmirror_dispatch, 'Tie::IxHash');
    # Note: The order of the types (keys) in the hash will be the same at the
    #       time of the execution, so, luns, then volumes and so on. It is important
    #       to preserve the order.
    %pre_lun_snapmirror_dispatch=(
        'Aggregate_WaflIron'         => sub {NACL::AppBuilder::EnGenX::Controller::Aggregate::WaflIron->delete(@_)},
        'Volume_Quota'               => sub {NACL::AppBuilder::EnGenX::Controller::VolumeQuota->delete(@_)},
        'Quota_Rule'                 => sub {NACL::AppBuilder::EnGenX::Controller::QuotaRule->delete(@_)},
        'Quota_Policy'               => sub {NACL::AppBuilder::EnGenX::Controller::QuotaPolicy->delete(@_)},
        'Lun_Bind'                   => sub {NACL::AppBuilder::EnGenX::Controller::LunBind->delete(@_)},
    );

    %lun_snapmirror_dispatch=(
        'Lun_Snapmirror'          => sub {NACL::AppBuilder::EnGenX::Controller::Snapmirror->delete(@_)},
    );

    %pre_volume_snapmirror_dispatch=(
        'Lun'                        => sub {NACL::AppBuilder::EnGenX::Controller::Lun->delete(@_)},
        'File'                       => sub {NACL::AppBuilder::EnGenX::Controller::File->delete(@_)},
        'Audit'                      => sub {NACL::AppBuilder::EnGenX::Controller::Audit->delete(@_)},
        'HomeDir_Search_Path'        => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::HomeDirSearchPath->delete(@_)},
        'CIFS_Branchcache'           => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::Branchcache->delete(@_)},
        'CIFS_Symlink'               => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::Symlink->delete(@_)},
        'Share_Access_Control'       => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::ShareAccessControl->delete(@_)},
        'CIFS_Share'                 => sub {NACL::AppBuilder::EnGenX::Controller::CIFSShare->delete(@_)},
        'CIFS_Domain_preferred_dc'   => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::VserverCifsDomainPreferredDc->delete(@_)},
        'Volume_Mount'               => sub{NACL::AppBuilder::EnGenX::Controller::Volume::Mount->delete(@_)},
        'Qtree'                      => sub {NACL::AppBuilder::EnGenX::Controller::Qtree->delete(@_)},
    );

    %volume_snapmirror_dispatch=(
        'Volume_Snapmirror'          => sub {NACL::AppBuilder::EnGenX::Controller::Snapmirror->delete(@_)},
    );

    %pre_vserver_snapmirror_dispatch=(
        'Volume_Clone'               => sub {NACL::AppBuilder::EnGenX::Controller::Volume::Clone->delete(@_)},
        'Volume'                     => sub {NACL::AppBuilder::EnGenX::Controller::Volume->delete(@_)},
        'Volume_Efficiency_Policy'   => sub {NACL::AppBuilder::EnGenX::Controller::VolumeEfficiencyPolicy->delete(@_)},
        'Export_Policy_Rule'         => sub {NACL::AppBuilder::EnGenX::Controller::ExportPolicyRule->delete(@_)},
        'Export_Policy'              => sub {NACL::AppBuilder::EnGenX::Controller::ExportPolicy->delete(@_)},
        'Portset'                    => sub {NACL::AppBuilder::EnGenX::Controller::Portset->delete(@_)},
        'Igroup'                     => sub {NACL::AppBuilder::EnGenX::Controller::Igroup->delete(@_)},
        'Security_Trace_Filter'      => sub {NACL::AppBuilder::EnGenX::Controller::Vserver::Security_Trace_Filter->delete(@_)},
        'External_Engine_Connection' => sub {NACL::AppBuilder::EnGenX::Controller::Fpolicy::ExternalEngineConnection->delete(@_)},
        'Fpolicy_Enable'             => sub {NACL::AppBuilder::EnGenX::Controller::Fpolicy::FpolicyEnable->delete(@_)},
        'Fpolicy_Scope'              => sub {NACL::AppBuilder::EnGenX::Controller::Fpolicy::FpolicyScope->delete(@_)},
        'Fpolicy'                    => sub {NACL::AppBuilder::EnGenX::Controller::Fpolicy::Fpolicy->delete(@_)},
        'Fpolicy_Event'               => sub {NACL::AppBuilder::EnGenX::Controller::Fpolicy::FpolicyEvent->delete(@_)},
        'External_Engine'             => sub {NACL::AppBuilder::EnGenX::Controller::Fpolicy::ExternalEngine->delete(@_)},
        'Ntfs_Dacl_Setting'          => sub {NACL::AppBuilder::EnGenX::Controller::Fsecurity::Dacl->delete(@_)},
        'Ntfs_Sacl_Setting'          => sub {NACL::AppBuilder::EnGenX::Controller::Fsecurity::Sacl->delete(@_)},
        'Policy_Task'                => sub {NACL::AppBuilder::EnGenX::Controller::Fsecurity::PolicyTask->delete(@_)},
        'File_Directory_Ntfs'        => sub {NACL::AppBuilder::EnGenX::Controller::Fsecurity::FileDirectoryNTFS->delete(@_)},
        'File_Directory_Policy'      => sub {NACL::AppBuilder::EnGenX::Controller::Fsecurity::FileDirectoryPolicy->delete(@_)},
        'On_Demand_Task'             => sub {NACL::AppBuilder::EnGenX::Controller::Vscan::OnDemandTask->delete(@_)},
        'Vscan'                      => sub {NACL::AppBuilder::EnGenX::Controller::Vscan->delete(@_)},
        'On_Access_Policy'           => sub {NACL::AppBuilder::EnGenX::Controller::Vscan::OnAccessPolicy->delete(@_)},
        'Scanner_Pool'               => sub {NACL::AppBuilder::EnGenX::Controller::Vscan::ScannerPool->delete(@_)},
        'Nfs_Kerberos_Interface'     => sub {NACL::AppBuilder::EnGenX::Controller::VserverNfsKerberosInterface->delete(@_)},
        'On_Access_Policy'           => sub {NACL::AppBuilder::EnGenX::Controller::Vscan::OnAccessPolicy->delete(@_)},
        'Nfs_Kerberos_Realm'         => sub {NACL::AppBuilder::EnGenX::Controller::VserverNfsKerberosRealm->delete(@_)},
        'iscsi_security'             => sub {NACL::AppBuilder::EnGenX::Controller::IscsiSecurity->delete(@_)},
        'ISNS'                       => sub {NACL::AppBuilder::EnGenX::Controller::Isns->delete(@_)},
        'Security_Login'             => sub {NACL::AppBuilder::EnGenX::Controller::SecurityLogin->delete(@_)},
        'CIFS_Local_Group'           => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::Local_Group->delete(@_)},
        'CIFS_Local_User'            => sub {NACL::AppBuilder::EnGenX::Controller::CIFS::Local_User->delete(@_)},
        'CIFS'                       => sub {NACL::AppBuilder::EnGenX::Controller::CIFS->delete(@_)},
        'NIS_Service'                => sub {NACL::AppBuilder::EnGenX::Controller::NIS->delete(@_)},
        'Lif'                        => sub {NACL::AppBuilder::EnGenX::Controller::Lif->delete(@_)},
        'Name_Mapping'               => sub {NACL::AppBuilder::EnGenX::Controller::Name_Mapping->delete(@_)},
        'Qos_Policy_Group'           => sub {NACL::AppBuilder::EnGenX::Controller::QosPolicyGroup->delete(@_)},
        'Failover_Group'             => sub {NACL::AppBuilder::EnGenX::Controller::FailoverGroup->delete(@_)},
        'Snapshot_Policy'            => sub {NACL::AppBuilder::EnGenX::Controller::SnapshotPolicy->delete(@_)},
        'Snapmirror_Policy_Rule'     => sub {NACL::AppBuilder::EnGenX::Controller::SnapmirrorPolicyRule->delete(@_)},
        'Snapmirror_Policy'          => sub {NACL::AppBuilder::EnGenX::Controller::SnapmirrorPolicy->delete(@_)},
    );

    %vserver_snapmirror_dispatch=(
        'Vserver_Snapmirror'         => sub {NACL::AppBuilder::EnGenX::Controller::Snapmirror->delete(@_)},
    );

    %vserver_peer_dispatch=(
        'Vserver_Peer'               => sub {NACL::AppBuilder::EnGenX::Controller::VserverPeer->delete(@_)},
    );

    %post_vserver_snapmirror_dispatch=(
        'Vserver'                    => sub {NACL::AppBuilder::EnGenX::Controller::Vserver->delete(@_)},
        'Aggregate'                  => sub {NACL::AppBuilder::EnGenX::Controller::Aggregate->delete(@_)},
        'Object_Store'               => sub {NACL::AppBuilder::EnGenX::Controller::Aggregate::ObjectStore->delete(@_)},
        'Cluster_Peer'               => sub {NACL::AppBuilder::EnGenX::Controller::ClusterPeer->delete(@_)},
        'Broadcast_Domain'           => sub {NACL::AppBuilder::EnGenX::Controller::IPspace::BroadcastDomain->delete(@_)},
        'IPspace'                    => sub {NACL::AppBuilder::EnGenX::Controller::IPspace->delete(@_)},
        # This is put after the cleanup of IPSpace and BD because it attempts to re-add
        # ports to the default IPSpace and BDs that were automatically removed during
        # IPSpace and BD deletion.
        'BD_Port'                    => sub {NACL::AppBuilder::EnGenX::Controller::IPspace::BroadcastDomain::Port->delete(@_)},
        'Vlan'                       => sub {NACL::AppBuilder::EnGenX::Controller::Vlan->delete(@_)},
        'Ifgrp'                      => sub {NACL::AppBuilder::EnGenX::Controller::Ifgrp->delete(@_)},
        'Job_Schedule_Cron' => sub {NACL::AppBuilder::EnGenX::Controller::JobScheduleCron->delete(@_)},
    );

    $self->_controller_execution_helper(
                dispatch_table=> \%pre_lun_snapmirror_dispatch,
                action => "CleanUp",
                %args
            );

    $self->_controller_execution_helper(
                dispatch_table=> \%lun_snapmirror_dispatch,
                action => "CleanUp",
                %args
            );

    $self->_controller_execution_helper(
                dispatch_table=> \%pre_volume_snapmirror_dispatch,
                action => "CleanUp",
                %args
            );

    $self->_controller_execution_helper(
                dispatch_table=> \%volume_snapmirror_dispatch,
                action => "CleanUp",
                %args
            );

    $self->_controller_execution_helper(
                dispatch_table=> \%pre_vserver_snapmirror_dispatch,
                action => "CleanUp",
                %args
            );

    $self->_controller_execution_helper(
                dispatch_table=> \%vserver_snapmirror_dispatch,
                action => "CleanUp",
                %args
            );

    $self->_controller_execution_helper(
                dispatch_table=> \%vserver_peer_dispatch,
                action => "CleanUp",
                %args
            );

    $self->_controller_execution_helper(
                dispatch_table=> \%post_vserver_snapmirror_dispatch,
                action => "CleanUp",
                %args
            );

    # On cleanup need to delete the used_ip list because it will no longer be valid
    $self->{parent}->{used_ips}->{ipv4} = ();
    $self->{parent}->{used_ips}->{ipv6} = ();

    $Log->comment("Cluster Cleanup completed");
    $Log->exit() if $may_exit;
    return TRUE;
} ## end sub cleanup

=head2 validate

Runs set of validation methods to verify that controllers are
in expected state.  Currently controller validate is a no-op.

  $self->{sm_controller_object}->validate();

=over

=item Options

None

=back

=item Return

TRUE

=item Exceptions

None

=cut

sub validate {
    my ($self, %args) = @_;

    # TODO: Add controller cluster validation
    $Log->comment("\n\nCurrently No cluster validation is done in "
                . "NACL::AppBuilder::EnGenX::Controller::validate()\n\n"
    );
    return TRUE;
} ## end sub validate

#==============================================================================
# Method:     _setup_snapmirror_initialize_execution_helper
# Objective:  Setup snapmirror initializes in the correct order.
#==============================================================================
sub _setup_snapmirror_initialize_execution_helper {
    my ($self, @opts) = @_;
    $Log->enter() if $may_enter;
    # Locals
    my %args=();

    # Process arguments.
    # Allowing extra arguments to be used when calling
    # ResourceList->get_resources()
    try {
        %args = Params::Validate::validate_with(
            params => \@opts,
            spec => {resource_list => {type => HASHREF, optional => FALSE},
                     dispatch_table=> {type => HASHREF, optional => FALSE},
                     run_id_prefix => {type => SCALAR, optional => TRUE,
                                       default => "EnGenX_"},
                    }
        );
    }
    otherwise {
        my $xcptn = shift;
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw($xcptn->text());
    };

    my $dispatch_type = 'Snapmirror_Initialize';
    my $type = lc($dispatch_type);
    my $dispatch_table = delete $args{dispatch_table};
    # Gather values
    # Delete the arguments so that ARGs only holds the information the user
    # intends to use on Resource_List->get_resources()
    my $resource_list = delete $args{resource_list};
    my $run_id_prefix = delete $args{run_id_prefix};
    my @clusters = $resource_list->get_categories(category => '^CLUSTER_');

    my %ordered_snapmirror_initialize_hash = ();
    my %unordered_snapmirror_initializes = ();

    # Sort the snapmirror initialize resource by cluster and _snapmirror_initalize_order
    $Log->debug("Resource List before _setup_snapmirror_initialize_execution_helper is:\n". Dumper($resource_list))
        if ($Log->may_debug());
    foreach my $cluster (@clusters) {
        my $clusters_SM_INIT_resources = $resource_list->get_resources(
                                        category => $cluster,
                                        type     => "^snapmirror_initialize\$",
                                        data     => {_setup => 1,
                                                     _setup_done => 0},
                                    );

        if (not scalar @{$clusters_SM_INIT_resources}) {
            my $cluster_name = substr($cluster,8);
            $Log->comment("NACL::AppBuilder::EnGenX::Controller::_setup_snapmirror_initialize_execution_helper "
                  ."Resource list for cluster '$cluster' "
                  ."has no Snapmirror Initalize(s) to setup"
                );
        }
        foreach my $SM_INIT (@{$clusters_SM_INIT_resources}) {
            # If the order is zero we assume it can be the same as unordered.
            if($SM_INIT->{_snapmirror_initialize_order}) {
                push(@{$ordered_snapmirror_initialize_hash{$SM_INIT->{_snapmirror_initialize_order}}{$cluster}},$SM_INIT);
            } else {
                push(@{$unordered_snapmirror_initializes{$cluster}},$SM_INIT);
            }
        }
    }

    # First do unordered_snapmirror initializes.
    foreach my $cluster (keys %unordered_snapmirror_initializes) {
        # In order to get the name of the cluster we have to get rid of the
        # 'CLUSTER_' string that is in the top_key name.
        my $cluster_name = substr($cluster,8);
        $Log->comment("Unordered Snapmirror Initializes for cluster '$cluster_name'");

        my $obj_list_ref = $unordered_snapmirror_initializes{$cluster};

        # Update all the _interface attributes for the objects that contain a percent.
        my %interface_percent_hash = ();
        foreach my $object (@{$obj_list_ref}) {
            # Skip the object unless it has a percent defined in _interface
            next unless (exists ($object->{_interface}) and $object->{_interface} =~ /\%/);

            my $interface_percent = uc($object->{_interface});
            $interface_percent =~ /(\d+)%(zapi|cli)/i;
            # The percent requested
            my $requested_percent = $1;
            # The interface the above percent is requested.
            my $requested_interface = $2;

            # Set the unrequested_interface. This will be used later to know the opposite of the
            # requested_interface.
            my $unrequested_interface = 'ZAPI';
            $unrequested_interface = "CLI" if ($requested_interface =~ /zapi/i);

            # If the iterface_percent_hash doesn't have this _interface with a percent defined then add it.
            unless (exists $interface_percent_hash{$interface_percent}) {
                $interface_percent_hash{$interface_percent} = { CLI     => 0,
                                                                ZAPI    => 0, };

                # Now update the object and the interface_percent_hash and go to the next object.
                $object->{_interface} = $requested_interface;
                $interface_percent_hash{$interface_percent}->{$requested_interface}++;
                next;
            }

            # The hash above will look like the following
            # %interface_percent_hash = { 60%ZAPI => {CLI           = 0,
            #                                         ZAPI          = 1, },
            #                             50%CLI  => {CLI           = 3,
            #                                         ZAPI          = 3, },}

            # Get values to calculate the current percent
            my $current_numerator = $interface_percent_hash{$interface_percent}->{$requested_interface};
            my $current_denominator = $interface_percent_hash{$interface_percent}->{CLI}
                                    + $interface_percent_hash{$interface_percent}->{ZAPI};
            # Calculate the current percent.
            my $current_percent   = ($current_numerator/$current_denominator)*100;

            # If current_percent is less than or equal to the requested_percent
            # then we need more of that interface, else we need more of the other interface.
            if( $current_percent <= $requested_percent ) {
                $object->{_interface} = $requested_interface;
                $interface_percent_hash{$interface_percent}->{$requested_interface}++;
            } else {
                $object->{_interface} = $unrequested_interface;
                $interface_percent_hash{$interface_percent}->{$unrequested_interface}++;
            }
        } # end foreach my $object (@{$obj_list_ref}) { used to update _interface.
        $Log->debug("The interface_percent_hash for $type is: \n" . Dumper(\%interface_percent_hash)) if ($Log->may_debug);
        my %params = ( resource_list    => $obj_list_ref,
                       'method-timeout' => 3600,
                    );
        # At this point we know that the cluster exists, now let's
        # validate that the nodes key actually holds an Array. If not
        # continue, since Task.pm will do a SystemNode->find()
        if( scalar(@{$self->{nodes_by_cluster}->{$cluster_name}->{nodes}})) {
            $params{nacl_nodes} =
                    $self->{nodes_by_cluster}->{$cluster_name}->{nodes};
        } else {
            $Log->warn( "Controller::_setup_snapmirror_initialize_execution_helper "
                       ."'nodes_by_cluster' has cluster '$cluster_name' "
                       ."defined but there were no NODES associated to "
                       ."this cluster, consequently each "
                       ."Controller::Task:Components will discover a "
                       ."SystemNode to be used as a command_interface "
                       ."for NACL."
                    );
        }

        # Pass the EnGenX obj through so that it can be accessed for
        # Global data.
        $params{EnGenX} = $self->{parent};
        # The actual execution.
        # This is an implementation of a dispatch table that is just a
        # mapping of keys (used as pointers) to actions (or methods).
        # By calling the dispatch table with an specific key
        # $dispatch_table->{$key}, we are invoking the call to the
        # subroutine that the key holds. Then, we are passing the params
        # $dispatch_table->{$key}->(%params) to that method call.
        $Log->comment("NACL::AppBuilder::EnGenX::Controller::_setup_snapmirror_initialize_execution_helper "
                      ."action: setup unordered, resources type: '$type', "
                      ."cluster: '$cluster_name'"
                    );
        my $start = [Time::HiRes::gettimeofday()];
        $dispatch_table->{$dispatch_type}->(%params);
        my $difference = Time::HiRes::tv_interval($start);
        $EnGenXUtil::OBJ_COUNT += scalar(@$obj_list_ref);
        # Redirect the $Log calls to write to performance.log
        $Log->redirect(path=>(Tharn::param('LOGDIR').'/performance.log'));
        $Log->comment(scalar(@$obj_list_ref) . " objects for " . $dispatch_type . " unordered :" . "_setup_snapmirror_initialize_execution_helper" .
                      " took $difference seconds with an average of ".
                      $difference/(scalar(@$obj_list_ref)) . " seconds per object    ");
        # Direct $Log calls to write back to the original log file
        $Log->reset_path();
    } # End for each cluster in unordered_snapmirror_initializes

    # Next perform the ordered snapmirror initializes in ascending order. 1,2,3..N
    foreach my $order_index (sort {$a <=> $b} keys %ordered_snapmirror_initialize_hash) {
        foreach my $cluster (keys $ordered_snapmirror_initialize_hash{$order_index}) {
            # In order to get the name of the cluster we have to get rid of the
            # 'CLUSTER_' string that is in the top_key name.
            my $cluster_name = substr($cluster,8);
            $Log->comment("Unordered Snapmirror Initializes for cluster '$cluster_name'");

            my $obj_list_ref = $ordered_snapmirror_initialize_hash{$order_index}{$cluster};

            # Update all the _interface attributes for the objects that contain a percent.
            my %interface_percent_hash = ();
            foreach my $object (@{$obj_list_ref}) {
                # Skip the object unless it has a percent defined in _interface
                next unless (exists ($object->{_interface}) and $object->{_interface} =~ /\%/);

                my $interface_percent = uc($object->{_interface});
                $interface_percent =~ /(\d+)%(zapi|cli)/i;
                # The percent requested
                my $requested_percent = $1;
                # The interface the above percent is requested.
                my $requested_interface = $2;

                # Set the unrequested_interface. This will be used later to know the opposite of the
                # requested_interface.
                my $unrequested_interface = 'ZAPI';
                $unrequested_interface = "CLI" if ($requested_interface =~ /zapi/i);

                # If the iterface_percent_hash doesn't have this _interface with a percent defined then add it.
                unless (exists $interface_percent_hash{$interface_percent}) {
                    $interface_percent_hash{$interface_percent} = { CLI     => 0,
                                                                    ZAPI    => 0, };

                    # Now update the object and the interface_percent_hash and go to the next object.
                    $object->{_interface} = $requested_interface;
                    $interface_percent_hash{$interface_percent}->{$requested_interface}++;
                    next;
                }

                # The hash above will look like the following
                # %interface_percent_hash = { 60%ZAPI => {CLI           = 0,
                #                                         ZAPI          = 1, },
                #                             50%CLI  => {CLI           = 3,
                #                                         ZAPI          = 3, },}

                # Get values to calculate the current percent
                my $current_numerator = $interface_percent_hash{$interface_percent}->{$requested_interface};
                my $current_denominator = $interface_percent_hash{$interface_percent}->{CLI}
                                        + $interface_percent_hash{$interface_percent}->{ZAPI};
                # Calculate the current percent.
                my $current_percent   = ($current_numerator/$current_denominator)*100;

                # If current_percent is less than or equal to the requested_percent
                # then we need more of that interface, else we need more of the other interface.
                if( $current_percent <= $requested_percent ) {
                    $object->{_interface} = $requested_interface;
                    $interface_percent_hash{$interface_percent}->{$requested_interface}++;
                } else {
                    $object->{_interface} = $unrequested_interface;
                    $interface_percent_hash{$interface_percent}->{$unrequested_interface}++;
                }
            } # end foreach my $object (@{$obj_list_ref}) { used to update _interface.
            $Log->debug("The interface_percent_hash for $type is: \n" . Dumper(\%interface_percent_hash)) if ($Log->may_debug);
            my %params = ( resource_list    => $obj_list_ref,
                           'method-timeout' => 3600,
                        );
            # At this point we know that the cluster exists, now let's
            # validate that the nodes key actually holds an Array. If not
            # continue, since Task.pm will do a SystemNode->find()
            if( scalar(@{$self->{nodes_by_cluster}->{$cluster_name}->{nodes}})) {
                $params{nacl_nodes} =
                        $self->{nodes_by_cluster}->{$cluster_name}->{nodes};
            } else {
                $Log->warn( "Controller::_setup_snapmirror_initialize_execution_helper "
                           ."'nodes_by_cluster' has cluster '$cluster_name' "
                           ."defined but there were no NODES associated to "
                           ."this cluster, consequently each "
                           ."Controller::Task:Components will discover a "
                           ."SystemNode to be used as a command_interface "
                           ."for NACL."
                        );
            }

            # Pass the EnGenX obj through so that it can be accessed for
            # Global data.
            $params{EnGenX} = $self->{parent};
            # The actual execution.
            # This is an implementation of a dispatch table that is just a
            # mapping of keys (used as pointers) to actions (or methods).
            # By calling the dispatch table with an specific key
            # $dispatch_table->{$key}, we are invoking the call to the
            # subroutine that the key holds. Then, we are passing the params
            # $dispatch_table->{$key}->(%params) to that method call.
            $Log->comment("NACL::AppBuilder::EnGenX::Controller::_setup_snapmirror_initialize_execution_helper "
                          ."action: 'setup ordered $order_index', resources type: '$type', "
                          ."cluster: '$cluster_name'"
                        );
            my $start = [Time::HiRes::gettimeofday()];
            $dispatch_table->{$dispatch_type}->(%params);
            my $difference = Time::HiRes::tv_interval($start);
            $EnGenXUtil::OBJ_COUNT += scalar(@$obj_list_ref);
            # Redirect the $Log calls to write to performance.log
            $Log->redirect(path=>(Tharn::param('LOGDIR').'/performance.log'));
            $Log->comment(scalar(@$obj_list_ref) . " objects for " . $dispatch_type . " ordered $order_index:" . "_setup_snapmirror_initialize_execution_helper" .
                          " took $difference seconds with an average of ".
                          $difference/(scalar(@$obj_list_ref)) . " seconds per object    ");
            # Direct $Log calls to write back to the original log file
            $Log->reset_path();
        } # End for each cluster in order number #
    } # End for each order index
    $Log->debug(
        "The Resource List After _setup_snapmirror_initialize_execution_helper is:\n" . Dumper($resource_list))
    if ($Log->may_debug());

    $Log->exit() if $may_exit;
    return TRUE;
} ## end _setup_snapmirror_initialize_execution_helper

#==============================================================================
# Method:     _controller_execution_helper
# Objective:  Abstract the duplicated code from config, setup, deconfig and
#             cleanup and created a simple way of adding/deleting resources to
#             the controller.
# Details:    Steps: 1)   Get all the categories that have information about
#                         the controller, basically all keys that start with
#                         CLUSTER
#                    2)   For each found CLUSTER key get all the TYPEs (vserver,
#                         vol, lun, etc) from the dispatch table.
#                    3)   For each found TYPE get the resources (resource_list)
#                    4)   Create each resource.
# Inputs:     dispatch_table. The hash table that holds the 'key' that is the
#                             type of the resource and the subroutine to be
#                             executed.
#             resource_list. The EnGenX xml resource list.
#             types. Depending on the caller, there could be different types.
#                    (this discrepancy could be eliminated when setup and
#                     config are consolidated).
#             run_id. Id to identify the subprocess that could be created.
# Outputs:    Boolean, True if no problem was encountered.
# Exceptions: Arguments exception if the passed arguments are wrong.
# TODO: Remove the resource_list argument in the logic and get the resource
#       list from $self ($self->{resource_list}.
#       Also remove the resource list argument on the caller (EnGenX::setup)
#
#==============================================================================
sub _controller_execution_helper {
    my ($self, @opts) = @_;
    $Log->enter() if $may_enter;
    # Locals
    my %args=();

    # Process arguments.
    # Allowing extra arguments to be used when calling
    # ResourceList->get_resources()
    try {
        %args = Params::Validate::validate_with(
            params => \@opts,
            spec => {resource_list => {type => HASHREF, optional => FALSE},
                     dispatch_table=> {type => HASHREF, optional => FALSE},
                     action        => {type => SCALAR, optional => FALSE},
                     run_id_prefix => {type => SCALAR, optional => TRUE,
                                       default => "EnGenX_"},
                    }
        );
    }
    otherwise {
        my $xcptn = shift;
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw($xcptn->text());
    };

    # Gather values
    # Delete the arguments so that ARGs only holds the information the user
    # intends to use on Resource_List->get_resources()
    my $resource_list = delete $args{resource_list};
    my $dispatch_table = delete $args{dispatch_table};
    my $action = delete $args{action};
    my $run_id_prefix = delete $args{run_id_prefix};
    my @clusters = $resource_list->get_categories(category => '^CLUSTER_');
    # Create the correct filters for cleanup and setup
    # The filter hash will filter out objects that dont need to be cleaned up
    # or setup based on the action to be perform so only need resources are
    # put through the processing engine.
    my %filter_hash = ();
    if ($action =~ /^cleanup$/i) {
        %filter_hash = ( _cleanup => 1,
                         _cleanup_done => 0 );
    } else {
        %filter_hash = ( _setup => 1,
                         _setup_done => 0 );
    }

    my %neg_filter_hash = ();
    # If the action is setup mean the setup before snapmirror.
    # Then we do NOT want to setup dp-destination _subtype objects.
    if ($action =~ /^setup$/i) {
        %neg_filter_hash = (neg_data => {_subtype => 'dp-destination'})
    } # otherwise we want to include the _subtype =>'dp-destination' objects.

    # If the action is setup_after_snapmirror,
    # then we want to only include the dp-destination _subtype objects.
    if ($action =~ /^setup_after_snapmirror$/i) {
        $filter_hash{_subtype} = 'dp-destination';
    }

    # Setup each cluster. Assumes a Resource List:
    # [cluster_<name1> => Aggr => array_of_aggr_resources,
    #                     VOL => array_of_volumes
    # ],
    # [cluster_<name2> => Aggr => array_of_aggr_resources,
    #                     VOL => array_of_volumes
    # ]...
    $Log->debug("Resource List before $action is:\n". Dumper($resource_list))
        if ($Log->may_debug());

    # Dispatch all the components for each found Cluster
    foreach my $cluster (@clusters) {
        # In order to get the name of the cluster we have to get rid of the
        # 'CLUSTER_' string that is in the top_key name.
        my $cluster_name = substr($cluster,8);
        $Log->comment("$action cluster '$cluster_name'");

        # Verify if the Cluster has been created
        $self->_cluster_validate(action => $action,
                                 cluster_name => $cluster_name,
                                 dispatch_table => $dispatch_table,
                                );

        $Log->debug("NACL::AppBuilder::EnGenX::Controller::Controller_execution_helper: "
                    ."THE DISPATCH TABLE". Dumper($dispatch_table))
            if ($Log->may_debug());

        # Create each requested type
        # Use the keys in the correct order, starting from top to buttom.
        foreach my $type (keys(%{$dispatch_table})) {
            # Gather the parameters needed for calling CREATE or DELETE for each
            # resource.
            # Start with the list of resources:
            my $dispatch_type = $type;
            $type = lc($type);
            my $obj_list_ref = undef;
            if ($type =~ /QOS_POLICY_GROUP/i){
                my $nacl_cluster_obj = $self->{parent}->{_objects}->
                                              {cluster}->{$cluster_name};
                # See if the release is SN or after because qos is only supported in SN or after.
                my $release_SN_or_after;
                $release_SN_or_after = $nacl_cluster_obj->version_manager()->is_version_ge(release => '8.2');
                unless ($release_SN_or_after) {
                    # QOS is not support in RRollingrock and before.
                    next;
                }
            }
            # We dont want to create volumes or luns after snapmirror because
            # the snapmirror will copy the volumes and luns from the src to the dest.
            if ($type =~ /^(volume|lun|volume_efficiency|cifs|qtree|dns_service|lif)$/i and $action =~ /^setup_after_snapmirror$/i){
                $obj_list_ref = [];
            }
            # We only want to cleanup non dp destination volumes and luns
            # As those get cleaned up during the snapmirror update after
            # the source config gets deleted.
            elsif ($type =~ /^(volume|lun|qtree)$/i and $action =~ /^cleanup$/i){
                $obj_list_ref = $resource_list->get_resources(
                    category => $cluster,
                    type     => "^".$type."\$",
                    data     => {%filter_hash},
                    neg_data => {_subtype => 'dp-destination'},
                );
            }
            elsif ($type =~ /^Volume_Efficiency$/i){
                $obj_list_ref = $resource_list->get_resources(
                                category => $cluster,
                                type     => '^efficiency_options$',
                                data     => {_efficiency => 1,
                                            %filter_hash},
                                %neg_filter_hash
                            );
            }
            elsif($type =~ /^Snapmirror_Initialize$/i) {
                $obj_list_ref = $resource_list->get_resources(
                                category => $cluster,
                                type     => "^".$type."\$",
                                data     => {%filter_hash},
                            );
            }
            elsif($type =~ /^(dns_service|lif)$/i and $action =~ /^setup_snapmirror$/) {
                $obj_list_ref = $resource_list->get_resources(
                                category => $cluster,
                                type     => "^".$type."\$",
                                data     => {_subtype        => 'dp-destination',
                                             %filter_hash},
                            );
            }
            elsif($type =~ /^(CIFS)$/i){
                my $service_type = "^" . $type . "_service\$";
                # Cleanup all CIFS service
                if($action =~ /^cleanup$/i) {
                    $obj_list_ref = $resource_list->get_resources(
                                category => $cluster,
                                type     => $service_type,
                                data     => {service_enabled => 1,
                                            %filter_hash},
                                );
                } # setup between snapmirror create and initialize only
                  # the dp-dest CIFS services
                elsif ($action =~/^setup_snapmirror$/i) {
                    $obj_list_ref = $resource_list->get_resources(
                                category => $cluster,
                                type     => $service_type,
                                data     => {service_enabled => 1,
                                             _subtype        => 'dp-destination',
                                            %filter_hash},
                                );
                } # setup before snapmirror create only the non dp-dest CIFS services
                elsif ($action =~/^setup$/i) {
                    $obj_list_ref = $resource_list->get_resources(
                                category => $cluster,
                                type     => $service_type,
                                data     => {service_enabled => 1,
                                            %filter_hash},
                                neg_data => {_subtype        => 'dp-destination',},
                                );
                }
            }
            elsif($type =~ /^(FCP|ISCSI|ISNS|NFS)$/i){
                my $service_type = "^" . $type . "_service\$";
                $obj_list_ref = $resource_list->get_resources(
                            category => $cluster,
                            type     => $service_type,
                            data     => {service_enabled => 1,
                                        %filter_hash},
                            %neg_filter_hash
                            );
            }
            elsif ($type =~ /^ISCSI_SECURITY$/i){
                $self->_expand_iscsi_security_settings_to_resources(
                                              resource_list => $resource_list,
                                              category => $cluster);
                $obj_list_ref = $resource_list->get_resources(
                                category => $cluster,
                                type     => "^".$type."\$",
                                data     => {%filter_hash},
                                %neg_filter_hash
                            );
            }
            elsif ($type =~ /^INITIATOR$/i){
                # Only do the conversion once during setup.
                # Initiators are not part of cleanup currently.
                if($action =~ /^setup$/i) {
                    $self->_convert_clients_to_initiators(
                                                  resource_list => $resource_list,
                                                  category => $cluster);
                }

                $obj_list_ref = $resource_list->get_resources(
                                category => $cluster,
                                type     => "^".$type."\$",
                                data     => {name => qr/^iqn\.\S+|
                                               ^[a-fA-F0-9]{16}$|
                                               ^[a-fA-F0-9]{2}:[a-fA-F0-9]{2}:
                                                [a-fA-F0-9]{2}:[a-fA-F0-9]{2}:
                                                [a-fA-F0-9]{2}:[a-fA-F0-9]{2}:
                                                [a-fA-F0-9]{2}:[a-fA-F0-9]{2}$
                                              /x,
                                            %filter_hash},
                                %neg_filter_hash
                            );
            }
            elsif ($type =~ /^Aggregate$/i and $action =~ /^cleanup$/i){
                $obj_list_ref = $resource_list->get_resources(
                                category => $cluster,
                                type     => "^".$type."\$",
                                data     => {%filter_hash},
                                neg_data => {'_cleanup_policy' => qr/^noaggr$/i},
                            );
            }
            elsif($type =~ /^(vserver_snapmirror)$/i){
                $obj_list_ref = $resource_list->get_resources(
                            category => $cluster,
                            type     => '^snapmirror$',
                            data => {%filter_hash,
                                     _snapmirror_object_level => 'vserver'},
                            );
            }
            elsif($type =~ /^(volume_snapmirror)$/i){
                $obj_list_ref = $resource_list->get_resources(
                            category => $cluster,
                            type     => '^snapmirror$',
                            data => {%filter_hash,
                                     _snapmirror_object_level => 'volume'},
                            );
            }
            elsif($type =~ /^(lun_snapmirror)$/i){
                $obj_list_ref = $resource_list->get_resources(
                            category => $cluster,
                            type     => '^snapmirror$',
                            data => {%filter_hash,
                                     _snapmirror_object_level => 'lun'},
                            );
            }
            else {
                $obj_list_ref = $resource_list->get_resources(
                                category => $cluster,
                                type     => "^".$type."\$",
                                data     => {%filter_hash},
                                %neg_filter_hash
                            );
            }


            if($action =~ /^setup/i and scalar(@{$obj_list_ref}) and $type =~ /Qos_Policy_Group/i) {
                my @temp_list = @{$obj_list_ref};
                @{$obj_list_ref} = $self->_fetch_and_filter_resources(resource_list => \@temp_list,
                                                                      type          => $type,
                                                                      cluster_name  => $cluster_name);
            }

            if($action =~ /^setup/i and scalar(@{$obj_list_ref}) and $type =~ /NFS_KERBEROS_INTERFACE/i) {
                my @temp_list = @{$obj_list_ref};
                @{$obj_list_ref} = $self->_add_lif_ip_address_to_kerberos_interface(
                                                          resource_list => \@temp_list,
                                                          cluster_name  => $cluster_name);
            }

            if($action =~ /^setup/i and scalar(@{$obj_list_ref}) and $type =~ /^VOLUME$/i) {
                my @temp_list = @{$obj_list_ref};
                @{$obj_list_ref} = $self->_create_aggr_list_on_volumes(
                                                            resource_list  => $resource_list,
                                                            resource_array => \@temp_list,
                                                            category  => $cluster);
            }

            if($action =~ /^setup/i and scalar(@{$obj_list_ref}) and $type =~ /^LIF$/i) {
                my @temp_list = @{$obj_list_ref};
                @{$obj_list_ref} = $self->_normalize_ip_ranges_on_lifs(
                                                            resource_list  => $resource_list,
                                                            resource_array => \@temp_list,
                                                            category  => $cluster);
            }

            # Now, if we have a list of resources, execute.
            if (scalar(@{$obj_list_ref})) {
                # Update all the _interface attributes for the objects that contain a percent.
                my %interface_percent_hash = ();
                foreach my $object (@{$obj_list_ref}) {
                    # Skip the object unless it has a percent defined in _interface
                    next unless (exists ($object->{_interface}) and $object->{_interface} =~ /\%/);

                    my $interface_percent = uc($object->{_interface});
                    $interface_percent =~ /(\d+)%(zapi|cli)/i;
                    # The percent requested
                    my $requested_percent = $1;
                    # The interface the above percent is requested.
                    my $requested_interface = $2;

                    # Set the unrequested_interface. This will be used later to know the opposite of the
                    # requested_interface.
                    my $unrequested_interface = 'ZAPI';
                    $unrequested_interface = "CLI" if ($requested_interface =~ /zapi/i);

                    # If the iterface_percent_hash doesn't have this _interface with a percent defined then add it.
                    unless (exists $interface_percent_hash{$interface_percent}) {
                        $interface_percent_hash{$interface_percent} = { CLI     => 0,
                                                                        ZAPI    => 0, };

                        # Now update the object and the interface_percent_hash and go to the next object.
                        $object->{_interface} = $requested_interface;
                        $interface_percent_hash{$interface_percent}->{$requested_interface}++;
                        next;
                    }

                    # The hash above will look like the following
                    # %interface_percent_hash = { 60%ZAPI => {CLI           = 0,
                    #                                         ZAPI          = 1, },
                    #                             50%CLI  => {CLI           = 3,
                    #                                         ZAPI          = 3, },}

                    # Get values to calculate the current percent
                    my $current_numerator = $interface_percent_hash{$interface_percent}->{$requested_interface};
                    my $current_denominator = $interface_percent_hash{$interface_percent}->{CLI}
                                            + $interface_percent_hash{$interface_percent}->{ZAPI};
                    # Calculate the current percent.
                    my $current_percent   = ($current_numerator/$current_denominator)*100;

                    # If current_percent is less than or equal to the requested_percent
                    # then we need more of that interface, else we need more of the other interface.
                    if( $current_percent <= $requested_percent ) {
                        $object->{_interface} = $requested_interface;
                        $interface_percent_hash{$interface_percent}->{$requested_interface}++;
                    } else {
                        $object->{_interface} = $unrequested_interface;
                        $interface_percent_hash{$interface_percent}->{$unrequested_interface}++;
                    }
                } # end foreach my $object (@{$obj_list_ref}) { used to update _interface.
                $Log->debug("The interface_percent_hash for $type is: \n" . Dumper(\%interface_percent_hash)) if ($Log->may_debug);
                my %params = ( resource_list    => $obj_list_ref,
                               'method-timeout' => 3600,
                            );
                # At this point we know that the cluster exists, now let's
                # validate that the nodes key actually holds an Array. If not
                # continue, since Task.pm will do a SystemNode->find()
                if( scalar(@{$self->{nodes_by_cluster}->{$cluster_name}->{nodes}})) {
                    $params{nacl_nodes} =
                            $self->{nodes_by_cluster}->{$cluster_name}->{nodes};
                } else {
                    $Log->warn( "Controller::_controller_execution_helper "
                               ."'nodes_by_cluster' has cluster '$cluster_name' "
                               ."defined but there were no NODES associated to "
                               ."this cluster, consequently each "
                               ."Controller::Task:Components will discover a "
                               ."SystemNode to be used as a command_interface "
                               ."for NACL."
                            );
                }

                # Pass the EnGenX obj through so that it can be accessed for
                # Global data.
                $params{EnGenX} = $self->{parent};
                # The actual execution.
                # This is an implementation of a dispatch table that is just a
                # mapping of keys (used as pointers) to actions (or methods).
                # By calling the dispatch table with an specific key
                # $dispatch_table->{$key}, we are invoking the call to the
                # subroutine that the key holds. Then, we are passing the params
                # $dispatch_table->{$key}->(%params) to that method call.
                $Log->comment("NACL::AppBuilder::EnGenX::Controller::_controller_execution_helper "
                              ."action: '$action', resources type: '$type', "
                              ."cluster: '$cluster_name'"
                            );
                my $start = [Time::HiRes::gettimeofday()];
                $dispatch_table->{$dispatch_type}->(%params);
                my $difference = Time::HiRes::tv_interval($start);
                $EnGenXUtil::OBJ_COUNT += scalar(@$obj_list_ref);
                # Redirect the $Log calls to write to performance.log
                $Log->redirect(path=>(Tharn::param('LOGDIR').'/performance.log'));
                $Log->comment(scalar(@$obj_list_ref) . " objects for " . $dispatch_type . ":" . $action .
                              " took $difference seconds with an average of ".
                              $difference/(scalar(@$obj_list_ref)) . " seconds per object    ");
                # Direct $Log calls to write back to the original log file
                $Log->reset_path();
            } else {
                $Log->comment("NACL::AppBuilder::EnGenX::Controller::_controller_execution_helper "
                              ."Resource list for cluster '$cluster_name' "
                              ."has no $type(s) to $action"
                            );
            }
        }# foreach type
    }# foreach cluster

    $Log->debug(
        "The Resource List After $action is:\n" . Dumper($resource_list))
    if ($Log->may_debug());

    $Log->exit() if $may_exit;
    return TRUE;
} ## end _controller_execution_helper

###############################################################################
# Method:    create_aggr_list_on_volumes
# Objective: Normalizes the lifs IPS
# Inputs:    $resource_list - reference to a Resource List
#            $category       - the current category
# Returns:   none
###############################################################################
sub _normalize_ip_ranges_on_lifs() {
    my ($self, %args) = @_;

    $Log->enter() if $may_enter;

     # Locals
    my $resource_list = $args{resource_list};
    my $resource_array = $args{resource_array};
    my $category  = $args{category};

    # Update all the resources "ips" with a normalized list of ips.
    foreach my $resource (@{$resource_array}) {
        # Skip if there are no ips.
        next unless (defined $resource->{ips});
        my $current_ips = delete $resource->{ips};
        my @normalized_ips = ();
        foreach my $current_ip (@{$current_ips}) {
            push (@normalized_ips, $self->_normalize_ip_range(ip_range => $current_ip));
        }
        $resource->{'ips'} = \@normalized_ips;
    }

    $Log->exit() if $may_exit;
    return (wantarray()) ? @{$resource_array} : $resource_array;
} ## end sub _create_aggr_list_on_volumes()

###############################################################################
# Method:    create_aggr_list_on_volumes
# Objective: Add aggr_list to volumes.
# Inputs:    $resource_list - reference to a Resource List
#            $category       - the current category
# Returns:   none
###############################################################################
sub _create_aggr_list_on_volumes() {
    my ($self, %args) = @_;

    $Log->enter() if $may_enter;

     # Locals
    my $resource_list = $args{resource_list};
    my $resource_array = $args{resource_array};
    my $category  = $args{category};

    # Get all aggregate resources.
    my @aggregrate_resources = $resource_list->get_resources(
                                category => $category,
                                type     => '^aggregate$'
                                );

    # Make an array for all the aggregate names
    my @aggregate_names = ();
    foreach my $aggregate (@aggregrate_resources) {
        push(@aggregate_names, $aggregate->{name});
    }

    # Update all the resource with an aggr list based on aggr_list_regex
    foreach my $resource (@{$resource_array}) {
        next unless (defined $resource->{aggr_list_regex});
        my $filter = delete $resource->{aggr_list_regex};
        my @temp_aggr_list = grep(/$filter/,@aggregate_names);
        $resource->{'aggr-list'} = \@temp_aggr_list;
    }

    $Log->exit() if $may_exit;
    return (wantarray()) ? @{$resource_array} : $resource_array;
} ## end sub _create_aggr_list_on_volumes()

###############################################################################
# Method:    add_lif_ip_address_to_kerberos_interface
# Objective: Adds the lif ip address to the kerberos interface resources
# Inputs:    $resource_list - reference to a Resource List
#            $category       - the current category
# Returns:   none
###############################################################################
sub _add_lif_ip_address_to_kerberos_interface() {
    my ($self, %args) = @_;

    $Log->enter() if $may_enter;

     # Locals
    my $resource_list = $args{resource_list};
    my $cluster_name  = $args{cluster_name};

    my $command_interface = $self->{parent}->{_objects}->
                                   {cluster}->{$cluster_name};
    $Log->comment("Fetching lif objects from the cluster $cluster_name.");
    my @fetched_objs  = (NACL::CS::NetworkInterface->fetch(
                                    allow_empty       => 1,
                                    command_interface => $command_interface,
                                    requested_fields  => ['vserver','lif','address']));

    # Create a sorted lif hash to lookup the address for the resources
    # of the format _cluster->vserver->lif = address
    my %sorted_lif_hash = ();
    foreach my $lif (@fetched_objs) {
        $sorted_lif_hash{$cluster_name}->{$lif->{vserver}}->{$lif->{lif}} = $lif->{address};
    }

    my @filtered_list = ();
    foreach my $resource (@{$resource_list}) {
        $resource->{address} = $sorted_lif_hash{$cluster_name}->{$resource->{vserver}}->{$resource->{lif}};
        push(@filtered_list,$resource);
    }

    $Log->exit() if $may_exit;
    return (wantarray()) ? @filtered_list : \@filtered_list;
} ## end sub _add_lif_ip_address_to_kerberos_interface()

#==============================================================================
# Method:     _fetch_and_filter_resources
# Objective:  Filter out objects from an array of resource that already exists
#                 on the filer.
# Details:    Steps: 1)   Fetch the correct resource type
#                    2)   Compare all the primary keys to check whether the object
#                         matches an existing object.
# Inputs:     resourece_list. An array of resource objects.
#             type            The type of resource
#             cluster_name    Name of the cluster related to the resources
# Outputs:    An array of resources filtered by existence on filer.
# Exceptions: None.
#==============================================================================
sub _fetch_and_filter_resources {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;
    # A hash to convert our types to NACL packages.
    my %type_to_pkg_hash =  (qos_policy_group => {'name' => 'policy-group',
                                                  'pkg'  => 'QosPolicyGroup'},
                            );
    # Locals
    my $resource_list = $args{resource_list};
    my $type          = lc($args{type});
    my $cluster_name  = $args{cluster_name};
    my $type_info     = $type_to_pkg_hash{$type};
    my @primary_keys  = ("NACL::C::".$type_info->{'pkg'})->get_primary_keys();
    # NACL primary keys dont include vserver for qos_policy_group in primary keys
    # but we need it to make sure that the resource is the same as what is on the
    # the filer.
    push(@primary_keys, "vserver") if ($type =~ /qos_policy_group/i);
    my $command_interface = $self->{parent}->{_objects}->
                                   {cluster}->{$cluster_name};
    $Log->comment("Fetching ".$type_info->{'pkg'}." objects from the cluster $cluster_name.");
    my @fetched_objs  = ("NACL::CS::".$type_info->{'pkg'})->fetch(
                                    allow_empty       => 1,
                                    command_interface => $command_interface,
                                    requested_fields  => \@primary_keys);
    # Delete Command Interface from CS objects to save space.
    foreach my $obj (@fetched_objs){
        delete $obj->{command_interface};
    }
    my @filtered_list = ();

    # Only loop through if filer has this type of object.
    if(scalar(@fetched_objs)) {
        my $match_count = 0;
        foreach my $resource (@{$resource_list}){
            # Match is a count of how many keys matched.
            # If all primary keys matched then we have a match.
            my $match = 0;
            foreach my $obj (@fetched_objs){
                $match = 0;
                foreach my $key (@primary_keys){
                    if($type_info->{name} =~ /^$key$/ and exists $resource->{name}) {
                        $match++ if ($resource->{name} eq $obj->$key());
                    } else {
                        $match++ if ($resource->{$key} eq $obj->$key());
                    }
                }
                if($match == scalar(@primary_keys)) {
                    $match_count++;
                    $resource->{_setup_done}=1;
                    $resource->{_reused}=1;
                    last;
                }
            }
            push(@filtered_list, $resource) unless ($match == scalar(@primary_keys));
            last if($match_count >= scalar(@fetched_objs));
        }
    } else { # Otherwise return the list passed in.
        return (wantarray()) ? @{$resource_list} : $resource_list;
    }
    $Log->exit() if $may_exit;
    return (wantarray()) ? @filtered_list : \@filtered_list;

} ## end _fetch_and_filter_resources

###############################################################################
# Method:    expand_iscsi_security_settings_to_resources
# Objective: Expands the vserver resource to iscsi security resources per client
# Inputs:    $resource_list - reference to a Resource List
#            $category       - the current category
# Returns:   none
###############################################################################
sub _expand_iscsi_security_settings_to_resources() {
    my ($self, %args) = @_;

    $Log->enter() if $may_enter;

    &ParamsMgr::process_args(%args);
    # Grab the resource list. This should have been passed as a
    # reference.
    my $resource_list = &ParamsMgr::get_arg(
        -arg      => "resource_list",
        -required => 1
    );
    my $category = &ParamsMgr::get_arg(
        -arg      => "category",
        -required => 1
    );
    if (&ParamsMgr::error()) {
        $Log->exit() if $may_exit;
        my $errmsg = join "\n", &ParamsMgr::get_error();
        NATE::BaseException->throw("Argument parsing failed: $errmsg");
    }

    my $vserver_resources = $resource_list->get_resources(
                                category => $category,
                                type     => '^vserver$',
                                data     => {iscsi_enabled => 1}
                                );

    foreach my $resource_data (@{$vserver_resources}) {

        my $cluster_name = $resource_data->{_cluster};
        my $vserver_name = $resource_data->{name};

        # find all iscsi clients related to this vserver/cluster
        my @iscsi_clients = $self->{parent}->get_clients_associated_to_vserver(-cluster_name => $cluster_name,
                                                                     -vserver_names => [$vserver_name],
                                                                     -protocols => ["iscsi", "mixed"]);
        if ($self->error()) {
            NATE::BaseException->throw($self->errmsg());
        }

        foreach my $client (@iscsi_clients) {
            # if this client matches an iSCSI nodename or a FCP WWPN, then we won't be able to create a specific iSCSI setting for it
            # because we can't accurately map it to a real client. Assume that it will use the default vserver iSCSI security settings
            if ($client =~ /^(iqn\.\d+\-\d+\.\S+?:\S+|[A-Fa-f0-9]{2}(?::?[A-Fa-f0-9]{2}){7})$/i) {
                next;
            }
            $Log->trace("Searching for $client in category client,hypervisor");
            my ($client_obj);
            try {
                ($client_obj) = $self->get_objects(types => ["client", "hypervisor"],
                                                   names => [$client]);
            } otherwise {
                # if we could not find this client's object, then we can't treat this
                # as an error per se, just do not build an iscsi security resource object
                # for the client
            };
            if (not defined($client_obj)) {
                next;
            }
            # skip this client if its not doing iSCSI
            if ($client_obj->get_protocol() !~ /iscsi/i) {
                next;
            }

            # obtain the chap settings from this client object
            my $chap_mode = $client_obj->{iscsi}->{CONFIG_ISCSI_CHAP_MODE};
            my $chap_inusername = $client_obj->{iscsi}->{CONFIG_ISCSI_CHAP_USERNAME};
            my $chap_outusername = $client_obj->{iscsi}->{CONFIG_ISCSI_CHAP_OUT_USERNAME};
            my $chap_inpassword = $client_obj->{iscsi}->{CONFIG_ISCSI_CHAP_IN_PASSWORD};
            my $chap_outpassword = $client_obj->{iscsi}->{CONFIG_ISCSI_CHAP_OUT_PASSWORD};

            # fetch the initiator objects for this client, filter out non-iscsi initiators
            my @initiator_objs = $client_obj->get_initiators();
            @initiator_objs = grep { $_->get_protocol() =~ /iscsi/ } @initiator_objs;
            if (not scalar(@initiator_objs)) {
                $Log->exit() if $may_exit;
                NATE::BaseException->throw("There are no iSCSI initiators for the client '$client'");
            }

            # pull out all the nodenames for this initiator
            my @nodenames = map { $_->get_nodename() } @initiator_objs;

            # remove dups from the nodename array
            my %nodename_hash;
            foreach (@nodenames) {
                $nodename_hash{$_}++;
            }
            @nodenames = keys %nodename_hash;

            my $iscsi_security_data = {};

            # Replace the EnGenX fields in the element data with
            # what we determined the current values were.
            $iscsi_security_data->{_cleanup_policy}  = $resource_data->{_cleanup_policy};
            $iscsi_security_data->{if_exists}        = $resource_data->{if_exists};
            $iscsi_security_data->{_parallel}        = $resource_data->{_parallel};
            $iscsi_security_data->{_interface}       = $resource_data->{_interface};
            $iscsi_security_data->{_load_balance}    = $resource_data->{_load_balance};
            $iscsi_security_data->{_setup}           = $resource_data->{_setup};
            $iscsi_security_data->{_cleanup}         = $resource_data->{_cleanup};
            $iscsi_security_data->{_config}          = $resource_data->{_config};
            $iscsi_security_data->{_setup_done}      = 0;
            $iscsi_security_data->{_cleanup_done}    = 0;
            $iscsi_security_data->{vserver}          = $vserver_name;
            $iscsi_security_data->{_cluster}         = $cluster_name;

            # store the chap mode that this initiator is doing
            $iscsi_security_data->{chap_mode} = $chap_mode;

            # if chap_mode is none, then we have to make sure that chap is turned off for this initiator
            if ($chap_mode =~ /none/i) {
                foreach my $nodename (@nodenames) {
                    # set up the initiator name field
                    $iscsi_security_data->{'initiator-name'} = $nodename;

                    # create a resource object for this nodename
                    # the iscsi security create code for this resource
                    # will recognize that there is nothing to set up for this iscsi security
                    # setting, it should explicitly remove it
                    $resource_list->add_resource(category => $category,
                                                         type     => 'iscsi_security',
                                                         data     => $iscsi_security_data);
                }
            } else {
                # setup up unidirectional chap
                $iscsi_security_data->{'auth-type'} = "CHAP";
                $iscsi_security_data->{'user-name'} = $chap_inusername;
                $iscsi_security_data->{'password'} = $chap_inpassword;

                if ($chap_mode =~ /bidirectional/i) {
                    # bidirectional chap
                    $iscsi_security_data->{'outbound-password'} = $chap_outpassword;
                    $iscsi_security_data->{'outbound-user-name'} = $chap_outusername;
                }

                # create a resource obj for each nodename
                foreach my $nodename (@nodenames) {
                    # default the in username as the nodename
                    if (not defined($chap_inusername) or
                        not length($chap_inusername)) {
                        $iscsi_security_data->{'user-name'} = $nodename;
                    }
                    # if doing bidirectional, default the outbound user name to the nodename
                    if ($chap_mode =~ /bidirectional/i) {
                        if (not defined($chap_outusername) or
                            not length($chap_outusername)) {
                            $iscsi_security_data->{'outbound-user-name'} = $nodename;
                        }
                    }

                    # set up the initiator name field
                    $iscsi_security_data->{'initiator-name'} = $nodename;

                    # add the resource
                    $resource_list->add_resource(category => $category,
                                                         type     => 'iscsi_security',
                                                         data     => $iscsi_security_data);
                }
            }
        }
    }
    $Log->exit() if $may_exit;
    return;
} ## end sub _expand_iscsi_security_settings_to_resources()

#==============================================================================
# Method:     _convert_clients_to_initiators
# Objective:  Create the resource list for Initiators.
# Details:    Steps: 1)   Get the initiator resources with client names
#                    2)   Get the initiators from those clients and add them.
# Inputs:     category. The category (key) under wich Initiators (type)
#                       would be looked for on the ResourceList object,
#             resourece_list. The EnGenX xml resource list.
# Outputs:    None
# Exceptions: None.
#
#==============================================================================
sub _convert_clients_to_initiators {
    my ($self, %args) = @_;

    $Log->enter() if $may_enter;

    &ParamsMgr::process_args(%args);
    # Grab the resource list. This should have been passed as a
    # reference.
    my $resource_list = &ParamsMgr::get_arg(
        -arg      => "resource_list",
        -required => 1
    );
    my $top_key = &ParamsMgr::get_arg(
        -arg      => "category",
        -required => 1
    );
    if (&ParamsMgr::error()) {
        $Log->exit() if $may_exit;
        my $errmsg = join "\n", &ParamsMgr::get_error();
        NATE::BaseException->throw("Argument parsing failed: $errmsg");
    }
    # Get the initiators that have names that dont match know WWPN and IQN
    # formats. We assume the these initiators have client names.
    # neg_data means find resources in the list that do NOT match
    # the data.
    my @initiators_with_client_names = $resource_list->get_resources(
                                category => $top_key,
                                type     => '^initiator$',
                                neg_data => {name => qr/^iqn\.\S+|
                                               ^[a-fA-F0-9]{16}$|
                                               ^[a-fA-F0-9]{2}:[a-fA-F0-9]{2}:
                                                [a-fA-F0-9]{2}:[a-fA-F0-9]{2}:
                                                [a-fA-F0-9]{2}:[a-fA-F0-9]{2}:
                                                [a-fA-F0-9]{2}:[a-fA-F0-9]{2}$
                                              /x},
                                );


    foreach my $client (@initiators_with_client_names) {
        my @initiator_list = ();

        next unless (defined($client->{name}));
        my $initiator_data = Clone::clone($client);
        my ($client_obj) = $self->get_objects(types => ["client", "hypervisor"],
                                              names => [$client->{name}]);
        my $igroup_protocol = $client->{_protocol};

        my $all_clients = $resource_list->get_resources(category => "CLIENTS",
                                                        type => "clients");
        my $all_hypervisors = $resource_list->get_resources(category => "HYPERVISORS",
                                                            type => '^hyperv$|^esx$|^cxs$');

        foreach my $data (@{$all_clients}, @{$all_hypervisors}) {
            if ($data->get_field("NAME") eq $client->{name}) {
                # found it, update the datastructure object
                $client = $data;
                last;
            }
        }

        # first check to see if this client has an initiators field, if it doesn then use the
        # initiators from that rather than dynamically searching the client object.
        my @fc_override_init_list = ();
        my @iscsi_override_init_list = ();
        my $perform_dynamic_lookup = 1;
        if ($client->field_defined("INITIATORS") and ref($client->get_field("INITIATORS")) eq "HASH") {
            my $initiator_hash = $client->get_field("INITIATORS");
            foreach my $key (keys %{$initiator_hash}) {
                if ($igroup_protocol =~ /mixed|fcp/i) {
                    if ($key =~ /WWPN/) {
                        # pull out the initiator value for this wwpn entry
                        # and push it on the init list
                        push(@fc_override_init_list, $initiator_hash->{$key}->{VALUE});
                    }
                } elsif ($igroup_protocol =~ /mixed|iscsi/i) {
                    if ($key =~ /ISCSI_NODENAME/) {
                        # pull out the initiator value for this wwpn entry
                        # and push it on the init list
                        push(@iscsi_override_init_list, $initiator_hash->{$key}->{VALUE});
                    }
                }
            }
            # add the inits to the final initiator list
            push(@initiator_list, @fc_override_init_list, @iscsi_override_init_list);

            # check to see if we are being requested to fall back to the dynamic lookup algorithm
            # in the event that we dont have a wwpn/nodename override for one of the protocols
            $perform_dynamic_lookup = $initiator_hash->{DYNAMIC_LOOKUP};
        }

        # perform a dynamic lookup for initiators if we were requested to
        if ($perform_dynamic_lookup == 1) {
            # search the client dynamically and add any additional initiators that are needed
            # for fcp/iscsi
            my @initiator_objs = $client_obj->get_initiators();
            if ($client_obj->error()) {
                NATE::BaseException->throw("Failed to obtain initators for client " .
                                            $client->get_field("NAME"),
                                            $client_obj->errmsg());
            }

            foreach my $initiator_obj (@initiator_objs) {
                # What protocol is this initiator?
                my $initiator_protocol = $initiator_obj->get_protocol();

                # If it's fcp, get the portname.
                if ($initiator_protocol eq "fcp") {
                    # skip this initiator if the user specified an override fcp init list
                    next if (scalar(@fc_override_init_list));

                    if ($igroup_protocol =~ /mixed|fcp/i) {
                        # store the fcp wwpn if the igroup is fcp/mixed
                        push(@initiator_list, $initiator_obj->get_portname());
                    }
                } elsif ($initiator_protocol eq "iscsi") {
                    # skip this initiator if the user specified an override iscsi init list
                    next if (scalar(@iscsi_override_init_list));

                    # Else if it's iscsi, get the nodename.
                    if ($igroup_protocol =~ /mixed|iscsi/i) {
                        # store the iscsi nodename if the igroup is iscsi/mixed
                        push(@initiator_list, $initiator_obj->get_nodename());
                    }
                } else {
                    # Else unknown protocol, return error
                     NATE::BaseException->throw(
                                    "Initiator " . $client->get_field("NAME") .
                                    " has an interface that " .
                                    "is not using either iSCSI or FCP");
                }
            }
        }

        foreach my $init_name (@initiator_list) {
            $initiator_data->{name} = $init_name;
            $resource_list->add_resource(
                                        category => $top_key,
                                        type     => 'initiator',
                                        data     => Clone::clone($initiator_data)
                                        );
        }
    }
    $Log->exit() if $may_exit;
}

#==============================================================================
# Method:     _load_resource
# Objective:  Load Controller NACL resources into EnGenX
# Inputs:     TBD
# Outputs:    TBD
# Exceptions: TBD
#==============================================================================
sub _load_resource {
    my ($self) = @_;
    $Log->enter() if $may_enter;

    # Check for presence of cDOT/cmode resources in the resource_list
    # Return if none are found, otherwise continue
    my $cluster_resources = $self->{parent}->{resource_list}->get_resources(
                            category    => "^CLUSTER_.",
                            );
    if (scalar(@$cluster_resources) == 0) {
        return $NACL::AppBuilder::EnGenX::TRUE;
    }

    # Go through all resources provided by caller (XML file) and create
    # necessary objects for each to be stored on SanMgr for later use
    my @nacl_nodes;
    my $defined_nodes_ref;
    if (exists $self->{parent}->{nodes}) {
        @nacl_nodes = @{delete $self->{parent}->{nodes}};
    } else {
        # Get all the Nodes that are defined.
        @nacl_nodes = NACL::C::Node->find();
    }

    # Filter only the nodes that are defined in the 'NODES" attribute for each
    # 'CLUSTER' tag in the XML.
    $defined_nodes_ref = $self->_get_defined_nodes(\@nacl_nodes);

    my @nacl_cluster_objs_array;
    if (exists $self->{parent}->{clusters}) {
        @nacl_cluster_objs_array = @{delete $self->{parent}->{clusters}};
    } else {
        # Get all the unique cluster objects from the Nodes that are defined.
        @nacl_cluster_objs_array =
            NACL::STask::Cluster->find_unique_clusters(nodes =>
                $defined_nodes_ref);
    }
    # Need to store the cluster objects in nacl_node because SanMgr libraries
    # may search node instead of cluster for the cluster because of how they
    # currently work. When we stop relying on SanMgr we can remove this.
    foreach my $cluster (@nacl_cluster_objs_array) {
        my $cluster_name = $cluster->name();
        $self->{parent}->{_objects}->{nacl_cluster}->{$cluster_name} = $cluster;
        $self->{parent}->{_objects}->{nacl_node}->{$cluster_name} = $cluster;
        # Adding cluster and node because SanMgr get_obj relies on it existing.
        $self->{parent}->{_objects}->{cluster}->{$cluster_name} = $cluster;
        $self->{parent}->{_objects}->{node}->{$cluster_name} = $cluster;

        # Store the name of the nodes under the corresponding cluster. Using
        # these names, we will populate the cluster->nodes hash with the
        # versioned objects.

        #TODO: When Process.pm Supports NODE versioning, use the following
        #      code to assign the retrieve the name of the nodes that each
        #      cluter hash needs to hold.
=break
        my @nodes = $cluster->nodes();
        $self->{nodes_by_cluster}->{$cluster_name}->{names} = \@nodes;
        $self->{nodes_by_cluster}->{$cluster_name}->{nodes} = [];
=cut
        my @node_objs = $cluster->node_objs();

        # Filter only the nodes that are defined in the 'NODES" attribute for each
        # 'CLUSTER' tag in the XML.
        my $unversioned_node_objs_ref = $self->_get_defined_nodes(\@node_objs);

        $self->{nodes_by_cluster}->{$cluster_name}->{nodes} =
            $unversioned_node_objs_ref;
    }

    # Now store all the nodes that we found earlier in nacl_node. These node
    # objects should already be version by this point.
    foreach my $nacl_node_obj (@nacl_nodes) {
        $self->{parent}->{_objects}->{nacl_node}->{$nacl_node_obj->name()}
            = $nacl_node_obj;
        # Adding node because SanMgr get_obj relies on it existing.
        $self->{parent}->{_objects}->{node}->{$nacl_node_obj->name()}
            = $nacl_node_obj;
        #TODO: When Process.pm Supports NODE versioning, use the following
        #      code to assign the versioned nodes to each cluster hash.
=break
        # Add the node objects to its corresponding cluster
        foreach my $cluster_name ( keys(%{$self->{nodes_by_cluster}}) ){
            my $names_array = $self->{nodes_by_cluster}->{$cluster_name}->{names};
            my $nodes_array = $self->{nodes_by_cluster}->{$cluster_name}->{nodes};
            foreach my $found_name (@{$names_array}){
                if ($node_name eq $found_name){
                    push(@{$nodes_array}, $nacl_node_obj);
                    last;
                }
            }
        }# end foreach cluster
=cut
    }

    # Print a friendly output of what nodes have been assigned to what cluster
    foreach my $cls_nm ( keys(%{$self->{nodes_by_cluster}}) ){
        foreach my $nd ( @{$self->{nodes_by_cluster}->{$cls_nm}->{nodes}} ){
            $Log->comment("NACL::AppBuilder::EnGenX::Controller::_load_resource: "
                          ."FOUND NODES_BY_CLUSTER->[$cls_nm]->[nodes]: "
                          ."NODE[". $nd->{node}."]"
                        );
        }
    }

    $Log->exit() if $may_exit;
    return $NACL::AppBuilder::EnGenX::TRUE;
}

#==============================================================================
# Method:     _cluster_validate
# Objective:  Validate that the cluster has been PREVIOUSLY created.
# Details:    Steps: 1) Validate that the action is Config (no need to see if
#                        the cluster has been created for cleanup)
#                    2) Validate that the provided cluster name is indeed a
#                       configured cluster in the test bed.
#                    3) Add the Cluster key to the dispatch table.
# Inputs:     action. The subroutine the user called.
#             cluster_name. The name of the cluster.
#             dispatch_table. The hash table that holds the 'key' that is the
#                             type of the resource and the subrutine to be
#                             executed.
# Outputs:    Boolean, True if no problem was encountered.
# Exceptions: Arguments exception if the passed arguments are wrong.
#
# TODO: We could configure the cluster here.
#
#==============================================================================
sub _cluster_validate {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;
    #Locals
    my $action = $args{action};
    my $cluster_name = $args{cluster_name};

    # ONLY VERIFY THE CLUSTER IF THE CALLING ACTION IS CONFIG
    if ($action =~ /^setup$/i) {
        # AT EnGenX::New::load_resources A CLUSTER IDENTITIY SHOW WAS PERFORMED
        # AND THE NAME OF ALL THE AVAILABLE CLUSTER WERE SAVED, IF THE CLUSTER
        # HAS ALREADY BEEN CREATED, THE NAME OF THE CLUSTER SHOULD BE FOUND.
        if( !exists($self->{nodes_by_cluster}->{$cluster_name}) ){
            # IF NOT FOUND THE USER COULD HAVE USED THE NAME OF A NODE (FILER)
            # AS A CLUSTER NAME. SanMgr COMPATIBILITY.
            if (defined($self->{parent}->{_objects}->{cluster}->{$cluster_name})) {
                # THROW A WARNING
                my $wrnng = "User specified in the XML file at the cluster tag that "
                        ."the name of the cluster is '$cluster_name' when in "
                        ."reality, that is the name of a NODE or Filer. Please "
                        ."modify the 'THARNHOST' or '.NTEST.TB' and specify the "
                        ."name of the cluster like this: CLUSTER_NAME=name_of_the_cluster "
                        ."and use that variable in the cluster name attribute.";
                # TODO: THROW AN ERROR WHEN NO LONGER SUPPORTED
                $Log->warn("NACL::AppBuilder::EnGenX::Controller::_cluster_validate " . $wrnng);

                # NOW, FIND THE NODES ASSOCIATED TO THIS "CLUSTER NAME". NEED TO
                # LOOK INTO EACH NODE INSIDE OF EACH CLUSTER
                foreach my $cls_nm (keys(%{$self->{nodes_by_cluster}})) {
                    foreach my $nd (@{$self->{nodes_by_cluster}->{$cls_nm}->{nodes}}) {
                        if($nd->{node} eq $cluster_name) {
                            $self->{nodes_by_cluster}->{$cluster_name}->{nodes} =
                                    $self->{nodes_by_cluster}->{$cls_nm}->{nodes};
                            last;
                        }
                    }
                }#foreach cls_nm
            }
            # IF THE CLUSTER NAME IS NOT A NODE NAME, THEN THE CLUSTER IS EITHER
            # NOT CONFIGURED OR NOT CORRECT EITHER WAY, WE NEED TO THROW AN
            # EXCEPTION
            else {
                # TODO: Cluster config call would go here.
                $Log->exit() if $may_exit;
                NATE::BaseException->throw(
                                     "EnGenX couldn't find a clusters with name "
                                    ."'$cluster_name'. At this point EnGenX does "
                                    ."not create the cluster for you.  Until "
                                    ."that functionality is added all clusters "
                                    ."will have to be created prior to setup."
                                );
            }
        }
    }
    # ELSE,
    # TODO: If the action is CleanUp or DeConfig it should not matter if the
    #       cluster is configured or not. But we do need to verify that the
    #       found nodes (if any) are from the specified cluster name.


    $Log->exit() if $may_exit;
    return $NACL::AppBuilder::EnGenX::TRUE;
}

#==============================================================================
# Method:     _get_defined_nodes
# Objective:  Returns an array reference to all found nodes also
# defined in the XML.
# config.
# Inputs:     Array reference of found nodes
# Outputs:    Returns an array reference to all found nodes also defined in XML
# Exceptions: TBD
#==============================================================================
sub _get_defined_nodes(){
    my ($self, $found_nodes) = @_;
    $Log->enter() if $may_enter;

    my @all_defined_nodes;

    my $resource_list = $self->{parent}->{resource_list};

    my $nodes = $resource_list->get_resources(category => qr/^cluster_.*/i,
                                              type     => qr/^node$/i);

    # Get all specified nodenames for each CLUSTER tag in the config.
    my %all_node_names;

    if (scalar(@$nodes)) {
        foreach my $node (@$nodes) {
            my $node_name = $node->get_field('name');
            $all_node_names{$node_name} = '';
        }
    } else {
        # Get the hash of clusters from the XML.
        my $xml_clusters = &ParamsMgr::search(
            -search_path => [],
            -key         => "CLUSTERS",
            -key_filter  => qr/^CLUSTER\d*$/,
            paramset => $self->NACL::AppBuilder::EnGenX::Util::get_paramset()
        );

        if (&ParamsMgr::error()) {
            $self->_set_err(&ParamsMgr::get_error());
            $Log->exit() if $may_exit;
            return (wantarray()) ? () : [];
        }


        foreach my $cluster (values %{$xml_clusters}) {

            if(defined $cluster->{NODES}) {
                # Split returned string from CLUSTER tag's NODES attribute
                # into an array.
                my @cluster_node_names_array = split(',', $cluster->{NODES});

                # Store nodenames into a hash. This will ensure duplicate nodes
                # are not passed.
                foreach my $nodename (@cluster_node_names_array){
                    $all_node_names{ $nodename } = '';
                }
            } else{
                # Log an error that no nodes were found in the cluster tag
                $Log->comment("Error: No nodes listed for the CLUSTER tag.");
            }
        }
    }

    # Get an array reference for all found nodes that are
    # defined in the config.
    my $defined_nodes_ref = $self->_filter_matching_nodes(
        $found_nodes,
        \%all_node_names);

    # Add cmode nodes to the array of all defined Nodes.
    push(@all_defined_nodes, @$defined_nodes_ref);

    $Log->exit() if $may_exit;
    return \@all_defined_nodes;
}


#==============================================================================
# Method:     _filter_matching_nodes
# Objective:  Filters and returns all found nodes that are defined in the
# passed hash.
# Inputs:     Array reference of found nodes, Hash reference to XML nodes
# Outputs:    Array reference of found nodes that are also defined in the XML
# Exceptions: TBD
#==============================================================================
sub _filter_matching_nodes(){
    my($self, $found_nodes, $xml_node_names_hash) = @_;
    $Log->enter() if $may_enter;

    my @defined_nodes;
    #my $name_return;
    # Create an array of found nodes defined in the XML. By looping through the
    # Hash of node names retrieved from the XML and comparing node names to
    # the elements in the '$found_nodes' array.
    foreach my $nodename (keys %$xml_node_names_hash) {
        foreach my $node (@$found_nodes) {
            if($node->{node} eq $nodename) {
                push(@defined_nodes, $node);
            }
        }
    }
    $Log->exit() if $may_exit;
    return \@defined_nodes;
}

=head2 xml_to_list

Translate the XML stored by ParamsMgr into a ResourceList

    $Egx_obj->xml_to_list();

=over

=item Options

None

=item Return

Nothing, resource_list attribure is populated

=item Exceptions

BaseException

=back

=cut

###############################################################################
sub xml_to_list {
    my ($self) = @_;
    $Log->enter() if $may_enter;

    $Log->comment("Translating Controller XML to Resource List");

    # Get the actions for the environment and save them (may have override)
    my $actions = &ParamsMgr::search(search_path => [], key => "ACTIONS",
                                     paramset => $self->{parent}->NACL::AppBuilder::EnGenX::Util::get_paramset());
    if (&ParamsMgr::error()) {
        $Log->exit() if $may_exit;
        my $errmsg = &ParamsMgr::errmsg();
        NATE::BaseException->throw("Failed to find ACTIONS param: $errmsg");
    }

    # Set action variables.  The defaults are for completeness sake, the
    # defaults set in the DTD file should be in effect before we get here
    my $action_setup    = defined($actions->{SETUP})    ? $actions->{SETUP}    : 1;
    my $action_cleanup  = defined($actions->{CLEANUP})  ? $actions->{CLEANUP}  : 1;
    my $cmd_interface = "zapi";

    # Get the policies for the environment and save them (may have override)
    my $policies
        = &ParamsMgr::search(search_path => [], key => "REUSE_POLICY",
                             paramset => $self->{parent}->NACL::AppBuilder::EnGenX::Util::get_paramset());
    if (&ParamsMgr::error()) {
        $Log->exit() if $may_exit;
        my $errmsg = &ParamsMgr::errmsg();
        NATE::BaseException->throw(
            "Failed to find REUSE_POLICY param: $errmsg");
    }

    my $reuse_policy = ($policies->{SETUP}) ? $policies->{SETUP} : "reuse";
    my $cleanup_policy = ($policies->{CLEANUP}) ? $policies->{CLEANUP} : 0;

    # Get the environment global options
    $self->_retrieve_iscsi_global_options();

    # Get the information in the CLUSTERS tag.  This should be any CLUSTER
    # tags, setup, config, deconfig, etc.
    my $clusters = &ParamsMgr::search(
        -search_path => [],
        -key         => "CLUSTERS",
        paramset => $self->{parent}->NACL::AppBuilder::EnGenX::Util::get_paramset()
    );
    if (&ParamsMgr::error()) {
        $Log->exit() if $may_exit;
        my $errmsg = &ParamsMgr::errmsg();
        NATE::BaseException->throw("Failed to find CLUSTERS param: $errmsg");
    }

    # We default these to what is set in the actions and reuse_policy tags.
    # Then we check to see if the clusters tag has overrides.  As we process
    # each cluster, we check them there as well.

    my $data_hash = {
        _cleanup_policy => $cleanup_policy,
        _reuse_policy   => $reuse_policy,
        _parallel       => 0,
        _interface      => $cmd_interface,
        _load_balance   => 0,
        _setup          => $action_setup,
        _cleanup        => $action_cleanup,
    };

    $data_hash = $self->_get_current_action(
                    cluster_hash => $clusters,
                    data_hash => $data_hash,
                    label    => "CLUSTERS"
                );

    # We are going to store the clusters attributes in the ResourceList.
    # Look at each key, skip the hashes and store the data.
    my $clusters_attributes = {};
    while (my ($clusters_key, $clusters_value) = each(%$clusters)) {
        next if (ref($clusters_value));
        $clusters_attributes->{$clusters_key} = $clusters_value;
    }

    $self->{parent}->{resource_list}->add_resource(
        category => "CLUSTERS",
        type     => "attributes",
        data     => $clusters_attributes,
    );

    # When we walk the XML we send in 3 hash references
    #   - data_hash - EnGenX data
    #   - pkey_hash - Primary Keys (cluster, vserver, etc)
    #   - walk_hash - the current hash to walk (cluster at this level)
    my $pkey_hash = {};

    # Now go through the CLUSTERS keys and process each CLUSTER tag
    while (my ($clusters_key, $clusters_value) = each(%$clusters)) {
        # If the value is not a HASH then this is an attribute. skip attributes.
        next if (ref($clusters_value) ne 'HASH');
        # Need to wait till all clusters are processed before processing
        # Vserver Peer or Snapmirror relationships.
        next if ($clusters_key =~ /^(VSERVER_PEER|SNAPMIRROR)\d*$/);
        $self->_xml_walk(
            $clusters_key, $clusters_key, $clusters_value,
            $data_hash,    $pkey_hash
        );
    }    # end while

    # Now go through the CLUSTERS keys and process each Vserver Peer tag
    # and Snapmirror tag.
    while (my ($clusters_key, $clusters_value) = each(%$clusters)) {
        # If the value is not a HASH then this is an attribute. skip attributes.
        next if (ref($clusters_value) ne 'HASH');
        # Need to only process
        # Vserver Peer and snapmirror relationships.
        next if ($clusters_key !~ /^(VSERVER_PEER|SNAPMIRROR)\d*$/);
        $self->_xml_walk(
            $clusters_key, $clusters_key, $clusters_value,
            $data_hash,    $pkey_hash
        );
    }    # end while
    $Log->comment("Completed building the Controller side Resource List");
    $Log->exit() if $may_exit;
} ## end sub xml_to_list

###############################################################################
# Method:    _xml_walk
# Objective: Walk thorugh the hash from the XML and create Resources
# Inputs:    top level key, current key, current hash,
#            data from parent, primary_keys hash
# Outputs:   Returns nothing, ResourceList populated
#
###############################################################################
sub _xml_walk() {
    my ($self, $category, $current_key, $walk_hash, $data_hash, $pkey_hash)
        = @_;

    if ($Log->may_debug()) {
        $Log->debug("Category is $category, Current key is $current_key"
                . "\nDumper of $current_key walk_hash: "
                . Dumper($walk_hash));
        $Log->debug(
            "Dumper of $current_key data_hash: " . Dumper($data_hash));
    }

    # If the current key is "VOL" or "AGGR" change to "VOLUME", "AGGREGATE"
    # Need to account for trailing numbers, since ParamsManager adds them when
    # it encounters more than one cluster defined in the xml.
    if ($category =~ /^CLUSTER($|\d*)/i and $current_key =~ /^VOL(\d*)$/i) {
        $current_key = "VOLUME$1";
    }
    if ($category =~ /^CLUSTER($|\d*)/i and $current_key =~ /^AGGR(\d*)/i) {
        $current_key = "AGGREGATE$1";
    }

    # Set the EnGenX data based on either the data_hash or the walk hash
    # or a default.  The defaults match what's in our DTD files.
    # The values for current_cluster and current_vserver are set
    # after we generate copies because they depend on the output from
    # generate_copies().
    #
    # Policies
    my $current_cleanup_policy
        = $self->_get_parent_value("CLEANUP_POLICY", "_cleanup_policy",
        $walk_hash, $data_hash, "new");
    my $current_reuse
        = $self->_get_parent_value("REUSE", "_reuse_policy", $walk_hash,
        $data_hash, "reuse");
    my $current_parallel
        = $self->_get_parent_value("PARALLEL", "_parallel", $walk_hash,
        $data_hash, 0);
    my $current_cmd_interface
        = $self->_get_parent_value("CMD_INTERFACE", "_interface", $walk_hash,
        $data_hash, "zapi");
    my $current_load_balance
        = $self->_get_parent_value("LOAD_BALANCE_PROVISIONING",
        "_load_balance", $walk_hash, $data_hash, 0);

    # Actions
    my $current_setup
        = $self->_get_parent_value("SETUP", "_setup", $walk_hash, $data_hash,
        1);
    my $current_cleanup
        = $self->_get_parent_value("CLEANUP", "_cleanup", $walk_hash,
        $data_hash, 1);

    # Grab a host typ to be used for the add resource call
    my $hash_type = $current_key;
    $hash_type =~ s/(\D*)(\d+)/$1/;    # remove any trailing digits LUN2->LUN

    # Use generate_copies to create multiple XML entries for resources
    # with a count > 0
    my @element_array = ();
    push(
        @element_array,
        $self->_generate_copies(
            element => $walk_hash,
            'type'  => $hash_type,
        )
    );

    # Now that we have an array of copies (can be just one), we
    # process each one (depth first recursion, etc
    foreach my $element (@element_array) {

        # Make a copy of the pkey_hash to send to the next level
        # since we are passing around referecnes to hashes, if we
        # don't create a clone, we could affect this level when we edit
        # it at the next level.
        my $primary = Clone::clone($pkey_hash);

        my $pkey = $current_key;

        # Strip the digits off the end of the key
        if ($pkey =~ /(\D*)(\d+)/) {
            $pkey = $1;
        }
        $pkey = '_CLUSTER' if($pkey eq 'CLUSTER');
        # If element contains a NAME field, add it to the primary key hash
        if (defined $walk_hash->{NAME}) {
            if($element->{PREPEND_VSERVER} and defined($primary->{VSERVER})) {
                delete $element->{PREPEND_VSERVER};
                my $name = '' . $primary->{VSERVER} . '_' . $element->{NAME};

                $element->{NAME} = $name;
            }
            my $name = $element->{NAME};
            $primary->{$pkey} = $name;
        }
        $primary->{_protocol} = $walk_hash->{PROTOCOL} if ($pkey eq 'IGROUP');
        $primary->{_subtype}  = $walk_hash->{SUBTYPE} if ($pkey eq 'VSERVER');
        $primary->{policy_enabled} = $walk_hash->{POLICY_ENABLED} if ($pkey eq 'QUOTA_POLICY');

        # Pass the username and pass for the cifs service to its children
        $primary->{'cifs_service_username'} = $walk_hash->{USERNAME} if ($pkey eq 'CIFS_SERVICE');
        $primary->{'cifs_service_password'} = $walk_hash->{PASSWORD} if ($pkey eq 'CIFS_SERVICE');
        $primary->{'cifs_service_domain'} = $walk_hash->{DOMAIN} if ($pkey eq 'CIFS_SERVICE');
        $primary->{snapmirror_policy_policy} = $walk_hash->{POLICY} if ($pkey eq 'SNAPMIRROR_POLICY');
        $primary->{'group-name'} = $walk_hash->{'GROUP-NAME'} if ($pkey eq 'CIFS_LOCAL_GROUP');

        # Pass the ntfs security descriptor to all file-directory policy children
        $primary->{'ntfs-sd'} = $walk_hash->{'NTFS-SD'} if ($pkey eq 'FILE_DIRECTORY_POLICY');

        # Process each key in the elements hash. Skip scalars and for each
        # HASH that we encounter, create the data hash needed by this method
        # and recursively call _xml_walk() to process the key.
        while (my ($key, $value) = each(%$element)) {
            next if (ref($value) ne 'HASH');
            # Skip CG ITEM MAPPINGS as that needs to be passed through as a hash.
            next if ($key =~ /^CG-ITEM-MAPPINGS$/);
            next if ($key =~ /^IPM$/i);
            # Skip CG ITEM as that is taken care of in generate copies.
            next if ($key =~ /^CG_ITEM\d*$/);

            my $element_data = {
                _cleanup_policy => $current_cleanup_policy,
                _reuse_policy   => $current_reuse,
                _parallel       => $current_parallel,
                _interface      => $current_cmd_interface,
                _load_balance   => $current_load_balance,
                _setup          => $current_setup,
                _cleanup        => $current_cleanup,
            };

            # We clone $primary, so that siblings can not modify each others primary keys.
            $self->_xml_walk($category,
                             $key,
                             $value,
                             $element_data,
                             Clone::clone($primary));
        }    # end while

        # When we get here we have gone through all of our curent hash keys
        # and have drilled down into any children.  We now get our push
        # data together and push/add ourselves as a resource.

        # Go through the hash pull out the data that we will store
        my $push_data = {};
        while (my ($data_key, $data_value) = each(%$element)) {
            # Skip anything that's not a scalar
            if (ref($data_value)){
                if ($hash_type =~ /LIF/i){
                    # IP_RANGE has been transformed to an ARRAY REF that needs
                    # to be added to it's corresponding resource obj.
                    if ($data_key =~/ips/i){
                        $push_data->{$data_key} = $data_value;
                    }
                    # DATA-PROTOCOL should be an array ref, so we must assigne
                    # it to the corresponding key
                    elsif($data_key =~/DATA-PROTOCOL/i){
                        $push_data->{$data_key} = $data_value;
                    } elsif ($data_key =~/^IPM$/i){
                        $push_data->{$data_key} = $data_value;
                    }
                } elsif ($hash_type =~ /^PORTSET\d*$/i) {
                    if ($data_key =~ /^igroups$/i) {
                        $push_data->{$data_key} = $data_value;
                    }
                } elsif ($hash_type =~ /^VOLUME$/i) {
                    if ($data_key =~ /^data-aggr-list$/i) {
                        $push_data->{$data_key} = $data_value;
                    }
                    elsif ($data_key =~ /^aggr-list$/i) {
                        $push_data->{$data_key} = $data_value;
                    }
                    elsif ($data_key =~ /^candidate_aggregates$/i) {
                        $push_data->{$data_key} = $data_value;
                    }
                } elsif ($hash_type =~ /^VSERVER$/i) {
                    if ($data_key =~ /^aggr-list$/i) {
                        $push_data->{$data_key} = $data_value;
                    }
                } elsif ($hash_type =~ /^SNAPMIRROR$/i) {
                    if ($data_key =~ /^cg-item-mappings$/i) {
                        $push_data->{$data_key} = $data_value;
                    }
                }
                next;
            }

            # Skip the stuff stored in the $current_ variables, added later
            # This also Skips things that we do not care about such as config,deconfig,validate,precleanup.
            # We only dont care about the above for controller so this may change when other things are added
            next if ($data_key =~ /^(CLEANUP_POLICY|REUSE|PARALLEL|CMD_INTERFACE|LOAD_BALANCE_PROVISIONING|SETUP|CLEANUP|CONFIG|DECONFIG|VALIDATE|PRECLEANUP)$/);
            if ($data_key eq "AGGR") {
                $data_key = "AGGREGATE";
            }
            $push_data->{$data_key} = $data_value;
        }

        # Add the primary keys to the data to be stored
        while (my ($data_key, $data_value) = each(%$pkey_hash)) {
            $push_data->{$data_key} = $data_value;
        }

        # If the lif doesn't have a vserver then we assign the cluster_name
        # as the vserver because the lif was created under the cluster
        # tag. The same for network_routes
        # The ips_key will be used to search for the next available IP
        # Address when creating the lif. It needs to be unique, so
        # concatenate the vserver and the lif name.
        if ($hash_type =~/LIF|NETWORK_ROUTE/i) {
            unless (exists($push_data->{VSERVER})) {
                $push_data->{VSERVER} = $push_data->{_CLUSTER};
            }
        }

        if ($hash_type =~ /^VOLUME$/i and exists($push_data->{EXPORT_POLICY})) {
            if($push_data->{PREPEND_VSERVER_TO_EXPORT_POLICY}) {
                delete $push_data->{PREPEND_VSERVER_TO_EXPORT_POLICY};
                my $vserver_name = $push_data->{VSERVER};
                my $replace = $vserver_name . '_';
                $push_data->{'EXPORT_POLICY'} = $replace . $push_data->{'EXPORT_POLICY'};
            }
        }

        if ($hash_type =~ /^LUN_MAP$/i) {
            if($push_data->{PREPEND_VSERVER_TO_IGROUP}) {
                delete $push_data->{PREPEND_VSERVER_TO_IGROUP};
                my $vserver_name = $push_data->{VSERVER};
                my $replace = $vserver_name . '_';
                $push_data->{'IGROUP'} = $replace . $push_data->{'IGROUP'};
            }
        }

        if ($hash_type =~ /^LUN$/i) {
            if($push_data->{PREPEND_VSERVER_TO_IGROUP}) {
                delete $push_data->{PREPEND_VSERVER_TO_IGROUP};
                my $vserver_name = $push_data->{VSERVER};
                #prepend vserver name after every comma
                my $replace = ',' . $vserver_name . '_';
                $push_data->{'MAP_LIST'} =~ s/,/$replace/g;
                #prepend vserver name to first in list.
                $replace = $vserver_name . '_';
                $push_data->{'MAP_LIST'} = $replace . $push_data->{'MAP_LIST'};
            }
        }

        if ($hash_type =~ /PORTSET/i) {
            if($push_data->{PREPEND_VSERVER_TO_PORT_NAMES}) {
                delete $push_data->{PREPEND_VSERVER_TO_PORT_NAMES};
                my $vserver_name = $push_data->{VSERVER};
                #prepend vserver name after every comma
                my $replace = ',' . $vserver_name . '_';
                $push_data->{'PORT-NAME'} =~ s/,/$replace/g;
                #prepend vserver name to first in list.
                $replace = $vserver_name . '_';
                $push_data->{'PORT-NAME'} = $replace . $push_data->{'PORT-NAME'};
            }
        }

        if ($current_reuse =~ /check|fail/i) {
            $current_reuse = "die";
        }

        # Replace the EnGenX fields in the element data with
        # what we determined the current values were.
        $push_data->{_cleanup_policy}  = $current_cleanup_policy;
        $push_data->{if_exists}        = $current_reuse;
        $push_data->{_parallel}        = $current_parallel;
        $push_data->{_interface}       = $current_cmd_interface;
        $push_data->{_load_balance}    = $current_load_balance;
        $push_data->{_setup}           = $current_setup,
        $push_data->{_cleanup}         = $current_cleanup,
        $push_data->{_setup_done}      = 0;
        $push_data->{_cleanup_done}    = 0;

        # When you add more than one cluster or multiple clients, you'll need
        # the category to be CLUSTER_<NAME> or CLIENT_<NAME>
        my $cat_to_add = $category;
        # Remove any trailing numbers
        $cat_to_add =~ s/(\D*)(\d+)/$1/;
        # Assign the catergory either as a key or an attribute
        if ($hash_type =~ /^CLUSTER$/) {
            $cat_to_add .= "_" . $walk_hash->{NAME};
        } elsif($hash_type =~ /^vserver_peer$/i) {
            #This will be fully set later.
            $cat_to_add = 'CLUSTER_';
            # Set _cluster incase home_cluster and this is not based on an engenx_relationship_id
            # Set this only if home_cluster exists
            $push_data->{_cluster} = $push_data->{HOME_CLUSTER} if exists $push_data->{HOME_CLUSTER};
        } elsif($hash_type =~ /^snapmirror$/i) {
            #This will be fully set later.
            $cat_to_add = 'CLUSTER_';
            # Set _cluster incase destination-cluster and this is not based on an engenx_relationship_id
            # Set this only if destination-cluster exists
            $push_data->{_cluster} = $push_data->{'DESTINATION-CLUSTER'} if exists $push_data->{'DESTINATION-CLUSTER'};
            # Need to remove destination-cluster if destination-path is passed in.
            delete $push_data->{'DESTINATION-CLUSTER'} if exists $push_data->{'DESTINATION-PATH'};
        }
        else {
            $cat_to_add .= "_" . $push_data->{_CLUSTER};
        }

        $push_data->{_parallel} = 5 if ($push_data->{_parallel} =~ /max/i);

        # Generate special case resources before adding the resource so that
        # if there are special cases then the special case fields can be
        # removed before adding the resource.
        $self->_generate_special_case_resources(
            $cat_to_add,
            $hash_type,
            $push_data,
            $pkey_hash
        );

        foreach my $key (keys(%$push_data)) {
            $push_data->{lc($key)} = delete $push_data->{$key};
        }
        if($hash_type =~ /^cluster_peer/i) {
            $push_data->{cluster} = $push_data->{_cluster};
        }
        if($hash_type =~ /^(snapmirror)$/i) {
            if (not defined ($push_data->{_cluster}) or
                $push_data->{_cluster} eq '' or
                $push_data->{_cluster} eq 'CLUSTER_') {
                             NATE::BaseException->throw("Cluster to run the snapmirror command from is unknown.\n".
                                           "Either include destination-cluster or use engenx_relationship_id functionality\n" . Dumper($push_data));
            }
            $cat_to_add .= $push_data->{_cluster}
        }
        if($hash_type =~ /^(vserver_peer)$/i) {
            if (not defined ($push_data->{_cluster}) or
                $push_data->{_cluster} eq '' or
                $push_data->{_cluster} eq 'CLUSTER_') {
                             NATE::BaseException->throw("Cluster to run the vserver_peer command from is unknown.\n".
                                           "Either include home_cluster or use engenx_relationship_id functionality\n" . Dumper($push_data));
            }
            $cat_to_add .= $push_data->{_cluster}
        }
        $self->{parent}->{resource_list}->add_resource(
            category => $cat_to_add,
            type     => lc($hash_type),
            data     => $push_data
        );

    }    # end foreach element

    if($hash_type =~ /^vserver/i) {
        delete $self->{parent}->{EFFICIENCY_POLICY_NAME};
    }

    if (defined $self->{parent}->{rgname})
    {
        my $cbdata = $self->{parent}->{resource_list}->get_resources( category => "^CLUSTER", type => lc($hash_type) );
        $Log->debug("Adding type $hash_type to configbroker\n");
        $Log->debug(Dumper($cbdata));
        $self->{parent}->{rgh}->add_resources(resource_type => lc($hash_type), resourcearrayref => $cbdata) if (defined $self->{parent}->{rgname});
    }

    if ($Log->may_debug()) {
        $Log->debug("Out Dumper of $current_key walk_hash: "
                . Dumper($walk_hash)
                . "\nOut Dumper of $current_key data_hash: "
                . Dumper($data_hash)
                . "\nOut Dumper of resource list: "
                . Dumper($self->{parent}->{resource_list}));
    }

} ## end sub _xml_walk()

###############################################################################
# Method:    _generate_special_case_resources
# Objective: Calls methods to take care of special resource cases.
# Inputs:    $resource_list - reference to a Resource List
#            $category - the category of the resource to generate special cases
#            $type - the type of the resource containing the map list
#            $resource_data - the data for the resource to generate special cases
#            $resource_pkey_hash - the primary key hash for the resource
#                                  to generate special cases
# Returns:   none
###############################################################################
sub _generate_special_case_resources() {
    my ($self,$category,$type,$resource_data,$resource_pkey_hash) = @_;

    if($type =~ /^LUN$/i) {
        if (exists $resource_data->{MAP_LIST}) {
            $self->_expand_map_list_to_resources(
                $category,
                $type,
                $resource_data,
                $resource_pkey_hash
            );
        }
        if(exists $resource_data->{BIND_LIST}) {
            $self->_expand_bind_list_to_resources(
                $category,
                $type,
                $resource_data,
                $resource_pkey_hash
            );
        }
        # Policy groups cannot start with _ so _default
        # is the indicator that this should be
        # changed to the primary keys generated name.
        if (exists $resource_data->{'QOS-POLICY-GROUP'} and
            $resource_data->{'QOS-POLICY-GROUP'} =~ /^_default/i) {
            $self->_build_policy_group_name(
                $category,
                $type,
                $resource_data,
                $resource_pkey_hash,
                FALSE
            );
        }
    } elsif($type =~ /^VSERVER_PEER$/i and exists $resource_data->{ENGENX_RELATIONSHIP_ID}) {
        $self->_gather_vserver_peer_args_from_relationship_id(
            $category,
            $type,
            $resource_data,
            $resource_pkey_hash
        );
    } elsif($type =~ /^SNAPMIRROR$/i) {
        $self->_gather_snapmirror_args_from_relationship_id(
            $category,
            $type,
            $resource_data,
            $resource_pkey_hash
        );
    } elsif($type =~ /^IGROUP$/i and exists $resource_data->{INITIATOR_LIST}) {
        $self->_expand_initiator_list_to_resources(
            $category,
            $type,
            $resource_data,
            $resource_pkey_hash
        );
    } elsif($type =~ /^Aggregate$/i and $resource_data->{WAFLIRON} == 1) {
        $self->_expand_wafliron_to_resource(
            $category,
            $type,
            $resource_data,
            $resource_pkey_hash
        );
    } elsif($type =~ /^FAILOVER_GROUP$/i) {
        $self->_add_failover_group_resources_for_extra_ports(
            $category,
            $type,
            $resource_data,
            $resource_pkey_hash
        );
    } elsif ($type =~ /^VSERVER$/i) {
        if( exists $resource_data->{HOMEDIR_SEARCH_PATH_LIST}) {
            $self->_expand_search_paths_to_resources(
                $category,
                $type,
                $resource_data,
                $resource_pkey_hash
            );
        }
        # Policy groups cannot start with _ so _default
        # is the indicator that this should be
        # changed to the primary keys generated name.
        if (exists $resource_data->{'QOS-POLICY-GROUP'} and
            $resource_data->{'QOS-POLICY-GROUP'} =~ /^_default/i) {
            $self->_build_policy_group_name(
                $category,
                $type,
                $resource_data,
                $resource_pkey_hash,
                FALSE
            );
        }
        if (exists $resource_data->{USE_DEEP_JUNCTION}) {
            $self->_sort_volumes_by_junction_path(
                $category,
                $type,
                $resource_data,
                $resource_pkey_hash
            );
        }
    } elsif ($type =~ /^VOLUME$/i) {
        # Policy groups cannot start with _ so _default
        # is the indicator that this should be
        # changed to the primary keys generated name.
        if (exists $resource_data->{'QOS-POLICY-GROUP'} and
            $resource_data->{'QOS-POLICY-GROUP'} =~ /^_default/i) {
            $self->_build_policy_group_name(
                $category,
                $type,
                $resource_data,
                $resource_pkey_hash,
                FALSE
            );
        }
    # Policy groups cannot start with _ so _default
    # is the indicator that this should be
    # changed to the primary keys generated name.
    } elsif ($type =~ /^QOS_POLICY_GROUP$/i and $resource_data->{NAME} =~ /^_default/i) {
        $self->_build_policy_group_name(
            $category,
            $type,
            $resource_data,
            $resource_pkey_hash,
            TRUE
        );
    } elsif ($type =~ /^QUOTA_POLICY$/i and $resource_data->{POLICY_ENABLED} =~ /^1$/) {
        $self->_sort_volumes_by_quota_settings(
            $category,
            $type,
            $resource_data,
            $resource_pkey_hash
        );
    } elsif ($type =~ /^FPOLICY$/i) {
        $self->_expand_fpolicy_enable_to_resource(
            $category,
            $type,
            $resource_data,
            $resource_pkey_hash
        );
    } elsif ($type =~ /^FILE_DIRECTORY_POLICY$/i) {
        $self->_expand_file_directory_policy_to_resource(
            $category,
            $type,
            $resource_data,
            $resource_pkey_hash
        );
    } elsif ($type =~ /^EXTERNAL_ENGINE$/i) {
        $self->_expand_external_engine_connection_to_resource(
            $category,
            $type,
            $resource_data,
            $resource_pkey_hash
        );
    } elsif ($type =~ /^EFFICIENCY_OPTIONS/i and exists $resource_data->{policy}){
        $self->_make_efficiency_policy(
            $category,
            $type,
            $resource_data,
            $resource_pkey_hash
        );
    } elsif ($type =~ /^CIFS_LOCAL_USER/i and exists $resource_data->{PRIVILEGES}){
        $self->_make_cifs_local_user_privilege(
            $category,
            $type,
            $resource_data,
            $resource_pkey_hash
        );
    } elsif ($type =~ /^CIFS_LOCAL_GROUP/i){
        if(exists $resource_data->{PRIVILEGES}){
            $self->_make_cifs_local_group_privilege(
                $category,
                $type,
                $resource_data,
                $resource_pkey_hash
            );
        }
        if (exists $resource_data->{'MEMBER-NAMES'}){
            $self->_make_cifs_local_group_membership(
                $category,
                $type,
                $resource_data,
                $resource_pkey_hash
            );
        }
    }

    return;
} ## end sub _generate_special_case_resources()

#==============================================================================
# Method:     _gather_snapmirror_args_from_relationship_id
# Objective:  Gather Snapmirror args based on the relationship id.
# Details:    Gather Snapmirror args based on the relationship id.
# Inputs:     %args from create or delete
# Outputs:    The created name
# Exceptions: Node
#==============================================================================
sub _gather_snapmirror_args_from_relationship_id {
    my ($self,$category,$type,$resource_data,$resource_pkey_hash,$is_qos_policy_group) = @_;
    $Log->enter() if $may_enter;

    if(exists $resource_data->{ENGENX_RELATIONSHIP_ID}) {
        # get the list of vservers that have same relationship id.
        # from the resource list
        my @vservers = $self->{parent}->{resource_list}->get_resources(
            category => "^CLUSTER",
            type     => '^vserver$',
            data     => {engenx_relationship_id => '^' . $resource_data->{ENGENX_RELATIONSHIP_ID} . '$'}
        );

        if (scalar @vservers > 1) {
            my $found_source = 0;
            my $found_destination = 0;
            foreach my $vserver (@vservers) {
                if($vserver->{engenx_relationship_type} eq 'source') {
                    $found_source = 1;
                    $resource_data->{'source-vserver'} = exists ($vserver->{name}) ? $vserver->{name} : $vserver->{vserver};
                    $resource_data->{'source-cluster'} = $vserver->{_cluster};
                } elsif($vserver->{engenx_relationship_type} eq 'destination') {
                    $found_destination = 1;
                    $resource_data->{'destination-vserver'} = exists ($vserver->{name}) ? $vserver->{name} : $vserver->{vserver};
                    $resource_data->{'destination-cluster'} = $vserver->{_cluster};
                    $resource_data->{'_cluster'} = $vserver->{_cluster};
                } else {
                    NATE::BaseException->throw("Snapmirror was defined with an engenx_relationship_id " .
                                   "but the vserver found with the same relationship id did not contain " .
                                   "a recognized engenx_relationship_type of source or destination.\n" . Dumper(\@vservers));
                }
                last if ($found_source and $found_destination);
            }
            unless ($found_source and $found_destination) {
                NATE::BaseException->throw("Snapmirror was defined with an engenx_relationship_id " .
                   "but did not find both a source and destination vserver with the same relationship id " .
                   "even with > 1 vserver found with the same relationship id.\n" . Dumper(\@vservers));
            }
        } else {
            # get the list of volumes that have same relationship id.
            # from the resource list
            my @volumes = $self->{parent}->{resource_list}->get_resources(
                category => "^CLUSTER",
                type     => '^volume$',
                data     => {engenx_relationship_id => '^' . $resource_data->{ENGENX_RELATIONSHIP_ID} . '$'}
            );

            if (scalar @volumes > 1) {
                my $found_source = 0;
                my $found_destination = 0;
                foreach my $volume (@volumes) {
                    if($volume->{engenx_relationship_type} eq 'source') {
                        $found_source = 1;
                        $resource_data->{'source-volume'} = exists ($volume->{name}) ? $volume->{name} : $volume->{volume};
                        $resource_data->{'source-vserver'} = $volume->{vserver};
                        $resource_data->{'source-cluster'} = $volume->{_cluster};
                    } elsif($volume->{engenx_relationship_type} eq 'destination') {
                        $found_destination = 1;
                        $resource_data->{'destination-volume'} = exists ($volume->{name}) ? $volume->{name} : $volume->{volume};
                        $resource_data->{'destination-vserver'} = $volume->{vserver};
                        $resource_data->{'destination-cluster'} = $volume->{_cluster};
                        $resource_data->{'_cluster'} = $volume->{_cluster};
                    } else {
                        NATE::BaseException->throw("Snapmirror was defined with an engenx_relationship_id " .
                                       "but the volume found with the same relationship id did not contain " .
                                       "a recognized engenx_relationship_type of source or destination.\n" . Dumper(\@volumes));
                    }
                    last if ($found_source and $found_destination);
                }
                unless ($found_source and $found_destination) {
                    NATE::BaseException->throw("Snapmirror was defined with an engenx_relationship_id " .
                       "but did not find both a source and destination volume with the same relationship id " .
                       "even with > 1 volume found with the same relationship id.\n" . Dumper(\@volumes));
                }
            } else {
                NATE::BaseException->throw("Snapmirror was defined with an engenx_relationship_id " .
                    "but was unable to find 2 vservers or 2 volumes with the same engenx_relationship_id");
            }
        }
    }

    # Check if this is the lun/file level, volume level or vserver level snapmirror.
    # The _snapmirror_object_level is used to determine at which level the snapmirror is being created.
    # This will be used to determine when the snapmirror can be cleaned up.
    if(exists $resource_data->{'cg-item-mappings'} or
       exists $resource_data->{'CG-ITEM-MAPPINGS'}) {
        $resource_data->{'_snapmirror_object_level'} = 'lun';
    } elsif(exists $resource_data->{'destination-volume'} or
       exists $resource_data->{'DESTINATION-VOLUME'} or
       (exists $resource_data->{'destination-path'} and
       $resource_data->{'destination-path'} =~ /^((\S+):(\S+))|((\S+):\/\/(\S+)\/(\S+))$/) or
       (exists $resource_data->{'DESTINATION-PATH'} and
       $resource_data->{'DESTINATION-PATH'} =~ /^((\S+):(\S+))|((\S+):\/\/(\S+)\/(\S+))$/)) {
        $resource_data->{'_snapmirror_object_level'} = 'volume';
    } else {
       $resource_data->{'_snapmirror_object_level'} = 'vserver';
    }

    if(delete $resource_data->{INITIALIZE}) {
        # Add a snapmirror_initialize resource for the snapmirror.
        $self->{parent}->{resource_list}->add_resource(category => $category . $resource_data->{'_cluster'},
                                             type     => 'snapmirror_initialize',
                                             data     => Clone::clone($resource_data));
    }
    $Log->exit() if $may_exit;
    return;
}

#==============================================================================
# Method:     _gather_vserver_peer_args_from_relationship_id
# Objective:  Gather Vserver Peer args based on the relationship id.
# Details:    Gather Vserver Peer args based on the relationship id.
# Inputs:     %args from create or delete
# Outputs:    The created name
# Exceptions: Node
#==============================================================================
sub _gather_vserver_peer_args_from_relationship_id {
    my ($self,$category,$type,$resource_data,$resource_pkey_hash,$is_qos_policy_group) = @_;
    $Log->enter() if $may_enter;

    # get the list of vservers that have same relationship id.
    # from the resource list
    my $vservers_ref = $self->{parent}->{resource_list}->get_resources(
        category => "^CLUSTER",
        type     => '^vserver$',
        data     => {engenx_relationship_id => '^' . $resource_data->{ENGENX_RELATIONSHIP_ID} . '$'}
    );

    my $first_vserver = pop @$vservers_ref;
    my $second_vserver = pop @$vservers_ref;

    unless (defined $first_vserver and defined $second_vserver) {
        NATE::BaseException->throw("Vserver_Peer was defined with an engenx_relationship_id " .
                                   "but was unable to find 2 vservers with the same engenx_relationship_id");
    }


    $resource_data->{'vserver'} = exists ($first_vserver->{name}) ? $first_vserver->{name} : $first_vserver->{vserver};
    $resource_data->{'peer-vserver'} = exists ($second_vserver->{name}) ? $second_vserver->{name} : $second_vserver->{vserver};
    $resource_data->{_cluster}  = $first_vserver->{_cluster};
    $resource_data->{'peer-cluster'}  = $second_vserver->{_cluster} if ($first_vserver->{_cluster} ne $second_vserver->{_cluster});

    $Log->exit() if $may_exit;
    return;
}

#==============================================================================
#Method:    _make_cifs_local_user_privilege
#Objective: create a cifs privilege resource for local user if it does not exist
#Outputs: created cifs privilege if one does not exist with the given privileges.
#Exceptions: none
#==============================================================================
sub _make_cifs_local_user_privilege {
    my ($self, $category, $type, $resource_data, $resource_pkey_hash) = @_;

    $Log->enter() if $may_enter;
    my $privilege_data = {};

    # Replace the EnGenX fields in the element data with
    # what we determined the current values were.
    $privilege_data->{_cleanup_policy} = $resource_data->{_cleanup_policy};
    $privilege_data->{if_exists}       = $resource_data->{if_exists};
    $privilege_data->{_parallel}       = $resource_data->{_parallel};
    $privilege_data->{_interface}      = $resource_data->{_interface};
    $privilege_data->{_load_balance}   = $resource_data->{_load_balance};
    $privilege_data->{_setup}          = $resource_data->{_setup};
    $privilege_data->{_cleanup}        = $resource_data->{_cleanup};
    $privilege_data->{_setup_done}     = 0;
    $privilege_data->{_cleanup_done}   = 0;
    $privilege_data->{privileges}      = $resource_data->{PRIVILEGES};
    # Add the primary keys
    while (my ($data_key, $data_value) = each(%$resource_pkey_hash)) {
        $privilege_data->{lc($data_key)} = $data_value;
    }

    $privilege_data->{'user-or-group-name'} = $resource_data->{'USER-NAME'};

    $self->{parent}->{resource_list}->add_resource(category => $category,
                                                   type     => 'cifs_privilege',
                                                   data     => Clone::clone($privilege_data),
                                                );

    delete $resource_data->{PRIVILEGES};
    $Log->exit() if $may_exit;
    return;

} ## end sub _make_cifs_local_user_privilege

#==============================================================================

#==============================================================================
#Method:    _make_cifs_local_group_privilege
#Objective: create a cifs privilege resource for local group if it does not exist
#Outputs: created cifs privilege if one does not exist with the given privileges.
#Exceptions: none
#==============================================================================
sub _make_cifs_local_group_privilege {
    my ($self, $category, $type, $resource_data, $resource_pkey_hash) = @_;

    $Log->enter() if $may_enter;
    my $privilege_data = {};

    # Replace the EnGenX fields in the element data with
    # what we determined the current values were.
    $privilege_data->{_cleanup_policy} = $resource_data->{_cleanup_policy};
    $privilege_data->{if_exists}       = $resource_data->{if_exists};
    $privilege_data->{_parallel}       = $resource_data->{_parallel};
    $privilege_data->{_interface}      = $resource_data->{_interface};
    $privilege_data->{_load_balance}   = $resource_data->{_load_balance};
    $privilege_data->{_setup}          = $resource_data->{_setup};
    $privilege_data->{_cleanup}        = $resource_data->{_cleanup};
    $privilege_data->{_setup_done}     = 0;
    $privilege_data->{_cleanup_done}   = 0;
    $privilege_data->{privileges}      = $resource_data->{PRIVILEGES};
    # Add the primary keys
    while (my ($data_key, $data_value) = each(%$resource_pkey_hash)) {
        $privilege_data->{lc($data_key)} = $data_value;
    }

    $privilege_data->{'user-or-group-name'} = $resource_data->{'GROUP-NAME'};

    $self->{parent}->{resource_list}->add_resource(category => $category,
                                                   type     => 'cifs_privilege',
                                                   data     => Clone::clone($privilege_data),
                                                );

    delete $resource_data->{PRIVILEGES};
    $Log->exit() if $may_exit;
    return;

} ## end sub _make_cifs_loacl_group_privilege

#==============================================================================

#==============================================================================
#Method:    _make_cifs_local_group_membership
#Objective: create a cifs membership resource for local group if it does not exist
#Outputs: created cifs membership if one does not exist with the given privileges.
#Exceptions: none
#==============================================================================
sub _make_cifs_local_group_membership {
    my ($self, $category, $type, $resource_data, $resource_pkey_hash) = @_;

    $Log->enter() if $may_enter;
    my $membership_data = {};

    # Replace the EnGenX fields in the element data with
    # what we determined the current values were.
    $membership_data->{_cleanup_policy} = $resource_data->{_cleanup_policy};
    $membership_data->{if_exists}       = $resource_data->{if_exists};
    $membership_data->{_parallel}       = $resource_data->{_parallel};
    $membership_data->{_interface}      = $resource_data->{_interface};
    $membership_data->{_load_balance}   = $resource_data->{_load_balance};
    $membership_data->{_setup}          = $resource_data->{_setup};
    $membership_data->{_cleanup}        = $resource_data->{_cleanup};
    $membership_data->{_setup_done}     = 0;
    $membership_data->{_cleanup_done}   = 0;
    $membership_data->{'member-names'}      = $resource_data->{'MEMBER-NAMES'};
    # Add the primary keys
    while (my ($data_key, $data_value) = each(%$resource_pkey_hash)) {
        $membership_data->{lc($data_key)} = $data_value;
    }

    $membership_data->{'group-name'} = $resource_data->{'GROUP-NAME'};

    $self->{parent}->{resource_list}->add_resource(category => $category,
                                                   type     => 'local_group_membership',
                                                   data     => Clone::clone($membership_data),
                                                );

    delete $resource_data->{'MEMBER-NAMES'};
    $Log->exit() if $may_exit;
    return;

} ## end sub _make_cifs_loacl_group_privilege

#==============================================================================

#==============================================================================
#Method:    _make_efficiency_policy
#Objective: create an efficiency policy if it does not exist
#Outputs: created efficiency policy if one does not exist with the given name.
#Exceptions: none
#==============================================================================
sub _make_efficiency_policy {
    my ($self, $category, $type, $resource_data, $resource_pkey_hash) = @_;

    $Log->enter() if $may_enter;
    my $efficiency_data = {};

    my $policy_ref = $self->{parent}->{EFFICIENCY_POLICY_NAME};

    my $policy_exists = 0;
    if ($self->{parent}->{EFFICIENCY_POLICY_NAME}->{$resource_data->{vserver}}->{$resource_data->{policy}}){
        $policy_exists = 1;
    }

    if(!$policy_exists){
        # Replace the EnGenX fields in the element data with
        # what we determined the current values were.
        $efficiency_data->{_cleanup_policy} = $resource_data->{_cleanup_policy};
        $efficiency_data->{if_exists}       = $resource_data->{if_exists};
        $efficiency_data->{_parallel}       = $resource_data->{_parallel};
        $efficiency_data->{_interface}      = $resource_data->{_interface};
        $efficiency_data->{_load_balance}   = $resource_data->{_load_balance};
        $efficiency_data->{_setup}          = $resource_data->{_setup};
        $efficiency_data->{_cleanup}        = $resource_data->{_cleanup};
        $efficiency_data->{_setup_done}     = 0;
        $efficiency_data->{_cleanup_done}   = 0;
        $efficiency_data->{policy}          = $resource_data->{policy};
        # Add the primary keys
        while (my ($data_key, $data_value) = each(%$resource_pkey_hash)) {
            $efficiency_data->{lc($data_key)} = $data_value;
        }

        delete $efficiency_data->{volume};

        $self->{parent}->{resource_list}->add_resource(category => $category,
                                                       type     => 'volume_efficiency_policy',
                                                       data     => Clone::clone($efficiency_data),
                                          );
        #add policy name to policy hash.
        $self->parent->{EFFICIENCY_POLICY_NAME}->{$resource_data->{vserver}}->{$resource_data->{policy}} = 1
    }
    $Log->exit() if $may_exit;
    return;

} ## end sub _make_efficiency_policy.

#==============================================================================
# Method:     _build_policy_group_name
# Objective:  Build the policy group name from args.
# Details:    Build policy-group name from vserver,vol,qtree,lun.
# Inputs:     %args from create or delete
# Outputs:    The created name
# Exceptions: Node
#==============================================================================
sub _build_policy_group_name {
    my ($self,$category,$type,$resource_data,$resource_pkey_hash,$is_qos_policy_group) = @_;
    $Log->enter() if $may_enter;
    my $policy_name =  "";
    if($is_qos_policy_group) {
        $policy_name = delete $resource_data->{'NAME'} if exists($resource_data->{'NAME'});
        $policy_name = delete $resource_data->{'POLICY-GROUP'} if exists($resource_data->{'POLICY-GROUP'});
    }
    else {
        $policy_name = delete $resource_data->{'QOS-POLICY-GROUP'} if exists($resource_data->{'QOS-POLICY-GROUP'});
    }
    $policy_name =~ s/_default//i;
    $policy_name = "pg" . $policy_name;
    $policy_name = $resource_data->{NAME} . "_" . $policy_name
                       unless ($is_qos_policy_group);
    $policy_name = $resource_pkey_hash->{LUN} . "_" . $policy_name
                       if (exists($resource_pkey_hash->{LUN})
                       and length($resource_pkey_hash->{LUN}));
    $policy_name = $resource_pkey_hash->{QTREE} . "_" . $policy_name
                       if (exists($resource_pkey_hash->{QTREE})
                       and length($resource_pkey_hash->{QTREE}));
    $policy_name = $resource_pkey_hash->{VOLUME} . "_" . $policy_name
                       if (exists($resource_pkey_hash->{VOLUME})
                       and length($resource_pkey_hash->{VOLUME}));
    $policy_name = $resource_pkey_hash->{VSERVER} . "_" . $policy_name
                       if (exists($resource_pkey_hash->{VSERVER})
                       and length($resource_pkey_hash->{VSERVER}));
    if($is_qos_policy_group) {
        $resource_data->{'POLICY-GROUP'} = $policy_name;
    }
    else {
        $resource_data->{'QOS-POLICY-GROUP'} = $policy_name;
    }
    $Log->exit() if $may_exit;
    return;
}

###############################################################################
# Method:    expand_wafliron_to_resource
# Objective: Expands the wafliron parameter on Aggrs to individual wafliron
#            resources
# Inputs:    $resource_list - reference to a Resource List
#            $category - the category of the resource containing the map list
#            $type - the type of the resource containing the map list
#            $resource_data - the data for the resource containing the map list
#            $resource_pkey_hash - the primary key hash for the resource
#                                  containing the map list
# Returns:   none
###############################################################################
sub _expand_wafliron_to_resource() {
    my ($self,$category,$type,$resource_data,$resource_pkey_hash) = @_;
    $Log->enter() if $may_enter;
    my $wafliron_data = {};

    # Replace the EnGenX fields in the element data with
    # what we determined the current values were.
    $wafliron_data->{_cleanup_policy}  = $resource_data->{_cleanup_policy};
    $wafliron_data->{if_exists}        = $resource_data->{if_exists};
    $wafliron_data->{_parallel}        = $resource_data->{_parallel};
    $wafliron_data->{_interface}       = $resource_data->{_interface};
    $wafliron_data->{_load_balance}    = $resource_data->{_load_balance};
    $wafliron_data->{_setup}           = $resource_data->{_setup};
    $wafliron_data->{_cleanup}         = $resource_data->{_cleanup};
    $wafliron_data->{_setup_done}      = 0;
    $wafliron_data->{_cleanup_done}    = 0;

    # Add the primary keys from aggr to the data to be stored
    while (my ($data_key, $data_value) = each(%$resource_pkey_hash)) {
        $wafliron_data->{lc($data_key)} = $data_value;
    }

    # Add the name key from the aggr resource because it is a pkey of
    # the wafliron
    $wafliron_data->{aggregate} = $resource_data->{NAME};

    # add the aggr map resource
    $self->{parent}->{resource_list}->add_resource(category => $category,
                                         type     => 'aggregate_wafliron',
                                         data     => Clone::clone($wafliron_data));
    $Log->exit() if $may_exit;
    return;
} ## end sub _expand_wafliron_to_resources()

###############################################################################
# Method:    add_failover_group_resources_for_extra_ports
# Objective:
# Returns:   none
###############################################################################
sub _add_failover_group_resources_for_extra_ports() {
    my ($self,$category,$type,$resource_data,$resource_pkey_hash) = @_;
    $Log->enter() if $may_enter;
    my $failover_group_data = {};

    # Replace the EnGenX fields in the element data with
    # what we determined the current values were.
    $failover_group_data->{_cleanup_policy}  = $resource_data->{_cleanup_policy};
    $failover_group_data->{if_exists}        = $resource_data->{if_exists};
    $failover_group_data->{_parallel}        = $resource_data->{_parallel};
    $failover_group_data->{_interface}       = $resource_data->{_interface};
    $failover_group_data->{_load_balance}    = $resource_data->{_load_balance};
    $failover_group_data->{_setup}           = $resource_data->{_setup};
    $failover_group_data->{_cleanup}         = $resource_data->{_cleanup};
    $failover_group_data->{_setup_done}      = 0;
    $failover_group_data->{_cleanup_done}    = 0;

    # Add the primary keys from lun to the data to be stored
    while (my ($data_key, $data_value) = each(%$resource_pkey_hash)) {
        $failover_group_data->{lc($data_key)} = $data_value;
    }

    # Add the name key from the lun resource because it is a pkey of
    # the failover_group
    $failover_group_data->{'failover-group'} = $resource_data->{NAME};

    # Add the other required data that is not ports.
    $failover_group_data->{'node'} = $resource_data->{NODE};

    # store the ports that are part of this failover group in an array
    my $ports_array = [split(/\s*,\s*/, $resource_data->{PORTS})];

    # remove ports since you dont need it.
    $resource_data->{PORT} = pop @$ports_array;
    delete $resource_data->{PORTS};
    while (scalar @$ports_array) {
        $failover_group_data->{port} = pop @$ports_array;
        # add the failover group resource
        $self->{parent}->{resource_list}->add_resource(category => $category,
                                             type     => 'failover_group',
                                             data     => Clone::clone($failover_group_data));
    }
    $Log->exit() if $may_exit;
    return;
} ## end sub _add_failover_group_resources_for_extra_ports()

###############################################################################
# Method:    expand_map_list_to_resources
# Objective: Expands the map_list parameter on Luns to individual lun_map
#            resources
# Inputs:    $resource_list - reference to a Resource List
#            $category - the category of the resource containing the map list
#            $type - the type of the resource containing the map list
#            $resource_data - the data for the resource containing the map list
#            $resource_pkey_hash - the primary key hash for the resource
#                                  containing the map list
# Returns:   none
###############################################################################
sub _expand_map_list_to_resources() {
    my ($self,$category,$type,$resource_data,$resource_pkey_hash) = @_;
    $Log->enter() if $may_enter;
    my $lun_map_data = {};

    # Replace the EnGenX fields in the element data with
    # what we determined the current values were.
    $lun_map_data->{_cleanup_policy}  = $resource_data->{_cleanup_policy};
    $lun_map_data->{if_exists}        = $resource_data->{if_exists};
    $lun_map_data->{_parallel}        = $resource_data->{_parallel};
    $lun_map_data->{_interface}       = $resource_data->{_interface};
    $lun_map_data->{_load_balance}    = $resource_data->{_load_balance};
    $lun_map_data->{_setup}           = $resource_data->{_setup};
    $lun_map_data->{_cleanup}         = $resource_data->{_cleanup};
    $lun_map_data->{_setup_done}      = 0;
    $lun_map_data->{_cleanup_done}    = 0;

    # Add the primary keys from lun to the data to be stored
    while (my ($data_key, $data_value) = each(%$resource_pkey_hash)) {
        $lun_map_data->{lc($data_key)} = $data_value;
    }

    # Add the name key from the lun resource because it is a pkey of
    # the lun_map
    $lun_map_data->{lun} = $resource_data->{NAME};

    # store the igroups that are part of this lun map on the igroups array
    $lun_map_data->{igroups} = [split(/\s*,\s*/, $resource_data->{MAP_LIST})];

    # store a few extra fields required for explicit lun id mappings
    $lun_map_data->{require_same_lun_id_per_igroup} = $resource_data->{REQUIRE_SAME_LUN_ID_PER_IGROUP};
    $lun_map_data->{'lun-id'} = $resource_data->{'LUN-ID'};
    $lun_map_data->{lun_id_range} = $resource_data->{LUN_ID_RANGE};
    $lun_map_data->{reporting_nodes} = $resource_data->{REPORTING_NODES} if(defined $resource_data->{REPORTING_NODES});
    $lun_map_data->{report_all_nodes} = $resource_data->{REPORT_ALL_NODES} if(defined $resource_data->{REPORT_ALL_NODES});
    $lun_map_data->{report_local_nodes} = $resource_data->{REPORT_LOCAL_NODES} if(defined $resource_data->{REPORT_LOCAL_NODES});
    $lun_map_data->{report_dest_aggr} = $resource_data->{REPORT_DEST_AGGR} if(defined $resource_data->{REPORT_DEST_AGGR});
    $lun_map_data->{report_dest_vol} = $resource_data->{REPORT_DEST_VOL} if(defined $resource_data->{REPORT_DEST_VOL});
    $lun_map_data->{remove_reporting_nodes} = $resource_data->{REMOVE_REPORTING_NODES} if(defined $resource_data->{REMOVE_REPORTING_NODES});
    $lun_map_data->{remove_remote_nodes} = $resource_data->{REMOVE_REMOTE_NODES} if(defined $resource_data->{REMOVE_REMOTE_NODES});

    # add the lun map resource
    $self->{parent}->{resource_list}->add_resource(category => $category,
                                         type     => 'lun_map',
                                         data     => Clone::clone($lun_map_data));
    $Log->exit() if $may_exit;
    return;
} ## end sub _expand_map_list_to_resources()

###############################################################################
# Method:    expand_bind_list_to_resources
# Objective: Expands the bind_list parameter on Luns to individual lun_bind
#            resources
# Inputs:    $resource_list - reference to a Resource List
#            $category - the category of the resource containing the bind list
#            $type - the type of the resource containing the bind list
#            $resource_data - the data for the resource containing the bind list
#            $resource_pkey_hash - the primary key hash for the resource
#                                  containing the bind list
# Returns:   none
###############################################################################
sub _expand_bind_list_to_resources() {
    my ($self,$category,$type,$resource_data,$resource_pkey_hash) = @_;
    $Log->enter() if $may_enter;
    my $lun_bind_data = {};

    # Replace the EnGenX fields in the element data with
    # what we determined the current values were.
    $lun_bind_data->{_cleanup_policy}  = $resource_data->{_cleanup_policy};
    $lun_bind_data->{if_exists}        = $resource_data->{if_exists};
    $lun_bind_data->{_parallel}        = $resource_data->{_parallel};
    $lun_bind_data->{_interface}       = $resource_data->{_interface};
    $lun_bind_data->{_load_balance}    = $resource_data->{_load_balance};
    $lun_bind_data->{_setup}           = $resource_data->{_setup};
    $lun_bind_data->{_cleanup}         = $resource_data->{_cleanup};
    $lun_bind_data->{_setup_done}      = 0;
    $lun_bind_data->{_cleanup_done}    = 0;

    # Add the primary keys from lun to the data to be stored
    while (my ($data_key, $data_value) = each(%$resource_pkey_hash)) {
        $lun_bind_data->{lc($data_key)} = $data_value;
    }

    # Make the VVOL PATH from the lun resource because it is a pkey of
    # the lun_bind
    my $volume =  $lun_bind_data->{volume};
    my $qtree =  $lun_bind_data->{qtree};
    my $lun    =  $resource_data->{NAME};
    if (defined($qtree) and length($qtree)) {
        $lun_bind_data->{'vvol-path'} = "/vol/" . $volume . "/" . $qtree . "/" . $lun;
    } else {
        $lun_bind_data->{'vvol-path'} = "/vol/" . $volume . "/" . $lun;
    }

    # Delete volume and qtree because they are not part of the lun_bind
    # args and are contained in the vvol path
    delete $lun_bind_data->{volume};
    delete $lun_bind_data->{qtree};

    # create a resource obj for all the bind list entries
    foreach my $pep (split(/,/, delete $resource_data->{BIND_LIST})) {
        # the protocol endpoint path may come in different styles,
        # the purpose of this logic is to convert all the possible
        # scenarios into an absolute lun path

        # Possible values of $pep:
        # (1) Absolute path
        #     e.g: /vol/myvol/mypep
        #     e.g: /vol/myvol/myqtree/mypep
        # (2) Relative path of a volume (vol: /vol/myvol)
        #     e.g: ./mypep --> /vol/myvol/mypep
        #     e.g: mypep   --> /vol/myvol/mypep
        #     e.g: myqtree/mypep   --> /vol/myvol/myqtree/mypep
        # (3) Relative path of a qtree inside a vol (vol: /vol/myvol/myqtree)
        #     e.g: ./mypep --> /vol/myvol/myqtree/mypep
        #     e.g: mypep   --> /vol/myvol/myqtree/mypep
        #     e.g: ../mypep --> /vol/myvol/mypep
        #     e.g: ../myqtree2/mypep   --> /vol/myvol/myqtree2/mypep

        # get rid of spaces
        $pep =~ s/\s+//g;

        # get rid of ./ in the bind list param if the user specified it
        $pep =~ s/^\.\///;

        if ($pep !~ /^\/vol/) {
            # pep is a relative path to a lun
            # add the vol name to the path
            # check if it has a qtree in it
            if (not defined($qtree) or not length($qtree)) {
                # the current vvol lun is not in a qtree
                # this handles implcict and explicit relative qtree pep cases
                $pep = "/vol/" . $volume . "/" . $pep;
            } else {
                # the current vvol is in a qtree
                if ($pep =~ /^\.\.\//) {
                    # remove the ../ from the pep
                    $pep =~ s/^\.\.\///;

                    # the user does not want this pep to be in the qtree, prepend just the vol name
                    $pep = "/vol/" . $volume . "/" . $pep;
                } else {
                    # this pep is a relative path with an implicit qtree name,
                    # ie: it does not contain a qtree name, but the vvol is in a qtree
                    # prepend the current vol name with the qtree
                    $pep = "/vol/" . $volume . "/" . $qtree . "/" . $pep;
                }
            }
        }

        $lun_bind_data->{'protocol-endpoint-path'} = $pep;

        $self->{parent}->{resource_list}->add_resource(
            category => $category,
            type     => 'lun_bind',
            data     => Clone::clone($lun_bind_data)
        );
    }
    $Log->exit() if $may_exit;
    return;
} ## end sub _expand_bind_list_to_resources()

###############################################################################
# Method:    expand_search_paths_to_resources
# Objective: Expands the seach_path_list parameter on vservers to individual
#            search_path resources
# Inputs:    $resource_list - reference to a Resource List
#            $category - the category of the resource containing the map list
#            $type - the type of the resource containing the map list
#            $resource_data - the data for the resource containing the map list
#            $resource_pkey_hash - the primary key hash for the resource
#                                  containing the map list
# Returns:   none
###############################################################################
sub _expand_search_paths_to_resources() {
    my ($self,$category,$type,$resource_data,$resource_pkey_hash) = @_;
    $Log->enter() if $may_enter;
    my $search_path_data = {};

    # Replace the EnGenX fields in the element data with
    # what we determined the current values were.
    $search_path_data->{_cleanup_policy}  = $resource_data->{_cleanup_policy};
    $search_path_data->{if_exists}        = $resource_data->{if_exists};
    $search_path_data->{_parallel}        = $resource_data->{_parallel};
    $search_path_data->{_interface}       = $resource_data->{_interface};
    $search_path_data->{_load_balance}    = $resource_data->{_load_balance};
    $search_path_data->{_setup}           = $resource_data->{_setup};
    $search_path_data->{_cleanup}         = $resource_data->{_cleanup};
    $search_path_data->{_setup_done}      = 0;
    $search_path_data->{_cleanup_done}    = 0;

    # Add the primary keys from vserver to the data to be stored
    while (my ($data_key, $data_value) = each(%$resource_pkey_hash)) {
        $search_path_data->{lc($data_key)} = $data_value;
    }

    # Need to add the vserver name
    $search_path_data->{vserver} = $resource_data->{NAME};
    # Now we have all the required data that will not change,
    # split the searchpath list based on ',' and loop through the paths
    # to add a searchpath resource to the resource list for each path.
    foreach my $path (split(/,/,$resource_data->{HOMEDIR_SEARCH_PATH_LIST})) {
        $search_path_data->{path} = $path;
        $self->{parent}->{resource_list}->add_resource(
            category => $category,
            type     => 'homedir_search_path',
            data     => Clone::clone($search_path_data)
        );
    }

    # Now that we have created the search path resources
    # delete the list.
    delete $resource_data->{HOMEDIR_SEARCH_PATH_LIST};

    $Log->exit() if $may_exit;
    return;
} ## end sub _expand_search_paths_to_resources()

###############################################################################
# Method:    _expand_initiator_list_to_resources
# Objective: Expands the initiator_list parameter on Igroups to individual
#            initiator resources
# Inputs:    $resource_list - reference to a Resource List
#            $category - the category of the resource containing the init list
#            $type - the type of the resource containing the init list
#            $resource_data - the data for the resource containing the init list
#            $resource_pkey_hash - the primary key hash for the resource
#                                  containing the init list
# Returns:   none
###############################################################################
sub _expand_initiator_list_to_resources() {
    my ($self,$category,$type,$resource_data,$resource_pkey_hash) = @_;
    $Log->enter() if $may_enter;
    my $initiator_data = {};
    # Replace the EnGenX fields in the element data with
    # what we determined the current values were.
    $initiator_data->{_cleanup_policy}  = $resource_data->{_cleanup_policy};
    $initiator_data->{if_exists}        = $resource_data->{if_exists};
    $initiator_data->{_parallel}        = $resource_data->{_parallel};
    $initiator_data->{_interface}       = $resource_data->{_interface};
    $initiator_data->{_load_balance}    = $resource_data->{_load_balance};
    $initiator_data->{_setup}           = $resource_data->{_setup};
    $initiator_data->{_cleanup}         = $resource_data->{_cleanup};
    $initiator_data->{_setup_done}      = 0;
    $initiator_data->{_cleanup_done}    = 0;

    # Add the primary keys from igroup to the data to be stored
    while (my ($data_key, $data_value) = each(%$resource_pkey_hash)) {
        $initiator_data->{lc($data_key)} = $data_value;
    }

    # Add the name key from the igroup resource because it is a pkey of
    # the initiator
    $initiator_data->{igroup} = $resource_data->{NAME};
    $initiator_data->{_protocol} = $resource_data->{PROTOCOL};
    # Now we have all the required data that will not change,
    # split the initiator_list based on ',' and loop through the igroups
    # to add a initiator resource to the resource list for each igroup.
    foreach my $initiator (split(/,/,$resource_data->{INITIATOR_LIST})) {
        $initiator_data->{name} = $initiator;
        $self->{parent}->{resource_list}->add_resource(
            category => $category,
            type     => 'initiator',
            data     => Clone::clone($initiator_data)
        );
    }
    $Log->exit() if $may_exit;
    return;
} ## end sub _expand_initiator_list_to_resources()

###############################################################################
# Method:    _expand_file_directory_policy_to_resource
# Objective: Creates the file_directory_policy_apply element in order to
#            enable the fsecurity policy on the vserver later.
# Inputs:    resource category, resource type, resource data and the resource
#            primary key hash
# Returns:   Returns nothing, adds a resource
###############################################################################
sub _expand_file_directory_policy_to_resource {
    my ($self,$category,$type,$resource_data,$resource_pkey_hash) = @_;
    $Log->enter() if $may_enter;

    # Get fpolicy name from the fpolicy element it is needed to create
    # the resource, fpolicy_enable, that will enable the fpolicy.
    my $fsecurity_policy_name = $resource_data->{NAME};

    my $fsecurity_enable_data = {};

    # Replace the EnGenX fields in the element data with
    # what we determined the current values were.
    $fsecurity_enable_data->{_cleanup_policy}  = $resource_data->{_cleanup_policy};
    $fsecurity_enable_data->{if_exists}        = $resource_data->{if_exists};
    $fsecurity_enable_data->{_parallel}        = $resource_data->{_parallel};
    $fsecurity_enable_data->{_interface}       = $resource_data->{_interface};
    $fsecurity_enable_data->{_load_balance}    = $resource_data->{_load_balance};
    $fsecurity_enable_data->{_setup}           = $resource_data->{_setup};
    $fsecurity_enable_data->{_cleanup}         = $resource_data->{_cleanup};
    $fsecurity_enable_data->{_setup_done}      = 0;
    $fsecurity_enable_data->{_cleanup_done}    = 0;

    # Add the primary keys from igroup to the data to be stored
    while (my ($data_key, $data_value) = each(%$resource_pkey_hash)) {
        $fsecurity_enable_data->{lc($data_key)} = $data_value;
    }

    $fsecurity_enable_data->{'policy-name'} = $fsecurity_policy_name;

    # add the file_directory_policy_apply resource
    $self->{parent}->{resource_list}->add_resource(category => $category,
                                         type     => 'file_directory_policy_apply',
                                         data     => Clone::clone($fsecurity_enable_data));

    $Log->exit() if $may_exit;
    return;
} ## end sub _expand_file_directory_policy_to_resource

###############################################################################
# Method:    _sort_volumes_by_quota_settings
# Objective: Sorts all of the volumes with quota rules defined for a quota policy
#            that is set to the vserver.
# Inputs:    resource category, resource type, resource data and the resource
#            primary key hash
# Returns:   Returns nothing, calls _expand_vol_quota_to_resource with the
#            sorted volumes organized in a hash
###############################################################################
sub _sort_volumes_by_quota_settings {

    my ($self, $category, $type, $resource_data, $resource_pkey_hash) = @_;

    my %volume_list;

    # get the list of quota rules for the enabled quota policy
    # from the resource list
    my $rules_ref = $self->{parent}->{resource_list}->get_resources(
        category => "^CLUSTER",
        type     => "^quota_rule",
    );

    # Gather hash of volumes quota rules are being defined for
    foreach my $quota_rule (@$rules_ref){
        $volume_list{$quota_rule->{volume}}=$quota_rule->{vserver};
    }

    $self->_expand_volume_quota_to_resource(
            $category,
            $resource_data,
            $resource_pkey_hash,
            \%volume_list,
        );

} ## end sub _sort_volumes_by_quota_settings()

###############################################################################
# Method:    _expand_volume_quota_to_resource
# Objective: Create a resource object for the volumes with quota policies
#            that need to be enabled
# Inputs:    resource category, resource data, resource primary hash key, and
#            the volume resources created in _sort_volumes_by_junction_path
# Returns:   Returns nothing, Resourcelist populated
###############################################################################
sub _expand_volume_quota_to_resource() {

    my ($self,$category,$resource_data,$resource_pkey_hash,$volume_list) = @_;
    $Log->enter() if $may_enter;

    my $vol_quota_data = {};

    # Replace the EnGenX fields in the element data with
    # what we determined the current values were.
    $vol_quota_data->{_cleanup_policy}  = $resource_data->{_cleanup_policy};
    $vol_quota_data->{if_exists}        = $resource_data->{if_exists};
    $vol_quota_data->{_parallel}        = $resource_data->{_parallel};
    $vol_quota_data->{_interface}       = $resource_data->{_interface};
    $vol_quota_data->{_load_balance}    = $resource_data->{_load_balance};
    $vol_quota_data->{_setup}           = $resource_data->{_setup};
    $vol_quota_data->{_cleanup}         = $resource_data->{_cleanup};
    $vol_quota_data->{_config}          = $resource_data->{_config};
    $vol_quota_data->{_deconfig}        = $resource_data->{_deconfig};
    $vol_quota_data->{_validate}        = $resource_data->{_validate};
    $vol_quota_data->{_precleanup}      = $resource_data->{_precleanup};
    $vol_quota_data->{_validate_done}   = 0;
    $vol_quota_data->{_config_done}     = 0;
    $vol_quota_data->{_setup_done}      = 0;
    $vol_quota_data->{_deconfig_done}   = 0;
    $vol_quota_data->{_cleanup_done}    = 0;
    $vol_quota_data->{_precleanup_done} = 0;

    $vol_quota_data->{volumes} = $volume_list;

    # add the vol quota resource
    $self->{parent}->{resource_list}->add_resource(category => $category,
                                         type     => 'volume_quota',
                                         data     => Clone::clone($vol_quota_data));
    $Log->exit() if $may_exit;

} ## end sub _expand_volume_quota_to_resource()

#==============================================================================
# Method:     _expand_fpolicy_enable_to_resource
# Objective:  Create a resource object for enabling fpolicies.
# Inputs:     %args from create or delete
# Outputs:    Return nothing, Resourcelist is populated
#==============================================================================
sub _expand_fpolicy_enable_to_resource {
    my ($self,$category,$type,$resource_data,$resource_pkey_hash) = @_;
    $Log->enter() if $may_enter;

    # Get fpolicy name from the fpolicy element it is needed to create
    # the resource, fpolicy_enable, that will enable the fpolicy.
    my $fpolicy_name = $resource_data->{NAME};
    my $fpolicy_sequence_num = $resource_data->{'SEQUENCE-NUMBER'};

    my $fpolicy_enable_data = {};

    # Replace the EnGenX fields in the element data with
    # what we determined the current values were.
    $fpolicy_enable_data->{_cleanup_policy}  = $resource_data->{_cleanup_policy};
    $fpolicy_enable_data->{if_exists}        = $resource_data->{if_exists};
    $fpolicy_enable_data->{_parallel}        = $resource_data->{_parallel};
    $fpolicy_enable_data->{_interface}       = $resource_data->{_interface};
    $fpolicy_enable_data->{_load_balance}    = $resource_data->{_load_balance};
    $fpolicy_enable_data->{_setup}           = $resource_data->{_setup};
    $fpolicy_enable_data->{_cleanup}         = $resource_data->{_cleanup};
    $fpolicy_enable_data->{_setup_done}      = 0;
    $fpolicy_enable_data->{_cleanup_done}    = 0;

    # Add the primary keys from fpolicy to the data to be stored
    while (my ($data_key, $data_value) = each(%$resource_pkey_hash)) {
        $fpolicy_enable_data->{lc($data_key)} = $data_value;
    }

    $fpolicy_enable_data->{'policy-name'} = $fpolicy_name;
    $fpolicy_enable_data->{'sequence-number'} = $fpolicy_sequence_num;

    # add the fpolicy_enable resource
    $self->{parent}->{resource_list}->add_resource(category => $category,
                                         type     => 'fpolicy_enable',
                                         data     => Clone::clone($fpolicy_enable_data));

    $Log->exit() if $may_exit;
    return;
}

#==============================================================================
# Method:     _expand_external_engince_connection_to_resource
# Objective:  Create a resource object for enabling fpolicy external engines.
# Inputs:     %args from create or delete
# Outputs:    Return nothing, Resourcelist is populated
#==============================================================================
sub _expand_external_engine_connection_to_resource {
    my ($self,$category,$type,$resource_data,$resource_pkey_hash) = @_;
    $Log->enter() if $may_enter;

    # Get fpolicy name from the fpolicy element it is needed to create
    # the resource, external_engine_connection, that will connect the fpolicy to the external-engine.
    my $fpolicy_name =  $resource_data->{FPOLICY};

    # Retrieve the first server from the external_engine element's 'primary-servers' attribute
    # In the future, the whole list will likely be needed to be passed to the external_engine_connection
    # element. burt#690612
    my $servers = [ split(/,/ ,$resource_data->{'PRIMARY-SERVERS'}) ];

    # Get the node for connection
    my $node = $resource_data->{NODE};

    my $external_engine_connection_data = {};

    # Replace the EnGenX fields in the element data with
    # what we determined the current values were.
    $external_engine_connection_data->{_cleanup_policy}  = $resource_data->{_cleanup_policy};
    $external_engine_connection_data->{if_exists}        = $resource_data->{if_exists};
    $external_engine_connection_data->{_parallel}        = $resource_data->{_parallel};
    $external_engine_connection_data->{_interface}       = $resource_data->{_interface};
    $external_engine_connection_data->{_load_balance}    = $resource_data->{_load_balance};
    $external_engine_connection_data->{_setup}           = $resource_data->{_setup};
    $external_engine_connection_data->{_cleanup}         = $resource_data->{_cleanup};
    $external_engine_connection_data->{_setup_done}      = 0;
    $external_engine_connection_data->{_cleanup_done}    = 0;

    # Add the primary keys from external-engine to the data to be stored
    while (my ($data_key, $data_value) = each(%$resource_pkey_hash)) {
        $external_engine_connection_data->{$data_key} = $data_value;
    }

    $external_engine_connection_data->{'policy-name'} = $fpolicy_name;
    $external_engine_connection_data->{'server'} = $servers->[0];
    $external_engine_connection_data->{'node'} = $node;

    # add the external_engine_connection resource
    $self->{parent}->{resource_list}->add_resource(category => $category,
                                         type     => 'external_engine_connection',
                                         data     => Clone::clone($external_engine_connection_data));

    $Log->exit() if $may_exit;
    return;
}

###############################################################################
# Method:    _sort_volumes_by_junction_path
# Objective: Sorts all of the volumes with deep-junction paths by the junction
#            path name
# Inputs:    resource category, resource type, resource data and the resource
#            primary key hash
# Returns:   Returns nothing, calls _expand_vol_mount_to_resource with the
#            sorted volumes organized in a hash
###############################################################################
sub _sort_volumes_by_junction_path {

    my ($self, $category, $type, $resource_data, $resource_pkey_hash) = @_;

    # get the list of volumes which are using deep junctions
    # from the resource list
    my $volumes_ref = $self->{parent}->{resource_list}->get_resources(
        category => "^CLUSTER",
        type     => "^volume\$",
        data     => {use_deep_junction => 1}
    );

    my @volumes = @{$volumes_ref};

    my $junction_path;
    my @grouped_volumes;
    my %grouped_volumes_hash;

    my $previous_junction_path;
    my $last_volume = $volumes[-1];

    # loop over all path names once organizing them
    # into groups by top directory. The hash maps:
    # common top directory -> deep junction path
    while (scalar(@volumes)) {
        my $volume = shift(@volumes);

        # if junction path is not set and use_junction_path
        # is not set skip to the next volume
        if (not $volume->{'junction-path'} and
            not $volume->{use_junction_path}) {
            next;
        }

        # pull out the name of the top directory in the
        # junction-path name.
        if ($volume->{'junction-path'} =~ /^(\/[a-zA-Z0-9\._\-]+\/?)/) {
            $junction_path = $1;
        } else {
            $self->{parent}->_set_err("junction-path did not match the correct " .
                            "format.");
        }

        # check if the junction path and previous junction path match
        if (defined($previous_junction_path)
            and     $junction_path !~ /^$previous_junction_path$/) {
            # add the array of path-names for that group-name to the hash
            $grouped_volumes_hash{$previous_junction_path}
                                                = [@grouped_volumes];
            # reset the array of path-names to empty
            @grouped_volumes = ();
        }

        # set previous group-name to the current group-name
        $previous_junction_path = $junction_path;
        # push the current path name on to the grouped path-names array
        push (@grouped_volumes, $volume);

        # check if the current path-name is the last path-name in the list
        if (not scalar(@volumes)) {
            $grouped_volumes_hash{$previous_junction_path}
                = [@grouped_volumes];
        }
    }

    # create and add a resource for each junction-path key
    foreach my $volume_key (keys(%grouped_volumes_hash)) {
        $self->_expand_vol_mount_to_resource(
            $category,
            $resource_data,
            $resource_pkey_hash,
            $grouped_volumes_hash{$volume_key},
        );
    }
} ## end sub _sort_volumes_by_junction_path()

###############################################################################
# Method:    _expand_vol_mount_to_resource
# Objective: Create a resource object for the volumes with deep junction-paths
#            that need to be mounted
# Inputs:    resource category, resource data, resource primary hash key, and
#            the volume resources created in _sort_volumes_by_junction_path
# Returns:   Returns nothing, Resourcelist populated
###############################################################################
sub _expand_vol_mount_to_resource() {

    my ($self,$category,$resource_data,$resource_pkey_hash,$vol_resources_object) = @_;
    $Log->enter() if $may_enter;

    my $vol_mount_data = {};

    # Replace the EnGenX fields in the element data with
    # what we determined the current values were.
    $vol_mount_data->{_cleanup_policy}  = $resource_data->{_cleanup_policy};
    $vol_mount_data->{if_exists}        = $resource_data->{if_exists};
    $vol_mount_data->{_parallel}        = $resource_data->{_parallel};
    $vol_mount_data->{_interface}       = $resource_data->{_interface};
    $vol_mount_data->{_load_balance}    = $resource_data->{_load_balance};
    $vol_mount_data->{_setup}           = $resource_data->{_setup};
    $vol_mount_data->{_cleanup}         = $resource_data->{_cleanup};
    $vol_mount_data->{_setup_done}      = 0;
    $vol_mount_data->{_cleanup_done}    = 0;

    $vol_mount_data->{volumes} = $vol_resources_object;

    # add the vol mount resource
    $self->{parent}->{resource_list}->add_resource(category => $category,
                                         type     => 'volume_mount',
                                         data     => Clone::clone($vol_mount_data));
    $Log->exit() if $may_exit;

} ## end sub _expand_vol_mount_to_resource()

###############################################################################
# Method:    _get_parent_value
# Objective: For things like cleanup_policy, reuse_policy, etc, we will
#            use a value if we are given one.  If not we will check the
#            parent data, otherwise we use a default. This is to be used
#            by _xml_walk()
# Inputs:    walk_hash_key, data_hash_key, walk_hash, data_hash, default
# Returns:   value of hash_key (or default)
###############################################################################
sub _get_parent_value() {
    my ($self, $walk_hash_key, $data_hash_key, $walk_hash, $data_hash,
        $default)
        = @_;

    my $current_value = $default;

    if (defined $walk_hash->{$walk_hash_key}) {
        # We have our own value of this key, use it
        $current_value = $walk_hash->{$walk_hash_key};
    } elsif (defined $data_hash->{$data_hash_key}) {
        # We don't have our own value but one of our
        # acestors had a value, inherit that value
        $current_value = $data_hash->{$data_hash_key};
    }

    return $current_value;
} ## end sub _get_parent_value()

###############################################################################
# Method:    _get_current_action
# Objective: Check the given hash for value and set it to either that
#            value or the given default.
# Inputs:    hash, hash_key, default, label
###############################################################################
sub _get_current_action() {
    my ($self, @opts) = @_;
    $Log->enter() if $may_enter;
    # Locals
    my %args=();

    # Process arguments.
    try {
        %args = Params::Validate::validate( @opts, {
            cluster_hash  => {type => HASHREF, optional => FALSE},
            data_hash  => {type => HASHREF, optional => FALSE},
            label => {type => SCALAR,  optional => FALSE}
            }
        );
    }
    otherwise {
        my $xcptn = shift;
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw($xcptn->text());
    };

    # Gather values
    my $data_hash = $args{data_hash};
    my $label = $args{label};

    foreach my $key (keys( %{$data_hash} )) {
        # 1. Get the default value
        my $default = $data_hash->{$key};
        # 2. Transform the KEY to a HASH_KEY
        #    That is get rid of the first underscore
        #    and  change it to capital letters.
        my $hash_key = uc(substr($key,1));
        if( $hash_key =~ /INTERFACE/i ){
            $hash_key = "CMD_".$hash_key;
        }
        # 3. Get the current_action by making the
        #    corresponding call
        $data_hash->{$key} = $self->_get_xml_override(
                hash     => $args{cluster_hash},
                hash_key => $hash_key,
                default  => $default,
                label    => $label
            );
    }

    $Log->exit() if $may_exit;
    return (wantarray()) ? %{$data_hash} : $data_hash;
} ## end _get_current_action

###############################################################################
# Method:    _get_xml_override
# Objective: Check the given hash for value and set it to either that
#            value or the given default.
# Inputs:    hash, hash_key, default, label
###############################################################################
sub _get_xml_override() {
    my ($self, %args) = @_;
    $Log->enter() if $may_enter;

    # Process agrs and validate mandatory params
    &ParamsMgr::process_args(%args);

    # Get the hash
    my $hash
        = &ParamsMgr::get_arg(-arg => "hash", -required => $NACL::AppBuilder::EnGenX::TRUE);
    if (&ParamsMgr::error()) {
        $Log->exit() if $may_exit;
        my $errmsg = &ParamsMgr::errmsg();
        NATE::BaseException->throw("Failed to find hash params: $errmsg");
    }

    # Get the hash_key
    my $hash_key
        = &ParamsMgr::get_arg(-arg => "hash_key", -required => $NACL::AppBuilder::EnGenX::TRUE);
    if (&ParamsMgr::error()) {
        $Log->exit() if $may_exit;
        my $errmsg = &ParamsMgr::errmsg();
        NATE::BaseException->throw("Failed to find hash_key params: $errmsg");
    }

    # Get the default
    my $default
        = &ParamsMgr::get_arg(-arg => "default", -required => $NACL::AppBuilder::EnGenX::TRUE);
    if (&ParamsMgr::error()) {
        $Log->exit() if $may_exit;
        my $errmsg = &ParamsMgr::errmsg();
        NATE::BaseException->throw("Failed to find default params: $errmsg");
    }

    # Get the label
    my $label = &ParamsMgr::get_arg(-arg => "label", -required => $NACL::AppBuilder::EnGenX::TRUE);
    if (&ParamsMgr::error()) {
        $Log->exit() if $may_exit;
        my $errmsg = &ParamsMgr::errmsg();
        NATE::BaseException->throw("Failed to find label params: $errmsg");
    }

    my $current_value = $default;
    if (defined($hash->{$hash_key})) {
        $current_value = $hash->{$hash_key};
    }

    $Log->exit() if $may_exit;
    return $current_value;
} ## end sub _get_xml_override()

###############################################################################
# Method:    _generate_copies
# Objective: Generates "n" copies of an object defined in the params. It's
#            responsible for renaming and appending indexes to the names
#            if needed. It also assigns IPs to copies of iSCSI lifs
# Input:     A reference to a hash containing object attributes
# Returns:   An array or a reference to an array containing all hashes
#            with all the attributes corresponding to a certain object.
###############################################################################
sub _generate_copies {
    my ($self, %args) = @_;

    # Local variables
    my @objs      = ();
    my @ips_range = ();
    my $num_copies = 1;    # By default create 1 object.
    my $range      = 1;
    my $offset     = 0;    # start counting at 0

    # Process agrs and validate mandatory params
    &ParamsMgr::process_args(%args);

    my $element = &ParamsMgr::get_arg(
        -arg      => "element",
        -required => $NACL::AppBuilder::EnGenX::TRUE
    );

    my $parent_name = &ParamsMgr::get_arg(
        -arg      => "parent_name",
        -required => $NACL::AppBuilder::EnGenX::FALSE,
        -default  => ""
    );

    my $type = &ParamsMgr::get_arg(
        -arg      => "type",
        -required => $NACL::AppBuilder::EnGenX::FALSE,
        -default  => "none"
    );

    # Make sure we don't mess up with the original data structure.
    my $data = Clone::clone($element);

    # Check for count or loop
    if (defined($data->{COUNT})) {
        $num_copies = delete $data->{COUNT};
    } elsif (defined($data->{LOOP})) {
        $num_copies = delete $data->{LOOP};
    }

    # Check for an index or offset. i.e. an offset of
    # 50 results in ob_50, ob_51 instead of ob_0, ob_1, etc.
    if (defined($data->{INDEX})) {
        $offset = delete $data->{INDEX};
    } elsif (defined($data->{OFFSET})) {
        $offset = delete $data->{OFFSET};
    }

    # If the type is Vserver,Volume, or Lun take care of the children QOS
    # Policy Groups.
    if ($type =~/^vserver\d*$/i or
        $type =~/^volume\d*$/i or
        $type =~/^lun\d*$/i) {

        # Need to loop through all children qos and check:
        # 1. If no name attribute add NAME=>_default
        # 2. If parent use_default_qos_policy_group
        #        If all have name attr then create one
        #        With NAME=>_default
        #        Policy groups cannot start with _ so it
        #        is a good indicator that this should be
        #        changed in lower level libraries to the
        #        primary keys generated name.
        # 3. Set assign_to_vserver if vserver and correct
        #        qos resource
        my $found_no_name = FALSE;
        # This is the number appended to the qos hash key
        my $qos_hash_number = 0;
        # Name of policy group to assign to this object.
        my $qos_policy_name = exists $data->{'QOS-POLICY-GROUP'} ? $data->{'QOS-POLICY-GROUP'} : "_NOT_SET";
        # Will store the hash key of the qos policy matching
        # $qos_policy_name if we find one.
        my $vserver_qos_key = undef;
        my $vserver_no_name_key = undef;
        foreach my $key ( keys(%{$data})) {
            if ($key =~/^(QOS_POLICY_GROUP)/i){
                $qos_hash_number = $1 if($key =~ /(\d+)/);
                if(not exists($data->{$key}->{NAME})) {
                    $data->{$key}->{NAME} = "_default";
                    if(exists $data->{$key}->{COUNT} and $data->{$key}->{COUNT} <= 1) {
                        $vserver_no_name_key = $key;
                        $found_no_name = TRUE;
                    }
                }
                if($type =~/^vserver/i and $data->{$key}->{NAME} =~ /^$qos_policy_name$/) {
                    # Since this is the qos-policy-group defined to
                    # assigned to the vserver tell the qos resource
                    # this.
                    $data->{$key}->{ASSIGN_TO_VSERVER} = 1;
                    $vserver_qos_key = $key;
                }
            }

            if ($type =~/^vserver/i and $key =~/^(VOLUME_EFFICIENCY_POLICY)/i) {
                $self->{parent}->{EFFICIENCY_POLICY_NAME}->{$data->{NAME}}->{$data->{$key}->{POLICY}} = 1;
            }

            # Want to make sure that the lif tags get the ipspace of the vserver so that
            # ip_address usage can be per ipspace.
            if ($type =~/^vserver/i and $key =~/^LIF/i) {
                # If the ipspace exists then pass it to the lif child.
                if(exists($data->{IPSPACE})) {
                    $data->{$key}->{IPSPACE} = $data->{IPSPACE};
                } else { # otherwise make the ipspace of the lif child default
                    $data->{$key}->{IPSPACE} = 'Default';
                }
            }

            # If PortSet has an IGROUP_BIND tag under it, means that the user wants to
            # associate (bind) this portset with that specific igroup. Also need the vserver
            # name in case we need to prepend vserver to the igroup
            if ($type =~/^vserver/i and $key =~ /^portset/i) {
                # Look if the user provided igroups to bind to the portset
                foreach my $key2 (keys %{$data->{$key}}) {
                    if ($key2 =~ /^IGROUP_BIND\d*$/) {
                        if (exists($data->{$key}->{$key2}->{NAME})) {
                            if($data->{$key}->{$key2}->{PREPEND_VSERVER}) {
                                push(@{$data->{$key}->{igroups}}, '' . $data->{NAME} . '_' . $data->{$key}->{$key2}->{NAME});
                            } else {
                                push(@{$data->{$key}->{igroups}}, $data->{$key}->{$key2}->{NAME});
                            }
                            # store the igroup name in the igroup array

                            delete($data->{$key}->{$key2});
                        }
                    }
                }
            }
        }

        # Replace/Create the qos-policy-group creation argument if
        # 'use_default_qos_policy_group' is true(1).
        if($data->{USE_DEFAULT_QOS_POLICY_GROUP}) {
            $data->{'QOS-POLICY-GROUP'} = '_default';
            if ($found_no_name) {
                # Since this is the qos-policy-group defined to
                # assigned to the vserver tell the qos resource
                # this.
                $data->{$vserver_no_name_key}->{ASSIGN_TO_VSERVER} = 1;
                # use_default_qos_policy_group takes precedence
                # so remove assign_to_verver if it was added because
                # qos-policy-group is defined on the vserver.
                $data->{$vserver_qos_key}->{ASSIGN_TO_VSERVER} = 0
                    if(defined($vserver_qos_key) and
                       exists $data->{$vserver_qos_key});
            } else {
                my $key = "QOS_POLICY_GROUP$qos_hash_number";
                $data->{$key} = {NAME => "_default"};
                if($type =~/^vserver/i) {
                    # Since this is the qos-policy-group defined to
                    # assigned to the vserver tell the qos resource
                    # this.
                    $data->{$key}->{ASSIGN_TO_VSERVER} = 1;
                    # use_default_qos_policy_group takes precedence
                    # so remove assign_to_verver if it was added because
                    # qos-policy-group is defined on the vserver.
                    $data->{$vserver_qos_key}->{ASSIGN_TO_VSERVER} = 0
                        if(defined($vserver_qos_key) and
                           exists $data->{$vserver_qos_key});
                }
            }
            # Remove this because it is not used anywhere else.
            delete $data->{USE_DEFAULT_QOS_POLICY_GROUP};
            # Remove this because vserver assignment is taken care of
            # in the qos creation.
        }
        delete $data->{'QOS-POLICY-GROUP'} if($type =~/^vserver/i);
    }

    # Deal with create_from_file
    if ($type =~ /^lun\d*$/i and $data->{CREATE_FROM_FILE}) {
        $data->{FILE}->{SIZE} = delete $data->{SIZE};
    }
    # Convert DATA-AGGR-LIST and AGGR-LIST to an array if it exists.
    elsif ($type =~ /^vol/i) {
        if (exists($data->{'DATA-AGGR-LIST'})) {
            my @aggrs = split(/,/, $data->{'DATA-AGGR-LIST'});
            $data->{'DATA-AGGR-LIST'} = \@aggrs;
        }
        if (exists($data->{'AGGR-LIST'})) {
            my @aggrs = split(/,/, $data->{'AGGR-LIST'});
            $data->{'AGGR-LIST'} = \@aggrs;
        }
        if (exists($data->{'CANDIDATE_AGGREGATES'})) {
            my @aggrs = split(/,/, $data->{'CANDIDATE_AGGREGATES'});
            $data->{'CANDIDATE_AGGREGATES'} = \@aggrs;
        }
        if (exists($data->{'EFFICIENCY_OPTIONS'})) {
            if(exists($data->{'EFFICIENCY'}) and $data->{'EFFICIENCY'}) {
                $data->{'EFFICIENCY_OPTIONS'}->{'_EFFICIENCY'} = 1;
            } else {
                $data->{'EFFICIENCY_OPTIONS'}->{'_EFFICIENCY'} = 0;
            }
        }
    }

    # Pass down delete_policy for ipspace to broadcast domain
    elsif ($type =~ /^ipspace/i) {
        # Loop through to find all the broadcast domains
        foreach my $key (keys(%{$data})) {
            # If its a broadcast domain then do make sure it either has delete_policy
            # or get it from the parent ipspace. Also pass down the delete_policy and mtu
            # to the bd_port children of the broadcast_domain.
            if ($key =~/^(broadcast_domain)/i){
                # Copy delete policy unless it exists
                $data->{$key}->{DELETE_POLICY} = $data->{DELETE_POLICY}
                    unless(exists ($data->{$key}->{DELETE_POLICY}));

                # Loop through the keys of the broadcast domain to find the bd_ports
                foreach my $port_key (keys(%{$data->{$key}})) {
                    # If its a broadcast domain then do make sure it either has delete_policy
                    # or get it from the parent ipspace. Also pass down the delete_policy and mtu
                    # to the bd_port children of the broadcast_domain.
                    if ($port_key =~/^(bd_port)/i){
                        # Copy delete policy unless it exists
                        $data->{$key}->{$port_key}->{DELETE_POLICY} = $data->{$key}->{DELETE_POLICY}
                            unless(exists ($data->{$key}->{$port_key}->{DELETE_POLICY}));

                        # Copy the mtu from the broadcast domain to the bd_port
                        $data->{$key}->{$port_key}->{MTU} = $data->{$key}->{MTU};
                    }
                } # End of foreach to get bd_ports from broadcast domain.
            } # End if broadcast domain
        } # End of foreach to get broacast domains from ipspace.
    }

    # If iSCSI or FCP enabled flags are defined, need to create the corresponding
    # services. And Convert AGGR-LIST to an array if it exists
    elsif ($type =~ /^vserver/i) {
        if (exists($data->{ISCSI_ENABLED}) or
            exists($data->{FCP_ENABLED})   or
            exists($data->{NFS_ENABLED}) or
            exists($data->{CIFS_ENABLED})) {
            $data = $self->_protocol_services_update($data);
        }
        if (exists($data->{'AGGR-LIST'})) {
            my @aggrs = split(/,/, $data->{'AGGR-LIST'});
            $data->{'AGGR-LIST'} = \@aggrs;
        }
        # if USE_DEEP_JUNCTION is set on the vserver tag
        # pass it down to each vol
        if (exists($data->{USE_DEEP_JUNCTION})) {
            foreach my $key (keys(%{$data})) {
                if ($key =~/^(VOL)/i){
                    $data->{$key}->{USE_DEEP_JUNCTION} = $data->{USE_DEEP_JUNCTION};
                }
            }
        }
    }
    # Collect all IP addresses and lifs
    elsif ($type =~ /^lif/i) {
        # A few of things to do with lifs
        #
        # 1. Normalize and Store IP addresses range for each lif in it's own object.
        #    They will be assigned to each resource accordingly.
        # 2. Transform the 'data-protocol' input list into an array.
        # 3. Convert gateway into a network route
        foreach my $lif_key (sort keys %{$data}) {
            # 1
            # There could be more than one tag of ADDRESS_RANGEs
            # so we must verify that the tag name is either plain
            # ADDRESS_RANGE or ADDRESS_RANGE with a number that
            # is actually the index of the tag. The regex only
            # supports 9 address range tags
            if ($lif_key =~ /^ADDRESS_RANGE\d*$/i) {
                push (@ips_range, $self->_normalize_ip_range(ip_range => $data->{$lif_key}->{IP}));
                # Need to delete the Address Range key (tag) to
                # avoid that the recursive call of _walk_xml generates
                # an entry for this tag that is not needed.
                delete $data->{$lif_key};
            }
            # 2
            # Transform the 'comma separated list' to an array, and store
            # the rereference to that array. (as required by NACL)
            elsif ($lif_key =~ /^DATA-PROTOCOL/i) {
                my @protocols = split(/,/, $data->{$lif_key});
                $data->{$lif_key} = \@protocols;
            }
            # 3
            # Convert gateway into a network_route tag.
            elsif ($lif_key =~ /^GATEWAY$/i and length ($data->{$lif_key})) {
                $data->{NETWORK_ROUTE}->{$lif_key} = $data->{$lif_key};
            }
        }

        # If role is intercluster the this will be in the Cluster ipspace.
        $data->{ipspace} = 'Cluster' if ($data->{ROLE} =~ /intercluster/i);
    }

    # /vol/volA/F1:@/vol/volB/F1',/vol/volA/F2:@/vol/volB/F2'
    # need to convert cg-item-mappings to the format below
    # 'cg-item-mappings' => {
    #    '/vol/src0/lun1' => '/vol/dst0/lun1',
    #    '/vol/src0/lun2' => '/vol/dst0/lun2',
    # }
    elsif ($type =~ /^snapmirror$/i) {
        my $new_cg_item_mappings = {};
        foreach my $key (keys %{$data}) {
            if ($key =~ /^CG-ITEM-MAPPINGS$/) {
                my $cg_item_mappings = delete $data->{$key};
                my @cg_items = split(',',$cg_item_mappings);
                foreach my $cg_item (@cg_items) {
                    $cg_item =~ /(.+):@(.+)/;
                    $new_cg_item_mappings->{$1} = $2;
                }
            } elsif ($key =~ /^CG_ITEM\d*$/) {
                my $source = $data->{$key}->{SOURCE_PATH};
                my $destination = $data->{$key}->{DESTINATION_PATH};
                delete $data->{$key};
                $new_cg_item_mappings->{$source} = $destination;
            }
        }
        if (keys %$new_cg_item_mappings) {
            $data->{'CG-ITEM-MAPPINGS'} = $new_cg_item_mappings;
        }
    }

    # If an IPSPACE has IPSPACE_MEMBER tags under it without DELETE_POLICY defined,
    # pass down the DELETE_POLICY from IPSPACE to IPSPACE_MEMBER
    elsif ($type =~ /^ipspace/i) {
        foreach my $key (keys %{$data}) {
            if ($key =~ /^IPSPACE_MEMBER\d+$/) {
                if (not exists($data->{$key}->{DELETE_POLICY})) {
                    $data->{$key}->{DELETE_POLICY} = $data->{DELETE_POLICY};
                }
            }
        }
    }# end if

    # If there's only one copy, leave the name of the resource alone
    if ($num_copies <= 1) {
        if ($type =~ /^lif/i) {
            # Verify there is a range of ips available and
            # assign it.
            if (scalar(@ips_range)) {
                $data->{ips} = \@ips_range;
            }
        }
        push(@objs, $data);
    } else {

        # Go through each element, tweak the name
        for (my $i = 0; $i < $num_copies; $i++) {
            my %tmp_copy;

            # keep the insertion order of the hash so that
            # everything is kept in the proper order
            tie(%tmp_copy, "Tie::IxHash");

            %tmp_copy = %{$data};
            if (    $type =~ /^VOL/i
                and (($i + $offset) >= 100)
                and (($i + $offset) < 1000))
            {
                $tmp_copy{NAME} = $data->{NAME} . "_0" . ($i + $offset);
            } else {
                $tmp_copy{NAME} = $data->{NAME} . "_" . ($i + $offset);
            }

            # TODO: Not sure if the PROTOCOL_COUNT is used, will leave
            #       logic in place to be consistent, but we could delete
            #       the following conde.
            if ($type =~ /^VSERVER/i and scalar(@ips_range)) {
                $tmp_copy{PROTOCOL_COUNT} = $i;
            }
            # Store a range of IPs for each lif in EnGenX's object.
            if ($type =~ /^LIF/i){
                # If the reference of the var ips has been defined, means
                # that there was an AddressRange asociated with this LIF
                # so all the ip_addresses will be assigned to the assigned
                # to each Lif, The subprocess will take care of distributing
                # the appropiate ip_address.
                if (scalar(@ips_range)) {
                    $tmp_copy{ips} = \@ips_range;
                }
            }
            push(@objs, \%tmp_copy);
        }# end for
    }# end else

    return (wantarray()) ? @objs : \@objs;
}    # End _generate_copies

###############################################################################
# Method:    _retrieve_iscsi_global_options
# Objective: Verify if the user entered any ISCSI GLOBAL OPTION in the
#            environment tag and retrieve those values.
# Input:     No input.
# Returns:   TRUE or propagates an exception.
###############################################################################
sub _retrieve_iscsi_global_options {
    my ($self) = @_;
    $Log->enter() if $may_enter;

    my $environment = $self->resource_list()->get_resources(category => "ENVIRONMENT",
                                                            type => "environment");
    my $filtered_environment = $environment->[0];

    # We know that iSCSI GLOBAL OPTIONS have been defaulted or specified, so let's
    # find out if the ISNS address was provided
    if( defined($filtered_environment->{GLOBAL_ISCSI_OPTIONS}->{ISNS_SERVER}) &&
        ($filtered_environment->{GLOBAL_ISCSI_OPTIONS}->{ISNS_SERVER} ne "") ){
            $self->{parent}->{globals}->{iscsi}->{isns_server} =
                                        $filtered_environment->{GLOBAL_ISCSI_OPTIONS}->{ISNS_SERVER};
    }
    # Also let's find and save the CHAP USER NAMES if provided. NOTE, the
    # passwords would always be defined and not blank because they have a
    # default value on the DTD file. No need to save those values
    if( defined($filtered_environment->{GLOBAL_ISCSI_OPTIONS}->{CHAP_IN_USERNAME}) &&
        ($filtered_environment->{GLOBAL_ISCSI_OPTIONS}->{CHAP_IN_USERNAME} ne "") ){
            $self->{parent}->{globals}->{iscsi}->{chap_in_username} =
                                    $filtered_environment->{GLOBAL_ISCSI_OPTIONS}->{CHAP_IN_USERNAME};
    }

    if( defined($filtered_environment->{GLOBAL_ISCSI_OPTIONS}->{CHAP_OUT_USERNAME}) &&
        ($filtered_environment->{GLOBAL_ISCSI_OPTIONS}->{CHAP_OUT_USERNAME} ne "") ) {
            $self->{parent}->{globals}->{iscsi}->{chap_out_username} =
                                    $filtered_environment->{GLOBAL_ISCSI_OPTIONS}->{CHAP_OUT_USERNAME};
    }

    $Log->exit() if $may_exit;
    return TRUE;
}

###############################################################################
# Method:    _protocol_services_update
# Objective: Verifies that the iSCSI, FCP, NFS, and CIFS services tags are created if
#            vserver information that is been worked has those flags enabled.
#            Also uses the GLOBAL OPTIONS values to create or update those
#            tags.
# Input:     The data hash that has the vserver xml information.
# Returns:   The data hash updated.
###############################################################################
sub _protocol_services_update {
    my ($self, $data) = @_;
    $Log->enter() if $may_enter;

    # Locals
    my $iscsi_service_found = (exists($data->{ISCSI_SERVICE})) ? TRUE : FALSE;
    my $fcp_service_found   = (exists($data->{FCP_SERVICE}))   ? TRUE : FALSE;
    my $isns_service_found  = (exists($data->{ISNS_SERVICE}))  ? TRUE : FALSE;
    my $cifs_service_found  = (exists($data->{CIFS_SERVICE}))  ? TRUE : FALSE;
    my $nfs_service_found   = (exists($data->{NFS_SERVICE}))   ? TRUE : FALSE;

    # Start with FCP
    # If FCP_SERVICE @vserver is ENABLED, need to make sure the service
    # is in place.
    if ($data->{FCP_ENABLED}) {
        # If service was not provided by the user, create it.
        if (!$fcp_service_found) {
            $data->{FCP_SERVICE}->{NAME}= $data->{NAME};
            $data->{FCP_SERVICE}->{'STATUS-ADMIN'} = 'up';
        }
        # Propagate the ENABLED flag
        $data->{FCP_SERVICE}->{SERVICE_ENABLED}= TRUE;
    }
    # If FCP_SERVICE @vserver is NOT ENABLED
    else {
        if ($fcp_service_found) {
            # Disable the service flag
            $data->{FCP_SERVICE}->{SERVICE_ENABLED}= FALSE;
        }
    }

    # If iSCSI_SERVICE @vserver is ENABLED, we need to make sure the service
    # is in place.
    if ($data->{ISCSI_ENABLED}) {
        # If iSCSI SERVICE was not provided by the user, create it.
        if (!$iscsi_service_found) {
            $data->{ISCSI_SERVICE}->{NAME}= $data->{NAME};
            $data->{ISCSI_SERVICE}->{'STATUS-ADMIN'} = 'up';
        }
        # Propagate the ENABLED flag
        $data->{ISCSI_SERVICE}->{SERVICE_ENABLED}= TRUE;

        # If the ISNS SERVICE exists
        if ($isns_service_found) {
            # Propagate the ENABLED flag
            $data->{ISNS_SERVICE}->{SERVICE_ENABLED}= TRUE;
            # If the ADDRESS is NOT defined, use the provided ISNS_SERVER global
            # option
            if( (!defined($data->{ISNS_SERVICE}->{ADDRESS})) &&
                (defined($self->{parent}->{globals}->{iscsi}->{isns_server})) ){
                $data->{ISNS_SERVICE}->{ADDRESS} =
                                    $self->{parent}->{globals}->{iscsi}->{isns_server};
            }
        } else {
            # If user provided an ISNS_SERVER global option, create an ISNS
            # service tag
            if( defined($self->{parent}->{globals}->{iscsi}->{isns_server}) ) {
                # Propagate the ENABLED flag
                $data->{ISNS_SERVICE}->{SERVICE_ENABLED}= TRUE;
                $data->{ISNS_SERVICE}->{FORCE}= "true";
                $data->{ISNS_SERVICE}->{ADDRESS} =
                                        $self->{parent}->{globals}->{iscsi}->{isns_server};
            }
        }
    }
    # If ISCSI_SERVICE @vserver is NOT ENABLED, disable iSCSI and ISNS services
    else {
        if ($iscsi_service_found) {
            # Disable the service flag
            $data->{ISCSI_SERVICE}->{SERVICE_ENABLED}= FALSE;
        }
        if ($isns_service_found) {
            # Disable the service flag
            $data->{ISNS_SERVICE}->{SERVICE_ENABLED}= FALSE;
        }
    }

    # If CIFS_SERVICE @vserver is ENABLED, need to make sure the service
    # is in place.
    if ($data->{CIFS_ENABLED}) {
        # If service was not provided by the user
        if (!$cifs_service_found) {
            # create a default cifs service tag
            $data->{CIFS_SERVICE}->{NAME}= $data->{NAME};
            $data->{CIFS_SERVICE}->{'STATUS-ADMIN'} = 'up';
            # use the global settings from the global_cifs_options tag
        }
        # Propagate the ENABLED flag
        $data->{CIFS_SERVICE}->{SERVICE_ENABLED} = TRUE;
    } else {
        # If CIFS_SERVICE @vserver is NOT ENABLED
        if ($cifs_service_found) {
            # Disable the service flag
            $data->{CIFS_SERVICE}->{SERVICE_ENABLED} = FALSE;
        }
    }

    # If NFS_SERVICE @vserver is ENABLED, need to make sure the service
    # is in place.
    if ($data->{NFS_ENABLED}) {
        # If service was not provided by the user
        if (!$nfs_service_found) {
            # create a default nfs service tag
            $data->{NFS_SERVICE}->{NAME}= $data->{NAME};
            # use the global settings from the global_nfs_options tag
        }
        # Propagate the ENABLED flag
        $data->{NFS_SERVICE}->{SERVICE_ENABLED} = TRUE;
    } else {
        # If NFS_SERVICE @vserver is NOT ENABLED
        if ($nfs_service_found) {
            # Disable the service flag
            $data->{NFS_SERVICE}->{SERVICE_ENABLED} = FALSE;
        }
    }

    $Log->exit() if $may_exit;
    return (wantarray()) ? %{$data} : $data;
}

1;
